<?php

/**
 * UTF8
 * Pavel Rudol, RON Software
 *  11.2.2015
 * seznam osob generovanych do terminalu
 */
namespace Ron\RonPortal;

use Ron\Browser;
use Ron\DB\SqlStatement;
use function Ron\ronText;

require_once('ron/browser.php');


/**
 * Seznam osob generovanych do terminalu
 *
 * @author pavel.rudol
 */
class OsobyTerminaluBrowser extends Browser {
    
    private $pravaZobrazit;
    private $terminal;
            
    function __construct($database, $terminal) {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');
        $this->terminal = $terminal;
        
        $this->title = ronText('seznam_osob');

        $this->code = 'browserosobyterminalu';
        $this->tableId = 'browserosobyterminalu';
        
        $this->sqlSelectPart = 
            'PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu,  '. // 0 - 2            
            'Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, PracovniPomer.Zamestnanec, PrirazeniKarty.KodKarty, S_TypOsoby.Popis '; // 3 - 8
        
        $this->sqlFromPart = 
                'Terminal, Osoba, PracovniPomer, PrirazeniKarty, Karta, S_TypOsoby';
        
        $this->sqlWherePart =
                '(Osoba.RC = PracovniPomer.RC) '.
                'AND (PrirazeniKarty.RC = PracovniPomer.RC) '.
                'AND (PrirazeniKarty.OsobniCislo = PracovniPomer.OsobniCislo) '.
                'AND (PrirazeniKarty.DatumNastupu = PracovniPomer.DatumNastupu) '.
                'AND (Karta.kod = PrirazeniKarty.kodKarty) '.
                'AND ((Karta.blokovana = 0) or (Karta.blokovana is null)) '.
                'AND (S_TypOsoby.Kod = PracovniPomer.Zamestnanec)';                              
                
        $this->defaultSort = 3;
        $this->primaryField = 0;
        $this->distinct = true;
        
        $this->browserUrl = '?menuid=terminaly&record='.$this->terminal.'&';

        $this->addField( [
            BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_LINE] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Osoba.Titul', BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_HEADER_TEXT => 'Titul',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_HEADER_TEXT => 'Prijmeni',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_HEADER_TEXT => 'Jmeno',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'PracovniPomer.OsobniCislo', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'Osobni_cislo',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'PrirazeniKarty.KodKarty', BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_HEADER_TEXT => 'karta',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'S_TypOsoby.Popis', BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_HEADER_TEXT => 'typPPomeru',
            BROWSER_FIELD_ORDERING => 1] );
    }
    
    function renderFilter()
    {
        $form = new RonPortalFilterForm($this->database, 'terminaly&record='.$this->terminal);
        $form->NastavDatumOdDo(true, false, ronText('terminal_platnost') . ' ' . ronText('Od'));
        $form->render();
    }
    
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result =
            "(Terminal.kod = " . $params->addParameter($this->terminal) . ") ".
            " AND ";

        if (!RonPortalSession::getFiltrovatDatum()) {
            $datumOd = time();
            $datumDo = $datumOd;
        } else {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        }
        
        if ($this->pravaZobrazit->jsouNejakePrava()) {
            $result .=
                '( ( '.
                "    (PracovniPomer.DatumNastupu <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((PracovniPomer.DatumUkonceni is null) or (PracovniPomer.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam($datumOd)) . ")) ".
                "AND (PrirazeniKarty.DatumPrideleni <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((PrirazeniKarty.DatumVraceni is null) or (PrirazeniKarty.DatumVraceni >= " . $params->addParameter($this ->database ->dateToParam($datumOd)) . ")) ".
                "AND (Karta.DatumPorizeni <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((Karta.DatumVyrazeni is null) or (Karta.DatumVyrazeni >= " . $params->addParameter($this->database->dateToParam($datumOd)) . ")) ) ".

                'and ( (EXISTS (SELECT * FROM RozdeleniOsob RO, PovoleniPristupuRozdeleni PPR        '.
                'WHERE (PPR.Rozdeleni = RO.Rozdeleni)  '.
                'AND (PPR.TypRozdeleni = RO.KodTypuRozdeleni) '.
                'AND (PPR.Terminal = Terminal.kod) '.
                "AND (PPR.PlatnostOd <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((PPR.PlatnostDo is null) or (PPR.PlatnostDo >= " . $params->addParameter($this->database->dateToParam($datumOd)) . ")) ".
                'AND (RO.KodTypuRozdeleni = 7) '.
                'AND (RO.RC = PracovniPomer.RC) '.
                'AND (RO.OsobniCislo = PracovniPomer.OsobniCislo) '.
                'AND (RO.DatumNastupu = PracovniPomer.DatumNastupu) '.
                "AND (RO.PlatnostOd <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((RO.PlatnostDo is null) or (RO.PlatnostDo >= " . $params->addParameter($this->database->dateToParam($datumOd)) . "))        )) ".

                'OR (EXISTS (SELECT * FROM PovoleniPristupuOsoby PPO '.
                'WHERE (PPO.Terminal = Terminal.kod) '.
                'AND (PPO.RC = PracovniPomer.RC) '.
                'AND (PPO.OsobniCislo = PracovniPomer.OsobniCislo) '.
                'AND (PPO.DatumNastupu = PracovniPomer.DatumNastupu) '.
                "AND (PPO.PlatnostOd <= " . $params->addParameter($this->database->dateToParam($datumDo)) . ") ".
                "AND ((PPO.PlatnostDo is null) or (PPO.PlatnostDo >= " . $params->addParameter($this->database->dateToParam($datumOd)) . "))        )) ) )";
        } else {
            $result .= '1 = 0';
        }
                
        return new SqlStatement($result, $params);
    }
}
