<?php

/**
 * UTF8
 * Pavel Rudol, RON Software
 *  13.3.2015
 * seznam skupin generovanych do terminalu
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Browser;
use Ron\DB\SqlStatement;
use function Ron\ronText;

require_once('ron/browser.php');


/**
 * Seznam skupin generovanych do terminalu
 *
 * @author pavel.rudol
 */
class SkupinyTerminaluBrowser extends Browser {
    
    private $pravaZobrazit;
    private $terminal;
    
    function __construct($database, $terminal) {
        parent::__construct($database);
        
        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');
        $this->terminal = $terminal;
        
        $this->title = ronText('terminal_skupiny');

        $this->code = 'browserskupinyterminalu';
        $this->tableId = 'browserskupinyterminalu';
        
        $this->sqlSelectPart = 
            'Rozdeleni.Kod, Rozdeleni.Popis  '; // 0 - 2            
            
        
        $this->sqlFromPart = 
                'PovoleniPristupuRozdeleni JOIN Rozdeleni ON (Rozdeleni.Kod = PovoleniPristupuRozdeleni.Rozdeleni '.
                '  AND Rozdeleni.KodTypuRozdeleni = PovoleniPristupuRozdeleni.TypRozdeleni)';
        
//        $this->sqlWherePart =
//                "(PovoleniPristupuRozdeleni.Terminal = '" . $this->database->addSlashes($this->terminal) . "') ";
                
        $this->defaultSort = 2;
        $this->primaryField = 0;
        //$this->f_distinct = true;
        
        $this->browserUrl = '?menuid=terminaly&record='.$this->terminal.'&';
        
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Kod', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER_TEXT => 'Kod',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'rozdeleni',
            BROWSER_FIELD_ORDERING => 1] );          
                
    }

    /**
     * @throws Exception
     */
    function renderFilter()
    {
        $form = new RonPortalFilterForm($this->database, 'terminaly&record='.$this->terminal);
        $form->nastavDatumOdDo(true, false, ronText('terminal_platnost') . ' ' . ronText('Od'));
        $form->loadData();
        $form->render();        
    }
    
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result =
            "(PovoleniPristupuRozdeleni.Terminal = " . $params->addParameter($this->terminal) . ") " .
            " AND ";
        
        if (!RonPortalSession::getFiltrovatDatum()) {
            $datumOd = time();
            $datumDo = $datumOd;
        } else {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        }
        
        if ($this->pravaZobrazit->jsouNejakePrava()) {
            $result .=
                '  PovoleniPristupuRozdeleni.PlatnostOd  <= ' . $params->addParameter($this->database->dateToParam($datumDo)) .
                ' AND (PovoleniPristupuRozdeleni.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam($datumOd)) . ' OR PovoleniPristupuRozdeleni.PlatnostDo IS NULL)';
        } else {
            $result .= '1 = 0';
        }
                
        return new SqlStatement($result, $params);
    }
}
