<?php

/**
 * UTF8
 * Pavel Rudol, RON Software
 *  10.2.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalbrowser.php');
require_once('ron/guiutils.php');

/*
 * Zobrazi filtr rozdeleni, umisteni, urceni
 */
class TerminalyFilterForm extends RonPortalFilterForm
{

    /**
     * TerminalyFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param PravaNaOsoby $prava
     */
    function __construct($database, $action, $prava)
    {
        parent::__construct($database, $action);

        $this->f_classname = 'filtercontainer leftcaptions';        

        $this->f_formaction = '?menuid=terminaly';

        //TODO: rozdeleni v session
        $this->nastavRozdeleni(true, $prava->typRozdeleni, 2);
        $this->showVcetnePodrizenychRozdeleni = false;
        $this->f_pravarozdeleni = $prava;
    }

    /**
     * @throws Exception
     */
    function renderUmisteniTerminalu()
    {
        echo '<DIV class="filterrow">';

        // checkbox "Jen umisteni"
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenUmisteniTerminalu', ronText('terminal_jen_umisteni') . '&nbsp;&nbsp;', isset($_SESSION['jenUmisteniTerminalu']), 'onClick="submit()"');

        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            'SELECT Kod, Popis FROM Umisteni ';        

       $sql .=
            ' ORDER BY Popis';

        // vyber umisteni podle popisu
        $combo = new ComboBox('umisteniTerminalu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->loadData($this ->database, $sql, isset($_SESSION['umisteniTerminalu']) ? $_SESSION['umisteniTerminalu'] : '');
        $combo->htmlParams = 'onChange="if (jenUmisteniTerminalu.checked) submit();"';
        $combo->render();
        
        echo "</DIV>\n";
    }
    

    /**
     * @throws Exception
     */
    function renderUrceniTerminalu()
    {
        echo '<DIV class="filterrow">';

        // checkbox "Jen umisteni"
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenUrceniTerminalu', ronText('terminal_jen_urceni') . '&nbsp;&nbsp;', isset($_SESSION['jenUrceniTerminalu']), 'onClick="submit()"');

        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            'SELECT Kod, Popis FROM S_UrceniTerminalu ';        

       $sql .=
            ' ORDER BY Popis';

        // vyber urceni podle popisu
        $combo = new ComboBox('urceniTerminalu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->loadData($this ->database, $sql, isset($_SESSION['urceniTerminalu']) ? $_SESSION['urceniTerminalu'] : '');
        $combo->htmlParams = 'onChange="if (jenUrceniTerminalu.checked) submit();"';
        $combo->render();
        
        echo "</DIV>\n";
        
    }
    
    // --------------------------------------------------------------------
    function renderFiltrStav()
    {        
        echo '<DIV class="filterrow">';

        // checkbox "Jen neuspesne nactene"
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenNeuspesneNastavene', ronText('terminal_jen_neuspesne_nastavene') . '&nbsp;&nbsp;', isset($_SESSION['jenNeuspesneNastavene']), 'onClick="submit()"');

        echo '</SPAN>';
        
        echo "</DIV>\n";
        
    }
    
    private function loadUmisteniTerminalu()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['jenUmisteniTerminalu']))
                $_SESSION['jenUmisteniTerminalu'] = true;
            else
                unset($_SESSION['jenUmisteniTerminalu']);
            
            $_SESSION['umisteniTerminalu'] = $_POST['umisteniTerminalu'];
        }
    }
    
    private function loadUrceniTerminalu()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['jenUrceniTerminalu']))
                $_SESSION['jenUrceniTerminalu'] = true;
            else
                unset($_SESSION['jenUrceniTerminalu']);
            
            $_SESSION['urceniTerminalu'] = $_POST['urceniTerminalu'];
        }
    }
    
    private function loadFiltrStav()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['jenNeuspesneNastavene']))
                $_SESSION['jenNeuspesneNastavene'] = true;
            else
                unset($_SESSION['jenNeuspesneNastavene']);
        }
    }
    
    function loadData()
    {
        parent::loadData();
        
        $this->loadUmisteniTerminalu();
        $this->loadUrceniTerminalu();
        $this->loadFiltrStav();
    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        parent::renderFilters(); // rozdeleni
       
        $this->renderUrceniTerminalu();
        
        $this->renderUmisteniTerminalu();
       
        $this->renderFiltrStav();
        
        echo "<BR />";
    }
}


/**
 * Seznam terminalu, na ktere ma prihlaseny uzivatel pravo podle rozdeleni. 
 * Pokud nelze pro filtrovani pouzit rozdeleni, budou se zobrazovat vsechny terminaly.
 *
 * @author pavel.rudol
 */
class TerminalyBrowser extends RonPortalBrowser
{
    
    /** @var PravaNaOsoby */
    private $pravaZobrazit;

    /**
     * TerminalyBrowser constructor.
     * @param Database $database
     * @param $action
     */
    public function __construct($database, $action) {
        parent::__construct($database, $action);        
        
        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');
        
        $this->title = ronText('terminaly');

        $this->code = 'browserterminaly';
        $this->tableId = 'browserterminaly';
        
        $this->sqlSelectPart = 
            'Terminal.Kod, Terminal.Popis, Terminal.KodUrceni, S_UrceniTerminalu.Popis, '. // 0 - 3            
            'Terminal.KodUmisteni, Umisteni.Popis, Rozdeleni.Popis, '.$database ->castWideMemo('Terminal.Poznamka').', '. // 4 - 7
            'StavTerminalu.NeuspesnychNastaveni'; //8
        
        $this->sqlFromPart = 
                'Terminal JOIN S_UrceniTerminalu ON (S_UrceniTerminalu.Kod = Terminal.KodUrceni) '.
                'LEFT JOIN Umisteni ON (Umisteni.Kod = Terminal.KodUmisteni) '.
                'LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = Terminal.KodTypuRozdeleni AND Rozdeleni.Kod = Terminal.KodRozdeleni)'.
                'LEFT JOIN StavTerminalu ON (StavTerminalu.KodTerminalu = Terminal.Kod)';
        
        
        $this->defaultSort = 1;
        $this->primaryField = 0;
        
        $this->browserUrl = '?menuid=terminaly&';
        
        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'prohl_terminaly';
        
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Terminal.Kod', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER_TEXT => 'kod',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Terminal.Popis', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'popis',
            BROWSER_FIELD_LINK => '?menuid=terminaly', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'S_UrceniTerminalu.Popis', BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_HEADER_TEXT => 'terminal_urceni',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Umisteni.Popis', BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_HEADER_TEXT => 'umisteni',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis', BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_HEADER_TEXT => 'rozdeleni',
            BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'StavTerminalu.NeuspesnychNastaveni', BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_HEADER_TEXT => 'terminal_pocet_nastaveni_browser',
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1] );

    }
    

    function setFilterForm() {
        $this->filterForm = new TerminalyFilterForm($this->database, $this->action, $this->pravaZobrazit);        
    }
    
    
    /**
     * Vraci cast retezce, ktera se vlozi do WHERE casti dotazu
     * muze vracet napriklad filtr podle zvolenych filtru (viz renderFilter)
     * 
     * @return string
     */
    function getSQLFilter()
    {

        if ($this->pravaZobrazit->jsouNejakePrava())
        {
            //$result = '((Rozdeleni.KodTypuRozdeleni = 8) OR (' . $this->f_pravaZobrazit->SQLfiltrRozdeleni(time(), time(), 1) . '))'; //rozdeleni na ktere je pravo vcetne vsech jidelen
            $result = '(' . $this->pravaZobrazit->sqlfiltrRozdeleni(time(), time(), 1) . ')'; //rozdeleni na ktere je pravo
        }
        else 
        {   
            $result = '1 = 0';   
        } 
                
        if (isset($_SESSION['jenUmisteniTerminalu'])) {
            if ($result) {
                $result .= ' AND';
            }
            $result .= " Terminal.KodUmisteni = '" . $this->database->addSlashes($_SESSION['umisteniTerminalu']) . "'";
        }
        
        if (isset($_SESSION['jenUrceniTerminalu'])) {
            if ($result) {
                $result .= ' AND';
            }
            $result .= " Terminal.KodUrceni = '" . $this->database->addSlashes($_SESSION['urceniTerminalu']) . "'";
        }
        
        if ($this->filterForm->kodRozdeleni) {
            if ($result) {
                $result .= ' AND';
            }
            $result .=
                ' Terminal.KodTypuRozdeleni = '. (int)$this->pravaZobrazit->typRozdeleni .
                " AND Terminal.KodRozdeleni = '" . $this->database->addSlashes($this->filterForm->kodRozdeleni) . "'";
        }
        
        if (isset($_SESSION['jenNeuspesneNastavene'])) {
            if ($result) {
                $result .= ' AND';
            }
            $result .= " StavTerminalu.NeuspesnychNastaveni > 0";
        }
        
        return $result;
    }
    
    function renderLine($values, $classname, $rownumber, $edit = false)
    {      
        
        if ($values[8] > 0) {             
            $classname = $rownumber % 2 ? 'terminal_neuspesnenastaveni1' : 'terminal_neuspesnenastaveni2';
        }
        
        parent::renderLine($values, $classname, $rownumber, $edit);
       
        
        if ($values[7]) // podrobny popis
        {
            echo "<tr class='$classname'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '"><span class="browserpoznamka terminalpoznamka">' . StringUtils::htmlTextConvert($values[7]) . '</span></td>';
            echo '</tr>';
                        
        }
    }
}
