<?php
/**
 * UTF8
 * stanislav.elbl, 01.11.2017
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

require_once 'ron/browser.php';


class ZastupJmenoPrijmeniField extends BrowserField
{
    public $fieldJmeno;
    public $fieldPrijmeni;

    /**
     * ZastupJmenoPrijmeniField constructor.
     * @param int $fieldJmeno
     * @param int $fieldPrijmeni
     * @param string $header
     */
    public function __construct($fieldJmeno, $fieldPrijmeni, $header)
    {
        parent::__construct();

        $this->fieldJmeno = $fieldJmeno;
        $this->fieldPrijmeni = $fieldPrijmeni;

        $this ->header = $header;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return StringUtils::htmlTextConvert($rowData[$this ->fieldJmeno] . ' ' . $rowData[$this ->fieldPrijmeni]);
    }
}


class LoginAsLinkField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[3] < 2) { // nezobrazuje link pro zrusene a zamitnute
            if ($this->database->stringToTimestamp($rowData[1]) <= time() && (!$rowData[2] || $this->database->stringToTimestamp($rowData[2]) >= DateUtils::startOfTheDay(time()))) {
                return '<a class="confirmQuestion" href="?menuid=zastupuzivatele&action=loginas&id=' . $rowData[0] . '" data-ron-confirm-id="Opravdu_prihlasit_jako_zastup">' . \Ron\ronText('Prihlasit_se_jako_zastup') . '</a>';
            }
        }

        return '';
    }
}


class ZastupyBrowser extends Browser
{
    const JA_ZASTUPUJI = 1;
    const JSEM_ZASTUPOVAN = 2;
    const VSECHNY_ZASTUPY = 3;

    public $coZobrazit;

    /**
     * @param \Ron\DB\Database $database
     * @param int $coZobrazit  konstanty JA_ZASTUPUJI/JSEM_ZASTUPOVAN/VSECHNY_ZASTUPY
     */
    function __construct($database, $coZobrazit)
    {
        parent::__construct($database);
        $this ->coZobrazit = $coZobrazit;

        $this->rowClickLink = '?menuid=zastupuzivatele';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->title = \Ron\ronText('Vy_zastupujete_uzivatele');
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $this->title = \Ron\ronText('Zastupuje_vas_uzivatel');
        } else {
            $this->title = '';
        }

        $this->sqlFromPart =
            'ZastupUzivatele' .
            //' INNER JOIN RoleProZastupUzivatele ON RoleProZastupUzivatele.Kod = ZastupUzivatele.Role' .
            ' INNER JOIN Osoba Zastupovany ON Zastupovany.RC = ZastupUzivatele.Zastupovany' .
            ' INNER JOIN Osoba Zastupujici ON Zastupujici.RC = ZastupUzivatele.Zastupujici';

        $this ->sqlSelectPart =
            'ZastupUzivatele.Id, ' . //  ZastupUzivatele.Role, RoleProZastupUzivatele.Popis, ' . // 0
            'ZastupUzivatele.PlatnostOd, ZastupUzivatele.PlatnostDo, ZastupUzivatele.Potvrzeni, ZastupUzivatele.CasPotvrzeni, ' . // 1..4
            'Zastupovany.RC, Zastupovany.Jmeno, Zastupovany.Prijmeni, ' . // 5..7
            'Zastupujici.RC, Zastupujici.Jmeno, Zastupujici.Prijmeni, ' . // 8..10
            $database ->castWideMemo('ZastupUzivatele.Poznamka') . ', ' . $database ->castWideMemo('ZastupUzivatele.VyjadreniZastupujiciho'); //11,12

        $this ->primaryField = 0;

        $this->sqlOrderPart = 'PlatnostOd DESC';

        $params = $database ->createParameters();
        $where = '(ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $where .= ' AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRC());
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $where .= ' AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC());
        }

        $this ->sqlWherePart = new SqlStatement($where, $params);

        $this ->addField( [BROWSER_FIELD_SQL_NAME => 'ZastupUzivatele.PlatnostOd',  BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_HEADER_TEXT => 'Platnost_od'] );
        $this ->addField( [BROWSER_FIELD_SQL_NAME => 'ZastupUzivatele.PlatnostDo',  BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_HEADER_TEXT => 'Platnost_do'] );
        //$this ->addField( [BROWSER_FIELD_SQL_NAME => 'RoleProZastupUzivatele.Popis',  BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_HEADER_TEXT => 'Role'] );

        if ($coZobrazit == self::JA_ZASTUPUJI  ||  $coZobrazit == self::VSECHNY_ZASTUPY) {
            // TODO: predelat na JmenoPrijmeniBrowserField
            $this->addField( new ZastupJmenoPrijmeniField(6, 7, \Ron\ronText('Zastupovany_uzivatel')));
        }

        if ($coZobrazit == self::JSEM_ZASTUPOVAN  ||  $coZobrazit == self::VSECHNY_ZASTUPY) {
            $this->addField( new ZastupJmenoPrijmeniField(9, 10, \Ron\ronText('Zastupujici_uzivatel')));
        }

        $this ->addField( [BROWSER_FIELD_SQL_NAME => 'ZastupUzivatele.Potvrzeno',  BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_SELECT,
            BROWSER_FIELD_OPTIONS => [ 0 => \Ron\ronText('Zastup_potvrzeni_0'), 1 => \Ron\ronText('Zastup_potvrzeni_1'), 2 => \Ron\ronText('Zastup_potvrzeni_2'), 3 => \Ron\ronText('Zastup_potvrzeni_3')]
        ] );

        $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_TEXT, BROWSER_FIELD_TEXT => \Ron\ronText('Podrobnosti'), BROWSER_FIELD_LINK => '?menuid=zastupuzivatele'] );

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->addField( new LoginAsLinkField($this ->database));
        } else {
            $this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_TEXT, BROWSER_FIELD_TEXT => \Ron\ronText('Upravit'), BROWSER_FIELD_LINK => '?menuid=zastupuzivatele&formaction=edit'] );
        }
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter); // TODO: Change the autogenerated stub
        $this ->cursor = new ArrayCursor($this ->cursor);
    }


    function enableLine($values)
    {
        return
            $this ->coZobrazit == self::JA_ZASTUPUJI  ||
            $this ->coZobrazit == self::JSEM_ZASTUPOVAN ||
            Prava::vratPrava('zast_uz_komu') ->lzeZobrazitRC($values[5], time(), time());
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[11]) {
            // zobrazi poznamku
            $colSpan = $this->columnCount - 1;

            echo "<tr class='$className'><td></td><td colspan='$colSpan'><div class='poznamka browserpoznamka'>" . StringUtils::htmlTextConvert($values[11]) . '</div></td>';
        }
    }


    function render()
    {
        if ($this ->cursor ->numRows()) {
            parent::render();
        } else {
            if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
                echo '<div class="infomsg">' . \Ron\ronText('Nemate_naplanovan_zadny_zastup') . '</div>';
            } else if ($this->coZobrazit == self::JA_ZASTUPUJI) {
                echo '<div class="infomsg">' . \Ron\ronText('Nikoho_nezastupujete') . '</div>';
            }
        }

        if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novazadost">' . \Ron\ronText('Pozadat_o_zastup') . '</a></div>';
        }

        if ($this->coZobrazit == self::VSECHNY_ZASTUPY) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novyzastup">' . \Ron\ronText('Naplanovat_zastup_uzivatele') . '</a></div>';
        }
    }


}
