<?php

/**
 * UTF8
 * Pavel Rudol, RON Software
 *  10.2.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use function Ron\ronText;

class TerminalyController extends HttpController
{
    /** @var $database Database */
    private $database;
    
    public function __construct($database) 
    {
        $this->database = $database;
        
        HtmlHeaders::getInstance()->addScript('script/terminaly.js');
        TextManager::getInstance() ->loadStrings('terminaly.ini');
        HtmlHeaders::getInstance()->addHeader("<script type='text/javascript' src='?menuid=terminaly&action=translations'></script>");        
        
    }

    /**
     *
     * @param PravaNaOsoby $prava pravo na zobrazeni terminalu
     * @param string $terminal kod terminalu
     * @return true pokud ma uzivatel pravo na zvoleny terminal
     * @throws DBException
     * @throws Exception
     */
    function jsouPravaNaTerminal($prava, $terminal)
    {
        $sql =  "SELECT Terminal.KodTypuRozdeleni, Terminal.KodRozdeleni, Terminal.Kod FROM Terminal ".
                "LEFT JOIN Rozdeleni ON (Terminal.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND Terminal.KodRozdeleni = Rozdeleni.Kod)".
                "WHERE Terminal.Kod = '" . $this->database->addSlashes($terminal) . "'".
                ' AND (' . $prava->sqlfiltrRozdeleni(time(), time(), 1) . ')'; //pravo na rozdeleni
                //' AND ((Terminal.KodTypuRozdeleni = 8) OR (' . $prava->SQLfiltrRozdeleni(time(), time(), 2) . '))'; //pravo na rozdeleni a jidelnove terminaly

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->select($sql);
        if (!$cursor) 
        {            
            throw new DBException($this->database,$sql);
        }
        $row = $cursor->fetch();

        if (isset($row[2])) {
            return true;
        }

        return false;
    }

    /**
     * @param string $terminal kod terminalu
     * @return string vychozi view portalu
     * @throws DBException
     * @throws Exception
     */
    function detailTerminalu($terminal)
    {
        require_once('adsapp/b_terminalosoby.php');
        require_once('adsapp/b_terminalskupiny.php');
        require_once('adsapp/ronportalfilter.php');

        $d = new Terminal($this->database, $terminal);
        $d->nactiTerminal();
        $d->nactiStavTerminalu();
        Model::addData('terminal',$d);
        Model::addViewObject(new BaseViewObject('view/terminal_detail.php'));
        
        $bSkupiny = new SkupinyTerminaluBrowser($this->database, $terminal);
        $bSkupiny->load();        
        Model::addViewObject($bSkupiny);
        
        $b = new OsobyTerminaluBrowser($this->database, $terminal);
        $b->load();
        Model::addViewObject($b);

        Model::addViewObject(new TextViewObject("<A href='?menuid=terminaly'>". ronText('Zpet_na_seznam') ."</A>",'textlink'));
        return 'view/appview.php';
    }

    /**
     * Spusti agenta terminalu
     * @param string $terminal kod terminalu
     * @param boolean $lzeGenerovat je mozne (je pravo) provest generovani
     * @return JsonOutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    function provedAkciTerminalu($terminal, $lzeGenerovat)
    {
        if (isset($_POST['terminal_kod']))
        {   
            $_SESSION['terminalUspesnyTest'] = false;
            
            $d = new Terminal($this->database, $terminal);
            if (isset($_POST['test']) && $d->spustAgentaTerminalu('test')) {
                $_SESSION['terminalUspesnyTest'] = true;                
                return new Redirect(null,'?menuid=terminaly&record='.$terminal);
            }
            elseif (isset($_POST['nastaveni']) && $lzeGenerovat) {
                return $this->provedGenerovaniTerminalu($d);
            }
        }

        if (isset($_GET['kontrolastavu'])) {
            return $this->vratStavTerminalu($terminal);
        }

        return $this->detailTerminalu($terminal);
    }

    /**
     * Spusti agenta pro generovani, pri pozadavku na JSON vrati i aktualni stav terminalu.
     * Jinak pri uspesnem generovani provede redirect na detail terminalu.
     * @param Terminal $terminal vybrany terminal
     * @return JsonOutputObject|string
     * @throws DBException
     * @throws Exception
     */
    function provedGenerovaniTerminalu($terminal)
    {
        $vysledekAgenta = $terminal->spustAgentaTerminalu('nastaveni');
        if ($vysledekAgenta) {
            if (isset($_POST['json'])) {
                return $this->vratVysledeGenerovaniJSON($terminal, $vysledekAgenta);
            }   
            else {
                return new Redirect(null,'?menuid=terminaly&record='.$terminal->kod);
            }
        }
        else {
            if (isset($_POST['json'])) {
                return $this->vratVysledeGenerovaniJSON($terminal, $vysledekAgenta);
            }   
            else {
                return $this->detailTerminalu($terminal->kod);
            }
        }
    }

    /**
     * @param Terminal $terminal
     * @param bool $vysledekAgenta
     * @return JsonOutputObject
     * @throws Exception
     */
    function vratVysledeGenerovaniJSON($terminal, $vysledekAgenta)
    {
        $terminal->nactiStavTerminalu();

        $json = new JsonOutputObject(['posledniNastaveni' => $terminal->casuspesnehonastaveni, 'posledniNacteni' => $terminal->casuspesnehonacteni,
            'posledniNastaveniStr' => StringUtils::jsonText(DateUtils::fineDateString($terminal->casuspesnehonastaveni, TRUE)),
            'posledniNacteniStr' => StringUtils::jsonText(DateUtils::fineDateString($terminal->casuspesnehonacteni, TRUE)),
            'pocetNastaveni' => $terminal->pocetneuspesnychnastaveni, 'pocetNacteni' => $terminal->pocetneuspesnychnacteni,
            'vysledekAgenta' => $vysledekAgenta,
            'result' => JsonOutputObject::RESULT_OK]);
        $json->addMessages(MessageManager::getInstance());
        return $json;
    }

    /**
     * Vraci JSON objekt s aktualnim stavem terminalu nactenym z databaze.
     * @param string $terminal kod terminalu
     * @return JsonOutputObject
     * @throws Exception
     */
    function vratStavTerminalu($terminal)
    {
        $t = new Terminal($this->database, $terminal);
        $t->nactiStavTerminalu();
        
        return new JsonOutputObject(['posledniNastaveni' => $t->casuspesnehonastaveni, 'posledniNacteni' => $t->casuspesnehonacteni,
            'posledniNastaveniStr' => StringUtils::jsonText(DateUtils::fineDateString($t->casuspesnehonastaveni, TRUE)),
            'posledniNacteniStr' => StringUtils::jsonText(DateUtils::fineDateString($t->casuspesnehonacteni, TRUE)),
            'pocetNastaveni' => $t->pocetneuspesnychnastaveni, 'pocetNacteni' => $t->pocetneuspesnychnacteni, 
            'result' => JsonOutputObject::RESULT_OK]);
    }

    /**
     * @param string $action
     * @return InternalRedirect|JsonOutputObject|OutputObject|Redirect|TranslationsOutputObject|ViewObject|TerminalyBrowser|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        Prava::nastavObdobiProPrava(time(), time());

        $action = isset($_GET['action']) ? $_GET['action'] : '';

        if ($action == 'translations') {
            return $this->translations();
        }

        $pravaProhlizet = Prava::vratPrava('prohl_terminaly');

        if (!$pravaProhlizet->jsouNejakePrava())
        {
            throw new Exception(ronText('Nemate_pravo'));
        }
        
        $terminal = isset($_GET['record']) ? $_GET['record'] : '';
        

        
        //je vybrany konkretni terminal
        if ($terminal)
        {   
            if ($this->jsouPravaNaTerminal($pravaProhlizet, $terminal))
            {   
                Model::addData('prava', $pravaProhlizet);
                
                return $this->provedAkciTerminalu($terminal, $pravaProhlizet->priznak);
            }
            else
            {
                throw new Exception(ronText('Nemate_pravo'));
            }
        }
        else
        {
            require_once('adsapp/b_terminaly.php'); 
            $b = new TerminalyBrowser($this->database, $action);
            $b->load();            
            return $b;            
        }       
    }
}


/**
 * Objekt s detaily terminalu
 */
class Terminal
{
    public $kod;
    public $popis;
    public $poznamka;
    
    public $urceni;
    public $umisteni;
    public $rozdeleni;
    public $typrozdeleni;
    
    public $driver;
    public $typterminalu;
    public $typkomunikacekod;
    public $typkomunikacepopis;
    public $hostitel;
    public $port;
    public $comport;
    public $parita;
    public $baudrate;
    
    public $stav;
    
    public $casuspesnehonacteni;
    public $casuspesnehonastaveni;
    public $casuspesnehospojeni;
    public $pocetneuspesnychnacteni;
    public $pocetneuspesnychnastaveni;
    public $pocetneuspesnychspojeni;
    
    public $verzefirmware;
    public $seriovecislo;
    public $macadresa;
    
    public $kodAplikace;
    
    /** @var Database $database */
    private $database;

    public function __construct($database, $kodTerminalu)
    {
        $this->database = $database;
        $this->kod = $kodTerminalu;

        try {
            $this->zjistiAplikaci();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
        }
    }

    /**
     * @throws DBException
     * @throws Exception
     */
    public function nactiTerminal()
    {
        $row = $this ->nactiDetailTerminalu();        

        $this->popis = $row[1];
        $this->poznamka = $row[7];
        
        $this->urceni = $row[3];
        $this->umisteni = $row[5];
        $this->rozdeleni = $row[6];
        $this->typrozdeleni = $row[8];
        
        $this->driver = $row[9];
        $this->typterminalu = $row[10];
        $this->typkomunikacekod = (int)$row[11];
        $this->typkomunikacepopis = DBUtils::getSqlValue($this->database, 'SELECT Popis FROM S_CommType WHERE Kod = ' . (int)$row[11] );
        $this->hostitel = $row[12];
        $this->port = $row[13];
        
        $this->comport = $row[14];
        $this->baudrate = DBUtils::getSqlValue($this->database, 'SELECT Popis FROM S_BaudRate WHERE Kod = ' . (int)$row[15] );
        $this->parita = DBUtils::getSqlValue($this->database, 'SELECT Popis FROM S_Parity WHERE Kod = ' . (int)$row[16] );
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function nactiStavTerminalu()
    {
        $this->stav = true;
        $rowStav = $this ->nactiStavTerminaluDB();
        if (!$rowStav) //return false;
            $this->stav = false;
        
        $this->casuspesnehonacteni = $this ->database ->stringToTimestamp($rowStav[1]);
        $this->casuspesnehonastaveni = $this ->database ->stringToTimestamp($rowStav[2]);
        $this->casuspesnehospojeni = $this ->database ->stringToTimestamp($rowStav[3]);
        
        $this->pocetneuspesnychnacteni = (int)$rowStav[4];
        $this->pocetneuspesnychnastaveni = (int)$rowStav[5];
        $this->pocetneuspesnychspojeni = (int)$rowStav[6];
        
        $this->verzefirmware = $rowStav[7];
        $this->seriovecislo = $rowStav[8];
        $this->macadresa = $rowStav[9];
        
        return true;
    }

    /**
     * Spusti agenta terminalu
     * @param string $action typ ulohy agenta terminalu ktery se ma provest
     * @return bool
     * @throws Exception
     */
    public function spustAgentaTerminalu($action)
    {
        switch ($action) {
            case 'nastaveni':
                $uloha = 1;
                break;
            case 'test':
                $uloha = 4;
                break;    
            default:
                throw new Exception('Chyba v metode TerminalyController:spustAgentaTerminalu - pro hodnotu ' . StringUtils::htmlTextConvert($action) . ' neni definovana uloha agenta.');
        }

        require_once('adsapp/agentmanager.php');
        if ($this->kodAplikace != '') {
            //return \ExecuteAgent_Internal($this->kodAplikace.'_Agent_Terminal.exe', $uloha.' -terminals:"' . $this->kod . '"');
            try {
                AgentManager::executeADS($this->kodAplikace . '_Agent_Terminal.exe', $uloha . ' -terminals:"' . $this->kod . '"');
                return true;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                return false;
            }
        }
        else {
            return FALSE;
        }
    }

    /**
     * @return array|bool
     * @throws Exception
     */
    private function nactiDetailTerminalu()
    {
        if (!$this->kod) { return false; }

        $params = $this->database->createParameters();

        $sql =
            'SELECT Terminal.Kod, Terminal.Popis, Terminal.KodUrceni, S_UrceniTerminalu.Popis, '. // 0 - 3            
            'Terminal.KodUmisteni, Umisteni.Popis, Rozdeleni.Popis, ' . $this->database->castWideMemo('Terminal.Poznamka') . ', '. // 4 - 7
            'TypRozdeleni.Popis, Terminal.Driver, Terminal.C_TerminalType, Terminal.C_CommType, '. //8 - 11
            'Terminal.C_Host, Terminal.C_Port, Terminal.C_ComPort, Terminal.C_BaudRate, Terminal.C_Parity '. //12 - 16
                
            ' FROM Terminal JOIN S_UrceniTerminalu ON (S_UrceniTerminalu.Kod = Terminal.KodUrceni) '.
                'LEFT JOIN Umisteni ON (Umisteni.Kod = Terminal.KodUmisteni) '.
                'LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = Terminal.KodTypuRozdeleni AND Rozdeleni.Kod = Terminal.KodRozdeleni) ' .
                'LEFT JOIN TypRozdeleni ON (TypRozdeleni.Kod = Terminal.KodTypuRozdeleni) '.
            " WHERE (Terminal.kod = " . $params->addParameter($this->kod) . ") ";

        $cursor = $this->database->select($sql, $params);
                
        if ($row = $cursor->fetch()) {
            return $row;
        } else {
            throw new Exception('Terminal se zadanym kodem neexistuje!');
        }
    }

    /**
     * @return array|bool
     * @throws Exception
     */
    private function nactiStavTerminaluDB()
    {
        if (!$this->kod) { return false; }

        $params = $this->database->createParameters();

        $sql =
            'SELECT  KodTerminalu, CasPoslednihoNacteni, CasPoslednihoNastaveni, CasPoslednihoTestuSpojeni, '. //0 - 3
            'NeuspesnychNacteni, NeuspesnychNastaveni, NeuspesnychTestuSpojeni, INF_VerzeFirmware, INF_SerioveCislo, '. //4 - 8
            'INF_MAC_Adresa, INF_PocetOperaci, INF_PocetOsob, INF_PocetSalda, INF_PocetDatovychZaznamu '. //9 - 13
            'FROM StavTerminalu'.
            " WHERE (KodTerminalu = " . $params->addParameter($this->kod) . ") ";

        if ($cursor = $this->database->select($sql, $params)) {
            if ($row = $cursor->fetch()) {
                return $row;
            } else {
                return false;
            }
        }

        return false;
    }

    /**
     * @throws Exception
     */
    private function zjistiAplikaci()
    {
        $sql = "SELECT applicationID FROM SYS_DDEF WHERE applicationID = 'ADS' OR applicationID = 'APS' OR applicationID = 'ZAKAZKY' ORDER BY applicationID";
        
        $cursor = $this->database->select($sql);
        
        if ($row = $cursor->fetch()) {        
            $this->kodAplikace = $row[0];
        }
        else {
            $this->kodAplikace = '';
        }
    }
}