<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 6.4.2018
 */

namespace Ron\RonPortal\Benefity;


use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\MessageManager;
use Ron\RonPortal\ComboBoxOsoba;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\StringUtils;

require_once "ron/browser.php";
require_once 'ron/httputils.php';
require_once "adsapp/ronportalfilter.php";


/**
 * Class BenefityFiltrForm
 * @package Ron\RonPortal\Benefity
 */
class BenefityFiltrForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby $pravo_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

    public $osoba;

    public $timestamp;

    public $filtrDlePrava;

    /**
     * BenefityFiltrForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     * @param $prava_rezervovat
     * @param $prava_schvalit
     * @param $prava_vydat
     * @param $prava_vznik
     */
    public function __construct($database, $action, $useGet, $prava_rezervovat, $prava_schvalit, $prava_vydat, $prava_vznik)
    {
        parent::__construct($database, $action, $useGet = false);

        $this->pravo_rezervovat = $prava_rezervovat;
        $this->pravo_schvalit = $prava_schvalit;
        $this->pravo_vydat = $prava_vydat;
        $this->pravo_vznik = $prava_vznik;

        $this->skrytTlacitkoNastavit = true;
        $this->f_formaction = '?menuid=benefity';
        $this->f_classname = 'filtercontainer leftcaptions';


        if($_SESSION['omezeniBenefitu'] == ""){
            $_SESSION['omezeniBenefitu'] = 0;
        }

        if (isset($_POST['datumFiltruBenefitu'])){
            try {
                $this->timestamp = DateUtils::dateStringToTimestamp($_POST['datumFiltruBenefitu']);
            } catch(\Exception $e) {
                MessageManager::addErrorText( 'Bad_time_format' );
                $this->timestamp = time();
            }
        } else {

            if ($_SESSION['datumFiltruBenefitu']) {
                $this->timestamp = $_SESSION['datumFiltruBenefitu'];
            } else {
                $this->timestamp = time();
            }
        }
    }


    function renderSchvaleniRezervaceCerpani()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        $radio = '';
        $radio .= "<input type='radio' name='omezeniBenefitu' value='0' " . ($_SESSION['omezeniBenefitu'] == 0 ? 'checked="checked"' : "") . "onclick='submit();'>" . "&nbsp;&nbsp;" . \Ron\ronText('vse');
        $radio .= "&nbsp;&nbsp;&nbsp;";
        echo $radio;
        echo '</span>';

        $radio = "<input type='radio' name='omezeniBenefitu' value='1' " . ($_SESSION['omezeniBenefitu'] == 1 ? 'checked="checked"' : "") . "onclick='submit();'>" . "&nbsp;&nbsp;" . \Ron\ronText('jenRezervaci');
        $radio .= "&nbsp;&nbsp;&nbsp;";
        $radio .= "<input type='radio' name='omezeniBenefitu' value='2' " . ($_SESSION['omezeniBenefitu'] == 2 ? 'checked="checked"' : "") . "onclick='submit();'>" . "&nbsp;&nbsp;" . \Ron\ronText('jenSchvaleni');
        $radio .= "&nbsp;&nbsp;&nbsp;";
        $radio .= "<input type='radio' name='omezeniBenefitu' value='3' " . ($_SESSION['omezeniBenefitu'] == 3 ? 'checked="checked"' : "") . "onclick='submit();'>" . "&nbsp;&nbsp;" . \Ron\ronText('jenCerpani');
        echo $radio;
        echo "&nbsp;&nbsp;&nbsp;";
        echo "&nbsp;&nbsp;&nbsp;";

        echo \Ron\ronText('k_datu');
        echo "&nbsp;&nbsp;";
        DateInput::renderDateSelect('datumFiltruBenefitu', date('j.n.Y', $this->timestamp), true);

        echo '</div>';
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderDruhPohybu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenDruh', \Ron\ronText('druh_pohybu'). '&nbsp;&nbsp;', $_SESSION['jenDruh'],
                'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT S_DruhPohybuBenefitu.Kod, S_DruhPohybuBenefitu.Popis 
              FROM PohybBenefitu LEFT JOIN S_DruhPohybuBenefitu ON (PohybBenefitu.DruhPohybu = S_DruhPohybuBenefitu.Kod) 
                WHERE PohybBenefitu.DruhPohybu IS NOT NULL 
              ORDER BY S_DruhPohybuBenefitu.Popis";

        $combo = new ComboBox('druh');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenDruh.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['druh']);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    function renderBenefit()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenBenefit', \Ron\ronText('benefit'). '&nbsp;&nbsp;', $_SESSION['jenBenefit'],
                'onClick="submit()"');
        echo '</span>';

        $prava = array($this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        $this->filtrDlePrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', time(), time());

        $sql =
            "SELECT DISTINCT Benefit.ID, Benefit.Popis 
              FROM Benefit Left JOIN PohybBenefitu ON (Benefit.ID = PohybBenefitu.IdBenefitu) LEFT JOIN PracovniPomer ON (PracovniPomer.RC = PohybBenefitu.RC)
                WHERE PohybBenefitu.IdBenefitu IS NOT NULL";

        if ($this->filtrDlePrava){
            $sql .= " AND " . $this->filtrDlePrava;
        }

        $sql .= "        
              ORDER BY Benefit.Popis";

        $combo = new ComboBox('filtr_benefit');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenBenefit.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['filtr_benefit']);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderTypBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenTypBenefitu', \Ron\ronText('typ_benefitu'). '&nbsp;&nbsp;', $_SESSION['jenTypBenefitu'],
                'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT TypBenefitu.Kod, TypBenefitu.Popis 
              FROM TypBenefitu Left JOIN Benefit ON (TypBenefitu.Kod = Benefit.TypBenefitu)
                WHERE Benefit.TypBenefitu IS NOT NULL 
              ORDER BY TypBenefitu.Popis";

        $combo = new ComboBox('filtr_typ_benefit');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypBenefitu.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['filtr_typ_benefit']);
        $combo->render();
        echo '</div>';
    }



    function renderOsobaBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenOsoba', \Ron\ronText('osoba_benefitu'). '&nbsp;&nbsp;', $_SESSION['jenOsoba'],
                'onClick="submit()"');
        echo '</span>';

        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('osoba');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->dataTypeComparison = false;
        $combo->htmlParams = 'onChange="if (jenOsoba.checked) submit();"';
        $combo->filtr = ' EXISTS (Select PohybBenefitu.RC From PohybBenefitu Where PohybBenefitu.RC = Osoba.RC)';
        $prava = array($this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        $combo->loadDataPP($this->database, $datumOd, $datumDo, $prava, false, isset($_SESSION['osoba']) ? $_SESSION['osoba'] : "", false);
        $combo->render();

        echo '</div>';
    }


    function renderFiltryBenefitu()
    {
        echo '<DIV>';
        echo '<span class="filtercaption">';

            GuiUtils::renderCheckbox('jenNeschvaleneBenefity', \Ron\ronText('neschvalene_benefity') . '&nbsp;&nbsp;', $_SESSION['jenNeschvaleneBenefity'],
                'onClick="submit()"');

            GuiUtils::renderCheckbox('jenNecerpaneBenefity', \Ron\ronText('necerpane_benefity') . '&nbsp;&nbsp;', $_SESSION['jenNecerpaneBenefity'],
                'onClick="submit()"');


        echo "</span>";
        echo '</DIV>';
    }


    function renderVznikBodu()
    {
        echo '<DIV class="filterrow">';
        echo    '<SPAN class="filtercaption">';
        echo        '<br />';
        echo        "<a href='?menuid=benefity&action=vznikbodu'>";
        echo            \Ron\ronText('vznik');
        echo        '</a>';
        echo    '</SPAN>';
        echo "</DIV>";
    }


    function renderRezervaceBenefitu()
    {
        echo '<DIV class="filterrow">';
        echo    '<SPAN class="filtercaption">';
        echo        "<a href='?menuid=benefity&action=rezervacebodu'>";
        echo            \Ron\ronText('rezervace_benefitu');
        echo        '</a>';
        echo    '</SPAN>';
        echo "</DIV>";
    }


    function renderVytvoreniSablonyBenefitu()
    {
        echo "<br />";
        echo '<DIV class="filterrow">';
        echo    '<SPAN class="filtercaption">';
        echo        "<a href='?menuid=benefity&action=browsersablonybenefitu'>";
        echo            \Ron\ronText('sablony_benefitu');
        echo        '</a>';
        echo    '</SPAN>';
        echo "</DIV>";
    }


    function renderOdkazy()
    {
        echo "<br />";

        echo '<DIV class="filterrow">';
        echo '<ul class="linkslist">';

        if ($this->pravo_rezervovat->jsouNejakePrava()) {
            echo "<li class='selectedlink'><a href='?menuid=benefity&action=rezervacebodu'>" . \Ron\ronText('rezervace_benefitu') . "</a></li>";
        }

        if ($this->pravo_vznik->jsouNejakePrava()) {
            echo "<li class='selectedlink'><a href='?menuid=benefity&action=vznikbodu'>" . \Ron\ronText('vznik') . "</a></li>";
        }

        if ($this->pravo_rezervovat->jsouNejakePrava()) {
            echo "<li class='selectedlink'><a href='?menuid=benefity&action=browsersablonybenefitu'>" . \Ron\ronText('sablony_benefitu') . "</a></li>";
        }

        echo "</ul>";
        echo "</DIV>";
    }

    private function loadOmezeniBenefitu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['omezeniBenefitu'] = $_POST['omezeniBenefitu'];
        }
    }


    function loadDatumFiltru()
    {
        if (isset($_POST['adsfilterform'])) {
            try{
                $_SESSION['datumFiltruBenefitu'] = DateUtils::dateStringToTimestamp($_POST['datumFiltruBenefitu']);
            } catch (\Exception $ex){
                $_SESSION['datumFiltruBenefitu'] = time();
            }
        }
    }

    function loadDruhPohybu()
    {
        if (isset($_GET['druhpohybubenefitu'])) {
            $_SESSION['jenDruh'] = isset($_GET['druhpohybubenefitu']);
            $_SESSION['druh'] = $_GET['druhpohybubenefitu'];
        }

        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenDruh'] = isset($_POST['jenDruh']);
            $_SESSION['druh'] = $_POST['druh'];
        }
    }

    function loadTypBenefitu()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenTypBenefitu'] = isset($_POST['jenTypBenefitu']);
            $_SESSION['filtr_typ_benefit'] = $_POST['filtr_typ_benefit'];
        }
    }

    function loadBenefit()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenBenefit'] = isset($_POST['jenBenefit']);
            $_SESSION['filtr_benefit'] = $_POST['filtr_benefit'];
        }
    }

    function loadOsobaBenefitu(){
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenOsoba'] = isset($_POST['jenOsoba']);
            $_SESSION['osoba'] = $_POST['osoba'];
        }
    }


    function loadNeschvaleneBenefitu(){
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenNeschvaleneBenefity'] = isset($_POST['jenNeschvaleneBenefity']);
        }
    }

    function loadNecerpaneBenefitu(){
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenNecerpaneBenefity'] = isset($_POST['jenNecerpaneBenefity']);
        }
    }


    function loadData()
    {
        $this->loadTypBenefitu();
        $this->loadDatumFiltru();
        $this->loadOmezeniBenefitu();
        $this->loadOsobaBenefitu();
        $this->loadBenefit();
        $this->loadDruhPohybu();
        $this->loadNecerpaneBenefitu();
        $this->loadNeschvaleneBenefitu();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        $this->renderSchvaleniRezervaceCerpani();
        $this->renderOsobaBenefitu();
        $this->renderTypBenefitu();
        $this->renderBenefit();
        $this->renderDruhPohybu();
        $this->renderFiltryBenefitu();


        $this->renderOdkazy();


    }
}


/**
 * Class DatumPosledniAktualizaceField
 * @package Ron\RonPortal\Benefity
 */
class DatumPosledniAktualizaceField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('posledni_aktualizace');
        $this->align = 'center';
        $this->sqlFieldName = "PB.Update_Timestamp";
        $this->ordering = 1;
        $this->sortSQL = "Update_Timestamp";
        $this->sortSQLDesc = "Update_Timestamp DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[19]) {
            $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[19])) . ' (' . StringUtils::htmlTextConvert($rowData[20]) . ') ' . "</div>" ;
        }
        return $result;
    }
}


/**
 * Class DatumVlozeniField
 * @package Ron\RonPortal\Benefity
 */
class DatumVlozeniField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('datum_osoba_vlozeni');
        $this->align = 'center';
        $this->sqlFieldName = "DatumVlozeni";
        $this->ordering = 1;
        $this->sortSQL = "Insert_Timestamp";
        $this->sortSQLDesc = "Insert_Timestamp DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[18]) {
            $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[18])) . ' (' . $rowData[17] . ') ' . "</div>" ;
        }
        return $result;
    }
}

/**
 * Class DruhBenefituField
 * @package Ron\RonPortal\Benefity
 */
class DruhBenefituField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('druh_pohybu_benefitu');
        $this->align = 'center';
        $this->sqlFieldName = "DruhPohybu";
        $this->ordering = 1;
        $this->sortSQL = "DruhPohybu";
        $this->sortSQLDesc = "DruhPohybu DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            //'<a href="?menuid=benefity&druhpohybubenefitu=' . HttpUtils::urlEncode($rowData[2]) . '">' .
            StringUtils::htmlTextConvert($rowData[21]) ;
            //'</a>';
    }
}

/**
 * Class DatumPlatnostiRezervaceField
 * @package Ron\RonPortal\Benefity
 */
class DatumPlatnostiRezervaceField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('datum_platnosti_rezervace');
        $this->align = 'right';
        $this->sqlFieldName = "DatumRezervaceDo";
        $this->ordering = 1;
        $this->sortSQL = "DatumOmezeniPlatnosti, DatumRezervaceDo";
        $this->sortSQLDesc = "DatumOmezeniPlatnosti DESC, DatumRezervaceDo DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($rowData[8] != "") {
            $result .= "<div title=''>" . \Ron\ronText('rezervace_do') . ": <strong>" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[8])) . "</strong></div>" ;
        } elseif ($rowData[15] != ""){
            $result .= "<div >" . \Ron\ronText('platnost') . ": <strong>" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[15])) . "</strong></div>" ;
        }
        return $result;
    }
}


/**
 * Class DatumSchvaleniField
 * @package Ron\RonPortal\Benefity
 */
class DatumSchvaleniField extends BrowserField
{
    /** @var PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var PravaNaOsoby $pravo_vznik_bodu */
    public $pravo_vznik_bodu;


    /**
     * DatumSchvaleniField constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_schvalit
     * @param PravaNaOsoby $pravo_vznik_bodu
     */
    function __construct($database, $pravo_schvalit, $pravo_vznik_bodu)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('datum_osoba_schvaleni');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sqlFieldName = "DatumSchvaleni";
        $this->sortSQL = "DatumSchvaleni";
        $this->sortSQLDesc = "DatumSchvaleni DESC";

        $this->pravo_schvalit = $pravo_schvalit;
        $this->pravo_vznik_bodu = $pravo_vznik_bodu;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        /*
         * "PB.ID, PB.RC, DruhPohybu, DruhZdroje, IdBenefitu, DatumVlozeni, DatumSchvaleni, Schvalil, " .        //0..7
            "DatumRezervaceDo, DatumCerpani, Cerpani_Username, PocetBodu, PocetBenefitu, TypDoplatku, " .       //8..13
            "PB.DoplatekCastka, DatumOmezeniPlatnosti, " . $database->castWideMemo('PB.Poznamka') .               //14..16
        */

        if (!($rowData[2] == "31" || $rowData[2] == "51"))
        {
            $zobrazitOdkaz = true;
            //Odkaz pro splneni pokud je platnostDo nebo RezervaceDo větší než akt. datum
            if ($rowData[8] != ""){
                $platnostDo = $this->database->stringToTimestamp($rowData[8]);
                $konecDne = DateUtils::incDayStart($platnostDo)-1;
                $zobrazitOdkaz = (($konecDne >= time()) ? true : false);
            } elseif ($rowData[15] != "") {
                $platnostDo = $this->database->stringToTimestamp($rowData[15]);
                $konecDne = DateUtils::incDayStart($platnostDo)-1;
                $zobrazitOdkaz = (($konecDne >= time()) ? true : false);
            }

            if ($rowData[6] != "") {
                $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[6])) . ' (' . $rowData[7] . ') ' . "</div>";
            } else {
                if ($this->pravo_schvalit->lzeZobrazitRC($rowData[1]) || ($this->pravo_vznik_bodu->lzeZobrazitRC($rowData[1]) && $rowData[2] == "11")) {
                    if ($zobrazitOdkaz) {
                        $result = GuiUtils::changeLocationQuest("<strong>" . \Ron\ronText('schvalit') . "</strong>", \Ron\ronText('vlozit_schvaleni'),
                            ' ?menuid=benefity&action=schvalenirezervace&idpohybubenefitu=' . $rowData[0]);
                    }
                }
            }
        }
        return $result;
    }
}


/**
 * Class DatumCerpaniField
 * @package Ron\RonPortal\Benefity
 */
class DatumCerpaniField extends BrowserField
{

    /** @var PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /**
     * DatumCerpaniField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $pravo
     */
    function __construct($database, $pravo)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('datum_osoba_cerpani');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sqlFieldName = "DatumCerpani";
        $this->pravo_vydat = $pravo;
        $this->sortSQL = "DatumSchvaleni";
        $this->sortSQLDesc = "DatumSchvaleni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if (!($rowData[2] == "31" || $rowData[2] == "51" || $rowData[2] == "11"))
        {
            $zobrazitOdkaz = true;
            //Odkaz pro splneni pokud je platnostDo nebo RezervaceDo větší než akt. datum
             if ($rowData[8] != ""){
                $platnostDo = $this->database->stringToTimestamp($rowData[8]);
                $konecDne = DateUtils::incDayStart($platnostDo)-1;
                $zobrazitOdkaz = (($konecDne >= time()) ? true : false);
            } elseif ($rowData[15] != "") {
                $platnostDo = $this->database->stringToTimestamp($rowData[15]);
                $konecDne = DateUtils::incDayStart($platnostDo)-1;
                $zobrazitOdkaz = (($konecDne >= time()) ? true : false);
            }


            if ($rowData[9] != "") {
                $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[9])) . ' (' . $rowData[10] . ') ' . "</div>";
            } else {

                $zobrazitDanyTyp = true;
                if (trim($this->pravo_vydat->priznak3) != '') {
                    $povoleneTypyBenefitu = explode(';', $this->pravo_vydat->priznak3);
                    $zobrazitDanyTyp = in_array($rowData[27], $povoleneTypyBenefitu);
                }

                if ($this->pravo_vydat->lzeZobrazitRC($rowData[1]) && $zobrazitDanyTyp) {
                    if ($zobrazitOdkaz) {
                        $result = GuiUtils::changeLocationQuest("<strong>" . \Ron\ronText('vydat') . "</strong>", \Ron\ronText('vlozit_cerpani'),
                            ' ?menuid=benefity&action=cerpanirezervace&idpohybubenefitu=' . $rowData[0]);
                    }
                }
            }
        }
        return $result;
    }
}


/**
 * Class PopisBenefituField
 * @package Ron\RonPortal\Benefity
 */
class PopisBenefituField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('benefit_popis');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "Insert_Timestamp";
        $this->sortSQLDesc = "Insert_Timestamp DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        switch ($rowData[2])
        {
            case '11':
                $result .= "<div >" . \Ron\ronText('vznik') . "</div>" ;
                break;

            case '31';
                $result .= "<div >" . \Ron\ronText('zanik') . "</div>" ;
                break;

            case '21':
            case '41':
                $result .= "<div >" . StringUtils::htmlTextConvert($rowData[23]) . ' (' . $rowData[25] . ') ' . "</div>" ;
                break;

            case '51':
                $result .= "<div >" . \Ron\ronText('zanik_rezervace_bodu') . ' (' . $rowData[23] . ') ' . "</div>" ;
                break;

        }

        return $result;
    }
}


/**
 * Class SchvaleniBenefituField
 * @package Ron\RonPortal\Benefity
 */
class StavBenefituField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('stav_benefitu');
        $this->align = 'left';
        $this->ordering = 0;
        $this->sqlFieldName = "DruhPohybu";
        $this->sortSQL = "DatumSchvaleni";
        $this->sortSQLDesc = "DatumSchvaleni DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

//        "PB.ID, PB.RC, DruhPohybu, DruhZdroje, IdBenefitu, DatumVlozeni, DatumSchvaleni, Schvalil, " .        //0..7
//        "DatumRezervaceDo, DatumCerpani, Cerpani_Username, PocetBodu, PocetBenefitu, TypDoplatku, " .       //8..13


        $result .= "<a href='?menuid=benefity&action=detailpohybu&idpohybubenefitu=$rowData[0]'>";
        switch ($rowData[2]) {

            case '11':
                if ($rowData[6]) {
                    $result .= "<div >" . \Ron\ronText('vznik_bodu') . " (" . $rowData[26] . ")" . "</div>";
                } else {
                    $result .= "<div ><strong>" . \Ron\ronText('vznik') . " (" . \Ron\ronText('neschvaleno') . ")" . "</strong></div>";
                }
                break;

            case '31';
                $result .= "<div >" . \Ron\ronText('zanik_bodu') . " (" . $rowData[26] .  ")" . "</div>" ;
                break;

            case '21':
            case '41':
                if ($rowData[6]) {

                    if ($rowData[9]) {
                        $result .= "<div >" . \Ron\ronText('cerpano') . " - " . StringUtils::htmlTextConvert($rowData[23]) . ' (' . StringUtils::htmlTextConvert($rowData[25]) . ') ' . "</div>";
                    } else {
                        $result .= "<div >" . \Ron\ronText('necerpano') . " - " . StringUtils::htmlTextConvert($rowData[23]) . ' (' . StringUtils::htmlTextConvert($rowData[25]) . ') ' . "</div>";
                    }

                } else {
                    $result .= "<div ><strong>" . StringUtils::htmlTextConvert($rowData[23]) . " (" . \Ron\ronText('neschvaleno') . ")" . "</strong></div>";
                }
                break;

            case '51':
                $title = StringUtils::htmlTextConvert($rowData[23])  . ' (' . StringUtils::htmlTextConvert($rowData[25]) . ')';
                $result .= "<div title='$title' >" . \Ron\ronText('zanik_rezervace') . " (" . $rowData[26] .  ")" . "</div>" ;
                break;
        }

        $result .= "</a>";

        return $result;
    }
}


/**
 * Class BenefityBrowser
 * @package Ron\RonPortal\Benefity
 */
class BenefityBrowser extends Browser
{

    /** @var  PravaNaOsoby $prava_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $prava_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

    /** @var  BenefityFiltrForm $filtrForm */
    public $filtrForm;


    /**
     * BenefityBrowser constructor.
     * @param \Ron\DB\Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database);

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');


        if ($this->pravo_schvalit->prava == 1 || $this->pravo_rezervovat->prava == 1 || $this->pravo_vznik->prava == 1 || $this->pravo_vydat->prava == 1){
            $this->title = \Ron\ronText('prehled_pohybu_benefitu');
        } else {
            $this->title = \Ron\ronText('prehled_pohybu_vlastnich_benefitu');
        }

        $this->code = 'browserbenefity';
        $this->tableId = 'browserbenefity';
        $this->browserUrl = "?menuid=benefity";

        $this->sqlSelectPart =
            "PB.ID, PB.RC, DruhPohybu, DruhZdroje, IdBenefitu, DatumVlozeni, DatumSchvaleni, Schvalil, " .        //0..7
            "DatumRezervaceDo, DatumCerpani, Cerpani_Username, PocetBodu, PocetBenefitu, TypDoplatku, " .       //8..13
            "PB.DoplatekCastka, DatumOmezeniPlatnosti, " . $database->castWideMemo('PB.Poznamka') .               //14..16
            ", PB.Insert_Username, PB.Insert_Timestamp, PB.Update_Timestamp, PB.Update_Username, " .                //17..20
            "S_DruhPohybuBenefitu.Popis, COALESCE(Osoba.Jmeno,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Prijmeni, '') OsobaBenefitu, " .    //21..22
            "Benefit.Popis, PracovniPomer.ID, TypBenefitu.Popis, PB.PocetBodu, TypBenefitu.Kod";        //23..27

        $this->sqlFromPart =
            "PohybBenefitu PB 
                LEFT JOIN S_DruhPohybuBenefitu ON (S_DruhPohybuBenefitu.Kod = PB.DruhPohybu) 
                 LEFT JOIN Osoba ON (PB.RC = Osoba.RC) 
                  LEFT JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) 
                   LEFT JOIN Benefit ON (Benefit.ID = PB.IdBenefitu)
                    LEFT JOIN TypBenefitu ON  (TypBenefitu.Kod = Benefit.TypBenefitu)";


        $this->filtrForm = new BenefityFiltrForm($this->database, null, false, $this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        $this->filtrForm->loadData();


        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'PB.ID',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('ID') . "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'DatumVlozeni',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('datum_vlozeni'), BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'date']);

        if ($this->pravo_rezervovat->prava != 1 || $this->pravo_vydat->prava != 1 || $this->pravo_vznik->prava != 1 || $this->pravo_schvalit->prava != 1) {
            //Pokud má právo pouze na sebe, pak se mu zobrazují pouze vlastní a pole osoba benefitu nemá smysl
            $this->addField([BROWSER_FIELD_SQL_INDEX => 22, BROWSER_FIELD_SQL_NAME => 'OsobaBenefitu',
                BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('osoba_benefitu'), BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'tucne']);
        }

        $this->addField(new StavBenefituField($this->database));

        $this->addField(new DruhBenefituField($this->database));

//        $this->addField([BROWSER_FIELD_SQL_INDEX => 21, BROWSER_FIELD_SQL_NAME => 'S_DruhPohybuBenefitu.Popis',
//            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('druh_pohybu_benefitu'), BROWSER_FIELD_ORDERING => 1]);

//        $this->addField(new PopisBenefituField($this->database));

        $this->addField(new DatumPlatnostiRezervaceField($this->database));

        $this->addField(new DatumVlozeniField($this->database));
        $this->defaultSort = -count($this->fields);

        $this->addField(new DatumSchvaleniField($this->database, $this->pravo_schvalit, $this->pravo_vznik));

        $this->addField(new DatumCerpaniField($this->database, $this->pravo_vydat));

        $this->addField(new DatumPosledniAktualizaceField($this->database));

    }

    function renderFilter()
    {
        parent::renderFilter();
        $this->filtrForm->render();
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        $rezerva = $this->pravo_rezervovat->lzeZobrazitRC($values[1]);
        $schvaleni = $this->pravo_schvalit->lzeZobrazitRC($values[1]);
        $vydat = $this->pravo_vydat->lzeZobrazitRC($values[1]);
        $vznik = $this->pravo_vznik->lzeZobrazitRC($values[1]);

        if ($rezerva || $schvaleni || $vydat || $vznik){

            //Vytvářím seznam názvů benefitů oddělených ;
            $upresneni = $values[23] != "" ? "ID pohybu: " . $values[0] . " - " . $values[23] : "";
            if ($upresneni == ""){
                $upresneni = $values[11] != "" ? "ID pohybu: " . $values[0] . " - " . $values[21] . " (" . $values[11] . ")" : "";
            }

            $_SESSION['seznambenefitu'] .=  ($upresneni ? $upresneni : "" ). ';;';

            if ($_SESSION['pocetbenefitufilru'] != ''){
                $_SESSION['pocetbenefitufilru'] .= ',';
            }
            $_SESSION['pocetbenefitufilru'] .= $values[0];

            return true;
        }
        return false;
    }



    /**
     * @return string
     */
    function getSQLFilter()
    {
        $result = "";
        $params = $this ->database ->createParameters();
        $datum = ($_SESSION['datumFiltruBenefitu']);

        $result .= "PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = PB.RC) ";

        if ($_SESSION['jenDruh']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= "PB.DruhPohybu = " . $params->addParameter($_SESSION['druh']);
        }

        if ($_SESSION['jenBenefit']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " PB.idBenefitu = " . $params->addParameter($_SESSION['filtr_benefit']);
        }

        if ($_SESSION['jenTypBenefitu']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " TypBenefitu.Kod = " . $params->addParameter($_SESSION['filtr_typ_benefit']);
        }

        if ($_SESSION['jenOsoba']){
            if ($result != ''){
                $result .= ' AND ';
            }

            $result .= "PracovniPomer.ID = " . $params->addParameter($_SESSION['osoba']);
        }

        if ($_SESSION['jenNeschvaleneBenefity']){
            if ($result != ''){
                $result .= ' AND ';
            }

            $result .= "( PB.DatumSchvaleni IS NULL AND PB.DruhPohybu IN ('11', '21', '41') )";
        }

        if ($_SESSION['jenNecerpaneBenefity']){
            if ($result != ''){
                $result .= ' AND ';
            }

            $result .= "( PB.DatumSchvaleni IS NOT NULL AND PB.DatumCerpani IS NULL AND PB.DruhPohybu IN ('21', '41'))";
        }


        if ($_SESSION['omezeniBenefitu'] == 1) {
            // Jen rezervace
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= "( PB.DruhPohybu IN ('41'))";
            $result .= " AND ((PB.DatumRezervaceDo IS NOT NULL) AND (PB.DatumRezervaceDo IS NOT NULL AND PB.DatumRezervaceDo <= " . $params->addParameter($this->database->timestampToParam($datum)) . "))";


        }  elseif ($_SESSION['omezeniBenefitu'] == 2){
            //Jen schválené
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " (PB.DatumSchvaleni IS NOT NULL AND PB.DatumSchvaleni <= " . $params->addParameter($this->database->timestampToParam($datum)) . "";
            $result .= " AND PB.DruhPohybu NOT IN ('51') )";


        } elseif ($_SESSION['omezeniBenefitu'] == 3){
            //Jen čerpání
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " (PB.DatumCerpani <= " . $params->addParameter($this->database->timestampToParam($datum)) . " )";
        }


        return new SqlStatement($result, $params);
    }
}