<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 6.11.2018
 */

namespace Ron\RonPortal\Benefity;


use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;


require_once "ron/browser.php";
require_once 'ron/httputils.php';
require_once "adsapp/ronportalfilter.php";

/**
 * Class SablonyBenefituFiltrForm
 * @package Ron\RonPortal\Benefity
 */
class SablonyBenefituFiltrForm extends RonPortalFilterForm
{

    /** @var  PravaNaOsoby $pravo_sablony */
    public $pravo_sablony;

    public $jedenDen;
    public $kalendarniTyden;
    public $kalendarniMesic;
    public $kalendarniKvartal;
    public $kalendarniPulrok;
    public $kalendarniRok;


    /**
     * SablonyBenefituFiltrForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $action, $useGet = false)
    {
        parent::__construct($database, $action, $useGet);

//        $this->pravo_sablony = $pravo_sablony;

        $this->skrytTlacitkoNastavit = true;
        $this->f_formaction = '?menuid=benefity&action=browsersablonybenefitu';
        $this->f_classname = 'filtercontainer leftcaptions';


        $this->jedenDen = new Option(1,\Ron\ronText('den'));
        $this->kalendarniTyden = new Option(2,\Ron\ronText('kalendarni_tyden'));
        $this->kalendarniMesic = new Option(3,\Ron\ronText('kalendarni_mesic'));
        $this->kalendarniKvartal = new Option(4,\Ron\ronText('kalendarni_kvartal'));
        $this->kalendarniPulrok = new Option(5,\Ron\ronText('kalendarni_pulrok'));
        $this->kalendarniRok = new Option(6,\Ron\ronText('kalendarni_rok'));
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderTypBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenTypBenefituSablony', \Ron\ronText('typ_benefitu'). '&nbsp;&nbsp;', $_SESSION['jenTypBenefituSablony'],
                'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT TypBenefitu.Kod, TypBenefitu.Popis 
              FROM TypBenefitu Left JOIN Benefit ON (TypBenefitu.Kod = Benefit.TypBenefitu) LEFT JOIN SablonaRezervaceBenefitu ON (Benefit.ID = SablonaRezervaceBenefitu.IdBenefitu)
                WHERE SablonaRezervaceBenefitu.IdBenefitu IS NOT NULL 
              ORDER BY TypBenefitu.Popis";

        $combo = new ComboBox('filtr_typ_benefit_sablony');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypBenefituSablony.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['filtr_typ_benefit_sablony']);
        $combo->render();
        echo '</div>';
    }


    function renderTypCasoveJednotkySablonyRezervaceBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenTypCasoveJednotkyRezervace', \Ron\ronText('pouze_casova_jednotka_opakovani'). '&nbsp;&nbsp;', $_SESSION['jenTypCasoveJednotkyRezervace'],
                'onClick="submit()"');
        echo '</span>';

        $combo = new ComboBox('filtr_typ_casove_jednotky_sablony', [$this->jedenDen, $this->kalendarniTyden, $this->kalendarniMesic, $this->kalendarniKvartal, $this->kalendarniPulrok, $this->kalendarniRok]);
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypCasoveJednotkyRezervace.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->selectedOption = $combo->findOption((int)$_SESSION['filtr_typ_casove_jednotky_sablony'] != '' ? (int)$_SESSION['filtr_typ_casove_jednotky_sablony'] : 1);
        $combo->render();
        echo '</div>';
    }


    function renderVytvoreniSablonyBenefitu()
    {
        echo "<br />";
        echo '<div class="filterrow">';

        echo '<ul class="linkslist">';
        echo "<li class='selectedlink'><a href='?menuid=benefity'>" . \Ron\ronText('zpet_pohyby') . "</a></li>";
        echo "<li class='selectedlink'><a href='?menuid=benefity&action=vytvoritsablonurezervace'>" . \Ron\ronText('vytvorit_sablonu') . "</a></li>";
        echo "</ul>";

        echo "</div>";
    }



    function loadTypBenefitu()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenTypBenefituSablony'] = isset($_POST['jenTypBenefituSablony']);
            $_SESSION['filtr_typ_benefit_sablony'] = $_POST['filtr_typ_benefit_sablony'];
        }
    }



    function loadTypCasoveJednotkySablonyRezervaceBenefitu()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenTypCasoveJednotkyRezervace'] = isset($_POST['jenTypCasoveJednotkyRezervace']);
            $_SESSION['filtr_typ_casove_jednotky_sablony'] = $_POST['filtr_typ_casove_jednotky_sablony'];
        }
    }


    /**
     * @throws \Exception
     */
    function loadData()
    {
        $this->loadTypCasoveJednotkySablonyRezervaceBenefitu();
        $this->loadTypBenefitu();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        parent::renderFilters();

        $this->renderTypBenefitu();
        $this->renderTypCasoveJednotkySablonyRezervaceBenefitu();
        $this->renderVytvoreniSablonyBenefitu();
    }
}


/**
 * Class DatumVlozeniField
 * @package Ron\RonPortal\Benefity
 */
class DatumVlozeniSablonyField extends BrowserField
{

    /**
     * DatumVlozeniSablonyField constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('datum_osoba_vlozeni');
        $this->align = 'center';
        $this->sqlFieldName = "Insert_UserName";
        $this->ordering = 1;
        $this->sortSQL = "Insert_Timestamp";
        $this->sortSQLDesc = "Insert_Timestamp DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[12]) {
            $result .= "<div >" . $rowData[11] . ' (' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[12])) . ') ' . "</div>" ;
        }
        return $result;
    }
}


/**
 * Class DatumVlozeniSablonyField
 * @package Ron\RonPortal\Benefity
 */
class DatumAktualizaceSablonyField extends BrowserField
{

    /**
     * DatumAktualizaceSablonyField constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('posledni_aktualizace');
        $this->align = 'center';
        $this->sqlFieldName = "Update_UserName";
        $this->ordering = 1;
        $this->sortSQL = "Update_UserName";
        $this->sortSQLDesc = "Update_UserName DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[13]) {
            $result .= "<div >" . $rowData[13] . ' (' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[14])) . ') ' . "</div>" ;
        }
        return $result;
    }
}


/**
 * Class CasovaJednotkaSablonyField
 * @package Ron\RonPortal\Benefity
 */
class CasovaJednotkaSablonyField extends BrowserField
{
    /**
     * CasovaJednotkaSablonyField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('casova_jednotka');
        $this->align = 'center';
        $this->sqlFieldName = "CasovaJednotka";
        $this->ordering = 1;
        $this->sortSQL = "CasovaJednotka";
        $this->sortSQLDesc = "CasovaJednotka DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        
        switch ($rowData[9]){
            case '1':
                $result .= "<div >" .  \Ron\ronText('den'). "</div>";
                break;

            case '2':
                $result .= "<div >" .  \Ron\ronText('kalendarni_tyden'). "</div>";
                break;

            case '3':
                $result .= "<div >" .  \Ron\ronText('kalendarni_mesic'). "</div>";
                break;

            case '4':
                $result .= "<div >" .  \Ron\ronText('kalendarni_kvartal'). "</div>";
                break;

            case '5':
                $result .= "<div >" .  \Ron\ronText('kalendarni_pulrok'). "</div>";
                break;

            case '6':
                $result .= "<div >" .  \Ron\ronText('kalendarni_rok'). "</div>";
                break;

        }
        
        return $result;
    }
}

/**
 * Class SmazatSablonuField
 * @package Ron\RonPortal\Benefity
 */
class SmazatSablonuField extends BrowserField
{
    /**
     * CasovaJednotkaSablonyField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('odstraneni_sablony');
        $this->align = 'center';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $text = \Ron\ronText("odstranit_sablonu_info");

        //Momentálně na portále lze rušit pouze své šablony
        if ($rowData[1] == User::getRC()) {
            $result .= "<span class='rectangle confirmQuestion odkaz' data-ron-confirm-text='$text' 
                            data-ron-href='?menuid=benefity&action=odstranitsablonu&record=$rowData[0]' >" . \Ron\ronText('odstranit_sablonu') . "</span>";
        }
        return $result;
    }
}



/**
 * Class DetailSablonyField
 * @package Ron\RonPortal\Benefity
 */
class DetailSablonyField extends BrowserField
{
    /**
     * DetailSablonyField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct(Database $database = null, array $options = null)
    {
        parent::__construct($database, $options);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $result .= "<a href='?menuid=benefity&action=detailsablonyrezervace&sablona=$rowData[0]'>" . \Ron\ronText("detail_sablony") . "</a>";

        return $result;
    }
}


/**
 * Class BrowserSablonyBenefitu
 * @package Ron\RonPortal\Benefity
 */
class BrowserSablonyBenefitu extends Browser
{

    /** @var SablonyBenefituFiltrForm $filterForm */
    public $filterForm;

    /** @var PravaNaOsoby $pravo_rezervace */
    public $pravo_sablony;


    /**
     * BrowserSablonyBenefitu constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database);

        $this->title = \Ron\ronText('sablony_benefitu');
        $this->code = 'browsersablonybenefitu';
        $this->tableId = 'browsersablonybenefitu';
        $this->browserUrl = '?menuid=benefity&action=browsersablonybenefitu';

        $this->pravo_sablony = Prava::vratPrava('benefit_sablona');


        $this->sqlSelectPart =
            'SablonaRezervaceBenefitu.ID, SablonaRezervaceBenefitu.RC, DruhPohybu, IdBenefitu, Poradi, ' .                               //0..4
            'PocetBenefitu, PocetBodu, TypDoplatku, SablonaRezervaceBenefitu.DoplatekCastka, ' .                //5..8
            'CasovaJednotka, PocetCasJednotek, SablonaRezervaceBenefitu.Insert_UserName, SablonaRezervaceBenefitu.Insert_Timestamp, ' .                           //9..12
            'SablonaRezervaceBenefitu.Update_UserName, SablonaRezervaceBenefitu.Update_Timestamp, TypKontrolySablony, Benefit.Popis, S_AnoNe.Popis,' .              //13..17
            "COALESCE(Osoba.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno, '') OsobaBenefitu";                                                                      //18

        $this->sqlFromPart =
            "SablonaRezervaceBenefitu  
                LEFT JOIN Benefit ON (Benefit.ID = SablonaRezervaceBenefitu.IdBenefitu) 
                 LEFT JOIN S_AnoNe ON (SablonaRezervaceBenefitu.TypKontrolySablony = S_AnoNe.Kod)
                  LEFT JOIN Osoba ON (SablonaRezervaceBenefitu.RC = Osoba.RC)";


        $this->filterForm = new SablonyBenefituFiltrForm($database, NULL, FALSE);
        $this->filterForm->loadData();
        
        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'ID', BROWSER_FIELD_HEADER_TEXT => 'ID',
            BROWSER_FIELD_ORDERING => 1]);


        $this->addField([BROWSER_FIELD_SQL_INDEX => 16, BROWSER_FIELD_SQL_NAME => 'Benefit.Popis', BROWSER_FIELD_HEADER_TEXT => 'benefit_popis',
            BROWSER_FIELD_ORDERING => 'center']);

        if ($this->pravo_sablony->prava == 3) {
            $this->addField([BROWSER_FIELD_SQL_INDEX => 18, BROWSER_FIELD_SQL_NAME => 'OsobaBenefitu', BROWSER_FIELD_HEADER_TEXT => 'osoba_benefitu',
                BROWSER_FIELD_ORDERING => 'center']);
        }
        
        $this->addField([BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'PocetBenefitu', BROWSER_FIELD_HEADER_TEXT => 'pocet_benefitu',
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new DetailSablonyField($this->database));

        $this->addField(new CasovaJednotkaSablonyField($this->database));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_SQL_NAME => 'PocetCasJednotek', BROWSER_FIELD_HEADER_TEXT => 'opakovani',
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 17, BROWSER_FIELD_SQL_NAME => 'S_AnoNe.Popis', BROWSER_FIELD_HEADER_TEXT => 'povolit_vlozit_mene',
            BROWSER_FIELD_ORDERING => 'center', BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new SmazatSablonuField($this->database));

        $this->addField(new DatumVlozeniSablonyField($this->database));
        $this->defaultSort = -count($this->fields);

        $this->addField(new DatumAktualizaceSablonyField($this->database));
    }


    function renderFilter()
    {
        parent::renderFilter();
        $this->filterForm->render();
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        if ($this->pravo_sablony->lzeZobrazitRC($values[1])){
            return true;
        }

        return false;
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $result = "";
        $params = $this->database->createParameters();

        if ($_SESSION['jenTypBenefituSablony']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " Benefit.TypBenefitu = " . $params->addParameter($_SESSION['filtr_typ_benefit_sablony']);
        }


        if ($_SESSION['jenTypCasoveJednotkyRezervace']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= " SablonaRezervaceBenefitu.CasovaJednotka = " . $params->addParameter($_SESSION['filtr_typ_casove_jednotky_sablony']);
        }

        return new SqlStatement($result, $params);
    }
}





