<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 9.4.2018
 */

namespace Ron\RonPortal\Benefity;

use Ron\DB\Database;
use Ron\MessageManager;



/**
 * Class Benefit
 * @package Ron\RonPortal\Benefity
 */
class Benefit
{
    public $database;

    public $id;
    public $popis;
    public $platnostOd;
    public $platnostDo;
    public $typ;
    public $detailniPopis;
    public $pocetCelkem;
    public $neomezenyPocet;
    public $hodnotaVBodech;
    public $variabilniHodnotaVBodech;
    public $doplatekCastka;
    public $maximalniPocetJedenZamestnanec;
    public $neomezovatNaOsobu;
    public $jednotkaCerpaniMaxima;
    public $pocetJednotekProCerpani;
    public $idDokumentu;
    public $automatickeSchvaleni;
    public $automatickeSchvaleni_popis;
    public $jizRezervovano;


    /**
     * @param Database $database
     * @param $id
     * @return Benefit
     * @throws \Exception
     */
    public static function nactiUdajeBenefitu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT  
             ID, Popis, TypBenefitu, PlatnostOd, PlatnostDo, " . $database->castWideMemo('PopisDetail') .', ' .         //0..5
            "PocetCelkem, NeomezenyPocet, HodnotaVBodech, VariabilniHodnotaVBodech, DoplatekCastka, " .                     //6..10
            "MaxPocetJedenZam, NeomezenyPocetJedenZam, JednProCerpaniMaxPoctu, PocetJednotekProCerpani, idDokumentu, AutomatickeSchvaleni " .       //11..16

            "FROM Benefit " .

            "WHERE ID = " . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()){
            throw new \Exception(\Ron\ronText('benefit_neexistuje'));
        }

        $benefit = new Benefit();
        $benefit->id = $id;
        $benefit->nactiUdajeBenefituZRadku($database, $row);
        $cursor->close();
        return $benefit;
    }


    /**
     * @param Database $database
     * @param $row
     */
    public function nactiUdajeBenefituZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->popis = $row[1];
        $this->typ = $row[2];
        $this->platnostOd = $database->stringToTimestamp($row[3]);
        $this->platnostDo = $database->stringToTimestamp($row[4]);
        $this->detailniPopis = $row[5];
        $this->pocetCelkem = $row[6];
        $this->neomezenyPocet = $row[7];
        $this->hodnotaVBodech = $row[8];
        $this->variabilniHodnotaVBodech = $row[9];
        $this->doplatekCastka = $row[10];
        $this->maximalniPocetJedenZamestnanec = (int)$row[11];
        $this->neomezovatNaOsobu = $row[12];
        $this->jednotkaCerpaniMaxima = $row[13];
        $this->pocetJednotekProCerpani = $row[14];
        $this->idDokumentu = $row[15];
        $this->automatickeSchvaleni = $row[16];
        $this->automatickeSchvaleni_popis = $row[16] ? \Ron\ronText('Str_Yes') : \Ron\ronText('Str_No');
        //TODO kolik má osoba již rezervováno
//        $this->jizRezervovano = $row[17];

    }
}


/**
 * Class PohybBenefitu
 * @package Ron\RonPortal\Benefity
 */
class PohybBenefitu
{
    public $rc;

    public $typ;

    public $idPohybu;
    public $id_benefitu;
    public $idPP;
    public $druhPohybu;
    public $druhZdroje;
    public $schvalil;
    public $vydal;
    public $typDoplatku;
    public $doplatekCastka;

    public $datumCerpani;
    public $datumOmezeniPlatnosti;
    public $datumRezervaceDo;
    public $datumVlozeni;
    public $datumSchvaleni;
    public $pocetBodu;
    public $pocetBenefitu;
    public $popisPohybuBenefitu;
    public $typBenefitu;

    public $poznamka;
//    public $database;


    /**
     * Načte 1 záznam pohybu benefitu, který představuje rezervování/vydání benefitu nebo pohyb s body (vznik/zánik)
     * @param Database $database
     * @param $id
     * @return PohybBenefitu
     * @throws \Exception
     */
    public static function nactiPohybBenefitu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT ' .
            ' PB.ID, PB.RC, PB.DruhPohybu, PB.DruhZdroje, PB.IdBenefitu, PB.DatumVlozeni,' . // 0..5
            ' PB.DatumSchvaleni, PB.Schvalil, PB.DatumRezervaceDo, ' .  // 6..8
            ' PB.DatumCerpani, PB.Cerpani_Username, PB.PocetBodu, PB.PocetBenefitu, PB.TypDoplatku, PB.DoplatekCastka, ' .  // 9..14
            ' PB.DatumOmezeniPlatnosti, ' . $database->castWideMemo('PB.Poznamka') . ' , PB.Insert_Username, ' .            // 15..17
            ' PB.Insert_Timestamp, PB.Update_Timestamp, PB.Update_Username,' .  // 18..20
            ' PracovniPomer.ID, S_DruhPohybuBenefitu.Popis, TypBenefitu.Kod' .  // 21..23
            ' FROM PohybBenefitu PB' .
            ' LEFT JOIN PracovniPomer ON (PB.RC = PracovniPomer.RC)' .
            ' LEFT JOIN S_DruhPohybuBenefitu ON (S_DruhPohybuBenefitu.Kod = PB.DruhPohybu)' .
            ' LEFT JOIN Benefit ON (Benefit.ID = PB.IdBenefitu)' .
            ' LEFT JOIN TypBenefitu ON (TypBenefitu.Kod = Benefit.TypBenefitu)' .
            ' WHERE PB.ID = ' . $params->addParameter((int)$id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);
        if (!$row = $cursor->fetch()){
            throw new \Exception(\Ron\ronText('pohyb_benefitu_neexistuje'));
        }
        $cursor->close();


        $pohybBenefitu = new PohybBenefitu();
        $pohybBenefitu->nactiPohybyBenefituZRadku($database, $row);

        return $pohybBenefitu;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiPohybyBenefituZRadku($database, $row)
    {
        $this->idPohybu = $row[0];
        $this->rc = $row[1];
        $this->druhPohybu = $row[2];
        $this->druhZdroje = $row[3];
        // Pro body je hodnota "id_benefitu" NULL, jelikoz body nejsou napojeny na konkretni benefit
        $this->id_benefitu = $row[4];
        $this->datumVlozeni = $database->stringToTimestamp($row[5]);
        $this->datumSchvaleni = $database->stringToTimestamp($row[6]);
        $this->schvalil = $row[7];
        $this->datumRezervaceDo = $database->stringToTimestamp($row[8]);
        $this->datumCerpani = $database->stringToTimestamp($row[9]);
        $this->vydal = $row[10];
        $this->pocetBodu = $row[11];
        $this->pocetBenefitu = $row[12];
        $this->typDoplatku = $row[13];
        $this->doplatekCastka = (float)$row[14];
        $this->datumOmezeniPlatnosti = $database->stringToTimestamp($row[15]);
        $this->poznamka = $row[16];
        $this->idPP = $row[21];
        $this->popisPohybuBenefitu = $row[22];
        $this->typBenefitu = $row[23];
    }
}



/**
 * Class RezervaceBenefitu
 * @package Ron\RonPortal\Benefity
 */
class BenefityOsoby
{
    public $rc;
    public $vznikBoduCelkem = 0;
    public $cerpanoBodu = 0;
    public $rezervovanoBodu = 0;
    public $zanikRezervaceBodu = 0;
    public $zanikBodu = 0;
    public $zustatekBodu = 0;

    /**
     * @param Database $database
     * @param $rc
     * @throws \Ron\DB\DBException
     */
    public function nactiStavBoduOsoby($database, $rc)
    {
        $this->rc = $rc;

        $params = $database->createParameters();
        $sql = "SELECT PocetBodu, DatumOmezeniPlatnosti, DatumSchvaleni, PohybBenefitu.DruhPohybu, PohybBenefitu.ID
                FROM PohybBenefitu
                WHERE (RC = " . $params->addParameter($this->rc) . ") 
                  AND (   ((DruhPohybu IN ('41', '21', '51')) AND (DatumSchvaleni IS NOT NULL) ) OR (DruhPohybu IN ('31')) OR ((DruhPohybu IN ('11')) AND (DatumSchvaleni IS NOT NULL) 
                  AND (DatumVlozeni <= " .  $params->addParameter($database->dateToParam(time())) . ")))
                ORDER BY DatumVlozeni";

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){

            switch ($row[3]){
                case '11':
                    $this->vznikBoduCelkem += $row[0];
                    break;

                case '21':
                    $this->cerpanoBodu += $row[0];
                    break;

                case '31':
                    $this->zanikBodu += $row[0];
                    break;

                case '41':
                    $this->rezervovanoBodu += $row[0];
                    break;

                case '51':
                    //Body které byly vázány se vracejí/resp.uvolní
                    $this->zanikRezervaceBodu += $row[0];
            }

        }

        $this->zustatekBodu = $this->vznikBoduCelkem - $this->cerpanoBodu - $this->zanikBodu - $this->rezervovanoBodu;
        $cursor->close();
    }
}


/**
 * Class SablonaBenefitu
 * @package Ron\RonPortal\Benefity
 */
class SablonaBenefitu
{
    public $id;
    public $rc;
    public $osoba;
    public $druhPohybu;
    public $idBenefitu;
    public $poradi;
    public $pocetBenefitu;
    public $pocetBodu;
    public $typDoplatku;
    public $doplatekCastka;
    public $typKontrolySablony;
    public $zdrojPoctuBenefitu;
    public $casovaJednotka;
    public $typCasoveSlozkyMzdy;
    public $pocetCasJednotek;
    public $insert_UserName;
    public $insert_Timestamp;
    public $update_UserName;
    public $update_Timestamp;

    /**
     * @param Database $database
     * @param $id
     * @return SablonaBenefitu
     * @throws \Exception
     */
    public static function nactiUdajeSablonyBenefitu($database, $id)
    {
        $params = $database->createParameters();
        $sql = "SELECT 
            ID, RC, DruhPohybu, IdBenefitu, Poradi, 
            PocetBenefitu, PocetBodu, TypDoplatku, DoplatekCastka, 
            TypKontrolySablony, CasovaJednotka, PocetCasJednotek, Insert_UserName, 
            Insert_Timestamp, Update_UserName, Update_Timestamp, ZdrojPoctuBenefitu, TypCasoveSlozkyMzdy

            FROM SablonaRezervaceBenefitu 

            WHERE ID = " . $params->addParameter($id);

        MessageManager::addDebugLog($sql);
        $cursor = $database->getSqlResult($sql, $params);
        if (!$row = $cursor->fetch()){
            throw new \Exception(\Ron\ronText('sablona_benefitu_neexistuje'));
        }

        $sablonaBenefitu = new SablonaBenefitu();
        $sablonaBenefitu->id = $id;
        $sablonaBenefitu->nactiUdajeSablonyBenefituZRadku($database, $row);
        $cursor->close();

        return $sablonaBenefitu;
    }


    /**
     * @param Database $database
     * @param $row
     */
    public function nactiUdajeSablonyBenefituZRadku($database, $row)
    {
        $this->id = (int)$row[0];
        $this->rc = $row[1];
        $this->druhPohybu = (int)$row[2];
        $this->idBenefitu = (int)$row[3];
        $this->poradi = (int)$row[4];
        $this->pocetBenefitu = (int)$row[5];
        $this->pocetBodu = (int)$row[6];
        $this->typDoplatku = (int)$row[7];
        $this->doplatekCastka = (float)$row[8];
        $this->typKontrolySablony = (int)$row[9];
        $this->casovaJednotka = (int)$row[10];
        $this->pocetCasJednotek = (int)$row[11];

        $this->insert_UserName = $row[12];
        $this->insert_Timestamp = $database->stringToTimestamp($row[13]);
        $this->update_UserName = $row[14];
        $this->update_Timestamp = $database->stringToTimestamp($row[15]);

        $this->zdrojPoctuBenefitu = (int)$row[16];
        $this->typCasoveSlozkyMzdy = $row[17];
    }
}



