<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 17.4.2018
 */

namespace Ron\RonPortal\Benefity;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\AgentManagerResult;
use Ron\RonPortal\ComboBoxOsoba;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\DokumentyBrowser;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\TextManager;
use stdClass;

require_once('benefity/benefit.php');
require_once ('adsapp/ronportalcontrols.php');
require_once('ron/guiutils.php');
require_once('ron/controls.php');
require_once('adsapp/b_dokumenty.php');
require_once('adsapp/dokumentyutils.php');


/**
 * Class DokumentyBenefituBrowser
 * @package Ron\RonPortal\Benefity
 */
class DokumentyBenefituBrowser extends DokumentyBrowser
{

    /**
     * DokumentyBenefituBrowser constructor.
     * @param Database $database
     * @param $IDBenefitu
     */
    function __construct($database, $IDBenefitu)
    {
        parent::__construct($database, "?menuid=benefity&action=dokumentdownload&id=$IDBenefitu", 0, NULL);

        $this->sqlFromPart =
            'Dokument, TypDokumentu, DokumentBenefitu';

        $params = $database->createParameters();
        $where =
            'Dokument.Id = DokumentBenefitu.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            " AND DokumentBenefitu.idBenefitu = " . $params->addParameter($IDBenefitu);
        $this ->sqlWherePart = new SqlStatement($where, $params);
    }
}


/**
 * Class Benefity
 * @package Ron\RonPortal\Benefity
 */
class BenefityController extends HttpController
{
    /** @var  Database $database*/
    private $database;

    /** @var  PravaNaOsoby $pravo_rezervovat */
    private $pravo_rezervovat;

    /** @var  PravaNaOsoby $pravo_schvalit*/
    private $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat*/
    private $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

    /** @var PravaNaOsoby $pravo_sablony */
    public $pravo_sablony;


    /**
     * Benefity constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        Prava::nastavObdobiProPrava(time(), time());

        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');
        $this->pravo_sablony = Prava::vratPrava('benefit_sablona');

        $this->database = $database;

        TextManager::getInstance() ->loadStrings('benefity.ini');
        HtmlHeaders::getInstance() ->addCss('css/s_ron.css');
        HtmlHeaders::getInstance() ->addScript('script/benefity.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=benefity&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);
    }


    /**
     * @param Database $database
     * @param string $url
     * @param $prava
     * @return ComboBoxOsoba
     */
    private function comboOsobBenefitu($database, $url, $prava)
    {
        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('osoba');
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = 'onChange="window.location.href = &quot;' . $url . '&amp;osoba=&quot; + encodeURIComponent(this.value)"';
        $combo->emptyRow = ronText('nevybrano');
        $combo->loadDataPP($database, $datumOd, $datumDo, $prava, false, isset($_SESSION['osoba']) ? $_SESSION['osoba'] : "", false);

        return $combo;
    }


    /**
     * Pokud má právo pouze na sebe, vrátí přihlášenou osobu
     * @param $rc
     * @return mixed
     * @throws \Ron\DB\DBException
     */
    public function najdiOsobuRezervace($rc=NULL)
    {
        $osoba = (($rc != "") ? $rc :  User::getRC());

        $params = $this->database->createParameters();
        $sql = "SELECT ID FROM PracovniPomer Where RC = " . $params->addParameter($osoba);
        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $osoba = $row[0];
        $cursor->close();

        return $osoba;
    }


    /**
     * Vrací pole: [0] - editace; [1] - schvaleni; [2] - cerpani
     * @param PracovniPomer $pp
     * @param PohybBenefitu $hodnoty
     * @param PohybBenefitu $hodnotyPost
     * @return array
     * @throws Exception
     */
    public function vyhodnoceniPodminekProEditaceZaznamu($pp, $hodnoty=null, $hodnotyPost=null)
    {
        $editaceZaznamu['editace'] = false;
        $editaceZaznamu['schvaleni'] = false;
        $editaceZaznamu['cerpani'] = false;

        //Hodnoty z DB
        if ($hodnoty->datumRezervaceDo == "")
        {
            //Platnost neomezená - teoreticky editace možná

            if ($hodnoty->datumSchvaleni == ""){
                //Není schváleno
                $editaceZaznamu['editace'] = true;

                if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc)){
                    $editaceZaznamu['schvaleni'] = true;
                } else {
                    $editaceZaznamu['schvaleni'] = false;
                }
            } else {
                //Je schváleno
                $editaceZaznamu['schvaleni'] = false;
                $editaceZaznamu['editace'] = false;

            }

            if ($hodnoty->datumCerpani == ""){
                if ($this->pravo_vydat->lzeZobrazitRC($pp->rc)){
                    $editaceZaznamu['cerpani'] = true;
                } else {
                    $editaceZaznamu['cerpani'] = false;
                }
            }
        }
        else
        {
            //Omezení dle platnosti - teoreticky editace možná

            if ($hodnoty->datumRezervaceDo >= time())
            {
                //V rámci platnosti
                if ($hodnoty->datumSchvaleni == "") {
                    $editaceZaznamu['editace'] = true;

                    if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc)) {
                        $editaceZaznamu['schvaleni'] = true;
                    } else {
                        $editaceZaznamu['schvaleni'] = false;
                    }
                } else {
                    $editaceZaznamu['schvaleni'] = false;
                    $editaceZaznamu['editace'] = false;
                }

                if ($hodnoty->datumCerpani == ""){
                    if ($this->pravo_vydat->lzeZobrazitRC($pp->rc)){
                        $editaceZaznamu['cerpani'] = true;
                    } else {
                        $editaceZaznamu['cerpani'] = false;
                    }
                }
            }
            else
            {
                //Platnost vypršela
                $editaceZaznamu['editace'] = false;
                $editaceZaznamu['schvaleni'] = false;
                $editaceZaznamu['cerpani'] = false;
            }
        }

        //Hodnoty z formuláře
        if (isset($hodnotyPost))
        {
            $editaceZaznamu['editace'] = true;

            if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc)){
                $editaceZaznamu['schvaleni'] = true;

            } else {
                $editaceZaznamu['schvaleni'] = false;
            }
        }

        //Vrátí pole: [0] - editace;[1] - schválení;[2] - čerpání
        return $editaceZaznamu;
    }


    /**
     * @param PracovniPomer $pp
     * @param PohybBenefitu $hodnoty
     * @param PohybBenefitu $hodnotyPost
     * @return string
     * @throws Exception
     */
    public function sekceVznikuBodu($pp, $hodnoty=null, $hodnotyPost=null)
    {
        $result = "<div class='row form-section'>";
        $result .=     ronText("data_vlozeni_a_platnosti");
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("datum_vlozeni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $dnes = StringUtils::format("date", time());
        $datumVlozeni = $hodnoty->datumVlozeni != "" ? StringUtils::format("date", $hodnoty->datumVlozeni): $dnes;
        $result .= "$datumVlozeni";
        $result .=          "<input type='hidden' name='datumVlozeni' value='$datumVlozeni'>";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("omezeni_platnosti_vzniku");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";

        $datumOmezeniPlatnosti = (($hodnotyPost->datumOmezeniPlatnosti != "") ? StringUtils::format("date", $hodnotyPost->datumOmezeniPlatnosti) : StringUtils::format("date", $hodnoty->datumOmezeniPlatnosti));
        if ($hodnoty->datumSchvaleni != ""){
            //Pokud je již schváleno pak nelze měnit
            $datumOmezeniPlatnosti = (($hodnoty->datumSchvaleni != "") ? ($hodnoty->datumOmezeniPlatnosti != "" ? StringUtils::format("date", $hodnoty->datumOmezeniPlatnosti) :  ronText('neomezeno')):"");
            $result .= "$datumOmezeniPlatnosti";
        } else {

            if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc) || $this->pravo_vznik->lzeZobrazitRC($pp->rc)) {
                $result .= "<input type='text' class='inputDate' size='10' maxlength='10' name='datumOmezeniPlatnosti' title='" . ronText("titulek_omezeni_platnosti_vzniku") . "' value='$datumOmezeniPlatnosti'>";
                $result .= "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='datumOmezeniPlatnosti'></i>";

            } else {
                $result .= "$datumOmezeniPlatnosti";
            }
        }

        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("pocet");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";


        //Pokud existuje hodnota v postu pak ji zobrazit - zajistit že POST bude null pokud bude schváleno !!
        $hodnotaPoctuBodu = (($hodnotyPost->pocetBodu != "") ? $hodnotyPost->pocetBodu : $hodnoty->pocetBodu);
        if ($hodnoty->datumSchvaleni != ""){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$hodnotaPoctuBodu";
        } else {
            if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc) || $this->pravo_vznik->lzeZobrazitRC($pp->rc)) {
                $result .= "<input type='number' name='pocetBodu' min='0' class='nonempty' title='Počet bodů' step='1' value='$hodnotaPoctuBodu'>";
            } else {
                $result .= "$hodnotaPoctuBodu";
            }
        }

        $result .=      "</div>";
        $result .= "</div>";


        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          $hodnoty->datumSchvaleni != "" ?  ronText("schvalil") :  ronText("schvaleni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";

        $datumSchvaleni = (($hodnoty->datumSchvaleni != "") ? StringUtils::format("date", $hodnoty->datumSchvaleni) : "");
        $checkDatumSchvaleni = $hodnoty->datumSchvaleni != "" ? 'checked disabled' : "";
        $schvalit = $datumSchvaleni . (($hodnoty->schvalil != "" ) ? " (" . $hodnoty->schvalil . ")" : "");

        if ($hodnoty->datumSchvaleni != ""){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$schvalit";
        } else {
            if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc) || $this->pravo_vznik->lzeZobrazitRC($pp->rc)) {
                $result .= "<input type='checkbox' name='checkDatumSchvaleni' id='checkDatumSchvaleni' value='1' $checkDatumSchvaleni><p id='datumSchvaleniInfo'></p>";
                $result .= "$schvalit";
            }
        }

        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("poznamka");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";

        $hodnotaPoznamky = (($hodnotyPost->poznamka != "") ? $hodnotyPost->poznamka : $hodnoty->poznamka);
        if ($hodnoty->datumSchvaleni != ""){
            //Pokud je již schváleno pak nelze již měnit
            $result .= $hodnotaPoznamky;
        } else {
            if ($this->pravo_schvalit->lzeZobrazitRC($pp->rc) || $this->pravo_vznik->lzeZobrazitRC($pp->rc)) {
                $result .= "<textarea class='col col-12 col-md-9' rows='4' cols='30' name='poznamka' title=''>$hodnotaPoznamky</textarea>";
            } else {
            $result .= $hodnotaPoznamky;
            }
        }

        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<input type='hidden' name='typDoplatku' value='0'>";


        return $result;
    }

    /**
     * @param Database $database
     * @param $rc
     * @param $benefit
     * @param $url
     * @return ComboBox
     * @throws \Ron\DB\DBException
     */
    public function comboVyberuBenefitu($database, $rc, $benefit, $url)
    {
        $params = $database->createParameters();

        $sql = "SELECT Benefit.ID, Benefit.Popis
                 FROM Benefit
                  INNER JOIN TypBenefitu ON (TypBenefitu.Kod = Benefit.TypBenefitu)
                 
                  WHERE (Benefit.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . ")
                    AND ((Benefit.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . ") OR (Benefit.PlatnostDo IS NULL))
                    AND
                   ( (Benefit.NeomezenyPocet = 1)
                       OR
                      ((SELECT COALESCE(SUM(PB.PocetBenefitu), 0) AS CerpanoBenefitu
                        FROM PohybBenefitu AS PB
                         WHERE (Benefit.ID = PB.IdBenefitu) AND (PB.DatumSchvaleni <= " . $params->addParameter($database->dateToParam((DateUtils::incDay(time())-1))) . ") 
                         AND (PB.DruhPohybu IN ('22','21','41','42','32'))) < Benefit.PocetCelkem)) 
                    AND 
                   ( ((SELECT COUNT(*) FROM BenefitProRozdeleni WHERE (BenefitProRozdeleni.IdBenefitu = Benefit.ID)) = 0)
                       OR
                        (EXISTS (SELECT * FROM BenefitProRozdeleni
                                 INNER JOIN RozdeleniOsob as RO ON (RO.KodTypuRozdeleni = BenefitProRozdeleni.KodTypuRozdeleni)
                                                               AND (RO.Rozdeleni = BenefitProRozdeleni.Rozdeleni)
                                 WHERE (BenefitProRozdeleni.IdBenefitu = Benefit.ID)
                                   AND (RO.RC = " . $params->addParameter($rc) . ")
                                   AND (RO.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . ")
                                   AND ((RO.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . ") OR (RO.PlatnostDo IS NULL)))) )
            
                  ORDER BY Benefit.Popis, Benefit.PlatnostOd";

        $combo = new ComboBox('benefit');
        $combo->emptyRow = ronText('nevybrano');
        $combo->dataTypeComparison = false;
        $combo->className .= 'form-input adsfilterforminput_full nonempty';
        $combo->htmlParams = "title='" . ronText("vyber_benefitu") . "'" . 'onChange="window.location.href=&quot;' . $url . '&amp;benefit=&quot; + encodeURIComponent(this.value);"';
        $combo->showCode = false;
        $combo->loadData($database, $sql, $benefit, $params);

        return $combo;
    }


    /**
     * Volá se pokud bude uživatel požadovat rezervaci benefitu z bodů
     * @param PracovniPomer $pp
     * @param PohybBenefitu $hodnoty
     * @param PohybBenefitu $hodnotyPost
     * @param Benefit $benefit
     * @return string
     * @throws Exception
     */
    public function sekceRezervaceZBodu($pp, $hodnoty, $hodnotyPost, $benefit=null)
    {

        $jeMoznoEditovat = $this->vyhodnoceniPodminekProEditaceZaznamu($pp, $hodnoty, $hodnotyPost);

        $result = "";
        $result .= "<div class='row form-section'>";
        $result .=      ronText("hodnoty_rezervace");
        $result .= "</div >";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("datum_vlozeni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $dnes = StringUtils::format("date", time());
        $datumVlozeni = $hodnoty->datumVlozeni != "" ? StringUtils::format("date", $hodnoty->datumVlozeni): $dnes;
        $result .= "$datumVlozeni";
        $result .=          "<input type='hidden' name='datumVlozeni' value='$datumVlozeni'>";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("platnost_rezervace");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        //Z DB nebo POSTu
        $datumRezervaceDo = (($hodnotyPost->datumRezervaceDo != "") ? StringUtils::format("date", $hodnotyPost->datumRezervaceDo) : StringUtils::format("date", $hodnoty->datumRezervaceDo));
        if (!$jeMoznoEditovat['editace']){
            //Platnost nebo Neomezeno
            $datumRezervaceDoInfo = (($hodnoty->datumSchvaleni != "") ? ($hodnoty->datumRezervaceDo != "" ? StringUtils::format("date", $hodnoty->datumRezervaceDo) :  ronText('neomezeno')): $datumRezervaceDo);
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$datumRezervaceDoInfo";
            $result .= "<input type='hidden' name='datumRezervaceDo' value='$datumRezervaceDo'>";
        } else {
            $result .=  "<input type='text' class='inputDate' size='10' maxlength='10' name='datumRezervaceDo' title='" . ronText("titulek_omezeni_platnosti_vzniku") . "' value='$datumRezervaceDo'>";
            $result .=  "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='datumRezervaceDo'></i>";
        }
        $result .=      "</div>";
        $result .= "</div>";
        $result .= "<div class='mezera'><div></div></div>";



        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption tucne'>";
        $result .=          ronText("pocet_benefitu");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $hodnotaPoctuBenefitu = (($hodnotyPost->pocetBenefitu != "") ? ($hodnotyPost->pocetBenefitu ) : $hodnoty->pocetBenefitu);
        if (!$jeMoznoEditovat['editace']){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$hodnotaPoctuBenefitu";
            $result .= "<input type='hidden' name='pocetBenefitu' value='$hodnotaPoctuBenefitu'>";
        } else {
            $result .= "<input type='number' step='1' name='pocetBenefitu' max='' min='0' class='nonempty' title='" . ronText('pocet_benefitu') . "'  value='$hodnotaPoctuBenefitu'>";
        }
        $result .=      "</div>";
        $result .= "</div>";



        // *** Hodnota v bodech
        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("pocet_bodu");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        //Pokud variabilní pak bez disabled
        $variabilni = (($benefit->variabilniHodnotaVBodech == 1) ? "" : "disabled");
        $hodnotaPoctuBodu = (($hodnotyPost->pocetBodu != "") ? $hodnotyPost->pocetBodu : $hodnoty->pocetBodu);
        if (!$jeMoznoEditovat['editace']){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$hodnotaPoctuBodu";
            $result .= "<input type='hidden' name='pocetBodu' value='$hodnotaPoctuBodu'>";
        } else {

            $titulek = ronText('benefity_pocet_bodu');
            if ($benefit->variabilniHodnotaVBodech == 1) {
                $result .= "<input type='number' name='pocetBodu' $variabilni class='' title='$titulek'  value='$hodnotaPoctuBodu'>";
            } else {
                $result .= "<input type='number' name='pocetBoduInfo' $variabilni class='' title='$titulek'  value='$hodnotaPoctuBodu'>";
                $result .= "<input type='hidden' name='pocetBodu' value='$hodnotaPoctuBodu'>";
            }
        }
        $result .=      "</div>";
        $result .= "</div>";



        // *** Doplatek castka
        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("doplatek");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $hodnotaDoplatkuCastka = (($hodnotyPost->doplatekCastka != "") ? $hodnotyPost->doplatekCastka : $hodnoty->doplatekCastka);
        if (!$jeMoznoEditovat['editace']){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "$hodnotaDoplatkuCastka";
            $result .= "<input type='hidden' name='doplatekCastka' value='$hodnotaDoplatkuCastka'>";
        } else {
            $result .= "<input type='number' step='1' name='doplatekCastka' class='' title=''  value='$hodnotaDoplatkuCastka'>";
        }
        $result .=      "</div>";
        $result .= "</div>";


        $result .= "<div class='mezera'><div></div></div>";


        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          $hodnoty->datumSchvaleni != "" ?  ronText("schvalil") :  ronText("schvaleni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";

        // *** Datum schválení
        $datumSchvaleni = (($hodnoty->datumSchvaleni != "") ? StringUtils::format("date", $hodnoty->datumSchvaleni) : ($benefit->automatickeSchvaleni ? $dnes : ""));
        if (!$jeMoznoEditovat['editace']){
            //Pokud je datum pak checked a disabled
            $checkDatumSchvaleni = 'checked disabled';
        } else {
            //Pokud neni datum pak jake je nastaveni automaticke schvaleni
            if ($benefit->automatickeSchvaleni){
                $checkDatumSchvaleni = 'checked';
            } else {
                $checkDatumSchvaleni = "";
            }
        }
        $schvalit = $datumSchvaleni . (($hodnoty->schvalil != "" ) ? " (" . $hodnoty->schvalil . ")" : "");
        if (!$jeMoznoEditovat['editace']) {
            $result .= "$schvalit";
            $result .= "<input type='hidden' name='datumSchvaleni' value='$datumSchvaleni'>";
            $result .= "<input type='hidden' name='schvalil' value='$hodnoty->schvalil'>";
            //Pokud je již schváleno pak nelze měnit
        } else {
            if ($benefit->automatickeSchvaleni || $this->pravo_schvalit->lzeZobrazitRC($pp->rc)) {
                $result .= "<input type='checkbox' name='checkDatumSchvaleni' id='checkDatumSchvaleni' value='1' $checkDatumSchvaleni><p id='datumSchvaleniInfo'></p>";
                $result .= "$schvalit";
                $result .= "<input type='hidden' name='datumSchvaleni' value='$datumSchvaleni'>";
                $result .= "<input type='hidden' name='schvalil' value='$hodnoty->schvalil'>";
            }
        }
        $result .=      "</div>";
        $result .= "</div>";


        $result .= "<div class='row'>";
        $result .= "<div class='col col-12 col-md-3 caption'>";
        $result .= $hodnoty->datumCerpani != "" ? ronText("vydal") : ronText("cerpani");
        $result .= "</div>";
        $result .= "<div class='col col-12 col-md-9'>";
        $datumCerpani = (($hodnoty->datumCerpani != "") ? StringUtils::format("date", $hodnoty->datumCerpani) : "");
        $checkDatumCerpani = $hodnoty->datumCerpani != "" ? 'checked disabled' : "";
        $vydal = $datumCerpani . (($hodnoty->vydal != "") ? " (" . $hodnoty->vydal . ")" : "");


        $zobrazitDanyTyp = true;
        if (trim($this->pravo_vydat->priznak3) != '') {
            $povoleneTypyBenefitu = explode(';', $this->pravo_vydat->priznak3);
            $zobrazitDanyTyp = in_array($hodnoty->typBenefitu, $povoleneTypyBenefitu);
        }


        if (!$jeMoznoEditovat['cerpani'] || $hodnoty->druhPohybu == 51) {
            $result .= "$vydal";
            $result .= "<p id='datumCerpaniInfo'></p>";
            $result .= "<input type='hidden' name='datumCerpani' value='$datumCerpani'>";
        } else {
            if ($this->pravo_vydat->lzeZobrazitRC($pp->rc) && $zobrazitDanyTyp) {
                $result .= "<input type='checkbox' name='checkDatumCerpani' id='checkDatumCerpani' value='1' $checkDatumCerpani><p id='datumCerpaniInfo'></p>";
                $result .= "$vydal";
                $result .= "<input type='hidden' name='datumCerpani' value='$datumCerpani'>";
            }
        }
        $result .= "</div>";
        $result .= "</div>";


        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("poznamka");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";

        $hodnotaPoznamky = (($hodnotyPost->poznamka != "") ? $hodnotyPost->poznamka : $hodnoty->poznamka);
        if (!$jeMoznoEditovat['editovat'] || $hodnoty->druhPohybu == 51){
            //Pokud je již schváleno pak nelze již měnit
            $result .= "<div class='benefityPevnaSirka' >" . $hodnotaPoznamky . "</div>";
            $result .= "<input type='hidden' name='poznamka' value='$hodnotaPoznamky'>";
        } else {
            if ($this->pravo_vydat->lzeZobrazitRC($pp->rc)) {
                $result .= "<textarea class='' rows='4' cols='50' name='poznamka' title=''>$hodnotaPoznamky</textarea>";
            } else {
                $result .= "<div class='' title=''>$hodnotaPoznamky</div>";
                $result .= "<input type='hidden' name='poznamka' value='$hodnotaPoznamky'>";

            }
        }
        $result .=      "</div>";
        $result .= "</div>";


        return $result;
    }


    /**
     * @param $hodnoty
     * @return string
     */
    public function sekceZanikuBodu($hodnoty)
    {
        $datumVlozeni = StringUtils::format("date", $hodnoty->datumVlozeni);
        $datumOmezeniPlatnosti = StringUtils::format("date", $hodnoty->datumOmezeniPlatnosti);
        $datumSchvaleni = StringUtils::format("date", $hodnoty->datumSchvaleni);

        $result = "<div class='row form-section'>";
        $result .=      ronText("pocet");
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("pocet_bodu");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $result .=          "$hodnoty->pocetBodu";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='mezera'><div></div></div>";

        $result .= "<div class='row form-section'>";
        $result .=          ronText("data_vlozeni_a_platnosti");
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("datum_vlozeni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $result .=          "$datumVlozeni";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("omezeni_platnosti_vzniku");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $result .=          "$datumOmezeniPlatnosti";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='mezera'><div></div></div>";

        $result .= "<div class='row form-section'>";
        $result .=          ronText("schvaleni");
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("datum_schvaleni");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $result .=          "$datumSchvaleni";
        $result .=      "</div>";
        $result .= "</div>";

        $result .= "<div class='row'>";
        $result .=      "<div class='col col-12 col-md-3 caption'>";
        $result .=          ronText("poznamka");
        $result .=      "</div>";
        $result .=      "<div class='col col-12 col-md-9'>";
        $result .=          "$hodnoty->poznamka";
        $result .=      "</div>";
        $result .= "</div>";


        return $result;
    }

    /**
     * @param $osoba
     * @param PohybBenefitu $hodnotyPost
     * @return BaseViewObject
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function vznikBodu($osoba, $hodnotyPost=NULL)
    {
        require_once ('adsapp/ppomer.php');

        if (!($this->pravo_vznik->jsouNejakePrava())) {
            RonPortalUtils::neniPravoException();
        }

        if ($hodnotyPost->idPP != ""){
            $osoba = $hodnotyPost->idPP;
        }

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);

        $bodyOsoby = "";
        if ($osoba){

            if (!($this->pravo_vznik->lzeZobrazitRC($pp->rc))) {
                RonPortalUtils::neniPravoException();
            }

            //Po generování zániků načíst body
            $bodyOsoby = new BenefityOsoby();
            $bodyOsoby->nactiStavBoduOsoby($this->database, $pp->rc);

        }

        $url = "?menuid=benefity&action=vznikbodu&osoba=$osoba";

        $nadpis =  ronText('vznik');

        Model::addData('pp', $pp);
        Model::addData('url', $url);
        Model::addData('druhPohybuBenefitu', "11");
        Model::addData('nadpis', $nadpis);
        Model::addData('osobabenefitu', $osoba);
        Model::addData('bodyOsoby', $bodyOsoby);
        Model::addData('comboOsobBenefitu', $this->comboOsobBenefitu($this->database, "?menuid=benefity&action=vznikbodu", $this->pravo_vznik));
        Model::addData('sekceBenefitu', $this->sekceVznikuBodu($pp, null, $hodnotyPost));

        Model::addData('jeMoznoEditovat', $this->vyhodnoceniPodminekProEditaceZaznamu($pp, NULL, $hodnotyPost));
        Model::addData('pravo_vznik_bodu', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vznik_bodu'));
        Model::addData('pravo_rezervace', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_rezervace'));
        Model::addData('pravo_schvaleni', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_schvaleni'));
        Model::addData('pravo_vydani', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vydani'));

        return new BaseViewObject('benefity/view_benefity.php');
    }


    /**
     * Slouží jako upřesňující info v detailu
     * @param PohybBenefitu $pohybBenefitu
     * @return string
     */
    public function popisStavuBenefitu($pohybBenefitu)
    {
        $result = '';

        switch ($pohybBenefitu->druhPohybu) {

            case '11':
                if ($pohybBenefitu->datumSchvaleni) {
                    $result .= ronText('vznik') . " (" . $pohybBenefitu->pocetBodu . ")" ;
                } else {
                    $result .= ronText('neschvaleno') . " (" . $pohybBenefitu->pocetBodu .  ")";
                }
                break;

            case '31';
                $result .= ronText('zanik_bodu') . " (" . $pohybBenefitu->pocetBodu .  ")";
                break;

            case '21':
            case '41':
                if ($pohybBenefitu->datumSchvaleni) {
                    $result .= $pohybBenefitu->popisPohybuBenefitu;
                } else {
//                    $result .= $pohybBenefitu->popisPohybuBenefitu . " (" . \Ron\ronText('neschvaleno') . ")";
                    $result .= ronText('neschvaleno') . " - " . $pohybBenefitu->popisPohybuBenefitu;
                }
                break;

            case '51':
                $result .= ronText('zanik_rezervace') . " (" . $pohybBenefitu->pocetBodu .  ")";
                break;
        }

        return $result;
    }


    /**
     * @param $pohyb
     * @param PohybBenefitu | stdClass $hodnotyPost
     * @return BaseViewObject
     * @throws Exception
     */
    public function detailPohybuBenefitu($pohyb, $hodnotyPost)
    {
        if ($hodnotyPost->idPohybu != ""){
            $pohyb = $hodnotyPost->idPohybu;
        }

        /** @var PohybBenefitu $pohybBenefitu */
        $hodnotyPohybuBenefitu = PohybBenefitu::nactiPohybBenefitu($this->database, $pohyb);

        require_once('adsapp/ppomer.php');
        /** @var PracovniPomer $pp */
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $hodnotyPohybuBenefitu->idPP);

        if (!$hodnotyPohybuBenefitu->idPP){
            throw new Exception(ronText('neni_vybrana_osoba'));
        }

        $lze_rezervovat = $this->pravo_rezervovat->lzeZobrazitRC($hodnotyPohybuBenefitu->rc);
        $lze_schvalit = $this->pravo_schvalit->lzeZobrazitRC($hodnotyPohybuBenefitu->rc);
        $lze_vydat= $this->pravo_vydat->lzeZobrazitRC($hodnotyPohybuBenefitu->rc);
        $lze_vznik = $this->pravo_vznik->lzeZobrazitRC($hodnotyPohybuBenefitu->rc);
        if (!($lze_rezervovat || $lze_schvalit || $lze_vydat || $lze_vznik)){
            RonPortalUtils::neniPravoException();
        }

        $objektBenefitu = '';
        if ($hodnotyPohybuBenefitu->id_benefitu) {
            /** @var Benefit $objektBenefitu */
            $objektBenefitu = Benefit::nactiUdajeBenefitu($this->database, $hodnotyPohybuBenefitu->id_benefitu);
        }

        if (in_array($hodnotyPohybuBenefitu->druhPohybu, ["11"])) {
            //Pro vznik a zánik
            Model::addData('sekceBenefitu', $this->sekceVznikuBodu($pp, $hodnotyPohybuBenefitu, $hodnotyPost));

        } elseif (in_array($hodnotyPohybuBenefitu->druhPohybu, ["21", "32", "41", "51", "52"])){
            //Pro benefity (rezervace/potvrzeni/schválení/vydání)
            if ($objektBenefitu->id != "") {
                Model::addData('sekceBenefitu', $this->sekceRezervaceZBodu($pp, $hodnotyPohybuBenefitu, $hodnotyPost, $objektBenefitu));
            }

        } elseif (in_array($hodnotyPohybuBenefitu->druhPohybu, ["31"])){
            Model::addData('sekceBenefitu', $this->sekceZanikuBodu($hodnotyPohybuBenefitu));
        }


        $bodyOsoby = new BenefityOsoby();
        $bodyOsoby->nactiStavBoduOsoby($this->database, $pp->rc);

        /** @var DokumentyBenefituBrowser $dokumentBenefitu */
        $dokumentBenefitu = new DokumentyBenefituBrowser($this->database, $objektBenefitu->id);
        $dokumentBenefitu->load();

        $nadpis = ronText('detail') . " - " . ($objektBenefitu != "" ? StringUtils::htmlTextConvert($objektBenefitu->popis) : $hodnotyPohybuBenefitu->popisPohybuBenefitu);

        $url = "?menuid=benefity&action=detailpohybu&idpohybubenefitu=$pohyb";
        $this->strankovaniDetailu();
        Model::addData('pp', $pp);
        Model::addData('jeMoznoEditovat', $this->vyhodnoceniPodminekProEditaceZaznamu($pp, $hodnotyPohybuBenefitu, $hodnotyPost));
        Model::addData('druhPohybuBenefitu', $hodnotyPohybuBenefitu->druhPohybu);
        Model::addData('pohybBenefitu', $hodnotyPohybuBenefitu);
        Model::addData('osobabenefitu', $hodnotyPohybuBenefitu->idPP);
        Model::addData('bodyOsoby', $bodyOsoby);
        Model::addData('objektBenefitu', $objektBenefitu);

        Model::addData('nadpis', $nadpis);
        Model::addData('url', $url);
        Model::addData('popisStavuBenefitu', $this->popisStavuBenefitu($hodnotyPohybuBenefitu));
        Model::addData('jumpMenu', $this->jumpMenu());
        Model::addData('dokumentBenefitu', $dokumentBenefitu);

        Model::addData('pravo_vznik_bodu', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vznik_bodu'));
        Model::addData('pravo_rezervace', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_rezervace'));
        Model::addData('pravo_schvaleni', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_schvaleni'));
        Model::addData('pravo_vydani', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vydani'));

        return new BaseViewObject('benefity/view_benefity.php');
    }


    /**
     * @param $osoba
     * @param $benefit
     * @param $hodnotyPost
     * @return BaseViewObject
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function rezervaceBenefitu($osoba, $benefit, $hodnotyPost=null)
    {
        if (!($this->pravo_rezervovat->jsouNejakePrava())) {
            RonPortalUtils::neniPravoException();
        }

        //Pokud má osoba právo pouze na sebe, pak se osoba nastaví dle přihlášení
        if ($this->pravo_rezervovat->prava == 1){
            $osoba = $this->najdiOsobuRezervace();
        }

        require_once ('adsapp/ppomer.php');
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);

        $objektBenefitu = "";
        $bodyOsoby = "";
        if ($osoba){

            if (!($this->pravo_rezervovat->lzeZobrazitRC($pp->rc))) {
                RonPortalUtils::neniPravoException();
            }

            $bodyOsoby = new BenefityOsoby();
            $bodyOsoby->nactiStavBoduOsoby($this->database, $pp->rc);

            if ($benefit) {
                require_once('benefity/benefit.php');
                $objektBenefitu = Benefit::nactiUdajeBenefitu($this->database, $benefit);
            }
        }


        /** @var DokumentyBenefituBrowser $dokumentBenefitu */
        $dokumentBenefitu = new DokumentyBenefituBrowser($this->database, $objektBenefitu->id);
        $dokumentBenefitu->load();

        $nadpis =  ronText('rezervace_benefitu') . ($objektBenefitu != "" ? (" - " . StringUtils::htmlTextConvert($objektBenefitu->popis)):"") ;
//        $url = "?menuid=benefity&action=rezervace&osoba=$osoba&benefit=$benefit";
        $url = "?menuid=benefity&action=rezervacebodu&osoba=" . ($_GET['osoba'] != "" ? $_GET['osoba'] : $osoba);

        Model::addData('pp', $pp);
        Model::addData('nadpis', $nadpis);
        Model::addData('druhPohybuBenefitu', "41");
        Model::addData('akce', "rezervace");
        Model::addData('url', $url);
        Model::addData('benefit', $benefit);
        Model::addData('osobabenefitu', $osoba);
        Model::addData('objektBenefitu', $objektBenefitu);
        Model::addData('bodyOsoby', $bodyOsoby);
        Model::addData('comboOsobBenefitu', $this->comboOsobBenefitu($this->database, "?menuid=benefity&action=rezervacebodu&osoba=$osoba", $this->pravo_rezervovat));
        Model::addData('comboVyberuBenefitu', $this->comboVyberuBenefitu($this->database, $pp->rc, $benefit, $url));
        Model::addData('dokumentBenefitu', $dokumentBenefitu);
        if ($objektBenefitu->id != "") {
            Model::addData('sekceBenefitu', $this->sekceRezervaceZBodu($pp, null, $hodnotyPost, $objektBenefitu));
        }
        Model::addData('jeMoznoEditovat', $this->vyhodnoceniPodminekProEditaceZaznamu($pp, $objektBenefitu, $hodnotyPost));
        Model::addData('pravo_vznik_bodu', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vznik_bodu'));
        Model::addData('pravo_rezervace', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_rezervace'));
        Model::addData('pravo_schvaleni', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_schvaleni'));
        Model::addData('pravo_vydani', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_vydani'));

        return new BaseViewObject('benefity/view_benefity.php');
    }


    /**
     * @param $osoba
     * @return string|Redirect
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function zapsatPohyb($osoba)
    {
        //Načíst hodnoty z POSTu
        $parametryBenefitu = $this->nactiPOSTProAgenta();

        $idPohybuBenefitu = $parametryBenefitu['BenefitIDPohybu'];
        $idBenefitu = $parametryBenefitu['BenefitID'];

        $kontrolaSchvaleniCerpani = true;

        //Pokud jsou již data schválení-vydání vyplněna pak Redirect
        if ($idPohybuBenefitu != "") {
            $kontrolaSchvaleniCerpani = $this->kontrolaSchvaleniCerpani($idPohybuBenefitu);
        }

        //50 - Kontrola benefitu
        $uloha = 50;

        $params = $uloha .
            ' -BenefitRC:' . $parametryBenefitu['BenefitRC'] .
            ' -BenefitDruhPohybu:' . $parametryBenefitu['BenefitDruhPohybu'] .
            ' -BenefitID:' . $parametryBenefitu['BenefitID'] .
            ' -BenefitIDPohybu:' . $parametryBenefitu['BenefitIDPohybu'] .
            ' -BenefitDatumStavu:' . $parametryBenefitu['BenefitDatumStavu'] .
            ' -BenefitDatumSchvaleni:' . $parametryBenefitu['BenefitDatumSchvaleni'] .
            ' -BenefitDatumRezervaceDo:' . $parametryBenefitu['BenefitDatumRezervaceDo'] .
            ' -BenefitDatumCerpani:' . $parametryBenefitu['BenefitDatumCerpani'] .
            ' -BenefitCerpaniUsername:' . $parametryBenefitu['BenefitCerpaniUsername'] .
            ' -BenefitDatumVlozeni:' . $parametryBenefitu['BenefitDatumVlozeni'] .
            ' -BenefitDatumOmezeniPlatnosti:' . $parametryBenefitu['BenefitDatumOmezeniPlatnosti'] .
            ' -BenefitPocetBodu:' . $parametryBenefitu['BenefitPocetBodu'] .
            ' -BenefitPocet:' . $parametryBenefitu['BenefitPocet'] .
            ' -BenefitDruhZdroje:' . $parametryBenefitu['BenefitDruhZdroje'] .
            ' -BenefitTypDoplatku:' . $parametryBenefitu['BenefitTypDoplatku'] .
            ' -BenefitDoplatekCastka:' . $parametryBenefitu['BenefitDoplatekCastka'] .
            ' -BenefitPoznamka:"' . $parametryBenefitu['BenefitPoznamka'] .
            '" ';


        //Kontrola zdali schvaleno/čerpáno
        if ($kontrolaSchvaleniCerpani){
            //Zaznam zapsat/změnit
            return $this->agentBenefitu($params, $osoba, $idPohybuBenefitu, $idBenefitu);
        } else {
            //Zaznam je již zapsán
            return new  Redirect(null, "?menuid=benefity");
        }
    }

    /**
     * Zavolá agenta s parametry a vrátí ID pohybu pro Redirect, v případě chyby vrátí zpět
     * @param $parametry
     * @param $osoba
     * @param $idPohybuBenefitu
     * @param $idBenefitu
     * @return string
     * @throws Exception
     */
    public function agentBenefitu($parametry, $osoba, $idPohybuBenefitu, $idBenefitu)
    {
        /**
         * Parametry agenta
         *
         * -BenefitRC:
         * -BenefitDruhPohybu:
         * -BenefitID:
         * -BenefitIDPohybu:
         * -BenefitDatumStavu:
         * -BenefitDatumSchvaleni:
         * -BenefitDatumRezervaceDo:
         * -BenefitDatumCerpani:
         * -BenefitDatumVlozeni:
         * -BenefitDatumOmezeniPlatnosti:
         * -BenefitPocetBodu:
         * -BenefitPocet:
         * -BenefitDruhZdroje:
         * -BenefitPoznamka:
         */

        if ($osoba == "" || $parametry == ""){
            throw new Exception(ronText('chyba_hodnot_agenta'));
        }

        require_once('adsapp/agentmanager.php');
        try {
            /** @var AgentManagerResult  $agentResult */
            $agentResult = AgentManager::executePersonAgent($parametry);
            MessageManager::addInfoMessage( ronText('vlozeni_ok') );

            $idPohybu = $agentResult->resultParams['idpohybu'];

            $returnValue = ($agentResult && $agentResult ->resultParams['idpohybu'])
                ? new Redirect(null, "?menuid=benefity&action=detailpohybu&idpohybubenefitu=$idPohybu")
                : new Redirect(null, '?menuid=benefity');

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            if ($idBenefitu == ""){
                $returnValue =  $this->vznikBodu($osoba, $this->nactiPOSTProDialog());
            } else {

                if ($idPohybuBenefitu != "") {
                    $returnValue = $this->detailPohybuBenefitu($idPohybuBenefitu, $this->nactiPOSTProDialog());
                } else {
                    $returnValue = $this->rezervaceBenefitu($osoba, $idBenefitu, $this->nactiPOSTProDialog());

                }
            }
        }
        return $returnValue;
    }


    /**
     * Pokud dojde k chybě načtou se vložené hodnoty do pole a to je předáno zpět
     * @return stdClass
     * @throws Exception
     */
    public function nactiPOSTProDialog()
    {
        $hodnoty = new stdClass();
        foreach ($_POST as $key => $value) {

            switch ($key){
                case "datumVlozeni":
                case "datumOmezeniPlatnosti":
                case "datumRezervaceDo":
                case "datumSchvaleni":
                case "datumCerpani":
                    if ($value != "") {
                        $hodnoty->$key = DateUtils::dateStringToTimestamp($value);
                    }
                    break;

                default:
                    $hodnoty->$key = $value;
            }
        }

        return $hodnoty;
    }


    /**
     * Vytvoří tlačítka pro přechod mezi jednotlivými detaily
     * @return string
     */
    private function jumpMenu()
    {
//        $result = "";
//        $result .= '<div class="strankovani">';
//
//        if ($_SESSION['benefity_predchozi']) {
//            $ronDataUrl = "?menuid=benefity&action=detailpohybu&idpohybubenefitu=" . $_SESSION['benefity_predchozi'];
//            $result .= '<input type="button" class="strankovani_tlacitko jumpMenu"  title="' . \Ron\ronText('predchozi_benefit_filtru') . '" data-ron-url="' . $ronDataUrl . '" value="' . \Ron\ronText('<') . '">';
//        } else {
//            $result .= '<input type="button" disabled="disabled" class="strankovani_tlacitko_disabled"  title="" value="">';
//        }
//
//        if ($_SESSION['benefity_pozice'] && $_SESSION['benefity_pocetnapaduvefiltru']) {
//            $result .= '<input type="button" disabled="disabled" class="strankovani_tlacitko"  title="' . $_SESSION['seznambenefitusukazatelem'] . '" value="' . $_SESSION['benefity_pozice'] . '/' . $_SESSION['benefity_pocetnapaduvefiltru'] . '">';
//        } else {
//            $result .= '<input type="button" disabled="disabled" class="strankovani_tlacitko_disabled"  title="" value="">';
//        }
//
//        if ($_SESSION['benefity_dalsi']) {
//            $ronDataUrl = "?menuid=benefity&action=detailpohybu&idpohybubenefitu=" . $_SESSION['benefity_dalsi'];
//            $result .= '<input type="button" class="strankovani_tlacitko jumpMenu"  title="' . \Ron\ronText('nasledujici_benefit_filtru') . '" data-ron-url="' . $ronDataUrl . '" value="' . \Ron\ronText('>') . '">';
//        } else {
//            $result .= '<input type="button" disabled="disabled" class="strankovani_tlacitko_disabled"  title="" value="">';
//        }
//
//        $result .= "</div>";
//        return $result;
    }

    /**
     * Pomocná funkce, umožňuje přímý přechod mezi jednotlivými detaily benefitů
     */
    private function strankovaniDetailu()
    {

        $polepocetbenefitufiltru = '';
        $polenazvubenefitu = '';

        //Split obou session do polí
        if ($_SESSION['pocetbenefitufilru']) {
            $polepocetbenefitufiltru = explode(",", $_SESSION['pocetbenefitufilru']);
        }
        if ($_SESSION['seznambenefitu']) {
            $polenazvubenefitu = explode(";;", $_SESSION['seznambenefitu']);
        }
        //Nulování pomocného seznamu titulku, aby se vždy uložila pouze čista aktuální data, budou sloužit k přidání ukazatele na aktuální název benefitu
        $_SESSION['seznambenefitusukazatelem'] = '';
        $polenazvunapadusukazatelem = '';
        $predchozi = '';
        $dalsi = '';

        $pozice = '';
        $pocetnapaduvefiltru = is_array($polepocetbenefitufiltru) ? count($polepocetbenefitufiltru) : 0;

        //Cyklus pro vytvoření tlačítek na procházení mezi jednotlivými benefity přes detail s přidáním stručného vyčtu jmen benefitů (v titulku) s ukazatelem na kterém je focus
        if ($polepocetbenefitufiltru) {
            for ($i = 0; $i < count($polepocetbenefitufiltru); $i++) {

                if ($polepocetbenefitufiltru[$i] == $_GET['idpohybubenefitu']) {
                    //Přidání ukazatele na konkrétní benefit
//                    $pom = $polenazvunapadusukazatelem[$i] . '&#187;&#009' . $polenazvubenefitu[$i];
//                    $polenazvunapadusukazatelem[$i] = $pom;

                    $pozice = $i + 1;

                    //Dokud je předchozí
                    $mene = $i - 1;
                    if ($mene >= 0) {
                        $predchozi = $polepocetbenefitufiltru[$mene];
                    }

                    //Dokud je následující
                    $vice = $i + 1;
                    if ($vice < count($polepocetbenefitufiltru)) {
                        $dalsi = $polepocetbenefitufiltru[$vice];
                    }

                } else {
                    //Přidání tabulátoru
//                    $pom = $polenazvunapadusukazatelem[$i] . '&#009' . $polenazvubenefitu[$i];
//                    $polenazvunapadusukazatelem[$i] = $pom;
                }
                //O5 sestavuji a přidávám odřádkování &#010 - LF
                $pom = $_SESSION['seznambenefitusukazatelem'] . $polenazvunapadusukazatelem[$i] . '&#010';
                $_SESSION['seznambenefitusukazatelem'] = $pom;
            }
        }

        $_SESSION['benefity_predchozi'] = $predchozi;
        $_SESSION['benefity_dalsi'] = $dalsi;
        $_SESSION['benefity_pozice'] = $pozice;
        $_SESSION['benefity_pocetnapaduvefiltru'] = $pocetnapaduvefiltru;
    }

    /**
     * @param $idBenefitu
     * @param $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    private function downloadDokument($idBenefitu, $idDokumentu)
    {
        //Kontrola, zdali existuje u dotyčného benefitu prohlížený dokument
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentBenefitu.ID ' .
            'FROM DokumentBenefitu LEFT JOIN Benefit ON (DokumentBenefitu.id = Benefit.idDokumentu) ' .
            "WHERE DokumentBenefitu.idBenefitu = " . $params->addParameter($idBenefitu) . " AND DokumentBenefitu.ID = " . $params->addParameter($idDokumentu);

        $existujeDokument = DBUtils::getSqlValue($this->database, $sql, $params);

        if ($existujeDokument == ''){
            throw new Exception(ronText('Nemate_pravo'));
        }

        return new DokumentOutputObject($this->database,$idDokumentu);
    }


    /**
     * Při vzniku bodů a rezervaci se nejprve generují zániky, po té se pokouší zapsat požadované změny
     * @throws Exception
     */
    private function generovaniZaniku()
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Val FROM SYS_VAR_GLOBAL Where Name = " . $params->addParameter("Benefit_GenerovaniZanikuKeDni");

        //Pokud je datum zániku včerejší, pak provede generování zániků
        $generovaniZanikuDatum = DBUtils::getSqlValue($this->database, $sql, $params);

        $datumZaniku = "";
        if ($generovaniZanikuDatum != "") {
            $datumZaniku = DateUtils::dateStringToTimestamp(explode("_", $generovaniZanikuDatum)[0]);
        }

        $dnes = DateUtils::dateStringToTimestamp(date("j.n.Y", time()));

        if ($datumZaniku < $dnes){
            require_once('adsapp/agentmanager.php');
            try {
                AgentManager::executePersonAgent(51);
            } catch (Exception $e) {
                MessageManager::addInfoMessage( ronText('chyba_generovani_zaniku') . ": " . $e ->getMessage() );
            }
        }
    }


    /**
     * @return array
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function nactiPOSTProAgenta()
    {
        require_once ('adsapp/ppomer.php');
        $id = filter_input(INPUT_POST,'osoba',FILTER_VALIDATE_INT);
        $autoSchvaleni = filter_input(INPUT_POST,'automatickeSchvaleni',FILTER_VALIDATE_INT);
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $id);

        $cerpat = $_POST['checkDatumCerpani'];
        $datumCerpani = null;
        $cerpaniUsername = null;
        $datumSchvaleni = null;
        $schvalil = null;

        if ($cerpat) {
            //Čerpání

            if ( $this->pravo_vydat->lzeZobrazitRC($pp->rc)) {

                $datumCerpani = date("Y-n-j", time());
                $cerpaniUsername = User::getLoginName();

                if ($_POST['checkDatumSchvaleni']) {
                    $datumSchvaleni = date("Y-n-j", DateUtils::dateStringToTimestamp($_POST['datumSchvaleni']));
                    $schvalil = $_POST['schvalil'] != "" ? $_POST['schvalil'] : User::getLoginName();
                } else {
                    $datumSchvaleni = date("Y-n-j", time());
                    $schvalil = User::getLoginName();
                }
            }
        } else {

            if ($_POST['datumSchvaleni'] != ""){
                $datumSchvaleni = date("Y-n-j", DateUtils::dateStringToTimestamp($_POST['datumSchvaleni']));
                $schvalil = $_POST['schvalil'] != "" ? $_POST['schvalil'] : "";
            } else {
                if ($autoSchvaleni || $this->pravo_schvalit->lzeZobrazitRC($pp->rc) || ($this->pravo_vznik->lzeZobrazitRC($pp->rc) && $_POST['druhPohybu'] == 11 )) {
                    //Pouze schválení
                    if ($_POST['checkDatumSchvaleni'] || $_POST['datumSchvaleni'] != "") {
                        $datumSchvaleni = date("Y-n-j", time());
                        $schvalil = User::getLoginName();
                    }
                }
            }
        }

        $poleNazvu = [];
        $poleNazvu["BenefitRC"] = $pp->rc;
        $poleNazvu["BenefitDruhPohybu"] = filter_input(INPUT_POST,'druhPohybu',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitID"] = filter_input(INPUT_POST,'benefit',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitIDPohybu"] = filter_input(INPUT_POST,'idPohybu',FILTER_SANITIZE_SPECIAL_CHARS);
        $poleNazvu["BenefitDatumStavu"] = date("Y-n-j",time());
        $poleNazvu["BenefitDatumSchvaleni"] = $datumSchvaleni;
        $poleNazvu["BenefitSchvalil"] = $schvalil;
        $poleNazvu["BenefitDatumRezervaceDo"] = $_POST['datumRezervaceDo'] != "" ? date("Y-n-j", DateUtils::dateStringToTimestamp($_POST['datumRezervaceDo'])) : NULL;
        $poleNazvu["BenefitDatumCerpani"] = $datumCerpani;
        $poleNazvu["BenefitCerpaniUsername"] = $cerpaniUsername;
        $poleNazvu["BenefitDatumVlozeni"] = $_POST['datumVlozeni'] != "" ? date("Y-n-j", DateUtils::dateStringToTimestamp($_POST['datumVlozeni'])) : NULL;
        $poleNazvu["BenefitDatumOmezeniPlatnosti"] = $_POST['datumOmezeniPlatnosti'] != "" ? date("Y-n-j", DateUtils::dateStringToTimestamp($_POST['datumOmezeniPlatnosti'])) : NULL;
        $poleNazvu["BenefitPocetBodu"] = filter_input(INPUT_POST,'pocetBodu',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitPocet"] = filter_input(INPUT_POST,'pocetBenefitu',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitDruhZdroje"] = 3;
        $poleNazvu["BenefitTypDoplatku"] = filter_input(INPUT_POST,'typDoplatku',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitDoplatekCastka"] = filter_input(INPUT_POST,'doplatekCastka',FILTER_VALIDATE_INT);
        $poleNazvu["BenefitPoznamka"] = filter_input(INPUT_POST,'poznamka',FILTER_SANITIZE_SPECIAL_CHARS);


        return $poleNazvu;
    }


    /**
     * @param $idPohybuBenefitu
     * @return bool | Redirect
     * @throws Exception
     */
    public function kontrolaSchvaleniCerpani($idPohybuBenefitu)
    {
        /** @var PohybBenefitu $hodnotyPohybuBenefitu */
        $hodnotyPohybuBenefitu = PohybBenefitu::nactiPohybBenefitu($this->database, $idPohybuBenefitu);

        switch ($hodnotyPohybuBenefitu->druhPohybu){
            case '11':
                if ($hodnotyPohybuBenefitu->datumSchvaleni != ""){
                    //Pokud omylem uživatel ukládá znova (z jiné záložky..) pak již pouze Redirect
                    return false;
                }
                break;

            case '21':
            case '41':
                if ($hodnotyPohybuBenefitu->datumCerpani != ""){
                    //Pokud omylem uživatel ukládá znova (z jiné záložky..) pak již pouze Redirect
                    return false;
                }
                break;

            default:
                return true;
        }

        return true;
    }


    /**
     * Funkce, k zapsání schválení pokud je parametr vydat=false, jinak zapíše čerpání
     * @param $idPohybuBenefitu
     * @param bool $cerpat
     * @return Redirect
     * @throws Exception
     */
    public function schvaleniVydaniOdkazem($idPohybuBenefitu, $cerpat=false)
    {
        /** @var PohybBenefitu $pohybBenefitu */
        $hodnotyPohybuBenefitu = PohybBenefitu::nactiPohybBenefitu($this->database, $idPohybuBenefitu);


        //Pokud je požadavek na "vydání-čerpání" pak kontrola schvaleni - pokud je prázdné pak jej vyplnit také
        $datumCerpani = null;
        $cerpaniUsername = null;

        if ($cerpat) {
            //Čerpání

            if (!$this->pravo_vydat->lzeZobrazitRC($hodnotyPohybuBenefitu->rc)) {
                throw new Exception(ronText('Nemate_pravo'));
            }

            if ($hodnotyPohybuBenefitu->datumCerpani != ""){
                //Pokud omylem uživatel ukládá znova (z jiné záložky..) pak již pouze Redirect
                return new Redirect(null, "?menuid=benefity");
            }

            $datumCerpani = date("Y-n-j", time());
            $cerpaniUsername = User::getLoginName();

            if ($hodnotyPohybuBenefitu->datumSchvaleni != "") {
                $datumSchvaleni = date("Y-n-j", $hodnotyPohybuBenefitu->datumSchvaleni);
                $schvalil = $hodnotyPohybuBenefitu->schvalil;
            } else {
                $datumSchvaleni = date("Y-n-j", time());
                $schvalil = User::getLoginName();
            }

        } else {

            //Dočasné - pokud již má právo na vznik bodů neschválit v jednom kroku s vložením probrat s JiSz ??
            $pravo_vzniku = $this->pravo_vznik->lzeZobrazitRC($hodnotyPohybuBenefitu->rc);
            $druh_vzniku = $hodnotyPohybuBenefitu->druhPohybu == 11;
            if (!($this->pravo_schvalit->lzeZobrazitRC($hodnotyPohybuBenefitu->rc)) && !( $pravo_vzniku && $druh_vzniku)) {
                throw new Exception(ronText('Nemate_pravo'));
            }

            if ($hodnotyPohybuBenefitu->datumSchvaleni != ""){
                //Pokud omylem uživatel ukládá znova (z jiné záložky..) pak již pouze Redirect
                return new Redirect(null, "?menuid=benefity");
            }

            //Pouze schválení - u vzniku a schválení benefitu
            $datumSchvaleni = date("Y-n-j", time());
            $schvalil = User::getLoginName();
        }

        //50 - Kontrola benefitu
        $uloha = 50;

        $params = $uloha .
            ' -BenefitRC:' . $hodnotyPohybuBenefitu->rc .
            ' -BenefitDruhPohybu:' . $hodnotyPohybuBenefitu->druhPohybu .
            ' -BenefitID:' . $hodnotyPohybuBenefitu->id_benefitu .
            ' -BenefitIDPohybu:' . $hodnotyPohybuBenefitu->idPohybu .
            ' -BenefitDatumStavu:' . date("Y-n-j",time()) .
            ' -BenefitDatumSchvaleni:' . $datumSchvaleni .
            ' -BenefitSchvalil:' . $schvalil .
            ' -BenefitDatumRezervaceDo:' . ($hodnotyPohybuBenefitu->datumRezervaceDo != "" ? date("Y-n-j",($hodnotyPohybuBenefitu->datumRezervaceDo)) : NULL) .
            ' -BenefitDatumCerpani:' . ($cerpat ? $datumCerpani : NULL) .
            ' -BenefitCerpaniUsername:' . ($cerpat  ? $cerpaniUsername : NULL) .
            ' -BenefitDatumVlozeni:' . ($hodnotyPohybuBenefitu->datumVlozeni != "" ? date("Y-n-j",$hodnotyPohybuBenefitu->datumVlozeni) : NULL) .
            ' -BenefitDatumOmezeniPlatnosti:' . ($hodnotyPohybuBenefitu->datumOmezeniPlatnosti != "" ? date("Y-n-j",$hodnotyPohybuBenefitu->datumOmezeniPlatnosti) : NULL) .
            ' -BenefitPocetBodu:' . $hodnotyPohybuBenefitu->pocetBodu .
            ' -BenefitPocet:' . $hodnotyPohybuBenefitu->pocetBenefitu .
            ' -BenefitDruhZdroje:' . $hodnotyPohybuBenefitu->druhZdroje .
            ' -BenefitTypDoplatku:' . ($hodnotyPohybuBenefitu->typDoplatku != ""  ? $hodnotyPohybuBenefitu->typDoplatku : 0)  .
            ' -BenefitDoplatekCastka:' . $hodnotyPohybuBenefitu->doplatekCastka .
            ' -BenefitPoznamka:"' . $hodnotyPohybuBenefitu->poznamka .
            '" ';

        require_once('adsapp/agentmanager.php');
        try {
            //Kontrola OK - zapsat pohyb

            AgentManager::executePersonAgent($params);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return new Redirect(null, "?menuid=benefity");
    }


    /**
     * @param $typ
     * @return ComboBox
     * @throws \Ron\DB\DBException
     */
    public function comboTypCasoveSlozky($typ)
    {
        $params = $this->database->createParameters();

        if (trim($this->pravo_sablony->priznak3) != '') {
            $povoleneTypyCS = explode(';', $this->pravo_sablony->priznak3);
            $sqlWhere = "Where " .  SQLUtils::getINCondition($this->database, 'Kod', $povoleneTypyCS, $params);
        } else {
            $sqlWhere = "";
        }

        $sql = "SELECT Kod, Popis FROM TypCasoveSlozkyMzdy " . $sqlWhere . " ORDER BY Popis";

        $combo = new ComboBox("typCasoveSlozkyMzdy");
        $combo->emptyRow = ronText('nevybrano');
        $combo->className .= 'adsfilterforminput_full nonempty';
        $combo->showCode = true;
        $combo->htmlParams = "id='typcasoveslozky' title='" . ronText('benefit_typ_casove_slozky') . "'";

        $combo->loadData($this->database, $sql, $typ, $params);

        return $combo;
    }


    /**
     * @param $benefit
     * @param $osoba
     * @param null $hodnotyPost
     * @return BaseViewObject
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function vytvoritSablonuRezervace($benefit, $osoba, $hodnotyPost=null)
    {
        if (!$this->pravo_sablony->jsouNejakePrava()){
            RonPortalUtils::neniPravoException();
        }

        //Pokud má osoba právo pouze na sebe, pak se osoba nastaví dle přihlášení
        if ($this->pravo_rezervovat->prava == 1 && $osoba == ""){
            $osoba = $this->najdiOsobuRezervace();
        }

        require_once "adsapp/ppomer.php";
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);


        $objektBenefitu = "";
        if ($benefit) {
            require_once('benefity/benefit.php');
            /** @var Benefit $objektBenefitu */
            $objektBenefitu = Benefit::nactiUdajeBenefitu($this->database, $benefit);

            /** @var DokumentyBenefituBrowser $dokumentBenefitu */
            $dokumentBenefitu = new DokumentyBenefituBrowser($this->database, $objektBenefitu->id);
            $dokumentBenefitu->load();

            Model::addData('objektBenefitu', $objektBenefitu);
            Model::addData('dokumentBenefitu', $dokumentBenefitu);
        }

        $jeDochazka = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'ADS');
        $jsouMzdy = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY');

        $nadpis =  ronText('sablona_rezervace_benefitu_vznik') . ($objektBenefitu != "" ? (" - " . StringUtils::htmlTextConvert($objektBenefitu->popis)):"");
        Model::addData('nadpis', $nadpis);
        Model::addData('akce', "sablona");
        Model::addData('druhPohybu', "41");
        Model::addData('osobaSablonyBenefitu', $osoba);
        Model::addData('hodnotyPost', $hodnotyPost);
        Model::addData('dochazkaNainstalovano', $jeDochazka);
        Model::addData('mzdyNainstalovano', $jsouMzdy);

        if ($jeDochazka || $jsouMzdy)
            Model::addData('comboTypCasoveSlozky', $this->comboTypCasoveSlozky(null));

        Model::addData('comboOsobBenefitu', $this->comboOsobBenefitu($this->database, "?menuid=benefity&action=vytvoritsablonurezervace", $this->pravo_rezervovat));
        Model::addData('comboVyberuBenefitu', $this->comboVyberuBenefitu($this->database, $pp->rc, $benefit, "?menuid=benefity&action=vytvoritsablonurezervace&osoba=$pp->idPP"));

        Model::addData('pravo_sablony', Prava::nactiPravaUzivatele($this->database, User::getRC(), 'benefit_sablona'));

        return new BaseViewObject('benefity/view_sablonabenefitu.php');
    }


    /**
     * @param $rc
     * @return array|bool
     * @throws \Ron\DB\DBException
     */
    public function nactiJmenoOsoby($rc)
    {
        $params = $this->database->createParameters();
        $sql = "SELECT PP.OsobniCislo, O.Prijmeni, O.Jmeno FROM PracovniPomer PP LEFT JOIN Osoba O ON (PP.RC = O.RC) WHERE PP.RC = " . $params->addParameter($rc);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        $osoba = $row[0] . " " . $row[1] . " " . $row[2];

        $cursor->close();

        return $osoba;
    }


    /**
     * @param $benefit
     * @return string
     * @throws \Ron\DB\DBException
     */
    public function nactiPopisBenefituSablony($benefit)
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Popis FROM Benefit WHERE ID = " . $params->addParameter($benefit);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        $benefit = $row[0];
        $cursor->close();

        return $benefit;
    }


    /** Detail šablony
     * @param $idSablony
     * @param null $osoba
     * @return BaseViewObject
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function upravitSablonuRezervace($idSablony, $osoba)
    {
        if ($idSablony) {
            require_once('benefity/benefit.php');
            $objektSablonyBenefitu = SablonaBenefitu::nactiUdajeSablonyBenefitu($this->database, $idSablony);

            Model::addData('objektSablonyBenefitu', $objektSablonyBenefitu);
        } else {
            throw new Exception(ronText('id_sablony_neni_zadano'));
        }

        if (!$this->pravo_sablony->lzeZobrazitRC($objektSablonyBenefitu->rc)){
            RonPortalUtils::neniPravoException();
        }

        require_once('benefity/benefit.php');
        /** @var Benefit $objektBenefitu */
        $objektBenefitu = Benefit::nactiUdajeBenefitu($this->database, $objektSablonyBenefitu->idBenefitu);

        /** @var DokumentyBenefituBrowser $dokumentBenefitu */
        $dokumentBenefitu = new DokumentyBenefituBrowser($this->database, $objektBenefitu->id);
        $dokumentBenefitu->load();

        $jeDochazka = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'ADS');
        $jsouMzdy = RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY');

        $nadpis =  ronText('sablona_rezervace_benefitu_detail') . ($objektBenefitu != "" ? (" - " . StringUtils::htmlTextConvert($objektBenefitu->popis)):"") ;
        Model::addData('nadpis', $nadpis);
        Model::addData('id', $objektSablonyBenefitu->id);
        Model::addData('druhPohybu', $objektSablonyBenefitu->druhPohybu);
        Model::addData('objektBenefitu', $objektBenefitu);
        Model::addData('dokumentBenefitu', $dokumentBenefitu);
        Model::addData('dochazkaNainstalovano', $jeDochazka);
        Model::addData('mzdyNainstalovano', $jsouMzdy);
        Model::addData('osobaSablonyBenefitu', $this->najdiOsobuRezervace($objektSablonyBenefitu->rc));

        if ($jeDochazka || $jsouMzdy)
            Model::addData('comboTypCasoveSlozky', $this->comboTypCasoveSlozky($objektSablonyBenefitu->typCasoveSlozkyMzdy));

        //Pouze výpis - zobrazení detailu
        Model::addData('osobaSablony', $this->nactiJmenoOsoby($objektSablonyBenefitu->rc));
        Model::addData('popisBenefitSablony', $this->nactiPopisBenefituSablony($objektSablonyBenefitu->idBenefitu));

        //Vytvoření nové šablony
        Model::addData('comboOsobBenefitu', $this->comboOsobBenefitu($this->database, "?menuid=benefity&action=sablonarezervace", $this->pravo_rezervovat));
        Model::addData('comboVyberuBenefitu', $this->comboVyberuBenefitu($this->database, $objektSablonyBenefitu->rc,
            $objektSablonyBenefitu->idBenefitu, "?menuid=benefity&action=detailsablonyrezervace&osoba=$osoba&sablona=$idSablony"));

        return new BaseViewObject('benefity/view_sablonabenefitu.php');
    }


    /**
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function vlozitSablonuBenenfitu()
    {
        /** @var SablonaBenefitu $hodnoty */
        $hodnoty = $this->nactiPOSTProDialog();

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'SablonaRezervaceBenefitu');
        $rec->logDescription = "Vložení/editace šablony benefitu - portál";

        require_once "adsapp/ppomer.php";
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $hodnoty->osoba);

        if ($hodnoty->id == "") {
            require_once('ron/sqlutils.php');
            $id = SQLUtils::newSerialNumber($this->database, 'SablonaRezervaceBenefitu', 'ID');
        } else {
            $id = $hodnoty->id;
        }

        $rec->addField('Id', 'int', true, 0, $id);
        $rec->addField('RC', '', true, 15, $pp->rc);
        $rec->addField('DruhPohybu', 'int', false, 0, $hodnoty->druhPohybu);
        $rec->addField('IdBenefitu', 'int', true, 0, $hodnoty->idBenefitu);
        $rec->addField('Poradi', 'int', false, 0, $hodnoty->poradi);
        $rec->addField('PocetBenefitu', 'int', false, 0, (((int)$hodnoty->zdrojPoctuBenefitu != 0) ? 1 : $hodnoty->pocetBenefitu));
        $rec->addField('PocetBodu', 'int', false, 0, $hodnoty->pocetBodu);
        $rec->addField('TypDoplatku', 'int', false, 0, $hodnoty->typDoplatku);
        $rec->addField('DoplatekCastka', 'float', false, 0, $hodnoty->doplatekCastka);
        $rec->addField('TypKontrolySablony', 'int', false, 0, $hodnoty->typKontrolySablony == "" ? 0 : $hodnoty->typKontrolySablony);
        $rec->addField('CasovaJednotka', 'int', false, 0, $hodnoty->casovaJednotka);
        $rec->addField('TypCasoveSlozkyMzdy', '', false, 10, $hodnoty->typCasoveSlozkyMzdy);
        $rec->addField('ZdrojPoctuBenefitu', 'int', false, 0, $hodnoty->zdrojPoctuBenefitu);
        $rec->addField('PocetCasJednotek', 'int', false, 0, $hodnoty->pocetCasJednotek);
        $rec->addField('Insert_UserName', '', true, 25, User::getLoginName());
        $rec->addField('Insert_Timestamp', 'timestamp', true, 0, time());
        $rec->addField('Update_UserName', '', false, 25, $hodnoty->id == "" ? NULL : User::getLoginName());
        $rec->addField('Update_Timestamp', 'timestamp', false, 0, $hodnoty->id == "" ? NULL : time());

        $params = $this->database->createParameters();
        $sqlWhere =
            ' ID = ' . $params->addParameter($id);

        $rec->setWhere($sqlWhere, $params);

        if ($rec->loadRecord() == 0) {
            if (!$rec->insertRecord()) {
                throw new Exception(StringUtils::fromUTF8('Chyba pri vložení šablony benefitu!'));
            }
        } else {
            if (!$rec->loadAndUpdateRecord()) {
                throw new Exception(StringUtils::fromUTF8('Chyba pri aktualizaci šablony benefitu!'));
            }
        }


        return new  Redirect(null, "?menuid=benefity&action=browsersablonybenefitu");
    }


    /**
     * @param Database $database
     * @return BrowserSablonyBenefitu
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function browserSablonyBenefitu($database)
    {
        if (!$this->pravo_sablony->lzeZobrazitRC(User::getRC())){
            RonPortalUtils::neniPravoException();
        }

        require_once 'benefity/b_sablonybenefitu.php';
        $b = new BrowserSablonyBenefitu($database);
        $b->load();
        return $b;
    }


    /**
     * @param $id
     * @return Redirect
     * @throws \Ron\DB\DBException
     * @throws Exception
     */
    public function odstranitSablonu($id)
    {
        $params = $this->database->createParameters();

        require_once('adsapp/ronportaldbrecord.php');
        $smazat = new RonPortalDBRecord($this->database, 'SablonaRezervaceBenefitu');
        $smazat->logDescription = "Odstranění šablony benefitu - portál";

        $smazat->addField('Id', 'int', false);
        $smazat->addField('RC', '', false);
        $smazat->addField('DruhPohybu', 'int', false);
        $smazat->addField('IdBenefitu', 'int', false);
        $smazat->addField('Poradi', 'int', false);
        $smazat->addField('PocetBenefitu', 'int', false);
        $smazat->addField('PocetBodu', 'int', false);
        $smazat->addField('TypDoplatku', 'int', false);
        $smazat->addField('DoplatekCastka', 'float', false);
        $smazat->addField('TypKontrolySablony', 'int', false);
        $smazat->addField('CasovaJednotka', 'int', false);
        $smazat->addField('PocetCasJednotek', 'int', false);
        $smazat->addField('Insert_UserName', '', true);
        $smazat->addField('Insert_Timestamp', 'timestamp');
        $smazat->addField('Update_UserName', '', false);
        $smazat->addField('Update_Timestamp', 'timestamp', false);

        //Odstranit šablonu jiné osoby lze pouze přes aplikaci
        $smazat->setWhere("ID = " . $params->addParameter($id) .
                            " AND RC = " . $params->addParameter(User::getRC()) , $params);

        if (!$smazat->deleteRecord()){
            throw new Exception(StringUtils::fromUTF8("Chyba při odstranění šablony"));
        }

        return new  Redirect(null, "?menuid=benefity&action=browsersablonybenefitu");
    }

    function translations()
    {
        require_once('ron/utils.php');
        return new TranslationsOutputObject();
    }

    /**
     * @param string $action
     * @return mixed
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!($this->pravo_vznik->jsouNejakePrava() || $this->pravo_schvalit->jsouNejakePrava() ||
            $this->pravo_vydat->jsouNejakePrava() || $this->pravo_rezervovat->jsouNejakePrava() || $this->pravo_sablony->jsouNejakePrava())){
            RonPortalUtils::neniPravoException();
        }


        $rezervaceBenefituForm = isset($_POST['rezervacebenefituform']) ? $_POST['rezervacebenefituform'] : "";
        $osoba = isset($_POST['osoba']) ? $_POST['osoba'] : $_GET['osoba'];
        $benefit = isset($_POST['benefit']) ? $_POST['benefit'] : $_GET['benefit'];
        $sablona = isset($_POST['sablona']) ? $_POST['sablona'] : $_GET['sablona'];
        $idPohybuBenefitu = isset($_POST['idpohybubenefitu']) ? $_POST['idpohybubenefitu'] : $_GET['idpohybubenefitu'];

        if ($rezervaceBenefituForm){
            return $this->zapsatPohyb($osoba);
        }


        switch ($_GET['action']){

            case 'detailpohybu':
                return $this->detailPohybuBenefitu($idPohybuBenefitu, null);

            case 'vznikbodu':
                return $this->vznikBodu($osoba);

            case 'rezervacebodu':
                return $this->rezervaceBenefitu($osoba, $benefit);

            case 'schvalenirezervace':
                //Přes odkaz
                return $this->schvaleniVydaniOdkazem($idPohybuBenefitu);

            case 'cerpanirezervace':
                //Přes odkaz
                return $this->schvaleniVydaniOdkazem($idPohybuBenefitu, true);


            case 'browsersablonybenefitu':
                return $this->browserSablonyBenefitu($this->database);

            case 'vytvoritsablonurezervace':
                return $this->vytvoritSablonuRezervace($benefit, $osoba);

            case 'detailsablonyrezervace':
                return $this->upravitSablonuRezervace($sablona, $osoba);

            case 'zapissablony':
                return $this->vlozitSablonuBenenfitu();

            case 'odstranitsablonu':
                return $this->odstranitSablonu($_GET['record']);

            case 'translations':
                return $this->translations();

            case 'dokumentdownload':
                return $this->downloadDokument((int)$_GET['id'], (int)$_GET['record']);

            default:
                //Generování zániků (nutné pro rezervování z bodů), standardně by měl spustit agent automatizovaně
                //(zde jen pro případ, že se tak nestane, dohodnuto s DaVa)
                $this->generovaniZaniku();

                //Pří zobrazeni Browseru se maže seznam ID benefitů v session aby se vždy vytvořil aktuální seznam při každé změně filtrů
                $_SESSION['pocetbenefitufilru'] = '';
                $_SESSION['seznambenefitu'] = '';

                require_once('benefity/b_benefity.php');
                $b = new BenefityBrowser($this->database);
                $b->load();
                return $b;
        }
    }
}