<?php
/**
 * UTF8
 * stanislav.elbl, 31.07.2015
 * roman.szablatura, 4.5.2016
 */

namespace Ron\RonPortal;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MVC;
use Ron\MVC\ViewObject;
use Ron\Personalistika\DistribuovaneDokumentyDetail;
use Ron\TextManager;
use RonPortal\Common\PotvrzeniPrecteniForm;

require_once('adsapp/dokumentyutils.php');

class DokumentyController implements MVC\Controller
{
    /** @var Database */
    var $database;

    private $osobaRC;
    private $dokumentPrihlaseneho = true;


    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance()->loadStrings('person.ini');
        HtmlHeaders::getInstance()->addCss('css/person.css');
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
    }


    /**
     * @throws DBException
     */
    private function nactiOsobu()
    {
        if (isset($_GET['osoba'])) {
            $params = $this->database->createParameters();
            $this->osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($_GET['osoba']), $params);
            $this->dokumentPrihlaseneho = false;
        }
        elseif (isset($_POST['idosoby'])) {
            $params = $this->database->createParameters();
            $this->osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($_POST['idosoby']), $params);
            $this->dokumentPrihlaseneho = false;
        }
        else {
            $this->osobaRC = User::getRC();
        }
    }

    /**
     * @param int $idDokumentu
     * @param string $rcOsoby
     * @throws DBException
     * @throws Exception
     */
    private function overPravoNaTypDokumentu($idDokumentu, $rcOsoby)
    {
        $params = $this->database->createParameters();
        $typPrava = DBUtils::getSqlValue($this->database,"SELECT DefinicePrava FROM TypDokumentu JOIN Dokument ON (Dokument.typ = TypDokumentu.kod) WHERE Dokument.id = " . $params->addParameter($idDokumentu), $params);

        if ($typPrava) {
            $prava = Prava::nactiPravaUzivatele($this->database,User::getRC(),$typPrava);

            if (!$prava->lzeZobrazitRC($rcOsoby, time(), time())) {
                RonPortalUtils::neniPravoException();
            }
        }
    }

//    /**
//     * @param Database $database
//     * @return bool
//     * @throws Exception
//     */
//    public static function lzeZobrazitVyplatniListky($database)
//    {
//        $params = $database->createParameters();
//
//        $sql =
//            "SELECT zobrazitNaIntrawebu FROM TypDokumentu WHERE kod = " . $params->addParameter(-1);
//
//        $cursor = $database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        $cursor->close();
//
//        return !!$row[0];
//    }

    /**
     * @return DokumentUploadOutput
     */
    public function uploadDokument()
    {
        $result = new DokumentUploadOutput();

        try {
            if (!Prava::vratPrava('prohl_osobni')->lzeZobrazitRC($this->osobaRC)) {
                throw new NeniPravoException();
            }

            // vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $_POST['typ'], $_POST['popis'], 'Dokument osoby');

            // Vlozi zaznam do vazebni tabulky
            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby');
            $rec ->logDescription = 'Dokument osoby';
            $rec ->addField('RC', '', false, 15, $this->osobaRC);
            $rec ->addField('Id', 'int', false, 0, $id);
            $rec ->insertRecord();
        } catch (Exception $e) {
            $result ->setError($e ->getMessage());
        }

        return $result;
    }

    /**
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    public function downloadDokumentOsoby($idDokumentu) {
        // zkontroluje, jestli je zadany dokument prirazen uzivateli
        // a je povoleno jeho zobrazeni na IW
        if (!$this->dokumentPrihlaseneho) {
            $this->overPravoNaTypDokumentu($idDokumentu,$this->osobaRC);
        }

        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Id ' .
            ' FROM Dokument, TypDokumentu, DokumentyOsoby ' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
            '   AND Dokument.Id = DokumentyOsoby.Id ' .
            '   AND TypDokumentu.Kod = Dokument.Typ ' .
            '   AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            "   AND (DokumentyOsoby.RC = " . $params->addParameter($this->osobaRC) . ")";

        if (!$cursor = $this ->database ->getSqlResult($sql, $params)) {
            throw new DBException($this ->database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws Exception
     */
    function downloadDokumentRozdeleni($idDokumentu)
    {
        if (!$this->dokumentPrihlaseneho) {
            $this->overPravoNaTypDokumentu($idDokumentu, $this->osobaRC);
        }

        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Id ' .
            ' FROM Dokument, TypDokumentu, DokumentyRozdeleni, RozdeleniOsob ' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
            ' AND Dokument.Id = DokumentyRozdeleni.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni ' .
            ' AND RozdeleniOsob.KodTypuRozdeleni <> -1 ' . // dokumenty prac. mist jsou zvlast
            ' AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod ' .
            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->osobaRC) . ")" .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)';

        if (!$cursor = $this ->database ->getSqlResult($sql, $params)) {
            throw new DBException($this ->database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws DBException
     * @throws Exception
     */
    function downloadDokumentPracMista($idDokumentu)
    {
        if (!$this->dokumentPrihlaseneho) {
            $this->overPravoNaTypDokumentu($idDokumentu, $this->osobaRC);
        }

        $params = $this->database->createParameters();

        $sql =
            'SELECT Dokument.Id ' .
            ' FROM Dokument, TypDokumentu, DokumentyRozdeleni, RozdeleniOsob ' .
            ' WHERE Dokument.Id = ' . $params->addParameter($idDokumentu) .
            ' AND Dokument.Id = DokumentyRozdeleni.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni ' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = -1 ' . // jen dokumenty prac. mist
            ' AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod ' .
            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->osobaRC) . ")" .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)';


        if (!$cursor = $this ->database ->getSqlResult($sql, $params)) {
            throw new DBException($this ->database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }

    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @return DokumentOutputObject
     * @throws DBException
     * @throws Exception
     */
    function downloadDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu)
    {
        if (!$this->dokumentPrihlaseneho) {
            $this->overPravoNaTypDokumentu($idDokumentu, $this->osobaRC);
        }

        $params = $this->database->createParameters();

        $sql =
            'SELECT  VerzeDokumentu.id ' .
            'FROM VerzeDokumentu JOIN DokumentVzkazu ON (VerzeDokumentu.id = DokumentVzkazu.idDokumentu AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd)' .
            'JOIN Vzkaz ON (Vzkaz.Id = DokumentVzkazu.IdVzkazu) ' .
            'WHERE VerzeDokumentu.id = ' . $params->addParameter($idDokumentu) .
            ' AND VerzeDokumentu.PlatnostOd = ' . $params->addParameter($this->database->timestampToParam($platnostOd)) .
            ' AND Vzkaz.RCPrijemce = ' . $params->addParameter($this->osobaRC) .
            ' AND DokumentVzkazu.IdVzkazu = ' . $params->addParameter($idVzkazu);


        if (!$cursor = $this ->database ->getSqlResult($sql, $params)) {
            throw new DBException($this ->database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            // dokument neexistuje, nebo neni prirazen osobe
            throw new Exception('Chyba! Zadany dokument nebyl nalezen!');
        }


        return new DokumentOutputObject($this ->database, $idDokumentu, true, $platnostOd);
    }


    /**
     * @return MVC\Redirect|PotvrzeniPrecteniForm
     * @throws Exception
     */
    private function potvrditPrecteni()
    {
        $idDokumentu = isset($_POST['dokumentid']) ? (int)$_POST['dokumentid'] : (int)$_GET['dokumentid'];
        $platnostOd = isset($_POST['platnostod']) ? $_POST['platnostod'] : $_GET['platnostod'];
        $idVzkazu = isset($_POST['vzkazid']) ? (int)$_POST['vzkazid'] : (int)$_GET['vzkazid'];

        $platnostOd = DateUtils::dateStringToTimestamp($platnostOd);

        //kontrola jestli je vzkaz pro prihlasenou osobu a jestli uz vzkaz nebyl potvrzen
        $this->kontrolaPrihlaseneOsoby($idDokumentu, $platnostOd,$idVzkazu);

        $f = new PotvrzeniPrecteniForm($this->database, $idDokumentu, $platnostOd, $idVzkazu);

        if ($f->handleAction()) {
            return new MVC\Redirect(null,'?menuid=dokumenty');
        }
        return $f;
    }

    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @throws DBException
     * @throws Exception
     */
    private function kontrolaPrihlaseneOsoby($idDokumentu, $platnostOd, $idVzkazu)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT RCPrijemce, Precteno ' .
            'FROM Vzkaz JOIN DokumentVzkazu ON (DokumentVzkazu.idvzkazu = Vzkaz.id) JOIN Osoba ON (Vzkaz.RCPrijemce = Osoba.RC) ' .
            'WHERE IdVzkazu = ' . $params->addParameter($idVzkazu) .
            ' AND idDokumentu = ' . $params->addParameter($idDokumentu) .
            ' AND PlatnostOd = ' . $params->addParameter($this->database->timestampToParam($platnostOd));

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row[0] != User::getRC()) {
            RonPortalUtils::neniPravoException();
        }

        if ($row[1] == 1) {
            throw new Exception(\Ron\ronText('precteni_potvrzeno'));
        }
    }

    /**
     * @param int $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @return DistribuovaneDokumentyDetail
     * @throws Exception
     */
    private function detailDokumentu($idDokumentu, $platnostOd, $idVzkazu)
    {
        $prava = Prava::vratPrava('dokumenty_prohl');

        if (!$prava->jsouNejakePrava())
            RonPortalUtils::neniPravoException();

        require_once ('person/b_distribuovanedokumenty.php');

        $b = new DistribuovaneDokumentyDetail($this->database, $idDokumentu, $platnostOd, $idVzkazu);
        $b->load();

        return $b;
    }

    /**
     * @return DokumentOutputObject|VyplatniListkyBrowser
     * @throws Exception
     */
    private function handleVyplatniListky()
    {
        if (User::isLoggedAs()) {
            // pri "prihlaseni jako" vyplatni listky nezobrazovat!
            throw new NeniPravoException();
        }
        if (!DokumentyUtils::lzeZobrazitVyplatniListky($this->database)) {
            throw new NeniPravoException();
        }

        // stahnuti vyplatniho listku osoby
        if (isset($_GET['record'])) {
            $this->osobaRC = User::getRC();
            $this->dokumentPrihlaseneho = true;
            return $this ->downloadDokumentOsoby((int) $_GET['record']);
        }

        require_once('person/b_vyplatnilistky.php');

        TextManager::getInstance()->loadStrings('users.ini');

        $browser = new VyplatniListkyBrowser($this->database);
        $browser->load();
        return $browser;
    }

    /**
     * @param string $action
     * @return ViewObject|DokumentOutputObject|DokumentUploadOutput
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOKUMENTY)) {
            throw new Exception('Zvolena funkce neni povolena!');
        }

        if ($action == 'vyplatnilistky')
            return $this->handleVyplatniListky();

        $idDokumentu = isset($_POST['dokumentid']) ? (int)$_POST['dokumentid'] : (isset($_GET['dokumentid']) ? (int)$_GET['dokumentid'] : null);
        $idVzkazu = isset($_POST['vzkazid']) ? (int)$_POST['vzkazid'] : (isset($_GET['vzkazid']) ? (int)$_GET['vzkazid'] : null);
        $platnostOd = isset($_POST['platnostod']) ? $_POST['platnostod'] : (isset($_GET['platnostod']) ? $_GET['platnostod'] : null);
        if ($platnostOd) {
            $platnostOd = DateUtils::dateStringToTimestamp($platnostOd);
        }

        $this->nactiOsobu();

        if (isset($_GET['action']) && ($_GET['action'] == 'dokumentupload')) {
            return $this->uploadDokument();
        }

        // stahnuti dokumentu osoby
        if (isset($_GET['dokumentosoby'])  &&  $_GET['dokumentosoby']  &&  isset($idDokumentu)) {
            return $this ->downloadDokumentOsoby((int)$idDokumentu);
        }

        // stahnuti dokumentu rozdeleni
        if (isset($_GET['dokumentrozdeleni'])  &&  $_GET['dokumentrozdeleni']  &&  isset($idDokumentu)) {
            return $this ->downloadDokumentRozdeleni((int)$idDokumentu);
        }

        // stahnuti dokumentu pracovniho mista
        if (isset($_GET['dokumentpmista'])  &&  $_GET['dokumentpmista']  &&  isset($idDokumentu)) {
            return $this ->downloadDokumentPracMista((int)$idDokumentu);
        }

        // stahnuti distribuovaneho dokumentu
        if(isset($_GET['distribuovanedokumenty'])  &&  $_GET['distribuovanedokumenty'] && $idDokumentu && $idVzkazu && $platnostOd){
            return $this->downloadDistribuovanyDokument($idDokumentu, $platnostOd, $idVzkazu);
        }

        //Potvrzeni precteni
        if(isset($_GET['potvrzeniprecteniform'])  &&  $_GET['potvrzeniprecteniform']){
            return $this->potvrditPrecteni();
        }

        //Detail distribuovaneho dokumentu
        if(isset($_GET['distribuovanedetail']) && $_GET['distribuovanedetail'] && $idDokumentu && $idVzkazu && $platnostOd){
            return $this->detailDokumentu($idDokumentu, $platnostOd, $idVzkazu);
        }

        require_once('person/b_dokumentyosoby.php');

        // prozatim jen zobrazeni dokumentu osoby
        // Seznam dokumentu
        $result = new DokumentyOsobyBrowser($this ->database);
        $result ->load();
        return $result;
    }
}

class NovyDokumentOsobyForm extends DokumentUploadForm
{
    private $idOsoby = 0;


    function __construct($database, $idPP)
    {
        parent::__construct($database, '?menuid=dokumenty&action=dokumentupload');
        $this ->idOsoby = $idPP;

        $this ->hiddenFields = [
            'dokumentdosoby' => '1',
            'idosoby' => $this ->idOsoby,
        ];
    }
}

return new DokumentyController($this ->database);
