<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 27.12.2017
 */

namespace Ron\RonPortal\Dochazka;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\DochazkaUtils;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\PrirazeniKalendare;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\User;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\StringUtils;
use Ron\TextManager;

require_once('ron/controls.php');
require_once('adsapp/dochazkautils.php');
require_once('dochazka/prirazenikalendare.php');
require_once('adsapp/ronportaldbrecord.php');
require_once('adsapp/ronportalfilter.php');


/**
 * Class PrirazeniKalendareFilterForm
 * @package Ron\RonPortal\Dochazka
 */
class PrirazeniKalendareFilterForm extends RonPortalFilterForm
{
    /**
     * PrirazeniKalendareFilterForm constructor.
     * @param Database $database
     * @param string $osoba
     */
    public function __construct($database, $osoba)
    {
        parent::__construct($database, 'kalendarosoby', true);

        $this ->addHiddenField('action', 'detailkalendare');
        $this ->addHiddenField('osoba', (int)($osoba));

        $this ->skrytTlacitkoNastavit = true;
        $this ->nastavObdobi(true, 'ADS', true, true);

        $this ->formClass = '';

        $this ->showMessages = false;
    }
}


/**
 * Class KalendarOsobyController
 * @package Ron\RonPortal\Dochazka
 */
class KalendarOsobyController implements Controller
{
    /** @var  Database $database */
    private $database;

    /** @var  PravaNaOsoby $pravo */
    public $pravo;


    /**
     * KalendarOsobyController constructor.
     * @param Database $database
     * @param int $osoba
     * @throws \Exception
     */
    public function __construct($database,  $osoba)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo = Prava::nactiPravaUzivatele($this->database, User::getRC(), 'prirazenikalendare');
    }


    /**
     * @param Database $database
     * @param $idPP
     * @param $datumOd
     * @return array
     * @throws \Ron\DB\DBException
     */
    public function nactiKalendarOsoby($database, $idPP, $datumOd)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT PrirazeniKalendare.Kalendar, Kalendar.Popis, PrirazeniKalendare.PlatnostOd, PrirazeniKalendare.PlatnostDo ' .
            '  FROM PrirazeniKalendare, Kalendar, PracovniPomer ' .
            ' WHERE PrirazeniKalendare.Kalendar = Kalendar.Kod ' .
            '   AND (PrirazeniKalendare.PlatnostOd >= ' . $params->addParameter($database->dateToParam($datumOd)) .
            '   OR PrirazeniKalendare.PlatnostDo IS NULL  OR  PrirazeniKalendare.PlatnostDo >= ' . $params->addParameter($database->dateToParam($datumOd)) . ')' .
            '   AND PrirazeniKalendare.RC = PracovniPomer.RC' .
            '   AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND PracovniPomer.Id = ' . $params->addParameter($idPP) .
            ' ORDER BY PrirazeniKalendare.PlatnostOd ';

        $cursor = $database->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor->fetch()) {
            $kalendar = new PrirazeniKalendare();
            $kalendar->kod = $row[0];
            $kalendar->popis = $row[1];
            $kalendar->platnostOd = $cursor ->stringToTimestamp($row[2]);
            $kalendar->platnostDo = $cursor ->stringToTimestamp($row[3]);

            $result[] = $kalendar;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param int $osoba
     * @return BaseViewObject
     * @throws \Exception
     */
    private function detailKalendareOsoby($osoba)
    {
        $editace_povolena = RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA);

        $filterForm = new PrirazeniKalendareFilterForm($this->database, $osoba);
        $filterForm ->loadData();

        if ($osoba != "") {
            //nacist osobu z GET[osoba] a pri neuspechu ze session
            $idPP = $osoba;

            require_once('adsapp/ppomer.php');
            $pracovniPomer = PracovniPomer::nactiPP($this ->database, $idPP);
            $pracovniPomer->vyberNactenyPP($this->database, $this->pravo);
        } else {
            require_once('adsapp/ppomer.php');
            $pracovniPomer = PracovniPomer::nactiPP($this ->database, RonPortalSession::vybranaOsoba());
            //$this->pracovniPomer->nactiPPPodleRC($this->database, RonPortalSession::osobaRC(), RonPortalSession::osobaOsobniCislo(), RonPortalSession::osobaNastup());
        }

        $zacatekObdobi = RonPortalSession::obdobi();
        $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer->idPP, $zacatekObdobi);

        if ($editace_povolena){
            $nadpis = \Ron\ronText('zmena_kalendare_osoby');
        } else {
            $nadpis = \Ron\ronText('detail_kalendare_osoby');
        }

        Model::addData('nadpis', $nadpis);
        Model::addData('pp', $pracovniPomer);
        Model::addData('editace_povolena', $editace_povolena);
        Model::addData('filtrObdobi', $filterForm);
        Model::addData('kalendare', $kalendareVDanemObdobi);
        Model::addData('comboBox', $this->comboBoxKalendarePrirazeni($this->database));
        Model::addData('schvaleniDochazky', $this->kontrolaSchvaleniDochazky($pracovniPomer, $zacatekObdobi));

        return new BaseViewObject('editace/view_zmenakalendare.php');
    }

    /**
     * @param Database $database
     * @return ComboBox
     * @throws \Exception
     */
    public function comboBoxKalendarePrirazeni($database)
    {
        $comboBox = new ComboBox('kalendar_osoby');
        $comboBox->className = 'form-input';
        $comboBox->showCode = true;

        require_once 'xml/xmlformportal.php';
        require_once 'adsapp/events.php';

        $osoba = [
            'IDPP' => RonPortalSession::vybranaOsoba(),
            'ObdobiOd' => $database->timestampToString(RonPortalSession::obdobi()),
            'ObdobiDo' => $database->timestampToString(RonPortalSession::konecObdobi())
        ];

        $select = trim(RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::PRIRAZENI_KALENDARE_SELECT, $osoba, '', true));

        if ($select)
            $comboBox->loadData($database, $select, '');
        else
            $comboBox->loadData($database,
                'SELECT Kod, Popis FROM Kalendar Where Aktivni = 1 ORDER BY Kod',
                '');

        return $comboBox;
    }

    /**
     * @return int|null
     * @throws \Ron\DB\DBException
     */
    private function kontrolaObdobi()
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT Max(konecObdobi)
                 FROM obdobi
                 WHERE (uzavreno <> 0)";

        MessageManager::addDebugLog('Obdobi: ' . $sql);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return $this->database->stringToTimestamp($row[0]);
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param int $platnostOd
     * @return mixed
     * @throws \Ron\DB\DBException
     */
    private function kontrolaSchvaleniDochazky($pracovniPomer, $platnostOd)
    {
        $params = $this->database->createParameters();

        $pocatekObdobi = DateUtils::firstDayOfMonth($platnostOd);

        $sql =
            'SELECT Max(ZpracovanoDo), Max(ZkontrolovanoDo), Max(SchvalenoDo), Max(ZpracovanoVeMzdach)' .
            ' FROM SchvaleniDochazky' .
            " WHERE Aplikace = 'ADS'" .
            ' AND RC = ' . $params->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($this->database->dateToParam($pracovniPomer ->datumNastupu)) .
            ' AND PocatekObdobi = ' . $params ->addParameter($this->database ->dateToParam($pocatekObdobi));

        MessageManager::addDebugLog('Schvaleni: ' . $sql);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $datum = $row[0];
        $zpracovanoDo = [];

        if ($row[3] == 0)
        {
            //Neni zpracovano ve mzdach
            foreach ($row as $d) {
                if ($datum < $d) {
                    $datum = $d;
                }
            }
            $zpracovanoDo[0] = $this->database->stringToTimestamp($datum);
            $zpracovanoDo[1] = 0;

        } else {
            //Obdobi je zpracovano ve mzdach proto nastavuji datum na posledni den v mesici - prirazeni kalendare je mozne az po tomto datu
            $zpracovanoDo[0] = DateUtils::lastDayOfMonth($platnostOd);
            $zpracovanoDo[1] = 1;
        }
        $cursor ->close();

        return $zpracovanoDo;
    }


    /**
     * @param $osoba
     * @return Redirect
     * @throws \Exception
     */
    private function editaceKalendareOsoby($osoba)
    {
        try {
            if ($osoba == '') {
                throw new \Exception(\Ron\ronText('osoba_neni_vybrana'));
            }

            require_once('adsapp/ppomer.php');

            $pracovniPomer = PracovniPomer::nactiPP($this->database, (int)$osoba);
            $pracovniPomer->vyberNactenyPP($this->database, $this->pravo);


            if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
                throw new \Exception('editace_neni_povolena');
            }

            $platnostOdZmeny = DateUtils::dateStringToTimestamp($_POST['Platnost_od']);
            if (!$platnostOdZmeny) {
                throw new \Exception(\Ron\ronText('platnost_od_neni_zadana'));
            }

            if ($_POST['platnost_do'] != '') {
                $platnostDoZmeny = DateUtils::dateStringToTimestamp($_POST['Platnost_do']);
            } else {
                $platnostDoZmeny = '';
            }


            $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer->idPP, RonPortalSession::obdobi());
            $kalendarZmeny = $_POST['kalendar_osoby'];

            if (!$kalendareVDanemObdobi) {
                throw new \Exception(\Ron\ronText('obdobi_nema_kalendar'));
            }

            if ($platnostOdZmeny != '' && $platnostDoZmeny != '') {
                if ($platnostOdZmeny > $platnostDoZmeny) {
                    throw new \Exception('platnost_do_je_mala');
                }
            }

            //Kontroly - zmenu umoznit jen pro data ktere nejsou schvalene a obdobi je otevrene.
            $this->kontroly($pracovniPomer, $platnostOdZmeny);


            //Zmena pouze s platnosti od
            if ($platnostOdZmeny != '' && $platnostDoZmeny == '') {
                $this->upravaKalendareTrvalaZmena($pracovniPomer, $kalendareVDanemObdobi, $platnostOdZmeny, $platnostDoZmeny, $kalendarZmeny);
            }

            //Zmena s platnosti od-do
            if ($platnostOdZmeny != '' && $platnostDoZmeny != '') {
                $this->upravaKalendareDocasnaZmena($pracovniPomer, $kalendareVDanemObdobi, $platnostOdZmeny, $platnostDoZmeny, $kalendarZmeny);
            }


            //Provest prepocet - zpracovat zadane obdobi podle puvodni dochazky ($uloha=8),
            //ale jen maximalne do data do kdy je dochazka zpracovana.
            $zpracovat = $_POST['zpracovat'];

            if ($zpracovat == 1) {
                //zpracovat zadane obdobi podle puvodni dochazky - 8
                $uloha = 8;
                $this->upravaKalendarePrepocetDochazky($pracovniPomer->idPP, $platnostOdZmeny, $uloha);
            }

            return new Redirect(null, '?menuid=kalendarosoby&action=detailkalendare&osoba=' . $osoba);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);

            return $this ->detailKalendareOsoby($osoba);
        }
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param $platnostOdZmeny
     * @throws \Exception
     */
    public function kontroly($pracovniPomer, $platnostOdZmeny)
    {
        if ($platnostOdZmeny <= $this->kontrolaObdobi()){
            throw new \Exception(\Ron\ronText('obdobi_uzavrene'));
        }

        $dochazkaSchvalena = $this->kontrolaSchvaleniDochazky($pracovniPomer, $platnostOdZmeny);

        if ($dochazkaSchvalena[1]){
            throw new \Exception(\Ron\ronText('obdobi_zpracovano'));
        }

        if ($platnostOdZmeny <= $dochazkaSchvalena[0]){
            throw new \Exception(\Ron\ronText('dochazka_schvalena'));
        }
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param $kalendareVDanemObdobi
     * @param int $zmenaOd
     * @param int $zmenaDo
     * @param $kalendarZmeny
     * @throws \Exception
     */
    private function upravaKalendareDocasnaZmena($pracovniPomer, $kalendareVDanemObdobi, $zmenaOd, $zmenaDo, $kalendarZmeny)
    {
        $platnostOdZmeny = $zmenaOd;
        $platnostDoZmeny = $zmenaDo;

        //Prekryti - nejdrive odstrani prekryte
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi) {

            $puvodniOd = $kalendarVDanemObdobi->platnostOd;
            $puvodniDo = $kalendarVDanemObdobi->platnostDo;

            if ($platnostOdZmeny <= $puvodniOd && $puvodniDo <= $platnostDoZmeny && $puvodniDo != '')
            {
                $this->odstraneniKalendare($pracovniPomer, $puvodniOd);
            }
        }

        //Zmena je v ramci
        $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer ->idPP, RonPortalSession::obdobi());
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi)
        {
            $puvodniOd = $kalendarVDanemObdobi->platnostOd;
            $puvodniDo = $kalendarVDanemObdobi->platnostDo;
            $kodPuvodniho = $kalendarVDanemObdobi->kod;

            if (($puvodniOd < $platnostOdZmeny && $puvodniDo == '') || ($puvodniOd < $platnostOdZmeny && $platnostDoZmeny < $puvodniDo && $puvodniDo != ''))
            {
                $this->upravaPlatnostiKalendare($pracovniPomer, $puvodniOd, $puvodniOd, DateUtils::incDay($platnostOdZmeny, -1), $kodPuvodniho);

                $this->upravaPlatnostiKalendare($pracovniPomer, DateUtils::incDay($platnostDoZmeny, 1), DateUtils::incDay($platnostDoZmeny, 1), $puvodniDo, $kodPuvodniho);
            }
        }

        $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer ->idPP, RonPortalSession::obdobi());

        //Zmena zkrati kalendar pred sebou
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi) {
            $puvodniOd = $kalendarVDanemObdobi->platnostOd;
            $puvodniDo = $kalendarVDanemObdobi->platnostDo;
            $kodPuvodniho = $kalendarVDanemObdobi->kod;

            if ($puvodniOd < $platnostOdZmeny && $platnostOdZmeny <= $puvodniDo && $puvodniDo <= $platnostDoZmeny && $puvodniDo != '')
            {
                $this->upravaPlatnostiKalendare($pracovniPomer, $puvodniOd, $puvodniOd, DateUtils::incDay($platnostOdZmeny, -1), $kodPuvodniho);
            }
        }

        $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer ->idPP, RonPortalSession::obdobi());

        //Zmena posune kalendar ktery nasleduje
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi) {
            $puvodniOd = $kalendarVDanemObdobi->platnostOd;
            $puvodniDo = $kalendarVDanemObdobi->platnostDo;
            $kodPuvodniho = $kalendarVDanemObdobi->kod;

            if (($platnostOdZmeny <= $puvodniOd && $puvodniOd <= $platnostDoZmeny && $puvodniDo == '') || ($platnostOdZmeny <= $puvodniOd && $puvodniOd <= $platnostDoZmeny && $puvodniDo != ''))
            {
                $this->upravaPlatnostiKalendare($pracovniPomer, $puvodniOd, DateUtils::incDay($platnostDoZmeny, 1), $puvodniDo, $kodPuvodniho);
            }
        }

        //Vlozeni noveho kalendare
        $this->prirazeniNovehoKalendare($pracovniPomer, $platnostOdZmeny, $platnostDoZmeny, $kalendarZmeny);
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param $kalendareVDanemObdobi
     * @param $platnostOdZmeny
     * @param $platnostDoZmeny
     * @param $kalendar
     * @throws \Exception
     */
    private function upravaKalendareTrvalaZmena($pracovniPomer, $kalendareVDanemObdobi, $platnostOdZmeny, $platnostDoZmeny, $kalendar)
    {
        //Prekryti  - nejdrive odstrani prekryte
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi) {

            $puvodniOd = $kalendarVDanemObdobi->platnostOd;

            if ($platnostOdZmeny <= $kalendarVDanemObdobi->platnostOd)
            {
                $this->odstraneniKalendare($pracovniPomer, $puvodniOd);
            }
        }

        $kalendareVDanemObdobi = $this->nactiKalendarOsoby($this->database, $pracovniPomer ->idPP, RonPortalSession::obdobi());
        foreach ($kalendareVDanemObdobi as $kalendarVDanemObdobi) {

            $puvodniOd = $kalendarVDanemObdobi->platnostOd;
            $puvodniDo = $kalendarVDanemObdobi->platnostDo;
            $kodPuvodniho = $kalendarVDanemObdobi->kod;

            if (($puvodniOd < $platnostOdZmeny && $puvodniDo == '') || ($puvodniOd < $platnostOdZmeny && $platnostOdZmeny <= $puvodniDo && $puvodniDo != '') )
            {
                $this->upravaPlatnostiKalendare($pracovniPomer, $puvodniOd, $puvodniOd, DateUtils::incDay($platnostOdZmeny, -1), $kodPuvodniho);
            }
        }

        //Vlozeni noveho kalendare
        $this->prirazeniNovehoKalendare($pracovniPomer, $platnostOdZmeny, $platnostDoZmeny, $kalendar);
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param int   $platnostOd
     * @param int   $novaPlatnostOd
     * @param int   $novaPlatnostDo
     * @param string $kalendar
     */
    public function upravaPlatnostiKalendare($pracovniPomer, $platnostOd, $novaPlatnostOd, $novaPlatnostDo, $kalendar)
    {
        $upravit = new RonPortalDBRecord($this->database, 'PrirazeniKalendare', $this);

        $upravit->addField('RC', '', false, 15, $pracovniPomer ->rc);
        $upravit->addField('OsobniCislo', '', false, 15, $pracovniPomer ->osobniCislo);
        $upravit->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer ->datumNastupu);
        $upravit->addField('PlatnostOd', 'timestamp', false, 0, $novaPlatnostOd);
        $upravit->addField('Kalendar', '', false, 30, $kalendar);
        $upravit->addField('PlatnostDo', 'timestamp', false, 0, $novaPlatnostDo);
        $upravit->addField('ZkracenyUvazekHodin', 'timestamp', false, NULL);
        $upravit->addField('Poznamka', '', false, 120, NULL);

        $params = $this->database->createParameters();
        $upravit->setWhere(
            'RC = ' . $params->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($this->database->dateToParam(($pracovniPomer ->datumNastupu))) .
            ' AND PlatnostOd = ' . $params->addParameter($this->database->dateToParam(($platnostOd))),
            $params
        );

        $upravit->updateOrInsert();
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param int   $platnostOdZmeny
     * @param int   $platnostDoZmeny
     * @param string $kalendar
     * @throws \Exception
     */
    public function prirazeniNovehoKalendare($pracovniPomer, $platnostOdZmeny, $platnostDoZmeny, $kalendar)
    {
        $params = $this->database->createParameters();

        $vlozit = new RonPortalDBRecord($this->database, 'PrirazeniKalendare', $this);

        if (!$platnostOdZmeny){
            throw new \Exception(\Ron\ronText('platnost_od_neni_zadana'));
        }

        $vlozit->addField('RC', '', false, 15, $pracovniPomer ->rc);
        $vlozit->addField('OsobniCislo', '', false, 15, $pracovniPomer ->osobniCislo);
        $vlozit->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer ->datumNastupu);
        $vlozit->addField('PlatnostOd', 'timestamp', false, 0, $platnostOdZmeny);
        $vlozit->addField('Kalendar', '', false, 30, $kalendar);
        $vlozit->addField('PlatnostDo', 'timestamp', false, 0, $platnostDoZmeny);
        $vlozit->addField('ZkracenyUvazekHodin', 'timestamp', false, NULL);
        $vlozit->addField('Poznamka', '', false, 120, NULL);

        $vlozit->setWhere( 'RC = ' . $params->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($this->database->dateToParam($pracovniPomer ->datumNastupu)) .
            ' AND PlatnostOd = ' . $params->addParameter($this->database->dateToParam(($platnostOdZmeny))), $params);

        $vlozit->insertRecord();
    }


    /**
     * @param PracovniPomer $pracovniPomer
     * @param int $platnostOd
     * @throws \Exception
     */
    public function odstraneniKalendare($pracovniPomer, $platnostOd)
    {
        $params = $this->database->createParameters();

        $smazat = new RonPortalDBRecord($this->database, 'PrirazeniKalendare', $this);

        $smazat->addField('RC', '', false);
        $smazat->addField('OsobniCislo', '', false);
        $smazat->addField('DatumNastupu', 'timestamp', false);
        $smazat->addField('PlatnostOd', 'timestamp', false);
        $smazat->addField('Kalendar', '', false);
        $smazat->addField('PlatnostDo', 'timestamp', false);
        $smazat->addField('ZkracenyUvazekHodin', 'timestamp', false);
        $smazat->addField('Poznamka', '', false);

        $smazat->setWhere(
            'RC = ' . $params->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($this->database->dateToParam($pracovniPomer ->datumNastupu)) .
            ' AND PlatnostOd = ' . $params->addParameter($this->database->dateToParam(($platnostOd))),
            $params);

        $smazat->deleteRecord();
    }


    /**
     * Prepocet - zpracovat zadane obdobi podle puvodni dochazky ($uloha=8), ale jen maximalne do data do kdy je dochazka zpracovana.
     * @param int $idpp
     * @param int $platnostOdZmeny
     * @param int $uloha
     * @throws \Exception
     */
    public function upravaKalendarePrepocetDochazky($idpp, $platnostOdZmeny, $uloha)
    {
        $zpracovaniDochazky =  DochazkaUtils::datumCasZpracovaniDochazky($this->database);

        MessageManager::addDebugMessage("Provedu zpracovani podle puvodni (uloha $uloha) pro interval " . date('j.n.Y', $platnostOdZmeny) . ' - ' . date('j.n.Y', $zpracovaniDochazky));

        $params = $uloha .
            ' -interval:' . date('Y-n-j', $platnostOdZmeny) . ';' . date('Y-n-j', $zpracovaniDochazky) .
            ' -IdPP:' . $idpp;

        require_once('adsapp/agentmanager.php');
        AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
    }


    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        $osoba = isset($_POST['osoba']) ? $_POST['osoba'] : $_GET['osoba'];

        if (!$this->pravo->lzeZobrazitPP($osoba)){
            RonPortalUtils::neniPravoException();
        }

        TextManager::getInstance() ->loadStrings('kalendarosoby.ini');
        //  HtmlHeaders::getInstance() ->addScript('script/kalendarosoby.js');
        HtmlHeaders::getInstance() ->addCss('css/s_ron.css');


        switch ($_GET['action']){

            case 'detailkalendare':
                return $this->detailKalendareOsoby($osoba);
                break;

            case 'upravakalendare':
                return $this->editaceKalendareOsoby($osoba);
                break;

            default:
                return new Redirect(null, '?menuid=zpracovanadochazka');
        }
    }
}

return new KalendarOsobyController($this->database, $_GET['osoba']);