<?php
/**
 * UTF8
 * pavel.rudol, 28.12.2017
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use function Ron\ronText;

require_once('editace/f_rozdeleniMV.php');
require_once('adsapp/ronportaldbrecord.php');
require_once('adsapp/ppomer.php');
require_once('adsapp/obdobi.php');

class RozdeleniMVController extends HttpController
{
    /** @var Database */
    private $database;

    /** @var PracovniPomer */
    private $pracovniPomer;
    /** @var Obdobi */
    private $obdobi;
    private $nastaveniEditace;

    /**
     * RozdeleniMVController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }

    /**
     * @throws DBException
     */
    private function nactiNastaveni()
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = " . $params->addParameter('RonPortal_Rozdeleni_mv');

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $text = $row[0];

        require_once('ron/xmlloader.php');

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniEditace = $loader->readXml($text);
    }

    /**
     * @throws Exception
     */
    private function nactiOsobuObdobi()
    {
        $idpp = isset($_POST['osoba']) ?
            $_POST['osoba'] :
            (isset($_GET['idpp']) ? $_GET['idpp'] : RonPortalSession::vybranaOsoba());
        $pp = new PracovniPomer();
        if (!$pp ->nactiPPPodleId($this ->database, $idpp)) throw new Exception('Pracovni pomer nebyl nalezen!');

        $pocatekObdobi = isset($_POST['obdobi']) ?
            DateUtils::urlStringToDate($_POST['obdobi']) :
            (isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi());
        $obdobi = new Obdobi();
        $obdobi ->nactiObdobi($this ->database, 'ADS', $pocatekObdobi);
        
        $this->pracovniPomer = $pp;
        $this->obdobi = $obdobi;
    }

    /**
     * @throws Exception
     */
    private function zkontrolujPrava()
    {
        $prava = Prava::vratPrava('editace');

        $jePravoEditovat = $prava ->lzeZobrazitPP($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
        if (!$jePravoEditovat) {
            // nemuze editovat, jeste zkontroluje, jestli ma pravo editovat rozdeleni vysledku
            $prava = Prava::vratPrava('editace_r');
            $jePravoEditovat = $prava ->lzeZobrazitPP($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
        }
        
        if (!$jePravoEditovat)
            throw new NeniPravoException();

        if ($this->obdobi->uzavreno) {
            throw new Exception(ronText('Obdobi_uzavreno'));
        }

        require_once('dochazka/schvalenidochazky.php');

        $schvaleniDochazky = new SchvaleniDochazky();
        $schvaleniDochazky->load($this->database, $this->pracovniPomer->rc, $this->pracovniPomer->osobniCislo, $this->pracovniPomer->datumNastupu, $this->obdobi->zacatek, $this->obdobi->konec);
        if (!$schvaleniDochazky->lzeEditovatPP($this->pracovniPomer->idPP, $this->obdobi->konec))
            throw new Exception('Nelze editovat, dochazka je jiz schvalena');
    }

    /**
     * @param bool $zobrazPost
     * @return BaseViewObject
     * @throws Exception
     * @throws DBException
     */
    private function zobrazFormular($zobrazPost = false)
    {
        $this->nactiNastaveni();

        require_once('ron/controls.php');

        $mesicniData = RozdeleniMV::nactiData($this->database, $this->pracovniPomer, $this->obdobi, $this->nastaveniEditace);

        if (!$zobrazPost) {
            try {
                $mesicniData->zkontrolujSoucty();
            } catch (Exception $exception) {
                MessageManager::addErrorMessage($exception->getMessage());
            }
        }

        $typRozdeleni = $this->nastaveniEditace['typrozdeleni'];

        $paramsCS = $this->database->createParameters();
        $sqlCasovaSlozka =
            "SELECT TypCasoveSlozkyMzdy.Kod, TypCasoveSlozkyMzdy.Popis " .
            "FROM TypCasoveSlozkyMzdy JOIN VlastnostCasoveSlozky ON TypCasoveSlozkyMzdy.Kod = VlastnostCasoveSlozky.Kod " .
            "WHERE VlastnostCasoveSlozky.PlatnostOd <= " . $paramsCS->addParameter($this->database->dateToParam($this->obdobi->konec)) .
            "  AND (VlastnostCasoveSlozky.PlatnostDo >= " . $paramsCS->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR VlastnostCasoveSlozky.PlatnostDo IS NULL) " .
            "  AND VlastnostCasoveSlozky.ManualniEditaceRozdelniMV = 1" .
            "  AND TypCasoveSlozkyMzdy.Kod NOT IN ('" . implode("', '", $mesicniData->kodyCS_R) .  "')";

        $comboCS = new ComboBox('rozdelenimv_casova_slozka');
        $comboCS->loadData($this->database, $sqlCasovaSlozka, $zobrazPost ? $_POST['rozdelenimv_casova_slozka'] : null, $paramsCS);

        foreach ($mesicniData->mesicniDataRozdeleni as $mesicniVysledekRozdeleni) {
            $sqlCasoveSlozky =
                "SELECT Kod, Popis " .
                "FROM TypCasoveSlozkyMzdy " .
                "WHERE Kod IN ('" . implode("', '", $mesicniVysledekRozdeleni->kodyCS) . "') ";
            $comboCSRozdeleni = new ComboBox('rozdelenimv_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS);
            $comboCSRozdeleni->loadData($this->database, $sqlCasoveSlozky, $zobrazPost ? $_POST['rozdelenimv_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS] : null, null);

            MessageManager::addDebugLog('Combo casove slozky pro souctovou CS ' . $mesicniVysledekRozdeleni->kodCS . ' : ' . $sqlCasoveSlozky);


            Model::addData('combo_casova_slozka_' . $mesicniVysledekRozdeleni->kodCS, $comboCSRozdeleni);
        }

        Model::addData('rozdeleni_mv', $mesicniData);
        Model::addData('combo_casova_slozka', $comboCS);
        Model::addData('typ_rozdeleni', $typRozdeleni);
        Model::addData('zobraz_post', $zobrazPost);

        return new BaseViewObject('editace/view_rozdeleniMV.php');
    }

    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     * @throws DBException
     */
    private function zpracujPost()
    {
        $this->nactiNastaveni();

        try {
            if (RozdeleniMV::zpracujPost($this->database, $this->pracovniPomer, $this->obdobi, $this->nastaveniEditace)) {
                if ((is_null($this->nastaveniEditace['prepocetdochazky'])) || ($this->nastaveniEditace['prepocetdochazky'] == 1)) {
                    require_once('adsapp/agentmanager.php');
                    AgentManager::prepocetDochazky($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
                }

                return new Redirect(null, '?menuid=rozdelenimv&osoba=' . $this->pracovniPomer->idPP . '&obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek));
            } else {
                return $this->zobrazFormular();
            }
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage($exception->getMessage());
            return $this->zobrazFormular(true);
        }
    }

    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     * @throws DBException
     */
    private function smazRadek()
    {
        $record = new RonPortalDBRecord($this->database, 'MesicniVysledek_R_E');
        $record->logDescription = StringUtils::fromUTF8('Editace rozdělení měsíčních výsledků');

        $record->addField('RC', '', true);
        $record->addField('OsobniCislo', '', true);
        $record->addField('DatumNastupu', 'timestamp', true);
        $record->addField('Mesic', 'timestamp', true);
        $record->addField('TypCasoveSlozkyMzdy', '', true);
        $record->addField('KodTypuRozdeleni', 'int', true);
        $record->addField('Rozdeleni', '', true);
        $record->addField('Hodiny', 'float', true);
        $record->addField('Dny', 'float', true);
        $record->addField('KalendarniDny', 'float', true);
        $record->addField('Poznamka', '', true);

        $params = $this->database->createParameters();
        $where =
            "     RC = " . $params->addParameter($this->pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->dateToParam($this->pracovniPomer->datumNastupu)) .
            " AND Mesic = " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) .
            " AND TypCasoveSlozkyMzdy = " . $params->addParameter($_GET['casovaslozka']) .
            " AND KodTypuRozdeleni = " . $params->addParameter($_GET['typrozdeleni']) .
            " AND Rozdeleni = " . $params->addParameter($_GET['rozdeleni']);

        $record->setWhere($where, $params);

        if ($record->deleteRecord()) {
            require_once('adsapp/agentmanager.php');
            AgentManager::prepocetDochazky($this->pracovniPomer->idPP, $this->obdobi->zacatek, $this->obdobi->konec);
            return new Redirect(null, '?menuid=rozdelenimv&osoba=' . $this->pracovniPomer->idPP . '&obdobi=' . DateUtils::dateToUrlString($this->obdobi ->zacatek));
        }
        else {
            return $this->zobrazFormular();
        }
    }

    /** Vraci seznam rozdeleni pro ronselect
     * @return JsonOutputObject
     * @throws Exception
     */
    private function seznamRozdeleni()
    {
        $output = new JsonOutputObject();

        $this->nactiNastaveni();

        $typRozdeleni = $this->nastaveniEditace['typrozdeleni'];

        if ($this->nastaveniEditace['pouzevlastni']) {
            $params = $this->database->createParameters();

            $sqlRozdeleni =
                "SELECT Kod, Popis " .
                "FROM Rozdeleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod) " .
                "WHERE RozdeleniOsob.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni)  .
                "  AND RozdeleniOsob.RC = " . $params->addParameter($this->pracovniPomer->rc) .
                "  AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
                "  AND RozdeleniOsob.DatumNastupu = " . $params->addParameter($this->database->dateToParam($this->pracovniPomer->datumNastupu)) .
                "  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR RozdeleniOsob.PlatnostDo IS NULL)" .
                "  AND DatumVytvoreni <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (DatumZruseni >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR DatumZruseni IS NULL)";
        }
        else {
            $params = $this->database->createParameters();

            $sqlRozdeleni =
                "SELECT Kod, Popis 
                 FROM Rozdeleni 
                 WHERE KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) . " 
                   AND DatumVytvoreni <= " . $params->addParameter($this->database->dateToParam($this->obdobi->konec)) .
                "  AND (DatumZruseni >= " . $params->addParameter($this->database->dateToParam($this->obdobi->zacatek)) . " OR DatumZruseni IS NULL)";
        }

        $sqlRozdeleni .= " ORDER BY Kod";

        MessageManager::addDebugLog('Combo rozdeleni: ' . $sqlRozdeleni);

        $cursor = $this->database->getSqlResult($sqlRozdeleni, $params);

        $seznamRozdeleni = [];
        while ($row = $cursor->fetch()) {
            $rozdeleni = [];
            $rozdeleni['kod'] = StringUtils::jsonText($row[0]);
            $rozdeleni['popis'] = StringUtils::jsonText($row[1]);
            $seznamRozdeleni[] = $rozdeleni;
        }

        $output->addData('items', $seznamRozdeleni);

        return $output;
    }

    /**
     * @param string $action
     * @return bool|InternalRedirect|OutputObject|Redirect|ViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            throw new Exception('V nastaveni portalu neni povolena editace!');
        }
        TextManager::getInstance()->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance()->addScript('script/dochazka.js');
        HtmlHeaders::getInstance()->addCss('css/dochazka.css');
        HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('Rozdeleni_mesicnich_vysledku'));

        $this->nactiOsobuObdobi();

        Prava::nastavObdobiProPrava( $this->obdobi ->zacatek, $this->obdobi ->konec );
        Prava::nactiPravaUzivatele($this->database, User::getRC());

        $this->zkontrolujPrava();
        
        if (isset($_POST['ulozitRozdeleniMV']) && $_POST['ulozitRozdeleniMV']) {
            return $this->zpracujPost();
        }

        if (isset($_GET['action'])) {
            if ($_GET['action'] == 'smazat') {
                return $this->smazRadek();
            }

            if ($_GET['action'] == 'seznamrozdeleni') {
                return $this->seznamRozdeleni();
            }
        }

        return $this->zobrazFormular();

    }

}

return new RozdeleniMVController($this ->database);
