<?php
/**
 * UTF8
 * stanislav.elbl, 01.11.2015 * roman.szablatura, 19.10.2016
 */

namespace Ron\Personalistika;

use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\RonPortal\RonPortalUtils;


/**
 * Class UkolyController
 * @package Ron\Personalistika
 */
class UkolyController extends JsonController
{
    /** @var  Database */
    var $database;

    /**
     * UkolyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @return SeznamUkoluBrowser
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function seznamUkolu()
    {
        require_once('person/b_seznamukolu.php');
        Prava::nastavObdobiProPrava(time(), time());

        $b = new SeznamUkoluBrowser($this ->database);
        $b ->load();
        return $b;
    }


    /**
     * @param $kodSerial
     * @return BaseViewObject
     * @throws \Exception
     */
    public function zobrazUkol($kodSerial)
    {
        require_once('person/ukol.php');

        $ukol = new Ukol();
        $ukol ->load($this ->database, $kodSerial);
        Model::addData('ukol', $ukol);

        //Zrusene ukoly je mozne spravovat pouze pres aplikaci
        if ($ukol->stavUkolu == -1) {
            throw new \Exception('Chyba! Pres RON portal nelze prohlizet zrusene ukoly.');
        }
        Prava::nastavObdobiProPrava(time(), time());
        $prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"

        if (!$prava->lzeZobrazitRC($ukol ->RC_Osoby)) {
            RonPortalUtils::neniPravoException();
        }
        return new BaseViewObject('view/ukol_formular.php');
    }


    /**
     * @param $kodSerial
     * @return BaseViewObject|Redirect
     * @throws \Exception
     */
    public function save($kodSerial)
    {
        require_once('person/ukol.php');
        $ukol = new Ukol();
        $ukol ->load($this ->database, $kodSerial);

        $prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"
        Prava::nastavObdobiProPrava(time(), time());

        if (!$prava->lzeZobrazitRC($ukol->RC_Osoby)) {
            RonPortalUtils::neniPravoException();
        }

        try {
            $ukol->ulozZmenenyZaznam($this->database, $prava->priznak);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return $this ->zobrazUkol($kodSerial);
        }

        $link = '';


        if (RonPortalSettings::getIntValue('typ_tlacitka')) {
            if (true) {
                $link = "&record=" . $kodSerial;
            }
            return new Redirect(null, "?menuid=ukoly" . $link);
        } else {
            return new Redirect(null, "?menuid=ukoly");
        }
    }

    /**
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    public function mojeUkoly($output)
    {
        require_once('ron/httputils.php');

        $html = '<table class="dashboardTable" data-ron-href="?menuid=ukoly">';
        $html .='<tr>';
        $html .= "<td>" . \Ron\ronText('') . '</td>';
        $html .= "<td>" . \Ron\ronText('typ_ukolu') . '</td>';
        $html .= "<td>" . \Ron\ronText('popis') . '</td>';
        $html .= "<td>" . \Ron\ronText('termin') . '</td>';
        $html .= '</tr>';


        $params = $this ->database ->createParameters();
        $sql = 'SELECT UkolOsoby.KodSerial, UkolOsoby.RC_Osoby, UkolOsoby.StavUkolu, UkolOsoby.RC_Hodnotitele_Def, UkolOsoby.Hodnoceno, ' .                 //0 - 4
            'UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.Schvaleno1, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.Schvaleno2, KategorieUkolu.Popis, ' .       //5 - 9
            'UkolOsoby.ZkracenyPopis, datumSplneni_Narizene ' .                                                                                        //10..11

            'FROM UkolOsoby LEFT JOIN KategorieUkolu ON (KategorieUkolu.kod = UkolOsoby.kodKategorieUkolu) ' .

            'WHERE StavUkolu <> -1 AND (RC_Osoby = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Hodnotitele_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele1_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele2_Def = ' . $params->addParameter(User::getRC()) . ')';

        $cursor = $this->database->getSqlResult($sql,$params);

        $prihlasenaOsoba = User::getRC();
        while ($row = $cursor->fetch()){

            $html .="<tr>";

                //Ukol ke splnění
                if ($row[1] == $prihlasenaOsoba && $row[2] == 0 ){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_splnit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

                //Ukol připraven k hodnoceni
                if ($row[3] == $prihlasenaOsoba && $row[4] == 0 && $row[2] == 1){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_hodnotit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

                //Ukol připraven ke schvaleni 1
                //Ukol připraven ke schvaleni 2
                if (($row[5] == $prihlasenaOsoba && $row[2] == 1 && $row[4] == 1 && $row[6] == 0) || ($row[7] == $prihlasenaOsoba && $row[2] == 1 && $row[6] == 1 && $row[8] == 0)){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_schvalit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

            $html .="</tr>";
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'mojeukoly':
                $this->mojeUkoly($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|SeznamUkoluBrowser|string
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('ukoly.ini');

        if (isset($_POST['kodserial'])  &&  $_POST['kodserial']) {
            return $this ->save($_POST['kodserial']);
        }

        if (isset($_GET['record'])  &&  $_GET['record']) {
            return $this ->zobrazUkol($_GET['record']);
        }

        if (isset($_GET['action']) && ($_GET['action'] == 'mojeukoly')) {
            //json action
            return parent::handleAction($action);
        }


        return $this->seznamUkolu();
    }
}

return new UkolyController($this ->database);
