<?php /** @noinspection SqlResolve */

/**
 * UTF8
 * User: pavel.rudol
 * Date: 15.5.2018
 * Time: 13:16
 */

namespace Ron\RonPortal\Vyroba;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\VyrobaUtils;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use function Ron\ronText;

require_once('vyroba/f_dennivyroba.php');

class VyrobaController extends HttpController
{
    const NASTAVENI_VYCHOZI_CINNOST = 'vyroba_vychozi_cinnost';
    const NASTAVENI_VYCHOZI_VYROBEK = 'vyroba_vychozi_vyrobek';
    const NASTAVENI_VYCHOZI_ZAKAZKA = 'vyroba_vychozi_zakazka';

    /** @var Database */
    private $database;

    private $pravaZobrazit;

    private $nastaveniEditace;

    private $kodyCSMesicni;
    private $kodyCSDenni;

    private $nacistVysledkyVyroby;

    private $datumCasZpracovaniVyroby;

    public static $vychoziCinnost;
    public static $vychoziVyrobek;
    public static $vychoziZakazka;
    public static $existujeSloupecCinnost;
    public static $existujeVysledekVyroby;

    public function __construct($database)
    {
        $this->database = $database;

        $this->pravaZobrazit = Prava::vratPrava('editacevyroby');
        static::$vychoziCinnost = RonPortalSettings::getValue(static::NASTAVENI_VYCHOZI_CINNOST);
        static::$vychoziVyrobek = RonPortalSettings::getValue(static::NASTAVENI_VYCHOZI_VYROBEK);
        static::$vychoziZakazka = RonPortalSettings::getValue(static::NASTAVENI_VYCHOZI_ZAKAZKA);

        $this->kodyCSMesicni = [];
        $this->kodyCSDenni = [];

        $this->nacistVysledkyVyroby = false;
    }

    /**
     * @throws Exception
     */
    private function nactiNastaveni()
    {
        $params = $this->database->createParameters();
        $sql = "SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = " . $params->addParameter('RonPortal_vyroba_editace');

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $text = $row[0];

        require_once('ron/xmlloader.php');

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniEditace = $loader->readXml($text);

        static::$existujeSloupecCinnost = false;
        foreach ($this->nastaveniEditace['sloupeceditace'] as $sloupecEditace) {
            if (($sloupecEditace['typ'] == 'CINNOST') && !static::$existujeSloupecCinnost) {
                static::$existujeSloupecCinnost = true;
            }
            if ($sloupecEditace['typ'] == 'MV')
                $this->kodyCSMesicni[] = $sloupecEditace['kod'];
            if ($sloupecEditace['typ'] == 'DV')
                $this->kodyCSDenni[] = $sloupecEditace['kod'];
            if (!$this->nacistVysledkyVyroby && ($sloupecEditace['typ'] == 'VYSLEDEKHODINY'))
                $this->nacistVysledkyVyroby = true;
        }
        if (!(static::$existujeSloupecCinnost || !is_null(static::$vychoziCinnost))) {
            throw new Exception(ronText('Vyroba_neni_cinnost'));
        }

        require_once 'vyroba/vyrobautils.php';
        $this->datumCasZpracovaniVyroby = VyrobaUtils::datumCasZpracovaniVyroby($this->database);
    }


    /**
     * @param $den
     * @param bool $nacistPost
     * @param null|ZaznamEditaceVyroby[] $chybneZaznamy
     * @return BaseViewObject
     */
    private function denniVyrobaFormular($den, $nacistPost = false, $chybneZaznamy = null)
    {
        try {
            $this->nactiNastaveni();

            Prava::nastavObdobiProPrava($den);

            $filter = new VyrobaFilterForm($this->database, $this->pravaZobrazit, $den, DateUtils::endOfTheDay($this->datumCasZpracovaniVyroby), $this->nastaveniEditace['typrozdeleni'], $this->nastaveniEditace['typrozdeleni2']);
            $filter->loadData();

            $vyroba = ZaznamEditaceVyroby::nactiOsobyProVyrobu($this->database, $den, $this->pravaZobrazit, $this->kodyCSDenni, $this->kodyCSMesicni, $this->nacistVysledkyVyroby);

            if (!is_null($chybneZaznamy))
                $vyroba = array_merge($chybneZaznamy, $vyroba);

            Model::addData('filter', $filter);
            Model::addData('vyroba', $vyroba);
            Model::addData('pocet_chyb', $chybneZaznamy ? count($chybneZaznamy) : 0);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
        }

        Model::addData('den', $den);
        Model::addData('zobrazitpost', $nacistPost);
        Model::addData('nastaveni_formulare', $this->nastaveniEditace);
        Model::addData('datum_zpracovani', $this->datumCasZpracovaniVyroby);

        return new BaseViewObject('vyroba/view_dennivyroba.php');
    }

    /**
     * @return JsonOutputObject
     * @throws DBException
     */
    private function seznamVyrobku()
    {
        $output = new JsonOutputObject();

        $sql = "SELECT Kod, Popis FROM Zbozi WHERE Neaktivni = 0 ORDER BY Popis";

        $cursor = $this->database->getSqlResult($sql);

        $vyrobky = [];
        while ($row = $cursor->fetch()) {
            $vyrobek = [];
            $vyrobek['kod'] = StringUtils::jsonText($row[0]);
            $vyrobek['popis'] = StringUtils::jsonText($row[1]);
            $vyrobky[] = $vyrobek;
        }

        $output->addData('items', $vyrobky);

        return $output;
    }

    /**
     * @return JsonOutputObject
     * @throws DBException
     */
    private function seznamCinnosti()
    {
        $output = new JsonOutputObject();

        $sql = "SELECT Kod, Popis FROM VyrobniCinnost WHERE Aktivni = 1 ORDER BY Popis";

        $cursor = $this->database->getSqlResult($sql);

        $cinnosti = [];
        while ($row = $cursor->fetch()) {
            $cinnost = [];
            $cinnost['kod'] = StringUtils::jsonText($row[0]);
            $cinnost['popis'] = StringUtils::jsonText($row[1]);
            $cinnosti[] = $cinnost;
        }

        $output->addData('items', $cinnosti);

        return $output;
    }

    /**
     * @return JsonOutputObject
     * @throws DBException
     */
    private function seznamZakazek()
    {
        $output = new JsonOutputObject();

        $params = $this->database->createParameters();
        $sql = "SELECT cisloZakazky, Popis, DatumPrijeti FROM VyrobniZakazka ORDER BY Popis";
        //"WHERE platnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " AND (platnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR platnostDo IS NULL)";

        $cursor = $this->database->getSqlResult($sql, $params);

        $zakazky = [];
        while ($row = $cursor->fetch()) {
            $zakazka = [];
            $zakazka['kod'] = StringUtils::jsonText($row[0]);
            $zakazka['popis'] = StringUtils::jsonText($row[1]);
            $zakazka['datumprijeti'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[2]));
            $zakazky[] = $zakazka;
        }

        $output->addData('items', $zakazky);

        return $output;
    }

    /**
     * @param $den
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    private function zapisVyrobu($den)
    {
        $this->nactiNastaveni();

        $filter = new VyrobaFilterForm($this->database, $this->pravaZobrazit, $den, DateUtils::endOfTheDay($this->datumCasZpracovaniVyroby), $this->nastaveniEditace['typrozdeleni'], $this->nastaveniEditace['typrozdeleni2']);
        $filter->loadData();

        try {
            $chybneZaznamy = ZaznamEditaceVyroby::zpracujPost($this->database, $den, $this->pravaZobrazit, (int)$this->nastaveniEditace['prepocetvyroby'], $this->datumCasZpracovaniVyroby,
                (int)$this->nastaveniEditace['prepocetdochazky'], (int)$this->nastaveniEditace['kontroladochazky'], $this->nastaveniEditace);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->denniVyrobaFormular($den, true);
        }

        if (!is_null($chybneZaznamy)) {
            return $this->denniVyrobaFormular($den, false, $chybneZaznamy);
        }

        MessageManager::addInfoMessage(ronText('Vyroba_zapsano'));
        return new Redirect(null, '?menuid=vyroba&action=denni&den=' . StringUtils::format('date', $den));
    }


    /**
     * @param $den
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    private function provedZnovuZpracovani($den)
    {
        $this->nactiNastaveni();

        //pokud neni v nastaveni povoleno znovuzpracovani podle puvodni vyroby, jen zobrazi formular z POSTu
        if (!($this->nastaveniEditace['podlepuvodni'] == 1))
            return $this->denniVyrobaFormular($den, true);

        if ($den > $this->datumCasZpracovaniVyroby)
            throw new Exception(ronText('Vyroba_nesmi_prekrocit_cas_zpracovani'));

        $zaznamyVyroby = ZaznamEditaceVyroby::nactiOsobyProVyrobu($this->database, $den, $this->pravaZobrazit);

        $idPPList = [];

        foreach ($zaznamyVyroby as $zaznamVyroby) {
            if (!in_array($zaznamVyroby->idPP, $idPPList))
                $idPPList[] = $zaznamVyroby->idPP;
        }

        try {
            VyrobaController::znovuZpracovaniHromadne($idPPList, $den);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->denniVyrobaFormular($den, true);
        }

        MessageManager::addInfoMessage(ronText('Vyroba_znovuzpracovano'));
        return new Redirect(null, '?menuid=vyroba&action=denni&den=' . StringUtils::format('date', $den));
    }

    /**
     * @param int $serialVyroba
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    private function smazZaznamVyroby($serialVyroba)
    {
        $this->nactiNastaveni();

        require_once('adsapp/ronportaldbrecord.php');

        $record = new RonPortalDBRecord($this->database, 'Vyroba', StringUtils::fromUTF8('Editace výroby'));

        $record->addField('serial', 'int', true);
        $record->addField('RC', 'string', true);
        $record->addField('OsobniCislo', 'string', true);
        $record->addField('DatumNastupu', 'timestamp', true);
        $record->addField('den', 'timestamp', true);
        $record->addField('vyrobek', 'string', true);
        $record->addField('cinnost', 'string', true);
        $record->addField('VyrobniZakazka', 'string', true);
        $record->addField('mnozstviDobre', 'float', true);
        $record->addField('hodiny', 'float', true);
        $record->addField('casOd', 'timestamp', true);
        $record->addField('casDo', 'timestamp', true);

        $params = $this->database->createParameters();

        $record->setWhere("serial = " . $params->addParameter($serialVyroba), $params);

        $den = null;

        try {
            $record->loadRecord();
            $den = $record['den'];

            require_once('adsapp/ppomer.php');
            $pp = new PracovniPomer();
            $pp->nactiPPPodleRC($this->database, $record['RC'], $record['OsobniCislo'], $record['DatumNastupu']);
            Prava::nastavObdobiProPrava($den);
            if (!$pp->lzeZobrazitPP($this->pravaZobrazit, $den, $den)) {
                RonPortalUtils::neniPravoException();
            }

            if ($record->deleteRecord()) {

                if ((int)$this->nastaveniEditace['prepocetdochazky'] > 0) {
                    require_once('adsapp/agentmanager.php');

                    try {
                        AgentManager::prepocetDochazky($pp->idPP, DateUtils::startOfTheDay($record['casOd']), DateUtils::endOfTheDay($record['casDo']), false, (int)$this->nastaveniEditace['kontroladochazky']);
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                }

                MessageManager::addInfoMessage(ronText('Zaznam_smazan'));
            }
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->denniVyrobaFormular((is_null($den) ? VyrobaFilterForm::getDen() : $den));
        }
        
        return new Redirect(null, '?menuid=vyroba&action=denni&den=' . StringUtils::format('date', $den));
    }

    public static function existujeSloupecNastaveni($nastaveni, $sloupec)
    {
        foreach ($nastaveni['sloupeceditace'] as $sloupecEditace) {
            if (($sloupecEditace['typ'] == $sloupec)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param int $denOd
     * @param int $denDo
     * @return mixed
     */
    public static function prepocetVysledku($rc, $osobniCislo, $datumNastupu, $denOd, $denDo = 0)
    {
        $datumOd = $denOd;
        $datumDo = $denDo;

        if ($denDo == 0)
            $datumDo = $datumOd;

        try {
            require_once('adsapp/agentmanager.php');

            AgentManager::executeVyroba('_Agent_Production', '3 -interval:' . date('Y-m-d', $datumOd) . ';' . date('Y-m-d', $datumDo) .
                ' -PP:"' . $rc . ';' . $osobniCislo .';' . date('Y-m-d',$datumNastupu) . '"' . ' -alwaysLog', true);

            return true;
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
            return false;
        }
    }

    /**
     * @param int[]|int|string $idPPList    Id pracovniho pomeru, seznam ID, nebo seznam ID oddelenych strednikem
     * @param int $denOd
     * @param int $denDo
     * @return void
     */
    public static function prepocetVysledkuHromadne($idPPList, $denOd, $denDo = null)
    {
        try {
            require_once 'vyroba/vyrobautils.php';
            VyrobaUtils::prepocetVysledku($idPPList, $denOd, $denDo);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }
    }

    /**
     * @param int[]|int $idPPList
     * @param int $denOd
     * @param int $denDo
     * @return bool
     */
    public static function znovuZpracovaniHromadne($idPPList, $denOd, $denDo = null)
    {
        try {
            require_once 'vyroba/vyrobautils.php';
            VyrobaUtils::znovuZpracovaniHromadne($idPPList, $denOd, $denDo);
            return true;
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
            return false;
        }
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue('vyroba')) {
            RonPortalUtils::neniPravoException();
        }

        if (!$this->pravaZobrazit->jsouNejakePrava())
            RonPortalUtils::neniPravoException();

        TextManager::getInstance()->loadStrings('vyroba.ini');
        HtmlHeaders::getInstance()->addCss('css/dochazka.css');
        HtmlHeaders::getInstance()->addScript('script/vyroba.js');

        if (isset($_POST['zapsat'])) {
            return $this->zapisVyrobu(DateUtils::dateStringToTimestamp($_POST['den']));
        }
        if (isset($_POST['zpracovat'])) {
            return $this->provedZnovuZpracovani(DateUtils::dateStringToTimestamp($_POST['den']));
        }

        switch ($_GET['action']) {
            case 'seznamvyrobku':
                return $this->seznamVyrobku();

            case 'seznamcinnosti':
                return $this->seznamCinnosti();

            case 'seznamzakazek':
                return $this->seznamZakazek();
                
            case 'zrusitradek':  
                return $this->smazZaznamVyroby($_GET['record']);

            case 'denni':
            default :
                return $this->denniVyrobaFormular(VyrobaFilterForm::getDen());
        }
    }

}

return new VyrobaController($this->database);
