<?php
/**
 * UTF8
 * roman.Szablatura 17.02.2017
 */

namespace Ron\RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MVC\HttpController;
use Ron\RonPortal\FotoOutputObject;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\TextManager;

require_once ('zaznamysnimacu/b_zaznamysnimacu.php');

/**
 * Class ZaznamySnimacuController
 * @package Ron\RonPortal\Dochazka
 */
class ZaznamySnimacuController extends HttpController
{
    /** @var  Database */
    private $database;

    /** @var  PravaNaOsoby $prava */
    public $prava;


    /**
     * ZaznamySnimacuController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->prava = Prava::vratPrava('prohl_terminaly');
        Prava::nastavObdobiProPrava(time(), time());
    }

    /**
     * @param $idFotoZKarty
     * @return FotoOutputObject
     */
    public function downloadFoto($idFotoZKarty)
    {
        $this->jeMoznoZobrazitFotoZKarty($idFotoZKarty);

        require_once('adsapp/fotoutils.php');
        $result = new FotoOutputObject($this->database, $idFotoZKarty);
        return $result;
    }

    /**
     * @param $idFotoZKarty
     * @throws \Exception
     */
    private function jeMoznoZobrazitFotoZKarty($idFotoZKarty)
    {
        //Kontrola, zdali je pravo na rc vlastnika fotografie
        $params = $this->database->createParameters();

        $sqlTabulkaFotografie =
            'SELECT Osoba.RC ' .
            'FROM Osoba JOIN Fotografie ON (Osoba.idFotografie = Fotografie.id) ' .
            "WHERE Fotografie.id = " . $params->addParameter($idFotoZKarty);

        $rcOsoby = DBUtils::getSqlValue($this->database, $sqlTabulkaFotografie, $params);

        if (!($this->prava->lzeZobrazitRC($rcOsoby, time(),time()))) {
            throw new \Exception(\Ron\ronText('Nemate_pravo'));
        }
    }

    /**
     * @param $idFotoZTerminalu
     * @return FotoOutputObject
     */
    public function downloadFotoZeSnimace($idFotoZTerminalu)
    {
        //Dle Pavla neni potreba tato kontrola, jelikoz jiz jednou kontrolou uzivatel musi projit v handleAction
        //$this->jeMoznoZobrazitFotoZTerminalu($idFotoZTerminalu);

        require_once('adsapp/fotoutils.php');
        $result = new FotoOutputObject($this->database, $idFotoZTerminalu, 'FotoZTerminalu');
        return $result;
    }


//    private function jeMoznoZobrazitFotoZTerminalu($idFotoZTerminalu)
//    {
//        //Kontrola, zdali je pravo na rc vlastnika fotografie z terminalu
//        $params = $this->database->createParameters();
//
//        $sqlTabulkaFotoZTerminalu =
//            'SELECT FotoZTerminalu.RC ' .
//            'FROM FotoZTerminalu ' .
//            "WHERE FotoZTerminalu.id = " . $params->addParameter($idFotoZTerminalu);
//
//        $rcFotoZTerminalu = DBUtils::getSqlValue($this->database, $sqlTabulkaFotoZTerminalu, $params);
//
//        if (!$this->prava->lzeZobrazitRC($rcFotoZTerminalu,time(),time())) {
//            throw new \Exception(\Ron\ronText('Nemate_pravo'));
//        }
//    }



    public function handleAction($action)
    {
        if (!$this->prava->jsouNejakePrava()){
            throw new \Exception(\Ron\ronText('Nemate_pravo'));
        }

        //TODO Zkusebni uprava - pridani plovouciho kontejneru pro snadnejsi vyhledavani konkretni informace(datum) bez nutnosti prepinat filtr
        HtmlHeaders::getInstance()->addScript('script/zaznamysnimacu.js');
        TextManager::getInstance()->loadStrings('zaznamysnimacu.ini');
        HtmlHeaders::getInstance()->addCss('zaznamysnimacu/zaznamysnimacu.css');

        if (isset($_GET['fotozterminalu'])) {
            return $this ->downloadFotoZeSnimace( (int)$_GET['fotozterminalu'] );
        }

        if (isset($_GET['foto'])) {
            return $this ->downloadFoto( (int)$_GET['foto'] );
        }

        $b = new BrowserZaznamySnimacu($this->database, $action);
        $b->load();
        return $b;
    }
}
return new ZaznamySnimacuController($this->database);