<?php

/**
 * UTF8
 */

namespace Ron\RonPortal\Dochazka;

use Exception;
use Ron\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\Rozdeleni;
use function Ron\ronText;

require_once('ron/sqlutils.php');
require_once('adsapp/ronportalbrowser.php');

class ENeschopenkaFilterForm extends RonPortalFilterForm
{
    /** @var int 0 = vse, 1 = ukoncene, 2 = neukoncene */
    public $ukoncene = 0;

    private function loadUkoncene()
    {
        if (isset($_GET['ukoncene'])) {
            $_SESSION['ukoncene'] = $_GET['ukoncene'];
        }

        $this->ukoncene = isset($_SESSION['ukoncene']) ? $_SESSION['ukoncene'] : 0;
    }

    function loadData()
    {
        parent::loadData();
        $this->loadUkoncene();
    }

    private function renderJenUkoncene()
    {
        echo '<div class="filterrow wrap">';
        echo '<span class="filtercaption"></span>';

        require_once('ron/guiutils.php');
        LinksList::renderLinks(
            [
                0 => ronText('Vse'),
                1 => ronText('E_neschopenka_ukoncene'),
                2 => ronText('E_neschopenka_neukoncene'),
            ],
            $this->ukoncene,
            '?menuid=eneschopenka&ukoncene='
        );

        echo "</div>\n";
    }

    function renderFilters()
    {
        parent::renderFilters();
        $this->renderJenUkoncene();
    }
}

class PricinaNemociBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ($value) {
            case 0:
            default:
                return ronText('E_neschopenka_pricina_0');
            case 1:
                return ronText('E_neschopenka_pricina_1');
            case 2:
                return ronText('E_neschopenka_pricina_2');
            case 3:
                return ronText('E_neschopenka_pricina_3');
            case 4:
                return ronText('E_neschopenka_pricina_4');
        }
    }

}

class ENeschopenkaBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    private $prava;
    
    /** @var ENeschopenkaFilterForm */
    public $filterForm;

    /**
     * ENeschopenkaBrowser constructor.
     * @param Database $database
     * @param PravaNaOsoby $prava
     * @throws Exception
     */
    function __construct($database, $prava)
    {
        parent::__construct($database, 'eneschopenka');

        $this->prava = $prava;

        $this->code = 'browsereneschopenka';

        $this->title = ronText('E_neschopenka');
        $this->cssClass = 'browser';

        $this->sqlSelectPart = "Osoba.Jmeno, Osoba.Prijmeni, DruhDavkyNZ.Popis, ENeschopenka.DatumPocatkuDavky, ENeschopenka.DatumUkonceniDavky, ENeschopenka.UkoncenaDavka"; // 0 - 5

        $novaVerze = false;
        if (RonPortalUtils::jeNainstalovanaAplikace($this->database, 'ADS') > DateUtils::dateStringToTimestamp('25.11.2020') ||
            RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY') > DateUtils::dateStringToTimestamp('25.11.2020')) {
            $novaVerze = true;
            $this->sqlSelectPart .= ', ENeschopenka.PricinaNemoci, {fn_CASTWIDEMEMO(ENeschopenka.Poznamka)}'; // 6,7
        }

        $this->sqlFromPart =
            "ENeschopenka JOIN Osoba ON ENeschopenka.RC = Osoba.RC 
                JOIN DruhDavkyNZ ON DruhDavkyNZ.Kod = ENeschopenka.DruhDavky";

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'eneschopenka_prohl';
        $this ->filterFormVcetneExternich = true;

        $this->setFilterForm();
        $this->filterForm->loadData();

        if ($this->filterForm->vsechnyOsoby) {
            $this->addFields([
                [BROWSER_FIELD_SQL_NAME => "Osoba.Prijmeni", BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'Prijmeni',
                    BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'font-weight-bold', BROWSER_FIELD_WRAP => 1],
                [BROWSER_FIELD_SQL_NAME => "Osoba.Jmeno", BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER_TEXT => 'Jmeno',
                    BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'font-weight-bold', BROWSER_FIELD_WRAP => 1]
            ]);
        }

        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_druh', BROWSER_FIELD_SQL_NAME => "DruhDavkyNZ.Popis", BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center' ]);
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_datum_pocatku', BROWSER_FIELD_SQL_NAME => "ENeschopenka.DatumPocatkuDavky", BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'date' ]);

        if (!$this->filterForm->vsechnyOsoby) {
            $this->orderFieldIndex = count($this->fields);
        }
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_datum_ukonceni', BROWSER_FIELD_SQL_NAME => "ENeschopenka.DatumUkonceniDavky", BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'date' ]);
        $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_ukoncena_davka', BROWSER_FIELD_SQL_NAME => "ENeschopenka.UkoncenaDavka", BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_FORMAT => 'anone' ]);

        if ($novaVerze) {
            $this->addField(new PricinaNemociBrowserField($this->database, [ BROWSER_FIELD_HEADER_TEXT => 'E_neschopenka_pricina_nemoci', BROWSER_FIELD_SQL_NAME => "ENeschopenka.PricinaNemoci", BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center' ]));
            $this->addField([ BROWSER_FIELD_HEADER_TEXT => 'Poznamka', BROWSER_FIELD_SQL_NAME => "ENeschopenka.Poznamka", BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'left' ]);
        }
    }


    function setFilterForm()
    {
        $this->filterForm = new ENeschopenkaFilterForm($this ->database, $this ->action);

        if ($this ->filterFormAppCode) {
            $this->filterForm->nastavObdobi(true, $this->filterFormAppCode, false); // jenotevrene = false
        }

        $prava = Prava::vratPrava($this ->filterFormRights);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
        $this->filterForm ->nastavAkciGet('eneschopenka');
        $this->filterForm->povolitVsechyOsoby = true;
    }

    function getSQLFilter()
    {
        require_once('adsapp/rozdeleni.php');

        $params = $this->database->createParameters();

        $sql =
            'ENeschopenka.DatumPocatkuDavky <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) . ' AND ' .
            '(ENeschopenka.DatumUkonceniDavky >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR ENeschopenka.DatumUkonceniDavky IS NULL OR ENeschopenka.UkoncenaDavka = 0)';

        if ($this->filterForm->vsechnyOsoby) {
            $sql .=
                ' AND EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) .
                '           AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                ' AND ' . $this->prava->sqlFiltr('PracovniPomer', $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $params);

            if ($this->filterForm->kodRozdeleni) {
                $sql .= ' AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $this->filterForm->vcetnePodrizenychRozdeleni, $params);
            }

            if ($this->filterForm->kodRozdeleni2) {
                $sql .= ' AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this->filterForm->kodRozdeleni2,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $this->filterForm->vcetnePodrizenychRozdeleni2, $params);
            }

            $sql .=
                ')';
        } else {
                $sql .= ' AND ENeschopenka.RC = ' . $params->addParameter($this->filterForm->vybranaOsobaRC);
        }

        switch ($this->filterForm->ukoncene) {
            case 0 :
            default :
                break;
            case 1 :
                $sql .= ' AND ENeschopenka.UkoncenaDavka = 1';
                break;
            case 2 :
                $sql .= ' AND ENeschopenka.UkoncenaDavka = 0';
                break;
        }

        return new SqlStatement($sql, $params);
    }


}
