<?php
/**
 * UTF8
 */

namespace Ron\ELearning;

require_once('adsapp/ronportalform.php');
require_once('adsapp/ronportaldbrecord.php');

use Ron\DB;
use Ron\InputForm;
use Ron\DBRecord;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\User;


class KurzFormular extends RonPortalForm
{
    public $recId = null; // nastavi se pri insertu


    function __construct($database, $formAction = 'new')
    {
        parent::__construct($database, $formAction);

        if (isset($_POST['Id'])) $record = (int)$_POST['Id'];
        else if (isset($_GET['record'])) $record = (int)$_GET['record'];
        else $record = 0;

        $this->htmlAction = '?menuid=kurzy';
        $this->stornoAction = '?menuid=kurzy' . ($record ? '&action=show&record=' . $record : '');

        $this->title = \Ron\ronText('Kurz');
        $this->sqlTableName = 'EKurz';

        $this->sqlSelectPart =
            'Id, Nazev, Stav, Popis, Poznamka, PlatnostOd, PlatnostDo,' . // 0..6
            ' NULL, NULL'; // 7,8 - Revize.SestavaProCertifikat, Revize.UlozitCertifikatDoDok

        $this->addField(FORM_FIELD_TYPE_ID)
            ->setFieldName('Id') ->setSqlFieldIndex(0) ->setPrimary(1);

        $this->addField(FORM_FIELD_TYPE_EDIT)
            ->setFieldName('Nazev') ->setSqlFieldIndex(1)
            ->setCaption('Nazev') ->setMaxLength(160) ->setRequired(1);

        $this->addField(FORM_FIELD_TYPE_COMBO)
            ->setFieldName('Stav') ->setSqlFieldIndex(2)
            ->setCaption('Stav_kurzu') ->setRequired(1)
            ->setOptions([0 => \Ron\ronText('Kurz_Stav_0'), 1 => \Ron\ronText('Kurz_Stav_1'), 2 => \Ron\ronText('Kurz_Stav_2')])
            ->setSelectDataTypeTest(false) ->setDefaultValue(0);

        $this->addField(FORM_FIELD_TYPE_MEMO)
            ->setFieldName('Popis') ->setSqlFieldIndex(3)
            ->setCaption('Popis');

        $this->addField( [
            FORM_FIELD_TYPE => 'memo', FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_SQL_INDEX => 4,
            FORM_FIELD_CAPTION => \Ron\ronText('Poznamka'), FORM_FIELD_EDIT => 1 ] );

        $this->addField( [
            FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostOd', FORM_FIELD_SQL_INDEX => 5, FORM_FIELD_DEFAULT_VALUE => date('j.n.Y'),
            FORM_FIELD_CAPTION => \Ron\ronText('Platnost_od'), FORM_FIELD_EDIT => 1, FORM_FIELD_REQUIRED => 1 ] );

        $this->addField( [
            FORM_FIELD_TYPE => 'date', FORM_FIELD_NAME => 'PlatnostDo', FORM_FIELD_SQL_INDEX => 6,
            FORM_FIELD_CAPTION => \Ron\ronText('Platnost_do'), FORM_FIELD_EDIT => 1 ] );

        $this->addField(FORM_FIELD_TYPE_SELECT)
            ->setFieldName('SestavaProCertifikat') ->setSqlFieldIndex(7) ->setSqlFieldName(null) // POZOR! Pole je v tabulce RevizeEKurzu
            ->setSelectSql('SELECT Kod, Nazev FROM SestavyProIntraweb ORDER BY Nazev')
            ->setCaption('Sestava_pro_certifikat') ->setEditable(1) ->setShowEmptyValue(true);

        $this->addField( [
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_CHECK, FORM_FIELD_NAME => 'UlozitCertifikatDoDok',
            FORM_FIELD_SQL_INDEX => 8, FORM_FIELD_SQL_NAME => null,  // POZOR! Pole je v tabulce RevizeEKurzu
            FORM_FIELD_CAPTION => \Ron\ronText('Ulozit_certifikat_do_dokumentu'), FORM_FIELD_EDIT => 1 ] );
    }


    /**
     * @param int $idKurzu
     * @return array|null
     * @throws DB\DBException
     */
    protected function getRevize($idKurzu)
    {
        // nacte ID platne revize
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Id, SestavaProCertifikat, UlozitCertifikatDoDok FROM RevizeEKurzu' .
            ' WHERE IdKurzu = ' . $params ->addParameter($idKurzu) .
            '  AND Stav < 2' . // jen aktivni, pripadne pripravovane
            ' ORDER BY PlatnostOd DESC', // prvni je nejnovejsi
            $params);

        $row = $cursor ->fetch();
        $cursor ->close();

        return $row;
    }


    function loadRecord()
    {
        $result = parent::loadRecord();

        if ($result) {
            // nacte podrobnosti z posledni revize
            $rowRevize = $this->getRevize($result[0]);
            $result[7] = $rowRevize[1]; // Revize.SestavaProCertifikat
            $result[8] = $rowRevize[2]; // Revize.UlozitCertifikatDoDok
        }

        return $result;
    }


    /**
     * @param int $idKurzu
     * @param int $platnostOd
     * @return RonPortalDBRecord
     */
    protected function createRevizeRec($idKurzu, $platnostOd)
    {
        $revizeRec = new RonPortalDBRecord($this ->database, 'RevizeEKurzu');
        $revizeRec ->addField('ID', 'int', true);
        $revizeRec ->addField('IDKurzu', 'int', true, 0, $idKurzu);
        $revizeRec ->addField('PlatnostOd', 'date', true, 0, $platnostOd);
        $revizeRec ->addField('Popis', '', true, 160, \Ron\ronText('Revize') . ' 1');
        $revizeRec ->addField('Autor', '', true, 160, User::getUserName());
        $revizeRec ->addField('Stav', 'int', true, 0, 1); // aktivni

        $revizeRec ->addField('SestavaProCertifikat', '', false, 40, $_POST['SestavaProCertifikat'] ?: null);
        $revizeRec ->addField('UlozitCertifikatDoDok', 'int', false, 0, $_POST['UlozitCertifikatDoDok'] ? 1 : 0);

        return $revizeRec;
    }


    protected  function insertRevize($idKurzu, $platnostOd)
    {
        $revizeRec = $this ->createRevizeRec($idKurzu, $platnostOd);
        $idField = $revizeRec ->findFieldByName('ID');

        require_once('ron/sqlutils.php');
        $idField ->setNewValue( DB\SQLUtils::newSerialNumber($this ->database, 'RevizeEKurzu', 'ID') );

        return $revizeRec ->insertRecord();
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterInsertRecord($rec)
    {
        $this -> recId = $rec['Id'];

        $this ->insertRevize($this ->recId, $rec['PlatnostOd']);

//        $idField = $revizeRec ->findFieldByName('ID');

//        $revizeRec = new RonPortalDBRecord($this ->database, 'RevizeEKurzu');
//        $idField = $revizeRec ->addField('ID', 'int');
//        $revizeRec ->addField('IDKurzu', 'int', false, 0, $this ->recId);
//        $revizeRec ->addField('PlatnostOd', 'date', false, 0, $rec['PlatnostOd']);
//        $revizeRec ->addField('Popis', '', false, 160, \Ron\ronText('Revize') . ' 1');
//        $revizeRec ->addField('Autor', '', false, 160, User::getUserName());
//        $revizeRec ->addField('Stav', 'int', false, 0, 1); // aktivni
//        $revizeRec ->addField('SestavaProCertifikat', '', false, 40, $_POST['SestavaProCertifikat'] ?: null);
//        $revizeRec ->addField('UlozitCertifikatDoDok', 'int', false, 0, (int)$_POST['UlozitCertifikatDoDok'] ? 1 : 0);

//        require_once('ron/sqlutils.php');
//        $idField ->setNewValue( DB\SQLUtils::newSerialNumber($this ->database, 'RevizeEKurzu', 'ID') );
//
//        return $revizeRec ->insertRecord();

        return parent::afterInsertRecord($rec);
    }


    function afterUpdateRecord($rec)
    {
        $rowRevize = $this ->getRevize($rec['Id']);
        $idRevize = $rowRevize ? $rowRevize[0] : null;

        if (isset($idRevize)) {
            $revizeRec = $this ->createRevizeRec($rec['ID'], $rec['PlatnostOd']);

            $params = $this->database->createParameters();
            $revizeRec->setWhere('Id = ' . $params ->addParameter($idRevize), $params);

            $revizeRec ->loadRecord();
            $revizeRec ->updateRecord();
        } else {
            $this ->insertRevize($rec['ID'], $rec['PlatnostOd']);
        }

        return parent::afterUpdateRecord($rec);
    }


}


