<?php
/**
 * UTF8
 * stanislav.elbl, 08.07.2015
 */

namespace Ron\ELearning;


use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\FileUtils;
use Ron\RonPortal\Log;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\StringUtils;


class ObsahKurzu
{
    /** @var int */
    private $id;
    /** @var int */
    private $idRevizeKurzu;

    /** @var int */
    private $poradi;

    /** @var int */
    private $stav;  // 1 = aktivni, 2 = zrusena

    /** @var string */
    private $nazev;

    /** @var string */
    private $text;

    /** @var string   kod pro html odkazy */
    private $kodStrany;

    private $zobrazitHlavicku;
    private $zobrazitPaticku;

    /** @var int  Pripojeny test */
    var $druhHodnoceni;
    var $temaCislo;

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getIdRevizeKurzu()
    {
        return $this->idRevizeKurzu;
    }

    /**
     * @param int $idRevizeKurzu
     */
    public function setIdRevizeKurzu($idRevizeKurzu)
    {
        $this->idRevizeKurzu = $idRevizeKurzu;
    }


    /**
     * @return int
     */
    public function getPoradi()
    {
        return $this->poradi;
    }

    /**
     * @return int
     */
    public function getStav()
    {
        return $this->stav;
    }


    /**
     * @return string
     */
    public function getNazev()
    {
        return $this->nazev;
    }

    /**
     * @param string $nazev
     */
    public function setNazev($nazev)
    {
        $this->nazev = $nazev;
    }



    /**
     * @return string
     */
    public function getText()
    {
        return $this->text;
    }

    /**
     * @return string
     */
    public function getKodStrany()
    {
        return $this->kodStrany;
    }

    /**
     * @return mixed
     */
    public function getZobrazitHlavicku()
    {
        return $this->zobrazitHlavicku;
    }

    /**
     * @return mixed
     */
    public function getZobrazitPaticku()
    {
        return $this->zobrazitPaticku;
    }




    /**
     * @param Database $database
     * @param int $idRevizeKurzu
     * @return ObsahKurzu[]|bool
     */
    public static function loadObsah($database, $idRevizeKurzu)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, Poradi, Nazev, KodStrany, ZobrazitHlavicku, ZobrazitPaticku, ' . // 0..5
            $database->castWideMemo('Obsah') . ', ' . // 6
            ' DruhHodnoceni, TemaCislo' . // 7,8
            ' FROM ObsahEKurzu' .
            ' WHERE IdRevizeKurzu = ' . $params ->addParameter((int)$idRevizeKurzu) .
            '   AND Stav <> 2' .
            ' ORDER BY Poradi, Id';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani obsahu kurzu!');

        $result = [];
        while ($rowData = $cursor->fetch()) {
            $obsah = new ObsahKurzu();

            $obsah->id = $rowData[0];
            $obsah->idRevizeKurzu = $idRevizeKurzu;
            $obsah->poradi = $rowData[1];
            $obsah->nazev = $rowData[2];
            $obsah->kodStrany = $rowData[3];
            $obsah->zobrazitHlavicku = (int)$rowData[4];
            $obsah->zobrazitPaticku = (int)$rowData[5];
            $obsah->text = $rowData[6];

            $obsah ->druhHodnoceni = $rowData[7];
            $obsah ->temaCislo = $rowData[8];

            $result[] = $obsah;
        }

        return $result;
    }


    public function getJsonObsah()
    {
        if (!isset($this->text)) return null;

        $text = str_replace('%DOWNLOAD%', '%DOWNLOAD%&amp;revize=' . $this ->idRevizeKurzu, $this->text);

        return StringUtils::jsonText($text);
    }


    public function getJson()
    {
        return [
            'id' => $this->id,
            'poradi' => $this->poradi,
            'stav' => $this ->stav,
            'nazev' => StringUtils::jsonText($this->nazev),
            'kodStrany' => isset($this ->kodStrany) ? StringUtils::jsonText($this ->kodStrany) : null,
            'zobrazitHlavicku' => (int)$this ->zobrazitHlavicku,
            'zobrazitPaticku' => (int)$this ->zobrazitPaticku,
            'obsah' => $this ->getJsonObsah(), //  isset($this->text) ? StringUtils::jsonText($this->text) : null,
            'druhHodnoceni' => isset($this ->druhHodnoceni) ? (int)$this ->druhHodnoceni : null,
            'temaCislo' => isset($this ->temaCislo) ? (int)$this ->temaCislo : null
        ];
    }


    /**
     * @param Database $database
     * @param int $idRevizeKurzu
     * @param string $obsah
     * @throws DBException
     */
    public function insertRecord($database, $idRevizeKurzu, $obsah = null)
    {
        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($database, 'ObsahEKurzu');

        $this ->idRevizeKurzu = (int)$idRevizeKurzu;

        require_once('ron/sqlutils.php');
        $this ->poradi = SQLUtils::newSerialNumber($database, 'ObsahEKurzu', 'Poradi', 'IdRevizeKurzu = ' . $this ->idRevizeKurzu . ' AND Stav <> 2');
        $this ->id = SQLUtils::newSerialNumber($database, 'ObsahEKurzu', 'Id');
        $this ->stav = 1;
        $this ->zobrazitHlavicku = 1;
        $this ->zobrazitPaticku = 1;

        if (!$this ->nazev) {
            $this ->nazev = \Ron\ronText('Nazev_nove_strany') . ' ' . $this ->poradi;
        }

        $rec ->addField('Id', 'int', false, 0, $this ->id);
        $rec ->addField('IdRevizeKurzu', 'int', false, 0, $this ->idRevizeKurzu);
        $rec ->addField('Poradi', 'int', false, 0, $this ->poradi);
        $rec ->addField('Stav', 'int', false, 0, $this ->stav);

        $rec ->addField('Nazev', '', false, 160, $this ->nazev);

        $rec ->addField('ZobrazitHlavicku', 'int', false, 0, 1);
        $rec ->addField('ZobrazitPaticku', 'int', false, 0, 1);

        if ($obsah) {
            $rec ->addField('Obsah', '', false, 0, $obsah);
            $this ->text = $obsah;
        }

        $rec ->insertRecord();
    }
}


class TestKurzu
{
    var $druhHodnoceni;
    var $temaCislo;
    var $kod;
    var $popis;


    /**
     * @param Database $database
     * @param int  $idRevize
     * @return TestKurzu[]|bool
     * @throws DBException
     */
    public static function loadTesty($database, $idRevize)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT TemaHodnoceni.DruhHodnoceni, TemaHodnoceni.TemaCislo, TemaHodnoceni.Kod, TemaHodnoceni.PopisTematu' . // 0..5
            ' FROM TestEKurzu, TemaHodnoceni ' .
            ' WHERE TemaHodnoceni.DruhHodnoceni = TestEKurzu.DruhHodnoceni '.
            '   AND TemaHodnoceni.TemaCislo = TestEKurzu.TemaCislo '.
            '   AND TestEKurzu.IdRevize = ' . $params ->addParameter((int)$idRevize) .
            ' ORDER BY TemaHodnoceni.Kod';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani pripojenych testu!');

        $result = [];
        while ($rowData = $cursor ->fetch()) {
            $test = new TestKurzu();

            $test ->druhHodnoceni = $rowData[0];
            $test ->temaCislo = $rowData[1];
            $test ->kod = $rowData[2];
            $test ->popis = $rowData[3];

            $result[] = $test;
        }

        return $result;
    }


    /**
     * @return array
     */
    public function getJson()
    {
        return [
            //'id' => $this->id,
            'druhHodnoceni' => $this->druhHodnoceni,
            'temaCislo' => $this->temaCislo,
            'kod' =>StringUtils::jsonText($this->kod),
            'popis' => StringUtils::jsonText($this ->popis)
        ];
    }
}



class RevizeKurzu
{
    /** @var int */
    private $id;
    /** @var int */
    private $idKurzu;

    /** @var int */
    private $stav;

    /** @var string */
    private $popis;

    /** @var string */
    private $autor;

    /** @var int */
    private $platnostOd;
    /** @var int */
    private $platnostDo;

    private $zpusobPrezentace;
    private $hlavicka;
    private $paticka;


    /** @var PrilohaKurzu[] */
    private $prilohy;

    /** @var ObsahKurzu[] */
    private $obsah;

    /** @var  TestKurzu[] */
    private $testy;


    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getIdKurzu()
    {
        return $this->idKurzu;
    }

    /**
     * @return int
     */
    public function getStav()
    {
        return $this->stav;
    }

    /**
     * @return string
     */
    public function getPopis()
    {
        return $this->popis;
    }

    /**
     * @return string
     */
    public function getAutor()
    {
        return $this->autor;
    }

    /**
     * @return int
     */
    public function getPlatnostOd()
    {
        return $this->platnostOd;
    }

    /**
     * @return int
     */
    public function getPlatnostDo()
    {
        return $this->platnostDo;
    }

    /**
     * @return mixed
     */
    public function getZpusobPrezentace()
    {
        return $this->zpusobPrezentace;
    }

    /**
     * @return mixed
     */
    public function getHlavicka()
    {
        return $this->hlavicka;
    }

    /**
     * @return mixed
     */
    public function getPaticka()
    {
        return $this->paticka;
    }


    /**
     * @return ObsahKurzu[]
     */
    public function getObsah()
    {
        return $this->obsah;
    }

    /**
     * @return PrilohaKurzu[]
     */
    public function getPrilohy()
    {
        return $this->prilohy;
    }

    /**
     * @return TestKurzu[]
     */
    public function getTesty()
    {
        return $this->testy;
    }



    /**
     * @param Database $database
     * @param int $idKurzu
     * @param int $idRevize   Pokud je null, nacte vsechny revize, pokud je 0, nacte jen aktivni, jinak zadanou
     * @return RevizeKurzu[]|bool
     */
    public static function loadRevize($database, $idKurzu = null, $idRevize = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT Id, Popis, Stav, PlatnostOd, PlatnostDo, Autor, ' . // 0..5
            $database ->castWideMemo('Hlavicka') . ', ' . $database ->castWideMemo('Paticka') . ', ZpusobPrezentace, IdKurzu ' . // 6..9
            ' FROM RevizeEKurzu';

        if ( isset($idRevize) ) {
            if ($idRevize) {
                $sql .= ' WHERE Id = ' . $params ->addParameter((int)$idRevize);
            } else {
                // posledni aktivni
                $sql .=
                    ' WHERE IdKurzu = ' . $params ->addParameter((int)$idKurzu) .
                    ' AND Stav = 1';
            }
        } else {
            $sql .= ' WHERE IdKurzu = ' . $params ->addParameter((int)$idKurzu);
        }

        $sql .=
            ' ORDER BY PlatnostOd DESC';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($rowData = $cursor ->fetch()) {
            $revize = new RevizeKurzu();

            $revize ->id = $rowData[0];
            $revize ->idKurzu = $rowData[9];
            $revize ->stav = $rowData[2];
            $revize ->platnostOd = $database->stringToTimestamp($rowData[3]);
            $revize ->platnostDo = $rowData[4] ? $database->stringToTimestamp($rowData[4]) : null;
            $revize ->popis = $rowData[1];
            $revize ->autor = $rowData[5];

            $revize ->hlavicka = $rowData[6];
            $revize ->paticka = $rowData[7];
            $revize ->zpusobPrezentace = (int)$rowData[8];

            $result[] = $revize;
        }

        return $result;
    }


    function loadContent($database)
    {
        $this->obsah = ObsahKurzu::loadObsah($database, $this->getId());

        require_once 'elearning/prilohakurzu.php';
        $this->prilohy = PrilohaKurzu::loadPrilohy($database, $this->getId());
        $this->testy = TestKurzu::loadTesty($database, $this->getId());
    }


    /**
     * Vraci stranku se zadanym kodem / id / indexem
     * @param string $kod
     * @param int $id
     * @param int $index
     * @return int
     * @throws \Exception
     */
    function findPage($kod, $id = null, $index = null)
    {
        if ($kod) {
            foreach ($this->getObsah() as $index => $strana) {
                if ($strana->getKodStrany() == $kod) return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym kodem neexistuje!');
        } else if ($id) {
            foreach ($this->getObsah() as $index => $strana) {
                if ($strana->getId() == $id) return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym Id neexistuje!');
        } else {
            if ($index === 'first') $index = 0;
            if ($index === 'last') $index = sizeof($this->getObsah()) - 1;

            if ($index >= 0 && $index < sizeof($this->getObsah())) {
                return $index;
            }

            throw new \Exception('Chyba - stranka se zadanym indexem neexistuje!');
        }
    }


    function getJson()
    {
        $obj = [
            'id' => $this->id,  // id kurzu
            'revize' => $this->getId(),
            'obsahKurzu' => [],
            'prilohy' => [],
            'testy' => []
        ];

        foreach ($this->getObsah() as $strana) {
            $obj['obsahKurzu'][] = $strana->getJson();
        }

        foreach ($this->getPrilohy() as $priloha) {
            $obj['prilohy'][] = $priloha->getJson();
        }

        foreach ($this->getTesty() as $test) {
            $obj['testy'][] = $test->getJson();
        }


        return json_encode($obj);
    }



}



class Kurz
{
    /** @var int */
    private $id;

    /** @var int */
    private $stav;

    /* @var string */
    private $nazev;
    /* @var string */
    private $popis;
    /* @var string */
    private $poznamka;

    /* @var int */
    private $platnostOd;
    /* @var int */
    private $platnostDo;

    /** @var RevizeKurzu[] */
    private $revize;

    function __construct()
    {

    }


    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getStav()
    {
        return $this->stav;
    }

    /**
     * @return string
     */
    public function getNazev()
    {
        return $this->nazev;
    }

    /**
     * @return string
     */
    public function getPopis()
    {
        return $this->popis;
    }

    /**
     * @return string
     */
    public function getPoznamka()
    {
        return $this->poznamka;
    }

    /**
     * @return int
     */
    public function getPlatnostOd()
    {
        return $this->platnostOd;
    }

    /**
     * @return int
     */
    public function getPlatnostDo()
    {
        return $this->platnostDo;
    }

    /**
     * @return RevizeKurzu[]
     */
    public function getRevize()
    {
        return $this->revize;
    }

    /**
     * @param Database $database
     * @param int $id
     * @return bool
     * @throws \Exception
     */
    function load($database, $id)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, Nazev, Stav, PlatnostOd, PlatnostDo, ' . // 0..4
            $database->castWideMemo('Popis') . ', ' . $database->castWideMemo('Poznamka') .
            ' FROM EKurz WHERE Id = ' . $params ->addParameter((int)$id) .
            ' ORDER BY PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params, 'Chyba pri nacitani kurzu!');

        if (!$rowData = $cursor->fetch()) {
            throw new \Exception('Zadany kurz nebyl nalezen!');
        }

        $this->id = $rowData[0];
        $this->nazev = $rowData[1];
        $this->stav = $rowData[2];
        $this->platnostOd = $database->stringToTimestamp($rowData[3]);
        $this->platnostDo = $rowData[4] ? $database->stringToTimestamp($rowData[4]) : null;

        $this->popis = $rowData[5];
        $this->poznamka = $rowData[6];

        return true;
    }


    /**
     * @param Database $database
     * @param int $idRevize Pokud je null, nacte vsechny revize, pokud je 0, nacte posledni aktivni, jinak zadanou
     */
    function loadRevize($database, $idRevize = null)
    {
        $this->revize = RevizeKurzu::loadRevize($database, $this->id, $idRevize);
    }


    function loadContent($database)
    {
        if (sizeof($this->revize)) {
            $this->revize[0] ->loadContent($database);
        }
    }


    // Export revize do ZIPu
    function export()
    {
        // vlastnosti kurzu
        // vlastnosti revize
        // prilohy (filename z databaze) ID, Nazev, Obsah
        // seznam priloh (ID, Nazev)
    }
}



class KurzOsoby
{
    public $id;  // id zaznamu
    public $rc;  // osoba, ktere patri
    public $idRevize;
    public $idKurzu; // nacteno z revize
    public $stav;

    public $datumCasZahajeni;
    public $stranka;
    public $posledniPristup;

    // RC_Skoleni, DruhSkoleni, DatumVznikuPovinnosti
    public $rcSkoleni;
    public $druhSkoleni;
    public $datumVznikuPovinnosti;

    /**
     * @param Database $database
     * @param int $id
     * @return KurzOsoby
     * @throws \Exception
     */
    public static function load($database, $id)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT EKurzOsoby.Id, EKurzOsoby.RC, EKurzOsoby.IdRevize, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.Stranka, EKurzOsoby.PosledniPristup, ' . // 0..6
            '       RevizeEKurzu.IdKurzu, EKurzOsoby.RC_Skoleni, EKurzOsoby.DruhSkoleni, EKurzOsoby.DatumVznikuPovinnosti, ' . // 7..10
            '       RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.ulozitCertifikatDoDok' . // 11,12
            '  FROM EkurzOsoby, RevizeEKurzu ' .
            ' WHERE EKurzOsoby.Id = ' . $params->addParameter($id) .
            '   AND EKurzOsoby.IdRevize = RevizeEKurzu.Id';

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) throw new \Exception("Zaznam EKurzOsoby nenalezen!");

        $kurzOsoby = new KurzOsoby();
        $kurzOsoby->id = $row[0];
        $kurzOsoby->rc = $row[1];

        $kurzOsoby->idRevize = $row[2];
        $kurzOsoby->idKurzu = $row[7];
        $kurzOsoby->stav = $row[3];
        $kurzOsoby->datumCasZahajeni = $database->stringToTimestamp($row[4]);
        $kurzOsoby->stranka = $row[5];
        $kurzOsoby->posledniPristup = $database->stringToTimestamp($row[6]);

        $kurzOsoby ->rcSkoleni = $row[8];
        $kurzOsoby ->druhSkoleni = $row[9];
        $kurzOsoby ->datumVznikuPovinnosti = $database ->stringToTimestamp($row[10]);

        return $kurzOsoby;
    }


    /**
     * Ulozi posledni navstivenou stranku
     * @param Database $database
     * @param int $strana
     */
    function ulozStranku($database, $strana)
    {
        $params = $database ->createParameters();

        $sql =
            'UPDATE EKurzOsoby ' .
            ' SET Stranka = ' . $params ->addParameter($strana) . ', ' .
            ' PosledniPristup = ' . $params ->addParameter( $database ->timestampToParam(time()) ) .
            ' WHERE Id = ' . $params ->addParameter($this ->id);

        try {
            if ( !$database->sql($sql, $params) ) {
                throw new DBException($database, $sql);
            }
        } catch (DBException $e) {
            require_once('adsapp/ronportallog.php');
            Log::addRecordToDBLog($database, 'EKurzOsoby', 'ERROR', 'ELearning', $e ->dbMessage . "\n\n" . $e ->getErrorDescription());
        }
    }
}

