<?php
/**
 * UTF8
 * stanislav.elbl, 02.03.2016
 */

namespace Ron\ELearning;

use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\User;

require_once('adsapp/ronportaldbrecord.php');


class EKurzOsobyDBRecord extends RonPortalDBRecord
{
    public $fId;
    public $fIdRevize;
    public $fDruhSkoleni;
    public $fDatumVznikuPovinnosti;
    public $fStav;

    public function __construct($database, $id = null)
    {
        parent::__construct($database, 'EKurzOsoby');

        $this ->logDescription = 'Elearning - prezentace';

        // vse readonly
        $this ->fId = $this ->addField('Id', 'int', true);
        $this ->fIdRevize = $this ->addField('IdRevize', 'int', true);
        // uzivatel
        $this ->addField('RC', '', true, 15, User::getRC());
        $this ->addField('', '', true, 0, '   ' . User::getUserName());
        // skoleni
        $this ->addField('RC_Skoleni', '', true, 15, User::getRC());
        $this ->fDruhSkoleni = $this ->addField('DruhSkoleni', '', true, 10);
        $this ->fDatumVznikuPovinnosti = $this->addField('DatumVznikuPovinnosti', 'timestamp', true);

        // stav atd
        $this ->fStav = $this ->addField('Stav', 'int', true, 0, 1);

        $this ->addField('DatumCasZahajeni', 'timestamp', true, 0, time());
        $this ->addField('Stranka', 'int', true, 0, 0);

        $params = $this ->database ->createParameters();
        if ( isset($id) ) {
            $this ->setWhere('Id = ' . $params ->addParameter((int)$id), $params);
        }
    }

}


class GenerovaneSkoleniDBRecord extends RonPortalDBRecord
{
    const ABSOLVOVANO_USPESNE = 3;
    const ABSOLVOVANO_NEUSPESNE = 4;

    var $fDatumSkoleni;
    var $fPotvrzeni;

    /**
     * @param \Ron\DB\Database $database
     * @param string $rc
     * @param string $druhSkoleni
     * @param int    $datumVznikuPovinnosti
     */
    public function __construct($database, $rc, $druhSkoleni, $datumVznikuPovinnosti)
    {
        parent::__construct($database, 'GenerovaneSkoleni');

        $this ->logDescription = 'Elearning - prezentace';

        $this ->addField('RC', '', true);
        $this ->addField('DruhSkoleni', '', true);
        $this ->addField('DatumVznikuPovinnosti', 'timestamp', true);
        $this ->addField('CisloSkoleni', '', true);

        $this ->fDatumSkoleni = $this ->addField('DatumSkoleni', 'timestamp', true);
        $this ->fDatumSkoleni ->updateLog = true;  // zapisuje se jen pri zmene stavu

        $this ->fPotvrzeni = $this ->addField('Potvrzeni', 'int', true);

        $params = $this ->database ->createParameters();
        $this ->setWhere(
            'RC = ' . $params ->addParameter($rc) .
            ' AND DruhSkoleni = ' . $params ->addParameter($druhSkoleni) .
            ' AND DatumVznikuPovinnosti = ' . $params ->addParameter($database ->timestampToParam($datumVznikuPovinnosti)),
            $params
        );
    }


    function nastavStav($stav = self::ABSOLVOVANO_USPESNE)
    {
        $this ->fPotvrzeni ->setNewValue($stav);
        $this ->fDatumSkoleni ->setNewValue( time() );

        if ($this ->loadRecord()) {
            $this ->updateRecord();
        }
    }
}
