<?php
/**
 * UTF8
 * stanislav.elbl, 06.08.2015
 */

namespace Ron\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\FileUtils;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;
use Ron\StringUtils;


class PrilohaKurzu
{
    private $idPrilohy;
    private $idRevize;
    private $id; // id v ramci kurzu
    private $nazev;

    // ?menuid=kurzy&%DOWNLOAD%&priloha=<id>
    // ?menuid=kurzy&action=download&kurzosoby=<id_kurzu_osoby>&priloha=<id>

    /**
     * @return int
     */
    public function getIdPrilohy()
    {
        return $this->idPrilohy;
    }

    /**
     * @return int
     */
    public function getIdRevize()
    {
        return $this->idRevize;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return mixed
     */
    public function getNazev()
    {
        return $this->nazev;
    }


    /**
     * @param Database $database
     * @param int  $idRevize
     * @return PrilohaKurzu[]|bool
     * @throws DBException
     */
    public static function loadPrilohy($database, $idRevize) {
        $params = $database ->createParameters();

        $sql =
            'SELECT PrilohaRevizeEKurzu.IdRevizeKurzu, PrilohaRevizeEKurzu.IdPrilohy, PrilohaRevizeEKurzu.Id, Priloha.Nazev' . // 0..3
            ' FROM PrilohaRevizeEKurzu, Priloha ' .
            ' WHERE PrilohaRevizeEKurzu.IdPrilohy = Priloha.Id '.
            '   AND PrilohaRevizeEKurzu.IdRevizeKurzu = ' . $params ->addParameter((int)$idRevize) .
            ' ORDER BY Priloha.Nazev';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani priloh!');

        $result = [];
        while ($rowData = $cursor ->fetch()) {
            $priloha = new PrilohaKurzu();

            $priloha ->idRevize = $rowData[0];
            $priloha ->idPrilohy = $rowData[1];
            $priloha ->id = $rowData[2];
            $priloha ->nazev = $rowData[3];

            $result[] = $priloha;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @return array
     */
    public function getJson()
    {
        require_once('ron/fileutils.php');

        return [
            'id' => $this->id,
            'idPrilohy' => $this->idPrilohy,
            'nazev' => StringUtils::jsonText($this->nazev),
            'type' => FileUtils::getFileType($this->nazev)
        ];
    }


    /**
     * @param PrilohaKurzu[] $prilohy
     * @return array|null
     */
    public static function getJsonPrilohy($prilohy)
    {
        if (!isset($prilohy)) {
            return null;
        }

        $result = [];

        foreach ($prilohy as $priloha) {
            $result[] = $priloha ->getJson();
        }

        return $result;
    }
}



/**
 * OutputObject pro stahnuti prilohy z databaze
 * Class PrilohaOutputObject
 */
class PrilohaOutputObject implements OutputObject
{
    /** @var  \Ron\DB\Database */
    protected $database;
    /** @var  int */
    protected $idPrilohy;


    /**
     * @param \Ron\DB\Database $database
     * @param int  $idPrilohy
     */
    function __construct($database, $idPrilohy = null)
    {
        $this ->database = $database;
        $this ->idPrilohy = $idPrilohy;
    }


    /**
     * @return \Ron\DB\Cursor
     * @throws DBException
     * @throws \Exception
     */
    function getData()
    {
        if (!isset($this ->idPrilohy)) {
            throw new \Exception('Chyba v PrilohaOutputObject - neni zadani id prilohy!');
        }

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Nazev, Obsah FROM Priloha ' .
            ' WHERE Id = ' . $params ->addParameter((int)$this ->idPrilohy);

        return $this ->database ->getSqlResult($sql, $params);
    }

    /**
     * @return void
     * @throws \Exception
     */
    public function output()
    {
        $cursor = $this ->getData();

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba! Pozadovana priloha neexistuje!');
        }

        // headers
        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($row[0]);
        HttpResponse::renderHeaders();

        print $cursor ->stringToBinary($row[1]);
        exit;
    }
}


class PrilohaRevizeKurzuOutputObject extends PrilohaOutputObject
{
    /** @var  int */
    protected $idRevizeKurzu;
    /** @var  int */
    protected $id;

    /**
     * PrilohaRevizeKurzuOutputObject constructor.
     * @param \Ron\DB\Database $database
     * @param int $idRevizeKurzu
     * @param int $id
     */
    public function __construct($database, $idRevizeKurzu, $id)
    {
        parent::__construct($database);

        $this->idRevizeKurzu = $idRevizeKurzu;
        $this->id = $id;
    }


    function getData()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Nazev, Obsah FROM Priloha, PrilohaRevizeEKurzu ' .
            ' WHERE Priloha.ID = PrilohaRevizeEKurzu.IdPrilohy' .
            '   AND PrilohaRevizeEKurzu.IdRevizeKurzu = ' . $params ->addParameter((int)$this ->idRevizeKurzu) .
            '   AND PrilohaRevizeEKurzu.Id = ' . $params ->addParameter((int)$this ->id);

        return $this ->database ->getSqlResult($sql, $params);
    }
}
