<?php
/**
 * UTF8
 * stanislav.elbl, 25.11.2015
 */

namespace Ron\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Personalistika\Dotaznik;
use Ron\Personalistika\DotaznikViewObject;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;

require_once('person/dotaznik.php');

class Test extends Dotaznik
{
    const TEST_STAV_NEABSOLVOVANO = 0;
    const TEST_STAV_PROBIHA = 1;
    const TEST_STAV_NEDOKONCENO_V_LIMITU = 2;
    const TEST_STAV_ABSOLVOVANO_USPESNE = 3;
    const TEST_STAV_ABSOLVOVANO_NEUSPESNE = 4;

    /** @var  int   Kurz Osoby (pri vyplnovani testu) */
    public $idKurzuOsoby;

    /** @var  int   Index stranky (pouziva se pri vyplnovani testu) */
    public $pageIndex;


    function getJsonData()
    {
        $obj = [
            'druhHodnoceni' => $this ->druhHodnoceni,
            'temaCislo' => $this ->temaCislo,
            'test' => $this ->test,
            'otazky' => []

//            'id' => $this ->id,
//            'revize' => sizeof($this ->revize) ? $this ->revize[0] ->getId() : '',
//            'obsahKurzu' => [],
//            'prilohy' => []
        ];

        foreach ($this ->otazky as $otazka) {
            $obj['otazky'][] = $otazka ->getJson();
        }

        return $obj;
    }


    function getJson()
    {
        return json_encode($this ->getJsonData());
    }


    /**
     * @param Database $database
     * @param int $idPripravenehoTestu  ID zaznamu z tabulky PripraveneTemaHodnoceni
     * @return Test
     */
    private static function nactiTest($database, $idPripravenehoTestu)
    {
        $test = new Test($database);
        $test ->nactiPripravenyDotaznik($idPripravenehoTestu);

        return $test;
    }


    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param KurzOsoby $kurzOsoby
     * @return Test
     */
    public static function vygenerujTest($database, $druhHodnoceni, $temaCislo, $kurzOsoby)
    {
        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($database, 'PripraveneTemaHodnoceni');

        $newId = SQLUtils::newSerialNumber($database, 'PripraveneTemaHodnoceni', 'ID');
        $rec ->addField('ID', 'int', false, 0, $newId);
        $rec ->addField('DruhHodnoceni', 'int', false, 0, $druhHodnoceni);
        $rec ->addField('TemaCislo', 'int', false, 0, $temaCislo);
        $rec ->addField('DatumHodnoceni', 'timestamp', false, 0, time());
        $rec ->addField('Stav', 'int', false, 0, 1); // povinne pole
        $rec ->addField('Test_Stav', 'int', false, 0, self::TEST_STAV_PROBIHA);
        $rec ->addField('Test_datumCasPocatku', 'timestamp', false, 0, time());
        $rec ->addField('EKurzOsoby', 'int', false, 0, $kurzOsoby ->id);

        $rec ->insertRecord();

        return self::nactiTest($database, $newId);
    }


    /**
     * Vraci instanci testu osoby k zadanemu kurzu osoby. Pokud neexistuje, vraci null
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param KurzOsoby $kurzOsoby
     * @param string $filtr
     * @return Test
     */
    public static function nactiTestOsoby($database, $druhHodnoceni, $temaCislo, $kurzOsoby, $filtr = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT ID FROM PripraveneTemaHodnoceni ' .
            ' WHERE EKurzOsoby = ' . $params ->addParameter($kurzOsoby ->id) .
            '   AND DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            '   AND TemaCislo = ' . $params ->addParameter($temaCislo);

        if ($filtr) {
            $sql .= ' AND ' . $filtr;
        }

        $sql .= ' ORDER BY Test_datumCasPocatku DESC'; // aby nacetl posledni pokus

        $cursor = $database ->select($sql, $params);

        $row = $cursor ->fetch();
        if (!$row) return null; // self::vygenerujTest($database, $druhHodnoceni, $temaCislo, $kurzOsoby);

        return self::nactiTest($database, $row[0]);
    }


//    /**
//     * Nacte dodatecne udaje z tabulky TestEKurzu (casovy limit, pocet pokusu)
//     * @param Database $database
//     * @param int $idRevize
//     */
//    public function nactiTestEKurzu($database, $idRevize)
//    {
//        $params = $database ->createParameters();
//
//        $sql =
//            'SELECT PocetPokusu, CasovyLimit FROM TestEKurzu ' .
//            ' WHERE IdRevize = ' . $params ->addParameter($idRevize) .
//            '   AND DruhHodnoceni = ' . $params ->addParameter($this ->druhHodnoceni) .
//            '   AND TemaCislo = ' . $params ->addParameter($this ->temaCislo);
//
//        $cursor = $database ->select($sql, $params);
//
//        $row = $cursor ->fetch();
//        if ($row) {
//            $this ->pocetPokusu = $row[0];
//            $this ->casovyLimit = $row[1];
//        }
//    }


    function zobrazTlacitka($editace)
    {
        if ($editace) {
            echo '<input type="submit" name="ulozit" class="button" value="' . \Ron\ronText('Ulozit_test') . '" />';
            echo '<span style="float:right">';
            echo '<input type="submit" name="odeslat" class="submitbutton" value="' . \Ron\ronText('Odeslat_test') . '" data-ron-confirm-id="' . \Ron\ronText('Opravdu_odeslat_test') . '" />';
            echo '</span>';
        }
    }


    function zobrazDotaznik($editace, $postValues)
    {
        echo "<FORM name='dotaznik' class='inputform' method='post' action='?menuid=prezentace&kurzosoby={$this->idKurzuOsoby}&pageindex={$this ->pageIndex}'>\n";

//        if ($editace) $action = 'ulozdotaznik';
//        else if ($this->jeEditovatelnyStav()) $action = 'uzavridotaznik';
//        else $action = '';
        $action='uloztest';

        echo "<INPUT type='hidden' name='testaction' value='$action'>\n";

        if ($this->fXMLSablona) {
            $this->zobrazDotaznikXML($editace, $postValues);
        } else {
            $oznacitSpravne = 0;

            if (!$editace) {
                switch ($this ->test_stav) {
                    case 3 :
                        $oznacitSpravne = $this->zobrazitOdpovediUspesne;
                        break;

                    case 2 :
                    case 4 :
                        // neuspesne nebo pozde dokonceny test
                        $oznacitSpravne = $this ->zobrazitOdpovediNeuspesne;
                        break;
                }
            }

            if ($oznacitSpravne) {
                $this ->nactiSpravneOdpovedi();
            }

            $this->zobrazObsahDotazniku($editace, $oznacitSpravne, RonPortalSettings::getIntValue('povolitHTMLvTestech', 1));
        }

        echo "</FORM>\n";

        echo "<script>$( function() { ronPortal.hodnoceni(); } ); </script>";
    }


    /**
     * @param int $novyStav
     * @return bool|void
     * @throws DBException
     */
    function ulozVyplnenyTest($novyStav = 1) // 1=TEST_STAV_PROBIHA
    {
        if (!$this ->loadPost()) {
            return false;
        }

        $result = $this ->ulozOdpovedi();

        // zapise jmeno hodnoticiho a cas
        $params = $this ->database ->createParameters();
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            '   SET HodnotilRC = ' . $params->addParameter(User::getRC()) . ', ' .
            '       Hodnotici = ' . $params->addParameter(User::getLoginName()) . ', ' .
            '       DatumHodnoceni = ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ', ' .
            '       Test_Stav = ' . $params ->addParameter($novyStav) . ', ' . // 1 = probiha
            '       Poznamka = ' . $params ->addParameter( $this ->poznamkaHodnoceni ) .
            ' WHERE Id = ' . $params ->addParameter((int)$this ->fId);

        if (!$this ->database ->sql($sql, $params)) {
            throw new DBException($this ->database, $sql, 'Pri aktualizaci testu doslo k chybe!');
        }

        return $result;
    }
}



class TestEKurzu
{
    /** @var Test */
    public $test;

    public $idRevize;
    public $druhHodnoceni;
    public $temaCislo;

    // Hodnoty z tabulky TestEKurzu
    public $pocetPokusu;
    public $casovyLimit;

    // Hodnoty uzivatele
    public $vycerpanoPokusu;


    /**
     * @param Database $database
     * @param int $idRevize
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @return TestEKurzu
     * @throws \Exception
     */
    public static function nactiTestEKurzu($database, $idRevize, $druhHodnoceni, $temaCislo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PocetPokusu, CasovyLimit FROM TestEKurzu ' .
            ' WHERE IdRevize = ' . $params ->addParameter($idRevize) .
            '   AND DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            '   AND TemaCislo = ' . $params ->addParameter($temaCislo);

        $cursor = $database ->select($sql, $params);

        $row = $cursor ->fetch();
        if ($row) {
            $result = new TestEKurzu();

            $result ->idRevize = $idRevize;
            $result ->druhHodnoceni = $druhHodnoceni;
            $result ->temaCislo = $temaCislo;

            $result ->pocetPokusu = $row[0];
            $result ->casovyLimit = $row[1];

            return $result;
        }

        // vyhodit chybu?
        throw new \Exception('TestEKurzu: zadany zaznam nebyl nalezen!' );
    }


    /**
     * @param Database $database
     * @param KurzOsoby $kurzOsoby
     */
    public function loadPocetAbsolvovanych($database, $kurzOsoby)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT COUNT(*) FROM PripraveneTemaHodnoceni ' .
            ' WHERE EKurzOsoby = ' . $kurzOsoby ->id;

        $cursor = $database ->select($sql, $params);

        $row = $cursor ->fetch();
        $this ->vycerpanoPokusu = $row ? $row[0] : 0;
    }


    /**
     * Vraci true, pokud je mozne spustit (dalsi) test
     * @return bool
     */
    public function lzeSpustitTest()
    {
        return !$this ->pocetPokusu || $this ->vycerpanoPokusu < $this ->pocetPokusu;
    }


    function getJson()
    {
        $data = $this ->test
            ? $this ->test ->getJsonData()
            : [];

        $data['idRevize'] = $this ->idRevize;
        if ($this ->pocetPokusu) $data['pocetPokusu'] = $this ->pocetPokusu;
        if ($this ->casovyLimit) $data['casovyLimit'] = $this ->casovyLimit;

        return json_encode($data);
    }

}


class TestViewObject extends DotaznikViewObject
{
    /** @var  TestEKurzu */
    public $testEKurzu;

    /**
     * TestViewObject constructor.
     * @param TestEKurzu $testEKurzu
     * @param bool $editace
     */
    public function __construct(TestEKurzu $testEKurzu, $editace)
    {
        parent::__construct($testEKurzu ->test, $editace, false);
        $this->testEKurzu = $testEKurzu;
    }


    /**
     * Vraci obrazek pro zadany stav. Pokud je $stav=null, tak vraci obrazek pro aktualni stav
     * @param null $stav
     * @return string
     */
    public function getInfoImage($stav = null)
    {
        if (!$stav) $stav = $this ->testEKurzu ->test ->test_stav;

        switch ($stav) {
            case '3' :
                //return 'images/32_ok.png';
                return 'fa-check';
            case '2' :
            case '4' :
                //return 'images/32_cancel.png';
                return 'fa-times';
            default :
                return '';
        }
    }


    function getJsonPrezentace()
    {
        $obj = ['editace' => $this->editace ? 1 : 0];

        if ($this ->testEKurzu ->casovyLimit) {
            $obj['casovyLimit'] = $this ->testEKurzu ->casovyLimit; // pocet sekund od zacatku
            $obj['casDo'] = date('j.n.Y H:i:s', $this ->testEKurzu ->test ->test_datumCasPocatku + $this ->testEKurzu ->casovyLimit);
        }

        return json_encode($obj);
    }

}
