<?php
/**
 * UTF8
 * stanislav.elbl, 07.12.2015
 */

namespace Ron\ELearning;

use Ron\DB\SQLUtils;
use Ron\DBRecordField;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\DB\Database;

require_once('adsapp/ronportaldbrecord.php');


class OtazkaDBRecord extends RonPortalDBRecord
{
    var $fPoradi;

    var $fPopis;
    var $fUpresnujiciPopis;
    var $fTypRozsahu;
    var $fBodovaVaha;
    var $fPovinnaOdpoved;
    var $fSpravnaHodnota;

    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param string $kodKriteria
     * @param string $filtrKod Pokud je zadan, nastavi se where podminka
     * @param object|string $logDescription
     */
    public function __construct($database, $druhHodnoceni, $temaCislo, $kodKriteria, $filtrKod = null, $logDescription = 'Testy')
    {
        parent::__construct($database, 'HodnoticiKriterium', $logDescription);

        $this ->addField('DruhHodnoceni', 'int', true, 0, $druhHodnoceni); // readonly
        $this ->addField('TemaCislo', 'int', true, 0, $temaCislo);
        $this ->addField('KodKriteria', 'string', false, 10, $kodKriteria );

        $this ->fPoradi = $this ->addField('Poradi', 'int', true);

        $this ->fPopis = $this ->addField('Popis', 'string', true, 250);
        $this ->fUpresnujiciPopis = $this ->addField('UpresnujiciPopis', 'memo', true, 0);
        $this ->fTypRozsahu = $this ->addField('TypRozsahu', 'string', true, 10 );
        $this ->fBodovaVaha = $this ->addField('BodovaVaha', 'int', true, 1);
        $this ->fPovinnaOdpoved = $this ->addField('PovinnaOdpoved', 'int', true, 0, 0);
        $this ->fSpravnaHodnota = $this ->addField('SpravnaHodnota', 'int', true);

        if ( isset($filtrKod)  &&  $filtrKod != '' ) {
            $params = $database ->createParameters();
            $this ->setWhere(
                'DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
                ' AND TemaCislo = ' . $params ->addParameter($temaCislo) .
                ' AND KodKriteria = ' . $params->addParameter( $filtrKod ),
                $params);
        }
    }


    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @return int
     * @throws \Ron\DB\DBException
     */
    static function novePoradi($database, $druhHodnoceni, $temaCislo)
    {
        require_once('ron/sqlutils.php');

        $params = $database ->createParameters();
        return SQLUtils::newSerialNumber(
            $database, 'HodnoticiKriterium', 'Poradi',
            'DruhHodnoceni = ' . $params ->addParameter((int)$druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter((int)$temaCislo),
            $params
            );
    }
}


class VycetOdpovediDBRecord extends RonPortalDBRecord
{
    /** @var  DBRecordField */
    var $fHodnota;
    /** @var  DBRecordField */
    var $fPopis;
    /** @var  DBRecordField */
    var $fKoeficient;

    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param string $kodKriteria
     */
    public function __construct($database, $druhHodnoceni, $temaCislo, $kodKriteria)
    {
        parent::__construct($database, 'VycetOdpovedi');
        $this ->logDescription = 'Testy';

        $this ->addField('DruhHodnoceni', 'int', true, 0, $druhHodnoceni);
        $this ->addField('TemaCislo', 'int', true, 0, $temaCislo);
        $this ->addField('KodKriteria', 'string', true, 10, $kodKriteria);

        $this ->fHodnota = $this ->addField('Hodnota', 'int', true);
        $this ->fPopis = $this ->addField('Popis', 'memo');
        $this ->fKoeficient = $this ->addField('Koeficient', 'float');

        $params = $database ->createParameters();
        $this ->setWhere(
            'DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter($temaCislo) .
            ' AND KodKriteria = ' . $params->addParameter($kodKriteria),
            $params);
    }
}


class TestDBRecord extends RonPortalDBRecord
{
    const DRUH_HODNOCENI_TEST = 6;

    public $fDruhHodnoceni;
    public $fTemaCislo;

    public $fKod;
    public $fPopisTematu;
    public $fPoznamka;
    public $fMinPocetBodu;
    public $fMaxPocetBodu;
    public $fSirkaSloupce;
    public $fEditovatPoznamku;
    public $fZobrazitOdpovediUspesne;
    public $fZobrazitOdpovediNeuspesne;

    public function __construct($database, $druhHodnoceni, $temaCislo = null)
    {
        parent::__construct($database, 'TemaHodnoceni');
        $this ->logDescription = 'Testy';

        $this ->fDruhHodnoceni = $this ->addField('DruhHodnoceni', 'int', true, 0, $druhHodnoceni);
        $this ->fTemaCislo = $this ->addField('TemaCislo', 'int', true, $temaCislo);

        $this ->fKod = $this ->addField('Kod', 'string', true, 20);
        $this ->fPopisTematu = $this ->addField('PopisTematu', 'string', true, 120);
        $this ->fPoznamka = $this ->addField('Poznamka', 'memo', true);
        $this ->fMinPocetBodu = $this ->addField('MinPocetBodu', 'int', true);
        $this ->fMaxPocetBodu = $this ->addField('MaxPocetBodu', 'int', true);
        $this ->fSirkaSloupce = $this ->addField('SirkaSloupce', 'int', true);
        $this ->fEditovatPoznamku = $this ->addField('EditovatPoznamku', 'int', true, 0, 0);
        $this ->fZobrazitOdpovediUspesne = $this ->addField('ZobrazitOdpovediUspesne', 'int', true, 0, 0);
        $this ->fZobrazitOdpovediNeuspesne = $this ->addField('ZobrazitOdpovediNeuspesne', 'int', true, 0, 0);
    }


    function insertRecord()
    {
        // vygeneruje nove temaCislo
        $params = $this ->database ->createParameters();
        $temaCislo = SQLUtils::newSerialNumber($this ->database,
            'TemaHodnoceni', 'TemaCislo',
            'DruhHodnoceni = ' . $params ->addParameter((int)$this ->fDruhHodnoceni ->value),
            $params);

        $this ->fTemaCislo ->value = $temaCislo;

        return parent::insertRecord();
    }
}
