<?php

namespace Ron\RonPortal;

use Ron\DB\SqlStatement;

require_once('adsapp/ronportalbrowser.php');


class MzdovePolozkyBrowser extends RonPortalBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'mzdovepolozky');

        $this->code = 'browsermzdovepolozky';
        $this->title = \Ron\ronText('Prehled_mzdovych_polozek');

        $this->sqlFromPart =
            '( MzdovaPolozkaVeMzde INNER JOIN TypMzdovePolozky ' .
            '  ON TypMzdovePolozky.Kod = MzdovaPolozkaVeMzde.MzdovaPolozka ' .
            ') ' .
            'LEFT OUTER JOIN VlastnostMzdovePolozky ' .
            'ON VlastnostMzdovePolozky.Kod = TypMzdovePolozky.Kod ' .
            '   AND VlastnostMzdovePolozky.PlatnostOd <= MzdovaPolozkaVeMzde.Obdobi' .
            '   AND (VlastnostMzdovePolozky.PlatnostDo >= MzdovaPolozkaVeMzde.Obdobi OR  VlastnostMzdovePolozky.PlatnostDo IS NULL)';

        $this->sqlWherePart = 'MzdovaPolozkaVeMzde.DruhZdrojeMzdovePolozky = 4';   // jen dochazkove MP

        global $STR_Rozdeleni; // kompat.

        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MzdovaPolozkaVeMzde.MzdovaPolozka',
            BROWSER_FIELD_CODE => 'col-kod', BROWSER_FIELD_HEADER_TEXT => 'Kod_mzdove_polozky'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'TypMzdovePolozky.Popis',
            BROWSER_FIELD_CODE => 'col-popis', BROWSER_FIELD_HEADER_TEXT => 'Mzdova_polozka'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MzdovaPolozkaVeMzde.PocetJednotek',
            BROWSER_FIELD_CODE => 'col-pocet',
            BROWSER_FIELD_HEADER_TEXT => 'Pocet_jednotek', BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ALIGN => 'right'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MzdovaPolozkaVeMzde.CenaZaJednotku',
            BROWSER_FIELD_CODE => 'col-cenazajednotku',
            BROWSER_FIELD_HEADER_TEXT => 'Cena_za_jednotku', BROWSER_FIELD_FORMAT => 'number2', BROWSER_FIELD_ALIGN => 'right'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MzdovaPolozkaVeMzde.Castka',
            BROWSER_FIELD_CODE => 'col-castka',
            BROWSER_FIELD_HEADER_TEXT => 'Castka', BROWSER_FIELD_FORMAT => 'number2', BROWSER_FIELD_ALIGN => 'right'] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'VlastnostMzdovePolozky.NazevJednotky',
            BROWSER_FIELD_CODE => 'col-jednotka',] );
        $this->addField( [
            BROWSER_FIELD_SQL_NAME => 'MzdovaPolozkaVeMzde.Rozdeleni',
            BROWSER_FIELD_CODE => 'col-rozdeleni', BROWSER_FIELD_HEADER => $STR_Rozdeleni ?: \Ron\ronText('Rozdeleni')] );

        $this->defaultSort = 1;
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        parent::setFilterForm();

        // jina prava pro mzdove polozky
        $prava = Prava::vratPrava('zobrazit_mzdpol');
        $this->filterForm->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi

        $this->filterForm->f_pravarozdeleni = $prava;
        $this->filterForm->nastavAkciGet('mzdovepolozky');
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            ' MzdovaPolozkaVeMzde.Obdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND MzdovaPolozkaVeMzde.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND MzdovaPolozkaVeMzde.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND MzdovaPolozkaVeMzde.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }

}

