<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 14.02.2019
 */

namespace Ron\RonPortal;


use Ron\Browser;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;

require_once 'adsapp/ronportalbrowser.php';


/**
 * Class StavZadostiField
 * @package Ron\RonPortal
 */
class StavZadostiField extends BrowserField
{

    /** @var Database $database */
    public $database;

    /**
     * ProhlaseniField constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct();

        $this->database = $database;
        $this->className = 'center';
        $this->header = \Ron\ronText('stav_zadosti');
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        switch ($rowData[4]){
            case '0':
                $result .= \Ron\ronText('Mzdy_navrh');
                break;

            case '1':
                $result .= "<span class=''>" . \Ron\ronText('Mzdy_schvaleno') . "</span>";
                break;

            case '2':
                $result .= "<span class=''>" . \Ron\ronText('Mzdy_zamitnuto') . "</span>";
                break;

        }

        return $result;
    }
}


/**
 * Class RocniZuctovaniZadamField
 * @package Ron\RonPortal
 */
class RocniZuctovaniZadamField extends BrowserField
{

    /** @var Database $database */
    public $database;

    /**
     * ProhlaseniField constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct();

        $this->database = $database;
        $this->className = 'center';
        $this->header = \Ron\ronText('zadost_o_rocni_zuctovani');
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        switch ($rowData[4]){
            case '0':
                $result .= \Ron\ronText('zadost_nezadam');
                break;

            case '1':
                $result .= "<span class='bold'>" . \Ron\ronText('zadost_zadam') . "</span>";
                break;


        }

        return $result;
    }
}


/**
 * Class RocniZuctovaniZadamNezadamField
 * @package Ron\RonPortal
 */
class RocniZuctovaniZadamNezadamField extends BrowserField
{
    /** @var Database $database */
    public $database;

    /**
     * ProhlaseniField constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct();

        $this->database = $database;
        $this->className = 'center';
        $this->header = \Ron\ronText('zadost_o_rocni_zuctovani');
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        $obdobi = $this->database->stringToTimestamp($rowData[0]);
        $datumProhlaseni = $this->database->stringToTimestamp($rowData[3]);

        //"RVD.ZadostPortalStav, RVD.ProhlaseniPortal, S_AnoNe.Popis, RVD.ProvestRocniVyuctovani, RVD.DatumProhlaseniPortal";     //4..8
        switch ($rowData[4]){
            //Stav zadosti na Portale
            case '0':
                if ($rowData[7] == 0){
                    //Pozaduje provest rocni vyuctovani
                    $result .= "<span class='bold'>" .
                                    \Ron\ronText('zadost_nezadam') .
                                "</span>";
                    $result .= " / ";
                    $result .= "<span class=''>" .
                                    "<a href='?menuid=dane&action=rocnizuctovani&obdobi=$obdobi&datumprohlaseni=$datumProhlaseni&zadamzuctovani=1'>" .
                                    \Ron\ronText('zadost_zadam') .
                        "</a>";

                    $result .= "</span>";
                } else {
                    //Nepozaduje rocni vyuctovani dane
                    $result .= "<span class=''>" .
                                    "<a href='?menuid=dane&action=rocnizuctovani&obdobi=$obdobi&datumprohlaseni=$datumProhlaseni&zadamzuctovani=0'>" . \Ron\ronText('zadost_nezadam') . "</a>" .
                                "</span>";
                    $result .= " / ";
                    $result .= "<span class='bold'>" .
                                    \Ron\ronText('zadost_zadam') .
                                "</span>";
                }

                break;

            case '1':
                $result .= "<span class=''>" . \Ron\ronText('zadost_zadam') . "</span>";
                break;

        }

        return $result;
    }
}


/**
 * Class ZadostField
 * @package Ron\RonPortal
 */
class ZadostField extends BrowserField
{
    /**
     * ZadostField constructor.
     * @param Database $database
     * @param array|null $options
     */
    public function __construct($database, array $options = null)
    {
        parent::__construct($database, $options);

        $this->database = $database;
        $this->className = 'center';
        $this->header = \Ron\ronText('formular_zadosti');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";
        $obdobi = $this->database->stringToTimestamp($rowData[0]);

        //TODO pokud je po datu 15.2. a není povoleno "Možnost podepsat žádost po termínu"
        //pak editaci neumožnit


        if ($obdobi > 0) {
            $result .= "<a href='?menuid=dane&action=formularzadosti&obdobi=$obdobi' >";

            $aktualniRok = date('Y', DateUtils::incDay(DateUtils::firstDayOfYear(time()), -1));
            $rokVSystemu = date("Y", $this->database->stringToTimestamp($rowData[0]));

            $povolitPoTerminu = (bool)(RonPortalSettings::getIntValue('zadostizuctovanidanepoterminu'));

            if ($povolitPoTerminu) {
                $lzeEditovat = true;
            } else {
                $lzeEditovat = (mktime(0, 0, 0, 2, 15, (int)date("y", time()))) > time();
            }

            //pokud bude zadost schvalena ucetni, pak umoznit pouze nahled jiz ne editaci
            if ($rowData[4] == 0 && ($rokVSystemu == $aktualniRok) && $rowData[7] == 1 && $lzeEditovat) {
                $result .= "<span class='bold'>" . \Ron\ronText("Mzdy_vyplnit") . "</span>";
            } else {
                $result .= "<span>" . \Ron\ronText("nahled") . "</span>";
            }

            $result .= "</a>";
        }

        return $result;
    }
}



/**
 * Class BrowserZadostORocniZuctovaniZaloh
 * @package Ron\RonPortal
 */
class BrowserZadostORocniZuctovaniZaloh extends Browser
{

    /**
     * BrowserZadostORocniZuctovaniZaloh constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);

        $this->title = \Ron\ronText('zadost_o_rocni_zuctovani_zaloh');

        //TODO upravit pro žádost viz RocniVyuctovaniDane.DatumZadosti, RocniVyuctovaniDane.ZadostPortalStav, RocniVyuctovaniDane.ZadostPortal
        $this->sqlSelectPart =
            "RVD.Rok,  RVD.DatumProhlaseni,  Obdobi.Popis, RVD.DatumZadostiPortal, " .       //0..3
            "RVD.ZadostPortalStav, RVD.ProhlaseniPortal, S_AnoNe.Popis, RVD.ProvestRocniVyuctovani, RVD.DatumProhlaseniPortal";     //4..8

        $this->sqlFromPart =
            "RocniVyuctovaniDane RVD LEFT JOIN Obdobi ON (RVD.Rok = Obdobi.PocatekObdobi AND Obdobi.Aplikace = 'Mzdy')" .
            " LEFT JOIN S_AnoNe ON (RVD.ProhlaseniPortal = S_AnoNe.Kod)";



        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => "RVD.Rok",
            BROWSER_FIELD_FORMAT => "year", BROWSER_FIELD_HEADER_TEXT => 'rok', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField(new StavZadostiField($this->database));

//        $this->addField(new RocniZuctovaniZadamField($this->database));

        $this->addField(new RocniZuctovaniZadamNezadamField($this->database));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => "RVD.DatumProhlaseni",
            BROWSER_FIELD_FORMAT => "date", BROWSER_FIELD_HEADER_TEXT => 'datum_prohlaseni', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);


        $this->addField([BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => "RVD.DatumZadostiPortal",
            BROWSER_FIELD_FORMAT => "date", BROWSER_FIELD_HEADER_TEXT => 'zadost_datum_zadosti_portal', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField(new ZadostField($database));

    }


    /**
     * @return \Ron\DB\SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = "";
        $aktualniRok = mktime(0,0,0,1,1, (int)date("Y", time()));

        $result .= "Rok < " .  $params->addParameter($this->database->timestampToParam($aktualniRok));
        $result .= " AND RC = " .  $params->addParameter(User::getRC());

        return new SqlStatement($result, $params);
    }
}



