<?php
/** UTF-8
 * stanislav.elbl, 06.11.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Mzdy\SrazkyBrowser;

class Dovolena
{
    public $obdobi;
    public $prepocetNaHodiny;
    public $typDovolene;
    public $narokDnu;
    public $narokHodin;
    public $cerpanoDnu;
    public $cerpanoHodin;
    public $zustatekDnu;
    public $zustatekHodin;
}

class MzdyController implements Controller
{
    /** @var Database */
    var $database;


    /**
     * MzdyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param $action
     * @return RonPortalFilterForm
     * @throws Exception
     */
    function getFilterForm($action)
    {
        require_once('adsapp/ronportalfilter.php');

        $datumOd = DateUtils::firstDayOfMonth(time());
        $datumDo = DateUtils::lastDayOfMonth(time());

        $frm = new RonPortalFilterForm($this ->database, $action);
        $frm->nastavOsoby(true, Prava::vratPrava('prohl_mzdy'), $datumOd, $datumDo); // datum od-do podle obdobi
        $frm->nastavAkciGet($action);
        $frm->loadData();

        if (!RonPortalSession::vybranaOsoba()) {
            RonPortalUtils::neniPravoException();
        }

        return $frm;
    }


    function nactiDovolenou()
    {
        //26.4.2018 dle konzultace s Kvetou se nacita z CSM 297, 298, 299 k obdobi posledni uzavrene mzdy zamestnance

        //TODO: nacitat lonsky zustatek, stejne jako ve vyplatnich listcich
        $dovolena = new Dovolena();

        $params = $this->database->createParameters();

        $sql =
            "SELECT MAX(Obdobi) ".
            "FROM Mzda " .
            "WHERE RC = " . $params->addParameter(RonPortalSession::osobaRC()) .
            "  AND OsobniCislo = " . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            "  AND DatumNastupu = " . $params->addParameter($this->database->dateToParam(RonPortalSession::osobaNastup())) .
            "  AND Schvaleno = 1";

        try {
            $obdobi = $this->database->stringToTimestamp(DBUtils::getSqlValue($this->database, $sql, $params));
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při načítání posledního schváleného období mzdy ') . $exception->getMessage());
            return;
        }

        $dovolena->obdobi = $obdobi;

        $params = $this->database->createParameters();

        $sql =
            "SELECT TypCasoveSlozkyMzdy, Dny, Hodiny ".
            "FROM CasovaSlozkaMzdy " .
            "WHERE RC = " . $params->addParameter(RonPortalSession::osobaRC()) .
            "  AND OsobniCislo = " . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            "  AND DatumNastupu = " . $params->addParameter($this->database->dateToParam(RonPortalSession::osobaNastup())) .
            "  AND Mesic = " . $params->addParameter($this->database->dateToParam($dovolena->obdobi)) .
            "  AND TypCasoveSlozkyMzdy IN ('297', '298', '299')";

        try {
            $cursor = $this->database->getSqlResult($sql, $params);

            while ($row = $cursor->fetch()) {
                switch ($row[0]) {
                    case '297':
                        $dovolena->narokDnu = $row[1];
                        $dovolena->narokHodin = $row[2];
                        break;
                    case '298':
                        $dovolena->cerpanoDnu = $row[1];
                        $dovolena->cerpanoHodin = $row[2];
                        break;
                    case '299':
                        $dovolena->zustatekDnu = $row[1];
                        $dovolena->zustatekHodin = $row[2];
                        break;
                }
            }

            $cursor->close();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při načítání dovolené zaměstnance ') . $exception->getMessage());
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT TypDovolene, NarokDnu, CerpanoDnuCelkem, ZustatekDovolene, PrepocetNaHodiny " .
            " FROM Dovolena " .
            " WHERE RC = " . $params->addParameter(RonPortalSession::osobaRC()) .
            " AND OsobniCislo = " . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            " AND DatumNastupu = " . $params->addParameter($this ->database ->dateToParam(RonPortalSession::osobaNastup())) .
            " AND Rok = " . $params->addParameter($this ->database ->dateToParam(DateUtils::firstDayOfYear($dovolena->obdobi)));

        try {
            $cursor = $this->database->getSqlResult($sql, $params);

            $row = $cursor->fetch();

            $dovolena->prepocetNaHodiny = $row ? $row[4] : null;
            $dovolena->typDovolene = $row ? $row[0] : null;

            $cursor ->close();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri nacitani nastaveni dovolene zamestnance ') . $exception->getMessage());
        }

        Model::addData('dovolena', $dovolena);
    }


    /**
     * @throws Exception
     */
    function nactiVyplaceniMzdy()
    {
        $params = $this ->database ->createParameters();

        $sql =
            "SELECT V.ZpusobPlatbyVyplaty, ZP1.Popis, V.SerioveCisloUctuVyplaty, " . //0..2
            " V.ZpusobPlatbyZalohy, ZP2.Popis, V.SerioveCisloUctuZalohy, " . //3..5
            " V.Castka, V.DefinovatCastkuHotove, " . //6,7
            " V.Adresat, V.Ulice, V.Mesto, V.PSC " . //8..11

            " FROM VyplaceniMzdy V, S_ZpusobPlatby ZP1, S_ZpusobPlatby ZP2 " .
            " WHERE ZP1.Kod = V.ZpusobPlatbyVyplaty " .
            "   AND ZP2.Kod = V.ZpusobPlatbyZalohy " .
            "   AND V.RC = " . $params->addParameter(RonPortalSession::osobaRC()) .
            "   AND V.OsobniCislo = " . $params->addParameter(RonPortalSession::osobaOsobniCislo()) .
            "   AND V.DatumNastupu = " . $params->addParameter($this->database->dateToParam(RonPortalSession::osobaNastup())) .
            "   AND V.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(time())) .
            "   AND (V.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(time())) . " OR V.PlatnostDo IS NULL)";

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        Model::addData('vyplaceniMzdy', $row);

        if ($row[0] = 1) {
            if (isset($row[2])) {
                // na ucet
                $params = $this->database->createParameters();
                $cursor = $this->database->getSqlResult(
                    'SELECT KodBanky, CisloUctu FROM BankovniUcet WHERE SerioveCislo = ' . $params->addParameter((int)$row[2]),
                    $params);

                $ucet = $cursor->fetch();
                $cursor->close();
            } else {
                $ucet = null;
            }

            Model::addData('cisloUctu', $ucet ? $ucet[1] . '/' . $ucet[0] : '');
        }

    }


    /**
     * @return string
     * @throws Exception
     */
    function mzdoveUdaje()
    {
        // vyber osoby (provede i load)
        Model::addData('filterForm', $this ->getFilterForm('mzdoveudaje'));

        // nacte data
        $this ->nactiDovolenou();
        $this ->nactiVyplaceniMzdy();

        // zakladni udaje
        Model::addViewObject( new BaseViewObject('mzdy/mzdoveudaje_view.php') );

        // srazky ze mzdy
        $b = new SrazkyBrowser($this ->database);
        $b ->load();

        Model::addViewObject($b);

        return('view/appview.php');
    }


//    function mzdovePolozky()
//    {
//        require_once('mzdy/b_mzdovepolozky.php');
//
//        $b = new MzdovePolozkyBrowser($this ->database);
//        $b ->load();
//
//        return $b;
//    }

    /**
     * @param string $action
     * @return InternalRedirect|OutputObject|Redirect|ViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('mzdy.ini');

        switch ($action) {
            case 'mzdoveudaje' :
                return $this->mzdoveUdaje();
//
//            case 'mzdovepolozky' :
//                return $this ->mzdovePolozky();
        }

        throw new Exception(StringUtils::fromUTF8("Chyba v metode MzdyController::handleAction - neznama akce ($action)!"));
    }
}

