<?php
/**
 * UTF8
 * roman.Szablatura 22.11.2016
 */

namespace Ron\RonPortal\Napady;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\RonPortal\ComboBoxOsoba;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

require_once('ron/guiutils.php');
require_once('ron/controls.php');
require_once('ron/httputils.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/ronportalfilter.php');
require_once('adsapp/prava.php');


/**
 * Class NapadyFilterForm
 * @package Ron\RonPortal\Napady
 */
class NapadyFilterForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby */
    var $prava;

    /** @var  PravaNaOsoby */
    var $prava_admin;


    /**
     * NapadyFilterForm constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'napady');

        $this->prava = Prava::vratPrava('napady');
        $this->prava_admin = Prava::vratPrava('napady_admin');
        Prava::nastavObdobiProPrava(time(),time());

        $this->skrytTlacitkoNastavit = true;
        $this->f_formaction = '?menuid=napady';
        $this->f_classname = 'filtercontainer leftcaptions';
    }

    /**
     * @throws DBException
     */
    function renderStavNapadu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenStav', ronText('stav_napadu'). '&nbsp;&nbsp;', $_SESSION['jenStav'],
            'onClick="submit()"');
        echo '</span>';

        $sql = "SELECT DISTINCT Kod, Popis FROM StavNapadu";

        $combo = new ComboBox('stav');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenStav.checked) submit();"';
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['stav']);
        $combo->render();
        echo '</div>';
    }

    function renderAutorNapadu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenAutor', ronText('autor'). '&nbsp;&nbsp;', $_SESSION['jenAutor'],
            'onClick="submit()"');
        echo '</span>';

//        $sql =
//            "SELECT DISTINCT PracovniPomer.ID, Coalesce (Autor.Prijmeni, '') + ' ' + Coalesce (Autor.Jmeno, '') + Coalesce (', ' +  Autor.Titul,'') AutorNapadu " .
//            "FROM Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC) LEFT JOIN PracovniPomer ON (Autor.RC = PracovniPomer.RC)".
//            "ORDER BY AutorNapadu";

        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new \RonPortal\Controls\ComboBoxOsoba('autor');
        $combo->filtr = ' EXISTS(SELECT Napad.Autor FROM Napad WHERE Napad.Autor = Osoba.RC)';
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenAutor.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->prava,
            false,
            $_SESSION['autor'],
            false);
//        $combo->showCode = 0;
//        $combo->loadData($this->database, $sql, $_SESSION['autor']);
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderOblastNapadu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenOblast', ronText('oblast_napadu'). '&nbsp;&nbsp;', $_SESSION['jenOblast'],
            'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT OblastNapadu.Kod, OblastNapadu.Popis " .
            "FROM Napad LEFT JOIN OblastNapadu ON (Napad.Oblast = OblastNapadu.Kod)";

        $combo = new ComboBox('oblast');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenOblast.checked) submit();"';
        $combo->showCode = 1;
        $combo->dataTypeComparison = false;
        $combo->loadData($this->database, $sql, $_SESSION['oblast']);
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderFilterPilot()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenPilot', ronText('pilot'). '&nbsp;&nbsp;', $_SESSION['jenPilot'],
            'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT PracovniPomer.ID, Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot" .
            " FROM Napad LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC) LEFT JOIN PracovniPomer ON (Pilot.RC = PracovniPomer.RC)" .
            " WHERE Napad.Pilot IS NOT NULL".
            " ORDER BY Pilot";

        $combo = new ComboBox('pilot');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenPilot.checked) submit();"';
        $combo->showCode = 0;
        $combo->emptyRow = ronText('pilot_neni_urcen');
        $combo->dataTypeComparison = false;
        $combo->loadData($this->database, $sql, $_SESSION['pilot']);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws DBException
     */
    function renderFilterPriorita()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenPriorita', ronText('priorita'). '&nbsp;&nbsp;', $_SESSION['jenPriorita'],
            'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT DISTINCT Napad.Priorita, Napad.Priorita " .
            "FROM Napad " .
            "WHERE Napad.Priorita IS NOT NULL";

        $combo = new ComboBox('priorita');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenPriorita.checked) submit();"';
        $combo->showCode = 0;
        $combo->dataTypeComparison = false;
        $combo->emptyRow = ronText('priorita_neni_urcena');
        $combo->loadData($this->database, $sql, $_SESSION['priorita']);
        $combo->render();
        echo '</div>';
    }

    function renderZapisNovehoNapadu()
    {
        echo '<DIV class="filterrow margin-top-2">';
        echo    '<SPAN class="filtercaption">';
        echo        '<a href="?menuid=napady&formaction=new">';
        echo            ronText('zapis_noveho_napadu');
        echo        '</a>';
        echo    '</SPAN>';
        echo "</DIV>";
    }

    function loadStavNapadu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenStav'] = isset($_POST['jenStav']);
            $_SESSION['stav'] = $_POST['stav'];
        }
    }

    function loadAutorNapadu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenAutor'] = isset($_POST['jenAutor']);
            $_SESSION['autor'] = $_POST['autor'];
        }
    }

    function loadOblastNapadu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenOblast'] = isset($_POST['jenOblast']);
            $_SESSION['oblast'] = $_POST['oblast'];
        }
    }

    function loadFilterPilot()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPilot'] = isset($_POST['jenPilot']);
            $_SESSION['pilot'] = $_POST['pilot'];
        }
    }

    function loadFilterPriorita()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPriorita'] = isset($_POST['jenPriorita']);
            $_SESSION['priorita'] = $_POST['priorita'];
        }
    }

    function loadData()
    {
        $this->loadStavNapadu();
        $this->loadAutorNapadu();
        $this->loadOblastNapadu();
        $this->loadFilterPilot();
        $this->loadFilterPriorita();
        parent::loadData();
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        $this->renderStavNapadu();
        $this->renderAutorNapadu();
        $this->renderOblastNapadu();
        $this->renderFilterPilot();
        $this->renderFilterPriorita();
        $this->renderZapisNovehoNapadu();
    }
}

class PrirazeniPilotaKNapadu extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava_admin;

    /**
     * PrirazeniPilotaKNapadu constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $prava_admin
     */
    function __construct($database, $prava_admin)
    {
        parent::__construct($database);
        $this->header = ronText('pilot');
        $this->ordering = 1;
        $this->prava_admin = $prava_admin;
        $this->sortSQL = "Pilot";
        $this->sortSQLDesc = "Pilot DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if (trim($rowData[8]) != '') {
            $result .= '&nbsp;' . StringUtils::htmlTextConvert($rowData[8]) . '&nbsp;';
        } else {
            if ($this->prava_admin->jsouNejakePrava()) {
                $result .= '&nbsp;' . '<a id="odkaz_pilot" href="?menuid=napady&formaction=edit&record=' . $rowData[0] . '">' . ronText('prirazeni_pilota') . '</a>';
            }
        }
        return $result;
    }
}

/**
 * Class BarvaNapadu
 * @package Ron\RonPortal\Napady
 */
class BarvaNapadu extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $colorHtml = isset($rowData[15])
            ? 'style="color: ' . GuiUtils::getHtmlColor($rowData[15]) . '"'
            : '';

        return
            "<i class='inline-glyph fa glyph-napad glyph-napad-{$rowData[11]}' $colorHtml></i>";
    }
}

///**
// * Class OblastNapadu
// * @package Ron\RonPortal\Napady
// */
//class OblastNapadu extends BrowserField
//{
//    function __construct($database)
//    {
//        parent::__construct($database);
//        $this->header = ronText('oblast_napadu');
//        $this->ordering = 1;
//        $this->sortSQL = "OblastNapadu.Popis";
//        $this->sortSQLDesc = "OblastNapadu.Popis DESC";
//    }
//
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result= '';
//        $result .= StringUtils::htmlTextConvert($rowData[17]) . ($rowData[5] ? ' (' . StringUtils::htmlTextConvert($rowData[5]) . ')' : '');
//        return $result;
//    }
//}

/**
 * Class PocetDokumentuNapaduField
 * @package Ron\RonPortal\Napady
 */
class PocetDokumentuNapaduField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[16]) {
            return '<i class="fa inline-glyph glyph-attachment" title="' . ronText('Pocet_priloh') . ': ' . $rowData[16] . '"></i>';
        }
        return '';
    }
}

//class DatumPosledniAktualizaceField extends BrowserField
//{
//    function __construct($database)
//    {
//        parent::__construct($database);
//        $this->header = ronText('datum_editace');
//        $this->align = 'center';
//        $this->ordering = 1;
//        $this->sortSQL = "Napad.Update_Timestamp";
//        $this->sortSQLDesc = "Napad.Update_Timestamp DESC";
//    }
//
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result = '';
//        if ($rowData[19]) {
//            $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[19])) . ' (' . StringUtils::htmlTextConvert($rowData[18]) . ') ' . "</div>" ;
//        }
//        return $result;
//    }
//}



/**
 * Class NapadyBrowser
 * @package Ron\RonPortal\Napady
 */
class NapadyBrowser extends RonPortalBrowser
{
    /** @var  PravaNaOsoby */
    var $prava;

    /** @var  PravaNaOsoby */
    var $prava_admin;


    /**
     * NapadyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct( $database)
    {
        parent::__construct($database, 'napady');

        $this->prava = Prava::vratPrava('napady');
        $this->prava_admin = Prava::vratPrava('napady_admin');

        Prava::nastavObdobiProPrava(time(),time());

        $this->title = ronText('evidence_napadu');
        $this->code = 'browsernapady';
        $this->tableId = 'browsernapady';
        $this->browserUrl = '?menuid=napady';

        $this->sqlSelectPart =
            "Napad.ID, Napad.Datum, StavNapadu.Popis, " .  // 0 - 2
            "Coalesce (Autor.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Autor.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Autor.Titul,'') Autor," .
            ' Autor.RC, Oblast, Nazev, Rozpocet, ' .       // 4 - 7
            " Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot," .
            ' Pilot.RC, Priorita, StavNapadu.Kod,' .       // 9 - 11
            ' {fn_CASTWIDEMEMO(Napad.PodrobnyPopis)}, OdmenaCastka, OdmenaVyplacenaVObdobi, StavNapadu.Barva, ' .   // 12 - 15
            ' (SELECT COUNT(*) FROM DokumentNapadu WHERE DokumentNapadu.IDNapadu = Napad.ID),' .    // 16
            ' OblastNapadu.Popis, Napad.Update_UserName, Napad.Update_Timestamp';                   // 17 - 19

        $this->sqlFromPart =
            'Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC)' .
            ' LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC)' .
            ' LEFT JOIN StavNapadu ON (Napad.Stav = StavNapadu.Kod)' .
            ' LEFT JOIN OblastNapadu ON (Napad.Oblast = OblastNapadu.Kod)';

        $this->primaryField = 0;

        $this->filterForm = new NapadyFilterForm($this->database);
        $this->filterForm->loadData();

        $this->addField(new BarvaNapadu($this->database))
            ->setCode('col-icon');

        $this->addField()
            ->setSqlFieldName('Napad.Datum', 1) ->setHeaderText('datum_vlozeni')
            ->setCode('col-datum')
            ->setFormat('finedate') ->setOrdering(1) ->setAlign('center');
        // Seradit dle data
        $this->defaultSort = count($this->fields);

        $this->addField()
            ->setSqlFieldName('StavNapadu.Popis', 2) ->setHeaderText('stav_napadu')
            ->setCode('col-stav')
            ->setOrdering(1) ->setLink('?menuid=napady&action=zaznam');

        $this->addField()
            ->setSqlFieldName('Autor', 3) ->setHeaderText('autor') ->setOrdering(1)
            ->setCode('col-autor');

        // Oblast napadu s kodem
        $this->addField()
            ->setSqlFieldName('OblastNapadu.Popis') ->setSqlFieldIndex(17) ->setSqlFieldIndex2(5)
            ->setCode('col-oblast')
            ->setOrdering(1) ->setHeaderText('oblast_napadu');

        $this->addField()
            ->setSqlFieldName('Nazev', 6) ->setHeaderText('nazev_napadu')
            ->setCode('col-nazev')
            ->setOrdering(1) ->setLink('?menuid=napady&action=zaznam');

        // Pilot
        $this->addField(new PrirazeniPilotaKNapadu($this->database, $this->prava_admin))
            ->setCode('col-pilot');

        // Priorita
        $this->addField()
            ->setSqlFieldName('Priorita', 10) ->setHeaderText('priorita') ->setOrdering(1)
            ->setCode('col-priorita');

        $this->addField()
            ->setSqlFieldName('Napad.Rozpocet', 7) ->setHeaderText('rozpocet')
            ->setCode('col-rozpocet')
            ->setFormat('number_2') ->setOrdering(1) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaCastka', 13) ->setHeaderText('odmena')
            ->setCode('col-odmena')
            ->setFormat('number_2') ->setOrdering(1) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaVyplacenaVObdobi', 14) ->setHeaderText('obdobi_vyplaceni_odmeny')
            ->setCode('col-obdobiodmeny')
            ->setFormat('finedate') ->setOrdering(1) ->setAlign('center');

        $this->addField(new PocetDokumentuNapaduField())
            ->setCode('col-dokumenty');

        $this->addField() //new DatumPosledniAktualizaceField($this->database));
            ->setSqlFieldName('Napad.Update_Timestamp', 19) ->setSqlFieldIndex2(18) ->setHeaderText('datum_editace')
            ->setCode('col-aktualizace')
            ->setFormat('date') ->setOrdering(1) ->setAlign('center');

        //Pokud vchazim do Browseru mazu seznam ID napadu v session aby se vzdy vytvoril aktualni seznam pri kazde zmene filtru
        unset($_SESSION['pocetnapadufilru']);
        unset($_SESSION['seznamnapadu']);
    }

    function setFilterForm()
    {
        // nedela nic - filter form nastaven v konstuktoru
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na autora pravo nebo je pilotem napadu
        if ($this->prava->lzeZobrazitRC($values[4]) || $this->prava_admin->lzeZobrazitRC($values[4]) || User::hasRC($values[9])) {
            //Vytvarim seznam nazvu napadu oddelenych ;
            $_SESSION['seznamnapadu'] .=  $values[6] . ';;';

            if ($_SESSION['pocetnapadufilru'] != ''){
                $_SESSION['pocetnapadufilru'] .= ',';
            }
            $_SESSION['pocetnapadufilru'] .= $values[0];
            return true;
        }
        return false;
    }


    /**
     * @return SqlStatement|string
     * @throws DBException
     */
    public function getSQLFilter()
    {
        $result = '';

        require_once('adsapp/ppomer.php');
        $pracovniPomerAutora = new PracovniPomer();
        $pracovniPomerAutora->nactiPPPodleId($this->database, $_SESSION['autor']);

        $pracovniPomerPilota = new PracovniPomer();
        $pracovniPomerPilota->nactiPPPodleId($this->database, $_SESSION['pilot']);

        $params = $this->database->createParameters();

        if ($_SESSION['jenStav']){
            $result .= " StavNapadu.Kod = " . $params->addParameter($_SESSION['stav']);
        }

        if ($_SESSION['jenAutor']){
            if ($result){
                $result .= ' AND ';
            }
            $result .= " Napad.Autor = " . $params->addParameter($pracovniPomerAutora->rc);
        }

        if ($_SESSION['jenOblast']){
            if ($result){
                $result .= ' AND ';
            }
            $result .= " Napad.Oblast = " . $params->addParameter($_SESSION['oblast']);
        }

        if ($_SESSION['jenPilot']){
            if ($result){
                $result .= ' AND ';
            }
            if ($_SESSION['pilot'] != ''){
                $result .= " Napad.Pilot = " . $params->addParameter($pracovniPomerPilota->rc);
            } else {
                $result .= ' Napad.Pilot IS NULL';
            }
        }

        if ($_SESSION['jenPriorita']){
            if ($result){
                $result .= ' AND ';
            }
            if ($_SESSION['priorita'] != ''){
                $result .= " Napad.Priorita = " . $params->addParameter($_SESSION['priorita']);
            } else {
                $result .= ' Napad.Priorita IS NULL';
            }
        }
        return new SqlStatement($result, $params);
    }
}