<?php
/**
 * UTF8
 * roman.Szablatura 24.11.2016  20:02
 */

namespace Ron\RonPortal\Napady;

use Ron\Controls\ComboBox;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalForm;
use Ron\DB\Database;
use Ron\RonPortal\User;
use Ron\RonPortal\UserLoginField;
use Ron\StringUtils;
use function Ron\ronText;

require_once ('adsapp/ronportalform.php');
require_once('ron/controls.php');
require_once('adsapp/ronportalcontrols.php');


/**
 * Class VlozeniAutora
 * @package Ron\RonPortal\Napady
 */
class VlozeniAutora extends FormField
{
    /** @var  PravaNaOsoby $prava_admin */
    var $prava_admin;

    var $autor;

    /**
     * VlozeniAutora constructor.
     * @param $prava_admin
     */
    function __construct($prava_admin)
    {
        parent::__construct();
        $this->prava_admin = $prava_admin;
        $this->maxLength = 15;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $this->autor = $rowData[4];

        // Prevod RC na ID PP
        $params = $database->createParameters();
        $osobaID = DBUtils::getSqlValue($database,
            'SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = ' .  $params->addParameter(User::getRC()),
            $params);

        $params = $database->createParameters();
        $sql =
            'SELECT MAX(PracovniPomer.Id),' .
            " COALESCE(Osoba.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} Osoba.Titul,'')" .
            ' FROM PracovniPomer, Osoba' .
            ' WHERE Osoba.RC = PracovniPomer.RC' .
            ' AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($database ->dateToParam(time())) .
            ' AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params->addParameter($database ->dateToParam(time())) . ')' .
            ' AND PracovniPomer.Zamestnanec > 0' .
            ' GROUP BY Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' .
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno';

        if ($this->editable  &&  $this->prava_admin->jsouNejakePrava()) {
            $combo = new ComboBox($this ->fieldName);
            $combo->emptyRow = '';
            $combo->className = 'form-input ronselect';
            $combo->htmlParams = ['data-columns' => 'text'];
            $combo->loadData($database, $sql, $osobaID, $params);
            $combo->render();
        } else  if ($this->editable) {
            echo StringUtils::htmlTextConvert(User::getUserName());
            echo '<input type="hidden" name="Autor" value="' . $osobaID . '">';
        } else {
            echo StringUtils::htmlTextConvert($rowData[17]);
            echo '<input type="hidden" name="Autor" value="' . $osobaID . '">';
        }
    }

    /**
     * @param Database $database
     * @param string $tableName
     * @param int $insert
     * @param bool $utf8
     * @return DBRecordField
     */
    function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        if ($this->editable) {
            // Prevod ID PP na RC
            $params = $database->createParameters();
            $osobaRC = DBUtils::getSqlValue($database, "SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($_POST['Autor']), $params);
            $field->setValue($osobaRC);
        } else {
            $field->setValue($this->autor);
        }
        return $field;
    }
}

/**
 * Class VlozeniPilota
 * @package Ron\RonPortal\Napady
 *
 * Jine chovani nez autor
 */
class PilotFormField extends FormField
{
    /** @var  PravaNaOsoby $prava_admin */
    var $prava_admin;

    var $pilot;

    function __construct($fieldname, $edit, $prava_admin)
    {
        parent::__construct();
        $this->prava_admin = $prava_admin;
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->editable = $edit;
        $this->maxLength = 15;
        $this->caption = ronText('pilot');
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $this->pilot = $rowData[8];

        // Prevod RC na ID PP
        if (isset($this ->pilot)) {
            $params1 = $database->createParameters();
            $osobaID = DBUtils::getSqlValue($database,
                "SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC  = " . $params1->addParameter($this->pilot),
                $params1);
        } else {
            $osobaID = null;
        }

        $params = $database->createParameters();
        $sql =
            'SELECT (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC' .
            ' AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($database ->dateToParam(date(time()))) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($database ->dateToParam(date(time()))) . ' OR PracovniPomer.DatumUkonceni IS NULL)), ' .
            "   COALESCE(Osoba.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} Osoba.Titul,'') " .
            ' FROM Osoba' .
            ' WHERE EXISTS(SELECT P.RC FROM PracovniPomer P WHERE (P.RC = Osoba.RC) AND (P.DatumNastupu <= ' . $params->addParameter($database ->dateToParam(date(time()))) . ' ) ' .
            ' AND ((P.DatumUkonceni IS NULL) OR (P.DatumUkonceni >= ' . $params->addParameter($database ->dateToParam(date(time()))) . ')) AND (P.Zamestnanec > 0))' .
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno';

        if ($this->editable && $this->prava_admin->jsouNejakePrava()) {
            $combo = new ComboBox('Pilot');
            $combo->emptyRow = '';
            $combo->className = 'form-input ronselect';
            $combo->htmlParams = ['data-columns' => 'text'];
            $combo->loadData($database, $sql, $osobaID, $params);
            $combo->render();
        } else {
            echo StringUtils::htmlTextConvert($rowData[18]);
            echo '<input type="hidden" name="Pilot" value="' . $osobaID . '">';
        }
    }

    /**
     * @param Database $database
     * @param $tableName
     * @param $insert
     * @return DBRecordField
     * @throws DBException
     */
    function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
        if ($this->editable) {
            //Prevod ID PP na RC
            if (isset($_POST['Pilot']) && $_POST['Pilot'] != '') {
                $params = $database->createParameters();
                $osobaRC = DBUtils::getSqlValue($database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($_POST['Pilot']), $params);
                $field->setValue($osobaRC);
            }
        }

        return $field;
    }
}


/**
 * Class VolbaPriority
 * @package Ron\RonPortal\Napady
 */
class VolbaPriority extends FormField
{
    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            $sql =
                'SELECT DISTINCT Napad.Priorita, Napad.Priorita' .
                ' FROM Napad' .
                ' WHERE Napad.Priorita IS NOT NULL' .
                ' ORDER BY Napad.Priorita';

            $combo = new ComboBox($this ->fieldName);
            $combo->emptyRow = ronText('priorita_neni_urcena');
            $combo->alwaysShowSelected = true;
            $combo->loadData($database, $sql, $rowData[9]);
            $combo->render();

            echo '<span class="js toggleClass" data-remove-class="d-none" data-selector=".row-nova-priorita">' . ronText('priorita_nova') . '</span>';
        } else {
            echo StringUtils::htmlTextConvert($value);
        }
    }


    function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        $field = parent::createDBRecordField($database, $tableName, $insert, $utf8);
        if ($_POST['priorita_nova'] != '') {
            $field->setValue($_POST['priorita_nova']);
        }
        return $field;
    }
}

///**
// * Class UzivatelJmenoField
// * @package Ron\RonPortal\Napady
// */
//class UzivatelJmenoField extends FormField
//{
//
//    /**
//     * UzivatelJmenoField constructor.
//     * @param $fieldname
//     * @param $edit
//     */
//    function __construct($fieldname, $edit)
//    {
//        parent::__construct();
//        $this->fieldName = $fieldname;
//        $this->sqlFieldName = $fieldname;
//        $this->maxLength = 25;
//        $this->editable = $edit;
//    }
//
//    function createDBRecordField($database, $tableName, $insert)
//    {
//        $field = new DBRecordField($this->sqlFieldName, '', !$this->editable, $this->maxLength);
//        $field->setValue(User::getUserName());
//        return $field;
//    }
//
//    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
//    }
//}


/**
 * Class VytvoreniNovehoNapaduFrom
 * @package Ron\RonPortal\Napady
 */
class NapadForm extends RonPortalForm
{
    /** @var  PravaNaOsoby $prava */
    var $prava;

    /** @var  PravaNaOsoby $prava_admin */
    var $prava_admin;

    //Pokud pilot zadal vyjadreni neumoznit (mimo admina) autorovi editaci napadu - dokud bude nejake vyjadreni pilota
    var $autorMuzeEditovat;

    //Pokud je zadan pilot umoznit editaci priority a dalsich poli, ktera jsou v kompetenci pilota
    var $pilotJeUrcen;

    var $id;
    var $pilot;

    /** @var  Napad $napad */
    var $napad;


    /**
     * NapadForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param PravaNaOsoby $prava
     * @param PravaNaOsoby $prava_admin
     * @param $autorMuzeEditovat
     * @param $pilotJeUrcen
     * @param $napad
     */
    function __construct($database, $formAction, $prava, $prava_admin, $autorMuzeEditovat, $pilotJeUrcen, $napad)
    {
        parent::__construct($database, $formAction);

        $this->title = ronText($this->isInsert() ? 'zapis_noveho_napadu' : 'editace_napadu');
        $this->htmlAction = '?menuid=napady';

        $this->code = 'formnapad';

        $this->autorMuzeEditovat = /*isset($_POST['autorMuzeEditovat']) ? $_POST['autorMuzeEditovat']:*/ $autorMuzeEditovat;
        $this->pilotJeUrcen = /*isset($_POST['pilotJeUrcen']) ? $_POST['pilotJeUrcen']:*/ $pilotJeUrcen;
        $this->prava = $prava;
        $this->prava_admin = $prava_admin;

        $this->stornoAction = ($formAction == 'edit')
            ? '?menuid=napady&action=zaznam&record=' . $_GET['record']
            : '?menuid=napady';

        $this->sqlSelectPart =
            'ID, Datum, Stav, Oblast, Autor, Nazev, {fn_CASTWIDEMEMO(Napad.PodrobnyPopis)}, Rozpocet,' .            //0 - 7
            ' Pilot, Priorita, {fn_CASTWIDEMEMO(Napad.VyjadreniPilota)}, OdmenaCastka, OdmenaVyplacenaVObdobi,' .   //8 - 12
            ' Insert_UserName, Insert_Timestamp, Update_UserName, Update_Timestamp, ' . // 13 - 16
            " (SELECT COALESCE(Osoba.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} Osoba.Titul,'')" .
            "  FROM Osoba WHERE Osoba.RC = Autor), " .      // 17
            " (SELECT COALESCE(Pilot.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Pilot.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} Pilot.Titul,'')" .
            "  FROM Osoba Pilot WHERE Pilot.RC = Pilot)" ;  // 18

        $this->sqlTableName = 'Napad';


        $this->addField(FORM_FIELD_TYPE_ID)
            ->setFieldName('ID') ->setPrimary(true) ->setSqlFieldIndex(0)
            ->setCode('field-id');

        if ($this->prava_admin->jsouNejakePrava()) {
            // Stav napadu - implicitne jako 0 - novy, jinak Admin nebo Pilot mohou stavy menit
            $this->addField(FORM_FIELD_TYPE_SELECT)
                ->setFieldName('Stav') ->setSqlFieldIndex(2) ->setDefaultValue(0) ->setCaption('stav_napadu') ->setRequired(1)
                ->setCode('field-stav-admin')
                ->setSelectSql('SELECT Kod, Popis FROM StavNapadu ORDER BY Kod') ->setSelectDataTypeTest(false) ->setEditable(1);
        } else {
            // Skryte pole: duvod novy napad - hodnota napevno nastavena na novy napad
            $this->addField(FORM_FIELD_TYPE_HIDDEN)
                ->setFieldName('Stav') ->setDefaultValue(0) ->setDataType('int')
                ->setCode('field-stav');
        }

        //$this->addField(FORM_FIELD_TYPE_SECTION) ->setCaption('autor') ->setRowClass('form-section');

        // Autor
        // Moznost upravovat autora pouze pri vytvoreni s pravy napady_admin, jinak je autorem automaticky prihlasena osoba
        $this->addField(new VlozeniAutora($this->prava_admin))
            ->setFieldName('Autor')
            ->setEditable($this->formAction == 'new' || $this->formAction == 'insert' ? 1 : 0)
            ->setRequired(true)
            ->setCaption('autor')
            ->setCode('field-autor');

        // Datum
        $this->addField(FORM_FIELD_TYPE_ACTUAL_DATE)
            ->setFieldName('Datum') ->setEditable(false)
            ->setCode('field-datum');

        // Sekce detaily
        $this->addField(FORM_FIELD_TYPE_SECTION)
            ->setCaption('detaily') ->setRowClass('form-section margin-top-2 margin-bottom-2')
            ->setCode('sekce-detaily');

        // Nazev
        $this->addField(FORM_FIELD_EDIT)
            ->setSqlFieldIndex(5) ->setFieldName('Nazev') ->setRequired(true)
            ->setCode('field-nazev')
            ->setCaption('nazev_napadu') ->setEditable($this->autorMuzeEditovat);

        // Oblast napadu
        $this->addField(FORM_FIELD_TYPE_SELECT)
            ->setFieldName('Oblast') ->setSqlFieldIndex(3) ->setCaption('oblast_napadu')
            ->setCode('field-oblast')
            ->setSelectSql('SELECT OblastNapadu.Kod, OblastNapadu.Popis FROM OblastNapadu ORDER BY OblastNapadu.Popis')
            ->setShowEmptyValue(true)
            ->setEditable($this->autorMuzeEditovat);

        // Podrobny popis
        $this->addField(FORM_FIELD_TYPE_MEMO)
            ->setFieldName('PodrobnyPopis') ->setSqlFieldIndex(6) ->setCaption('podrobny_popis')
            ->setCode('field-podrobnypopis')
            ->setEditable($this->autorMuzeEditovat);

        // Rozpocet
        $this->addField(FORM_FIELD_TYPE_FLOAT)
            ->setFieldName('Rozpocet') ->setSqlFieldIndex(7) ->setCaption('rozpocet')
            ->setCode('field-rozpocet')
            ->setEditable($this->autorMuzeEditovat);

        //Pokud se nejedna o admina nebo neni pilot urcen pak nezobrazovat
        if ($this->pilotJeUrcen || $this->prava_admin->jsouNejakePrava()) {

            //Urceny pilot
            $this->addField(new PilotFormField('Pilot', $this->prava_admin->jsouNejakePrava(), $this->prava_admin));

            //Priorita - z combo pripadne nova
            $this->addField(new VolbaPriority())
                ->setFieldName('Priorita') ->setEditable(true) ->setDataType('string')
                ->setCaption('priorita');

            $this->addField()
                ->setFieldName('*priorita_nova') //->setCaption(ronText('priorita_nova'))
                ->setRowClass('form-row row-nova-priorita d-none');


            //Vyjadreni pilota
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_NAME => 'VyjadreniPilota', FORM_FIELD_CAPTION_TEXT => ronText('vyjadreni_pilota'),
                FORM_FIELD_SQL_INDEX => 10, FORM_FIELD_SQL_NAME => 'VyjadreniPilota', FORM_FIELD_EDIT => true]);

            //Odmena
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_INT, FORM_FIELD_DATA_TYPE => 'int', FORM_FIELD_NAME => 'OdmenaCastka', FORM_FIELD_SQL_NAME => 'OdmenaCastka',
                FORM_FIELD_SQL_INDEX => 11, FORM_FIELD_CAPTION => ronText('odmena'), FORM_FIELD_EDIT => true]);

            //Termin vyplaceni odmeny
            $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_NAME => 'OdmenaVyplacenaVObdobi', FORM_FIELD_CAPTION_TEXT => ronText('obdobi_vyplaceni_odmeny'),
                FORM_FIELD_SQL_NAME => 'OdmenaVyplacenaVObdobi', FORM_FIELD_SQL_INDEX => 12, FORM_FIELD_EDIT => true]);
        } else {
            //Vyjadreni pilota - text
            $this->addFields([
                [FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => ronText('pilot'), FORM_FIELD_TEXT => $napad->pilotPrijmeniJmeno],
                [FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => ronText('vyjadreni_pilota'), FORM_FIELD_TEXT => $napad->vyjadreniPilota],
                [FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => ronText('odmena'), FORM_FIELD_TEXT => $napad->odmenaCastka],
                [FORM_FIELD_TYPE => FORM_FIELD_TYPE_TEXT, FORM_FIELD_CAPTION_TEXT => ronText('obdobi_vyplaceni_odmeny'), FORM_FIELD_TEXT => StringUtils::format('date',$napad->odmenaVyplacenaVObdobi)]
            ]);
        }

        $this->addField(new UserLoginField('Update_UserName', true))
            ->setCode('field-update_username');
        // Datum editace napadu
        $this->addField(FORM_FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp') ->setEditable(true)
            ->setCode('field-update_timestamp');

        $this->addField(new UserLoginField('Insert_UserName', false))
            ->setCode('field-insert_username');
        // Datum vlozeni napadu
        $this->addField(FORM_FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp') ->setEditable(false)
            ->setCode('field-insert_timestamp');
    }

//    function showCommonFields()
//    {
//        parent::showCommonFields();
//        /* echo '<input type="hidden" value="' . $this->pilotJeUrcen . '" name="pilotJeUrcen" />';
//         echo '<input type="hidden" value="' . $this->autorMuzeEditovat . '" name="autorMuzeEditovat" />';*/
//    }

    function afterInsertRecord($rec)
    {
        $this->id = $rec['ID'];
        return parent::afterInsertRecord($rec);
    }

    function afterUpdateRecord($rec)
    {
        $this->id = $rec['ID'];
        return parent::afterUpdateRecord($rec);
    }
}
