<?php
/** roman.Szablatura 22.11.2016 */

namespace Ron\RonPortal\Napady;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\MessageManager;
use function Ron\ronText;


/**
 * Class Napad
 * @package Ron\RonPortal\Napady
 */
class Napad
{
    var $id;
    var $datum;
    var $stavNapaduKod;
    var $stavNapaduPopis;
    var $autorRC;
    var $oblast;
    var $nazev;
    var $rozpocet;
    var $pilotRC;
    var $priorita;
    var $vyjadreniPilota;
    var $podrobnyPopis;
    var $odmenaCastka;
    var $odmenaVyplacenaVObdobi;
    var $barvaNapadu;
    var $autorPrijmeniJmeno;
    var $pilotPrijmeniJmeno;
    var $insert_UserName;
    var $insert_Timestamp;
    var $update_UserName;
    var $update_Timestamp;
    var $oblastNapaduPopis;
    var $pocetDokumentu;


    /**
     * @param Database $database
     * @return Napad
     * @param $id
     * @throws Exception
     */
    public static function nactiNapad($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT ID, Napad.Datum, StavNapadu.Kod, StavNapadu.Popis, " .          //0 - 3
            " Autor.RC, Oblast, Nazev, Rozpocet, Pilot.RC, Priorita, " . $database->castWideMemo('VyjadreniPilota') .', ' .            //4 - 10
            $database ->castWideMemo('PodrobnyPopis ') . ", OdmenaCastka, OdmenaVyplacenaVObdobi, StavNapadu.Barva, " .             //11 - 14
            "Coalesce (Autor.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Autor.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Autor.Titul,'') Autor, " .       //15
            "Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot, " .     //16
            "Napad.Insert_UserName, Napad.Insert_Timestamp, Napad.Update_UserName, Napad.Update_Timestamp, " .                //17 - 20
            "OblastNapadu.Popis, (SELECT COUNT(*) FROM DokumentNapadu WHERE DokumentNapadu.IDNapadu = Napad.ID) " .         //21 - 22

            "FROM Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC) ".
            "LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC) " .
            "LEFT JOIN StavNapadu ON (Napad.Stav = StavNapadu.Kod) ".
            "LEFT JOIN OblastNapadu ON  (OblastNapadu.Kod = Napad.Oblast)" .

            "WHERE ID = " . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()){
            throw new Exception(ronText('napad_neexistuje'));
        }

        $napad = new Napad();
        $napad->id = $id;
        $napad->nactiNapadZRadku($database, $row);
        return $napad;
    }


    /**
     * @param Database $database
     * @param $row
     */
    public function nactiNapadZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->datum = $database->stringToTimestamp($row[1]);
        $this->stavNapaduKod = $row[2];
        $this->stavNapaduPopis = $row[3];
        $this->autorRC = $row[4];
        $this->oblast = $row[5];
        $this->nazev = $row[6];
        $this->rozpocet = $row[7];
        $this->pilotRC = $row[8];
        $this->priorita = $row[9];
        $this->vyjadreniPilota = $row[10];
        $this->podrobnyPopis = $row[11];
        $this->odmenaCastka = $row[12];
        $this->odmenaVyplacenaVObdobi = $database->stringToTimestamp($row[13]);
        $this->barvaNapadu = GuiUtils::getHtmlColor($row[14]);
        $this->autorPrijmeniJmeno = $row[15];
        $this->pilotPrijmeniJmeno = $row[16];
        $this->insert_UserName = $row[17];
        $this->insert_Timestamp = $database->stringToTimestamp($row[18]);
        $this->update_UserName = $row[19];
        $this->update_Timestamp = $database->stringToTimestamp($row[20]);
        $this->oblastNapaduPopis = $row[21];
        $this->pocetDokumentu = $row[22];
    }
}

/**
 * Class NapadyDokumenty
 * @package Ron\RonPortal\Napady
 */
class NapadyDokumenty
{
    public $idDokumentu;
    public $popisDokumentu;
    public $typDokumentu;
    public $datumVlozeniDokumentu;
    public $nazevDokumentu;
    public $poznamkaDokumentu;

    /**
     * @param Database $database
     * @param $id
     * @return NapadyDokumenty[]
     * @throws DBException
     */
    public static function nactiDokumentyNapadu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT DokumentNapadu.id, popis, typ, datum, jmenoSouboru, ' . $database->castWideMemo('poznamka') .       //0 - 5
            'FROM DokumentNapadu LEFT JOIN Dokument ON (Dokument.id = DokumentNapadu.ID) ' .
            'WHERE IDNapadu = ' . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql,$params);

        $seznamDokumentu = [];

        while ($row = $cursor->fetch())
        {
            $dokumentNapadu = new NapadyDokumenty();
            $dokumentNapadu->nactiDokumentNapaduZRadku($database,$row);
            $seznamDokumentu[] = $dokumentNapadu;
        }
        return $seznamDokumentu;
    }

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiDokumentNapaduZRadku($database,$row)
    {
        $this->idDokumentu = $row[0];
        $this->popisDokumentu = $row[1];
        $this->typDokumentu = $row[2];
        $this->datumVlozeniDokumentu = $database->stringToTimestamp($row[3]);
        $this->nazevDokumentu = $row[4];
        $this->poznamkaDokumentu = $row[5];
    }
}