<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 27.3.2017
 */

namespace Ron\RonPortal\Navstevy;

use Exception;
use Ron\DB\DBException;
use Ron\BrowserField;
use Ron\Controls\Option;
use Ron\Controls\DateInput;
use Ron\Controls\ComboBox;
use Ron\DB\SqlStatement;
use Ron\RonPortal\Prava;
use Ron\MessageManager;
use Ron\DB\Database;
use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\RonPortal\RonPortalBrowser;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalFilterForm;
use RonPortal\RonPortalSession;

require_once('ron/guiutils.php');
require_once('ron/controls.php');
require_once('ron/httputils.php');
require_once('adsapp/prava.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/ronportalfilter.php');

/**
 * Class NavstevyFilter
 * @package Ron\RonPortal\Navstevy
 */
class NavstevyFilter extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby $prava */
    public $prava;
    public $ciziOsoba;
    public $zamestnanec;
    public $timestamp;
    public $omezeniNavstevHodnota;
    public $database;


    /**
     * NavstevyFilter constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $action, $useGet)
    {
        parent::__construct($database, $action, $useGet);
        $this->prava = Prava::vratPrava('navstevy');
        Prava::nastavObdobiProPrava(time(), time());
        $this->database = $database;

        $this->f_formaction = '?menuid=navstevy&';
        $this->f_classname = 'filtercontainer leftcaptions';

        $this->ciziOsoba = new Option(0, ronText('cizi'));
        $this->zamestnanec = new Option(1, ronText('vlastni'));

        if ($_SESSION['vyberpocetdnu'] == '') {
            $_SESSION['vyberpocetdnu'] = 1;
        }

        if($_SESSION['omezeniNavstev'] == ""){
            $_SESSION['omezeniNavstev'] = 2;
        }

        if($_SESSION['omezeniVratnice'] == ""){
            $_SESSION['omezeniVratnice'] = 1;
        }


        //Nastaveni timestamp (pokud je datum ve spatnem formatu nebo neni v session nebo neni, nastavi se na aktu. datum)
        if (isset($_POST['datum'])){
            try {
                $this->timestamp = DateUtils::dateStringToTimestamp($_POST['datum']);
            } catch(Exception $e) {
                MessageManager::addErrorText( 'Bad_time_format' );
                $this->timestamp = time();
            }
        } else {

            if ($_SESSION['datum']) {
                $this->timestamp = $_SESSION['datum'];
            } else {
                $this->timestamp = time();
            }

        }

        //Nastaveni casu pokud jeste neni nastaveno od pulnoci do vteriny pred dvanactou
        $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
        $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $_SESSION['vyberpocetdnu'])) - 1);
    }

    /**
     * @param $pocetDni
     * @return string
     */
    private function textTitulkuZaznamyOd($pocetDni)
    {
        switch ($pocetDni){
            case 1:
                $titulek = ronText('1_den');
                break;

            case 10:
                $titulek = ronText('10_dni');
                break;

            case 20:
                $titulek = ronText('20_dni');
                break;

            case 31:
                $titulek = ronText('31_dni');
                break;

            default:
                $titulek = "";
        }
        return $titulek;
    }

    function renderOmezeniNavstev()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            // jen ocekavane
            '<label class="margin-right-2">' .
            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='0' " . ($_SESSION['omezeniNavstev'] == 0 ? 'checked="checked"' : "") . '> ' .
            ronText('jenOcekavany') .
            '</label>' .
            // pritomne
            '<label>' .
            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='1' " . ($_SESSION['omezeniNavstev'] == 1 ? 'checked="checked"' : "") . '> ' .
            ronText('pritomni') .
            '</label>' .
            '</span>';

        //Combo volby dni a datumu
        $zvolenaDoba =
            ronText('zvolena_doba') . ': ' .
            date('j.n.Y', $_SESSION['datum']) . ' - ' .
            date('j.n.Y', $_SESSION['datumdodata']);

        echo "<span title='$zvolenaDoba'>";

        echo
            '<label class="zaznamy_od">' .
            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='2' " .
            ($_SESSION['omezeniNavstev'] == 2 ? 'checked="checked"' : "") . '> ' .
            ronText('zobrazit_navstevy_za') .
            '</label> ';

        if ($_SESSION['omezeniNavstev'] == 2) {
            $combo = new ComboBox('vyberpocetdnu', [
                1 => ronText('1_den'),
                10 => ronText('10_dni'),
                20 => ronText('20_dni'),
                31 => ronText('31_dni')]
            );
            $combo->className = 'vyber_dni_combo adsfilterforminput_kod';
            $combo->htmlParams = 'data-ron-autosubmit="1"';
            $combo->selectedOption = $combo->findOption((int)$_SESSION['vyberpocetdnu'] != '' ? (int)$_SESSION['vyberpocetdnu'] : 1);
            $combo->render();

            $pouzeDnesniDen = ronText('pouze_dnesni_den');
            echo
                "<span class='margin-left-2' title='$pouzeDnesniDen'>" .
                ronText('Od') . ' ';
            DateInput::renderDateSelect('datum', date('j.n.Y', $this->timestamp), true);
            echo
                '</span>';
        } else {

            $text =
                "<input type='text' size='10' class='input_pocet_dni' disabled='disabled' value='" . $this->textTitulkuZaznamyOd($_SESSION['vyberpocetdnu']) . "'/>" .
                ' ' . ronText('od') . ' ' .
                "<input type='text' size='10' class='input_pocet_dni' disabled='disabled' value='" . date('j.n.Y', $_SESSION['datum']) . "'/>";
            echo $text;
        }
        echo '</span>';

        echo '</div>';


        echo
            '<div class="filterrow" id="filterrow_omezenivratnice">' .
            '<span class="filtercaption">' .
            // vsechny vratnice
            '<label class="margin-right-2">' .
            "<input type='radio' class='clickAutoSubmit' name='omezeniVratnice' value='0' " . ($_SESSION['omezeniVratnice'] == 0 ? 'checked="checked"' : "") . '> ' .
            ronText('vsechnyVratnice') .
            '</label>' .
            // jen aktualni vratnice
            '<label>' .
            "<input type='radio' class='clickAutoSubmit' name='omezeniVratnice' value='1' " . ($_SESSION['omezeniVratnice'] == 1 ? 'checked="checked"' : "") . '> ' .
            ronText('pouzeMojeVratnice') .
            '</label>' .
            '</span>' .
            '</div>';
    }

    function renderTypNavstevy()
    {
        echo '<div class="filterrow" id="filterrow_typnavstevy">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypNavstevy', ronText('podle_typu_navstevy'), $_SESSION['jenTypNavstevy'],
            ['class' => 'clickAutoSubmit', 'id' => 'jen_typ_navstevy_checkbox']);
        echo '</span>';

        $combo = new ComboBox('typNavstevy', [$this->ciziOsoba, $this->zamestnanec]);
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_typ_navstevy_checkbox'];
        $combo->selectedOption = $combo->findOption($_SESSION['typNavstevy'] != '' ? (int)$_SESSION['typNavstevy'] : 0);
        $combo->showCode = 0;
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderDuvodNavstevy()
    {
        echo '<div class="filterrow" id="filterrow_duvodnavstevy">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenDuvodNavstevy', ronText('podle_duvodu_navstevy'), $_SESSION['jenDuvodNavstevy'],
            ['class' => 'clickAutoSubmit', 'id' => 'jen_duvod_navstevy_checkbox']);
        echo '</span>';

        $sql = "SELECT Kod, Popis FROM DuvodNavstevy ORDER BY Popis";

        $combo = new ComboBox('duvodNavstevy');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_duvod_navstevy_checkbox'];
        $combo->loadData($this->database, $sql, $_SESSION['duvodNavstevy']);
        $combo->showCode = 0;
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderOsobaNavstevy()
    {
        echo '<div class="filterrow" id="filterrow_osobanavstevy">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenOsoba', ronText('podle_navstivene_osoby'), $_SESSION['jenOsoba'],
            ['class' => 'clickAutoSubmit', 'id' => 'jen_navstivena_osoba_checkbox']);
        echo '</span>';

        $params = null;
        if (RonPortalSession::getValue('rozdelenivratnice') != '') {
            $params = $this->database->createParameters();
            $sql =
                'SELECT DISTINCT JmenoNavstiveneOsoby, JmenoNavstiveneOsoby Jmeno FROM Navsteva' .
                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) .
                ' ORDER BY JmenoNavstiveneOsoby';
        } else {
            $sql =
                'SELECT DISTINCT JmenoNavstiveneOsoby, JmenoNavstiveneOsoby Jmeno FROM Navsteva ORDER BY jmenoNavstiveneOsoby';
        }

        $combo = new ComboBox('osoba');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_navstivena_osoba_checkbox'];
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('osoba'), $params);
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderFirma()
    {
        echo '<div class="filterrow"  id="filterrow_firmanavstevy">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenFirma', ronText('podle_firmy'), $_SESSION['jenFirma'],
            ['class' => 'clickAutoSubmit', 'id' => 'jen_firma_checkbox']);
        echo '</span>';

        $params = null;
        if (RonPortalSession::getValue('rozdelenivratnice') != '') {
            $params = $this->database->createParameters();
            $sql =
                'SELECT DISTINCT Firma FROM Navsteva' .
                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) .
                ' ORDER BY Firma';
        } else {
            $sql =
                'SELECT DISTINCT Firma FROM Navsteva ORDER BY Firma';
        }
        $combo = new ComboBox('firma');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_firma_checkbox'];
        $combo->showCode = 0;
        $combo ->captionIndex = 0;
        $combo->loadData($this->database, $sql, $_SESSION['firma'], $params);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws Exception
     */
    function renderSchvaleniNavstevy()
    {
        $navstevaNeschvalena = new Option(0, ronText('navsteva_neschvalena'));
        $navstevaSchvalena = new Option(1, ronText('navsteva_schvalena'));

        echo '<div class="filterrow" id="filterrow_schvaleninavstevy">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenSchvaleno', ronText('podle_schvaleni'), $_SESSION['jenSchvaleno'], 'id="checkbox_jen_schvaleno"', 'clickAutoSubmit');
        echo '</span>';

        $combo = new ComboBox('schvaleniNavstevy', [$navstevaNeschvalena, $navstevaSchvalena]);
        $combo->className = 'adsfilterforminput_full';
        //$combo->htmlParams = 'onChange="if (jenSchvaleno.checked) submit();"';
        $combo->htmlParams = 'data-ron-autosubmit="1" data-ron-autosubmit-check="#checkbox_jen_schvaleno"';
        $combo->selectedOption = $combo->findOption($_SESSION['schvaleniNavstevy'] != '' ? (int)$_SESSION['schvaleniNavstevy'] : 0);
        $combo->showCode = 0;
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderPrijmeni()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenPrijmeni', ronText('podle_prijmeni'). '&nbsp;&nbsp;', $_SESSION['jenPrijmeni'], 'id="checkbox_jen_prijmeni"', 'clickAutoSubmit');
        echo '</span>';

        $params = null;
        if (RonPortalSession::getValue('rozdelenivratnice') != '') {
            $params = $this->database->createParameters();
            $sql =
                'SELECT DISTINCT prijmeni, prijmeni p FROM Navsteva' .
                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) . ' ORDER BY p';
        } else {
            $sql =
                'SELECT DISTINCT prijmeni, prijmeni p FROM Navsteva ORDER BY p';
        }
        $combo = new ComboBox('prijmeni');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = 'data-ron-autosubmit="1" data-ron-autosubmit-check="#checkbox_jen_prijmeni"';
        //$combo->htmlParams = 'onChange="if (jenPrijmeni.checked) submit();"';
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('prijmeni'), $params);
        $combo->render();

        echo '</div>';
    }


    /**
     * @throws DBException
     */
    function renderMedium()
    {
        echo '<div class="filterrow">';
        $titulekFiltruZapujcenehoMedia = ronText('filtr_zapujceneho_media');
        echo "<span class='filtercaption' title='$titulekFiltruZapujcenehoMedia'>";
        GuiUtils::renderCheckbox('jenMedium', ronText('vyhledat_zapujcena_media'), $_SESSION['jenMedium'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();

        $sql =
            "SELECT DISTINCT Karta.Kod, Karta.OznaceniMedia " .
            "FROM Navsteva LEFT JOIN PrirazeniKarty ON (Navsteva.osobaRC = PrirazeniKarty.RC 
                                AND Navsteva.osobniCislo = PrirazeniKarty.osobniCislo
                                AND Navsteva.datumNastupu = PrirazeniKarty.datumNastupu) 
                           LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty) " .
            "WHERE Navsteva.prichod IS NOT NULL AND Navsteva.odchod IS NULL AND Karta.Kod IS NOT NULL ";

        if (RonPortalSession::getValue('rozdelenivratnice') != '') {
            $sql .= "AND Navsteva.Rozdeleni_Vratnice = " . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

        $sql .= " ORDER BY Karta.OznaceniMedia";

        $combo = new ComboBox('medium');
        $combo->className = 'adsfilterforminput_full';
        $combo->emptyRow = ronText("nevybrano");
        $combo->htmlParams = 'onChange="if (jenMedium.checked) submit();"';
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['medium'], $params);
        $combo->render();
        echo '</div>';
    }


    public function renderUzivatelskePolozkyVPoznamceBrowseru()
    {
        echo '<div class="filterrow">';
        echo '<label>';
        GuiUtils::renderCheckbox('uzivatelskePoznamky', ronText('navstevyUzivatelskePoznamky'), $_SESSION['uzivatelskePoznamky'],
            '', 'clickAutoSubmit');
        echo '</label>';
        echo '</div>';
    }

    function renderVozidloNavstevy()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenVozidloNavstevy', ronText('podle_typu_vozidla'), $_SESSION['jenVozidloNavstevy'],
            '', 'clickAutoSubmit');
        echo '</span>';

        $combo = new ComboBox('typVozidla', [0 => ronText("bez_vozidla"), 1 => ronText("s_vozidlem")]);
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenVozidloNavstevy.checked) submit();"';
        $combo->selectedOption = $combo->findOption($_SESSION['typVozidla'] != '' ? (int)$_SESSION['typVozidla'] : 0);
        $combo->render();

        echo '</div>';
    }

    function renderZapisNoveNavstevy()
    {
        echo '<ul class="linkslist">';

        echo "<li class='selectedlink'><a href='?menuid=navstevy&formaction=new&ocekavana=1'>" . ronText('ocekavana_navsteva') . "</a></li>";

        if (!$this ->prava ->priznak) { // zapis nove nasvtevy se nezobrazuje, pokud ma pravo jen na ocekavane navstevy
            echo "<li class='selectedlink'><a href='?menuid=navstevy&formaction=new&ocekavana=0'>" . ronText('zapis_nove_navstevy') . "</a></li>";
        }

        echo "<li class='selectedlink'><a id='najit_navstevu_pomoci_id_medium'>" . ronText('najit_navstevu_pomoci_id_medium') . "</a></li>";

        echo "</ul>";
    }

    function loadDatumFiltru()
    {
        if (isset($_POST['adsfilterform'])) {
            try{
                //V pripade ze se zada neexistujici den v mesici tak skript skoncil chybou, ze ktere se jen tak neda odejit - proto jsem toto osetril
                // a nastavuji v pripade chyby aktualni datum
                $_SESSION['datum'] = DateUtils::dateStringToTimestamp($_POST['datum']);
            } catch (Exception $ex){
                $_SESSION['datum'] = time();
            }
        }
    }

    private function loadTypNavstevy()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypNavstevy'] = isset($_POST['jenTypNavstevy']);
            $_SESSION['typNavstevy'] = $_POST['typNavstevy'];
            $this->nastaveniZobrazeniRozsahu();
        }
    }

    private function loadOsobaNavstevy()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenOsoba'] = isset($_POST['jenOsoba']);
            $_SESSION['osoba'] = $_POST['osoba'];
        }
    }

    private function loadUzivatelskePoznamky()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['uzivatelskePoznamky'] = isset($_POST['uzivatelskePoznamky']);
        }
    }

    private function loadVozidloNavstevy()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenVozidloNavstevy'] = isset($_POST['jenVozidloNavstevy']);
            $_SESSION['typVozidla'] = $_POST['typVozidla'];
        }
    }

    private function loadDuvodNavstevy()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenDuvodNavstevy'] = isset($_POST['jenDuvodNavstevy']);
            $_SESSION['duvodNavstevy'] = $_POST['duvodNavstevy'];
        }
    }

    private function loadFirma()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenFirma'] = isset($_POST['jenFirma']);
            $_SESSION['firma'] = $_POST['firma'];
        }
    }

    private function loadSchvaleniNavstevy()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenSchvaleno'] = isset($_POST['jenSchvaleno']);
            $_SESSION['schvaleniNavstevy'] = $_POST['schvaleniNavstevy'];
        }
    }

    private function loadPrijmeni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPrijmeni'] = isset($_POST['jenPrijmeni']);
            $_SESSION['prijmeni'] = $_POST['prijmeni'];
        }
    }

    private function loadMedium()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenMedium'] = isset($_POST['jenMedium']);
            $_SESSION['medium'] = $_POST['medium'];
        }
    }


    private function loadOmezeniNavstev()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['omezeniNavstev'] = $_POST['omezeniNavstev'];
            $this->nastaveniZobrazeniRozsahu();
        } else {
            $this->nastaveniZobrazeniRozsahu();
        }
    }

    private function loadOmezeniVratnice()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['omezeniVratnice'] = $_POST['omezeniVratnice'];
        }
    }

    private function loadRozsahPocetDnu()
    {
        if (isset($_POST['adsfilterform'])) {
            if ($_POST['vyberpocetdnu'] != "") {
                $_SESSION['vyberpocetdnu'] = $_POST['vyberpocetdnu'];
            }
        }
    }

    private function nastaveniZobrazeniRozsahu()
    {
        $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
        $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $_SESSION['vyberpocetdnu'])) - 1);
    }


    function loadData()
    {
        $this->loadDatumFiltru();
        $this->loadRozsahPocetDnu();
        $this->loadOmezeniNavstev();
        $this->loadTypNavstevy();
        $this->loadOsobaNavstevy();
        $this->loadPrijmeni();
        $this->loadFirma();
        $this->loadSchvaleniNavstevy();
        $this->loadMedium();
        $this->loadOmezeniVratnice();
        $this->loadDuvodNavstevy();
        $this->loadVozidloNavstevy();
        $this->loadUzivatelskePoznamky();
        parent::loadData();
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    function renderFilters()
    {
        $this->renderOmezeniNavstev();
        parent::renderFilters();
        $this->renderOsobaNavstevy();
        $this->renderPrijmeni();
        $this->renderTypNavstevy();
        $this->renderFirma();
        $this->renderSchvaleniNavstevy();
        $this->renderMedium();
        $this->renderDuvodNavstevy();
        $this->renderVozidloNavstevy();
        $this->renderUzivatelskePolozkyVPoznamceBrowseru();
        $this->renderZapisNoveNavstevy();
    }
}


/**
 * Class TypNavstevy
 * @package Ron\RonPortal\Navstevy
 */
class TypNavstevy extends BrowserField
{

    /**
     * TypNavstevy constructor.
     * @param Database $database
     * @param $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = ronText('typ_navstevy');
        $this->ordering = 1;
        $this->align = 'center';
        $this->sortSQL = 'TypNavstevy, Navsteva.OcekavanyPrichod, prichod';
        $this->sortSQLDesc = 'TypNavstevy DESC, Navsteva.OcekavanyPrichod DESC, prichod DESC';
        $this->sqlFieldIndex = 1;
        $this->sqlFieldName = 'TypNavstevy';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=navstevy&formaction=edit&record=' . (int)$rowData[0] . '" title="' . ronText($rowData[1] ? 'vlastni' : 'cizi') . '">' .
            ($rowData[1] ? '<strong>Z</strong>' : '-->') .
            '</a>';
    }
}

/**
 * Class CasNavstevy
 * @package Ron\RonPortal\Navstevy
 */
class CasNavstevy extends BrowserField
{
    private $datumRadku;

    /**
     * CasNavstevy constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->ordering = 1;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        $time = $this->database->stringToTimestamp($rowData[$this->sqlFieldIndex]);
        if (DateUtils::startOfTheDay($time) == DateUtils::today()) {
            $result =
                '<strong>' . $result . '</strong>';
        }

        return $result;
    }
}


/**
 * Class PotvrzeniOdchoduNavstevyField
 * @package Ron\RonPortal\Navstevy
 */
class PotvrzeniOdchoduNavstevyField extends BrowserField
{
    var $lzeZapsatOdchod;

    /**
     * PotvrzeniPrijetiField constructor.
     * @param Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->lzeZapsatOdchod = !Prava::vratPrava('navstevy') ->priznak;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // Jestlize je zadan prichod a odchod jeste ne
        if (!$rowData[$this ->sqlFieldIndex]  &&  $this ->lzeZapsatOdchod  &&  $rowData[14]) {
            return GuiUtils::changeLocationQuest(
                ronText('vlozit_odchod_navstevy'), ronText('opravdu_ukoncit_navstevu'),
                '?menuid=navstevy&action=potvrzeniodchodu&id=' . $rowData[0]);
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class SchvaleniNavstevyField
 * @package Ron\RonPortal\Navstevy
 */
class SchvaleniNavstevyField extends BrowserField
{

    /** @var PravaNaOsoby $pravo_schvaleni */
    private $pravo_schvaleni;

    /**
     * PotvrzeniPrijetiField constructor.
     * @param Database $database
     * @param array|null $options
     * @param PravaNaOsoby $pravo_schvaleni
     */
    public function __construct($database, $options, $pravo_schvaleni)
    {
        parent::__construct($database, $options);
        $this->header = ronText('navstevu_schvalil');
        $this->align = "center";
        $this->pravo_schvaleni = $pravo_schvaleni;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        //Jestlize neschvaleno
        if (!$rowData[47]) {
            //
            if ($this->pravo_schvaleni->jsouNejakePrava() && $rowData[27]) {  // pokud je to ocekavana navsteva
                $result = GuiUtils::changeLocationQuest(ronText('Schvalit'), ronText('opravdu_schvalujete_navstevu'),
                    ' ?menuid=navstevy&action=schvaleninavstevy&id=' . $rowData[0]);
            }
        } else {
            $result = StringUtils::htmlTextConvert($rowData[47]) . " (" .StringUtils::format('datetime', $this->database->stringToTimestamp($rowData[48])) . ")";
        }
        return $result;
    }
}


/**
 * Class NavstevuSchvalilaOsoba
 * @package Ron\RonPortal\Navstevy
 */
class NavstevuSchvalilaOsoba extends BrowserField
{

    /** @var Database $database */
    public $database;

    /** @var PravaNaOsoby $pravo_schvaleni */
    private $pravo_schvaleni;


    /**
     * NavstevuSchvalilaOsoba constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_schvaleni
     */
    public function __construct($database, $pravo_schvaleni)
    {
        parent::__construct($database);
        $this->header = ronText('navstevu_schvalil');
        $this->database = $database;
        $this->pravo_schvaleni = $pravo_schvaleni;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
         $result = "";

        //Jestlize neschvaleno
        if ($rowData[47] == "") {
            //
            $rcNavstiveneOsoby = $rowData[21];
            $maPravoNaOsobu = $this->pravo_schvaleni->lzeZobrazitRC($rcNavstiveneOsoby);
            if ($maPravoNaOsobu && $rowData[27]) {
                $result = GuiUtils::changeLocationQuest(ronText('Schvalit'), ronText('opravdu_schvalujete_navstevu'),
                    ' ?menuid=navstevy&action=schvaleninavstevy&id=' . $rowData[0] .
                    '&browser=1');
            }
        } else {
            $result = StringUtils::htmlTextConvert($rowData[47]) . " (" .StringUtils::format('datetime', $this->database->stringToTimestamp($rowData[48])) . ")";
        }
        return $result;
    }
}



/**
 * Class NavstevyBrowser
 * @package Ron\RonPortal\Navstevy
 */
class NavstevyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby $prava */
    private $prava;

    /** @var  PravaNaOsoby $pravo_schvaleni */
    private $pravo_schvaleni;

    /**
     * NavstevyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'navstevy');
        $this->prava = Prava::vratPrava('navstevy');
        $this->pravo_schvaleni = Prava::vratPrava('navstevy_schvaleni');
        Prava::nastavObdobiProPrava(time(),time());

        $this->code = 'browsernavstevy';
        $this->tableId = 'browsernavstevy';
        //$this->browserUrl = "?menuid=navstevy&";

        $this->filterForm = new NavstevyFilter($database, null, null);
        $this->filterForm->loadData();

        $this->title = ronText('navstevy') . ' ' .
            (($_SESSION['omezeniVratnice'] == 1)
                ? $this->rozdeleniVratnice($database)
                : (($_SESSION['omezeniVratnice'] == 0) ? ronText('vsechnyVratnice') : ''));

        $this->sqlSelectPart =
            'Navsteva.id, TypNavstevy, Navsteva.kod, Navsteva.titul, Navsteva.prijmeni, Navsteva.jmeno, zamestnanecRC, zamestnanecOsobniCislo,' .   //0 - 7
            ' ZamestnanecDatumNastupu, ZamestnanecTypRozdeleni, ZamestnanecRozdeleni, Firma, ' .                                                    //8 - 11
            ' KodTypuRozdeleni_Firma, Rozdeleni_Firma, prichod, Navsteva.odchod, Navsteva.osobaRC, Navsteva.osobniCislo, Navsteva.datumNastupu,' .  //12 - 18
            ' PristeStejneMedium, Vozidlo, NavstivenaOsobaRC, JmenoNavstiveneOsoby, Doklad,' .                                                      //19 - 23
            $database ->castWideMemo('duvodNavstevy') . ',' .                                                                                       //24
            ' KodTypuRozdeleni, Rozdeleni, PredemOcekavanaNavsteva, Navsteva.OcekavanyPrichod, KodTypuRozdeleni_Vratnice, Rozdeleni_Vratnice,' .    //25 - 29
            ' Navsteva.Uzivatelska1, Navsteva.Uzivatelska2, Navsteva.Uzivatelska3, Navsteva.Uzivatelska4, Navsteva.Uzivatelska5,' .                 //30 - 35
            ' Navsteva.Uzivatelska6, Navsteva.Uzivatelska7, Navsteva.Uzivatelska8, Navsteva.Uzivatelska9, Navsteva.Uzivatelska10,' .                //36 - 40
            ' NavstivenaOsoba.Telefon,' .                                           // 41
            " COALESCE(Zapsal.Jmeno,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Zapsal.Prijmeni,''), " .   // 42
            ' Karta.OznaceniMedia, PrichodZapsalRC, Karta.Kod,' .                   // 43 - 45

            ' (Select Rozdeleni.Popis FROM TypRozdeleni LEFT JOIN Rozdeleni ON (TypRozdeleni.Kod = Rozdeleni.KodTypuRozdeleni)' .
            '  WHERE TypRozdeleni.Kod = KodTypuRozdeleni_Vratnice AND Rozdeleni.Kod = Rozdeleni_Vratnice) AS Rozdeleni_vratnice_popis,' .           //46
            ' Navsteva.Schvalil, Navsteva.DT_Schvaleni, DuvodNavstevy.Popis';                                                                       //47..49

        $this->primaryField = 0;

        $this->sqlFromPart =
            'Navsteva LEFT JOIN Osoba NavstivenaOsoba ON (Navsteva.navstivenaOsobaRC = NavstivenaOsoba.RC) ' .
            ' LEFT JOIN Osoba Zapsal ON (Navsteva.prichodZapsalRC = Zapsal.RC)' .
            ' LEFT JOIN PrirazeniKarty' .
            '  ON (Navsteva.OsobaRC = PrirazeniKarty.RC) AND (Navsteva.OsobniCislo = PrirazeniKarty.OsobniCislo) AND (Navsteva.DatumNastupu = PrirazeniKarty.DatumNastupu)' .
            '  AND PrirazeniKarty.DatumPrideleni <= Navsteva.Prichod' .
            '  AND (PrirazeniKarty.DatumVraceni >= Navsteva.Prichod  OR  PrirazeniKarty.DatumVraceni IS NULL)' .
            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            ' LEFT JOIN S_AnoNe schvalenaNavsteva ON (Navsteva.Schvalil = PrirazeniKarty.kodKarty)' .
            ' LEFT JOIN DuvodNavstevy ON ( (DuvodNavstevy.Kod = Navsteva.duvodNavstevy2))';

        // Typ navstevy
        $this->addField( new TypNavstevy($database, null))
            ->setCode('col-typnavstevy');

        // Vratnice
        $this->addField()
            ->setSqlFieldName('Rozdeleni_vratnice_popis', 46) ->setHeaderText('vratnice')
            ->setCode('col-vratnice')
            ->setVisible(!$_SESSION['omezeniVratnice']);

        // Duvod navstevy
        $this->addField()
            ->setSqlFieldName('DuvodNavstevy.Popis', 49) ->setHeaderText('duvod_navstevy')
            ->setCode('col-duvodnavstevy');

        //Ocekavany?
        $this->addField(BROWSER_FIELD_TYPE_SELECT)
            ->setSqlFieldName('Navsteva.PredemOcekavanaNavsteva', 27) ->setHeaderText('ocekavany') ->setOrdering(1)
            ->setCode('col-ocekavany')
            ->setOptions([0 => ronText('Str_No'), 1 => ronText('Str_Yes')])
            ->setAlign('center');

        //Navsteva.prijmeni
        $this->addField()
            ->setSqlFieldName('Navsteva.prijmeni', 4) ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            ->setOrdering(1) ->setClassName('font-weight-bold')
            ->setLink('?menuid=navstevy&action=zaznam');

        //Navsteva.jmeno
        $this->addField()
            ->setSqlFieldName('Navsteva.jmeno', 5) ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setOrdering(1)
            ->setLink('?menuid=navstevy&action=zaznam');

        //Ocekavany prichod
        $this->addField( new CasNavstevy($database))
            ->setSqlFieldName('OcekavanyPrichod', 28) ->setHeaderText('ocekavany_datum') ->setFormat('weektimestamp')
            ->setCode('col-ocekavanyprichod')
            ->setLink('?menuid=navstevy&formaction=edit');

        //doklad
        $this->addField()
            ->setSqlFieldName('doklad', 23) ->setHeaderText('doklad')
            ->setCode('col-doklad');

        //vozidlo
        $this->addField()
            ->setSqlFieldName('vozidlo', 20) ->setHeaderText('vozidlo')
            ->setCode('col-vozidlo');

        //firma
        $this->addField()
            ->setSqlFieldName('firma', 11) ->setHeaderText('firma') ->setOrdering(1)
            ->setCode('col-firma');

        //OznaceniMedia
        $this->addField()
            ->setSqlFieldName('OznaceniMedia', 43) ->setHeaderText('oznaceni_media') ->setOrdering(1)
            ->setCode('col-oznacenimedia');

        //Prichod
        $this->addField( new CasNavstevy($database))
            ->setSqlFieldName('Prichod', 14) ->setHeaderText('prichod') ->setFormat('weektimestamp')
            ->setCode('col-prichod')
            ->setTitleFieldIndex(42, ronText('prichod_zapsal') . ': ') ->setLink('?menuid=navstevy&formaction=edit');

        $this->defaultSort = -count($this ->fields);

        //Odchod
        $this->addField(new PotvrzeniOdchoduNavstevyField($database, null))
            ->setSqlFieldName('Odchod', 15) ->setHeaderText('Odchod') ->setFormat('weektimestamp')
            ->setCode('col-odchod');

        //NavstivenaOsoba
        $this->addField()
            ->setSqlFieldIndex(22) ->setTitleFieldIndex(41, ronText('telefon') . ': ') ->setHeaderText('navstivena_osoba')
            ->setCode('col-navstivenaosoba');

        $this->addField( new NavstevuSchvalilaOsoba($database, $this->pravo_schvaleni))
            ->setCode('col-schvalil');
    }


    function setFilterForm()
    {
        // nedela nic - filter nastaven v constructoru
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[24]) {
            echo
                "<tr class='$className poznamka-navstevy'>" .
                '<td></td>' .
                '<td colspan="' . ($this->columnCount - 1) . '">' .
                '<span class="browserpoznamka">' . ronText('duvod_navstevy') . ': ' . StringUtils::htmlTextConvert($values[24]) . '</span>' .
                '</td>' .
                '</tr>';
        }

        if ($_SESSION['uzivatelskePoznamky']) {
            for ($x = 1; $x <= 10; $x++) {
                if ($values[30 + $x]) {
                    echo
                        "<tr class='$className uzivatelska-pole-navstev'>" .
                        '<td></td><td></td>' .
                        '<td colspan="' . ($this->columnCount - 2) . '">' .
                        '<span class="browserpoznamka">'. RonPortalSettings::getValue('navstevy_uzivatelska' . $x) . ': ' . StringUtils::htmlTextConvert($values[30 + $x]) . '</span>' .
                        '</td>' .
                        '</tr>';
                }
            }
        }
    }

    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na RC navstivene osoby pravo
        if ($this->prava->lzeZobrazitRC($values[21])){
            return true;
        }
        return false;
    }

    /**
     * @param Database $database
     * @return mixed
     * @throws DBException
     */
    private function rozdeleniVratnice($database)
    {
        if (!RonPortalSession::getValue('rozdelenivratnice')) {
            return '';
        }

        $params = $database->createParameters();
        $sql =
            'SELECT Rozdeleni.Popis' .
            ' FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenivratnice')) .
            ' AND Rozdeleni.Kod = ' . $params->addParameter($_SESSION['rozdelenivratnice']);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        return $row[0];
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $dnesZacatekDne = DateUtils::startOfTheDay(time());
        $result = '';

        if (RonPortalSession::getValue('omezeniVratnice') == 1 && RonPortalSession::getValue('rozdelenivratnice') != '') {
            $result =
                "Navsteva.Rozdeleni_Vratnice = " . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

        if (RonPortalSession::getValue('jenMedium')){
            $result .=
                ($result ? ' AND ' : '') .
                " Karta.Kod = " . $params->addParameter(RonPortalSession::getValue('medium')) . " AND prichod IS NOT NULL AND odchod IS NULL";
        } else {
            //timestamp -> string
            if (RonPortalSession::getValue('omezeniNavstev') == 0) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " (Navsteva.OcekavanyPrichod >= " . $params->addParameter($this->database->timestampToParam($dnesZacatekDne)) . " AND Navsteva.prichod IS NULL AND Navsteva.odchod IS NULL )";

            } else if (RonPortalSession::getValue('omezeniNavstev') == 1) {
                $result .=
                    ($result ? ' AND ' : '') .
                    //' (Navsteva.prichod >= ' . $params->addParameter($this->database->timestampToParam($dnesZacatekDne)) .
                    ' (Navsteva.prichod <= ' . $params->addParameter($this->database->timestampToParam(time())) . // uz prisla a jeste neodesla
                    ' AND Navsteva.odchod IS NULL';
                $result .= ")";

            } else if (RonPortalSession::getValue('omezeniNavstev') == 2) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " ((Navsteva.OcekavanyPrichod >= " . $params->addParameter($this->database->timestampToParam($_SESSION['datum'])) .
                    " AND Navsteva.OcekavanyPrichod <= " . $params->addParameter($this->database->timestampToParam($_SESSION['datumdodata'])) .
                    " ) OR (" .
                    " Navsteva.prichod >= " . $params->addParameter($this->database->timestampToParam($_SESSION['datum'])) .
                    " AND Navsteva.prichod <= " . $params->addParameter($this->database->timestampToParam($_SESSION['datumdodata'])) .
                    "))";
            }

            if (RonPortalSession::getValue('jenTypNavstevy')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.TypNavstevy = " . $params->addParameter($_SESSION['typNavstevy']);
            }

            if (RonPortalSession::getValue('jenOsoba')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.jmenoNavstiveneOsoby = " . $params->addParameter($_SESSION['osoba']);
            }

            if (RonPortalSession::getValue('jenFirma')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.firma = " . $params->addParameter($_SESSION['firma']);
            }

            if (RonPortalSession::getValue('jenPrijmeni')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.prijmeni = " . $params->addParameter($_SESSION['prijmeni']);
            }

            if (RonPortalSession::getValue('jenKod')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.kod = " . $params->addParameter($_SESSION['kod']);
            }

            if ($_SESSION['jenDuvodNavstevy']) {
                $result .=
                    ($result ? ' AND ' : '') .
                    " Navsteva.duvodNavstevy2 = " . $params->addParameter($_SESSION['duvodNavstevy']);
            }


            if ($_SESSION['jenVozidloNavstevy']) {
                if ($result) {
                    $result .= ' AND ';
                }

                if ($_SESSION['typVozidla'] == 1){
                    $result .= " Navsteva.vozidlo IS NOT NULL ";
                } else {
                    $result .= " Navsteva.vozidlo IS NULL ";
                }

            }

            if ($_SESSION['jenSchvaleno']) {
                if ($result) {
                    $result .= ' AND ';
                }

                if ($_SESSION['schvaleniNavstevy'] == 0){
                    $result .= " Navsteva.Schvalil IS NULL " ;
                } else {
                    $result .= "Navsteva.Schvalil IS NOT NULL ";
                }
            }
        }

        return new SqlStatement($result, $params);
    }
}