<?php
/**
 * UTF-8
 * Roman.Szablatura RON Software 28.3.2017
 */

namespace Ron\RonPortal\Navstevy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use function Ron\ronText;

/**
 * Class Navsteva
 * @package Ron\RonPortal\Navstevy
 */
class Navsteva
{
    public $id;
    public $typNavstevy;
    public $kod;
    public $titul;
    public $prijmeni;
    public $jmeno;
    public $zamestnanecRC;
    public $zamestnanecOsobniCislo;
    public $zamestnanecDatumNastupu;
    public $zamestnanecTypRozdeleni;
    public $zamestnanecRozdeleni;
    public $firma;
    public $kodTypuRozdeleni_Firma;
    public $rozdeleni_Firma;
    public $prichod;
    public $odchod;

    /** @var string     RC prirazene externi osoby (karty) */
    public $osobaRC;
    /** @var string     OsobniCislo prirazene externi osoby (karty) */
    public $osobniCislo;
    /** @var int        DatumNastupu prirazene externi osoby (karty) */
    public $datumNastupu;
    /** @var string     Oznaceni prirazene karty (pres osobu) */
    public $oznaceniMedia;
    //public $osobaPrijmeni;

    public $pristeStejneMedium;

    public $vozidlo;
    public $navstivenaOsobaRC;
    public $jmenoNavstiveneOsoby;
    public $doklad;
    public $duvodNavstevy;
    public $duvodNavstevy2;
    public $kodTypuRozdeleni;
    public $rozdeleni;
    public $predemOcekavanaNavsteva;
    public $ocekavanyPrichod;
    public $kodTypuRozdeleni_Vratnice;
    public $rozdeleni_Vratnice;
    public $uzivatelskaPole;
    public $rozdeleni_VratnicePopis;
    public $typNavstevyPopis;
    public $zamestnanecPrijmeniJmenoTitul;
    //public $dobaNavstevy;
    public $insert_UserName;
    public $insert_Timestamp;
    public $update_UserName;
    public $update_Timestamp;
    public $prichodZapsalRC;
    public $navstevuSchvalil;
    public $navstevuSchvalilDatum;


    /**
     * @param Database $database
     * @param $id
     * @return Navsteva
     * @throws Exception
     */
    public static function nactiNavstevu($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT id, TypNavstevy, Navsteva.kod, Navsteva.titul, Navsteva.prijmeni, Navsteva.jmeno, zamestnanecRC, zamestnanecOsobniCislo, ' .                //0 - 7
            'zamestnanecDatumNastupu, zamestnanecTypRozdeleni, zamestnanecRozdeleni, firma, ' .                                                         //8 - 11
            'KodTypuRozdeleni_Firma, Rozdeleni_Firma, prichod, odchod, osobaRC, osobniCislo, datumNastupu, ' .                                                  //12 - 18
            'pristeStejneMedium, vozidlo, navstivenaOsobaRC, jmenoNavstiveneOsoby, doklad, {fn_CASTWIDEMEMO(Navsteva.duvodNavstevy)}, ' .                     //19 - 24
            'KodTypuRozdeleni, Rozdeleni, PredemOcekavanaNavsteva, Navsteva.OcekavanyPrichod, KodTypuRozdeleni_Vratnice, ' .                            //25 - 29
            'Rozdeleni_Vratnice, Navsteva.Uzivatelska1, Navsteva.Uzivatelska2, Navsteva.Uzivatelska3, Navsteva.Uzivatelska4, Navsteva.Uzivatelska5, ' .        //30 - 35
            'Navsteva.Uzivatelska6, Navsteva.Uzivatelska7, Navsteva.Uzivatelska8, Navsteva.Uzivatelska9, Navsteva.Uzivatelska10, ' .                    //36 - 40
            'insert_UserName, insert_Timestamp, update_UserName, update_Timestamp, prichodZapsalRC, Navsteva.Schvalil, Navsteva.DT_Schvaleni, ' .       //41 - 47
            'DuvodNavstevy.popis ';            //48

        $sql .= 'FROM Navsteva LEFT JOIN DuvodNavstevy ON (Navsteva.duvodNavstevy2 = DuvodNavstevy.kod) ';
        $sql .= 'WHERE id = ' . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()){
            throw new Exception(ronText('navsteva_neexistuje'));
        }

        $navsteva = new Navsteva();
        $navsteva->id = $id;
        $navsteva->nactiNavstevuZRadku($database, $row);
        return $navsteva;
    }

    /**
     * @param Database $database
     * @param $rc
     * @return string
     * @throws DBException
     */
    private function jmenoZamestnance($database, $rc)
    {
        $sql = "SELECT COALESCE(Osoba.Prijmeni,'') + ' ' + COALESCE(Osoba.Jmeno,'') + COALESCE(' ' + Osoba.Titul,'') FROM Osoba WHERE RC = '" . $database->addSlashes($rc) . "'";
        $cursor = $database->getSqlResult($sql);
        $row = $cursor->fetch();

        return $row ? $row[0] : '';
    }

    /**
     * @param Database $database
     * @param $row
     * @throws DBException
     */
    private function nactiNavstevuZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->typNavstevy = $row[1];
        $this->kod = $row[2];
        $this->titul = $row[3];
        $this->prijmeni = $row[4];
        $this->jmeno = $row[5];
        $this->zamestnanecRC = $row[6];
        $this->zamestnanecOsobniCislo = $row[7];
        $this->zamestnanecDatumNastupu = $database->stringToTimestamp($row[8]);
        $this->zamestnanecTypRozdeleni = $row[9];
        $this->zamestnanecRozdeleni = $row[10];
        $this->firma = $row[11];
        $this->kodTypuRozdeleni_Firma = $row[12];
        $this->rozdeleni_Firma = $row[13];
        $this->prichod = $database->stringToTimestamp($row[14]);
        $this->odchod = $database->stringToTimestamp($row[15]);

        // prirazena karta
        $this->osobaRC = $row[16];
        $this->osobniCislo = $row[17];
        $this->datumNastupu = $database->stringToTimestamp($row[18]);
        $this->pristeStejneMedium = $row[19];

        $this->vozidlo = $row[20];
        $this->navstivenaOsobaRC = $row[21];
        $this->jmenoNavstiveneOsoby = $row[22];
        $this->doklad = $row[23];
        $this->duvodNavstevy = $row[24];
        $this->duvodNavstevy2 = $row[48];
        $this->kodTypuRozdeleni = $row[25];
        $this->rozdeleni = $row[26];
        $this->predemOcekavanaNavsteva = $row[27];
        $this->ocekavanyPrichod = $database->stringToTimestamp($row[28]);
        $this->kodTypuRozdeleni_Vratnice = $row[29];
        $this->rozdeleni_Vratnice = $row[30];

        $this->uzivatelskaPole = [];
        for ($i=1; $i <=10; $i++){ $this->uzivatelskaPole[$i] = $row[30+$i];}

        $this->rozdeleni_VratnicePopis = $this->nacteniRozdeleni_VratnicePopis($database);
        $this->typNavstevyPopis = ($row[1] == 0) ? ronText('cizi') : ronText('vlastni');

        $this->zamestnanecPrijmeniJmenoTitul = $this->jmenoZamestnance($database, $row[6]);
        //$this->dobaNavstevy = $this->dobaNavstevy();
        $this->insert_UserName = $row[41];
        $this->insert_Timestamp = $database->stringToTimestamp($row[42]);
        $this->update_UserName = $row[43];
        $this->update_Timestamp = $database->stringToTimestamp($row[44]);
        $this->prichodZapsalRC = $this->jmenoZamestnance($database, $row[45]);

        $this->oznaceniMedia = $this->nacteniOznaceniMedia($database);
        //$this->osobaPrijmeni = $this->nacteniOsobaPrijmeni($database);

        $this->navstevuSchvalil = $row[46];
        $this->navstevuSchvalilDatum = $database->stringToTimestamp($row[47]);
    }

//    /** Informace o dobe navstevy
//     * @return string
//     */
//    private function dobaNavstevy()
//    {
//        $casVypis = '';
//        if ($this->odchod != "") {
//            $casMezi = round(abs($this->odchod - $this->prichod) / 60, 2);
//            $casVypis = ((round(($casMezi * (1/60)), 2) . " h"));
//            $casVypis .= "  / " . $casMezi . " " . ronText('minut');
//        }
//        return $casVypis;
//    }

    /**
     * @param Database $database
     * @return mixed
     * @throws DBException
     */
    private function nacteniOznaceniMedia($database)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Karta.OznaceniMedia' .
            ' FROM PrirazeniKarty, Karta' .
            ' WHERE (PrirazeniKarty.KodKarty = Karta.Kod)' .
            ' AND PrirazeniKarty.RC = ' . $params->addParameter($this->osobaRC) .
            ' AND PrirazeniKarty.OsobniCislo = ' . $params->addParameter($this->osobniCislo) .
            ' AND PrirazeniKarty.DatumNastupu = ' . $params->addParameter($database->dateToParam($this->datumNastupu));

        return DBUtils::getSqlValue($database, $sql, $params);
    }


//    /**
//     * @param Database $database
//     * @return mixed
//     * @throws DBException
//     */
//    private function nacteniOsobaPrijmeni($database)
//    {
//        $params = $database->createParameters();
//
//        $sql =
//            'SELECT Osoba.Prijmeni' .
//            ' FROM Karta JOIN PrirazeniKarty PK ON (Karta.Kod = PK.kodKarty)' .
//            ' JOIN Osoba ON (PK.RC = Osoba.RC) ' .
//            ' WHERE PK.RC = ' . $params->addParameter($this->osobaRC) .
//            ' AND PK.osobniCislo = ' . $params->addParameter($this->osobniCislo) .
//            ' AND PK.datumNastupu = ' . $params->addParameter($database->dateToParam($this->datumNastupu));
//
//        return DBUtils::getSqlValue($database, $sql, $params);
//    }


    /**
     * @param Database $database
     * @return mixed
     * @throws DBException
     */
    private function nacteniRozdeleni_VratnicePopis($database){
        $params = $database->createParameters();

        $sql =
            "Select Rozdeleni.Popis " .
            "FROM Rozdeleni " .
            "WHERE Rozdeleni.KodTypuRozdeleni = " . $params->addParameter($this->kodTypuRozdeleni_Vratnice) . " AND Rozdeleni.Kod = " . $params->addParameter($this->rozdeleni_Vratnice);

        return DBUtils::getSqlValue($database, $sql, $params);
    }
}
