<?php
/**
 * UTF8
 * roman.Szablatura 27.3.2017
 */

namespace Ron\RonPortal\Navstevy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\RonPortal\User;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;

require_once('ron/controls.php');
require_once('adsapp/b_dokumenty.php');
require_once ('Navstevy/navsteva.php');
require_once('adsapp/dokumentyutils.php');


/**
 * Class NavstevyController
 * @package Ron\RonPortal\Navstevy
 */
class NavstevyController extends JsonController
{
    const XML_EVENT_SCHVALENI = 'NAVSTEVA_SCHVALENI';
    const XML_EVENT_AFTER_SCHVALENI = 'NAVSTEVA_AFTER_SCHVALENI';

    /** @var  Database $database */
    private $database;

    /** @var PravaNaOsoby $prava */
    private $prava;

    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->prava = Prava::vratPrava('navstevy');
    }

    /**
     * @param $formaction
     * @param null $zamestnanec
     * @param null $ocekavana
     * @param null $navsteva
     * @return Redirect|NavstevaForm
     * @throws Exception
     */
    private function navstevyForm($formaction, $zamestnanec = null, $ocekavana = null, $navsteva = null)
    {
        require_once ('navstevy/f_navstevy.php');
        $form = new NavstevaForm($this->database, $formaction, $zamestnanec, $ocekavana, $navsteva);

        if($form->handleAction()){
            $rozdelenivratnice = (isset($_SESSION['rozdelenivratnice']) && $_SESSION['rozdelenivratnice'] != "") ? ("&rozdelenivratnice=" . $_SESSION['rozdelenivratnice']) : "";
            return new Redirect(null, "?menuid=navstevy$rozdelenivratnice");
        }
        return $form;
    }

    /**
     * @param int $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    private function potvrzeniOdchodu($id)
    {
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT navstivenaOsobaRC FROM Navsteva WHERE ID = '. (int)$id);

        // Pokud ma pravo jen na ocekavane navstevy (priznak) nebo nema pravo na RC navstivene osoby
        if ($this->prava->priznak  ||  !$this->prava->lzeZobrazitRC($osobaRC)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $recVlozitOdchod = new RonPortalDBRecord($this->database, 'Navsteva', $this);

        $recVlozitOdchod->addField('id', 'int', true);
        $recVlozitOdchod->addField('TypNavstevy', 'int', true);
        $recVlozitOdchod->addField('Prijmeni', 'string', true);
        $recVlozitOdchod->addField('PredemOcekavanaNavsteva', 'int', true);

        $recVlozitOdchod->addField('Odchod', 'timestamp', false, 0,time());
        $recVlozitOdchod->addField('Update_UserName', 'string', false, 25, User::getLoginName());
        $recVlozitOdchod->addField('Update_Timestamp', 'timestamp', false, 0,time());

        $params = $this->database->createParameters();

        $recVlozitOdchod->setWhere('id = ' . $params->addParameter($id), $params);

        $recVlozitOdchod->loadAndUpdateRecord();

        return new Redirect(null,'?menuid=navstevy#odchod_vlozen_uspesne');
    }


    /**
     * @param $idNavstevy
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailNavstevy($idNavstevy)
    {

        $detailNavstevy = Navsteva::nactiNavstevu($this->database, $idNavstevy);
        Model::addData('navsteva', $detailNavstevy);

        return new BaseViewObject('navstevy/view_navsteva.php');
    }

    /**
     * @param $idpp
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function nacteniZamestnance($idpp, $output)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul' . // , RO.Rozdeleni, Rozdeleni.Popis' .
            ' FROM PracovniPomer PP INNER JOIN Osoba ON (Osoba.rc = PP.rc)' .
            //' LEFT OUTER JOIN RozdeleniOsob RO ON ((RO.rc = PP.rc) AND (RO.OsobniCislo = PP.OsobniCislo) ' .
            //' AND (RO.DatumNastupu = PP.DatumNastupu) AND (RO.KodTypuRozdeleni = 6) AND (RO.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .') '.
            //' AND ((RO.PlatnostDo >=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) OR (RO.PlatnostDo IS NULL)))) ' .
            //' LEFT JOIN Rozdeleni ON ((Rozdeleni.KodTypuRozdeleni = RO.KodTypuRozdeleni) AND (Rozdeleni.Kod = RO.Rozdeleni))) '.
            ' WHERE ( (PP.datumNastupu <=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) ' .
            ' AND ((PP.datumUkonceni >=  ' . $params->addParameter($this->database->timestampToParam(time())) . ' ) OR (PP.datumUkonceni IS NULL)) AND (PP.Zamestnanec > 0) ) ' .
            ' AND PP.ID = ' . $params->addParameter($idpp);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if ($row) {
            $output ->addData('prijmeni', StringUtils::jsonText($row[0]));
            $output ->addData('jmeno', StringUtils::jsonText($row[1]));
            $output ->addData('titul', StringUtils::jsonText($row[2]));
            //$output ->addData('rozdeleni', StringUtils::jsonText($row[3]));
            //$output ->addData('popis', StringUtils::jsonText($row[4]));
        }
    }


    /**
     * @param $id
     * @param $returnToBrowser
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    private function schvaleniNavstevy($id, $returnToBrowser)
    {
        try {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT navstivenaOsobaRC FROM Navsteva WHERE ID = ' . $params->addParameter($id), $params);

            //Pokud je pravo na RC navstivene osoby
            if (!$this->prava->lzeZobrazitRC($osobaRC)) {
                throw new Exception(ronText('Nemate_pravo'));
            }

            $rec = new RonPortalDBRecord($this->database, 'Navsteva', $this);

            $rec->addField('id', 'int', true);
            $rec->addField('TypNavstevy', 'int', true);
            $rec->addField('PredemOcekavanaNavsteva', 'int', true);

            $rec->addField('navstivenaOsobaRC', '', true);
            $rec->addField('jmenoNavstiveneOsoby', '', true);
            $rec->addField('jmeno', '', true);
            $rec->addField('prijmeni', 'string', true);
            $rec->addField('prichod', 'timestamp', true);
            $rec->addField('OcekavanyPrichod', 'timestamp', true);
            $rec->addField('duvodNavstevy', 'memo', true);

            $rec->addField('Schvalil', 'string', false, 25, User::getLoginName());
            $rec->addField('DT_Schvaleni', 'timestamp', false, 0, time());
            $rec->addField('Update_UserName', 'string', false, 25, User::getLoginName());
            $rec->addField('Update_Timestamp', 'timestamp', false, 0, time());

            $params = $this->database->createParameters();

            $rec->setWhere('id = ' . $params->addParameter($id), $params);

            $rec->loadRecord();

            $rec->executeXmlEvents(self::XML_EVENT_SCHVALENI);
            $rec->updateRecord();
            $rec->executeXmlEvents(self::XML_EVENT_AFTER_SCHVALENI);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if ($returnToBrowser) {
            return new Redirect(null, '?menuid=navstevy#schvaleno');
        }

        return new Redirect(null,'?menuid=navstevy&action=zaznam&record=' . $id);
    }



    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'nactenizamestnance':
                $this->nacteniZamestnance($_GET['idpp'], $output);
                break;
        }
    }


    /**
     * @param string $action
     * @return mixed
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        Prava::nastavObdobiProPrava(time(), time());

        TextManager::getInstance()->loadStrings('navstevy.ini');

        //HtmlHeaders::getInstance()->addCss('css/s_ron.css');
        HtmlHeaders::getInstance()->addScript('script/navstevy.js');
        HtmlHeaders::getInstance()->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Navstevy'));

        $this ->addTranslationsScript('navstevy');

        if (!$this->prava->jsouNejakePrava()){
            throw new NeniPravoException();
        }

        // Pri spusteni se kontroluje v URL parametr "rozdelenivratnice" pokud je, nacte se jeho hodnota do session.
        // Pote se jiz pracuje pouze s danou session
        if (isset($_GET['rozdelenivratnice'])){
            $_SESSION['rozdelenivratnice'] = $_GET['rozdelenivratnice'];
        }

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if (isset($_POST['record'])) $idNavstevy = $_POST['record'];
        else if (isset($_GET['record'])) $idNavstevy = $_GET['record'];
        else $idNavstevy = '';

        if (isset($_POST['ocekavana'])) $ocekavana = $_POST['ocekavana'];
        else if (isset($_GET['ocekavana'])) $ocekavana = $_GET['ocekavana'];
        else $ocekavana = '';

        if ($formaction){
            $navsteva = ($formaction == 'edit')
                ? Navsteva::nactiNavstevu($this->database, $idNavstevy)
                : null;

            return $this->navstevyForm($formaction, $idNavstevy, $ocekavana, $navsteva);
        }


        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'nactenizamestnance' :
                    return parent::handleAction($action);

                case 'translations':
                    return $this->translations();

                case 'zaznam':
                    return $this->zobrazDetailNavstevy($_GET['record']);

                case 'potvrzeniodchodu':
                    return $this->potvrzeniOdchodu($_GET['id']);

                case 'schvaleninavstevy':
                    return $this->schvaleniNavstevy($_GET['id'], $_GET['browser']);

//            case 'vzornovenavstevy':
//                return $this->navstevyForm('new', null, 0, $_GET['record']);
            }
        }

        require_once ('navstevy/b_navstevy.php');
        $b = new NavstevyBrowser($this->database);
        $b->load();
        return $b;
    }
}

