<?php
/** UTF8 Roman.Szablatura, 26.4.2016 */

namespace Ron\Personalistika;

use Exception;
use Ron\Browser;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\StringUtils;
use function Ron\ronText;

require_once ('adsapp/ronportalfilter.php');
require_once('adsapp/ronportalbrowser.php');
require_once ('ron/browser.php');
require_once ('ron/guiutils.php');

class DistribuovaneDokumentyInfoField extends RonPortalFilterForm
{
    private $idDokumentu;
    private $idVzkazu;
    private $platnostOd;

    function __construct($database, $idDokumentu, $idVzkazu, $platnostOd)
    {
        parent::__construct($database, '');

        $this->skrytTlacitkoNastavit = true;

        $this->idDokumentu = $idDokumentu;
        $this->idVzkazu = $idVzkazu;
        $this->platnostOd = $platnostOd;
    }

    function renderFiltrPrectene()
    {
        echo '<DIV class="filterrow">';
        echo '<span class="filtercaption">' . ronText('prectene_dokumenty') . '</span>';

        $data = array(
            0 => ronText('Vse'),
            1 => ronText('Str_Yes'),
            2 => ronText('Str_No')
        );

        LinksList::renderLinks($data, $_SESSION['jenPrectene'],
            '?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $_GET['dokumentid'] . '&vzkazid=' . $_GET['vzkazid'] . '&platnostod=' . $_GET['platnostod'] . '&prectene=', '', false);
        echo '</DIV>';
    }

    /**
     * @throws DBException
     */
    function nactiZobrazDetail()
    {
        $params = $this->database->createParameters();

        $sql =
            ' SELECT  Dokument.datum, VerzeDokumentu.Popis, DatumCas, Dokument.jmenoSouboru,' .    //0 - 3
            ' DokumentVzkazu.Precteno, TypDokumentu.popis, VerzeDokumentu.PotvrditDo ' .    //4 - 6
            ' FROM Vzkaz, DokumentVzkazu, VerzeDokumentu, Osoba, Dokument, TypDokumentu ' .
            ' WHERE DokumentVzkazu.idvzkazu = Vzkaz.id ' .
            ' and Dokument.typ = TypDokumentu.kod ' .
            ' and Vzkaz.RCPrijemce = Osoba.RC ' .
            ' and DokumentVzkazu.idDokumentu = ' . $params->addParameter($this->idDokumentu) .
            ' and DokumentVzkazu.PlatnostOd = ' . $params->addParameter($this->database->timestampToParam($this->platnostOd)) .
            ' and Dokument.id = VerzeDokumentu.id ' .
            ' and DokumentVzkazu.idDokumentu = VerzeDokumentu.id and DokumentVzkazu.PlatnostOd = VerzeDokumentu.PlatnostOd ';


        $cursor = $this->database->getSqlResult($sql,$params);

        $row = $cursor->fetch();

        echo '<strong>' . ronText('typ_dokumentu') . '</strong>: ' . $row[5];
        echo '<br />';
        echo '<strong>' . ronText('Popis') . '</strong>: ' . $row[1];
        echo '<br />';
        echo '<strong>' . ronText('platnost_dokumentu_od') . '</strong>: ' . DBUtils::getFormatedValue($this ->database, 'date', $row[0]);
        echo '<br />';
        echo '<strong>' . ronText('datum_a_cas_distribuce') . '</strong>: ' . DBUtils::getFormatedValue($this ->database, 'datetime', $row[2]);
        echo '<br />';
        echo '<strong>' . ronText('dokumenty_potvrdit_do') . '</strong>: ' . DBUtils::getFormatedValue($this ->database, 'finedate', $row[6]);
    }

    function odkazZpet()
    {
        $result = '<DIV>';
        $result .= '<a href="?menuid=dokumenty&">';
        $result .= ronText('Zpet');
        $result .= '</a>';
        $result .= '</DIV>';

        echo $result;
    }

    function odkazNaStazeni()
    {
        $result = '';
        $result .= '<a href="?menuid=dokumenty&distribuovanedokumenty=1&dokumentid=' . $_GET['dokumentid'] . '&vzkazid=' . $_GET['vzkazid'] . '&platnostod=' . $_GET['platnostod'] . '&prectene=' . $_GET['prectene'] . '" target="_blank">';
        $result .= ronText('nahled_dokumentu');
        $result .='</a>';
        echo $result;
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        $this->nactiZobrazDetail();
        echo '<br />';
        $this->odkazNaStazeni();
        echo '<br />';
        echo '<br />';
        $this->renderFiltrPrectene();
        echo '<br />';
        $this->odkazZpet();

        parent::renderFilters();
    }


    function loadPrectena()
    {
        if(isset($_GET['prectene'])){
            $_SESSION['jenPrectene'] = $_GET['prectene'];
        }
    }

    function loadData()
    {
        $this->loadPrectena();
    }
}

/**
 * Class DistribuovaneDokumentyDetail
 * @package Ron\Personalistika
 */
class DistribuovaneDokumentyDetail extends Browser
{
    /** @var \Ron\RonPortal\PravaNaOsoby  */
    var $pravaProhlizet;

    /** @var DistribuovaneDokumentyInfoField*/
    var $filterForm;

    private $seznamOsob;

    /**
     * DistribuovaneDokumentyDetail constructor.
     * @param Database $database
     * @param $idDokumentu
     * @param $platnostOd
     * @param $idVzkazu
     * @throws Exception
     */
    function __construct($database, $idDokumentu, $platnostOd, $idVzkazu)
    {
        parent::__construct($database);

        $this->pravaProhlizet = Prava::vratPrava('dokumenty_prohl');

        $this->title = ronText('detail_distribuovane_dokumenty');

        $this->browserUrl =
            '?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $idDokumentu
            . '&vzkazid=' . $idVzkazu
            . '&platnostod=' . StringUtils::format('date', $platnostOd)
            . '&prectene=' . $_SESSION['jenPrectene']
            . '&';
        $this->defaultSort = 1;
        $this->primaryField = 0;

        $this->sqlSelectPart =
            'Osoba.Update_Timestamp, Osoba.Prijmeni, Osoba.Jmeno, Vzkaz.Predmet, ' . $database->castWideMemo('Popis') . ', Precteno, ' .   //0 - 5
            $database->castWideMemo('DokumentVzkazu.Vyjadreni') . ', ' . $database->castWideMemo('DokumentVzkazu.IdentifikacePodpisu') . ', DatumCas, Osoba.RC, PracovniPomer.ID';    //6 - 10
        $this->sqlFromPart =
            'Vzkaz JOIN DokumentVzkazu ON (DokumentVzkazu.idvzkazu = Vzkaz.id) ' .
            'JOIN Osoba ON (Vzkaz.RCPrijemce = Osoba.RC)'.
            'JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC)';
        $this->sqlWherePart =
            ' PracovniPomer.DatumNastupu <= ' . $this->database->timestampToString(time()) .
            ' and (PracovniPomer.DatumUkonceni >= ' . $this->database->timestampToString(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL) ' .
            ' and DokumentVzkazu.idDokumentu = ' . (int)$idDokumentu .
            ' and DokumentVzkazu.PlatnostOd = ' . $database->timestampToString($platnostOd);

        $this->filterForm = new DistribuovaneDokumentyInfoField($database, $idDokumentu, $idVzkazu, $platnostOd);
        $this->filterForm ->loadData();

//        //Datum a čas
//        $this->addField([BROWSER_FIELD_SQL_INDEX => 8,BROWSER_FIELD_SQL_NAME => 'DatumCas',
//                        BROWSER_FIELD_HEADER_TEXT => 'DatumCas',
//                        BROWSER_FIELD_FORMAT =>'date',
//                        BROWSER_FIELD_ALIGN =>'center',
//                        BROWSER_FIELD_ORDERING => 1]);
        //Příjmení
        $this->addField([BROWSER_FIELD_SQL_INDEX => 1,BROWSER_FIELD_SQL_NAME => 'Prijmeni',
                        BROWSER_FIELD_HEADER_TEXT => 'Prijmeni',
                        BROWSER_FIELD_ORDERING => 1]);
        //Jméno
        $this->addField([BROWSER_FIELD_SQL_INDEX => 2,BROWSER_FIELD_SQL_NAME => 'Jmeno',
                        BROWSER_FIELD_HEADER_TEXT => 'Jmeno',
                        BROWSER_FIELD_ORDERING => 1]);

        //Přečteno
        $this->addField([BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_SELECT,
                        BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'Precteno',
                        BROWSER_FIELD_HEADER_TEXT => 'precteno',
                        BROWSER_FIELD_OPTIONS => [0 => ronText('Str_No'), 1 => ronText('Str_Yes')], 'ordering' => 1,
                        BROWSER_FIELD_ALIGN => 'center']);

        //Identifikace podpisu
        $this->addField([BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_SQL_NAME => 'IdentifikacePodpisu',
                        BROWSER_FIELD_HEADER_TEXT => 'identifikace_podpisu']);

        //Vyjádření
        $this->addField([BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'DokumentVzkazu.Vyjadreni',
                        BROWSER_FIELD_HEADER_TEXT => 'vyjadreni']);

        $this->seznamOsob = [];

    }


    function renderFilter()
    {
        parent::renderFilter();
        $this->filterForm->render();
    }

    function getSQLFilter()
    {
        $result = '(' . $this->pravaProhlizet->sqlFiltr('PracovniPomer', time(), time()) .') AND ';

        switch ($_SESSION['jenPrectene']) {
            case 0:
                $result .= '1 = 1';
                break;
            case 1:
                $result .= 'DokumentVzkazu.Precteno = 1';
                break;
            case 2:
                $result .= 'DokumentVzkazu.Precteno = 0';
                break;
        }
        return $result;
    }

    /**
     * @param array $values
     * @return bool
     */
    function enableLine($values)
    {
        //RC jiz v seznamu je, nebude se vypisovat
        if (array_key_exists($values[9],$this->seznamOsob)) {
            return false;
        }
        else {
            $this->seznamOsob[$values[9]] = 1;
            return true;
        }
    }


}
