<?php
/** UTF-8
 * stanislav.elbl, 30.10.2015
 */

namespace Ron\RonPortal;
use Exception;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/b_dokumenty.php');
require_once('adsapp/ronportalfilter.php');
require_once('adsapp/ronportalbrowser.php');


class DokumentOsobyLinkField extends BrowserField
{
    private $nazevIndex;

    public function __construct($database, $options, $nazevIndex)
    {
        parent::__construct($database, $options);

        $this->nazevIndex = $nazevIndex;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '<a href="' . $this->link;

        $distribuovany = '';
        //dokument je distribuovany
        if (isset($rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX]) && $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] != 0) {
            $distribuovany = '&vzkazid=' . $rowData[6] .
                '&platnostod=' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[7]));
        }

        $osobaID = '';
        //je vybrana jina osoba nez prihlaseny uzivatel, do odkazu na stazeni se doplni id vybraneho PP (kontroluje se pravo podle RC)
        if (RonPortalSession::osobaRC() != User::getRC()) {
            $osobaID = '&osoba=' . RonPortalSession::vybranaOsoba();
        }

        if ($recordId != '') $result .= '&dokumentid=' . $recordId . $osobaID . $distribuovany;
        $result .= '" target="_blank">' . ($value ? StringUtils::htmlTextConvert($value) : StringUtils::htmlTextConvert($rowData[$this->nazevIndex])) . '</a>';

        return $result;
    }
}

class DokumentyOsobyForm extends RonPortalFilterForm
{
    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->f_formaction = '?menuid=dokumenty&';
        $this->f_classname = 'filtercontainer leftcaptions';

        $this->nastavPersonalistiku([Prava::vratPrava('prohl_osobni'), Prava::vratPrava('dokumenty_prohl'), Prava::vratPrava('dokumenty_seznam')]);
    }
}

class DistribuovanyDokumentField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $pravaProhlizet;

    function __construct($database)
    {
        parent::__construct($database);

        $this->header = ronText('distribuovany');
        $this->align = 'center';
        $this->className = 'dokument_distribuovany';
        $this->code = 'col-distribuovany';

        $this->pravaProhlizet = Prava::vratPrava('dokumenty_prohl');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $distribuovanyTyp = isset($rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX]) ? $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] : 0;

        //typ dokumentu neni distribuovany
        if ($distribuovanyTyp == 0) {
            $result = ronText('Str_No');
            return $result;
        } elseif ($distribuovanyTyp == 1 || $distribuovanyTyp == 2) {
            //pokud ma uzivatel prava na vybranou osobu
            // TODO: neuvidim i osoby na ktere nemam pravo?
            if($this->pravaProhlizet->lzeZobrazitRC(RonPortalSession::osobaRC())) {
                $result .='<a href=?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $rowData[0] . '&vzkazid=' . $rowData[6] .
                    '&platnostod=' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[7])) .
                    ' title="' . ronText('odkaz_na_detail') . '">';
                $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[10]));
                $result .='</a>';
                return $result;
            }
            //pokud neni pravo, zobrazi se jen datum distribuce bez odkazu
                $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[10]));
            return $result;
        }
        return  $result;
    }
}

class PrectenyDokumentField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);

        $this->header = ronText('identifikace_podpisu');
        $this->className = 'dokument_identifikace_podpisu';
        $this->code = 'col-prectenidokumentu';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $distribuovanyTyp = isset($rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX]) ? $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] : 0;

        //typ dokumentu neni distribuovany
        if ($distribuovanyTyp == 0)
            return $result;

//TODO: u dokumentu ktere maji zadane potvrdit do zobrazit datum a neumoznit potvrzeni po tomto datu (doplnit kontrolu do controlleru!)
        //pokud je dokument precteny, zobrazi identifikaci podpisu
        if ($rowData[9] == 1) {
            $result .= $rowData[11];
        }
        //pokud je dokument distribuovany = je nastaven datumCas u vzkazu, zobrazi odkaz na potrvrzeni precteni
        //Potvrdit je mozno jen dokument distribuovany prihlasene osobe.
        elseif ($rowData[10] && ($rowData[13] == User::getRC())) {

            $result .= '<a href="?menuid=dokumenty&potvrzeniprecteniform=1&formaction=edit&vzkazid=' . $rowData[6] . '&dokumentid=' . $rowData[0] .
                '&platnostod=' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[7])) . '">';
            $result .= ronText('potvrzeni_precteni');
            $result .= '</a>';
        }

        if ($rowData[14]) {
            $class = '';
            if ($rowData[14] && (DateUtils::startOfTheDay(time()) >= $this->database->stringToTimestamp($rowData[14]))) {
                $class = 'class="distribuovany_po_terminu"';
            }
            if ($result) {
                $result .= ', ';
            }
            $result .= '<span ' . ($class ?: '') . '>' . ronText('dokumenty_potvrdit_do') . ': ' .
                DBUtils::getFormatedValue($this->database, 'finedate', $rowData[14])  .'</span>';
        }
        return $result;
    }
}


class DokumentyOsobyBrowser extends RonPortalBrowser
{
    const ZOBRAZOVAT_KATEGORIE_DOKUMENTU = "'1','2','50','51'";
    const DISTRIBUOVANY_TYP_INDEX = 8;
    const DOKUMENT_PRECTENO_INDEX = 9;
    

    var $downloadLink = '';

    var $deleteType = 0;
    var $deleteLink = null;

    var $datumField = null;

    var $prvniDokument;  // pomocne promenne pro zobrazeni sekci
    var $nadpisSekce;

    var $downloadField;

    /** @var  DokumentyOsobyForm */
    var $filterForm;

    var $defaultSelectPart;
    var $distribuovanePrvni = false;

    /**
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'dokumenty'); // odkaz pro download se nastavuje az u radku, protoze se zobrazuje vice typu dokumentu (osoby, prac.mista, rozdeleni)

        $this->code = 'browserdokumenty';

        $this->title = ronText('Dokumenty');
        $this->tableId = 'Dokumenty';
        $this->sqlFromPart = 'Dokument, TypDokumentu';
        $this->sqlWherePart = 'Dokument.Typ = TypDokumentu.Kod';
        $this->sqlSelectPart =  //U distribuovanych se doplnuji dalsi sloupce!
            'Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru, TypDokumentu.DefinicePrava';
        $this->defaultSelectPart = $this->sqlSelectPart;

        $this ->primaryField = 0;
        $this->distinct = true; //U distribuovanych se distinct nepouziva!

        $this->cssClass = 'browser minwidth';

        $this->prvniDokument = true;

        $this->defaultSort = 1; // podle datumu
        $this ->datumField = $this ->addField(
            ['sqlFieldName' => 'Dokument.Datum',     'sqlIndex' => 1, BROWSER_FIELD_HEADER_TEXT => 'Date', 'format' => 'date', 'ordering' => 1, BROWSER_FIELD_CLASS_NAME => 'dokument_datum', BROWSER_FIELD_CODE => 'col-datum'] );

        $this ->addFields( [
            ['sqlFieldName' => 'TypDokumentu.Popis', 'sqlIndex' => 2, BROWSER_FIELD_HEADER_TEXT => 'Typ_dokumentu',   'ordering' => 1, BROWSER_FIELD_CLASS_NAME => 'dokument_typ', BROWSER_FIELD_CODE => 'col-typ'],
            ['sqlFieldName' => 'Dokument.Popis',     'sqlIndex' => 3, BROWSER_FIELD_HEADER_TEXT => 'Popis', 'ordering' => 1, BROWSER_FIELD_CLASS_NAME => 'dokument_popis', BROWSER_FIELD_CODE => 'col-popis']
        ]);

        // odkaz na stahnuti souboru
        $this->downloadField = new DokumentOsobyLinkField($this ->database,
            ['sqlFieldName' => 'Dokument.JmenoSouboru', 'sqlIndex' => 4, 'link' => $this ->downloadLink, BROWSER_FIELD_HEADER_TEXT => 'Dokument_nazev', BROWSER_FIELD_ORDERING => 1, 'col-nazev'],
            3);
        $this->addField($this->downloadField);

        // datum distribuce s odkazem na prehled distribuce podle prav
        $this->addField(new DistribuovanyDokumentField($database));

        // u prectenych identifikace podpisu, u neprectenych odkaz na potvrzeni precteni
        $this->addField(new PrectenyDokumentField($database));

        $this ->browserUrl = '?menuid=dokumenty&';
    }

    /**
     * @throws Exception
     */
    function setFilterForm()
    {
        $this ->filterForm = new DokumentyOsobyForm($this->database, null);
        $this ->filterForm ->nastavAkciGet('dokumenty');
        $this ->filterForm ->loadData();
    }

    function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key == 'distribuovaneprvni') {
            $this ->distribuovanePrvni = (int)$valueText;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    /**
     * Dokumenty prirazene osobe nebo pracovnimu pomeru
     * @throws Exception
     */
    function showDokumentyOsoby()
    {
        $this->downloadField->link = '?menuid=dokumenty&dokumentosoby=1';

        $params = $this->database->createParameters();

        $this->sqlFromPart = 'Dokument INNER JOIN TypDokumentu ON (Dokument.typ  =  TypDokumentu.kod)
            INNER JOIN DokumentyOsoby ON (Dokument.id  =  DokumentyOsoby.id)';
        $this->sqlWherePart = new SqlStatement(
            ' (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            " AND (DokumentyOsoby.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) . ")" .
            " AND TypDokumentu.KategorieDokumentu IN (" . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ") ".
            " AND TypDokumentu.Kod >= 0" . //-1 = vyplatni pasky - jsou ve vlastnim browseru
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )', $params);


        $sqltext = $this ->getSQL('');
        $this ->showDokumentySQL($sqltext, ronText('Dokumenty_osobni'));

        echo '<tr><td colspan="' . $this ->columnCount . '">';
        echo "<div class='normaltext'><a href='#' onclick='showDokumentOsobyDialog(" . "" . RonPortalSession::vybranaOsoba() . ");'>" . ronText('Vlozit_novy_dokument') . "</a></div>";
        $form = new NovyDokumentOsobyForm($this->database, RonPortalSession::vybranaOsoba());
        $form->render();
        echo '</td></tr>';
    }


    /**
     * Dokumenty k rozdeleni osoby
     * @throws Exception
     */
    function showDokumentyRozdeleni()
    {
        $this ->downloadField ->link = '?menuid=dokumenty&dokumentrozdeleni=1';

        $params = $this->database->createParameters();

        $this ->sqlFromPart = 'Dokument INNER JOIN TypDokumentu ON (Dokument.typ  =  TypDokumentu.kod)
            INNER JOIN DokumentyRozdeleni ON (Dokument.Id = DokumentyRozdeleni.Id)
            INNER JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod)';

        $this ->sqlWherePart = new SqlStatement(
            '  (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni <> -1 ' .    // vynecha pracovni mista
            " AND TypDokumentu.KategorieDokumentu IN (" . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ") ".
            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) . ")" .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )', $params);

        $sqltext = $this ->getSQL();
        $this ->showDokumentySQL($sqltext, ronText('Dokumenty_rozdeleni'));
    }


    /**
     * Dokumenty pripojene k pracovnimu mistu
     * @throws Exception
     */
    function showDokumentyPracMista()
    {
        $this ->downloadField->link = '?menuid=dokumenty&dokumentpmista=1';

        $params = $this->database->createParameters();

        $this ->sqlFromPart = 'Dokument INNER JOIN TypDokumentu ON (Dokument.typ  =  TypDokumentu.kod)
            INNER JOIN DokumentyRozdeleni ON (Dokument.Id = DokumentyRozdeleni.Id)
            INNER JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod)  ';
        $this ->sqlWherePart = new SqlStatement(
            'Dokument.Id = DokumentyRozdeleni.Id ' .
            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = -1 ' .    // jen pracovni mista
            " AND TypDokumentu.KategorieDokumentu IN (" . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ") ".
            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) . ")" .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(time())) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )', $params);

        $sqltext = $this ->getSQL();
        $this ->showDokumentySQL($sqltext, ronText('Dokumenty_pracovni_misto'));
    }

    /**
     * @throws Exception
     */
    function showDistribuovaneDokumenty()
    {
        $this ->downloadField->link ='?menuid=dokumenty&distribuovanedokumenty=1';

        $this->distinct = false;

        $params = $this->database->createParameters();

        $this->sqlSelectPart .= ', DokumentVzkazu.IdVzkazu, DokumentVzkazu.PlatnostOd, TypDokumentu.Distribuovany, DokumentVzkazu.Precteno, Vzkaz.DatumCas, ' . //6 - 10
                $this->database->castWideMemo('DokumentVzkazu.IdentifikacePodpisu') . ', ' . $this->database->castWideMemo('DokumentVzkazu.Vyjadreni') . ', ' . // 11 - 12
                'Vzkaz.RCPrijemce, VerzeDokumentu.PotvrditDo'; //13 - 14

        $this->sqlFromPart =
            'Vzkaz JOIN DokumentVzkazu ON (Vzkaz.id=DokumentVzkazu.idVzkazu)
            JOIN VerzeDokumentu ON (VerzeDokumentu.id = DokumentVzkazu.idDokumentu AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd)
            JOIN Dokument ON (Dokument.id = VerzeDokumentu.id) JOIN TypDokumentu ON (Dokument.typ  =  TypDokumentu.kod)';

        $this->sqlWherePart = new SqlStatement(
            " RCPrijemce= " . $params->addParameter($this->filterForm->vybranaOsobaRC), $params);

        $sqltext = $this->getSQL();
        $this->showDokumentySQL($sqltext, ronText('distribuovane_dokumenty'));
    }



    /**
     * @throws Exception
     */
    function renderData()
    {
        if ($this->distribuovanePrvni) {
            //metoda rusi pouziti distinctu a meni SELECT
            $this ->showDistribuovaneDokumenty();
            $this->distinct = true;
            $this->sqlSelectPart = $this->defaultSelectPart;
        }

        // Osobni dokumenty
        $this ->showDokumentyOsoby();
        $this ->showDokumentyPracMista();
        $this ->showDokumentyRozdeleni();

        if (!$this->distribuovanePrvni) {
            //metoda rusi pouziti distinctu
            $this->showDistribuovaneDokumenty();
        }
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //pokud je u typu dokumentu prirazen typ prava
        if ($values[5]) {
            $prava = Prava::vratPrava($values[5]);
            if (!$prava->lzeZobrazitRC($this->filterForm->vybranaOsobaRC)) return false;
        }
        return true;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        if ($this->prvniDokument && $this->nadpisSekce) {
            echo '<tr><td colspan="' . $this->columnCount . '"></br><b>' . $this->nadpisSekce . '</b></td></tr>';
            $this ->prvniDokument = false;
        }

        parent::renderLine($values, $className, $rowNumber, $edit);

        if (isset($values[12])) {
            echo '<tr class="' . $className . '"><td></td><td colspan="' . ($this->columnCount - 1) . '"><em>' . ronText('vyjadreni') . ': &nbsp;'
                . StringUtils::longTextToHtml($values[12]) . '</em></td></tr>';
        }
    }


    /**
     * @param SqlStatement $sql
     * @param string $nadpisSekce
     * @throws Exception
     */
    function showDokumentySQL($sql, $nadpisSekce)
    {
        $this->prvniDokument = true;
        $this->nadpisSekce = $nadpisSekce;

        MessageManager::addDebugLog('Browser sekce ' . $nadpisSekce . $sql->sqlText);

        $cursor = $this ->database ->select($sql);
        if ($cursor) {
            $this ->renderCursor($cursor);
        } else {
            echo '<tr><td colspan="' . $this ->columnCount . '">';
            MessageManager::addDbErrorMessage($this->database,$sql);
            echo '</td></tr>';
        }

        if (!$this ->prvniDokument) $this ->renderSeparator();
    }
    
}
