<?php
/** UTF8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 13.3.2017
 * Time: 11:03
 */

namespace Ron\Personalistika;

use Ron\Browser;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\HttpUtils;
use function Ron\ronText;

require_once('ron/browser.php');

class SmazaniTypovehoMistaField extends BrowserField
{
    private $osobaId;
    private $kodRozdeleniIndex;
    private $popisRozdeleniIndex;
    private $platnostOdIndex;

    /**
     * SmazaniTypovehoMistaField constructor.
     * @param null|Database $database
     * @param int $osobaId
     * @param $kodRozdeleniIndex
     * @param $popisRozdeleniIndex
     * @param $platnostOdIndex
     */
    public function __construct($database, $osobaId, $kodRozdeleniIndex, $popisRozdeleniIndex, $platnostOdIndex)
    {
        parent::__construct($database);

        $this->osobaId = $osobaId;
        $this->kodRozdeleniIndex = $kodRozdeleniIndex;
        $this->popisRozdeleniIndex = $popisRozdeleniIndex;
        $this->platnostOdIndex = $platnostOdIndex;
    }

    /** Zobrazi odkaz pro smazani prirazeneho typoveho mista.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        require_once('ron/httputils.php');

        return GuiUtils::changeLocationQuest(
                '<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('Zrusit_zaznam') . '"></i>',
                sprintf(ronText('Dovednosti_smazat_misto'), $rowData[$this->popisRozdeleniIndex]),
                '?menuid=dovednosti&action=zrusittpm&osoba=' . $this->osobaId . '&misto=' . HttpUtils::urlEncode($rowData[$this->kodRozdeleniIndex]) .
                '&platnostod=' .  DBUtils::getFormatedValue($this->database, 'date', $rowData[$this->platnostOdIndex])
            );
    }
}

class UkonceniTypovehoMistaField extends BrowserField
{
    private $osobaId;
    private $kodRozdeleniIndex;
    private $popisRozdeleniIndex;
    private $platnostOdIndex;

    /**
     * SmazaniTypovehoMistaField constructor.
     * @param null|Database $database
     * @param int $osobaId
     * @param $kodRozdeleniIndex
     * @param $popisRozdeleniIndex
     * @param $platnostOdIndex
     */
    public function __construct($database, $osobaId, $kodRozdeleniIndex, $popisRozdeleniIndex, $platnostOdIndex)
    {
        parent::__construct($database);

        $this->osobaId = $osobaId;
        $this->kodRozdeleniIndex = $kodRozdeleniIndex;
        $this->popisRozdeleniIndex = $popisRozdeleniIndex;
        $this->platnostOdIndex = $platnostOdIndex;
    }

    /** Zobrazi odkaz pro ukonceni prirazeneho typoveho mista.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        require_once('ron/httputils.php');

        return "<a href='#' class='ukonceniTPM' data-tpm-kod='" . $rowData[$this->kodRozdeleniIndex] . "'
                    data-tpm-platnost-od='" . DBUtils::getFormatedValue($this->database, 'date', $rowData[$this->platnostOdIndex]) . "'>"
                . ronText('Dovednosti_ukoncit_tpm') . "</a>";
    }
}

class TypovePracovniMistoOsobyBrowser extends Browser
{

    private $idOsoby;

    /**
     * TypovePracovniMistoOsobyBrowser constructor.
     * @param Database $database
     * @param string $url url na ktere je browser zobrazen
     * @param int $idOsoby id PP osoby
     * @param int $typRozdeleniTypovePracovniMisto
     * @throws DBException
     */
    public function __construct($database, $url, $idOsoby, $typRozdeleniTypovePracovniMisto)
    {
        parent::__construct($database);

        $this ->idOsoby = $idOsoby;

        $this->tableId = 'tpm';

        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . (int)$this->idOsoby);

        $this->title = ronText('Dovednosti_typova_mista_zamestnance');

        $this->browserUrl = $url . '&';

        $this->sqlSelectPart =
            "RozdeleniOsob.Rozdeleni, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, PracovniPomer.ID, ". //0 - 4
            "RozdeleniOsob.Poznamka";

        $this->sqlFromPart =
            "RozdeleniOsob 
                JOIN PracovniPomer ON ((PracovniPomer.RC = RozdeleniOsob.RC) AND (PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo) 
                    AND (PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)) 
                JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni)";

        $this->sqlWherePart =
            " RozdeleniOsob.KodTypuRozdeleni = " . $typRozdeleniTypovePracovniMisto .
            " AND RozdeleniOsob.RC = '" . $this->database->addSlashes($osobaRC) . "'" .
            " AND PracovniPomer.DatumNastupu <= " . $this->database->timestampToString(DateUtils::today()) .
            " AND (PracovniPomer.DatumUkonceni >= " . $this->database->timestampToString(DateUtils::today()) . " OR PracovniPomer.DatumUkonceni IS NULL)";



        $this->addFields([
            [BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'RozdeleniOsob.Rozdeleni', BROWSER_FIELD_HEADER_TEXT => 'Kod',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis', BROWSER_FIELD_HEADER_TEXT => 'Popis',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'RozdeleniOsob.PlatnostOd', BROWSER_FIELD_HEADER_TEXT => 'Platnost_od',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'date'],

        ]);

        $this->defaultSort = count($this->fields);

        $this->addFields([
            [BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'RozdeleniOsob.PlatnostDo', BROWSER_FIELD_HEADER_TEXT => 'Platnost_do',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'date'],
            [BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'RozdeleniOsob.Poznamka', BROWSER_FIELD_HEADER_TEXT => 'Poznamka',
                BROWSER_FIELD_ALIGN => 'left'
            ]
        ]);

        //$this->addField(new SmazaniTypovehoMistaField($this->database, $this->idOsoby, 0, 1, 2));
        $this->addField(new UkonceniTypovehoMistaField($this->database, $this->idOsoby, 0, 1, 2));
    }

}