<?php

/**
 * UTF-8
 */

namespace Ron\Personalistika;

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\App;
use Ron\RonPortal\PredavaciProcesyUtils;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

require_once('ron/browser.php');
require_once('adsapp/predavaciproces.php');
require_once('person/hodnoceni_filter.php');


/**
 * Odkaz na zobrazeni hodnoceni - podle prav
 * Class OdkazHodnoceniField
 * @package Ron\Personalistika
 */
class OdkazHodnoceniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (($rowData[1] !== '') && ((int)$rowData[8] !== 0) && ((int)$rowData[8] !== 4)) // jestli je vyplneno RC hodnotitele a u hodnoceni
            //  jiz byl zmenen stav z 0 (nezpracovano) nebo 4 (vraceno - nezpracovano)
        {
            // hodnoceni uz nekdo vyplnil - bude zde odkaz na zobrazeni
            $link = "?menuid=hodnoceni&record={$rowData[0]}";
            global $STR_ZobrazitHodnoceni; // kompat.
            $text = $STR_ZobrazitHodnoceni ?: ronText('zobrazit_hodnoceni');
        } else {
            global $STR_VyplnitHodnoceni; // kompat.
            $link = "?menuid=hodnoceni&hodnoceniaction=editujhodnoceni&record={$rowData[0]}";
            $text = $STR_VyplnitHodnoceni ?: ronText('vyplnit_hodnoceni');
        }

        return '<A href="' . $link . '">' . $text . "</A>";
    }
}


/**
 * Odkaz na zobrazeni hodnoceni - podle prav
 * Class VyberCheckboxField
 * @package Ron\Personalistika
 */
class VyberCheckboxField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return "<input type='checkbox'  name='hodnocenicheckbox_{$rowData[0]}' />";
    }
}


/**
 * Datum pro zobrazeni hodnoceni - od kdy
 * Class DatumZobrazeniField
 * @package Ron\Personalistika
 */
class DatumZobrazeniField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->align = 'right';
        global $STR_ZobrazitOd; // kompat.
        $this->header = $STR_ZobrazitOd ?: ronText('Zobrazit_hodnoceni_od');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if  ($rowData[19] != 1) {
            // je za limitni urovni
            $date = DateUtils::incDay($this ->database ->stringToTimestamp($rowData[7]), $rowData[20]);

            return date('j.n.Y', $date);
        }

        return '';
    }
}



class HodnoceniBrowser extends Browser
{
    //var $f_pravaeditace;
    //var $f_pravaprohlizeni;

    var $hromadneSchvaleni; // priznak, jestli se ma zobrazit hromadne schvalovani

    var $fieldRozdeleni;

    /** @var  HodnoceniFilterForm */
    var $filterForm;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        global $STR_PrehledHodnoceni; // kompat.
        $this->title = $STR_PrehledHodnoceni ?: ronText('Prehled_hodnoceni');

        $this->code = 'browserseznamhodnoceni';
        $this->tableId = 'browserseznamhodnoceni';
        $this->browserUrl = '?menuid=hodnoceni&';

        $this->sqlSelectPart =
            'PripraveneTemaHodnoceni.ID, PripraveneTemaHodnoceni.HodnotilRC, PripraveneTemaHodnoceni.HodnocenyRC, ' . // 0 - 2
            'TemaHodnoceni.PopisTematu, Osoba.Jmeno, Osoba.Prijmeni, ' . // 3 - 5
            'VysledekHodnoceni.PocatekObdobi, VysledekHodnoceni.KonecObdobi, PripraveneTemaHodnoceni.Stav, ' . // 6 - 8
            'HodnoceniOsoby.Znamka, PripraveneTemaHodnoceni.Hodnotici, PripraveneTemaHodnoceni.DatumHodnoceni, ' . // 9 - 11
            'HodnoceniOsoby.KodProcesu, HodnoceniOsoby.PocatecniUroven, HodnoceniOsoby.PoradiPredavani, HodnoceniOsoby.UrovenPredavani, PoradiPredavani.KodUrovne, ' . // 12-16
            'HodnoceniOsoby.IdVysledku, TemaHodnoceni.XMLProZobrazeni, PoradiPredavani.LimitniUroven, '. // 17,18,19
            '(SELECT PresahProVlastniHodnoceni FROM UcelHodnoceni WHERE Kod = VysledekHodnoceni.KodUcelu), PripraveneTemaHodnoceni.IndividualniPoradi '; //20, 21

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            // pripoji pole pro zobrazeni rozdeleni
            $this->sqlSelectPart .= ', RozdeleniOsob.Rozdeleni, Rozdeleni.Popis'; // 22,23
        }


        $this->sqlFromPart =
            ' ( ( ( ( PripraveneTemaHodnoceni LEFT OUTER JOIN HodnoceniOsoby ' .
            '         ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
            '       ) ' .
            '       LEFT OUTER JOIN PoradiPredavani ' .
            '       ON PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu ' .
            '          AND PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven ' .
            '          AND PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani ' .
            '     ) ' .
            '     LEFT OUTER JOIN Osoba ' .
            '     ON Osoba.RC = PripraveneTemaHodnoceni.HodnocenyRC ' .
            '   )' .
            '   LEFT OUTER JOIN VysledekHodnoceni ' .
            '   ON VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku ' .
            ' )' .
            ' INNER JOIN TemaHodnoceni ' .
            ' ON TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
            '    AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ';


        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            // vybere rozdeleni pouze pro HPP platny v posledni den hodnoceneho obdobi
            $this->sqlFromPart =
                '( ( (' . $this->sqlFromPart .
                '    ) ' .
                '    LEFT OUTER JOIN PracovniPomer ' .
                '    ON PracovniPomer.RC = Osoba.RC ' .
                "    AND PracovniPomer.TypPracovnihoPomeru = 'HPP' " .
                '    AND PracovniPomer.DatumNastupu <= VysledekHodnoceni.KonecObdobi ' . // platny na konci hodnoceneho obdobi
                '    AND (PracovniPomer.DatumUkonceni >= VysledekHodnoceni.KonecObdobi  OR  PracovniPomer.DatumUkonceni IS NULL)' .
                '  ) ' .
                '  LEFT OUTER JOIN RozdeleniOsob ' .
                '  ON RozdeleniOsob.RC = PracovniPomer.RC ' .
                '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '  AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                '  AND RozdeleniOsob.PlatnostOd <= VysledekHodnoceni.KonecObdobi ' .
                '  AND (RozdeleniOsob.PlatnostDo >= VysledekHodnoceni.KonecObdobi  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        // ---------------------------------------------------------------------
        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $seznamUrovni = PredavaciProcesyUtils::seznamUrovniPredavaniUzivatele($database);
        $this->hromadneSchvaleni = PredavaciProcesyUtils::muzeHromadneSchvalovatHodnoceni($seznamUrovni);

        $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        // vynechani hodnoceni skoleni
        $this->sqlWherePart =
            'NOT EXISTS(SELECT HodnoceniSkoleni.DruhHodnoceni FROM HodnoceniSkoleni ' .
            '   WHERE PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni ' .
            '     AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo)';

        // vybere zaznamy, na ktere ma uzivatel pravo 
        $this->sqlWherePart .=
            ' AND ((PripraveneTemaHodnoceni.IndividualniPoradi = 0 OR' .    // POZOR! tato podminka to muze zpomalit (testovano SE)!
            " PripraveneTemaHodnoceni.HodnotilRC = '" . $database ->addSlashes(User::getRC()) . "')" . // je nastaveny "individualni" hodnotitel
            'AND (' .
            " PripraveneTemaHodnoceni.HodnotilRC = '" . $database ->addSlashes(User::getRC()) . "'";   // vyplnil jsem "ja"

        $this->sqlWherePart .=
            " OR ( PoradiPredavani.Vlastni = 1   AND  PripraveneTemaHodnoceni.HodnocenyRC = '" . $database ->addSlashes(User::getRC()) . "') ";


        // vybere hodnoceni zamestnancu, na ktereho mam pravo (podle pracovniho mista a hierarchie)
        $this->sqlWherePart .=
            ' OR ' .
            ' (PoradiPredavani.Vlastni = 0  AND ' .
            '  PoradiPredavani.NekontrolovatHierarchii = 0 AND ' .
            '( EXISTS ' .
            '  ( SELECT PracovniPomer.Id ' .
            '  FROM PracovniPomer, RozdeleniOsob R_PracovniMisto, PMProUrovne, ' .
            '       RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny, PredavaciProces ' .
            " WHERE PracovniPomer.RC = '" . $database ->addSlashes(User::getRC()) . "'" .
            // jen "moje" urovne predavani podle pracovniho mista
            '   AND R_PracovniMisto.RC = PracovniPomer.RC ' .
            '   AND R_PracovniMisto.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_PracovniMisto.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND R_PracovniMisto.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (R_PracovniMisto.PlatnostDo >= ' . $database ->timestampToString(time()) . '  OR  R_PracovniMisto.PlatnostDo IS NULL) ' .
            '   AND R_PracovniMisto.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND R_PracovniMisto.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND HodnoceniOsoby.UrovenPredavani = PMProUrovne.KodUrovne ' .
            '   AND PredavaciProces.Kod = PoradiPredavani.KodProcesu' .
            // jen pordrizena rozdeleni (vzhledem k PP!)
            '   AND R_Ja.RC = PracovniPomer.RC ' .
            '   AND R_Ja.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_Ja.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '   AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '   AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '   AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            "   AND (R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni OR (PredavaciProces.KodTypuRozdeleni IS NULL AND R_JA.KodTypuRozdeleni = $kodTypuRozdeleni))" .
            "   AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni" .
            '   AND R_Hodnoceny.RC = VysledekHodnoceni.HodnocenyRC' .
            '   AND R_Hodnoceny.OsobniCislo = VysledekHodnoceni.HodnocenyOsobniCislo' .
            '   AND R_Hodnoceny.DatumNastupu = VysledekHodnoceni.HodnocenyDatumNastupu' .
            '   AND R_Ja.PlatnostOd <= VysledekHodnoceni.KonecObdobi' .
            '   AND (R_Ja.PlatnostDo >= VysledekHodnoceni.PocatekObdobi  OR  R_Ja.PlatnostDo IS NULL)' .
            '   AND R_Hodnoceny.PlatnostOd <= VysledekHodnoceni.KonecObdobi' .
            '   AND (R_Hodnoceny.PlatnostDo >= VysledekHodnoceni.PocatekObdobi  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
            ')' .
            '))';

        // vybere hodnoceni zamestnancu, na ktereho mam pravo (podle pracovniho mista bez hierarchie)
        $this->sqlWherePart .=
            ' OR ' .
            ' (PoradiPredavani.Vlastni = 0  AND ' .
            '  PoradiPredavani.NekontrolovatHierarchii = 1 AND ' .
            '( EXISTS ' .
            '  ( SELECT PracovniPomer.Id ' .
            '  FROM PracovniPomer, RozdeleniOsob R_PracovniMisto, PMProUrovne ' .
            " WHERE PracovniPomer.RC = '" . $database ->addSlashes(User::getRC()) . "'" .
            // jen "moje" urovne predavani podle pracovniho mista
            '   AND R_PracovniMisto.RC = PracovniPomer.RC ' .
            '   AND R_PracovniMisto.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_PracovniMisto.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND R_PracovniMisto.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (R_PracovniMisto.PlatnostDo >= ' . $database ->timestampToString(time()) . '  OR  R_PracovniMisto.PlatnostDo IS NULL) ' .
            '   AND R_PracovniMisto.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND R_PracovniMisto.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND HodnoceniOsoby.UrovenPredavani = PMProUrovne.KodUrovne '.
            ')' .
            '))';

        $this->sqlWherePart .=
            '))';

        $this->defaultSort = 1;
        $this->primaryField = 0;

        if (App::getDebugMode()) {
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 0,  BROWSER_FIELD_SQL_NAME => 'PripraveneTemaHodnoceni.ID', BROWSER_FIELD_HEADER => 'DEBUG> Id', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 17, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.IDVysledku',  BROWSER_FIELD_HEADER => 'DEBUG> IdVysledku', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1] );
        }

        if ($this->hromadneSchvaleni) {
            $this->addField( new VyberCheckboxField() );
        }

        $this->addField( [BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'TemaHodnoceni.PopisTematu', BROWSER_FIELD_HEADER => ronText('Popis'), BROWSER_FIELD_LINK => "?menuid=hodnoceni", BROWSER_FIELD_ORDERING => 1] );

        // rozdeleni
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $coZobrazit = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT);
            if ($coZobrazit == 1 || $coZobrazit == 2) {
                // jen kod | kod+popis
                $f = $this->addField( [
                    BROWSER_FIELD_SQL_INDEX => 22, BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Kod',
                    BROWSER_FIELD_HEADER => RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS), BROWSER_FIELD_ORDERING => 1] );
                $f ->sortSQL = 'Rozdeleni.Kod, Osoba.Prijmeni';
            }

            if ($coZobrazit == 0 || $coZobrazit == 2) {
                // jen popis | kod+popis
                $f = $this->addField( [
                    BROWSER_FIELD_SQL_INDEX => 23, BROWSER_FIELD_SQL_NAME => 'Rozdeleni.Popis',
                    BROWSER_FIELD_HEADER => RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS), BROWSER_FIELD_ORDERING => 1] );
                $f ->sortSQL = 'Rozdeleni.Popis, Osoba.Prijmeni';
            }
        }

        $this->addField( [BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_HEADER => ronText('Jmeno'), BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_HEADER => ronText('Prijmeni'), BROWSER_FIELD_ORDERING => 1] );

        // Obdobi hodnoceni (od-do)
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'VysledekHodnoceni.PocatekObdobi', BROWSER_FIELD_HEADER => ronText('Datum_od'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_SQL_NAME => 'VysledekHodnoceni.KonecObdobi', BROWSER_FIELD_HEADER => ronText('Datum_do'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1] );
        
        $this->addField( new DatumZobrazeniField($database) );
        
        global $STR_H_0; // kompat.
        global $STR_H_1;
        global $STR_H_2;
        global $STR_H_3;
        global $STR_H_4;
        global $STR_H_5;
        global $STR_StavHodnoceni; // kompat.
        $this->addField( [
                BROWSER_FIELD_TYPE => 'select', BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'PripraveneTemaHodnoceni.Stav',
                BROWSER_FIELD_HEADER => $STR_StavHodnoceni ?: ronText('Stav_hodnoceni'),
                BROWSER_FIELD_OPTIONS => [$STR_H_0 ?: ronText('Stav_hodnoceni_0'), $STR_H_1 ?: ronText('Stav_hodnoceni_1'), $STR_H_2 ?: ronText('Stav_hodnoceni_2'),
                    $STR_H_3 ?: ronText('Stav_hodnoceni_3'), $STR_H_4 ?: ronText('Stav_hodnoceni_4'), $STR_H_5 ?: ronText('Stav_hodnoceni_5')],
                BROWSER_FIELD_ORDERING => 1] );

        if (App::getDebugMode()) {
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 18, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.XMLProZobrazeni', BROWSER_FIELD_HEADER => 'DEBUG> Id XML', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'PripraveneTemaHodnoceni.Stav', BROWSER_FIELD_HEADER => 'DEBUG> Stav', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1] );
        }

        // navrzene znamky vidi jen ti, kteri mohou hromadne schvalovat
        if ($this->hromadneSchvaleni) {
            global $STR_Znamka; // kompat.
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 9, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.Znamka', BROWSER_FIELD_HEADER => $STR_Znamka ?: ronText('Znamka'), BROWSER_FIELD_FORMAT => 'number_2', BROWSER_FIELD_ORDERING => 1] );
        }

        $this->addField( new OdkazHodnoceniField() ); // odkaz zobrazit/vyplnit

        $this->addField( [BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_SQL_NAME => 'PripraveneTemaHodnoceni.Hodnotici', BROWSER_FIELD_HEADER_TEXT => 'Hodnotil', BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_SQL_NAME => 'PripraveneTemaHodnoceni.DatumHodnoceni', BROWSER_FIELD_FORMAT => 'datetime', BROWSER_FIELD_ORDERING => 1] );

        $this->addField( [BROWSER_FIELD_SQL_INDEX => 14, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.PoradiPredavani', BROWSER_FIELD_HEADER_TEXT => 'Poradi', BROWSER_FIELD_ORDERING => 1] );

        if (App::getDebugMode()) {
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 12, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.KodProcesu', BROWSER_FIELD_HEADER => 'Proces', BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 13, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.PocatecniUroven', BROWSER_FIELD_HEADER => StringUtils::fromUTF8('Poč.úroveň'), BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 15, BROWSER_FIELD_SQL_NAME => 'HodnoceniOsoby.UrovenPredavani', BROWSER_FIELD_HEADER => StringUtils::fromUTF8('Kat.PM hodnocení'), BROWSER_FIELD_ORDERING => 1] );
            $this->addField( [BROWSER_FIELD_SQL_INDEX => 16, BROWSER_FIELD_SQL_NAME => 'PoradiPredavani.KodUrovne', BROWSER_FIELD_HEADER => 'Kat.PM dle procesu', BROWSER_FIELD_ORDERING => 1] );
        }

        $this->maxPageRows = 100;
    }


    /**
     * @param string $sqlFilter
     * @throws \Exception
     */
    function load($sqlFilter = '')
    {
        if (isset($_GET['stavhodnoceni'])) {
            $_SESSION['filtrstavhodnoceni'] = (int)$_GET['stavhodnoceni'];
        }

        $stav = HodnoceniFilterForm::getStavHodnoceni();
        if ( !isset($stav) ) {
            $_SESSION['filtrstavhodnoceni'] = -1;  // vychozi hodnota - neukoncene
        }

        $this ->filterForm = new HodnoceniFilterForm($this ->database, 'hodnoceni', false);
        $this ->filterForm->zobrazitZnamky = $this->hromadneSchvaleni;
        $this ->filterForm ->loadData();

        parent::load($sqlFilter);
    }


    function renderFilterStav()
    {
        global $STR_H_0; // kompat.
        global $STR_H_1;
        global $STR_H_2;
        global $STR_H_3;
        global $STR_H_5;

        $stav = (int)HodnoceniFilterForm::getStavHodnoceni();

        $data = array(
            -1 => ronText('Neukonceno'),
            0 => $STR_H_0 ?: ronText('Stav_hodnoceni_0'),
            1 => $STR_H_1 ?: ronText('Stav_hodnoceni_1'),
            2 => $STR_H_2 ?: ronText('Stav_hodnoceni_2'),
            3 => $STR_H_3 ?: ronText('Stav_hodnoceni_3'),
            5 => $STR_H_5 ?: ronText('Stav_hodnoceni_4'),
            100 => ronText('Stav_hodnoceni_vse'));

        LinksList::renderLinks($data, $stav, '?menuid=hodnoceni&stavhodnoceni=');
    }


    function renderFilter()
    {
        $this->renderFilterStav();
        echo "<BR>\n";

        $this ->filterForm ->render();

        if ($this->hromadneSchvaleni) {
            echo "<br/>\n";
            echo '<input type="button" class="button" name="vybratvse" id="vyberVsechnaHodnoceni" value="' . ronText('Vybrat_vse') . '" /> ';
            echo '<input type="button" class="button" name="vybratvse" id="zrusVyberHodnoceni" value="' . ronText('Zrusit_vyber') . '" /> ';

            echo '<span style="float:right"> <input type="button" class="button" name="odeslathodnoceni" value="' . ronText('Odeslat_vybrane') . '"  OnClick="hromadneUzavreniHodnoceni()" />  </span>';
            echo "\n";
        }
    }


    function getSQLFilter()
    {
        $result = '';

        $stav = (int)HodnoceniFilterForm::getStavHodnoceni();

        // ---------------------------------------------------------------------
        // filtr pro stav
        switch ($stav) {
            case 1 : // rozpracovano
            case 2 : // ukonceno
            case 3 : // ukonceno - vraceno
            case 5 : // vyrazene
                $result = ' PripraveneTemaHodnoceni.Stav = ' . $stav;
                break;

            case 0 :
                $result = ' (PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 4) ';
                break;

            case -1 :
                // neukoncene = pripravene + rozpracovane + vracene k prepracovani
                $result = ' (PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 1  OR PripraveneTemaHodnoceni.Stav = 4) ';
                break;

            // 100 = vse
        }

        if (HodnoceniFilterForm::getJenRozdeleniHodnoceni() && HodnoceniFilterForm::getRozdeleniHodnoceni()) {
            if ($result) $result .= '  AND  ';

            $result .=
                ' EXISTS ' .
                '( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                '  FROM HierarchieRozdeleni H, RozdeleniOsob R ' .
                '  WHERE R.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
                '    AND R.RC = PripraveneTemaHodnoceni.HodnocenyRC' .
                '    AND H.TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
                "    AND H.KodNadrazene = '" . $this ->database ->addSlashes(HodnoceniFilterForm::getRozdeleniHodnoceni()) . "'" .
                '    AND H.KodPodrizene = R.Rozdeleni ' .
                '    AND R.PlatnostOd <= VysledekHodnoceni.KonecObdobi' .
                '    AND (R.PlatnostDo >= VysledekHodnoceni.PocatekObdobi  OR  R.PlatnostDo IS NULL)' .
                ')';
        }

        // pro nektere stavy je filtr na datum povinny, pro jine jde vypnout checkboxem
        if ($stav == 2 || $stav == 3 || $stav == 100 || RonPortalSession::getFiltrovatDatum()) {
            // Obdobi
            if (RonPortalSession::getDatumOd()) {
                if ($result) $result .= ' AND ';
                $result .= ' VysledekHodnoceni.KonecObdobi >= ' . $this ->database ->dateToString(RonPortalSession::getDatumOd());
            }

            if (RonPortalSession::getDatumDo()) {
                if ($result) $result .= ' AND ';
                $result .= ' VysledekHodnoceni.PocatekObdobi <= ' . $this ->database ->dateToString(RonPortalSession::getDatumDo());
            }
        }

        // "Jen ucel hodnoceni"
        if (HodnoceniFilterForm::getJenUcelHodnoceni() && HodnoceniFilterForm::getUcelHodnoceni()) {
            if ($result) $result .= '  AND  ';

            $result .= ' (VysledekHodnoceni.KodUcelu = ' . (int)HodnoceniFilterForm::getUcelHodnoceni() . ') ';
        }

        // "Jen tema hodnoceni"
        if (HodnoceniFilterForm::getJenTemaHodnoceni() && HodnoceniFilterForm::getTemaHodnoceni()) {
            if ($result) $result .= ' AND ';
            $result .= ' PripraveneTemaHodnoceni.TemaCislo = ' . (int)HodnoceniFilterForm::getTemaHodnoceni();
        }

        if ($this->hromadneSchvaleni && HodnoceniFilterForm::getFiltrovatZnamky()) {
            if ($result) $result .= ' AND ';
            $result .= ' HodnoceniOsoby.Znamka >= ' . (float)HodnoceniFilterForm::getZnamkaOd() .
                ' AND HodnoceniOsoby.Znamka <= ' . (float)HodnoceniFilterForm::getZnamkaDo();
        }

        //jen osoba
        if ($this->filterForm->f_showosoba && !RonPortalSession::vsechnyOsoby()) {
            if ($result) $result .= ' AND ';
            $result .= " PripraveneTemaHodnoceni.HodnocenyRC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "'";
        }

        return $result;
    }


    function enableLine($values)
    {
        //pokud je hodnoceni na urovni predavani, ktera ma limitni uroven ruznou od 1 (je za limitni urovni, kdy se nadefinovane otazky stanou otazkami na ktere odpovida)
        if ($values[19] != 1) {
            //a aktualni cas je vetsi nez konec obdobi hodnoceni + PresahProVlastniHodnoceni (pocet dnu), zobrazi se hodnoceni
            if (time() > DateUtils::incDay($this ->database ->stringToTimestamp($values[7]), $values[20])) {
                return true;
            } else {
                return false;
            }
        } else {
            return true;
        }
    }


    function render()
    {
        parent::render();

        echo '<script>$( function() { ronPortal.hromadneHodnoceni() } );</script>';
    }


}
