<?php

namespace Ron\Personalistika;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RozdeleniOsobyField;
use Ron\StringUtils;
use function Ron\ronText;

require_once('ron/browser.php');
require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/predavaciproces.php');
require_once('person/hodnoceni_filter.php');

class GenerovaniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return "<span class='generovani clickLink' data-typdialogu='generovani' data-id='$rowData[3]'>" .  ronText('Generovat') . "</span>";
    }

}

class ExpandField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /**
         * Zmeni class u elementu this, nebo u elementu se selektorem zadanym v data-selector (pokud je zadan)
         * - odstrani tridu data-remove-class
         * - pridat tridu data-add-class
         * - prepne tridu data-toggle-class
         */
        if ($rowData[4])
            return "<div class='hodnoceniToggle expand' data-expand-selector='.vysledkyhodnoceni_{$rowNumber}' data-hide-selector='.vysledkyhodnoceni_{$rowNumber}, .hodnoceni_radek_{$rowNumber}' data-toggle-class='hidden'>&nbsp;</div>";

        return '';
    }
}

class HodnoceniOsobBrowser extends Browser
{
    /** @var  HodnoceniFilterForm */
    var $filterForm;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->title = ronText('Hodnoceni_osob');

        $this->sqlFromPart =
            "Osoba";

        $this ->sqlSelectPart =
            'Osoba.Jmeno, Osoba.Prijmeni, Osoba.RC, ' . // 0..2
            '(SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC), ' . //3
            '(SELECT COUNT(ID) FROM VysledekHodnoceni WHERE HodnocenyRC = Osoba.RC)'; //4

        $this->defaultSort = 3;
        $this->primaryField = 0;

        // rozdeleni
        $fRozdeleni = new RozdeleniOsobyField($database, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));
        $fRozdeleni ->nastavIndexPole(2); // index pole HodnocenyRC
        $this ->addField($fRozdeleni);

        // osoba
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_HEADER => ronText('Jmeno'), BROWSER_FIELD_ORDERING => 1] );
        $this->addField( [BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni', BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER => ronText('Prijmeni'), BROWSER_FIELD_ORDERING => 1] );

        $this->addField(new GenerovaniField($this->database));

        $this->addField(new ExpandField());
    }


    /**
     * @param string $sqlFilter
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        $this ->filterForm = new HodnoceniFilterForm($this ->database, 'hodnoceniosob', false);
        $this ->filterForm ->loadData();

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        echo "<div><a class='hodnoceniToggle' data-selector='.hodnoceni, .vysledekhodnoceni' data-expanded-selector='.hodnoceniToggle' data-toggle-class='hidden' href='#' >" . ronText('Zobrazit_vse') . "</a></div>";
    }


    function getSQLFilter()
    {
        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        if (RonPortalSession::getFiltrovatDatum()) {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        }

        $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        $prava = Prava::vratPrava('hodnoceni');

        $params = $this->database->createParameters();

        $result =
            ' EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.ZpracovatPersonalistiku = 1 ' .
            '        AND EXISTS(SELECT RC FROM EvidencniStav JOIN TypEvidencnihoStavu ON (TypEvidencnihoStavu.Kod = EvidencniStav.KodEvidencnihoStavu) ' .
            '                   WHERE (PracovniPomer.RC = EvidencniStav.RC) AND (PracovniPomer.OsobniCislo = EvidencniStav.OsobniCislo) AND (PracovniPomer.DatumNastupu = EvidencniStav.DatumNastupu)' .
            '                     AND (PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam($datumDo)) . ')' .
            '                     AND ((PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam($datumDo)) . ') OR (PracovniPomer.DatumUkonceni IS NULL))' .
            '                     AND (EvidencniStav.PlatnostOd <= ' . $params->addParameter($this->database->dateToParam($datumDo)) . ')' .
            '                     AND ((EvidencniStav.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam($datumDo)) . ') OR (EvidencniStav.PlatnostDo IS NULL))' .
            '                     AND (TypEvidencnihoStavu.ZahrnoutDoAgend = 1)  ' .
            '                   )' .
            '        AND ' . $prava->sqlFiltr('PracovniPomer', $datumOd, $datumDo, $params) . ')';


        if ($this->filterForm->f_showosoba && !RonPortalSession::vsechnyOsoby()) {
            $result .= " AND Osoba.RC = " . $params->addParameter(RonPortalSession::osobaRC());
        }

        // "Jen rozdeleni"
        if (HodnoceniFilterForm::getJenRozdeleniHodnoceni() && HodnoceniFilterForm::getRozdeleniHodnoceni()) {
            $result .=
                ' AND EXISTS ' .
                '( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                '  FROM HierarchieRozdeleni H JOIN RozdeleniOsob R ON (H.typPodrizene = R.KodTypuRozdeleni AND H.KodPodrizene = R.Rozdeleni)' .
                '  WHERE R.KodTypuRozdeleni = ' . $kodTypuRozdeleni .
                '    AND R.RC = Osoba.RC ' .
                '    AND H.TypNadrazene = ' . $kodTypuRozdeleni .
                "    AND H.KodNadrazene = " . $params->addParameter(HodnoceniFilterForm::getRozdeleniHodnoceni()) .
                '    AND R.PlatnostOd <= ' . $params->addParameter($this->database->dateToParam($datumDo)) .
                '    AND (R.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam($datumOd)) . ' OR R.PlatnostDo IS NULL) ' .
                ')';
        }

        // "Jen ucel hodnoceni"
        if (HodnoceniFilterForm::getJenUcelHodnoceni() && HodnoceniFilterForm::getUcelHodnoceni()) {
            $result .= ' AND (EXISTS(SELECT ID FROM VysledekHodnoceni WHERE VysledekHodnoceni.HodnocenyRC = Osoba.RC AND VysledekHodnoceni.KodUcelu = ' . $params->addParameter(HodnoceniFilterForm::getUcelHodnoceni()) . ')) ';
        }

        return new SqlStatement($result, $params);
    }

    /**
     * @param string $rc
     * @param $rowNumber
     * @param $className
     * @throws Exception
     */
    private function zobrazVysledky($rc, $rowNumber, $className)
    {
        $datumPocatku = isset($_GET['datumpocatku']) ? DateUtils::dateStringToTimestamp($_GET['datumpocatku']) : null;
        $ucelHodnoceni = isset($_GET['ucel']) ? $_GET['ucel'] : null;

        $params = $this->database->createParameters();

        $sql =
            "SELECT ID, PocatekObdobi, KonecObdobi, UcelHodnoceni.Popis, UcelHodnoceni.Kod 
              FROM VysledekHodnoceni JOIN UcelHodnoceni ON (VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod)
              WHERE HodnocenyRC = " . $params->addParameter($rc);

        if (HodnoceniFilterForm::getJenUcelHodnoceni())
            $sql .= ' AND VysledekHodnoceni.KodUcelu = ' . $params->addParameter(HodnoceniFilterForm::getUcelHodnoceni());

        if (RonPortalSession::getFiltrovatDatum())
            $sql .= ' AND VysledekHodnoceni.KonecObdobi >= ' . $params->addParameter($this->database->dateToParam(RonPortalSession::getDatumOd())) .
                    ' AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addParameter($this->database->dateToParam(RonPortalSession::getDatumDo()));

        if (HodnoceniFilterForm::getJenTemaHodnoceni())
            $sql .=
                ' AND EXISTS(SELECT PripraveneTemaHodnoceni.ID ' .
                ' FROM HodnoceniOsoby INNER JOIN PripraveneTemaHodnoceni ON HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
                '      INNER JOIN TemaHodnoceni ON TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ' .
                ' WHERE HodnoceniOsoby.IdVysledku = VysledekHodnoceni.ID ' .
                '   AND TemaHodnoceni.TemaCislo = ' . $params->addParameter(HodnoceniFilterForm::getTemaHodnoceni()) . ')';

        $sql .=
            " ORDER BY ID";

        $cursor = $this->database->getSqlResult($sql, $params);

       while ($row = $cursor->fetch()) {
           $hiddenStr = 'hidden';
           $expandedStr = '';
           if ((!is_null($datumPocatku) && ($datumPocatku == $this->database->stringToTimestamp($row[1]))) && (!is_null($ucelHodnoceni) && ($ucelHodnoceni == $row[4]))) {
               $hiddenStr = '';
               $expandedStr = 'expanded';
           }
           echo "<tr class='$className vysledekhodnoceni vysledkyhodnoceni_{$rowNumber} {$hiddenStr}'>";
           echo "<td></td>";
           echo "<td class=''>" . StringUtils::htmlTextConvert($row[3]) . "</td>";
           echo "<td class=''>" . DBUtils::getFormatedValue($this->database,'date', $row[1]) . "</td>";
           echo "<td class=''>" . DBUtils::getFormatedValue($this->database,'date', $row[2]) . "</td>";
           echo "<td class='hodnoceniToggle expand {$expandedStr}' data-selector='.hodnoceni_{$row[0]}' data-toggle-class='hidden'>&nbsp;</td>";
           echo "</tr>";
           $this->zobrazSlozky($row[0], $rowNumber, $className, $hiddenStr);
       }

    }

    /**
     * zobrazi (pod)radky se slozkama hodnoceni
     * @param $idVysledku
     * @param $rowNumber
     * @param $className
     * @param $hiddenStr
     * @throws DBException
     */
    function zobrazSlozky($idVysledku, $rowNumber, $className, $hiddenStr)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT PripraveneTemaHodnoceni.ID, PripraveneTemaHodnoceni.Stav, " . // 0,1
            "   PripraveneTemaHodnoceni.DatumHodnoceni, PripraveneTemaHodnoceni.Hodnotici, " . //2,3
            "   TemaHodnoceni.PopisTematu, HodnoceniOsoby.Znamka, HodnoceniOsoby.Koeficient, " . //4 - 6
            "   PripraveneTemaHodnoceni.HodnotilRC, Osoba.Prijmeni, Osoba.Jmeno, PoradiPredavani.Vlastni" . //7 - 10

            " FROM HodnoceniOsoby " .
            " INNER JOIN PripraveneTemaHodnoceni ON (HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID) " .
            " INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo) " .
            " INNER JOIN PoradiPredavani ON ((PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu) AND (PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven) AND (PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani))" .
            " LEFT OUTER JOIN Osoba ON (Osoba.RC = PripraveneTemaHodnoceni.HodnotilRC)" .
            " WHERE HodnoceniOsoby.IdVysledku = " . $params->addParameter($idVysledku);

        if (HodnoceniFilterForm::getJenTemaHodnoceni()) {
            $sql .=
                ' AND TemaHodnoceni.TemaCislo = ' . $params->addParameter(HodnoceniFilterForm::getTemaHodnoceni());
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            echo "<tr class='$className hodnoceni hodnoceni_radek_{$rowNumber} hodnoceni_{$idVysledku} {$hiddenStr}'>";
            echo "<td colspan='1'></td>";
            echo "<td class='hodnocenipodrobnosti'><a href='?menuid=hodnoceni&backaction=hodnoceniosob&record={$row[0]}'>" . StringUtils::htmlTextConvert($row[4]) . "</a></td>"; // popis tematu
            echo "<td class='hodnocenipodrobnosti'>" .
                //pokud je prirazeny hodnotitel, zobrazi jmeno a prijmeni pokud zatim neni a v aktualni urovni neni predavani nastaveno jako vlastni zobrazi odkaz na prirazeni hodnotitele
                ($row[7] ? "$row[9] $row[8]" : ($row[10] ? "" : "<span class='nastavhodnotitele clickLink' data-typdialogu='nastavhodnotitele' data-id='{$row[0]}'>" . ronText('Nastav_hodnotitele') . "</span>")) . // hodnotici
                "</td>";
            echo "<td class='hodnocenipodrobnosti'>" . DBUtils::getFormatedValue($this ->database, "finedatetime", $row[2]) . "</td>"; // datum hodnoceni
            // stav
            global $STR_H_0; // kompat.
            global $STR_H_1;
            global $STR_H_2;
            global $STR_H_3;
            global $STR_H_4;
            global $STR_H_5;

            echo "<td class='hodnocenipodrobnosti'>";
            switch ($row[1]) {
                case 0 :
                    echo $STR_H_0 ?: ronText('Stav_hodnoceni_0');
                    break;
                case 1 :
                    echo $STR_H_1 ?: ronText('Stav_hodnoceni_1');
                    break;
                case 2 :
                    echo $STR_H_2 ?: ronText('Stav_hodnoceni_2');
                    break;
                case 3 :
                    echo $STR_H_3 ?: ronText('Stav_hodnoceni_3');
                    break;
                case 4 :
                    echo $STR_H_4 ?: ronText('Stav_hodnoceni_4');
                    break;
                case 5 :
                    echo $STR_H_5 ?: ronText('Stav_hodnoceni_5');
                    break;
                default :
                    echo $row[1];
            }
            echo "</td>";

            echo "</tr>\n";
        }
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     * @throws Exception
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $this->zobrazVysledky($values[2], $rowNumber, $className);

        $this->renderSeparator();
    }



}
