<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 3.10.2017
 */

namespace Ron\Personalistika;

use Ron\Controls\Option;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\BrowserField;
use Ron\DB\Database;
use Ron\Browser;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RozdeleniUtils;
use Ron\StringUtils;
use Ron\Controls\ComboBox;

require_once('adsapp/ronportalbrowser.php');
require_once('adsapp/ronportalfilter.php');


/**
 * Class NahlaseneZmenyFiltr
 * @package Ron\RonPortal
 */
class NahlaseneZmenyFiltr extends RonPortalFilterForm
{

    /**
     * NahlaseneZmenyFiltr constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $action, $useGet = false)
    {
        parent::__construct($database, $action, $useGet);

        $this->f_classname = 'filtercontainer';
        $this->f_formaction = '?menuid=nahlasenezmeny';
        $this->setUseGet(true, 'nahlasenezmeny');

        $this->nastavDatumOdDo(true, false, \Ron\ronText('obdobi_nahlasenych_zmen') . ' ' . \Ron\ronText('Od'));
        $this->nastavOsoby(true, [Prava::vratPrava('ou_zmeny_nahlaseni'), Prava::vratPrava('ou_zmeny_schval')], time(), time(), true);
        $this->povolitVsechyOsoby = true;

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderTypZmeny()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenKodTypuZmeny', \Ron\ronText('typ_zmeny'), RonPortalSession::getValue('jenKodTypuZmeny'),
            ['id' => 'jen_typ_zmeny_checkbox'], 'clickAutoSubmit', 1, true);
        echo '</span>';

        $sql = "SELECT DISTINCT kod, popis FROM TypNahlaseneZmeny ORDER BY popis";

        $combo = new ComboBox('kodTypuZmeny');
        $combo->className = 'adsfilterforminput_full';
        $combo->showCode = 1;
        $combo->dataTypeComparison = false;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('kodTypuZmeny'));
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_typ_zmeny_checkbox'];
        $combo->render();
        echo '</div>';
    }


//    /**
//     * @throws \Ron\DB\DBException
//     */
//    function renderOsobaZmeny()
//    {
//        echo '<div class="filterrow">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenOsobaZmeny', \Ron\ronText('osoba_zmeny'), $_SESSION['jenOsobaZmeny'], '', 'clickAutoSubmit');
//        echo '</span>';
//
//        $sql =
//            "SELECT DISTINCT PracovniPomer.ID, Coalesce (OsobaZmeny.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (OsobaZmeny.Jmeno, '') OsobaZmeny " .
//            "FROM NahlasenaZmena LEFT JOIN Osoba OsobaZmeny ON (NahlasenaZmena.RC = OsobaZmeny.RC) LEFT JOIN PracovniPomer ON (OsobaZmeny.RC = PracovniPomer.RC)".
//            "ORDER BY OsobaZmeny";
//
//        $combo = new ComboBox('osobaZmeny');
//        $combo->className = 'form-input adsfilterforminput_full';
//        $combo->htmlParams = 'onChange="if (jenOsobaZmeny.checked) submit();"';
//        $combo->showCode = 0;
//        $combo->dataTypeComparison = false;
//        $combo->loadData($this->database, $sql, $_SESSION['osobaZmeny']);
//        $combo->render();
//        echo '</div>';
//    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderZpracovatel()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenZpracovatel', \Ron\ronText('zpracoval'), RonPortalSession::getValue('jenZpracovatel'),
            ['id' => 'jen_zpracovatelcheckbox'], 'clickAutoSubmit', 1, true);
        echo '</span>';

        $sql =
            "SELECT DISTINCT MAX(PracovniPomer.ID), Coalesce (Zpracovatel.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Zpracovatel.Jmeno, '') Zpracovatel" .
            ' FROM NahlasenaZmena, Osoba Zpracovatel, PracovniPomer' .
            ' WHERE NahlasenaZmena.Zpracoval = Zpracovatel.RC' .
            ' AND Zpracovatel.RC = PracovniPomer.RC' .
            ' GROUP BY Zpracovatel.Prijmeni, Zpracovatel.Jmeno' .
            ' ORDER BY Zpracovatel';

        $combo = new ComboBox('zpracovatel');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_zpracovatelcheckbox'];
        $combo->showCode = 0;
        $combo->dataTypeComparison = false;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('zpracovatel'));
        $combo->render();
        echo '</div>';
    }


    function renderStavZmeny()
    {
        echo '<div class="filterrow">';
            echo '<span class="filtercaption">';
                GuiUtils::renderCheckbox('jenStavZmeny', \Ron\ronText('stav_zmeny'), RonPortalSession::getValue('jenStavZmeny'),
                    ['id' => 'jen_stav_zmeny_checkbox'], 'clickAutoSubmit', 1, true);
            echo '</span>';

            $combo = new ComboBox('stavZmeny', [
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);

            $combo->className = 'adsfilterforminput_full';
            $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_stav_zmeny_checkbox'];
            $combo->dataTypeComparison = false;
            $combo->selectedOption = $combo->findOption(RonPortalSession::getValue('stavZmeny') != '' ? (int)RonPortalSession::getValue('stavZmeny') : 1);
            $combo->render();
        echo '</div>';
    }


    function renderJenNezapsaneZmeny()
    {
        //Dlouhodoby kurz
        echo '<DIV class="">';

        echo '<span>';
        echo "<a href='?menuid=osobniudaje'>" . \Ron\ronText('osobni_udaje') . "</a>";
        echo '</span>';
        echo '</DIV>';
    }


    function loadZpracovatel()
    {
        if(isset($_GET['zpracovatel'])){
            RonPortalSession::setValue('jenZpracovatel', isset($_GET['jenZpracovatel'])  &&  (int)$_GET['jenZpracovatel']);
            RonPortalSession::setValue('zpracovatel', $_GET['zpracovatel']);
        }
    }


    function loadTypZmeny()
    {
        if(isset($_GET['kodTypuZmeny'])){
            RonPortalSession::setValue('jenKodTypuZmeny', isset($_GET['jenKodTypuZmeny']) && (int)$_GET['jenKodTypuZmeny']);
            RonPortalSession::setValue('kodTypuZmeny', $_GET['kodTypuZmeny']);
        }
    }


    function loadStavZmeny()
    {
        if(isset($_GET['stavZmeny'])){
            RonPortalSession::setValue('jenStavZmeny', isset($_GET['jenStavZmeny'])  &&  (int)$_GET['jenStavZmeny']);
            RonPortalSession::setValue('stavZmeny', $_GET['stavZmeny']);
        }
    }


    function loadData(){
        $this->loadTypZmeny();
        $this->loadZpracovatel();
        $this->loadStavZmeny();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderTypZmeny();
        //$this->renderOsobaZmeny();
        $this->renderZpracovatel();
        $this->renderStavZmeny();
        $this->renderJenNezapsaneZmeny();
    }
}

/**
 * Class ZobrazeniDetailuNahlaseneZmenyField
 * @package Ron\RonPortal
 */
class ZobrazeniDetailuNahlaseneZmenyField extends BrowserField
{

    /** @var Database */
    public $database;

    /**
     * ZobrazeniDetailuNahlaseneZmenyField constructor.
     * @param null|Database $database
     */
    public function __construct($database = null)
    {
        parent::__construct($database);
        $this->database = $database;
        $this->className = 'center sloupec';
        $this->header = \Ron\ronText('detail');
        $this->code = 'col-detail';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        //$result .= "<a href='?menuid=nahlasenezmeny&action=detail&record=$rowData[0]&osoba=$rowData[18]'>";
        $result .=
            "<a href='?menuid=nahlasenezmeny&record=$rowData[0]'>" .
            \Ron\ronText(($rowData[14]) ? 'detail' : 'detail_zmeny_s_potvrzenim') .
            '</a>';

        return $result;
    }
}


///**
// * Class StavField
// * @package Ron\RonPortal
// */
//class StavZmenyField extends BrowserField
//{
//    /**
//     * ZobrazeniDetailuNahlaseneZmenyField constructor.
//     * @param null|Database $database
//     * @param array|null $options
//     */
//    public function __construct($database = null, $options = null)
//    {
//        parent::__construct($database, $options);
//        $this->header = \Ron\ronText('stav_zmeny');
//        $this->className = 'center sloupec';
//
//    }
//
//    /**
//     * @param mixed $value
//     * @param int $rowNumber
//     * @param string $recordId
//     * @param array $rowData
//     * @return int|string
//     */
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result = "";
//
//        if ($rowData[3] != '') {
//
//            $result .= "<a href='?menuid=nahlasenezmeny&action=nastavitfiltr&filtr=stavzmeny&stav=$rowData[14]'>" . \Ron\ronText('zpracovano') . "</a>";
//
//            if ($rowData[14] == 1) {
//                $result = "<a href='?menuid=nahlasenezmeny&action=nastavitfiltr&filtr=stavzmeny&stav=$rowData[14]'>" . \Ron\ronText('zpracovano') . "</a>";
//            }
//
//            if ($rowData[14] == 2) {
//                $result = "<a href='?menuid=nahlasenezmeny&action=nastavitfiltr&filtr=stavzmeny&stav=$rowData[14]'>" . \Ron\ronText('odmitnuto') . "</a>";
//            }
//
//        } else {
//            $result .= "<a href='?menuid=nahlasenezmeny&action=nastavitfiltr&filtr=stavzmeny&stav=$rowData[14]'>" . \Ron\ronText('nezpracovano') . "</a>";
//        }
//
//        return $result;
//    }
//}

///**
// * Class TypZmenyField
// * @package Ron\RonPortal
// */
//class TypZmenyField extends BrowserField
//{
//    /**
//     * TypZmenyField constructor.
//     * @param null|Database $database
//     * @param array|null $options
//     */
//    public function __construct(Database $database = null, array $options = null)
//    {
//        parent::__construct($database, $options);
//        $this->header = \Ron\ronText('typ_zmeny');
//        $this->className = "center sloupec";
//    }
//
//    /**
//     * @param mixed $value
//     * @param int $rowNumber
//     * @param string $recordId
//     * @param array $rowData
//     * @return string
//     */
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result = "";
//
//        $result .= "<a href='?menuid=nahlasenezmeny&action=nastavitfiltr&filtr=typzmeny&stav=$rowData[7]'>" . \Ron\ronText($rowData[6]) . "</a>";
//
//        return $result;
//    }
//}



///**
// * Class FieldCisloRadku
// * @package Ron\RonPortal
// */
//class FieldCisloRadku extends BrowserField
//{
//    public function __construct($database, $options)
//    {
//        parent::__construct($database, $options);
//        $this->align = 'left';
//    }
//
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result = '';
//        $result .= '<a' . ' href="#top">' . $rowNumber . '</a>';
//        return $result;
//    }
//}


/**
 * Class Zpracovatel
 * @package Ron\RonPortal
 */
class ZpracovatelField extends BrowserField
{
    public function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('zpracoval');
        $this->code = 'col-zpracoval';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($rowData[14] != "") {
            $result .= StringUtils::htmlTextConvert($rowData[15]) . " " . StringUtils::htmlTextConvert($rowData[16]) . ($rowData[17]!="" ? ", " : "") . StringUtils::htmlTextConvert($rowData[17]);
        }

        return $result;
    }
}



/**
 * Class NahlaseneZmenyBrowser
 * @package Ron\RonPortal
 */
class NahlaseneZmenyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby */
    var $pravo_nahlaseni;

    /** @var  PravaNaOsoby */
    var $pravo_schvaleni;


    /**
     * NahlaseneZmenyBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'nahlasenezmeny');

        Prava::nastavObdobiProPrava(time(),time());
        $this->pravo_nahlaseni = Prava::vratPrava('ou_zmeny_nahlaseni');
        $this->pravo_schvaleni = Prava::vratPrava('ou_zmeny_schval');

        $this->code = 'browsernahlasenezmeny';
        $this->tableId = 'browsernahlasenezmeny';
        $this->browserUrl = '?menuid=nahlasenezmeny';
        
        $this->title = \Ron\ronText('nahlasene_zmeny');

        $this->sqlSelectPart =
            "NahlasenaZmena.ID, NahlasenaZmena.TypZmeny, NahlasenaZmena.DatumCas, NahlasenaZmena.Zpracoval, " .    //0..3
             $database->castWideMemo("NahlasenaZmena.Poznamka") . ", " . $database->castWideMemo("NahlasenaZmena.Hodnoty") .   //4..5
            ", TypNahlaseneZmeny.Popis, TypNahlaseneZmeny.Kod, NahlasenaZmena.DatumCasZpracovani, " .        //6..8
            "COALESCE(OsobaZmeny.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(OsobaZmeny.Jmeno,'') OsobaZmeny, " .  //9
            "COALESCE(Zpracovatel.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Zpracovatel.Jmeno,'') Zpracovatel, " .  //10
            "COALESCE(Nahlasil.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Nahlasil.Jmeno,'') Nahlasil, NahlasenaZmena.RC, " .  //11..12
            "NahlasenaZmena.ID, NahlasenaZmena.Stav, " .   //13..14
            "Zpracovatel.Prijmeni, Zpracovatel.Jmeno, Zpracovatel.Titul, (SELECT MAX(PP.ID) FROM PracovniPomer PP WHERE PP.RC = NahlasenaZmena.RC)";   //15..18

        $this->primaryField = 0;

        $this->sqlFromPart =
            "(NahlasenaZmena LEFT JOIN TypNahlaseneZmeny ON (NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod)) 
                LEFT JOIN Osoba OsobaZmeny ON (NahlasenaZmena.RC = OsobaZmeny.RC) " .
                "LEFT JOIN Osoba Zpracovatel ON (NahlasenaZmena.Zpracoval = Zpracovatel.RC)" .
                 "LEFT JOIN Osoba Nahlasil ON (NahlasenaZmena.Nahlasil = Nahlasil.RC)";
                  //"LEFT JOIN PracovniPomer PP ON (PP.RC = NahlasenaZmena.RC)";

        $this->filterForm = new NahlaseneZmenyFiltr($database, null);
        $this->filterForm->loadData();

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCas', 2) ->setHeaderText('datum_hlaseni') ->setFormat('date')
            ->setCode('col-datum')
            ->setOrdering(true) ->setAlign('center');

        $this->defaultSort = -1;
        
        //$this->addField(new StavZmenyField());
        $this->addField(BROWSER_FIELD_TYPE_SELECT)
            ->setSqlFieldIndex(14) -> setSqlFieldName('NahlasenaZmena.Stav') ->setOrdering(true)
            ->setHeaderText('stav_zmeny') //->setClassName('potvrzeni_zmeny')
            ->setCode('col-stavzmeny')
            //->setLink('?menuid=nahlasenezmeny') // . ($backMenuId ? '&back=' . $backMenuId : ''))
            ->setOptions([
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);

        //$this->addField(new StavZmenyField());
        $this->addField()
            ->setSqlFieldIndex(6) ->setSqlFieldName('TypNahlaseneZmeny.Popis') ->setOrdering(1)
            ->setHeaderText('typ_zmeny')
            ->setCode('col-typzmeny')
            ->setLink('?menuid=nahlasenezmeny'); // . ($backMenuId ? '&back=' . $backMenuId : ''))
        //$this->addField(new TypZmenyField());

        $this->addField(new ZobrazeniDetailuNahlaseneZmenyField($this->database));

        $this->addField()
            ->setSqlFieldName('OsobaZmeny', 9) ->setHeaderText('osoba_zmeny') ->setOrdering(true)
            ->setCode('col-osoba');

        $this->addField()
            ->setSqlFieldName('Nahlasil', 11) ->setHeaderText('zmenu_nahlasil') ->setOrdering(true)
            ->setCode('col-nahlasil');

        $this->addField(new ZpracovatelField($this->database));

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCasZpracovani', 8)
            ->setCode('col-datumzpracovani')
            ->setHeaderText('datum_zpracovani') ->setFormat('date') ->setOrdering(true) ->setAlign('center');
    }

    function setFilterForm()
    {
        // uz nastaveno v konstruktoru
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na osobu zmeny pravo
        if ($this->pravo_nahlaseni->lzeZobrazitRC($values[12]) || $this->pravo_schvaleni->lzeZobrazitRC($values[12])){
            return true;
        }
        return false;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[4]) {
            echo
                "<tr class='$className'>" .
                '<td></td><td colspan="' . ($this->columnCount - 1) . '" >' .
                '<span class="browserpoznamka">' . \Ron\ronText("Poznamka") . ': ' . StringUtils::htmlTextConvert($values[4]) . '</span>' .
                '</td>' .
                '</tr>';
        }

        $this->renderSeparator();
    }

    /**
     * @return SqlStatement
     * @throws \Ron\DB\DBException
     */
    public function getSQLFilter()
    {
        $result = '';
        $params = $this ->database ->createParameters();

        require_once('adsapp/ppomer.php');
//        $pracovniPomerOsobyZmeny = new PracovniPomer();
//        $pracovniPomerOsobyZmeny->nactiPPPodleId($this->database, $_SESSION['osobaZmeny']);
        $pp = null;
        if (!$this ->filterForm ->vsechnyOsoby) {
            $pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->vybranaOsoba);
        }


        if ($this ->filterForm ->filtrovatDatum) {
            $result .= ' NahlasenaZmena.DatumCas >= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->datumOd)) .
                ' AND NahlasenaZmena.DatumCas <= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->datumDo));
        }

        if (RonPortalSession::getValue('jenKodTypuZmeny')) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " TypNahlaseneZmeny.kod = " . $params->addParameter(RonPortalSession::getValue('kodTypuZmeny'));
        }

        if (RonPortalSession::getValue('jenStavZmeny')){
            if ($result) {
                $result .= ' AND ';
            }

            if (!RonPortalSession::getValue('stavZmeny')) {
                $result .= " (NahlasenaZmena.DatumCasZpracovani IS NULL OR NahlasenaZmena.Zpracoval IS NULL)";
            }

            if (RonPortalSession::getValue('stavZmeny') == 1) {
                $result .=
                    " (NahlasenaZmena.Stav = " . $params->addParameter(RonPortalSession::getValue('stavZmeny')) . ')' .
                    "  AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL)" .
                    " OR (NahlasenaZmena.Stav =  0 AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL))";
            }

            if (RonPortalSession::getValue('stavZmeny') == 2) {
                $result .= " NahlasenaZmena.Stav = " . $params->addParameter(RonPortalSession::getValue('stavZmeny'));
            }
        }

        if($pp) {
            $result .=
                ($result ? ' AND ' : '') .
                'NahlasenaZmena.RC = ' . $params->addParameter($pp->rc);
        } else {
            if ($this ->filterForm ->kodRozdeleni) {
                require_once 'adsapp/rozdeleniutils.php';

                $rozdeleniFiltr =
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'NahlasenaZmena',
                        $this ->filterForm ->f_typrozdeleni, $this ->filterForm ->kodRozdeleni,
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumOd : time(),
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumDo : time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true); // jenRC = true

                if ($rozdeleniFiltr) {
                    $result .= ($result ? ' AND ' : '') . '(' . $rozdeleniFiltr . ')';
                }
            }

            if ($this ->filterForm ->kodRozdeleni2) {
                require_once 'adsapp/rozdeleniutils.php';

                $rozdeleniFiltr2 =
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'NahlasenaZmena',
                        $this ->filterForm ->f_typrozdeleni2, $this ->filterForm ->kodRozdeleni2,
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumOd : time(),
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumDo : time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni2, $params, true); // jenRC = true

                if ($rozdeleniFiltr2) {
                    $result .= ($result ? ' AND ' : '') . '(' . $rozdeleniFiltr2 . ')';
                }
            }
        }

        if(RonPortalSession::getValue('jenZpracovatel')) {
            $pracovniPomerZpracovatel = new PracovniPomer();
            $pracovniPomerZpracovatel->nactiPPPodleId($this->database, RonPortalSession::getValue('zpracovatel'));

            $result .=
                ($result ? ' AND ' : '') .
                'NahlasenaZmena.Zpracoval = ' . $params->addParameter($pracovniPomerZpracovatel->rc);
        }

        return new SqlStatement($result, $params);
    }
}