<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 11.6.2018
 */

namespace Ron\RonPortal;


use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use function Ron\ronText;
use Ron\StringUtils;

require_once "ron/httputils.php";


/**
 * Class PlanovaneSkoleniKurzyForm
 * @package Ron\RonPortal
 */
class PlanovaneSkoleniKurzyForm extends RonPortalFilterForm
{

    /** @var PravaNaOsoby $prava*/
    var $pravoNaPlan;

    /**
     * PlanovaneSkoleniKurzyForm constructor.
     * @param Database $database
     * @param $pravo
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $pravo, $action, $useGet = false)
    {
        parent::__construct($database, $action, $useGet);

        $this->nastavDatumOdDo(true,false);
        Prava::nastavObdobiProPrava(time(),time());

        $this->pravoNaPlan = $pravo;
        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = '?menuid=skoleni&action=planovaneskolenikurzy';

    }


    /**
     * @throws DBException
     */
    function renderTypSkoleni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenDruhSkoleniAKurzu', ronText('planovany_kurz') . '&nbsp;&nbsp;', $_SESSION['jenDruhSkoleniAKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT DISTINCT PS.CisloSkoleni AS Kod, PS.Popis 
                FROM PlanovaneSkoleni PS LEFT JOIN PlanovaneSkoleniOsob ON (PS.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni)";

        if (!($this->pravoNaPlan->prava == 3)){
               $sql .= "WHERE PlanovaneSkoleniOsob.RC = " . $params->addParameter(User::getRC());
        }

        $sql .= " ORDER BY PS.Popis";

        $combo = new ComboBox('druhSkoleniAKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenDruhSkoleniAKurzu.checked) submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['druhSkoleniAKurzu'], $params);
        $combo->render();
        echo '</div>';

    }

    function loadTypSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenDruhSkoleniAKurzu'] = isset($_POST['jenDruhSkoleniAKurzu']);
            $_SESSION['druhSkoleniAKurzu'] = $_POST['druhSkoleniAKurzu'];
        }
    }


    function loadData()
    {
        $this->loadTypSkoleni();
        parent::loadData();
    }


    /**
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderTypSkoleni();
    }
}

/**
 * Class DetailPlanovanehoSkolenikurzu
 * @package Ron\RonPortal
 */
class DetailPlanovanehoSkolenikurzu extends BrowserField
{

    /**
     * DetailPlanovanehoSkolenikurzu constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = ronText('detail_planovaneho_skoleni');
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= '<a href="?menuid=skoleni&action=detailplanovanehoskolenikurzu&record=' . HttpUtils::urlEncode($rowData[0]) . '">';
        $result .= StringUtils::htmlTextConvert($rowData[1]);
        $result .= '</a>';
        return $result;
    }
}

/**
 * Class DatumPosledniAktualizacePlanovanehoField
 * @package Ron\RonPortal
 */
class DatumPosledniAktualizacePlanovanehoField extends BrowserField
{

    /**
     * DatumPosledniAktualizacePlanovanehoField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = ronText('posledni_aktualizace');
        $this->align = 'center';
        $this->sqlFieldName = "PS.Update_Timestamp";
        $this->ordering = 1;
        $this->sortSQL = "PS.Update_Timestamp";
        $this->sortSQLDesc = "PS.Update_Timestamp DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[29]) {
            $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[29])) . ' (' . StringUtils::htmlTextConvert( $rowData[28]) . ') ' . "</div>" ;
        }
        return $result;
    }
}

/**
 * Class DatumVlozeniPlanovanehoField
 * @package Ron\RonPortal
 */
class DatumVlozeniPlanovanehoField extends BrowserField
{

    /**
     * DatumVlozeniPlanovanehoField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = ronText('datum_vlozeni');
        $this->align = 'center';
        $this->sqlFieldName = "PS.Update_Timestamp";
        $this->ordering = 1;
        $this->sortSQL = "PS.Insert_Timestamp";
        $this->sortSQLDesc = "PS.Insert_Timestamp DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[27]) {
            $result .= "<div >" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[27])) . ' (' . StringUtils::htmlTextConvert( $rowData[26]) . ') ' . "</div>" ;
        }
        return $result;
    }
}


/**
 * Class PlanovaneSkoleniKurzy
 */
class PlanovaneSkoleniKurzy extends Browser
{
    /** @var PravaNaOsoby $pravo*/
    public $pravo;


    /** @var  PlanovaneSkoleniKurzy $filtrForm */
    public $filtrForm;

    /**
     * PlanovaneSkoleniKurzy constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database);


        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo = Prava::vratPrava('skoleni_plan');

        $this->code = 'browserplanovaneskolenikurzy';
        $this->tableId = 'browserplanovaneskolenikurzy';
        $this->browserUrl = "?menuid=skoleni&action=planovaneskolenikurzy";

        $this->sqlSelectPart =
            "PS.CisloSkoleni, PS.Popis, PS.DruhSkoleni, " .                   //0..2
            "PS.DatumPocatkuSkoleni, PS.CasPocatkuSkoleni, PS.DatumUkonceniSkoleni, " .       //3..5
            "PS.CasUkonceniSkoleni, PS.MistoSkoleni, PS.PocetOsob, " .        //6..8
            "PS.KonkretniUcastnici, PS.SkoliciFirma, PS.Skolitel, " .         //9..11
            "PS.CenaZaSkoleni, PS.DPH1, PS.DPH2, PS.DPH3, " .       //12..15
            "PS.Dan1, PS.Dan2, PS.Dan3, PS.Stav, " .                //16..19
            $this->database->castWideMemo("PS.Poznamka") . ", S_StavPlanovanehoSkoleni.Kod, S_StavPlanovanehoSkoleni.Popis, " . //20..22
            "PS.DelkaSkoleniDnu, PS.DelkaSkoleniHodin, Firma.Nazev1, PS.Insert_UserName, PS.Insert_Timestamp, PS.Update_UserName, PS.Update_Timestamp, " .               //23..29
            "(Select RC From PlanovaneSkoleniOsob Where DatumVlozeni = (Select MIN(DatumVlozeni) From PlanovaneSkoleniOsob Where (PS.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni))) ";  //30

        $this->sqlFromPart =
            "PlanovaneSkoleni PS 
              LEFT JOIN S_StavPlanovanehoSkoleni ON (PS.Stav = S_StavPlanovanehoSkoleni.Kod)
               LEFT JOIN Firma ON (PS.SkoliciFirma = Firma.kodFirmy)";

        $this->filtrForm = new PlanovaneSkoleniKurzyForm($database, $this->pravo, null);
        $this->filtrForm->loadData();

        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'PS.CisloSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('ID') . "&nbsp;&nbsp;", BROWSER_FIELD_ORDERING => 1]);

        $this->addField(new DetailPlanovanehoSkolenikurzu($this->database));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'PS.DruhSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('druh_skoleni') . "&nbsp;&nbsp;", BROWSER_FIELD_ORDERING => 1]);


        $this->addField([BROWSER_FIELD_SQL_INDEX => 22, BROWSER_FIELD_SQL_NAME => 'S_StavPlanovanehoSkoleni.Popis',
            BROWSER_FIELD_HEADER_TEXT => ronText('stav') . "&nbsp;&nbsp;", BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'PS.DatumPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('datum_pocatku_skoleni'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'PS.CasPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('cas_pocatku_skoleni'), BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);


        $this->addField([BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'PS.CasUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('cas_ukonceni_skoleni'), BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'PS.DatumUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('datum_ukonceni_skoleni'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 25, BROWSER_FIELD_SQL_NAME => 'Firma.Nazev1', BROWSER_FIELD_HEADER_TEXT => ronText('skolici_firma'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_SQL_NAME => 'PS.Skolitel', BROWSER_FIELD_HEADER_TEXT => ronText('skolitel'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField(new DatumVlozeniPlanovanehoField($this->database));
        $this->addField(new DatumPosledniAktualizacePlanovanehoField($this->database));
    }


    function renderFilter()
    {
        $this->filtrForm->render();
        parent::renderFilter();
    }

    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        if ($this->pravo->prava == 3){
            return true;

        } elseif ($values[30] != "" && $this->pravo->lzeZobrazitRC($values[30])){
            return true;
        }
        return false;
    }


    /**
     * @return string
     */
    function getSQLFilter()
    {
        $result = "";
        $params = $this->database->createParameters();
        //Nebylo mozno zobrazit omylem zamitnute kurzy
//        $result .= "PS.Stav < 2 ";

        if ($_SESSION['jenDruhSkoleniAKurzu']){
            if ($result != ''){
                $result .= ' AND ';
            }
            $result .= "PS.DruhSkoleni = " . $params->addParameter($_SESSION['druhSkoleniAKurzu']);
        }

        if (RonPortalSession::getFiltrovatDatum()) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                ' AND PS.DatumPocatkuSkoleni <= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumDo()));
        }

        return new SqlStatement($result, $params);
    }
}