<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 23.11.2017
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\LinksList;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\DB\Database;
use Ron\BrowserField;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\Browser;
use Ron\Controls\ComboBox;

require_once('ron/browser.php');
require_once('adsapp/prava.php');
require_once('ron/httputils.php');
require_once('adsapp/ronportalfilter.php');


/**
 * Class PrihlaseniNaPlanovaneSkoleniForm
 * @package Ron\RonPortal
 */
class PrihlaseniNaPlanovaneSkoleniForm extends RonPortalFilterForm
{
    /** @var PravaNaOsoby $pravaPrihlasit */
    public $pravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /**
     * PrihlaseniNaPlanovaneSkoleniForm constructor.
     * @param Database $database
     * @param $pravoPrihlasit
     * @param $pravoSchvalit
     * @param bool $useGet
     */
    public function __construct(Database $database, $pravoPrihlasit, $pravoSchvalit, $useGet = false)
    {
        parent::__construct($database, $useGet);

        $this->pravoPrihlasit = $pravoPrihlasit;
        $this->pravoSchvalit = $pravoSchvalit;
        Prava::nastavObdobiProPrava(time(),time());

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = '?menuid=skoleni&action=prihlaseninaplanovaneskoleni';

        //Zobrazeni FilterFormu pro nastaveni data od - do
        $this->nastavDatumOdDo(true, false, ronText('pocatek_skoleni'));
    }


    /**
     * @throws DBException
     */
    function renderTypSkoleni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypSkoleniAKurzu', ronText('druh_skoleni'). '&nbsp;&nbsp;', $_SESSION['jenTypSkoleniAKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Distinct DruhSkoleni.Kod, DruhSkoleni.Popis 
                FROM DruhSkoleni Left JOIN PlanovaneSkoleni ON  (DruhSkoleni.Kod = PlanovaneSkoleni.DruhSkoleni) JOIN PlanovaneSkoleniOsob ON (PlanovaneSkoleni.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni)
                WHERE PlanovaneSkoleniOsob.RC = " . $params->addParameter(User::getRC()) . " 
                AND Stav = 0 AND DruhSkoleni.Aktivni = 1
                ORDER By DruhSkoleni.Popis";

        $combo = new ComboBox('typSkoleniAKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypSkoleniAKurzu.checked) submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['typSkoleniAKurzu'], $params);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws DBException
     */
    function renderStavPrihlaseni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenStavPrihlaseni', ronText('stav_prihlaseni'). '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;', $_SESSION['jenStavPrihlaseni'],
            'onClick="submit()"');
        echo '</span>';

        $sql = "SELECT DISTINCT Kod, Popis FROM S_StavPlanovanehoSkoleni ORDER BY Popis";

        $combo = new ComboBox('stavPrihlaseni');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenStavPrihlaseni.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['stavPrihlaseni']);
        $combo->render();
        echo '</div>';
    }

    public function renderPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        // vlastnik ukolu - moje / vsechny
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            LinksList::renderLinks(
                [
                    'planypodleosoby' => ronText('plany_podle_osoby'),
                    'planypodlekurzu' => ronText('plany_podle_kurzu')
                ],
                $_SESSION['filtrprihlaseninaplanovaneskoleni'], '?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=');
        }
        echo "<br>";
    }


    function renderNeprihlaseneSkoleni()
    {
        echo '<div class="">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenNeprihlasenaSkoleni', ronText('neprihlasena_skoleni'). '&nbsp;&nbsp;', $_SESSION['jenNeprihlasenaSkoleni'],
            'onClick="submit()"');
        echo '</span>';
        echo '</div>';
    }


    private function loadPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        //Pokud nema pravo na ostatni pak se hned nastavi na volbu mojeukoly
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            if (isset($_GET['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = $_GET['filtrprihlaseninaplanovaneskoleni'];
            }
            //Pri prvnim zobrazeni stranky se implicitne nastavi Moje ukoly, pokud nema prihlasena osoba pravo na schvaleni nebo hodnoceni
            if (!isset($_SESSION['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = 'planypodleosoby';
            }
        }
    }

    function loadTypSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypSkoleniAKurzu'] = isset($_POST['jenTypSkoleniAKurzu']);
            $_SESSION['typSkoleniAKurzu'] = $_POST['typSkoleniAKurzu'];
        }
    }

    function loadNeprihlaseneSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenNeprihlasenaSkoleni'] = isset($_POST['jenNeprihlasenaSkoleni']);
        }
    }

    function loadStavPrihlaseni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenStavPrihlaseni'] = isset($_POST['jenStavPrihlaseni']);
            $_SESSION['stavPrihlaseni'] = $_POST['stavPrihlaseni'];
        }
    }

    function loadData()
    {
        $this->loadTypSkoleni();
        $this->loadStavPrihlaseni();
        $this->loadNeprihlaseneSkoleni();
        $this->loadPodMenuPrihlaseniNaPlanovaneSkoleni();
        parent::loadData();
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        $this->renderPodMenuPrihlaseniNaPlanovaneSkoleni();
        parent::renderFilters();
        $this->renderTypSkoleni();
        $this->renderStavPrihlaseni();
        $this->renderNeprihlaseneSkoleni();
    }
}


/**
 * Class PrihlaseniNaSkoleniField
 * @package Ron\RonPortal
 */
class PrihlaseniNaSkoleniField extends BrowserField
{

    /**
     * PrihlaseniNaSkoleniField constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = ronText('prihlaseni');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "Prihlaseno";
        $this->sortSQLDesc = "Prihlaseno DESC";
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[12] == 0) {
            $result .= "<a href='?menuid=prihlaseninaplanovaneskoleni&action=prihlasit&cisloskoleni=$rowData[0]'>" . ronText('prihlasit_na_skoleni') . "</a>" ;
        } else {
            if ($rowData[14] == 0) {
                $result .= "<a href='?menuid=prihlaseninaplanovaneskoleni&action=odhlasit&cisloskoleni=$rowData[0]'>" . ronText('odhlasit_ze_skoleni') . '&nbsp;&nbsp;(' . $rowData[13] . ')' . "</a>";
            } else {
                $result .= "<strong>" . $rowData[13] . "</strong>";
            }
        }
        return $result;
    }
}


/**
 * Class PrihlaseniNaPlanovaneSkoleniBrowser
 * @package Ron\RonPortal
 */
class PrihlaseniNaPlanovaneSkoleniBrowser extends Browser
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    var $pravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /** @var PrihlaseniNaPlanovaneSkoleniForm $filtrForm */
    var $filtrForm;

    //TODO nahrazeno pomoci b_schvalovaniplanukurzuosoby.php dle MaKr. smazat?
    


    /**
     * PrihlaseniNaPlanovaneSkoleniBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database);

        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        Prava::nastavObdobiProPrava(time(), time());

        $this->title = ronText('Moje_kurzy');
        $this->code = 'browserprihlaseninaplanovaneskoleni';
        $this->tableId = 'prihlaseninaplanovaneskoleni';
        $this->browserUrl = '?menuid=skoleni&action=prihlaseninaplanovaneskoleni';


        $this->sqlSelectPart =
            'PS.CisloSkoleni, PS.Popis, DruhSkoleni.Popis, Stav, PS.DatumPocatkuSkoleni, PS.CasPocatkuSkoleni, PS.DatumUkonceniSkoleni, ' .      //0..6
            'PS.CasUkonceniSkoleni, PS.MistoSkoleni, PocetOsob, Firma.Nazev1, PS.Skolitel, ' .       //7..11
            "(SELECT count(rc) FROM PlanovaneSkoleniOsob 
                WHERE PS.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni AND rc = '" . $this->database->addSlashes(User::getRC()) . "') Prihlaseno, " .    //12
            "(SELECT S_StavPotvrzeni.Popis FROM PlanovaneSkoleniOsob LEFT JOIN S_StavPotvrzeni ON (S_StavPotvrzeni.Kod = PlanovaneSkoleniOsob.Potvrzeni)
                WHERE PS.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni AND rc = '" . $this->database->addSlashes(User::getRC()) . "') , " .    //13
            "(SELECT S_StavPotvrzeni.Kod FROM PlanovaneSkoleniOsob LEFT JOIN S_StavPotvrzeni ON (S_StavPotvrzeni.Kod = PlanovaneSkoleniOsob.Potvrzeni)
                WHERE PS.CisloSkoleni = PlanovaneSkoleniOsob.CisloSkoleni AND rc = '" . $this->database->addSlashes(User::getRC()) . "') , " .    //14
            $this->database->castWideMemo('PS.Poznamka'); //15

        $this->sqlFromPart =
            'PlanovaneSkoleni PS LEFT JOIN DruhSkoleni ON (DruhSkoleni.Kod = PS.DruhSkoleni) ' .
                'LEFT JOIN Skoleni ON (PS.CisloSkoleni = Skoleni.CisloPlanovanehoSkoleni) ' .
                    'LEFT JOIN Firma ON (PS.SkoliciFirma = Firma.kodFirmy) ';

        $this->sqlOrderPart = "PS.DatumPocatkuSkoleni DESC";

        $this->filtrForm = new PrihlaseniNaPlanovaneSkoleniForm($database, $this->pravoPrihlasit, $this->pravoSchvalit);
        $this->filtrForm->loadData();

        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'CisloSkoleni',
            BROWSER_FIELD_HEADER_TEXT => 'cislo_skoleni', BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'Popis', BROWSER_FIELD_HEADER_TEXT => ronText('popis_planu_kurzu'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'DruhSkoleni.Popis', BROWSER_FIELD_HEADER_TEXT => ronText('druh_skoleni'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);


        $this->addField([BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'DatumPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('datum_pocatku_skoleni'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'CasPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('cas_pocatku_skoleni'), BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new PrihlaseniNaSkoleniField($database));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_SQL_NAME => 'CasUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('cas_ukonceni_skoleni'), BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'DatumUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => ronText('datum_ukonceni_skoleni'), BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_SQL_NAME => 'SkoliciFirma', BROWSER_FIELD_HEADER_TEXT => ronText('skolici_firma'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_SQL_NAME => 'Skolitel', BROWSER_FIELD_HEADER_TEXT => ronText('skolitel'),
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_ORDERING => 1]);
    }


    /**
     * Render filtru
     */
    function renderFilter()
    {
        parent::renderFilter();
        $this->filtrForm->render();
    }

    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[15]) {
            echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 5) . '" style="width:12rem;">';
                    echo '<span class="info tucne">' . ronText('poznamka_kurzu') . ':&nbsp;&nbsp;&nbsp;&nbsp;' . '</span>';
                    echo '<span class="info">' . StringUtils::htmlTextConvert($values[15]) . '</span>';
                echo '<td colspan="4"></td>';
            echo '</tr>';
        }

        $this->renderSeparator();
    }

    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = '';

        $result .= 'Stav = 0';

        if (!$_SESSION['jenAktualniObdobi']) {
            if (RonPortalSession::getFiltrovatDatum()) {
                if ($result) {
                    $result .= ' AND ';
                }
                $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                    ' AND PS.DatumPocatkuSkoleni <= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumDo()));
            }
        }

        if($_SESSION['jenTypSkoleniAKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " PS.DruhSkoleni = " . $params->addParameter($_SESSION['typSkoleniAKurzu']);
        }

        if($_SESSION['jenStavPrihlaseni']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= "EXISTS(SELECT Potvrzeni FROM PlanovaneSkoleniOsob
                            Where PS.CisloSkoleni=PlanovaneSkoleniOsob.CisloSkoleni AND Potvrzeni = " . $params->addParameter($_SESSION['stavPrihlaseni']) .
                            " AND PlanovaneSkoleniOsob.RC = " . $params->addParameter(User::getRC());
        }

        if($_SESSION['jenNeprihlasenaSkoleni']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= "NOT EXISTS(SELECT Potvrzeni FROM PlanovaneSkoleniOsob
                            Where PS.CisloSkoleni=PlanovaneSkoleniOsob.CisloSkoleni AND PlanovaneSkoleniOsob.RC = " . $params->addParameter(User::getRC());
        }


        return new SqlStatement($result, $params);
    }
}
