<?php

namespace Ron\RonPortal;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\User;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\PravaNaOsoby;

class PersonalniProcesyFilterForm extends RonPortalFilterForm
{
    /** @var int typ zobrazeni, 0 = procesy vytvorene pro uzivatele, 1 = procesy, kde se ma uzivatel vyjadrit */
    public $typZobrazeni;

    public $typProcesu;

    public $stav;

    private function loadZobrazeni()
    {
        if (isset($_GET['zobrazeni'])) {
            $_SESSION['pers_procesy_zobrazeni'] = $_GET['zobrazeni'];
        }

        $this->typZobrazeni = isset($_SESSION['pers_procesy_zobrazeni']) ? $_SESSION['pers_procesy_zobrazeni'] : 0;
    }

    private function loadTypProcesu()
    {
        if (isset($_GET['typprocesu'])) {
            $_SESSION['pers_procesy_proces'] = $_GET['typprocesu'];
        }

        $this->typProcesu = isset($_SESSION['pers_procesy_proces']) ? $_SESSION['pers_procesy_proces'] : null;
    }

    private function loadStav()
    {
        if (isset($_GET['stav'])) {
            $_SESSION['pers_procesy_stav'] = $_GET['stav'];
        }

        $this->stav = isset($_SESSION['pers_procesy_stav']) ? $_SESSION['pers_procesy_stav'] : 2;
    }

    function loadData()
    {
        $this->loadZobrazeni();
        $this->loadTypProcesu();
        $this->loadStav();

        if ($this->typZobrazeni == 0) {
            $this->nastavOsoby(false);
            $this->nastavRozdeleni(false);
            $this->nastavRozdeleni2(false);
        }

        parent::loadData();
    }

    public function renderTypProcesu()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo TextManager::_('Pers_procesy_typ');
        echo '</SPAN>';


        // combobox pro vyber
        $params = $this->database->createParameters();
        $sql =
            "SELECT Kod, Popis 
            FROM TypPersProcesu
            ORDER BY TypPersProcesu.Popis";

        // vyber typu procesu podle popisu
        $combo = new ComboBox("typprocesu");
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, $this->typProcesu, $params);
        $combo->render();

        echo "</DIV>\n";
    }

    function renderFilters()
    {
        LinksList::renderLinks(
            [0 => TextManager::_('Pers_procesy_moje'), 1 => TextManager::_('Pers_procesy_vyplneni')], $this->typZobrazeni,'?menuid=personalniprocesy&zobrazeni=');
        $this->renderTypProcesu();
        parent::renderFilters();

        LinksList::renderLinks(
            [
//                1 => TextManager::_('Pers_procesy_stav_1'),
                2 => TextManager::_('Pers_procesy_stav_2'),
                11 => TextManager::_('Pers_procesy_stav_11'),
                12 => TextManager::_('Pers_procesy_stav_12'),
                13 => TextManager::_('Pers_procesy_stav_13'),
                14 => TextManager::_('Pers_procesy_stav_14'),
                128 => TextManager::_('Pers_procesy_stav_128')
            ],
            $this->stav, '?menuid=personalniprocesy&stav='
        );
    }
}

class StavProcesuBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ($value) {
            case 1:
            default:
                return TextManager::_('Pers_procesy_stav_1');
            case 2:
                return TextManager::_('Pers_procesy_stav_2');
            case 128:
                return TextManager::_('Pers_procesy_stav_128');
        }
    }

}

class PersonalniProcesyBrowser extends RonPortalBrowser
{
    const BROWSER_PRAVA_INDEX = 23;

    /** @var PersonalniProcesyFilterForm */
    public $filterForm;

    public $pravaProhlizeni;
    public $pravaEditovat;

    public function __construct(Database $database, PravaNaOsoby $pravaProhlizeni, PravaNaOsoby $pravaEditovat)
    {
        parent::__construct($database, 'personalniprocesy');

        $this->pravaProhlizeni = $pravaProhlizeni;
        $this->pravaEditovat = $pravaEditovat;

        $this->title = TextManager::_('Pers_procesy');

        $this->code = 'browserpersonalniprocesy';
        $this->browserUrl = '?menuid=personalniprocesy';

        $this->sqlSelectPart =
            "PersProces.ID, PersProces.RC, PersProces.OsobniCislo, PersProces.DatumNastupu, PersProces.KodTypuProcesu, PersProces.ZalozilRC, PersProces.ZalozilDatum, " . // 0 - 6
            "PersProces.DatumUkonceniPozadovany, PersProces.DatumUkonceniSkutecny, PersProces.UkoncilRC, PersProces.Stav, O_Zalozil.Jmeno, O_Zalozil.Prijmeni, O_Zalozil.Titul, " . // 7 - 13
            "{fn_CASTWIDEMEMO(PersProces.PoznamkaZamestnance)}, {fn_CASTWIDEMEMO(ZalozilPoznamka)}, O_Ukoncil.Jmeno, O_Ukoncil.Prijmeni, O_Ukoncil.Titul, TypPersProcesu.Popis, ". // 14 - 19
            "O_Procesu.Jmeno, O_Procesu.Prijmeni, O_Procesu.Titul, TypPersProcesu.DefinicePrava"; // 20 - 23

        $this->sqlFromPart =
            "PersProces JOIN TypPersProcesu ON (TypPersProcesu.Kod = PersProces.KodTypuProcesu) " .
            " LEFT OUTER JOIN Osoba O_Zalozil ON (O_Zalozil.RC = PersProces.ZalozilRC)" .
            " LEFT OUTER JOIN Osoba O_Ukoncil ON (O_Ukoncil.RC = PersProces.UkoncilRC)" .
            " LEFT OUTER JOIN Osoba O_Procesu ON (O_Procesu.RC = PersProces.RC)";

        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('TypPersProcesu.Popis', 19)
            ->setHeaderText('Pers_procesy_typ')
            ->setCode('col-popis-procesu')
            ->setOrdering(true)
            ->setLink('?menuid=personalniprocesy');

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 20, 21, null, 22))
            ->setSqlFieldName('O_Procesu.Prijmeni, O_Procesu.Jmeno, O_Procesu.Titul')
            ->setHeaderText('Pers_procesy_osoba')
            ->setCode('col-osoba')
            ->setOrdering(true);

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 11, 12, null, 13))
            ->setSqlFieldName('O_Zalozil.Prijmeni, O_Zalozil.Jmeno, O_Zalozil.Titul')
            ->setHeaderText('Pers_procesy_zalozil')
            ->setCode('col-zalozil')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('PersProces.ZalozilDatum', 6)
            ->setHeaderText('Pers_procesy_zalozeno_datum')
            ->setFormat('datetime')
            ->setCode('col-zalozeno-datum-cas')
            ->setOrdering(true);

        $this->addField(new StavProcesuBrowserField())
            ->setSqlFieldName('PersProces.Stav', 10)
            ->setHeaderText('Pers_procesy_stav')
            ->setCode('col-stav')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('PersProces.DatumUkonceniPozadovany', 7)
            ->setHeaderText('Pers_procesy_datum_ukonceni_pozadovane')
            ->setFormat('date')
            ->setCode('col-datum-ukonceni-pozadavek')
            ->setOrdering(true);

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 16, 17, null, 18))
            ->setSqlFieldName('O_Ukoncil.Prijmeni, O_Ukoncil.Jmeno, O_Ukoncil.Titul')
            ->setHeaderText('Pers_procesy_ukoncil')
            ->setCode('col-ukoncil')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('PersProces.DatumUkonceniSkutecny', 8)
            ->setHeaderText('Pers_procesy_datum_ukonceni_skutecne')
            ->setFormat('date')
            ->setCode('col-datum-ukonceni-skutecnost')
            ->setOrdering(true);
    }

    function setFilterForm()
    {
        $this ->filterForm = new PersonalniProcesyFilterForm($this ->database, $this ->action);

        $prava = $this->pravaProhlizeni;

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
        $this->filterForm->povolitVsechyOsoby = true;
        $this->filterForm->setUseGet(true, 'personalniprocesy');
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        switch ($this->filterForm->typZobrazeni) {
            case 0:
            default:
                $sql = "PersProces.RC = " . $params->addParameter(User::getRC());
                break;
            case 1:
                if (!$this->filterForm->vsechnyOsoby) {
                    $sql =
                    "     PersProces.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) .
                    " AND PersProces.OsobniCislo = " . $params->addParameter($this->filterForm->vybranaOsobaOsobniCislo) .
                    " AND PersProces.DatumNastupu = " . $params->addParameter($this->database->dateToParam($this->filterForm->vybranaOsobaDatumNastupu));
                } else {
                    $sql =
                        " (" . $this->pravaProhlizeni->sqlFiltr('PersProces', DateUtils::today(), DateUtils::today(), $params) . " OR " .
                        $this->pravaEditovat->sqlFiltr('PersProces', DateUtils::today(), DateUtils::today(), $params) . ")";

                    $sql .= " AND ";

                    if ($this->pravaProhlizeni->priznak || $this->pravaEditovat->priznak) {
                        $sql .= "1 = 1";
                    } else {
                        $sql .= PersonalniProcesyController::getSQLFilterProcesy($params);
                    }

                    if ($this->filterForm->kodRozdeleni) {
                        $sql .= " AND " .
                            RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PersProces', $this->filterForm->f_typrozdeleni, $this->filterForm->kodRozdeleni, DateUtils::today(), DateUtils::today(),
                                $this->filterForm->vcetnePodrizenychRozdeleni, $params);
                    }
                    if ($this->filterForm->kodRozdeleni2) {
                        $sql .= " AND " .
                            RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PersProces', $this->filterForm->f_typrozdeleni2, $this->filterForm->kodRozdeleni2, DateUtils::today(), DateUtils::today(),
                                $this->filterForm->vcetnePodrizenychRozdeleni2, $params);
                    }
                }
                break;
        }

        if ($this->filterForm->stav) {
            $sql .=
                " AND PersProces.Stav = " . $params->addParameter($this->filterForm->stav);
        } else {
            $sql .=
                " AND PersProces.Stav > " . $params->addParameter(1);
        }

        if ($this->filterForm->typProcesu) {
            $sql .=
                " AND PersProces.KodTypuProcesu = " . $params->addParameter($this->filterForm->typProcesu);
        }

        return new SqlStatement($sql, $params);
    }


}
