<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 19.07.2019
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\StringUtils;

require_once 'adsapp/ronportalbrowser.php';
require_once 'ron/httputils.php';


/**
 * Class SchvalovaniOsobNaPlanuKurzuFilter
 * @package Ron\RonPortal
 */
class SchvalovaniOsobNaPlanuKurzuFilter extends RonPortalFilterForm
{
    /** @var PravaNaOsoby $prava */
    public $pravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;


    /**
     * SchvalovaniOsobNaPlanuKurzuFilter constructor.
     * @param $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     * @param $pravoPrihlasit
     * @param $pravoSchvalit
     * @param bool $useGet
     */
    public function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni, $pravoPrihlasit, $pravoSchvalit, $useGet = false)
    {
        parent::__construct($database, $useGet);
        $this->pravoPrihlasit = $pravoPrihlasit;
        $this->pravoSchvalit = $pravoSchvalit;
        Prava::nastavObdobiProPrava(time(),time());

        $this->nastavDatumOdDo(true,false);

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrPrihlaseniNaPlanovaneSkoleni";
    }

    /**
     * @throws DBException
     */
    public function renderPouzeTypKurzu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypKurzu', \Ron\ronText('pouze_typ_kurzu'). '&nbsp;&nbsp;', $_SESSION['jenTypKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Distinct DruhSkoleni.Kod, DruhSkoleni.Popis
             FROM DruhSkoleni
             WHERE DruhSkoleni.Aktivni = 1
             ORDER BY DruhSkoleni.Popis";

        $combo = new ComboBox('typKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'data-ron-select2="typKurzu_kod" onChange="if (jenTypKurzu.checked) submit();"';
        $combo->dataTypeComparison = false;
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['typKurzu'], $params);
        $combo->render();

        $combo = new ComboBox('typKurzu_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->htmlParams = 'data-ron-select2="typKurzu" data-ron-autosubmit="1"';
        $combo->showCode = 1;
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql, $_SESSION['typKurzu'], $params);
        $combo->render();

        echo '</div>';
    }

    /**
     * @throws DBException
     */
    public function renderPouzeKodKurzu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenKodKurzu', \Ron\ronText('pouze_kod_kurzu'). '&nbsp;&nbsp;', $_SESSION['jenKodKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Distinct PS.CisloSkoleni, PS.Popis
             FROM PlanovaneSkoleni PS
             WHERE PS.Stav <> 2 " .
             "ORDER BY PS.Popis";

        $combo = new ComboBox('kodKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenKodKurzu.checked) submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['kodKurzu'], $params);
        $combo->render();

        $combo = new ComboBox('kodKurzu_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->htmlParams = 'data-ron-select2="kodKurzu" data-ron-autosubmit="1"';
        $combo->showCode = 1;
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql, $_SESSION['kodKurzu'], $params);
        $combo->render();

        echo '</div>';
    }

    public function renderPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            LinksList::renderLinks(
                [
                    'planypodleosoby' => \Ron\ronText('plany_podle_osoby'),
                    'planypodlekurzu' => \Ron\ronText('plany_podle_kurzu')
                ],
                $_SESSION['filtrprihlaseninaplanovaneskoleni'], '?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=');
        }
        echo "<br>";
        require_once 'person/view_poznamkaschvaleniplanukurzu.php';
    }


     private function loadPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        //Moje kurzy odstraneno (na zadost Martina) jelikoz funkcnost nahrazuje "pro osobu"
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            if (isset($_GET['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = $_GET['filtrprihlaseninaplanovaneskoleni'];
            }
            //Pri prvnim zobrazeni stranky se implicitne nastavi Pro osobu
            if (!isset($_SESSION['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = 'planypodleosoby';
            }
        }
    }


    public function renderPouzeNeschvalenePlanyKurzu()
    {
        echo "<div class='filterrow'>";
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('aktualniplanykurzu', \Ron\ronText('filtr_pouze_aktualni_plany') . '&nbsp&nbsp&nbsp', $_SESSION['aktualniplanykurzu'],
                'onClick="submit()"');
        echo "</span>";
        echo "</div>";
    }

    private function loadAktualniPlany()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['aktualniplanykurzu'] = isset($_POST['aktualniplanykurzu']);
        }
    }

    private function loadTypKurzu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypKurzu'] = isset($_POST['jenTypKurzu']);
            $_SESSION['typKurzu'] = $_POST['typKurzu'];
        }
    }

    private function loadKodKurzu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenKodKurzu'] = isset($_POST['jenKodKurzu']);
            $_SESSION['kodKurzu'] = $_POST['kodKurzu'];
        }
    }

    function loadData()
    {
        parent::loadData();
        $this->loadPodMenuPrihlaseniNaPlanovaneSkoleni();
        $this->loadTypKurzu();
        $this->loadKodKurzu();
        $this->loadAktualniPlany();
    }


    /**
     * @throws DBException
     */
    public function renderFilters()
    {
        $this->renderPodMenuPrihlaseniNaPlanovaneSkoleni();
        parent::renderFilters();
        $this->renderPouzeTypKurzu();
        $this->renderPouzeKodKurzu();
        $this->renderPouzeNeschvalenePlanyKurzu();
    }
}


/**
 * Class CasKurzuOsobField
 * @package Ron\RonPortal
 */
class CasKurzuOsobField extends BrowserField
{

    /**
     * CasKurzuOsobField constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('cas_planovaneho_kurzu');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PS.CasPocatkuSkoleni";
        $this->sortSQLDesc = "PS.CasPocatkuSkoleni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= "<div>";
        $result .= StringUtils::format('time', $this->database->stringToTimestamp($rowData[6])) .
            " - " . StringUtils::format('time', $this->database->stringToTimestamp($rowData[7]));
        $result .= "</div>";
        return $result;
    }
}


/**
 * Class KurzyPodrizenehoField
 * @package Ron\RonPortal
 */
class KurzyPodrizenehoField extends BrowserField
{

    /**
     * CasKurzuOsobField constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PS.CasPocatkuSkoleni";
        $this->sortSQLDesc = "PS.CasPocatkuSkoleni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $result .= "<div class=''>";

        $urlNaKurzy = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=planypodleosoby&osoba=$rowData[0]";
        $textOznameni = \Ron\ronText('prejit_na_kurzy_osoby');

        if ($rowData[15] == User::getRC()) {
            $textOznameni = \Ron\ronText('prejit_na_moje_plany');
        }
        $result .= "&nbsp;&nbsp;&nbsp;" . "<a title=''  href='$urlNaKurzy'>$textOznameni</a>" . "&nbsp;&nbsp;&nbsp;";

        $result .= "</div>";

        return $result;
    }
}

/**
 * Class TypKurzuField
 * @package Ron\RonPortal
 */
class TypKurzuField extends BrowserField
{
    public $filtrPrihlaseniNaPlanovaneSkoleni;

    /**
     * TypKurzuField constructor.
     * @param $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     */
    function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('typ_kurzu');
        $this->ordering = 1;
        $this->sortSQL = "PS.DruhSkoleni";
        $this->sortSQLDesc = "PS.DruhSkoleni DESC";

        $this->filtrPrihlaseniNaPlanovaneSkoleni = $filtrPrihlaseniNaPlanovaneSkoleni;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= "<a href='?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$this->filtrPrihlaseniNaPlanovaneSkoleni&druhskoleni=" . HttpUtils::urlEncode($rowData[2]) . "'>";
        $result .= StringUtils::htmlTextConvert($rowData[2]);
        $result .= '</a>';
        return $result;
    }
}

/**
 * Class ObsazenostNaKurzuField
 * @package Ron\RonPortal
 */
class ObsazenostNaKurzuField extends BrowserField
{

    /**
     * PocetOsobNaKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('kapacita_obsazenost_volne');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PSO.RC";
        $this->sortSQLDesc = "PSO.RC DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $vycerpano = (int)((int)$rowData[12]-(int)$rowData[13]) <= 0 ? 'color-red' : '';

        $kapacita = (int)$rowData[12];
        $volno = ((int)$rowData[12]-(int)$rowData[13]);

        $result .= "<div>";
        $result .= "<span title=''>" . $kapacita . "</span>/<span class=' $vycerpano' >" . $volno . "</span>";
        $result .= "</div>";

        return $result;
    }
}


/**
 * Class StavHromadneField
 * @package Ron\RonPortal
 */
class StavHromadneField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('stav');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PSO.Potvrzeni";
        $this->sortSQLDesc = "PSO.Potvrzeni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $stav = "";
        $result = '';

        $stavCiselne = $rowData[9];
        switch ($stavCiselne){
            case 0:
                if ($rowData[9] === null) {
                    $stav = \Ron\ronText('-');
                } else {
                    $stav = \Ron\ronText('osoba_plan_kurzu_prihlasen_neschvaleno');
                }
                break;

            case 1:
                $stav = \Ron\ronText('osoba_plan_kurzu_prihlasen_schvaleno');
                break;

            case 2:
                $stav = \Ron\ronText('osoba_plan_kurzu_zamitnuto');
                break;

        }

        $result .= "<div>";
        $result .= $stav;
        $result .= "</div>";
        return $result;
    }

}


/**
 * Class SchvaleniPrihlaseniOsobyNaKurzuField
 * @package Ron\RonPortal
 */
class SchvaleniPrihlaseniOsobyNaKurzuField extends BrowserField
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;
    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /**
     * SchvaleniPrihlaseniOsobyNaKurzuField constructor.
     * @param Database $database
     * @param $pravoPrihlasit
     * @param $pravoSchvalit
     */
    function __construct($database, $pravoPrihlasit, $pravoSchvalit)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('schvaleni');
        $this->align = 'left';
        $this->ordering = 1;
        $this->sortSQL = "PSO.Potvrzeni";
        $this->sortSQLDesc = "PSO.Potvrzeni DESC";
        $this->pravoPrihlasit = $pravoPrihlasit;
        $this->pravoSchvalit = $pravoSchvalit;
    }



     /**
     * @param $rowData
     * @param $textOznameniPoznamky
     * @return string
     */
    private function schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky)
    {
        $ajaxSchvalit = "<span class='pripojit-poznamku-schvaleni js'  id='schvalit_planek'
                                                data-ron-akce='schvalit'
                                                data-ron-osoba='$rowData[0]'
                                                data-ron-cislokurzu='$rowData[3]'
                                                data-ron-poznamkaschvaleni='$rowData[10]'>$textOznameniPoznamky</span>";

        return $ajaxSchvalit;
    }


    /**
     * @param $rowData
     * @param $textOznameniPoznamky
     * @return string
     */
    private function zamitnoutPrihlaseniAjax($rowData, $textOznameniPoznamky)
    {
        $ajaxSchvalit = "<span class='pripojit-poznamku-schvaleni js'  id='schvalit_planek'
                                                data-ron-akce='zamitnout'
                                                data-ron-osoba='$rowData[0]'
                                                data-ron-cislokurzu='$rowData[3]'
                                                data-ron-poznamkaschvaleni='$rowData[10]'>$textOznameniPoznamky</span>";

        return $ajaxSchvalit;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $volnaMista = (int)((int)$rowData[12]-(int)$rowData[13]);
        $datumPocatkuKurzu = $this->database->stringToTimestamp($rowData[5]);

        //Prava
        $muzePrihlasit = $this->pravoPrihlasit->lzeZobrazitRC(RonPortalSession::osobaRC());
        $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC(RonPortalSession::osobaRC());
        //Url
        $urlSchvaleni = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=schvalit&cisloskoleni=$rowData[3]&osoba=$rowData[0]";
        $urlOdhlaseni = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=zrusit&cisloskoleni=$rowData[3]&osoba=$rowData[0]";
        $urlZamitnuti = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=zamitnout&cisloskoleni=$rowData[3]&osoba=$rowData[0]";
        //Stav
        $jePrihlasen = $rowData[9] === 0;
        $jeSchvalen = $rowData[9] == 1;
        $jeZamitnut = $rowData[9] == 2;



        //Pokud ma pravo na schvaleni ale neni zapsan na planu pak nabidnout prihlaseni a schvaleni jinak pouze prihlaseni
        //Pokud je prihlasen a nema pravo schvalit pak zobrazit pouze text uzivatel je prihlasen
        if ($muzePrihlasit || $muzeSchvalit)
        {
            if ($jeSchvalen)
            {
                //Je schvalen -> je i prihlasen
                if ($datumPocatkuKurzu >= DateUtils::today() && $muzeSchvalit)
                {
                    $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                    $result .= "<div>";
                    $result .= "<a title=''  href='$urlOdhlaseni'>$textOznameni</a>";
                    $result .= "</div>";
                } else {
                    $textOznameni = \Ron\ronText('osoba_je_na_planu');
                    $result .= "<div>$textOznameni</div>";
                }

            } else {
                //Neni schvalen
                if ($jePrihlasen)
                {
                    if ($datumPocatkuKurzu >= DateUtils::today()) {
                        //Je prihlasen, ale neschvalen ->  moznost schvalit nebo ignorovat
                        $textOznameni = \Ron\ronText('');
                        $textOznameniPoznamky = \Ron\ronText('prihlasit_plan_kurzu_poznamka');
                        if ($muzeSchvalit) {
                            $result .= "<div>";
                                $textOznameniPoznamky = \Ron\ronText('schvalit_plan_kurzu_poznamka');
                                $textOznameni = \Ron\ronText('schvalit_plan_skoleni_uzivatel_ma_zajem');
                                $result .= "<a href='$urlSchvaleni'>$textOznameni</a>";
                                $result .=  "&nbsp;&nbsp;&nbsp;" . $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);
                            $result .= "</div>";

                            if (!$jeZamitnut) {
                                $result .= "<div>";
                                $textOznameni = \Ron\ronText('zamitnout');
                                $textOznameniPoznamky = \Ron\ronText('zamitnout_plan_kurzu_poznamka');
                                $result .= "<a href='$urlZamitnuti'>$textOznameni</a>";
                                $result .=  "&nbsp;&nbsp;&nbsp;" . $this->zamitnoutPrihlaseniAjax($rowData, $textOznameniPoznamky);
                                $result .= "</div>";
                            }
                        } else {
                            $result .= "<div>";
                            if ($muzePrihlasit){
                                $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                                $result .= "<a href='$urlOdhlaseni'>$textOznameni</a>";
                            } else {
                                $result .= $textOznameni;
                            }
                            $result .= "</div>";
                        }
                    }

                } else {
                    //Neni prihlasen
                    //Kapacita je
                    if ($volnaMista > 0)
                    {
                        if ($datumPocatkuKurzu >= DateUtils::today())
                        {
                            //Volne misto je -> moznost prihlasit i schvalit
                            $textOznameni = \Ron\ronText('prihlasit_plan_skoleni_uzivatel_neprojevil_zajem');
                            $textOznameniPoznamky = \Ron\ronText('prihlasit_plan_kurzu_poznamka');
                            if ($muzeSchvalit)
                            {
                                $textOznameni = \Ron\ronText('prihlasit_schvalit_plan_skoleni_uzivatel_neprojevil_zajem');
                                $textOznameniPoznamky = \Ron\ronText('prihlasit_schvalit_plan_kurzu_poznamka');
                                $result .= "<div>";
                                $result .= "<a title=''  href='$urlSchvaleni'>$textOznameni</a>";
                                $result .= "&nbsp;&nbsp;&nbsp;";
                                $result .= $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);
                                $result .= "</div>";
                            } elseif ($muzePrihlasit && !$muzeSchvalit){
                                $result .= "<div>";
                                $result .= "<a title=''  href='$urlSchvaleni'>$textOznameni</a>";
                                $result .= "&nbsp;&nbsp;&nbsp;";
                                $result .= $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);

                                $result .= "</div>";
                            }
                        } else {
                            $textOznameni = \Ron\ronText('expirace_kurzu');
                            $result .= "<div>$textOznameni</div>";
                        }

                    }else{
                        //neni prihlasen a neni volne misto

                        if ($datumPocatkuKurzu >= time())
                        {
                            $textOznameni = \Ron\ronText('kapacita_planu_naplnena');
                            $result .= "<div>$textOznameni</div>";
                        } else {
                            $result .= "<div></div>";
                        }
                    }
                }
            }
        }

        return $result;
    }
}


/**
 * Class SchvalovaniPodrizenychNaPlanuKurzuBrowser
 */
class SchvalovaniPodrizenychNaPlanuKurzuBrowser extends Browser
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;
    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /** @var SchvalovaniOsobNaPlanuKurzuFilter $filterForm */
    public $filterForm;

    /**
     * SchvalovaniPodrizenychNaPlanuKurzuBrowser constructor.
     * @param Database $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     * @throws \Exception
     */
    public function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni)
    {
        parent::__construct($database);

        $this->code = 'browserschvalovaniosobpodleplanukurzu';
        $this->tableId = 'schvalovaniosobpodleplanukurzu';
        $this->browserUrl = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrPrihlaseniNaPlanovaneSkoleni";
        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        Prava::nastavObdobiProPrava(time(), time());

        $this->filterForm = new SchvalovaniOsobNaPlanuKurzuFilter($database, $filtrPrihlaseniNaPlanovaneSkoleni, $this->pravoPrihlasit, $this->pravoSchvalit);
        $this->filterForm->loadData();

        $this->sqlSelectPart =
            "PP.ID, COALESCE(O.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(O.Jmeno,''), PS.DruhSkoleni, PS.CisloSkoleni, PS.Popis, " .          //0..4
            "PS.DatumPocatkuSkoleni, PS.CasPocatkuSkoleni, PS.CasUkonceniSkoleni,PS.DatumUkonceniSkoleni, PSO.Potvrzeni, " .            //5..9
            "{fn_CASTWIDEMEMO(PSO.PoznamkaSchvalovani)},{fn_CASTWIDEMEMO(PS.Poznamka)}, PS.PocetOsob AS Kapacita, " .               //10..12
            "(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob Where PlanovaneSkoleniOsob.CisloSkoleni = PS.CisloSkoleni 
                AND PlanovaneSkoleniOsob.Potvrzeni < 2) AS CelkemPrihlasenych, PSO.RC, O.RC";   //13..15

        $this->sqlFromPart =
            "PracovniPomer PP 
                INNER JOIN Osoba O ON (O.RC = PP.RC)
                CROSS JOIN PlanovaneSkoleni PS
                LEFT OUTER JOIN PlanovaneSkoleniOsob PSO ON ((PSO.RC = O.RC) AND (PSO.CisloSkoleni = PS.CisloSkoleni))";

        $this->sqlWherePart =
            "(PP.Zamestnanec = 1  OR  PP.Zamestnanec = 2) 
                 AND PP.DatumNastupu <= " . $this->database->timestampToString(time()) . " 
                 AND (PP.DatumUkonceni >= " . $this->database->timestampToString(time()) .
                    " OR PP.DatumUkonceni IS NULL) 
                 AND PP.ID = (SELECT Max(ID) FROM PracovniPomer WHERE PracovniPomer.RC = PP.RC)";

        $this->defaultSort = -6;

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1 ,
            BROWSER_FIELD_SQL_NAME => "O.Prijmeni, O.Jmeno",
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('Jmeno'),
            BROWSER_FIELD_ORDERING => true,
            BROWSER_FIELD_ALIGN => 'right']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2,
            BROWSER_FIELD_SQL_NAME => 'PS.DruhSkoleni',
            BROWSER_FIELD_HEADER_TEXT => 'typ_skoleni',
            BROWSER_FIELD_ALIGN => 'right',
            BROWSER_FIELD_CLASS_NAME => '',
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 3,
            BROWSER_FIELD_SQL_NAME => 'PS.CisloSkoleni',
            BROWSER_FIELD_HEADER_TEXT => 'cislo_skoleni',
            BROWSER_FIELD_ALIGN => 'left',
            BROWSER_FIELD_CLASS_NAME => '',
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 4,
            BROWSER_FIELD_SQL_NAME => 'PS.Popis',
            BROWSER_FIELD_HEADER_TEXT => 'popis_planu_kurzu',
            BROWSER_FIELD_ALIGN => 'center',
            BROWSER_FIELD_CLASS_NAME => 'tucne',
            BROWSER_FIELD_ORDERING => 1]);

        //Sloupec s proklikem na Schvalovani kurzu podrizeneho
        $this->addField(new KurzyPodrizenehoField($database));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 5,
            BROWSER_FIELD_SQL_NAME => 'PS.DatumPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('datum_pocatku_skoleni'),
            BROWSER_FIELD_FORMAT => 'date',
            BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 8,
            BROWSER_FIELD_SQL_NAME => 'PS.DatumUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('datum_ukonceni_skoleni'),
            BROWSER_FIELD_FORMAT => 'date',
            BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new ObsazenostNaKurzuField($database));

        $this->addField(new StavHromadneField($database));

        $this->addField(new SchvaleniPrihlaseniOsobyNaKurzuField($database, $this->pravoPrihlasit, $this->pravoSchvalit));
    }


    public function renderFilter()
    {
        $this->filterForm->render();
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        //PSO.PoznamkaSchvalovani: PSO.PoznamkaSchvalovani
        if ($values[10]) {
            echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 4) . '" style="width:12rem;">';
                    echo '<span class="info tucne">' . \Ron\ronText('poznamka_schvaleni') . ':&nbsp;&nbsp;&nbsp;&nbsp;' . '</span>';
                    echo '<span class="info">' . StringUtils::htmlTextConvert($values[10]) . '</span>';
                echo '<td colspan="2"></td>';
            echo '</tr>';
        }
    }

    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        //Overeni prav
        $result =
            '(' .
            $this->pravoPrihlasit->sqlFiltr('PP',time(),time()) .
            ' OR ' .
            $this->pravoSchvalit->sqlFiltr('PP',time(),time()) .
            ')';

        $result .= " AND PS.Stav < 2 ";

        if($_SESSION['jenTypKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " PS.DruhSkoleni = " . $params->addParameter($_SESSION['typKurzu']);
        }

        if (RonPortalSession::getFiltrovatDatum()) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                ' AND PS.DatumPocatkuSkoleni <= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumDo()));
        }


        if($_SESSION['aktualniplanykurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(DateUtils::today()));
        }

        if($_SESSION['jenKodKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " PS.CisloSkoleni = " . $params->addParameter($_SESSION['kodKurzu']);
        }

        return new SqlStatement($result, $params);
    }
}
