<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 17.07.2019
 */

namespace Ron\RonPortal;


use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;

require_once('adsapp/ronportalfilter.php');
require_once('ron/browser.php');



/**
 * Class SchvalovaniPlanuKurzuOsobyFilter
 * @package Ron\RonPortal
 */
class SchvalovaniPlanuKurzuOsobyFilter extends RonPortalFilterForm
{
/** @var PravaNaOsoby $pravaPrihlasit */
    public $pravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;


    /**
     * SchvalovaniPlanuKurzuOsobyFilter constructor.
     * @param $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     * @param bool $useGet
     */
    public function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni, $useGet = false)
    {
        parent::__construct($database, $useGet);
        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrPrihlaseniNaPlanovaneSkoleni";


        $this ->nastavOsoby(true, [Prava::vratPrava('skoleni_prihlasit'), Prava::vratPrava('skoleni_schvaleni_planu')], time(), time()); // prava podle aktualniho casu
        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        Prava::nastavObdobiProPrava(time(),time());

        $this->nastavDatumOdDo(true,false);
    }


    public function renderPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            LinksList::renderLinks(
                [
                    'planypodleosoby' => \Ron\ronText('plany_podle_osoby'),
                    'planypodlekurzu' => \Ron\ronText('plany_podle_kurzu')
                ],
                $_SESSION['filtrprihlaseninaplanovaneskoleni'], '?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=');
        }
        echo "<br>";
    }


    /**
     * @throws DBException
     */
    public function renderPouzeTypKurzu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypKurzu', \Ron\ronText('skoleni_druh'). '&nbsp;&nbsp;', $_SESSION['jenTypKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $params = $this->database->createParameters();

        $sql =
            "SELECT Distinct DruhSkoleni.Kod, DruhSkoleni.Popis
             FROM DruhSkoleni
             WHERE DruhSkoleni.Aktivni = 1
             ORDER BY DruhSkoleni.Popis";

        $combo = new ComboBox('typKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->showCode = 1;
        $combo->htmlParams = 'data-ron-select2="typKurzu_kod" onChange="if (jenTypKurzu.checked) submit();"';
        $combo->loadData($this->database, $sql, $_SESSION['typKurzu'], $params);
        $combo->dataTypeComparison = false;
        $combo->render();

        $combo = new ComboBox('typKurzu_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->htmlParams = 'data-ron-select2="typKurzu" data-ron-autosubmit="1"';
        $combo->captionIndex = 0;
        $combo->showCode = 1;
        $combo->keyIndex = 0;
        $combo->dataTypeComparison = false;
        $combo->loadData($this->database, $sql, $_SESSION['typKurzu'], $params);
        $combo->render();

        echo '</div>';
    }


    public function renderPouzeNeschvalenePlanyKurzu()
    {
        echo "<div class='filterrow'>";
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('aktualniplanykurzu', \Ron\ronText('filtr_pouze_aktualni_plany') . '&nbsp&nbsp&nbsp', $_SESSION['aktualniplanykurzu'],
                'onClick="submit()"');
        echo "</span>";

        echo '<span class="">';
            GuiUtils::renderCheckbox('neschvaleneplanykurzu', \Ron\ronText('filtr_pouze_neschvalene') . '&nbsp&nbsp&nbsp', $_SESSION['neschvaleneplanykurzu'],
                'onClick="submit()"');
        echo "</span>";

        echo '<span class="">';
            GuiUtils::renderCheckbox('pouzenaplanukurzu', \Ron\ronText('filtr_pouze_na_planu_kurzu') . '&nbsp&nbsp&nbsp', $_SESSION['pouzenaplanukurzu'],
                'onClick="submit()"');
        echo "</span>";
        echo "</div>";

        require_once 'person/view_poznamkaschvaleniplanukurzu.php';
    }


    private function loadPodMenuPrihlaseniNaPlanovaneSkoleni()
    {
        //Moje kurzy odstraneno (na zadost Martina) jelikoz funkcnost nahrazuje "pro osobu"
        if ($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava()) {
            if (isset($_GET['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = $_GET['filtrprihlaseninaplanovaneskoleni'];
            }
            //Pri prvnim zobrazeni stranky se implicitne nastavi Pro osobu
            if (!isset($_SESSION['filtrprihlaseninaplanovaneskoleni'])) {
                $_SESSION['filtrprihlaseninaplanovaneskoleni'] = 'planypodleosoby';
            }
        }
    }


    private function loadNeschvalene()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['neschvaleneplanykurzu'] = isset($_POST['neschvaleneplanykurzu']);
        }
    }

    private function loadAktualniPlany()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['aktualniplanykurzu'] = isset($_POST['aktualniplanykurzu']);
        }
    }

    private function loadPouzeNaPlanu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['pouzenaplanukurzu'] = isset($_POST['pouzenaplanukurzu']);
        }
    }

    private function loadTypKurzu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypKurzu'] = isset($_POST['jenTypKurzu']);
            $_SESSION['typKurzu'] = $_POST['typKurzu'];
        }
    }

    /**
     * @throws Exception
     */
    function loadData()
    {
        parent::loadData();
        $this->loadTypKurzu();
        $this->loadPodMenuPrihlaseniNaPlanovaneSkoleni();
        $this->loadNeschvalene();
        $this->loadAktualniPlany();
        $this->loadPouzeNaPlanu();
    }


    /**
     * @throws DBException
     */
    function renderFilters()
    {
        $this->renderPodMenuPrihlaseniNaPlanovaneSkoleni();
        parent::renderFilters();
        $this->renderPouzeTypKurzu();
        $this->renderPouzeNeschvalenePlanyKurzu();
    }
}


/**
 * Class PopisSkoleniField
 * @package Ron\RonPortal
 */
class PopisSkoleniField extends BrowserField
{

    /** @var PravaNaOsoby $pravoNaPlan */
    public $pravoNaPlan;

    /**
     * PopisSkoleniField constructor.
     * @param $database
     * @param $pravoNaPlan
     */
    function __construct($database, $pravoNaPlan)
    {
        parent::__construct($database);
        $this->pravoNaPlan = $pravoNaPlan;
        $this->header = "";
        $this->align = 'center';
        $this->ordering = 0;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if (($this->pravoNaPlan->lzeZobrazitRC(($rowData[14]) )&& $rowData[14] != "") || $this->pravoNaPlan->prava == 3)
        {
            $result .= "<a href='?menuid=skoleni&action=detailplanovanehoskolenikurzu&record=$rowData[0]&filtrprihlaseninaplanovaneskoleni=planypodleosoby' target='_blank'>";
            $result .= '<i class="inline-glyph fa glyph-edit" title="' . \Ron\ronText('editace_planu_kurzu') . '"></i>';
            $result .= "</a>";
        }

        return $result;
    }
}


/**
 * Class CasKurzuField
 * @package Ron\RonPortal
 */
class CasKurzuField extends BrowserField
{

    /**
     * CasKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('cas_planovaneho_kurzu');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "PS.CasPocatkuSkoleni";
        $this->sortSQLDesc = "PS.CasPocatkuSkoleni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= "<div>";
        $result .= StringUtils::format('time', $this->database->stringToTimestamp($rowData[4])) .
            " - " . StringUtils::format('time', $this->database->stringToTimestamp($rowData[5]));
        $result .= "</div>";
        return $result;
    }
}


/**
 * Class StavNaKurzuField
 * @package Ron\RonPortal
 */
class StavNaKurzuField extends BrowserField
{

    /**
     * PocetOsobNaKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('stav');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "Schvaleni";
        $this->sortSQLDesc = "Schvaleni DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $stav = "";

        $stavCiselne = $rowData[10];
        switch ($stavCiselne){
            case 0:
                if ($rowData[9] == 1) {
                    $stav = \Ron\ronText('osoba_plan_kurzu_prihlasen_neschvaleno');
                } else {
                    $stav = \Ron\ronText('-');
                }
                break;

            case 1:
                $stav = \Ron\ronText('osoba_plan_kurzu_prihlasen_schvaleno');
                break;

            case 2:
                $stav = \Ron\ronText('osoba_plan_kurzu_zamitnuto');
                break;

        }


        $result .= "<div>";
        $result .= $stav;
        $result .= "</div>";
        return $result;
    }
}


/**
 * Class PocetOsobNaKurzuField
 * @package Ron\RonPortal
 */
class PocetOsobNaKurzuField extends BrowserField
{

    /**
     * PocetOsobNaKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = \Ron\ronText('kapacita_obsazenost_volne');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = "Kapacita";
        $this->sortSQLDesc = "Kapacita DESC";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $vycerpano = (int)((int)$rowData[7]-(int)$rowData[8]) <= 0  ? 'color-red' : '';

        $kapacita = (int)$rowData[7];
        $volno = ((int)$rowData[7]-(int)$rowData[8]);

        $result .= "<div>";
        $result .= "<span>" . $kapacita . "</span>/<span class='$vycerpano'>" . $volno . "</span>";
        $result .= "</div>";
        return $result;
    }
}

/**
 * Class SchvaleniPrihlaseniNaKurzuField
 * @package Ron\RonPortal
 */
class SchvaleniPrihlaseniNaKurzuField extends BrowserField
{
    /** @var PravaNaOsoby $pravaPrihlasit */
    public $pravoPrihlasit;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /**
     * SchvaleniPrihlaseniNaKurzuField constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrihlasit
     * @param PravaNaOsoby $pravoSchvalit
     */
    function __construct($database, $pravoPrihlasit, $pravoSchvalit)
    {
        parent::__construct($database);
        $this->align = 'left';
        $this->sortSQL = "PSO.Potvrzeni";
        $this->ordering = 1;
        $this->sortSQLDesc = "PSO.Potvrzeni DESC";
        $this->pravoPrihlasit = $pravoPrihlasit;
        $this->pravoSchvalit = $pravoSchvalit;
    }


    /**
     * @param $rowData
     * @param $textOznameniPoznamky
     * @return string
     */
    private function schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky)
    {
        $ajaxSchvalit = "<span class='pripojit-poznamku-schvaleni js' id='schvalit_planek'
                                                data-ron-akce='schvalit'
                                                data-ron-osoba='$rowData[13]'
                                                data-ron-cislokurzu='$rowData[0]'
                                                data-ron-poznamkaschvaleni='$rowData[11]'>$textOznameniPoznamky</span>";

        return $ajaxSchvalit;
    }


    /**
     * @param $rowData
     * @param $textOznameniPoznamky
     * @return string
     */
    private function zamitnoutPrihlaseniAjax($rowData, $textOznameniPoznamky)
    {
        $ajaxSchvalit = "<span class='pripojit-poznamku-schvaleni js' id='schvalit_planek'
                                                data-ron-akce='zamitnout'
                                                data-ron-osoba='$rowData[13]'
                                                data-ron-cislokurzu='$rowData[0]'
                                                data-ron-poznamkaschvaleni='$rowData[11]'>$textOznameniPoznamky</span>";

        return $ajaxSchvalit;
    }



    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $volnaMista = (int)((int)$rowData[7]-(int)$rowData[8]);
        $datumPocatkuKurzu = $this->database->stringToTimestamp($rowData[3]);
        $muzePrihlasit = $this->pravoPrihlasit->lzeZobrazitRC(RonPortalSession::osobaRC());
        $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC(RonPortalSession::osobaRC());
        //Pokud ma pravo schvalit osobu
        $urlSchvaleni = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=schvalit&cisloskoleni=$rowData[0]&osoba=$rowData[13]";
        //Odhlasit - zrusit (odstrani se zaznam)
        $urlOdhlaseni = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=zrusit&cisloskoleni=$rowData[0]&osoba=$rowData[13]";
        $urlZamitnuti = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&plankurzuzamestnance=zamitnout&cisloskoleni=$rowData[0]&osoba=$rowData[13]";
        $jePrihlasen = $rowData[10] == 0 && $rowData[9] == 1;
        $jeSchvalen = $rowData[10] == 1;
        $jeZamitnut = $rowData[10] == 2;

        //Pokud ma pravo na schvaleni ale neni zapsan na planu pak nabidnout prihlaseni a schvaleni jinak pouze prihlaseni
        //Pokud je prihlasen a nema pravo schvalit pak zobrazit pouze text uzivatel je prihlasen
        if ($muzePrihlasit || !$muzeSchvalit) {

            if ($jeSchvalen)
            {
                //Je schvalen -> je i prihlasen
                if ($datumPocatkuKurzu > time() && $muzeSchvalit)
                {
                    $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                    $result .= "<div>";
                    $result .= "<a title=''  href='$urlOdhlaseni'>$textOznameni</a>";
                    $result .= "</div>";

                } else {
                    $textOznameni = \Ron\ronText('osoba_je_na_planu');
                    $result .= "<div>$textOznameni</div>";
                }

            } else {
                //Neni schvalen
                if ($jePrihlasen)
                {
                    if ($datumPocatkuKurzu > time())
                    {
                        //Je prihlasen, ale neschvalen ->  moznost chvalit nebo ignorovat
                        $textOznameni = \Ron\ronText('osoba_je_prihlasena_na_planu');

                        $result .= "<div>";

                            if ($muzeSchvalit) {
                                $result .= "<div>";
                                    $textOznameni = \Ron\ronText('schvalit_plan_skoleni_uzivatel_ma_zajem');
                                    $textOznameniPoznamky = \Ron\ronText('schvalit_plan_kurzu_poznamka');
                                    $result .= "<a href='$urlSchvaleni'>$textOznameni</a>";
                                    $result .= "&nbsp;&nbsp;&nbsp;";
                                    $result .= $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);
                                $result .= "</div>";

                                if (!$jeZamitnut) {
                                    $result .= "<div>";
                                    $textOznameni = \Ron\ronText('zamitnout');
                                    $textOznameniPoznamky = \Ron\ronText('zamitnout_plan_kurzu_poznamka');
                                    $result .= "<a href='$urlZamitnuti'>$textOznameni</a>";
                                    $result .= "&nbsp;&nbsp;&nbsp;";
                                    $result .= $this->zamitnoutPrihlaseniAjax($rowData, $textOznameniPoznamky);
                                    $result .= "</div>";
                                }

                            } else {
                                if ($muzePrihlasit){
                                    $textOznameni = \Ron\ronText('odhlasit_z_planu_skoleni');
                                    $result .= "<a title=''  href='$urlOdhlaseni'>$textOznameni</a>";
                                } else {
                                    $result .= $textOznameni;
                                }
                            }

                        $result .= "</div>";
                    }

                } else {
                    //Neni prihlasen
                    //Kapacita je
                    if ($volnaMista > 0)
                    {
                        if ($datumPocatkuKurzu > time()) {
                            //Volne misto je -> moznost prihlasit i schvalit
                            $textOznameni = \Ron\ronText('prihlasit_plan_skoleni_uzivatel_neprojevil_zajem');
                            $textOznameniPoznamky = \Ron\ronText('prihlasit_plan_kurzu_poznamka');
                            $result .= "<div>";
                            if ($muzeSchvalit) {
                                $textOznameni = \Ron\ronText('prihlasit_schvalit_plan_skoleni_uzivatel_neprojevil_zajem');
                                $textOznameniPoznamky = \Ron\ronText('prihlasit_schvalit_plan_kurzu_poznamka');
                                $result .= "<a href='$urlSchvaleni'>$textOznameni</a>";
                                $result .= "&nbsp;&nbsp;&nbsp;";
                                $result .= $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);
                            }elseif ($muzePrihlasit && !$muzeSchvalit){
                                $result .= "<div>";
                                $result .= "<a title=''  href='$urlSchvaleni'>$textOznameni</a>";
                                $result .= "&nbsp;&nbsp;&nbsp;";
                                $result .= $this->schvalitPrihlaseniAjax($rowData, $textOznameniPoznamky);

                                $result .= "</div>";
                            }
                        } else {
                            $textOznameni = \Ron\ronText('expirace_kurzu');
                            $result .= "<div>$textOznameni</div>";
                        }
                    }else{
                        //neni prihlasen a neni volne misto
                        if ($datumPocatkuKurzu > time()) {
                            $textOznameni = \Ron\ronText('kapacita_planu_naplnena');
                            $result .= "<div>$textOznameni</div>";
                        } else {
                            $result .= "<div></div>";
                        }
                    }
                }
            }
        }

        return $result;
    }
}


/**
 * Class SchvalovaniPlanuKurzuPodrizenehoBrowser
 */
class SchvalovaniPlanuKurzuPodrizenehoBrowser extends Browser
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;
    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /** @var PravaNaOsoby $pravoPlan */
    public $pravoPlan;

    public $filterForm;

    /**
     * SchvalovaniPlanuKurzuPodrizenehoBrowser constructor.
     * @param Database $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     * @throws Exception
     */
    public function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni)
    {
        parent::__construct($database);

        $this->code = 'browserschvalovaniplanukurzuosoby';
        $this->tableId = 'schvalovaniplanukurzuosoby';
        $this->browserUrl = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrPrihlaseniNaPlanovaneSkoleni";
        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        $this->pravoPlan = Prava::vratPrava('skoleni_plan');

        Prava::nastavObdobiProPrava(time(), time());

        $this->filterForm = new SchvalovaniPlanuKurzuOsobyFilter($database, $filtrPrihlaseniNaPlanovaneSkoleni);
        $this->filterForm->loadData();

        $this->sqlSelectPart =
            "PS.CisloSkoleni, PS.Popis, PS.DruhSkoleni, PS.DatumPocatkuSkoleni, PS.CasPocatkuSkoleni, " .        //0..4
            "PS.CasUkonceniSkoleni, PS.DatumUkonceniSkoleni, PS.PocetOsob AS Kapacita, " .                      //5..7
            "(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.Potvrzeni < 2 ) AS CelkemPrihlasenych, " .     //8

            "(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.RC = '" . RonPortalSession::osobaRC() . "') AS JePrihlasen," .            //9

            "(SELECT PSO.Potvrzeni FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.RC = '" . RonPortalSession::osobaRC() . "') AS Schvaleni," .            //10

            "(SELECT {fn_CASTWIDEMEMO(PSO.PoznamkaSchvalovani)} FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.RC = '" . RonPortalSession::osobaRC() . "'),{fn_CASTWIDEMEMO(PS.Poznamka)}, " .        //11..12

            "(SELECT MAX(ID) FROM PracovniPomer Where PracovniPomer.RC = '" . RonPortalSession::osobaRC() . "')," .                           //13
            "(SELECT RC FROM PlanovaneSkoleniOsob 
                            Where DatumVlozeni = (Select MIN(DatumVlozeni) From PlanovaneSkoleniOsob Where PlanovaneSkoleniOsob.CisloSkoleni = PS.CisloSkoleni))"; //14

        $this->sqlFromPart = "PlanovaneSkoleni PS";


        $this->addField(new PopisSkoleniField($database, $this->pravoPlan));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2,
            BROWSER_FIELD_SQL_NAME => 'PS.DruhSkoleni',
            BROWSER_FIELD_HEADER_TEXT => 'druh_skoleni',
            BROWSER_FIELD_ALIGN => 'left',
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1,
            BROWSER_FIELD_SQL_NAME => 'PS.Popis',
            BROWSER_FIELD_HEADER_TEXT => 'popis_planu_kurzu',
            BROWSER_FIELD_ALIGN => 'center',
            BROWSER_FIELD_CLASS_NAME => 'tucne',
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 3,
            BROWSER_FIELD_SQL_NAME => 'DatumPocatkuSkoleni',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('datum_pocatku_skoleni'),
            BROWSER_FIELD_FORMAT => 'date',
            BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 6,
            BROWSER_FIELD_SQL_NAME => 'DatumUkonceniSkoleni',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('datum_ukonceni_skoleni'),
            BROWSER_FIELD_FORMAT => 'date',
            BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new PocetOsobNaKurzuField($database));

        $this->addField(new StavNaKurzuField($database));

        $this->addField(new SchvaleniPrihlaseniNaKurzuField($database, $this->pravoPrihlasit, $this->pravoSchvalit));

        $this->defaultSort = -4;
    }


    function renderFilter()
    {
        $this->filterForm->render();
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        //PS.Poznamka : PS.Poznamka
        //PSO.PoznamkaSchvalovani: PSO.PoznamkaSchvalovani
        if ($values[11]) {
            echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 3) . '" style="width:12rem;">';
                    echo '<span class="info tucne">' . \Ron\ronText('poznamka_schvaleni') . ':&nbsp;&nbsp;&nbsp;&nbsp;' . '</span>';
                    echo '<span class="info">' . StringUtils::htmlTextConvert($values[11]) . '</span>';
                echo '<td colspan="3"></td>';
            echo '</tr>';
        }

        if ($values[12]) {
            echo "<tr class='$className'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 3) . '" style="width:12rem;">';
                    echo '<span class="info tucne">' . \Ron\ronText('poznamka_kurzu') . '</strong>:&nbsp;&nbsp;&nbsp;&nbsp;' . '</span>';
                    echo '<span class="info">' . StringUtils::htmlTextConvert($values[12]) . '</span>';
                echo '<td colspan="3"></td>';
            echo '</tr>';
        }
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = "PS.Stav < 2";

        if (RonPortalSession::getFiltrovatDatum()) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                ' AND PS.DatumPocatkuSkoleni <= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumDo()));
        }

        if($_SESSION['jenTypKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " PS.DruhSkoleni = " . $params->addParameter($_SESSION['typKurzu']);
        }

        if($_SESSION['neschvaleneplanykurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= "(SELECT PSO.Potvrzeni FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.RC = " . $params->addParameter(RonPortalSession::osobaRC()) . ") = 0 ";
        }

        if($_SESSION['aktualniplanykurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' PS.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(DateUtils::today()));
        }

        if($_SESSION['pouzenaplanukurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= "(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO 
                Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.RC = " . $params->addParameter(RonPortalSession::osobaRC()) . ") = 1 ";
        }


        return new SqlStatement($result, $params);
    }
}
