<?php
/** UTF-8
 * roman.szablatura, 10.10.2016
 */

namespace Ron\Personalistika;

require_once('adsapp/ronportalbrowser.php');
require_once('ron/controls.php');
require_once('adsapp/ronportalfilter.php');
require_once('ron/guiutils.php');

use Exception;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Controls\LinksList;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\RonPortal\RonPortalBrowser;
use Ron\StringUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\User;
use function Ron\ronText;


/**
 * Class UkolFilterForm
 * @package Ron\Personalistika
 */
class UkolFilterForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby */
    var $prava;

    var $ukoluKeSplneni;
    var $ukoluKHodnoceni;
    var $ukoluKeSchvaleni;

    /**
     * UkolFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     * @param $splnit
     * @param $hodnot
     * @param $schval
     */
    function __construct(Database $database, $action, $useGet, $splnit, $hodnot, $schval)
    {
        parent::__construct($database, $action, $useGet);

        $this->f_formaction = '?menuid=ukoly';
        $this->f_classname = 'filtercontainer leftcaptions';
        $this->prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
        $this->nastavOsoby(true,$this->prava,time(),time());
        $this->povolitVsechyOsoby = true;
        //Zobrazeni FilterFormu pro nastaveni data od - do
        $this->nastavDatumOdDo(true, false, ronText('Termin_ukolu') . ' ' . ronText('Od'));

        $this->ukoluKeSplneni = $splnit;
        $this->ukoluKHodnoceni = $hodnot;
        $this->ukoluKeSchvaleni = $schval;
    }

    /**
     * @throws Exception
     */
    function renderTypUkolu()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
            GuiUtils::renderCheckbox('jenTypUkolu', ronText('Typ_ukolu'), isset($_SESSION['jenTypUkolu']) && $_SESSION['jenTypUkolu'],
                'onClick="submit()"');
        echo '</SPAN>';

        $sql = 'SELECT kod, popis FROM KategorieUkolu ORDER BY popis';

        $combo = new ComboBox('typUkolu');
        $combo ->className = 'app_combo adsfilterforminput_full';
        $combo ->htmlParams = 'onChange="if (jenTypUkolu.checked) submit();"';
        $combo ->showCode = 1;
        $combo ->loadData($this->database, $sql, isset($_SESSION['typUkolu']) ? $_SESSION['typUkolu'] : '');
        $combo ->render();

        echo '</DIV>';
    }


    function renderStavUkolu()
    {
        // stav ukolu -  nesplnene, nesplnene po terminu, atd.
        echo '<div class="padding-top-2 padding-bottom-2">';
        //echo '<span>';
            GuiUtils::renderCheckbox('nesplnene', ronText('Nesplnene'), isset($_SESSION['nesplnene']) && $_SESSION['nesplnene'],
                'onClick="submit()"');

            GuiUtils::renderCheckbox('poterminu', ronText('Po_terminu_nesplnene'), isset($_SESSION['poterminu']) && $_SESSION['poterminu'],
                'onClick="submit()"', 'margin-left-2');

            GuiUtils::renderCheckbox('splnene', ronText('Splnene'), isset($_SESSION['splnene']) && $_SESSION['splnene'],
                'onClick="submit()"', 'margin-left-2');

            GuiUtils::renderCheckbox('mam_hodnotit', ronText('Mam_hodnotit'), isset($_SESSION['mam_hodnotit']) && $_SESSION['mam_hodnotit'],
                'onClick="submit()"', 'margin-left-2');

            GuiUtils::renderCheckbox('mam_schvalit', ronText('Mam_schvalit'), isset($_SESSION['mam_schvalit']) && $_SESSION['mam_schvalit'],
                'onClick="submit()"', 'margin-left-2');

        //echo '</span>';
        echo '</div>';
    }

    function renderInfoNovinky()
    {
        if ($this->ukoluKeSplneni == 0  && $this->ukoluKHodnoceni == 0 && $this->ukoluKeSchvaleni == 0) {
            // nic
        } else {
            $url = '?menuid=ukoly';

            if($this->ukoluKeSplneni != 0) {
                $url .= '&nesplnene=1';
            }

            if($this->ukoluKHodnoceni != 0) {
                $url .= '&mam_hodnotit=1';
            }

            if($this->ukoluKeSchvaleni != 0) {
                $url .= '&mam_schvalit=1';
            }

            echo
                '<div class="font-weight-bold ukoly_cekaji" title="' . $this->vytvoreniTitleNovinek() . '">' .
                "<a href='$url'>" . ronText('ukoly_cekajici') . '</a>' .
                '</div>';
        }
    }

    function vytvoreniTitleNovinek()
    {
        $result = '';
        if ($this->ukoluKeSplneni != 0) {
            $result .= ronText('ukolu_ke_splneni'). ": " . $this->ukoluKeSplneni . '&#010';
        }
        if ($this->ukoluKHodnoceni != 0) {
            $result .= ronText('ukolu_k_hodnoceni'). ": " .  $this->ukoluKHodnoceni . '&#010';
        }
        if ($this->ukoluKeSchvaleni != 0) {
            $result .= ronText('ukolu_ke_schvaleni'). ": " . $this->ukoluKeSchvaleni;
        }
        return $result;
    }

    private function loadNesplnene()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['nesplnene'] = isset($_POST['nesplnene']);
        }elseif (isset($_GET['nesplnene'])) {
            $_SESSION['nesplnene'] = 1;
        }elseif ($this->ukoluKeSplneni == 0){
            $_SESSION['nesplnene'] = 0;
        }
    }

    private function loadPoterminu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['poterminu'] = isset($_POST['poterminu']);
        }
    }

    private function loadSplnene(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['splnene'] = isset($_POST['splnene']);
        }
    }

    private function loadMamHodnotit(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['mam_hodnotit'] = isset($_POST['mam_hodnotit']);
        }
        elseif (isset($_GET['mam_hodnotit'])) {
            $_SESSION['mam_hodnotit'] = 1;
        }elseif($this->ukoluKHodnoceni == 0){
            $_SESSION['mam_hodnotit'] = 0;
        }
    }

    private function loadMamSchvalit(){
        if(isset($_POST['adsfilterform'])){
            $_SESSION['mam_schvalit'] = isset($_POST['mam_schvalit']);
        }
        elseif (isset($_GET['mam_schvalit'])) {
            $_SESSION['mam_schvalit'] = 1;
        }elseif ($this->ukoluKeSchvaleni == 0){
            $_SESSION['mam_schvalit'] = 0;
        }
    }

    private function loadTypUkolu()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypUkolu'] = isset($_POST['jenTypUkolu']);
            $_SESSION['typUkolu'] = isset($_POST['typUkolu']) ? $_POST['typUkolu'] : null;
        }
    }

    private function loadVlastnikUkolu()
    {
        //Pokud nema pravo na ostatni pak se hned nastavi na volbu mojeukoly
        if (!$this->prava->pouzeSebe()) {
            if (isset($_GET['filtrresiteleukolu'])) {
                $_SESSION['filtrresiteleukolu'] = $_GET['filtrresiteleukolu'];
            }
            //Pri prvnim zobrazeni stranky se implicitne nastavi Moje ukoly, pokud nema prihlasena osoba pravo na schvaleni nebo hodnoceni
            if (!isset($_SESSION['filtrresiteleukolu'])) {
                $_SESSION['filtrresiteleukolu'] = 'mojeukoly';
            }
        }
    }

    function renderVlastnikUkolu()
    {
        // vlastnik ukolu - moje / vsechny
        if (!$this->prava->pouzeSebe()) {
            LinksList::renderLinks(
                ['mojeukoly' => ronText('Moje_ukoly'), 'ukolyvsech' => ronText('Vsechny_ukoly')],
                $_SESSION['filtrresiteleukolu'], '?menuid=ukoly&filtrresiteleukolu=');
        }
    }

    function loadData()
    {
        $this->loadVlastnikUkolu();
        $this->loadNesplnene();
        $this->loadPoterminu();
        $this->loadSplnene();
        $this->loadMamHodnotit();
        $this->loadMamSchvalit();
        $this->loadTypUkolu();
        parent::loadData();
    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderTypUkolu();
        $this->renderStavUkolu();
        $this->renderVlastnikUkolu();
        $this->renderInfoNovinky();
    }
}

/**
 * Class SplneniUkoluBrowser
 * @package Ron\Personalistika
 */
class SplneniUkoluBrowserField extends BrowserField
{
    function __construct($database, $options)
    {
        parent::__construct($database, $options);

        $this->header = ronText('Splneno');
        $this->ordering = 1;
        $this->link = '?menuid=ukoly';

        $this->sortSQL = 'UkolOsoby.StavUkolu';
        $this->sortSQLDesc = 'UkolOsoby.StavUkolu DESC';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if($rowData[19] == 1){
            $result = ronText('Str_Yes');
        } else if($rowData[19] == 0){
            if($rowData[20] == 1){
                $result = ronText('Prekazka_ukolu');
            } else {
                $result = ronText('Str_No');
            }
        }
        return $result . '&nbsp&nbsp';
    }
}

/**
 * Class Ukol_OsobaField
 * Sloupec se jmenem osoby (resitel, zadavatel)
 */
class Ukol_OsobaField extends BrowserField
{
    //var $sqlFieldIndex;
    var $bold; // urcuje, jestli ma byt bold kdyz je to "moje vlastni"

    /**
     * @param int $sqlFieldIndex
     * @param string $header
     * @param bool $bold
     */
    function __construct($sqlFieldIndex, $header, $bold) // index prvniho pole v sql (pole jsou v poradi jmeno, prijmeni, titul, sql pole dle ktereho se ma radit)
    {
        parent::__construct();

        $this->sqlFieldIndex = $sqlFieldIndex;
        $this->bold = $bold;

        $this ->header = $header;
        $this ->ordering = 1;
        $this->sortSQL = $sqlFieldIndex + 2 ;
        $this->sortSQLDesc = $sqlFieldIndex + 2 . ' DESC';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // prijmeni, jmeno
        $result = StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex + 1]) . ' ' . StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex]);

        // titul
        if ($rowData[$this->sqlFieldIndex + 2]) {
            $result = $result . ', ' .  StringUtils::htmlTextConvert($rowData[$this->sqlFieldIndex + 2]);
        }

        //Pokud radek patri osobe prave prihlasene a parametr bold je true, pak se radek ve sloupci zadano_osobe zvyrazni
        if ($this->bold && User::hasRC($rowData[$this->sqlFieldIndex + 3])){
            $result = '<strong>' . $result . '</strong>';
        }
        return $result;
    }
}

/**
 * Class Ukol_TerminField
 * @package Ron\Personalistika
 */
class Ukol_TerminField extends BrowserField
{
    /**
     * Ukol_TerminField constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database);

        $this ->sqlFieldName = 'UkolOsoby.DatumSplneni_Narizene';
        $this ->header = ronText('Termin_ukolu');
        $this ->format = 'finedate';
        $this ->link = '?menuid=ukoly';
        $this ->ordering = 1;
        $this ->sqlFieldIndex = 9;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($rowData[$this ->sqlFieldIndex], $rowNumber, $recordId, $rowData);

        $class = '';
        //Pokud existuje zaznam DatumSplneni_Skutecne, pak se promenna $class naplni hodnotou 'splnene'
        if ($rowData[10]) {
            $class = 'splnene';
        } else {
            //ulozeni hodnoty datumu - DatumSplneni_Narizene
            $datum = $this ->database ->stringToTimestamp($rowData[$this ->sqlFieldIndex]);
            //Pokud je aktualni datum vyssi nez DatumSplneni_Narizene je pouzito k obarveni data tridy po_terminu
            if (DateUtils::endOfTheDay($datum) < time()) {
                $class = 'po_terminu';
            }
        }

        //Vysledne obarveni datumu na zaklade splneni nebo po_terminu
        if ($class) {
            $result = '<DIV class="' . $class . '">' . $result . '</DIV>';
        }
        return $result;
    }
}

/**
 * Class SeznamUkoluBrowser
 * @package Ron\Personalistika
 */
class SeznamUkoluBrowser extends RonPortalBrowser
{
    var $prava;    //Prava na prohlizeni a "splneni"

    var $ukoluKeSplneni = 0;
    var $ukoluKHodnoceni = 0;
    var $ukoluKeSchvaleni = 0;


    /**
     * SeznamUkoluBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'ukoly');

        $this->prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"

        $this->title = ronText('Seznam_ukolu');
        $this->code = 'browserukoly';

        $this->sqlSelectPart =
            ' UkolOsoby.KodSerial, ' . // 0
            ' Resitel.Jmeno, Resitel.Prijmeni, Resitel.Titul, UkolOsoby.RC_Osoby, ' . // 1..4
            ' Zadavatel.Jmeno, Zadavatel.Prijmeni, Zadavatel.Titul, UkolOsoby.RC_KdoZadal, ' . // 5..8
            ' UkolOsoby.DatumSplneni_Narizene, UkolOsoby.DatumSplneni_Skutecne, ' . // 9,10
            $database ->castWideMemo('UkolOsoby.PodrobnyPopis').', UkolOsoby.Prekazka, UkolOsoby.DatumZapisuPrekazky, ' . // 11..13
            $database ->castWideMemo('UkolOsoby.PoznamkaKPrekazce') . ', UkolOsoby.ZkracenyPopis, ' . // 14,15
            ' KategorieUkolu.Popis, UkolOsoby.DatumZadani,' . // 16,17
            " COALESCE(Hodnotitel.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Hodnotitel.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} Hodnotitel.Titul, '')," .  // 18
            ' UkolOsoby.StavUkolu, UkolOsoby.Prekazka, UkolOsoby.PP_Osoby_OsobniCislo, UkolOsoby.PP_Osoby_DatumNastupu, UkolOsoby.RC_Hodnotitele_Def, ' .     // 19 - 23
            " UkolOsoby.RC_OsobaUkolu, COALESCE(VazanaOsoba.Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(VazanaOsoba.Jmeno,'') {fn_concat()} COALESCE(', ' {fn_concat()} VazanaOsoba.Titul, ''), " .  //24 - 25
            ' KategorieUkolu.PopisVazbyNaOsobu, VazanaOsoba.Prijmeni, UkolOsoby.Hodnoceno, UkolOsoby.Schvaleno1, UkolOsoby.Schvaleno2, ' .    //26 - 30
            ' UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.PP_Hodnotitel_RC, S_AnoNe.Popis, RC_Hodnotitele_Def ';    //31 - 35

        $this->sqlFromPart =
            '((((((UkolOsoby INNER JOIN Osoba Resitel' .
            '           ON UkolOsoby.RC_Osoby = Resitel.RC ' .
            '             )' .
            '        LEFT OUTER JOIN Osoba Zadavatel ' .
            '        ON UkolOsoby.RC_KdoZadal = Zadavatel.RC ' .
            '        )' .
            '       LEFT OUTER JOIN Osoba Hodnotitel ' .
            '       ON UkolOsoby.RC_Hodnotitele_Def = Hodnotitel.RC' .
            '       )'.
            '     LEFT OUTER JOIN Osoba VazanaOsoba ' .
            '     ON UkolOsoby.RC_OsobaUkolu = VazanaOsoba.RC' .
            '     )'.
            '    LEFT OUTER JOIN KategorieUkolu ' .
            '    ON KategorieUkolu.Kod = UkolOsoby.KodKategorieUkolu ' .
            '    )'.
            '   LEFT OUTER JOIN S_AnoNe ' .
            '   ON UkolOsoby.Hodnoceno = S_AnoNe.Kod ' .
            ')';

//        $this->defaultSort = 2;
        $this->primaryField = 0;
        $this->defaultSort = 1;

        $this->najdiUkolyVyzadujiciPozornost($database);

        //Filtr 1 a 2, Osoba a nastaveni Od a Do
        $this->filterForm = new UkolFilterForm($this->database,'ukoly', false, $this->ukoluKeSplneni, $this->ukoluKHodnoceni, $this->ukoluKeSchvaleni);
        $this->filterForm ->loadData();

        // termin
        $terminfield = new Ukol_TerminField($database);
        $this->addField($terminfield);

        //Splneno - Ano/Ne/Prekazka
        $this->addField(new SplneniUkoluBrowserField($this->database, null));

        //Datum splneni
        $this->addField()
            ->setSqlFieldName('UkolOsoby.DatumSplneni_Skutecne', 10) ->setHeaderText('Datum_splneni')
            ->setCode('col-datumsplneni') ->setVisible(isset($_SESSION['splnene']) && ($_SESSION['splnene']))
            ->setFormat('finedate') ->setOrdering(1);

        // Zadano osobe - pokud jsou vybrany vsechny osoby pak sloupec skryt
        $resitelfield = new Ukol_OsobaField(1, ronText('Resitel'), true); // index pole v sql
        $this->addField($resitelfield)
            ->setCode('col-resitel') ->setVisible($this ->filterForm ->vsechnyOsoby);

        // typ ukolu - v KODAMU(kategorie)
        $this->addField()
            ->setSqlFieldName('KategorieUkolu.Popis', 16) ->setHeaderText('Typ_ukolu') ->setOrdering(1)
            ->setCode('col-kategorie');

        // popis
        $this->addField()
            ->setSqlFieldName('UkolOsoby.ZkracenyPopis', 15) ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setLink('?menuid=ukoly') ->setOrdering(1) ->setWrap(1);

        // Datum zadani
        $this->addField()
            ->setSqlFieldName('UkolOsoby.DatumZadani', 17) ->setHeaderText('Datum_zadani')
            ->setCode('col-datumzadani')
            ->setFormat('finedate') ->setOrdering(true);

        // Zadal
        $zadavatelfield = new Ukol_OsobaField(5, ronText('Zadavatel'), false); // index pole v sql
        $this->addField($zadavatelfield)
            ->setCode('col-zadavatel');

        // Hodnotitel
        $this->addField()
            ->setSqlFieldName('19', 18)
            ->setHeaderText('Hodnotitel')
            ->setCode('col-hodnotitel')
            ->setOrdering(1);

        // Hodnoceno
        $this->addField()
            ->setSqlFieldName('Hodnoceno', 34) ->setHeaderText('Hodnoceno') ->setOrdering(1) ->setAlign('center')
            ->setCode('col-hodnoceno');

        // zjisti, jestli existuje typ ukolu s nastavenou vazbou
        $cursor = $database ->getSqlResult('SELECT MAX(PopisVazbyNaOsobu) FROM KategorieUkolu');
        $rowVazba = $cursor ->fetch();
        $cursor ->close();

        // Vazana osoba - popis
        $this->addField()
            ->setSqlFieldName('KategorieUkolu.PopisVazbyNaOsobu', 26) ->setHeaderText('Vazana_osoba_popis') ->setOrdering(1)
            ->setCode('col-popisvazby') ->setVisible(false); // skryto - je v titulku sloupce, lze zobrazit pomoci xml nastaveni

        // Vazana osoba - prijmeni a jmeno
        $this->addField()
            ->setSqlFieldName('VazanaOsoba.Prijmeni', 25) ->setHeaderText('Vazana_osoba_prijmeni_jmeno') ->setOrdering(1)
            ->setCode('col-vazanaosoba') ->setVisible($rowVazba  &&  $rowVazba[0]) // jen pokud existuje nejaky typ ukolu s nastavenou vazbou
            ->setTitleFieldIndex(26);
    }

    function setFilterForm()
    {
        // nedela nic - filterform nastaven uz v konstruktoru
    }


    function load($sqlFilter = '')
    {
        //Pokud nema pravo na ostatni pak se hned nastavi na volbu mojeukoly
//        if (!$this->prava->pouzeSebe()) {
//            if (isset($_GET['filtrresiteleukolu'])) {
//                $_SESSION['filtrresiteleukolu'] = $_GET['filtrresiteleukolu'];
//            }
//            //Pri prvnim zobrazeni stranky se implicitne nastavi Moje ukoly, pokud nema prihlasena osoba pravo na schvaleni nebo hodnoceni
//            if (!isset($_SESSION['filtrresiteleukolu'])) {
//                $_SESSION['filtrresiteleukolu'] = 'mojeukoly';
//            }
//        }
        parent::load($sqlFilter);
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    function najdiUkolyVyzadujiciPozornost($database)
    {
        $params = $database->createParameters();

        // nacte ukoly zgrupovane podle RC zamestnancu a stavu (at nemusi nacitat kazdy zvlast)
        $sql =
            'SELECT RC_Osoby, UkolOsoby.StavUkolu, UkolOsoby.RC_Hodnotitele_Def, UkolOsoby.Hodnoceno,' . // 0 - 3
            ' UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.Schvaleno1, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.Schvaleno2,' . // 4 - 7
            ' COUNT(*)' .   // 8
            ' FROM UkolOsoby' .
            ' WHERE StavUkolu <> -1' .
            ' AND (UkolOsoby.RC_Osoby = ' . $params ->addParameter(User::getRC()) .
            '  OR UkolOsoby.RC_Hodnotitele_Def = ' . $params ->addParameter(User::getRC()) .
            '  OR UkolOsoby.RC_Schvalovatele1_Def = ' . $params ->addParameter(User::getRC()) .
            '  OR UkolOsoby.RC_Schvalovatele2_Def = ' . $params ->addParameter(User::getRC()) .
            ' )' .
            ' GROUP BY RC_Osoby, UkolOsoby.StavUkolu, UkolOsoby.RC_Hodnotitele_Def, UkolOsoby.Hodnoceno,' .
            '  UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.Schvaleno1, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.Schvaleno2';

        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            // Ukoly ke splneni
            if (User::hasRC($row[0]) && !$row[1]) {
                $this->ukoluKeSplneni += $row[8];
            }

            // Ukoly pripravene k hodnoceni
            if (User::hasRC($row[2]) && !$row[3] && $row[1] == 1) {
                $this->ukoluKHodnoceni += $row[8];
            }

            // Ukoly pripravene ke schvaleni 1
            if (User::hasRC($row[4]) && $row[1] == 1 && $row[3] == 1 && $row[5] == 0) {
                $this->ukoluKeSchvaleni += $row[8];
            }

            // Ukoly pripravene ke schvaleni 2
            if (User::hasRC($row[6]) && $row[1] == 1 && $row[5] == 1 && $row[7] == 0) {
                $this->ukoluKeSchvaleni += $row[8];
            }
        }

        $cursor ->close();
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazit pokud ma na radek pravo
        if ($this->prava->lzeZobrazitRC($values[4])) {
            //Nezobrazovat zrusene ukoly
            if($values[19] != -1){
                return true;
            }
        }
        return false;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber);

        if ($values[11]) {
            // podrobny popis
            echo "<tr class='$className row-detailni-popis'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 1) . '"><span class="browserpoznamka">' . ronText('Detailni_popis'). ': '.  $values[11] . '</span></td>';
            echo '</tr>';
        }

        if ($values[12] && ($values[13] || $values[14])) {
            // je zadana prekazka - zobrazi text prekazky
            // Pole 12 - Prekazka, 13 - DatumZapisuPrekazky, 14 - PoznamkaKPrekazce
            echo "<tr class='$className row-prekazka'>";
                echo '<td></td><td colspan="' . ($this->columnCount - 1) . '">';
                echo '<span class="browserpoznamka prekazkaukolu">';
                echo ronText('Prekazka_ukolu');
                if ($values[13]) {
                    echo ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $values[13]) . ') ';
                }
                if ($values[14]) {
                    echo ': ' . StringUtils::htmlTextConvert($values[14]);
                }
                echo '</span>';
                echo '</td>';
            echo '</tr>';
        }
        $this->renderSeparator();
    }


    function getSQLFilter()
    {
        $result = '';
        $resultFilter = '';
        $resultVlsTypDatum = '';
        $resultStavUkolu = '';

    // Filtr rozdeleni $resultFilter
        if (!$this ->filterForm ->vsechnyOsoby) {
            $resultFilter .= " UkolOsoby.RC_Osoby = '" . $this->database->addSlashes($this ->filterForm ->vybranaOsobaRC) . "'\n";
        } else {

            if ($this ->filterForm ->kodRozdeleni != '') {
                $sqlPodrizene = '';

                switch ($this ->filterForm ->vcetnePodrizenychRozdeleni) {
                    case 0:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
                        break;
                    case 1:
                        $sqlPodrizene = '';
                        break;
                    case 2:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
                        break;
                    case 3:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
                        break;
                }

                $resultFilter .=
                    ($resultFilter ? ' AND ' : '') .
                    "EXISTS(SELECT RozdeleniOsob.RC FROM RozdeleniOsob JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni " .
                    "     AND HierarchieRozdeleni.typNadrazene = HierarchieRozdeleni.typPodrizene) " .
                    " where (RozdeleniOsob.RC = UkolOsoby.RC_Osoby ) " .
                    //"and (RozdeleniOsob.osobniCislo = UkolOsoby.PP_Osoby_OsobniCislo )" .
                    //"and (RozdeleniOsob.datumNastupu = UkolOsoby.PP_Osoby_DatumNastupu ) " .
                    " and (RozdeleniOsob.KodTypuRozdeleni = " . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . ")" .
                    " and (HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "') " .
                    $sqlPodrizene .
                    ' and (RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(time()) . ') ' .
                    ' and ((RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(time()) . ') or (RozdeleniOsob.PlatnostDo IS NULL))' .
                    ")\n";
            }

            if ($this ->filterForm ->kodRozdeleni2 != '') {
                $sqlPodrizene = '';

                switch ($this ->filterForm ->vcetnePodrizenychRozdeleni2) {
                    case 0:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
                        break;
                    case 1:
                        $sqlPodrizene = '';
                        break;
                    case 2:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
                        break;
                    case 3:
                        $sqlPodrizene = 'and (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
                        break;
                }

                $resultFilter .=
                    ($resultFilter ? ' AND ' : '') .
                    "EXISTS(SELECT RozdeleniOsob.RC FROM RozdeleniOsob JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni " .
                    "     AND HierarchieRozdeleni.typNadrazene = HierarchieRozdeleni.typPodrizene) " .
                    " where (RozdeleniOsob.RC = UkolOsoby.RC_Osoby ) " .
                    //" and (RozdeleniOsob.osobniCislo = UkolOsoby.PP_Osoby_OsobniCislo )" .
                    //" and (RozdeleniOsob.datumNastupu = UkolOsoby.PP_Osoby_DatumNastupu ) " .
                    " and (RozdeleniOsob.KodTypuRozdeleni = " . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) . ")" .
                    " and (HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni2) . "') " .
                    $sqlPodrizene .
                    ' and (RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(time()) . ') ' .
                    ' and ((RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(time()) . ') or (RozdeleniOsob.PlatnostDo IS NULL))' .
                    ")\n";
            }
        }


    //Filtr vlastnika - $resultVlsTypDatum
        if ($_SESSION['filtrresiteleukolu'] == 'mojeukoly' || $this->prava->pouzeSebe()) {
            if ($resultVlsTypDatum) {
                $resultVlsTypDatum .= ' AND ';
            }
            $resultVlsTypDatum .= " RC_Osoby = '" . $this ->database ->addSlashes(User::getRC()) . "'";
        }

        if (isset($_SESSION['jenTypUkolu']) && $_SESSION['jenTypUkolu']) {
            if ($resultVlsTypDatum) {
                $resultVlsTypDatum .= ' AND ';
            }
            $resultVlsTypDatum .= " KategorieUkolu.kod = '" . $this->database->addSlashes($_SESSION['typUkolu']) . "'";
        }

        if ($this ->filterForm ->filtrovatDatum) {
            if ($resultVlsTypDatum) {
                $resultVlsTypDatum .= ' AND ';
            }
            $resultVlsTypDatum .= ' DatumSplneni_Narizene >= ' . $this ->database ->timestampToString($this ->filterForm ->datumOd) .
                ' AND DatumSplneni_Narizene <= ' . $this ->database ->timestampToString($this ->filterForm ->datumDo);
        }

    //Filtr stav ukolu  $resultStavUkolu
        if (isset($_SESSION['nesplnene']) && $_SESSION['nesplnene']) {
            $resultStavUkolu .= " UkolOsoby.StavUkolu = 0";
        }

        if (isset($_SESSION['poterminu']) && $_SESSION['poterminu']) {
            if ($resultStavUkolu) {
                $resultStavUkolu .= ' OR ';
            }
            $resultStavUkolu .= " DatumSplneni_Skutecne IS NULL AND DatumSplneni_Narizene < " .
                $this ->database ->timestampToString(DateUtils::incDay(DateUtils::startOfTheDay(time()), 0));
        }

        if (isset($_SESSION['splnene']) && $_SESSION['splnene']) {
            if ($resultStavUkolu) {
                $resultStavUkolu .= ' OR ';
            }
            $resultStavUkolu .= ' DatumSplneni_Skutecne IS NOT NULL ';
        }

        //Aby se vyfiltroval musi byt ukol splnen, musi byt jeste neohodnocen a musi RC_Hodnotitele_Def souhlasit s prihlasenym uzivatelem
        if ($_SESSION['mam_hodnotit']) {
            if ($resultStavUkolu) {
                $resultStavUkolu .= ' OR ';
            }
            $resultStavUkolu .= "UkolOsoby.StavUkolu = 1 AND UkolOsoby.Hodnoceno = 0 AND UkolOsoby.RC_Hodnotitele_Def = '" . $this ->database ->addSlashes(User::getRC()) . "'";
        }

        //Aby se vyfiltroval musi byt ukol splnen, musi byt neschvaleny a musi RC_Schvalovatele1_Def nebo RC_Schvalovatele2_Def souhlasit s prihlasenym uzivatelem
        if ($_SESSION['mam_schvalit']) {
            if ($resultStavUkolu) {
                $resultStavUkolu .= ' OR ';
            }

            $resultStavUkolu .= " ((UkolOsoby.StavUkolu = 1 AND UkolOsoby.Hodnoceno = 1) ";
            $resultStavUkolu .= " AND ((UkolOsoby.Schvaleno1 = 0 AND UkolOsoby.RC_Schvalovatele1_Def = '" . $this ->database ->addSlashes(User::getRC()) . "' ) ";
            $resultStavUkolu .= " OR (UkolOsoby.Schvaleno1 = 1 AND UkolOsoby.Schvaleno2 = 0 AND UkolOsoby.RC_Schvalovatele2_Def = '" . $this ->database ->addSlashes(User::getRC()) . "' ) ";
            $resultStavUkolu .= " ))\n";
        }

    //slozeni where
        if ($resultFilter) {
            if ($result != '') {
                $result .= ' AND ';
            }
            $result .= $resultFilter;
        }
        if ($resultVlsTypDatum) {
            if ($result != '') {
                $result .= ' AND ';
            }
            $result .= $resultVlsTypDatum;
        }
        if ($resultStavUkolu) {
            if ($result != '') {
                $result .= ' AND ';
            }
            $result .= '(' . $resultStavUkolu . ')';
        }
        return $result;
    }
}
