<?php
/**
 * UTF8
 */

namespace Ron\RonPortal;

require_once('adsapp/ronportalbrowser.php');
require_once('ron/browserfield.php');
require_once('ron/guiutils.php');

use Exception;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\Personalistika\OsobniCisloOsobyField;
use Ron\StringUtils;
use function Ron\ronText;
use RonPortal\RonPortalSession;


/**
 * Class SkoleniFilterForm
 * @package Ron\RonPortal
 */
class SkoleniFilterForm extends RonPortalFilterForm
{
    protected $filterLinks;

    /**
     * SkoleniFilterForm constructor.
     * @param $database
     * @param $action
     */
    public function __construct($database, $action) {
        parent::__construct($database, $action);

        $this->setUseGet(true, $action);

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->filterLinks = new LinksList();
        $this->filterLinks->itemsIdPrefix = 'skoleni_stav';
        $this->povolitVsechyOsoby = true;

        $this->nastavDatumOdDo(true, 0 );
        $this ->nastavOsoby(true, Prava::vratPrava('skoleni_prohlizeni'), time(), time()); // prava podle aktualniho casu

        $this->nastavRozdeleni(true);

        $data = self::getStavySkoleni($database);
        $this->filterLinks->addItemsFromArray($data, null, false);
        $this->filterLinks->url = '?menuid=skoleni&';
        return;
    }


    public static $stavySkoleni = null;

    /**
     * @param Database $database
     * @return null
     */
    public static function getStavySkoleni($database)
    {
        if (self::$stavySkoleni) {
            return self::$stavySkoleni;
        }

        $sql =
            'SELECT Kod, Popis FROM S_StavPotvrzeni ORDER BY Kod';

        $data[-100] = ronText('Vse');

        try {
            $cursor = $database->getSqlResult($sql);

            while ($row = $cursor->fetch()) {
                $data[$row[0]] = ronText('Stav_potvrzeni_skoleni_' . $row[0], '') ?: $row[1];
            }
            $cursor ->close();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při načítání stavů potvrzení školení ') . $exception->getMessage());
        }

        $data[100] = ronText('Skoleni_jen_hodnoceni');

        self::$stavySkoleni = $data;
        return $data;
    }




    /**
     * @throws DBException
     */
    public function renderTypSkoleni()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Skoleni_typ_skoleni');
        echo '</SPAN>';


        // combobox pro vyber
        $params = $this->database->createParameters();
        $sql =
            "SELECT DISTINCT GenerovaneSkoleni.DruhSkoleni, DruhSkoleni.Popis 
                FROM GenerovaneSkoleni INNER JOIN DruhSkoleni ON (GenerovaneSkoleni.DruhSkoleni = DruhSkoleni.Kod ) 
                WHERE DruhSkoleni.Aktivni = 1 ";

        if (!RonPortalSession::vsechnyOsoby()) {
            $sql .= " AND GenerovaneSkoleni.RC = " . $params->addParameter($this->vybranaOsobaRC);
        }

        $sql .= ' ORDER BY DruhSkoleni.Popis';

        // vyber typu skoleni podle popisu
        $combo = new ComboBox('druhskoleni');
        $combo->emptyRow = ronText('_vse_');
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, RonPortalSession::getValue('typSkoleni'), $params);
        $combo->render();

        echo "</DIV>\n";

        require_once 'person/view_detailskolenikurzu.php';

    }


    public function renderElearning()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo ronText('Skoleni_filtr_elearning');
        echo '</span>';

        $vse = new Option(0, ronText('_vse_'));
        $jeElearning = new Option(1, ronText('skoleni_pouze_elearning'));
        $neniElearning = new Option(2, ronText('skoleni_bez_elearning'));

        $combo = new ComboBox('filtrNaElearing', [$vse, $jeElearning, $neniElearning]);
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->selectedOption = $combo->findOption((int)RonPortalSession::getValue('filtrNaElearing'));
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws DBException
     */
    public function renderSpecifikaceKurzu()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        echo ronText('Skoleni_specifikace_filtr');

        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Kod, Popis FROM SpecifikaceKurzu ORDER BY Popis";

        $combo = new ComboBox('specifikaceSkoleni');
        $combo->emptyRow = ronText('_vse_');

        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, RonPortalSession::getValue('specifikaceSkoleni'), $params);
        $combo->render();

        echo "</div>";
    }


    /**
     * @throws DBException
     */
    public function renderKategorieKurzu()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        echo ronText('Skoleni_kategorie_filtr');

        echo '</span>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT Kod, Popis FROM KategorieSkoleni ORDER BY Popis";

        $combo = new ComboBox('kategorieSkoleni');
        $combo->emptyRow = ronText('_vse_');

        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
        $combo->loadData($this ->database, $sql, RonPortalSession::getValue('kategorieSkoleni'), $params);
        $combo->render();

        echo "</div>";
    }


    public function renderStavSkoleni()
    {
        echo '<div class="filterrow wrap">';
        echo '<span class="filtercaption">' . ronText('Stav_skoleni') . '</span>';

        $this->filterLinks->render();

        echo "</div>\n";
    }


    public function renderJenPosledniZaznam()
    {
        GuiUtils::renderCheckbox('jenPosledniZaznam',
            ronText('Skoleni_jen_posledni_zaznam'),
            RonPortalSession::getValue('jenPosledniZaznam'), '', 'clickAutoSubmit', 1);
    }


    private function loadTypSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypSkoleni'] = ($_POST['druhskoleni'] !== ''); // isset($_POST['jenTypSkoleni']);
            $_SESSION['typSkoleni'] = $_POST['druhskoleni'];
        }

        if (isset($_GET['druhskoleni'])) {
            $_SESSION['jenTypSkoleni'] = ($_GET['druhskoleni'] !== ''); // isset($_POST['jenTypSkoleni']);
            $_SESSION['typSkoleni'] = $_GET['druhskoleni'];
        }
    }

    private function loadSpecifikaceSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['specifikaceSkoleni'] = $_POST['specifikaceSkoleni'];
        }
        if (isset($_GET['specifikaceSkoleni'])) {
            $_SESSION['specifikaceSkoleni'] = $_GET['specifikaceSkoleni'];
        }
    }

    private function loadKategorieSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['kategorieSkoleni'] = $_POST['kategorieSkoleni'];
        }
        if (isset($_GET['kategorieSkoleni'])) {
            $_SESSION['kategorieSkoleni'] = $_GET['kategorieSkoleni'];
        }
    }

    private function loadPosledniZaznam()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenPosledniZaznam'] = isset($_POST['jenPosledniZaznam']);
        }
        if (isset($_GET['druhskoleni'])) {
            $_SESSION['jenPosledniZaznam'] = isset($_GET['jenPosledniZaznam']);
        }
    }

    private function loadElearning()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenFiltrNaElearing'] = ((int)$_POST['filtrNaElearing'] !== 0); //isset($_POST['jenFiltrNaElearing']);
            $_SESSION['filtrNaElearing'] = $_POST['filtrNaElearing'];
        }
        if (isset($_GET['filtrNaElearing'])) {
            $_SESSION['jenFiltrNaElearing'] = ((int)$_GET['filtrNaElearing'] !== 0); //isset($_POST['jenFiltrNaElearing']);
            $_SESSION['filtrNaElearing'] = $_GET['filtrNaElearing'];
        }
    }


    private function loadStavSkoleni()
    {
        require_once('adsapp/usersettings.php');
        $vybraneStavy = NastaveniUzivatele::getUserSettingsItem($this->database, 'skoleni_filtr_stav');
        $this ->filterLinks ->loadSettingString($vybraneStavy);
        $this ->filterLinks ->loadGet();

        //je vybrano vse
        if (isset($_GET['-100']) && ($_GET['-100'] == 1)) {
            foreach ($this ->filterLinks->items as $item) {
                if ($item ->key != -100) $item->active = false;
            }
            $_SESSION['jenStavSkoleni'] = false;
        } else {
            $this->filterLinks->findItem(-100)->active = false;
            $_SESSION['jenStavSkoleni'] = true;
        }

        $result = null;
        $_SESSION['skoleniHodnocene'] = false;
        if ($_SESSION['jenStavSkoleni']) {
            foreach ($this->filterLinks->items as $item) {
                if ((!(($item->key == -100) || ($item->key == 100))) && $item->active) $result[] = $item->key;
                //jen skoleni s hodnocenim
                if (($item->key == 100) && $item->active) $_SESSION['skoleniHodnocene'] = true;
            }
            if ($result == null) $_SESSION['jenStavSkoleni'] = false;
            $_SESSION['stavSkoleni'] = $result;
        }

        if (($vybraneStavy == '') || (($result == null) && !$_SESSION['skoleniHodnocene']))  {
            $this->filterLinks->activateItem(-100);
        }

        $vybraneNew = $this ->filterLinks ->getSettingsString();
        if ($vybraneNew != $vybraneStavy) NastaveniUzivatele::setUserSettingsItem($this->database, 'skoleni_filtr_stav', $vybraneNew, '');
    }

    /**
     *
     */
    function renderDatumOdDo()
    {
        if ($this->nadpisDatumOdDo) $nadpis = $this->nadpisDatumOdDo;
        else $nadpis = ronText('Skoleni_jen_datum_od_do');

        echo '<DIV class="filterrow">';

            echo '<SPAN class="filtercaption">';
            if ($this->datumOdDoPovinne) {
                echo $nadpis; // .' &nbsp;&nbsp;';
            } else {
                require_once('ron/guiutils.php');
                //Na prani MaKR checkbox jako prvni
                GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', null, false);
            }
            echo '</SPAN>';

            echo '<SPAN style="white-space:nowrap;">';
            DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter

            echo '&nbsp;&nbsp;&nbsp; ' . ronText('Do') . ' &nbsp;&nbsp;';
            DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter
            echo '</SPAN>';

        echo "</DIV>\n";
    }

    public function loadData()
    {
        $this->loadTypSkoleni();
        $this->loadStavSkoleni();
        $this->loadElearning();
        $this->loadPosledniZaznam();
        $this->loadSpecifikaceSkoleni();
        $this->loadKategorieSkoleni();

        if (!RonPortalSession::getValue('skoleniPrvniPristup')) {
            RonPortalSession::setValue('skoleniPrvniPristup', true);

            $params = $this->database->createParameters();
            $idpp = DBUtils::getSqlValue($this->database, 'SELECT MAX(ID) FROM PracovniPomer WHERE RC = ' . $params->addParameter(User::getRC()) .
                ' AND DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
                ' AND (DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR DatumUkonceni IS NULL)', $params);
            RonPortalSession::setVybranaOsoba($idpp, null, null, null);
        }

        parent::loadData();
    }

    public function renderFilters()
    {
        $this->comboOsoba->className .= ' ronselect';
        $this->comboOsoba->htmlParams['data-ronselect-class'] = 'ronselect adsfilterforminput_full';
        $this->comboOsoba->htmlParams['data-columns'] = 'text';

        parent::renderFilters(); // osoba, rozdeleni

        try {
            $this->renderElearning();
            $this->renderTypSkoleni();
            $this->renderSpecifikaceKurzu();
            $this->renderKategorieKurzu();
            $this->renderJenPosledniZaznam();
            $this->renderStavSkoleni();
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při načítání filtru ') . $exception->getMessage());
        }

    }
}


class PrihlaseniField extends BrowserField
{
    private $druhSkoleniIndex;
    private $stavPotvrzeniIndex;
    private $elearningIndex;
    private $pocetSkoleniIndex;
    private $idPPOsobyIndex;
    private $cisloSkoleniIndex;
    private $stavPrihlaskyIndex;
    private $datumSkoleniIndex;

    /**
     * PrihlaseniField constructor.
     * @param Database $database
     * @param int $druhSkoleniIndex
     * @param int $stavPotvrzeniIndex
     * @param int $elearningIndex
     * @param int $pocetSkoleniIndex
     * @param int $idPPOsobyIndex
     * @param int $cisloSkoleniIndex
     * @param int $stavPrihlaskyIndex
     * @param int $datumSkoleniIndex
     */
    public function __construct($database, $druhSkoleniIndex, $stavPotvrzeniIndex, $elearningIndex, $pocetSkoleniIndex, $idPPOsobyIndex, $cisloSkoleniIndex, $stavPrihlaskyIndex, $datumSkoleniIndex)
    {
        parent::__construct($database);

        $this->header = ronText('Skoleni_prihlaseni');
        $this->code = 'col-prihlaseni-skoleni';

        $this->wrap = true;

        $this->className = 'prehled-skoleni-prihlaseni-na-kurz';
        $this->druhSkoleniIndex = $druhSkoleniIndex;
        $this->stavPotvrzeniIndex = $stavPotvrzeniIndex;
        $this->elearningIndex = $elearningIndex;
        $this->pocetSkoleniIndex = $pocetSkoleniIndex;
        $this->idPPOsobyIndex = $idPPOsobyIndex;
        $this->cisloSkoleniIndex = $cisloSkoleniIndex;
        $this->stavPrihlaskyIndex = $stavPrihlaskyIndex;
        $this->datumSkoleniIndex = $datumSkoleniIndex;
    }

    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[$this->elearningIndex])
            return '';

        if ($rowData[$this->stavPotvrzeniIndex] == 1) {
            $osoba = '';
            if (RonPortalSession::vsechnyOsoby())
                $osoba = '&osoba=' . $rowData[$this->idPPOsobyIndex];

            if ($rowData[$this->cisloSkoleniIndex]) {
                $stav = '';
                if ($rowData[$this->stavPrihlaskyIndex] != '')
                    $stav = " ({$rowData[$this->stavPrihlaskyIndex]})";

//                return '<a href="?menuid=skoleni&action=zobrazprihlaseni' . $osoba . '&cisloskoleni=' . $rowData[$this->cisloSkoleniIndex] . '">' . ronText('Skoleni_Prihlasen') . '</a>';

               $datumSkoleni = DBUtils::getFormatedValue($this->database, 'date', $rowData[SkoleniBrowser::DATUM_POCATKU_SKOLENI_INDEX]);

                return ronText('Skoleni_Prihlasen') . $stav . ' ' . $datumSkoleni;
            } else if ((int)$rowData[$this->pocetSkoleniIndex] > 0) {
                return '<a href="?menuid=skoleni&action=zobrazprihlaseni' . $osoba . '&typskoleni=' . $rowData[$this->druhSkoleniIndex] . '">' . ronText('Skoleni_prihlaseni') . /*' (' . $rowData[$this->pocetSkoleniIndex] . ')' .*/ '</a>';
            } else {
                return ronText('Skoleni_nejsouTerminy');
            }
        }

        return '';

    }
}


/**
 * Class AbsolvovaniField
 * @package Ron\RonPortal
 */
class AbsolvovaniField extends BrowserField
{
    private $idOsobyIndex;
    private $datumPovinnostiIndex;
    private $druhSkoleniIndex;
    private $stavPotvrzeniIndex;
    private $elearningIndex;
    private $rcOsobyIndex;

    /**
     * PrihlaseniField constructor.
     * @param Database $database
     * @param int $idOsobyIndex
     * @param int $druhSkoleniIndex
     * @param int $datumPovinnostiIndex
     * @param int $stavPotvrzeniIndex
     * @param int $elearningIndex
     * @param int $rcOsobyIndex
     */
    public function __construct($database, $idOsobyIndex, $druhSkoleniIndex, $datumPovinnostiIndex, $stavPotvrzeniIndex, $elearningIndex, $rcOsobyIndex)
    {
        parent::__construct($database);

        $this->header = ronText('Skoleni_zapsat_absolvovani');

        $this->idOsobyIndex = $idOsobyIndex;
        $this->druhSkoleniIndex = $druhSkoleniIndex;
        $this->datumPovinnostiIndex = $datumPovinnostiIndex;
        $this->stavPotvrzeniIndex = $stavPotvrzeniIndex;
        $this->elearningIndex = $elearningIndex;
        $this->rcOsobyIndex = $rcOsobyIndex;
        $this->className = 'prehled-skoleni-absolvovat';
        $this->code = 'col-zapsat-absolvovani';
    }

    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        try {
            //pro druhy skoleni, ktere nejsou elearning
            if (($rowData[$this->elearningIndex] == 0) &&
                //u stavu potvrzeni 0,1,5,6 umoznit zapis absolvovani
                (($rowData[$this->stavPotvrzeniIndex] == 0) || ($rowData[$this->stavPotvrzeniIndex] == 1) || ($rowData[$this->stavPotvrzeniIndex] == 5) ||
                    ($rowData[$this->stavPotvrzeniIndex] == 6))
                && Prava::vratPrava('skoleni_absolvovani')->lzeZobrazitRC($rowData[$this->rcOsobyIndex])
            ) {
                return '<a href="?menuid=skoleni&action=zapisabsolvovani&formaction=edit&osoba=' . $rowData[$this->idOsobyIndex] . '&druhskoleni=' . $rowData[$this->druhSkoleniIndex] .
                    '&datumpovinnosti=' . $this->database->stringToTimestamp($rowData[$this->datumPovinnostiIndex]) . '">'
                    . ronText('Skoleni_zapsat_absolvovani') . '</a>';
            }
        } catch (Exception $exception) {
            return '';
        }

        return '';
    }
}

/**
 * Class DatumSkoleniField
 * @package Ron\RonPortal
 */
class DatumSkoleniField extends BrowserField
{
    private $stavPotvrzeniIndex;

    /**
     * @param Database $database
     * @param int $stavPotvrzeniIndex
     * @param array|null $options
     */
    public function __construct(Database $database, $stavPotvrzeniIndex, $options = null)
    {
        parent::__construct($database, $options);

        $this->stavPotvrzeniIndex = $stavPotvrzeniIndex;
    }

    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this->database->stringToTimestamp($value) > DateUtils::today())
            $this->className = 'skoleni_planovane';
        else
            $this->className = 'skoleni_uplynule';

        //absolvovano uspesne                               absolvovano neuspesne                       neucast, prominuto
        if (!(($rowData[$this->stavPotvrzeniIndex] == 3) || ($rowData[$this->stavPotvrzeniIndex] == 4) || ($rowData[$this->stavPotvrzeniIndex] >= 7)))
            $value = null;

        return parent::getFieldCellText($value, $rowNumber, $recordId, $rowData);
    }
}

/**
 * Class TypPopisSkoleni
 * @package Ron\RonPortal
 */
class TypPopisSkoleni extends BrowserField
{

    /**
     * TypPopisSkoleni constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->header = ronText('Druh_skoleni');
        $this->align = 'left';
        $this->ordering = 0;
        $this->className = "font-weight-bold";
        $this->code = 'col-popis-skoleni';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= StringUtils::htmlTextConvert($rowData[6] . ($rowData[11] != "" ? ", " . $rowData[11] : ""));
        return $result;
    }
}


class DetailSkoleni extends BrowserField
{

    /**
     * TypPopisSkoleni constructor.
     * @param $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->align = 'center';
        $this->ordering = 0;
        $this->code = 'col-detail';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        if ($rowData[0] && $rowData[5] != "") {

            $datumPovinnosti = $this->database->stringToTimestamp($rowData[5]);
            $result .= "<div class='zobrazit-detail-skoleni-kurzu js'
                    data-ron-idpp='$rowData[8]'
                    data-ron-druhskoleni='$rowData[0]'
                    data-ron-cisloskoleni='$rowData[1]'
                    data-ron-datum='$datumPovinnosti'  >" . ronText('Detail') . "</div>";
        }
        return $result;
    }
}


class DokumentySkoleniBrowserField extends BrowserField
{
    private $pocetDokumentuKurzu;
    private $pocetDokumentuAbsolvovanehoSkoleni;


    /**
     * DokumentySkoleniBrowserField constructor.
     * @param $database
     * @param int $pocetDokumentuAbsolvovanehoSkoleni
     * @param int $pocetDokumentuKurzu
     */
    public function __construct($database, $pocetDokumentuAbsolvovanehoSkoleni, $pocetDokumentuKurzu)
    {
        parent::__construct($database);

        $this->pocetDokumentuAbsolvovanehoSkoleni = $pocetDokumentuAbsolvovanehoSkoleni;
        $this->pocetDokumentuKurzu = $pocetDokumentuKurzu;
    }


    /** Vrátí HTML pro vypsání seznamu dokumentů kurzu, každý dokument je vnořen do tagu span s class dokument_kurzu.
     * @param string $cisloSkoleni
     * @return string
     */
    function vratDokumentyKurzu($cisloSkoleni)
    {
        $result = '';

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru " . //0..2
            "FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentSkoleni.cisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $linkStr = "?menuid=skoleni&action=download&cisloskoleni=" . urlencode($cisloSkoleni) . "&dokument={$row[0]}";

                if ($linkStr) $result .= "<span class='dokument_kurzu'><a target='_blank' href='$linkStr'>";
                $title = StringUtils::htmlTextConvert($row[1]) ; //. ' - ' . StringUtils::htmlTextConvert($row[2]);
                $result .= "<i title='$title' class='inline-glyph fa glyph-attachment'></i>";
                if ($linkStr) $result .= "</a></span>";
            }
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání dokumentů školení ") . StringUtils::htmlTextConvert($cisloSkoleni) . "\n" . $exception->getMessage());
        }

        return $result;
    }


    /** Vrátí HTML pro vypsání seznamu dokumentů absolvovaného školení, každý dokument je vnořen do tagu span s class dokument_absolvovani.
     * @param array $values
     * @return string
     */
    function vratDokumentyAbsolvovanehoSkoleni($values)
    {
        $result = '';

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.popis, Dokument.jmenoSouboru " . //0..2
            "FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentyOsoby.RC = " . $params->addParameter($values[4]) .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = " . $params->addParameter($values[4]) .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = " . $params->addParameter($values[0]) . //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = " . $params->addParameter($this ->database ->timestampToParam( $this ->database ->stringToTimestamp($values[5]) )) .  //datum vzniku povinnosti
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $linkStr = "?menuid=skoleni&action=download&osoba=" . $values[8] . "&druhskoleni=" . $values[0] .
                    "&datum=" . $this->database->stringToTimestamp($values[5]) .
                    "&dokument={$row[0]}";

                if ($linkStr) $result .= "<span class='dokument_absolvovani'><a target='_blank' href='$linkStr'>";
                $title = StringUtils::htmlTextConvert($row[1]) ; //. ' - ' . StringUtils::htmlTextConvert($row[2]);
                $result .= "<i title='$title' class='inline-glyph fa glyph-attachment'></i>";
                if ($linkStr) $result .= "</a></span>";
            }
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání dokumentů absolvovaného školení ") . StringUtils::htmlTextConvert($values[0]) . "\n" . $exception->getMessage());
        }

        return $result;
    }

    /** Zobrazí dokumenty školení v jednom řádku tabulky. Dokumenty kurzu v divu s class skoleni_dokumenty_kurzu a dokumenty absolvovaného školení
     * v divu s class skoleni_dokumenty_absolvovani.
     * @param array $values
     * @return string
     */
    function zobrazDokumentySkoleni($values)
    {
        $result = '';

        $dokumentyKurzu = '';
        if ((int)$values[$this->pocetDokumentuKurzu] > 0)
            $dokumentyKurzu = $this->vratDokumentyKurzu($values[1]);

        $dokumentyAbsolvovane = '';
        if ((int)$values[$this->pocetDokumentuAbsolvovanehoSkoleni] > 0)
            $dokumentyAbsolvovane = $this->vratDokumentyAbsolvovanehoSkoleni($values);

        if (($dokumentyKurzu != '') || ($dokumentyAbsolvovane != '')) {
            if ($values[1]) {
                $result .= "<div class='skoleni_dokumenty_kurzu'>";
                $result .= $dokumentyKurzu;
                $result .= "</div>";
            }
            $result .= "<div class='skoleni_dokumenty_absolvovani'>";
            $result .= $dokumentyAbsolvovane;
            $result .= "</div>";
        }

        return $result;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Zobrazeni dokumentu jen pokud nejake jsou
        if ((int)$rowData[$this->pocetDokumentuKurzu] > 0 || (int)$rowData[$this->pocetDokumentuAbsolvovanehoSkoleni] > 0)
            return $this->zobrazDokumentySkoleni($rowData);

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }


}

/**
 * Class SkoleniBrowser
 * @package Ron\RonPortal
 */
class SkoleniBrowser extends RonPortalBrowser
{
    const DATUM_POCATKU_SKOLENI_INDEX = 20;

    /** @var PravaNaOsoby $pravoSkoleniProhlizeni */
    public $pravoSkoleniProhlizeni;

    /**
     * SkoleniBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'skoleni');

        $this->pravoSkoleniProhlizeni = Prava::vratPrava('skoleni_prohlizeni');
        Prava::nastavObdobiProPrava(time(), time());

        $this ->filterForm = new SkoleniFilterForm($this ->database, $this ->action);
        $this->filterForm->loadData();

        $this->code = 'browserskoleni';
        $this ->maxPageRows = 100;

        /** @noinspection PhpDeprecationInspection V konstruktoru zatim nelze pouzit parametry */
        $this ->sqlSelectPart =
            'SZ.DruhSkoleni, SZ.CisloSkoleni, SZ.Potvrzeni, SZ.DatumSkoleni, SZ.RC, SZ.DatumVznikuPovinnosti, ' . // 0..5
            'D.Popis, P.Popis, (SELECT MAX(ID) FROM PracovniPomer WHERE RC = SZ.RC  AND PracovniPomer.DatumNastupu <= ' . $this->database->timestampToString(time()) . // 6 .. 8
            '   AND (PracovniPomer.DatumUkonceni >= ' . $this->database->timestampToString(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)), D.Elearning, SZ.PozadovaneDatumSkoleni, SZ.PopisSkoleni, '.  // 8..11
            'Osoba.Jmeno, Osoba.Prijmeni, ' .                //12, 13
            '(SELECT COUNT(HodnoceniSkoleni.TemaCislo) FROM HodnoceniSkoleni WHERE (S.CisloSkoleni = HodnoceniSkoleni.CisloSkoleni)), ' . //14

            //15 - zjisteni poctu vypsanych kurzu pro dany typ skoleni, presunuto z BrowserField pro zrychleni
            "(SELECT COUNT(Skoleni.CisloSkoleni) " .
            "FROM Skoleni " .
            "WHERE Skoleni.DruhSkoleni = D.Kod" .
            "  AND Skoleni.StavSkoleni IN (0, 1) " .
            '  AND (Skoleni.DatumUkonceniSkoleni >= ' . $this->database->timestampToString(time()) . ' OR Skoleni.DatumUkonceniSkoleni IS NULL)' .
            "  AND Skoleni.DatumPocatkuSkoleni >= " . $this->database->timestampToString(time()) .
            "  AND (Skoleni.DatumPotvrzeniUcasti >= "  . $this->database->timestampToString(time()) . " OR Skoleni.DatumPotvrzeniUcasti IS NULL)" .
            "  AND (Kapacita > (SELECT COUNT(RC) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni))" .
            "  AND D.Elearning = 0" .
            "), " .

            //16 - pocet dokumentu pripojenych ke kurzu
            "(SELECT COUNT(Dokument.ID) " .
            "FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentSkoleni.cisloSkoleni = SZ.cisloSkoleni" .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))" .
            "), " .

            //17 - pocet dokumentu absolvovaneho skoleni
            "(SELECT COUNT(Dokument.ID) " .
            "FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            "WHERE DokumentyOsoby.RC = SZ.RC" .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = SZ.RC" .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = SZ.DruhSkoleni" . //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = SZ.DatumVznikuPovinnosti" .  //datum vzniku povinnosti
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))" .
            "), " .

            //18..20
            "S_StavSchvalPrihlNaKurz.Popis, S.Popis, S.DatumPocatkuSkoleni"
        ;

        //'S.CasPocatkuSkoleni, S.MistoSkoleni, S.DelkaSkoleniDnu, S.DelkaSkoleniHodin '

        $this->sqlFromPart =
            'GenerovaneSkoleni SZ LEFT OUTER JOIN Skoleni S  ON S.CisloSkoleni = SZ.CisloSkoleni ' .
            '   LEFT OUTER JOIN DruhSkoleni D ON SZ.DruhSkoleni = D.Kod  ' .
            '   LEFT OUTER JOIN S_StavPotvrzeni P ON P.Kod = SZ.Potvrzeni' .
            '   LEFT OUTER JOIN Osoba ON SZ.RC = Osoba.RC' .
            '   LEFT OUTER JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = SZ.SchvaleniPrihlasky)';


        if (RonPortalSession::vsechnyOsoby()) {
            require_once('person/person_browserfield.php');
            $this->addField(new OsobniCisloOsobyField($database, 4));

            $this ->addField()
                ->setSqlFieldName('Osoba.Prijmeni', 13) ->setHeaderText('Prijmeni')
                ->setCode('col-prijmeni')
                ->setOrdering(1) ->setClassName('font-weight-bold') ->setWrap(1);

            $this ->addField()
                ->setSqlFieldName('Osoba.Jmeno', 12) ->setHeaderText('Jmeno')
                ->setCode('col-jmeno')
                ->setOrdering(1) ->setClassName('font-weight-bold') ->setWrap(1);
        }

        $this->addField()
            ->setSqlFieldName('S.Popis', 19) ->setHeaderText('Skoleni_popis_kurzu')
            ->setCode('col-popis-kurzu')
            ->setOrdering(1) ->setVisible(0) ->setClassName('font-weight-bold') ->setWrap(1);

        $this->addField(new TypPopisSkoleni($database));


        $this->addField()
            ->setSqlFieldName('SZ.PozadovaneDatumSkoleni', 10) ->setHeaderText('Pozadovane_datum_skoleni')
            ->setCode('col-pozadovane-datum-skoleni')
            ->setFormat('finedate') ->setOrdering(1) ->setAlign('center') ->setWrap(1);


        $this->addField(new DatumSkoleniField($database, 2))
            ->setSqlFieldName('SZ.DatumSkoleni', 3) ->setHeaderText('Skoleni_datum')
            ->setCode('col-skoleni-datum')
            ->setFormat('finedate') ->setAlign('center') ->setOrdering(1) ->setWrap(1);

        $this->defaultSort = -count($this->fields);

        $this ->addField(BROWSER_FIELD_TYPE_SELECT)
            ->setSqlFieldName('SZ.Potvrzeni', 2) ->setHeaderText('Stav_skoleni') ->setOptions(SkoleniFilterForm::getStavySkoleni($database))
            ->setCode('col-stav-skoleni')
            ->setOrdering(1) ->setAlign('center') ->setWrap(1);

        $this->addField()
            ->setSqlFieldName('P.Popis', 7) ->setHeaderText('Stav_skoleni')
            ->setCode('col-popis-stavu-skoleni') ->setVisible(false)
            ->setOrdering(1) ->setAlign('center') ->setWrap(1);


        $this->addField(new DetailSkoleni($database));

        $this->addField(new DokumentySkoleniBrowserField($database, 17, 16))
            ->setCode('col-dokumenty') ->setHeaderText('Dokumenty')
            ->setAlign('center') ->setOrdering(0);

        $this->addField(new AbsolvovaniField($database, 8, 0, 5, 2, 9, 4));

        $this->addField(new PrihlaseniField($this->database, 0, 2, 9, 15, 8, 1, 18, 3));

        $this->primaryField = 0;
    }


    /**
     * @param string $sqlFilter
     * @throws DBException
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->orderFieldIndex = $this ->getOrderFieldIndex();
        $sqlText = $this ->getSQL($sqlFilter);

        $this ->loadSQL($sqlText);
    }


    /** Zobrazí podrobnosti školení, na které je uživatel přihlášen. Formát: <br/>
     * Začátek školení d.m.rrrr [hh:nn] <strong>Popis skoleni</strong> <br>
     * [Popis umisteni] <br>
     * [Ulice, Mesto]
     * @param array $values detaily skoleni zamestnance
     * @param string $className CSS class
     */
    function zobrazPodrobnostiSkoleni($values, $className)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Skoleni.MistoSkoleni, ' . // 0
            ' Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni, Skoleni.DelkaSkoleniDnu, Skoleni.DelkaSkoleniHodin, ' . // 1..4
            ' Umisteni.Popis, Umisteni.Mesto, Umisteni.Ulice, Umisteni.Patro, Umisteni.CisloMistnosti, Skoleni.Popis ' . // 5..10
            ' FROM Skoleni LEFT OUTER JOIN Umisteni ON Skoleni.MistoSkoleni = Umisteni.Kod ' .
            " WHERE Skoleni.CisloSkoleni = " . $params ->addParameter($values[1]);

        try {
            $cursor = $this ->database ->getSqlResult($sql, $params);
            if ($row = $cursor ->fetch()) {
                echo "<tr class='$className'><td></td><td class='browserpoznamka' colspan='" . ($this->columnCount-1) . "'>";

                echo ronText('podrobnosti_skoleni') . ": " . ronText('Zacatek_skoleni') . ' ' . DBUtils::getFormatedValue($this ->database, 'finedate', $row[1]); // datum zacatku

                if ($row[2]) echo ' ' . DBUtils::getFormatedValue($this ->database, 'time', $row[2]);             // cas zacatku

                echo '&nbsp;-&nbsp;' . StringUtils::htmlTextConvert($row[10]);

                echo '<br/>';

                if ($row[5]) echo StringUtils::htmlTextConvert($row[5]) . '<br/> '; // popis umisteni

                if ($row[6] || $row[7]) {
                    echo StringUtils::htmlTextConvert($row[7]);
                    if ($row[6] && $row[7]) echo ", ";
                    echo StringUtils::htmlTextConvert($row[6]);
                    echo '<br/>';
                }

                echo "</td></tr>\n";
            }
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání podrobností školení ") . StringUtils::htmlTextConvert($values[1]) . "\n" . $exception->getMessage());
        }
    }


    function zobrazEKurzy($values, $className)
    {
        /** @noinspection PhpDeprecationInspection ponecháno pro kompatibilitu s ODBC driverem (generuje chybu pro parametry v LEFT JOIN nebo v EXISTS) */
        $sql =
            'SELECT EKurz.ID, EKurz.Nazev, EKurz.PlatnostOd, EKurz.PlatnostDo, ' . // 0..3
            '  EKurzOsoby.Id, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.PosledniPristup, ' . // 4..7
            '  RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.UlozitCertifikatDoDok, SestavyProIntraweb.Nazev' . // 8..10
            ' FROM (EKurzDruhuSkoleni INNER JOIN EKurz  ' .
            '   ON EKurzDruhuSkoleni.IDKurzu = EKurz.ID) ' .
            ' LEFT OUTER JOIN (EKurzOsoby INNER JOIN RevizeEKurzu ON EKurzOsoby.IdRevize = RevizeEKurzu.Id)' .
            "   ON EKurzOsoby.RC_Skoleni = '" . $this ->database ->addSlashes($values[4]) . "'" .
            "  AND EKurzOsoby.DruhSkoleni = '" . $this ->database ->addSlashes($values[0]) . "'" .
            '  AND EKurzOsoby.DatumVznikuPovinnosti = ' . $this ->database ->timestampToString( $this ->database ->stringToTimestamp($values[5]) ) .
            '  AND RevizeEKurzu.IDKurzu = EKurz.ID ' .
            ' LEFT OUTER JOIN SestavyProIntraweb' .
            '  ON RevizeEKurzu.SestavaProCertifikat = SestavyProIntraweb.Kod' .
            " WHERE EKurzDruhuSkoleni.KodDruhuSkoleni = '" . $this ->database ->addSlashes($values[0]) . "'" .
            '   AND EKurz.Stav = 1' . // jen aktivni
            ' ORDER BY EKurzDruhuSkoleni.Poradi, EKurzOsoby.DatumCasZahajeni DESC';

        try {
            $cursor = $this->database->getSqlResult($sql);
            while ($row = $cursor->fetch()) {
                echo "<tr class='$className'><td colspan='" . ($this->columnCount) . "'>";

                if ($row[3] && $this->database->stringToTimestamp($row[3]) < DateUtils::startOfTheDay(time())) {
                    echo ronText('Platnost_kurzu_skoncila');
                } else {
                    if ($row[4]) {
                        // existuje zaznam EKurzOsoby
                        $linkStr = '?menuid=prezentace&kurzosoby=' . $row[4];
                    } else {
                        //MaKr: nezobrazovat odkaz pokud je kurz uspesne absolvovan
                        //    abs. uspesne           abs. neuspesne        prominuto            prominuto trvale
                        if (!(($values[2] == 3) || ($values[2] == 4) || ($values[2] == 9) || ($values[2] == 10))) {
                            $linkStr =
                                '?menuid=prezentace&zahajitkurz=' . $row[0] .
                                '&druhskoleni=' . StringUtils::htmlTextConvert($values[0]) .
                                '&datumvznikupovinnosti=' . $this->database->stringToTimestamp($values[5]); // date('j.n.Y', $this ->database ->stringToTimestamp($values[5]) ); - je tam i cas!
                        }
                    }

                    if (isset($linkStr)) {
                        echo "<a target='_blank' href='$linkStr'>";
                        echo StringUtils::htmlTextConvert($row[1]);
                        echo "</a>";
                    }

                    switch ($row[5]) {
                        case 1 :
                            echo ' - <span class="ekurz_probiha">' . ronText('EKurz_probiha') . '</span>';
                            break;

                        case 3 :
                            echo ' - <span class="ekurz_absolvovan_uspesne">' . ronText('EKurz_absolvovan_uspesne') . '</span>';
                            if ($row[8]) {
                                // download certifikatu
                                // TODO: presunout do kontroleru skoleni?
                                echo ", <a target='_blank' href='?menuid=prezentace&action=certifikat&id={$row[4]}'>" . StringUtils::htmlTextConvert($row[10]) . '</a>';
                            }
                            break;

                        case 4 :
                            echo ' - <span class="ekurz_absolvovan_neuspesne">' . ronText('EKurz_absolvovan_neuspesne') . '</span>';
                            break;
                    }
                }

                echo "</td></tr>\n";
            }
            $cursor->close();
        }
        catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání ekurzů ") . $exception->getMessage());
        }
    }




    private function zobrazHodnoceniSkoleni($values, $className)
    {
        /** @var Parameters $params */
        $params = $this->database->createParameters();

        $sql =
            "SELECT DISTINCT HodnoceniSkoleni.TypGenerovani, TemaHodnoceni.PopisTematu, PripraveneTemaHodnoceni.ID, HodnoceniSkoleni.TypGenerovani " . // 0 - 3
            "FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON (PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni " .
            "    AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
            "   INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
            "   LEFT JOIN RozdeleniOsob RO_Hodnoceny ON (RO_Hodnoceny.RC = PripraveneTemaHodnoceni.HodnocenyRC AND RO_Hodnoceny.KodTypuRozdeleni = HodnoceniSkoleni.TypRozdeleni)" .
            "   LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RO_Hodnoceny.KodTypuRozdeleni AND Rozdeleni.Kod = RO_Hodnoceny.Rozdeleni) " .
            "   LEFT JOIN RozdeleniOsob RO_Nadrizeny ON (RO_Nadrizeny.KodTypuRozdeleni = Rozdeleni.RozdeleniRodice AND RO_Nadrizeny.Rozdeleni = Rozdeleni.Rodic) ".
            "WHERE PripraveneTemaHodnoceni.HodnocenyRC = " . $params->addParameter($values[4]) .
            " AND HodnoceniSkoleni.CisloSkoleni = " . $params->addParameter($values[1]) .
            " AND ((HodnoceniSkoleni.TypGenerovani = 3 AND PripraveneTemaHodnoceni.HodnocenyRC = " . $params->addParameter(User::getRC()) . ")" .
            "     OR (HodnoceniSkoleni.TypGenerovani = 2 AND HodnoceniSkoleni.RCHodnoticiOsoby = " . $params->addParameter(User::getRC()) . ")" .
            "     OR (HodnoceniSkoleni.TypGenerovani = 1 " .
            "            AND RO_Hodnoceny.PlatnostOd <= " . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            "            AND (RO_Hodnoceny.PlatnostDo >= " . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            "                   OR RO_Hodnoceny.PlatnostDo IS NULL) " .
            "            AND RO_Nadrizeny.RC = " . $params->addParameter(User::getRC()) .
            "            AND RO_Nadrizeny.PlatnostOd <= " . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            "            AND (RO_Nadrizeny.PlatnostDo >= " . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            "                   OR RO_Nadrizeny.PlatnostDo IS NULL) " .
            "        ) " .
            "     )" .
            "ORDER BY HodnoceniSkoleni.TypGenerovani DESC";

        $hodnoceni = null;

        try {
            $cursor = $this->database->select($sql, $params);

            while ($row = $cursor->fetch()) {
                $typGenerovani = '';
                switch ($row[3]) {
                    case 1 : //ucastnik dle typu rozdeleni
                    case 2 : //ucastnik zvolenou osobou
                        $typGenerovani = ronText('Skoleni_Hodnoceni_Ucastnika');
                        break;
                    case 3 : //skoleni ucastnikem
                        $typGenerovani = ronText('Skoleni_Hodnoceni');
                        break;
                }
                $hodnoceni .= "<span class='skoleni_hodnoceni'><strong><a href='?menuid=skoleni&action=hodnoceni&record=$row[2]'>" .
                    StringUtils::htmlTextConvert($row[1]) . "</a></strong> - " . $typGenerovani . "</span>";
            }
        } catch (Exception $exception) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8("Chyba při načítání hodnocení školení ") . StringUtils::htmlTextConvert($values[1]) . "\n" . $exception->getMessage());
        }

        if (!is_null($hodnoceni)) {
            echo "<tr class='$className'><td colspan='" . ($this->columnCount - 1) . "'>";
            echo "<div class='skoleni_seznam_hodnoceni'>$hodnoceni</div>";
            echo "</td></tr>";
        }
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        // Pokud je vybrano konkretni skoleni, zobrazi podrobnosti
//        if ($values[1]) {
        //Zobrazeni podrobnosti presunuto do detailu
//            $this->zobrazPodrobnostiSkoleni($values, $className);

        //Pokud ma prirazeny kurz nastavene sa
        if ($values[14])
            $this->zobrazHodnoceniSkoleni($values, $className);
//        }


        //e-learningove kurzy hleda jen pokud je druh skoleni e-learning
        if (RonPortalSettings::getIntValue(RonPortalSettings::ELEARNING_POVOLEN) && $values[9] && $values[4] == User::getRC()) {
            // pokud zobrazuje vlastni, tak zobrazi prokliky na elearningove kurzy
            $this ->zobrazEKurzy($values, $className);
        }

        $this ->renderSeparator();
    }

    function getSQLFilter()
    {
        //Ověření práv
        $sql =
            "EXISTS(SELECT PP.RC FROM PracovniPomer PP " .
            "WHERE (PP.RC = Osoba.RC) " .
            ' AND (' . $this->pravoSkoleniProhlizeni->sqlFiltr('PP', DateUtils::today(), DateUtils::today()) . ')' .
            " AND (PP.DatumNastupu <= " . $this->database->dateToString(RonPortalSession::getDatumDo()) . ")" .
            " AND ((PP.DatumUkonceni >= " . $this->database->dateToString(RonPortalSession::getDatumOd()) . ") OR (PP.DatumUkonceni IS NULL))";

        if (!$this->filterForm->vsechnyOsoby) {
            $sql .= " AND PP.RC = '" . $this->database->addSlashes($this->filterForm->vybranaOsobaRC) . "'";
        } else {
            if ($this->filterForm->kodRozdeleni) {
                require_once('adsapp/rozdeleni.php');
                $sql .= ' AND ' .
                    Rozdeleni::sqlFilterRozdeleni($this->database, 'PP', (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                        DateUtils::today(), DateUtils::today(), $this->filterForm->vcetnePodrizenychRozdeleni);
            }
        }

        $sql .=
            ')';



        if (RonPortalSession::getFiltrovatDatum()) {
            if ($sql) {
                $sql .= ' AND ';
            }
            $sql .= ' SZ.DatumSkoleni >= ' . $this->database->dateToString(RonPortalSession::getDatumOd()) .
                ' AND SZ.DatumSkoleni <= ' . $this->database->dateToString(RonPortalSession::getDatumDo());
        }


        if (RonPortalSession::getValue('jenStavSkoleni')) {
            if ($sql) {
                $sql .= ' AND ';
            }
            $sql .= SQLUtils::getINCondition($this->database, "SZ.Potvrzeni", $_SESSION['stavSkoleni'], false);
        }


        if ($_SESSION['skoleniHodnocene']) {
            $sql .= " AND " . self::getSQLFilterHodnocene($this->database, 'SZ');
        }


        if (RonPortalSession::getValue('jenTypSkoleni')) {
            $sql .= " AND SZ.DruhSkoleni = '" . $this->database->addSlashes(RonPortalSession::getValue('typSkoleni')) . "'";
        }

        if (RonPortalSession::getValue('specifikaceSkoleni')) {
            $sql .= " AND S.Specifikace = '" . $this->database->addSlashes(RonPortalSession::getValue('specifikaceSkoleni')) . "'";
        }


        if (RonPortalSession::getValue('kategorieSkoleni') != "") {
            $sql .= " AND D.Kategorie = '" . $this->database->addSlashes(RonPortalSession::getValue('kategorieSkoleni')) . "'";
        }


        if(RonPortalSession::getValue('jenPosledniZaznam')){
            $sql .= " AND SZ.DatumVznikuPovinnosti = (Select Max(DatumVznikuPovinnosti) FROM GenerovaneSkoleni GES WHERE GES.DruhSkoleni = D.Kod AND GES.RC = SZ.RC ";

            if (RonPortalSession::getValue('jenStavSkoleni')) {
                $sql .= " AND GES.Potvrzeni = SZ.Potvrzeni ";
            }

            if (RonPortalSession::getValue('skoleniHodnocene')) {
                $sql .= " AND " . self::getSQLFilterHodnocene($this->database, 'GES');
            }
            $sql .= " ) ";
        }


        if(RonPortalSession::getValue('jenFiltrNaElearing')){
            if ($sql) {
                $sql .= ' AND ';
            }
            if (RonPortalSession::getValue('filtrNaElearing') == 1) {
                $sql .= "D.Elearning = 1";
            } elseif (RonPortalSession::getValue('filtrNaElearing') == 2) {
                $sql .= "D.Elearning = 0";
            }
        }

        return $sql;
    }
    
    public static function getSQLFilterHodnocene($database, $nazevTabulky)
    {
        return " EXISTS(SELECT DISTINCT HodnoceniSkoleni.TypGenerovani, TemaHodnoceni.PopisTematu, PripraveneTemaHodnoceni.ID, HodnoceniSkoleni.TypGenerovani " . // 0 - 3
        "FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON (PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni " .
        "    AND PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
        "   INNER JOIN TemaHodnoceni ON (TemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni AND TemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo) " .
        "   LEFT JOIN RozdeleniOsob RO_Hodnoceny ON (RO_Hodnoceny.RC = PripraveneTemaHodnoceni.HodnocenyRC AND RO_Hodnoceny.KodTypuRozdeleni = HodnoceniSkoleni.TypRozdeleni)" .
        "   LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RO_Hodnoceny.KodTypuRozdeleni AND Rozdeleni.Kod = RO_Hodnoceny.Rozdeleni) " .
        "   LEFT JOIN RozdeleniOsob RO_Nadrizeny ON (RO_Nadrizeny.KodTypuRozdeleni = Rozdeleni.RozdeleniRodice AND RO_Nadrizeny.Rozdeleni = Rozdeleni.Rodic) " .
        "WHERE PripraveneTemaHodnoceni.HodnocenyRC = $nazevTabulky.RC " .
        " AND HodnoceniSkoleni.CisloSkoleni = $nazevTabulky.CisloSkoleni " .
        " AND PripraveneTemaHodnoceni.Stav <> 2 " .
        " AND ((HodnoceniSkoleni.TypGenerovani = 3 AND PripraveneTemaHodnoceni.HodnocenyRC = '" . $database->addSlashes(User::getRC()) . "')" .
        "     OR (HodnoceniSkoleni.TypGenerovani = 2 AND HodnoceniSkoleni.RCHodnoticiOsoby = '" . $database->addSlashes(User::getRC()) . "')" .
        "     OR (HodnoceniSkoleni.TypGenerovani = 1 " .
        "            AND RO_Hodnoceny.PlatnostOd <= " . $database->dateToString(RonPortalSession::getDatumDo()) .
        "            AND (RO_Hodnoceny.PlatnostDo >= " . $database->dateToString(RonPortalSession::getDatumDo()) .
        "                   OR RO_Hodnoceny.PlatnostDo IS NULL) " .
        "            AND RO_Nadrizeny.RC = '" . $database->addSlashes(User::getRC()) . "'" .
        "            AND RO_Nadrizeny.PlatnostOd <= " . $database->dateToString(RonPortalSession::getDatumDo()) .
        "            AND (RO_Nadrizeny.PlatnostDo >= " . $database->dateToString(RonPortalSession::getDatumDo()) .
        "                   OR RO_Nadrizeny.PlatnostDo IS NULL) " .
        "        ) " .
        "     )" .
        ")";
    }
}
