<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 30.8.2017
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Browser;
use Ron\Controls\ComboBox;
use function Ron\ronText;


/**
 * Class SkoleniKurzyFilterForm
 * @package Ron\RonPortal
 */
class SkoleniKurzyFilterForm extends RonPortalFilterForm
{
    /** @var PravaNaOsoby $prava*/
    var $prava;

    /**
     * SkoleniKurzyFilterForm constructor.
     * @param Database $database
     * @param bool $useGet
     * @internal param string $action
     */
    public function __construct(Database $database, $useGet = false)
    {
        parent::__construct($database, $useGet);

        $this->prava = Prava::vratPrava('skoleni_lektor');
        Prava::nastavObdobiProPrava(time(),time());

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->skrytTlacitkoNastavit = false;
        $this->f_formaction = '?menuid=skoleni&action=skolenikurzy';

        //Zobrazeni FilterFormu pro nastaveni data od - do
        $this->nastavDatumOdDo(true, false, ronText('skoleni_v_obdobi') . ' ' . ronText('Od'));

    }


    /**
     * Volba druhu skoleni
     * @throws DBException
     */
    function renderTypSkoleni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypSkoleniAKurzu', ronText('skoleni_druh'). '&nbsp;&nbsp;', $_SESSION['jenTypSkoleniAKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $sql =
            "SELECT Distinct DruhSkoleni.Kod, DruhSkoleni.Popis
             FROM DruhSkoleni LEFT JOIN Skoleni ON (DruhSkoleni.Kod = Skoleni.DruhSkoleni)
             WHERE (Skoleni.StavSkoleni < 2)
             ORDER BY DruhSkoleni.Popis";

        $combo = new ComboBox('typSkoleniAKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenTypSkoleniAKurzu.checked) submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['typSkoleniAKurzu']);
        $combo->render();
        echo '</div>';
    }


    /**
     * Filtr pro aktualni obdobi nabidne aktualni mesic, aktualni ctvrtleti, aktualni pololeti
     */
    function renderAktualniObdobi()
    {
        $aktualniMesic = $this->vratAktualniMesic();
        $aktualniCtvrtleti = $this->vratAktualniCtvrtleti();
        $aktualniPololeti = $this->vratAktualniPololeti();


        //Hodnoty obdobi[0] 1-mesic, 3-ctvrtleti, 6-pololeti
        $obdobi = explode('_', $_SESSION['aktualniObdobi']);

        $mesic = "1_$aktualniMesic[0]_$aktualniMesic[1]";
        $ctvrtleti = "3_$aktualniCtvrtleti[0]_$aktualniCtvrtleti[1]";
        $pololeti = "6_$aktualniPololeti[0]_$aktualniPololeti[1]";

        $mesicText =  date('j.n.Y G:i:s', $aktualniMesic[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniMesic[1]);
        $ctvrtletiText =  date('j.n.Y G:i:s', $aktualniCtvrtleti[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniCtvrtleti[1]);
        $pololetiText =  date('j.n.Y G:i:s', $aktualniPololeti[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniPololeti[1]);
        $titulekCombaZvolenehoObdobi = "";

        if ($obdobi[0] == 1){
            $titulekCombaZvolenehoObdobi = $mesicText;
        }

        if ($obdobi[0] == 3){
            $titulekCombaZvolenehoObdobi = $ctvrtletiText;
        }

        if ($obdobi[0] == 6){
            $titulekCombaZvolenehoObdobi = $pololetiText;
        }

        echo "<div class='filterrow'>";
        echo "<span title='" . ronText('filtr_pouze_v_obdobi') . "' class='filtercaption'>";
        GuiUtils::renderCheckbox('jenAktualniObdobi', ronText('skoleni_v_obdobi'). '&nbsp;&nbsp;', $_SESSION['jenAktualniObdobi'],
            'onClick="submit()"');
        echo '</span>';


        $combo = "";
        $combo .= '<div class="tooltipek">';
        $combo .=
            "<select class='app_combo adsfilterforminput_full' name='aktualniObdobi' id='aktualniObdobi' onchange='this.form.submit()'>" .
                    "<option value='$mesic' " . ($obdobi[0] == 1 ? 'selected':'') . ">" . ronText('aktualni_mesic') . '</option>' .
                "</optgroup>" .

                    "<option value='$ctvrtleti' " . ($obdobi[0] == 3 ? 'selected':'') . ">" . ronText('aktualni_ctvrtleti') . '</option>' .
                "</optgroup>" .

                    "<option value='$pololeti' " . ($obdobi[0] == 6 ? 'selected':'') . ">" . ronText('aktualni_pololeti') . '</option>' .
                "</optgroup>" .
            '</select>';

        $combo .= '<span>' . ronText('skoleni_v_obdobi') . ':&nbsp;&nbsp;' .  $titulekCombaZvolenehoObdobi . '</span></div>';

        echo $combo;
        echo "<br>";
        echo '</div>';
    }


    /**
     * Z aktualniho data zjisti mesic
     * @return array
     */
    private function vratAktualniMesic()
    {
        $zacatekMesice = (DateUtils::firstDayOfMonth(time()));
        $konecMesice = (DateUtils::incMonth(DateUtils::firstDayOfMonth( time()), 1)-1);

        //Vraci pole s timestamp
        return [$zacatekMesice, $konecMesice];
    }


    /**
     * Z aktualniho data zjisti rozsah ctvrtleti
     * @return array
     */
    private function vratAktualniCtvrtleti()
    {
        $aktualniMesic = date('n', time());
        $aktualniCtvrtleti = [];

        if ($aktualniMesic >= 1 && $aktualniMesic <= 3){
            // 1 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 1, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);
        } elseif ($aktualniMesic >= 4 && $aktualniMesic <= 6){
            // 2 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 4, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);
        } elseif ($aktualniMesic >= 7 && $aktualniMesic <= 9) {
            // 3 ctvrtleti;
            $zacatek = mktime(0, 0, 0, 7, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);


        } elseif($aktualniMesic >= 10 && $aktualniMesic <= 12) {
            // 4 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 10, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);

        }

        //Vraci pole s timestamp
        return $aktualniCtvrtleti;
    }


    /**
     * Z aktualniho data zjisti rozsah pololeti
     * @return array
     */
    private function vratAktualniPololeti()
    {
        $aktualniMesic = date('n', time());
        $aktualniPololeti = [];

        if ($aktualniMesic >= 1 && $aktualniMesic <= 6){
            // 1 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 1, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 6) - 1;

            $aktualniPololeti[0] = ($zacatek);
            $aktualniPololeti[1] = ($konec);
        } elseif ($aktualniMesic >= 7 && $aktualniMesic <= 12) {
            // 2 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 7, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 6) - 1;

            $aktualniPololeti[0] = ($zacatek);
            $aktualniPololeti[1] = ($konec);
        }

        //Vraci pole s timestamp
        return $aktualniPololeti;
    }


    /**
     * Stav skoleni
     * @throws DBException
     */
     function renderStavSkoleni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenStavSkoleniAKurzu', ronText('Stav_skoleni'). '&nbsp;&nbsp;', $_SESSION['jenStavSkoleniAKurzu'],
            'onClick="submit()"');
        echo '</span>';

        $sql = "SELECT DISTINCT Kod, Popis FROM S_StavSkoleni  WHERE S_StavSkoleni.Kod < 2 ORDER BY Popis";

        $combo = new ComboBox('stavSkoleniAKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->htmlParams = 'onChange="if (jenStavSkoleniAKurzu.checked) submit();"';
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, $_SESSION['stavSkoleniAKurzu']);
        $combo->render();
        echo '</div>';
    }

    /**
     * Check pro pouze dlouhodoby kurz
     */
    function renderDlouhodobyKurz()
    {
        //Dlouhodoby kurz
        echo '<DIV>';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('dlouhodobyKurz', ronText('jen_dlouhodoby') . '&nbsp&nbsp&nbsp', $_SESSION['dlouhodobyKurz'],
                'onClick="submit()"');

        echo '</span>';
        echo '</DIV>';
    }

    function loadTypSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypSkoleniAKurzu'] = isset($_POST['jenTypSkoleniAKurzu']);
            $_SESSION['typSkoleniAKurzu'] = $_POST['typSkoleniAKurzu'];
        }
    }

    function loadAktualniObdobi()
    {
        if (isset($_POST['adsfilterform'])) {

            $_SESSION['jenAktualniObdobi'] = isset($_POST['jenAktualniObdobi']);
            $_SESSION['aktualniObdobi'] = $_POST['aktualniObdobi'];

        }
    }

    function loadStavSkoleni()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenStavSkoleniAKurzu'] = isset($_POST['jenStavSkoleniAKurzu']);
            $_SESSION['stavSkoleniAKurzu'] = $_POST['stavSkoleniAKurzu'];
        }
    }

    private function loadDlouhodobyKurz()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['dlouhodobyKurz'] = isset($_POST['dlouhodobyKurz']);
        }
    }

    function loadData()
    {
        $this->loadAktualniObdobi();
        $this->loadTypSkoleni();
        $this->loadStavSkoleni();
        $this->loadDlouhodobyKurz();
        parent::loadData();
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
//        $this->renderAktualniObdobi();
        $this->renderTypSkoleni();
        $this->renderStavSkoleni();
        $this->renderDlouhodobyKurz();
    }
}


/**
 * Class ZaznamenatUcastSkoleniAKurzu
 * @package Ron\RonPortal
 */
class ZaznamenatUcastSkoleniAKurzu extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
        $this->header = ronText('ucast_na_skoleni');
        $this->className = '';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        if ($rowData[18] == 1){
            $result .= "<div><a href='?menuid=skoleni&action=skoleniucast&record=$rowData[0]'>" . ronText('ucast_na_skoleni') . '</div></a>';
        }
        return $result;
    }
}


/**
 * Class ZaznamenatHodnoceniSkoleniAKurzu
 * @package Ron\RonPortal
 */
class ZaznamenatHodnoceniSkoleniAKurzu extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
        $this->header = ronText('hodnoceni_skoleni');
        $this->className = '';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        if ($rowData[18] == 1){

//            $result .= "<div><a href='#'>" . \Ron\ronText('hodnoceni_ucastniku_dlouhodobeho_skoleni') . '</div></a>';
        } else {
            $result .= "<div><a href='?menuid=skoleni&action=skolenihodnoceni&record=$rowData[0]'>" . ronText('hodnoceni_skoleni') . '</div></a>';
        }

        return $result;
    }
}



/**
 * Class VsechnaSkoleniAKurzy
 * @package Ron\RonPortal
 */
class SkoleniAKurzy extends Browser
{
    /** @var  PravaNaOsoby $pravaLektora */
    private $pravaLektora;

    var $filtrForm;

    /**
     * VsechnaSkoleniAKurzy constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database);

        $this->pravaLektora = Prava::vratPrava('skoleni_lektor');
        Prava::nastavObdobiProPrava(time(),time());

        $this->code = 'browserskolenikurzy';
        $this->tableId = 'browserskolenikurzy';
        $this->browserUrl = "?menuid=skoleni&action=skolenikurzy";

        $this->sqlSelectPart =
            "CisloSkoleni, Skoleni.Popis, DruhSkoleni, DatumPocatkuSkoleni, CasPocatkuSkoleni, DatumUkonceniSkoleni, CasUkonceniSkoleni, " .    //0..6
            "DelkaSkoleniDnu, DelkaSkoleniHodin, Kapacita, StavSkoleni, Skolitel, " .      //7..11
            "Hodnotici, DatumPotvrzeniUcasti, " .$database->castWideMemo('HodnoceniSkoleni') . ", " .   //12..14
            "SHodnocenim, Specifikace, Kredity, DlouhodobyKurz, " . $database->castWideMemo('ObsahSkoleni') . ", S_StavSkoleni.Popis S_SP, S_AnoNe.Popis S_AP";       //15..21

        $this->sqlFromPart = "Skoleni " .
            "LEFT JOIN S_StavSkoleni ON (Skoleni.StavSkoleni = S_StavSkoleni.Kod) " .
            "LEFT JOIN S_AnoNe ON (Skoleni.DlouhodobyKurz = S_AnoNe.Kod) ";

        $this->filtrForm = new SkoleniKurzyFilterForm($database);
        $this->filtrForm->loadData();

        $this->addField([BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'CisloSkoleni', BROWSER_FIELD_HEADER_TEXT => ronText("Skoleni_CisloSkoleni"),
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'DruhSkoleni', BROWSER_FIELD_HEADER_TEXT => ronText('skoleni_druh'),
            BROWSER_FIELD_ORDERING => 1 ]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'Popis', BROWSER_FIELD_HEADER_TEXT => ronText('Skoleni_Popis'),
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'popisSkoleniKurzy' ]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 20, BROWSER_FIELD_SQL_NAME => 'S_SP', BROWSER_FIELD_HEADER_TEXT => ronText('Stav_skoleni'),
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_WRAP => 1 ]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'DatumPocatkuSkoleni', BROWSER_FIELD_HEADER_TEXT => ronText('Skoleni_DatumZacatku'),
            BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);
        $this->defaultSort = count($this ->fields);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'DatumUkonceniSkoleni', BROWSER_FIELD_HEADER_TEXT => ronText('Skoleni_DatumUkonceni'),
            BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new ZaznamenatUcastSkoleniAKurzu());

        $this->addField(new ZaznamenatHodnoceniSkoleniAKurzu());

        $this->addField([BROWSER_FIELD_SQL_INDEX => 21, BROWSER_FIELD_SQL_NAME => 'S_AP', BROWSER_FIELD_HEADER_TEXT => ronText('Skoleni_DlouhodobyKurz'),
            BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center' ]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 9, BROWSER_FIELD_SQL_NAME => 'Kapacita', BROWSER_FIELD_HEADER_TEXT => ronText('Skoleni_Kapacita'),
            BROWSER_FIELD_FORMAT => 'number0', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

    }

    function renderFilter()
    {
        parent::renderFilter();
        $this->filtrForm->render();
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);



        echo "<tr class='$className'>";
                echo '<td></td>';
                echo '<td colspan="' . ($this->columnCount -1) . '" style="max-width:650px;"></td>';
        echo '</tr>';
    }


    public function enableLine($values)
    {
        //Dle priznaku 3 zobrazovat kurzy

        if ($this->pravaLektora->prava == 3 )
        {
            $zobrazitDanyTyp = true;

        } else {
            $zobrazitDanyTyp = false;
            if (trim($this->pravaLektora->priznak3) != '')
            {
                $povoleneTypyKurzu = explode(';', $this->pravaLektora->priznak3);
                $zobrazitDanyTyp = in_array($values[0], $povoleneTypyKurzu);
            }
        }

        return $zobrazitDanyTyp;
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();


        $result = '';
        $result .= 'Skoleni.StavSkoleni < 2';

        $obdobi = explode('_', $_SESSION['aktualniObdobi']);

        if($_SESSION['jenTypSkoleniAKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " Skoleni.DruhSkoleni = " . $params->addParameter($_SESSION['typSkoleniAKurzu']) ;
        }

        if($_SESSION['jenStavSkoleniAKurzu']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " Skoleni.StavSkoleni = " . $params->addParameter($_SESSION['stavSkoleniAKurzu']) . "'";
        }

        if($_SESSION['dlouhodobyKurz']){
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " Skoleni.DlouhodobyKurz = 1";
        }


        if (!$_SESSION['jenAktualniObdobi']) {
            if (RonPortalSession::getFiltrovatDatum()) {
                if ($result) {
                    $result .= ' AND ';
                }
                $result .= ' Skoleni.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumOd())) .
                    ' AND Skoleni.DatumUkonceniSkoleni <= ' . $params->addParameter( $this->database->timestampToParam(RonPortalSession::getDatumDo()));
            }
        }


        if ($_SESSION['jenAktualniObdobi']) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' Skoleni.DatumPocatkuSkoleni >= ' . $params->addParameter( $this->database->timestampToParam($obdobi[1])) .
                ' AND Skoleni.DatumUkonceniSkoleni <= ' . $params->addParameter( $this->database->timestampToParam($obdobi[2]));
        }

        return new SqlStatement($result, $params);
    }
}