<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 16.6.2017
 */

namespace Ron\RonPortal;

use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\Browser;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\Controls\DateInput;


/**
 * Jednoducha pomocna trida pro vkladani TD do tabulky, text se vklada primo pres ronText.
 * Class VlozTD
 * @package Ron\RonPortal
 */
class VlozTD
{
    private $class;
    private $titulek;
    private $data;

    /**
     * Jednoducha pomocna trida pro vkladani TD do tabulky, text se vklada primo pres ronText.
     * @param string $class Trida
     * @param string $titulek ronText
     * @param string $data Data ktera je nutno vypsat
     */
    public function __construct($class, $data, $titulek = null)
    {
        $this->class = $class;
        $this->titulek = $titulek;
        $this->data = $data;
    }

    /**
     * @return string
     */
    function __toString()
    {
        return $this->vlozData($this->class, $this->titulek, $this->data);
    }


    /**
     * @param $trida
     * @param $titulek
     * @param $data
     * @return string
     */
    function vlozData($trida, $titulek, $data)
    {
        $result = "<td ";
        $class = "";

        if ($trida){
            $class .= "class='" . $trida . "'";
        }

        if ($titulek){
            $titulek .= "title='" . \Ron\ronText($titulek) . "'";
        }

        $result .= " $class $titulek >";
        $result .= $data;
        $result .= "</td>";

        return $result;
    }
}


/**
 * Class UcastNaSkoleniFiltr
 * @package Ron\RonPortal
 */
class UcastNaSkoleniFiltr extends RonPortalFilterForm
{
    public $timestamp;
    public $jedenDen;
    public $desetDni;
    public $dvacetDni;
    public $tricetJednaDni;
    public $sedesatDni;
    public $rozsahZobrazeni;

    private $filtrObdobi;

    /**
     * UcastNaSkoleni constructor.
     * @param Database $database
     * @param string $kodKurzu
     * @param bool $useGet
     * @internal param string $action
     */
    public function __construct(Database $database, $kodKurzu, $useGet)
    {
        parent::__construct($database, $useGet);
        $this->f_formaction = '?menuid=skoleni&action=skoleniucast';
        $this->f_classname = 'filtercontainer leftcaptions';

        $this->jedenDen = new Option(1,\Ron\ronText('1_den'));
        $this->desetDni = new Option(10,\Ron\ronText('10_dni'));
        $this->dvacetDni = new Option(20,\Ron\ronText('20_dni'));
        $this->tricetJednaDni = new Option(31,\Ron\ronText('31_dni'));
        $this->sedesatDni = new Option(60,\Ron\ronText('60_dni'));


        if ($kodKurzu != ""){
            $_SESSION['kodDlouhodobehoKurzu'] = $kodKurzu;
        }

        //Nastaveni timestamp (pokud je datum ve spatnem formatu nebo neni v session nebo neni, nastavi se na aktu. datum)
        if (isset($_POST['datumTerminu'])){
            try {
                $this->timestamp = DateUtils::dateStringToTimestamp($_POST['datumTerminu']);
            } catch(\Exception $e) {
                MessageManager::addErrorText( 'Bad_time_format' );
                $this->timestamp = time();
            }
        } else {

            if ($_SESSION['datumTerminu']) {
                $this->timestamp = $_SESSION['datumTerminu'];
            } else {
                $this->timestamp = time();
            }
        }


        /**
         * Po vstupu do browseru
         */
        if (!isset($_SESSION['jenObdobiUcastiNaSkoleni']) && $_SESSION['jenObdobiUcastiNaSkoleni'] == ""){
            $_SESSION['jenObdobiUcastiNaSkoleni'] = 1;
            $this->filtrObdobi = $_SESSION['jenObdobiUcastiNaSkoleni'];
            $aktualniMesic = $this->vratAktualniMesic();
            $mesic = "1_$aktualniMesic[0]_$aktualniMesic[1]";
            $_SESSION['aktualniObdobiUcastiNaSkoleni'] = $mesic;
        }

        if (!isset($_SESSION['skrytFormular']) && $_SESSION['skrytFormular'] == ""){
            $_SESSION['skrytFormular'] = 1;
        }

        if (!isset($_SESSION['vyberpocetdnu']) && $_SESSION['vyberpocetdnu'] == ""){
            $_SESSION['vyberpocetdnu'] = 60;
        }

        //Nastaveni casu pokud jeste neni nastaveno od pulnoci do vteriny pred dvanactou
        $_SESSION['datumTerminu'] = DateUtils::startOfTheDay($this->timestamp);
        $_SESSION['datumdodataTerminu'] = ((DateUtils::incDay($_SESSION['datumTerminu'], $_SESSION['vyberpocetdnu'])) - 1);
    }


    function renderVyberSkoleni()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo \Ron\ronText('dlouhodoby_kurz');
        echo '</span>';

        $sql = "SELECT DISTINCT CisloSkoleni, Popis FROM Skoleni WHERE DlouhodobyKurz = 1 ORDER BY Popis";

        $combo = new ComboBox('kodDlouhodobehoKurzu');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->emptyRow = \Ron\ronText('nevybrano');
        $combo->htmlParams = 'onChange="submit();"';
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, $_SESSION['kodDlouhodobehoKurzu']);
        $combo->render();
        echo '</div>';
    }


    function renderNevyplneneFormulare(){
        echo '<DIV>';
        echo '<span class="filtercaption">';
            GuiUtils::renderCheckbox('jenNevyplneneFormulare', \Ron\ronText('jen_nevyplnene_formulare') . '&nbsp&nbsp&nbsp', $_SESSION['jenNevyplneneFormulare'],
                'onClick="submit()"');

        echo '</span>';
        echo '</DIV>';
    }


    function renderAktualniObdobi()
    {
        $aktualniMesic = $this->vratAktualniMesic();
        $aktualniCtvrtleti = $this->vratAktualniCtvrtleti();
        $aktualniPololeti = $this->vratAktualniPololeti();


        //Hodnoty obdobi[0] 1-mesic, 3-ctvrtleti, 6-pololeti
        $obdobi = explode('_', $_SESSION['aktualniObdobiUcastiNaSkoleni']);

        $mesic = "1_$aktualniMesic[0]_$aktualniMesic[1]";
        $ctvrtleti = "3_$aktualniCtvrtleti[0]_$aktualniCtvrtleti[1]";
        $pololeti = "6_$aktualniPololeti[0]_$aktualniPololeti[1]";

        $mesicText =  date('j.n.Y G:i:s', $aktualniMesic[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniMesic[1]);
        $ctvrtletiText =  date('j.n.Y G:i:s', $aktualniCtvrtleti[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniCtvrtleti[1]);
        $pololetiText =  date('j.n.Y G:i:s', $aktualniPololeti[0]) . "&nbsp;&nbsp;-&nbsp;&nbsp;" . date('j.n.Y G:i:s', $aktualniPololeti[1]);
        $titulekCombaZvolenehoObdobi = "";

        if ($obdobi[0] == 1){
            $titulekCombaZvolenehoObdobi = $mesicText;
        }

        if ($obdobi[0] == 3){
            $titulekCombaZvolenehoObdobi = $ctvrtletiText;
        }

        if ($obdobi[0] == 6){
            $titulekCombaZvolenehoObdobi = $pololetiText;
        }

        echo "<div class='filterrow'>";
        echo "<span class='filtercaption'>";

        $this->filtrObdobi = $_SESSION['jenObdobiUcastiNaSkoleni'] == 1 ? "checked" : "";
        echo "<input type='radio' name='jenObdobiUcastiNaSkoleni' value='1' $this->filtrObdobi onclick='submit();'>" . "&nbsp;" . \Ron\ronText('zobrazit_terminy_za');
        echo '</span>';


        $combo = "";
        $combo .= '<div class="popis_filtru_terminu">';
        $combo .=
            "<select class='app_combo adsfilterforminput_full' name='aktualniObdobiUcastiNaSkoleni' id='aktualniObdobiUcastiNaSkoleni' >" .
                    "<option value='$mesic' " . ($obdobi[0] == 1 ? 'selected':'') . ">" . \Ron\ronText('aktualni_mesic') . '</option>' .

                    "<option value='$ctvrtleti' " . ($obdobi[0] == 3 ? 'selected':'') . ">" . \Ron\ronText('aktualni_ctvrtleti') . '</option>' .

                    "<option value='$pololeti' " . ($obdobi[0] == 6 ? 'selected':'') . ">" . \Ron\ronText('aktualni_pololeti') . '</option>' .
            '</select>';

        $combo .= '<span>' . \Ron\ronText('skoleni_v_obdobi') . ':&nbsp;&nbsp;' .  $titulekCombaZvolenehoObdobi . '</span></div>';

        echo $combo;
        echo "<br>";
        echo '</div>';
    }


    function renderOmezeniTerminu()
    {
        echo '<div class="filterrow">';

        if ($_SESSION['vyberpocetdnu'] == 1) {
            $zobrazit_terminy = \Ron\ronText('zobrazit_termin_za');
        } else {
            $zobrazit_terminy = \Ron\ronText('zobrazit_terminy_za');
        }

        echo "<span class='filtercaption'>";
//        GuiUtils::renderCheckbox('jenObdobiUcastiNaSkoleni', $zobrazit_terminy . '&nbsp;&nbsp;', $_SESSION['jenObdobiUcastiNaSkoleni'],
//            'onClick="submit()"');

        $this->filtrObdobi = $_SESSION['jenObdobiUcastiNaSkoleni'] == 2 ? "checked" : "";
        echo "<input type='radio' name='jenObdobiUcastiNaSkoleni' value='2' $this->filtrObdobi onclick='submit();'>" . "&nbsp;" . $zobrazit_terminy;
        echo '</span>';

        echo '<span class="zaznamy_od">';
        echo '<div class="popis_filtru_terminu">';
        $combo = new ComboBox('vyberpocetdnu', [$this->jedenDen, $this->desetDni, $this->dvacetDni, $this->tricetJednaDni, $this->sedesatDni]);
        $combo->className = 'vyber_dni_combo adsfilterforminput_kod';
        $combo->htmlParams = 'onChange="submit();"';
        $combo->selectedOption = $combo->findOption((int)$_SESSION['vyberpocetdnu'] != '' ? (int)$_SESSION['vyberpocetdnu'] : 1);
        $combo->render();
        echo '<span>' . \Ron\ronText('zvolena_doba') . ': ' . $this->rozsahZobrazeni . '</span>';
        echo '</div>';

        echo '<div class="popis_filtru_terminu">';
        echo '&nbsp;&nbsp;' . \Ron\ronText('Od') . '&nbsp;&nbsp;&nbsp;';
        DateInput::renderDateSelect('datumTerminu', date('j.n.Y', $this->timestamp), true);
        echo '<span>' . \Ron\ronText('pouze_dnesni_den') . '</span>';
        echo '</div>';

        echo '</span>';

        echo '</div>';
    }

    function renderDetailDlouhodobehoKurzu()
    {
        $sql = "SELECT " .
            "CisloSkoleni, Popis, DruhSkoleni, DatumPocatkuSkoleni, CasPocatkuSkoleni, DatumUkonceniSkoleni, " .        //0- 5
            "CasUkonceniSkoleni, MistoSkoleni, DelkaSkoleniDnu, DelkaSkoleniHodin, Kapacita, NabidkaSkoleni, " .        //6 - 11
            "StavSkoleni, SkoliciFirma, Skolitel, Hodnotici, HodnoceniUzavreno, DatumPotvrzeniUcasti, " .               //12 - 17
            $this->database->castWideMemo('HodnoceniSkoleni') . ", UrceniCenySkoleni, SkoleniSplatneDne, SkoleniUhrazenoDne, SHodnocenim, " .       //18 - 22
            "ZaplacenaZaloha, ZalohaUhrazenaDne, ZakladDPH1, CenaZaSkoleni, Specifikace, ZakladDPH2, " .                //23 - 28
            "ZakladDPH3, DPH1, DPH2, DPH3, Dan1, Dan2, Dan3, Kredity, DlouhodobyKurz," . $this->database->castWideMemo('ObsahSkoleni') . ", " .       //29 - 38
            "(Select Popis From S_StavSkoleni Where Kod = StavSkoleni) Stav_skoleni ";
        $sql .= "FROM Skoleni ";
        $sql .= "WHERE Skoleni.CisloSkoleni = '" . $_SESSION['kodDlouhodobehoKurzu'] . "' AND Skoleni.DlouhodobyKurz = 1 ";
        $cursor = $this->database->getSqlResult($sql);
        $row = $cursor->fetch();

        echo '<div class="filterrow">';
        echo    '<strong><span class="filtercaption">' . "&nbsp;&nbsp;&nbsp;" .  '</span></strong>';
        echo    '<span class="filtercaption">' . '</span>';
        echo '</div>';

        $_SESSION['popisKurzu'] = $row[1];

        /**
         * Popis zacatku - konce casem od - do skoleni napr.        1.7.2017 - 30.9.2017     8:00 - 16:00
         **/
        echo '<div class="filterrow">';
        echo    '<span class="filtercaption">' . \Ron\ronText('zacatek_konec_skoleni').  '</span>';
        echo    '<strong><span class="filtercaption">' .
                    StringUtils::format('date', $this->database->stringToTimestamp($row[3])) .
                    "&nbsp;&nbsp;-" .
                    "&nbsp;&nbsp;" .
                StringUtils::format('date', $this->database->stringToTimestamp($row[5]));
        echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';

        echo        StringUtils::format('time', $this->database->stringToTimestamp($row[4])) .
                    "&nbsp;&nbsp;" .
                    "-&nbsp;&nbsp;" .
                    StringUtils::format('time', $this->database->stringToTimestamp($row[6]));
        echo    '</span></strong>';
        echo '</div>';

        /**
         * Popis se jmenem skolitele
         */
        echo '<div class="filterrow">';
        echo    '<span class="filtercaption">' . \Ron\ronText('skolitel').  '</span>';
        echo    '<strong><span class="filtercaption">' . $row[14] . '</span></strong>';
        echo '</div>';
    }


    function renderPridaniTerminu()
    {
        if ($_SESSION['terminJeObsazen'] == 2) {
            //Kombinuji javascript a ronText
            echo "<script type='text/javascript'>alert('" . \Ron\ronText('termin_je_obsazen') . "');</script>";
            $_SESSION['terminJeObsazen'] = 0;
        }

        echo '<div class="filterrow">';
        echo '<span class="filtercaption radekTerminu">';
        echo '</span>';
        echo '</div>';

        echo '<div class="filterrow">';
            echo "<button type='button' id='buttonNovyTermin' class='buttonJakoOdkaz'>" . \Ron\ronText("vytvorit_termin") . "</button>";

            echo '<span class="filtercaption">';
            echo    $this->menuVytvoritTermin();
            echo '</span>';

            echo '<span class="filtercaption">';
            echo '</span>';

            echo '<span class="filtercaption" style="text-align: right">';
            echo "<a href='?menuid=skoleni&action=skolenikurzy'>" . \Ron\ronText('zpet_na_seznam')  . "</a>";
            echo '</span>';
        echo '</div>';
    }

    /**
     * @return string
     */
    private function menuVytvoritTermin()
    {
        $result  = "<input type='hidden' name='vytvoritNovyTermin' value='1'>";
        $result  .= "<input type='hidden' name='kodKurzu' value='" . $_SESSION['kodDlouhodobehoKurzu'] . "'>";
        $result  .= "<ul class='vytvoritTerminSeznam'>";

        $novyTermin = "<input class='button' id='vytvoritNovyTerminBtn' name='vytvoritNovyTerminBtn' type='submit' value='" . \Ron\ronText("vytvorit_novy_termin") . "'>";

        $datumCasTerminu = '<input title="' . \Ron\ronText("datum_koleckem_mysi") . '" type="text" class="app_input inputDate" size="10" value="' . '" id="pridaniTerminu" name="pridaniTerminu">';
        $datumCasTerminu .= "<img src='images/16_calendar.png' class='showcalendar' data-roncalendar-inputname='pridaniTerminu' alt='showcalendar' />";
        $datumCasTerminu .= '&nbsp;';
        $datumCasTerminu .= '<input title="' . \Ron\ronText("cas_koleckem_mysi") . '" type="text" size="3" class="editaceCasuTerminu" value="' . '" id="pridaniTerminuTime_' . 1 . '" name="pridaniTerminu_time">';

        $textarea = "<textarea id='poznamkaKNovemuTerminu' name='poznamkaKNovemuTerminu' class='' rows='1' cols='30'>" . $_SESSION['poznamkaNovehoTerminu'] . "</textarea>";

        $result .= "<li>" . $datumCasTerminu . "</li>";
        $result .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
        $result .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
        $result .= "<li>" . $novyTermin . "</li>";
        $result .= "<br>";
        $result .= "<li>" . $textarea . "</li>";

        $result .= "</ul>";

        return $result;
    }


    //TODO Smazat?
    function renderDokumentyTerminu()
    {
        echo "</form>";
        echo '<div class="filterrow">';
        echo '<span class="filtercaption radekTerminu">';
        echo \Ron\ronText('dokumenty_kurzu');
        echo "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
        echo '</span>';

        echo '<span id="pripojit_novy_dokument_terminu_img" class="plus">';
        echo "<img  src='images/plus_big.png'>";
        echo '</span>';
        echo "</div>";

        echo "<div id='dokumentyKurzu' style='display: none'>";
        echo    $this->menuPripojitDokumentKurzu();
        echo "</div>";
        echo "<br>";

        echo '</div>';
    }

    //TODO Smazat?
    private function menuPripojitDokumentKurzu()
    {
        $result  = "<input type='hidden' name='kodKurzu' value='" . $_SESSION['kodDlouhodobehoKurzu'] . "'>";
        $result  .= "<ul class='pripojitDokumentKurzu'>";
        $result .= "</ul>";
        $result .= "<br>";

        $result .="<div id='dokumentyKurzu'>";
        $model = Model::getInstance();
        $browserDokumentuKurzu = $model->getData('browserDokumentuKurzu');
        $browserDokumentuKurzu->render();
        $result .="</div>";

        return $result;
    }


    function loadDatumFiltru()
    {
        if (isset($_POST['adsfilterform'])) {
            try{
                //V pripade ze se zada neexistujici den v mesici tak skript skoncil chybou, ze ktere se jen tak neda odejit - proto jsem toto osetril
                // a nastavuji v pripade chyby aktualni datum
                $_SESSION['datumTerminu'] = DateUtils::dateStringToTimestamp($_POST['datumTerminu']);
            } catch (\Exception $ex){
                $_SESSION['datumTerminu'] = time();
            }
        }
    }


    private function loadNevyplneneFormulare()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenNevyplneneFormulare'] = isset($_POST['jenNevyplneneFormulare']);
        }
    }

    function loadAktualniObdobi()
    {
        if (isset($_POST['adsfilterform']))
        {
            $_SESSION['aktualniObdobiUcastiNaSkoleni'] = $_POST['aktualniObdobiUcastiNaSkoleni'];
        }
    }

    function loadVolbaObdobi()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['jenObdobiUcastiNaSkoleni'] = $_POST['jenObdobiUcastiNaSkoleni'];
        }
    }


    function loadSkoleni()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['kodDlouhodobehoKurzu'] = $_POST['kodDlouhodobehoKurzu'];
        }
    }


    private function loadRozsahPocetDnu()
    {
        if (isset($_POST['adsfilterform'])) {
            if ($_POST['vyberpocetdnu'] != "") {
                $_SESSION['vyberpocetdnu'] = $_POST['vyberpocetdnu'];
            }
        }
    }


    private function loadNastaveniZobrazeniRozsahu()
    {
        $_SESSION['datumTerminu'] = DateUtils::startOfTheDay($this->timestamp);
        $_SESSION['datumdodataTerminu'] = ((DateUtils::incDay($_SESSION['datumTerminu'], $_SESSION['vyberpocetdnu'])) - 1);

        $this->rozsahZobrazeni = '';
        $this->rozsahZobrazeni .= '&nbsp;&nbsp;' . date('j.n.Y G:i:s', $_SESSION['datumTerminu']);
        $this->rozsahZobrazeni .= '&nbsp;&nbsp;&nbsp;-&nbsp;&nbsp;&nbsp;' . date('j.n.Y G:i:s', $_SESSION['datumdodataTerminu']) . '&nbsp;&nbsp;';
    }

    function loadZobrazenaDoba()
    {
        if(isset($_POST['adsfilterform'])) {
            $_SESSION['rozsahTerminu'] = $this->rozsahZobrazeni;
        }
    }

    function loadData()
    {
        $this->loadVolbaObdobi();
        $this->loadAktualniObdobi();
        $this->loadDatumFiltru();
        $this->loadRozsahPocetDnu();
        $this->loadSkoleni();
        $this->loadZobrazenaDoba();
        $this->loadNevyplneneFormulare();
        $this->loadNastaveniZobrazeniRozsahu();
    }


    function renderFilters()
    {
        $this->renderVyberSkoleni();
        $this->renderOmezeniTerminu();
        $this->renderAktualniObdobi();
        $this->renderNevyplneneFormulare();
        $this->renderDetailDlouhodobehoKurzu();
        $this->renderPridaniTerminu();
//        $this->renderDokumentyTerminu();
    }



    /**
     * Z aktualniho data zjisti mesic
     * @return array
     */
    private function vratAktualniMesic()
    {
        $zacatekMesice = (DateUtils::firstDayOfMonth(time()));
        $konecMesice = (DateUtils::incMonth(DateUtils::firstDayOfMonth( time()), 1)-1);

        //Vraci pole s timestamp
        return [$zacatekMesice, $konecMesice];
    }


    /**
     * Z aktualniho data zjisti rozsah ctvrtleti
     * @return array
     */
    private function vratAktualniCtvrtleti()
    {
        $aktualniMesic = date('n', time());
        $aktualniCtvrtleti = [];

        if ($aktualniMesic >= 1 && $aktualniMesic <= 3){
            // 1 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 1, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);
        } elseif ($aktualniMesic >= 4 && $aktualniMesic <= 6){
            // 2 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 4, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);
        } elseif ($aktualniMesic >= 7 && $aktualniMesic <= 9) {
            // 3 ctvrtleti;
            $zacatek = mktime(0, 0, 0, 7, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);


        } elseif($aktualniMesic >= 10 && $aktualniMesic <= 12) {
            // 4 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 10, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 3) - 1;

            $aktualniCtvrtleti[0] = ($zacatek);
            $aktualniCtvrtleti[1] = ($konec);

        }

        //Vraci pole s timestamp
        return $aktualniCtvrtleti;
    }


    /**
     * Z aktualniho data zjisti rozsah pololeti
     * @return array
     */
    private function vratAktualniPololeti()
    {
        $aktualniMesic = date('n', time());
        $aktualniPololeti = [];

        if ($aktualniMesic >= 1 && $aktualniMesic <= 6){
            // 1 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 1, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 6) - 1;

            $aktualniPololeti[0] = ($zacatek);
            $aktualniPololeti[1] = ($konec);
        } elseif ($aktualniMesic >= 7 && $aktualniMesic <= 12) {
            // 2 ctvrtleti;

            $zacatek = mktime(0, 0, 0, 7, 1, date('Y', time()));
            $konec = DateUtils::incMonth($zacatek, 6) - 1;

            $aktualniPololeti[0] = ($zacatek);
            $aktualniPololeti[1] = ($konec);
        }

        //Vraci pole s timestamp
        return $aktualniPololeti;
    }
}


/**
 * Class UcastNaSkoleniField
 * @package Ron\RonPortal
 */
class UcastNaSkoleniField extends BrowserField
{

    private $ucast = 0;
    private $neucast = 0;

    /**
     * UcastNaSkoleniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('ucast_na_skoleni');
        $this->align = 'center';
    }


    /**
     * @param $datum
     * @return array
     * @throws \Ron\DB\DBException
     */
    private function nacistUcastnikyZajezdu($datum)
    {
        $params = $this->database->createParameters();
        $cnt = 0;
        $ucastnici = [];
        $cisloSkoleni = $_SESSION['kodDlouhodobehoKurzu'];

        $sql = "SELECT PracovniPomer.ID, COALESCE (Osoba.prijmeni, '') {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.jmeno,'') '" . \Ron\ronText('jmeno') . "', " .           //0 - 1
                    "UTS.Datum, UTS.Pritomen, UTS.DuvodNepritomnosti, " . $this->database->castWideMemo('UTS.Poznamka') . ", " .         //2 - 5
                    "(SELECT Count(GS.RC) FROM GenerovaneSkoleni GS INNER JOIN Osoba ON (Osoba.RC = GS.RC) WHERE (GS.CisloSkoleni = " . $params->addParameter($cisloSkoleni) . ") AND (GS.Potvrzeni<5)) 'PocetUcastniku', " .   //6
                    "(SELECT COUNT(Pritomen) FROM UcastNaTerminuSkoleni WHERE UcastNaTerminuSkoleni.Pritomen = 1 AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum) 'Pritomnych' " .   //7
                " FROM GenerovaneSkoleni GS LEFT JOIN Osoba ON (Osoba.RC = GS.RC) LEFT JOIN PracovniPomer On (GS.RC = PracovniPomer.RC) " .
                    "LEFT JOIN TerminSkoleni ON (TerminSkoleni.CisloSkoleni=GS.CisloSkoleni) " .
                    "LEFT JOIN UcastNaTerminuSkoleni UTS ON (UTS.CisloSkoleni=TerminSkoleni.CisloSkoleni AND UTS.RC=GS.RC AND UTS.Datum=TerminSkoleni.Datum AND UTS.RC = PracovniPomer.RC) " .
                " WHERE (GS.CisloSkoleni = " . $params->addParameter($cisloSkoleni) . ") AND (GS.Potvrzeni<5)  AND TerminSkoleni.Datum = " . $params ->addParameter( $this->database->dateToParam($datum)) .
                " AND PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = GS.RC)" .

                " ORDER BY Osoba.Prijmeni";


        MessageManager::addDebugLog($sql);
        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $ucastnici[$cnt] = $row;
            $cnt++;
        }
        return $ucastnici;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws \Ron\DB\DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);
        $sestavenyForm = $this->sestavFormularUcasti($rowData);

        //Kontejner pro zobrazeni dialogu
        $dialog = "";
        $dialog .= '<div class="dialog-overlay" id="dialog-overlay_' . $timestampTerminuSkoleni .'"></div>';
        $dialog .=   '<div class="dialog-box" id="dialog-box_' . $timestampTerminuSkoleni .'">';
        $dialog .=       '<div class="dialog-content">';
        $dialog .=       '<div class="dialog-message" id="dialog-message_' . $timestampTerminuSkoleni .'">';
        $dialog .=          $sestavenyForm;   //Vygenerovany formular, umisti se do kontejneru cimz vznikne dialog na ktery se dale aplikuje jQuery
        $dialog .=   '</div></div>';
        $dialog .= '</div>';

        $result .= $dialog;
        //Button na ktery se navaze udalost pro zobrazeni dialogu
        $result .= "<button type='button' id='buttonProDialog_" . $timestampTerminuSkoleni . "' class='do_nabidky_tlacitko'>" . \Ron\ronText("zobraz_dialog_ucasti") . "</button>";

        return $result;
    }


    /** Sestaveni formulare
     * @param $rowData
     * @return string
     * @throws \Ron\DB\DBException
     */
    function sestavFormularUcasti($rowData)
    {
        $datumSkoleni = DBUtils::getFormatedValue($this ->database, 'date', $rowData[1]);
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);
        $formular = "";
        $radkyTabulky = "";
        $cnt = 0;
        $pocetPritomnych = 0;
        $pocetNepritomnych = 0;
        $pocetUcastniku = 0;
        $vsichniUcastniciPritomni = true;
        $vsichniUcastniciOmluveni = true;
        $ucastnici = $this->nacistUcastnikyZajezdu($timestampTerminuSkoleni);

        $formular .= "<form enctype='multipart/form-data' method='post' id='ucastNaSkoleni_$timestampTerminuSkoleni' name='ucastNaSkoleni' action='?menuid=skoleni&action=skoleniucast&' class='formular_ucasti'>";
        $formular .= '<input type="hidden" name="timestampTerminuSkoleni" value="' . $timestampTerminuSkoleni . '">';
        $formular .= '<table class="tabulka_ucast">';

        $prazdnaPoznamka = false;
        //Pokud existuje alespoň jedna prazdna poznamka - pak ma smysl zobrazit "tlacitko" pro kopirovani poznamky
        foreach ($ucastnici as $ucastnik => $row)
        {
            if ($row[5] == ""){
                $prazdnaPoznamka = true;
                break;
            }
        }

        //Vytvoreni jednotlivych radku
        foreach ($ucastnici as $ucastnik => $row)
        {
            $classJmeno = "jmeno";
            $cnt++;
            $omluveno = "";
            $pocetUcastniku = $row[6];

            //Obarveni radku
            if ($cnt % 2 != 0) {
                $classRadku = "licheRadky radekTabulky";
            } else {
                $classRadku = "sudeRadky radekTabulky";
            }

            //Pritomen
            if ($row[3] == 1) {
                $classRadku .= " pritomni";
                //Pro procentualni informaci o ucasti
                ++$pocetPritomnych;
                $pritomen = "<input class='' data-ron-timestamp='$timestampTerminuSkoleni' type='checkbox' id='" . $cnt .  "_pritomenNaTerminu_$timestampTerminuSkoleni' name='" . $cnt .  "_pritomenNaTerminu' checked value='1'>";
                //Pokud je duvod pro vykresleni checkboxu pak pridat tridu pro manipulaci v jQ
                $classPritomen = "tucne vpravo sirkaCheck oblastCheckboxu";
                $classCheckboxu = " vpravo sirkaCheck";
                $classJmeno .= " tucne sirkaJmena";
            } else {
                $classRadku .= " nepritomni";
                $vsichniUcastniciPritomni = false;
                $pritomen = "<input data-ron-timestamp='$timestampTerminuSkoleni' type='checkbox' id='" . $cnt .  "_pritomenNaTerminu_$timestampTerminuSkoleni' name='" . $cnt .  "_pritomenNaTerminu' value='1'>";
                //Pokud je duvod pro vykresleni checkboxu pak pridat class pro manipulaci v jQ
                $classPritomen = " vpravo sirkaCheck oblastCheckboxu";
                $classCheckboxu = " vpravo sirkaCheck oblastCheckboxu";
            }

            //Nepritomen - dva stavy omluveno / neomluveno
            if ($row[3] == 0){
                $classCheckboxu .= " vpravo sirkaCheck";
                if ($row[4] == 1) {
                    ++$pocetNepritomnych;
                    //Pokud je duvod pro vykresleni checkboxu pak pridat tridu pro manipulaci v jQ
                    $classCheckboxu .= " oblastCheckboxu";
                    $omluveno = "<input  class='' data-ron-timestamp='$timestampTerminuSkoleni' type='checkbox' id='" . $cnt .  "_omluvenaUcast_$timestampTerminuSkoleni' name='" . $cnt .  "_omluvenaUcast' checked value='1' >";
                } else {
                    $vsichniUcastniciOmluveni = false;
                    $omluveno = "<input data-ron-timestamp='$timestampTerminuSkoleni' type='checkbox' id='" . $cnt .  "_omluvenaUcast_$timestampTerminuSkoleni' name='" . $cnt .  "_omluvenaUcast' value='1' >";
                }
            }

            $radkyTabulky .= "<tr class='" . $classRadku . "'>";
            //Sloupec 1 ------------------------------------------------------            //Jmeno, v hidden poli dale prenasim puvodni hodnoty, slozi k porovnani a pokud doslo ke zmene pak zapsat zmenu do logu
            $radkyTabulky .=  new VlozTD($classJmeno, StringUtils::htmlTextConvert($row[1]) . "<input id='" . $cnt . "_idUcastnika' name='" . $cnt . "_idUcastnika' type='hidden' value='" . $row[0] . "'>"
             . "<input id='" . $cnt . "_idUcastnikaPuvodniHodnoty' name='" . $cnt . "_idUcastnikaPuvodniHodnoty' type='hidden' value='" . $row[3]. ";;" . $row[4]. ";;" . $row[5]. "'>");

            //Sloupec 3 ------------------------------------------------------     //checkbox zdali je pritomen
            $radkyTabulky .=  new VlozTD($classPritomen, $pritomen);

            //Sloupec 4 ------------------------------------------------------  //checkbox zdali je omluven
            $radkyTabulky .=  new VlozTD($classCheckboxu, $omluveno);

            //Sloupec 5 ------------------------------------------------------
            $radkyTabulky .=  new VlozTD("center sirkaPoznamky", "<textarea id='" . $cnt . "_poznamkaKUchazeci_$timestampTerminuSkoleni' name='" . $cnt . "_poznamkaKUchazeci' class='textarea' rows='1' cols='30'>" . $row[5] . "</textarea>");

            //Sloupec 6 ------------------------------------------------------
            if ($prazdnaPoznamka == true) {
                if ($row[5] != "") {
                    $radkyTabulky .=  "<td title='" . \Ron\ronText('kopirovat_do_prazdnych_poznamek') . "' class='edit_textarea' id='" . $cnt . "_copyPoznamkyKUchazeci_$timestampTerminuSkoleni'>";
                    $radkyTabulky .= "<span ><img  src='images/16_new_as_copy.png'></span>";
                    $radkyTabulky .=  "</td>";
                }
            }
            $radkyTabulky .= '</tr>';
        }


        //Popis pripojuji az znam promenne k vypoctu
        $formular .= $this->popisTabulky($datumSkoleni, $pocetUcastniku, $pocetPritomnych, $timestampTerminuSkoleni, $pocetNepritomnych);

        $skoleniHeader = "skoleniHeader_" . $timestampTerminuSkoleni;
        $nadpisy  = '<tbody id="' . $skoleniHeader . '" class="hlavickaHodnoceniUcasti tbody_info">';
        $nadpisy .=   '<th class="center nadpisUcastiJmeno">' . \Ron\ronText('Jmeno') . '</th>';

        $nadpisy .=   '<th class="vpravo nadpisUcastiPritomen">' . \Ron\ronText('pritomen') .
            '<input data-ron-timestamp="' . $timestampTerminuSkoleni . '" id="pritomenNaTerminuAll_' . $timestampTerminuSkoleni .
            '" type="checkbox" ' . (($vsichniUcastniciPritomni==true) ? "checked" : "") . '></th>';

        $nadpisy .=   '<th class="vpravo nadpisUcastiOmluven">' . \Ron\ronText('omluven') .
            '<input data-ron-timestamp="' . $timestampTerminuSkoleni . '" id="omluvenaUcastAll_' . $timestampTerminuSkoleni .
            '" type="checkbox" ' . (($vsichniUcastniciOmluveni==true && $vsichniUcastniciPritomni!=true) ? "checked" : "") . '></th>';

        $nadpisy .=   '<th colspan="2" class="center nadpisUcastiPoznamka">' . \Ron\ronText('Poznamka') . '</th>';
        $nadpisy .= '</tbody>';


        //Sestaveni celeho formulare
        $formular .=    $nadpisy;
        $formular .=    "<tbody class='tbody_ucasti'>";
        $formular .=    $radkyTabulky;
        $formular .=    "</tbody>";
        $formular .=    "</table>";
        $formular .=    "<input type='hidden' name='pocetRadku' value='" . $cnt ."'>";

        $formular .=    "<table id='tlacitkaObsluhy_$timestampTerminuSkoleni' >";
        $radek  =       '<tr>';
        $radek .=           '<td></td>';
        $radek .=           new VlozTD("", '<button type="button" id="tlacitkoStorno_' . $timestampTerminuSkoleni . '" class="button">' . \Ron\ronText("Zpet_na_seznam") . '</button>');
        $radek .=           new VlozTD("", "<input class='button' type='reset' value='" . \Ron\ronText('form.Original') . "' >");
        $radek .=           new VlozTD("", "<input class='button' type='submit' value='" . \Ron\ronText('form.Store') . "' >");
        $radek .=       "</tr>";
        $radek .=       "<tr>";
        $radek .=           '<td>&nbsp;</td>';
        $radek .=       "</tr>";
        $formular .=    $radek;
        $formular .=    '</table>';
        $formular .= '</form>';


        return $formular;
    }

    /** Vygeneruje popis tabulky
     * @param $datumSkoleni
     * @param $pocetUcastniku
     * @param $pocetPritomnych
     * @param $timestampTerminuSkoleni
     * @param $pocetNepritomnych
     * @return string
     */
    private function popisTabulky ($datumSkoleni, $pocetUcastniku, $pocetPritomnych, $timestampTerminuSkoleni, $pocetNepritomnych)
    {
        $this->ucast = 0;
        $this->neucast = 0;

        if ($pocetPritomnych != "") {
            $this->ucast = round((($pocetPritomnych / $pocetUcastniku) * 100), 0);
        }

        if ($pocetNepritomnych != "") {
            $this->neucast = round((($pocetNepritomnych / $pocetUcastniku) * 100), 0);
        }

        $infoHromadnehoVyberuCheckboxu = \Ron\ronText('infoHromadnehoVyberuCheckboxu');
        //radek 1 ----------
        $popisTabulky  =    '<tbody class="tbody_info">';
        $popisTabulky .=    '<tr>';
        $popisTabulky .=        '<td class="nadpis_sekce" colspan="5">';
        $popisTabulky .=            \Ron\ronText('editace_ucasti_na_skoleni');
        $popisTabulky .=        '</td>';
        $popisTabulky .=    '</tr>';

        $popisTabulky .=    '<tr class="">';
        $popisTabulky .=        '<td class="infoFormuKurzu" colspan="1">';
        $popisTabulky .=            \Ron\ronText('nazev') . ": ";
        $popisTabulky .=        '</td>';
        $popisTabulky .=        '<td colspan="3" class="tucne">';
        $popisTabulky .=            StringUtils::htmlTextConvert($_SESSION['popisKurzu']);
        $popisTabulky .=        '</td>';
        $popisTabulky .=    '</tr>';

        //radek 2 ----------
        $popisTabulky .=    '<tr class="">';
        $popisTabulky .=        '<td class="infoFormuKurzu">';
        $popisTabulky .=            \Ron\ronText('termin') . ": ";
        $popisTabulky .=        '</td>';
        $popisTabulky .=        '<td  colspan="3"  class="tucne">';
        $popisTabulky .=            $datumSkoleni;
        $popisTabulky .=        '</td>';
        $popisTabulky .=    '</tr>';

        //radek 3 ----------
        $popisTabulky .=    '<tr class="">';
        $popisTabulky .=        '<td class="infoFormuKurzu">';
        $popisTabulky .=            \Ron\ronText('pocetUcastniku') . " / " . \Ron\ronText('pritomni') . " / " . \Ron\ronText('ucast') . ": ";
        $popisTabulky .=        '</td>';
        $popisTabulky .=        '<td  colspan="3"  class="tucne">';
        $popisTabulky .=            $pocetUcastniku;
        $popisTabulky .=        ' / ';
        $popisTabulky .=            $pocetPritomnych;
        $popisTabulky .=        ' / ';
        $popisTabulky .=        (($this->ucast != 0) ? $this->ucast . "%" : \Ron\ronText('zadna'));
        $popisTabulky .=        "<input type='hidden' id='hodnotaUcastiNaSkoleni_" . $timestampTerminuSkoleni . "' value='$this->ucast'>";
        $popisTabulky .=        "<input type='hidden' id='hodnotaNeucastiNaSkoleni_" . $timestampTerminuSkoleni . "' value='$this->neucast'>";
        $popisTabulky .=        '</td>';
        $popisTabulky .=    '</tr>';
        $popisTabulky .=    '<tr><td>&nbsp;</td></tr>';

        //radek 4 ----------
        $popisTabulky .=    '<tr class="">';
        $popisTabulky .=        '<td class="infoFormuKurzu">';
        $popisTabulky .=            \Ron\ronText('zobrazit') . ": ";
        $popisTabulky .=        '</td>';
        $popisTabulky .=        '<td colspan="3">';
        $popisTabulky .=           \Ron\ronText("vsechny") . "<input type='radio' id='filtr_pritomnych_" . $timestampTerminuSkoleni . "' name='filtr_pritomnych' checked value='0'>&nbsp;&nbsp;&nbsp;";
        $popisTabulky .=           \Ron\ronText("pritomne") . "<input type='radio' id='filtr_pritomnych_" . $timestampTerminuSkoleni . "' name='filtr_pritomnych'  value='1'>&nbsp;&nbsp;&nbsp;";
        $popisTabulky .=           \Ron\ronText("nepritomne") . "<input type='radio' id='filtr_pritomnych_" . $timestampTerminuSkoleni . "' name='filtr_pritomnych'  value='2'>";
        $popisTabulky .=        '</td>';
        $popisTabulky .=    '</tr>';
        $popisTabulky .=    '<tr><td>&nbsp;</td></tr>';
        $popisTabulky .=    '</tbody>';

        return $popisTabulky;
    }
}


/**
 * Class TerminUpraven
 * @package Ron\RonPortal
 */
class UpravitTermin extends BrowserField
{

    /**
     * UpravitTermin constructor.
     * @param Database $database
     * @param null $options
     */
    public function __construct($database = null, $options = null)
    {
        parent::__construct($database, $options);
        $this->align = 'center';
//        $this->className = 'sloupecEditovanoInfo';
        $this->header = \Ron\ronText("editace_terminu");
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        $sestavenyForm = $this->tabulkaEditace($rowData, $timestampTerminuSkoleni);

        //Kontejner pro zobrazeni dialogu
        $dialog = "";
        $dialog .= '<div class="dialog-overlay" id="dialog-overlay-edit_' . $timestampTerminuSkoleni .'"></div>';
        $dialog .=   '<div class="dialog-box" id="dialog-box-edit_' . $timestampTerminuSkoleni .'">';
        $dialog .=       '<div class="dialog-content">';
        $dialog .=       '<div class="dialog-message" id="dialog-message-edit_' . $timestampTerminuSkoleni .'">';
        $dialog .=          $sestavenyForm;   //Vygenerovany formular, umisti se do kontejneru cimz vznikne dialog na ktery se dale aplikuje jQuery
        $dialog .=   '</div></div>';
        $dialog .= '</div>';

        $result .= $dialog;
        //Button na ktery se navaze udalost pro zobrazeni dialogu
        $result .= "<button type='button' id='buttonProDialog-edit_" . $timestampTerminuSkoleni . "' class='do_nabidky_tlacitko'>" . \Ron\ronText("zobraz_dialog_editace") . "</button>";

        return $result;
    }


    function tabulkaEditace($rowData, $timestampTerminuSkoleni)
    {
        $datumSkoleni = DBUtils::getFormatedValue($this ->database, 'date', $rowData[1]);
        $casSkoleni = date('H:i', $this->database ->stringToTimestamp($rowData[2]));
        $tabulka = "";
        $radek = "";

        $formular  = "<form enctype='multipart/form-data' method='post' id='editaceSkoleni_$timestampTerminuSkoleni' name='editaceSkoleni' action='?menuid=skoleni&action=skoleniucast&' class=''>";
        $formular .= '<input type="hidden" name="timestampEditovanehoTerminuSkoleni" value="' . $timestampTerminuSkoleni . '">';
        $formular  .= "<input type='hidden' name='kodKurzu' value='" . $_SESSION['kodDlouhodobehoKurzu'] . "'>";

        $tabulka .= '<table class="inputtable">';
        $radek .= "<tr>";
        $radek .= $this->popisTabulky($datumSkoleni);
        $radek .= "</tr>";

        $radek .= '<tr><td>&nbsp;</td></tr>';

        $nadpisy  = '<tr>';
        $nadpisy .=   '<th class="center">' . \Ron\ronText('cas') . '</th>';
        $nadpisy .=   '<th class="center">' . \Ron\ronText('Poznamka') . '</th>';
        $nadpisy .= '</tr>';

        $radek .= $nadpisy;

        $radek .= "<tr>";
        $radek .=           new VlozTD("center", "<input title='" .
            \Ron\ronText("cas_koleckem_mysi") . "' id='editaceCasuTerminu_" . $timestampTerminuSkoleni . "' class='editaceCasuTerminu' name='time' value='$casSkoleni' />");
        $radek .=           new VlozTD("center", "<textarea id='" .
            "poznamkaKUchazeci_$timestampTerminuSkoleni' name='poznamkaKEditovanemuTerminu' class='textareaEditace' rows='1' cols='30'>" . $rowData[3] .
            "</textarea>");
        $radek .= "</tr>";

        $radek .= '<tr><td>&nbsp;</td></tr>';

        $radek .= "<tr>";
        $radek .=  "<td style='text-align: right; margin:  0 0; '>";
        $radek .=     '<button type="button" id="tlacitkoStorno-edit_' . $timestampTerminuSkoleni . '" class="button zarovnatDoprava">' . \Ron\ronText("Zpet_na_seznam") . '</button>';
        $radek .=  "</td>";
        $radek .=  "<td style='text-align: right; margin:  0 0; '>";
        $radek .=     "<input class='button' type='reset' value='" . \Ron\ronText('form.Original') . "' >";
        $radek .=     "<input class='button' type='submit' value='" . \Ron\ronText('form.Store') . "' >";
        $radek .=  "</td>";
        $radek .= "</tr>";

        $tabulka .= $radek;
        $tabulka .= '</table>';

        //Sestaveni tabulky
        //Sestaveni formulare
        $formular .= $tabulka;
        $formular .= '</form>';

        return $formular;
    }

    private function popisTabulky ($datumSkoleni)
    {

        //radek 1 ----------
        $nadpisTabulky  =    '<tr class="infoFormuKurzu">';
        $nadpisTabulky .=        '<td height="500px" class="nadpis_sekce" colspan="5">';
        $nadpisTabulky .=            \Ron\ronText('editace_terminu_skoleni');
        $nadpisTabulky .=        '</td>';
        $nadpisTabulky .=    '</tr>';

        $nadpisTabulky .=    '<tr class="infoFormuKurzu">';
        $nadpisTabulky .=        '<td colspan="1">';
        $nadpisTabulky .=            \Ron\ronText('nazev') . ": ";
        $nadpisTabulky .=        '</td>';
        $nadpisTabulky .=        '<td colspan="3" class="tucne">';
        $nadpisTabulky .=            StringUtils::htmlTextConvert($_SESSION['popisKurzu']);
        $nadpisTabulky .=        '</td>';
        $nadpisTabulky .=    '</tr>';

        //radek 2 ----------
        $nadpisTabulky .=    '<tr class="infoFormuKurzu">';
        $nadpisTabulky .=        '<td>';
        $nadpisTabulky .=            \Ron\ronText('termin') . ": ";
        $nadpisTabulky .=        '</td>';
        $nadpisTabulky .=        '<td  class="tucne">';
        $nadpisTabulky .=            $datumSkoleni;
        $nadpisTabulky .=        '</td>';
        $nadpisTabulky .=    '</tr>';

        return $nadpisTabulky;
    }
}


/**
 * Class TerminUpravenInfo
 * @package Ron\RonPortal
 */
class UcastZaznamenanaEditovan extends BrowserField
{
    public function __construct($database = null, $options = null)
    {
        parent::__construct($database, $options);
        $this->align = 'center';
        $this->className = 'sloupecEditovanoInfo';
        $this->header = \Ron\ronText("ucast_zaznamenana");
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);

        $result .=  "<span id='info_vlozenaUcast_$timestampTerminuSkoleni'>$rowData[4]</span>";

        return $result;
    }
}

/**
 * Class ProcentualniUcastNaSkoleniField
 * @package Ron\RonPortal
 */
class ProcentualniUcastNaSkoleniField extends BrowserField
{
    /**
     * ProcentualniUcastNaSkoleniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct($database = null, $options = null)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText("Ucast");
        $this->align = 'center';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);

        $result .= "<div class='center' id='sloupecUcasti_" . $timestampTerminuSkoleni . "' disabled='disabled'></div>";


        return $result;
    }
}

/**
 * Class ProcentualniNeucastNaSkoleniField
 * @package Ron\RonPortal
 */
class ProcentualniNeucastNaSkoleniField extends BrowserField
{

    /**
     * ProcentualniNeucastNaSkoleniField constructor.
     * @param null $database
     * @param null $options
     */
    public function __construct($database = null, $options = null)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText("neucast");
        $this->align = 'center';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);
        $timestampTerminuSkoleni = $this->database->stringToTimestamp($rowData[1]);

        $result .= "<div class='center' id='sloupecNeucasti_" . $timestampTerminuSkoleni . "' disabled='disabled'></div>";

        return $result;
    }
}

/**
 * Class SkoleniUcastBrowser
 * @package Ron\RonPortal
 */
class SkoleniUcastBrowser extends Browser
{
    /** @var  PravaNaOsoby $prava */
    private $prava;

    /** @var UcastNaSkoleniFiltr $filterForm */
    private $filterForm;

    private $kodKurzu;

    /**
     * SkoleniUcastBrowser constructor.
     * @param Database $database
     * @param $kodKurzu
     */
    public function __construct(Database $database, $kodKurzu)
    {
        parent::__construct($database);
        $this->kodKurzu = $kodKurzu;

        $this->prava = Prava::vratPrava('skoleni_prihlasit');

        $this->code = 'browserskoleniucast';
        $this->cssClass = "browser";
        $this->tableId = 'browserskoleniucast';
        $this->browserUrl = "?menuid=skoleni&action=skoleniucast";
        $this->title = \Ron\ronText('ucast_na_skoleni');

        $this->sqlSelectPart = "CisloSkoleni, Datum, Cas, " . $database->castWideMemo('Poznamka');      //0 - 3
        $this->sqlFromPart = 'TerminSkoleni';

        $this->filterForm = new UcastNaSkoleniFiltr($database, $this->kodKurzu, null);
        $this->filterForm->loadData();

        $this->defaultSort = -1;

        $this->addField([BROWSER_FIELD_SQL_INDEX => 1 ,BROWSER_FIELD_SQL_NAME => 'Datum',
            BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('Datum'), BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => true,
        BROWSER_FIELD_ALIGN => 'center']);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 2 ,BROWSER_FIELD_SQL_NAME => 'Cas',
            BROWSER_FIELD_HEADER_TEXT => 'cas', BROWSER_FIELD_FORMAT => 'time', BROWSER_FIELD_ALIGN => 'center']);

        $this->addField(new UcastZaznamenanaEditovan($database, null));

        $this->addField(new ProcentualniUcastNaSkoleniField($database, null));

        $this->addField(new ProcentualniNeucastNaSkoleniField($database, null));

        $this->addField(new UcastNaSkoleniField($database, null));

        $this->addField(new UpravitTermin($database, null));
    }


    function renderFilter()
    {
        $this->filterForm->render();
        parent::renderFilter();
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        echo "<tr class='$className'>";
                echo '<td></td>';
                echo '<td colspan="' . ($this->columnCount) . '" style="max-width:650px;">';
                echo '<em><span class="info">' . '&nbsp;&nbsp;&nbsp;&nbsp;' . StringUtils::htmlTextConvert($values[3]) . '</em> ' .'</span>';
                echo '</td>';
        echo '</tr>';

        echo "<tr class='$className'>";
                echo '<td></td>';
                echo '<td colspan="' . ($this->columnCount -1) . '" style="max-width:650px;"></td>';
        echo '</tr>';
    }


    function getSQLFilter()
    {
        $result= '';
        $obdobi = explode('_', $_SESSION['aktualniObdobiUcastiNaSkoleni']);
        $result .= "TerminSkoleni.CisloSkoleni = '" . $this->database->addSlashes($_SESSION['kodDlouhodobehoKurzu']) . "'";

        if ($_SESSION['jenNevyplneneFormulare']) {
            $result .= " AND ((SELECT COUNT(Pritomen) FROM UcastNaTerminuSkoleni WHERE UcastNaTerminuSkoleni.Pritomen = 1 AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum) = 0  ";
            $result .= " AND (SELECT COUNT(DuvodNepritomnosti) FROM UcastNaTerminuSkoleni WHERE UcastNaTerminuSkoleni.DuvodNepritomnosti = 1 AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum) = 0 )";
        }

        if ($_SESSION['jenObdobiUcastiNaSkoleni'] == 2) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= " (TerminSkoleni.Datum >= " . $this->database->timestampToString($_SESSION['datumTerminu']) .
                        " AND TerminSkoleni.Datum <= " . $this->database->timestampToString($_SESSION['datumdodataTerminu']);
            $result .= " )";
        }

        if ($_SESSION['jenObdobiUcastiNaSkoleni'] == 1) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= ' TerminSkoleni.Datum >= ' . $this->database ->timestampToString($obdobi[1]) .
                ' AND TerminSkoleni.Datum <= ' . $this->database ->timestampToString($obdobi[2]);
        }

        return $result;
    }
}