<?php
/**
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 2.10.2018
 * Time: 13:16
 */

namespace Ron\RonPortal;

use Ron\Browser;
use Ron\DateUtils;
use Ron\DB\SqlStatement;
use function Ron\ronText;

require_once('ron/browser.php');

class VyplatniListkyBrowser extends Browser
{
    public function __construct($database)
    {
        parent::__construct($database);

        $this->code = 'browservyplatnilistky';

        $this->title = ronText('Vyplatni_listky');
        $this->tableId = 'Vyplatnilistky';
        $this->sqlSelectPart =
            'Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru';

        $this->sqlFromPart =
            'Dokument JOIN TypDokumentu ON (Dokument.Typ = TypDokumentu.Kod) ' .
            'JOIN DokumentyOsoby ON (Dokument.id = DokumentyOsoby.id)';
        $this->sqlWherePart =
            'TypDokumentu.Kod = -1' . //jen vyplatni listky
            ' AND TypDokumentu.zobrazitNaIntrawebu = 1';

        $this->primaryField = 0;
        $this->distinct = true;

        $this->cssClass = 'browser minwidth';

        $this->defaultSort = 1; // podle datumu

        $this ->addFields([
            [BROWSER_FIELD_SQL_NAME => 'Dokument.Datum',     BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_HEADER_TEXT => 'Date', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'dokument_datum'],
            [BROWSER_FIELD_SQL_NAME => 'Dokument.Popis',     BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_HEADER_TEXT => 'Popis', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_CLASS_NAME => 'dokument_popis'],
            [BROWSER_FIELD_SQL_NAME => 'Dokument.JmenoSouboru', BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_LINK => '?menuid=vyplatnilistky', BROWSER_FIELD_HEADER_TEXT => 'Dokument_nazev', BROWSER_FIELD_ORDERING => 1]
        ]);

        $this ->browserUrl = '?menuid=vyplatnilistky&';
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql =
            "DokumentyOsoby.RC = " . $params->addParameter(User::getRC());

        return new SqlStatement($sql, $params);
    }

    function renderHeader()
    {
        parent::renderHeader();

        Prava::nastavObdobiProPrava(DateUtils::today());
        if (Prava::vratHodnotuPrava('osobni_heslo') != 0)
            echo "<div id='vl_personal_paswd'><a href='?menuid=personalpassword'>" . ronText('Nastaveni_osobniho_hesla') . "</a></div>";
    }


}