<?php
/**
 * stanislav.elbl, 16.01.2016
 * pavel.rudol, 27.04.2016 predelani na MVC
 */

namespace Ron\Personalistika;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\DataOutputObject;
use Ron\MVC\HttpController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\RonPortal\DochazkaUtils;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\DokumentUploadOutput;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\FotoOutputObject;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;
use RonPortal\Personalistika\DovednostOsobyForm;

require_once('person/dovednostiutils.php');

class DovednostiController extends HttpController
{
    /** @var Database */
    public $database;

    /** @var PravaNaOsoby  */
    private $pravaZobrazeni;
    /** @var PravaNaOsoby  */
    private $pravaEditace;
    /** @var PravaNaOsoby */
    private $pravaSmazani;

    /**
     * DovednostiController constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;

        TextManager::getInstance() ->loadStrings('person.ini');

        Prava::nastavObdobiProPrava(time(), time());

        Prava::nactiPravaUzivatele($database, User::getRC());
        $this->pravaZobrazeni = Prava::vratPrava('doved_ziskane');
        $this->pravaEditace = Prava::vratPrava('doved_edit');
        $this->pravaSmazani = Prava::vratPrava('doved_smazani');

        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');
        HtmlHeaders::getInstance()->addCss('css/person.css');
    }

    /**
     * Zjisti jestli je k ziskane dovednosti pripojen dokument.
     * @param Database $database
     * @param string $osobaRC
     * @param string $dovednost kod dovednosti
     * @param int $uroven kod urovne
     * @param int $datumZiskani datum ziskani dovednosti
     * @return bool true, pokud je k ziskane dovednosti pripojen dokument
     * @throws DBException
     * @throws Exception
     */
    public static function jePripojenDokumentDovednosti($database, $osobaRC, $dovednost, $uroven, $datumZiskani)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT COUNT(RC) FROM DokumentyOsoby
                 WHERE DokumentyOsoby.Dovednost_RC = " . $params->addParameter($osobaRC) . " 
                   AND DokumentyOsoby.Dovednost_Kod = " . $params->addParameter($dovednost) . "
                   AND DokumentyOsoby.Dovednost_Uroven = " . $params->addParameter($uroven) . "
                   AND DokumentyOsoby.Dovednost_DatumZiskani = " . $params->addParameter($database->timestampToParam($datumZiskani));

        $cursor = $database->getSqlResult($sql, $params);

        return ($cursor->fetch()[0] > 0);
    }

    /**
     * Provede smazani ziskane dovednosti osoby.
     * Pred smazaznim kontroluje pravo na zvolenou osobu a zda neni k dovednosti pripojen dokument.
     * @return Redirect
     * @throws Exception
     */
    private function zrusDovednost()
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($_GET['osoba']), $params);
        $kodDovednosti = $_GET['dovednost'];
        $urovenDovednosti = (int)$_GET['uroven'];
        $datumZiskani = $_GET['datumziskani'];

        if (!$this->pravaSmazani->lzeZobrazitRC($osobaRC)) {
            throw new NeniPravoException();
        }

        if (DovednostiController::jePripojenDokumentDovednosti($this->database, $osobaRC, $kodDovednosti, $urovenDovednosti, $datumZiskani)) {
            throw new Exception(ronText('Dovednosti_nelze_smazat_dokument'));
        }

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'DovednostOsoby');
        $rec->addField('RC', '', true);
        $rec->addField('KodDovednosti', '', true);
        $rec->addField('KodUrovne', 'int', true);
        $rec->addField('DatumZiskani', 'timestamp', true);

        $params = $this->database->createParameters();

        $sql =
            " RC = " . $params->addParameter($osobaRC) .
            " AND KodDovednosti = " . $params->addParameter($kodDovednosti) .
            " AND KodUrovne = " . $params->addParameter($urovenDovednosti) .
            " AND DatumZiskani = " . $params->addParameter($this->database->timestampToParam($datumZiskani));

        $rec->setWhere($sql, $params);

        try {
            if ($rec->deleteRecord()) {
                MessageManager::addInfoMessage(ronText('Dovednosti_smazano'));
                return new Redirect('', '?menuid=maticedovednosti&action=dovednostiosoby&idosoby=' . (int)$_GET['osoba']);
            }
        }
        catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $this->redirectToView();
    }

    /** Vrati JSON objekt s informacemi o poslednim pruchodu zvolene osoby/konkretniho prac. pomeru.
     * @param int $idPPosoby id libovolneho pracovniho pomeru osoby (budou se kontrolovat vsechny PP osoby)
     * @param PravaNaOsoby $prava prava pro kontrolu opravneni na zvolenou osobu/prac. pomer
     * @return JsonOutputObject
     * @throws Exception
     */
    private function pritomnostZamestnance($idPPosoby, $prava)
    {
        require_once('adsapp/dochazkautils.php');

        //vyhledani posledniho pruchodu pro osobu, vyhledava se podle RC (RC se najde podle libovolneho ID prac. pomeru osoby)
        if ($idPPosoby > 0) {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . $params->addParameter($idPPosoby), $params);
            if (!$prava->lzeZobrazitRC($osobaRC, time(), time())) {
                RonPortalUtils::neniPravoException();
            }

            $posledniOperace = DochazkaUtils::vratPosledniPruchodSQL($this->database, time(), $osobaRC, null, null, false);
        }


        if (!isset($posledniOperace)) {
            throw new Exception(StringUtils::fromUTF8('Nebyl nalezen poslední průchod'));
        }

        $json = new JsonOutputObject();

        $json->addData('pritomnost', ['pritomen' => $posledniOperace[4], 'datumcasoperace' => $posledniOperace[0], 'popisoperace' => StringUtils::jsonText($posledniOperace[3])]);

        return $json;
    }

    /**
     * @param int $idOsoby id pracovniho pomeru osoby
     * @return FotoOutputObject
     * @throws NeniPravoException
     * @throws Exception
     * @throws DBException
     */
    private function downloadFoto($idOsoby)
    {
        $params = $this->database->createParameters();
        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($idOsoby), $params);

        if (!(($this->pravaEditace->lzeZobrazitRC($osobaRC, time(), time())) || ($this->pravaZobrazeni->lzeZobrazitRC($osobaRC, time(), time())))) {
            throw new NeniPravoException();
        }

        require_once('adsapp/fotoutils.php');
        $foto = new FotoOutputObject($this->database);
        $foto->nactiIdFotkyOsoby($osobaRC);

        return $foto;
    }

    /**
     * @param int $idDokumentu
     * @param int $idOsoby
     * @return OutputObject
     * @throws Exception
     */
    private function downloadDokument($idDokumentu, $idOsoby)
    {
        if ( !RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOVEDNOSTI) ) {
            throw new Exception('Tato funkce neni povolena!');
        }

        $sql =
            "SELECT Dokument.ID " .
            "FROM Dokument JOIN DokumentyOsoby ON (DokumentyOsoby.ID = Dokument.ID) " .
            "  JOIN DovednostOsoby ON (DovednostOsoby.RC = DokumentyOsoby.RC AND DovednostOsoby.KodDovednosti = DokumentyOsoby.Dovednost_Kod " .
            "       AND DovednostOsoby.KodUrovne = DokumentyOsoby.Dovednost_Uroven AND DovednostOsoby.DatumZiskani = DokumentyOsoby.Dovednost_DatumZiskani) " .
            "  JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ) " .
            "WHERE Dokument.ID = " . (int)$idDokumentu .
            "  AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1)) " .
            "  AND DokumentyOsoby.RC = DovednostOsoby.RC " .
            "  AND EXISTS(SELECT RC FROM PracovniPomer WHERE PracovniPomer.RC = DokumentyOsoby.RC AND PracovniPomer.ID = " . (int)$idOsoby .
            "           AND ((" . $this->pravaEditace->sqlFiltr('PracovniPomer',time(),time()) . ") OR ".
            "                (" . $this->pravaZobrazeni->sqlFiltr('PracovniPomer',time(),time()) . ")) ".
            ")";

        $cursor = $this ->database ->getSqlResult($sql);
        if (!$row = $cursor->fetch()) {
            throw new Exception('Zadany dokument nebyl nalezen nebo na nej nemate pravo!');
        }

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @return DokumentUploadOutput
     */
    function uploadDokument()
    {
        require_once('adsapp/dokumentyutils.php');

        $result = new DokumentUploadOutput();

        try {
            // zkontroluje prava
            $idOsoby = (int)$_POST['idosoby'];

            if (!$this->pravaEditace ->lzeZobrazitPP($idOsoby)) {
                throw new NeniPravoException();
            }

            // nacte RC osoby
            $rc = DBUtils::getSqlValue( $this ->database,  'SELECT RC FROM PracovniPomer WHERE ID = ' . $idOsoby );
            if ($rc === false) {
                throw new Exception(StringUtils::fromUTF8("Chyba při vkládání dokumentu osoby - nenalezen pracovní poměr, ID: $idOsoby!"));
            }

            // vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $_POST['typ'], $_POST['popis'], 'Dovednost osoby');

            //Dovednost osoby ve tvaru: kod dovednosti~kod urovne~datum ziskani
            $dovednost = explode('~', $_POST['Dovednost']);

            // Vlozi zaznam do vazebni tabulky
            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this ->database, 'DokumentyOsoby');
            $rec ->addField('RC', '', false, 15, $rc);
            $rec ->addField('Dovednost_RC', '', false, 15, $rc);
            $rec ->addField('Dovednost_Kod', '', false, 40, $dovednost[0]);
            $rec ->addField('Dovednost_Uroven', 'int', false, 0, (int)$dovednost[1]);
            $rec ->addField('Dovednost_DatumZiskani', 'timestamp', false, 0, DateUtils::dateStringToTimestamp($dovednost[2]));
            $rec ->addField('Id', 'int', false, 0, $id);
            $rec ->insertRecord();
        } catch (Exception $e) {
            $result ->setError($e ->getMessage());
        }

        return $result;
    }

    /** Umožní stažení CSV souboru se seznamem dovedností z browseru dovedností.
     * @param int $idOsoby
     * @return DovednostiBrowser
     * @throws Exception
     * @throws DBException
     */
    private function downloadDovednostiCSV($idOsoby)
    {
        require_once('person/b_dovednosti.php');
        $b = new DovednostiBrowser($this->database, '?' . $_SERVER['QUERY_STRING'].'&', $idOsoby);
        $b ->load();

        return $b;
    }

    /** Provede vložení/ukončení typového pracovního místa (RozdeleniOsob - konkretni typ z nastaveni)
     * @return Redirect
     * @throws Exception
     */
    private function vlozeniAktualizaceTPM()
    {
        $idOsoby = $_POST['idosoby'];

        $tpm = (isset($_POST['tpm_kod']) && ($_POST['tpm_kod'] != '')) ? $_POST['tpm_kod'] : $tpm = $_POST['typovemisto'];
        if ((isset($_POST['tpm_platnost_od'])) && ($_POST['tpm_platnost_od'] != '')) {
            $platnostOd = DateUtils::dateStringToTimestamp($_POST['tpm_platnost_od']);
            $platnostDo = DateUtils::dateStringToTimestamp($_POST['platnost_do']);
        }
        else {
            $platnostOd = DateUtils::dateStringToTimestamp($_POST['platnost_od']);
            $platnostDo = null;
        }

        $poznamka = isset($_POST['poznamka']) ? $_POST['poznamka'] : null;

        require_once('adsapp/ppomer.php');
        $pPomer = new PracovniPomer();
        $pPomer->nactiPPPodleId($this->database, $idOsoby);
        if (!$pPomer->lzeZobrazitPP($this->pravaEditace, $platnostOd, $platnostOd)) {
            throw new NeniPravoException();
        }

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'RozdeleniOsob');
        $rec->addField('KodTypuRozdeleni', 'int', true, 0, RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm'));
        $rec->addField('Rozdeleni', '', true, 30, $tpm);
        $rec->addField('RC', '', true, 15, $pPomer->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pPomer->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $pPomer->datumNastupu);
        $rec->addField('PlatnostOd', 'timestamp', true, 0, $platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $platnostDo);
        $rec->addField('Poznamka', '', false, 80, $poznamka);

        $params = $this->database->createParameters();

        $sqlWhere =
            '     KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm')) .
            " AND Rozdeleni = " . $params->addParameter($tpm) .
            " AND RC = " . $params->addParameter($pPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($pPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($pPomer->datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($this->database->timestampToParam($platnostOd));

        $rec->setWhere($sqlWhere, $params);

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception(StringUtils::fromUTF8('Chyba při vložení TPM: ' . $exception->getMessage()));
        }

        return new Redirect('', '?menuid=maticedovednosti&formaction=new&idosoby='.$idOsoby);
    }

    /** Vrací jméno a příjmení osoby podle ID PP ve tvaru: Titul Jméno Příjmení, TitulZa
     * @param Database $database
     * @param int $idPP
     * @return string vrátí jméno osoby: Titul Jméno Příjmení, TitulZa
     * @throws DBException
     */
    public static function vratJmenoOsoby($database, $idPP)
    {
        $params = $database->createParameters();

        $sql = 'SELECT Jmeno, Prijmeni, Titul, TitulZa ' .
            'FROM Osoba JOIN PracovniPomer ON PracovniPomer.RC = Osoba.RC ' .
            'WHERE PracovniPomer.ID = ' . $params->addParameter($idPP);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row) {
            return '';
        }

        return $row[2] . ' ' . $row[0] . ' ' . $row[1] . ($row[3] ? ', ' . $row[3] : '');
    }

    /** Zajistí přesměrování po vložení podle práv.
     * @return Redirect
     */
    private function redirectToView()
    {
        if ($this->pravaZobrazeni->jsouNejakePrava()) {
            $action = '&action=' . $_SESSION['dovednostibrowser'] ?: 'pozadovane';
        }
        else {
            $action = '&action=dovednostiosoby';
        }

        return new Redirect(null,'?menuid=maticedovednosti'.$action);
    }

    /** Zobrazí přehled typových pracovních míst osoby s možností přidání a ukončení.
     * @param int $typRozdeleniTypovePracovniMisto
     * @param int $idOsoby
     * @throws DBException
     */
    private function typovaPracovniMistaOsoby($typRozdeleniTypovePracovniMisto, $idOsoby)
    {
        require_once('person/b_dovednostitpm.php');
        $b = new TypovePracovniMistoOsobyBrowser($this->database, '?' . $_SERVER['QUERY_STRING'].'&', $idOsoby, $typRozdeleniTypovePracovniMisto);
        $b->load();

        Model::addViewObject($b);

        $params = $this->database->createParameters();

        $sqlTypoveMista =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis 
            FROM Rozdeleni 
                JOIN VazbaRozdeleni ON (VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni AND VazbaRozdeleni.kodPodrizene = Rozdeleni.Kod)
                JOIN Rozdeleni PM ON (VazbaRozdeleni.typNadrazene = PM.KodTypuRozdeleni AND VazbaRozdeleni.kodNadrazene = PM.Kod)
                JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PM.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PM.Kod)
                JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
            WHERE PM.KodTypuRozdeleni = -1 
              AND Rozdeleni.KodTypuRozdeleni = ' . $params->addParameter($typRozdeleniTypovePracovniMisto) . ' 
              AND PracovniPomer.DatumNastupu <=  ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' 
              AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)
              AND PracovniPomer.ID = ' . $params->addParameter($idOsoby) . '
              AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . '
              AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        $comboTypoveMista = new ComboBox('typovemisto');
        $comboTypoveMista->loadSQL($this->database, $sqlTypoveMista, $params);
        $comboTypoveMista->htmlParams = 'data-ron-select2="typovemisto_kod"';

        $comboTypoveMistaKod = new ComboBox('typovemisto_kod');
        $comboTypoveMistaKod->loadSQL($this->database, $sqlTypoveMista, $params, 0,0);
        $comboTypoveMistaKod->htmlParams = 'data-ron-select2="typovemisto"';

        require_once('adsapp/ppomer.php');

        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $idOsoby);

        Model::addData('pracPomerOsoby', $pracPomer);
        Model::addData('combotypovemista', $comboTypoveMista);
        Model::addData('combotypovemistakod', $comboTypoveMistaKod);
        Model::addViewObject(new BaseViewObject('person/view_novetpm.php'));
    }

    /** Zobrazí browser dovednosti osoby + seznam dokumentu osoby s možností vložit nový.
     * @param int $idOsoby
     * @param bool $zobrazitJmeno
     * @return string
     * @throws Exception
     */
    private function prehledDovednostiOsoby($idOsoby, $zobrazitJmeno = false)
    {
        if ($zobrazitJmeno) {
            Model::addViewObject(new TextViewObject('<h1>' . ronText('Dovednosti_detail_osoby') . '</h1>'));
            $table =
                "<table class='infotable'>" .
                "<tr><td><strong>" . StringUtils::htmlTextConvert(self::vratJmenoOsoby($this->database, $idOsoby)) . "</strong></td></tr>" .
                "</table>" .

                "<br/>" .
                "<a href='?menuid=maticedovednosti&action=" . $_SESSION['dovednostibrowser']  . "'>" . ronText('Zpet') . "</a>";

            Model::addViewObject(new TextViewObject($table));
        }

        $typRozdeleniTypovePracovniMisto = (int)RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm');
        if ($typRozdeleniTypovePracovniMisto) {
            $this->typovaPracovniMistaOsoby($typRozdeleniTypovePracovniMisto, $idOsoby);
        }

        require_once('person/b_dovednosti.php');
        $b = new DovednostiBrowser($this->database, '?' . $_SERVER['QUERY_STRING'].'&', $idOsoby);
        $b ->load();

        Model::addViewObject($b);

        Model::addViewObject(new TextViewObject("<A id='dokumenty'></A>\n"));

        Model::addViewObject(new TextViewObject("<form id='b_dokumentydovednosti'>"));

        require_once('person/b_dokumentyDovednosti.php');
        $b = new DokumentyDovednosti($this->database,$idOsoby); /*::zobrazDokumentyDovednosti($database, $idPP, $lzeEditovat);*/
        $b->load();

        Model::addViewObject($b);

        Model::addViewObject(new TextViewObject("</form>"));

        // Pokud lze editovat, zobrazi odkaz "vlozit novy"
        if ($this->pravaEditace->lzeZobrazitPP($idOsoby)) {
            Model::addViewObject(new TextViewObject("<br/>
                <div class='normaltext'><a href='#' onclick='showDokumentDovednostiDialog(" . "$idOsoby);'>" . ronText('Vlozit_novy_dokument') . "</a></div>"));

            $form = new NovyDokumentDovednostiForm($this->database, $idOsoby);
            Model::addViewObject($form);
        }

        return 'view/appview.php';
    }

    /** Zobrazi/zpracuje formular nove dovednosti.
     * @return Redirect|string
     * @throws Exception
     */
    private function dovednostiFormular()
    {
        if (!$this->pravaEditace->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $idOsoby = (int)(isset($_GET['idosoby']) ? $_GET['idosoby'] : (isset($_POST['osoba']) ? $_POST['osoba'] : 0));

        //pokud neni zadane id PP osoby v URL, pouzije se vybrana osoba ze session (pokud je nastavena)
        //22.8.2019 zruseno nastaveni osoby ze session, muze zpusobovat problem s pravy, kdy v jinem browseru vybere osobu na kterou zde nema pravo
        /*if (($idOsoby == 0) && (RonPortalSession::vybranaOsoba())) {
            $idOsoby = RonPortalSession::setVybranaOsoba()vybranaOsoba();
        }*/

        //uzivatel muze editovat dovednosti, pokud je zadana osoba, tak musi mit pravo primo na ni
        if (!($this->pravaEditace->jsouNejakePrava() && (($idOsoby == 0) || $this->pravaEditace->lzeZobrazitPP($idOsoby)))) {
            throw new NeniPravoException();
        }

        $form = new DovednostOsobyForm($this->database, $idOsoby);

        if ($form->handleAction()) {
            MessageManager::addInfoMessage(ronText('Dovednosti_vlozeni_probehlo'));
            return $this->redirectToView();
        }

        $this->hlavickaVyberAkce();

        Model::addViewObject($form);

        return $this->prehledDovednostiOsoby($idOsoby);
    }

    /**
     * @return string
     * @throws Exception
     */
    private function dovednostiOsoby()
    {
        $idOsoby = 0;
        if (isset($_GET['idosoby'])) {
            $idOsoby = $_GET['idosoby'];
        }
        
        if (!$this->pravaZobrazeni->lzeZobrazitPP($idOsoby)) {
            throw new NeniPravoException();
        }

        return $this->prehledDovednostiOsoby($idOsoby, true);
    }

    /** Zobrazí přehled - matici získaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsob
     * @throws NeniPravoException
     * @throws Exception
     */
    private function ziskaneDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsob($this->database, 'ziskane');

        $params = $this->database->createParameters();

        $selectZiskane =
            "SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Dovednost.Popis AS PopisDovednosti, Dovednost.Kod AS KodDovednosti, " . //0 - 4
            "UrovenDovednosti.KodUrovne, 'ziskane', PracovniPomer.ID, PracovniPomer.OsobniCislo, PracovniPomer.TypPracovnihoPomeru, Osoba.idFotografie, " . // 5 - 10
//            "PM.Rozdeleni, PMNazev.Popis, RO.Rozdeleni, R.Popis, Osoba.idFotografie, PK.Kalendar AS PrirazenyKalendar, Kalendar.Popis, ".  //10 - 16
            "DovednostOsoby.PlatnostDo, UrovenDovednosti.Popis, DovednostOsoby.SpecifikaceUrovne "; //11 - 13
        //podrizene ", D2.Popis AS PopisPodrazene, UD2.KodUrovne AS UrovenPodrizene" .

        $fromZiskane   =
            "FROM DovednostOsoby " .
            " INNER JOIN Osoba ON (DovednostOsoby.RC = Osoba.RC) " .
            " INNER JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) " .
            " INNER JOIN UrovenDovednosti ON (DovednostOsoby.KodDovednosti = UrovenDovednosti.KodDovednosti AND DovednostOsoby.KodUrovne = UrovenDovednosti.KodUrovne) " .
            " INNER JOIN Dovednost ON (UrovenDovednosti.KodDovednosti = Dovednost.Kod) ";
//            " LEFT JOIN RozdeleniOsob PM ON (PM.RC = PracovniPomer.RC AND PM.OsobniCislo = PracovniPomer.OsobniCislo AND PM.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND PM.KodTypuRozdeleni = " . $matice->typRozdeleniPracMisto->kod . " AND PM.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PM.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PM.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni PMNazev ON (PMNazev.KodTypuRozdeleni = PM.KodTypuRozdeleni AND PMNazev.Kod = PM.Rozdeleni)" .
//            " LEFT JOIN RozdeleniOsob RO ON (RO.RC = PracovniPomer.RC AND RO.OsobniCislo = PracovniPomer.OsobniCislo AND RO.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND RO.KodTypuRozdeleni = " . $matice->typRozdeleniFiltr->kod .
//            "       AND RO.PlatnostOd <= " . $this->database->dateToString(time()) . " AND (RO.PlatnostDo >= " . $this->database->dateToString(time()) . " OR RO.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni R ON (R.KodTypuRozdeleni = RO.KodTypuRozdeleni AND R.Kod = RO.Rozdeleni) " .
//            " LEFT JOIN PrirazeniKalendare PK ON (PK.RC = PracovniPomer.RC AND PK.OsobniCislo = PracovniPomer.OsobniCislo  " .
//            "       AND PK.DatumNastupu = PracovniPomer.DatumNastupu AND PK.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PK.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PK.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Kalendar ON (Kalendar.Kod = PK.Kalendar)";
        /* podrizene dovednosti: LEFT OUTER JOIN PodrazenaDovednost ON (Dovednost.Kod = PodrazenaDovednost.KodNadrazene)
        LEFT OUTER JOIN Dovednost D2 ON (D2.Kod = PodrazenaDovednost.KodPodrazene)
        LEFT OUTER JOIN UrovenDovednosti UD2 ON (UD2.KodDovednosti = D2.Kod)
        LEFT OUTER JOIN DovednostOsoby DO2 ON (DO2.KodDovednosti = UD2.KodDovednosti AND DO2.KodUrovne = UD2.KodUrovne)*/


        $whereZiskane   =
            "WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL) " .
            "  AND DovednostOsoby.DatumZiskani <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (DovednostOsoby.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(time())) . " OR DovednostOsoby.PlatnostDo IS NULL) ".
            //17.3.2017 RON-PR doplnena podminka jen na aktivni dovednosti
            "  AND Dovednost.Aktivni = 1";


        $matice->nactiSeznam($selectZiskane, $fromZiskane, $whereZiskane, $params);

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /** Zobrazí přehled - matici získaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsob
     * @throws NeniPravoException
     * @throws Exception
     */
    private function vyhledaniOsobDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsobVyhledani($this->database, 'ziskane');

        if (!(isset($_POST['find']) || isset($_POST['showdata']))) {
            $matice->nactiFiltr();
            return $matice;
        }

        $params = $this->database->createParameters();

        $selectZiskane =
            "SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Dovednost.Popis AS PopisDovednosti, Dovednost.Kod AS KodDovednosti, " . //0 - 4
            "UrovenDovednosti.KodUrovne, 'ziskane', PracovniPomer.ID, PracovniPomer.OsobniCislo, PracovniPomer.TypPracovnihoPomeru, Osoba.idFotografie, " . // 5 - 10
//            "PM.Rozdeleni, PMNazev.Popis, RO.Rozdeleni, R.Popis, Osoba.idFotografie, PK.Kalendar AS PrirazenyKalendar, Kalendar.Popis, ".  //10 - 16
            "DovednostOsoby.PlatnostDo, UrovenDovednosti.Popis, DovednostOsoby.SpecifikaceUrovne "; //11 - 13
        //podrizene ", D2.Popis AS PopisPodrazene, UD2.KodUrovne AS UrovenPodrizene" .

        $fromZiskane   =
            "FROM DovednostOsoby " .
            " INNER JOIN Osoba ON (DovednostOsoby.RC = Osoba.RC) " .
            " INNER JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) " .
            " INNER JOIN UrovenDovednosti ON (DovednostOsoby.KodDovednosti = UrovenDovednosti.KodDovednosti AND DovednostOsoby.KodUrovne = UrovenDovednosti.KodUrovne) " .
            " INNER JOIN Dovednost ON (UrovenDovednosti.KodDovednosti = Dovednost.Kod) ";
//            " LEFT JOIN RozdeleniOsob PM ON (PM.RC = PracovniPomer.RC AND PM.OsobniCislo = PracovniPomer.OsobniCislo AND PM.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND PM.KodTypuRozdeleni = " . $matice->typRozdeleniPracMisto->kod . " AND PM.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PM.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PM.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni PMNazev ON (PMNazev.KodTypuRozdeleni = PM.KodTypuRozdeleni AND PMNazev.Kod = PM.Rozdeleni)" .
//            " LEFT JOIN RozdeleniOsob RO ON (RO.RC = PracovniPomer.RC AND RO.OsobniCislo = PracovniPomer.OsobniCislo AND RO.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND RO.KodTypuRozdeleni = " . $matice->typRozdeleniFiltr->kod .
//            "       AND RO.PlatnostOd <= " . $this->database->dateToString(time()) . " AND (RO.PlatnostDo >= " . $this->database->dateToString(time()) . " OR RO.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni R ON (R.KodTypuRozdeleni = RO.KodTypuRozdeleni AND R.Kod = RO.Rozdeleni) " .
//            " LEFT JOIN PrirazeniKalendare PK ON (PK.RC = PracovniPomer.RC AND PK.OsobniCislo = PracovniPomer.OsobniCislo  " .
//            "       AND PK.DatumNastupu = PracovniPomer.DatumNastupu AND PK.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PK.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PK.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Kalendar ON (Kalendar.Kod = PK.Kalendar)";
        /* podrizene dovednosti: LEFT OUTER JOIN PodrazenaDovednost ON (Dovednost.Kod = PodrazenaDovednost.KodNadrazene)
        LEFT OUTER JOIN Dovednost D2 ON (D2.Kod = PodrazenaDovednost.KodPodrazene)
        LEFT OUTER JOIN UrovenDovednosti UD2 ON (UD2.KodDovednosti = D2.Kod)
        LEFT OUTER JOIN DovednostOsoby DO2 ON (DO2.KodDovednosti = UD2.KodDovednosti AND DO2.KodUrovne = UD2.KodUrovne)*/


        $whereZiskane   =
            "WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL) " .
            "  AND DovednostOsoby.DatumZiskani <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (DovednostOsoby.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(time())) . " OR DovednostOsoby.PlatnostDo IS NULL) ".
            //17.3.2017 RON-PR doplnena podminka jen na aktivni dovednosti
            "  AND Dovednost.Aktivni = 1";


        $matice->nactiSeznam($selectZiskane, $fromZiskane, $whereZiskane, $params);

        $matice->seradOsoby();

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /** Zobrazí přehled - matici požadovaných dovedností.
     * @return DataOutputObject|SeznamDovednostiOsob
     * @throws NeniPravoException
     * @throws Exception
     */
    private function pozadovaneDovednosti()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        $matice = new SeznamDovednostiOsob($this->database, 'pozadovane');

        $params = $this->database->createParameters();

        $selectPozadovane =
            "SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Dovednost.Popis AS PopisDovednosti, Dovednost.Kod AS KodDovednosti, " . //0 - 4
            "UrovenDovednosti.KodUrovne, 'pozadovane', PracovniPomer.ID, PracovniPomer.OsobniCislo, PracovniPomer.TypPracovnihoPomeru, Osoba.idFotografie, " . // 5 - 10
//            "PM.Rozdeleni, PMNazev.Popis, RO.Rozdeleni, R.Popis, Osoba.idFotografie, PK.Kalendar, Kalendar.Popis, " . //10 - 16
            "Rozdeleni.Popis, UDZ.KodUrovne AS KodUrovneZiskane, UrovenDovednosti.Popis, UDZ.Popis, DovednostOsoby.SpecifikaceUrovne " ; //11 - 15

        $fromPozadovane =
            "FROM Osoba " .
            " INNER JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) " .
            " INNER JOIN RozdeleniOsob ON (RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu) " .
            " INNER JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni) " .
            " INNER JOIN PozadavekNaDovednost ON (PozadavekNaDovednost.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND PozadavekNaDovednost.KodRozdeleni = RozdeleniOsob.Rozdeleni) " .
            " INNER JOIN UrovenDovednosti ON (PozadavekNaDovednost.KodDovednosti = UrovenDovednosti.KodDovednosti AND PozadavekNaDovednost.PozadovanaUroven = UrovenDovednosti.KodUrovne) " .
            " INNER JOIN Dovednost ON (UrovenDovednosti.KodDovednosti = Dovednost.Kod) " .
            " LEFT JOIN DovednostOsoby ON (DovednostOsoby.KodDovednosti = Dovednost.Kod AND DovednostOsoby.RC = Osoba.RC " .
            "               AND (DovednostOsoby.PlatnostDo >= " . $this->database->dateToString(time()) . " OR DovednostOsoby.PlatnostDo IS NULL)) " .
            " LEFT JOIN UrovenDovednosti UDZ ON (UDZ.KodDovednosti = DovednostOsoby.KodDovednosti AND UDZ.KodUrovne = DovednostOsoby.KodUrovne) ";
//            " LEFT JOIN RozdeleniOsob PM ON (PM.RC = PracovniPomer.RC AND PM.OsobniCislo = PracovniPomer.OsobniCislo AND PM.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND PM.KodTypuRozdeleni = " . $matice->typRozdeleniPracMisto->kod . "  AND PM.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PM.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PM.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni PMNazev ON (PMNazev.KodTypuRozdeleni = PM.KodTypuRozdeleni AND PMNazev.Kod = PM.Rozdeleni)" .
//            " LEFT JOIN RozdeleniOsob RO ON (RO.RC = PracovniPomer.RC AND RO.OsobniCislo = PracovniPomer.OsobniCislo AND RO.DatumNastupu = PracovniPomer.DatumNastupu " .
//            "       AND RO.KodTypuRozdeleni = " . $matice->typRozdeleniFiltr->kod .
//            "       AND RO.PlatnostOd <= " . $this->database->dateToString(time()) . " AND (RO.PlatnostDo >= " . $this->database->dateToString(time()) . " OR RO.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Rozdeleni R ON (R.KodTypuRozdeleni = RO.KodTypuRozdeleni AND R.Kod = RO.Rozdeleni)" .

//            " LEFT JOIN PrirazeniKalendare PK ON (PK.RC = PracovniPomer.RC AND PK.OsobniCislo = PracovniPomer.OsobniCislo  " .
//            "       AND PK.DatumNastupu = PracovniPomer.DatumNastupu AND PK.PlatnostOd <= " . $this->database->dateToString(time()) .
//            "       AND (PK.PlatnostDo >= " . $this->database->dateToString(time()) . " OR PK.PlatnostDo IS NULL)) " .
//            " LEFT JOIN Kalendar ON (Kalendar.Kod = PK.Kalendar)";

        $wherePozadovane =
            "WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL) " .
            "  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(time())) .
            "  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(time())) . " OR RozdeleniOsob.PlatnostDo IS NULL) " .
            //17.3.2017 RON-PR doplnena podminka jen na aktivni dovednosti
            "  AND Dovednost.Aktivni = 1" .
            "  AND NOT EXISTS(SELECT VyssiUroven.PozadovanaUroven FROM PozadavekNaDovednost AS VyssiUroven WHERE VyssiUroven.KodTypuRozdeleni = PozadavekNaDovednost.KodTypuRozdeleni AND VyssiUroven.KodRozdeleni = PozadavekNaDovednost.KodRozdeleni " .
            "   AND VyssiUroven.KodDovednosti = PozadavekNaDovednost.KodDovednosti AND VyssiUroven.PozadovanaUroven > PozadavekNaDovednost.PozadovanaUroven)";

        $matice->nactiSeznam($selectPozadovane, $fromPozadovane, $wherePozadovane, $params);

        if (isset($_GET['format']) && $_GET['format'] == 'csv') {
            return $matice->outputCSV();
        }

        return $matice;
    }

    /**
     * @param string $kodDovednosti kód dovednosti
     * @return JsonOutputObject
     */
    function nactiUrovneDovednosti($kodDovednosti)
    {
        $result = new JsonOutputObject();

        try {
            if (!$kodDovednosti) {
                throw new Exception(StringUtils::fromUTF8('Není vybrána dovednost.'));
            }

            require_once('person/dovednostiutils.php');
            $dovednost = new DovednostOsoby($kodDovednosti);

            if (!$dovednost->nactiUrovne($this ->database)) {
                throw new Exception(StringUtils::fromUTF8('Nelze načíst úrovně dovednosti.'));
            }

            $urovne = [];

            if (is_array($dovednost->urovneDovednosti)) {
                foreach ($dovednost->urovneDovednosti as $kodUrovne => $popisUrovne) {
                    $uroven = array(
                        'kod' => (int)$kodUrovne,
                        'popis' => StringUtils::jsonText($popisUrovne),
                    );
                    $urovne[] = $uroven;
                }
            }

            $result ->addData('urovne', $urovne);
        } catch (Exception $e) {
            $result ->addError($e);
        }

        return $result;
    }

    private function hlavickaVyberAkce()
    {
        $sekce = [];
        if ($this->pravaZobrazeni->jsouNejakePrava())
            $sekce = ['ziskane' => ronText('MI_Dovednosti_ziskane'), 'pozadovane' => ronText('MI_Dovednosti_pozadovane'),
                'vyhledani' => ronText('MI_Dovednosti_hledani')];
        if ($this->pravaEditace->jsouNejakePrava())
            $sekce = array_merge($sekce, ['novadovednost' => ronText('MI_Dovednosti_nove')]);
        
        Model::addData('header',  ronText('MI_Dovednosti'));
        Model::addData('aktivni_sekce', (isset($_GET['action']) ? $_GET['action'] : (isset($_GET['formaction']) ? 'novadovednost' : '')));
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=maticedovednosti&action=');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /** Zobrazí matici (přehled) dovedností/formulář pro vložení.
     * @return Redirect|SeznamDovednostiOsob|string
     * @throws NeniPravoException
     * @throws Exception
     */
    private function handleMaticeDovednosti()
    {
        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->dovednostiFormular();
        }

        $dovednostiaction = isset($_GET['action']) ? $_GET['action'] : (isset($_SESSION['dovednostiaction']) ? $_SESSION['dovednostiaction'] : '');

        $_SESSION['dovednostiaction'] = $dovednostiaction;

        switch ($dovednostiaction)
        {
            case 'novadovednost':
                return $this->dovednostiFormular();
            case 'dovednostiosoby':
                return $this->dovednostiOsoby();
            case 'pozadovane' :
                return $this->pozadovaneDovednosti();
            case 'zrusitdovednost' :
                return $this->zrusDovednost();
            case 'vyhledani' :
                return $this->vyhledaniOsobDovednosti();
            case 'ziskane' :
            default :
                return $this->ziskaneDovednosti();
        }
    }


    /** Práce s dokumenty (nahrání/zobrazení) + stažení seznamu úrovní pro dovednosti.
     * @return bool|JsonOutputObject|OutputObject|Redirect|DokumentUploadOutput
     * @throws Exception
     */
    private function handleDovednosti()
    {
        switch ($_GET['action']) {
            case 'pritomnost' :
                return $this->pritomnostZamestnance((int)$_GET['osoba'], $this->pravaZobrazeni);

            case 'foto' :
                return $this->downloadFoto((int)$_GET['osoba']);

            case 'download' :
                return $this ->downloadDokument((int)$_GET['record'], (int)$_GET['dovednostosoby']);

            case 'downloadcsv' :
                return $this->downloadDovednostiCSV((int)$_GET['osoba']);

            case 'dokumentupload' :
                return $this ->uploadDokument();

            case 'loaddata' :
                return $this ->nactiUrovneDovednosti($_GET['dovednost']);

            case 'editujtpm' :
                return $this->vlozeniAktualizaceTPM();

        }

        throw new Exception('Chyba v DovednostiController::handleDovednosti! Neznama akce!');
    }

    /**
     * @param string $action
     * @return bool|JsonOutputObject|OutputObject|Redirect|SeznamDovednostiOsob|DokumentUploadOutput|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        //pokud je povoleno v aplikaci a uzivatel ma pravo na zobrazeni, nebo editaci dovednosti
        if (!(RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOVEDNOSTI) && ($this->pravaZobrazeni->jsouNejakePrava() || $this->pravaEditace->jsouNejakePrava())))
        {
            throw new NeniPravoException();
        }

        switch ($action) {
            case 'dovednosti':
                return $this->handleDovednosti();
            case 'maticedovednosti':
                return $this->handleMaticeDovednosti();
        }

        throw new Exception('Chyba v DovednostiController::handleAction! Neznama akce!');
    }

}
