<?php

/** UTF8
 * Pavel Rudol, RON Software
 *  15.7.2015
 */
namespace Ron\Personalistika;

require_once('ron/sqlutils.php');
require_once('ron/controls.php');
require_once('adsapp/ronportalfilter.php');
require_once('adsapp/prava.php');
require_once('adsapp/dokumentyutils.php');
require_once('adsapp/rozdeleni.php');

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\MVC\DataOutputObject;
use Ron\MVC\ViewObject;
use Ron\RonPortal\DokumentUploadForm;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\RonPortal\RozdeleniOsob;
use Ron\RonPortal\TypRozdeleni;
use Ron\StringUtils;
use function Ron\ronText;
use Ron\TextManager;
use RonPortal\RonPortalSession;

/*
 * Zobrazi filtr rozdeleni
 */
class DovednostiFilterForm extends RonPortalFilterForm
{
    public $vedouciJmenoPrijmeni;
    public $zastupceJmenoPrijmeni;

    /** @var string urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti */
    private $typZobrazeni;

    private $vedouciIDosoby;
    private $zastupceIDosoby;

    private $typRozdeleniFiltr1;
    private $typRozdeleniPracMisto;

    private $zobrazitFiltrKalendar;

    /**
     * DovednostiFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param string $typZobrazeni urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti
     * @param int $typRozdeleniFiltr1
     * @param int $typRozdeleniPracMisto
     * @param bool $zobrazitFiltrKalendar
     */
    public function __construct($database, $action, $typZobrazeni, $typRozdeleniFiltr1, $typRozdeleniPracMisto, $zobrazitFiltrKalendar)
    {
        parent::__construct($database, $action/*, true*/); //GET blbne pokud je v action &

        $this->typZobrazeni = $typZobrazeni;

        $this->f_classname = 'filtercontainer leftcaptions dovednostifilter';

        $this->typRozdeleniFiltr1 = $typRozdeleniFiltr1;
        $this->typRozdeleniPracMisto = $typRozdeleniPracMisto;

        $this->nastavRozdeleni(true, $this->typRozdeleniFiltr1, 2, '', true);
        $this->nastavRozdeleni2(true, $this->typRozdeleniPracMisto, 0, '', true);
        $this->f_pravarozdeleni = Prava::vratPrava('doved_ziskane');

        $this->zobrazitFiltrKalendar = $zobrazitFiltrKalendar;
    }

    /**
     * @throws DBException
     */
    private function renderKalendar()
    {
        echo '<DIV class="filterrow">';

        // checkbox "Jen umisteni"
        echo '<SPAN class="filtercaption">';
        echo ronText('Kalendar');
        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            'SELECT Kod, Popis FROM Kalendar
            WHERE Aktivni = 1
            ORDER BY Kod';

        // vyber kalendare
        $combo = new ComboBox('kalendar');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo ->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('kalendar'));
        $combo->htmlParams = "data-ron-select2='kalendar_kod' data-ron-autosubmit='1'";
        $combo->render();

        // vyber kalendare
        $combo = new ComboBox('kalendar_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->captionIndex = 0;
        $combo ->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('kalendar'));
        $combo->htmlParams = "data-ron-select2='kalendar' data-ron-autosubmit='1'";
        $combo->render();

        echo "</DIV>\n";
    }

    /**
     * @throws DBException
     */
    protected function renderTypDovednosti()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dovednosti_typ_dovednosti');
        echo '</SPAN>';

        // combobox pro vyber
        $sql = 'SELECT Kod, Popis FROM TypDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.TypDovednosti = TypDovednosti.Kod AND Aktivni = 1)';

        // vyber typu dovednosti
        $combo = new ComboBox('typDovednosti');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Popis', RonPortalSession::getValue('typDovednosti'));
        $combo->htmlParams = "data-ron-select2='typDovednosti_kod' data-ron-autosubmit='1'";
        $combo->render();

        // vyber typu dovednosti
        $combo = new ComboBox('typDovednosti_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->emptyRow = ronText('_vse_');
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql . ' ORDER BY Kod', RonPortalSession::getValue('typDovednosti'));
        $combo->htmlParams = "data-ron-select2='typDovednosti' data-ron-autosubmit='1'";
        $combo->render();

        echo "</DIV>\n";
    }

    /**
     * @throws DBException
     */
    protected function renderDovednost()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =
            "SELECT Kod, Popis FROM Dovednost WHERE Aktivni = 1 ";

        if (RonPortalSession::getValue('typDovednosti')) {
            $sql .= "AND TypDovednosti = " . $params->addParameter(RonPortalSession::getValue('typDovednosti'));
        }

        // vyber dovednosti
        $combo = new ComboBox('dovednost');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Popis', RonPortalSession::getValue('dovednost'), $params);
        $combo->htmlParams = "data-ron-select2='dovednost_kod' data-ron-autosubmit='1'";
        $combo->render();


        // vyber  dovednosti
        $combo = new ComboBox('dovednost_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->captionIndex = 0;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Kod', RonPortalSession::getValue('dovednost'), $params);
        $combo->htmlParams = "data-ron-select2='dovednost' data-ron-autosubmit='1'";
        $combo->render();

        echo "</DIV>\n";
    }

    /**
     * @throws DBException
     */
    protected function renderUrovenDovednosti()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dovednosti_uroven_dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =
            "SELECT DISTINCT KodUrovne, KodUrovne FROM UrovenDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.Kod = UrovenDovednosti.KodDovednosti AND Aktivni = 1)";

        if (RonPortalSession::getValue('dovednost')) {
            $sql =
                "SELECT KodUrovne, Popis FROM UrovenDovednosti " .
                "WHERE KodDovednosti = " . $params->addParameter(RonPortalSession::getValue('dovednost'));
        }

        // vyber urovne dovednosti
        $combo = new ComboBox('uroven_dovednosti');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->captionIndex = 1;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY 2', RonPortalSession::getValue('uroven_dovednosti'), $params);
        $combo->htmlParams = "data-ron-select2='uroven_dovednosti_kod' data-ron-autosubmit='1'";
        $combo->render();

        // vyber urovne dovednosti
        $combo = new ComboBox('uroven_dovednosti_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->showCode = true;
        $combo->captionIndex = 0;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY 1', RonPortalSession::getValue('uroven_dovednosti'), $params);
        $combo->htmlParams = "data-ron-select2='uroven_dovednosti' data-ron-autosubmit='1'";
        $combo->render();

        echo "</DIV>\n";
    }

    private function renderExpiraceDovednosti()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo '</SPAN>';

        require_once('ron/guiutils.php');
        GuiUtils::renderCheckbox('zobrazit_expiraci', ronText('Dovednosti_zobrazit_expiraci'), RonPortalSession::getValue('zobrazit_expiraci'), 'onChange=submit();');

        echo "</DIV>\n";
    }

    function renderFoto()
    {
        if ($this->vedouciIDosoby) {
            echo
                "<td><div class='fotoDochazka ronFoto'><img src='?menuid=dovednosti&action=foto&osoba=$this->vedouciIDosoby' alt='vedouci'>" .
                "<strong>" . ronText('Dovednosti_vedouci') . ": </strong>" . StringUtils::htmlTextConvert($this->vedouciJmenoPrijmeni) . "</div></td>";
        }
        if ($this->zastupceIDosoby) {
            echo
                "<td><div class='fotoDochazka ronFoto'><img src='?menuid=dovednosti&action=foto&osoba=$this->zastupceIDosoby' alt='zastupce'>" .
                "<strong>" . ronText('Dovednosti_zastupce') . ": </strong>" . StringUtils::htmlTextConvert($this->zastupceJmenoPrijmeni) . "</div></td>";
        }
    }

    /**
     * @throws DBException
     */
    protected function renderDovednostiFilter()
    {
        if ($this->zobrazitFiltrKalendar) {
            $this->renderKalendar();
        }

        $this->renderTypDovednosti();

        $this->renderDovednost();

        $this->renderUrovenDovednosti();

        if ($this->typZobrazeni == 'ziskane') {
            $this->renderExpiraceDovednosti();
        }
    }

    private function loadKalendar()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['kalendar'] = $_POST['kalendar'];
        }
        elseif (isset($_GET['kalendar'])  &&  $_GET['kalendar'] != '') {
            $_SESSION['kalendar'] = $_GET['kalendar'];
        }
    }
    
    protected function loadTypDovednosti()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['typDovednosti'] = $_POST['typDovednosti'];
        }
        elseif (isset($_GET['typDovednosti'])  &&  $_GET['typDovednosti'] != '') {
            $_SESSION['typDovednosti'] = $_GET['typDovednosti'];
        }
    }
    
    protected function loadDovednost()
    {
        if (isset($_POST['adsfilterform'])) {            
            $_SESSION['dovednost'] = $_POST['dovednost'];
        }
        elseif (isset($_GET['dovednost'])  &&  $_GET['dovednost'] != '') {
            $_SESSION['dovednost'] = $_GET['dovednost'];
        }
    }

    protected function loadUrovenDovednosti()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['uroven_dovednosti'] = $_POST['uroven_dovednosti'];
        }
        elseif (isset($_GET['uroven_dovednosti'])  &&  $_GET['uroven_dovednosti'] != '') {
            $_SESSION['uroven_dovednosti'] = $_GET['uroven_dovednosti'];
        }
    }

    private function loadExpiraceDovednosti()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['zobrazit_expiraci'] = $_POST['zobrazit_expiraci'];
        }
        elseif (isset($_GET['zobrazit_expiraci']) &&  $_GET['zobrazit_expiraci'] != '') {
            $_SESSION['zobrazit_expiraci'] = $_GET['zobrazit_expiraci'];
        }
    }

    /**
     * @throws DBException
     */
    private function loadVedouciRozdeleni()
    {
        if (RonPortalSession::rozdeleni($this->f_typrozdeleni)) {
            $params = $this->database->createParameters();
            $sql =
                'SELECT MAX(PP_Vedouci.ID), MAX(PP_Zastupce.ID), O_Vedouci.Jmeno, O_Vedouci.Prijmeni, O_Vedouci.Titul, O_Zastupce.Jmeno, O_Zastupce.Prijmeni, O_Zastupce.Titul
                 FROM Rozdeleni
                   LEFT OUTER JOIN PracovniPomer PP_Vedouci ON (PP_Vedouci.RC = Rozdeleni.VedouciRC)
                   LEFT OUTER JOIN Osoba O_Vedouci ON (O_Vedouci.RC = PP_Vedouci.RC)
                   LEFT OUTER JOIN PracovniPomer PP_Zastupce ON (PP_Zastupce.RC = Rozdeleni.ZastupceRC)
                   LEFT OUTER JOIN Osoba O_Zastupce ON (O_Zastupce.RC = PP_Zastupce.RC)
                 WHERE KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) .
                    ' AND Kod = ' . $params->addParameter(RonPortalSession::rozdeleni($this->f_typrozdeleni)) . ' ' .
                'GROUP BY O_Vedouci.Jmeno, O_Vedouci.Prijmeni, O_Vedouci.Titul, O_Zastupce.Jmeno, O_Zastupce.Prijmeni, O_Zastupce.Titul';

            $cursor = $this->database->getSqlResult($sql, $params);

            $row = $cursor->fetch();

            $this->vedouciIDosoby = $row[0];
            $this->vedouciJmenoPrijmeni = ($row[4] ? $row[4] . ' ' : '') . $row[3] . ' ' . $row[2];
            $this->zastupceIDosoby = $row[1];
            $this->zastupceJmenoPrijmeni = ($row[7] ? $row[7] . ' ' : '') . $row[6] . ' ' . $row[5];
        }

    }

    /**
     * @throws Exception
     */
    function loadData()
    {
        $this->loadKalendar();
        
        $this->loadTypDovednosti();
        
        $this->loadDovednost();

        $this->loadUrovenDovednosti();

        $this->loadExpiraceDovednosti();

        if ($this ->f_showrozdeleni) $this ->loadRozdeleni();

        $filtrTPM = '';

        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->f_typrozdeleni : null;

        //3.10.2017 odfiltrovat seznam TPM podle vybraneho rozdeleni ve filtru 1,
        // jen TPM, ktera maji zamestnanci (vybrani ve filtru 1) prirazene ve vazbe na jim prirazenem pracovnim miste
        if (($this->typRozdeleniPracMisto != -1) && (RonPortalSession::rozdeleni($typRozdeleniSuffix))) {

            $filtrTPM =
                'EXISTS(SELECT 1 ' .
                'FROM RozdeleniOsob R_ZAM JOIN RozdeleniOsob R_PM ON (R_PM.RC = R_ZAM.RC AND R_PM.OsobniCislo = R_ZAM.OsobniCislo AND R_PM.DatumNastupu = R_ZAM.DatumNastupu) ' .
                '   JOIN VazbaRozdeleni ON (VazbaRozdeleni.typNadrazene = R_PM.KodTypuRozdeleni AND VazbaRozdeleni.kodNadrazene = R_PM.Rozdeleni) ' .
                'WHERE R_ZAM.KodTypuRozdeleni = ' . (int)$this->typRozdeleniFiltr1 .
                "  AND R_ZAM.Rozdeleni = '" . $this->database->addSlashes(RonPortalSession::rozdeleni($typRozdeleniSuffix)) . "'" .
                '  AND R_ZAM.PlatnostOd <= ' . $this->database->timestampToString(time()) .
                '  AND (R_ZAM.PlatnostDo >= ' . $this->database->timestampToString(time()) . ' OR R_ZAM.PlatnostDo IS NULL) ' .
                '  AND R_PM.KodTypuRozdeleni = -1' .  //pracovni misto
                '  AND R_PM.PlatnostOd <= ' . $this->database->timestampToString(time()) .
                '  AND (R_PM.PlatnostDo >= ' . $this->database->timestampToString(time()) . ' OR R_PM.PlatnostDo IS NULL) ' .
                '  AND VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                '  AND VazbaRozdeleni.kodPodrizene = Rozdeleni.Kod ' .
                '  )';
        }

        $this->f_filtrrozdeleni2 = $filtrTPM;

        if ($this ->f_showrozdeleni2) $this ->loadRozdeleni2();

        $this->loadVedouciRozdeleni();
    }
                

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters(); // rozdeleni

        $this->renderDovednostiFilter();
        
        echo "<BR />";
    }
}

/**
 * Nacita do pameti (a nasledne zobrazi) matici dovednosti zamestnancu
 */
class SeznamDovednostiOsob implements ViewObject
{
    const CSV_SEPARATOR = ';';

    /** @var Database */
    public $database;

    /** @var OsobaSDovednosti[] */
    public $osoby;

    public $seznamIDPP;
    public $rcPracPomeru;
    
    /** @var DovednostOsoby[] */
    public $seznamDovednosti;
    
    /** @var PravaNaOsoby */
    public $pravaEditace;
    /** @var PravaNaOsoby */
    public $pravaZobrazeni;
    /** @var PravaNaOsoby[] */
    public $pravaZobrazeniOsoby;

    /** @var TypRozdeleni */
    public $typRozdeleniFiltr;
    /** @var TypRozdeleni  */
    public $typRozdeleniPracMisto;

    public $zobrazitFoto;
    public $zobrazitTypPP;
    public $zobrazitPracovniMisto;
    public $zobrazitRozdeleni;
    public $zobrazitKalendar;
    public $zobrazitPritomnost;
    public $zobrazitPocetDosazenychDovednosti;
    /** @var int typ zobrazeni urovne: 0 = kod, 1 = popis */
    public $zobrazeniUrovneDovednosti;

    /** @var string urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti */
    protected $typZobrazeni;
    protected $sqlSelect;
    protected $sqlFrom;
    protected $sqlWhere;
    protected $sqlOrder;
    /** @var Parameters */
    protected $sqlParams;

    /** @var  DovednostiFilterForm */
    protected $filterForm;

    /**
     *
     * @param Database $database
     * @param string $typZobrazeni urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti
     * @throws Exception
     */
    function __construct($database, $typZobrazeni)
    {
        $this ->database = $database;
        $this ->osoby = [];
        $this ->seznamDovednosti = [];
        $this ->typZobrazeni = $typZobrazeni;
        $this->seznamIDPP = [];
        $this->rcPracPomeru = [];
        
        Prava::nastavObdobiProPrava(time(), time());
        
        $this ->pravaEditace = Prava::vratPrava('doved_edit');
        $this ->pravaZobrazeni = Prava::vratPrava('doved_ziskane');

        $prava = Prava::vratPrava('prohl_osobni');
        $pravadoch = Prava::vratPrava('prohlizeni'); // osoby u kterych muze zobrazit zakladni udaje

        $this ->pravaZobrazeniOsoby = [$prava, $pravadoch];
        
        $_SESSION['dovednostibrowser'] = $this->typZobrazeni;

        $this->typRozdeleniFiltr = TypRozdeleni::getTypRozdeleni($this->database,
            (RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_filtr1') ?: RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)));
        //pokud je v nastavení jiný typ rozdělení použije se místo pracovního místa
        $this->typRozdeleniPracMisto = TypRozdeleni::getTypRozdeleni($this->database, (RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm') ?: -1));

        $this->zobrazitFoto = RonPortalSettings::getIntValue('dovednosti_sloupce_foto', 1);
        $this->zobrazitTypPP = RonPortalSettings::getIntValue('dovednosti_sloupce_typ_pp', 1);
        $this->zobrazitPracovniMisto = RonPortalSettings::getIntValue('dovednosti_sloupce_prac_misto', 1);
        $this->zobrazitRozdeleni = RonPortalSettings::getIntValue('dovednosti_sloupce_rozdeleni', 1);
        $this->zobrazitKalendar = RonPortalSettings::getIntValue('dovednosti_sloupce_kalendar', 1);
        $this->zobrazitPritomnost = RonPortalSettings::getIntValue('dovednosti_sloupce_pritomnost', 0);
        $this->zobrazitPocetDosazenychDovednosti = false;
        $this->zobrazeniUrovneDovednosti = RonPortalSettings::getIntValue('dovednosti_typ_zobrazeni_urovne', 0);

        $this->filterForm = new DovednostiFilterForm($this->database,"maticedovednosti&action=$this->typZobrazeni", $this->typZobrazeni,
            $this->typRozdeleniFiltr->kod, $this->typRozdeleniPracMisto->kod, $this->zobrazitKalendar);
    }


    /** Sestavi SQL dotaz.
     *
     * @param string $select
     * @param string $from
     * @param string $where
     * @param Parameters $params
     */
    function nastavSQL($select, $from, $where, $params)
    {
        
        $this ->sqlSelect = $select;
        $this ->sqlFrom   = $from;
        $this ->sqlWhere  = $where;
        $this ->sqlParams = $params;

        $prava = Prava::vratPrava('doved_ziskane');
        
        Prava::nastavObdobiProPrava(time(), time());
        
        if ($this ->sqlWhere == '') {
            $this->sqlWhere = "WHERE 1 = 1 ";
        }

        $this ->sqlWhere .=
            " AND " . $prava->sqlFiltr('PracovniPomer', time(), time());

        $this ->sqlOrder =
            " ORDER BY Osoba.Prijmeni, Osoba.Jmeno";
    }

    /**
     * @param int $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $datumOd
     * @param int $datumDo
     * @param int $vcetnePodrizenych
     * @return string
     */
    function vratFiltrRozdeleni($typRozdeleni, $kodRozdeleni, $datumOd, $datumDo, $vcetnePodrizenych)
    {
        $sqlPodrizene = '';

        switch ($vcetnePodrizenych) {
            case 0:
                $sqlPodrizene = 'and (H.kodNadrazene = H.kodPodrizene) ';
                break;
            case 1:
                $sqlPodrizene = '';
                break;
            case 2:
                $sqlPodrizene = 'and (H.kodNadrazene <> H.kodPodrizene) ';
                break;
            case 3:
                $sqlPodrizene = 'and (H.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = H.typNadrazene AND RozdeleniRodice = H.typNadrazene AND Rodic = H.kodNadrazene)) ';
                break;
        }

        return
            " AND EXISTS(SELECT RO.RC FROM RozdeleniOsob RO JOIN PracovniPomer PP ON (PP.RC = RO.RC AND PP.OsobniCislo = RO.OsobniCislo AND PP.DatumNastupu = RO.DatumNastupu)".
            "   JOIN HierarchieRozdeleni H ON (H.typPodrizene = RO.KodTypuRozdeleni AND H.kodPodrizene = RO.Rozdeleni) " .
            "WHERE RO.RC = Osoba.RC " .
            "  AND PP.DatumNastupu <= " . $this ->database ->dateToString($datumDo) .
            "  AND (PP.DatumUkonceni >= " . $this ->database ->dateToString($datumOd) . " OR PP.DatumUkonceni IS NULL) " .
            "  AND H.typNadrazene = " . $typRozdeleni .
            "  AND H.kodNadrazene = '" . $this ->database ->addSlashes($kodRozdeleni) . "'" .
            $sqlPodrizene .
            "  AND RO.PlatnostOd <= " . $this ->database ->dateToString($datumDo) .
            "  AND (RO.PlatnostDo >= " . $this ->database ->dateToString($datumOd) . " OR RO.PlatnostDo IS NULL) ) ";
    }

    /**
     * Doplni do SQL dotazu filtr na rozdeleni, PM podle zvoleneho typu zobrazeni.
     */
    function doplnFiltr()
    {
        switch ($this->typZobrazeni)
        {
            case 'pozadovane' :
                if ($this->filterForm->kodRozdeleni != '') {
                    $this ->sqlWhere .=
                        $this->vratFiltrRozdeleni($this->typRozdeleniFiltr->kod, $this->filterForm->kodRozdeleni, DateUtils::today(), DateUtils::today(), $this->filterForm->vcetnePodrizenychRozdeleni);
                }

                if ($this->filterForm->kodRozdeleni2 != '') {
                    $this ->sqlWhere .=
                        $this->vratFiltrRozdeleni($this->typRozdeleniPracMisto->kod, $this->filterForm->kodRozdeleni2, DateUtils::today(), DateUtils::today(), $this->filterForm->vcetnePodrizenychRozdeleni2);

                    $this ->sqlWhere .=
                        " AND EXISTS(SELECT H.kodPodrizene FROM HierarchieRozdeleni H " .
                        "WHERE H.typPodrizene = PozadavekNaDovednost.KodTypuRozdeleni" .
                        "  AND H.kodPodrizene = PozadavekNaDovednost.KodRozdeleni" .
                        "  AND H.typNadrazene = " . $this->typRozdeleniPracMisto->kod .
                        "  AND H.kodNadrazene = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni2($this->typRozdeleniPracMisto->kod)) . "'" .
                        (RonPortalSession::rozdeleni2VcetnePodrizenych($this->typRozdeleniPracMisto->kod) ? '' : ' AND H.kodNadrazene = H.kodPodrizene') .
                        "}";
                }

                break;
            case 'ziskane':
            default:               
                if ($this->filterForm->kodRozdeleni != '') {
                    $this ->sqlWhere .=
                        $this->vratFiltrRozdeleni($this->typRozdeleniFiltr->kod, $this->filterForm->kodRozdeleni, DateUtils::today(), DateUtils::today(), $this->filterForm->vcetnePodrizenychRozdeleni);
                }                 
                
                if ($this->filterForm->kodRozdeleni2 != '') {
                    $this ->sqlWhere .=
                        $this->vratFiltrRozdeleni($this->typRozdeleniPracMisto->kod, $this->filterForm->kodRozdeleni2, DateUtils::today(), DateUtils::today(), $this->filterForm->vcetnePodrizenychRozdeleni2);
                }

                break;
        } //switch

        if (RonPortalSession::getValue('kalendar')) {
            $this->sqlWhere .=
                                " AND EXISTS(SELECT PK.RC FROM PrirazeniKalendare PK JOIN PracovniPomer PP ON (PP.RC = PK.RC AND PP.OsobniCislo = PK.OsobniCislo AND PP.DatumNastupu = PK.DatumNastupu)".
                                "WHERE PK.RC = Osoba.RC " .
                                "  AND PP.DatumNastupu <= " . $this ->database ->dateToString(time()) .
                                "  AND (PP.DatumUkonceni >= " . $this ->database ->dateToString(time()) . " OR PP.DatumUkonceni IS NULL) " .
                                "  AND PK.Kalendar = '" . $this ->database ->addSlashes(RonPortalSession::getValue('kalendar')) . "'" .
                                "  AND PK.PlatnostOd <= " . $this ->database ->dateToString(time()) .
                                "  AND (PK.PlatnostDo >= " . $this ->database ->dateToString(time()) . " OR PK.PlatnostDo IS NULL) ) ";
        }
        
        if (RonPortalSession::getValue('typDovednosti')) {
            $this->sqlWhere .= " AND Dovednost.TypDovednosti = '" . $this->database->addSlashes(RonPortalSession::getValue('typDovednosti')) . "'";
        }
        
        if (RonPortalSession::getValue('dovednost')) {
            $this->sqlWhere .= " AND Dovednost.Kod = '" . $this->database->addSlashes(RonPortalSession::getValue('dovednost')) . "'";
        }

        if (RonPortalSession::getValue('uroven_dovednosti')) {
            $this->sqlWhere .=
                " AND EXISTS(SELECT RC FROM DovednostOsoby DO 
                             WHERE DO.KodUrovne = " . (int)(RonPortalSession::getValue('uroven_dovednosti')) .
                "            AND DO.RC = Osoba.RC " .
                (RonPortalSession::getValue('dovednost') ? " AND DO.KodDovednosti = '" . $this->database->addSlashes(RonPortalSession::getValue('dovednost')) . "'" : '') .
                "            AND DO.DatumZiskani <= " . $this->database->timestampToString(time()) .
                "            AND (DO.PlatnostDo >= " . $this->database->timestampToString(time()) . " OR DO.PlatnostDo IS NULL))";
        }
    }

    /** Nacte ziskane/pozadovane dovednosti z databaze.
     *
     * @throws Exception
     */
    function nactiSeznamSQL()
    {             
        $sql = $this -> sqlSelect . $this -> sqlFrom . $this -> sqlWhere . $this -> sqlOrder;
        
        MessageManager::addDebugLog($sql);
        
        $cursor = $this->database->getSqlResult($sql, $this->sqlParams);
        if (!$cursor) return;
        
        while ($row = $cursor->fetch()) {
            $this ->nactiRadek($row);
        }

        uasort($this->seznamDovednosti, array($this, "seradDovednosti"));
    }

    /**
     * @throws DBException
     */
    public function doplnRozdeleniKalendar()
    {
        if ($this->zobrazitRozdeleni || $this->zobrazitPracovniMisto) {
            $typyRozdeleni = [];
            if ($this->zobrazitRozdeleni) {
                $typyRozdeleni[] = $this->typRozdeleniFiltr->kod;
            }
            if ($this->zobrazitPracovniMisto) {
                $typyRozdeleni[] = $this->typRozdeleniPracMisto->kod;
            }

            require_once('adsapp/rozdeleniosob.php');
            $seznamRozdeleni = RozdeleniOsob::nactiRozdeleniOsoby($this->database, $this->seznamIDPP, $typyRozdeleni, DateUtils::today());

            if (!empty($seznamRozdeleni)) {
                foreach ($seznamRozdeleni as $rozdeleni) {
                    $rc = $this->rcPracPomeru[$rozdeleni->idPP];
                    $this->osoby[$rc]->priradRozdeleniOsoby($rozdeleni, $this->typRozdeleniFiltr->kod, $this->typRozdeleniPracMisto->kod);
                }
            }
        }

        if ($this->zobrazitKalendar) {
            $seznamKalendaru = KalendarOsoby::nactiKalendarOsoby($this->database, $this->seznamIDPP, DateUtils::today());

            if (!empty($seznamKalendaru)) {
                foreach ($seznamKalendaru as $kalendar) {
                    $this->osoby[$kalendar->rc]->priradKalendar($kalendar);
                }
            }
        }
    }

    /**
     * @param $row
     * @throws Exception
     */
    function nactiRadek($row)
    {        
        //pokud osoba jeste nebyla nactena do seznamu vytvori se pro ni objekt
        if (!array_key_exists($row[0], $this ->osoby)) {
            $osoba = new OsobaSDovednosti($row[0], $row[1], $row[2]);
            $osoba ->idOsoby = $row[7];
            $osoba ->osobniCislo = $row[8];
            $osoba ->typPP = $row[9];
            $osoba ->idFotografie = $row[10];
            $osoba ->lzeEditovat = $this->pravaEditace->lzeZobrazitPP($osoba->idOsoby);
            $osoba ->lzeZobrazit = $this->pravaZobrazeni->lzeZobrazitPP($osoba->idOsoby);
            $osoba ->lzeZobrazitOsobniUdaje = Prava::lzeZobrazitPP($this->pravaZobrazeniOsoby, $osoba->idOsoby);

            /*if ($row[15]) {
                $osoba ->kodKalendare = $row[15];
                $osoba->popisKalendare = $row[16];
            }*/
            $this ->osoby[$row[0]] = $osoba;
        } else {
            //doplni osobni cislo do seznamu osobnich cisel osoby, pokud jeste v seznamu neni
            if (($this ->osoby[$row[0]] ->osobniCislo <> '') && (strpos($this ->osoby[$row[0]] ->osobniCislo,$row[8]) === false)) {
                $this ->osoby[$row[0]] ->osobniCislo .= ', ' . $row[8];
            }
            
            //doplni typ PP do seznamu typu PP osoby, pokud jeste v seznamu neni
            if (($this ->osoby[$row[0]] ->typPP <> '') && (strpos($this ->osoby[$row[0]] ->typPP,$row[9]) === false)) {
                $this ->osoby[$row[0]] ->typPP .= ', ' . $row[9];
            }            
        }

        if (!in_array($row[7], $this->seznamIDPP)) {
            $this->seznamIDPP[] = $row[7];
        }
        $this->rcPracPomeru[$row[7]] = $row[0];
        
        //pokud ma zamestnanec pracovni misto, doplni ho do seznamu prac. mist osoby
       /* if ($row[10])
        {
            $pracMisto = new RozdeleniOsoby($row[10], $row[11]);
            $this ->osoby[$row[0]] ->priradPracovniMisto($pracMisto);
        }
        
        //pokud ma zamestnanec rozdeleni pro zobrazeni, doplni ho do seznamu
        if ($row[12])
        {
            $rozdeleni = new RozdeleniOsoby($row[12], $row[13]);
            $this ->osoby[$row[0]] ->priradRozdeleniOsoby($rozdeleni);
        }*/

        //pokud dovednost jeste neni v seznamu dovednosti pro hlavicku, doplni ji
        $dovednost = isset($this ->seznamDovednosti[$row[4]]) ? $this ->seznamDovednosti[$row[4]] : null;
        if (!$dovednost) {
            $dovednost = new DovednostOsoby($row[4], $row[3]);
            $this ->seznamDovednosti[$row[4]] = $dovednost;
        }

        if ($row[6] == 'ziskane') {
            $ziskanaDovednost = new DovednostOsoby($row[4], $row[3], $row[5]);
            $ziskanaDovednost->popisUrovne = $row[12];
            $ziskanaDovednost->platnostDo = $this->database->stringToTimestamp($row[11]);
            $this ->osoby[$row[0]]->priradZiskanouDovednost($ziskanaDovednost, $row[13]);

            if (!isset($dovednost ->htmlPopisUrovni[$row[5]])) {
                $dovednost ->htmlPopisUrovni[$row[5]] = StringUtils::htmlTextConvert($row[12]);
            }
        }
        elseif ($row[6] == 'pozadovane') {
            $pozadovanaDovednost = new DovednostOsoby($row[4], $row[3], $row[5]);
            $pozadovanaDovednost->popisUrovne = $row[13];
            $this ->osoby[$row[0]]->priradPozadovanouDovednost($pozadovanaDovednost);

            if (!isset($dovednost ->htmlPopisUrovni[$row[5]])) {
                $dovednost ->htmlPopisUrovni[$row[5]] = StringUtils::htmlTextConvert($row[13]);
            }

            //pokud ma osoba i ziskane dovednosti doplni je do seznamu ziskanych dovednosti
            if ($row[12]) {
                $ziskanaDovednost = new DovednostOsoby($row[4], $row[3], $row[12]);
                $ziskanaDovednost->popisUrovne = $row[14];
                $this ->osoby[$row[0]]->priradZiskanouDovednost($ziskanaDovednost, $row[15]);
            }
        }
        


    }

    /**
     * @param string $select
     * @param string $from
     * @param string $where
     * @param Parameters $params
     * @throws Exception
     */
    function nactiSeznam($select, $from, $where, $params)
    {
        $this ->nastavSQL($select, $from, $where, $params);

        $this->filterForm->loadData();

        $this ->doplnFiltr();
        $this ->nactiSeznamSQL();
        $this ->doplnRozdeleniKalendar();
    }

    /**
     * @throws Exception
     * @throws DBException
     */
    public function zobrazFiltrProTisk()
    {
        $rozdeleni = new Rozdeleni();
        if (RonPortalSession::rozdeleni($this->typRozdeleniFiltr->kod) != '') {
            $rozdeleni->load($this->database, $this->typRozdeleniFiltr->kod, RonPortalSession::rozdeleni($this->typRozdeleniFiltr->kod));
        }

        $params = $this->database->createParameters();
        $typDovednosti = DBUtils::getSqlValue($this->database,
            'SELECT Popis FROM TypDovednosti WHERE Kod = ' . $params->addParameter(RonPortalSession::getValue('typDovednosti')), $params);

        echo "<div class='dovednostifilterprint'>";
        if ($rozdeleni->kod) {
            echo "<strong>" . $this->typRozdeleniFiltr->popis . "</strong>: <span class='$rozdeleni->cssClass'> $rozdeleni->popis ($rozdeleni->kod) </span> <br>";
        }
        if (RonPortalSession::getValue('typDovednosti')) {
            echo "<strong>" . ronText('Dovednosti_typ_dovednosti') . "</strong>: <span>" . StringUtils::htmlTextConvert($typDovednosti) . "</span> <br>";
        }
        if ($this->filterForm->vedouciJmenoPrijmeni) {
            echo "<strong>" . ronText('Dovednosti_vedouci') . "</strong>: <span>" . StringUtils::htmlTextConvert($this->filterForm->vedouciJmenoPrijmeni) . "</span> <br>";
        }
        if ($this->filterForm->zastupceJmenoPrijmeni) {
            echo "<strong>" . ronText('Dovednosti_zastupce') . "</strong>: <span>" . StringUtils::htmlTextConvert($this->filterForm->zastupceJmenoPrijmeni) . "</span> <br>";
        }
        echo "</div>";
    }

    function zobrazMatici()
    {
        switch ($this->typZobrazeni)
        {
            case 'pozadovane':
                require_once('person/view_pozadovaneDovednosti.php');
                break;
            case 'ziskane':
            default :                
                require_once('person/view_ziskaneDovednosti.php');
                break;
        }        
    }

    /**
     * @throws Exception
     * @throws DBException
     */
    public function render()
    {
        $this->filterForm->render();
        $this->zobrazFiltrProTisk();
        $this ->zobrazMatici();
    }

    public function hlavickaCSV()
    {
        $data = '';

        $data .= ronText('Zamestnanec') . static::CSV_SEPARATOR;
        $data .= ronText('Osobni_cislo') . static::CSV_SEPARATOR;
        if ($this->zobrazitTypPP) {
            $data .= ronText('Dovednosti_typ_PP') . static::CSV_SEPARATOR;
        }
        if ($this->zobrazitPracovniMisto) {
            $data .= $this->typRozdeleniPracMisto->popis . static::CSV_SEPARATOR;
        }
        if ($this->zobrazitRozdeleni) {
            $data .= $this->typRozdeleniFiltr->popis . static::CSV_SEPARATOR;
        }
        if ($this->zobrazitKalendar) {
            $data .= ronText('Kalendar') . static::CSV_SEPARATOR;
        }
        $data .= ronText('Dovednosti_stav') . static::CSV_SEPARATOR;

        foreach($this->seznamDovednosti as $dovednost)
        {
            $data .= $dovednost->popis . static::CSV_SEPARATOR;
            //zobrazit platnost
            if (RonPortalSession::getValue('zobrazit_expiraci')) {
                $data .=  ronText('Platnost_do') . static::CSV_SEPARATOR;
            }

        }

        $data .= "\n";

        return $data;
    }

    public function dataDovednostiDoCSV()
    {
        $data = $this->hlavickaCSV();

        foreach($this->osoby as $osoba)
        {
            $data .= $osoba->prijmeni . ' ' . $osoba->jmeno . static::CSV_SEPARATOR;

            $data .= $osoba->osobniCislo . static::CSV_SEPARATOR;

            if ($this->zobrazitTypPP == 1) {
                $data .= $osoba->typPP . static::CSV_SEPARATOR;
            }

            if ($this->zobrazitPracovniMisto) {
                //vypise seznam pracovnich mist osoby
                $seznamPM = '';
                $seznamPopisuPM = '';
                foreach ($osoba->pracovniMista as $pracovniMisto) {
                    if ($seznamPM) {
                        $seznamPM .= ', ';
                    }
                    if ($seznamPopisuPM) {
                        $seznamPopisuPM .= ', ';
                    }
                    $seznamPopisuPM .= $pracovniMisto->popis;
                    $seznamPM .= $pracovniMisto->kod;
                }
                if ($this->zobrazitPracovniMisto == 1) {
                    $data .= $seznamPM . static::CSV_SEPARATOR;
                } elseif ($this->zobrazitPracovniMisto == 2) {
                    $data .= $seznamPopisuPM . static::CSV_SEPARATOR;
                }
            }

            if ($this->zobrazitRozdeleni) {
                //vypise seznam kodu rozdeleni osoby
                $seznamRozdeleni = '';
                $seznamPopisuRozdeleni = '';
                foreach ($osoba->prirazenaRozdeleniOsoby as $rozdeleni) {
                    if ($seznamRozdeleni) {
                        $seznamRozdeleni .= ', ';
                    }
                    if ($seznamPopisuRozdeleni) {
                        $seznamPopisuRozdeleni .= ', ';
                    }
                    $seznamPopisuRozdeleni .= $rozdeleni->popis;
                    $seznamRozdeleni .= $rozdeleni->kod;
                }
                if ($this->zobrazitRozdeleni == 1) {
                    $data .= $seznamRozdeleni . static::CSV_SEPARATOR;
                } elseif ($this->zobrazitRozdeleni == 2) {
                    $data .= $seznamPopisuRozdeleni . static::CSV_SEPARATOR;
                }
            }

            if ($this->zobrazitKalendar) {
                if ($this->zobrazitKalendar == 1) {
                    $data .= $osoba->kodKalendare . static::CSV_SEPARATOR;
                } elseif ($this->zobrazitKalendar == 2) {
                    $data .= $osoba->popisKalendare . static::CSV_SEPARATOR;
                }
            }

            $data .= $osoba->pocetZiskanychDovednosti . ($this->typZobrazeni == 'pozadovane' ? '/' . $osoba->pocetPozadovanychDovednosti : '') . static::CSV_SEPARATOR;

            /** @var DovednostOsoby $dovednost */
            foreach($this->seznamDovednosti as $dovednost)
            {
                if ($this->typZobrazeni == 'pozadovane') {
                    $data .= $osoba->urovenZiskaneDovednosti($dovednost->kod, true) . '/' . $osoba->urovenPozadovaneDovednosti($dovednost->kod, true) . static::CSV_SEPARATOR;
                }
                else {
                    $data .=
                        ($this->zobrazeniUrovneDovednosti == 0 ? $osoba->urovenZiskaneDovednosti($dovednost->kod) : $dovednost ->htmlPopisUrovni[$osoba->ziskaneDovednosti[$dovednost->kod]->uroven]) . static::CSV_SEPARATOR;

                    //zobrazit platnost do
                    if (RonPortalSession::getValue('zobrazit_expiraci')) {
                        if ($osoba->ziskaneDovednosti[$dovednost->kod]->platnostDo > 0) {
                            $data .= StringUtils::format('date', $osoba->ziskaneDovednosti[$dovednost->kod]->platnostDo) . static::CSV_SEPARATOR;
                        }
                    }
                }
            }

            $data .= "\n";
        }

        return $data;
    }

    /**
     * @return DataOutputObject
     */
    public function outputCSV()
    {
        $data = $this->dataDovednostiDoCSV();
        switch ($this->typZobrazeni)
        {
            case 'pozadovane':
                $fileName = 'pozadovaneDovednosti.csv';
                break;
            case 'ziskane':
            default :
                $fileName = 'ziskaneDovednosti.csv';
                break;
        }
        return new DataOutputObject($data, $fileName, TextManager::getInstance() ->getCharSet());
    }

    /**
     * @param DovednostOsoby $dovednost1
     * @param DovednostOsoby $dovednost2
     * @return int
     */
    public function seradDovednosti($dovednost1, $dovednost2)
    {
        return StringUtils::compareStrings($dovednost1->popis, $dovednost2->popis);
    }
}

class DovednostOsoby
{
    /** @var string  */
    public $kod;
    /** @var string  */
    public $popis;
    /** @var int */
    public $uroven;
    /** @var  string */
    public $popisUrovne;

    /** @var array  */
    public $specifikaceUrovne;
    
    /** @var DovednostOsoby[] */
    public $podrizeneDovednosti;
    
    public $urovneDovednosti;

    public $platnostDo;

    /** @var string[] */
    public $htmlPopisUrovni = [];

    /**
     *
     * @param string $kod kod Dovednosti
     * @param string $popis popis Dovednosti
     * @param int $uroven nejvyssi uroven Dovednosti pro zamestnance
     */
    function __construct($kod, $popis = '', $uroven = 0)
    {
        $this -> kod = $kod;
        $this -> popis = $popis;
        $this -> uroven = $uroven;

        $this->specifikaceUrovne = [];
    }

    /**
     * Nacita seznam vsech urovni dovednosti pro nacteni AJAXem do vyberu urovni dovednosti po vybrani konkretni dovednosti*
     * @param Database $database
     * @return bool
     * @throws Exception
     */
    function nactiUrovne($database)
    {
        $params = $database->createParameters();

        $sql = "SELECT KodUrovne, Popis FROM UrovenDovednosti WHERE PovolitRucniZadani = 1 AND KodDovednosti = " . $params->addParameter($this->kod);

        $cursor = $database ->getSqlResult($sql, $params);
        if ($cursor)
        {
            $row = $cursor->fetch();

            if (!$row)
            {
                throw new Exception('Chyba - úrovně dovednosti: ' . $this->kod . ' nebyly nalezeny!');
            }
            else
            {
                $urovne = array();
                $urovne[$row[0]] = $row[1];
            }

            while ($row = $cursor->fetch())
            {
                $urovne[$row[0]] = $row[1];
            }

            $this ->urovneDovednosti = $urovne;

        }

        return true;
    }
}

class KalendarOsoby
{
    public $kod;
    public $popis;
    public $idpp;
    public $rc;

    /**
     * @param Database $database
     * @param int|int[] $pp
     * @param $obdobiOd
     * @param $obdobiDo
     * @return KalendarOsoby[]|null
     * @throws DBException
     */
    public static function nactiKalendarOsoby($database, $pp, $obdobiOd, $obdobiDo = null)
    {
        if (!isset($obdobiDo)) {
            $obdobiDo = $obdobiOd;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar.Kod, Kalendar.Popis, PracovniPomer.ID, PracovniPomer.RC ' .
            'FROM PrirazeniKalendare JOIN Kalendar ON PrirazeniKalendare.Kalendar = Kalendar.Kod ' .
            '   JOIN PracovniPomer on PrirazeniKalendare.RC = PracovniPomer.RC and PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo and PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu ' .
            ' WHERE PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ' OR PlatnostDo IS NULL)';

        if ($pp  &&  is_array($pp)) {
            // neprazdny seznam idpp
            $sql .= ' AND PracovniPomer.Id IN (';
            $first = true;
            foreach ($pp as $idPP) {
                if ($first) {
                    $first = false;
                } else {
                    $sql .= ', ';
                }
                $sql .= $params ->addParameter($idPP);
            }
            $sql .= ')';
        } else {
            $sql .=
                '  AND PracovniPomer.Id = ' . $params ->addParameter((int)$pp);
        }

         $sql .=
            ' ORDER BY PlatnostOd ';

        $kalendare = [];

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        while ($row = $cursor ->fetch()) {
            $kalendar = new KalendarOsoby();
            $kalendar->kod = $row[0];
            $kalendar->popis = $row[1];
            $kalendar->idpp = $row[2];
            $kalendar->rc = $row[3];
            $kalendare[] = $kalendar;
        }
        $cursor ->close();
        return !empty($kalendare) ? $kalendare : null;
    }
}

class OsobaSDovednosti
{
    protected $rc;
    
    public $jmeno;
    public $prijmeni;
    
    public $pocetZiskanychDovednosti;
    public $pocetPozadovanychDovednosti;
    
    /** @var RozdeleniOsob[] */
    public $prirazenaRozdeleniOsoby;
    public $idOsoby;
    public $osobniCislo;
    public $typPP;
    public $idFotografie;
    public $kodKalendare;
    public $popisKalendare;
    /** @var RozdeleniOsob[] */
    public $pracovniMista;
    
    /** @var DovednostOsoby[] */
    public $pozadovaneDovednosti;
    
    /** @var DovednostOsoby[] */
    public $ziskaneDovednosti;
    
    public $lzeEditovat;
    public $lzeZobrazit;
    public $lzeZobrazitOsobniUdaje;
    
    public function __construct($rc, $jmeno = '', $prijmeni = '')
    {
        $this -> rc = $rc;
        $this -> jmeno = $jmeno;
        $this -> prijmeni = $prijmeni;
        
        $this -> ziskaneDovednosti = [];
        $this -> pozadovaneDovednosti = [];
        $this -> pracovniMista = [];
        $this -> prirazenaRozdeleniOsoby = [];
        
        $this -> pocetZiskanychDovednosti = 0;
        /** @var int pocetPozadovanychDovednosti */
        $this -> pocetPozadovanychDovednosti = 0;
    }

    /**
     * @param DovednostOsoby $dovednost
     */
    public function priradPozadovanouDovednost($dovednost)
    {
        //pokud zamestnanec jeste nema pozadavek na dovednost, doplni se do seznamu pozadavku
        if (!array_key_exists($dovednost->kod, $this->pozadovaneDovednosti))
        {
            $this ->pozadovaneDovednosti[$dovednost->kod] = $dovednost;
            $this ->pocetPozadovanychDovednosti += $dovednost->uroven;
        }
        else 
        {
            //pokud je jiny pozadavek dovednosti na vyssi uroven, zvysi se pozadovana uroven dovednosti a celkovy pocet pozadovanych urovni
            if ($this ->pozadovaneDovednosti[$dovednost->kod]->uroven < $dovednost->uroven)
            {
                $rozdil = $dovednost->uroven - $this ->pozadovaneDovednosti[$dovednost->kod]->uroven;
                $this ->pozadovaneDovednosti[$dovednost->kod]->uroven = $dovednost->uroven;
                $this ->pozadovaneDovednosti[$dovednost->kod]->popisUrovne = $dovednost->popisUrovne;
                $this ->pocetPozadovanychDovednosti += $rozdil;
            }
        }
    }

    /**
     *
     * @param DovednostOsoby $dovednost
     * @param int $specifikaceUrovneDovednosti
     */
    public function priradZiskanouDovednost($dovednost, $specifikaceUrovneDovednosti)
    {
        //pokud zamestnanec jeste nema prirazenou dovednost, doplni se do seznamu ziskanych dovednosti
        if (!array_key_exists($dovednost->kod, $this->ziskaneDovednosti))
        {
            $this ->ziskaneDovednosti[$dovednost->kod] = $dovednost;
            $this ->pocetZiskanychDovednosti += $dovednost->uroven;
        }
        else 
        {
            //pokud je jina ziskana dovednost na vyssi uroven, zvysi se ziskana uroven dovednosti a celkovy pocet ziskanych urovni
            if ($this ->ziskaneDovednosti[$dovednost->kod]->uroven < $dovednost->uroven)
            {
                $rozdil = $dovednost->uroven - $this ->ziskaneDovednosti[$dovednost->kod]->uroven;
                $this ->ziskaneDovednosti[$dovednost->kod]->uroven = $dovednost->uroven;
                $this ->ziskaneDovednosti[$dovednost->kod]->popisUrovne = $dovednost->popisUrovne;
                $this ->pocetZiskanychDovednosti += $rozdil;
            }
        }

        $this ->ziskaneDovednosti[$dovednost->kod]->specifikaceUrovne[$dovednost->uroven] = $specifikaceUrovneDovednosti;
    }
    
    /**
     *
     * @param RozdeleniOsob $rozdeleni
     * @param int $typRozdeleni
     * @param int $typRozdeleniPM
     */
    public function priradRozdeleniOsoby($rozdeleni, $typRozdeleni, $typRozdeleniPM)
    {
        if ($rozdeleni->kodTypuRozdeleni == $typRozdeleni) {
            if (!array_key_exists($rozdeleni->kod, $this->prirazenaRozdeleniOsoby)) {
                $this->prirazenaRozdeleniOsoby[$rozdeleni->kod] = $rozdeleni;
            }
        }
        if ($rozdeleni->kodTypuRozdeleni == $typRozdeleniPM) {
            if (!array_key_exists($rozdeleni->kod, $this->pracovniMista)) {
                $this->pracovniMista[$rozdeleni->kod] = $rozdeleni;
            }
        }

    }

    /**
     * @param KalendarOsoby $kalendar
     */
    public function priradKalendar($kalendar)
    {
        $this->kodKalendare = $kalendar->kod;
        $this->popisKalendare = $kalendar->popis;
    }
            
    public function urovenZiskaneDovednosti($kodDovednosti, $zobrazitNulu = false)
    {
        //if (array_key_exists($kodDovednosti, $this->ziskaneDovednosti))
        if (isset($this->ziskaneDovednosti[$kodDovednosti]))
        {
            return $this->ziskaneDovednosti[$kodDovednosti]->uroven;
        }

        return $zobrazitNulu ? 0 : '';
    }
    
    public function urovenPozadovaneDovednosti($kodDovednosti, $zobrazitNulu = false)
    {
        //if (array_key_exists($kodDovednosti, $this->pozadovaneDovednosti))
        if (isset($this->pozadovaneDovednosti[$kodDovednosti]))
        {
            return $this->pozadovaneDovednosti[$kodDovednosti]->uroven;
        }

        return $zobrazitNulu ? 0 : '';
    }
}


class DovednostOsobyCombo extends ComboBox implements ICreateItem
{
    /** @var Database */
    public $database;

    /**
     * @param string $varName
     * @param Database $database
     */
    function __construct($varName, $database)
    {
        parent::__construct($varName);
        $this ->database = $database;
    }

    function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[1] . '~' . $row[2] . '~' . DBUtils::getFormatedValue($this ->database, 'date', $row[3]); // kod dovednosti~kod urovne~datum ziskani
        $caption = $row[4] . ' - ' . $row[5] . StringUtils::fromUTF8(' - získáno: ') . DBUtils::getFormatedValue($this ->database, 'finedate', $row[3]); // Popis dovednosti - popis urovne : datum ziskani
        
        return new Option($key, $caption);
    }
}


class NovyDokumentDovednostiForm extends DokumentUploadForm
{
    private $idOsoby = 0;    


    function __construct($database, $idPP)
    {
        parent::__construct($database, '?menuid=dovednosti&action=dokumentupload');
        $this ->idOsoby = $idPP;

        $this ->hiddenFields = [
            'dokumentdovednosti' => '1',
            'idosoby' => $this ->idOsoby,
        ];
    }


    /**
     * @throws DBException
     */
    function dalsiRadkyKDokumentu()
    {
        echo '<div class="form-gap"></div>';

        // zobrazi vyber radku, ke kteremu se ma dokument pripojit
        echo '<div class="row" id="novydokument_dovednostrow">';

        echo '<div class="col col-12 col-sm-4 caption">' . ronText('Dovednosti_pripojit_dokument') . '</div>';
        echo '<div class="col col-12 col-sm-8">';
        $combo = new DovednostOsobyCombo('Dovednost', $this ->database);
        $sql =
            'SELECT DovednostOsoby.RC, DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, Dovednost.Popis, UrovenDovednosti.Popis' .
            '  FROM DovednostOsoby ' .
            '    JOIN UrovenDovednosti ON (UrovenDovednosti.KodDovednosti = DovednostOsoby.KodDovednosti AND UrovenDovednosti.KodUrovne = DovednostOsoby.KodUrovne)' .
            '    JOIN Dovednost ON (Dovednost.Kod = UrovenDovednosti.KodDovednosti)' .
            ' WHERE DovednostOsoby.DatumZiskani <= ' . $this ->database->dateToString(time()) .
            '   AND (DovednostOsoby.PlatnostDo >= ' . $this ->database->dateToString(time()) . ' OR DovednostOsoby.PlatnostDo IS NULL)' .
            '   AND EXISTS(SELECT RC FROM PracovniPomer WHERE PracovniPomer.RC = DovednostOsoby.RC AND ID = ' . (int)$this ->idOsoby . ')';

        //$combo ->LoadData($this ->database, $sql, 0, 1, '', null, false);
        $combo ->loadData($this ->database, $sql, null);

        $combo ->render();
        echo '</div>';

        echo '</div>';
    }
}

class VyhledaniDovednostiFilterForm extends DovednostiFilterForm
{
    private $pocetDovednosti;

    public function __construct(Database $database, $action, $typZobrazeni, $typRozdeleniFiltr1, $typRozdeleniPracMisto)
    {
        parent::__construct($database, $action, $typZobrazeni, $typRozdeleniFiltr1, $typRozdeleniPracMisto, false);
        $this->skrytTlacitkoNastavit = true;
        $this->pocetDovednosti = 1;

    }

    /**
     * @param int $index index prvku ve strance
     * @throws DBException
     */
    protected function renderTypDovednostiOsoby($index)
    {
        echo '<DIV class="filterrow_dovednost">';
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dovednosti_typ_dovednosti');
        echo '</SPAN>';

        // combobox pro vyber
        $sql = 'SELECT Kod, Popis FROM TypDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.TypDovednosti = TypDovednosti.Kod AND Aktivni = 1)';

        // vyber typu dovednosti
        $combo = new ComboBox('typ_dovednosti'.$index);
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql. ' ORDER BY Popis', RonPortalSession::getValue('typydovednosti')[$index]);
        $combo->htmlParams = "data-ron-select2='typ_dovednosti_kod$index' data-ron-autosubmit='1'";
        $combo->render();

        // vyber typu dovednosti
        $combo = new ComboBox('typ_dovednosti_kod'.$index);
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->emptyRow = ronText('_vse_');
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql. ' ORDER BY Kod', RonPortalSession::getValue('typydovednosti')[$index]);
        $combo->htmlParams = "data-ron-select2='typ_dovednosti$index' data-ron-autosubmit='1'";
        $combo->render();

        echo '<span class="dovednosti_caption"></span>';
        GuiUtils::renderCheckbox('deletefilter' . $index, ronText('Odebrat'), false);

        echo '</DIV>';
    }

    /**
     * @param int $index index prvku ve strance
     * @throws DBException
     */
    protected function renderDovednostOsoby($index)
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

//        $sqlFiltr = "WHERE TypDovednosti = " . $params->addParameter($_SESSION['typydovednosti'][$index]); filtrovat jen dle vybraneho typu?

        // combobox pro vyber
        $sql =  "SELECT Kod, Popis FROM Dovednost WHERE Aktivni = 1 ";

        if (isset($_SESSION['typydovednosti'][$index]) && ($_SESSION['typydovednosti'][$index] != '')) {
            $sql .= "AND TypDovednosti = " . $params->addParameter($_SESSION['typydovednosti'][$index]);
        }

        // vyber typu dovednosti
        $combo = new ComboBox('dovednost'.$index);
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->emptyRow = ''; //\Ron\ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Popis', $_SESSION['seznamdovednosti'][$index], $params);
        $combo->htmlParams = "data-ron-select2='dovednost_kod$index' data-ron-autosubmit='1'";
        $combo->render();

        // vyber typu dovednosti
        $combo = new ComboBox('dovednost_kod'.$index);
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->captionIndex = 0;
        $combo->emptyRow = ''; //\Ron\ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Kod', $_SESSION['seznamdovednosti'][$index], $params);
        $combo->htmlParams = "data-ron-select2='dovednost$index' data-ron-autosubmit='1'";
        $combo->render();
    }

    /**
     * @param int $index index prvku ve strance
     * @throws DBException
     */
    protected function renderUrovenDovednostiOsoby($index)
    {
        echo '<span class="dovednost_filter_uroven">';

        echo '<SPAN class="dovednosti_caption">';
        echo ronText('Dovednosti_uroven_dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

       // combobox pro vyber
        $sql =
            "SELECT DISTINCT KodUrovne FROM UrovenDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.Kod = UrovenDovednosti.KodDovednosti AND Aktivni = 1)";

        if (isset($_SESSION['seznamdovednosti'][$index]) && ($_SESSION['seznamdovednosti'][$index] != '')) {
            $sql .= " AND KodDovednosti = " . $params->addParameter($_SESSION['seznamdovednosti'][$index]);
        }

        // vyber urovne dovednosti
        $combo = new ComboBox('uroven_dovednosti'.$index);
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->showCode = true;
        $combo->captionIndex = 0;
        $combo->emptyRow = ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY KodUrovne', $_SESSION['seznam_urovni_dovednosti'][$index], $params);
        $combo->render();

        echo "</span></DIV></DIV>\n";
    }

    /**
     * @throws DBException
     */
    private function renderSeznamDovednosti()
    {
        for ($i = 1; $i <= $this->pocetDovednosti; $i++) {
            $this->renderTypDovednostiOsoby($i);
            $this->renderDovednostOsoby($i);
            $this->renderUrovenDovednostiOsoby($i);
        }

        echo '<INPUT type="submit" class="submitbutton" name="addfilter" value="' . ronText('Pridat') . '">';
        echo '<INPUT type="submit" class="submitbutton" name="deletefilter" value="' . ronText('Odebrat') . '">';
    }

    /**
     * @throws DBException
     */
    protected function renderDovednostiFilter()
    {
        $this->renderSeznamDovednosti();
    }

    function renderFoto()
    {
        echo "<td><div><INPUT type='submit' class='submitbutton' name='find' value='" . ronText('Vyhledat') . "'></div></td>";
    }

    protected function loadSeznamDovednosti($pridejDovednost)
    {
        if (isset($_POST['pocet_dovednosti']))
            $this->pocetDovednosti = $_POST['pocet_dovednosti'];
        elseif (isset($_SESSION['dovednost_pocet_dovednosti']))
            $this->pocetDovednosti = $_SESSION['dovednost_pocet_dovednosti'];

        if ($pridejDovednost)
            $this->pocetDovednosti++;

        $puvodniPocetDovednosti = (int)$this->pocetDovednosti;
        $pocetSmazanych = 0;
        $indexSession = 1;

        if (isset($_POST['adsfilterform'])) {
            for ($index = 1; $index <= $puvodniPocetDovednosti; $index++) {
                $nacistDovednost = true;
                if (isset($_POST['deletefilter' . $index])) {
                    $nacistDovednost = false;
                    $this->pocetDovednosti--;
                    $pocetSmazanych++;
                }
                if ($nacistDovednost) {
                    $_SESSION['typydovednosti'][$indexSession] = $_POST['typ_dovednosti' . ($index)];
                    $_SESSION['seznamdovednosti'][$indexSession] = $_POST['dovednost_kod' . ($index)];
                    $_SESSION['seznam_urovni_dovednosti'][$indexSession] = $_POST['uroven_dovednosti' . ($index)];
                    $indexSession++;
                }
            }

            //probehlo mazani, zrusi se prvky z  pole
            if ($pocetSmazanych > 0) {
                for ($index = ($puvodniPocetDovednosti + 1) - $pocetSmazanych; $index <= $puvodniPocetDovednosti; $index++) {
                    unset($_SESSION['typydovednosti'][$index]);
                    unset($_SESSION['seznamdovednosti'][$index]);
                    unset($_SESSION['seznam_urovni_dovednosti'][$index]);
                }
            }
        }

        $this->hiddenFields['pocet_dovednosti'] = $this->pocetDovednosti;
        $_SESSION['dovednost_pocet_dovednosti'] = $this->pocetDovednosti;
    }

    /**
     * @throws Exception
     */
    function loadData()
    {
        $pridejDovednost = isset($_POST['addfilter']);

        $this->loadTypDovednosti();

        $this->loadSeznamDovednosti($pridejDovednost);

        if (isset($_POST['find']) || isset($_POST['showdata']))
            $this->hiddenFields['showdata'] = 1;

        if ($this ->f_showrozdeleni) $this ->loadRozdeleni();

        if ($this ->f_showrozdeleni2) $this ->loadRozdeleni2();
    }


}

class SeznamDovednostiOsobVyhledani extends SeznamDovednostiOsob
{
    /**
     * SeznamDovednostiOsobVyhledani constructor.
     * @param Database $database
     * @param $typZobrazeni
     * @throws Exception
     */
    public function __construct(Database $database, $typZobrazeni)
    {
        parent::__construct($database, $typZobrazeni);

        $this->zobrazitKalendar = false;
        $this->zobrazitPocetDosazenychDovednosti = true;

        $this->filterForm = new VyhledaniDovednostiFilterForm($this->database, "maticedovednosti&action=vyhledani", $this->typZobrazeni,
            $this->typRozdeleniFiltr->kod, $this->typRozdeleniPracMisto->kod);
    }

    /**
     * @throws Exception
     */
    public function nactiFiltr()
    {
        $this->filterForm->loadData();
    }

    public function doplnFiltr()
    {
        if (RonPortalSession::rozdeleni($this->typRozdeleniFiltr->kod)) {
            $this ->sqlWhere .=
                " AND EXISTS(SELECT RO.RC FROM RozdeleniOsob RO JOIN PracovniPomer PP ON (PP.RC = RO.RC AND PP.OsobniCislo = RO.OsobniCislo AND PP.DatumNastupu = RO.DatumNastupu)".
                "WHERE RO.RC = Osoba.RC " .
                "  AND PP.DatumNastupu <= " . $this ->database ->dateToString(time()) .
                "  AND (PP.DatumUkonceni >= " . $this ->database ->dateToString(time()) . " OR PP.DatumUkonceni IS NULL) " .
                //20.3.2017 RON-PR typ rozdeleni se nove nastavuje pro dovednosti zvlast
                "  AND RO.KodTypuRozdeleni = " . $this->typRozdeleniFiltr->kod .
                "  AND RO.Rozdeleni = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->typRozdeleniFiltr->kod)) . "'" .
                "  AND RO.PlatnostOd <= " . $this ->database ->dateToString(time()) .
                "  AND (RO.PlatnostDo >= " . $this ->database ->dateToString(time()) . " OR RO.PlatnostDo IS NULL) ) ";
        }

        if (RonPortalSession::rozdeleni2($this->typRozdeleniPracMisto->kod)) {
            $this ->sqlWhere .=
                " AND EXISTS(SELECT RO.RC FROM RozdeleniOsob RO JOIN PracovniPomer PP ON (PP.RC = RO.RC AND PP.OsobniCislo = RO.OsobniCislo AND PP.DatumNastupu = RO.DatumNastupu)".
                "WHERE RO.RC = Osoba.RC " .
                "  AND PP.DatumNastupu <= " . $this ->database ->dateToString(time()) .
                "  AND (PP.DatumUkonceni >= " . $this ->database ->dateToString(time()) . " OR PP.DatumUkonceni IS NULL) " .
                "  AND RO.KodTypuRozdeleni = " . $this->typRozdeleniPracMisto->kod .
                "  AND RO.Rozdeleni = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni2($this->typRozdeleniPracMisto->kod)) . "'" .
                "  AND RO.PlatnostOd <= " . $this ->database ->dateToString(time()) .
                "  AND (RO.PlatnostDo >= " . $this ->database ->dateToString(time()) . " OR RO.PlatnostDo IS NULL) ) ";
        }

        if ($_SESSION['typydovednosti']) {
            $typy = '';
            $first = true;

            foreach ($_SESSION['typydovednosti'] as $typDovednosti) {
                if ($typDovednosti != '') {
                    if (!$first) $typy .= ', ';
                    else $first = false;

                    $typy .= "'" . $this->database->addSlashes($typDovednosti) . "'";
                }
            }

            if ($typy != '')
                $this->sqlWhere .= " AND Dovednost.TypDovednosti IN ({$typy})";
        }

        if (isset($_SESSION['seznamdovednosti'])) {

            $dovednostiFiltr = SQLUtils::getINCondition($this->database, 'DovednostOsoby.KodDovednosti', $_SESSION['seznamdovednosti']);
            $dovednostiFiltrUrovne = '';

            foreach ($_SESSION['seznam_urovni_dovednosti'] as $index => $urovenDovednosti) {
                if ($urovenDovednosti) {
                    if ($dovednostiFiltrUrovne) {
                        $dovednostiFiltrUrovne .= ' AND ';
                    }
                    $dovednostiFiltrUrovne .=
                        " (EXISTS(SELECT DovednostOsoby.RC FROM DovednostOsoby
                                  WHERE DovednostOsoby.RC = Osoba.RC
                                    AND DovednostOsoby.KodDovednosti = '" . $this->database->addSlashes($_SESSION['seznamdovednosti'][$index]) . "'
                                    AND DovednostOsoby.KodUrovne >= " .  (int)$urovenDovednosti .
                        ' ))';
                }
            }

            if ($dovednostiFiltr) {
                $this->sqlWhere .= " AND (" . $dovednostiFiltr . ")";
            }
            if ($dovednostiFiltrUrovne) {
                $this->sqlWhere .= " AND (" . $dovednostiFiltrUrovne . ")";
            }
        }
    }

    /**
     * @param OsobaSDovednosti $osoba1
     * @param OsobaSDovednosti $osoba2
     * @return int
     */
    public function porovnejDovednost($osoba1, $osoba2)
    {
        if (count($osoba1->ziskaneDovednosti) == count($osoba2->ziskaneDovednosti)) {
            if ($osoba1->pocetZiskanychDovednosti == $osoba2->pocetZiskanychDovednosti) {
                return StringUtils::compareStrings($osoba1->prijmeni, $osoba2->prijmeni);
            }
            return ($osoba1->pocetZiskanychDovednosti > $osoba2->pocetZiskanychDovednosti) ? -1 : 1;
        }

        return (count($osoba1->ziskaneDovednosti) > count($osoba2->ziskaneDovednosti)) ? -1 : 1;
    }

    public function seradOsoby()
    {
        uasort($this->osoby, array($this, "porovnejDovednost"));
    }



}