<?php
/** UTF8
 * Pavel Rudol, RON Software
 * 28.1.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DBRecordField;
use Ron\FormField;
use Ron\MessageManager;
use Ron\Personalistika\ProvozniDenikController;
use Ron\StringUtils;
use function Ron\ronText;

require_once('ron/formfield.php');
require_once('adsapp/ronportalform.php');
require_once('adsapp/rozdeleni.php');
require_once('ron/controls.php');


class VyberRozdeleniField extends FormField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $typrozdeleni;
    var $rozdeleniPK;
    
    function __construct($typRozdeleni, $prava, $fieldname, $edit, $rozdeleniPK)
    {
        parent::__construct();
        $this->prava = $prava;
        $this->typrozdeleni = $typRozdeleni;

        $this->fieldType = "rozdeleni";
        $this->fieldName = $fieldname;
        $this->sqlFieldName = $fieldname;
        $this->required = true;     //
        $this->editable = $edit;
        $this->maxLength = 30;

        $this->rozdeleniPK = $rozdeleniPK;

        $this->caption = ronText('rozdeleni');
    }


    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws Exception
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {


        if ($canEdit) {
            $params = $database->createParameters();

            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass '.
                ' FROM Rozdeleni'.
                ' WHERE Rozdeleni.KodTypuRozdeleni = '. $params->addParameter($this->typrozdeleni) .
                ' AND (Rozdeleni.DatumZruseni >= ' . $params->addParameter($database->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL)' ;

            //dle nastavení místo podmínky na práva je použito rozdělení z plánování kapacit - kapacity_typrozdeleni
            if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity') && (isset($this->rozdeleniPK))) {
                $params = $database->createParameters();

                $sql =
                    'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass '.
                    'FROM Rozdeleni JOIN PlanovaniKapacit ON (PlanovaniKapacit.Rozdeleni = Rozdeleni.Kod) '.
                    'WHERE Rozdeleni.KodTypuRozdeleni = '. $params->addParameter($this->typrozdeleni) .
                    '  AND (Rozdeleni.DatumZruseni >= ' . $params->addParameter($database->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                    '  AND PlanovaniKapacit.KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI))  .
                    '  AND PlanovaniKapacit.RC = ' . $params->addParameter(User::getRC())  .
                    '  AND PlanovaniKapacit.Datum = ' . $params->addParameter($database->timestampToParam(DateUtils::startOfTheDay(time())));
            }
            else {
                $sql .= '  AND (' . $this->prava->sqlfiltrRozdeleni(time(), time()) . ')';
            }

            $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            MessageManager::addDebugLog('Combo vyber rozdeleni: ' . $sql);

//Combo uzavrena do regionu
//<editor-fold desc="Combo 1">
            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', $rowData[5], $params);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col1';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' OnChange='ComboBoxChange(this, ".'"'.$this->fieldName.'_nazev"'.")'";
            $combo ->render();
//</editor-fold>
//<editor-fold desc="Combo 2">
            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', $rowData[5], $params);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col23';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' OnChange='ComboBoxChange(this, ".'"'.$this->fieldName.'"'.")'";
            $combo ->render();
//</editor-fold>

        } else {
            $rozdeleni =  new Rozdeleni();
            $rozdeleni->load($database, $this->typrozdeleni, $rowData[5]);
            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . $rozdeleni->kod . '" />';
            echo StringUtils::htmlTextConvert($rozdeleni->popis) . "\n";
        }
    } 
    
    function createDBRecordField($connection, $tablename, $insert, $utf8 = false)
    {        
        $field = new DBRecordField($this->fieldName, $this->fieldType, !$this->editable, $this->maxLength);
        $field->value = $this ->getPostValue($utf8);
        
        return $field;
    }
}

// ----------------------------------------------------------------------------
class ProvozniDenikFormular extends RonPortalForm
{
    const XML_EVENT_BEFORE = 'PROVOZNI_DENIK_BEFORE';
    const XML_EVENT_AFTER = 'PROVOZNI_DENIK_AFTER';
    const XML_CONTEXT_INS = 'insert';
    const XML_CONTEXT_UPD = 'update';

    var $prava;
    var $rozdeleniPK;

    /**
     * ProvozniDenikFormular constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->rozdeleniPK = '';

        if (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity')) {
            try {
                $this->rozdeleniPK = ProvozniDenikController::vratRozdeleniPlanovaniKapacit($database, User::getRC(), time());
            } catch (Exception $e) {
                $this->rozdeleniPK = null;
                MessageManager::addInfoMessage($e->getMessage());
            }
        }

        $this->prava = Prava::vratPrava('pr_denik_write');
        Prava::nastavObdobiProPrava(time(),time());
        $this ->htmlAction = '?menuid=provoznidenik&action=denikform';
        $this ->stornoAction = '?menuid=provoznidenik';
        
        if (!$this->formAction != 'edit') {
            $this ->title = ronText('nova_udalost');
        } else {
            $this ->title = ronText('editace_udalosti');
        }

        $this ->sqlTableName = 'ProvozniDenik';
        $this ->dbLogSrcDescription = 'ProvozniDenik';

        $this->sqlSelectPart =
            'ProvozniDenik.ID, ProvozniDenik.DatumCas, ProvozniDenik.Popis, ProvozniDenik.TypUdalosti, '. // 0 - 3
            'ProvozniDenik.KodTypuRozdeleni, ProvozniDenik.KodRozdeleni, ProvozniDenik.RC, ' . $database->castWideMemo('ProvozniDenik.Poznamka') . ', '. // 4 - 7
            'ProvozniDenik.Insert_UserName, ProvozniDenik.Insert_Timestamp, ProvozniDenik.Update_UserName, ProvozniDenik.Update_Timestamp'; //8 - 11    

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_ID, FORM_FIELD_NAME => 'ID', FORM_FIELD_PRIMARY => true, FORM_FIELD_SQL_INDEX => 0 ]);

        $this ->addField(new UserRCField('RC'));

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'DatumCas',  FORM_FIELD_NAME => 'DatumCas', FORM_FIELD_EDIT => false ]);

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Update_Timestamp',  FORM_FIELD_NAME => 'Update_Timestamp', FORM_FIELD_EDIT => true ]);

        $this ->addField(new UserLoginField('Update_UserName', true));

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Insert_Timestamp',  FORM_FIELD_NAME => 'Insert_Timestamp', FORM_FIELD_EDIT => false ]);

        $this ->addField(new UserLoginField('Insert_UserName', false));

        //$this->addField(new TypRozdeleniField('kodTypuRozdeleni', $this->prava->typRozdeleni, true));
        $this->addField(FORM_FIELD_TYPE_CONSTANT)
            ->setSqlFieldName('kodTypuRozdeleni') ->setEditable(true) ->setDataType('int')
            ->setDefaultValue($this->prava->typRozdeleni);

        $this ->addField(new VyberRozdeleniField((RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity') && isset($this->rozdeleniPK)) ? RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_TYP_ROZDELENI) :  $this->prava->typRozdeleni, $this->prava, 'kodRozdeleni', true, $this->rozdeleniPK));

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_SQL_NAME => 'TypUdalosti', FORM_FIELD_NAME => 'TypUdalosti', FORM_FIELD_CAPTION_TEXT => 'popis_typu_udalosti',
            FORM_FIELD_EDIT => 1, FORM_FIELD_REQUIRED => 1, FORM_FIELD_SQL_INDEX => 3,
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM TypUdalostiPD WHERE Aktivni = 1 ORDER BY Popis']);

        $this ->addField([FORM_FIELD_NAME => 'Popis', FORM_FIELD_SQL_NAME => 'Popis',  FORM_FIELD_SQL_INDEX => 2, FORM_FIELD_CAPTION_TEXT => 'Popis',
            FORM_FIELD_TYPE => 'edit', 'otherHtmlParams' => 'id="pd_f_popis"']);

        $this->addField( [ FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO,   FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'text_cinnosti',
            FORM_FIELD_EDIT => 1, FORM_FIELD_SQL_INDEX => 7]);
    }

    /**
     *
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeInsertRecord($rec)
    {
        if (!($this->prava->jsouNejakePrava()
            && (($this->prava->lzeZobrazitRC($rec['RC'], time(), time()))
            && (($this->prava->jePravoNaRozdeleni($this->prava->typRozdeleni,$rec['KodRozdeleni'])) || ($rec['KodRozdeleni'] == $this->rozdeleniPK))
               )
        ))
        {
            //throw new \Exception(\Ron\ronText('Nemate_pravo'));
            return false;
        }
        
        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_INS)) {
            return false;
        }

        return true;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeUpdateRecord($rec)
    {   
        if (!($this->prava->priznak
            && (($this->prava->priznak2 == 0) || ($rec['DatumCas']->originalvalue > DateUtils::incMinute(time(),- $this->prava->priznak2)))
            && (($this->prava->lzeZobrazitRC($rec['RC']->originalValue, time(), time()))
            && (($this->prava->jePravoNaRozdeleni($this->prava->typRozdeleni,$rec['KodRozdeleni']->originalValue))) || ($rec['KodRozdeleni']->originalValue == $this->rozdeleniPK))
                ))
        {
            throw new Exception(ronText('Nemate_pravo'));
        }
        
        if (!$rec->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_UPD)) {
            return false;
        }
        return true;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function afterInsertRecord($rec) {
        $_SESSION['typUdalostiPD'] = $rec['TypUdalosti'];

        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_INS)) {
            return false;
        }
        return true;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function afterUpdateRecord( $rec) {
        $_SESSION['typUdalostiPD'] = $rec['TypUdalosti'];

        if (!$rec->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_UPD)) {
            return false;
        }
        return true;
    }

    /**
     * @param array $dbrow
     * @return bool
     * @throws Exception
     */
    function canEditRecord( $dbrow)
    {
        if (!(($this->prava->priznak && (($this->prava->priznak2 == 0) || ($dbrow[1] > DateUtils::incMinute(time(),- $this->prava->priznak2)))
            && (($this->prava->lzeZobrazitRC($dbrow[6], time(), time())) && (($this->prava->jePravoNaRozdeleni($this->prava->typRozdeleni,$dbrow[5])) || ($dbrow[5] == $this->rozdeleniPK)))
                ))) {
            return false;
        }
        return true;
    }
}