<?php
/**
 * stanislav.elbl, 16.06.2020
 */

namespace Ron\Personalistika;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\RonException;
use Ron\RonPortal\DokumentyBrowser;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;
use Ron\StringUtils;

require_once 'adsapp/b_dokumenty.php';


/**
 * Class DokumentyNahlaseneZmenyBrowser
 * @package Ron\RonPortal
 */
class DokumentyNahlaseneZmenyBrowser extends DokumentyBrowser
{

    /**
     * DokumentyNahlaseneZmenyBrowser constructor.
     * @param Database $database
     * @param $idZmeny
     */
    function __construct($database, $idZmeny)
    {
        parent::__construct($database,
            "?menuid=nahlasenezmeny&action=dokumentdownload&id=$idZmeny",
            1, "?menuid=nahlasenezmeny&action=dokumentdelete&id=$idZmeny");

        $this->sqlFromPart =
            'Dokument, TypDokumentu, DokumentZmeny';

        $params = $database->createParameters();
        $where =
            'Dokument.Id = DokumentZmeny.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            ' AND DokumentZmeny.IDZmeny = ' . $params->addParameter($idZmeny);

        $this ->sqlWherePart = new SqlStatement($where, $params);
    }
}




class HlaseniZmenyForm extends BaseViewObject
{
    /** @var Database */
    var $database;
    /** @var NahlasenaZmena */
    var $nahlasenaZmena;

    // hodnoty potrebne pro jednotlive typy zmen
    /** @var mixed[]    asociativni pole s hodnotami aktualne platneho uctu osoby */
    var $aktualniUcetOsoby;

    /** @var DriveNahlaseneZmenyBrowser */
    var $historieNahlasenychZmenBrowser;

    /** @var DokumentyNahlaseneZmenyBrowser */
    var $dokumentBrowser;

    /** @var mixed[] */
    var $aktualniHodnoty;

    /** @var bool       Priznak, ze uzivatel muze zmenu editovat */
    var $lzeEditovat;
    /** @var bool       Priznak, ze uzivatel muze zmenu schvalit */
    var $lzeSchvalit;

    /** @var bool   Priznak, ze se jedna jen o zobrazeni detailu */
    var $detail;

    /**
     * HlaseniZmenyForm constructor.
     * @param Database $database
     * @param bool $detail
     */
    public function __construct($database, $detail = false)
    {
        parent::__construct('person/hlasenizmeny_view.php');

        $this->database = $database;
        $this->detail = $detail;
    }


    /**
     * @param $rc
     * @return array|bool
     * @throws \Ron\DB\DBException
     */
    private function nactiAktualniUcetOsoby($rc)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT BankovniUcet.CisloUctu, VyplaceniMzdy.PlatnostOd, BankovniUcet.SerioveCislo' .
            '  FROM BankovniUcet JOIN VyplaceniMzdy ON (BankovniUcet.SerioveCislo = VyplaceniMzdy.SerioveCisloUctuVyplaty)' .
            ' WHERE VyplaceniMzdy.RC = ' . $params->addParameter($rc) .
            ' ORDER BY VyplaceniMzdy.PlatnostOd DESC';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        return [
            'cisloUctu' => $row ? $row[0] : null,
            'platnostOd' => $row ? $this->database->stringToTimestamp($row[1]) : null,
            'serioveCislo' => $row ? $row[2] : null
        ];
    }


    /**
     * @throws \Ron\DB\DBException
     */
    public function nactiOsobniUdaje()
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Titul, TitulZa, Jmeno, Prijmeni, eMail, Telefon, Mobil' .
            ' FROM Osoba INNER JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC )' .
            ' WHERE PracovniPomer.ID = ' . $params->addParameter($this->nahlasenaZmena->idPPOsobyZmeny);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        $this->nahlasenaZmena->nacteneHodnoty = [
            'Titul' => $row[0],
            'TitulZa' => $row[1],
            'Jmeno' => $row[2],
            'Prijmeni' => $row[3],
            'Email' => $row[4],
            'Telefon' => $row[5],
            'Mobil' => $row[6]
        ];
    }


    /**
     * Nacte prukaz, ktery se ma editovat
     * @param string $cisloPrukazu
     * @param int $typPrukazu
     * @throws \Exception
     */
    public function nactiPrukazOsoby($cisloPrukazu, $typPrukazu)
    {
        if (!isset($this->nahlasenaZmena->rcOsobyZmeny)) {
            throw new \Exception('Chyba v metode HlaseniZmenyForm::nactiPrukazOsoby - neni nacten pracovni pomer!');
        }

        $params = $this->database->createParameters();

        $sql =
            'SELECT Prukaz.TypPrukazu, Prukaz.CisloPrukazu, Stat.Kod, Prukaz.PlatnostOd, Prukaz.PlatnostDo, ' . $this->database->castWideMemo('Poznamka') .
            ' FROM Prukaz ' .
            ' LEFT JOIN TypPrukazu TP ON (Prukaz.TypPrukazu = TP.Kod)' .
            ' LEFT JOIN Stat ON (Prukaz.VydalStat = Stat.Kod)' .
            ' WHERE Prukaz.RC = ' . $params->addParameter($this->nahlasenaZmena->rcOsobyZmeny) .
            '  AND Prukaz.CisloPrukazu = ' . $params->addParameter($cisloPrukazu) .
            '  AND Prukaz.TypPrukazu = ' . $params->addParameter($typPrukazu);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if ($row) {
            $this->nahlasenaZmena->nacteneHodnoty = [
                'TypPrukazu' => (int)$row[0],
                'CisloPrukazu' => $row[1],
                'Stat' => $row[2],
                'PlatnostOd' => $this->database->stringToTimestamp($row[3]),
                'PlatnostDo' => $this->database->stringToTimestamp($row[4]),
                'vlastnipoznamka' => $row[5]
            ];
        } else {
            $this->nahlasenaZmena->nacteneHodnoty = [];
        }
    }


    /**
     * Nacte zaznam rodinneho prislusnika pro editaci
     * @param $id
     * @return array|bool
     * @throws \Ron\DB\DBException
     */
    public function nactiRodinnehoPrislusnika($id)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.RodneCisloRP, RodinnyPrislusnik.Titul, RodinnyPrislusnik.DatumNarozeni,' .
            ' S_RodinnyVztah.Kod, ClenoveRodiny.PartnerDatumOd, ClenoveRodiny.PartnerDatumDo, S_ZdravotniStav.Popis, RodinnyPrislusnik.DrzitelPrukazuZTTP,' .
            ' RodinnyPrislusnik.DatumPriznaniZTTP, ClenoveRodiny.VyzivovanaOsoba, ClenoveRodiny.StejnaAdresa, RodinnyPrislusnik.KontaktniOsoba,' .
            ' RodinnyPrislusnik.Telefon, RodinnyPrislusnik.Email, RodinnyPrislusnik.RC_RodinnehoPrislusnika' .
            ' FROM ClenoveRodiny' .
            ' LEFT JOIN RodinnyPrislusnik ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika)' .
            ' LEFT JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = RodinnyPrislusnik.ZdravotniStav)' .
            ' LEFT JOIN S_RodinnyVztah ON (S_RodinnyVztah.Kod = ClenoveRodiny.RodinnyVztah)' .
            ' WHERE ClenoveRodiny.RC_RodinnehoPrislusnika = ' . $params->addParameter($id);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if ($row) {
            $this->nahlasenaZmena->nacteneHodnoty = [
                'Prijmeni' => $row[0],
                'Jmeno' => $row[1],
                'RC_RodinnehoPrislusnika' => $row[2],
                'Titul' => $row[3],
                'DatumNarozeni' => $this->database->stringToTimestamp($row[4]),
                'RodinnyVztah' => $row[5],
                'PartnerDatumOd' => $this->database->stringToTimestamp($row[6]),
                'PartnerDatumDo' => $this->database->stringToTimestamp($row[7]),
                'ZdravotniStav' => $row[8],
                'DrzitelPrukazuZTTP' => $row[9],
                'DatumPriznaniZTTP' => $this->database->stringToTimestamp($row[10]),
                'VyzivovanaOsoba' => $row[11],
                'StejnaAdresa' => $row[12],
                'KontaktniOsoba' => $row[13],
                'Telefon' => $row[14],
                'Email' => $row[15],
                'ID' => $row[16]
            ];
        } else {
            $this->nahlasenaZmena->nacteneHodnoty = [];
        }
    }


    /**
     * Nacte aktualni hodnoty uctu osoby
     * @throws \Ron\DB\DBException
     */
    public function nactiUcetOsoby()
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT BankovniUcet.JmenoUctu, S_TypBankovnihoUctu.Kod, BankovniUcet.CisloUctu, Banka.kod,' . // 0..3
            '  BankovniUcet.ZahranicniPlatba, BankovniUcet.IBAN, Mena.KodMeny_ISO, BankovniUcet.SpecifickySymbol,' . // 4..7
            '  BankovniUcet.KonstantniSymbol, BankovniUcet.VariabilniSymbol, VyplaceniMzdy.PlatnostOd, VyplaceniMzdy.PlatnostDo,' . // 8..11
            '  BankovniUcet.SerioveCislo, S_TypBankovnihoUctu.Popis' . // 12..13
            ' FROM BankovniUcet JOIN VyplaceniMzdy ON (BankovniUcet.SerioveCislo = VyplaceniMzdy.SerioveCisloUctuVyplaty)' .
            ' LEFT JOIN S_TypBankovnihoUctu ON (S_TypBankovnihoUctu.Kod = BankovniUcet.TypUctu)' .
            ' LEFT JOIN Banka ON (Banka.kod = BankovniUcet.KodBanky)' .
            ' LEFT JOIN Mena ON (Mena.KodMeny_ISO = BankovniUcet.KodMeny_ISO)' .
            ' LEFT JOIN KonstantniSymbol ON (BankovniUcet.KonstantniSymbol = KonstantniSymbol.kod)' .
            ' WHERE TypUctu = 1 AND BankovniUcet.RC = ' . $params->addParameter($this->nahlasenaZmena->rcOsobyZmeny) .
            ' ORDER BY VyplaceniMzdy.PlatnostOd DESC';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

//        E:JmenoUctu:Jmeno_uctu;S_TypBankovnihoUctu:TypUctu:Typ_uctu;E:CisloUctu:Cislo_uctu;Banka:Banka:Banka;
//        C:ZahranicniPlatba:Zahranicni_platba;IBAN:IBAN:IBAN;Mena:KodMeny_ISO:Kod_meny;E:SpecifickySymbol:specificky_symbol;
//        KonstantniSymbol:KonstantniSymbol:konstantni_symbol;E:VariabilniSymbol:Variabilni_symbol;
//        D:PlatnostOd:Platnost_od;D:PlatnostDo:Platnost_do;soubor:soubor:Pripojit_dokument
        if ($row) {
            $this->nahlasenaZmena->nacteneHodnoty = [
                'JmenoUctu' => $row[0],
                'TypUctu' => $row[1],
                'CisloUctu' => $row[2],
                'Banka' => $row[3],
                'ZahranicniPlatba' => $row[4],
                'IBAN' => $row[5],
                'KodMeny_ISO' => $row[6],
                'SpecifickySymbol' => $row[7],
                'KonstantniSymbol' => $row[8],
                'VariabilniSymbol' => $row[9],
                'PlatnostOd' => $this->database->stringToTimestamp($row[10]),
                'PlatnostDo' => $this->database->stringToTimestamp($row[11]),
                'SerioveCislo' => $row[12], // nezobrazuje se
                'PopisTypuUctu' => $row[13] // nezobrazuje se
            ];
        }
    }


    private function overClenaRodiny()
    {
        if (!$this ->nahlasenaZmena ->nacteneHodnoty['RC_RodinnehoPrislusnika']) {
            // neni zadano RC => neni co overovat
            return;
        }

        // zmena jeste neni zapsana - zjisti, jestli uz stejny clen rodiny neexistuje u jineho zamestnance
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Osoba.Jmeno, Osoba.Prijmeni' .
            ' FROM RodinnyPrislusnik, ClenoveRodiny, Osoba' .
            ' WHERE RodinnyPrislusnik.RodneCisloRP = ' . $params ->addParameter($this ->nahlasenaZmena ->nacteneHodnoty['RC_RodinnehoPrislusnika']) .
            ' AND ClenoveRodiny.RC_RodinnehoPrislusnika = RodinnyPrislusnik.RC_RodinnehoPrislusnika' .
            ' AND Osoba.RC = ClenoveRodiny.RC',
            $params
        );

        $message = '';
        while ($row = $cursor ->fetch()) {
            $message .= ($message ? ', ' : '') . StringUtils::htmlTextConvert($row[0] . ' ' . $row[1]);
        };

        $cursor ->close();

        if ($message) {
            $this ->nahlasenaZmena ->messages[] = sprintf(\Ron\ronText('Rodinny_prislusnik_uz_existuje_u_zamestnancu'), $message);
        }

    }


    private function loadMessages()
    {
        switch ($this ->nahlasenaZmena ->typZmeny) {
            case 'novyclenrodiny' :
                $this ->overClenaRodiny();
                break;
        }
    }


    private function loadData()
    {
        if (!$this->nahlasenaZmena->povolitVIntrawebu) {
            throw new \Exception(\Ron\ronText('Zmena_nepovolena'));
        }

        // musi mit pravo prohlizet a/nebo editovat a/nebo schvalovat zadanou zmenu
        NahlasenaZmena::zkontrolujPravoNaZmenu($this->nahlasenaZmena->typZmeny, $this->nahlasenaZmena->rcOsobyZmeny);

        $this->lzeEditovat = NahlasenaZmena::jePravoNaZmenu($this->nahlasenaZmena->typZmeny, $this->nahlasenaZmena->rcOsobyZmeny, true);
        $this->lzeSchvalit = NahlasenaZmena::jePravoNaZmenu($this->nahlasenaZmena->typZmeny, $this->nahlasenaZmena->rcOsobyZmeny, false, true);

        if ($this->nahlasenaZmena->typZmeny == 'ucetosoby') {
            $this->aktualniUcetOsoby = $this->nactiAktualniUcetOsoby($this->nahlasenaZmena->rcOsobyZmeny);
        }

        $this->historieNahlasenychZmenBrowser = new DriveNahlaseneZmenyBrowser($this->database,
            $this->nahlasenaZmena->idPPOsobyZmeny, $this->nahlasenaZmena->typZmeny, $this->nahlasenaZmena->id,
            'osobniudaje');
        $this->historieNahlasenychZmenBrowser->load();

        if ($this->nahlasenaZmena->id) {
            // dokumenty zobrazi jen pri editaci existujici zmeny
            $this->dokumentBrowser = new DokumentyNahlaseneZmenyBrowser($this->database, (int)$this->nahlasenaZmena->id);
            $this->dokumentBrowser ->showMessages = false;
            $this->dokumentBrowser->load();
        }

        $this->aktualniHodnoty = NahlasenaZmena::nactiAktualniHodnoty($this->database,
            $this->nahlasenaZmena->rcOsobyZmeny, $this->nahlasenaZmena->typZmeny);

        $this ->loadMessages();
    }


    /**
     * Nacte existujici nahlasenou zmenu
     * @param int $idZmeny
     * @throws \Exception
     */
    public function load($idZmeny)
    {
        require_once 'person/nahlasenazmena.php';
        $this->nahlasenaZmena = NahlasenaZmena::nactiNahlasenouZmenu($this->database, $idZmeny);

        $this->loadData();
    }


    /**
     * @param Database $database
     * @param string $typZmeny
     * @return NahlasenaZmena
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiNastaveniZmeny($database, $typZmeny)
    {
        require_once 'person/nahlasenazmena.php';
        $nahlasenaZmena = new NahlasenaZmena();

        $params = $database->createParameters();
        $sql =
            'SELECT PovolitVIntrawebu, Popis, ' . $database->castWideMemo('Nastaveni') . ', Kod ' .
            ' FROM TypNahlaseneZmeny ' .
            ' WHERE Kod = ' . $params->addParameter($typZmeny);
        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor->close();

        if (!$row) {
            throw new \Exception('Chyba pri hlaseni zmeny - neznamy typ zmeny');
        }

        $nahlasenaZmena->povolitVIntrawebu = (int)$row[0];
        $nahlasenaZmena->popisTypuZmeny = $row[1];
        $nahlasenaZmena->nastaveniZmeny = $row[2];
        $nahlasenaZmena->typZmeny = $row[3];

        return $nahlasenaZmena;
    }


    /**
     * Nove nahlaseni zmeny - nacte nastaveni z typu nahlasene zmeny a zadanych hodnot
     * @param PracovniPomer $pracovniPomer
     * @param string $typZmeny
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public function novaZmena($pracovniPomer, $typZmeny)
    {
        $this->nahlasenaZmena = self::nactiNastaveniZmeny($this->database, $typZmeny);

        $this->nahlasenaZmena->idPPOsobyZmeny = $pracovniPomer->idPP;
        $this->nahlasenaZmena->rcOsobyZmeny = $pracovniPomer->rc;
        $this->nahlasenaZmena->celeJmenoOsobyZmeny =
            ($pracovniPomer->titul ? $pracovniPomer->titul . ' ' : '') .
            ($pracovniPomer->jmeno ? $pracovniPomer->jmeno . ' ' : '') .
            ($pracovniPomer->prijmeni ?: '');

        $this->loadData();
    }


    /**
     * Podle nastaveni hlasene zmeny a udaju zadanych uzivatelem doplni hodnotu
     * @return string
     * @throws \Exception
     */
    public function sestavHodnotuHlaseni()
    {
        $this ->nahlasenaZmena->poznamka = $_POST['poznamka'];

        $polozky = explode(';', $this ->nahlasenaZmena->nastaveniZmeny);

        $allEmpty = true;

        // nacte vsechny odeslane hodnoty
        foreach ($polozky as $polozkaZmeny) {
            //list($typPolozky, $kodPolozky, $popisPolozky, $priznak) = explode(':', $polozkaZmeny);
            $poma = explode(':', $polozkaZmeny);
            $typPolozky = $poma[0];
            $kodPolozky = $poma[1];
            $priznak = isset($poma[3]) ? $poma[3] : null;

            if (strtolower($priznak) == 'ro') {
                // read only polozky nenacita
            } else if (strtolower($typPolozky) == 'd') {
                // datumova polozka - prozatim nacte jen jako text
                $this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky . '#STR'] = trim($_POST[$kodPolozky]);
            } else if (strtolower($typPolozky) == 'soubor') {
                // zaznamena jmeno prilozeneho souboru (jen pro info)
                if (is_uploaded_file($_FILES[$kodPolozky]['tmp_name'])) {
                    $this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky] = $_FILES[$kodPolozky]['name'];
                }
            } else {
                $this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky] = $_POST[$kodPolozky];
            }
        }

        $hodnoty = '';

        // sestavi vyslednou textovou hodnotu a zkontroluje, jestli jsou zadany vsechny povinne polozky
        foreach ($polozky as $polozkaZmeny) {
            //list($typPolozky, $kodPolozky, $popisPolozky, $priznak) = explode(':', $polozkaZmeny);
            $poma = explode(':', $polozkaZmeny);
            $typPolozky = $poma[0];
            $kodPolozky = $poma[1];
            $popisPolozky = $poma[2];
            $priznak = isset($poma[3]) ? $poma[3] : null;

            $hodnota = isset($_POST[$kodPolozky]) ? trim($_POST[$kodPolozky]) : '';

            //Je povinne pole prazdne?
            if (strtolower($priznak) == 'pr' || strtolower($priznak) == 'po') {
                //if (filter_input(INPUT_POST, $kodPolozky, FILTER_SANITIZE_SPECIAL_CHARS) == ''){
                if ($hodnota == '') {
                    throw new RonException(\Ron\ronText('Chybi_povinna_hodnota') . ' (' . \Ron\ronText($popisPolozky) . ')', $kodPolozky);
                }
            }

            // kontrola spravne zadaneho datumu
            if (strtoupper($typPolozky) == 'D') {
                try {
                    $this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky] = $this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky . '#STR'] == ''
                        ? null
                        : DateUtils::dateStringToTimestamp($this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky . '#STR']);
                } catch (\Exception $e) {
                    throw new RonException($e->getMessage(), $kodPolozky, null, 0, $e);
                }
            }

            // osetrit zalomeni radku (napr. v poznamce)
            $hodnoty .=
                $kodPolozky . '=' .
                (isset($_POST[$kodPolozky])
                    ? str_replace("\r", '\r', str_replace("\n", '\n', $_POST[$kodPolozky]))
                    : ''
                ) .
                "\n";

            //Odchyceni prazdneho formulare
            if ($hodnota != '') {
                $allEmpty = false;
            }
        }

        $this ->nahlasenaZmena->hodnoty = $hodnoty;

        if ($allEmpty) {
            throw new \Exception(\Ron\ronText('formular_neni_vyplnen'));
        }
    }

    // TODO: doplnit kontrolu platnostOd - nesmi byt mensi nez posledni platnost od

    private function nactiHodnotyProNovouZmenu()
    {
        $typZmeny = $_POST['sys_typzmeny'];

        require_once 'adsapp/ppomer.php';
        $pracovniPomer = PracovniPomer::nactiPP($this->database, (int)$_POST['idpp']);

        require_once 'person/nahlasenazmena.php';
        NahlasenaZmena::zkontrolujPravoNaZmenu($typZmeny, $pracovniPomer->rc, true);

        $this->novaZmena($pracovniPomer, $typZmeny);
    }


    private function nactiHodnotyProEditovanouZmenu()
    {
        require_once 'person/nahlasenazmena.php';
        $this->nahlasenaZmena = NahlasenaZmena::nactiNahlasenouZmenu($this->database, (int)$_POST['idZmeny']);

        NahlasenaZmena::zkontrolujPravoNaZmenu($this ->nahlasenaZmena ->typZmeny, $this ->nahlasenaZmena ->rcOsobyZmeny, true, isset($_POST['potvrdit_zmenu']) || isset($_POST['odmitnout_zmenu']));

        $this->loadData();
    }


    /**
     * @throws \Exception
     */
    private function platnostOdUctuOsoby()
    {
        $novaPlatnostOdUctuOsoby = DateUtils::firstDayOfMonth($this ->nahlasenaZmena ->nacteneHodnoty['PlatnostOd']);

        if ($novaPlatnostOdUctuOsoby < time()) {
            throw new \Exception(\Ron\ronText('ucet_upozorneni'));
        }

        $params = $this->database->createParameters();
        $cursor = $this->database->getSqlResult(
            'SELECT PlatnostOd FROM VyplaceniMzdy' .
            ' WHERE PlatnostDo IS NULL' .
            ' AND RC = ' . $params->addParameter($this ->nahlasenaZmena ->rcOsobyZmeny),
            $params);
        $row = $cursor->fetch();
        $cursor ->close();

        $aktualniPlatnostOdUctuOsoby = $this->database->stringToTimestamp($row[0]);

        // Pokud je platnost mensi nez platnost aktualni hodnoty, zahlasit chybu
        if ($aktualniPlatnostOdUctuOsoby >= $novaPlatnostOdUctuOsoby) {
            throw new RonException(
                \Ron\ronText('chyba_platnosti_od_nejdrive_od') . ' ' . date('j.n.Y', DateUtils::incDayStart($aktualniPlatnostOdUctuOsoby)),
                'PlatnostOd');
        }
    }



    /**
     * @param $trvale
     * @throws \Exception
     */
    private function platnostOdAdresy($trvale)
    {
        $novaPlatnostOdBydliste = $this ->nahlasenaZmena ->nacteneHodnoty['PlatnostOd'];

        $params = $this->database->createParameters();
        $sql =
            'SELECT PlatnostOd FROM AdresaOsoby' .
            ' WHERE TrvaleBydliste = ' . $params ->addParameter($trvale) .
            ' AND PlatnostDo IS NULL' .
            ' AND RC = ' . $params->addParameter($this ->nahlasenaZmena ->rcOsobyZmeny);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        $aktualniPlatnostOdBydliste = $this->database->stringToTimestamp($row[0]);

        // Pokud je platnost mensi nez platnost aktualni hodnoty, zahlasit chybu
        if ($aktualniPlatnostOdBydliste >= $novaPlatnostOdBydliste){
            throw new RonException(
                \Ron\ronText('chyba_platnosti_od_bydliste') . ' ' . date('j.n.Y', DateUtils::incDayStart($aktualniPlatnostOdBydliste)),
                'PlatnostOd');
        }
    }


    /**
     * @throws \Exception
     */
    public function kontrolaPlatnostiOd()
    {
        switch ($this ->nahlasenaZmena ->typZmeny){
            case 'ucetosoby':
                $this->platnostOdUctuOsoby();
                break;

            case 'trvale':
                $this->platnostOdAdresy(1);
                break;

            case 'prechodne':
                $this->platnostOdAdresy(0);
                break;
        }
    }

    public function zkontrolujZadaneHodnoty()
    {
        $this->kontrolaPlatnostiOd();
        //$this ->nahlasenaZmena->nacteneHodnoty[$kodPolozky]
    }

    /**
     * Nacte hodnoty zadane uzivatelem.
     * @throws RonException
     * @throws \Exception
     * @throws \Ron\RonPortal\NeniPravoException
     */
    public function nactiZadaneHodnoty()
    {
        if (isset($_POST['idZmeny']) && (int)$_POST['idZmeny']) {
            $this ->nactiHodnotyProEditovanouZmenu();
        } else {
            $this ->nactiHodnotyProNovouZmenu();
        }

        $this ->sestavHodnotuHlaseni();
        $this ->zkontrolujZadaneHodnoty();
    }


    public function ulozDokumenty()
    {
        $polozky = explode(';', $this ->nahlasenaZmena->nastaveniZmeny);

        // nacte vsechny odeslane hodnoty
        foreach ($polozky as $polozkaZmeny) {
            //list($typPolozky, $kodPolozky, $popisPolozky, $priznak) = explode(':', $polozkaZmeny);
            $poma = explode(':', $polozkaZmeny);
            $typPolozky = $poma[0];
            $kodPolozky = $poma[1];
            if (strtolower($typPolozky) == 'soubor') {
                // zaznamena jmeno prilozeneho souboru (jen pro info)
                if (is_uploaded_file($_FILES[$kodPolozky]['tmp_name'])) {
                    $popis = \Ron\ronText("dokument_nahlasene_zmeny") . ' ' . $this ->nahlasenaZmena ->id . ' (' . date('j.n.Y', time()) . ')';

                    require_once 'adsapp/dokumentyutils.php';
                    $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'soubor', RonPortalSettings::getIntValue('hlaseni_zmeny_typdokumentu'), $popis, $this);

                    try {
                        $rec = new RonPortalDBRecord($this->database, 'DokumentZmeny', $this);
                        $rec->addField('id', 'int', true, 0, $idDokumentu);
                        $rec->addField('IDZmeny', 'int', true, 0, (int)$this ->nahlasenaZmena ->id);

                        $rec->insertRecord();
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage($e);
                    }
                }
            }
        }
    }


    public function zapisZaznamZmeny()
    {
        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'NahlasenaZmena', $this);

        $rec->addStringField('TypZmeny', true, 20, $this->nahlasenaZmena->typZmeny);
        $rec->addStringField('RC', true, 15, $this->nahlasenaZmena->rcOsobyZmeny);
        $rec->addIntField('Stav', true, 0);

        $f = $rec->addTimestampField('DatumCas', false, time());
        $f ->setUpdateLog(true); // zmeni se jen v pripade zmeny jineho pole
        $f = $rec->addStringField('Nahlasil', false, 15, User::getRC());
        $f ->setUpdateLog(true);

        $rec->addField('Hodnoty', 'memo', false, 0, $this->nahlasenaZmena->hodnoty);
        $rec->addField('Poznamka', 'memo', false, 0, $this->nahlasenaZmena->poznamka);

        if (!$this->nahlasenaZmena->id) {
            require_once('ron/sqlutils.php');
            $idZmeny = SQLUtils::newSerialNumber($this->database, 'NahlasenaZmena', 'Id');
            $rec->addIntField('Id', true, $idZmeny);

            $rec->insertRecord();

            $this->nahlasenaZmena->id = $idZmeny;
        } else {
            $rec->addIntField('Id', true);

            $params = $this->database->createParameters();
            $rec->setWhere('Id = ' . $params->addParameter($this->nahlasenaZmena->id), $params);

            if (!$rec->loadRecord()) {
                throw new \Exception('Chyba pri aktualizaci zaznamu: editovana nahlasena zmena nebyla nalezena!');
            }

            $rec->updateRecord();
        }
    }


    public function zapisZmenu()
    {
        $this->zapisZaznamZmeny();
        $this->ulozDokumenty();
    }
}