<?php
/**
 * UTF-8
 * stanislav.elbl, 13.11.2015
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PredavaciProcesyUtils;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\RonPortal\XML\XMLFormViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;


class HodnoceniController extends  HttpController
{
    const ULOHA_PREPOCET = 0;

    /** @var Database */
    var $database;

    public $pravaZobraxit;

    /**
     * HodnoceniController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;

        $this->pravaZobraxit = Prava::vratPrava('hodnoceni');
        Prava::nastavObdobiProPrava(DateUtils::today(), DateUtils::today());
    }


    /**
     * @return BaseViewObject|HodnoceniBrowser|XMLFormViewObject|string
     * @throws Exception
     */
    function seznamHodnoceni()
    {
        if (RonPortalSettings::getMapovaniXML('b_hodnoceni')) {
            require_once('xml/xmlform.php');
            return XMLFormViewObject::zobrazXmlSablonu($this ->database, RonPortalSettings::getMapovaniXML('b_hodnoceni'));
//            $viewObject = new XMLFormViewObject($this->database);
//            return $viewObject ->processXml(RonPortalSettings::getMapovaniXML('b_hodnoceni'));
        } else {
            require_once('person/b_hodnoceni.php');

            $b = new HodnoceniBrowser($this->database);
            $b->load();

            return $b;
        }
    }


    /**
     * Zkontroluje, jestli ma uzivatel pravo na tento dotaznik
     * - jestli je pro zamestnance z jeho rozdeleni nebo z podrizeneho
     * - jestli je na jeho urovni predavani v predavacim procesu (podle pracovnich mist)
     * - jestli ma pravo na hodnoceneho zamestnance (typ prava hodnoceni) - doplneno 9/2018 - jen prohlizeni = 1
     * - vraci 0 nebo false - nema pravo,  1 - muze prohlizet,  2 - muze editovat
     * @param Database $database
     * @param int  $id
     * @param bool $editace  Pokud je $editace = true a zjisti, ze nemuze editovat, tak rovnou vraci 0 a nekontroluje, jestli lze i cist
     * @return int
     * @throws Exception
     */
    public static function vratPravoNaHodnoceni($database, $id, $editace)
    {
        $params = $database->createParameters();
        // zjisti stav, uroven predavani a rozdeleni osoby
        $sql =
            'SELECT PripraveneTemaHodnoceni.Stav, PripraveneTemaHodnoceni.HodnotilRC, ' . // 0, 1
            '    PripraveneTemaHodnoceni.HodnocenyRC, HodnoceniOsoby.UrovenPredavani, ' . // 2, 3
            '    VysledekHodnoceni.PocatekObdobi, VysledekHodnoceni.KonecObdobi, ' . // 4, 5
            '    HodnoceniOsoby.KodProcesu, HodnoceniOsoby.PocatecniUroven, PoradiPredavani.Vlastni, PripraveneTemaHodnoceni.IndividualniPoradi, ' . // 6..9
            '    PoradiPredavani.NekontrolovatHierarchii, VysledekHodnoceni.HodnocenyRC, VysledekHodnoceni.HodnocenyOsobniCislo, VysledekHodnoceni.HodnocenyDatumNastupu, ' . // 10..13
            '    (SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = VysledekHodnoceni.HodnocenyRC AND PracovniPomer.OsobniCislo = VysledekHodnoceni.HodnocenyOsobniCislo AND PracovniPomer.DatumNastupu = VysledekHodnoceni.HodnocenyDatumNastupu)' . // 14
            ' FROM PripraveneTemaHodnoceni, HodnoceniOsoby, VysledekHodnoceni, PoradiPredavani ' .
            ' WHERE PripraveneTemaHodnoceni.Id = ' . $params->addParameter($id) .
            '   AND HodnoceniOsoby.Id = ' . $params->addParameter($id) .
            '   AND HodnoceniOsoby.IDVysledku = VysledekHodnoceni.Id ' .
            '   AND PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu ' .
            '   AND PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven ' .
            '   AND PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani ';

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        if (!$row) {
            throw new Exception(StringUtils::fromUTF8("Chyba - zadaný dotazník (číslo $id) nebyl nalezen nebo k němu neexistuje záznam VysledekHodnoceni nebo PoradiPredavani!"));
        }

        $result = 2; // lze editovat
        $pravoZobrazitHodnoceneho = false;

        //doplneno 9/2018 - pokud ma uzivatel pravo na hodnocenou osobu, muze zobrazit needitovatelne hodnoceni
        if (Prava::vratPrava('hodnoceni')->lzeZobrazitPP($row[14], $database->stringToTimestamp($row[4]), $database->stringToTimestamp($row[5]))) $pravoZobrazitHodnoceneho = true;

        // pokud se ma editovat, zkontroluje, jestli neni uzavren
        if ($row[0] == 2 || $row[0] == 3) {
            // dotaznik je uzavren
            if ($editace) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - dotazník nelze editovat, protože je již uzavřen!'));
            }

            $result = 1; // nelze editovat - jeste bude kontrolovat jesli lze cist
        }

        if ($row[1] == User::getRC()) return $result;
            // dotaznik vyplnil aktualni uzivatel, takze ma na jeho zobrazeni pravo
            // uz neni potreba kontrolovat stromovou strukturu

        //15.6.2015 RON-PR pokud ma dotaznik individualni poradi predavani a k dotazniku neni prirazen aktualne prihlaseny uzivatel, na dotaznik neni pravo
        //9/2018 pristup je zamitnut jen pro editaci - zobrazit jej muze
        if ($editace && ($row[9] == 1  &&  $row[1] != User::getRC())) {
            return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolený dotazník nemáte právo (dotazník má jiného individuálního hodnotitele)!'));
        }

        // TODO: zmenit poradi kontrol - nejdrive $row[2] == S_GetUserRC()   a teprve potom $row[8] ... ?

        if ($row[8]) {// uroven je nastavena na "vlastni" => uzivatel hodnoti sam sebe
            if ($row[2] == User::getRC()) return $result; // je to vlastni - muze ho zobrazit i editovat

            if ($editace) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na editaci zvoleného dotazníku nemáte právo (uživatel hodnotí sám sebe)!'));
            }

            $result = 1; // nelze editovat - jeste bude kontrolovat jesli lze cist
            // OPRAVIT - uzivatel muze videt dotazniky kolegu, pokud zna link
            //      ale pritom je nutne zajistit, aby mohl nadrizeny videt dotazniky podrizenych, ktere jsou teprve na "vlastni urovni"
        }

        // Zmena 7.4.2014 - muze editovat sve vlastni hodnoceni, pokud to umoznuje predavaci proces
        // tj. v pripade editace pokracuje dale
        /*
        if ( $row[2] == S_GetUserRC() )
        {
          // je to hodnoceni aktualniho uzivatele
          if ( $editace )
          {
             // nemuze editovat sve vlastni hodnoceni
             $errormessage = "Chyba - na editaci zvoleného dotazníku nemáte právo!";
             return 0;
          }

          //if ( !$editace ) return 1; // muze prohlizet sve vlastni hodnoceni
        }
        */

        // kontrola, jestli je dotaznik na "moji" urovni predavani
        require_once('adsapp/predavaciproces.php');

        if (!PredavaciProcesyUtils::overUrovenPredavani($database, $row[3])) {
            // tento dotaznik neni na moji urovni
            if ($editace) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na editaci zvoleného dotazníku nemáte právo!'));
            }

            if ($row[2] != User::getRC()) { // podminka doplnena 7.4.2014 - aby videl tlacitka v hodnoceni
                // jeste overi, jestli je aktualni uzivatel na jinem miste v predavacim procesu
                // - v takovem pripade muze tento dotaznik zobrazit (nemuze editovat)
                if (!($pravoZobrazitHodnoceneho || PredavaciProcesyUtils::jeVPredavacimProcesu($database, $row[6], $row[7]))) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zobrazení zvoleného dotazníku nemáte právo!'));
                }
            }

            // je v predavacim procesu, nebo je to vlastni hodnoceni
            $result = 1; // muze prohlizet, nemuze editovat
        }

        $kodtypurozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);
        $obdobiod = $database ->stringToTimestamp($row[4]);
        $obdobido = $database ->stringToTimestamp($row[5]);

        // 25.10.2017 hierarchie se kontroluje jen v pripade, ze v predavacim procesu je nastaveno na aktualni urovni NekontrolovatHierarchii na 0
        if ($row[10] == 0) {

            $params = $database->createParameters();

            $sql =
                'SELECT * ' . // kontrola, ze jsem (byl) v hodnocenem obdobi ve stejnem nebo nadrazenem rozdeleni
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
                "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                '    AND R_JA.KodTypuRozdeleni = ' . $params->addParameter($kodtypurozdeleni) .
                '    AND R_Hodnoceny.KodTypuRozdeleni = ' . $params->addParameter($kodtypurozdeleni) .
                "    AND R_Hodnoceny.RC = " . $params->addParameter($row[11]) .
                "    AND R_Hodnoceny.OsobniCislo = " . $params->addParameter($row[12]) .
                "    AND R_Hodnoceny.DatumNastupu = " . $params->addParameter($database->timestampToParam($database->stringToTimestamp($row[13]))) .
                '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                '    AND R_Hodnoceny.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                '    AND (R_Hodnoceny.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)';

            $cursor = $database->getSqlResult($sql, $params);

            $row = $cursor->fetch();
            if (!($pravoZobrazitHodnoceneho || $row)) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zobrazení zvoleného dotazníku nemáte právo! Hodnocená osoba patří do jiného rozdělení.'));
            }
        }

        // vse je OK
        return $result; // nastaveno drive na 1 (jen cist) nebo 2 (i editace)
    }



    /**
     * Zobrazeni sablony (z aplikace)
     * @return DotaznikViewObject
     * @throws Exception
     */
    function sablonaHodnoceni()
    {
        require_once('person/hodnoceni.php');

        $dotaznik = new Hodnoceni($this ->database);
        $dotaznik ->nactiSablonu((int)$_GET['druhhodnoceni'], (int)$_GET['temacislo']);

        return new DotaznikViewObject($dotaznik, true, false); // editace = true, postvalues = false
    }


    /**
     * @param int  $id
     * @param bool $editace
     * @return Hodnoceni
     * @throws Exception
     */
    function nactiHodnoceni($id, $editace)
    {
        require_once('person/hodnoceni.php');

        /** @var $hodnoceni Hodnoceni */
        $hodnoceni = new Hodnoceni($this ->database, $id);

        $pravo = $this ->vratPravoNaHodnoceni($this ->database, $id, $editace); // editace = false - pri nacitani me to nezajima
        $hodnoceni ->fPravo = $pravo;

        if (!$pravo  ||  ($editace  &&  $pravo < 2)) {
            RonPortalUtils::neniPravoException();
        }

        return $hodnoceni;
    }


    /**
     * @param int $id
     * @param bool $editace
     * @param bool $postValues
     * @return ViewObject
     * @throws Exception
     */
    function pripravenyDotaznik($id, $editace, $postValues)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, $editace);

        // kontrola, jestli se needituje uzavreny dotaznik
        if ($editace  &&  !$dotaznik ->jeEditovatelnyStav()) { // ukonceno, ukonceno-vraceno
            MessageManager::addErrorMessage( ronText('Nelze_Editovat_uzavrene_hodnoceni') );
            $editace = false;
        }

        // Nastavi hodnotitele dotazniku. Pokud jej jiz nekdo jiny rozeditoval, tak zobrazi vyzvu na prevzeti a vraci false.
        if ($editace  &&  !$postValues) {
            $prevzit = false;
            if (!RonPortalSettings::getIntValue(RonPortalSettings::ZAKAZAT_PREVZIT_HODNOCENI)  &&  isset($_GET['prevzit'])) $prevzit = (int)$_GET['prevzit'];

            // TODO: opravit!
            if (!$dotaznik->nastavHodnotitele($prevzit)) {
                if ( !RonPortalSettings::getIntValue(RonPortalSettings::ZAKAZAT_PREVZIT_HODNOCENI)  &&  !$prevzit ) {
                    // jeste nebylo potvrzeno prevzeti
                    return new PrevzetiHodnoceniViewObject($dotaznik);
                }

                return new MessageManagerViewObject(); // zobrazi chybove hlasky
            }
        }

        return new DotaznikViewObject($dotaznik, $editace, $postValues);
    }


    /**
     * Provede odeslani nebo vraceni dotazniku. Pouziva se i v javascriptech!
     * @param int $id
     * @param int $uloha  1=predat dale, 2=vratit
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    private function odesliDotaznik($id, $uloha)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8("Chyba v metode odesliDotaznik - není zadáno Id dotazníku!"));

        MessageManager::addDebugMessage("HodnoceniController::odesliDotaznik id=" . StringUtils::htmlTextConvert($id) . " - uloha=" . StringUtils::htmlTextConvert($uloha));

        $params = $this->database->createParameters();

        // zapise jmeno hodnoticiho a cas
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            "   SET HodnotilRC = " . $params ->addParameter(User::getRC()) . ", " .
            "       Hodnotici = " . $params ->addParameter(RonPortalUtils::getLogUserName()) . ", " .
            '       DatumHodnoceni = ' . $params->addParameter($this ->database ->timestampToParam(time())) .
            ' WHERE Id = ' . $params->addParameter($id);

        $result = $this ->database ->sql($sql, $params);
        if (!$result) {
            throw new DBException($this ->database, $sql, StringUtils::fromUTF8('Chyba při nastavení hodnotitele!'));
        }

        require_once('adsapp/agentmanager.php');
        try {
            AgentManager::executePersonAgent("$uloha -IDHodnoceni:{$id} -alwaysLog");
            MessageManager::addInfoMessage( ronText('Uzavreni_hodnoceni_OK') );
        } catch (Exception $e) {
            MessageManager::addErrorMessage( ronText('Chyba_uzavreni_hodnoceni') );
            MessageManager::addErrorMessage($e ->getMessage());

            return new Redirect(null, '?menuid=hodnoceni&record=' . $id);
            //throw new \Exception( \Ron\ronText('Chyba_uzavreni_hodnoceni') . "\n" . $e ->getMessage(), 0, $e );
        }

//        if ( !AgentManager::executePerson("$uloha -IDHodnoceni:{$id} -alwaysLog") ) {
//            throw new \Exception( \Ron\ronText('Chyba_uzavreni_hodnoceni') );
//        }

        return new Redirect(null, "?menuid=hodnoceni");
    }


    /**
     * @return Redirect|ViewObject|DotaznikViewObject
     * @throws Exception
     */
    function ulozHodnoceni()
    {
        $id = (int)$_GET['record'];
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, true); // editace = true

        $dotaznik->fUlozenoVXML = null;
        $dotaznik->fPrepoctenoVXML = null;

        if ($dotaznik->fXMLSablona) {
            $dotaznik->vyplnDotaznikXML();
        }

        // pokud nebyl dotaznik ulozen uz v XML, nebo nema XML, tak jej ulozi nyni
        if (isset($dotaznik->fUlozenoVXML)) {
            MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí proběhlo přímo v XML formuláři'));
            $ulozeno_result = $dotaznik->fUlozenoVXML;
        } else {
            if ($dotaznik->fXMLSablona) {
                MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí neproběhlo v XML -> proběhne nyní'));
            }

            try {
                $ulozeno_result = $dotaznik->ulozVyplnenyDotaznik();
            } catch (RonException $e) {
                MessageManager::addErrorMessage($e ->getMessage(), $e ->fieldName, null, $e ->getErrorDescription());
                $ulozeno_result = false;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $ulozeno_result = false;
            }
        }

        if ($ulozeno_result) {
            MessageManager::addInfoMessage(ronText('Hodnoceni_ulozeno'));
        } else {
            return new DotaznikViewObject($dotaznik, true, true);
            // TODO: zkontrolovat XMLkovy dotaznik!
            //return $this ->pripravenyDotaznik($id, true, true); // editace = true, postvalues = true
        }

        $prepocet_result = false; // jen kvuli warningu

        if ($dotaznik->fVyplnenoVse) {
            // ulozeni probehlo v poradku - provede prepocet
            if (isset($dotaznik->fPrepoctenoVXML)) {
                MessageManager::addDebugMessage(StringUtils::fromUTF8('Přepočet výsledků proběhl přímo v XML formuláři'));
                $prepocet_result = $dotaznik->fPrepoctenoVXML;
            } else {
                MessageManager::addDebugMessage(StringUtils::fromUTF8('Přepočet výsledků neproběhl v XML -> proběhne nyní'));
                $prepocet_result = $dotaznik->prepocitejDotaznik();
            }

            if (!$prepocet_result) {
                // chyba pri prepoctu
                return $this ->pripravenyDotaznik($id, true, true); // editace = true, postvalues = true
            }
        }

        // test, jestli se ma zaroven s ulozenim vratit nebo uzavrit (stisknuto tlacitko "uzavrit")
        if (isset($_POST['uzavrit']) && $_POST['uzavrit']) {
            if ($dotaznik->fVyplnenoVse  &&  $prepocet_result) {
                return $this ->odesliDotaznik($id, 1); // 1=predat dale
            }

            if (!$dotaznik->fVyplnenoVse) {
                MessageManager::addErrorMessage( ronText('Nelze_uzavrit_nevyplnene_hodnoceni') );
            }

            // pouzit redirect ? ? ?
            return $this ->pripravenyDotaznik($id, true, false); // editace = true, postvalues = false (uz je ulozeno)

        } else if (isset($_POST['vratit']) && $_POST['vratit']) {
            // stisknuto tlacitko "vratit"
            return $this->odesliDotaznik($id, 2); // 2=vratit zpet
            //return VratDotaznik($connection, $id);
        } else if (isset($_POST['prepocitat']) && $_POST['prepocitat']) {
            return new Redirect(null, '?menuid=hodnoceni&hodnoceniaction=editujhodnoceni&record=' . $id);
        } else {
            // jen ulozeni - zobrazi v readonly
            return new Redirect(null, '?menuid=hodnoceni&record=' . $id);
            //ZobrazPripravenyDotaznik($connection, $id, false, false); // editace = false, postvalues = false
        }
    }


    /**
     * Uzavreni dotazniku (kliknuti na tlacitko Odeslat nebo Vratit k doplneni)
     * @throws Exception
     */
    public function uzavriHodnoceni()
    {
        $id = (int)$_GET['record'];
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        if (isset($_POST['uzavrit']) && $_POST['uzavrit']) {
            // bylo stisknuto tlacitko "uzavrit"
            return $this ->odesliDotaznik($id, 1); // 1=odeslat
        } else if (isset($_POST['vratit']) && $_POST['vratit']) {
            // bylo stisknuto tlacitko "vratit"
            return $this ->odesliDotaznik($id, 2); // 2=vratit
        }

        return new Redirect(null, '?menuid=hodnoceni&record=' . $id);
        //return $this ->pripravenyDotaznik($id, false, false);
    }


    /**
     * @return VysledekHodnoceniBrowser
     * @throws Exception
     */
    function vysledkyHodnoceni()
    {
        require_once('person/b_vysledekhodnoceni.php');

        $b = new VysledekHodnoceniBrowser($this->database);
        $b->load();

        return $b;
    }

    /**
     * @param $idPP
     * @return JsonOutputObject
     * @throws Exception
     */
    private function ucelyHodnoceniProOsobu($idPP)
    {
        $output = new JsonOutputObject();

        require_once('adsapp/ppomer.php');

        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $idPP);

        if (!$this->pravaZobraxit->lzeZobrazitRC($pracPomer->rc))
            RonPortalUtils::neniPravoException();

        $params = $this->database->createParameters();

        $sql =
            "SELECT UcelHodnoceni.Kod, UcelHodnoceni.Popis, UcelHodnoceni.DuplicitniGenerovani
                FROM UcelHodnoceni JOIN UcelyHodnoceniNaPM ON (UcelHodnoceni.Kod = UcelyHodnoceniNaPM.KodUcelu) 
                    JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = UcelyHodnoceniNaPM.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = UcelyHodnoceniNaPM.KodRozdeleni)
                    JOIN PracovniPomer ON (RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu)
                    JOIN PredavaciProces ON (PredavaciProces.Kod = UcelHodnoceni.KodProcesu)
                    JOIN VstupniUroven ON (VstupniUroven.KodProcesu = PredavaciProces.Kod)
                    JOIN UrovenPredavani ON (UrovenPredavani.Kod = VstupniUroven.VstupniUroven)
                    JOIN PMProUrovne ON (PMProUrovne.KodUrovne = UrovenPredavani.Kod)
                    JOIN RozdeleniOsob R_PM ON (R_PM.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni AND R_PM.Rozdeleni = PMProUrovne.KodRozdeleni AND R_PM.RC = PracovniPomer.RC AND R_PM.OsobniCislo = PracovniPomer.OsobniCislo AND R_PM.DatumNastupu = PracovniPomer.DatumNastupu) 
                WHERE PracovniPomer.RC = " . $params->addParameter($pracPomer->rc) . "
                  AND PracovniPomer.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR PracovniPomer.DatumUkonceni IS NULL)
                  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                  AND UcelyHodnoceniNaPM.DatumOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (UcelyHodnoceniNaPM.DatumDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR UcelyHodnoceniNaPM.DatumDo IS NULL)
                  AND R_PM.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (R_PM.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR R_PM.PlatnostDo IS NULL)";

        $cursor = $this->database->getSqlResult($sql, $params);

        $ucely = [];
        while ($row = $cursor->fetch()) {
            $ucel = [];
            $ucel['kod'] = StringUtils::jsonText($row[0]);
            $ucel['popis'] = StringUtils::jsonText($row[1]);
            $ucel['povolit_duplicitu'] = $row[2] ? true : false;
            $ucely[] = $ucel;
        }

        $cursor->close();

        $output->addData('items', $ucely);

        return $output;
    }

    /**
     * @param $idhodnoceni
     * @return JsonOutputObject
     * @throws Exception
     */
    private function hodnotiteleProHodnoceni($idhodnoceni)
    {
        $output = new JsonOutputObject();

        $params = $this->database->createParameters();

        $sql =
            "SELECT RozdeleniOsob.RC, Osoba.Prijmeni, Osoba.Jmeno, RozdeleniOsob.OsobniCislo, PP.ID 
                FROM HodnoceniOsoby 
                  JOIN PoradiPredavani ON ((PoradiPredavani.KodProcesu = HodnoceniOsoby.KodProcesu) AND (PoradiPredavani.PocatecniUroven = HodnoceniOsoby.PocatecniUroven) AND (PoradiPredavani.Poradi = HodnoceniOsoby.PoradiPredavani))
                  JOIN PMProUrovne ON (PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne)
                  JOIN RozdeleniOsob ON (PMProUrovne.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND PMProUrovne.KodRozdeleni = RozdeleniOsob.Rozdeleni)
                  JOIN VysledekHodnoceni ON (VysledekHodnoceni.ID = HodnoceniOsoby.IDVysledku)
                  JOIN RozdeleniOsob RHod ON (RHod.RC = VysledekHodnoceni.HodnocenyRC AND RHod.OsobniCislo = VysledekHodnoceni.HodnocenyOsobniCislo AND RHod.DatumNastupu = VysledekHodnoceni.HodnocenyDatumNastupu)                  
                  JOIN Osoba ON (RozdeleniOsob.RC = Osoba.RC)
                  JOIN PredavaciProces ON (PredavaciProces.Kod = PoradiPredavani.KodProcesu)
                  JOIN PracovniPomer PP ON PP.RC = RozdeleniOsob.RC AND PP.OsobniCislo = RozdeleniOsob.OsobniCislo AND PP.DatumNastupu = RozdeleniOsob.DatumNastupu 
                WHERE HodnoceniOsoby.ID = " . $params->addParameter($idhodnoceni) . "
                  AND (RHod.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni 
                        OR (PredavaciProces.KodTypuRozdeleni IS NULL AND RHod.KodTypuRozdeleni = " . $params->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI)) . "))
                  AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                  AND RHod.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (RHod.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RHod.PlatnostDo IS NULL)
                  AND PP.DatumNastupu <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                  AND (PP.DatumUkonceni >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR PP.DatumUkonceni IS NULL)
                  
                  AND ((PoradiPredavani.NekontrolovatHierarchii = 1) 
                        OR ((PoradiPredavani.NekontrolovatHierarchii = 0) AND EXISTS(SELECT RHodnotitel.RC 
                                                                                     FROM HierarchieRozdeleni 
                                                                                          JOIN RozdeleniOsob RHodnotitel ON (RHodnotitel.KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene 
                                                                                                    AND RHodnotitel.Rozdeleni = HierarchieRozdeleni.kodNadrazene 
                                                                                                    AND RHodnotitel.RC = RozdeleniOsob.RC 
                                                                                                    AND RHodnotitel.OsobniCislo = RozdeleniOsob.OsobniCislo 
                                                                                                    AND RHodnotitel.DatumNastupu = RozdeleniOsob.DatumNastupu) 
                                                                                     WHERE HierarchieRozdeleni.typPodrizene = RHod.KodTypuRozdeleni 
                                                                                       AND HierarchieRozdeleni.kodPodrizene = RHod.Rozdeleni
                                                                                       AND RHodnotitel.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . "
                                                                                       AND (RHodnotitel.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RHodnotitel.PlatnostDo IS NULL)
                                                                                     )
                           )
                      )
                  
                  AND (PoradiPredavani.Vlastni = 0 OR (PoradiPredavani.Vlastni = 1 AND RozdeleniOsob.RC = " . $params->addParameter(User::getRC()) . "))
             ORDER BY Osoba.Prijmeni, Osoba.Jmeno";

        MessageManager::addDebugMessage($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $hodnotitele = [];
        while ($row = $cursor->fetch()) {
            $hodnotitel = [];
            $hodnotitel['kod'] = $row[4];
            $hodnotitel['popis'] = StringUtils::jsonText($row[1], true) . ' ' . StringUtils::jsonText($row[2], true);
            $hodnotitel['osc'] = StringUtils::jsonText($row[3], true);
            $hodnotitele[] = $hodnotitel;
        }

        $cursor->close();

        $output->addData('items', $hodnotitele);

        return $output;
    }

    /**
     * @return JsonOutputObject
     * @throws Exception
     */
    private function generujHodnoceniOsoby()
    {
        $idPP = $_POST['idpp'];
        $ucel = $_POST['ucel'];
        $datumOd = date('Y-m-d', DateUtils::dateStringToTimestamp($_POST['datumod']));
        $pocetObdobi = $_POST['pocetobdobi'] ?: 1;
        $pocet = $_POST['pocet'] ?: 1;

        require_once('adsapp/ppomer.php');

        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $idPP);

        $output = new JsonOutputObject();

        require_once('adsapp/agentmanager.php');
        try {
            AgentManager::executePersonAgent("11 -datumDo:{$datumOd} -RCHodnoceneho:{$pracPomer->rc} -KodUceluHodnoceni:{$ucel} -PocetObdobi:{$pocetObdobi} -PovolitDuplicitu:{$pocet}");
            $output->addText('vysledekgenerovani', ronText('Generovani_hodnoceni_OK'));
        } catch (Exception $e) {
            MessageManager::addErrorMessage( ronText('Chyba_uzavreni_hodnoceni') );
            $output->addText('vysledekgenerovani', ronText('Generovani_hodnoceni_chyba'));
            $output->addError($e);
        }

        return $output;
    }

    /**
     * @return JsonOutputObject
     * @throws DBException
     */
    private function nastavHodnotitele()
    {
        $hodnotitel = $_POST['hodnotitel'];

        require_once 'adsapp/ppomer.php';
        $pracPomer = new PracovniPomer();
        $pracPomer->nactiPPPodleId($this->database, $hodnotitel);

        $idHodnoceni = $_POST['idhodnoceni'];

        $output = new JsonOutputObject();

        require_once('adsapp/ronportaldbrecord.php');

        $rec = new RonPortalDBRecord($this->database, 'PripraveneTemaHodnoceni');

        $rec->addField('ID', 'int', true);
        $rec->addField('HodnocenyRC', '', true);
        $rec->addField('HodnotilRC', '', false, 15, $pracPomer->rc);
        $rec->addField('IndividualniPoradi', 'int', false, 0, 1);

        $params = $this->database->createParameters();

        $rec->setWhere('ID = ' . $params->addParameter($idHodnoceni), $params);

        try {
            $rec->loadRecord();

            if (!$this->pravaZobraxit->lzeZobrazitRC($rec['HodnocenyRC']))
                RonPortalUtils::neniPravoException();

            $rec->updateRecord();

        } catch (Exception $e) {
            $output->addError($e);
        }

        return $output;
    }

    /**
     * @return JsonOutputObject|HodnoceniOsobBrowser
     * @throws Exception
     */
    function hodnoceniOsob()
    {
        if (!$this->pravaZobraxit->jsouNejakePrava())
            RonPortalUtils::neniPravoException();

        switch ($_GET['action']) {
            case 'nactiucely' :
                return $this->ucelyHodnoceniProOsobu($_GET['idpp']);
            case 'nactiosoby' :
                return $this->hodnotiteleProHodnoceni($_GET['idhodnoceni']);
            case 'generovani' :
                return $this->generujHodnoceniOsoby();
            case 'nastavhodnotitele' :
                return $this->nastavHodnotitele();
        }

        require_once('person/b_hodnoceniosob.php');

        $b = new HodnoceniOsobBrowser($this->database);
        $b->load();

        Model::addViewObject(new BaseViewObject('person/view_hodnocenigenerovani.php'));
        Model::addViewObject(new BaseViewObject('person/view_hodnocenipriradhodnotitele.php'));

        return $b;
    }


    /**
     * Volano javascriptem - hromadne uzavreni
     * @return JsonOutputObject
     */
    function hromadneUzavreniDotazniku()
    {
        $result = new JsonOutputObject();

        try {
            $id = (int)$_GET['id'];
            if (!$id) throw new Exception('Chyba! Neni zadan ID hodnoceni!');

            $pravo = self::vratPravoNaHodnoceni($this ->database, $id, true);

            if ($pravo) {
                $this ->odesliDotaznik($id, 1); // 1 = predat dale
            }

        } catch (Exception $e) {
            $result ->addError($e ->getMessage());
        }

        $result ->addMessages(MessageManager::getInstance());

        return $result;
    }


    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|JsonOutputObject|OutputObject|Redirect|TranslationsOutputObject|ViewObject|DotaznikViewObject|HodnoceniBrowser|HodnoceniOsobBrowser|VysledekHodnoceniBrowser|XMLFormViewObject|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dotaznik.ini');

        if (isset($_GET['translations'])) {
            return $this->translations();
        }

        if ($action == 'hodnoceni' &&  isset($_GET['hodnoceniaction'])  &&  $_GET['hodnoceniaction'] == 'hromadneuzavrenidotazniku') {
            return $this ->hromadneUzavreniDotazniku();
        }

        HtmlHeaders::getInstance() ->addScript('script/hodnoceni.js');
        HtmlHeaders::getInstance() ->addCss('person/dotaznik.css');
        HtmlHeaders::getInstance() ->addScript('?menuid=hodnoceni&translations=1');

        switch ($action) {
            case 'zobrazsablonuhodnoceni':
                // zobrazeni sablony (nahled z aplikace)
                return $this ->sablonaHodnoceni();

            case 'vysledkyhodnoceni' :
                return $this ->vysledkyHodnoceni();

            case 'hodnoceniosob' :
                return $this->hodnoceniOsob();
        }


        if (isset($_POST['hodnoceniaction'])  &&  $_POST['hodnoceniaction']) {
            // odeslani formulare - ulozeni hodnoceni nebo uzavreni
            switch ($_POST['hodnoceniaction']) {
                case 'ulozdotaznik' :
                    return $this ->ulozHodnoceni();

                case'uzavridotaznik' :
                    return $this ->uzavriHodnoceni();

                default :
                    throw new Exception('Chyba v metode HodnoceniController::handleAction - neznamy typ akce!');
            }
        }

        if (isset($_GET['record'])  &&  (int)$_GET['record']) {
            $id = (int)$_GET['record'];

            $hodnoceniAction = isset($_GET['hodnoceniaction']) ? $_GET['hodnoceniaction'] : '';
            return $this ->pripravenyDotaznik($id, $hodnoceniAction == 'editujhodnoceni', false);
        }

        return $this ->seznamHodnoceni();
    }

}
