<?php
/**
 * UTF-8
 * stanislav.elbl, 19.11.2015, pavel.rudol, 8.11.2017
 * Rozsireni obecneho dotazniku o funkce specificke pro hodnoceni skoleni
 */

namespace Ron\Personalistika;

use Exception;
use Ron\DB\DBException;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

require_once('person/hodnoceni.php');


class HodnoceniSkoleni extends Hodnoceni
{
    public $typSkoleni;
    public $nazevSkoleni;
    public $zacatekSkoleni;
    public $konecSkoleni;
    /** @var  int typ generovani: 1 Hodnocení účastníka dle typu rozdělení 2 Hodnocení účastníka zvolenou osobou 3 Hodnocení školení účastníkem */
    public $typGenerovaniHodnoceni;

    /**
     * HodnoceniSkoleni constructor.
     * @param $database
     * @param null $id
     * @throws Exception
     */
    public function __construct($database, $id = null)
    {
        parent::__construct($database, $id);

        $this->nactiDetailySkoleni();
    }

    /**
     * @throws Exception
     */
    private function nactiDetailySkoleni()
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT Skoleni.Popis, Skoleni.DruhSkoleni, Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni, " . // 0..3
            "   Skoleni.DatumUkonceniSkoleni, Skoleni.CasUkonceniSkoleni, HodnoceniSkoleni.TypGenerovani " . // 4, 5, 6
            "FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni " .
            "   ON ((PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni) AND (PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo)) " .
            " JOIN Skoleni ON (HodnoceniSkoleni.CisloSkoleni = Skoleni.CisloSkoleni) " .
            "WHERE PripraveneTemaHodnoceni.ID = " . $params->addParameter($this->fId);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $this->nazevSkoleni = $row[0];
        $this->typSkoleni = $row[1];
        $this->zacatekSkoleni = $this->database->stringToTimestamp($row[2]) + 3600 * $this->database->stringToTimeOfDay($row[3]);
        $this->konecSkoleni = $this->database->stringToTimestamp($row[4]) + 3600 * $this->database->stringToTimeOfDay($row[5]);
        $this->typGenerovaniHodnoceni = $row[6];
    }

    /**
     * Vyplni u zadaneho dotazniku pole HodnotilRC a Hodnotil (uzivatelske jmeno)
     * Pokud uz je to vyplneno a je tam nekdo jiny nez aktualni uzivatel, tak zobrazi dotaz, jestli chce uzivatel tento dotaznik prevzit
     * @param bool $prevzit
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function nastavHodnotitele($prevzit = false)
    {
        // TODO: upravit ! ! !
        if (!$this->fId) {
            throw new Exception(StringUtils::fromUTF8('Chyba při nastavování hodnotitele - není načten dotazník!'));
        }

        if ( $this->fXMLSablona) {
            // v XML sablone je mozne zabranit vyplneni (napr. uzivatelska kontrola prav)
            if (!$this->zobrazDotaznikXML(false, false, 'nastavhodnotitele')) {
                return false;
            }
        }

        $params = $this->database->createParameters();

        $sql =
            "UPDATE PripraveneTemaHodnoceni " .
            "  SET HodnotilRC = " . $params->addParameter(User::getRC()) . ", " .
            "   Hodnotici = " . $params->addParameter(User::getLoginName()) . ", " .
            '   DatumHodnoceni = ' . $params->addParameter($this ->database ->timestampToParam(time())) . ', ' .
            '   Stav = 1 ' . // = probiha
            '  WHERE Id = ' . $params->addParameter($this ->fId) .
            '   AND (HodnotilRC IS NULL)';

        if (!$this ->database ->sql($sql, $params)) {
            throw new DBException($this ->database, $sql, StringUtils::fromUTF8('Chyba při nastavení hodnotitele!'));
        }

        $params = $this->database->createParameters();

        // Overi, jestli je aktualni uzivatel zapsany jako resitel
        $sql =
            'SELECT PripraveneTemaHodnoceni.HodnotilRC, Osoba.Prijmeni, Osoba.Jmeno, ' .
            '       PripraveneTemaHodnoceni.Hodnotici, PripraveneTemaHodnoceni.DatumHodnoceni, ' .
            '       PripraveneTemaHodnoceni.Stav ' .
            '  FROM PripraveneTemaHodnoceni, Osoba ' .
            ' WHERE Id = ' . $params->addParameter($this->fId) .
            '   AND Osoba.RC = PripraveneTemaHodnoceni.HodnotilRC ';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if (!$row) throw new Exception(StringUtils::fromUTF8("Chyba při nastavování hodnotitele - zadaný záznam neexistuje!"));

        // updatne hodnoty
        $this->fHodnoticiPrijmeni = $row[1];
        $this->fHodnoticiJmeno = $row[2];
        $this->fHodnotici = $row[3];
        $this->fDatumHodnoceni = $this->database->stringToTimestamp($row[4]);
        $this->fStav = $row[5];

        if ($row[0] == User::getRC()) return true; // OK, uzivatel je nastaveny jako hodnotitel
        else throw new Exception(StringUtils::fromUTF8('Hodnocení má přiřazeno jiného hodnotitele!'));
    }


    function zobrazHlavicku()
    {
        require('person/view_hlavickahodnoceniskoleni.php');
    }


    function vratHTMLTlacitka($editace, $ulozitstr = null, $odeslatstr = null, $vratitstr = null, $uzavritstr = null)
    {
        if ($this->fPravo != 2) return ''; // pokud nemuze editovat, nic nezobrazi

        $result = '<DIV class="tlacitkahodnoceni">';

        // Tlacitka Odeslat a Upravit
        if ($editace) { //  || $postvalues) {
            if ($ulozitstr != 'null') {
                // tlacitko "odeslat" (ulozi odpovedi, ale nezmeni stav)
                if (!$ulozitstr) $ulozitstr = ronText('form.Store');

                if ($this->fId) $hlpstr = '';
                else $hlpstr = 'disabled';

                $result .= "<INPUT type='submit' class='submitbutton' $hlpstr value='$ulozitstr' />\n";
            }
        } else {
            if (($this->fPravo == 2) && // ma pravo editovat
                ($this->jeEditovatelnyStav())
            ) // neuzavreno
            {
                global $STR_UpravitDotaznik; // kompat.
                // moznost upravit ? ? ? - jen pokud mam pravo a neni uzavren
                //echo '<SPAN style="float:right"> ble </SPAN>';
                $result .= '<A href="?menuid=skoleni&#38;action=hodnoceni&#38;hodnoceniaction=editujhodnoceni&#38;record=' . $this->fId . '">' . ($STR_UpravitDotaznik ?: ronText('Upravit_hodnoceni')) . "</A>\n";
            }
        }


        // Tlacitka Uzavrit a Vratit
        if ($this->jeEditovatelnyStav()) // v reseni   // 0=pripraveno, 1=probiha, 2=ukonceno, 3=ukonceno-vraceno, 4=pripraveno (vraceno)
        {
            $result .= '<SPAN style="float:right">';

            $editace = (int)$editace; // prevede praznou hodnotu na 0 - jinak je prazdny parametr javascriptu a nefachci to

            // pokud neni vyplneno vse, nepovoli uzavrit
            // pokud se ale jedna o editaci, tak uzavreni povoli, protoze uzivatel mohl chybejici odpovedi doplnit
            if ($this->fVyplnenoVse || $editace) {
                global $STR_UzavritDotaznik; // kompat.

                $btntext = $uzavritstr ?: ($STR_UzavritDotaznik ?: ronText('Uzavrit_hodnoceni'));
                $msgId = 'Opravdu_uzavrit_hodnoceni';

                if ($msgId != 'null') {
                    $result .= '<INPUT type="submit" name="uzavrit" data-ron-confirm-id="' . $msgId . '" class="submitbutton" value="' . $btntext . '" />';
                }
            } else {
                $result .= ronText('Chybi_odpovedi');
            }

            $result .= '</SPAN>';
        }

        $result .= "</DIV>\n";

        return $result;
    }


    /**
     * @param bool $editace
     */
    function zobrazTlacitka($editace)
    {
        echo $this->vratHTMLTlacitka($editace);
    }

    function vratAkciDotazniku()
    {
        return "?menuid=skoleni&action=hodnoceni&record={$this->fId}";
    }

}
