<?php
/**
 * UTF8
 * stanislav.elbl, 20.09.2015
 * roman.szablatura 10.10.2017
 */

namespace Ron\Personalistika;


use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Redirect;
use Ron\MVC\TranslationsOutputObject;
use Ron\MVC\ViewObject;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\DokumentUploadOutput;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\User;
use Ron\TextManager;
use RonPortal\Personalistika\NahlasenaZmenaForm;

require_once ('person/nahlasenezmenycontroller.php');
require_once ('person/b_drivenahlasenezmeny.php');


/**
 * Class HlaseniZmenyController
 * @package Ron\RonPortal
 */
class HlaseniZmenyController extends HttpController
{
    /** @var Database */
    var $database;

    /**
     * HlaseniZmenyController constructor.
     * @param Database $database
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(), time());
    }


    /**
     * Pouze pro vypis pro potreby view
     * @param Database $database
     * @param $rc
     * @return array|bool
     * @throws \Ron\DB\DBException
     */
    public function nacteniUctuOsoby($database, $rc)
    {
        $params = $database->createParameters();

        $sql = 'SELECT BankovniUcet.JmenoUctu, S_TypBankovnihoUctu.Kod, BankovniUcet.CisloUctu, Banka.kod, '. // 0..3
            ' BankovniUcet.ZahranicniPlatba, BankovniUcet.IBAN, Mena.KodMeny_ISO, BankovniUcet.SpecifickySymbol,' . // 4..7
            " BankovniUcet.KonstantniSymbol, BankovniUcet.VariabilniSymbol, VyplaceniMzdy.PlatnostOd, BankovniUcet.CisloUctu, BankovniUcet.SerioveCislo
                     FROM BankovniUcet JOIN VyplaceniMzdy ON (BankovniUcet.SerioveCislo = VyplaceniMzdy.SerioveCisloUctuVyplaty)
                        LEFT JOIN S_TypBankovnihoUctu ON (S_TypBankovnihoUctu.Kod = BankovniUcet.TypUctu)
                          LEFT JOIN Banka ON (Banka.kod = BankovniUcet.KodBanky)
                            LEFT JOIN Mena ON (Mena.KodMeny_ISO = BankovniUcet.KodMeny_ISO)
                                LEFT JOIN KonstantniSymbol ON (BankovniUcet.KonstantniSymbol = KonstantniSymbol.kod)

                      WHERE TypUctu = 1 AND BankovniUcet.RC =" . $params ->addParameter($rc) .
            " ORDER BY VyplaceniMzdy.PlatnostOd ";

        $hodnotyUctu = [];
        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $hodnotyUctu[0] = $row[0];
            $hodnotyUctu[1] = $row[1];
            $hodnotyUctu[2] = $row[2];
            $hodnotyUctu[3] = $row[3];
            $hodnotyUctu[4] = $row[4];
            $hodnotyUctu[5] = $row[5];
            $hodnotyUctu[6] = $row[6];
            $hodnotyUctu[7] = $row[7];
            $hodnotyUctu[8] = $row[8];
            $hodnotyUctu[9] = $row[9];
            $hodnotyUctu[10] = $database->stringToTimestamp($row[10]);
            $hodnotyUctu[11] = $row[11];
            $hodnotyUctu[12] = $row[12];
        }

        return $hodnotyUctu;
    }


    /**
     * Nahlaseni nove zmeny - v pripade potreby nacte data z prislusne agendy a zobrazi formular
     * @param $typZmeny
     * @param $idPP
     * @param $editace
     * @param $detail
     * @return BaseViewObject
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    function hlaseniZmeny($typZmeny, $idPP, $editace, $detail)
    {
        if (isset($_GET['new'])) {
            $form = new NahlasenaZmenaForm($this ->database);
            $form ->handleAction();

            return $form;
        }

        require_once 'adsapp/ppomer.php';
        $pracovniPomer = PracovniPomer::nactiPP($this->database, $idPP);

        require_once 'person/nahlasenazmena.php';
        NahlasenaZmena::zkontrolujPravoNaZmenu($typZmeny, $pracovniPomer ->rc, true);

        require_once 'person/hlasenizmeny_form.php';
        $form = new HlaseniZmenyForm($this ->database, $detail);
        $form ->novaZmena($pracovniPomer, $typZmeny);

        //Pro potreby editace aktualnich udaju
        if (isset($editace)) {
            switch ($editace) {
                case 'prukazy' :
                    $form ->nactiPrukazOsoby($_GET['cisloprukazu'], (int)$_GET['typprukazu']);
                    break;

                case 'rodina' :
                    $form ->nactiRodinnehoPrislusnika($_GET['idrodinnehoprislusnika']);
                    break;

                case 'osobnich' :
                    $form ->nactiOsobniUdaje();
                    break;
            }
            //TODO doplnit upravu trvale adresy
            //$adresa = $this->nacteniAdresy($this->database, $osoba, $_GET['trvala'], $_GET['platnostod']);
            //if ($editace == 'adresa') {Model::addData('hodnotyKEditaci', $adresa);}
        }

        if ($typZmeny == 'ucetosoby') {
            $form ->nactiUcetOsoby();
        }

        return $form;
    }


    function editaceZmeny($idZmeny)
    {
        require_once 'person/hlasenizmeny_form.php';
        $form = new HlaseniZmenyForm($this ->database);
        $form ->load($idZmeny);

        return $form;
    }

    function redirect()
    {
        switch (isset($_GET['back']) ? $_GET['back'] : '') {
            case 'osobniudaje' :
                $menuId = $_GET['back'];
                break;

            default :
                $menuId = 'nahlasenezmeny';
        }

        return new Redirect(null, "?menuid=$menuId");
    }


    /**
     * @return ViewObject|Redirect
     * @throws \Exception
     */
    function odesliHlaseniZmeny()
    {
        require_once 'person/hlasenizmeny_form.php';
        $form = new HlaseniZmenyForm($this ->database, false);

        try {
            $form->nactiZadaneHodnoty();
            $form->zapisZmenu();

            // Kontrola, jestli uzivatel kliknul na "odmitnout"
            if (isset($_POST['odmitnout_zmenu'])){
                return $this ->odmitnutiNahlaseneZmeny();
            }

            if (isset($_POST['potvrdit_zmenu'])){
                return $this ->potvrzeniNahlaseneZmeny();
            }

        } catch (\Exception $e) {
            // doslo k chybe - zobrazi formular znovu
            MessageManager::addErrorMessage($e);
            return $form;
        }

        return new Redirect(null, "?menuid=nahlasenezmeny&record={$form ->nahlasenaZmena ->id}&back=osobniudaje");
    }


    /**
     * Po zapisu se aktualizuje nahlasena zmena o datum potvrzeni a osobu, ktera zmenu potvrdila
     * @param $idZaznamu
     * @param $poznamka
     * @param $stav
     * @param $logText
     * @throws \Exception
     * @throws \Ron\DB\DBException
     * @internal param $datumZapisu
     */
    private function nastavStavNahlaseneZmeny($idZaznamu, $poznamka, $stav, $logText)
    {
        $rec = new RonPortalDBRecord($this->database, 'NahlasenaZmena', $this);

        $rec ->addLogText($logText);

        $rec->addIntField('Id', true);
        $rec->addStringField('TypZmeny', true);
        $rec->addStringField('RC', true);
        $rec->addIntField('Stav', false, $stav);
        $rec->addTimestampField('DatumCasZpracovani', false, time());
        $rec->addStringField('Zpracoval', false, 15, User::getRC());
        $rec->addField('Hodnoty', 'memo', true);
        $rec->addField('Poznamka', 'memo', false, 0, $poznamka);

        $params = $this->database->createParameters();
        $rec->setWhere('ID = ' . $params->addParameter($idZaznamu), $params);

        if ($rec ->loadRecord()) {
            $rec->updateRecord();
        } else {
            throw new \Exception('Zaznam nahlasene zmeny nebyl nalezen!');
        }
    }



    private function odmitnutiNahlaseneZmeny()
    {
        // nacte zapsanou zmenu
        require_once 'person/nahlasenazmena.php';
        $zmena = NahlasenaZmena::nactiNahlasenouZmenu($this ->database, (int)$_POST['idZmeny']);
        // zkontroluje pravo na schvalovani
        NahlasenaZmena::zkontrolujPravoNaZmenu($zmena ->typZmeny, $zmena ->rcOsobyZmeny, false, true);

        // a nastavi novy stav
        $this ->nastavStavNahlaseneZmeny((int)$_POST['idZmeny'], $_POST['poznamka'], 2, 'Odmitnuti nahlasene zmeny'); // 2 = odmitnuto

        return $this ->redirect();
    }


    private function potvrzeniNahlaseneZmeny()
    {
        require_once 'person/nahlasenazmena.php';
        $zmena = NahlasenaZmena::nactiNahlasenouZmenu($this ->database, (int)$_POST['idZmeny']);
        // zkontroluje pravo na schvalovani
        NahlasenaZmena::zkontrolujPravoNaZmenu($zmena ->typZmeny, $zmena ->rcOsobyZmeny, false, true);

        try {
            // zapise nove hodnoty do databaze
            $zmena->zapisZmeny($this->database);

            // potvrdi zmenu
            $this ->nastavStavNahlaseneZmeny($zmena ->id, $zmena ->poznamka, 1, 'Potvrzeni nahlasene zmeny');
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return new Redirect(null, "?menuid=nahlasenezmeny&record={$zmena ->id}&back=osobniudaje");
        }


        return $this ->redirect();
    }


    private function zkontrolujPravoNaDokument($idZmeny, $idDokumentu, $editace)
    {
        //Kontrola, zdali existuje u dotycne zmeny prohlizeny dokument a uzivatel na nej ma pravo
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentZmeny.ID, NahlasenaZmena.TypZmeny, NahlasenaZmena.RC' .
            ' FROM DokumentZmeny JOIN NahlasenaZmena ON (DokumentZmeny.IDZmeny = NahlasenaZmena.ID)' .
            ' WHERE DokumentZmeny.IDZmeny = ' . $params->addParameter($idZmeny) .
            ' AND DokumentZmeny.ID = ' . $params->addParameter($idDokumentu);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }

        require_once 'person/nahlasenazmena.php';
        NahlasenaZmena::zkontrolujPravoNaZmenu($row[1], $row[2], $editace);
    }


    private function download($idZmeny, $idDokumentu)
    {
        $this ->zkontrolujPravoNaDokument($idZmeny, $idDokumentu, false);
        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    /**
     * @param $idZmeny
     * @param $idDokumentu
     * @return Redirect
     * @throws \Exception
     */
    private function smazPripojenyDokument($idZmeny, $idDokumentu)
    {
        $this ->zkontrolujPravoNaDokument($idZmeny, $idDokumentu, true);
        //Kontrola prava na zmenu - pokud nemam, neni treba dale kontrolovat???

        try {
            $params = $this ->database ->createParameters();
            DokumentyUtils::smazDokument($this ->database,
                $idDokumentu, $this, 'DokumentZmeny',
                'IdZmeny = ' . $params ->addParameter($idZmeny), $params);

            MessageManager::addInfoMessage(\Ron\ronText('odstraneni_dokumentu_zmeny_ok'));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return new Redirect(null, '?menuid=nahlasenezmeny&record=' . $idZmeny );
    }


    /**
     * @return DokumentUploadOutput
     */
    function uploadDokument()
    {
        $result = new DokumentUploadOutput();

        try {
            $idZmeny = (int)$_POST['IDZmeny'];

            require_once 'person/nahlasenazmena.php';
            NahlasenaZmena::zkontrolujPravoNaZapsanouZmenu($this ->database, $idZmeny, true);

            // Vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $_POST['typ'], $_POST['popis'], $this);

            // Vlozi vazbu na nahlasenou zmenu
            $rec = new RonPortalDBRecord($this ->database, 'DokumentZmeny', $this);
            $rec->addIntField('Id', true, $id);
            $rec->addIntField('IdZmeny', true, (int)$idZmeny);

            $rec->insertRecord();
        } catch (\Exception $e) {
            $result ->setError($e ->getMessage());
        }

        return $result;
    }


    /**
     * @param string $action
     * @return Redirect|TranslationsOutputObject|ViewObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('person.ini');
        TextManager::getInstance() ->loadStrings('nahlasenezmeny.ini');

        HtmlHeaders::getInstance() ->addScript('script/osobni_udaje.js');
        HtmlHeaders::getInstance() ->addScript('script/nahlasenezmeny.js');

        $this ->addTranslationsScript('nahlasenezmeny');

        $hlaseniAction = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($hlaseniAction) {
            case 'translations':
                return $this->translations();

            case 'dokumentdownload':
                return $this->download((int)$_GET['id'], (int)$_GET['record']);

            case 'dokumentdelete' :
                return $this->smazPripojenyDokument((int)$_GET['id'], (int)$_GET['record']);

            case 'dokumentupload':
                return $this->uploadDokument();
        }

        if (isset($_POST['sys_ulozzmenu']) && $_POST['sys_ulozzmenu']) {
            if (isset($_POST['cancel'])  &&  $_POST['cancel']) {
                return new Redirect(null, '?menuid=osobniudaje&osoba=' . (int)$_POST['idpp']);
            }

            return $this ->odesliHlaseniZmeny();
        }

        if (isset($_GET['record']) && (int)$_GET['record']) {
            return $this ->editaceZmeny((int)$_GET['record']);
        }

        if (isset($_GET['typzmeny'])) {
            // hlaseni nove zmeny - zobrazeni formulare
            return $this->hlaseniZmeny($_GET['typzmeny'], (int)$_GET['osoba'],
                isset($_GET['editace']) ? $_GET['editace'] : null,
                isset($_GET['detail']) ? $_GET['detail'] : null);
        }

        // seznam nahlasenych zmen
        require_once ('person/b_nahlasenezmeny.php');
        $b= new NahlaseneZmenyBrowser($this->database);
        $b->load();
        return $b;
    }
}
