<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 15.08.2019
 */

namespace Ron\RonPortal\Notifikace;


use Ron\DB\Database;
use Ron\MessageManager;
use Ron\XmlLoader;


require_once 'ron/xmlloader.php';

/**
 * Class NotifikaceObjekt
 * @package Ron\RonPortal\Notifikace
 */
class NotifikaceObjekt
{

    public $id;
    public $typ;
    public $poradiGenerovani;

    public $ppCiloveRc;
    public $ppCiloveOsobniCislo;
    public $ppCiloveDatumNastupu;

    public $ppZdrojoveRC;
    public $ppZdrojoveOsobniCislo;
    public $ppZdrojoveDatumNastupu;

    public $prectenoIntraweb;
    public $textNotifikace;
    public $jeTextNotifikace;

    public $insertUser;
    public $insertDate;
    public $updateUser;
    public $updateDate;


    /**
     * @param Database $database
     * @param $id
     * @return NotifikaceObjekt
     * @throws \Exception
     */
    public static function nactiHodnotyNotifikace($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT ID, Typ, PoradiGenerovani, PP_CiloveRC, PP_CiloveOsobniCislo, PP_CiloveDatumNastupu, " .                        //0..5
            "PP_ZdrojoveRC, PP_ZdrojoveOsobniCislo, PP_ZdrojoveDatumNastupu, PrectenoIntraweb, {fn_CASTWIDEMEMO(Notifikace.TextNotifikace)}, " .        //6..10
            "Insert_UserName, Insert_Timestamp, Update_UserName, Update_Timestamp " .                                       //11..14


            "FROM Notifikace " .

            "WHERE ID = " . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if (!$row = $cursor->fetch()){
            throw new \Exception(\Ron\ronText('notifikace_neexistuje'));
        }

        $notifikace = new NotifikaceObjekt();
        $notifikace->id = $id;
        $notifikace->nactiUdajeNotifikaceZRadku($database, $row);
        $cursor->close();
        return $notifikace;
    }


    /**
     * @param Database $database
     * @param $row
     */
    public function nactiUdajeNotifikaceZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->typ = $row[1];
        $this->poradiGenerovani = $row[2];

        $this->ppCiloveRc = $row[3];
        $this->ppCiloveOsobniCislo = $row[4];
        $this->ppCiloveDatumNastupu = $database->stringToTimestamp($row[5]);

        $this->ppZdrojoveRC = $row[6];
        $this->ppZdrojoveOsobniCislo = $row[7];
        $this->ppZdrojoveDatumNastupu = $database->stringToTimestamp($row[8]);

        $this->prectenoIntraweb = $row[9];

        $this->nactiXMLNotifikace($row);

        $this->insertUser = $row[11];
        $this->insertDate = $database->stringToTimestamp($row[12]);
        $this->updateUser = $row[13];
        $this->updateDate = $database->stringToTimestamp($row[14]);
//        var_dump($this->textNotifikace);

    }


    public function nactiXMLNotifikace($row)
    {

        if (!is_null($row[10])) {
            require_once('ron/xmlloader.php');
            $loader = new XmlLoader();
            $loader->includeAttributes = true;
            $loader->prefixToSubarray = false;
            $loader->ignorePrefixes = false;

            $loader->convertFromUtf8 = true;
            $hodnotaXml = $loader->readXml($row[10]);
            $this->textNotifikace = $hodnotaXml['sloupecnotifikace'];
            $this->jeTextNotifikace = !is_null($hodnotaXml['sloupecnotifikace']);
        } else {
            $this->textNotifikace = "";
            $this->jeTextNotifikace = false;
        }
    }
}

