<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 23.11.2017
 */

namespace Ron\RonPortal;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\MVC\HttpController;

require_once('ron/controls.php');
require_once('adsapp/ronportaldbrecord.php');


/**
 * Class PrihlaseniNaPlanovaneSkoleniController
 * @package Ron\RonPortal
 */
class PrihlaseniNaPlanovaneSkoleniController extends HttpController
{
    /** @var  PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;

    /** @var  Database */
    private $database;

    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /**
     * PrihlaseniNaPlanovaneSkoleniController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
        Prava::nastavObdobiProPrava(time(), time());

        TextManager::getInstance()->loadStrings('prihlaseninaplanovanaskoleni.ini');
        HtmlHeaders::getInstance()->addCss("css/skoleni.css");
        HtmlHeaders::getInstance()->addCss("css/s_ron.css");
        HtmlHeaders::getInstance()->addScript('script/skolenihodnoceni.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=skoleni&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);
    }


    /**
     * @param $cisloSkoleni
     * @return Redirect
     * @throws Exception
     */
    public function prihlaseniNaPlanovaneSkoleni($cisloSkoleni)
    {
        if (!$this->pravoPrihlasit->lzeZobrazitRC(User::getRC()) && !$this->pravoSchvalit->lzeZobrazitRC(User::getRC())){
            RonPortalUtils::neniPravoException();
        }

        //Dotaz na volne misto
        $volnaMista  = $this->dotazNaVolnaMista($cisloSkoleni);

        if ($volnaMista > 0) {
            $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob');
            $recZapis->logDescription = StringUtils::fromUTF8('Plánované školení - přihlašování na plánované školení');
            $recZapis->addField('CisloSkoleni', '', false, 10, $cisloSkoleni);
            $recZapis->addField('RC', '', false, 15, User::getRC());
            //Pokud ma pravo schvalit sebe
            $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC(User::getRC());
            $recZapis->addField('Potvrzeni', 'int', false, 0, ($muzeSchvalit) ? 1 : 0);
            $recZapis->addField('DatumVlozeni', 'timestamp', false, 0, time());
            $recZapis->addField('Vlozil', '', false, 25, User::getRC());
            $recZapis->addField('Vyhodnotil', '', false, 25, '');
            $recZapis->addField('PrihlasenoZAplikace', 'int', false, 0, 1);
            $recZapis->addField('DatumVyhodnoceni', 'timestamp', false, 0, '');

            $params = $this->database->createParameters();
            $recZapis->setWhere('RC  = ' . $params->addParameter(User::getRC()) .
                ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni), $params);

            if ($recZapis->loadRecord() == 0) {
                if (!$recZapis->insertRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při přihlašování na plánované školení.'));
                }
            } else {
                throw new Exception(StringUtils::fromUTF8('Chyba při přihlašování na plánované školení - osoba je již přihlášena.'));
            }

        } else {
            MessageManager::addErrorMessage(\Ron\ronText('chyba_kapacita_kurzu_naplnena'));
        }
        return new Redirect(null, '?menuid=skoleni&action=prihlaseninaplanovaneskoleni');
    }


    /**
     * @param $cisloSkoleni
     * @return Redirect
     * @throws Exception
     */
    public function odhlaseniZPlanovanehoSkoleni($cisloSkoleni)
    {
        if (!$this->pravoPrihlasit->lzeZobrazitRC(User::getRC()) && !$this->pravoSchvalit->lzeZobrazitRC(User::getRC())){
            RonPortalUtils::neniPravoException();
        }

        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob');

        $params = $this->database->createParameters();
        $recZapis->logDescription = StringUtils::fromUTF8('Plánované školení - odhlašování z plánovaného školení');
        $recZapis->addField('CisloSkoleni', '', false);
        $recZapis->addField('RC', '', false);
        $recZapis->addField('Potvrzeni', '', false);
        $recZapis->addField('Vlozil', '', false);
        $recZapis->addField('DatumVlozeni', 'timestamp', false);
        $recZapis->addField('Vyhodnotil', '', false);
        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false);
        $recZapis->addField('PrihlasenoZAplikace', '', false);

        $recZapis->setWhere( 'RC  = ' . $params->addParameter(User::getRC()) .
            ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni), $params);

        if (!$recZapis->deleteRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba při odhlašování z plánovaného školení.'));
        }

        $filtrprihlaseninaplanovaneskoleni = $_SESSION['filtrprihlaseninaplanovaneskoleni'];
        return new Redirect(null, "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrprihlaseninaplanovaneskoleni");
    }

    /**
     * @param $cisloSkoleni
     * @return int
     * @throws DBException
     */
    public function dotazNaVolnaMista($cisloSkoleni)
    {
        //Dotaz na volne misto
        $params = $this->database->createParameters();
        $sql = "SELECT PS.PocetOsob, (SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO Where PSO.CisloSkoleni = PS.CisloSkoleni AND PSO.Potvrzeni = 1) 
                FROM PlanovaneSkoleni PS WHERE PS.CisloSkoleni = " . $params->addParameter($cisloSkoleni);
        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $pocetVolnychMist  = (int)($row[0] - $row[1]);

        return $pocetVolnychMist;
    }


    /**
     * @param $cisloSkoleni
     * @param $osoba
     * @param $akce
     * @param null $poznamkaSchvalovani
     * @param null $ajax
     * @return bool|JsonOutputObject|Redirect
     * @throws DBException
     * @throws Exception
     */
    public function schvalitPlanKurzuZamestnance($cisloSkoleni, $osoba, $akce, $poznamkaSchvalovani=null, $ajax=null)
    {
        if (!$cisloSkoleni || !$osoba){
            throw new Exception(\Ron\ronText('chyba_nezadano_cislo_skoleni_nebo_osoba'));
        }

        require_once 'adsapp/ppomer.php';
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);

        if (!$this->pravoPrihlasit->lzeZobrazitPP($pp->idPP) && !$this->pravoSchvalit->lzeZobrazitPP($pp->idPP)){
            RonPortalUtils::neniPravoException();
        }


        //Dotaz na volne misto
        $volnaMista  = $this->dotazNaVolnaMista($cisloSkoleni);

        if ($volnaMista > 0)
        {
            $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob');
            $recZapis->logDescription = 'Planovane skoleni - schvaleni/pripojeni na planovany kurz';

            $recZapis->addField('CisloSkoleni', '', false, 10, $cisloSkoleni);
            $recZapis->addField('RC', '', false, 15, $pp->rc);
            //Pokud ma pravo schvalit danou osobu tak primo schvalit
            $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC($pp->rc);
            $recZapis->addField('Potvrzeni', 'int', false, 0, ($muzeSchvalit ) ? $akce : 0);
            $recZapis->addField('DatumVlozeni', 'timestamp', true, 0, time());
            $recZapis->addField('Vlozil', '', true, 25, User::getRC());
            $recZapis->addField('Vyhodnotil', '', true, 25, '');
            $recZapis->addField('DatumVyhodnoceni', 'timestamp', true, 0, '');
            $recZapis->addField('PrihlasenoZAplikace', 'int', true, 0, 1);
            //pri odhlaseni zustane poznamka zachovana
            if ($poznamkaSchvalovani == ""){
                $recZapis->addField('PoznamkaSchvalovani', 'memo', true, 0, $poznamkaSchvalovani);
            } else {
                $recZapis->addField('PoznamkaSchvalovani', 'memo', false, 0, $poznamkaSchvalovani);
            }

            $params = $this->database->createParameters();
            $recZapis->setWhere('RC  = ' . $params->addParameter($pp->rc) .
                ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni), $params);


            if ($recZapis->loadRecord() == 0) {
                if (!$recZapis->insertRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při přihlašování na plánovaný kurz.'));
                }
            } else {
                if (!$recZapis->loadAndUpdateRecord()) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při aktualizaci na plánované školení - osoba je již přihlášena.'));
                }
            }

        } else {
            MessageManager::addErrorMessage(\Ron\ronText('kapacita_kurzu_naplnena'));
        }

        if ($ajax){
            //Pokud je prihlaseni/schvaleni pomoci dialogu via ajax
            $json = new JsonOutputObject();
            $json->setResult('OK');
            return $json;
        }

        return new Redirect(null, "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni={$_SESSION['filtrprihlaseninaplanovaneskoleni']}");
    }


    /**
     * @param $cisloSkoleni
     * @param $osoba
     * @param $poznamkaSchvalovani
     * @param null $ajax
     * @return bool|JsonOutputObject|Redirect
     * @throws DBException
     * @throws Exception
     */
    public function odhlasitZPlanuKurzuZamestnance($cisloSkoleni, $osoba)
    {
        if (!$this->pravoPrihlasit->lzeZobrazitRC(User::getRC()) && !$this->pravoSchvalit->lzeZobrazitRC(User::getRC())){
            RonPortalUtils::neniPravoException();
        }

        if (!$cisloSkoleni || !$osoba){
            throw new Exception(\Ron\ronText('nezadano_cislo_skoleni_nebo_osoba'));
        }

        require_once 'adsapp/ppomer.php';
        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osoba);

        if (!$this->pravoPrihlasit->lzeZobrazitPP($pp->idPP) && !$this->pravoSchvalit->lzeZobrazitPP($pp->idPP)){
            RonPortalUtils::neniPravoException();
        }

        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob');

        $params = $this->database->createParameters();
        $recZapis->logDescription = StringUtils::fromUTF8('Plánované školení - odhlašování z plánovaného školení');
        $recZapis->addField('CisloSkoleni', '', false);
        $recZapis->addField('RC', '', false);
        $recZapis->addField('Vlozil', '', false);
        $recZapis->addField('Potvrzeni', '', false);
        $recZapis->addField('DatumVlozeni', 'timestamp', false);
        $recZapis->addField('Vyhodnotil', '', false);
        $recZapis->addField('PrihlasenoZAplikace', '', false);
        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false);

        $recZapis->setWhere( 'RC  = ' . $params->addParameter($pp->rc) .
            ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni), $params);

        if (!$recZapis->deleteRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba při odhlášení zaměstnance z plánu kurzu.'));
        }


        $filtrprihlaseninaplanovaneskoleni = $_SESSION['filtrprihlaseninaplanovaneskoleni'];
        return new Redirect(null, "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrprihlaseninaplanovaneskoleni");
    }



    /**
     * @param string $action
     * @return InternalRedirect|OutputObject|Redirect|ViewObject|PrihlaseniNaPlanovaneSkoleniBrowser|SchvalovaniPlanuKurzuPodrizenehoBrowser|SchvalovaniPodrizenychNaPlanuKurzuBrowser|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!($this->pravoPrihlasit->jsouNejakePrava() || $this->pravoSchvalit->jsouNejakePrava())){
            throw new Exception(\Ron\ronText('Nemate_pravo'));
        }

        //Schvalovani planu kurzu z odkazu
        if ($_GET['action'] == 'prihlaseninaplanovaneskoleni' && isset($_GET['plankurzuzamestnance']) ){
            if ($_GET['plankurzuzamestnance'] == 'schvalit'){
                return $this->schvalitPlanKurzuZamestnance($_GET['cisloskoleni'], $_GET['osoba'], 1, StringUtils::fromUTF8($_GET['poznamkaSchvalovani']), $_GET['ajax']);
            } elseif ($_GET['plankurzuzamestnance'] == 'zrusit'){
                return $this->odhlasitZPlanuKurzuZamestnance($_GET['cisloskoleni'], $_GET['osoba']);
            } elseif ($_GET['plankurzuzamestnance'] == 'zamitnout')
                return $this->schvalitPlanKurzuZamestnance($_GET['cisloskoleni'], $_GET['osoba'], 2, StringUtils::fromUTF8($_GET['poznamkaSchvalovani']), $_GET['ajax']);
        }

        //Schvalovani planu kurzu s dialogem s poznamkou
        if ($_POST['ajaxpoznamkakurzu'] == "true" ){
            if ($_POST['plankurzuzamestnance'] == 'schvalit'){
                return $this->schvalitPlanKurzuZamestnance($_POST['cisloskoleni'], $_POST['osoba'], 1, StringUtils::fromUTF8($_POST['poznamkaSchvalovani']), $_POST['ajaxpoznamkakurzu']);
            } elseif ($_POST['plankurzuzamestnance'] == 'zamitnout') {
                return $this->schvalitPlanKurzuZamestnance($_POST['cisloskoleni'], $_POST['osoba'], 2, StringUtils::fromUTF8($_POST['poznamkaSchvalovani']), $_POST['ajaxpoznamkakurzu']);
            }
        }

        $filtrPrihlaseniNaPlanovaneSkoleni = isset($_GET['filtrprihlaseninaplanovaneskoleni']) ? $_GET['filtrprihlaseninaplanovaneskoleni'] : '';
        if ($filtrPrihlaseniNaPlanovaneSkoleni == null && isset($_SESSION['filtrprihlaseninaplanovaneskoleni'])){
            $filtrPrihlaseniNaPlanovaneSkoleni = $_SESSION['filtrprihlaseninaplanovaneskoleni'];
        }

        if ($filtrPrihlaseniNaPlanovaneSkoleni == 'planypodleosoby' && !isset($_GET['plankurzuzamestnance']))
        {
            require_once('person/b_schvalovaniplanukurzuosoby.php');
            $b = new SchvalovaniPlanuKurzuPodrizenehoBrowser($this->database, $filtrPrihlaseniNaPlanovaneSkoleni);
            $b->load();
            return $b;
        }

        if ($filtrPrihlaseniNaPlanovaneSkoleni == 'planypodlekurzu' && !isset($_GET['plankurzuzamestnance']))
        {
            require_once('person/b_schvalovaniosobnaplanukurzu.php');
            $b = new SchvalovaniPodrizenychNaPlanuKurzuBrowser($this->database, $filtrPrihlaseniNaPlanovaneSkoleni);
            $b->load();
            return $b;
        }


        switch ($_GET['action'])
        {
            case 'prihlasit':
                return $this->prihlaseniNaPlanovaneSkoleni($_GET['cisloskoleni']);

            case 'odhlasit':
                return $this->odhlaseniZPlanovanehoSkoleni($_GET['cisloskoleni']);

            case 'translations':
                return $this->translations();

            default:
                require_once('person/b_schvalovaniplanukurzuosoby.php');
                $b = new SchvalovaniPlanuKurzuPodrizenehoBrowser($this->database, $filtrPrihlaseniNaPlanovaneSkoleni);
                $b->load();
                return $b;
        }
    }
}