<?php
/**
 * stanislav.elbl, 14.01.2016
 */

namespace Ron\Personalistika;

use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\FotoOutputObject;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;
use Ron\TextManager;
use RonPortal\Personalistika\KomunikaceUchazeceForm;

require_once ('person/uchazec.php');


class UchazeciController extends HttpController
{
    /**
     * @var Database
     */
    private $database;
    /**
     * @var \Ron\RonPortal\PravaNaOsoby
     */
    private $prava;
    /**
     * @var \Ron\RonPortal\PravaNaOsoby
     */
    private $pravaKomunikace;

    /**
     * UchazeciController constructor.
     * @param Database $database datab�ze
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->pravaKomunikace = Prava::vratPrava('uchazeci_komunikace');
        $this->prava = Prava::vratPrava('uchazeci_prohl');
        Prava::nastavObdobiProPrava(time(),time());
    }


    /**
     * @param $rc
     * @return BaseViewObject BaseViewObject
     * @throws \Exception
     */
    public function uchazecView($rc)
    {
        $uchazec = Uchazec::nactiUchazece($this->database,$rc);

        Model::addData('uchazec', $uchazec);
        Model::addData('pravaKomunikace', $this->pravaKomunikace);

        return new BaseViewObject('person/view_uchazec.php');
    }


    /**
     * @return KomunikaceUchazeceForm
     * @throws \Exception
     */
    public function uchazeciKomunikaceForm()
    {

        if (!($this->pravaKomunikace->jsouNejakePrava())) {
            throw new \Exception(\Ron\ronText('Nemate_pravo'));
        }

        $formAction = isset($_POST['formaction'])
            ? $_POST['formaction']
            : $_GET['formaction'];

        switch ($formAction) {
            case 'new' :
                $uchazec = Uchazec::nactiUchazece($this->database,$_GET['uchazec']);
                break;

            case 'edit' :
                if (User::getRC() != UchazecKomunikace::vratRCPersonalistyPodleID($this->database, $_GET['record'])) {
                    throw new \Exception(\Ron\ronText('Nemate_pravo'));
                }
                $uchazec = Uchazec::nactiUchazece($this->database, UchazecKomunikace::vratRCUchazecePodleID($this->database,$_GET['record']));
                break;

            case 'insert' :
                $this->vytvorPersonalistu(); // vytvori personalistu z prihlasene osoby
                $uchazec = Uchazec::nactiUchazece($this->database, $_POST['RCUchazece']);
                break;

            case 'save' :
                if (User::getRC() != UchazecKomunikace::vratRCPersonalistyPodleID($this->database, $_POST['record'])) {
                    throw new \Exception(\Ron\ronText('Nemate_pravo'));
                }
                $uchazec = Uchazec::nactiUchazece($this->database, UchazecKomunikace::vratRCUchazecePodleID($this->database,$_POST['IdKomunikace']));
                break;

            default:
                throw new \Exception('Chyba v UchazeciController::uchazeciKomunikaceForm!');
        }

        $f = new KomunikaceUchazeceForm($this ->database, $uchazec);

        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=uchazeci&action=zobrazuchazece&record='.$uchazec->rc);
        }
        return $f;
    }

    /**
     * @param $rc
     * @param $id
     * @return DokumentOutputObject
     * @throws \Exception
     */

    public function downloadDocument($rc, $id)
    {
        $params = $this->database->createParameters();

        $sql = 'SELECT ID ' .
                'FROM DokumentUchazece ' .
                'WHERE DokumentUchazece.ID = ' . $params->addParameter((int)$id) .
                ' AND IDUchazece = ' . $params->addParameter($rc);

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row){
            throw new \Exception(\Ron\ronText('Nemate_pravo'));
        }

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($this->database,$id);
    }

    /**
     * @param string $rc
     * @return DokumentOutputObject
     */
    public function downloadFoto($rc)
    {
        require_once('adsapp/fotoutils.php');

        $result = new FotoOutputObject($this->database);
        $result->nactiIdFotkyOsoby($rc,'P_Uchazec');
        return $result;
    }

    private function zapisZobrazeniUchazece($rc)
    {
        require_once('adsapp/ronportaldbrecord.php');

        $recHistorie = new RonPortalDBRecord($this->database,'HistorieUchazece', $this);

        $recHistorie->addField('ID','int',false,0,SQLUtils::newSerialNumber($this->database, 'HistorieUchazece', 'ID'));
        $recHistorie->addField('RCUchazece', '', false,15,$rc);
        $recHistorie->addField('Cas', 'timestamp', false, 0, time());
       //$recHistorie->addField('RCOsoby', '', false, 15, S_GetUserRC());
        $recHistorie->addField('Popis', 'memo', false, 0,  sprintf(\Ron\ronText('Uchazec_zobrazeni'),User::getUserName()));
        $recHistorie->addField('Insert_UserName', '', false, 25, User::getLoginName());
        $recHistorie->addField('Insert_Timestamp','timestamp',false,0,time());

        if(!$recHistorie->insertRecord()){
            throw new \Exception('Chyba p�i z�pisu zobrazen� uchaze�e.');
        }
    }

    private function vytvorPersonalistu()
    {
        require_once('adsapp/ronportaldbrecord.php');

        $params = $this->database->createParameters();
        $sql = "SELECT Jmeno, Prijmeni, Titul, TitulZa FROM Osoba WHERE RC = ". $params->addParameter(User::getRC());

        $cursor = $this->database->getSqlResult($sql,$params);

        if (!$row = $cursor->fetch()) {
            throw new \Exception('Prihlaseny_uzivatel_nenalezen');
        }

        $recPersonalista = new RonPortalDBRecord($this->database,'P_Osoba');
        $recPersonalista->logDescription = 'Zobrazen� uchaze�e - port�l';
        $recPersonalista->addField('RC','',false,15,User::getRC()); //puvodne S_GetUserRC()
        $recPersonalista->addField('Jmeno','',false,30,$row[0]);
        $recPersonalista->addField('Prijmeni','',false,30,$row[1]);
        $recPersonalista->addField('Titul','',false,20,$row[2]);
        $recPersonalista->addField('TitulZa','',false,20,$row[3]);
        $recPersonalista->addField('Insert_UserName','',false,25, User::getLoginName()); // S_GetLoginName());
        $recPersonalista->addField('Insert_Timestamp','timestamp',false,0,time());
        $recPersonalista->where = "RC = '" . $this->database->addSlashes(User::getRC()) . "'";

        if ($recPersonalista->loadRecord() == 0) {
            if (!$recPersonalista->insertRecord()) {
                throw new \Exception('Chyba p�i vytvo�en� personalisty (Z�pis komunikace uchaze�e).');
            }
        }
    }


    /**
     * @param string $action
     * @return string
     * @throws \Exception
     */
    public function handleAction($action)
    {
    	if (!(($this->prava->jsouNejakePrava() || $this->pravaKomunikace->jsouNejakePrava()) && RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_UCHAZECE))) {
    		throw new \Exception(\Ron\ronText('Nemate_pravo'));
    	}
        HtmlHeaders::getInstance()->addCss('./css/person.css');
        TextManager::getInstance() ->loadStrings('person.ini');

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'download':
                    return $this->downloadDocument($_GET['record'], $_GET['dokument']);
                case 'downloadfoto':
                    return $this->downloadFoto($_GET['record']);
                case 'zobrazuchazece':
                    $this->zapisZobrazeniUchazece($_GET['record']);
                    return $this->uchazecView($_GET['record']);
                case 'komunikace':
                    return $this->uchazeciKomunikaceForm();
            }
        }

        require_once('person/b_uchazeci.php');
        $b = new UchazeciBrowser($this ->database);
        $b ->load();
        return $b;
    }
}
