<?php
/**
 * stanislav.elbl, 28.02.2016
 */

namespace Ron\Personalistika;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\RonPortal\RonPortalDBRecord;

class VzdelaniOsoby
{
    public $nazevSkoly;
    public $kodSkoly;
    public $oborVzdelani;
    public $oborVzdelaniKod;
    public $datumUkonceni;
    public $ukonceniVzdelani;
    public $ukonceniVzdelaniKod;
    public $typVzdelani;
    public $typVzdelaniKod;
    public $popisUrovneVzdelani;
    public $popisTypuVzdelani;
    public $skolaOd;
    public $skolaDo;
    public $vzdelani;
    public $mistoStudia;
    public $oborVzdelaniISPV;

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiVzdelaniZRadku($database, $row)
    {
        $this->nazevSkoly = $row[0];
        $this->kodSkoly = $row[9];
        $this->oborVzdelani = $row[1];
        $this->oborVzdelaniKod = $row[7];
        $this->ukonceniVzdelani = $row[2];
        $this->ukonceniVzdelaniKod = $row[8];
        $this->popisUrovneVzdelani= $row[3];
        $this->skolaOd = $database->stringToTimestamp($row[4]);
        $this->skolaDo = $database->stringToTimestamp($row[5]);
        $this->popisTypuVzdelani = $row[6];
        $this->datumUkonceni = $database->stringToTimestamp($row[10]);
        $this->typVzdelaniKod = $row[11];
        $this->mistoStudia = $row[12];
        $this->oborVzdelaniISPV = $row[13];
    }

    /**
     * @param Database $database
     * @param $rc
     * @param string $tableName      VzdelaniOsoby nebo VzdelaniUchazece
     * @return VzdelaniOsoby[]
     * @throws \Exception
     */
    public static function nactiVzdelani($database, $rc, $tableName)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Skola.Nazev, OborVzdelani.Popis, S_ZpusobUkonceniSkoly.Popis, UrovenVzdelani.Popis, ' . // 0..3
            "$tableName.DatumOd, $tableName.DatumDo, TypVzdelani.Popis, OborVzdelani.Kod, S_ZpusobUkonceniSkoly.Kod, " . // 4..8
            "Skola.Kod, $tableName.DatumUkonceni, TypVzdelani.Kod, $tableName.MistoStudia, $tableName.OborVzdelaniISPV " . // 9..13
            "FROM $tableName JOIN  OborVzdelani ON ($tableName.OborVzdelani = OborVzdelani.Kod) " .
            " JOIN TypVzdelani ON ($tableName.KodVzdelani = TypVzdelani.Kod) " .
            ' JOIN UrovenVzdelani ON (TypVzdelani.UrovenVzdelani = UrovenVzdelani.Uroven) ' .
            " JOIN S_ZpusobUkonceniSkoly ON ($tableName.VzdelaniUkonceno = S_ZpusobUkonceniSkoly.Kod)".
            " LEFT JOIN Skola ON ($tableName.Skola = Skola.Kod)" .
            ' WHERE  RC = ' . $params->addParameter($rc) .
            " ORDER BY $tableName.DatumOd DESC, $tableName.DatumDo  DESC";

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql,$params);

        $seznamVzdelani = [];
        while($row = $cursor->fetch())
        {
            $vzdelani = new VzdelaniOsoby();
            $vzdelani ->nactiVzdelaniZRadku($database, $row);
            $seznamVzdelani[] = $vzdelani;
        }

        return $seznamVzdelani;
    }

    /**
     * @param Database  $database
     * @param string    $rc
     * @param string    $tableName
     * @param int $index
     * @param object|string $logDescription
     * @return bool|int
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function ulozVzdelani($database, $rc, $tableName, $index = null, $logDescription = 'VzdelaniOsoby')
    {
        $insert = false;
        $loaded = false;

        require_once('adsapp/ronportaldbrecord.php');

        $vzdelaniRec = new RonPortalDBRecord($database, $tableName, $logDescription);

        $vzdelaniRec->addField('RC','',true, 15);
        $vzdelaniRec->addField('KodVzdelani', 'int', true);
        $vzdelaniRec->addField('OborVzdelani','', true, 10);
        $vzdelaniRec->addField('DatumUkonceni', 'timestamp', true);
        $vzdelaniRec->addField('Skola', '', false, 10);
        $vzdelaniRec->addField('DatumOd', 'timestamp', false);
        $vzdelaniRec->addField('DatumDo', 'timestamp', false);
        $vzdelaniRec->addField('Zamereni', '', false, 40);
        $vzdelaniRec->addField('VzdelaniUkonceno', 'int', false);

        $vzdelaniRec->where =
            self::getWhereCondition($rc, $database, $index);

        //pri vkladani noveho zaznamu neexistuji puvodni hodnoty, provadi se jen insert a nenacita se tedy zaznam z DB
        if ($vzdelaniRec->where == '') {
            $vzdelaniRec->findFieldByName('RC')->setNewValue($rc);
            $insert = true;
        }
        else {
            $loaded = $vzdelaniRec->loadRecord();
        }

        self::nactiPost($vzdelaniRec, $index);

        //pokud nejsou zadana povinna pole vraci -1
        if (($vzdelaniRec['KodVzdelani'] == null) || ($vzdelaniRec['OborVzdelani'] == null) || ($vzdelaniRec['DatumUkonceni'] == null)) {
            return -1;
        }

        //zaznam o vzdelani jeste u osoby neexistuje, nebo ve formulari nebyl nacten zaznam pro editaci, provadi se vlozeni
        if ($insert) {
            $vzdelaniRec->insertRecord();
            return true;
        }

        if ($loaded === false) {
            throw new \Exception('Chyba pri nacitani zaznamu vzdelani.');
        }
        if (!$vzdelaniRec->isChanged()) {
            return 0;
        }
        if ($loaded) {
            return $vzdelaniRec->updateRecord();
        }

        return 0;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @param int $index
     */
    private static function nactiPost($rec, $index)
    {
        $poradi = '';

        if (!is_null($index)) {
            $poradi = '_' . $index;
        }

        $rec->findFieldByName('KodVzdelani')->setNewValue((int)$_POST['vzdelani_kod'.$poradi]);
        $rec->findFieldByName('OborVzdelani')->setNewValue($_POST['vzdelani_obor'.$poradi]);

        if ($_POST['vzdelani_datum_ukonceni'.$poradi] != '') {
            $rec->findFieldByName('DatumUkonceni')->setNewValue(DateUtils::dateStringToTimestamp($_POST['vzdelani_datum_ukonceni' . $poradi]));
        }
        if ($_POST['vzdelani_skola'.$poradi] != '') {
            $rec->findFieldByName('Skola')->setNewValue($_POST['vzdelani_skola' . $poradi]);
        }

        if ($_POST['vzdelani_datum_od'.$poradi] != '') {
            $rec->findFieldByName('DatumOd')->setNewValue(DateUtils::dateStringToTimestamp($_POST['vzdelani_datum_od' . $poradi]));
        }

        if ($_POST['vzdelani_datum_do' . $poradi] != '') {
            $rec->findFieldByName('DatumDo')->setNewValue(DateUtils::dateStringToTimestamp($_POST['vzdelani_datum_do' . $poradi]));
        }
        $rec->findFieldByName('Zamereni')->setNewValue($_POST['vzdelani_zamereni'.$poradi]);
        $rec->findFieldByName('VzdelaniUkonceno')->setNewValue((int)$_POST['vzdelani_ukonceno'.$poradi]);
    }

    /**
     * @param string $rc
     * @param Database $database
     * @param int $index
     * @return string
     */
    private static function getWhereCondition($rc, $database, $index)
    {
        $poradi = '';

        if (!is_null($index)) {
            $poradi = '_' . $index;
        }

        if ((isset($_POST['vzdelani_kod_puvodni'.$poradi]) && isset($_POST['vzdelani_obor_puvodni'.$poradi]) && isset($_POST['vzdelani_datum_ukonceni_puvodni'.$poradi]))
            && (($_POST['vzdelani_kod_puvodni'.$poradi] != '') && ($_POST['vzdelani_obor_puvodni'.$poradi] != '') && ($_POST['vzdelani_datum_ukonceni_puvodni'.$poradi] != ''))
        ) {
            return
                "RC = '" . $database->addSlashes($rc) . "'" .
                " AND KodVzdelani = " . (int)$_POST['vzdelani_kod_puvodni' . $poradi] .
                " AND OborVzdelani = '" . $database->addSlashes($_POST['vzdelani_obor_puvodni' . $poradi]) . "'" .
                " AND DatumUkonceni = " . $database->timestampToString($_POST['vzdelani_datum_ukonceni_puvodni' . $poradi]);
        }

        return '';
    }
}
