<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 10:12
 */

namespace Ron\RonPortal\PokladniKniha;

require_once('ron/browser.php');
require_once('adsapp/ronportalfilter.php');

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use function Ron\ronText;

class PokladniKnihaFilter extends RonPortalFilterForm
{
    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;

    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    /**
     * PokladniKnihaFilter constructor.
     * @param Database $database
     * @param string $action
     * @param SeznamPolozekPokladniKnihy $seznamPolozekPokladniKnihy
     */
    public function __construct($database, $action, $seznamPolozekPokladniKnihy)
    {
        $this->seznamPolozekPokladniKnihy = $seznamPolozekPokladniKnihy;

        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' . $this->seznamPolozekPokladniKnihy->cislo);
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $this->seznamPolozekPokladniKnihy->cislo);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' . $this->seznamPolozekPokladniKnihy->cislo);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $this->seznamPolozekPokladniKnihy->cislo);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $this->seznamPolozekPokladniKnihy->cislo);

        parent::__construct($database, $action);

        $this->f_formaction = '?menuid=' . $action . '&browser=' . $this->seznamPolozekPokladniKnihy->cislo;

        $this->nastavOsoby(true, [$this->pravaProhlizet, $this->pravaVlozit, $this->pravaSchvalit1, $this->pravaSchvalit2, $this->pravaSchvalit3], time(),time());
        $this->povolitVsechyOsoby = true;

        $this->nastavDatumOdDo(true,false);

        $this->nastavRozdeleni(true);
    }

    /**
     * Filtr na typ polozky pokladni knihy
     * @throws DBException
     */
    private function renderTypPolozky()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypPolozkyPK', ronText('jen_typ') . '&nbsp;&nbsp;', isset($_SESSION['jenTypPolozkyPK']) && $_SESSION['jenTypPolozkyPK'], 'onClick="submit()"');

        echo '</SPAN>';

        // combobox pro vyber
        $params = $this->database->createParameters();

        $sql =
            'SELECT Kod, Popis FROM TypPolozkyPokladniKnihy ';

        if (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL))
            $sql .=
                'WHERE ' . SQLUtils::getINCondition($this->database,'Kod',$this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params);

        $sql .=
            ' ORDER BY Popis';

        // vyber typu udalosti podle popisu
        $combo = new ComboBox('typPolozkyPK');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->loadData($this ->database, $sql, isset($_SESSION['typPolozkyPK']) ? $_SESSION['typPolozkyPK'] : '', $params);
        $combo ->htmlParams = 'onChange="if (jenTypPolozkyPK.checked) submit();"';
        $combo->render();

        echo "</DIV>\n";
    }

    private function renderFilterStav()
    {
        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption">' . ronText('PoklK_uroven_schvaleni') . '</span>';

        $data = array(
            0 => ronText('Vse'),
            -1 => ronText('PoklK_ke_schvaleni'),
            -2 => ronText('PoklK_bez_schvaleni'),
            1 => ronText('PoklK_schvaleni_uroven1'),
            2 => ronText('PoklK_schvaleni_uroven2'),
            3 => ronText('PoklK_schvaleni_uroven3')
        );

        LinksList::renderLinks($data, isset($_SESSION['urovenschvaleniPoklK']) ? $_SESSION['urovenschvaleniPoklK'] : 0, $this ->f_formaction .  '&urovenschvaleni=', '', false);

        echo "</div>\n";
    }

    /**
     * Vypise vsechny filtry
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();

        $this->renderTypPolozky();
        $this->renderFilterStav();

    }

    private function loadTypPolozky()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypPolozkyPK'] = isset($_POST['jenTypPolozkyPK']);

            $_SESSION['typPolozkyPK'] = $_POST['typPolozkyPK'];
        }
    }

    private function loadUrovenSchvaleni()
    {
        if (isset($_GET['urovenschvaleni'])) {
            $_SESSION['urovenschvaleniPoklK'] = $_GET['urovenschvaleni'];
        }
    }

    function loadData()
    {
        parent::loadData();

        $this->loadTypPolozky();
        $this->loadUrovenSchvaleni();
    }

}

class SchvaleniZaznamuPKField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->align = 'left';

        $this->header = ronText('PoklK_schvaleni');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hint = '';

        if ($rowData[15]) {
            $date = $rowData[15];
            $name = $rowData[16];
            $class = 'textschvaleni3';
        } else if ($rowData[13]) {
            $date = $rowData[13];
            $name = $rowData[14];
            $class = 'textschvaleni2';
        } else if ($rowData[11]) {
            $date = $rowData[11];
            $name = $rowData[12];
            $class = 'textschvaleni1';
        }  else {
            return '';
        }

        $result = "<SPAN class='$class' title='$hint'>";
        if ($date) $result .= DBUtils::getFormatedValue($this ->database, 'date', $date);
        if ($name) $result .= " ($name)";
        $result .= "</SPAN>\n";

        return $result;
    }
}

class SchvaleniZaznamuPKEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $indexSchvaleno;
    var $indexDatum;
    var $indexUzivatel;
    var $htmlIndex;

    /** @var  PokladniKnihaBrowser */
    var $browser;

    var $existsEdit;


    /**
     * @param Browser $browser
     * @param PravaNaOsoby $prava
     * @param int $indexSchvaleno
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param int $htmlIndex
     * @param string $caption
     */
    function __construct($browser, $prava, $indexSchvaleno, $indexDatum, $indexUzivatel, $htmlIndex, $caption)
    {
        parent::__construct($browser ->database);
        $this ->align = "center";

        $this ->header =
            "<span id='schvaleniPKMenu_{$htmlIndex}''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->htmlIndex = $htmlIndex;
        $this ->prava = $prava;
        $this ->indexSchvaleno = $indexSchvaleno;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection DuplicatedCode  */
        $schvaleno = isset($this ->indexSchvaleno) ? $rowData[$this ->indexSchvaleno] : $rowData[$this ->indexDatum];

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
        }

        if ($this ->prava ->lzeZobrazitPP($rowData[27])) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}_{$this ->htmlIndex}' value='1' />" .
                "<input type='checkbox' data-ronschvalenipoklk_index='{$this ->browser ->jsonRowIndex}' name='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' " .
                    "id='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . ronText($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL_{$this ->htmlIndex}' id='schvaleni_ALL_{$this ->htmlIndex}' title='' />\n";
        }
        else return '';
    }

}

class ZruseniPKField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /**
     * @param Database $database
     * @param PravaNaOsoby $prava
     */
    function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->prava = $prava;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[11] || $rowData[13] || $rowData[15]) return ''; // jiz je schvaleno, nejde smazat

        // zkontroluje prava na editaci zastupu
        if ($this->prava->lzeZobrazitPP($rowData[27], $this ->database ->stringToTimestamp($rowData[1]), $this ->database ->stringToTimestamp($rowData[1]))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('PoklK_zrusit_zaznam') . '"></i>', ronText('PoklK_opravdu_zrusit'),
                '?menuid=pokladnikniha&action=zrusitzaznam&record=' . $rowData[0]);
        }

        return '';
    }
}

class DokumentyPKField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[28]) {
            return
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . ronText('Pocet_priloh') . ': ' . $rowData[28] . '"></i>';
        }

        return '';
    }
}

class PokladniKnihaBrowser extends Browser
{
    /** @var  PokladniKnihaFilter */
    private $filterForm;
    private $schvalovani;
    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;

    public $jsonRowIndex;
    public $jsonObject;
    public $hiddenFields;

    /**
     * PokladniKnihaBrowser constructor.
     * @param $database
     * @param $cisloBrowseru
     * @param bool $schvalovani
     * @throws DBException
     */
    public function __construct($database, $cisloBrowseru, $schvalovani = false)
    {
        parent::__construct($database);

        $this->seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $cisloBrowseru);

        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' . $cisloBrowseru);
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $cisloBrowseru);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' . $cisloBrowseru);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $cisloBrowseru);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $cisloBrowseru);

        $this->title = ronText('pokladni_kniha' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''));

        $this->browserUrl = '?menuid=pokladnikniha&';
        $this->code = 'browserpokladnikniha';
        $this->tableId = 'seznampokladniknihy';

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //6 - 10
            'PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, ' . //11, 12
            'PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //13, 14
            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //15, 16
            'PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //17, 18
            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' . //19 - 23
            'Osoba.Jmeno, Osoba.Prijmeni, TypPolozkyPokladniKnihy.Popis, PracovniPomer.ID, ' . //24 - 27
            '(SELECT COUNT(*) FROM DokumentPokladniKnihy WHERE IDPokladniKnihy = PokladniKniha.ID), ' . //28
            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH'; //29 - 30

        $this->sqlFromPart =
            'PokladniKniha  ' .
            'JOIN Osoba ON (Osoba.RC = PokladniKniha.RC) ' .
            'JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky) ' .
            'JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ';

        $this->primaryField = 0;

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'Prijmeni', BROWSER_FIELD_SQL_INDEX => 25, BROWSER_FIELD_SQL_NAME => 'Osoba.Prijmeni',
            BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_LINK => '?menuid=pokladnikniha&action=zaznam', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'Jmeno', BROWSER_FIELD_SQL_INDEX => 24, BROWSER_FIELD_SQL_NAME => 'Osoba.Jmeno',
            BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_LINK => '?menuid=pokladnikniha&action=zaznam', BROWSER_FIELD_ORDERING => 1],

            [BROWSER_FIELD_HEADER_TEXT => 'datum_pohybu', BROWSER_FIELD_SQL_INDEX => 1,
                BROWSER_FIELD_SQL_NAME => 'PokladniKniha.DatumPohybu', BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ALIGN => 'right',
                BROWSER_FIELD_LINK => '?menuid=pokladnikniha&action=zaznam', BROWSER_FIELD_ORDERING => 1],
        ]);

        $this->defaultSort = count($this->fields);

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'typ_polozky', BROWSER_FIELD_SQL_INDEX => 26,
                BROWSER_FIELD_SQL_NAME => 'TypPolozkyPokladniKnihy.Popis', BROWSER_FIELD_ALIGN => 'left' ],

            [BROWSER_FIELD_HEADER_TEXT => 'popis' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''), BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'PokladniKniha.Popis',
            BROWSER_FIELD_ALIGN => 'left' ],
        ]);

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField([BROWSER_FIELD_HEADER_TEXT => 'castkaBezDPH', BROWSER_FIELD_SQL_INDEX => 29,
                BROWSER_FIELD_SQL_NAME => 'PokladniKniha.CastkaBezDPH', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'number2']);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField([BROWSER_FIELD_HEADER_TEXT => 'castkaDPH', BROWSER_FIELD_SQL_INDEX => 30,
                BROWSER_FIELD_SQL_NAME => 'PokladniKniha.CastkaDPH', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'number2']);
        }

        $this->addFields([
            [BROWSER_FIELD_HEADER_TEXT => 'castka', BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'PokladniKniha.Castka',
            BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_FORMAT => 'number2'],

            [BROWSER_FIELD_HEADER_TEXT => 'mena', BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'PokladniKniha.Mena',
            BROWSER_FIELD_ALIGN => 'left'],

            /*[BROWSER_FIELD_HEADER_TEXT => '', BROWSER_FIELD_SQL_INDEX => 0, BROWSER_FIELD_SQL_NAME => 'PokladniKniha.ID',
            BROWSER_FIELD_ALIGN => 'center', BROWSER_FIELD_LINK => '?menuid=pokladnikniha&action=zaznam', BROWSER_FIELD_ORDERING => 1],*/

        ]);

        if ($this->schvalovani) {
            $this->addField(new SchvaleniZaznamuPKEditField($this, $this->pravaSchvalit1, null, 11, 12, '1',
                ronText('PoklK_schvaleni_uroven1' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))));
            $this->addField(new SchvaleniZaznamuPKEditField($this, $this->pravaSchvalit2, null, 13, 14, '2',
                ronText('PoklK_schvaleni_uroven2' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))));
            $this->addField(new SchvaleniZaznamuPKEditField($this, $this->pravaSchvalit3, null, 15, 16, '3',
                ronText('PoklK_schvaleni_uroven3' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))));
        } else {
            $this->addField(new SchvaleniZaznamuPKField($this ->database));
        }

        $this->addField(new DokumentyPKField());
        $this->addField(new ZruseniPKField($this->database, $this->pravaVlozit));
    }

    /**
     * @param string $sqlFilter
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        $this->filterForm = new PokladniKnihaFilter($this->database,'pokladnikniha', $this->seznamPolozekPokladniKnihy);
        $this->filterForm->loadData();

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        $this->filterForm->render();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava())
               $this->renderNewRecord();

            echo "<div id='poklk_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . ronText('PoklK_Schvalit_hromadne') . "'/></div>";
        }
    }

    function renderNewRecord()
    {
        echo "<div class='normaltext poklk_links'><a href='?menuid=pokladnikniha&browser=" . $this->seznamPolozekPokladniKnihy->cislo . "&formaction=new'>" . ronText('New_record') . "</a></div>\n";
    }

    function getSQLFilter()
    {
        $result = '(1 = 1 ) ';

        $params = $this->database->createParameters();

        if (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL)) {
            $result .= ' AND ' . SQLUtils::getINCondition($this->database, 'PokladniKniha.TypPolozky', $this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params);
        }
        $result .= ' AND ';

        $urovenSchvaleni = isset($_SESSION['urovenschvaleniPoklK']) ? $_SESSION['urovenschvaleniPoklK'] : 0;
        // uroven schvaleni (vse, ke schvaleni, ...)
        switch ($urovenSchvaleni) {
            case -2 : // bez schvaleni
                $result .= 'PokladniKniha.Schvaleni1 IS NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL  ';
                break;

            case 1 :
                $result .= 'PokladniKniha.Schvaleni1 IS NOT NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL ';
                break;

            case 2 :
                $result .= 'PokladniKniha.Schvaleni2 IS NOT NULL  AND  PokladniKniha.Schvaleni3 IS NULL ';
                break;

            case 3 :
                $result .= 'PokladniKniha.Schvaleni3 IS NOT NULL ';
                break;

            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
                $result .= 'PokladniKniha.Schvaleni3 IS NULL ';
                // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")

                if (!$this->pravaSchvalit3->jsouNejakePrava()) {
                    $result .= ' AND PokladniKniha.Schvaleni2 IS NULL ';
                    // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje

                    if (!$this->pravaSchvalit2->jsouNejakePrava()) {
                        $result .= ' AND PokladniKniha.Schvaleni1 IS NULL ';
                        // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
                    }
                }
                break;

            default : // vse
                $result .= ' 1 = 1 ';
        }

        $datumOd = DateUtils::today();
        $datumDo = $datumOd;

        if ($this->filterForm->filtrovatDatum) {
            $datumOd = $this->filterForm->datumOd;
            $datumDo = $this->filterForm->datumDo;

            $result .=
                ' AND PokladniKniha.DatumPohybu <= ' . $params->addParameter($this->database->timestampToParam($datumOd)) .
                ' AND PokladniKniha.DatumPohybu >= ' . $params->addParameter($this->database->timestampToParam($datumDo));
        }

        if (!$this->filterForm->vsechnyOsoby) {
            $result .=
                " AND PokladniKniha.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) .
                " AND PokladniKniha.OsobniCislo = " . $params->addParameter($this->filterForm->vybranaOsobaOsobniCislo) .
                " AND PokladniKniha.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->filterForm->vybranaOsobaDatumNastupu));
        }

        if ($this->filterForm->kodRozdeleni != '') {
            require_once('adsapp/rozdeleni.php');
            $result .= ' AND ' . Rozdeleni::sqlFilterRozdeleni($this->database, 'PokladniKniha', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                   $datumOd, $datumDo, $this->filterForm->vcetnePodrizenychRozdeleni, $params);

        }

        if (isset($_SESSION['jenTypPolozkyPK']) && $_SESSION['jenTypPolozkyPK']) {
            $result .=
                " AND PokladniKniha.TypPolozky = " . $params->addParameter($_SESSION['typPolozkyPK']);
        }

        return new SqlStatement($result, $params);
    }

    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function poklkEnableLine($values, $datumod, $datumdo)
    {
        $this ->jsonRowIndex = null;

        if (isset($_SESSION['urovenschvaleniPoklK']) && ($_SESSION['urovenschvaleniPoklK'] == -1)) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values[15]) return false;
            // jiz je schvaleno nejvyssi urovni = neceka na nikoho

            if ($values[13]) return $this->pravaSchvalit3->lzeZobrazitPP($values[27], $datumod, $datumdo);
            // je schvaleno 2, ceka na 3 => pokud mam pravo, ceka na me

            if ($values[11]) return $this->pravaSchvalit2->lzeZobrazitPP($values[27], $datumod, $datumdo);

            // je schvaleno 1, ceka na 2 => pokud mam pravo, ceka na me

            return $this->pravaSchvalit1->lzeZobrazitPP($values[27], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo na 1, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
            if ($this->pravaProhlizet->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        }

        if ($this->pravaSchvalit1->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit2->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit3->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $datumod = $this ->database ->stringToTimestamp($values[1]);
        $datumdo = $this ->database ->stringToTimestamp($values[1]);

        $result = $this ->poklkEnableLine($values, $datumod, $datumdo);
        if (!$result) return false;

        if ($this ->schvalovani) {

            $this ->jsonRowIndex = count($this ->jsonObject['items']);

            $this ->hiddenFields .= "<input type='hidden' name='idPoklK_{$this ->jsonRowIndex}' value='{$values[0]}'/>";

            $this ->jsonObject['items'][] = [
                'id' => $values[0],
                'pravo_1' => $this ->pravaSchvalit1 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'pravo_2' => $this ->pravaSchvalit2 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'pravo_3' => $this ->pravaSchvalit3 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'schvaleno_1' => $values[11] ? 1 : 0,
                'schvaleno_2' => $values[13] ? 1 : 0,
                'schvaleno_3' => $values[15] ? 1 : 0
            ];

        } else {
            $this ->jsonRowIndex = null;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values[15]) $className .= ' poklk_schvalena3';
        else if ($values[13]) $className .= ' poklk_schvalena2';
        else if ($values[11]) $className .= ' poklk_schvalena1';

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="poklk_row_' . $values[0] . '"';

        return $result;
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[17]) {
            // zobrazi poznamku k vraceni
            echo "<tr class='$className'>";
            echo    '<td></td><td><div class="browserpoznamka" style="text-align:right"><strong>' . ronText('PoklK_zaznam_vracen') . '</strong>: </div></td>'.
                    '<td colspan="' . ($this->columnCount - 2) . '"><div class="browserpoznamka">' . $values[17] . '</div></td>';
            echo '</tr>';
        }
    }

    function renderContent()
    {
        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        $this ->hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniPoklK" action="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';
        }

        echo
            '<ul class="popupmenu"  id="schvaleniPoklKMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . ronText('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . ronText('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . ronText('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->hiddenFields) echo $this ->hiddenFields;
        echo '<input type="hidden" name="pocet" value="' . count($this ->jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . ronText('Provest') . "' id='provestHromadneSchvaleniPoklK' />" .
                '<input type="button" class="button" value="' . ronText('Cancel') . '" id="stornoHromadneSchvaleniPoklK" 
                    data-browserlink="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '" />' .
                '</div>'.
                "</form>\n";

            /** @noinspection BadExpressionStatementJS */
            echo
                '<script type="text/javascript">' .
                'ron.PokladniKniha.hromadneSchvaleni.init(' . json_encode($this ->jsonObject) . ')' .
                '</script>';
        } else {
            $pravo = Prava::vratHodnotuPrava('pkniha_vloz', 'pkniha_vloz_' . $this->seznamPolozekPokladniKnihy->cislo);
            if ($pravo) $this->renderNewRecord();
        }
    }

}