<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 13:21
 */
namespace Ron\RonPortal\PokladniKniha;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\DBRecord;
use Ron\FormField;
use Ron\RonPortal\OsobaField;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\UserLoginField;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalform.php');
require_once('adsapp/events.php');
require_once('adsapp/ppomer.php');

class CastkaFormField extends FormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo '<span id="castka_pokladni_knihy">' . $value . '</span>';
    }
}

class PracovniPomerPoklKnihaFormField extends OsobaField
{
    private $idPP;

    public function __construct($caption, $prava, $datumPlatnostiPP, $edit, $idPP = null)
    {
        parent::__construct($caption, $prava, $datumPlatnostiPP, $edit);

        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!is_null($this->idPP))
            $value = $this->idPP;

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

class PokladniKnihaForm extends RonPortalForm
{
    public $id;

    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    /**
     * PokladniKnihaForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param SeznamPolozekPokladniKnihy $browser
     * @param null|int $idPP
     * @throws Exception
     */
    public function __construct($database, $formAction, $browser, $idPP = null)
    {
        parent::__construct($database, $formAction);

        $this->seznamPolozekPokladniKnihy = $browser;

        $this ->htmlAction = '?menuid=pokladnikniha';
        $this ->stornoAction = '?menuid=pokladnikniha&browser='.$this->seznamPolozekPokladniKnihy->cislo;

        $this ->title = $formAction == 'new' ? ronText('New_record') //. ' ' . \Ron\ronText('pokladni_kniha' . ($browser->cislo != 1 ? "_$browser->cislo" : ''))
            : ronText('PoklK_editace_zaznamu' . ($browser->cislo != 1 ? "_$browser->cislo" : ''));

        $this ->sqlTableName = 'PokladniKniha';
        $this ->dbLogSrcDescription = StringUtils::fromUTF8('Pokladní kniha');

        $this->sqlSelectPart =
            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, ' . //6 - 9
            'PokladniKniha.IDDokumentu, PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //10 - 14
            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //15 - 18
            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' .  //19 - 23
            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH '; //24 - 25

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_ID, FORM_FIELD_NAME => 'ID', FORM_FIELD_PRIMARY => true, FORM_FIELD_SQL_INDEX => 0 ]);

        $this->addField(new PracovniPomerPoklKnihaFormField(ronText('Osoba'), 'pkniha_vloz', time(), false, $idPP));

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_CAPTION_TEXT => 'datum_pohybu',
            FORM_FIELD_REQUIRED => true, FORM_FIELD_SQL_INDEX => 1, FORM_FIELD_SQL_NAME => 'DatumPohybu',  FORM_FIELD_NAME => 'DatumPohybu',
            FORM_FIELD_EDIT => true, FORM_FIELD_MAXLENGTH => 10,
            FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',DateUtils::startOfTheDay(time())),
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'typ_polozky',
            FORM_FIELD_REQUIRED => true, FORM_FIELD_ROW_ID => 'typ_polozky', FORM_FIELD_SQL_NAME => 'TypPolozky',  FORM_FIELD_NAME => 'TypPolozky',
            FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 7,
            FORM_FIELD_SQL => 'SELECT TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.Popis FROM TypPolozkyPokladniKnihy WHERE ' .
                SQLUtils::getINCondition($this->database, 'TypPolozkyPokladniKnihy.Kod', $this->seznamPolozekPokladniKnihy->seznamTypuSQL),
        ]);

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_FLOAT, FORM_FIELD_CAPTION_TEXT => 'castkaBezDPH',
                FORM_FIELD_REQUIRED => false, FORM_FIELD_ROW_ID => 'row_castka_bez_dph', FORM_FIELD_SQL_NAME => 'CastkaBezDPH',
                FORM_FIELD_NAME => 'CastkaBezDPH', FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 24,
            ]);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_FLOAT, FORM_FIELD_CAPTION_TEXT => 'castkaDPH',
                FORM_FIELD_REQUIRED => false, FORM_FIELD_ROW_ID => 'row_castka_dph', FORM_FIELD_SQL_NAME => 'CastkaDPH', FORM_FIELD_NAME => 'CastkaDPH',
                FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 25,
            ]);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastku()) {
            $this->addField([
                FORM_FIELD_TYPE => FORM_FIELD_TYPE_FLOAT, FORM_FIELD_CAPTION_TEXT => 'castka', FORM_FIELD_REQUIRED => true,
                FORM_FIELD_SQL_NAME => 'Castka', FORM_FIELD_NAME => 'castka', FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 5,
            ]);
        }
        
        //needitovatelne pole s celkovu castkou - defaultne skryte, zobrazi se misto editovatelne castky (pokud nelze editovat)
        $this->addField(new CastkaFormField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_FLOAT, FORM_FIELD_CAPTION_TEXT => 'castka', FORM_FIELD_ROW_ID => 'poklkniha_castka_noedit',
            FORM_FIELD_REQUIRED => false, FORM_FIELD_EDIT => false]));

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_CAPTION_TEXT => 'mena', FORM_FIELD_REQUIRED => true,
            FORM_FIELD_SQL_NAME => 'Mena',  FORM_FIELD_NAME => 'Mena', FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 6,
            FORM_FIELD_SQL => 'SELECT Mena.KodMeny_ISO, Mena.KodISO FROM Mena WHERE Mena.Aktivni = 1', FORM_FIELD_DEFAULT_VALUE => RonPortalSettings::getValue('poklk_vychozi_mena')
        ]);

        $this->addField([
            FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_CAPTION_TEXT => 'popis' . ($browser->cislo != 1 ? "_$browser->cislo" : ''), FORM_FIELD_REQUIRED => true,
            FORM_FIELD_SQL_NAME => 'Popis',  FORM_FIELD_NAME => 'Popis', FORM_FIELD_EDIT => true, FORM_FIELD_SQL_INDEX => 8,
        ]);

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Update_Timestamp',  FORM_FIELD_NAME => 'Update_Timestamp', FORM_FIELD_EDIT => true ]);
        $this->addField(new UserLoginField('Update_UserName', true));

        $this->addField([ FORM_FIELD_TYPE => FORM_FIELD_TYPE_ACTUAL_TIME, FORM_FIELD_SQL_NAME => 'Insert_Timestamp',  FORM_FIELD_NAME => 'Insert_Timestamp', FORM_FIELD_EDIT => false ]);
        $this->addField(new UserLoginField('Insert_UserName', false));
    }



    function createDBRecordFields($rec, $insert)
    {
        $pracPomer = PracovniPomer::nactiPP($this->database, (int)$_POST['osoba']);

        $rec->addField('RC','',true,15, $pracPomer->rc);
        $rec->addField('OsobniCislo','',true,15, $pracPomer->osobniCislo);
        $rec->addField('DatumNastupu','timestamp',true,0, $pracPomer->datumNastupu);

        parent::createDBRecordFields($rec, $insert);
    }


    /** Pokud je typ pohybu vydej, nastavi castku na zapornou.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws DBException
     */
    private function nastavPodleTypuPohybu($rec)
    {
        $params = $this->database->createParameters();
        $jeVydej = (int)DBUtils::getSqlValue($this->database, "SELECT DruhPohybu FROM TypPolozkyPokladniKnihy WHERE Kod = " . $params->addParameter($rec['TypPolozky']), $params) == 2;
        //pro vydej bude castka zaporna
        if ($jeVydej) {
            $rec['Castka'] = - abs($rec['Castka']);
        }
        else {
            $rec['Castka'] = abs($rec['Castka']);
        }

        return $rec;
    }

    /** Overi, zda jsou zadana pole, ktera jsou u typu polozky povolena a pokud je povolena castka bez DPH a castka DPH secte je do pole castka.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws Exception
     */
    private function overVypoctiPodleTypuPohybu($rec)
    {
        $typPolozky = $this->seznamPolozekPokladniKnihy->vratTypPolozky($rec['TypPolozky']);
        
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && ($rec['CastkaBezDPH'] == '')) {
            throw new Exception(ronText('Nezadana_castka_bez_dph'));
        }

        if ($typPolozky->lzeZobrazitCastkuDPH() && ($rec['CastkaDPH'] == '')) { 
            throw new Exception(ronText('Nezadana_castka_dph'));
        }

        //zadava se castka bez dph a celkova castka, castka dph se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && !$typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaBezDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_bez_dph_vyssi'));
            }
            $rec['CastkaDPH'] = $rec['Castka'] - $rec['CastkaBezDPH'];
        }

        //zadava se castka dph a celkova castka, castka bez dph se dopocte
        if (!$typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_dph_vyssi'));
            }
            $rec['CastkaBezDPH'] = $rec['Castka'] - $rec['CastkaDPH'];
        }

        //zadava se castka bez dph a castka dph, celkova castka se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            $rec['Castka'] = $rec['CastkaBezDPH'] + $rec['CastkaDPH'];
        }
        
        return $rec;
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeInsertRecord($rec)
    {
        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeUpdateRecord($rec)
    {
        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'update',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    public function afterInsertRecord($rec)
    {
        $this->id = $rec['ID'];
        
        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    public function afterUpdateRecord($rec)
    {
        $this->id = $rec['ID'];

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'update',null,false);
    }

    public function showCommonFields()
    {
        parent::showCommonFields();
        
        echo '<input type="hidden" name="browser" value="' . $this->seznamPolozekPokladniKnihy->cislo . '" />';
    }

    public function showForm($formAction = '')
    {
        parent::showForm($formAction);

        echo
            '<script type="text/javascript">' .
            'ron.PokladniKniha.formular.init(' . json_encode($this ->seznamPolozekPokladniKnihy->jsonObject) . ');' .
            '</script>';
    }


}