<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:41
 */

namespace Ron\RonPortal\PokladniKniha;

use Exception;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\RonPortal\Prava;
use function Ron\ronText;

class PokladniKniha
{
    public $datumPohybu;
    public $osobniCislo;
    public $castka;
    public $mena;
    public $typPolozky;
    public $popis;
    public $poznamka;
    public $schvaleni;
    public $schvalil;
    public $poznamkaKVraceni;
    public $historieSchvalovani;
    public $obdobiVyuctovani;
    public $idPP;
    public $castkaBezDPH;
    public $castkaDPH;
    public $cisloBrowseru;

    public $typPolozkyKod;
    private $id;
    private $rc;
    private $jmeno;
    private $prijmeni;
    private $titul;

    public function __construct()
    {
        $this->schvaleni = [];
        $this->schvalil = [];
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->rc = $row[2];
        $this->jmeno = $row[22];
        $this->prijmeni = $row[23];
        $this->titul = $row[24];

        $this->datumPohybu = $database->stringToTimestamp($row[1]);
        $this->osobniCislo = $row[3];
        $this->castka = $row[5];
        $this->mena = $row[6];
        $this->typPolozky = $row[20];
        $this->popis = $row[8];
        $this->poznamka = $row[9];
        $this->schvaleni[1] = $database->stringToTimestamp($row[11]);
        $this->schvalil[1] = $row[12];
        $this->schvaleni[2] = $database->stringToTimestamp($row[13]);
        $this->schvalil[2] = $row[14];
        $this->schvaleni[3] = $database->stringToTimestamp($row[15]);
        $this->schvalil[3] = $row[16];
        $this->poznamkaKVraceni = $row[17];
        $this->historieSchvalovani = $row[18];
        $this->obdobiVyuctovani = $row[19];
        $this->idPP = $row[21];

        $this->castkaBezDPH = $row[25];
        $this->castkaDPH = $row[26];
        $this->typPolozkyKod = $row[27];
        $this->cisloBrowseru = $row[28] ?: 1;
    }

    /**
     * @param Database $database
     * @param int $id id zaznamu
     * @return PokladniKniha|false
     * @throws DBException
     */
    public static function nactiZaznam($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, ' . //0 - 4
            ' PokladniKniha.Castka, PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //5 - 10
            ' PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //11 - 16
            ' PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, PokladniKniha.ObdobiVyuctovani, TypPolozkyPokladniKnihy.Popis, ' . //17 - 20
            ' PracovniPomer.ID, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . //21 - 24
            ' PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.RonPortalBrowser ' . //25 - 28
            'FROM PokladniKniha JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky)' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) ' .
            'WHERE PokladniKniha.ID = ' . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            $zaznam = new PokladniKniha();
            $zaznam->nactiDataZRadku($database, $row);
            return $zaznam;
        }

        return false;
    }

    /**
     * @return string Prijmeni Jmeno, Titul
     */
    public function vratJmenoPrijmeni()
    {
        return $this->prijmeni . ' ' . $this->jmeno . ($this->titul ? ', ' . $this->titul : '');
    }

    /**
     * @return int id zaznamu pokladni knihy
     */
    public function vratID()
    {
        return $this->id;
    }

    /**
     * @param bool $raise vyhazovat vyjimku
     * @return bool
     * @throws Exception
     */
    public function lzeEditovat($raise = true)
    {
        // kontrola schvaleni (nelze editovat schvalene, ale pokud mam pravo na vyssi uroven, tak muzu)
        $schvalena = false;

        try {
            if ($this->schvaleni[3]) {
                $schvalena = true;
            } else if ($this->schvaleni[2]) {
                // pokud mam pravo na uroven 3, tak muzu editovat
                $prava = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $this->cisloBrowseru);
                $schvalena = !$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu);
            } else if ($this->schvaleni[1]) {
                $prava = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $this->cisloBrowseru);
                if (!$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
                    // nemam pravo na schvalovani 2, jeste zkontroluje schvalovani 3
                    $prava = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $this->cisloBrowseru);
                    $schvalena = !$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu);
                }
            }

            if ($schvalena) {
                throw new Exception(ronText('PoklK_zaznam_schvalen'));
            }

            // kontrola prav
            $prava = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $this->cisloBrowseru);
            if (!$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
                throw new Exception(ronText('Neni_pravo_editovat_poklk'));
            }
        }
        catch (Exception $exception) {
            if ($raise) {
                throw $exception;
            }

            return false;
        }

        return true;
    }

    /**
     * Vraci true, pokud je zaznam schvalen zadanym levelem nebo vyssim
     * @param $minimalniLevel
     * @return bool
     */
    public function jeZaznamSchvalen($minimalniLevel = 1)
    {
        if ($this ->schvaleni[3]) {
            return true;
        }

        if ($this ->schvaleni[2]) return $minimalniLevel <= 2;
        if ($this ->schvaleni[1]) return $minimalniLevel <= 1;

        return false;
    }

    /**
     * @param Database $database
     * @param int $id
     * @return Cursor 0 = id pokladni knihy, 1 = id dokumentu
     * @throws DBException
     */
    public static function nactiPripojeneDokumenty($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT IDPokladniKnihy, ID 
             FROM DokumentPokladniKnihy 
             WHERE IDPokladniKnihy = ' . $params->addParameter((int)$id);

        return $database->getSqlResult($sql, $params);
    }
}