<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:43
 */

namespace Ron\RonPortal\PokladniKniha;

require_once('adsapp/ronportaldbrecord.php');
require_once('adsapp/events.php');

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DBRecordField;
use Ron\MessageManager;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\RonPortalXMLEvents;
use Ron\RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

class PokladniKnihaRecord extends RonPortalDBRecord
{
    /** @var int */
    public $id;

    /** @var  int */
    public $idpp; // id PP pro kontrolu prav

    /** @var DBRecordField */
    public $datumpohybufield;
    /** @var DBRecordField */
    public $rcfield;

    /** @var DBRecordField */
    public $typPolozkyfield;

    /** @var DBRecordField */
    public $schvaleni1;
    /** @var DBRecordField */
    public $schvaleni2;
    /** @var DBRecordField */
    public $schvaleni3;

    /** @var DBRecordField */
    public $historiefield; // pole historie

    /** @var int cislo browseru pro kontrolu prav */
    public $cisloBrowseru;


    public function __construct($database, $id)
    {
        parent::__construct($database, 'PokladniKniha');
        $this->id = (int)$id;

        $this->logDescription = StringUtils::fromUTF8('Pokladní kniha');
        $this->where = 'PokladniKniha.Id = ' . (int)$id;

        // je nutne pridat jmeno tabulky PokladniKniha., protoze se pripojuje jeste PracovniPomer
        $this->addField('PokladniKniha.Id', 'int', true); // readonly

        $this->rcfield = $this->addField('PokladniKniha.RC', '', true); // readonly
        $this->addField('PokladniKniha.OsobniCislo', '', true);
        $this->addField('PokladniKniha.DatumNastupu', 'date', true);

        $this->datumpohybufield = $this->addField('PokladniKniha.DatumPohybu', 'date', true);

        $this->typPolozkyfield = $this->addField('PokladniKniha.TypPolozky','string',true);

        $this->addField('PokladniKniha.Popis', '', true);

        $this->schvaleni1 = $this->addField('PokladniKniha.Schvaleni1', 'datetime', true);
        $this->schvaleni2 = $this->addField('PokladniKniha.Schvaleni2', 'datetime', true);
        $this->schvaleni3 = $this->addField('PokladniKniha.Schvaleni3', 'datetime', true);

        $this->addField('PokladniKniha.Poznamka', 'memo', true);

        $this->historiefield = $this->addField('PokladniKniha.HistorieSchvalovani', 'memo', false); // readonly = false
        $this->historiefield->updateLog = true;
    }

    /**
     * @param $row
     * @return bool
     * @throws Exception
     */
    public function loadRecordValues($row)
    {
        if (!$this ->id) {
            throw new Exception(StringUtils::fromUTF8('Chyba - není zadáno ID záznamu pokladní knihy!'));
        }

        parent::loadRecordValues($row);

        $this ->idpp = $row[count($this ->fields)];
        $this ->cisloBrowseru = $row[count($this->fields)+1];

        return true;
    }



    public function getSelectSQL($filtr = '', $targetParams = null, $filtrParams = null)
    {
        $sql =
            'SELECT ' . $this ->getSelectPartSQL() . ', PracovniPomer.Id, TypPolozkyPokladniKnihy.RonPortalBrowser ' . // PP.Id predposledni!
            '  FROM PokladniKniha, PracovniPomer, TypPolozkyPokladniKnihy ' .
            ' WHERE PokladniKniha.RC = PracovniPomer.RC ' .
            '   AND PokladniKniha.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND PokladniKniha.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky' .
            '   AND (' . $this ->where . ')';

        if ($filtr) $sql .= ' AND (' . $filtr . ')';

        return $sql;
    }

    /**
     * @param $pravastr
     * @param bool $zruseni
     * @throws Exception
     */
    private function zkontrolujPrava($pravastr, $zruseni = false)
    {
        $prava = Prava::vratPrava($pravastr, $pravastr . '_' . $this->cisloBrowseru);
        if ($zruseni  &&  $prava ->priznak) {
            // je zaskrtnuto "zakazat zrusit schvaleni"
            throw new Exception(ronText('Nemate_pravo'));
        }

        Prava::nastavObdobiProPrava($this ->datumpohybufield ->value, $this ->datumpohybufield ->value);

        if (!$prava ->lzeZobrazitPP($this ->idpp)) {
            throw new Exception(ronText('Nemate_pravo'));
        }
    }


    private function xmlEventsSC($eventName, $xmlContext)
    {
        return $this ->executeXmlEvents($eventName, $xmlContext, ['pokladnikniha' => $this ->id]);
    }

    /**
     * @param $uroven
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function schval($uroven)
    {
        $pomf = $this ->addField('Schvalil' . $uroven, '', false, 0, RonPortalUtils::getLogUserName());
        $pomf ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        // schvaleni nastavuje az po LoadRecord, aby se zbytecne nevolal update, pokud uz bylo schvaleno
        // (neni to totiz priznak, ale datum)
        switch ($uroven) {
            case '1' :
                if ( !$this ->schvaleni1 ->value)  $this ->schvaleni1 ->setNewValue(time());
                break;

            case '2' :
                if ( !$this ->schvaleni2 ->value)  $this ->schvaleni2 ->setNewValue(time());
                break;

            case '3' :
                if ( !$this ->schvaleni3 ->value)  $this ->schvaleni3 ->setNewValue(time());
                break;

        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava("pkniha_schval$uroven");

        $this ->historiefield ->value = $this ->historiefield ->originalValue . "\n<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . sprintf(ronText('Schvaleni_poklk_log'), '' . $uroven);

        $this ->logText = StringUtils::fromUTF8('Schválení záznamu pokladní knihy');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'schvaleni' . $uroven . '_before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'schvaleni' . $uroven . '_after');
    }

    /**
     * @param $uroven
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function zrusSchvaleni($uroven)
    {
        switch ($uroven) {
            case '1' :
                $this ->schvaleni1 ->setNewValue(null);
                break;

            case '2' :
                $this ->schvaleni2 ->setNewValue(null);
                break;

            case '3' :
                $this ->schvaleni3 ->setNewValue(null);
                break;

        }

        $f = $this ->addField('Schvalil1', '', $uroven != 1, 0, null);
        $f ->updateLog = true;
        $f = $this ->addField('Schvalil2', '', $uroven != 2, 0, null);
        $f ->updateLog = true;
        $f = $this ->addField('Schvalil3', '', $uroven != 3, 0, null);
        $f ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava("pkniha_schval$uroven", true);

        if ( ( $uroven <= 2 && $this ->schvaleni3 ->value )
            || ( $uroven == 1 && $this ->schvaleni2 ->value )
        ) {
            throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy je již schválen vyšší úrovní - schválení nelze zrušit!'));
        }

        $this ->historiefield ->value = $this ->historiefield ->originalValue . "\n<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . sprintf(ronText('Odschvaleni_poklk_log'), '' . $uroven);

        $this ->logText = StringUtils::fromUTF8('Zrušeno schválení záznamu pokladní knihy');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'zrusitschvaleni' . $uroven . '_before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'zrusitschvaleni' . $uroven . '_after');
    }

    /**
     * Zkontroluje, jestli uzivatel muze vratit schvaleni zadane urovne
     * @param int $uroven
     * @return bool
     * @throws Exception
     */
    function lzeVratitZaznam($uroven)
    {
        // vetsina chyb by nemela vubec nastat
        // - nektere pouze v pripade, ze by 2 uzivatele soubezne pracovali se stejnym zaznamem

        if ($this ->schvaleni3 ->value) {
            // jiz je schvalena uroven 3 - nelze vratit
            throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - schvalování záznamu pokladní knihy již bylo dokončeno!'));
        }

        switch ($uroven) {
            case 2 :
                if (!$this ->schvaleni2 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 2 není schválena!'));
                }
                break;

            case 1 :
                if ($this ->schvaleni2 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 2 je již schválena!'));
                }

                if (!$this ->schvaleni1 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 1 není schválena!'));
                }
                break;

            default:
                throw new Exception(StringUtils::fromUTF8('Chyba - špatně zadaná úroveň schválení při vrácení záznamu pokladní knihy! (' . $uroven . ')'));
        }


        $this ->zkontrolujPrava('pkniha_schval' . ($uroven + 1));

        return true;
    }

    /**
     * @param int    $uroven
     * @param string $poznamka
     * @return bool
     * @throws Exception
     */
    function vratSchvaleni($uroven, $poznamka)
    {
        $this ->addField('PoznamkaKVraceni', '', false, 100, $poznamka);

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba při vrácení schválení - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        if (!$this ->lzeVratitZaznam($uroven)) return false;

        switch ($uroven) {
            case 1 :
                $this ->schvaleni1 ->setNewValue(null);
                break;

            case 2 :
                $this ->schvaleni2 ->setNewValue(null);
                break;

            default :
                throw new Exception(StringUtils::fromUTF8('Chyba při vrácení schválení pokladní knihy - špatně zadaná úroveň schválení!'));
        }

        $this ->historiefield ->setNewValue(
            $this ->historiefield ->originalValue .
            "\n<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . sprintf(ronText('Vraceni_poklk_log'), '' . $uroven) .
            "\n> " . $poznamka
        );

        $this ->logText = StringUtils::fromUTF8('Vráceno schválení záznamu pokladní knihy');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'vratitschvaleni' . $uroven . '_before')) return false;
        if (!$this ->updateRecord()) return false;
        return $this ->xmlEventsSC(RonPortalXMLEvents::POKLK_SCHVALENI, 'vratitschvaleni' . $uroven . '_after');
    }

    /**
     * @param Database $database
     * @return bool
     */
    public static function hromadneSchvaleni($database)
    {
        // uzivatel odeslal hromadne schvaleni zaznamu Pokladni knihy
        $pocet = (int)$_POST['pocet'];

        $result = true;

        for ($i = 0;  $i < $pocet;  $i++) {
            $id = $_POST["idPoklK_$i"];
            $rec = new PokladniKnihaRecord($database, $id);

            try {
                if (isset($_POST["NastavitSchvaleni_{$id}_1"]) && $_POST["NastavitSchvaleni_{$id}_1"]) {
                    if ($_POST["Schvaleni_{$id}_1"]) {
                        if (!$rec->schval(1)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(1)) $result = false;
                    }
                }

                if (isset($_POST["NastavitSchvaleni_{$id}_2"]) && $_POST["NastavitSchvaleni_{$id}_2"]) {
                    if ($_POST["Schvaleni_{$id}_2"]) {
                        if (!$rec->schval(2)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(2)) $result = false;
                    }
                }

                if (isset($_POST["NastavitSchvaleni_{$id}_3"]) && $_POST["NastavitSchvaleni_{$id}_3"]) {
                    if ($_POST["Schvaleni_{$id}_3"]) {
                        if (!$rec->schval(3)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(3)) $result = false;
                    }
                }
            } catch (Exception $e) {
                $result = false;
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (!$result) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Při hromadném schvalování záznamů pokladní knihy došlo k chybě!'));
        }

        return $result;
    }

}