<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 9:24
 */

namespace Ron\RonPortal\PokladniKniha;

require_once('adsapp/b_dokumenty.php');
require_once('adsapp/dokumentyutils.php');
require_once('adsapp/ronportaldbrecord.php');
require_once('pokladnikniha/pokladniknihautils.php');
require_once('pokladnikniha/pokladnikniha_record.php');

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonPortal\DokumentOutputObject;
use Ron\RonPortal\DokumentUploadOutput;
use Ron\RonPortal\DokumentyUtils;
use Ron\RonPortal\NeniPravoException;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use function Ron\ronText;

class PokladniKnihaController extends HttpController
{
    const POKLADNI_KNIHA_POVOLENA = 'povolitpokladniknihu';
    
    /** @var Database */
    private $database;

    private $cisloBrowseru;
    
    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;

    /**
     * PokladniKnihaController constructor.
     * @param $database
     * @throws Exception
     */
    public function __construct($database)
    {
        $this->database = $database; 
        TextManager::getInstance() ->loadStrings('pokladnikniha.ini');

        HtmlHeaders::getInstance()->addScript('script/pokladnikniha.js');

        $this->cisloBrowseru =
            isset($_POST['browser']) ? $_POST['browser'] :
            (isset($_GET['browser']) ? $_GET['browser'] : $_SESSION['pokladniKnihaCisloBrowseru']);

        if (is_null($this->cisloBrowseru))
            $this->cisloBrowseru = 1;

        Prava::nactiPravaUzivatele($this->database,User::getRC());
        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' .$this->cisloBrowseru);
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' .$this->cisloBrowseru);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' .$this->cisloBrowseru);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' .$this->cisloBrowseru);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' .$this->cisloBrowseru);
        Prava::nastavObdobiProPrava(time(),time());
    }

    /** Pokud ma jakekoliv pravo na zaznam, vrati true.
     * @param int $idZaznamu id zaznamu pokladni knihy
     * @param bool $vratZaznam pokud je true vrati i nacteny zaznam, jinak vrati jen true
     * @return false|PokladniKniha pokud je pravo na zaznam PK vrati ho, jinak vypise chybu
     * @throws Exception
     */
    private function jePravoNaZaznam($idZaznamu, $vratZaznam = false)
    {
        require_once('pokladnikniha/pokladnikniha.php');
        $zaznam = PokladniKniha::nactiZaznam($this->database, $idZaznamu);

        if ($zaznam === false) {
            throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy nebyl nalezen.'));
        }

        /* 1.6.2016 kontrola prav se lisi jen browserem, ne typem polozky
        $suffixPrava = $this->cisloBrowseru != 1 ? ('_' . $this->cisloBrowseru) : '';

        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1','pkniha_schval1' . $suffixPrava . '_'.$zaznam->typPolozky);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2','pkniha_schval2' . $suffixPrava . '_'.$zaznam->typPolozky);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3','pkniha_schval3' . $suffixPrava . '_'.$zaznam->typPolozky);*/

        if (!(
            $this->pravaProhlizet->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaVlozit->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit1->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit2->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit3->lzeZobrazitPP($zaznam->idPP)
        )) {
            throw new Exception('Nemate_pravo');
        }

        if ($vratZaznam) {
            return $zaznam;
        }
        return true;
    }

    /**
     * @param int $idDokumentu
     * @param int $idZaznamu
     * @return DokumentOutputObject
     * @throws Exception
     * @throws DBException
     */
    private function downloadDokument($idDokumentu, $idZaznamu)
    {
        if (!$this->jePravoNaZaznam($idZaznamu)) {
            throw new Exception('Nemate_pravo');
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT IDDokumentu FROM PokladniKniha JOIN DokumentPokladniKnihy ON (DokumentPokladniKnihy.IDPokladniKnihy = PokladniKniha.ID) " .
            " JOIN Dokument ON (Dokument.id = DokumentPokladniKnihy.ID) ".
            " JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.typ)" .
            "WHERE PokladniKniha.ID = " . $params->addParameter((int)$idZaznamu) .
            " AND Dokument.ID = " . $params->addParameter((int)$idDokumentu) .
            " AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        $cursor = $this->database->getSqlResult($sql,$params);

        $row = $cursor->fetch();

        if (!$row) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        return new DokumentOutputObject($this->database,$idDokumentu);
    }

    /** Nahraje dokument a priradi ho zaznamu pokladni knihy
     * @return DokumentUploadOutput
     */
    function uploadDokument()
    {
        $result = new DokumentUploadOutput();

        try {
            // zkontroluje prava
            $idOsoby = (int)$_POST['idosoby'];

            if (!$this->pravaVlozit->lzeZobrazitPP($idOsoby)) {
                RonPortalUtils::neniPravoException();
            }

            // vlozi dokument do databaze
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', RonPortalSettings::getIntValue('poklk_typ_dokumentu'), $_POST['popis'], StringUtils::fromUTF8('Pokladní kniha'));

            $idPokladniKnihy = (int)$_POST['id'];

            // Vlozi id dokumentu k zaznamu pokladni knihy
            require_once('adsapp/ronportaldbrecord.php');
            $rec = new RonPortalDBRecord($this ->database, 'DokumentPokladniKnihy');
            $rec ->addField('ID', 'int', true, 0, $id);

            $rec->addField('IDPokladniKnihy', 'int', true, 0, (int)$idPokladniKnihy);

            $rec ->insertRecord();
        } catch (Exception $e) {
            $result ->setError($e ->getMessage());
        }

        return $result;
    }

    /**
     * @param int $idZaznamu id zaznamu pokladni knihy
     * @param bool $vraceni
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailZaznamu($idZaznamu, $vraceni = false)
    {
        $zaznam = $this->jePravoNaZaznam($idZaznamu, true);

        if (!($zaznam instanceof PokladniKniha)) {
            throw new Exception(StringUtils::fromUTF8('Chyba při načítání záznamu pokladní knihy'));
        }

        $dokumentForm = new NovyDokumentPokladniKnihyForm($this->database, $idZaznamu, $zaznam->idPP);

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $this->cisloBrowseru);
        
        Model::addData('pokladnikniha',$zaznam);
        Model::addData('vraceni',$vraceni);
        Model::addData('pravaVlozit', $this->pravaVlozit);
        Model::addData('pravaSchvalit1', $this->pravaSchvalit1);
        Model::addData('pravaSchvalit2', $this->pravaSchvalit2);
        Model::addData('pravaSchvalit3', $this->pravaSchvalit3);
        Model::addData('dokumentForm', $dokumentForm);
        Model::addData('seznamPolozekPoklK', $seznamPolozekPokladniKnihy);

        $dokumentyBrowser = new DokumentyPokladniKnihyBrowser($this->database, $idZaznamu);
        $dokumentyBrowser->load();

        Model::addData('dokumentyBrowser', $dokumentyBrowser);

        return new BaseViewObject('pokladnikniha/pokladnikniha_detail.php');
    }

    /**
     * @param $formAction
     * @return Redirect|PokladniKnihaForm
     * @throws Exception
     */
    private function pokladniKnihaFormular($formAction)
    {
        if (!$this->pravaVlozit->jsouNejakePrava()) {
            throw new Exception('Nemate_pravo');
        }

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $this->cisloBrowseru);
        
        require_once('pokladnikniha/f_pokladnikniha.php');

        $zaznam = null;
        
        if (($formAction == 'edit') || ($formAction == 'save')) {
            $idZaznamu = isset($_POST['record']) ? $_POST['record'] : $_GET['record'];
            require_once('pokladnikniha/pokladnikniha.php');
            $zaznam = PokladniKniha::nactiZaznam($this->database, $idZaznamu);

            if ($zaznam === false) {
                throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy nebyl nalezen.'));
            }

            if (!$zaznam->lzeEditovat()) {
                throw new NeniPravoException();
            }

        }

        $form = new PokladniKnihaForm($this->database, $formAction, $seznamPolozekPokladniKnihy, $zaznam instanceof PokladniKniha ? $zaznam->idPP : null);

        if ($form->handleAction())
        {
            return $this->redirectToView($form->id, $this->cisloBrowseru);
        }

        return $form;
    }

    /**
     * @param int $id
     * @param int $uroven
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function schvaleni($id, $uroven)
    {
        $rec = new PokladniKnihaRecord($this ->database, $id);
        $result = $rec ->schval($uroven);

        if ($result) {
            return $this ->redirectToView($id);
        } else {
            return $this ->zobrazDetailZaznamu($id);
        }
    }

    /**
     * @param int $id
     * @param int $uroven
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function zrusSchvaleni($id, $uroven)
    {
        $rec = new PokladniKnihaRecord($this ->database, $id);
        try {
            if ($rec->zrusSchvaleni((int)$uroven)) {
                return $this->redirectToView($id);
            } else {
                return $this->zobrazDetailZaznamu($id);
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
            return $this->zobrazDetailZaznamu($id);
        }
    }

    /**
     * @param int $id
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function vratSchvaleni($id)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba - není zadán ID záznamu pokladní knihy!'));

        if (!isset($_POST['poznamkakvraceni']) || !$_POST['poznamkakvraceni']) {
            return $this ->zobrazDetailZaznamu($id, true);
        }

        $uroven = (int)$_POST['urovenvraceni'];
        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při vrácení - není zadána úroveň schválení!'));
            return $this ->zobrazDetailZaznamu($id, true);
        }

        $rec = new PokladniKnihaRecord($this ->database, $id);
        if ($rec ->vratSchvaleni($uroven, $_POST['poznamkakvraceni'])) {
            return $this ->redirectToView($id);
        } else {
            return $this ->zobrazDetailZaznamu($id);
        }
    }

    /** Smaze zaznam pokladni knihy, pokud neni schvalen.
     * @param int $id id zaznamu pokladni knihy
     * @return Redirect
     * @throws Exception
     */
    private function smazZaznam($id)
    {
        $zaznam = $this->jePravoNaZaznam($id, true);

        $browser = $zaznam->cisloBrowseru;

        try {
            $zaznam->lzeEditovat();

            $seznamDokumentuCursor = PokladniKniha::nactiPripojeneDokumenty($this->database, $id);

            while ($row = $seznamDokumentuCursor->fetch()) {
                try {
                    $this->smazDokument($row[1]);
                }
                catch (Exception $e) {
                    MessageManager::addErrorMessage($e);
                }
            }

            $rec = new PokladniKnihaRecord($this->database, $id);
            if ($rec->deleteRecord()) {
                MessageManager::addInfoMessage(ronText('PoklK_zruseni_ok'));
            } else {
                MessageManager::addDbErrorMessage($this->database, null, StringUtils::fromUTF8('Při rušení záznamu došlo k chybě!'));
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return new Redirect('','?menuid=pokladnikniha&browser=' . $browser);
    }

    /** Smaze dokument prirazeny k zaznamu pokladni knihy, pokud neni schvalena.
     * @param int $id id zaznamu pokladni knihy
     * @param int $idDokumentu
     * @return Redirect
     * @throws Exception
     */
    private function smazPripojenyDokument($id, $idDokumentu)
    {
        $zaznam = $this->jePravoNaZaznam($id, true);

        try {
            $zaznam->lzeEditovat();

            $seznamDokumentuCursor = PokladniKniha::nactiPripojeneDokumenty($this->database, $id);

            $dokumentPripojen = false;

            while ($row = $seznamDokumentuCursor->fetch()) {
                if ($row[1] == $idDokumentu) {
                    $dokumentPripojen = true;

                    try {
                        if ($this->smazDokument($idDokumentu)) {
                            MessageManager::addInfoMessage(ronText('PoklK_zruseni_dokumentu_ok'));
                        }
                    }
                    catch (Exception $e) {
                        MessageManager::addErrorMessage($e);
                    }
                }
            }

            if (!$dokumentPripojen) {
                throw new Exception('PoklK_dokument_nepripojen');
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToView($id);
    }

    /** Smaze dokument z databaze.
     * @param int $id
     * @return bool Vraci true, pokud byl smazan, false pokud nebyl nalezen a v pripade chyby vyhodi vyjimku
     * @throws Exception
     * @throws DBException
     */
    private function smazDokument($id)
    {
        $rec = new RonPortalDBRecord($this->database,'DokumentPokladniKnihy');
        $rec->logDescription = StringUtils::fromUTF8('Pokladní kniha');

        $rec->addField('ID', 'int', true);

        $rec->where = 'ID = ' . (int)$id;
        if ($rec->deleteRecord()) {

            $recDokument = new RonPortalDBRecord($this->database, 'Dokument');
            $recDokument->logDescription = 'Pokladní kniha';

            $recDokument->addField('ID', 'int', true);

            $recDokument->where = 'ID = ' . (int)$id;

            return $recDokument->deleteRecord();
        }
        
        return false;
    }

    /**
     * @param int $id
     * @param int|null $cisloBrowseru
     * @return Redirect
     */
    private function redirectToView($id, $cisloBrowseru = null)
    {
        return new Redirect(null, '?menuid=pokladnikniha&action=zaznam&record=' . $id . ($cisloBrowseru ? '&browser=' . $cisloBrowseru : ''));
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|PokladniKnihaBrowser|PokladniKnihaForm|string
     * @throws Exception
     * @throws DBException
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(self::POKLADNI_KNIHA_POVOLENA)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->pokladniKnihaFormular($formaction);
        }

        if (isset($_POST['hromadneSchvaleni'])  &&  $_POST['hromadneSchvaleni']) {
            if (PokladniKnihaRecord::hromadneSchvaleni($this ->database))
                return new Redirect(null, '?menuid=pokladnikniha&browser=' . $_GET['browser']);
        }

        $knihaaction = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($knihaaction) {
            case 'zaznam':
                return $this->zobrazDetailZaznamu($_GET['record']);

            case 'dokumentdownload':
                return $this->downloadDokument((int)$_GET['record'], (int)$_GET['id']);

            case 'dokumentupload' :
                return $this ->uploadDokument();

            case 'dokumentdelete' :
                return $this->smazPripojenyDokument((int)$_GET['id'], (int)$_GET['record']);
            
            case 'zrusitzaznam' :
                return $this->smazZaznam((int)$_GET['record']);

            case 'schval1' :
                return $this ->schvaleni((int)$_GET['record'], 1);
            case 'schval2' :
                return $this ->schvaleni((int)$_GET['record'], 2);
            case 'schval3' :
                return $this ->schvaleni((int)$_GET['record'], 3);

            case 'zrusschvaleni1' :
                return $this ->zrusSchvaleni((int)$_GET['record'], 1);
            case 'zrusschvaleni2' :
                return $this ->zrusSchvaleni($_GET['record'], 2);
            case 'zrusschvaleni3' :
                return $this ->zrusSchvaleni($_GET['record'], 3);

            case 'vrat' :
                return $this ->vratSchvaleni($_GET['record']);
            
            default:
                if (!$_GET['browser']) {
                    throw new Exception(StringUtils::fromUTF8('Chyba v pokladní kniha controller: není zadáno číslo browseru pro zobrazení!'));
                }
                //pro spravnou funkcnost "vratit zpet" z detailu zaznamu (predevsim pri presmerovani po schvaleni atd.)
                $_SESSION['pokladniKnihaCisloBrowseru'] = $_GET['browser'];

                require_once('pokladnikniha/b_pokladnikniha.php');
                $browser = new PokladniKnihaBrowser($this->database, $_GET['browser'], isset($_GET['schvalovani'])  &&  $_GET['schvalovani']);
                $browser->load();
                return $browser;
        }
    }

}