<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:46
 */

namespace Ron\RonPortal\PokladniKniha;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\RonPortal\DokumentUploadForm;
use Ron\RonPortal\DokumentyBrowser;
use Ron\RonPortal\RonPortalSettings;
use Ron\StringUtils;

class NovyDokumentPokladniKnihyForm extends DokumentUploadForm
{

    function __construct($database, $idZaznamu, $idPP)
    {
        parent::__construct($database, '?menuid=pokladnikniha&action=dokumentupload');

        $this->typDokumentu = RonPortalSettings::getIntValue('poklk_typ_dokumentu');

        $this->hiddenFields = [
            'dokumentpokladniknihy' => '1',
            'id' => $idZaznamu,
            'idosoby' => $idPP,
        ];
    }
}

class DokumentyPokladniKnihyBrowser extends DokumentyBrowser
{
    private $idZaznamuPoklK;

    function __construct($database, $idZaznamuPoklK)
    {
        parent::__construct($database, "?menuid=pokladnikniha&action=dokumentdownload&id=$idZaznamuPoklK", 1, "?menuid=pokladnikniha&action=dokumentdelete&id=$idZaznamuPoklK");

        $this->idZaznamuPoklK = $idZaznamuPoklK;

        $this->sqlFromPart = 'Dokument, TypDokumentu, DokumentPokladniKnihy';
        $this->sqlWherePart =
            'Dokument.Id = DokumentPokladniKnihy.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            " AND DokumentPokladniKnihy.IDPokladniKnihy = " . (int)$idZaznamuPoklK;
    }
}

class TypPolozkyPokladniKnihy
{
    public $kod;
    public $povolitCastkaBezDPH;
    public $povolitCastkaDPH;

    /**
     * TypPolozkyPokladniKnihy constructor.
     * @param string $kod
     * @param bool $povolitCastkaBezDPH
     * @param bool $povolitCastkaDPH
     */
    public function __construct($kod, $povolitCastkaBezDPH, $povolitCastkaDPH)
    {
        $this->kod = $kod;
        $this->povolitCastkaBezDPH = (bool)$povolitCastkaBezDPH;
        $this->povolitCastkaDPH = (bool)$povolitCastkaDPH;
    }

    /** Vraci true pokud lze pro typ polozky pokladni knihy zobrazit castku bez DPH.
     * @return bool
     */
    public function lzeZobrazitCastkuBezDPH()
    {
        return $this->povolitCastkaBezDPH;
    }

    /** Vraci true pokud lze pro typ polozky pokladni knihy zobrazit castku DPH.
     * @return bool
     */
    public function lzeZobrazitCastkuDPH()
    {
        return $this->povolitCastkaDPH;
    }
}

class SeznamPolozekPokladniKnihy
{
    public $cislo;
    /** @var string[] seznam typu polozek */
    public $seznamTypuSQL;
    /** @var  TypPolozkyPokladniKnihy[] */
    public $seznamTypu;

    public $jsonObject;

    /** 
     * @param int $cislo
     * @param string[] $seznamTypuSQL
     * @param TypPolozkyPokladniKnihy[] $seznamTypu
     * @param array $json
     */
    public function __construct($cislo, $seznamTypuSQL, $seznamTypu, $json)
    {
        $this->cislo = $cislo;
        $this->seznamTypuSQL = $seznamTypuSQL;
        $this->seznamTypu = $seznamTypu;
        $this->jsonObject = $json;
    }

    /** Vraci true, pokud lze pro nekterou z typu polozek zobrazit castku bez DPH.
     * @return bool
     */
    public function lzeZobrazitCastkuBezDPH()
    {
        foreach ($this->seznamTypu as $typPolozkyPokladniKnihy) {
            if ($typPolozkyPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
                return true;
            }
        }
        return false;
    }

    /** Vraci true, pokud lze pro nekterou z typu polozek zobrazit castku DPH.
     * @return bool
     */
    public function lzeZobrazitCastkuDPH()
    {
        foreach ($this->seznamTypu as $typPolozkyPokladniKnihy) {
            if ($typPolozkyPokladniKnihy->lzeZobrazitCastkuDPH()) {
                return true;
            }
        }
        return false;
    }

    /** Vraci true pokud lze zobrazit castku. Plati pro tyto kombinace:
     * <ul>
     *  <li>Lze zobrazit castku bez DPH, ale nejde zobrazit castku DPH.</li>
     *  <li>Nelze zobrazit castku bez DPH, ale lze zobrazit castku DPH.</li>
     *  <li>Nelze zobrazit castku bez DPH ani castku DPH.</li>
     * </ul>
     * @return bool
     */
    public function lzeZobrazitCastku()
    {
        foreach ($this->seznamTypu as $typPolozkyPokladniKnihy) {
            if ($typPolozkyPokladniKnihy->lzeZobrazitCastkuBezDPH() && !$typPolozkyPokladniKnihy->lzeZobrazitCastkuDPH()) {
                return true;
            }
            if (!$typPolozkyPokladniKnihy->lzeZobrazitCastkuBezDPH() && $typPolozkyPokladniKnihy->lzeZobrazitCastkuDPH()) {
                return true;
            }
            if (!$typPolozkyPokladniKnihy->lzeZobrazitCastkuBezDPH() && !$typPolozkyPokladniKnihy->lzeZobrazitCastkuDPH()) {
                return true;
            }
        }
        return false;
    }

    /** Vrati vybrany typ polozky pokladni knihy.
     * @param string $kodTypu kod typu polozky pokladni knihy
     * @return TypPolozkyPokladniKnihy
     */
    public function vratTypPolozky($kodTypu)
    {
        return $this->seznamTypu[$kodTypu];
    }

    /** Nacte a vrati z databaze seznam typu polozek pokladni knihy pro zvoleny browser.
     * @param Database $database
     * @param int $cisloBrowseru
     * @return SeznamPolozekPokladniKnihy
     * @throws DBException
     */
    public static function vratBrowser($database, $cisloBrowseru)
    {
        //pro číslo browseru 1 se vyhledávají i typy položek, které číslo browseru nemají zadané
        $sql =
            "SELECT Kod, PovolitCastkaBezDPH, PovolitCastkaDPH, RonPortalBrowser 
              FROM TypPolozkyPokladniKnihy
              WHERE (RonPortalBrowser = " . ($cisloBrowseru == 1 ? ($cisloBrowseru . " OR RonPortalBrowser IS NULL") : $cisloBrowseru)  . ")
              ORDER BY RonPortalBrowser";

        $cursor = $database->getSqlResult($sql);

        MessageManager::addDebugLog($sql);

        $typy = [];
        $polozky = [];
        $jsonObject = [];
        $jsonObject['items'] = [];

        while ($row = $cursor->fetch()) {
            $polozka = new TypPolozkyPokladniKnihy($row[0], $row[1], $row[2]);
            $polozky[$row[0]] = $polozka;
            $typy[] = $row[0];

            $jsonObject['items'][] = [
                'typ_polozky' => StringUtils::jsonText($row[0]),
                'povolit_castka_bez_dph' => $row[1] ? 1 : 0,
                'povolit_castka_dph' => $row[2] ? 1 : 0,
                'povolit_castka' => (!($row[1] && $row[2])) ? 1 : 0,

            ];

            $cisloBrowseru = $row[3];
        }

        return new SeznamPolozekPokladniKnihy((($cisloBrowseru != 0) ? $cisloBrowseru : 1), $typy, $polozky, $jsonObject);
    }
}
