<?php

/**
 * UTF-8
 * Pavel Rudol, RON Software
 *  4.9.2015
 */

namespace Ron\RonPortal\Zamestnanci;

use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\User;
use RonPortal\RozdeleniUtils;

require_once('pracovnipomery/b_zamestnanci.php');


class SeznamOsobBrowser extends SeznamZamestnancuBrowser
{
    public function __construct($database, $action, $prava)
    {
        $this->code = 'browserosoby';
        $this->tableId = 'browserosoby';
        
        $this->browserUrl = '?menuid=seznamosob&';

        $this->pouzitPracovniPomer = false;

        //musi byt za nastavenim pouzitPracovniPomer
        parent::__construct($database, $action, $prava);
    }
    
    function getSQLFilter()
    {
        if (!$this->pravaZobrazit->jsouNejakePrava()) {
            return '1 = 0';
        }

        $result = '';

        switch ($_SESSION['stavPP']) {
            case 'aktivni' :
                $result .= $this->getSQLFilterPracovniPomer();
                break;
            case 'neaktivni' :
                $result .= " NOT " . $this->getSQLFilterPracovniPomer();

                $result .= " AND " . $this->getSQLFilterPracovniPomer(false); //PP bez omezeni platnosti                
                break;
            default :
                $result .= ' ((NOT EXISTS(SELECT P.RC FROM PracovniPomer P WHERE (P.RC = Osoba.RC))';

                $result .= ' AND ' . $this->getSQLFilterPravaNaOsobu() . ')';

                $result .= " OR " . $this->getSQLFilterPracovniPomer(false); //PP bez omezeni platnosti
                $result .= ')';
        }

        return $result;
    }

    /**
     *
     * @param integer $typRozdeleni Typ rozdeleni pro filtrovani
     * @param string $rozdeleni kod rozdeleni pro filtrovani
     * @param int $vcetnePodrizenych 0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @return string SQL podminka existence prirazeneho rozdeleni
     */
    function getSQLFilterRozdeleni($typRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
//        $sqlPodrizene = '';
//
//        switch ($vcetnePodrizenych) {
//            case 0:
//                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
//                break;
//            case 1:
//                $sqlPodrizene = '';
//                break;
//            case 2:
//                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
//                break;
//            case 3:
//                $sqlPodrizene = 'and (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
//                break;
//        }
//
//        return "EXISTS(SELECT RozdeleniOsob.RC FROM RozdeleniOsob JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni "
//            . "     AND HierarchieRozdeleni.typNadrazene = HierarchieRozdeleni.typPodrizene) "
//            . " where (RozdeleniOsob.RC = P.RC ) "
//            . "and (RozdeleniOsob.osobniCislo = P.osobniCislo )"
//            . "and (RozdeleniOsob.datumNastupu = P.datumNastupu ) "
//            . "and (RozdeleniOsob.KodTypuRozdeleni = " . (int)$typRozdeleni . ")"
//            . "and (HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($rozdeleni) . "') "
//            . $sqlPodrizene
//            . 'and (RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(RonPortalSession::getDatumDo() ?: time()) . ') '
//            . 'and ((RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(RonPortalSession::getDatumOd() ?: time()) . ') or (RozdeleniOsob.PlatnostDo IS NULL))'
//            . ')';
        return RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
            'P', $typRozdeleni, $rozdeleni, RonPortalSession::getDatumOd() ?: time(), RonPortalSession::getDatumDo() ?: time(), $vcetnePodrizenych);
    }
    
    /**
     * 
     * @param boolean $omezitPlatnost pokud je TRUE omezi platnost filtrovaneho pracovniho pomeru
     * @return string SQL podminka na existenci pracovniho pomeru
     */
    function getSQLFilterPracovniPomer($omezitPlatnost = TRUE)
    {   
        return "EXISTS(SELECT P.RC FROM PracovniPomer P "
                ."WHERE (P.RC = Osoba.RC) "
                .'  AND (' . $this->pravaZobrazit->sqlFiltr('P', time(), time()) . ')'
                . (($omezitPlatnost) ? 
                    " AND (P.DatumNastupu <= " . $this->database->dateToString(RonPortalSession::getDatumDo()) . ")"
                    . " AND ((P.DatumUkonceni >= " . $this->database->dateToString(RonPortalSession::getDatumOd()) . ") OR (P.DatumUkonceni IS NULL))"
                    : "")
                . (($_SESSION['jenTypZamestnance']) ? " AND P.Zamestnanec = " . (int)$_SESSION['typZamestnance'] : "")
                . (((RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) != 0) && (RonPortalSession::rozdeleni() != '')) ? 'AND ' . $this->getSQLFilterRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), RonPortalSession::rozdeleni(), RonPortalSession::rozdeleniVcetnePodrizenych()) : '')
                . (((RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) != 0) && (RonPortalSession::rozdeleni2() != '')) ? 'AND ' . $this->getSQLFilterRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), RonPortalSession::rozdeleni2(), RonPortalSession::rozdeleni2VcetnePodrizenych()) : '')
                .')';
    }
    
    function getSQLFilterPravaNaOsobu($jmenoTabulky = 'Osoba')
    {
        if ($this ->pravaZobrazit->prava == 3) return ' (0 = 0) '; // vse

        if ($this ->pravaZobrazit->prava == 0) return ' (0 = 1) '; // nic
        if ($this ->pravaZobrazit->prava == 100) return ' (0 = 1) ';
        
        if ($this->pravaZobrazit->prava == 1) {
            return "($jmenoTabulky.RC = '" . $this->database->addSlashes(User::getRC()) . "')";
        }
        
        if ($this ->pravaZobrazit->prava == 2 || $this ->pravaZobrazit->prava == 4) {
            // vybrane osoby a skupiny ( 4 = krome sebe )

            $filtrosoby = '';

            $pomarray = explode(';', $this ->pravaZobrazit->osoby);
            foreach ($pomarray as $osoba) {
                if ($osoba != '') {
                    if ($filtrosoby) $filtrosoby .= ' OR ';
                    $filtrosoby .= "$jmenoTabulky.RC = '" . $this->database->addSlashes($osoba) . "'";
                }
            }

            $filtrskupiny = '';
            $pomarray = explode(';', $this ->pravaZobrazit->skupiny);
            foreach ($pomarray as $skupina) {
                if ($skupina != '') {
                    if ($filtrskupiny) $filtrskupiny .= ' OR ';
                    $filtrskupiny .= "PrirazeniOsobDoPristSkup.Kod = '" . $this->database->addSlashes($skupina) . "'";
                }
            }

            if ($filtrskupiny) {
                $filtrskupiny =
                    ' EXISTS (' .
                    'SELECT PrirazeniOsobDoPristSkup.Kod FROM PrirazeniOsobDoPristSkup' .
                    ' WHERE (' . $filtrskupiny . ') ' .
                    "   AND PrirazeniOsobDoPristSkup.RC = $jmenoTabulky.RC " .
                    /*"   AND PrirazeniOsobDoPristSkup.OsobniCislo = $jmenoTabulky.OsobniCislo " .
                    "   AND PrirazeniOsobDoPristSkup.DatumNastupu = $jmenoTabulky.DatumNastupu " .*/
                    '   AND PrirazeniOsobDoPristSkup.PlatnostOd <= ' . $this->database->dateToString(RonPortalSession::getDatumDo()) .
                    '   AND (PrirazeniOsobDoPristSkup.PlatnostDo IS NULL  OR  PrirazeniOsobDoPristSkup.PlatnostDo >= ' . $this->database->dateToString(RonPortalSession::getDatumOd()) . ')' .
                    ')';
            }

            if (!$filtrosoby && !$filtrskupiny) return '(0 = 1)';

            if ($filtrosoby != '') {
                if ($filtrskupiny) $filtr = " (($filtrosoby)  OR  ($filtrskupiny)) ";
                else $filtr = $filtrosoby;

            } else $filtr = $filtrskupiny;

            if ($this ->pravaZobrazit->prava == 4) $filtr = " ( $jmenoTabulky.RC <> '" . $this->database->addSlashes(User::getRC()) . "'  AND  ($filtr)) ";

            return $filtr;
        }              
        
        return ' (0 = 1) ';
    }
}