<?php

/**
 * UTF-8
 * Pavel Rudol, RON Software
 *  4.9.2015
 */

namespace Ron\RonPortal\Zamestnanci;

use Ron\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;

require_once('pracovnipomery/b_zamestnanci.php');

/**
 * Class IDMediumFilterForm
 * @package Ron\RonPortal\Zamestnanci
 */
class IDMediumField extends BrowserField
{
    private $volnaMedia;
    private $vyrazenaMedia;

    /**
     * IDMediumFilterForm constructor.
     * @param Database $database
     * @param bool $volnaMedia
     * @param $vyrazenaMedia
     */
    function __construct($database, $volnaMedia, $vyrazenaMedia)
    {
        parent::__construct($database);

        $this->volnaMedia = $volnaMedia;
        $this->vyrazenaMedia = $vyrazenaMedia;
        $this->header = \Ron\ronText('identifikacni_medium');
        $this->align = 'center';
        $this->ordering = 1;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if($rowData[8] != ''){

            if($this->volnaMedia) {
                $result .=
                    //Zobrazi odkaz v pripade ze jiz je karta k dane osobe pridelena
                    '<a href="?menuid=zamestnanec&action=prirazenikarty&formaction=edit&volnamedia=' . ($this->vyrazenaMedia ? 1 : 0) . '&osoba=' . $rowData[0] . '&kodkarty=' . $rowData[7] .
                    '&datumprideleni=' . $this->database->stringToTimestamp($rowData[9]) . '" >';
                $result .= $rowData[8];
                $result .= '</a>';
            } else {
                $result .=
                    //Zobrazi odkaz v pripade ze jiz je karta k dane osobe pridelena
                    '<a title="' . \Ron\ronText('nejsou_dalsi_volna_media') . '" href="?menuid=zamestnanec&action=prirazenikarty&formaction=edit&volnamedia=' . ($this->vyrazenaMedia ? 1 : 0) . '&osoba=' . $rowData[0] . '&kodkarty=' . $rowData[7] .
                    '&datumprideleni=' . $this->database->stringToTimestamp($rowData[9]) . '" >';
                $result .= $rowData[8];
                $result .= '</a>';
            }

        }else {
            if ($this->volnaMedia) {
                $result .=
                    '<a href="?menuid=zamestnanec&action=prirazenikarty&formaction=new&volnamedia=' . ($this->vyrazenaMedia ? 1 : 0) . '&osoba=' . $rowData[0] . '" >';
                $result .= \Ron\ronText('vydani_media');
                $result .= '</a>';
            } elseif ($this->vyrazenaMedia) {
                //$vyrazenaMedia
                $result .= '<a title="' . \Ron\ronText('nejsou_volna_nebo_obnovit') . '" href="?menuid=zamestnanec&action=obnovenimedia&formaction=edit">' . \Ron\ronText('nejsou_volna_media') . '</a>';
            } else {
                $result .= '';
            }
        }
        return $result;
    }
}

class SeznamPPBrowser extends SeznamZamestnancuBrowser
{
    public function __construct($database, $action, $prava)
    {
        parent::__construct($database, $action, $prava);

        $this->code = 'browserppomery';
        $this->tableId = 'browserppomery';

        $this->browserUrl = '?menuid=seznampp&';

        //TODO: nechat, nebo uplne presunout do formulare zamestnance?
        /*global $enable_seznamPP;
        if ($enable_seznamPP)
        $this->addField(new IDMediumField($this->database, $this->dotazNaVolnaMedia(), $this->dotazNaVyrazenaMedia()));*/

    }


    function getSQLFilter()
    {
        if ($this->pravaZobrazit->jsouNejakePrava()) {
            $result = '(' . $this->pravaZobrazit->sqlFiltr('PracovniPomer', time(), time()) . ')';
        } else {
            $result = '1 = 0';
        }

        switch ($_SESSION['stavPP']) {
            case 'aktivni' :
                $result .= " AND (PracovniPomer.DatumNastupu <= " . $this->database->dateToString(RonPortalSession::getDatumDo()) . ")"
                    . " AND ((PracovniPomer.DatumUkonceni >= " . $this->database->dateToString(RonPortalSession::getDatumOd()) . ") OR (PracovniPomer.DatumUkonceni IS NULL))";
                break;
            case 'neaktivni' :
                $result .= "AND (PracovniPomer.DatumUkonceni IS NOT NULL) ";

                $result .= "AND (PracovniPomer.DatumUkonceni <= " . $this->database->dateToString(RonPortalSession::getDatumOd()) . ")";

                break;
            default :
        }

        $result .= (($_SESSION['jenTypZamestnance']) ? " AND (PracovniPomer.Zamestnanec = " . (int)$_SESSION['typZamestnance'] . ")" : "");

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) != 0)
            $result .= (RonPortalSession::rozdeleni() != '' ? 'AND ' . $this->getSQLFilterRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), RonPortalSession::rozdeleni(), RonPortalSession::rozdeleniVcetnePodrizenych()) : '');
        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) != 0)
            $result .= (RonPortalSession::rozdeleni2() != '' ? 'AND ' . $this->getSQLFilterRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), RonPortalSession::rozdeleni2(), RonPortalSession::rozdeleni2VcetnePodrizenych()) : '');

        return $result;
    }

    /**
     * @return bool
     * @throws DBException
     */
    function dotazNaVolnaMedia()
    {
        $sql =
            "SELECT  Karta.Kod, Karta.OznaceniMedia  FROM Karta LEFT JOIN S_ZpusobVyrazeniKarty ON (Karta.ZpusobVyrazeni = S_ZpusobVyrazeniKarty.Kod)" .
            "WHERE DatumPorizeni <= " . $this->database->timestampToString(time()) .
            "  AND (DatumVyrazeni >= " . $this->database->timestampToString(time()) . " OR DatumVyrazeni IS NULL)" .
            " AND NOT EXISTS(SELECT kodKarty FROM PrirazeniKarty WHERE kodKarty = Karta.kod AND datumPrideleni <= " . $this->database->timestampToString(time()) .
            "  AND (datumVraceni >= " . $this->database->timestampToString(time()) . " OR datumVraceni IS NULL))";

        $cursor = $this->database->getSqlResult($sql);
        $row = $cursor->fetch();

        if (!$row) {
            return false;
        }
        return true;
    }

    /**
     * @return bool
     * @throws DBException
     */
    function dotazNaVyrazenaMedia()
    {
        $sql =
            'SELECT * FROM Karta WHERE DatumVyrazeni IS NOT NULL';

        $cursor = $this->database->getSqlResult($sql);
        $row = $cursor->fetch();

        if (!$row) {
            return false;
        }
        return true;
    }

    /**
     *
     * @param integer $typRozdeleni Typ rozdeleni pro filtrovani
     * @param string $rozdeleni kod rozdeleni pro filtrovani
     * @param int $vcetnePodrizenych 0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @return string SQL podminka existence prirazeneho rozdeleni
     */
    function getSQLFilterRozdeleni($typRozdeleni, $rozdeleni, $vcetnePodrizenych)
    {
        $sqlPodrizene = '';

        switch ($vcetnePodrizenych) {
            case 0:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 1:
                $sqlPodrizene = '';
                break;
            case 2:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 3:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
                break;
        }

        return "EXISTS(SELECT RozdeleniOsob.RC FROM RozdeleniOsob JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni "
            . "     AND HierarchieRozdeleni.typNadrazene = HierarchieRozdeleni.typPodrizene) "
            . " where (RozdeleniOsob.RC = PracovniPomer.RC ) "
            . "and (RozdeleniOsob.osobniCislo = PracovniPomer.osobniCislo )"
            . "and (RozdeleniOsob.datumNastupu = PracovniPomer.datumNastupu ) "
            . "and (RozdeleniOsob.KodTypuRozdeleni = " . (int)$typRozdeleni . ")"
            . "and (HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($rozdeleni) . "') "
            . $sqlPodrizene
            . 'and (RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString(RonPortalSession::getDatumDo() ?: time()) . ') '
            . 'and ((RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString(RonPortalSession::getDatumOd() ?: time()) . ') or (RozdeleniOsob.PlatnostDo IS NULL))'
            . ')';
    }
}
