<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 16.8.2016
 * Time: 12:28
 */

namespace Ron\RonPortal\Zamestnanci;

use Ron\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalbrowser.php');
require_once('ron/browserfield.php');

class PrijmeniDetailBrowserField extends BrowserField
{
    private $rcIndex;
    private $idPPIndex;

    public function __construct($database, $options, $rcIndex, $idPPIndex) {
        parent::__construct($database, $options);

        $this->rcIndex = $rcIndex;
        $this->idPPIndex = $idPPIndex;

        //TODO: odstranit podminku po dokonceni formulare
        global $enable_seznamPP;
        if ($enable_seznamPP)
            $this->link = '?menuid=zamestnanec';
    }

    function getLink($text, $rowData) {
        $result = '';

        if ($this ->link) {
            $result .= '<a href="' . $this ->link;
            if ($rowData[$this->idPPIndex] != '') {
                $result .= '&idpp=' . $rowData[$this->idPPIndex];
            }
            elseif ($rowData[$this->rcIndex] != '') {
                $result .= '&rc=' . $rowData[$this->rcIndex];
            }
            $result .= '" onclick="StopEventPropagation(event)">';
        }

        $result .= $text;

        if ($this ->link) $result .= '</a>';

        return $result;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData) {
        switch ($this ->type) {
            case BROWSER_FIELD_TYPE_LINE :
                if ($rowNumber > 0) return $rowNumber;
                else return '';

            case BROWSER_FIELD_TYPE_DEFAULT :
                return $this ->getLink(DBUtils::getFormatedValue($this ->database, $this ->format, $value), $rowData);

            case BROWSER_FIELD_TYPE_TEXT :
                return $this ->getLink($this ->text, $recordId);

            case BROWSER_FIELD_TYPE_SELECT :
                return $this ->getLink($this ->options[$value], $recordId);
        }

        return '';
    }
}

class FotoZamestnanceField extends BrowserField
{
    private $indexIDOsoby;
    private $indexFotografie;

    /**
     * FotoZamestnanceField constructor.
     * @param int $indexOsoby
     * @param int $indexFoto
     */
    public function __construct($indexOsoby, $indexFoto)
    {
        parent::__construct();

        $this->indexIDOsoby = $indexOsoby;
        $this->indexFotografie = $indexFoto;
        $this->header = ronText('Zamestnanci_foto');
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (($rowData[$this->indexIDOsoby] != '') && ($rowData[$this->indexFotografie] > 0)) {
            return
                '<div class="zamestnanec_browser_foto ronFoto">
                <img src="?menuid=zamestnanec&action=foto&osoba=' . $rowData[$this->indexIDOsoby] . '" alt="foto osoby"/>
            </div>';
        }

        return '';
    }

}


class PritomnostField extends BrowserField
{
    private $indexIDOsoby;

    /**
     * PritomnostField constructor.
     * @param int $indexIDOsoby
     */
    public function __construct($indexIDOsoby)
    {
        parent::__construct();
        $this->indexIDOsoby = $indexIDOsoby;
        $this->header = ronText('Zamestnanci_pritomnost');
        $this->align = 'center';
        $this->className = 'pritomnost';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<i class="fa pritomnost_info" data-idpp="' . $rowData[$this ->indexIDOsoby] . '" title="' . ronText('Zamestnanci_pritomnost_nenacteno') .'"></i>';
    }


}


/**
 * Class SeznamZamestnancuBrowser obecny predek pro prehled pracovnich pomeru/prehled osob, sloupce se nacitaji z nastaveni.
 * @package Ron\RonPortal\Zamestnanci
 */
class SeznamZamestnancuBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    protected $pravaZobrazit;

    /** @var boolean urcuje, zda bude dotaz i z tabulky PracovniPomer, nebo jen Osoba */
    protected $pouzitPracovniPomer = true;

    /** @var string nazev pole z nastaveni portalu s definici sloupcu (je mozne mit ruzne nastaveni pro pracovni pomery a osoby) */
    protected $nazevNastaveniSloupcu = 'pracovnipomery_sloupce';

    /** @var int index pole s RC v selectu */
    protected $indexRC;
    /** @var int index pole s ID PP v selectu, pokud se pouziva tabulka PracovniPomer, bere se primo zde, jinak posledni ID Pracovniho pomeru osoby, pokud existuje */
    protected $indexIdPP;

    /**
     * SeznamZamestnancuBrowser constructor.
     * @param Database $database
     * @param string $action
     * @param PravaNaOsoby $prava
     * @throws \Exception
     */
    public function __construct($database, $action, $prava)
    {
        parent::__construct($database, $action);

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = $prava;

        $this->sqlSelectPart =
            'Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, Osoba.TitulZa, OsobniUdaj.RodneJmeno, OsobniUdaj.DatumNarozeni, ' . //0 - 6
            'Osoba.eMail, Osoba.Telefon, OsobniUdaj.RC_Zahranicni, ' . //7 - 9
            'Osoba.uzivatelska1, Osoba.uzivatelska2, Osoba.uzivatelska3, Osoba.uzivatelska4, Osoba.uzivatelska5, ' . //10 - 14
            'Osoba.uzivatelska6, Osoba.uzivatelska7, Osoba.uzivatelska8, Osoba.uzivatelska9, Osoba.uzivatelska10, ' . //15 -19
            'Osoba.uzivatelska11, Osoba.uzivatelska12, Osoba.uzivatelska13, Osoba.uzivatelska14, Osoba.uzivatelska15, ' . //20 - 24
            'Osoba.uzivatelska16, Osoba.uzivatelska17, Osoba.uzivatelska18, Osoba.uzivatelska19, Osoba.uzivatelska20, ' . //25 - 29
            'Osoba.Insert_UserName, Osoba.Insert_Timestamp, Osoba.Update_UserName, Osoba.Update_Timestamp, Osoba.idFotografie, Osoba.Mobil, ' .   //30 - 35
            '(SELECT MAX(PracovniPomer.ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC)';  //36

        $this->indexIdPP = 36;

        if ($this->pouzitPracovniPomer) {
            $this->sqlSelectPart .= ', ' .
                'PracovniPomer.ID, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, ' . //37 - 39
                'PracovniPomer.uzivatelska1, PracovniPomer.uzivatelska2, PracovniPomer.uzivatelska3, PracovniPomer.uzivatelska4, PracovniPomer.uzivatelska5, ' . //40 - 44
                'PracovniPomer.uzivatelska6, PracovniPomer.uzivatelska7, PracovniPomer.uzivatelska8, PracovniPomer.uzivatelska9, PracovniPomer.uzivatelska10, ' . //45 -49
                'PracovniPomer.uzivatelska11, PracovniPomer.uzivatelska12, PracovniPomer.uzivatelska13, PracovniPomer.uzivatelska14, PracovniPomer.uzivatelska15, ' . //50 - 54
                'PracovniPomer.uzivatelska16, PracovniPomer.uzivatelska17, PracovniPomer.uzivatelska18, PracovniPomer.uzivatelska19, PracovniPomer.uzivatelska20, ' . //55 - 59
                'PracovniPomer.DatumUkonceni, PracovniPomer.ZkusebniDobaDoVypoctena, PracovniPomer.ZkusebniDobaDo, ZkusebniDobaDoRucne';  // 60 - 63
                // , PrirazeniKarty.kodKarty, Karta.OznaceniMedia, PrirazeniKarty.datumPrideleni '; //60 - 62

            $this->indexIdPP = 37;
        }

        $this->indexRC = 0;

        $this->sqlFromPart =
            'Osoba LEFT OUTER JOIN OsobniUdaj ON (Osoba.RC = OsobniUdaj.RC)' .
            ($this->pouzitPracovniPomer
                ? ' JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC)'
                : '');

        $this->filterFormAppCode = 'ADS';
        $this->filterFormRights = 'zamest_prohl';

        //TODO: odstranit podminku po dokonceni formulare
        global $enable_seznamPP;
        if ($enable_seznamPP)
            $this->rowClickLink = '?menuid=zamestnanec';

        $this->cssClass .= ' pritomnost-container';

//        $this->tableHtmlParams = [
//            'data-ron-pritomnost-url' => '?menuid=zamestnanec&action=pritomnost',
//            'data-ron-hledatpp' => $this->pouzitPracovniPomer ? 'true' : 'false'
//        ];

        //$this->sestavHlavicku();

        $str = RonPortalSettings::getBvarSettings($this ->database,
            'RonPortal_' . $this->nazevNastaveniSloupcu,
            RonPortalSettings::getValue($this->nazevNastaveniSloupcu)); // pro zpetnou kompatibilitu pouziva i getValue()
        if (!$str) {
            throw new \Exception(\Ron\ronText('Zamestnanci_neni_nastaveni_sloupcu_tabulky'));
        }

        $this ->loadConfigurationXml($str);
    }


    function createNewField($colSettings)
    {
        $newField = null;

        switch ($colSettings['typsloupce']) {
            case 'prijmeni' :
                $newField = $this ->addField(new PrijmeniDetailBrowserField($this ->database, null, $this ->indexRC, $this ->indexIdPP))
                    ->setSqlFieldName('Osoba.Prijmeni', 2) ->setHeaderText('Prijmeni') ->setOrdering(true);
                break;

            case 'jmeno' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Jmeno', 1) ->setHeaderText('Jmeno') ->setOrdering(true);
                break;

            case 'titul' :
                $newField = $this->addField() ->setSqlFieldName('Osoba.Titul', 3) ->setHeaderText('Titul');
                break;
            case 'titulza' :
                $newField = $this->addField() ->setSqlFieldName('Osoba.TitulZa', 4) ->setHeaderText('Titul_za');
                break;

            case 'rodneprijmeni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.RodneJmeno', 5) ->setHeaderText('Rodne_jmeno') ->setOrdering(true);
                break;

            case 'datumnarozeni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.DatumNarozeni', 6) ->setHeaderText('Datum_narozeni')
                    ->setOrdering(true) ->setFormat('date');
                break;

            case 'email' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.eMail', 7) ->setHeaderText('e_mail') ->setOrdering(true);
                break;

            case 'telefon' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Telefon', 8) ->setHeaderText('Telefon') ->setOrdering(true);
                break;

            case 'mobil' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Mobil', 35) ->setHeaderText('Mobil') ->setOrdering(true);
                break;

            case 'rczahranicni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.RC_Zahranicni', 9) ->setHeaderText('RC_Zahranicni') ->setOrdering(true);
                break;

            case 'osobauzivatelska' :
                $index = (int)$colSettings['index'];
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.uzivatelska' . $index, 9 + $index) ->setHeaderText("osoba_uzivatelska_{$index}") ->setOrdering(true);
                break;

            case 'ppomeruzivatelska' :
                $index = (int)$colSettings['index'];
                $newField = $this->addField()
                    ->setSqlFieldName('PracovniPomer.uzivatelska' . $index, 39 + $index) ->setHeaderText("ppomer_uzivatelska_{$index}") ->setOrdering(true);
                break;

            case 'rc' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.RC', 0) ->setHeaderText('RC') ->setOrdering(true);
                break;

            case 'foto' :
                $newField = $this->addField(new FotoZamestnanceField($this->indexIdPP, 34));
                break;

            case 'osobnicislo' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.OsobniCislo', 38) ->setHeaderText('Osobni_cislo')
                        ->setOrdering(true);
                }
                break;

            case 'datumnastupu' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.DatumNastupu', 39) ->setHeaderText('Datum_nastupu')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center');
                }
                break;

            case 'datumukonceni' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.DatumUkonceni', 60) ->setHeaderText('Datum_ukonceni')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center');
                }
                break;

            case 'zkusebnidobadovypoctena' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.ZkusebniDobaDoVypoctena', 61) ->setHeaderText('Zkusebni_doba_do_vypoctena')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center');
                }
                break;

            case 'pritomnost' :
                $newfield = new PritomnostField($this->indexIdPP);
                $this->addField($newfield);
                break;

            case 'kodrozdeleni' :
                if ($this->pouzitPracovniPomer) {
                    $typRozdeleni = (int)$colSettings['index'];

                    $colSettings['sqlFieldName'] =
                        '(SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob' .
                        ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $typRozdeleni .
                        ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
                        ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                        ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                        ' AND RozdeleniOsob.Hlavni = 1' .
                        ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString(RonPortalSession::getDatumDo() ?: time()) .
                        ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(RonPortalSession::getDatumOd() ?: time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        ') AS KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM');   //20.12.2016 RON-PR workaround pro typ rozdeleni -1 (PM)

                    $colSettings['sortSQL'] = 'KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ', Osoba.Prijmeni, Osoba.Jmeno';
                    $colSettings['sortSQLDesc'] = 'KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ' DESC, Osoba.Prijmeni DESC, Osoba.Jmeno DESC';

                    if (!isset($colSettings['header'])) {
                        $colSettings['header'] = StringUtils::htmlTextConvert(DBUtils::getSqlValue($this->database, 'SELECT Popis FROM TypRozdeleni WHERE Kod = ' . (int)$typRozdeleni));
                    }

                    if (!isset($colSettings['ordering'])) {
                        $colSettings['ordering'] = 1;
                    }
                    return parent::createNewField($colSettings);
                }
                break;

            case 'popisrozdeleni' :
                if ($this->pouzitPracovniPomer) {
                    $typRozdeleni = (int)$colSettings['index'];

                    $colSettings['sqlFieldName'] =
                        '(SELECT Popis FROM Rozdeleni WHERE KodTypuRozdeleni = ' . $typRozdeleni .
                        ' AND Kod = ' .
                        ' (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob ' .
                        ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $typRozdeleni .
                        '  AND RozdeleniOsob.RC = PracovniPomer.RC' .
                        '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                        '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                        '  AND RozdeleniOsob.Hlavni = 1' .
                        '  AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString(RonPortalSession::getDatumDo() ?: time()) .
                        '  AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(RonPortalSession::getDatumOd() ?: time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                        ')) AS PopisRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM'); //20.12.2016 RON-PR workaround pro typ rozdeleni -1 (PM)

                    $colSettings['sortSQL'] = 'PopisRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ', Osoba.Prijmeni, Osoba.Jmeno';
                    $colSettings['sortSQLDesc'] = 'PopisRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ' DESC, Osoba.Prijmeni DESC, Osoba.Jmeno DESC';

                    if (!isset($colSettings['header'])) {
                        $colSettings['header'] = StringUtils::htmlTextConvert(DBUtils::getSqlValue($this->database, 'SELECT Popis FROM TypRozdeleni WHERE Kod = ' . (int)$typRozdeleni));
                    }

                    if (!isset($colSettings['ordering'])) {
                        $colSettings['ordering'] = 1;
                    }

                    return parent::createNewField($colSettings);
                }
                break;

            default :
                return parent::createNewField($colSettings);
        }

        if ($newField) {
            $newField ->loadConfigurationArray($colSettings);
            if (($newField->ordering) && !$this->defaultSort) {
                $this->defaultSort = count($this->fields);
            }
        }

        return $this->fields ? count($this->fields) - 1 : -1;
    }


    function loadConfigurationXml($xmlText)
    {
        if ($xmlText  &&  $xmlText[0] == '<') {
            parent::loadConfigurationXml($xmlText);
        } else {
            $this ->sestavHlavicku($xmlText);
        }
    }


    function loadConfiguration()
    {
        // Nenacita konfiguraci - je to zbytecne, protoze se to konfiguruje primo v nastaveni
        // return parent::loadConfiguration();
    }

    /**
     * Prida k zobrazeni sloupce dle nastaveni. Pouze pro kompatibilitu se starym nastavenim ve formatu Nadpis:TypPolozky;...
     * @param string $str
     */
    public function sestavHlavicku($str)
    {
        $header = [];
        $sloupce = [];
        StringUtils::parseStringToArrays($str, $header, $sloupce, true);

        $this ->addField( [BROWSER_FIELD_TYPE => 'line'] );

        foreach ($sloupce as $i => $sloupec) {
            $newfield = null;

            $sloupecKod = preg_replace('/[0-9\-]/', '', $sloupec);
            $sloupecCislo = preg_replace('/[a-zA-Z]/', '', $sloupec);

            $colSettings = ['typsloupce' => $sloupecKod];
            if ($sloupecCislo) {
                $colSettings['index'] = $sloupecCislo;
            }
            if ($header[$i]) {
                $colSettings['header'] = ronText($header[$i]);
            }

            $this ->createNewField($colSettings);
        }
    }


    /**
     * Nastavi filtry.
     */
    function setFilterForm()
    {
        $this->filterForm = new SeznamZamestnancuFilterForm($this->database, $this->action);
    }

    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        if (!$this->tableHtmlParams) {
            $this->tableHtmlParams = [];
        }

        $this->tableHtmlParams['data-ron-pritomnost-url'] =
            '?menuid=zamestnanec&action=pritomnost' .
            '&rozdeleni1=' . StringUtils::htmlTextConvert($this ->filterForm ->kodRozdeleni) .
            '&vcetnepodrizenych1=' . (int)$this ->filterForm ->vcetnePodrizenychRozdeleni .
            '&rozdeleni2=' . StringUtils::htmlTextConvert($this ->filterForm ->kodRozdeleni2) .
            '&vcetnepodrizenych1=' . (int)$this ->filterForm ->vcetnePodrizenychRozdeleni2 .
            '&jenrc=' . ($this->pouzitPracovniPomer ? 0 : 1);

        //'data-ron-hledatpp' => $this->pouzitPracovniPomer ? 'true' : 'false'
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @return int|string
     */
    function getClickLink($values)
    {
        if ($this ->rowClickLink) {
            if ($this->pouzitPracovniPomer || ($values[$this->indexIdPP] != '')) {
                return $this ->rowClickLink . '&idpp=' . $values[$this->indexIdPP];
            }

            return $this ->rowClickLink .  '&rc=' . $values[$this->indexRC];
        }

        return 0;
    }

    /** Doplni k radku id pracovniho pomeru pro javascript.
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @return string
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        return 'data-ron-idpp="' . $values[$this->indexIdPP] . '" ' . parent::getRowAttributes($values, $className, $rowNumber);
    }

}