<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 23.8.2016
 * Time: 14:34
 */

namespace Ron\RonPortal\Zamestnanci;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalUtils;

class MzdovyVymerOsoby
{
    public $idPP;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $platnostOd;
    public $platnostDo;
    public $typMzdy;
    public $platovyVymer;
    public $zaloha;
    public $mzdovyStupen;

    /**
     *
     * @param Database $database
     * @param int $idPP
     * @param int $platnost
     * @return MzdovyVymerOsoby[]
     * @throws \Exception
     */
    public static function nactiVymerOsoby($database, $idPP, $platnost = 0)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PlatovyVymer.RC, PlatovyVymer.OsobniCislo, PlatovyVymer.DatumNastupu, PlatovyVymer.PlatnostOd, " . // 0 - 3
            "PlatovyVymer.PlatnostDo, PlatovyVymer.TypMzdy, PlatovyVymer.PlatovyVymer, PlatovyVymer.Zaloha, PlatovyVymer.MzdovyStupen " . // 4 - 8
            "FROM PlatovyVymer JOIN PracovniPomer ON (PracovniPomer.RC = PlatovyVymer.RC AND PracovniPomer.OsobniCislo = PlatovyVymer.OsobniCislo " .
            "   AND PracovniPomer.DatumNastupu = PlatovyVymer.DatumNastupu) ".
            "WHERE PracovniPomer.ID = " . $params->addParameter($idPP);

        if ($platnost > 0) {
            $sql .= ' AND PlatnostOd <= ' .  $params->addParameter($database->timestampToParam($platnost)) .
                ' AND (PlatnostDo >= ' . $params->addParameter($database->timestampToParam($platnost)) . ' OR PlatnostDo IS NULL) ';
        }

        $sql .= ' ORDER BY PlatnostOd ';

        $cursor = $database->getSqlResult($sql, $params);

        $seznamVymeru = [];

        while ($row = $cursor->fetch()) {
            $vymer = new MzdovyVymerOsoby();
            $vymer->idPP = $idPP;
            $vymer->nactiVymerZRadku($database, $row);

            $seznamVymeru[] = $vymer;
        }

        return $seznamVymeru;
    }

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param bool $platnostOdPrvnihoDneMesice pokud je true, nastavi se platnostod na prvni den v mesici zadane platnosti od
     * @param int $index index prvku v postu
     * @return bool
     * @throws \Exception
     */
    public function ulozVymerOsoby($database, $platnostOd = 0, $platnostOdPrvnihoDneMesice = true, $index = null)
    {
        $this->nactiPost($index);

        if ($platnostOd > 0) {
            $this->platnostOd = $platnostOd;
        }

        if (!($this->rc && $this->platnostOd)) {
            return false;
        }

        $this->platnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($this->platnostOd) : $this->platnostOd;

        $rec = new RonPortalDBRecord($database,'PlatovyVymer');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $this->rc);
        $rec->addField('OsobniCislo', '', true, 15, $this->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $this->datumNastupu);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $this->platnostDo ? $this->platnostDo : null);
        $rec->addField('TypMzdy', 'int', false, 0, $this->typMzdy);
        $rec->addField('PlatovyVymer', 'float', false, 0, $this->platovyVymer);
        $rec->addField('Zaloha', 'float', false, 0, $this->zaloha);
        $rec->addField('MzdovyStupen', 'int', false, 0, $this->mzdovyStupen);

        $params = $database->createParameters();

        $rec->setWhere(
            " RC = " . $params->addParameter($this->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($database->timestampToParam($this->datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($this->platnostOd)), $params);

        $rec->loadRecord();

        if (!$rec->updateOrInsert()) {
            throw new \Exception('Chyba pri vkladani/aktualizaci vymeru osoby.');
        }

        return true;
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @param bool $platnostOdPrvnihoDneMesice
     * @return bool
     * @throws \Exception
     */
    public static function aktualizujPlatnostVymeruOsoby($database, $pracovniPomer, $puvodniPlatnostOd, $novaPlatnostOd, $platnostOdPrvnihoDneMesice = true)
    {
        $novaPlatnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($novaPlatnostOd) : $novaPlatnostOd;

        $rec = new RonPortalDBRecord($database,'PlatovyVymer');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $pracovniPomer->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pracovniPomer->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer->datumNastupu);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $novaPlatnostOd);

        $params = $database->createParameters();

        $rec->setWhere(
            " RC = " . $params->addParameter($pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($database->timestampToParam($pracovniPomer->datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        $rec->loadRecord();

        if (!$rec->updateRecord()) {
            throw new \Exception('Chyba pri aktualizaci platnosti vymeru osoby.');
        }

        return true;
    }

    /**
     *
     * @param Database $database
     * @param array $row
     */
    private function nactiVymerZRadku($database, $row)
    {
        $this->rc = $row[0];
        $this->osobniCislo = $row[1];
        $this->datumNastupu = $database->stringToTimestamp($row[2]);
        $this->platnostOd = $database->stringToTimestamp($row[3]);
        $this->platnostDo = $database->stringToTimestamp($row[4]);
        $this->typMzdy = $row[5];
        $this->platovyVymer = $row[6];
        $this->zaloha = $row[7];
        $this->mzdovyStupen = $row[8];
    }

    /**
     * @param int $index index prvku v postu
     * @throws \Exception
     */
    private function nactiPost($index = null)
    {
        $poradiVymeru = '';

        if (!is_null($index)) {
            $poradiVymeru = '_' . $index;
        }

        if ($_POST['vymer_platnostOd'.$poradiVymeru] != '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'vymer_platnostOd'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS));
        }

        if ($_POST['vymer_platnostDo'.$poradiVymeru] != '') {
            $this->platnostDo = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'vymer_platnostDo'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS));
        }

        $this->typMzdy = filter_input(INPUT_POST,'vymer_typMzdy'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platovyVymer = filter_input(INPUT_POST,'vymer_platovyVymer'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zaloha = filter_input(INPUT_POST,'vymer_zaloha'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->mzdovyStupen = filter_input(INPUT_POST,'vymer_mzdovyStupen'.$poradiVymeru,FILTER_SANITIZE_SPECIAL_CHARS);

    }
}

/**
 * Class BankovniUcetOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class BankovniUcetOsoby
{
    public $serioveCislo;
    public $rc;
    public $cisloUctu;
    public $kodBanky;
    //tabulky BankovniUcet a VyplaceniMzdy

    public $specifickySymbol;
    public $konstantniSymbol;
    public $variabilniSymbol;

    public $vyplaceniMzdyPlatnostOd;
    public $vyplaceniMzdyPlatnostDo;


    public $insert_UserName;
    public $insert_Timestamp;
    public $update_UserName;
    public $update_Timestamp;

    /**
     * @param Database $database
     * @param string $rc
     * @return BankovniUcetOsoby[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiUcetOsoby($database, $rc)
    {
        $params = $database->createParameters();

        $sql =  'SELECT KodBanky, CisloUctu, SerioveCislo, SpecifickySymbol, KonstantniSymbol, VariabilniSymbol, VyplaceniMzdy.PlatnostOd, VyplaceniMzdy.PlatnostDo, ' .    //0..7
            '  Insert_UserName, Insert_Timestamp, Update_UserName, Update_Timestamp ' .   //8..11
            'FROM BankovniUcet JOIN VyplaceniMzdy ON (VyplaceniMzdy.SerioveCisloUctuVyplaty = BankovniUcet.SerioveCislo) ' .
            'WHERE BankovniUcet.RC = ' . $params->addParameter($rc) .
            ' ORDER BY VyplaceniMzdy.PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $ucty = [];

        while ($row = $cursor->fetch()) {
            $ucet = new BankovniUcetOsoby();
            $ucet->rc = $rc;
            $ucet->nactiDataZRadku($row, $database);

            $ucty[$ucet->serioveCislo] = $ucet;
        }

        return $ucty;
    }

    /** Vlozi/aktualizuje bankovni ucet. Pokud je zadan i pracovni pomer pri vkladani vlozi i zaznam vyplaceni mzdy.
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param int $platnostOd
     * @param bool $platnostOdPrvnihoDneMesice pokud je true, nastavi se platnostod na prvni den v mesici zadane platnosti od
     * @param bool $vlozitNovyUcet
     * @throws \Exception
     */
    public static function ulozUcetOsoby($database, $rc, $osobniCislo = '', $datumNastupu = 0, $platnostOd = 0, $platnostOdPrvnihoDneMesice = true, $vlozitNovyUcet = false)
    {
        $platnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($platnostOd) : $platnostOd;

        $rec = new RonPortalDBRecord($database, 'BankovniUcet');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('SerioveCislo', 'int',true, 0);
        $rec->addField('JmenoUctu', '', true, 40);
        $rec->addField('KodBanky', '', false, 10);
        $rec->addField('CisloUctu', '', false, 20);
        $rec->addField('TypUctu', 'int', true, 0, 1); //ucet zamestnance
        $rec->addField('RC', '', true, 15);

        $rec->addField('SpecifickySymbol', '', false, 20);
        $rec->addField('KonstantniSymbol', '', false, 10);
        $rec->addField('VariabilniSymbol', '', false, 20);

        $rec->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $rec->addField('Insert_Timestamp', 'timestamp', true, 0, time());
        $rec->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
            ->updateLog = true;
        $rec->addField('Update_Timestamp', 'timestamp', false, 0, time())
            ->updateLog = true;

        static::nactiPost($rec);

        if ($vlozitNovyUcet) {
            $rec->findFieldByName('SerioveCislo')->setNewValue(SQLUtils::newSerialNumber($database, 'BankovniUcet', 'SerioveCislo'));
            $rec->findFieldByName('JmenoUctu')->setNewValue(Zamestnanec::vratPrijmeniJmenoOsoby($database, $rc) . ' - vyplata mzdy');
            $rec->findFieldByName('RC')->setNewValue($rc);

            if (!$rec->insertRecord()) {
                throw new \Exception('Chyba pri vkladani bankovniho uctu osoby.');
            }

            if ($osobniCislo != '') {
                static::priradUcetVyplaceni($database, $rc, $osobniCislo, $datumNastupu, $platnostOd, $rec->findFieldByName('SerioveCislo')->value);
            }
        } else {

            $params = $database->createParameters();
            $rec->setWhere('SerioveCislo = ' . $params->addParameter($_POST['banka_seriove_cislo']), $params);

            if (!$rec->loadAndUpdateRecord()) {
                throw new \Exception('Chyba pri aktualizaci bankovniho uctu osoby.');
            }
        }

    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @param bool $platnostOdPrvnihoDneMesice
     * @throws \Exception
     */
    public static function aktualizujPlatnostVyplaceniMzdy($database, $pracovniPomer, $puvodniPlatnostOd, $novaPlatnostOd, $platnostOdPrvnihoDneMesice = true)
    {
        $novaPlatnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($novaPlatnostOd) : $novaPlatnostOd;

        $recVyplata = new RonPortalDBRecord($database, 'VyplaceniMzdy');

        $recVyplata->logDescription = 'Editace zamestnance - portal';

        $recVyplata->addField('RC', '', false, 15, $pracovniPomer->rc);
        $recVyplata->addField('OsobniCislo', '', false, 15, $pracovniPomer->osobniCislo);
        $recVyplata->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer->datumNastupu);
        $recVyplata->addField('PlatnostOd', 'timestamp', false, 0, $novaPlatnostOd);

        $params = $database->createParameters();

        $recVyplata->setWhere(" RC = " . $params->addParameter($pracovniPomer->rc) .
            " AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($database->timestampToParam($pracovniPomer->datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        if (!$recVyplata->updateRecord()) {
            throw new \Exception('Chyba pri editaci platnosti vyplaceni mzdy osoby.');
        }
    }

    /**
     *
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($row, $database)
    {
        $this->kodBanky = $row[0];
        $this->cisloUctu = $row[1];
        $this->serioveCislo = $row[2];
        $this->specifickySymbol = $row[3];
        $this->konstantniSymbol = $row[4];
        $this->variabilniSymbol = $row[5];
        $this->vyplaceniMzdyPlatnostOd = $database->stringToTimestamp($row[6]);
        $this->vyplaceniMzdyPlatnostDo = $database->stringToTimestamp($row[7]);
        $this->insert_UserName = $row[8];
        $this->insert_Timestamp = $database->stringToTimestamp($row[9]);
        $this->update_UserName = $row[10];
        $this->update_Timestamp = $database->stringToTimestamp($row[11]);

    }

    /**
     * @param RonPortalDBRecord $rec
     */
    private static function nactiPost($rec)
    {
        $rec->findFieldByName('SerioveCislo')->setNewValue($_POST['banka_seriove_cislo']);
        $rec->findFieldByName('CisloUctu')->setNewValue($_POST['banka_cislo_uctu']);
        $rec->findFieldByName('KodBanky')->setNewValue($_POST['KodBanky']);
        $rec->findFieldByName('SpecifickySymbol')->setNewValue($_POST['banka_specificky_symbol']);
        $rec->findFieldByName('KonstantniSymbol')->setNewValue($_POST['banka_konstantni_symbol']);
        $rec->findFieldByName('VariabilniSymbol')->setNewValue($_POST['banka_variabilni_symbol']);
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param int $platnostOd
     * @param int $serioveCislo
     * @throws \Exception
     */
    private static function priradUcetVyplaceni($database, $rc, $osobniCislo, $datumNastupu, $platnostOd, $serioveCislo)
    {
        $recVyplata = new RonPortalDBRecord($database, 'VyplaceniMzdy');

        $recVyplata->logDescription = 'Editace zamestnance - portal';

        $recVyplata->addField('RC', '', false, 15, $rc);
        $recVyplata->addField('OsobniCislo', '', FALSE, 15, $osobniCislo);
        $recVyplata->addField('DatumNastupu', 'timestamp', false, 0, $datumNastupu);
        $recVyplata->addField('PlatnostOd', 'timestamp', false, 0, $platnostOd);

        $recVyplata->addField('SerioveCisloUctuVyplaty', 'int',false, 0, $serioveCislo);

        $params = $database->createParameters();

        $recVyplata->setWhere(" RC = " . $params->addParameter($rc) .
            " AND OsobniCislo = " . $params->addParameter($osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($database->timestampToParam($datumNastupu)) .
            " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($platnostOd)), $params);

        if (!$recVyplata->updateOrInsert()) {
            throw new \Exception('Chyba pri vkladani/editaci vyplaceni mzdy osoby.');
        }
    }
}
