<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 21.1.2016
 * Time: 13:51
 */

namespace Ron\RonPortal\Zamestnanci;

require_once('ron/controls.php');

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\RonPortal\Log;
use Ron\RonPortal\RonPortalDBRecord;
use function Ron\ronText;


/**
 * Class RodinnyPrislusnik
 * @package Ron\RonPortal\Zamestnanci
 */
class RodinnyPrislusnik
{
    public $jmeno;
    public $prijmeni;
    public $rodnecislo;
    public $noverodnecislo;
    public $datumnarozeni;
    public $rodinnyvztah;
    public $rodinnyvztahstr;
    public $zdravotnistav;
    public $zdravotnistavstr;
    public $drzitelZTPod;
    public $vyzivovanaosoba;
    public $vyzivovanaosobastr;
    public $stejnaadresa;
    public $kontaktniosoba;
    public $telefon;
    public $email;


    /**
     * @param Database $database
     * @param array $row
     */
    public function nactiDataZRadku($database, $row)
    {
        $this->jmeno = $row[2];
        $this->prijmeni = $row[1];
        $this->rodnecislo = $row[0];
        if ($row[7]) {
            $this->datumnarozeni = $database->stringToTimestamp($row[7]);
        }
        $this->rodinnyvztah = $row[8];
        $this->rodinnyvztahstr = $row[11];
        $this->zdravotnistav = $row[4];
        $this->zdravotnistavstr = $row[12];
        if ($row[6]) {
            $this->drzitelZTPod = $database->stringToTimestamp($row[6]);
        }
        $this->vyzivovanaosoba = $row[10];
        $this->vyzivovanaosobastr = $row[16];
        $this->stejnaadresa = $row[9];
        $this->kontaktniosoba = $row[13];
        $this->telefon = $row[14];
        $this->email = $row[15];
    }

    /**
     * @param Database $database
     * @param string $rc
     * @return RodinnyPrislusnik[]
     * @throws DBException
     */
    public static function nactiRodinuZamestnance($database, $rc)
    {
        $params = $database->createParameters();

        $sql =  "SELECT RodinnyPrislusnik.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno, ". //0 - 2
            "RodinnyPrislusnik.Titul, RodinnyPrislusnik.ZdravotniStav, RodinnyPrislusnik.DrzitelPrukazuZTTP, RodinnyPrislusnik.DatumPriznaniZTTP, RodinnyPrislusnik.DatumNarozeni, " . //3 - 7
            "ClenoveRodiny.RodinnyVztah, ClenoveRodiny.StejnaAdresa, ClenoveRodiny.VyzivovanaOsoba, S_RodinnyVztah.Popis, S_ZdravotniStav.Popis, Kontakt.Popis, " . //8 - 13
            "Telefon, Email, VyzivovanaOsobaPopis.Popis " .     //14..16

            "FROM RodinnyPrislusnik JOIN ClenoveRodiny ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) " .
            " JOIN S_RodinnyVztah ON (S_RodinnyVztah.Kod = ClenoveRodiny.RodinnyVztah) " .
            " JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = RodinnyPrislusnik.ZdravotniStav) " .
            " LEFT JOIN S_AnoNe Kontakt ON (Kontakt.Kod = RodinnyPrislusnik.KontaktniOsoba) " .
            " LEFT JOIN S_AnoNe VyzivovanaOsobaPopis ON (VyzivovanaOsobaPopis.Kod = ClenoveRodiny.VyzivovanaOsoba) " .
            "WHERE ClenoveRodiny.RC  = " . $params->addParameter($rc) . ' AND (ClenoveRodiny.PartnerDatumOd IS NULL OR ClenoveRodiny.PartnerDatumOd <= ' . $params->addParameter($database->timestampToParam(time())) .
            ") AND (ClenoveRodiny.PartnerDatumDo IS NULL OR ClenoveRodiny.PartnerDatumDo >= " . $params->addParameter($database->timestampToParam(time())) . ") " .
            "ORDER BY RodinnyPrislusnik.DatumNarozeni";

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        $prislusnici = [];

        while ($row = $cursor->fetch()) {
            $prislusnik = new RodinnyPrislusnik();
            $prislusnik->nactiDataZRadku($database,$row);

            $prislusnici[] = $prislusnik;
        }

        return $prislusnici;
    }

    /**
     * @param Database $database
     * @param string $rc rc osoby, ktere se prirazuji clenove
     * @param int $index index prvku v postu
     * @return bool
     * @throws Exception
     */
    public function ulozRodinnehoPrislusnika($database, $rc, $index = null)
    {
        $this->nactiPost($index);

        if (!$this->rodnecislo) {
            return false;
        }

        $rec = new RonPortalDBRecord($database,'RodinnyPrislusnik');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC_RodinnehoPrislusnika','',false, 15, $this->noverodnecislo ? $this->noverodnecislo : $this->rodnecislo);
        $rec->addField('Prijmeni', '', false, 30, $this->prijmeni);
        $rec->addField('Jmeno','', false, 30, $this->jmeno);
        $rec->addField('ZdravotniStav', 'int', false, 0, $this->zdravotnistav);
        if ($this->drzitelZTPod > 0) {
            $rec->addField('DrzitelPrukazuZTTP', 'int', false, 0, 1);
            $rec->addField('DatumPriznaniZTTP', 'timestamp', false, 0, $this->drzitelZTPod);
        }
        $rec->addField('DatumNarozeni', 'timestamp', false, 0, $this->datumnarozeni);

        $params = $database->createParameters();

        $rec->setWhere(" RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

        $rec->loadRecord();
        
        if (!$rec->updateOrInsert()) {
            throw new Exception('Chyba pri vkladani/aktualizaci rodinneho prislusnika.');
        }

        //prirazeni clena rodiny k osobe se provede jen pri vlozeni, update je zajisten v databazi kaskadne!
        if (!isset($this->noverodnecislo)) {
            //přiřazení člena rodiny osobě
            $recClenove = new RonPortalDBRecord($database, 'ClenoveRodiny');

            $recClenove->logDescription = 'Editace zamestnance - portal';

            $recClenove->addField('RC', '', false, '15', $rc);
            $recClenove->addField('RC_RodinnehoPrislusnika', '', false, 15, $this->noverodnecislo ? $this->noverodnecislo : $this->rodnecislo);
            $recClenove->addField('RodinnyVztah', 'int', false, 0, $this->rodinnyvztah);
            $recClenove->addField('StejnaAdresa', 'int', false, 0, $this->stejnaadresa);
            $recClenove->addField('VyzivovanaOsoba', 'int', false, 0, $this->vyzivovanaosoba);

            $params = $database->createParameters();

            $recClenove->setWhere(" RC = " . $params->addParameter($rc) .
                " AND RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

            if ($recClenove->loadRecord() == 0) {
                if (!$recClenove->insertRecord()) {
                    throw new Exception('Chyba pri vkladani clena rodiny (prirazeni rodinneho prislusnika osobe).');
                }
            }
        }

        return true;
    }
    
    /**
     * 
     * @param Database $database
     * @param int $index index prvku v postu
     * @return boolean
     * @throws Exception
     */
    public function smazRodinnehoPrislusnika($database, $index = null)
    {
        $this->nactiPost($index);

        if (!$this->rodnecislo) {
            return false;
        }
        
        $rec = new RonPortalDBRecord($database,'RodinnyPrislusnik');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC_RodinnehoPrislusnika','',false, 15, $this->noverodnecislo ? $this->noverodnecislo : $this->rodnecislo);

        $params = $database->createParameters();

        $rec->setWhere(" RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

        if (!$rec->deleteRecord()) {
            throw new Exception('Chyba pri mazani rodinneho prislusnika.');
        }

        return true;
    }

    /**
     * @param int $index index prvku v postu
     * @throws Exception
     */
    private function nactiPost($index = null)
    {
        $poradiPrislusnika = '';

        if (!is_null($index)) {
            $poradiPrislusnika = '_' . $index;
        }

        $this->jmeno = filter_input(INPUT_POST,'rodina_prijmeni'.$poradiPrislusnika,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->prijmeni = filter_input(INPUT_POST, 'rodina_jmeno'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        $rc = filter_input(INPUT_POST, 'rodina_rc'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        if (is_null($this->rodnecislo)) {
            $this->rodnecislo = $rc;
        }
        elseif ($rc != $this->rodnecislo) {
            $this->noverodnecislo = $rc;
        }
        $this->datumnarozeni = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rodina_datumnarozeni'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS));
        $this->rodinnyvztah = filter_input(INPUT_POST,'rodina_vztah'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zdravotnistav = filter_input(INPUT_POST, 'rodina_zdravotni_stav'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['rodina_drzitelztp'.$poradiPrislusnika] != '') {
            $this->drzitelZTPod = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rodina_drzitelztp'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS));
        }
        $this->vyzivovanaosoba = isset($_POST['rodina_vyzivovanaosoba'. $poradiPrislusnika]);
        $this->stejnaadresa = isset($_POST['rodina_stejnaadresa' . $poradiPrislusnika]);
    }

}


/**
 * Class RodinnyStav
 * @package Ron\RonPortal\Zamestnanci
 * Pouziva se i v osobnich udajich!
 */
class RodinnyStav
{
    public $rc;
    public $platnostOd;
    public $platnostDo;

    public $rodinnyStav;
    public $rodinnyStavStr;

    /** Vraci vsechny rodinne stavy osoby, pokud je zadana i platnost od, pak zvoleny rodinny stav
     *  a pokud je $aktualni = true, tak jen aktualni.
     * @param Database $database
     * @param string $rc
     * @param int $platnostOd
     * @param bool $aktualni
     * @return RodinnyStav[]|RodinnyStav
     * @throws Exception
     */
    public static function nactiStav($database, $rc, $platnostOd = 0, $aktualni = false)
    {
        $params = $database->createParameters();

        $sql =  'SELECT PlatnostOd, PlatnostDo, KodRodinnehoStavu, S_RodinnyStav.Popis ' .
                ' FROM RodinnyStav JOIN S_RodinnyStav ON (S_RodinnyStav.Kod = RodinnyStav.KodRodinnehoStavu) ' .
                ' WHERE RodinnyStav.RC = ' . $params->addParameter($rc);
        if ($aktualni) {
            $sql .=
                ' AND RodinnyStav.PlatnostOd <= ' . $params->addParameter($database->dateToParam(time())) .
                ' AND (RodinnyStav.PlatnostDo >= ' . $params->addParameter($database->dateToParam(time())) . ' OR RodinnyStav.PlatnostDo IS NULL)';
        }

        $sql .= ' ORDER BY PlatnostOd';

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        $seznamStavu = [];

        while ($row = $cursor->fetch()) {
            if ($database->stringToTimestamp($row[0]) == $platnostOd) {
                $rodinnyStav = new RodinnyStav();
                $rodinnyStav->nactiStavZRadku($database, $row);

                return $rodinnyStav;
            }

            $rodinnyStav = new RodinnyStav();
            $rodinnyStav->nactiStavZRadku($database, $row);

            $seznamStavu[] = $rodinnyStav;
        }

        return $seznamStavu;
    }

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param string $rc
     * @param bool $povinne
     * @param bool $ukoncitPredchozi
     * @return bool
     * @throws Exception
     */
    public static function ulozStav($database, $platnostOd = 0, $rc = '', $povinne = false, $ukoncitPredchozi = false)
    {
        $rodinnyStav = new RodinnyStav();
        $rodinnyStav->nactiPost();

        if ($platnostOd) {
            $rodinnyStav->platnostOd = $platnostOd;
        }

        if ($rc != '') {
            $rodinnyStav->rc = $rc;
        }

        if ($povinne) {
            if ($rodinnyStav->rc == '' || $rodinnyStav->platnostOd == 0 || $rodinnyStav->rodinnyStav == '') {
                MessageManager::addErrorMessage(\Ron\ronText('Rodinny_stav_chybi_pole'));
                return false;
            }
        }
        elseif (!$rodinnyStav->rodinnyStav) {
            return true;
        }


        $rec = new RonPortalDBRecord($database,'RodinnyStav');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',false, '15', $rodinnyStav->rc);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $rodinnyStav->platnostOd);
        $rec->addField('KodRodinnehoStavu','int', false, 0, $rodinnyStav->rodinnyStav);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $rodinnyStav->platnostDo ? $rodinnyStav->platnostDo : null);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rodinnyStav->rc) .
                        ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($rodinnyStav->platnostOd)), $params);

        if (!$rec->updateOrInsert()) {
            return false;
        }

        if ($ukoncitPredchozi)
            $rodinnyStav->ukonciPredchozi($database);

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws DBException
     */
    public static function aktualizujPlatnostStavu($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $rec = new RonPortalDBRecord($database,'RodinnyStav');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',true, '15');
        $rec->addField('PlatnostOd', 'timestamp', true);
        $rec->addField('KodRodinnehoStavu','int', true);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        if (!$rec->loadRecord())
            return false;

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        if (!$rec->updateRecord())
            return false;

        return true;
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiStavZRadku($database, $row)
    {
        $this->platnostOd = $database->stringToTimestamp($row[0]);
        $this->platnostDo = $database->stringToTimestamp($row[1]);

        $this->rodinnyStav = $row[2];
        $this->rodinnyStavStr = $row[3];
    }

    /**
     *
     */
    private function nactiPost()
    {
        $this->rc = filter_input(INPUT_POST,'RC',FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostOd = filter_input(INPUT_POST, 'RodinnyStavPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostDo = filter_input(INPUT_POST, 'RodinnyStavPlatnostDo', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->rodinnyStav = filter_input(INPUT_POST, 'RodinnyStav', FILTER_SANITIZE_SPECIAL_CHARS);
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function ukonciPredchozi($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE RodinnyStav ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));

        $database->sql($sql, $params);
    }
}

/**
 * Class ZivotniPojisteniOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class ZivotniPojisteniOsoby
{
    public $rc;

    /** @var string */
    public $pojistovna;
    /** @var string     Nazev pojistovny */
    public $popis;
    /** @var string */
    public $cisloSmlouvy;

    /** @var int */
    public $platnostOd;
    /** @var int */
    public $platnostDo;
    /** @var int */
    public $serioveCisloUctu1;
    /** @var float */
    public $prispevekZamestnavatele;
    /** @var int */
    public $serioveCisloUctu2;
    /** @var float */
    public $prispevekZamestnance;


    /**
     * @param Database $database
     * @param $rc
     * @return array
     * @throws DBException
     */
    public static function nactiZivotniPojisteniOsoby($database, $rc){
        $params = $database->createParameters();
        $sql =  "SELECT Pojistovna, CisloSmlouvy, SmlouvaZivotniPojisteni.PrispevekZamestnance, SmlouvaZivotniPojisteni.PlatnostOd, SmlouvaZivotniPojisteni.SerioveCisloUctu1, " .    //0..4
                "SmlouvaZivotniPojisteni.PrispevekZamestnavatele, SmlouvaZivotniPojisteni.SerioveCisloUctu2, Pojistovna.Popis, SmlouvaZivotniPojisteni.PlatnostDo " .       //5..8
                "FROM SmlouvaZivotniPojisteni LEFT JOIN Pojistovna ON (SmlouvaZivotniPojisteni.Pojistovna = Pojistovna.Kod)  " .
                "WHERE (RC = " . $params->addParameter($rc) . ") ".
                "ORDER BY PlatnostOd";

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        $seznamZivotnichPojisteni = [];

        while ($row = $cursor->fetch()) {
            $zivotniPojisteni = new ZivotniPojisteniOsoby();
            $zivotniPojisteni->rc = $rc;
            $zivotniPojisteni->nactiDataZRadku($database, $row);
            $seznamZivotnichPojisteni[] = $zivotniPojisteni;
        }

        return $seznamZivotnichPojisteni;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiDataZRadku($database, $row){
        $this->pojistovna = $row[0];
        $this->cisloSmlouvy = $row[1];
        $this->prispevekZamestnance = $row[2];
        $this->platnostOd = $database->stringToTimestamp($row[3]);
        $this->platnostDo = $database->stringToTimestamp($row[8]);
        $this->serioveCisloUctu1 = $row[4];
        $this->prispevekZamestnavatele = $row[5];
        $this->serioveCisloUctu2 = $row[6];
        $this->popis = $row[7];
    }
}




/**
 * Class PenzijniPripojisteniOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class PenzijniPripojisteniOsoby
{
    public $rc;

    public $penzijniFond;
    public $cisloSmlouvy;
    public $platnostOd;
    public $platnostDo;
    public $serioveCisloUctu1;
    public $prispevekZamestnavatele;
    public $serioveCisloUctu2;
    public $prispevekZamestnance;
    public $popisPenzijnihoFondu;


    /**
     * @param Database $database
     * @param $rc
     * @return array
     * @throws DBException
     */
    public static function nactiPenzijniPripojisteniOsoby($database, $rc){
        $params = $database->createParameters();
        $sql =  "SELECT PenzijniFond, CisloSmlouvy, SmlouvaPenzijnihoPrip.PrispevekZamestnance, SmlouvaPenzijnihoPrip.PlatnostOd, SmlouvaPenzijnihoPrip.SerioveCisloUctu1, " .    //0..4
                "SmlouvaPenzijnihoPrip.PrispevekZamestnavatele, SmlouvaPenzijnihoPrip.SerioveCisloUctu2, PenzijniFond.Popis, SmlouvaPenzijnihoPrip.PlatnostDo " .       //5..8
                "FROM SmlouvaPenzijnihoPrip LEFT JOIN PenzijniFond ON (SmlouvaPenzijnihoPrip.PenzijniFond = PenzijniFond.Kod)  " .
                "WHERE (RC = " . $params->addParameter($rc) . ") ".
                "ORDER BY PlatnostOd";

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        $seznamPenzijnihoPripojisteni = [];

        while ($row = $cursor->fetch()) {
            $penzijniPripojisteni = new PenzijniPripojisteniOsoby();
            $penzijniPripojisteni->rc = $rc;
            $penzijniPripojisteni->nactiDataZRadku($database, $row);
            $seznamPenzijnihoPripojisteni[] = $penzijniPripojisteni;
        }

        return $seznamPenzijnihoPripojisteni;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiDataZRadku($database, $row){
        $this->penzijniFond = $row[0];
        $this->cisloSmlouvy = $row[1];
        $this->prispevekZamestnance = $row[2];
        $this->platnostOd = $database->stringToTimestamp($row[3]);
        $this->platnostDo = $database->stringToTimestamp($row[8]);
        $this->serioveCisloUctu1 = $row[4];
        $this->prispevekZamestnavatele = $row[5];
        $this->serioveCisloUctu2 = $row[6];
        $this->popisPenzijnihoFondu = $row[7];
    }
}


/**
 * Class PracovniPomuckaOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class PracovniPomuckaOsoby
{
    public $typVelikosti;
    public $typVelikostiPopis;
    public $velikost;

    /**
     * @param Database $database
     * @param string $rc
     * @return PracovniPomuckaOsoby[]
     * @throws DBException
     */
    public static function nactiVelikostiPomucekOsoby($database, $rc)
    {
        $params = $database->createParameters();
        $sql =  "SELECT TypVelikosti, Velikost " .
                "FROM VelikostPPOsoby " .
                "WHERE (RCOsoby = " . $params->addParameter($rc) . ") ".
                "ORDER BY TypVelikosti";
        
        MessageManager::addDebugLog($sql);
        
        $cursor = $database->getSqlResult($sql,$params);
        
        $pracovniPomucky = [];
        
        while ($row = $cursor->fetch()) {
            $pracovniPomucka = new PracovniPomuckaOsoby();
            $pracovniPomucka->typVelikosti = $row[0];
            $pracovniPomucka->velikost = $row[1];
            
            $pracovniPomucky[$row[0]] = $pracovniPomucka;
        }
        
        return $pracovniPomucky;
    }

    /**
     * @param Database $database
     * @return PracovniPomuckaOsoby[]
     * @throws DBException
     */
    public static function nactiTypyVelikostiPomucek($database)
    {
        $sql =  "SELECT Kod, Popis " .
                "FROM TypVelikosti " .
                "ORDER BY Kod";
        
        $cursor = $database->getSqlResult($sql);
        
        $seznamVelikosti = [];
        
        while ($row = $cursor->fetch()) {
            $typVelikosti = new PracovniPomuckaOsoby();
            $typVelikosti->typVelikosti = $row[0];
            $typVelikosti->typVelikostiPopis = $row[1];
            
            $seznamVelikosti[] = $typVelikosti;
        }
        
        return $seznamVelikosti;
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @param bool $povinne
     * @return bool
     * @throws Exception
     */
    public function ulozVelikostPomuckyOsoby($database, $rc, $povinne = false)
    {
        $this->nactiPost();

        if ($povinne and (($rc == '') or ($this->typVelikosti == '') or ($this->velikost == ''))) {
            throw new Exception(\Ron\ronText('Pomucky_chybi_povinne_pole'));
        }

        $rec = new RonPortalDBRecord($database,'VelikostPPOsoby');
        
        $rec->logDescription = 'Editace zamestnance - portal';
        
        $rec->addField('RCOsoby', '', false, 15, $rc);
        $rec->addField('TypVelikosti', '', false, 10, $this->typVelikosti);
        $rec->addField('Velikost', '', false, 20, $this->velikost);

        $params = $database->createParameters();

        $rec->setWhere(" RCOsoby = " . $params->addParameter($rc) .
                        " AND TypVelikosti = " . $params->addParameter($this->typVelikosti), $params);
        
        
        if (!$this->velikost) {
            if ($rec->loadRecord()) {
                //pokud je u osoby zaznam o velikosti, smaze se
                if (!$rec->deleteRecord()) {
                    throw new Exception('Chyba pri mazani velikosti pomucky osoby');
                }
                // smazani probehlo uspesne
                else {
                    return true;
                }
            }
            //zaznam velikosti neexistuje, nic se neprovede
            else {
                return false;
            }
        }
        
        if (!$rec->updateOrInsert()) {
            throw new Exception('Chyba pri vkladani/aktualizaci velikosti pomucky osoby');
        }
        else {
            return true;
        }
    }
    
    private function nactiPost()
    {
        $velikost = explode(';', $_POST['velikost_pomucky_'.$this->typVelikosti]);
        $this->velikost = $velikost[1];
    }
}

/**
 * Class ZdravotniPojistovnaOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class ZdravotniPojistovnaOsoby
{
    public $rc;
    public $kodZP;
    public $nazevZP;
    public $platnostOd;
    public $platnostDo;
    public $cisloPojistence;

    /**
     *
     * @param Database $database
     * @param null|string $cisloPojistence
     * @param int $platnostOd
     * @param bool $platnostOdPrvnihoDneMesice pokud je true, nastavi se platnostod na prvni den v mesici zadane platnosti od
     * @return bool
     * @throws Exception
     */
    public function ulozZdravotniPojistovnuOsoby($database, $cisloPojistence = null, $platnostOd = 0, $platnostOdPrvnihoDneMesice = true)
    {
        $this->nactiPost();
        
        if ($platnostOd > 0)
            $this->platnostOd = $platnostOd;

        if (!is_null($cisloPojistence))
            $this->cisloPojistence = $cisloPojistence;

        if (!$this->rc)
            return false;

        $this->platnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($this->platnostOd) : $this->platnostOd;

        $this->ukonciPredchoziPojistovnu($database);

        $rec = new RonPortalDBRecord($database,'PojisteniOsobyU_ZP');
        
        $rec->logDescription = 'Editace zamestnance - portal';
        
        $rec->addField('RC', '', false, 15, $this->rc);
        $rec->addField('KodZdravotniPojistovny', '', false, 10, $this->kodZP);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('CisloPojistence', '', false, 15, $this->cisloPojistence);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($this->rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($this->platnostOd)), $params);
        
        $rec->loadRecord();
        
        if (!$rec->updateOrInsert()) {
            throw new Exception('Chyba pri zapisu pojistovny zamestnance.');
        }
        else {
            return true;
        }
    }

    /**
     *
     * @param Database $database
     * @param string $rc RC osoby
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @param bool $platnostOdPrvnihoDneMesice
     * @param string $kodZP
     * @return bool
     * @throws Exception
     */
    public static function aktualizujPlatnostZdravotniPojistovnyOsoby($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd, $platnostOdPrvnihoDneMesice = true, $kodZP = null)
    {
        $novaPlatnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($novaPlatnostOd) : $novaPlatnostOd;

        $rec = new RonPortalDBRecord($database,'PojisteniOsobyU_ZP');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15);
        if (!is_null($kodZP)) {
            $rec->addField('KodZdravotniPojistovny', '', true, 10);
        }
        $rec->addField('PlatnostOd', 'timestamp', true, 0);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)) .
                        (is_null($kodZP) ? "" : " AND KodZdravotniPojistovny = " . $params->addParameter($kodZP)), $params);

        $rec->loadRecord();

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        if (!$rec->updateRecord()) {
            throw new Exception('Chyba pri aktualizaci platnosti pojistovny zamestnance.');
        }
        else {
            return true;
        }
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @param int $platnost den ke kteremu je pojistovna platna
     * @return ZdravotniPojistovnaOsoby
     * POZOR! Pouziva se i v osobnich udajich!
     * @throws DBException
     */
    public static function nactiZdravotniPojistovnuOsoby($database, $rc, $platnost = 0)
    {
        if ($platnost == 0) {
            $platnost = time();
        }
        
        $params = $database->createParameters();
        
        $sql =  "SELECT KodZdravotniPojistovny, PlatnostOd, CisloPojistence, ZdravotniPojistovna.Popis, PlatnostDo " .
                " FROM PojisteniOsobyU_ZP, ZdravotniPojistovna ".
                " WHERE ZdravotniPojistovna.Kod = PojisteniOsobyU_ZP.KodZdravotniPojistovny ".
                "  AND RC = " . $params->addParameter($rc) .
                "  AND PlatnostOd <= " . $params->addParameter($database->timestampToParam($platnost)) .
                "  AND (PlatnostDo >= " . $params->addParameter($database->timestampToParam($platnost)) . " OR PlatnostDo IS NULL)";
        
        MessageManager::addDebugLog($sql);        
        
        $cursor = $database->getSqlResult($sql,$params);
        
        if ($row = $cursor->fetch()) {
            $zdravotniPojistovna = new ZdravotniPojistovnaOsoby();
            $zdravotniPojistovna->rc = $rc;
            $zdravotniPojistovna->nactiDataZRadku($database, $row);
            return $zdravotniPojistovna;
        } else {
            return null;
        }
    }
    
    /**
     * 
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->kodZP = $row[0];
        $this->platnostOd = $database->stringToTimestamp($row[1]);
        $this->platnostDo = $database->stringToTimestamp($row[4]);
        $this->cisloPojistence = $row[2];
        $this->nazevZP = $row[3];
    }

    /**
     * @throws Exception
     */
    private function nactiPost()
    {
        //$this->rc = filter_input(INPUT_POST, 'RC', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->kodZP = filter_input(INPUT_POST, 'ZdravotniPojistovna', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->cisloPojistence = filter_input(INPUT_POST, 'ZdravotniPojistovnaCisloPojistence', FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['ZdravotniPojistovnaPlatnostOd'] != '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'ZdravotniPojistovnaPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS));
        }
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function ukonciPredchoziPojistovnu($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE PojisteniOsobyU_ZP ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));
        
        $database->sql($sql, $params);
    }
}


/**
 * Class AdresaOsoby
 * @package Ron\RonPortal\Zamestnanci
 * POZOR! Pouziva se i v osobnich udajich
 */
class AdresaOsoby
{
    public $rc;
    public $trvaleBydliste;
    public $kontaktniAdresa;
    public $platnostOd;
    public $platnostDo;
    public $ulice;
    public $mesto;
    public $obec;
    public $obecPopis;
    public $psc;
    public $stat;
    public $nazevStatu;
    public $telefon1;
    public $telefon2;
    public $mobil;
    public $email;
    public $zpusobDopravy;


    /**
     * @param Database $database
     * @param string $rc
     * @param int $platnost
     * @return AdresaOsoby[]
     * @throws DBException
     */
    public static function nactiAdresuOsoby($database, $rc, $platnost = 0, $trvale = null)
    {
        $params = $database->createParameters();
        
        $sql =  "SELECT RC, TrvaleBydliste, PlatnostOd, PlatnostDo, Ulice, Mesto, Obec, PSC, Stat, " . // 0 - 8
                "   Telefon1, Telefon2, Mobil, ZpusobDopravy, KontaktniAdresa, E_Mail, Stat.Popis, Obec.Popis " . // 9 - 16
                "  FROM AdresaOsoby LEFT OUTER JOIN Stat ON Stat.Kod = AdresaOsoby.Stat LEFT JOIN Obec ON (Obec.Kod = AdresaOsoby.Obec)".
                " WHERE RC = " . $params->addParameter($rc);
        
        if ($platnost > 0) {
            $sql .= ' AND PlatnostOd <= ' .  $params->addParameter($database->timestampToParam($platnost)) .
                    ' AND (PlatnostDo >= ' . $params->addParameter($database->timestampToParam($platnost)) . ' OR PlatnostDo IS NULL) ';
        }

        if (!is_null($trvale)) {
            $sql .= ' AND TrvaleBydliste = ' . $params->addParameter($trvale);
        }
        
        $sql .= ' ORDER BY TrvaleBydliste DESC';
        
        $cursor = $database->getSqlResult($sql, $params);
        
        $seznamAdres = [];
        
        while ($row = $cursor->fetch()) {
            $adresa = new AdresaOsoby();
            $adresa->rc = $rc;
            $adresa->nactiAdresuZRadku($database, $row);
            
            $seznamAdres[] = $adresa;
        }
        
        return $seznamAdres;
    }

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param int $index index prvku v postu
     * @return bool
     * @throws Exception
     */
    public function ulozAdresuOsoby($database, $platnostOd = 0, $index = null)
    {
        $this->nactiPost($index);

        if (is_null($this->platnostOd) && $platnostOd > 0)
            $this->platnostOd = $platnostOd;
        
        if (!($this->rc && $this->platnostOd))
            return false;
        
        $rec = new RonPortalDBRecord($database,'AdresaOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $this->rc);
        $rec->addField('TrvaleBydliste', 'int', false, 0, $this->trvaleBydliste);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $this->platnostDo ? $this->platnostDo : null);
        $rec->addField('Ulice', '', false, 40, $this->ulice ? $this->ulice : null);        
        $rec->addField('Mesto', '', false, 40, $this->mesto ? $this->mesto : null);
        $rec->addField('Obec', '', false, 10, $this->obec ? $this->obec : null);
        $rec->addField('PSC', '', false, 10, $this->psc ? $this->psc : null);
        $rec->addField('Stat', '', false, 10, $this->stat ? $this->stat : null);
        $rec->addField('Telefon1', '', false, 20, $this->telefon1 ? $this->telefon1 : null);
        $rec->addField('Telefon2', '', false, 20, $this->telefon2 ? $this->telefon2 : null);
        $rec->addField('Mobil', '', false, 20, $this->mobil ? $this->mobil : null);
        $rec->addField('ZpusobDopravy', 'int', false, 0, $this->zpusobDopravy ? 1 : 0);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($this->rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($this->platnostOd)) .
                        " AND TrvaleBydliste = " . $params->addParameter($this->trvaleBydliste), $params);

        if (!$rec->updateOrInsert()) {
            throw new Exception('Chyba pri vkladani/aktualizaci adresy osoby.');
        }

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws Exception
     */
    public static function aktualizujPlatnostAdresyOsoby($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $params = $database->createParameters();
        $sql =
            'UPDATE AdresaOsoby SET PlatnostOd = ' . $params->addParameter($database->dateToParam($novaPlatnostOd)) .
            ' WHERE RC = ' . $params->addParameter($rc) .
            '   AND PlatnostOd = ' . $params->addParameter($database->dateToParam($puvodniPlatnostOd));

        $database->sql($sql, $params);

        Log::addRecordToDBLog($database, 'AdresaOsoby', 'UPDATE', 'Editace zamestnance - portal', "$sql");

        return true;
    }

    /**
     * 
     * @param Database $database
     * @param array $row
     */
    private function nactiAdresuZRadku($database, $row)
    {
        $this->trvaleBydliste = (int)$row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->ulice = $row[4];
        $this->mesto = $row[5];
        $this->obec = $row[6];
        $this->psc = $row[7];
        $this->stat = $row[8];
        $this->telefon1 = $row[9];
        $this->telefon2 = $row[10];
        $this->mobil = $row[11];
        $this->zpusobDopravy = (int)$row[12];
        $this->kontaktniAdresa = (int)$row[13];
        $this->email = $row[14];
        $this->nazevStatu = $row[15];
        $this->obecPopis = $row[16];
    }
    
    /**
     * @param int $index index prvku v postu
     * @throws Exception
     */
    private function nactiPost($index = null)
    {
        $poradiAdresy = '';

        if (!is_null($index)) {
            $poradiAdresy = '_' . $index;
        }

        $this->trvaleBydliste = filter_input(INPUT_POST,'adresa_trvale'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->kontaktniAdresa = filter_input(INPUT_POST, 'adresa_kontaktni'.$poradiAdresy, FILTER_SANITIZE_SPECIAL_CHARS);
        
        if ($_POST['adresa_platnostOd'.$poradiAdresy] != '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'adresa_platnostOd'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if ($_POST['adresa_platnostDo'.$poradiAdresy] != '') {
            $this->platnostDo = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'adresa_platnostDo'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS));
        }
        
        $this->ulice = filter_input(INPUT_POST,'adresa_ulice'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->mesto = filter_input(INPUT_POST,'adresa_mesto'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->obec = filter_input(INPUT_POST,'adresa_obec'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->psc = filter_input(INPUT_POST,'adresa_psc'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->stat = filter_input(INPUT_POST,'adresa_stat'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->telefon1 = filter_input(INPUT_POST,'adresa_telefon1'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->telefon2 = filter_input(INPUT_POST,'adresa_telefon2'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->mobil = filter_input(INPUT_POST,'adresa_mobil'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zpusobDopravy = filter_input(INPUT_POST,'adresa_zpusob'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        
    }
}


/**
 * Class ComboBoxObec
 * @package Ron\RonPortal\Zamestnanci
 */
class ComboBoxObec extends ComboBox implements ICreateItem
{
    public function __construct($varName)
    {
        parent::__construct($varName);
        $this->emptyRow = ' ';
        $this->className .= ' adresaObec';
    }

    /**
     * @param $database
     * @param $selectedValue
     * @throws DBException
     */
    public function loadDataCustom($database, $selectedValue)
    {
        $sql = "SELECT Obec.Kod, Obec.Popis, Okres.Popis  FROM Obec JOIN Okres ON (Okres.Kod = Obec.Okres) ORDER BY Obec.Popis";

        parent::loadData($database, $sql, $selectedValue, null);
        $this->setSelectedOption($selectedValue);
    }

    public function createItem($row)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[0];
        $caption = $row[1] . ' (' . \Ron\ronText('okres') . ' ' . $row[2] . ')';

        return new Option($key, $caption);
    }

}

/**
 * Class ComboBoxMesto
 * @package Ron\RonPortal\Zamestnanci
 */
class ComboBoxMesto extends ComboBox implements ICreateItem
{
    public function __construct($varName, $data = null, $keyFieldName = 'key', $valueFieldName = 'value')
    {
        parent::__construct($varName, $data, $keyFieldName, $valueFieldName);
        $this->emptyRow = ' ';
        $this->className .= ' adresaMesto';
    }

    /**
     * @param $database
     * @param $selectedValue
     * @throws DBException
     */
    public function loadDataCustom($database, $selectedValue)
    {
        $sql =
            "SELECT Mesto.PSC, Mesto.Mesto, Mesto.Posta, Mesto.Stat, Obec.Kod, Obec.Popis, Okres.Popis 
             FROM Mesto LEFT JOIN Obec ON (Mesto.Obec = Obec.Kod) 
             LEFT JOIN Okres ON (Okres.Kod = Obec.Okres) 
             ORDER BY Mesto.Mesto, Obec.Popis";

        parent::loadData($database, $sql, $selectedValue, null);
        $this->setSelectedOption($selectedValue);
    }

    public function createItem($row)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[1];
        $caption = $row[1] . ' - ' . \Ron\ronText('posta') . ' ' . $row[2]; // . ' (' . \Ron\ronText('obec') . ' ' . $row[5] . ' - ' . \Ron\ronText('okres') . ' ' . $row[6];

        $option = new Option($key, $caption);
        $option->params = [
            'data-adresa-obec'  => $row[4],
            'data-adresa-psc'   => $row[0],
            'data-adresa-stat'  => $row[3]
        ];

        return $option;
    }
}

/**
 * Class PrukazOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class PrukazOsoby
{
    public $rc;
    public $typPrukazu;
    public $typPrukazuPopis;
    public $cisloPrukazu;
    public $platnostOd;
    public $platnostDo;
    public $poznamka;

    /**
     *  Nacte seznam vsech prukazu osoby/vsech prukazu vybraneho typu.
     * @param Database $database
     * @param string $rc
     * @param int $typPrukazu
     * @return PrukazOsoby[]
     * @throws DBException
     */
    public static function nactiPrukazyOsoby($database, $rc, $typPrukazu = 0)
    {
        $params = $database->createParameters();

        $sql =  "SELECT TypPrukazu, CisloPrukazu, PlatnostOd, PlatnostDo,  " . $database ->castWideMemo('Poznamka') .
                " FROM Prukaz" .
                " WHERE RC = " . $params->addParameter($rc);

        if ($typPrukazu > 0) {
            $sql .= ' AND TypPrukazu = ' . $params->addParameter($typPrukazu);
        }

        $sql .= ' ORDER BY TypPrukazu, PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $prukazy = [];

        while ($row = $cursor->fetch()) {
            $prukaz = new PrukazOsoby();
            $prukaz->rc = $rc;
            $prukaz->nactiDataZRadku($database, $row);

            if ($typPrukazu > 0) {
                $prukazy[] = $prukaz;
            }
            else {
                $prukazy[$row[0]][] = $prukaz;
            }
        }

        return $prukazy;
    }

    /**
     *
     * @param Database $database
     * @return PrukazOsoby[]
     * @throws DBException
     */
    public static function nactiTypyPrukazu($database)
    {
        $sql =  "SELECT Kod, Popis " .
                "FROM TypPrukazu " .
                "ORDER BY Kod";

        $cursor = $database->getSqlResult($sql);

        $seznamTypu = [];

        while ($row = $cursor->fetch()) {
            $typPrukazu = new PrukazOsoby();
            $typPrukazu->typPrukazu = $row[0];
            $typPrukazu->typPrukazuPopis = $row[1];

            $seznamTypu[] = $typPrukazu;
        }

        return $seznamTypu;
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @throws Exception
     * @throws DBException
     */
    public function ulozPrukazOsoby($database, $rc)
    {
        $this->nactiPost();

        $rec = new RonPortalDBRecord($database,'Prukaz');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $rc);
        $rec->addField('TypPrukazu', 'int', true, 0, $this->typPrukazu);
        $rec->addField('CisloPrukazu', '', false, 20, $this->cisloPrukazu);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $this->platnostDo ? $this->platnostDo : null);
        $rec->addField('Poznamka', 'memo', false, 0, $this->poznamka);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
                        " AND TypPrukazu = " . $params->addParameter($this->typPrukazu) .
                        " AND CisloPrukazu = " . $params->addParameter($this->cisloPrukazu), $params);


        $rec->loadRecord();

        if (!$rec->updateOrInsert()) {
            throw new Exception('Chyba pri vkladani/aktualizaci prukazu osoby');
        }
    }

    /**
     * @throws Exception
     */
    private function nactiPost()
    {
        $this->cisloPrukazu = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_cislo', FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['prukaz_' . $this->typPrukazu . '_platnostOd'] !== '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp($this->platnostOd = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_platnostOd', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if (isset($_POST['prukaz_' . $this->typPrukazu . '_platnostDo']) && ($_POST['prukaz_' . $this->typPrukazu . '_platnostDo'] !== '')) {
            $this->platnostDo = DateUtils::dateStringToTimestamp($this->platnostDo = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_platnostDo', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        $this->poznamka = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_poznamka', FILTER_SANITIZE_SPECIAL_CHARS);
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->typPrukazu = $row[0];
        $this->cisloPrukazu = $row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->poznamka = $row[4];
    }
}

/**
 * Class ZdravotniStav
 * @package Ron\RonPortal\Zamestnanci
 */
class ZdravotniStav
{
    public $rc;
    public $platnostOd;
    public $platnostDo;

    public $zdravotniStav;
    public $zdravotniStavStr;
    public $drzitelPrukazuZTTP;
    public $osetrujiciLekar;
    public $osetrujiciLekarJmenoPrijmeni;
    public $zps;
    public $zps_tzp;

    /** Vraci vsechny zdravotni stavy osoby, pokud je zadana i platnost od, pak zvoleny rodinny stav
     *  a pokud je $aktualni = true, tak jen aktualni.
     * @param Database $database
     * @param string $rc
     * @param int $platnostOd
     * @param bool $aktualni
     * @return ZdravotniStav[]|ZdravotniStav
     * @throws Exception
     */
    public static function nactiStav($database, $rc, $platnostOd = 0, $aktualni = false)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PlatnostOd, PlatnostDo, ZdravotniStav, S_ZdravotniStav.Popis, DrzitelPrukazuZTTP, ZdravotniStavOsoby.OsetrujiciLekar, ' . //0 - 5
            '  OsetrujiciLekar.JmenoLekare, OsetrujiciLekar.Prijmeni, ZPS, ZPS_TZP ' . //6 - 9
            ' FROM ZdravotniStavOsoby JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = ZdravotniStavOsoby.ZdravotniStav) ' .
            '   LEFT JOIN OsetrujiciLekar ON (OsetrujiciLekar.Kod = ZdravotniStavOsoby.OsetrujiciLekar) ' .
            ' WHERE ZdravotniStavOsoby.RC = ' . $params->addParameter($rc);
        if ($platnostOd) {
            $sql .=
                ' AND ZdravotniStavOsoby.PlatnostOd = ' . $params->addParameter($database->timestampToParam($platnostOd));
        } else if ($aktualni) {
            $sql .=
                ' AND ZdravotniStavOsoby.PlatnostOd <= ' . $params->addParameter($database->dateToParam(time())) .
                ' AND (ZdravotniStavOsoby.PlatnostDo >= ' . $params->addParameter($database->dateToParam(time())) . ' OR ZdravotniStavOsoby.PlatnostDo IS NULL)';
        }

        $sql .= ' ORDER BY PlatnostOd';

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        if ($platnostOd) {
            if (!$row = $cursor->fetch()) {
                return null;
            }

            $zdravotniStav = new ZdravotniStav();
            $zdravotniStav->nactiStavZRadku($database, $row);

            return $zdravotniStav;
        }

        $seznamStavu = [];

        while ($row = $cursor->fetch()) {
            $zdravotniStav = new ZdravotniStav();
            $zdravotniStav->nactiStavZRadku($database, $row);

            $seznamStavu[] = $zdravotniStav;
        }

        return $seznamStavu;
    }

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param string $rc
     * @param bool $povinne
     * @param bool $ukoncitPredchozi
     * @return bool
     * @throws Exception
     */
    public static function ulozStav($database, $platnostOd = 0, $rc = '', $povinne = false, $ukoncitPredchozi = false)
    {
        $zdravotniStav = new ZdravotniStav();
        $zdravotniStav->nactiPost();

        if ($platnostOd) {
            $zdravotniStav->platnostOd = $platnostOd;
        }

        if ($rc != '') {
            $zdravotniStav->rc = $rc;
        }

        if ($povinne) {
            if ($zdravotniStav->rc == '' || $zdravotniStav->platnostOd == 0 || $zdravotniStav->zdravotniStav == '') {
                MessageManager::addErrorMessage(\Ron\ronText('Zdravotni_stav_chybi_pole'));
                return false;
            }
        }
        elseif (!$zdravotniStav->zdravotniStav) {
            return true;
        }


        $rec = new RonPortalDBRecord($database,'ZdravotniStavOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',false, '15', $zdravotniStav->rc);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $zdravotniStav->platnostOd);
        $rec->addField('ZdravotniStav','int', false, 0, $zdravotniStav->zdravotniStav);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $zdravotniStav->platnostDo ?: null);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($zdravotniStav->rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($zdravotniStav->platnostOd)), $params);

        if (!$rec->updateOrInsert()) {
            return false;
        }

        if ($ukoncitPredchozi)
            $zdravotniStav->ukonciPredchozi($database);

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws DBException
     */
    public static function aktualizujPlatnostStavu($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $rec = new RonPortalDBRecord($database,'ZdravotniStavOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',true, 15);
        $rec->addField('PlatnostOd', 'timestamp', true);
        $rec->addField('ZdravotniStav','int', true);
        $rec->addField('PlatnostDo', 'timestamp', true);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        if (!$rec->loadRecord())
            return false;

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        if (!$rec->updateRecord())
            return false;

        return true;
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiStavZRadku($database, $row)
    {
        $this->platnostOd = $database->stringToTimestamp($row[0]);
        $this->platnostDo = $database->stringToTimestamp($row[1]);

        $this->zdravotniStav = $row[2];
        $this->zdravotniStavStr = $row[3];

        $this->drzitelPrukazuZTTP = $row[4];
        $this->osetrujiciLekar = $row[5];
        $this->osetrujiciLekarJmenoPrijmeni = $row[6] . ' ' . $row[7];
        $this->zps = $row[8];
        $this->zps_tzp = $row[9];
    }

    /**
     *
     */
    private function nactiPost()
    {
        $this->rc = filter_input(INPUT_POST,'RC',FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostOd = filter_input(INPUT_POST, 'ZdravotniStavPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostDo = filter_input(INPUT_POST, 'ZdravotniStavPlatnostDo', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zdravotniStav = filter_input(INPUT_POST, 'ZdravotniStav', FILTER_SANITIZE_SPECIAL_CHARS);
        //TODO: doplnit dalsi pole
    }

    /**
     * @param Database $database
     * @throws \Exception
     */
    private function ukonciPredchozi($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE ZdravotniStavOsoby ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));

        $database->sql($sql, $params);
    }
}

/**
 * Class EvidencniStav
 * @package Ron\RonPortal\Zamestnanci
 */
class EvidencniStav
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $idPP;
    public $typPP;

    public $kodStavu;
    public $popisStavu;

    public $platnostOd;
    public $platnostDo;
    public $poznamka;

    /**
     * @param Database $database
     * @param $rc
     * @return EvidencniStav[]|array
     * @throws DBException
     */
    public static function nactiEvidencniStavy($database, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT EvidencniStav.KodEvidencnihoStavu, TypEvidencnihoStavu.Popis, PlatnostOd, PlatnostDo,  " . $database ->castWideMemo('EvidencniStav.Poznamka') . ", " .
            "   PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.TypPracovnihoPomeru, PracovniPomer.ID " .
            "FROM EvidencniStav JOIN TypEvidencnihoStavu ON (TypEvidencnihoStavu.Kod = EvidencniStav.KodEvidencnihoStavu) " .
            "   JOIN PracovniPomer ON  (PracovniPomer.RC = EvidencniStav.RC AND PracovniPomer.OsobniCislo = EvidencniStav.OsobniCislo AND PracovniPomer.DatumNastupu = EvidencniStav.DatumNastupu) " .
            "WHERE PracovniPomer.RC = " . $params->addParameter($rc) .
            "  AND PracovniPomer.DatumNastupu <= " . $params->addParameter($database->dateToParam(DateUtils::today())) .
            "  AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->dateToParam(DateUtils::today())) . " OR PracovniPomer.DatumUkonceni IS NULL) " .

            'ORDER BY PracovniPomer.DatumNastupu, PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $stavy = [];

        while ($row = $cursor->fetch()) {
            $stav = new EvidencniStav();
            $stav->rc = $rc;
            $stav->nactiDataZRadku($database, $row);

            $stavy[] = $stav;
        }

        return $stavy;
    }

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->kodStavu = $row[0];
        $this->popisStavu = $row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->poznamka = $row[4];

        $this->idPP = $row[8];
        $this->osobniCislo = $row[5];
        $this->datumNastupu = $database->stringToTimestamp($row[6]);
        $this->typPP = $row[7];
    }
}