<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace Ron\Controls;

class AppMenu
{
    /** @var AppMenuItem[] */
    var $sections;
    /** @var bool Urcuje, jestli muze byt zaroven zapnuto vice sekci najednou */
    var $enableMultipleSections = true;

    /** @var AppMenuItem[] */
    var $items;

    var $className = 'mainmenu';

    var $logo;

    /** @var bool
     * pomocna promenna pro pridavani polozek (priznak, ze posledni pridana polozka byl prazdny radek
     * pokud je true, tak AddEmptyRow nic nedela
     */
    private $emptyRow;


    function __construct()
    {
        $this ->items = [];
        $this ->sections = [];

        $this ->logo = null;

        $this ->emptyRow = false; // zabrani pridani prazdneho radku na zacatku menu
    }

    /**
     * Pridani polozky do section menu - muze to byt sekce, nebo i primo polozka s menuId
     * @param AppMenuItem $item
     * @return $this
     */
    function addSectionItem($item)
    {
        if ($item ->sectionId) {
            if ($item ->active  &&  !$this ->enableMultipleSections) {
                foreach ($this ->sections as $section) {
                    // vypne ostatni sekce
                    $section ->active = false;
                }
            }
        }

        $index = $item ->sectionId ?: $item ->menuId;
        if ($index) $this ->sections[$index] = $item;
        else $this ->sections[] = $item;

        return $this;
    }


    function addSection($itemText, $sectionId, $active = true, $defaultQueryString = null, $htmlId = '', $glyph = null)
    {
        $newSection = new AppMenuItem($itemText, null, '', $htmlId);
        $newSection ->sectionId = $sectionId;
        $newSection ->active = $active;
        $newSection ->defaultQueryString = $defaultQueryString;
        $newSection ->glyph = $glyph;

        $this ->addSectionItem($newSection);

        return $this;
    }


    function findFirstMenuItemForSection($sectionId)
    {
        foreach ($this ->items as $item) {
            if ($item ->sectionId == $sectionId) return $item;
        }

        return null;
    }

    function setSectionsDefaultActions()
    {
        foreach ($this ->sections as $section) {
            if ($section ->sectionId  &&  !$section ->defaultQueryString) {
                $item = $this ->findFirstMenuItemForSection($section ->sectionId);
                if ($item) $section ->defaultQueryString = '&menuid=' . $item ->menuId;
            }
        }
    }


    function setActiveSection($sectionId)
    {
        $found = false;
        foreach ($this ->sections as $section) {
            if ($section ->sectionId  &&  $section ->sectionId == $sectionId) {
                $section ->active = true;
                $found = true;
            }
        }

        if ($found) {
            foreach ($this ->sections as $section) {
                if ($section ->sectionId  &&  $section ->sectionId != $sectionId) {
                    $section ->active = false;
                }
            }
        }
    }


    /**
     * @param AppMenuItem $item
     */
    function addMenuItem($item)
    {
        if ($this ->emptyRow) {
            $item ->className .= ($item ->className ? ' ' : '') . 'margin-top';
            $this ->emptyRow = false;
        }

        $this ->items[] = $item;
    }


    function addItem($itemtext, $action, $htmlid = '', $sectionId = null)
    {
        $newItem = new AppMenuItem($itemtext, $action, '', $htmlid);
        $newItem ->sectionId = $sectionId;
        $this ->addMenuItem($newItem);
    }


    function addSubItem($itemtext, $action, $htmlid = '')
    {
        $this ->addMenuItem( new AppMenuItem($itemtext, $action, 'submenu', $htmlid) );
    }


    function addEmptyRow($sectionId = null)
    {
//        if (!$this ->emptyRow) {
//            $newItem = new AppMenuItem('', '');
//            $newItem ->sectionId = $sectionId;
//            $this ->items[] = $newItem;
//            $this ->emptyRow = true;
//        }
        if ($this ->items) {
            $this->emptyRow = true;
        }
    }


    function getItemClass($menuItem)
    {
        $className = $menuItem ->className;

        $pomstr = 'menuid=' . $menuItem ->menuId;
        parse_str($pomstr, $url);

        if ($menuItem ->menuId  &&  isset($_GET['menuid'])  &&  isset($url['menuid'])  &&  $url['menuid'] == $_GET['menuid']) {
            $className .= ($className ? ' ' : '') . 'active';
        }

        return $className;
    }


    /**
     * @param $menuItem AppMenuItem
     */
    function renderItem($menuItem)
    {
        // Pokud je nejaky text, tak ho zobrazi, jinak zobrazi jen prazdny radek
        if ($menuItem ->text) {
            $className = $this ->getItemClass($menuItem);

            $pomstr = $className ? "class='$className' " : '';

            if ($menuItem ->htmlId) $pomstr .= "id='{$menuItem ->htmlId}'";

            echo "<li $pomstr>";
            if ($menuItem ->menuId) {
                $sidStr = SID ? '&' . SID : '';
                echo
                    '<a href="?menuid=' . $menuItem ->menuId . $sidStr . '" ' . ($menuItem ->htmlParams ?: '') . '>' .
                    ($menuItem ->glyph ? "<i class='{$menuItem ->glyph}'></i>" : '') .
                    $menuItem ->text .
                    '</a>';
            } else {
                echo $menuItem ->text;
            }
            echo "</li>\n";

            $this ->emptyRow = false;
        } else {
            if (!$this ->emptyRow) {
                echo '<li class="listseparator"></li>';
                $this ->emptyRow = true;
            }
        }
    }


    /**
     * Vraci true, pokud existuje nejaka sekce, respektive sekce se zadanym $id
     * @param string $id
     * @return bool
     */
    function existsSection($id = null) {
        foreach ($this ->sections as $section) {
            if (isset($id)) {
                if ($id == $section ->sectionId) return true;
            } else {
                if ($section->sectionId) return true;
            }
        }

        return false;
    }


    function renderItems($section, $className = null, $htmlId = null)
    {
        // zjisti, jestli existuje nejaka sekce - pokud ne, tak bude zobrazovat vsechny polozky bez ohledu na sekci
        $existSection = $this ->existsSection();
        if (!isset($className)) {
            $className = $this ->className;
        }

        $idStr = $htmlId ? "id='$htmlId'" : '';
        echo "<ul $idStr class='$className'>";

        $this ->emptyRow = true;

        foreach ($this ->items as $menuItem) {
            if (!$existSection) {
                $renderItem = true;
            } else if (isset($section)) {
                // zobrazuje se jen vybrana sekce - pouziva se pri rendrovani plovouciho menu
                $renderItem = $menuItem ->sectionId == $section;
            } else {
                // zobrazuji se vsechny zapnute moduly
                $renderItem = !$menuItem ->sectionId  ||  (isset($this ->sections[$menuItem ->sectionId])  &&  $this ->sections[$menuItem ->sectionId] ->active);
            }

            if ($renderItem) {
                $this->renderItem($menuItem);
            }
        }
        echo "</ul>\n";
    }


    function render()
    {
        $this ->renderItems(null);
    }


    /**
     * Renderuje menu pro sekce
     * @param bool $defaultQueryString Pokud je true, renderuje jako prvni ikonu "menu"
     * @param bool $renderDefaultSection
     * @param string $htmlId
     * @param bool $renderItems
     */
    function renderSections($defaultQueryString, $renderDefaultSection = false, $htmlId = null, $renderItems = false)
    {
        $idStr = $htmlId ? "id='$htmlId'" : '';
        echo "<div $idStr class='appselect'>";

        $cnt = 0;
        foreach ($this ->sections as $sectionItem) {
            if ($sectionItem ->sectionId) $cnt ++;
        }

        if ($defaultQueryString  &&  $defaultQueryString[0] != '&')  $defaultQueryString = '&' . $defaultQueryString;

        if ($renderDefaultSection) {
            echo "<div class='appselect-item' data-ron-itemid='default'><div class='defaultmenuitem fal fa-bars appselect-section'></div>";
            if ($renderItems) {
                $this->renderItems('', 'floatingmenulist popupmenulist', 'floatingmenulist_default');
            }
            echo "</div>\n";
        }

        foreach ($this ->sections as $sectionItem) {
            $htmlParams = $sectionItem ->htmlParams ?: '';
            $htmlId = $sectionItem -> htmlId ?: $sectionItem ->sectionId;

            if ($sectionItem ->sectionId) {
                // sekce
                $classStr = 'appselect-item' . (($sectionItem->active || $cnt == 1) ? ' active' : ''); // sekce je aktivni nebo je jen jedna

                if ($this ->enableMultipleSections) {
                    // zapinani/vypinani jednotlivych sekci
                    $queryString = $defaultQueryString;

                    $swichParam = $sectionItem ->active ? 'hidesection' : 'showsection';
                    //$value = $sectionItem ->active ? '0' : '1';
                    echo
                        "<div class='$classStr' id='$htmlId' data-ron-itemid='{$sectionItem ->sectionId}'>" .
                        "<div class='appselect-section'><a href='?session@{$swichParam}={$sectionItem ->sectionId}{$queryString}' $htmlParams>" .
                        ($sectionItem ->glyph ? "<i class='menu-glyph {$sectionItem ->glyph}'></i>" : '') .
                        $sectionItem ->text .
                        "</a></div>";
                    if ($renderItems) {
                        $this ->renderItems($sectionItem ->sectionId, 'floatingmenulist popupmenulist', 'floatingmenulist_' . $sectionItem ->sectionId);
                    }
                    echo
                    "</div>\n";
                } else {
                    // prepinani sekci - jen jedna aktivni
                    $queryString = $sectionItem ->defaultQueryString ?: $defaultQueryString;

                    echo
                        "<div class='$classStr' id='$htmlId' data-ron-itemid='{$sectionItem ->sectionId}'>" .
                        "<a href='?session@menusection={$sectionItem ->sectionId}$queryString' $htmlParams>" .
                        ($sectionItem ->glyph ? "<i class='menu-glyph {$sectionItem ->glyph}'></i>" : '') .
                        $sectionItem ->text .
                        "</a>";
                    if ($renderItems) $this ->renderItems($sectionItem ->sectionId, 'floatingmenulist popupmenulist', 'floatingmenulist_' . $sectionItem ->sectionId);
                    echo "</div>\n";
                }
            } else {
                // klasicka polozka menu
                $className = 'appselect-item' . ($sectionItem ->className ?: ' active');
                echo
                    "<div class='$className' id='$htmlId'>" .
                    "<a href='?menuid={$sectionItem ->menuId}' $htmlParams>" .
                    ($sectionItem ->glyph ? "<i class='menu-glyph {$sectionItem ->glyph}'></i>" : '') .
                    "{$sectionItem ->text}</a>" .
                    "</div>\n";
            }
        }

        echo '</div>';
    }


    function existItemWithoutSection()
    {
        foreach ($this ->items as $menuItem) {
            if (!$menuItem ->sectionId) return true;
        }

        return false;
    }


    function renderFloatingMenu($defaultQueryString)
    {
        $renderDefaultSection = $this ->existItemWithoutSection();
        $this ->renderSections($defaultQueryString, $renderDefaultSection, /* id = */ 'floatingmenulist', true);
    }
}
