<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;


class ComboBox extends Options implements ViewObject
{
    public $titleFieldName;    // jmeno datoveho pole objektu, ktere obsahuje title. V load se nastavuje na 'title'
    public $cssClassFieldName; // jmeno datoveho pole objektu, ktere obsahuje class. V load se nastavuje na 'cssClass'

    /** @var Option $selectedOption */
    var $selectedOption;

    /** @var int   Indexy jednotlivych poli v SQL dotazu */
    var $keyIndex;
    var $captionIndex;
    var $titleIndex;
    var $cssClassIndex;

    /** @var array  Asociativni pole indexu dalsich atributu, ktere se pridaji k polozkam (nazev atributu => index) */
    var $attrIndexes;

    /** @var bool $alwaysShowSelected    pokud je true, zobrazi se zadana hodnota, i kdyz neni nactena z DB */
    var $alwaysShowSelected;

    /** @var string $varName    Jmeno promenne v HTML */
    var $varName;
    var $className;

    /** @var bool $noHtmlConvert    priznak, ze se nema provest htmlTextConvert s popisem (defaultne = false) */
    var $noHtmlConvert;

    /** @var string $emptyRow   hodnota, ktera se ma zobrazit v prvnim (prazdnem) radku - pokud je null, nezobrazuje se */
    var $emptyRow;

    /** @var bool $showCode     Priznak, ze se ma zobrazovat i kod (v zavorce) */
    var $showCode;

    /** @var bool $dataTypeComparison      Priznak, ze se maji hodnoty porovnavat vcetne datoveho typu, default = true */
    var $dataTypeComparison;

    /** @var array|string       Asociativni pole HTML parametru, pripadne muze byt string */
    var $htmlParams;

    var $readOnly;

    /** @var mixed[]            Pole "preferovanych" hodnot. Pri nacitani z databaze se zobrazi na zacatku */
    var $preferredValues = [];


    /**
     * @param string $varName
     * @param Option[]|object[]|string[] [$data]
     * @param string $keyFieldName
     * @param string $valueFieldName
     */
    function __construct($varName, $data = null, $keyFieldName = 'key', $valueFieldName = 'value')
    {
        parent::__construct($data, $keyFieldName, $valueFieldName);

        $this->varName = $varName;

        $this ->keyIndex = 0;
        $this ->captionIndex = 1;

        $this ->dataTypeComparison = true;

        $this ->htmlParams = ['class' => 'form-input'];
        $this ->readOnly = false;
    }


    function setSelectedOption($selectedValue)
    {
        $selectedOption = isset($selectedValue) ? $this->findOption($selectedValue, $this ->dataTypeComparison) : null;

        if (!$selectedOption  &&  isset($selectedValue)) {
            if ($this ->alwaysShowSelected) {
                $selectedOption = new Option($selectedValue, $selectedValue);
                //$this ->data[] = $selectedOption;
            } else if (!isset($this ->emptyRow)  &&  sizeof($this ->data)) { // pokud je empty row, tak by nemela byt vybrana zadna jina
                $selectedOption = $this ->data[0];
            }
        }

        $this ->selectedOption = $selectedOption;
    }


    /**
     * @param Database      $database
     * @param string|SqlStatement $sql
     * @param mixed         $selectedValue
     * @param Parameters    $sqlParams
     * @throws DBException
     */
    function loadData($database, $sql, $selectedValue, $sqlParams = null)
    {
        if ($this instanceof ICreateItem) {
            /** @noinspection PhpUndefinedMethodInspection */
            /** @noinspection PhpUndefinedFieldInspection */
            $this ->loadSQLCustomCreate($database, $sql, $sqlParams, $this, $this ->emptyRow);
        } else {
            $this->titleFieldName = 'title';
            $this->cssClassFieldName = 'cssClass';

            $this->loadSQL($database, $sql, $sqlParams, $this->keyIndex, $this->captionIndex, $this->titleIndex, $this->cssClassIndex, $this ->attrIndexes, $this ->preferredValues);
            $this->setSelectedOption($selectedValue);
        }
    }


    /**
     * Vrati nactena data pro JSON
     */
    function getJsonData()
    {
        $result = [];
        foreach ($this ->data as $option) {
            $result[] = $option ->getJson();
        }

        return $result;
    }


    /**
     * @param string[] $params  Asociativni pole HTML atributu
     */
    function renderHtmlParams($params)
    {
//        if ($this->varName) $params['name'] = $this ->varName;
//        if ($this->className) $params['class'] = $this->className;

        foreach ($params as $name => $value) {
            switch ($name) {
                case 'name' :
                    $value = $this ->varName ?: $value;
                    break;

                case 'class' :
                    $value = $this ->className ?: $value;
                    break;
            }

            echo " $name='$value'";
        }

        if (!isset($params['name'])  &&  $this ->varName) {
            echo " name='{$this ->varName}'";
        }

        if (!isset($params['class'])  &&  $this ->className) {
            echo " class='{$this ->className}'";
        }
    }


    function render()
    {
        if ($this ->readOnly) {
            if ($this ->selectedOption) {
                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '<span title="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->titleFieldName}) . '">';
                }

                echo $this ->noHtmlConvert
                    ? $this->selectedOption->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($this->selectedOption->{$this ->valueFieldName});

                if ($this ->titleFieldName  &&  $this ->selectedOption ->{$this ->titleFieldName}) {
                    echo '</span>';
                }
            }
        } else {
            echo '<select';
            if (is_array($this->htmlParams)) {
                $this ->renderHtmlParams($this ->htmlParams);
//                if ($this->varName) $this ->htmlParams['name'] = $this ->varName;
//                if ($this->className) $this ->htmlParams['class'] = $this->className;
//
//                foreach ($this ->htmlParams as $name => $value) {
//                    echo " $name='$value'";
//                }
            } else {
                if ($this->varName) echo ' name="' . $this->varName . '"';
                if ($this->className) echo ' class="' . $this->className . '"';
                if ($this->htmlParams) echo ' ' . $this->htmlParams;
            }
            echo '>';

            $first = true;

            // empty row
            if (isset($this ->emptyRow)) {
                $emptyRowText = $this ->emptyRow === true
                    ? '&nbsp;'
                    :  $this ->emptyRow;

                echo "<option value=''>$emptyRowText</option>";

                $first = false;
            }

            // zapise jednotlive polozky
            $selectedOptionsRendered = false;

            foreach ($this->data as $i => $option) {
                echo
                    '<option ' .
                    ' value="' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . '"';

                if (($first && !isset($this->selectedOption)) || ($this->selectedOption === $option)) echo ' selected="selected"';

                $cssClass = ($this ->cssClassFieldName  &&  $option->{$this ->cssClassFieldName})
                    ? $option->{$this ->cssClassFieldName}
                    : '';

                if ($option ->cssClass) {
                    $cssClass .= ($cssClass ? ' ' : '') . $option ->cssClass;
                }
                if ($cssClass) echo ' class="' . StringUtils::htmlTextConvert($cssClass) . '"';
                if ($this ->titleFieldName  &&  $option->{$this ->titleFieldName}) echo ' title="' . StringUtils::htmlTextConvert($option->{$this ->titleFieldName}) . '"';

                if ($option ->params) {
                    foreach ($option ->params as $paramName => $paramValue) {
                        echo ' ' . $paramName . '="' . StringUtils::htmlTextConvert($paramValue) . '"';
                    }
                }


                echo '>';

                echo $this->noHtmlConvert
                    ? $option->{$this ->valueFieldName}
                    : StringUtils::htmlTextConvert($option->{$this ->valueFieldName});

                if ($this->showCode && $option->{$this ->keyFieldName} !== $option->{$this ->valueFieldName}  &&  $option ->{$this ->keyFieldName} !== '') {
                    echo ' (' . StringUtils::htmlTextConvert($option->{$this ->keyFieldName}) . ')';
                }

                echo '</option>';

                if ($this->selectedOption === $option) $selectedOptionsRendered = true;
                $first = false;
            }

            if ($this ->selectedOption  &&  !$selectedOptionsRendered  &&  $this ->alwaysShowSelected) {
                echo '<option value="' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->keyFieldName}) . '" selected="selected">' . StringUtils::htmlTextConvert($this ->selectedOption ->{$this ->valueFieldName}) . '</option>';
            }

            echo "</select>\n";
        }
    }


    static function renderComboFromArray($data, $varName, $selectedValue, $htmlParams = [], $dataTypeComparison = true)
    {
        if (!$data) return;

        $combo = new ComboBox($varName, $data);
        $combo ->dataTypeComparison = $dataTypeComparison;
        $combo ->htmlParams = $htmlParams;

        if (isset($selectedValue)) {
            $combo ->selectedOption = $combo ->findOption($selectedValue, $dataTypeComparison);
        }

        $combo ->render();
    }


    /**
     * @param string $varName
     * @param Option[]|object[] $options    Seznam polozek pro zobrazeni
     * @param mixed    [$selectedValue]     Vybrana hodnota
     * @param string[] [$comboBoxProps]     asociativni pole vlastnosti comboboxu
     * @param string [$keyFieldName]
     * @param string [$valueFieldName]
     */
    public static function renderComboBox($varName, $options, $selectedValue = null, $comboBoxProps = null)
    {
        $comboBox = new static($varName, $options);
        if ($comboBoxProps) {
            foreach ($comboBoxProps as $propName => $propValue) {
                $comboBox ->{$propName} = $propValue;
            }
        }

        if (isset($selectedValue)) {
            $comboBox ->setSelectedOption($selectedValue);
        }

        $comboBox ->render();
    }
}
