<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

class Option
{
    var $key;
    var $value;

    var $title;
    var $cssClass;

    /**
     * Asociativni pole dalsich atributu (napr. data)
     * @var array
     */
    var $params = [];

    function __construct($key, $value)
    {
        $this ->key = $key;
        $this ->value = $value;
    }


    function getJson()
    {
        $result = [
            'key' => StringUtils::jsonText($this ->key),
            'value' => StringUtils::jsonText($this ->value)
        ];

        if ($this ->cssClass) {
            $result['cssClass'] = $this ->cssClass;
        }

        if ($this ->title) {
            $result['title'] = StringUtils::jsonText($this ->title);
        }

        foreach ($this ->params as $paramName => $paramValue) {
            $result[$paramName] = StringUtils::jsonText($paramValue);
        }

        return $result;
    }


    public static function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
    {
        $option = new Option($row[$keyIndex], $row[$captionIndex]);

        if (isset($titleIndex)) $option ->title = $row[$titleIndex];
        if (isset($cssClassIndex)) $option ->cssClass = $row[$cssClassIndex];

        return $option;
    }


    /**
     * Nacte polozky ze zadaneho dotazu.
     * @param Database $database
     * @param string|SqlStatement $sql
     * @param Parameters $sqlParams
     * @param int $keyIndex Index pole s klicem v sql dotazu
     * @param int $captionIndex Index pole s hodnotou v sql dotazu
     * @param null $emptyValue
     * @param int $titleIndex Index pole s html titulkem
     * @param int $cssClassIndex Index pole s css tridou
     * @param array $attrIndexes Asociativni pole indexu poli, ktere se maji zapsat do params (jmeno parametru => index)
     * @param mixed[] $preferredValues  Pole "preferovanych" hodnot - nactou se na zacatek (mimo poradi z sql)
     * @return Option[]
     * @throws DBException
     */
    public static function loadSQL($database, $sql, $sqlParams, $keyIndex = 0, $captionIndex = 1, $emptyValue = null, $titleIndex = null, $cssClassIndex = null, $attrIndexes = null, $preferredValues = null)
    {
        $result = [];

        if ($preferredValues) {
            $preferredOptions = array_fill_keys($preferredValues, false);
        } else {
            $preferredOptions = null;
        }

        if (isset($emptyValue)) {
            // prida polozku pro "emptyValue"
            $emptyOption = new Option('', $emptyValue);
            if (!$preferredValues) {
                // neexistuji preddefinovane hodnoty - prida ji rovnou do vysledku
                $result[] = $emptyOption;
            } else {
                // doplni prazdnou hodnotu na zacatek preddefinovanych hodnot
                array_splice($preferredOptions, 0, 0, [$emptyOption]);
                //$preferredOptions[] = $emptyOption;
            }
        }

        $cursor = $database ->getSqlResult($sql, $sqlParams);

        while ($row = $cursor ->fetch()) {
            $newOption = static::createItem($row, $keyIndex, $captionIndex, $titleIndex, $cssClassIndex);
            if ($attrIndexes) {
                foreach ($attrIndexes as $dataParamName => $rowIndex) {
                    $newOption ->params[$dataParamName] = $row[$rowIndex];
                }

            }

            if ($preferredValues  &&  in_array($newOption ->key, $preferredValues)) {
                $preferredOptions[$newOption ->key] = $newOption;
            } else {
                $result[] = $newOption;
            }
        }

        $cursor ->close();
        if ($preferredOptions) {
            // spoji pole $preferredValues a $result
            return array_merge(
                array_values(array_filter($preferredOptions)), // odstrani nenalezene polozky a vyrobi standardni indexovane pole
                $result);
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param string|SqlStatement   $sql
     * @param Parameters    $sqlParams
     * @param ICreateItem   $createItemInterface
     * @return array
     */
    public static function loadSQLCustomCreate($database, $sql, $sqlParams, $createItemInterface) // , $emptyValue = false
    {
        $result = [];

//        if ($emptyValue) {
//            $result[] = $createItemInterface ->createItem(null);
//        }

        $cursor = $database ->getSqlResult($sql, $sqlParams);

        while ($row = $cursor ->fetch()) {
            $result[] = $createItemInterface ->createItem($row);
        }

        $cursor ->close();

        return $result;
    }
}
