<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

class ArrayCursor extends Cursor
{
    protected $data;
    protected $position = 0;

    /**
     * ArrayCursor constructor.
     * @param Cursor|array $data
     */
    public function __construct($data)
    {
        parent::__construct(null);

        if ($data instanceof Cursor) {
            $this->loadData($data);
        } else {
            $this ->data = $data;
        }
    }


    public function loadData(Cursor $cursor)
    {
        $this ->position = 0;
        $this ->data = [];

        if (!$cursor) {
            $this ->data = null;
            return;
        }

        $this ->database = $cursor ->database;

        while ($row = $cursor ->fetch()) {
            $this ->data[] = $row;
        }
    }


    public function fetch()
    {
        if ($this ->data  &&  $this ->position < count($this ->data)) {
            $result = $this ->data[$this ->position];
            $this ->position ++;

            return $result;
        }

        return null;
    }


    public function close()
    {
        $this ->data = null;
        $this ->position = 0;
    }


    public function numRows()
    {
        return $this ->data ? count($this ->data) : 0;
    }


    public function seek($rowNumber)
    {
        $this ->position = $rowNumber;
    }


    public function last()
    {
        $this ->position = $this ->data ? count($this ->data) - 1 : 0;
    }
}
