<?php
/**
 * stanislav.elbl, 25.02.2021
 */

namespace Ron\DB;

class OracleBinaryParameter
{
    public $data;
    public $parameterName;

    /**
     * OracleBinaryParameter constructor.
     * @param $data
     */
    public function __construct($data)
    {
        $this->data = $data;
    }
}


class OracleParameters extends NamedParameters
{
    /** @var OracleBinaryParameter[] */
    public $lobs = [];

    public function addParameter($value)
    {
        if ($value instanceof OracleBinaryParameter) {
//            $name = 'temp_lob_' . count($this ->lobs);
//            $value ->parameterName = $name;
//            $this ->lobs[] = $value;
//            return $name;

            // funguje pro text:
            $dataString = unpack("H*hex", $value ->data);
            $paramName = parent::addParameter($dataString['hex']);
            return "hextoraw(" . $paramName . ")";      // toto funguje jen pro textove data a asi jen do 4000 bytu


            // dalsi pokusy
            //return "hextoraw('" . $dataString['hex'] . "')";

            //$dataString = base64_encode($value ->data);
//            $paramName = parent::addParameter($dataString);
//            return 'utl_encode.base64_decode(' . $paramName . ')';

            //return "utl_encode.base64_decode('" . $dataString . "')";

            //return 'hextoraw(' . $paramName . ')';

            //$paramName = parent::addParameter($value ->data);

            //return 'utl_raw.cast_to_varchar2(utl_encode.base64_encode(utl_raw.cast_to_raw(' . $paramName . ')))';
            //return 'utl_encode.uuencode(utl_encode.base64_encode(' . $paramName . '))';
        }

        $paramName = parent::addParameter($value);
        return $paramName;
    }
}


class OracleDatabase extends Database
{

    /**
     * @var resource
     */
    private $connection;

    private $connectionString;

    private $lasterror_cmd_id;


    public function __construct($connectionString)
    {
        $this->connectionString = $connectionString;
        $this ->functions = [
            'GETDATE' => '#current_date',
            'LEN' => 'length'
        ];
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return string
     */
    public function getDbType()
    {
        return 'oracle';
    }

    /**
     * Musi byt spravne nastavena promenna prostreni ORACLE_HOME - a to uz pri spusteni IIS ! ! !
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        $this ->connection = oci_connect($user, $password, $this ->connectionString);

        if (!$this ->connection) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }

        $this ->executeOci("ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'");
        // nastavit aby bylo porovnavani case insensitive - kvuli prihlasovani
        $this ->executeOci('ALTER SESSION SET NLS_COMP = ANSI');
        $this ->executeOci('ALTER SESSION SET NLS_SORT = BINARY_CI');
        $this ->executeOci("ALTER SESSION SET NLS_NUMERIC_CHARACTERS = '. '");
    }

    /**
     * @return bool
     */
    public function close()
    {
        return oci_close($this ->connection);
    }


    public function createParameters()
    {
        return new OracleParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return resource
     * @throws DBException
     */
    private function executeOci($sqlText, $params = null)
    {
        $stmt = oci_parse($this ->connection, $sqlText);

        $this ->lasterror_cmd_id = $stmt;

        if ($params) {
            $data = $params ->getSqlParameters();
            foreach ($data as $name => $value) {
                $type = SQLT_CHR;
//                if (is_float($value)) {
//                    $type = SQLT_FLT;
//                }
                if (!oci_bind_by_name($stmt, ":$name", $data[$name], -1, $type)) { // POZOR, nelze pouzit $value! Predava se odkaz na promennou!
                    throw new DBException($this, $sqlText, 'Chyba pri volani oci_bind_by_name!');
                }
            }
        }

        if (!oci_execute($stmt)) {
            throw new DBException($this, $sqlText);
        }

        return $stmt;
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     * @return Cursor
     */
    public function executeSelect($sqlText, $params = null)
    {
        $stmt = $this ->executeOci($sqlText, $params);
        return new OracleCursor($this, $stmt);
    }

    /**
     * @param string $sqlText
     * @param OracleParameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        $this ->executeOci($sqlText, $params);
//        if (!$params  ||  !$params ->lobs  ||  !count($params ->lobs)) {
//            return !!$this ->select($sqlText, $params);
//        }
//
//        $sql =
//            "DECLARE\n";
//
//        foreach ($params ->lobs as $lobParameter) {
//            $sql .= $lobParameter ->parameterName . " blob;\n";
//        }
//
//        $sql .= "BEGIN\n" .
//            $sqlText . "\n" .
//            'RETURNING ';
//
//            $first = true;
//            foreach ($params ->lobs as $lobParameter) {
//                $sql .= ($first ? '' : ', ') . $lobParameter ->parameterName;
//                $first = false;
//            }
//
//        $sql .= ' INTO ';
//            $first = true;
//            foreach ($params ->lobs as $lobParameter) {
//                $sql .= ($first ? '' : ', ') . ':_' . $lobParameter ->parameterName;
//                $first = false;
//            }
//
//            $sql .= ";\n";
//        $sql .= "END;\n";
//
//        $stmt = oci_parse($this ->connection, $sql);
//
//        $this ->lasterror_cmd_id = $stmt;
//
//        echo "sql: $sqlText <br><br>";
//        var_dump($sql);
//        var_dump($params);
//
//        if ($params) {
//            $data = $params ->getSqlParameters();
//            foreach ($data as $name => $value) {
//                if (!oci_bind_by_name($stmt, ":$name", $data[$name])) return false; // POZOR, nelze pouzit $value! Predava se odkaz na promennou!
//            }
//        }
//
//        if (!oci_execute($stmt)) return false;
//
//        return true;
    }


    /**
     * @return string
     */
    public function lastErrorText()
    {
        if ($this ->lasterror_cmd_id) $erra = oci_error($this ->lasterror_cmd_id);
        else $erra = oci_error();

        if ( !$erra ) return '';

        if ( is_array($erra) ) return $erra['message'];

        return '';
    }


    /**
     * @return bool
     * @throws \Exception
     */
    public function beginTransaction()
    {
        throw new \Exception('Oracle - funkce beginTransaction neni implementovana!');
    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function commitTransaction()
    {
        throw new \Exception('Oracle - funkce commitTransaction neni implementovana!');
    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function rollbackTransaction()
    {
        throw new \Exception('Oracle - funkce rollbackTransaction neni implementovana!');
    }


    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return "EXTRACT(YEAR FROM $text)";
    }

    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if ( !isset($str) ) return NULL;

        $basetime = 0;
        $str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval

        $result = strtotime($str);
        if ($result != -1)
        {
            return $result - $basetime;
        }

        // nejaka chyba
        return -1;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if ( !isset($timestamp) ) return 'null';

        return "TO_DATE('" . date('Y-m-d H:i:s', $timestamp) . "', 'YYYY-MM-DD HH24:MI:SS')";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if ( !isset($timestamp) ) return 'null';

        return "TO_DATE('" . date('Y-m-d', $timestamp) . "', 'YYYY-MM-DD')";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ( $timestamp >= 0  &&  $timestamp < 86400)
        {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
//            $h = str_pad( (int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
//            $timestamp = $timestamp - 3600 * $h;
//            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
//            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            $h = (int)($timestamp / 3600);
            $timestamp = $timestamp - 3600 * $h;
            $m = (int)($timestamp / 60);
            $s = (int)$timestamp - 60 * $m;
            return "TO_DATE('1899-12-30 " . str_pad($h, 2, '0', STR_PAD_LEFT) . ':' . str_pad($m, 2, '0', STR_PAD_LEFT) . ':' . str_pad($s, 2, '0', STR_PAD_LEFT) . "', 'YYYY-MM-DD HH24:MI:SS')";
        }

        return "TO_DATE('1899-12-30 ".date('H:i:s', $timestamp) . "', 'YYYY-MM-DD HH24:MI:SS')";
    }


    public function timestampToParam($timestamp)
    {
        if ( !isset($timestamp) ) return null;
        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if ( !isset($timestamp) ) return null;
        return date('Y-m-d 00:00:00', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        if ( !isset($timestamp) ) return null;

        if ( $timestamp >= 0  &&  $timestamp < 86400)
        {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = (int)($timestamp / 3600);
            $timestamp = $timestamp - 3600 * $h;
            $m = (int)($timestamp / 60);
            $s = (int)$timestamp - 60 * $m;
            return '1899-12-30 ' . str_pad($h, 2, '0', STR_PAD_LEFT) . ':' . str_pad($m, 2, '0', STR_PAD_LEFT) . ':' . str_pad($s, 2, '0', STR_PAD_LEFT);
        }

        return '1899-12-30 ' . date('H:i:s', $timestamp);
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode (':', $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0 ;
    }



    function getBlobParam($data)
    {
        return new OracleBinaryParameter($data);
    }

}




class OracleCursor extends Cursor
{
    /**
     * @var resource
     */
    protected $data;


    /**
     * @param OracleDatabase $database
     * @param resource $data
     */
    function __construct($database, $data)
    {
        parent::__construct($database);
        $this->data = $data;
    }


    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        $result = oci_fetch_array($this ->data, OCI_NUM + OCI_RETURN_NULLS + OCI_RETURN_LOBS);

        return $result;
    }
}
