<?php
/**
 * UTF8
 * stanislav.elbl, 08.01.2018
 */

namespace Ron;


use Ron\MVC\ViewObject;

class DialogElement
{
    /** @var string     HTML tag */
    public $tag;

    /** @var string     Id elementu v HTML */
    public $id;
    /** @var string */
    public $class;

    /** @var string[]   Pole dalsich html parametru  */
    public $htmlParams = [];


    /**
     * DialogElement constructor.
     */
    public function __construct()
    {
    }


    /**
     * @return string
     */
    public function getTag()
    {
        return $this->tag;
    }

    /**
     * @param string $tag
     * @return $this
     */
    public function setTag($tag)
    {
        $this->tag = $tag;
        return $this;
    }


    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $id
     * @return $this
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }


    /**
     * @return string
     */
    public function getClass()
    {
        return $this->class;
    }

    /**
     * @param string $class
     * @return $this
     */
    public function setClass($class)
    {
        $this->class = $class;
        return $this;
    }


    public function setHtmlParam($name, $value)
    {
        $this ->htmlParams[$name] = $value;
    }


    /**
     * Vraci zakladni HTML parametry (podle vlastnosti id, class a pripadne dalsich)
     * @return string
     */
    protected function getBasicRenderParams()
    {
        $renderParams = [];
        if ($this ->class) $renderParams['class'] = $this ->class;
        if ($this ->id) $renderParams['id'] = $this ->id;

        return $renderParams;
    }

    public function getRenderParams()
    {
        $renderParams = $this ->getBasicRenderParams();

        foreach ($this ->htmlParams as $paramName => $paramValue) {
            $renderParams[$paramName] = $paramValue;
        }

        return $renderParams;
    }


    /**
     * Vraci retezec parametru pro vygenerovani do HTML
     * @param $renderParams
     * @return string
     */
    public function getParamsHtml($renderParams)
    {
        $result = '';
        foreach ($renderParams as $name => $value) {
            $result .= ' ' . $name . '="' . StringUtils::htmlTextConvert($value) . '"';
        }

        return $result;
    }
}


class DialogButton extends DialogElement
{
    /** @var string     Nazev elementu (parametr name) v HTML */
    public $name;
    /** @var string     Parametr value v HTML (text v buttonu) */
    public $value;
    /** @var string     typ inputu v HTML - obvykle submit nebo button */
    public $type = 'submit';


    /**
     * DialogButton constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->tag = 'input';
        $this->class = 'submitbutton';
    }

    /**
     * Vraci button "Storno"
     * @param string $id
     * @param string $value
     * @return $this
     */
    public static function cancelButton($id = 'btn_cancel', $value = null)
    {
        return
            (new DialogButton())
                ->setId($id)
                ->setType('button')
                ->setClass('button cancel')
                ->setValue($value ?: \Ron\ronText('Cancel'));
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return $this
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param string $value
     * @return $this
     */
    public function setValue($value)
    {
        $this->value = $value;
        return $this;
    }


    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     * @return $this
     */
    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }

    protected function getBasicRenderParams()
    {
        $renderParams = parent::getBasicRenderParams();
        if ($this ->name) $renderParams['name'] = $this ->name;
        if ($this ->type) $renderParams['type'] = $this ->type;
        if (isset($this ->value)) $renderParams['value'] = $this ->value;

        return $renderParams;
    }


    public function getHtml()
    {
        $renderParams = $this ->getRenderParams();

        return "<{$this ->tag}" . $this ->getParamsHtml($renderParams) . "/>\n";
    }
}


class Dialog extends DialogElement
{
    /** @var string */
    public $title;
    /** @var string|ViewObject  View pro vygenerovani obsahu formulare */
    public $view;
    /** @var string     Text hlasky o nacitani dat - pokud je nastaven, vytvori se */
    public $waitMessage;

    /** @var DialogButton[] */
    public $buttons = [];



    /**
     * Dialog constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this ->tag = 'div';
        $this ->class = 'rondialog';
    }

    /**
     * @return mixed
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param mixed $title
     * @return $this
     */
    public function setTitle($title)
    {
        $this->title = $title;
        return $this;
    }

    /**
     * @return string
     */
    public function getView()
    {
        return $this->view;
    }

    /**
     * @param string|ViewObject $view
     * @return $this
     */
    public function setView($view)
    {
        $this->view = $view;
        return $this;
    }

    /**
     * @return string
     */
    public function getWaitMessage()
    {
        return $this->waitMessage;
    }

    /**
     * Nastavi text hlasky informujici napriklad o nacitani dat
     * @param string|bool $waitMessage
     * @return $this
     */
    public function setWaitMessage($waitMessage)
    {
        if ($waitMessage === true) {
            $this ->waitMessage = \Ron\ronText('Probiha_nacitani');
        } else {
            $this ->waitMessage = $waitMessage;
        }

        return $this;
    }


    /**
     * @param DialogButton $button
     * @return $this
     */
    public function addButton(DialogButton $button)
    {
        $this ->buttons[] = $button;
        return $this;
    }


    protected function getBasicRenderParams()
    {
        $renderParams = parent::getBasicRenderParams();
        if ($this ->title) $renderParams['title'] = $this ->title;

        return $renderParams;
    }


    public function renderContent()
    {
        if (!$this ->view) return;

        if ($this ->view instanceof ViewObject) {
            $this ->view ->render();
        } else {
            require($this ->view);
        }
    }


    public function renderButtons()
    {
        if (!count($this ->buttons)) return;

        echo '<div class="buttons">';
        foreach ($this ->buttons as $button) {
            echo $button ->getHtml();
        }
        echo '</div>';
    }

    public function renderLoadingMessage()
    {
        if (!isset($this ->waitMessage)) return;
        echo '<div class="dialogwaitmessage" style="display: none">' . $this ->waitMessage . '</div>';
    }


    public function renderHead()
    {
        $renderParams = $this ->getRenderParams();

        echo "<{$this ->tag}" . $this ->getParamsHtml($renderParams) . ">\n";

        echo '<div class="content">';
        $this ->renderLoadingMessage();
    }

    public function renderTail()
    {
        echo '</div>';
        $this ->renderButtons();
        echo "</{$this ->tag}>\n";
    }

    public function render() {
        $this ->renderHead();
        $this ->renderContent();
        $this ->renderTail();
    }
}

