<?php
/**
 * UTF8
 * stanislav.elbl, 18.09.2015
 */

namespace Ron\Http;

class HtmlHeaders
{
    /** @var string[] */
    var $headers;

    /** @var string   Pripojuje se za jmeno souboru (css, script) pro zabraneni cachovani */
    var $versionId;

    private static $htmlHeaders;


    public function __construct($versionId = null)
    {
        $this ->headers = [];
        $this ->versionId = $versionId;
    }


    /**
     * Vraci globalni instanci objektu HtmlHeaders
     * @return HtmlHeaders
     */
    public static function getInstance()
    {
        if (!self::$htmlHeaders) {
            self::$htmlHeaders = new HtmlHeaders();
        };

        return self::$htmlHeaders;
    }


    /**
     * Prida radek do HTML head.
     * @param string $header
     * @param string $id        Pokud je zadan $id a uz existuje radek se stejnym $id, tak jej nahradi
     * @return HtmlHeaders      Vraci $this pro moznost zretezeni
     */
    public function addHeader($header, $id = null)
    {
        if ($id) {
            $this->headers[$id] = $header;
        } else {
            $this->headers[] = $header;
        }

        return $this;
    }


    public function addCss($filename, $params = '')
    {
        if (!$params  &&  $this ->versionId) {
            $params = '?ver=' . $this ->versionId;
        }


        $this ->addHeader(
            '<link rel="stylesheet" type="text/css" href="' .
            $filename . $params .
            '">'
        );

        return $this;
    }


    /**
     * @param string $filename
     * @param string $params
     * @param string $id    viz addHeader
     * @return HtmlHeaders  vraci $this pro moznost zretezeni
     */
    public function addScript($filename, $params = '', $id = null)
    {
        if (!$params  &&  $this ->versionId) {
            $params = '?ver=' . $this ->versionId;
        }

        $this ->addHeader(
            '<script type="text/javascript" src="' .
            $filename . $params .
            '"></script>',
            $id
        );

        return $this;
    }


    public function setTitle($title)
    {
        $this ->addHeader("<title>$title</title>", 'title');
        return $this;
    }


    /**
     * Nastavi ikonu zobrazovanou v prohlizeci
     * @param $iconFile
     * @return $this
     */
    public function setIcon($iconFile)
    {
        $this ->addHeader("<link rel='shortcut icon' href='$iconFile'>", 'shortcut_icon');
        return $this;
    }


    public function render()
    {
        foreach ($this ->headers as $header) {
            echo $header . "\n";
        }

        if (isset($this ->title)) {
            echo "<title>" . $this ->title . "</title>";
        }
    }


}
