<?php
/**
 * UTF8
 * stanislav.elbl, 02.11.2015
 */

namespace Ron;

class EMailAttachment
{
    var $fileName;
    var $content;

    /**
     * eMailAttachment constructor.
     * @param $fileName
     * @param $content
     */
    public function __construct($fileName, $content)
    {
        $this->fileName = $fileName;
        $this->content = $content;
    }
}



class EMail
{
    var $from;
    var $fromName;

    var $to;
    var $copyTo;
    var $blindCopyTo;
    var $replyTo;

    var $subject;
    var $text;

    /** @var EMailAttachment[] $attachments */
    var $attachments = [];

    var $sendHtml = null; // podle nastaveni maileru


    function addAttachment($fileName, $content)
    {
        $this ->attachments[] = new EMailAttachment($fileName, $content);
    }

    function __toString()
    {
        return
            "Od:  $this->from 
            Jmeno odesilatele: $this->fromName 
            Komu: $this->to 
            Kopie: $this->copyTo 
            Skryta kopie: $this->blindCopyTo
            Odpoved: $this->replyTo
            Predmet: $this->subject
            Text: $this->text";
    }

    function convertToUTF8()
    {
        $this ->subject = StringUtils::toUTF8($this ->subject);
        $this ->text = StringUtils::toUTF8($this ->text);
        $this ->fromName = StringUtils::toUTF8($this ->fromName);
    }

    function getSubject()
    {
        global $mailer_subject_prefix;
        return ($mailer_subject_prefix ?: '') . $this->subject;
    }

}



abstract class MailManager
{
    /**
     * @param eMail $eMail
     * @throws RonException
     */
    abstract function send($eMail);


    /**
     * @param EMail $eMail
     */
    public static function sendMail($eMail)
    {
        // pred require_once nastavit promenne jako globalni!
        global $sendmail_disable;
        //global $sendmail_lasterror; // chybova hlaska pri poslednim odeslani emailu
        global $sendmailpath;
        global $usephpmailer;
        global $phpmailerpath;
        global $additional_parameters;
        global $phpmailer_host;
        global $phpmailer_port;
        global $phpmailer_user;
        global $phpmailer_password;
        global $phpmailer_html;
        global $phpmailer_secure;
        global $phpmailer_options;
        global $mailer_subject_prefix;
        global $sendmail_debug_receiver;

        require_once("_email_set.php"); // nacte nastaveni odesilani emailu

        if ($sendmail_disable) {
            MessageManager::addDebugMessage('Odesilani emailu je vypnuto (zapnuta volba sendmail_disable). Email nebude odeslan.');
            MessageManager::addDebugLog($eMail);
            return;
        }

        $mailManager = null;

        global $usephpmailer;
        global $phpmailerpath;
        if ($usephpmailer  ||  $phpmailerpath) {
            $mailManager = new PhpMailerMailManager();
        } else {
            $mailManager = new BasicMailManager();
        }

        if ($mailManager) {
            $mailManager ->send($eMail);
        }
    }
}


/**
 * Odesilani emailu pres standardni funkci mail
 * Class MailFunctionManager
 * @package Ron
 */
class BasicMailManager extends  MailManager
{
    function send($eMail)
    {
        // TODO: Otestovat, pripadne predelat na UTF8
        $subject = "=?windows-1250?B?" . base64_encode($eMail ->getSubject()) . "?=";

        $headers = "From: {$eMail ->from}\r\n";
        $headers .= "Content-type: text/plain;charset=windows-1250\r\n";
        if ($eMail ->copyTo) $headers .= "Cc: {$eMail ->copyTo}\r\n";

        global $additional_parameters;

        if ( !mail($eMail ->to, $subject, $eMail ->text, $headers, $additional_parameters) ) {
            throw new RonException(ronText('Email_error'));
        }
    }
}


/**
 * Odesilani emailu pres PHPMailer
 * Class PhpMailerManager
 * @package Ron
 */
class PhpMailerMailManager extends MailManager
{
    var $useOldVersion = false;

    /**
     * phpMailerManager constructor.
     */
    public function __construct()
    {
        global $usephpmailer;
        global $phpmailerpath;
        if ($usephpmailer) {
//            if ($this ->useOldVersion) {
//                require_once('PHPMailer_old/PHPMailerAutoload.php');
//            } else {
                require_once 'PHPMailer/src/Exception.php';
                require_once 'PHPMailer/src/PHPMailer.php';
                require_once 'PHPMailer/src/SMTP.php';
//            }
        } else if ($phpmailerpath) {
            require_once($phpmailerpath);
        } else {
            throw new \Exception('Chyba v PhpMailerMailManager - neni zadana cesta k PHP Maileru!');
        }
    }


    /**
     * Nastavi zadany PHPMailer podle _email_set.php
     * @param \PHPMailer\PHPMailer\PHPMailer $mailer
     */
    function initMailer($mailer)
    {
        global $phpmailer_host;
        global $phpmailer_port;
        global $phpmailer_user;
        global $phpmailer_password;
        global $phpmailer_secure;
        global $phpmailer_html;
        global $phpmailer_options;

        //$this ->useOldVersion = version_compare(PHP_VERSION, '5.5.0', '<');

        MessageManager::addDebugLog(
            "PhpMailerMailManager.initMailer: \n" .
            "Host: $phpmailer_host \n" .
            "Port: $phpmailer_port \n" .
            "User: $phpmailer_user \n" .
            "HTML: $phpmailer_html \n" );

        // Parametry odesilani
        $mailer ->Port = $phpmailer_port;
        $mailer ->Host = $phpmailer_host;
        $mailer ->Username = $phpmailer_user;
        $mailer ->Password = $phpmailer_password;
        $mailer ->SMTPAuth = ($phpmailer_user != "");
        $mailer ->Hostname = $phpmailer_host;
        if ($phpmailer_options  &&  is_array($phpmailer_options)) {
            foreach ($phpmailer_options as $key => $value) {
                $mailer ->{$key} = $value;
            }
        }

        if ( isset($phpmailer_secure) ) {
            $mailer->SMTPSecure = $phpmailer_secure;
            $mailer ->SMTPAutoTLS = false;
        }
    }


    /**
     * @param EMail $eMail
     * @throws RonException
     */
    function send($eMail)
    {
        try {
            // Vytvoreni nove instance tridy PHPMailer se zapnutymi chybovymi hlasenimi
            if ($this ->useOldVersion) {
                $mailer = new \PHPMailer(true); // pouzije starou verzi (nefunguje v PHP 5.4)
            } else {
                /** @noinspection PhpUnnecessaryFullyQualifiedNameInspection */
                $mailer = new \PHPMailer\PHPMailer\PHPMailer(true);
            }

            $this->initMailer($mailer);

            $eMail ->convertToUTF8();
            $mailer ->CharSet = 'utf-8';

            global $sendmail_debug_receiver;

            // Parametry emailu
            $mailer ->isSMTP();
            if ($sendmail_debug_receiver) {
                $mailer->From = $sendmail_debug_receiver;
            } else {
                $mailer->From = $eMail->from;
            }
            $mailer ->FromName = $eMail ->fromName ? $eMail ->fromName : $eMail ->from;

            if ($sendmail_debug_receiver) {
                $mailer->addAddress($sendmail_debug_receiver);
            } else {
                $recps = explode(";", $eMail->to);
                foreach ($recps as $recp) $mailer->addAddress($recp);

                $recps = explode(";", $eMail->copyTo);
                foreach ($recps as $recp) if ($recp) $mailer->addCC($recp);

                $recps = explode(";", $eMail->blindCopyTo);
                foreach ($recps as $recp) if ($recp) $mailer->addBCC($recp);

                $sendrs = explode(";", $eMail->replyTo);
                foreach ($sendrs as $sendr) if ($sendr) $mailer->addReplyTo($sendr);
            }

            $mailer ->Subject = $eMail ->getSubject();
            //$mail -> WordWrap = 80;
            //$mail -> LE = "\n";


            global $phpmailer_html;
            $useHtml = isset($eMail ->sendHtml) ? $eMail ->sendHtml : $phpmailer_html;
            $mailer ->isHTML($useHtml);

            $textmsg = strip_tags($eMail ->text);
            $textmsg = str_replace('&nbsp;', ' ', $textmsg);

            // Telo emailu
            if ($useHtml) {
                $mailer ->AltBody = $textmsg . $mailer ->getLE();
                $mailer ->Body = $eMail ->text . $mailer ->getLE();
            } else {
                $mailer ->Body = $textmsg . $mailer ->getLE();
            }
            // nebo vyzkouset toto?
            // preg_replace("/(\s*[\r\n]+\s*|\s+)/", ' ', $mailbody);

            // Prilohy
            foreach ($eMail ->attachments as $attachment) {
                $mailer ->addStringAttachment($attachment ->content, $attachment ->fileName);
            }

            //$mail -> MsgHTML($body);			// Vlozeni HTML tela zpravy
            //$mail -> AltBody = "Pro spravne zobrazeni zpravy potrebujete klienta s podporou zobrazeni HTML.";

            $mailer ->send(); // Samotne odeslani emailu

        } catch (\phpmailerException $e) {
            throw new RonException(ronText('Email_error') . ' ' . $e ->getMessage(), null, null, 0, $e);
        }
    }
}
