<?php
/**
 * UTF8
 * stanislav.elbl, 08.11.2015
 */

namespace Ron;


class StringUtils
{
    public static $collator = null;

    /**
     * Vraci true, pokud je retezec spravne zadane cislo integer
     * @param $valueStr
     * @return bool
     */
    public static function isInteger($valueStr)
    {
        return preg_match('/^-?[0-9]+$/', $valueStr);
    }


    public static function stringToInteger($valueStr)
    {
        if (!self::isInteger($valueStr)) {
            throw new \Exception(\Ron\ronText('Bad_number_format'));
        }

        return (int)$valueStr;
    }


    /**
     * @param string $value
     * @return int float
     */
    public static function stringToFloat($value)
    {
        $value = str_replace(',', '.', $value);
        $value = str_replace(' ', '', $value);

        return (double)$value;
    }


    /**
     * Vraci true, pokud je retezec spravne zadany float
     * @param $valueStr
     * @return bool
     */
    public static function isFloat($valueStr)
    {
        $valueStr = str_replace(' ', '', $valueStr); // mezery ignoruje

        return preg_match("/^-?[0-9]+[.,]?[0-9]*$/", $valueStr);
    }


    /**
     * Zkontroluje, jestli je zadany retezec ve spravnem formatu
     * - int, integer - cele cislo
     * - float
     * @param string $format
     * @param string $valueStr
     * @return bool
     */
    public static function checkFormat($format, $valueStr)
    {
        // date : "^[0-9]{4}-((0?[0-9])|(1[012]))-(([0-2]?[0-9])|(3[01]))$"
        switch ($format) {
            case 'int' :
            case 'integer' :
                return self::isInteger($valueStr);

            case 'float' :
                return self::isFloat($valueStr);
        }

        return false;
    }


    /**
     * Porovna 2 retezce s pouzitim Collatoru (pokud je zapnuto rozsireni intl), nebo strcasecmp (pokud neni zapnuto)
     * @param $str1
     * @param $str2
     * @param bool $utf     Nastavte na true, pokud jsou porovnavane retezce v UTF8.
     * @return int
     */
    public static function compareStrings($str1, $str2, $utf = false)
    {
        if (!self::$collator) {
            self::$collator = class_exists('\Collator', false)
                ? new \Collator('cs_CZ')
                : false;
        }

        if (self::$collator) {
            return $utf
                ? self::$collator->compare($str1, $str2)
                : self::$collator->compare(self::toUTF8($str1), self::toUTF8($str2));
        } else {
            return strcasecmp($str1, $str2);
        }
    }


    /**
     * Ze zadaneho retezce vytvori 2 pole - jednotlive polozky jsou v retezci oddelene strednikem.
     * Do prvniho pole se vlozi cast pred dvojteckou, do druheho cast za dvojteckou. Napriklad z retezce
     *   "Prijmeni:prijmeni;Jmeno:jmeno;ABC:def"
     * se vytvori pole ["Prijmeni", "Jmeno", "ABC"]  a  ["prijmeni" "jmeno", "def"]
     *
     * @param string $str
     * @param array $array1
     * @param array $array2
     * @param bool $clearArrays   Urcuje, jestli se maji pole na zacatku vyprazdnit
     */
    public static function parseStringToArrays($str, &$array1, &$array2, $clearArrays = false)
    {
        if ($clearArrays) {
            $array1 = [];
            $array2 = [];
        }

        if (!$str) {
            return;
        }

        $pomcolumns = explode(";", $str);

        foreach ($pomcolumns as $colstr) {
            $pomarray = explode(":", $colstr);
            $array1[] = isset($pomarray[0]) ? $pomarray[0] : '';
            $array2[] = isset($pomarray[1]) ? $pomarray[1] : '';
        }
    }


    /**
     * @param string $str
     * @return string
     */
    public static function strEncrypt($str)
    {
        for ($i = 0; $i < strlen($str); $i++) {
            $char = ord($str[$i]);
            $rotfact = (($i + 1) & 0xff) % 7;
            $char = (($char << $rotfact) & 0xff) + ($char >> (8 - $rotfact));
            $str[$i] = chr($char);
        }

        return $str;
    }


    /**
     * Prevede zadany retezec na hexadecimalni tvar (kazde pismeno zakoduje jako 2 pismena
     * @param string $str
     * @return string
     */
    public static function strToHexa($str)
    {
        $result = '';
        for ($i = 0; $i < strlen($str); $i++) {
            $result = $result . dechex(ord($str[$i]));
        }

        return $result;
    }


    /**
     * Vraci podretezec retezce $haystack koncici pred poslednim podretezcem $needle
     * @param string $haystack
     * @param string $needle
     * @return string
     */
    public static function strBeforeLast($haystack, $needle)
    {
        $pos = strrpos($haystack, $needle);
        if ($pos === false) return $haystack; // podretezec nenalezen

        return substr($haystack, 0, $pos);
    }


    /**
     * @param string $haystack
     * @param string $needle
     * @return string
     */
    public static function strAfterLast($haystack, $needle)
    {
        $pos = strrpos($haystack, $needle);
        if ($pos === false) return $haystack; // podretezec nenalezen

        return substr($haystack, $pos + strlen($needle));
    }


    /**
     * Z retezce $string (seznamu podretezcu oddelenych znakem $delimiter) odstrani zadany podretezec
     * @param string $delimiter Oddelovac
     * @param string $string    Zdrojovy retezec
     * @param string $subString Podretezec k odstraneni
     * @return string
     */
    public static function removeSubstring($delimiter, $string, $subString)
    {
        $a = explode($delimiter, $string);

        $result = '';
        foreach ($a as $substr) {
            if ($substr != $subString) {
                $result .= ($result ? $delimiter : '') . $substr;
            }
        }

        return $result;
    }

    /**
     * Ve vyrazu nahradi identifikatory, ktere zacinaji znakem '_' - tento zacatek nahradi retezcem $identifierStart
     * Pokud je '_' uvnitr identifikatoru, tak ho ponecha
     * Slouzi napr. pro doplneni $ ve vzorcich pro eval
     * Napr. processIdentifiers ( _110 + _110_A, '$H_', '_' ) vraci $H_110 + $H_110_A
     *
     * @param string $expression    Vyraz, ve kterem se texty nahradi
     * @param string $identifierStart     Novy zacatek identifikatoru
     * @return string
     */
    public static function processIdentifiers($expression, $identifierStart)
    {
        $result = "";
        $identifier = false; // priznak, ze zpracovava identifikator
        for ($i = 0;  $i < strlen($expression);  $i++) {
            if ($identifier) {
                // je v identifikatoru - pokud je znak pismeno, cislo nebo podtrzitko, tak identifikator pokracuje, jinak konci
                if (!ctype_alnum($expression[$i])  &&  $expression[$i] != '_') {
                    // konec identifikatoru
                    $identifier = false;
                }

                $result .= $expression[$i];
            }
            else if ($expression[$i] == '_') {
                // neni v identifikatoru => podtrzitko je zacatek kodu casove slozky
                $result .= $identifierStart;
                $identifier = true;
            }
            else {
                if (ctype_alpha($expression[$i])) {
                    // zacatek identifikatoru (pismeno)
                    $identifier = true;
                }

                $result .= $expression[$i];
            }
        }

        return $result;
    }


    /**
     * Prevede dlouhy text na text s html znackama
     * @param string $text
     * @param bool   $htmlTextConvert   pokud je true, provede se i nejdrive htmlTextConvert()
     * @return string
     */
    public static function longTextToHtml($text, $htmlTextConvert = false)
    {
        if ($htmlTextConvert) $text = self::htmlTextConvert($text);
        $text = str_replace("\n", ' <br/> ', $text);

        return $text;
    }


    /**
     * Transformuje text pro pouziti v XML
     * @param string $text
     * @return string mixed
     */
    public static function xmlTextConvert($text)
    {
        $text = str_replace('&', '&amp;', $text); // toto musi byt prvni!

        $text = str_replace('<', '&lt;', $text);
        $text = str_replace('>', '&gt;', $text);

        $text = str_replace("'", '&apos;', $text);
        $text = str_replace('"', '&quot;', $text);

        //$text = str_replace ('%', '&#37;', $text);

        return $text;
    }


    /**
     * @param string $text
     * @param array $replaceTexts  asociativni pole dalsich retezcu, ktere se maji nahradit (napr ["\n" => '\n'])
     * @return string
     */
    public static function htmlTextConvert($text, $replaceTexts = [])
    {
        //return htmlspecialchars($text, ENT_COMPAT, 'windows-1250'); - windows-1250 not suported
        $text = self::xmlTextConvert($text);
        if ($replaceTexts) {
            foreach ($replaceTexts as $search => $replace) {
                $text = str_replace($search, $replace, $text);
            }
        }

        return $text;
    }


    const SECURE_STRICT = null;
    const SECURE_URL = ['"', "'", ">", "<"];
    const SECURE_QUOTAS = ['"', "'"];

    /**
     * Odstrani z retezce vsechny znaky krome pismen, cislic, podtrizitka. Pokud je zadana mnozina $removeCharSet, tak odstrani znaky z teto mnoziny.
     * @param $text
     * @param array|bool $removeCharSet  Pole znaku, ktere se maji odstranit
     * @return mixed
     */
    public static function secureString($text, $removeCharSet = null)
    {
        if (!$removeCharSet) {
            return preg_replace('/[^\w]/', '', $text);
        }

        if ($removeCharSet === true) {
            return str_replace(self::SECURE_URL, '', $text);
        }

        return str_replace($removeCharSet, '', $text);
    }

    /**
     * Nahradi v retezci znaky tak, aby se dal pouzit v css jako jmeno tridy nebo id
     * @param $text
     * @return mixed
     */
    public static function cssName($text)
    {
        return str_replace([' ', ',', '.', ';', ':', '&', '!', '?', '#', '$', '%',
            "'", '"', '(', ')', '/', '<', '>', '=', '\\', '[', ']', '|', '{', '}', '^', '@'], '_', $text);
    }


    /**
     * Prevede zadany text do UTF8. Pokud system bezi na utf8, nedela nic
     * @param $text
     * @return string
     */
    public static function toUTF8($text)
    {
        $charset = TextManager::getInstance() ->getCharSet();
        return ($charset && $charset != 'utf-8') ? iconv($charset, 'utf-8', $text) : $text;
    }


    /**
     * Prevede zadany text v UTF8 do lokalniho kodovani (charset). Pokud system bezi na utf8, nedela nic
     * @param $text
     * @return string
     */
    public static function fromUTF8($text)
    {
        $charset = TextManager::getInstance() ->getCharSet();
        return ($charset && $charset != 'utf-8') ? iconv('utf-8', $charset, $text) : $text;
    }


    public static function jsonText($text, $htmlConvert = false)
    {
        $result = self::toUTF8($text);
        if ($htmlConvert) $result = self::htmlTextConvert($result);
        return $result;
    }


    /**
     * Pripoji k url zadane parametry. Automaticky prida ? nebo &
     * @param string $url
     * @param string|string[] $params
     * @return string
     */
    public static function concatenateHtmlParams($url, $params)
    {
        if (!$params) return $url;

        $result = $url;

        if (strstr($url, '?') === false) {
            $url .= '?';
        }

        if (!is_array($params)) {
            $params = [$params];
        }

        foreach ($params as $param) {
            if ($param) {
                if (substr($result, -1) != '&'  &&  substr($result, -1) != '?') {
                    $result .= '&';
                }

                $result .= $param;
            }
        }

        return $result;
    }


    public static function jsonDate($date)
    {
        if ($date) return date('j.n.Y', $date);
        return '';
    }


    /**
     * Naformatuje zadanou hodnotu (podle formatu to muze byt float, int).
     * @param string $format Mozne hodnoty jsou:
     * <ul><li>date</li><li>time</li><li>timestamp</li><li>datetime</li><li>year</li><li>finedate</li><li>finedatetime</li>
     * <li>kc</li><li>curr</li><li>anone</li><li>int</li><li>float</li><li>number</li><li>number0</li><li>number1 (jedno desetinne misto</li><li>number_1 (max jedno des. misto)</li><li>number2 (2 des. mista)</li><li>number_2 (max 2 des. mista)</li>
     * <li>hodiny</li><li>minuty</li><li>sekundy</li><li>timeofday</li>
     * </ul>
     * @param mixed $value
     * @param bool $nullToEmptyString  Pokud je true, tak misto null vraci prazdny retezec
     * @return string
     */
    public static function format($format, $value, $nullToEmptyString = true)
    {
        if ($nullToEmptyString  &&  (!isset($value) ||
                ((!(($format === 'anone') || ($format === 'int') || ($format === 'number_1') || ($format === 'number_2'))) //pro tyto formaty muze byt na vstupu, prazdna hodnota
                    && ($value === false || $value === '')
                )
            )
        ) {
            return '';
        }

        switch ($format)
        {
            case 'date' :       return date('j.n.Y', $value); // if ($pom = db_stringtotimestamp($data)) return date('j.n.Y', $pom); else return NULL; // $pom kvuli datum s rokem vetsim nez 2038
            case 'time' :       return date('G:i', $value);
            case 'timestamp' :  return date('j.n.Y G:i', $value);
            case 'datetime' :   return date('j.n.Y G:i', $value);
            case 'year' :       return date('Y', $value);
            case 'finedate' :   return DateUtils::fineDateString($value, false);
            case 'finedatetime' : return DateUtils::fineDateString($value, true);
            case 'weekday' :    return DateUtils::dayName($value);
            case 'weekdayshort' : return DateUtils::dayName($value, true);
            case 'weektimestamp' : return DateUtils::dayName($value, true) . ' ' . date('j.n.Y G:i', $value);
            case 'kc' :         return number_format($value, 2, ',', ' ') . ' ' . ronText('_vychozi_mena');
            case 'curr' :       return number_format($value, 2, ',', ' ');
            case 'anone':       return ronText($value ? 'Str_Yes' : 'Str_No');
            case 'int' :        return (int)$value;
            case 'float' :      return str_replace('.', ',', $value);
            case 'number' :     return str_replace('.', ',', $value);
            case 'number0' :    return number_format($value, 0, ',', ' ');
            case 'number1' :    return number_format($value, 1, ',', ' ');
            case 'number_1' :   return str_replace('.', ',', round($value, 1)); // 1 nebo zadne desetinne misto
            case 'number2' :    return number_format($value, 2, ',', ' ');
            case 'number_2' :   return str_replace('.', ',', round($value, 2)); // 2, 1 nebo zadne desetinne misto
            case 'hodiny' :     return DateUtils::minutesToString($value * 60); // float pocet hodin => h:m
            case 'minuty' :     return DateUtils::minutesToString($value); // pocet minut => h:m
            case 'sekundy' :    return DateUtils::secondsToString($value); // int => cas ve formatu h:m:s
            case 'timeofday' :  return DateUtils::minutesToString($value * 60);
            case 'text' :       return $value;
            default :
                if ($format) return self::htmlTextConvert(sprintf($format, $value));
                else return self::htmlTextConvert($value);
        }
    }


    /**
     * Sestavi vysledny retezec z dane hodnoty dle potreby pro min/max (sql dotaz) nebo nove ID
     * @param string $mask
     * @param int $numericalPart
     * @param bool $incrementValue
     * @return string
     */
    static function assemblyStringIDFromValueAndMask($mask, $numericalPart, $incrementValue = FALSE)
    {
        //Spocita delku hash casti - pro potreby "natahnuti" hodnoty na pozadovanou velikost
        $hashLength = substr_count($mask, '#');

        //Numericka cast
        if ($incrementValue)
            //Nove ID - oprava 9.5. Delku ciselneho retezce lze pocitat az po navyseni, jinak chybne doplnuje nuly
            $numericalPart = (int)++$numericalPart;

        //Spocita pocet nul, ktery bude muset pridat tak aby se respektovala maska tedy ne ID13 ale ID000013
        $pocetNul = $hashLength - strlen($numericalPart);

        //Sestavi string ID + prislusny pocet nul + hodnota

        //Prefix
        $stringNewNumericalPart = substr($mask, 0, strpos($mask, "#"));

        //Doplni nuly aby opet vzniklo cislo v pozadovanem formatu masky
        $stringNewNumericalPart .= str_repeat("0", $pocetNul);

        $stringNewNumericalPart .= $numericalPart;

        //Postfix
        $stringNewNumericalPart .= substr($mask, strrpos($mask, "#")+1);

        return $stringNewNumericalPart;
    }


    /**
     * Nacte ciselnou cast na zaklade masky, ktera je ocekavana ve tvaru string#######string a vrati ji, nebo zbyvajici rozsah
     * @param $mask
     * @param $stringId
     * @param null | int $max
     * @return int
     */
    static function getNumberOrRangeFromStringID($mask, $stringId, $max = NULL)
    {
        $hashLength = substr_count($mask, '#');

        if ($max != ""){
            //Pokud bude existovat parametr $max pak vypocitat zbyvajici prostor v rozsahu (max - aktualni hodnota)
            $numericalPart = $max - (substr($stringId, strpos($mask, "#"), $hashLength));
        } else {
            $numericalPart = substr($stringId, strpos($mask, "#"), $hashLength);
        }

        return (int)$numericalPart;
    }
}


