<?php
/**
 * UTF8
 * stanislav.elbl, 1.08.2015
 */

namespace Ron;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\DBException;
use Ron\MVC\JsonOutputObject;


class Utils
{
    /**
     * Vraci JsonOutputObject, ktery obsahuje vysledek zadaneho dotazu.
     * @param Database $database
     * @param string $sql SQL text pro nacteni dat. Muze obsahovat i vice vyslednych poli.
     * @param Parameters $sqlParams
     * @param string|string[] $outputFormat Format vystupu. Pokud je $outputKey pole, tak musi byt taky pole o stejnem poctu prvku
     * @param string|string[] $outputKey    Nazev polozky, do ktere se zapise vysledek. Napr. pro implicitni 'text' vygeneruje [ 'text' => <hodnota> ]
     *                                      Pokud je to pole, tak se jednotlive polozky pouziji jako nazev poli vysledku - melo by obsahovat stejny pocet prvku jako vysledek sql
     * @param string|string[] $defaultValue Implicitni hodnota - pouzije se v pripade, ze neni vysledek nalezen.
     * @param string $notFoundError
     * @return JsonOutputObject
     */
    public static function sqlResultJsonOutput($database, $sql, $sqlParams = null, $outputFormat = null, $outputKey = 'text', $defaultValue = null, $notFoundError = null)
    {
        $result = new JsonOutputObject();

        try {
            $cursor = $database->select($sql, $sqlParams);
            if (!$cursor) {
                throw new DBException($database, $sql);
            }

            if ($row = $cursor->fetch()) {
                if (is_array($outputKey)) {
                    foreach ($outputKey as $i => $key) {
                        $value = $outputFormat ? DBUtils::getFormatedValue($database, $outputFormat[$i], $row[$i]) : $row[$i];
                        $result->addData($key, $value);
                    }
                } else {
                    $value = $outputFormat ? DBUtils::getFormatedValue($database, $outputFormat, $row[0]) : $row[0];
                    $result->addData($outputKey, $value);
                }
            } else {
                // nenasel zadny zaznam
                if (is_array($outputKey)) {
                    foreach ($outputKey as $index => $key) {
                        $result->addData($key, $defaultValue[$index]);
                    }
                } else {
                    $result->addData($outputKey, $defaultValue);
                }

                if ($notFoundError) $result->addError($notFoundError);
            }
        } catch (DBException $e) {
            $result->addError($e->getMessage(), null, $e->getErrorDescription());
        } catch (\Exception $e) {
            $result->addError($e->getMessage());
        }

        return $result;
    }
}
