<?php
/**
 * stanislav.elbl, 15.07.2015
 */

namespace RonPortal\Controller;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\HttpController;
use Ron\RonPortal\XML\RonPortalXMLForm;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalXMLEvents;
use RonPortal\SamlUtils;
use RonPortal\Security;
use RonPortal\SoftipUtils;
use RonPortal\User;

class LoginController extends HttpController
{
    /** @var Database */
    var $database;


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }



    /**
     * Pri logout vycisti session
     */
    private static function clearSession()
    {
        $terminalvkladani = isset($_SESSION['terminalprovkladani']) ? $_SESSION['terminalprovkladani'] : null;

        $lang = S_GetLanguage();

        session_unset();
        ////session_destroy();
        //session_start();

        S_SetLanguage($lang);

        User::setLoginName('');
        User::setUserAuthenticated(false);

        $_SESSION['terminalprovkladani'] = $terminalvkladani;
    }


    public static function logoutUser()
    {
        self::clearSession();
    }



    public static function nastavSessionLogin($rc, $jmenoPrijmeni, $skupinaiw)
    {
        User::setUserName($jmenoPrijmeni);

        User::setSkupinaUzivatele($skupinaiw);
        User::setRC($rc);

        //nactiPravaUzivatele($this->database, $rc); - presunuto do adsapp.php
    }


    public function loginPage()
    {
        global $login_xml_id;

        if ($login_xml_id) {
            return new InternalRedirect('xmllogin');
        }

        return 'view/login.php';
    }


    /**
     * @return Redirect|string
     * @throws Exception
     */
    public function handleLogout()
    {
        require_once 'xml/xmlformportal.php';

        if (User::isAuthenticated()) {
            RonPortalXMLForm::xmlEvents($this->database, RonPortalXMLEvents::LOGOUT_BEFORE, [], '');
        }

        global $simplesaml_sp;

        if ($simplesaml_sp) {
            SamlUtils::samlLogout();
//            global $simplesaml_path;
//
//            require_once ( ($simplesaml_path ?: '../simplesamlphp') .  '/lib/_autoload.php');
//            $as = new \SimpleSAML\Auth\Simple($simplesaml_sp);
//            if ($as ->isAuthenticated()) {
//                $as ->logout();
//            }
//
//            \SimpleSAML\Session::getSessionFromRequest()->cleanup();
            self::clearSession();
            MessageManager::addSuccessMessage(TextManager::_('Succesfully_logged_out'));
            return 'view/loggedoutview.php';
        }

        self::clearSession();

        if (RonPortalXMLForm::$eventRedirect) {
            return RonPortalXMLForm::$eventRedirect;
        }

        return $this ->loginPage();
    }


    /**
     * Vraci false, pokud je uzivatel povinen zmenit si heslo.
     * @param bool $vynutitZmenu
     * @param int $posledniZmena
     * @return bool
     * @throws Exception
     */
    protected function overPlatnostHesla($vynutitZmenu, $posledniZmena)
    {
        // kontrola, jestli je vyzadovana zmena hesla (vyprsela platnost nebo zaskrtnuto "vynutit zmenu")
        if ($vynutitZmenu) {
            return false;
        }

        $policy = Security::getPasswordPolicy($this ->database);

        if ($policy ->platnost > 0) {
            // kontrola, jestli nevyprselo heslo
            if (!$posledniZmena) {
                // jeste neni nastaveno "posledni zmena"
                return false;
            }

            if (DateUtils::incDay($posledniZmena, $policy ->platnost) < time()) {
                return false;
            }
        }

        return true;
    }


    /**
     * @param $skupinaIW
     * @return false|mixed
     * @throws DBException
     */
    private function setUserCssFile($skupinaIW)
    {
        $params = $this ->database ->createParameters();
        $sql = 'SELECT cssSoubor FROM SkupinaUzivateluIW WHERE kod = ' . $params ->addParameter($skupinaIW);
        $result = DBUtils::getSqlValue($this ->database, $sql, $params);
        User::setUserCss($result);

        return $result;
    }


    private function jePlatnyPP($rc)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Id FROM PracovniPomer ' .
            ' WHERE PracovniPomer.RC = ' . $params->addParameter($rc) .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            '   AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')';

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani pracovnich pomeru!');
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    /**
     * @param $loginName
     * @param $rc
     * @param $jmeno
     * @param $prijmeni
     * @param $skupinaIW
     * @param bool|string $autoLogin
     * @param bool $writeLog
     * @throws DBException
     */
    protected function userLoggedIn($loginName, $rc, $jmeno, $prijmeni, $skupinaIW, $autoLogin, $writeLog = true)
    {
        if ($autoLogin === true) {
            $autoLogin = 'ANO';
        }

        if (!$this ->jePlatnyPP($rc)) {
            Log::addRecordToLog($this->database,
                sprintf(StringUtils::fromUTF8(TextManager::_('Invalid_login_log')), $loginName),
                StringUtils::fromUTF8(TextManager::_('Login_log_category')), '', $loginName);
            sleep(2);
            throw new Exception(TextManager::_('Nemate_platny_pp'));
        }

        global $profilUzivatele;
        if ($profilUzivatele) {
            $skupinaIW = $profilUzivatele;
        }

        // TODO: asi by se nemelo provadet pri prihlaseni pres SimpleSamlPHP ! ! !
        session_regenerate_id(true);    // po kazdem prihlaseni zmeni session ID

        User::setLoginName($loginName);
        LoginController::nastavSessionLogin($rc, $jmeno . ' ' . $prijmeni, $skupinaIW);
        User::setUserAuthenticated(true);

        RonPortalSession::initSessionTimestamp();

        $userCss = $this ->setUserCssFile($skupinaIW);

        if ($writeLog) {
            global $db_utils;
            global $db_connectionstring;
            global $appVersion;

            Log::addRecordToLog($this->database, StringUtils::fromUTF8('Přihlášení uživatele: ') . $loginName, StringUtils::fromUTF8('Přihlášení/odh. uživatele'),
                StringUtils::fromUTF8('Uživatel: ') . $loginName . ' - ' . User::getUserName() . ' (profil ' . User::getSkupinaUzivatele() . ')' .
                "\nVerze Intrawebu: " . $appVersion .
                "\nVerze PHP: " . phpversion() .
                "\nURL: http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]" .
                StringUtils::fromUTF8("\nPřipojení k databázi: ") . $db_utils . ', ' . substr($db_connectionstring, 0, 20) . // nevypisuje cele, aby tam nebylo heslo
                "\n  odbc.defaultlrl: " . ini_get('odbc.defaultlrl') .
                "\n  mssql.textlimit: " . ini_get('mssql.textlimit') .
                "\n  mssql.textsize: " . ini_get('mssql.textsize') .
                StringUtils::fromUTF8("\nAutomatické přihlášení: ") . ($autoLogin ?: 'NE') .
                (!$autoLogin && isset($_POST['adblock']) && $_POST['adblock'] ? StringUtils::fromUTF8("\nPravděpodobně je zapnutý ADBLOCK nebo vypnutý Javascript!") : '') .
                "\nSession\n  save_path: " . ini_get('session.save_path') .
                "\n  gc_maxlifetime: " . ini_get('session.gc_maxlifetime') .
                "\n  gc_probability: " . ini_get('session.gc_probability') .
                "\n  gc_divisor: " . ini_get('session.gc_divisor') .
                "\nTimezone: " . ini_get('date.timezone') .
                "\ndisplay_errors: " . ini_get('display_errors') .
                "\nlog_errors: " . ini_get('log_errors') .
                "\nerror_log: " . ini_get('error_log') .
                "\nerror_reporting: " . sprintf('%X', error_reporting()) .
                "\nmax_execution_time: " . ini_get('max_execution_time') .
                "\npost_max_size: " . ini_get('post_max_size') .
                "\nupload_max_filesize: " . ini_get('upload_max_filesize') .
                "\nupload_tmp_dir: " . ini_get('upload_tmp_dir') .
                "\nSCRIPT_FILENAME: " . $_SERVER['SCRIPT_FILENAME'] .
                "\nREMOTE_USER: " . $_SERVER['REMOTE_USER'] .
                "\nREMOTE_HOST: " . $_SERVER['REMOTE_HOST'] .
                "\nHTTP_USER_AGENT: " . $_SERVER['HTTP_USER_AGENT'] .
                "\nUser CSS: " . $userCss
            );
        }
    }


    /**
     * @param string $id
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    function checkIDLogin($id)
    {
        global $idlogin_kod2;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Osoba.Jmeno, Osoba.Prijmeni, Osoba.IntrawebLogin, Osoba.SkupinaIW, Karta.Blokovana, Karta.DatumVyrazeni ' .
            ' FROM Osoba, PrirazeniKarty, Karta ' .
            ' WHERE Karta.Kod = PrirazeniKarty.KodKarty ' .
            ' AND ' .( $idlogin_kod2 ? 'Karta.Kod2' : 'PrirazeniKarty.KodKarty') . ' = ' . $params->addParameter($id) .
            ' AND PrirazeniKarty.RC = Osoba.RC ' .
            ' AND PrirazeniKarty.DatumPrideleni <= ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
            ' AND (PrirazeniKarty.DatumVraceni >= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::startOfTheDay(time()))) .
            ' OR  PrirazeniKarty.DatumVraceni IS NULL) ';

        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            // Nenasel prirazeni karty
            Log::addRecordToLog($this->database,
                sprintf(StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou %s - neplatná nebo nepřiřazená karta!'), $id),
                StringUtils::fromUTF8('Přihlášení/odh. uživatele'));
            throw new Exception( TextManager::_('Bad_id_medium') );
        }

        if ($row[5]) {
            Log::addRecordToLog($this->database,
                sprintf(StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou %s - karta je zablokovaná!'), $id),
                StringUtils::fromUTF8('Přihlášení/odh. uživatele'));
            throw new Exception( TextManager::_('Karta_je_zablokovana') );
        }

        if ($row[6]) if ($this ->database ->stringToTimestamp($row[6]) <= time()) {
            Log::addRecordToLog($this->database,
                sprintf(StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele kartou %s - karta je vyřazená!'), $id),
                StringUtils::fromUTF8('Přihlášení/odh. uživatele'));
            throw new Exception( TextManager::_('Karta_je_vyrazena') );
        }

        User::setLoginName($id);
        //S_SetIdLogin(true);

        $this ->userLoggedIn($row[3] ? $row[3] : $id, $row[0], $row[1], $row[2], $row[4], false);  // loginName, rc, jmeno, prijmeni, skupinaIW

        return true;
    }


    /**
     * @param $loginName
     * @throws Exception
     */
    private function badPassword($loginName)
    {
        Log::addRecordToLog($this->database,
            sprintf(StringUtils::fromUTF8('Chybný pokus o přihlášení uživatele: %s!'), $loginName),
            StringUtils::fromUTF8('Přihlášení/odh. uživatele'), '', $loginName);
        sleep(3);
        throw new Exception( TextManager::_('Bad_password') );
    }


    /**
     * @param Database $database
     * @param string $loginName
     * @param string $password
     * @param string $dbPassword    Pokud neni zadano, nacte se z databaze
     * @return bool|string          Pri spravnem overeni hesla pomoci LDAP vraci "ldap", jinak vraci true/false
     * @throws Exception
     */
    public static function overHeslo($database, $loginName, $password, $dbPassword = null)
    {
        global $ldap_server;
        global $ldap_only_existing_users; // priznak, jestli ma pres ldap prihlasovat pouze existujici uzivatele

        if ($ldap_server) {
            $result = Security::checkLDAPLogin($loginName, $password);
            if (!$ldap_only_existing_users  ||  isset($result)) {
                User::setPasswordChangeEnabled(false);
                return $result ? "ldap" : false;
                // pokud uzivatel neexistuje a je nastaveno $ldap_only_existing_users, pokracuje dale (na standardni overeni dochazkou)
            }
        }

        if (!isset($dbPassword)) {
            $params = $database ->createParameters();
            $sql =
                'SELECT IntrawebPassword FROM Osoba ' .
                ' WHERE UPPER(IntrawebLogin) = ' . $params->addParameter(strtoupper($loginName));
            $cursor = $database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if (!$row) return false; // neni nalezen zadany uzivatel

            $dbPassword = $row[0];
        }

        $result = Security::checkPassword( Security::getPasswordType($database), $password, $dbPassword);
        if ($result) {
            // probehlo overeni pomoci dochazky => muze si zmenit heslo
            User::setPasswordChangeEnabled(true);
        }

        return $result;
    }


    /**
     * Overi prihlaseni pomoci jmena a hesla. V pripade chyby (i spatne heslo) vyhodi vyjimku.
     *
     * @param string $loginName
     * @param string $password
     * @param bool   $autoLogin
     * @throws Exception
     */
    private function checkUserLogin($loginName, $password, $autoLogin)
    {
        if (trim($loginName) == '') {
            // jmeno nesmi byt prazdne!
            throw new Exception( TextManager::_('Bad_password') );
        }

        if (strlen($loginName) > 25) {
            $this ->badPassword($loginName); // vyhodi vyjimku
        }

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT RC, Jmeno, Prijmeni, SkupinaIW, IntrawebPassword, VynutitZmenuHesla, PosledniZmenaHesla, IntrawebLogin ' .
            '  FROM Osoba ' .
            ' WHERE UPPER(IntrawebLogin) = ' . $params->addParameter(strtoupper($loginName));

        $cursor = $this ->database ->select($sql, $params);
        $row = $cursor ->fetch();
        $row2 = $row ? $cursor ->fetch() : null;
        $cursor ->close();

        if (!$row) {
            $this ->badPassword($loginName); // vyhodi vyjimku
            //throw new \Exception( TextManager::_('Bad_password') );
        }

        //global $ldap_server;
        global $simplesaml_sp;

        $overeno = null;

        if (!$autoLogin) {
            // kontrola hesla
            $overeno = self::overHeslo($this ->database, $loginName, $password, $row[4]);
            if (!$overeno) {
                $this ->badPassword($loginName); // vyhodi vyjimku
            }
        }

        if ($row2) {
            // existuje jeste jiny uzivatel se stejnym jmenem
            MessageManager::addErrorMessage(StringUtils::fromUTF8('V databázi existuje ještě další uživatel se stejným přihlašovacím jménem! To může způsobovat problémy!'));
        }

        // heslo je zadano spravne, nebo je automaticke prihlasovani
        User::setVynutitZmenuHesla(false);

        if ( $overeno !== 'ldap'  &&  !$simplesaml_sp ) {
            User::setPasswordChangeEnabled(true);

            // kontrola, jestli heslo nevyprselo
            if (!$this ->overPlatnostHesla($row[5], $this ->database ->stringToTimestamp($row[6]))) {
                User::setVynutitZmenuHesla(true);
            }
        }

        $this ->userLoggedIn($row[7], $row[0], $row[1], $row[2], $row[3], $autoLogin);
    }


    private function redirect()
    {
        $urlParams = [];
        parse_str($_SERVER['QUERY_STRING'], $urlParams);

        $newParams = [];

        foreach ($urlParams as $key => $value) {
            if (strpos($key, 'session@') !== 0  &&  $key != 'loginpassword'  &&  $key != 'loginname'  &&  $key != 'token'  &&  $key != 'idlogin') {
                if ($key != 'menuid'  ||  $value != 'login') { // menuid=login dela problem, pokud existuje uzivatelsky login controller
                    $newParams[$key] = $value;
                }
            }
        }

        return new Redirect(null, '?' .http_build_query($newParams));
    }


    /**
     * @return bool|Redirect|string
     */
    protected function handleLogin()
    {
        global $activedirectorylogin; // nastaveno v _appdbset.php
        global $enableurlidlogin;     // => zapnuto nepovinne prihlaseni pomoci id v url ( ?idlogin=...)

        try {
            global $simplesaml_sp;
            global $enableidlogin;

            if ($simplesaml_sp) {
                $userName = SamlUtils::samlLogin();
                self::clearSession();
                try {
                    $this->checkUserLogin($userName, null, 'SAML');
                } catch (Exception $e) {
                    if (TextManager::_('SAML_login_error')) {
                        MessageManager::addErrorMessage(TextManager::_('SAML_login_error'));
                    }
                    MessageManager::addErrorMessage($e);
                    return 'view/loggedoutview.php';
                }

            } else if ($enableidlogin  &&  isset($_POST['loginname'])) {
                // povinne prihlaseni pomoci ID media
                self::clearSession();
                $this ->checkIDLogin($_POST['loginname']);

            } else if (isset($_POST['loginname'])) {
                // prihlaseni z logovaci stranky
                self::clearSession();
                $this ->checkUserLogin($_POST['loginname'], $_POST['loginpassword'], false);

            } else if (isset($_GET['loginname'])) {
                // prihlaseni primo v url
                self::clearSession();
                $this->checkUserLogin($_GET['loginname'], $_GET['loginpassword'], false);

            } else if ( $enableurlidlogin  &&  isset($_GET['idlogin']) ) {
                self::clearSession();
                $this->checkIDLogin($_GET['idlogin']);

            } else if ($activedirectorylogin) {
                // Prihlaseni pomoci active directory
                $remote_user = $_SERVER['REMOTE_USER'];

                if (!empty($remote_user)) {
                    self::clearSession();
                    // uzivatelske jmeno je za poslednim lomitkem
                    $actual_username = substr(strrchr($remote_user, '\\'), 1);
                    $this ->checkUserLogin($actual_username, '', true);
                }

                if (!User::userAuthenticated()) {
                    throw new Exception( TextManager::_('Nelze_automaticky_prihlasit') );
                }
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return $this ->loginPage();
        }

        if (User::userAuthenticated()) {
            $_SESSION['AfterLogin'] = 1;
            return $this ->redirect();
        } else {
            return $this ->loginPage();
        }
    }


    /**
     * Overi prihlaseni osoby se zadanym osobnim cislem. V pripade chyby (osoba neexistuje nebo nema platny PP) vyhodi vyjimku.
     * Pouziva se i z uzivatelskych kontroleru!
     * @param string $osobniCislo
     * @throws Exception
     */
    protected function checkPersonLogin($osobniCislo)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Jmeno, Prijmeni, SkupinaIW, IntrawebPassword, VynutitZmenuHesla, PosledniZmenaHesla, IntrawebLogin ' .
            '  FROM Osoba, PracovniPomer ' .
            ' WHERE Osoba.RC = PracovniPomer.RC ' .
            '   AND PracovniPomer.OsobniCislo = ' . $params->addParameter($osobniCislo) .
            '   AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            '   AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')';

        $cursor = $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani osoby!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception( TextManager::_('Osoba_k_tokenu_nenalezena') );
        }

        // osoba se zadanym osobnim cislem nalezena
        User::setVynutitZmenuHesla(false);

        $this ->userLoggedIn($row[7], $row[0], $row[1], $row[2], $row[3], false);  // loginName, rc, jmeno, prijmeni, skupinaIW
    }


    function handleTokenLogin()
    {
        try {
            global $softipLoginUrl;

            if (isset($softipLoginUrl)) {
                // login pomoci Softip tokenu
                $osobniCislo = SoftipUtils::tokenUserAuthentication( $_GET['token'] );

                if (!$osobniCislo) {
                    throw new Exception( TextManager::_('Neplatny_token') );
                }

                $this ->checkPersonLogin($osobniCislo);
            } else {
                throw new Exception('Chyba - neni zadana url pro overeni tokenu!');
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
            return $this ->loginPage();
        }

        if (User::userAuthenticated()) {
            return $this->redirect();
        } else {
            return $this ->loginPage();
        }

    }


    /**
     * @param string $oldpass
     * @param string $newpass1
     * @param string $newpass2
     * @return bool
     * @throws Exception
     */
    function setPassword($oldpass, $newpass1, $newpass2)
    {
        if ($newpass1 != $newpass2) {
            throw new Exception( TextManager::_('Hesla_se_neshoduji') );
        }

        if (User::getVynutitZmenuHesla() && $oldpass == $newpass1) {
            // pokud je povinna zmena hesla, tak nesmi byt stare heslo stejne jako nove
            throw new Exception( TextManager::_('Nove_heslo_je_stejne') );
        }

        $policy = Security::getPasswordPolicy($this ->database);

        Security::checkPasswordPolicy($newpass1, $policy);

        if (!Security::checkUserPassword($this ->database, User::getRC(), $oldpass)) {
            Log::addRecordToLog($this->database,
                sprintf(StringUtils::fromUTF8('Chybný pokus o změnu hesla uživatele: %s - špatně zadané původní heslo!'), User::getLoginName()),
                StringUtils::fromUTF8('Přihlášení/odh. uživatele'));
            throw new Exception( TextManager::_('Spatne_puvodni_heslo') );
        }

        $passwordType = Security::getPasswordType($this ->database);
        $newpass1 = Security::encodePassword($passwordType, $newpass1);

        $params = $this ->database ->createParameters();
        $this ->database ->sql(
                'UPDATE Osoba SET IntrawebPassword = ' . $params->addParameter($newpass1) .
                ', VynutitZmenuHesla = 0,  PosledniZmenaHesla = ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
                ' WHERE RC = ' . $params->addParameter(User::getRC()), $params);

        Log::addRecordToLog($this->database,
            StringUtils::fromUTF8('Proběhla změna hesla uživatele: ') . User::getLoginName(),
            StringUtils::fromUTF8('Přihlášení/odh. uživatele'));

        User::setVynutitZmenuHesla(0);

        return true;
    }


    /**
     * @return bool|Redirect|string
     */
    function handleChangePassword()
    {
        if (isset($_POST['setpassword'])  &&  $_POST['setpassword']) {
            try {
                $this ->setPassword($_POST['oldpass'], $_POST['newpass1'], $_POST['newpass2']);
                return new Redirect(null, '?menuid=changepassword&changed=1');
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                // pokracuje dale (zobrazi znovu dialog)
            }
        } else {
            if (isset($_GET['changed'])  &&  $_GET['changed']) {
                return new TextViewObject(
                    TextManager::_('Heslo_zmeneno') .
                    '<br/><br/>' .
                    "<a href='?menuid='>" . TextManager::_('Pokracovat') . "</a>\n"
                );
            }
        }

        if (User::getVynutitZmenuHesla()) {
            MessageManager::addErrorText( 'Platnost_hesla_vyprsela' );
        }

        return new BaseViewObject('view/change_password.php');
    }


    public function handleUserLogin()
    {
        return false;
    }


    /**
     * @param string $action
     * @return string|Redirect|InternalRedirect
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('users.ini');

        if ($action === 'logout') {
            return $this->handleLogout();
        }

        // uzivatelsky definovane prihlaseni - z pripadneho uzivatelskeho controleru
        $result = $this ->handleUserLogin();
        if ($result) return $result;

        global $tokenLogin;
        if (isset($tokenLogin)  &&  $tokenLogin  &&  isset($_GET['token'])) {
            return $this ->handleTokenLogin();
        }

        global $enableurlidlogin;
        if ($enableurlidlogin  &&  isset($_GET['idlogin'])) {
            // prihlaseni pomoci ID v URL
            return $this ->handleLogin();
        }

        if ( isset($_GET['loginname']) ) {
            // prihlaseni v url - i kdyz je uzivatel prihlaseny (aby se mohl prihlasit jiny)
            return $this ->handleLogin();
        }

        if (!User::userAuthenticated()) {
            return $this ->handleLogin();
        }

        if ($action == 'changepassword'  ||  User::getVynutitZmenuHesla() ) {
            return $this ->handleChangePassword();
        }

        return null;
    }


}
