<?php
/**
 * stanislav.elbl, 18.03.2021
 */

namespace RonPortal\Controller;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\Controller;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\Common\VyjadreniFormular;
use RonPortal\Common\VzkazBrowser;
use RonPortal\Common\VzkazFormular;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\User;

class VzkazyController implements Controller
{

    /** @var Database */
    var $database;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        $this->database = $database;
    }


    function seznamVzkazu()
    {
        $result = new VzkazBrowser($this ->database);
        $result ->load();

        return $result;
    }


    function jePravoNaVzkaz($id, $pouzeProMe)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT RCOdesilatele, RCPrijemce, KodTypuRozdeleni, Rozdeleni, KodPrijemce' .
            ' FROM Vzkaz' .
            ' WHERE Id = '.$params ->addParameter((int)$id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) return false;

        if ( $row[1] == User::getRC() ) {
            // je to vzkaz pro me
            return true;
        }

        if ($pouzeProMe) {
            return false;
        }

        if ( $row[0] == User::getRC() ) {
            // je to muj vzkaz
            return true;
        }

        if ( $row[2]  &&  $row[3] ) {
            // kontrola, jestli je to pro "moje" rozdeleni
            return Rozdeleni::jeVRozdeleni($this ->database,
                User::getRC(), null,     // kontroluje jen RC
                $row[2], $row[3], false, // $vcetnenadrazenych = false ? ?
                DateUtils::today(), DateUtils::today());
        }

        if ( !isset($row[0])   &&   !isset($row[1])  &&  !isset($row[2])  &&  !isset($row[3])   &&  !isset($row[4]) ) {
            // vzkaz pro vsechny
            return true;
        }

        return false;
    }


    function vyjadreni() {
        $id = isset($_POST['Id']) ? (int)$_POST['Id'] : (int)$_GET['record'];

        if (!$this ->jePravoNaVzkaz($id, true)) {
            throw new NeniPravoException();
        }

        $f = new VyjadreniFormular($this ->database);

        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=vzkazy');
        }

        return $f;
    }


    function downloadDokument()
    {
        $idVzkazu = (int)$_GET['idvzkazu'];
        $idDokumentu = (int)$_GET['iddokumentu'];

        if (!$this ->jePravoNaVzkaz($idVzkazu, false)) {
            throw new NeniPravoException();
        }

        $platnost = null;
        if ($_GET['platnostodtimestamp']) $platnost = $_GET['platnostodtimestamp'];
        else if ($_GET['platnostod']) $platnost = DateUtils::dateStringToTimestamp($_GET['platnostod']);

        if (!$platnost) {
            throw new \Exception('Chyba - neni zadana platnost pozadovaneho dokumentu!');
        }

        // kontrola jestli je zadany dokument pripojeny k zadanemu vzkazu
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT IdDokumentu FROM DokumentVzkazu ' .
            ' WHERE IdVzkazu = ' . $params ->addParameter((int)$idVzkazu) .
            '   AND IdDokumentu = ' . $params ->addParameter((int)$idDokumentu) .
            '   AND PlatnostOd = ' . $params ->addParameter($this ->database ->dateToParam($platnost));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba - pozadovany dokument neni pripojen k zadanemu vzkazu!');
        }

        return new DokumentOutputObject($this ->database, $idDokumentu, true, $platnost);
    }


    /**
     * @param string $action
     * @return ViewObject|OutputObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('vzkazy.ini');

        $action = isset($_GET['action']) ? $_GET['action'] : '';

        if ($action == 'download') {
            // pri downloadu nekontroluje, jestli jsou vzkazy zapnute (pouziva se to i v xml)
            return $this ->downloadDokument();
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VZKAZY)) {
            throw new \Exception('Tato funkce neni povolena!');
        }

        switch ($action) {
            case 'novyvzkaz' :
                return new VzkazFormular($this ->database);

            case 'odeslatvzkaz' :
                $f = new VzkazFormular($this ->database);
                if (!$f ->handleAction()) return $f;
                return new Redirect(null, '?menuid=vzkazy');

            case 'vyjadreni' :
                return $this ->vyjadreni();

            default :
                return $this ->seznamVzkazu();
        }
    }
}
