<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;


use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\DB\Database;
use Ron\MessageManager;
use RonPortal\PracovniPomer;
use RonPortal\TypRozdeleni;

class ComboBoxesRozdeleni extends ComboBoxes
{
    /** @var Database */
    protected $database;

    /** @var string */
    public $name;
    /** @var string */
    public $id;

    /** @var TypRozdeleni */
    public $typRozdeleni;


    /**
     * ComboBoxesRozdeleni constructor.
     * @param Database $database
     * @param string $name
     * @param string $id
     */
    public function __construct(Database $database, $name, $id)
    {
        $this->database = $database;

        $isError = MessageManager::getInstance() ->isError($name);

        $combo1 = new ComboBox($name);
        if ($id) {
            $combo1 ->htmlParams['id'] = "{$id}_popis";
        }
        $combo1 ->htmlParams['data-ron-select2'] = $name . '_kod';
        //$combo1 ->htmlParams = ($id ? "id='{$id}_popis' " : '') . "data-ron-select2='{$name}_kod'";
        $combo1 ->className .= ' form-control ronselect' . ($isError ? ' error' : '');


        $combo2 = new ComboBox($name.'_kod');
        //$combo2 ->htmlParams = ($id ? "id='{$id}_kod' " : '') . "data-ron-select2='$name'";
        if ($id) {
            $combo2 ->htmlParams['id'] = "{$id}_kod";
        }
        $combo2 ->htmlParams['data-ron-select2'] = $name;
        $combo2 ->htmlParams['data-drop-down-align'] = 'right';
        $combo2 ->className .= ' form-control ronselect' . ($isError ? ' error' : '');

        parent::__construct($combo1, $combo2);
    }


    private function loadData($sql, $params, $aktualniHodnota, $povinne)
    {
        $this ->combo1 ->alwaysShowSelected = isset($aktualniHodnota) && $aktualniHodnota != '';
        //$this ->combo2 ->cssClassIndex = 2; -
        $this ->combo1 ->htmlParams['data-columns'] = 'text;value';
        $this ->combo1 ->attrIndexes = ['title' => 1, 'class' => 2];
        $this ->combo1 ->emptyRow = (isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ';
        $this ->combo1 ->loadData($this ->database, $sql.' ORDER BY Popis', $aktualniHodnota, $params);

        $this ->combo2 ->alwaysShowSelected = isset($aktualniHodnota)  &&  $aktualniHodnota != '';
        $this ->combo2 ->cssClassIndex = 2;
        $this ->combo2 ->captionIndex = 0;
        $this ->combo2 ->titleIndex = 1;
        $this ->combo2 ->htmlParams['data-columns'] = 'value;popis';
        $this ->combo2 ->attrIndexes = ['data-popis' => 1];
        $this ->combo2 ->emptyRow = (isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ';
        $this ->combo2 ->loadData($this ->database, $sql.' ORDER BY Kod', $aktualniHodnota, $params);
    }


    /**
     * @param PracovniPomer $pp
     * @param int    $typRozdeleni
     * @param string $aktualniHodnota
     * @param bool   $povinne
     */
    public function loadVlastniRozdeleni($pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        if (!$platnostOd) {
            $platnostOd = time();
        }

        if (!$platnostDo) {
            $platnostDo = $platnostOd;
        }

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
            '  FROM RozdeleniOsob, Rozdeleni'.
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)'.
            '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostDo)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)'.
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp ? $pp->rc : '') .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp ? $pp->osobniCislo : '') .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ? $pp->datumNastupu : 0)) .
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
            "   AND (cssClass IS NULL  OR  cssClass <> 'hidden')";

        $this ->loadData($sql, $params, $aktualniHodnota, $povinne);
    }


    public function loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        if (!$platnostOd) {
            $platnostOd = time();
        }

        if (!$platnostDo) {
            $platnostDo = $platnostOd;
        }

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND ( (' .
            ($aktualniHodnota != '' ? 'Rozdeleni.Kod = ' . $params->addParameter($aktualniHodnota)  : '0 = 1')   // zvolenou hodnotu zobrazi vzdy - pozor pri volbe "pouze vlastni" je to moc pomale
            . ')' .
            ' OR (' .
            '   (DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) . ' OR DatumZruseni IS NULL)' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->timestampToParam($platnostDo))  .
            ' )' .
            ')' .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

        $this ->loadData($sql, $params, $aktualniHodnota, $povinne);
    }


    /**
     * @param bool      $pouzeVlastni
     * @param PracovniPomer $pp
     * @param int       $typRozdeleni
     * @param string    $aktualniHodnota
     * @param bool      $povinne
     * @param int       $platnostOd
     * @param int       $platnostDo
     * @throws Exception
     */
    public function loadRozdeleni($pouzeVlastni, $pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        $this ->typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $typRozdeleni);

        if ($pouzeVlastni) {
            $this ->loadVlastniRozdeleni($pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd, $platnostDo);
        } else {
            $this ->loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne, $platnostOd, $platnostDo);
        }
    }

}
