<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;

class ComboBoxObdobi extends ComboBox
{
    const SETTINGS_MAXIMALNI_POCET_OBDOBI = 'maximalnipocetobdobi';

    var $pocatekObdobi;
    var $konecObdobi;
    var $popisObdobi;
    var $uzavreno;


    function __construct($varname = 'obdobi')
    {
        parent::__construct($varname);
    }


    function nastavSession()
    {
        RonPortalSession::setObdobi($this->pocatekObdobi, $this->konecObdobi);
    }


    function nastavSessionVariantni()
    {
        RonPortalSession::setVariantniObdobi($this ->pocatekObdobi, $this->konecObdobi);
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param bool   $emptyRow
     * @param int    $selectedValue
     * @return bool
     * @throws /Exception
     */
    function loadSQLObdobi($database, $sql, $emptyRow, $selectedValue)
    {
        $this->data = [];

        $this->selectedOption = null;

        if ($emptyRow) {
            $this->data[] = new Option('', '');
        }

        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani dat v ComboBoxObdobi::loadSQLObdobi!');

        $first = true;
        $pocet = 0;
        $jeVybrane = false;

        // prida jednotlive radky
        while ( ($row = $cursor ->fetch())  &&
            (RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI) <= 0  ||  $pocet < RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI)) ) {
            $pocatek = $database->stringToTimestamp($row[1]);
            $konec = $database->stringToTimestamp($row[2]);

            $jeAktualni = (time() >= $pocatek) && (time() < DateUtils::incDay($konec, 1));

            $nastavit = false;
            if ($selectedValue == $pocatek) {
                $nastavit = true;
                $jeVybrane = true;
            } else if ($first || ($jeAktualni && !$jeVybrane)) {
                $nastavit = true;
            }

            $this->data[] = $option = new Option( DateUtils::dateToUrlString($pocatek), $row[0]);

            if ($nastavit) {
                $this->pocatekObdobi = $pocatek;
                $this->konecObdobi = $konec;
                $this->popisObdobi = $row[0];
                $this->uzavreno = $row[3];

                $this->selectedOption = $option;
            }

            $first = false;

            if ($pocatek <= time()) { // do maximalniho poctu zapocita jen predchozi obdobi
                $pocet++;
            }
        }
        $cursor ->close();

        if ($first) {
            // V databazi neni zadne obdobi - to je mozne pri pouziti pristupaku bez dalsich aplikaci
            // ! ! ! Co kdyz je v databazi napr. nejake starsi testovaci obdobi jine aplikace ? ? ?
            //       Pak v pristupaku neni mozne vybrat aktualni obdobi
            $maxpocet = RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI);
            if ($maxpocet < 1) $maxpocet = 5;

            $date = DateUtils::firstDayOfMonth(time());
            for ($i = 0; $i < $maxpocet; $i++) {
                $popis = date('n/Y', $date);
                $pocatek = $date;

                $this->data[] = $option = new Option($pocatek, $popis);

                if ($first || ($selectedValue == $pocatek)) {
                    //$selecteddbrow = array($popis, $pocatek, $konec, $uzavreno);
                    $this->pocatekObdobi = $pocatek;
                    $this->konecObdobi = DateUtils::lastDayOfMonth($date);
                    $this->popisObdobi = $popis;
                    $this->uzavreno = 0;

                    $this ->selectedOption = $option;
                }

                $first = false;

                $date = DateUtils::firstDayOfMonth(DateUtils::incDay($date, -10)); // prejde na predchozi mesic
            }
        }

        return true;
    }


    /**
     * Nacte seznam obdobi
     * @param Database $database
     * @param string $appId
     * @param bool $variantni
     * @param bool $pouzeNeuzavrene
     * @param bool $emptyRow
     * @param int  $selectedValue
     * @throws \Exception
     */
    function loadObdobi($database, $appId, $variantni, $pouzeNeuzavrene, $emptyRow, $selectedValue)
    {
        $params = $database ->createParameters();
        if ($variantni) {
            $sql =
                'SELECT PopisObdobi, PocatekObdobi, KonecObdobi, 0 FROM VariantniObdobi' .
                ' ORDER BY PocatekObdobi DESC';
        } else {
            $sql = 'SELECT DISTINCT Popis, PocatekObdobi, KonecObdobi, Uzavreno FROM Obdobi ';

            if ($appId) {
                $sql .= ' WHERE Aplikace = ' . $params->addParameter($appId);
            } else {
                // vybere obdobi ze vsech aplikaci (krome skladu)
                // pozor - muze zpusobit, ze se obdobi opakuje!
                $sql .= " WHERE (Aplikace = 'ADS'  OR  Aplikace = 'APS'  OR  Aplikace = 'JIDELNA'  OR  Aplikace = 'MZDY') ";
            }

            if ($pouzeNeuzavrene) $sql = $sql . ' AND (Uzavreno = 0  OR  Uzavreno IS NULL)';

            $sql = $sql . ' ORDER BY PocatekObdobi DESC';
        }

        $this->loadSQLObdobi($database, new SqlStatement($sql, $params), $emptyRow, $selectedValue);
    }
}
