<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Controls;
use Ron\DateUtils;
use Ron\DB\Cursor;
use RonPortal\Kalendar;
use RonPortal\RonPortalSession;

/**
 * Obecny browser, ktery zobrazi tabulku, ve ktere bude pro kazdy den radek
 * a na nem zobrazeny prislusny zaznam (napr. planovane smeny)
 * SQL dotaz musi vracet na prvni pozici datum a musi byt podle tohoto data serazeny.
 * Format prvniho pole by mel byt timestamp. Pokud bude pro jeden den vice zaznamu, tak bude ten den na vice radcich.
 *
 * Class DenniZaznamyBrowser
 */
class DenniZaznamyBrowser extends RonPortalBrowser
{
    var $dateFrom; // nastavi se v load
    var $dateTo;


    /**
     * @param \Ron\DB\Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->cssClass = 'browser browserdenni grid';
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        $this->dateFrom = RonPortalSession::obdobi();
        $this->dateTo = RonPortalSession::konecObdobi();
    }


    /**
     * Zobrazi pro kazdy den v obdobi minimalne jeden radek.
     * @param Cursor $cursor
     */
    function renderCursor($cursor)
    {
        $datum = $this->dateFrom;
        $i = 1;

        $zaznam = $cursor ->fetch();
        if ($zaznam) $zaznam[0] = $this ->database ->stringToTimestamp($zaznam[0]);

        // Pro kazdy den vytvori radek
        while ($datum <= $this->dateTo) {
            $dalsidatum = DateUtils::incDayStart($datum, 1);

            $zobrazeno = false;

            // Zobrazi vsechny zaznamy pro aktualni den
            while ($zaznam && $zaznam[0] < $dalsidatum) {
                $this->renderLine($zaznam, Kalendar::getStyleForDate($this ->database, $zaznam[0]), $i);

                $zobrazeno = true;
                $i++;
                $zaznam = $cursor ->fetch();
                if ($zaznam) $zaznam[0] = $this ->database ->stringToTimestamp($zaznam[0]);
            }

            // Pokud nebyl pro aktualni datum zobrazen zadny zaznam, tak zobrazi prazdny radek
            if (!$zobrazeno) {
                $emptyrow = [$datum];

                $this->renderLine($emptyrow, Kalendar::getStyleForDate($this ->database, $datum), $i);

                $i++;
            }

            $datum = $dalsidatum;
        }
    }

}
