<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\BrowserField;
use Ron\DB\Database;


class JmenoPrijmeniBrowserField extends BrowserField
{
    /** @var int    Index pole se jmenem */
    public $indexJmeno;
    /** @var int    Index pole s prijmenim */
    public $indexPrijmeni;
    /** @var int    Index pole s titulem - zobrazi se pred jmenem */
    public $indexTitul;
    /** @var int    Index pole s titulem - zobrazi se az za jmenem */
    public $indexTitul2;

    /** @var int    Index pole, ktere se zobrazi, kdyz jsou ostatni pole prazdna */
    public $indexIfNull;

    /** @var bool   Urcuje, jestli se ma zobrazit nejdrive jmeno a pak prijmeni */
    public $nejdriveJmeno = false;


    /**
     * JmenoPrijmeniBrowserField constructor.
     * @param null|Database $database
     * @param int $indexJmeno
     * @param int $indexPrijmeni
     * @param int $indexTitul       Index pole s titulem - zobrazi se pred jmenem
     * @param int $indexTitul2      Index pole s titulem - zobrazi se az za jmenem
     */
    public function __construct($database, $indexJmeno, $indexPrijmeni = null, $indexTitul = null, $indexTitul2 = null)
    {
        parent::__construct($database);

        $this->indexJmeno = $indexJmeno;
        $this->indexPrijmeni = $indexPrijmeni;
        $this->indexTitul = $indexTitul;
        $this->indexTitul2 = $indexTitul2;

        $this ->header = \Ron\ronText('Jmeno');
    }


    /**
     * @param boolean $nejdriveJmeno
     * @return JmenoPrijmeniBrowserField
     */
    public function setNejdriveJmeno($nejdriveJmeno)
    {
        $this->nejdriveJmeno = $nejdriveJmeno;
        return $this;
    }

    /**
     * @param int $indexIfNull
     * @return JmenoPrijmeniBrowserField
     */
    public function setIndexIfNull($indexIfNull)
    {
        $this->indexIfNull = $indexIfNull;
        return $this;
    }



    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $text =
            ((isset($this ->indexTitul)  &&  $rowData[$this ->indexTitul])
                ? $rowData[$this ->indexTitul]
                : '');

        // prijmeni/jmeno
        $i = $this ->nejdriveJmeno ? $this ->indexJmeno : $this ->indexPrijmeni;
        if (isset($i)  &&  $rowData[$i]) {
            $text .= ($text ? ' ' : '') . $rowData[$i];
        }

        // jmeno/prijmeni
        $i = $this ->nejdriveJmeno ? $this ->indexPrijmeni : $this ->indexJmeno;
        if (isset($i)  &&  $rowData[$i]) {
            $text .= ($text ? ' ' : '') . $rowData[$i];
        }

        if (isset($this ->indexTitul2)  &&  $rowData[$this ->indexTitul2]) {
            $text .= ($text ? ', ' : '') . $rowData[$this ->indexTitul2];
        }

        if (!$text  &&  isset($this ->indexIfNull)) {
            $text = $rowData[$this ->indexIfNull];
        }

        return parent::getViewText($text, $rowNumber, $recordId, $rowData);
    }


    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'indexJmeno' :
            case 'indexPrijmeni' :
            case 'indexTitul' :
            case 'indexTitul2' :
            case 'nejdriveJmeno' :
                $this->{$key} = (int)$value;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }
}
