<?php
/**
 * stanislav.elbl, 05.03.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\FormField;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;

/**
 * Class OsobaFormField
 * @package RonPortal\Controls
 * Formularove pole pro vyber osoby
 */
class OsobaFormField extends FormField
{
    var $prava;
    var $datumPlatnostiPP;

    /** @var bool Priznak, jestli se ma zobrazit button na vyber vlastniho PP */
    var $zobrazitButtonVlastni = true;

    var $filtr;

    /** @var PracovniPomer  Pouzije se pro zobrazeni readonly hodnoty. Pokud neni zadan, tak se nacte dodatecn. */
    var $pracovniPomer;

    /**
     * OsobaField constructor.
     * @param string $prava kod prava
     * @param int $datumPlatnostiPP datum ke kteremu se bere platnost prac. pomeru
     * @param PracovniPomer $pracovniPomer      V pripade, ze je zadan, pouzije se pro zobrazeni readonly hodnoty. Nepovinnyy
     */
    function __construct($prava, $datumPlatnostiPP = null, $pracovniPomer = null)
    {
        parent::__construct();

        $this->fieldType = 'osoba';
        $this->fieldName = 'osoba';
        $this->prava = $prava;
        $this->required = true;

        $this->fieldCssClass = 'form-control ronselect';

        $this->datumPlatnostiPP = $datumPlatnostiPP ?: time();
        $this->pracovniPomer = $pracovniPomer;
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws \Ron\DB\DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            echo '<div class="row no-margin-bottom no-flex-wrap">';

            if ( !isset($value) ) {
                $value = RonPortalSession::vybranaOsoba();
            }

            $combo = new ComboBoxZamestnanec($this->fieldName); // jmeno promenne
            $combo->filtr = $this->filtr;
            $combo->htmlParams = $this ->getFieldAttributes();
//            $combo->htmlParams = $this->otherHtmlParams ?: [];
//            if ($this ->fieldId && is_array($combo->htmlParams) ) {
//                $combo->htmlParams['id'] = $this ->fieldId;
//            }
            $combo->loadDataPP($database,
                $this->datumPlatnostiPP, $this->datumPlatnostiPP,
                Prava::vratPrava($this->prava), ' ', $value, false); // $emptyRow = '', $selectedValue, $vcetneExternich = false
            $combo->render();
            $combo->nastavOsobuDoSession();

            if ($combo ->obsahujeVlastniPP  &&  $this ->fieldId) {
                echo "<i class='inline-glyph glyph-button fa glyph-own-data comboBoxMove' data-value='{$combo ->idVlastnihoPP}' data-selector='#{$this ->fieldId}' data-trigger-change='1'></i>";
            }

            echo '</div>';
        } else {
            if ($this ->pracovniPomer || $value) {
                $pp = $this ->pracovniPomer ?: PracovniPomer::nactiPP($database, $value);
                echo
                    '<input type="hidden" name="' . $this->fieldName . '" value="' . ($pp ? $pp ->idPP : '') . '" />' .
                    $pp ->jmenoPrijmeni .
                    "\n";
                //echo StringUtils::htmlTextConvert($pp ->jmenoPrijmeni) . "\n";
//            } else if ($this ->required  &&  !$insert) {
//                // Puvodni kod, ale je nesmyslny, protoze zobrazuje osobu z session. Nepouzivat! Pouze pro kompatibilitu
//                echo '<div class="warning">POZOR! Chyba pri pouziti OsobaField - neni zadana hodnota (idpp)!</div>';
//                echo '<input type="hidden" name="' . $this->fieldName . '" value="' . RonPortalSession::vybranaOsoba() . '" />';
//                echo 'OSOBA ID ' . RonPortalSession::vybranaOsoba() . "\n";
            }
        }
    }
}
