<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\Browser;
use Ron\DB\Database;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;

class RonPortalBrowser extends Browser
{
    var $action;

    /** @var RonPortalFilterForm  */
    var $filterForm;
    var $filterFormAppCode; // kod aplikace pro vyber obdobi
    var $filterFormRights;  // typ prav pro vyber osob
    var $filterFormVcetneExternich = false;

    /**
     * @param Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database);

        $this ->action = $action;
        $this ->browserUrl = "?menuid=$action&";

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'prohlizeni';
        $this ->filterFormVcetneExternich = false;
    }



    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action);

        if ($this ->filterFormAppCode) {
            $this->filterForm->nastavObdobi(true, $this->filterFormAppCode, false); // jenotevrene = false
        }

        $prava = Prava::vratPrava($this ->filterFormRights);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
    }


    /**
     * @param \ArrayAccess $colSettings
     */
    public function loadColumnConfiguration($colSettings)
    {
        if (!$colSettings) {
            return;
        }

        if (!$colSettings['profil']  ||  User::jeVProfilu($colSettings['profil'])) {
            parent::loadColumnConfiguration($colSettings);
        }
    }


    /**
     * Nacte XML konfiguraci browseru se zadanym kodem
     * @param Database $database
     * @param string $browserCode
     * @return string|null
     */
    public static function loadBrowserConfiguration($database, $browserCode)
    {
        if (!$browserCode) {
            return null;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare WHERE Popis = ' . $params ->addParameter('config_' . $browserCode),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Nacte konfiguraci browseru z XML sablony
     * @throws \Exception
     */
    function loadConfiguration()
    {
        if (!$this ->code) {
            return;
        }

        $xml = self::loadBrowserConfiguration($this ->database, $this ->code);
        if ($xml) {
            $this ->loadConfigurationXml($xml);
        }
    }


    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->setFilterForm();

        if ($this ->filterForm) {
            $this ->filterForm ->loadData();
        }

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        if ($this ->filterForm) {
            $this->filterForm->render();
        }
    }
}
