<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\Controls\DateInput;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\Profiler;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use RonPortal\NastaveniUzivatele;


class RonPortalFilterForm implements ViewObject
{
    /** @var Database */
    var $database;

    private $useGet;
    private $getMenuId;     // pokud je nastavena a je $useGet=true, prida se hidden promenna menuid
    public $getAction = ''; // action formulare pouzita pri $useGet=true (neni povinna)

    var $f_formaction;      // akce formulare - nemusi byt nastavena

    protected $hiddenFields = []; // asociativni pole jmeno=>hodnota

    /** @var string         Nadpis stranky - pokud je zadan, vygeneruje <h1> */
    public $header = null;

    var $f_autosubmit;

    /** @var string[]   atributy HTML formulare */
    public $formAttributes = [];

    /** @var bool       priznak, jestli ma zobrazit vyber obdobi */
    var $showObdobi;
    /** @var string     aplikace, ze ktere ma zobrazit obdobi */
    var $obdobiAplikace;
    var $jenOtevreneObdobi;
    /** @var bool       pokud neni obdobi povinne, zobrazi se i zaskrtavatko "jen obdobi" */
    var $obdobiPovinne;
    /** @var ComboBoxObdobi */
    var $comboObdobi;

    /** @var int        Pocatek vybraneho obdobi (nastavi se v load) */
    var $vybraneObdobiOd;
    /** @var int        Konec vybraneho obdobi (nastavi se v load) */
    var $vybraneObdobiDo;
    /** @var bool       Priznak, ze se ma zobrazit jen jedno obdobi (zaskrtavatko) */
    var $jenObdobi = false;

    /** @var bool       priznak, jestli ma zobrazit vyber variantniho obdobi */
    var $showVariantni;
    /** @var ComboBoxObdobi */
    var $comboVariantni;

    /** @var bool           zapina zadavani datumu od-do */
    var $showDatumOdDo;
    /** @var bool           priznak, jestli je zadani obdobi povinne (pokud ano nezobrazuje checkbox) */
    var $datumOdDoPovinne;
    /** @var string         Titulek pro datum */
    var $nadpisDatumOdDo;
    /** @var string         Umoznuje nastavit text, ktery se zobrazi za filtrem pro datum od-do */
    var $datumOdDoLinks = null;

    /** @var bool       priznak, ze se ma filtrovat datum */
    var $filtrovatDatum;
    /** @var int        datum od zadane uzivatelem */
    var $datumOd;
    /** @var int        datum do zadane uzivatelem */
    var $datumDo;
    /** @var string     Umoznuje nastavit text, ktery se zobrazi za filtrem pro datum */
    var $datumLinks = null;

    /** @var bool           zadani konkretniho datumu */
    var $showDatum;
    /** @var int            maximalni datum, ktere je mozne zadat */
    var $maxDatum = null;
    /** @var int            minimalni datum, ktere je mozne zadat */
    var $minDatum = null;
    /** @var int            aktualni hodnota zadaneho datumu */
    var $datum;

    /** @var bool           priznak, jestli ma zobrazit vyber rozdeleni */
    var $f_showrozdeleni;
    var $f_typrozdeleni;
    var $vlastniTypRozdeleni;
    /** @var integer        0=vse, 1=jen vlastni, 2=podle prav (prava muzou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene  */
    var $jenVlastniRozdeleni;
    /** @var string */
    var $filtrRozdeleni;
    /** @var PravaNaOsoby   Typ prava, podle ktereho se muze odfiltrovat rozdeleni */
    var $f_pravarozdeleni;
    var $pouzitPopisTypuRozdeleni;
    var $popisTypuRozdeleni;
    var $showVcetnePodrizenychRozdeleni;
    /** @var int            Hodnota pro select "vcetne podrizenych" 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene. Po load() se nastavi na skutecnou hodnotu. */
    var $vcetnePodrizenychRozdeleni;
    /** @var  ComboBoxes    Vyber rozdeleni 1 (podle popisu a kodu) */
    var $comboBoxesRozdeleni;
    /** @var  string        Kod vybraneho rozdeleni - nastavi se v load */
    var $kodRozdeleni;

    var $f_showrozdeleni2;  // priznak, jestli ma zobrazit vyber rozdeleni
    var $f_typrozdeleni2;
    var $vlastniTypRozdeleni2;
    var $jenVlastniRozdeleni2;
    var $f_filtrrozdeleni2;
    var $pouzitPopisTypuRozdeleni2;
    var $popisTypuRozdeleni2;
    var $showVcetnePodrizenychRozdeleni2;
    /** @var int            Hodnota pro select "vcetne podrizenych" 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene. Po load() se nastavi na skutecnou hodnotu. */
    var $vcetnePodrizenychRozdeleni2;
    /** @var  ComboBoxes    Vyber rozdeleni 2 (podle popisu a kodu) */
    var $comboBoxesRozdeleni2;
    /** @var  string        Kod vybraneho rozdeleni 2 - nastavi se v load */
    var $kodRozdeleni2;

    //var $f_showosoba;      // priznak, jestli se maji zobrazit osoby - bud podle zadanych prav nebo podle predavaciho procesu
    var $showOsoby;
    /** @var string     kod predavaciho procesu, podle ktereho se vybira */
    var $kodProcesu;
    var $typRozdeleniProces;
    var $f_prava;          // prava na osoby - pokud jsou zadana, nepouzije se predavaci proces!
    /** @var int        obdobi pro urceni prav (zarazeni do pristupove skupiny v zadanem obdobi) */
    var $osobyDatumOd;
    /** @var int        pokud je 0 a zaroven je zobrazen vyber obdobi, pouzije se datum dle obdobi */
    var $osobyDatumDo;
    // vyber osoby probiha bud podle prav nebo podle predavaciho procesu
    /** @var string     SQL pro filtrovani seznamu osob */
    var $osobyFiltr;
    var $vcetneExternich;
    var $povolitVsechyOsoby;
    var $downloadFotoOsobyUrl;
    /** @var  ComboBoxZamestnanec */
    var $comboOsoba;

    /** @var int        Id PP vybrane osoby - nastavi se v load */
    var $vybranaOsoba;
    /** @var string     RC vybrane osoby - nastavi se v load */
    var $vybranaOsobaRC;
    /** @var string     Osobni cislo vybraneho pp - nastavi se v load */
    var $vybranaOsobaOsobniCislo;
    /** @var int        Datum nastupu vybraneho pp - nastavi se v load */
    var $vybranaOsobaDatumNastupu;
    /** @var bool       Priznak, ze se maji zobrazit vsechny osoby */
    var $vsechnyOsoby = false;

    var $f_link;  // link, ktery se zobrazi v poslednim radku

    var $name;

    var $showMessages;

    var $skrytTlacitkoNastavit = false;

    var $f_classname;

    var $formClass = 'form ronportalfilterform position-relative';
    var $htmlId;


    /**
     * @param Database $database
     * @param string $action    Pro POST se pouzije jako action formulare. Pro GET se prida promenna menuid=$action
     * @param bool $useGet      Priznak, jestli ma formular pouzivat GET
     */
    public function __construct($database, $action, $useGet = false)
    {
        $this ->database = $database;

        $this->showObdobi = false;
        $this->showVariantni = false;
        $this->f_showrozdeleni = false;
        $this->f_showrozdeleni2 = false;
        $this->showOsoby = false;
        $this->f_link = '';
        $this->f_autosubmit = true;

        $this->name = 'adsappfilter';
        //$this -> f_formaction = '';  - zmeneno 13.3. - po vlozeni dochazky filterform nefungoval

        $this ->useGet = $useGet;
        if ($this ->useGet) {
            if ($action) {
                $this ->addHiddenField('menuid', $action);
            }
        } else {
            $this->f_formaction = '?menuid=' . $action;
        }

        $this->f_classname = 'filtercontainer';

        $this->showMessages = true; // na konci zobrazi chybove hlasky errormanagera
    }


    /**
     * @param bool $value
     * @param string|string[] $action bud akce (menuid, napr. "dochazka") nebo asociativni pole parametru
     * @return RonPortalFilterForm
     * @throws Exception
     */
    public function setUseGet($value, $action)
    {
        // TODO: upravit useGet tak, aby se to dalo nastavit i dodatecne. Momentalne to funguje spravne jen primo v konstruktoru! Navic asi blbne vyber rozdeleni!
        if (!$value  &&  $this ->useGet) {
            throw new Exception("Chyba v RonPortalFilterForm! USE GET nelze vypnout!");
        }

        $this ->useGet = $value;
        if (is_array($action)) {
            foreach ($action as $key => $fieldValue) {
                $this ->addHiddenField($key, $fieldValue);
            }
        } else {
            if ($action) {
                $this->addHiddenField('menuid', $action);
            }
        }

        return $this;
    }


    /**
     * Nastavi akci pro POST
     * @param $akce
     * @return RonPortalFilterForm
     */
    public function nastavAkci($akce)
    {
        $this ->useGet = false;
        $this->f_formaction = $akce;

        return $this;
    }

    /**
     * @param string $menuId
     * @return RonPortalFilterForm
     */
    public function nastavAkciGet($menuId)
    {
        $this ->useGet = true;
        $this ->getMenuId = $menuId;
        return $this;
    }

    /**
     * Prida hidden pole, nebo nastavi jeho hodnotu
     * @param string $name
     * @param string $value
     * @return RonPortalFilterForm
     */
    public function addHiddenField($name, $value)
    {
        $this ->hiddenFields[$name] = $value;
        return $this;
    }


    //Metody pro nastaveni formulare (zapnuti/vypnuti jednotlivych filtru)

    /** Nastavi zobrazeni filtru na obdobi
     * @param bool $show zapnuti zobrazeni filtru na obdobi
     * @param string $aplikace kod aplikace (ADS, ...)
     * @param bool $jenOtevrene jen otevrena obdobi
     * @param bool $obdobiPovinne pokud je false je v nabidce i prazdny radek
     * @return RonPortalFilterForm
     */
    public function nastavObdobi($show, $aplikace, $jenOtevrene, $obdobiPovinne = true)
    {
        $this->showObdobi = $show;
        $this->obdobiAplikace = $aplikace;
        $this->jenOtevreneObdobi = $jenOtevrene;
        $this->obdobiPovinne = $obdobiPovinne;

        return $this;
    }


    /**
     * @param bool $show zapne zobrazeni vyberu variantniho obdobi
     * @return RonPortalFilterForm
     */
    public function nastavVariantni($show)
    {
        $this->showVariantni = $show;
        return $this;
    }


    /**
     * Zapne zobrazeni vyberu pro datum od, datum do
     * @param bool $show
     * @param bool $povinne
     * @param string $nadpis
     * @return RonPortalFilterForm
     */
    public function nastavDatumOdDo($show, $povinne, $nadpis = '')
    {
        $this->showDatumOdDo = $show;
        $this->datumOdDoPovinne = $povinne;
        $this->nadpisDatumOdDo = $nadpis;
        return $this;
    }


    /**
     * Zapne zobrazeni datumu
     * @param $show
     * @param int $minimalniDatum
     * @param int $maximalniDatum
     * @return RonPortalFilterForm
     */
    public function nastavDatum($show, $minimalniDatum = null, $maximalniDatum = null)
    {
        $this ->showDatum = $show;
        $this ->minDatum = $minimalniDatum;
        $this ->maxDatum = $maximalniDatum;
        return $this;
    }


    /**
     * Zobrazi vyber rozdeleni
     * @param bool $show
     * @param int $typRozdeleni
     * @param int $jenVlastni 0=vse, 1=jen vlastni, 2=podle prav (prava muzou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene
     * @param string $filtrRozdeleni podminka, ktera se doplni do SQL podminky
     * @param bool $pouzitPopisTypuRozdeleni jestli se zobrazi pred vyberem rozdeleni popis typu rozdeleni, nebo popis zadany v nastaveni
     * @return RonPortalFilterForm
     */
    public function nastavRozdeleni($show, $typRozdeleni = 0, $jenVlastni = 0, $filtrRozdeleni = '', $pouzitPopisTypuRozdeleni = false)
    {
        $this->f_showrozdeleni = $show;

        // pokud neni zadan typ rozdeleni, tak pouzije vychozi
        if ($typRozdeleni) {
            $this->f_typrozdeleni = $typRozdeleni;
            $this->jenVlastniRozdeleni = $jenVlastni;
            $this ->vlastniTypRozdeleni = true;
        } else {
            $this->f_typrozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
            $this->jenVlastniRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);
            $this ->vlastniTypRozdeleni = false;
        }

        $this->filtrRozdeleni = $filtrRozdeleni;
        $this ->pouzitPopisTypuRozdeleni = $pouzitPopisTypuRozdeleni;
        $this->showVcetnePodrizenychRozdeleni = true;
        $this->vcetnePodrizenychRozdeleni = null;

        return $this;
    }


    /**
     * @param bool $show
     * @param int|null $typRozdeleni
     * @param bool $jenVlastni
     * @param string $filtrRozdeleni
     * @param bool $pouzitPopisTypuRozdeleni jestli se zobrazi pred vyberem rozdeleni popis typu rozdeleni, nebo popis zadany v nastaveni
     * @return RonPortalFilterForm
     */
    public function nastavRozdeleni2($show, $typRozdeleni = null, $jenVlastni = false, $filtrRozdeleni = '', $pouzitPopisTypuRozdeleni = false)
    {
        $this->f_showrozdeleni2 = $show;

        if ($typRozdeleni) {
            $this->f_typrozdeleni2 = $typRozdeleni;
            $this->jenVlastniRozdeleni2 = $jenVlastni;
            $this ->vlastniTypRozdeleni2 = true;
        } else {
            $this->f_typrozdeleni2 = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP);
            $this->jenVlastniRozdeleni2 = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_JEN_VLASTNI);
            $this ->vlastniTypRozdeleni2 = false;
        }

        $this->f_filtrrozdeleni2 = $filtrRozdeleni;
        $this ->pouzitPopisTypuRozdeleni2 = $pouzitPopisTypuRozdeleni;
        $this->showVcetnePodrizenychRozdeleni2 = true;
        $this->vcetnePodrizenychRozdeleni2 = null;

        return $this;
    }



    /**
     * jestli se maji zobrazit osoby a podle jakych prav a v jakem obdobi
     * pokud neni obdobi zadano, tak se pouzije aktualne vybrane obdobi
     * @param bool $show zda se ma zobrazit vyber osoby
     * @param PravaNaOsoby|PravaNaOsoby[] $prava prava na zobrazeni osoby
     * @param int $datumOd      Datum pro kontrolu prav - pokud je 0, urci se automaticky podle obdobi
     * @param int $datumDo      -//-
     * @param bool $vcetneExternich vcetne externich osob
     * @return RonPortalFilterForm
     */
    public function nastavOsoby($show, $prava = null, $datumOd = 0, $datumDo = 0, $vcetneExternich = false)
    {
        $this->showOsoby = $show;
        $this->f_showosoba = $show; // pro kompatibilitu stareho kodu
        $this->f_prava = $prava;
        $this->osobyDatumOd = $datumOd;
        $this->osobyDatumDo = $datumDo;

        $this->vcetneExternich = $vcetneExternich;

        $this->kodProcesu = null;
        $this->typRozdeleniProces = null;

        return $this;
    }


    /**
     * zobrazeni osob podle predavaciho procesu
     * @param bool $show
     * @param string $kodProcesu
     * @param int $typRozdeleni
     * @return RonPortalFilterForm
     */
    public function nastavOsobyDleProcesu($show, $kodProcesu, $typRozdeleni)
    {
        $this->showOsoby = $show;
        $this->f_showosoba = $show; // pro kompatibilitu stareho kodu
        $this->kodProcesu = $kodProcesu;
        $this->typRozdeleniProces = $typRozdeleni;

        $this->f_prava = null;

        return $this;
    }

    /*
      function NastavUrovenSchvaleni($show)
      {
        $this -> f_showurovenschvaleni = $show;
      }
    */

    public function nastavLink($link)
    {
        $this->f_link = $link;
        return $this;
    }


    /**
     * Provede zakladni nastaveni formulare podle nastaveni Portalu
     * @param $aplikaceObdobi
     * @param $pravaNaOsoby
     * @param $vcetneExternich
     * @return RonPortalFilterForm
     */
    public function nastavFormular($aplikaceObdobi, $pravaNaOsoby, $vcetneExternich)
    {
        if ($aplikaceObdobi) {
            $this->nastavObdobi(true, $aplikaceObdobi, false); // jenotevrene = false
        }

        $prava = Prava::vratPrava($pravaNaOsoby);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->nastavRozdeleni(true);
            $this ->f_pravarozdeleni = $prava;
            $this ->vcetnePodrizenychRozdeleni = 1;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->nastavRozdeleni2(true);
                $this->vcetnePodrizenychRozdeleni2 = 1;
            }
        }

        $this ->nastavOsoby(true, $prava, 0, 0, $vcetneExternich); // datum od-do podle obdobi
        return $this;
    }

    /**
     * Nastavi filtry pro personalistiku (zatim jen vyber osob) dle nastaveni (pokud je nastaveno zobrazeni pracovnich pomeru do budoucna)
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     */
    public function nastavPersonalistiku($prava)
    {
        $datumOd = 0;
        $datumDo = 0;

        $pocetMesicu = RonPortalSettings::getIntValue('person_pocet_mesicu_dopredu');

        if ($pocetMesicu) {
            $datumOd = DateUtils::today();
            $datumDo = DateUtils::incMonth($datumOd, $pocetMesicu);
        }

        $this->nastavOsoby(true, $prava, $datumOd, $datumDo);
    }

    /**
     * Metody pro nacteni dat zadanych uzivatelem (LOAD)
     * @throws Exception
     */
    public function loadObdobi() {
        if (isset($_POST['adsfilterform']) && !$this->obdobiPovinne) {
            $this->jenObdobi = isset($_POST['jenobdobi']);
            RonPortalSession::setJenObdobi($this->jenObdobi);
        } else if (isset($_GET['obdobi'])  && !$this->obdobiPovinne) {
            $this->jenObdobi = isset($_GET['jenobdobi']) && $_GET['jenobdobi'];
            RonPortalSession::setJenObdobi($this->jenObdobi);
        } else {
            $this ->jenObdobi = RonPortalSession::jenObdobi();
        }

        $obdobi = RonPortalSession::obdobi();

        try {
            if (isset($_POST['obdobi'])) {
                $obdobi = DateUtils::urlStringToDate($_POST['obdobi'], true);  //(int)$_POST['obdobi'];
            }
            else if (isset($_GET['obdobi'])) {
                $obdobi = DateUtils::urlStringToDate($_GET['obdobi'], true); // (int)$_GET['obdobi'];
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        $this ->comboObdobi = new ComboBoxObdobi('obdobi');
        $this ->comboObdobi ->className = 'form-input adsfilterforminput_full';
        $this ->comboObdobi ->htmlParams = [
            'id' => 'adsfilterform_obdobi',
            'data-ron-autosubmit' => $this ->f_autosubmit ? '1' : '0'];
        $this ->comboObdobi ->loadObdobi($this ->database, $this->obdobiAplikace, false, $this->jenOtevreneObdobi, false, $obdobi); // $variantni = false, $emptyRow = false

        $this ->comboObdobi ->nastavSession();

        $this ->vybraneObdobiOd = $this ->comboObdobi ->pocatekObdobi;
        $this ->vybraneObdobiDo = $this ->comboObdobi ->konecObdobi;
    }


    /**
     * @throws Exception
     */
    public function loadVariantni()
    {
        //HttpResponse::setCookieAndSession('ads_variantni', 'variantni', true, false); - bylo puvodne v appcontroller.LoadCookies()

        $obdobi = RonPortalSession::variantniObdobi();

        try {
            if (isset($_POST['variantni'])) {
                $obdobi = DateUtils::urlStringToDate($_POST['variantni'], true);
            } else if (isset($_GET['variantni'])) {
                $obdobi = DateUtils::urlStringToDate($_GET['variantni'], true);
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        $this ->comboVariantni = new ComboBoxObdobi('variantni');
        $this ->comboVariantni ->className = 'form-input adsfilterforminput_full';
        $this ->comboVariantni ->htmlParams = ['id' => 'adsfilterform_variabilni'];
        if ($this ->f_autosubmit) {
            $this->comboVariantni->htmlParams['data-ron-autosubmit'] = '1';
        }
        $this ->comboVariantni ->loadObdobi($this ->database, '', true, false, false, $obdobi); // $variantni=true, $pouzeNeuzavrene=false, $emptyRow=false
        $this ->comboVariantni ->nastavSessionVariantni();

        $this ->vybraneObdobiOd = $this ->comboVariantni ->pocatekObdobi;
        $this ->vybraneObdobiDo = $this ->comboVariantni ->konecObdobi;
    }


    /**
     * @throws Exception
     */
    public function loadDatumOdDo()
    {
        $this ->filtrovatDatum = $this ->datumOdDoPovinne || RonPortalSession::getFiltrovatDatum();

        if (isset($_POST['adsfilterform'])) {
            $this ->filtrovatDatum = $this ->datumOdDoPovinne || isset($_POST['jendatumoddo']);

            $this ->datumOd =  (isset($_POST['datumod'])  &&  $_POST['datumod'])
                ? DateUtils::dateStringToTimestamp($_POST['datumod'])
                : null;

            $this ->datumDo = (isset($_POST['datumdo'])  &&  $_POST['datumdo'])
                ? DateUtils::dateStringToTimestamp($_POST['datumdo'])
                : null;

            RonPortalSession::setFiltrovatDatum(isset($_POST['jendatumoddo']), $this ->datumOd, $this ->datumDo);
        } else if (isset($_GET['datumod'])) {
            $this ->filtrovatDatum = $this ->datumOdDoPovinne || (isset($_GET['jendatumoddo']) && $_GET['jendatumoddo']);

            $this ->datumOd =  (isset($_GET['datumod'])  &&  $_GET['datumod'])
                ? DateUtils::dateStringToTimestamp($_GET['datumod'])
                : null;

            $this ->datumDo = (isset($_GET['datumdo'])  &&  $_GET['datumdo'])
                ? DateUtils::dateStringToTimestamp($_GET['datumdo'])
                : null;

            RonPortalSession::setFiltrovatDatum(isset($_GET['jendatumoddo']) && $_GET['jendatumoddo'], $this ->datumOd, $this ->datumDo);
        } else {
            $this->datumOd = RonPortalSession::getDatumOd();
            $this->datumDo = RonPortalSession::getDatumDo();
        }


        if (!$this ->datumOd) {
            $this ->datumOd = DateUtils::firstDayOfMonth(time());
            $this ->datumDo = DateUtils::lastDayOfMonth(time());

            RonPortalSession::setFiltrovatDatum($this ->filtrovatDatum,
                $this ->datumOd,
                $this ->datumDo );
        }
    }


    public function loadDatum()
    {
        try {
            $this ->datum = isset($_POST['datum'])
                ? DateUtils::dateStringToTimestamp($_POST['datum'])
                : (isset($_GET['datum'])
                    ? DateUtils::dateStringToTimestamp($_GET['datum'])
                    : DateUtils::today());
        } catch (Exception $exception) {
            $this ->datum = DateUtils::today();
        }

        if (isset($this ->maxDatum) && $this ->datum > $this ->maxDatum) {
            $this ->datum = $this ->maxDatum;
        }

        if (isset($this ->minDatum) && $this ->datum < $this ->minDatum) {
            $this ->datum = $this ->minDatum;
        }
    }


    public function nastavVybraneObdobiProPrava()
    {
        if ($this->showObdobi) {
            $datumOd = RonPortalSession::obdobi();
            $datumDo = RonPortalSession::konecObdobi();
        } else if ($this->showVariantni) {
            $datumOd = RonPortalSession::variantniObdobi();
            $datumDo = RonPortalSession::variantniObdobiKonec();
        } else if ($this->showDatumOdDo && ($this->datumOdDoPovinne || $this ->filtrovatDatum)) {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        } else if ($this->osobyDatumOd) {
            $datumOd = $this->osobyDatumOd;
            $datumDo = $this->osobyDatumDo;
        } else {
            $datumOd = time();
            $datumDo = time();
        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);
    }


    /**
     * @param int $podrizena    0=bez podrizenych, 1=vcetne podrizenych, 2=vcetne primo podrizenych
     * @return string
     */
    public function sestavFiltrRozdeleni($podrizena)
    {
        // zjisti datum platnosti rozdeleni
        if ($this->showObdobi) {
            $datumOd = RonPortalSession::obdobi();
            $datumDo = RonPortalSession::konecObdobi();
        } else if ($this->showVariantni) {
            $datumOd = RonPortalSession::variantniObdobi();
            $datumDo = RonPortalSession::variantniObdobiKonec();
        } else if ($this->showDatumOdDo && ($this->datumOdDoPovinne || $this ->filtrovatDatum)) {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        } else {
            $datumOd = time();
            $datumDo = time();
        }

        if ($podrizena == 1) {
            // vcetne podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                '    AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene' .
                '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene' .
                "    AND RozdeleniOsob.RC = '" . $this->database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else if ($podrizena == 2) {
            // vcetne primo podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, Rozdeleni R2' .
                ' WHERE R2.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '    AND R2.Kod = RozdeleniOsob.Rozdeleni' .
                '    AND R2.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '    AND (R2.Kod = Rozdeleni.Kod OR R2.Kod = Rozdeleni.Rodic)' .
                "    AND RozdeleniOsob.RC = '" . $this->database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this->database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else {
            // jen vlastni`
            return
                'EXISTS (SELECT * FROM RozdeleniOsob ' .
                '  WHERE Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
                '    AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                "    AND RozdeleniOsob.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        }
    }


    public function filtrRozdeleniPodlePrav($typRozdeleni)
    {
        if (!$this->f_pravarozdeleni) {
            return '';
        }

        if (!$this->f_pravarozdeleni->typRozdeleni) {
            // neni nastaven typ rozdeleni
            return '';
        }

        if ($this->f_pravarozdeleni->typRozdeleni != $typRozdeleni) {
            // filtruje se podle jineho typu rozdeleni nez prava
            return '';
        }

        switch ($this->f_pravarozdeleni->prava) {
            case 10 : // vlastni rozdeleni
            case 11 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni(0); // podrizena = 0 (bez podrizenych)

            case 12 : // vlastni a podrizena
            case 13 : //   -//- krome sebe
            case 14 : // podrizena
            case 15 : // podrizena a sebe
                return $this->sestavFiltrRozdeleni(1); // (vcetne podrizenych)

            case 22 : // vlastni a primo podrizena
            case 23 : //   -//- krome sebe
            case 24 : // primo podrizena
            case 25 : //   -//- a sebe
            case 26 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni(2); // vcetne primo podrizenych
        }

        return '';
    }



    public function setUserSettingsItem($name, $htmlName, $post, $get)
    {
        if ($post  &&  isset($_POST[$htmlName])) {
            if (User::isAuthenticated()) {
                NastaveniUzivatele::setUserSettingsItem($this->database, $name, $_POST[$htmlName]);
            }
            $_SESSION[$name] = $_POST[$htmlName];
            return $_POST[$htmlName];
        }

        if ($get  &&  isset($_GET[$htmlName])) {
            if (User::isAuthenticated()) {
                NastaveniUzivatele::setUserSettingsItem($this->database, $name, $_GET[$htmlName]);
            }
            $_SESSION[$name] = $_GET[$htmlName];
            return $_GET[$htmlName];
        }

        if (!User::isAuthenticated()) {
            return null;
        }

        if (!isset($_SESSION[$name])) {
            $_SESSION[$name] = NastaveniUzivatele::getUserSettingsItem($this->database, $name);
        }

        return $_SESSION[$name];
    }


    public function setUserSettingsItemCheckBox($name, $htmlName, $htmlNameSubmit, $post, $get, $defaultValue = null)
    {
        $newValue = null;

        if ($post  &&  isset($_POST[$htmlNameSubmit])) {
            NastaveniUzivatele::setUserSettingsItem($this ->database, $name, isset($_POST[$htmlName]));
            $newValue = isset($_POST[$htmlName]);
        } else if ($get  &&  isset($_GET[$htmlNameSubmit])) {
            NastaveniUzivatele::setUserSettingsItem($this->database, $name, isset($_GET[$htmlName]));
            $newValue = isset($_GET[$htmlName]);
        } else if (isset($_SESSION[$name])) {
            return $_SESSION[$name];
        } else {
            $newValue = NastaveniUzivatele::getUserSettingsItem($this ->database, $name, $defaultValue);
            if (!isset($newValue)) {
                $newValue = $defaultValue;
            }
        }

        if (isset($newValue)) {
            $_SESSION[$name] = $newValue;
        }

        return $newValue;
    }


    /**
     * @param $typ
     * @return bool
     * @throws Exception
     */
    public function loadPopisTypuRozdeleni($typ)
    {
        $typ = TypRozdeleni::getTypRozdeleni($this ->database, $typ);
        return $typ ->popis;
    }


    /**
     * @param Option $option1
     * @param Option $option2
     * @return int
     */
    protected static function compareOptionsRozdeleni($option1, $option2)
    {
        if (!$option1) {
            return $option2 ? -1 : 0;
        }

        if (!$option2) return 1;

        if ($option1 ->key < $option2 ->key) return -1;
        if ($option1 ->key > $option2 ->key) return 1;

        return 0;
    }

    /**
     * @param string $htmlKod
     * @param int $typRozdeleni
     * @param bool $jenVlastni
     * @param $filtr
     * @param $value
     * @return ComboBoxes
     * @throws Exception
     */
    protected function loadDataRozdeleni($htmlKod, $typRozdeleni, $jenVlastni, $filtr, $value)
    {
        if ($this->showObdobi || $this->showVariantni) {
            $datumOd = $this->showVariantni ? RonPortalSession::variantniObdobi() : RonPortalSession::obdobi();
            $datumDo = $this->showVariantni ? RonPortalSession::variantniObdobiKonec() : RonPortalSession::konecObdobi();
        } else if ($this->showDatumOdDo && ($this->datumOdDoPovinne || RonPortalSession::getFiltrovatDatum())) {
            $datumOd = RonPortalSession::getDatumOd();
            $datumDo = RonPortalSession::getDatumDo();
        } else if ($this->showDatum) {
            $datumOd = $this ->datum;
            $datumDo = $datumOd;
        } else {
            $datumOd = time();
            $datumDo = null;
        }

        if (($jenVlastni == 2 || $jenVlastni == 3)  &&  $this ->f_pravarozdeleni) {
            $sql = $this ->f_pravarozdeleni ->sqlRozdeleni($this ->database,
                'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass',
                (int)$typRozdeleni, $datumOd, $datumDo, $jenVlastni == 2);
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . (int)$typRozdeleni .
                ' AND (DatumZruseni >= ' . $this ->database ->dateToString($datumOd) . ' OR DatumZruseni IS NULL) ';

            if ($datumDo)
                $sql .=
                    ' AND DatumVytvoreni <= ' . $this ->database ->dateToString($datumDo);

            $filtrRozdeleni = ($jenVlastni == 1)
                ? $this->sestavFiltrRozdeleni(1)  // vcetne podrizenych
                : $this->filtrRozdeleniPodlePrav($typRozdeleni);

            if ($filtrRozdeleni) {
                $sql .= ' AND (' . $filtrRozdeleni . ')';
            }
        }

        if ($filtr) {
            if ($sql instanceof SqlStatement) {
                $sql ->append(' AND (' . $filtr . ')');
            } else {
                $sql .= ' AND (' . $filtr . ')';
            }
        }

        if ($this ->useGet  &&  isset($_GET[$htmlKod])) {
            // pokud pouziva GET, tak povoli v URL vybrat i rozdeleni, do ktereho vybrana osoba nepatri
            // (jinak by neslo prepnout rozdeleni)
            // POZOR! V URL pak muze byt jina osoba nez skutecne nactena
        } else if (isset($_GET['osoba'])) {
            if (isset($value) && $value !== '' ) {
                // pokud je nastavena osoba v URL adrese, tak zkontroluje, jestli tato osoba patri do aktualne vybraneho rozdeleni
                // a pokud ne, tak rozdeleni vypne - je to nutne, aby fungovaly napr. odkazy v emailech
                if (!Rozdeleni::jeVRozdeleni($this->database, null, $_GET['osoba'], $typRozdeleni, $value, 3, $datumOd, $datumDo)) { // 3 = vcetne podrizenych
                    $value = '';
                }
            }
        }

        // vyber rozdeleni podle popisu
        $combo1 = new ComboBox($htmlKod);
        $combo1->className = 'form-input adsfilterforminput_popis ronselect';
        $combo1->cssClassIndex = 2;
        $combo1->showCode = true;
        $combo1->emptyRow = TextManager::_('_vse_');
        $sqlPopis = $sql instanceof SqlStatement
            ? new SqlStatement($sql ->sqlText . ' ORDER BY 2', $sql ->parameters)
            : $sql . ' ORDER BY Rozdeleni.Popis';
        $combo1->loadData($this ->database, $sqlPopis, $value);
        $combo1->htmlParams =
            "data-ron-select2='#ronportalfilter_{$htmlKod}_kod'" .
            " data-ronselect-class='ronselect adsfilterforminput_popis'" .
            ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '');
        //$combo1->render(); // 0, "data-ron-select2='{$htmlkod}_kod'" . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') ); // .' style="width:100%;margin:0px" '

//        if (isset($combo1->f_selectedvalue)) {
//            $result = $combo1->f_selectedvalue;
//        }
        $options2 = [];
        foreach ($combo1 ->data as $option) {
            $option->params['data-popis'] = $option ->value;
            $options2[] = $option;
        }

        usort($options2, 'RonPortal\Controls\RonPortalFilterForm::compareOptionsRozdeleni');


        // vyber kodu rozdeleni
        $combo2 = new ComboBox(null); // $htmlKod . '_kod'
        $combo2->className = 'form-input adsfilterforminput_kod desktop-only ronselect';
        $combo2->cssClassIndex = 2;
        $combo2->captionIndex = 0; // kod - kod
        $combo2->emptyRow = TextManager::_('_vse_');
//        $sqlKod = $sql instanceof SqlStatement
//            ? new SqlStatement($sql ->sqlText . ' ORDER BY Rozdeleni.Kod', $sql ->parameters)
//            : $sql . ' ORDER BY Rozdeleni.Kod';
//        $combo2->loadData($this ->database, $sqlKod, $value);

        $combo2 ->valueFieldName = 'key';
        $combo2 ->keyFieldName = 'key';
        $combo2 ->titleFieldName = 'value';
        $combo2 ->cssClassFieldName = 'cssClass';

        $combo2 ->addOptions($options2);

        $combo2->setSelectedOption($value);

        $combo2->htmlParams =
            "data-ron-select2='$htmlKod'" . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') .
            " data-ronselect-class='ronselect adsfilterforminput_kod'" .
            " data-columns='value;popis'" .
            " id='ronportalfilter_{$htmlKod}_kod'";
        //$combo2->render(); // Show(0, "data-ron-select2='$htmlkod'" . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') ); // .' style="width:100%;margin:0px" '

        return new ComboBoxes($combo1, $combo2);
    }


    /**
     * @throws Exception
     */
    public function loadRozdeleni()
    {
        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->f_typrozdeleni : null;

        // nacte rozdeleni zadane uzivatelem
        $this ->setUserSettingsItem('ads_rozdeleni' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''), 'rozdeleni', true, true);
        $this ->vcetnePodrizenychRozdeleni = (int)$this ->setUserSettingsItem('ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
            'rozdeleni_vcetne_podrizenych', true, true);
//        $this ->vcetnePodrizenychRozdeleni = $this ->setUserSettingsItemCheckBox(
//            'ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
//            'rozdeleni_vcetne_podrizenych', 'rozdeleni',
//            true, true, $this->vcetnePodrizenychRozdeleni);

        if ($this->pouzitPopisTypuRozdeleni) {
            $this->popisTypuRozdeleni = $this->loadPopisTypuRozdeleni($this->f_typrozdeleni);
        }
;
        $this ->comboBoxesRozdeleni = $this ->loadDataRozdeleni('rozdeleni',
            $this->f_typrozdeleni, $this->jenVlastniRozdeleni, $this->filtrRozdeleni,
            RonPortalSession::rozdeleni($typRozdeleniSuffix));

        if (isset($this ->comboBoxesRozdeleni ->combo1 ->selectedOption)) {
            $this ->kodRozdeleni = $this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key;
            RonPortalSession::setRozdeleni($this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
        } else {
            $this ->kodRozdeleni = null;
        }

        //RonPortalSession::setRozdeleniVcetnePodrizenych(isset($_POST['rozdeleni_vcetne_podrizenych']), $typRozdeleniSuffix);

        //if (isset($noverozdeleni)) S_SetRozdeleni($noverozdeleni);
    }


    /**
     * @throws Exception
     */
    public function loadRozdeleni2()
    {
        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->f_typrozdeleni2 : null;

        $this->setUserSettingsItem('ads_rozdeleni2' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''), 'rozdeleni2', true, true);
        $this ->vcetnePodrizenychRozdeleni2 = (int)$this ->setUserSettingsItem('ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
            'rozdeleni2_vcetne_podrizenych', true, true);
//        $this ->vcetnePodrizenychRozdeleni2 = $this ->setUserSettingsItemCheckBox(
//            'ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
//            'rozdeleni2_vcetne_podrizenych', 'rozdeleni2',
//            true, true, $this->vcetnePodrizenychRozdeleni2);

        if ($this->pouzitPopisTypuRozdeleni2) {
            $this->popisTypuRozdeleni2 = $this->loadPopisTypuRozdeleni($this->f_typrozdeleni2);
        }
        $this ->comboBoxesRozdeleni2 = $this ->loadDataRozdeleni('rozdeleni2',
            $this->f_typrozdeleni2, $this->jenVlastniRozdeleni2,
            $this->f_filtrrozdeleni2, RonPortalSession::rozdeleni2($typRozdeleniSuffix));

        if (isset($this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption)) {
            $this ->kodRozdeleni2 = $this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption ->key;
            RonPortalSession::setRozdeleni2($this ->comboBoxesRozdeleni2 ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
        } else {
            $this ->kodRozdeleni2 = null;
        }

//        RonPortalSession::setRozdeleni2VcetnePodrizenych(isset($_POST['rozdeleni2_vcetne_podrizenych']), $typRozdeleniSuffix);
    }


    private function createComboOsoba()
    {
        $this ->comboOsoba = new ComboBoxZamestnanec('osoba');
        $this ->comboOsoba ->className = 'ronselect form-input adsfilterforminput_full';
        $this ->comboOsoba ->htmlParams['data-ron-autosubmit'] = $this ->f_autosubmit ? '1' : '0';
        $this ->comboOsoba ->htmlParams['id'] = 'adsfilterform_osoba';
        $this ->comboOsoba ->htmlParams['data-ronselect-class'] = 'ronselect adsfilterforminput_full';
        //    'data-columns' => 'text'
    }

    public function loadOsobyPodleProcesu()
    {
        $this ->createComboOsoba();
        $this ->comboOsoba ->loadDataPredavaciProces($this ->database,
            $this->kodProcesu, $this->typRozdeleniProces,
            false, RonPortalSession::vybranaOsoba()); // emptyrow = false

        $this ->comboOsoba ->nastavOsobuDoSession();
        $this ->vybranaOsoba = $this ->comboOsoba ->idPP;
        $this ->vybranaOsobaRC = $this ->comboOsoba ->rc;
        $this ->vybranaOsobaOsobniCislo = $this ->comboOsoba ->osobniCislo;
        $this ->vybranaOsobaDatumNastupu = $this ->comboOsoba ->datumNastupu;

        if (!$this ->vybranaOsobaRC || !$this ->vybranaOsobaOsobniCislo) {
            MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
        }
    }


    public function loadOsobyPodlePrav()
    {
        if ($this ->povolitVsechyOsoby) {
            $this->vsechnyOsoby = RonPortalSession::vsechnyOsoby();
        }

        if (!Prava::jsouNejakePrava($this->f_prava)) {
            RonPortalSession::smazudajezamestnance();

            MessageManager::addErrorMessage( TextManager::_('Nemate_pravo') );
        }

        $datumOd = $this->osobyDatumOd;
        $datumDo = $this->osobyDatumDo;

        if ($this->showObdobi) {
            if (!$datumOd) {
                $datumOd = RonPortalSession::obdobi();
            }
            if (!$datumDo) {
                $datumDo = RonPortalSession::konecObdobi();
            }
        } else if ($this->showVariantni) {
            if (!$datumOd) {
                $datumOd = RonPortalSession::variantniObdobi();
            }
            if (!$datumDo) {
                $datumDo = RonPortalSession::variantniObdobiKonec();
            }
        } else if ($this ->showDatum) {
            if (!$datumOd) {
                $datumOd = $this ->datum;
            }
        } else {
            if (!$datumOd) {
                $datumOd = time();
            }
        }

        if (!$datumDo) {
            $datumDo = $datumOd;
        }

        // pokud je zaroven zobrazeno rozdeleni, tak osoby profiltruje
        $this ->createComboOsoba();

        if ($this->f_showrozdeleni) {
            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->f_typrozdeleni : null;
            //$this ->comboOsoba->nastavFiltrRozdeleni($this->f_typrozdeleni, RonPortalSession::rozdeleni($typRozdeleniSuffix), RonPortalSession::rozdeleniVcetnePodrizenych($typRozdeleniSuffix));
            $this ->comboOsoba->nastavFiltrRozdeleni($this->f_typrozdeleni, $this ->kodRozdeleni, $this ->vcetnePodrizenychRozdeleni);
        }

        if ($this->f_showrozdeleni2) {
            //$typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->f_typrozdeleni2 : null;
            //$this ->comboOsoba->nastavFiltrRozdeleni2($this->f_typrozdeleni2, RonPortalSession::rozdeleni2($typRozdeleniSuffix), RonPortalSession::rozdeleni2VcetnePodrizenych($typRozdeleniSuffix));
            $this ->comboOsoba->nastavFiltrRozdeleni2($this->f_typrozdeleni2, $this ->kodRozdeleni2, $this ->vcetnePodrizenychRozdeleni2);

        }
        $this ->comboOsoba->filtr = $this->osobyFiltr; // obecny filtr, napr. jen osoby, kterym se zpracovava dochazka

        if ($this ->povolitVsechyOsoby) {
            if (isset($_POST['osoba'])) {
                $this ->vsechnyOsoby = $_POST['osoba'] == '';
            } else if (isset($_GET['osoba'])) {
                $this ->vsechnyOsoby = $_GET['osoba'] == '';
            } else if ( !RonPortalSession::vybranaOsoba() ) {
                $this ->vsechnyOsoby = true;
            }

            RonPortalSession::setVsechnyOsoby($this ->vsechnyOsoby);
        }

        $selectedValue = ($this ->povolitVsechyOsoby  &&  $this ->vsechnyOsoby)
            ? ''
            : RonPortalSession::vybranaOsoba();

        $this ->comboOsoba->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->f_prava,
            $this ->povolitVsechyOsoby ? TextManager::_('_vsichni_'): false,  // $emptyRow
            $selectedValue,
            $this->vcetneExternich); // vcetneExternich

        if ($this ->povolitVsechyOsoby) {
            if (!$this ->comboOsoba ->selectedOption  ||  !$this ->comboOsoba ->selectedOption ->key) {
                // Pokud jsou povoleny vsechny osoby a v selectu je vybrana volba "vsichni", tak nastavi setVsechnyOsoby
                // Je to potreba v situaci, kdy byla puvodne v session osoba, na kterou nema uzivatel pravo
                $this->vsechnyOsoby = true;
                RonPortalSession::setVsechnyOsoby(true);
            }
        }

        if (!$this ->povolitVsechyOsoby  ||  !$this ->vsechnyOsoby) {
            $this ->comboOsoba->nastavOsobuDoSession();
            $this ->vybranaOsoba = $this ->comboOsoba ->idPP;
            $this ->vybranaOsobaRC = $this ->comboOsoba ->rc;
            $this ->vybranaOsobaOsobniCislo = $this ->comboOsoba ->osobniCislo;
            $this ->vybranaOsobaDatumNastupu = $this ->comboOsoba ->datumNastupu;

            if (!$this ->vybranaOsobaRC || !$this ->vybranaOsobaOsobniCislo) {
                MessageManager::addErrorMessage( TextManager::_('Neni_pracovni_pomer') );
            }
        }
    }


    /**
     * @throws Exception
     */
    public function loadData()
    {
        Profiler::addItem("");
        $item = Profiler::addItem('RonPortalFilterForm::loadData ----------', 'APP');

        if ($this ->showObdobi) {
            $this ->loadObdobi();
        }
        if ($this ->showVariantni) {
            $this ->loadVariantni();
        }
        if ($this ->showDatumOdDo) {
            $this ->loadDatumOdDo();
        }
        if ($this ->showDatum) {
            $this ->loadDatum();
        }

        $this ->nastavVybraneObdobiProPrava();

        if ($this ->f_showrozdeleni) $this ->loadRozdeleni();
        if ($this ->f_showrozdeleni2) $this ->loadRozdeleni2();

        if ($this->showOsoby) {
            if ($this ->f_prava) {
                $this ->loadOsobyPodlePrav();
            } else {
                $this ->loadOsobyPodleProcesu();
            }
        }

        Profiler::endItem($item);
        Profiler::addItem("");
    }



    // --------------------------------------------------------------------------
    // Metody pro zobrazeni formulare (RENDER)

    /**
     * Zobrazi vyber obdobi (data uz musi byt nactena!)
     */
    public function renderObdobi()
    {
        echo '<div class="filterrow" id="filterrow_obdobi">';

        echo '<span class="filtercaption">';
        if ($this->obdobiPovinne) {
            echo TextManager::_('Obdobi') . ' ';
        } else {
            GuiUtils::renderCheckbox('jenobdobi', TextManager::_('Obdobi'), RonPortalSession::jenObdobi(), '', '', 1, true);
        }
        echo '</span>';

        $this ->comboObdobi ->render();

        // tlacitka pro posun na dalsi a predchozi obdobi
        echo '<span id="adsfilterform_obdobi_fast" class="desktop-only">';
        echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-submit="1" data-selector="#adsfilterform_obdobi"></i>';
        echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_obdobi"></i>';
        echo '</span>';

        // zobrazi zamek, pokud je obdobi uzavrene
        if ($this ->comboObdobi->uzavreno) {
            echo '<span class="inline-glyph fa glyph-lock"></span>';
        }

        echo "</div>\n";
    }


    /**
     * Zobrazi vyber variabilniho obdobi
     */
    public function renderVariantni()
    {
        echo '<DIV class="filterrow" id="filterrow_variabilni">';
        echo '<SPAN class="filtercaption">' . TextManager::_('Variabilni_obdobi') . '</SPAN>';

        $this ->comboVariantni ->render();

        // tlacitka pro posun na dalsi a predchozi obdobi
        echo '<span id="adsfilterform_variabilni_fast" class="desktop-only">';
        echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-submit="1" data-selector="#adsfilterform_variabilni"></i>';
        echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_variabilni"></i>';
        echo '</span>';

        echo "</DIV>\n";
    }


    /**
     * Umoznuje zobrazit dalsi obsah za vyber datumu od-do
     */
    public function renderDatumOdDoLinks()
    {
        echo $this ->datumOdDoLinks ?: '';
    }


    /**
     * Zobrazeni filtru datum od - do
     */
    public function renderDatumOdDo()
    {
        $nadpis = $this->nadpisDatumOdDo ?: TextManager::_('Datum_od');

        echo '<div class="filterrow" id="filterrow_datum_od_do">';

        echo '<span class="filtercaption">';
        if ($this->datumOdDoPovinne) {
            echo $nadpis; // .' &nbsp;&nbsp;';
        } else {
            GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', 1, true);
        }
        echo '</span>';

        echo '<span style="white-space:nowrap;">';
        DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter

        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
        DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter
        echo '</span>';

        $this ->renderDatumOdDoLinks();

        echo "</div>\n";
    }



    /**
     * Umoznuje zobrazit dalsi obsah za vyber datumu. Standardne zobrazi odkaz "Dnes" pokud je vybrany jiny datum
     */
    public function renderDatumLinks()
    {
        if (isset($this ->datumLinks)) {
            echo $this ->datumLinks;
            return;
        }

        $today = DateUtils::today();
        $action = $this ->getAction ?: $this->f_formaction;

        if ($action
            && $this ->datum != $today
            &&  (!$this ->maxDatum  ||  $today <= $this ->maxDatum)
            &&  (!$this ->minDatum  ||  $today >= $this ->maxDatum) )
        {
            echo
                '<span class="margin-left-2">' .
                '<a href="#" class="setValue" data-submit="1" data-selector="#ronportalfilter_datum" data-value="' . StringUtils::format('date', $today) . '">' .
                TextManager::_('Dnes') . '
                </a>' .
                '</span>';
        }
    }



    /**
     * Zobrazeni filtru datum od - do
     */
    public function renderDatum()
    {
        echo '<div class="filterrow" id="filterrow_datum">';

        echo '<span class="filtercaption">';
        echo TextManager::_('Datum');
        echo "</span>";

        echo '<span style="white-space:nowrap;">';
        $valueStr = date('j.n.Y', $this->datum);
        $submitStr = $this->f_autosubmit ? "data-roncalendar-submit='1'" : '';
        $limitsStr =
            (isset($this ->maxDatum) ? 'data-roncalendar-maxdate="' . date('j.n.Y', $this ->maxDatum) . '"' : '') .
            '' .
            (isset($this ->minDatum) ? 'data-roncalendar-mindate="' . date('j.n.Y', $this ->minDatum) . '"' : '');
        echo "<input id='ronportalfilter_datum' type='text' class='inputDate' size='10' maxlength='10' name='datum' value='$valueStr' $submitStr $limitsStr>";
        echo "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='datum'></i>";
        echo '</span>';

        $this ->renderDatumLinks();

        echo "</div>";
    }


    public function renderRozdeleni()
    {
        if (!$this->f_typrozdeleni) return false;

        echo '<div class="filterrow" id="filterrow_rozdeleni">';
        echo '<span class="filtercaption">' . ($this->popisTypuRozdeleni ? StringUtils::htmlTextConvert($this->popisTypuRozdeleni) : TextManager::_(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_NADPIS))) . '</span>';

        $this ->comboBoxesRozdeleni ->combo1 ->render();
        $this ->comboBoxesRozdeleni ->combo2 ->render();

        if ($this->showVcetnePodrizenychRozdeleni) {
            $typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->f_typrozdeleni : null;

            $value = RonPortalSession::rozdeleniVcetnePodrizenych($typRozdeleniSuffix);

            echo
                '<select name="rozdeleni_vcetne_podrizenych"' . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') . '>' .
                '<option value="0"' . (!$value ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_vybrane') . '</option>' .
                '<option value="1"' . ($value == 1 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_vcetne_podrizenych') . '</option>' .
                '<option value="2"' . ($value == 2 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_podrizena') . '</option>' .
                '<option value="3"' . ($value == 3 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_primo_podrizena') . '</option>' .
                '</select>';

//            GuiUtils::renderCheckbox('rozdeleni_vcetne_podrizenych', \Ron\ronText('Rozdeleni_vcetne_podrizenych'),
//                RonPortalSession::rozdeleniVcetnePodrizenych($typRozdeleniSuffix),
//                '', ($this ->f_autosubmit ? 'clickAutoSubmit' : ''));
        }

        echo "</div>\n";

        return true;
    }


    public function renderRozdeleni2()
    {
        if (!$this->f_typrozdeleni2) return false;

        echo '<DIV class="filterrow" id="filterrow_rozdeleni2">';
        echo '<SPAN class="filtercaption">' . ($this->popisTypuRozdeleni2 ? StringUtils::htmlTextConvert($this->popisTypuRozdeleni2) : TextManager::_(RonPortalSettings::getValue(RonPortalSettings::ROZDELENI2_NADPIS))) . '</SPAN>';

        $this ->comboBoxesRozdeleni2 ->combo1 ->render();
        $this ->comboBoxesRozdeleni2 ->combo2 ->render();

        if ($this->showVcetnePodrizenychRozdeleni2) {
            $typRozdeleniSuffix = $this ->vlastniTypRozdeleni2 ? $this ->f_typrozdeleni2 : null;

            $value = RonPortalSession::rozdeleni2VcetnePodrizenych($typRozdeleniSuffix);

            echo
                '<select name="rozdeleni2_vcetne_podrizenych"' . ($this ->f_autosubmit ? ' data-ron-autosubmit="1"' : '') . '>' .
                '<option value="0"' . (!$value ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_vybrane') . '</option>' .
                '<option value="1"' . ($value == 1 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_vcetne_podrizenych') . '</option>' .
                '<option value="2"' . ($value == 2 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_podrizena') . '</option>' .
                '<option value="3"' . ($value == 3 ? " selected='selected'" : '') . '>' . TextManager::_('Rozdeleni_jen_primo_podrizena') . '</option>' .
                '</select>';

//            GuiUtils::renderCheckbox('rozdeleni2_vcetne_podrizenych', \Ron\ronText('Rozdeleni_vcetne_podrizenych'), RonPortalSession::rozdeleni2VcetnePodrizenych($typRozdeleniSuffix), '', ($this ->f_autosubmit ? 'clickAutoSubmit' : ''));
        }

        echo "</DIV>\n";

        return true;
    }


    public function renderOsoby()
    {
        echo '<DIV class="filterrow" id="filterrow_osoba">';
        echo '<SPAN class="filtercaption">' . TextManager::_('Osoba') . '</SPAN>';

        $this ->comboOsoba ->render();

        if ($this ->f_prava) {
            if ($this->comboOsoba->data && count($this->comboOsoba->data) > 1) {
                // tlacitka pro posun na dalsi a predchozi PP
                echo '<span id="adsfilterform_pp_fast" class="desktop-only">';
                echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
                echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
                echo '</span>';
            }
        }

        if ($this ->comboOsoba ->obsahujeVlastniPP  &&  $this ->comboOsoba ->rc != User::getRC()) {
            // zobrazi ikonu pro prechod na vlastni PP
            echo
                '<i id="vybratVlastniBtn" class="inline-glyph glyph-button fa glyph-own-data comboBoxMove"' .
                " data-value='{$this ->comboOsoba ->idVlastnihoPP}'" .
                ' title="' . TextManager::_('Vybrat_vlastni_zaznamy') .'"' .
                ' data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
        }

        if ($this ->povolitVsechyOsoby  &&  !$this ->vsechnyOsoby) {
            // zobrazi ikonu pro prechod na vsechny osoby
            echo
                '<i id="vybratVsechnyOsobyBtn" class="inline-glyph glyph-button fa glyph-all-persons comboBoxMove"' .
                " data-value=''" .
                ' title="' . TextManager::_('Vse') .'"' .
                ' data-submit="1" data-selector="#adsfilterform_osoba" data-trigger-change="1"></i>';
        }

        echo "</DIV>\n";
    }


    public function renderLink()
    {
        echo '<DIV class="filterrow" id="filterrow_link">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';
        echo $this->f_link;
        echo '</SPAN>';
        echo "</DIV>\n";

    }



    public function renderFilters()
    {
        if ($this->showObdobi) {
            $this->renderObdobi();
        }
        if ($this->showVariantni) {
            $this->renderVariantni();
        }
        if ($this->showDatumOdDo) {
            $this->renderDatumOdDo();
        }
        if ($this->showDatum) {
            $this->renderDatum();
        }

        if ($this->f_showrozdeleni) {
            $this->renderRozdeleni();
        }
        if ($this->f_showrozdeleni2) {
            $this->renderRozdeleni2();
        }

        if ($this->showOsoby) {
            $this->renderOsoby();
        }

        if ($this->f_link) {
            $this->renderLink();
        }
    }

    public function renderFoto()
    {
        if ($this ->showOsoby  &&  $this ->downloadFotoOsobyUrl  &&  !($this ->povolitVsechyOsoby && $this ->vsechnyOsoby)  &&  $this ->comboOsoba  &&  $this ->comboOsoba ->idFotografie) {
            echo '<div class="fotoDochazka ronFoto desktop-only"><img alt="Foto" src="' . $this ->downloadFotoOsobyUrl . $this ->comboOsoba ->idPP . '"></div>';
        }
    }

    /**
     * vraci SQL filtr ( EXISTS ... ) pro vybrane rozdeleni
     * @param $tabulka
     * @param int $vcetnePodrizenych    0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param int $datumod
     * @param int $datumdo
     * @return string
     */
    public function vratSQLFiltrRozdeleni($tabulka, $vcetnePodrizenych, $datumod = 0, $datumdo = 0)
    {
        if (!$this->f_showrozdeleni) return '';
        if (!$this->f_typrozdeleni) return '';
        if (RonPortalSession::rozdeleni() === null) return '';

        if (!$datumod) {
            if ($this->showObdobi) $datumod = RonPortalSession::obdobi();
            else if ($this->showVariantni) $datumod = RonPortalSession::variantniObdobi();
            else if ($this ->showDatumOdDo) $datumdo = $this ->datumOd;
            else $datumod = time();
        }

        if (!$datumdo) {
            if ($this->showObdobi) $datumdo = RonPortalSession::konecObdobi();
            else if ($this->showVariantni) $datumdo = RonPortalSession::variantniObdobiKonec();
            else if ($this ->showDatumOdDo) $datumdo = $this ->datumDo;
            else $datumdo = $datumod;
        }

        return RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, $tabulka,
            $this->f_typrozdeleni, RonPortalSession::rozdeleni(),
            $datumod, $datumdo, $vcetnePodrizenych);
    }


    /**
     */
    public function render()
    {
        if ($this ->header) {
            echo "<h1>{$this ->header}</h1>\n";
        }

        $actionStr = '';
        if ($this ->getAction) {
            $actionStr = ' action="' . $this->getAction . '"';
        }
        else if ($this->f_formaction) {
            $actionStr = ' action="' . $this->f_formaction . '"';
        }

        $attrs = GuiUtils::getAttributesHtml($this ->formAttributes);
        echo
            '<form method="' . ($this ->useGet ? 'GET' : 'POST') . '" name="' . $this->name . '"' .
            ' data-ron-autosubmit="' . ($this ->f_autosubmit ? '1' : '0' ) . '" ' .
            $attrs .
            $actionStr . '>';

        if ($this ->useGet) {
            if ($this ->getMenuId) {
                echo "<input type='hidden' name='menuid' value='{$this ->getMenuId}' />\n";
            }
        } else {
            echo '<input type="hidden" name="adsfilterform" value="1">';
        }

        foreach ($this ->hiddenFields as $fieldName => $fieldValue) {
            echo "<input type='hidden' name='$fieldName' value='$fieldValue' />\n";
        }


        $idStr = $this ->htmlId ? " id='{$this ->htmlId}'" : '';
        echo '<div class="' . $this ->formClass . '"><div class="form-content">';
        echo "<div class='{$this->f_classname}' {$idStr}>\n";

        echo '<div class="filtercontainercol">';
        // vnitrni tabulka - jednotlive filtry
        $this->renderFilters();
        echo '</div>';

        // zobrazi tlacitko nastavit - kdyby nefungoval javascript
        if (!$this ->skrytTlacitkoNastavit) {
            echo '<div class="filtercontainercol">';
            echo '<input type="submit" class="submitbutton" value="' . TextManager::_('Nastavit') . '">';
            echo '</div>';
        }

        $this->renderFoto();

        echo
            "</div>\n" .
            "</div>" .  // form-content
            "<i class='ronportalfilterform-submit-indicator d-none fa glyph-spinner fa-spin submit-indicator position-absolute position-top position-right margin-all-2'></i>" .
            "</div>\n"; // ronportalfilterform

        echo "</form>\n";

        if ($this ->showMessages) {
            MessageManager::getInstance() ->showMessages();
        }
    }
}
