<?php
/**
 * stanislav.elbl, 02.03.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\InputForm;
use Ron\DB\Database;
use RonPortal\RonPortalDBRecord;

class RonPortalForm extends InputForm
{
    /** @var string     Nazev zdroje pro DB LOG (implicitne nazev tridy) */
    var $dbLogSrcDescription = null;

    /** @var string     Nazev XML udalosti spustene pred zapisem zaznamu */
    var $xmlEventBeforePost;

    /** @var string     Nazev XML udalosti spustene po zapisu zaznamu */
    var $xmlEventAfterPost;

    /**
     * @param \Ron\DB\Database $database
     * @param string $formAction
     * @param bool $dialogMode
     */
    function __construct($database, $formAction = 'new', $dialogMode = false)
    {
        parent::__construct($database, $formAction, $dialogMode);
    }


    /**
     * Nastavi nazvy XML udalosti - pridava sufixy _BEFORE_POST a AFTER_POST
     * @param $name
     */
    function setXmlEventsName($name)
    {
        $this ->xmlEventBeforePost = $name . '_BEFORE_POST';
        $this ->xmlEventAfterPost = $name . '_AFTER_POST';
    }


    function getHtmlFormParams()
    {
        return
            parent::getHtmlFormParams() .
            ($this ->xmlEventBeforePost
                ? " data-xml-beforepost='{$this ->xmlEventBeforePost}'"
                : '') .
            ($this ->xmlEventAfterPost
                ? " data-xml-afterpost='{$this ->xmlEventAfterPost}'"
                : '')
            ;
    }


    /**
     * @param null $tableName
     * @return RonPortalDBRecord
     */
    function createDBRecord($tableName = null)
    {
        return new RonPortalDBRecord($this ->database,
            $tableName ?: $this ->sqlTableName,
            $this->dbLogSrcDescription ?: $this);
    }


    function beforeInsertRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventBeforePost) {
            $rec ->executeXmlEvents($this ->xmlEventBeforePost, 'insert');
        }

        return parent::beforeInsertRecord($rec);
    }

    function afterInsertRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventAfterPost) {
            $rec ->executeXmlEvents($this ->xmlEventAfterPost, 'insert');
        }

        return parent::afterInsertRecord($rec);
    }

    function beforeUpdateRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventBeforePost) {
            $rec ->executeXmlEvents($this ->xmlEventBeforePost, 'update');
        }

        return parent::beforeUpdateRecord($rec);
    }

    function afterUpdateRecord($rec)
    {
        if ($rec instanceof RonPortalDBRecord  &&  $this ->xmlEventAfterPost) {
            $rec ->executeXmlEvents($this ->xmlEventAfterPost, 'update');
        }

        return parent::afterUpdateRecord($rec);
    }


    /**
     * Nacte XML konfiguraci browseru se zadanym kodem
     * @param Database $database
     * @param string $code
     * @return string|null
     */
    public static function loadFormConfiguration($database, $code)
    {
        if (!$code) {
            return null;
        }

        $params = $database ->createParameters();
        $cursor = $database ->select(
            'SELECT ObsahXML FROM SablonaXMLFormulare WHERE Popis = ' . $params ->addParameter('config_' . $code),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Nacte konfiguraci formulare z XML sablony
     * @throws \Exception
     */
    function loadConfiguration()
    {
        if (!$this ->code) {
            return;
        }

        $xml = self::loadFormConfiguration($this ->database, $this ->code);
        if ($xml) {
            $this ->loadConfigurationXml($xml);
        }
    }


    function handleAction()
    {
        $this ->loadConfiguration();
        return parent::handleAction();
    }
}
