<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\Controls\BrowserField;
use RonPortal\RonPortalSettings;

/**
 * Pole pro zobrazeni kodu a/nebo popisu rozdeleni (podle nastaveni) - data musi byt v SQL dotazu browseru
 * Class RozdeleniField
 * @package Ron\RonPortal
 */
class RozdeleniBrowserField extends BrowserField
{
    var $indexKod;
    var $indexPopis;

    /**
     * @param Database $database
     * @param int $indexKod
     * @param int $indexPopis
     * @param string $fieldCode
     */
    function __construct($database, $indexKod, $indexPopis, $fieldCode = 'col-rozdeleni')
    {
        parent::__construct($database);

        $this->indexKod = $indexKod;
        $this->indexPopis = $indexPopis;
        $this->code = $fieldCode;

        $this->header = \Ron\ronText('Rozdeleni');
        $this->type = BrowserField::FIELD_TYPE_DEFAULT;
        $this->align = 'left';

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1) $this->sqlFieldName = 'Rozdeleni.Kod';
        else $this->sqlFieldName = 'Rozdeleni.Popis';

        $this->ordering = 1;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) ) {
            case 0 : // jen popis
                return $rowData[$this->indexPopis];

            case 1 : // jen kod
                return $rowData[$this->indexKod];

            case 2 : // kod i popis
                $result = $rowData[$this->indexPopis];
                if (isset($rowData[$this->indexKod])) $result .= ' (' . $rowData[$this->indexKod] . ')';

                return $result;
        }

        return '';
    }
}
