<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\BrowserField;
use Ron\DB\Database;

/**
 * Pole pro nacteni a zobrazeni rozdeleni osoby - vyhledava jen podle RC nebo podle prac. pomeru
 * Class RozdeleniOsobyField
 * @package RonPortal\Controls
 */
class RozdeleniOsobyBrowserField extends BrowserField
{
    var $typRozdeleni;

    var $zobrazitKod;
    var $zobrazitPopis;

    var $indexRC;       // index pole s rodnym cislem osoby (povinne)
    var $indexOsobniCislo; // index pole s OsobnimCislem
    var $indexDatumNastupu; // a datem nastupu (nepovinne)

    var $indexDatumOd;  // index pole s datumem od (nepovinne)
    var $indexDatumDo;  // index pole s datumem do (nepovinne)

    var $datumOd;       // obdobi prirazeni na rozdeleni
    var $datumDo;


    /**
     * @param Database $database
     * @param int $typRozdeleni
     */
    function __construct($database, $typRozdeleni)
    {
        parent::__construct($database);

        $this->database = $database;
        $this->typRozdeleni = $typRozdeleni;

        $this->indexRC = -1;
        $this->indexOsobniCislo = -1;
        $this->indexDatumNastupu = -1;

        $this->indexDatumOd = -1;
        $this->indexDatumDo = -1;

        $this->zobrazitKod = 1;
        $this->zobrazitPopis = 0;

        $this->datumOd = time();
        $this->datumDo = time();

        global $STR_Rozdeleni; // kompat
        $this->header = $STR_Rozdeleni ?: \Ron\ronText('Rozdeleni');
    }


    /**
     * nastavni indexu poli v master tabulce (tj. v $rowdata predanem do GetViewText)
     * @param $indexRC
     * @param int $indexOsC
     * @param int $indexNastup
     */
    function nastavIndexPole($indexRC, $indexOsC = -1, $indexNastup = -1)
    {
        $this->indexRC = $indexRC;
        $this->indexOsobniCislo = $indexOsC;
        $this->indexDatumNastupu = $indexNastup;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->database) return '';
        if ($this->indexRC < 0) return '';

        if (!$rowData[$this->indexRC]) return ''; // zadna osoba

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
            ' FROM RozdeleniOsob, Rozdeleni' .
            ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$this->typRozdeleni) .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($rowData[$this->indexRC]);

        if ($this->indexOsobniCislo >= 0 && $this->indexDatumNastupu >= 0) {
            // vyber podle pracovniho pomeru (ne jen podle RC)
            $sql .=
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($rowData[$this->indexOsobniCislo]) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($rowData[$this->indexDatumNastupu]));
        }

        // urceni datumu, ke kteremu se rozdeleni nacita
        if ($this->indexDatumOd >= 0) {
            $datumod = $this ->database ->stringToTimestamp($rowData[$this->indexDatumOd]);
            $datumdo = 0;

            if ($this->indexDatumDo >= 0) $datumdo = $this ->database ->stringToTimestamp($rowData[$this->indexDatumDo]);
        } else {
            $datumod = $this->datumOd;
            $datumdo = $this->datumDo;
        }

        if (!$datumdo) $datumdo = $datumod;

        $sql .=
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($datumdo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($datumod)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

        $sql .= ' ORDER BY RozdeleniOsob.PlatnostOd ';

        $result = '';

        $cursor = $this ->database ->select($sql, $params);

        while ($row = $cursor ->fetch()) {
            // nacetl zaznam
            if ($result) $result .= ', ';

            if ($this->zobrazitKod) $result .= $row[0];
            if ($this->zobrazitKod && $this->zobrazitPopis) $result .= ' - ';
            if ($this->zobrazitPopis) $result .= $row[1];
        }
        $cursor ->close();

        return $result;
    }
}
