<?php

namespace RonPortal\Controls;

use Exception;
use Ron\Controls\ShiftForm;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Common\RozdeleniOsobRecord;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;

/**
 * Dialog pro prirazovani rozdeleni zamestnancum
 * Class RozdeleniShiftForm
 * @package RonPortal\Controls
 */
class RozdeleniShiftForm extends ShiftForm
{
    /** @var int */
    protected $typRozdeleni;
    /** @var PravaNaOsoby|PravaNaOsoby[]  */
    protected $prava;

    /**
     * RozdeleniShiftForm constructor.
     * @param Database $database
     * @param int    $typRozdeleni
     * @param string $prava
     */
    public function __construct($database, $typRozdeleni, $prava)
    {
        parent::__construct($database);

        $this ->typRozdeleni = (int)$typRozdeleni;
        $this ->prava = $prava;
    }


    /**
     * @throws DBException
     */
    public function loadData()
    {
        $result = [];

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);

        $forceLoad = isset($_GET['force'])  &&  (int) $_GET['force'];
        $loadAll = !(isset($_GET['jenvlastni']) && (int)$_GET['jenvlastni']); // jen vlastni - pouziva se pro aktualizaci comboboxu

        $filter = isset($_GET['filter'])
            ? urldecode($_GET['filter'])        // iconv('utf-8', 'windows-1250', $_GET['filter'])
            : null;

        $result['filter'] = StringUtils::jsonText($filter);

        $count = null;

        session_write_close(); // aby fungoval .abort() v javascriptu

        // podle poctu existujicich zaznamu automaticky urci, jestli se nactou rovnou vsechna data
        // (pri velkem poctu zaznamu je to pomale - proto se nenacitaji zaznamy ihned)
        if ($loadAll) {
            $params = $this ->database ->createParameters();

            $sql =
                'SELECT COUNT(*) FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            if (isset($filter)  &&  $filter !== '') {
                $sql .=
                    '   AND (Popis LIKE ' . $params->addParameter('%' . $filter . '%') .
                    '  OR  Kod LIKE ' . $params->addParameter('%' . $filter . '%') . ')';
            }

            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();

            $count = $row ? $row[0] : 0;
            $loadAll = $count < 3000;
        }

        $params = $this ->database ->createParameters();
        if ($loadAll  ||  $forceLoad) {
            // POZOR! Nefunguje na ODBC s driverem "SQL Server" - je potreba prepnout na jiny driver!
            // nacita vsechna rozdeleni vcetne neprirazenych
            $sql =
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
                ' FROM Rozdeleni LEFT OUTER JOIN RozdeleniOsob ' .
                '  ON RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                '  AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                '  AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '  AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '  AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
                '  AND (RozdeleniOsob.PlatnostDo > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden') " .

                (isset($filter)
                    ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . ' OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                    : '') .

                ' ORDER BY Rozdeleni.Popis ';

            $result['nactenoVse'] = 1;
        } else {
            // nacita jen vlastni rozdeleni
            $sql =
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
                ' FROM RozdeleniOsob, Rozdeleni ' .
                ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
                ' AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                ' AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu)) .
                ' AND (RozdeleniOsob.PlatnostDo > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
                ' AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->dateToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') " .

                (isset($filter)
                    ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . '  OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                    : '') .

                ' ORDER BY Rozdeleni.Popis ';

            $result['nactenoVse'] = 0;
        }

        if (isset($count)) {
            // pocet zaznamu
            $result['count'] = (int)$count;
        }

        $result['leftItems'] = [];
        $result['rightItems'] = [];

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            //$obj = ['kod' => jsonText($row[0]), 'popis' => jsonText($row[1])];
            $obj = [
                'kod' => StringUtils::jsonText($row[0], false),
                'popis' => StringUtils::jsonText($row[1], false)
            ];

            if ($row[3]) {
                $obj['cssClass'] = StringUtils::jsonText($row[3]);
            }

            if ($row[2]) {
                // prirazena
                $result['rightItems'][] = $obj;
            } else {
                $result['leftItems'][] = $obj;
            }
        }

        return $result;
    }


    /**
     * @param PracovniPomer $pp
     * @param string $kodRozdeleni
     * @throws DBException
     */
    protected function odstranRozdeleni($pp, $kodRozdeleni)
    {
        if ($kodRozdeleni) {
            $rec = new RozdeleniOsobRecord($this->database, $this, $pp, $this->typRozdeleni, $kodRozdeleni);
            $rec->checkMultiple = false;
            $rec->deleteRecords();
        }
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function handleMoveLeft()
    {

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);

        // zkontrolovat prava
        if (!Prava::lzeZobrazitPP($this ->prava, $pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->odstranRozdeleni($pp, $kod);
        }

        return [ 'rightAction' => 'remove', 'leftAction' => 'add' ];
    }


    /**
     * @param PracovniPomer $pp
     * @param string $kodRozdeleni
     * @param int    $platnostOd
     * @throws Exception
     */
    public function priradRozdeleni($pp, $kodRozdeleni, $platnostOd)
    {
        $rec = new RozdeleniOsobRecord($this ->database, $this, $pp, $this ->typRozdeleni, $kodRozdeleni);

        $params = $this ->database ->createParameters();
        $filter =
            'PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
            ' AND (PlatnostDo >=  ' . $params ->addParameter($this ->database ->timestampToParam(time())) . '  OR  PlatnostDo IS NULL )';

        $rec ->checkMultiple = false;
        if (!$rec ->loadRecord($filter, $params)) {
            $rec ->fPlatnostOd ->setNewValue($platnostOd);
            $rec ->insertRecord();
        }
    }


    /**
     * @throws NeniPravoException
     * @throws DBException
     * @throws Exception
     */
    public function handleMoveRight()
    {
        // zkontrolovat prava!
        $pp = PracovniPomer::nactiPP($this ->database, $_GET['idpp']);

        if (!Prava::lzeZobrazitPP($this ->prava, $pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->priradRozdeleni($pp, $kod, $pp->datumNastupu);
        }

        return [ 'leftAction' => 'remove', 'rightAction' => 'add' ];
    }

}
