<?php
/**
 * seznam planovanych absenci
 */

namespace RonPortal\Dochazka;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\PoznamkaKeDni;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


class AbsenceFilterForm extends RonPortalFilterForm
{
    function __construct($database, $action)
    {
        parent::__construct($database, $action);
    }


    function loadData()
    {
        if (isset($_GET['operace'])) {
            $_SESSION['absence_operace'] = $_GET['operace'];
        }

        // zjisti hodnoty filtru zadane uzivatelem - nutne provest pred nastavovanim filtru Obdobi kvuli checkboxu
        if (isset($_GET['planovani_filtr'])) {
            $filtr = $_GET['planovani_filtr'];

            NastaveniUzivatele::setUserSettingsItem($this ->database, 'browser_absence_filtr', $filtr);
        } else if ( isset($_SESSION['planovani_filtr']) ) {
            $filtr = $_SESSION['planovani_filtr'];
        } else {
            $filtr = NastaveniUzivatele::getUserSettingsItem($this ->database, 'browser_absence_filtr');
        }

        $_SESSION['planovani_filtr'] = $filtr;

        // nastaveni "skryt/zobrazit automaticky schvalovane"
        if (isset($_GET['skrytauto'])) {
            $skrytauto = (int)$_GET['skrytauto'];
            NastaveniUzivatele::setUserSettingsItem($this ->database, 'browser_absence_skrytauto', $skrytauto);
        }
        else if ( isset($_SESSION['planovani_skrytauto']) ) {
            $skrytauto = $_SESSION['planovani_skrytauto'];
        }
        else {
            $skrytauto = NastaveniUzivatele::getUserSettingsItem($this ->database, 'browser_absence_skrytauto');
        }

        $_SESSION['planovani_skrytauto'] = $skrytauto;

        // pokud se ma zobrazit "vse" nebo "zrusene" tak je filtr na obdobi povinny
        $this ->obdobiPovinne = $_SESSION['planovani_filtr'] == 1 || $_SESSION['planovani_filtr'] == 2;

        parent::loadData();
    }


    function renderKodOperace()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . \Ron\ronText('Jen_typ_absence') . '</span>';

        $sql = 'SELECT Kod, Popis FROM OperaceDochazky WHERE Absence_Intraweb = 1';

        $combo = new ComboBox('operace');
        $combo->showCode = true;
        $combo->className = 'adsfilterforminput_popis';
        $combo->emptyRow = \Ron\ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Popis, Kod', RonPortalSession::getValue('absence_operace'));
        $combo->htmlParams = 'id="filtr_operace" data-ron-select2="#filtr_operace_kod" data-ron-autosubmit="1"';
        $combo->render();

        $combo = new ComboBox('');
        $combo->className = 'adsfilterforminput_kod desktop-only'; // na mobilu se zobrazi jen popis
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->emptyRow = \Ron\ronText('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Kod', RonPortalSession::getValue('absence_operace'));
        $combo->htmlParams = 'id="filtr_operace_kod" data-ron-select2="#filtr_operace" data-ron-autosubmit="1"';
        $combo->render();
        echo '</div>';
    }


    function renderFilterStav()
    {
        $filterarray = [0 => \Ron\ronText('Absence_aktualni'), 1 => \Ron\ronText('Absence_vse'), 2 => \Ron\ronText('Absence_zrusene')];

        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            $filterarray[3] = \Ron\ronText('Absence_bez_schvaleni');

            if (Prava::vratHodnotuPrava('prohlizeni') != Prava::vratHodnotuPrava('absence_schval')) {
                // POZOR! Neresi situaci, kdy je uzivatelsky vytvorene pravo pro konkretni absenci
                $filterarray[4] = \Ron\ronText('Absence_ke_schvaleni');
            }
        }

        echo '<div class="filterrow" id="filtr_urovenschvaleni">';
        echo '<span class="filtercaption"></span>';
        LinksList::renderLinks($filterarray, $_SESSION['planovani_filtr'], $this ->f_formaction . '&planovani_filtr=', '', false);
        echo "</div>\n";

        echo '<div class="filterrow" id="filtr_automatickyschvalovane">';
        echo '<span class="filtercaption"></span>';
        if ( $_SESSION['planovani_skrytauto'] ) echo '<a href="' . $this ->f_formaction . '&skrytauto=0">' . \Ron\ronText('Zobrazit_automaticky_schvalovane') . '</a>';
        else echo '<a href="' . $this ->f_formaction . '&skrytauto=1">' . \Ron\ronText('Skryt_automaticky_schvalovane') . '</a>';
        echo "</div>\n";
    }


    function renderFilters()
    {
        parent::renderFilters();

        $this ->renderKodOperace();

        // vsechno / nezpracovano / nezkontrolovano / neschvaleno
        $this->renderFilterStav();
    }
}


/**
 * Zobrazeni delky absence - bud popis delky nebo cas od - do
 * Class DelkaAbsenceField
 * @package Ron\RonPortal
 */
class DelkaAbsenceField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this ->code = 'col-delka';
        $this ->align = 'left';
        $this ->header = \Ron\ronText('Delka_absence');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData[12]) {
            case 0 :
                return \Ron\ronText('Absence_cela_smena');

            case 1 :
                return \Ron\ronText('Absence_pulka_smeny1');

            case 2 :
                return \Ron\ronText('Absence_pulka_smeny2');

            case 3 : // cas od-do
                return
                    DateUtils::minutesToString((int)($this ->database ->stringToSecondsOfDay($rowData[14]) / 60)) .
                    ' - ' .
                    DateUtils::minutesToString((int)($this ->database ->stringToSecondsOfDay($rowData[15]) / 60));
//                    date('G:i', $this ->database ->stringToTimestamp($rowData[14])) .
//                    ' - ' .
//                    date('G:i', $this ->database ->stringToTimestamp($rowData[15]));

            case 4 : // kontinualne
                return
                    date('j.n', $this ->database ->stringToTimestamp($rowData[2]) ) . ' ' . date('G:i', $this ->database ->stringToTimestamp($rowData[14])) .
                    ' - ' .
                    date('j.n', $this ->database ->stringToTimestamp($rowData[3]) ) . ' ' . date('G:i', $this ->database ->stringToTimestamp($rowData[15]));
        }

        return $rowData[13]; // popis delky absence
    }
}



/**
 * Odkaz na zruseni absence
 * Class ZruseniAbsenceField
 */
class ZruseniAbsenceField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $prava = Prava::vratPrava('absence', 'absence_' . $rowData[8]); // prava podle Absence.OperaceDochazky

        if (!$prava ->lzeZobrazitPP($rowData[11],
            $this ->database ->stringToTimestamp($rowData[2]), $this ->database ->stringToTimestamp($rowData[3]))
        ) return ''; // nemuze editovat => nemuze zrusit

        if ($rowData[9] == 4) return ''; // jiz je zruseno
        if ($rowData[9] == 5) return ''; // zruseni zamitnuto -> zrusit jde jedine ve formulari

        // Pokud nemuze uzivatel schvalovat, tak muze zrusit jedine absenci, ktera jeste nebyla schvalena
        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI))
            if ($rowData[9]) {
                $pravaSchvalovani = Prava::vratPrava('absence_schval', 'absence_schval_' . $rowData[8]);

                if (!$pravaSchvalovani ->lzeZobrazitPP($rowData[11],
                    $this ->database ->stringToTimestamp($rowData[2]), $this ->database ->stringToTimestamp($rowData[3]))
                ) return '';
            }

        return GuiUtils::changeLocationQuest(\Ron\ronText('Zrusit'), \Ron\ronText('Opravdu_zrusit_absenci'),
            '?menuid=absence&action=zrusitabsenci&idpp=' . $rowData[11] .
            '&datum=' . DateUtils::dateToUrlString($this ->database ->stringToTimestamp($rowData[2])) .
            '&poradi=' . $rowData[16] . '&list=1');
    }
}


/**
 * Odkaz na zobrazeni absence: menuid=absence&action=zobrazabsenci&idpp=...&datum=...&poradi=...
 * Class EditaceAbsenceField
 * @package Ron\RonPortal
 */
class EditaceAbsenceField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this ->code = 'col-zobrazit';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=absence&action=zobrazabsenci&idpp=' . $rowData[11] .
            '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
            '&poradi=' . $rowData[16] . '">' . \Ron\ronText('Zobrazit') . '</a>';
    }
}


/**
 * Odkaz na schvaleni absence
 * Class SchvaleniAbsenceField
 * @package Ron\RonPortal
 */
class SchvaleniAbsenceField extends BrowserField
{
    var $pravaDoporuceni;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this ->code = 'col-schvaleni';
        $this ->header = \Ron\ronText('Schvaleno');
        $this ->align = 'center';

        $this ->pravaDoporuceni = Prava::vratPrava('absence_doporuc');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch($rowData[9]) {
            case 1 :
                return \Ron\ronText('Str_Yes') . ($rowData[10] ? ' (' . StringUtils::htmlTextConvert($rowData[10]) . ')' : ''); // uz schvaleno

            case 2 :
                return \Ron\ronText('Str_No') . ($rowData[10] ? ' (' . StringUtils::htmlTextConvert($rowData[10]) . ')' : ''); // zamitnuto

            case 3 :
                return \Ron\ronText('Zadost_o_zruseni');

            case 4 :
                return \Ron\ronText('Zruseno') . ($rowData[10] ? ' (' . StringUtils::htmlTextConvert($rowData[10]) . ')' : ''); // zruseno

            case 5 :
                return \Ron\ronText('Zruseni_zamitnuto') . ($rowData[10] ? ' (' . StringUtils::htmlTextConvert($rowData[10]) . ')' : '');
        }

        $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $rowData[8]);
        $lzeschvalit = $prava ->lzeZobrazitPP($rowData[11], $this ->database ->stringToTimestamp($rowData[2]), $this ->database ->stringToTimestamp($rowData[3]));

        $result = '';

        // schvaleni neni urceno
        if (RonPortalSettings::getIntValue(AbsenceController::SETTINGS_DOPORUCOVANI)) {
            if ($rowData[18] == 2) {
                // nedoporuceno - zamitnout
                $result = \Ron\ronText('Nedoporuceno') . ($rowData[19] ? ' (' . StringUtils::htmlTextConvert($rowData[19]) . ')' : '') . '&nbsp;';

                if ($lzeschvalit) $result .=
                    '<a href="?menuid=absence&action=zamitnoutabsenci&idpp=' . $rowData[11] .
                    '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
                    '&poradi=' . $rowData[16] . '&list=1' . '">' . \Ron\ronText('Zamitnout') . '</a>';

                return $result;

            } else if ($rowData[18] == 1) {
                // doporuceno
                $result = \Ron\ronText('Doporuceno') . ($rowData[19] ? ' (' . StringUtils::htmlTextConvert($rowData[19]) . ')' : '') . '&nbsp;';

                if ($lzeschvalit) $result .= // muze schvalovat - prida odkaz na schvaleni
                    '<a class="sabs_schvalit" href="?menuid=absence&action=schvalitabsenci&idpp=' . $rowData[11] .
                    '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
                    '&poradi=' . $rowData[16] . '&list=1' . '">' . \Ron\ronText('Schvalit') . '</a>';

                return $result;

            } else if (!$rowData[18]) {
                // doporuceni neni jeste nastaveno - pokud nemuze schvalovat, tak zobrazi "doporucit
                if (!$lzeschvalit && $this ->pravaDoporuceni ->lzeZobrazitPP($rowData[11], $this ->database ->stringToTimestamp($rowData[2]), $this ->database ->stringToTimestamp($rowData[3]))) {
                    return
                        '<a href="?menuid=absence&action=doporucitabsenci&idpp=' . $rowData[11] .
                        '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
                        '&poradi=' . $rowData[16] . '&list=1' . '">' . \Ron\ronText('doporucit') . '</a>';
                }
            }
        }

        if ($lzeschvalit) // muze schvalovat
            $result .=
                '<a class="sabs_schvalit" href="?menuid=absence&action=schvalitabsenci&idpp=' . $rowData[11] .
                '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
                '&poradi=' . $rowData[16] . '&list=1' . '">' . \Ron\ronText('Schvalit') . '</a>' .
                ' / ' .
                '<a class="sabs_zamitnout" href="?menuid=absence&action=zamitnoutabsenci&idpp=' . $rowData[11] .
                '&datum=' . date('j.n.Y', $this ->database ->stringToTimestamp($rowData[2])) .
                '&poradi=' . $rowData[16] . '&list=1' . '">' . \Ron\ronText('Zamitnout') . '</a>';

        return $result;
    }
}


/**
 * Poznamky ke dni v rozsahu zadane absence
 * Class Absence_PoznamkyKeDniField
 * @package Ron\RonPortal
 */
class Absence_PoznamkyKeDniField extends BrowserField
{
    var $typyPoznamek;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $typPoznamky = RonPortalSettings::getValue('absence_typpoznamky');
        $this ->typyPoznamek = ($typPoznamky == '*') ? null : explode(';', $typPoznamky);

        $this ->code = 'col-poznamkykedni';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $poznamky = PoznamkaKeDni::nactiPoznamkyProOsobu($this ->database, $rowData[11],
            $this ->database ->stringToTimestamp($rowData[2]), $this ->database ->stringToTimestamp($rowData[3]),
            $this ->typyPoznamek);

        $result = '';
        if ($poznamky) {
            $title = '';
            foreach ($poznamky as $poznamka) {
                $title .= StringUtils::htmlTextConvert($poznamka ->text) . "\n";
            }

            $result = '<i class="inline-glyph fa glyph-info" title="' . $title . '"></i>';
        }

        return $result;
    }
}

class AbsenceBrowser extends RonPortalBrowser
{
    var $pravaProhlizeni;
    var $pravaSchvalovani;
    var $pravaPlanovani;

    var $RC; // pokud je zadano, zeobrazi se data jen pro jednu osobu
    var $menuId; // cilova stranka pro odkazy

    var $jsonObject;
    var $jsonRowIndex;


    function __construct($database, $RC = null)
    {
        parent::__construct($database, 'planovani');

        $this ->title = \Ron\ronText('Prehled_planovanych_absenci');

        $this ->code = 'browserplanovaneabsence';
        $this ->tableId = 'schvalovaniAbsenciTable';

        $this ->RC = $RC;
        $this ->menuId = $RC ? 'planovani' : 'schvalovaniabsence';
        $this ->browserUrl = '?menuid=' . $this ->menuId . '&';

        // definice dotazu a hlavicky
        $this ->sqlSelectPart =
            'Osoba.Jmeno, Osoba.Prijmeni, ' . // 0,1
            'Absence.DatumOd, Absence.DatumDo, OperaceDochazky.Popis, ' . // 2..4
            'Absence.RC, Absence.OsobniCislo, Absence.DatumNastupu, ' . // 5..7
            'Absence.OperaceDochazky, Absence.Schvaleni, Absence.Schvalil, ' . // 8..10
            'PracovniPomer.Id, ' . // 11
            'Absence.Delka, S_DelkaAbsence.Popis, Absence.CasOd, Absence.CasDo, Absence.Poradi, ' . // 12..16
            $database ->castWideMemo('Absence.Poznamka') . ', Absence.Doporuceni, Absence.Doporucil, ' . // 17..19
            'Absence.KodTypuRozdeleni, Absence.Rozdeleni, Rozdeleni.Popis, PracovniPomer.OsobniCislo '; // 20..23


        $this ->sqlFromPart =
            '( ( ( ( Absence INNER JOIN OperaceDochazky ' .
            '        ON Absence.OperaceDochazky = OperaceDochazky.Kod ' .
            '      ) ' .
            '      INNER JOIN PracovniPomer ' .
            '       ON PracovniPomer.RC = Absence.RC' .
            '      AND PracovniPomer.OsobniCislo = Absence.OsobniCislo ' .
            '      AND PracovniPomer.DatumNastupu = Absence.DatumNastupu ' .
            '    ) ' .
            '    INNER JOIN Osoba ' .
            '     ON Osoba.RC = Absence.RC ' .
            '  ) ' .
            '  INNER JOIN S_DelkaAbsence ' .
            '   ON S_DelkaAbsence.Kod = Absence.Delka' .
            ') ' .
            'LEFT OUTER JOIN Rozdeleni ' .
            '  ON Rozdeleni.KodTypuRozdeleni = Absence.KodTypuRozdeleni ' .
            ' AND Rozdeleni.Kod = Absence.Rozdeleni ';


        if ($this ->RC) // jen data jedne osoby (pri planovani)
            $this ->sqlWherePart = "Osoba.RC = '" . $database ->addSlashes($this ->RC) . "'";

        $this ->pravaProhlizeni = Prava::vratPrava('prohlizeni');
        $this ->pravaSchvalovani = Prava::vratPrava('absence_schval');
        $this ->pravaPlanovani = Prava::vratPrava('absence');

        $this ->primaryField = 0;

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno', 0)
            ->setCode('col-jmeno') ->setHeaderText('Jmeno') ->setOrdering(1);

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni',  1)
            ->setCode('col-prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(1);

        $this ->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo', 23)
            ->setCode('col-osobnicislo') ->setHeaderText('Osobni_cislo') ->setOrdering(1);

        $this ->addField()
            ->setSqlFieldName('Absence.DatumOd', 2) ->setFormat('date')
            ->setCode('col-datumod') ->setHeaderText('Datum_od') ->setOrdering(1);
        $this ->defaultSort = count($this ->fields);

        $this ->addField()
            ->setSqlFieldName('Absence.DatumDo', 3) ->setFormat('date')
            ->setCode('col-datumdo') ->setHeaderText('Datum_do') ->setOrdering(1);

        $this ->addField()
            ->setSqlFieldName('OperaceDochazky.Popis', 4)
            ->setCode('col-operace') ->setHeaderText('Popis_absence') ->setOrdering(1);

        $this ->addField(new DelkaAbsenceField($database));
        $this ->addField(new EditaceAbsenceField($database));
        $this ->addField(new ZruseniAbsenceField($database));

        // pole pro schvaleni absence vcetne odkazu a osoby, ktera to schvalila
        $this ->addField(new SchvaleniAbsenceField($database))
            ->setVisible(RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI));

        // zjednodusene zobrazeni schvaleni - pouze textovy stav - implicitne se nezobrazuje
        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Absence.Schvaleni', 9) ->setHeaderText('Stav_absence') ->setOrdering(1)
            ->setCode('col-stav') ->setTitleFieldIndex(10) ->setVisible(false)
            ->setOptions([
                0 => \Ron\ronText('Stav_Absence_0'),
                1 => \Ron\ronText('Stav_Absence_1'),
                2 => \Ron\ronText('Stav_Absence_2'),
                3 => \Ron\ronText('Stav_Absence_3'),
                4 => \Ron\ronText('Stav_Absence_4'),
                5 => \Ron\ronText('Stav_Absence_5')
            ]);

        if (RonPortalSettings::getValue('absence_typpoznamky')) {
            $this ->addField(new Absence_PoznamkyKeDniField($database));
        }
    }

    function setFilterForm()
    {
        if (isset($_GET['poznamkyabsence'])) {
            $_SESSION['poznamkyabsence'] = (int)$_GET['poznamkyabsence'];
            setcookie('absence_zobrazitpoznamky', (int)$_GET['poznamkyabsence'], time() + 30000000);
        } else if (!isset($_SESSION['poznamkyabsence'])) {
            $_SESSION['poznamkyabsence'] = isset($_COOKIE['absence_zobrazitpoznamky']) ? (int)$_COOKIE['absence_zobrazitpoznamky'] : 0;
        }

        $this ->filterForm = new AbsenceFilterForm($this ->database, $this ->menuId);
        $this ->filterForm ->setUseGet(true, $this ->menuId);

        $this ->filterForm ->nastavObdobi(true, 'ADS', false, true); // jenotevrene = false, povinne - nastavi se v load

        if (!$this ->RC) {
            // filtr rozdeleni - v planovani se nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);

            $pravaPrehled = Prava::vratPrava('absence_prehled');
            //Nastavi pravo na rozdeleni, kde muze nekoho videt. POZOR: nejde nastavit soucet prav, pokud tedy u schvalovani ma pouze jednu osobu, uvidi pouze jeji rozdeleni.
            if ($this->pravaSchvalovani->jsouNejakePrava()) {
                $this->filterForm->f_pravarozdeleni = $this->pravaSchvalovani;
            }
            elseif ($this->pravaProhlizeni->jsouNejakePrava()) {
                $this->filterForm->f_pravarozdeleni = $this->pravaProhlizeni;
            }
            elseif ($pravaPrehled->jsouNejakePrava()) {
                $this->filterForm->f_pravarozdeleni = $pravaPrehled;
            }
            elseif ($this->pravaPlanovani->jsouNejakePrava()) {
                $this->filterForm->f_pravarozdeleni = $this->pravaPlanovani;
            }

            if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
            }

            $this->filterForm ->nastavOsoby(true, $this->filterForm->f_pravarozdeleni);
            $this->filterForm ->povolitVsechyOsoby = true;
        }
    }


    /**
     * Zobrazi formular pro hromadne schvalovani
     */
    function hromadneSchvalovaniForm()
    {
        if ( !RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) ) {
            return;
        }
        if ( !$this ->pravaSchvalovani ->jsouNejakePrava() ) {
            return;
        }

        echo
            '<div id="absence_hromadneschvalovani">' .
            '<form method="POST" name="hromadneSchvalovaniForm" action="?menuid=schvalovaniabsence">' .
            '<input type="hidden" name="hromadneSchvaleni" value="1" />' .
            '<input type="hidden" name="akceHromadneSchvaleni" value="0" />' . // nastavi se tlacitkem 1=schvalit, 2=zamitnout
            '<input type="hidden" name="vybraneZaznamy" value="" />' .

            '<span class="buttons">' .
            '<input type="button" class="button midbutton" name="schvaleniMenuButton" id="schvaleniMenuButton" value="..." />' .

            '<input id="schvalitVybraneAbsence" type="button" class="button" name="schvalitVybrane" disabled="disabled" value="' . \Ron\ronText('Schvalit_vybrane') . '" />' .
            '<input id="zamitnoutVybraneAbsence" type="button" class="button" name="zamitnoutVybrane" disabled="disabled" value="' . \Ron\ronText('Zamitnout_vybrane') . '" />' .
            '<input id="schvalitVsechnyAbsence" type="button" class="button" name="schvalitVse" value="' . \Ron\ronText('Schvalit_vse') . '" />' .

            '</span>' .
            '</form>'.
            "</div>\n";

        echo
            '<ul class="popupmenu"  id="schvaleniMenu">' .
            "<li id='schvaleniMenuVse'>" . \Ron\ronText('Oznacit_vse') . "</li>" .
            "<li id='schvaleniMenuNic'>" . \Ron\ronText('Zrusit_oznaceni') . "</li>" .
            '</ul>';
    }


    private function zobrazitPoznamky()
    {
        return $_SESSION['poznamkyabsence'];
    }


//    function load($sqlFilter = '')
//    {
//        parent::load($sqlFilter);
//    }


    function renderFilter()
    {
        parent::renderFilter();

        echo '<div class="text_link" id="absence_poznamky" >';
        if ($this ->zobrazitPoznamky()) echo '<a href="?menuid=' . $this ->menuId . '&poznamkyabsence=0">' . \Ron\ronText('Skryt_poznamky') . '</a>';
        else echo '<a href="?menuid=' . $this ->menuId . '&poznamkyabsence=1">' . \Ron\ronText('Zobrazit_poznamky') . '</a>';
        echo "</div>\n";

        $this ->hromadneSchvalovaniForm();
    }


    function getSQLFilter()
    {
        $result = '';

        $filtrSettings = isset($_SESSION['planovani_filtr']) ? $_SESSION['planovani_filtr'] : 0;

        switch ($filtrSettings) {
            case 1 : // vse
                break;

            case 2 : // zrusene
                $result = 'Absence.Schvaleni = 4';
                break;

            case 3 : // bez schvaleni
            case 4 : // ke schvaleni - jako bez schvaleni, ale jen ty, ktere muzu schvalovat
                $result = 'Absence.Schvaleni = 0';
                break;

            default : // aktualni a budouci
                $result =
                    'Absence.Schvaleni <> 4 ' . // vynecha zrusene
                    ' AND Absence.DatumDo >= ' . $this ->database ->dateToString(time());
                break;
        }

        if ( $_SESSION['planovani_skrytauto'] ) {
            $result .=
                ($result ? ' AND ' : '') .
                'OperaceDochazky.Absence_AutoSchvalovat = 0';
        }

        if (isset($_SESSION['absence_operace'])  &&  $_SESSION['absence_operace'] !== '') {
            $result .=
                ($result ? ' AND ' : '') .
                "Absence.OperaceDochazky = '" . $this ->database ->addSlashes($_SESSION['absence_operace']) . "'";
        }


        if ($this ->filterForm ->showOsoby  &&  !$this ->filterForm ->vsechnyOsoby) {
            // filtr na osobu
            $result .=
                ($result ? ' AND ' : '') .
                'PracovniPomer.Id = ' . (int)$this ->filterForm ->vybranaOsoba;
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && RonPortalSession::rozdeleni() !== null) {
            // rozdeleni
            $filtr = $this ->filterForm ->vratSQLFiltrRozdeleni('PracovniPomer', RonPortalSession::rozdeleniVcetnePodrizenych());

            if ($filtr) {
                $result .=
                    ($result ? ' AND ' : '') .
                    '(' . $filtr . ') ';
            }
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) && RonPortalSession::rozdeleni2()) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), RonPortalSession::rozdeleni2(),
                RonPortalSession::obdobi(), RonPortalSession::konecObdobi(), RonPortalSession::rozdeleni2VcetnePodrizenych());

            if ($filtr) {
                $result .=
                    ($result ? ' AND ' : '') .
                    '(' . $filtr . ') ';
            }
        }

        // obdobi (pro "vse" a "zrusene" je povinne)
        if ($_SESSION['planovani_filtr'] == 1 || $_SESSION['planovani_filtr'] == 2 || RonPortalSession::jenObdobi()) {
            $result .=
                ($result ? ' AND ' : '') .
                ' Absence.DatumDo >= ' . $this ->database ->timestampToString(RonPortalSession::obdobi()) .
                ' AND Absence.DatumOd <= ' . $this ->database ->timestampToString(RonPortalSession::konecObdobi());
        }

        return $result;
    }

    function getRowAttributes($values, $className, $rowNumber)
    {
        switch ($values[9]) {
            case 1 :
            case 5 : // zadost o zruseni zamitnuta = schvaleno
                $className .= ' absence_schvalena';
                break;

            case 2 :
                $className .= ' absence_zamitnuta';
                break;

            case 3 :
                $className .= ' absence_zadostozruseni';
                break;

            case 4 :
                $className .= ' absence_zrusena';
                break;

            default :
                $className .= ' absence_neschvalena';

                if (RonPortalSettings::getIntValue(AbsenceController::SETTINGS_DOPORUCOVANI)) {
                    if ($values[18] == 1) $className .= ' absence_doporucena';
                    else if ($values[18] == 2) $className .= ' absence_nedoporucena';
                }
                break;
        }

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        if (isset($this ->jsonRowIndex)) {
            $result .=
                " data-ron-indexabsence='{$this ->jsonRowIndex}' id='absencerow_{$this ->jsonRowIndex}'";
        }

        return $result;
    }


    function enableLine($values)
    {
        $datumod = $this ->database ->stringToTimestamp($values[2]);
        $datumdo = $this ->database ->stringToTimestamp($values[3]);

        $this ->jsonRowIndex = null;

        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            $pravo = Prava::vratPrava('absence_schval', 'absence_schval_' . $values[8]);

            if($pravo ->lzeZobrazitPP($values[11], $datumod, $datumdo)) {
                // ma pravo schvalit - prida radek do json a vraci true
                $this->jsonRowIndex = count($this->jsonObject['items']);
                $this->jsonObject['items'][] = ['idpp' => $values[11], 'datum' => date('j.n.Y', $datumod), 'poradi' => $values[16]];

                return true;
            }

            if ($_SESSION['planovani_filtr'] == 4) {
                // filtr "ke schvaleni" - pokud nemuzu schvalovat, tak ho ani nezobrazuju
                return false;
            }
        }

        if ($this ->pravaProhlizeni ->lzeZobrazitPP($values[11], $datumod, $datumdo)) {
            // ma pravo prohlizet
            return true;
        }

        $pravo = Prava::vratPrava('absence_prehled', 'absence_' .$values[8]);
        if ($pravo ->lzeZobrazitPP($values[11], $datumod, $datumdo)) {
            return true;
        }

        $pravo = Prava::vratPrava('absence', 'absence_' . $values[8]);
        return $pravo ->lzeZobrazitPP($values[11], $datumod, $datumdo);
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        // zobrazi poznamku a rozdeleni
        if ( $this ->zobrazitPoznamky()  &&  ($values[17]  ||  isset($values[21])) ) {
            echo "<tr class='$className'>";
            //$colspan = 7;
            $colspan = count($this ->fields);

            if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) $colspan++;
            echo "<td></td><td colspan='$colspan'>";

            // rozdeleni (kod - popis)
            if (isset($values[21])) {
                echo '<div class="poznamka browserpoznamka">' . StringUtils::htmlTextConvert($values[21]) . ' - ' . StringUtils::htmlTextConvert($values[22]) . '</div>';
            }

            // poznamka
            if ($values[17]) {
                echo '<div class="poznamka browserpoznamka">' . StringUtils::htmlTextConvert($values[17]) . '</div>';
            }

            echo '</td>';
            echo "</tr>\n";
        }
    }


    function render()
    {
        echo '<div id="browserabsence">';

        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        parent::render();

        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            echo
                '<script' . ' type="text/javascript">' .
                ' $( function() { ronPortal.schvalovaniAbsenci(' . json_encode($this ->jsonObject) . ') } );' . // inicializace
                '</script>';
        }

        echo '</div>';
    }
}

