<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use RonPortal\PracovniPomer;

/**
 * Spravuje data osoby pro dny dochazky (zaznamy zpracovane dochazky)
 * Class DenDochazky
 * @package Ron\RonPortal
 */

class DenDochazky
{
    /** @var int */
    var $datum;

    /** @var ZaznamZpracovaneDochazky[] Zaznamy zpracovane dochazky */
    var $zaznamy;

    function __construct($datum)
    {
        $this->datum = $datum;
        $this->zaznamy = [];
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @return DenDochazky[]
     */
    public static function seznamDnu($datumOd, $datumDo)
    {
        $result = [];

        $datum = DateUtils::startOfTheDay($datumOd);
        while ($datum <= $datumDo) {
            $den = new static($datum);
            $result[] = $den;
            $datum = DateUtils::incDayStart($datum);
        }

        return $result;
    }


    /**
     * Do zadaneho seznamu dnu nacte zaznamy zpracovane dochazky
     * @param Database $database
     * @param DenDochazky[] $dny
     * @param PracovniPomer $pp
     * @param $nacistPoznamky
     * @return ZaznamZpracovaneDochazky[]
     */
    public static function nactiZpracovanouDochazku($database, $dny, $pp, $nacistPoznamky)
    {
        if (!$dny) return null;

        foreach ($dny as $den) {
            $den ->zaznamy = [];
        }

        $datumOd = $dny[0] ->datum;
        $datumDo = $dny[count($dny) - 1] ->datum;


        $zaznamy = ZaznamZpracovaneDochazky::nactiOperaceDochazkyZDB($database, $pp, $datumOd, $datumDo, $nacistPoznamky);

        /** @var int $index     Index dne - behem nacitani zaznamu se postupne inkrementuje */
        $index = 0;

        foreach ($zaznamy as $novyZaznam) {
            $datum = DateUtils::startOfTheDay($novyZaznam ->datumCasPocatku);

            // najde den, do ktereho zaznam patri
            while ($index < count($dny)  &&  $dny[$index] ->datum < $datum) {
                $index++;
            }

            if ($index < count($dny)) {
                $radek = $index < count($dny) ? $dny[$index] : null;

                // v prvnim dnu v mesici muze byt zaznam, ktery patri do predchoziho mesice!
                if ($radek  &&  $radek ->datum <= $novyZaznam ->datumCasPocatku) {
                    $radek->zaznamy[] = $novyZaznam;
                    $dalsiIndex = $index + 1;
                } else {
                    $dalsiIndex = $index; // je to zaznam z predchoziho mesice - v prvnim dnu zobrazim jeho cast po pulnoci
                }

                if ($novyZaznam->datumCasUkonceni > DateUtils::incDayStart(DateUtils::startOfTheDay($novyZaznam->datumCasPocatku))) {
                    if ($dalsiIndex < count($dny)  &&  $novyZaznam->datumCasUkonceni > $dny[$dalsiIndex] ->datum) {
                        // zaznam presahuje do dalsiho dne - prida jeste dalsi zaznam
                        $radek2 = $dny[$dalsiIndex];

                        $zaznamPoPulnoci = $novyZaznam->kopiePoPulnoci();
                        $radek2->zaznamy[] = $zaznamPoPulnoci;
                    }
                }
            } else {
                // zaznam je v dalsim mesici, ale je prirazen do posledniho dne aktualniho mesice
                if ($novyZaznam ->den == $datumDo) {
                    $radek = $dny[count($dny) - 1];
                    $radek ->zaznamy[] = $novyZaznam;
                }
            }
        }

        return $zaznamy;
    }

}
