<?php
/**
 * UTF8
 * stanislav.elbl, 28.08.2015
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use RonPortal\RonPortalSettings;


class DochazkaUtils
{
    /**
     * Vraci datum a cas posledniho zpracovani dochazky
     * @param Database $database
     * @return int
     * @throws \Exception
     */
    static function datumCasZpracovaniDochazky($database)
    {
        $sql = "SELECT val FROM SYS_VAR WHERE (applicationID = 'ADS') AND (Name = 'DatumZpracovaniDochazky')";
        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani zpracovani dochazky!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row  ||  !$row[0]) return 0;

        try {
            $result = DateUtils::datetimeStringToTimestamp($row[0]); // Pozor! Hodnota v db ulozena jako text!
        } catch (\Exception $e) {
            throw new \Exception("Chyba pri zjistovani posledniho zpracovnani dochazky: {$row[0]}");
        }

        return $result;
    }


    /**
     * Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka (pracovni pomer). Z nich vybere ten posledni.
     * @param Database $database
     * @param int $datum
     * @param string $rc
     * @param string $osobnicislo
     * @param int $datumnastupu
     * @param bool $pouzitPP slouzi pro urceni, zda se vyhledava operace pro konkretni PP, nebo osobu
     * @return array|bool|null
     * @throws DBException
     */
    public static function vratPosledniPruchodSQL($database, $datum, $rc, $osobnicislo, $datumnastupu, $pouzitPP = true)
    {
        // TODO: predelat na parametry!
        $dnu = RonPortalSettings::getIntValue('monitorovani_dnu');
        if ($dnu < 1) $dnu = 1;

        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace as DatumCas, PuvodniDochazka.Terminal, PuvodniDochazka.Poznamka,' .
            ' OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' .
            ' Umisteni.popis as popisUmisteni, PuvodniDochazka.Rozdeleni, Rozdeleni.Popis, OperaceDochazky.Kod' .
            ' FROM ' .
            ' ( ( (PuvodniDochazka INNER JOIN OperaceDochazky' .
            '        ON OperaceDochazky.kod = PuvodniDochazka.KodOperace' .
            '     ) LEFT OUTER JOIN Terminal' .
            '       ON Terminal.kod = PuvodniDochazka.terminal' .
            '   ) LEFT OUTER JOIN Umisteni' .
            '     ON Umisteni.kod = Terminal.KodUmisteni' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.Kod = PuvodniDochazka.Rozdeleni ' .
            '   AND Rozdeleni.KodTypuRozdeleni = PuvodniDochazka.KodTypuRozdeleni ' .
            ' WHERE (PuvodniDochazka.datumCasOperace <= ' . $database ->timestampToString($datum) . ')' .
            ' AND (PuvodniDochazka.datumCasOperace > ' . $database ->timestampToString(DateUtils::incDay($datum, -$dnu)) . ')' .
            ' AND ((Terminal.bezMonitorovani <> 1) OR (Terminal.bezMonitorovani IS NULL))' .
            ' AND OperaceDochazky.PritomnostProMonitorovani <= 2 ' . // nastaveni 3=nemonitorovat uplne ignoruje
            " AND PuvodniDochazka.RC = '" . $database ->addSlashes($rc) . "'";

        if ($pouzitPP) {
            $sql .=
                " AND PuvodniDochazka.OsobniCislo = '" . $database ->addSlashes($osobnicislo) . "'" .
                " AND PuvodniDochazka.DatumNastupu = " . $database ->timestampToString($datumnastupu);
        }

        $sql .=
            ' UNION ' .

            'SELECT HistoriePruchodu.DatumCasPruchodu as DatumCas, HistoriePruchodu.Terminal,';

        //  if (db_type() == "interbase") $sql = $sql.'cast(null as varchar(40)),';
        //  else $sql = $sql.'NULL,';
        $sql = $sql . 'HistoriePruchodu.Poznamka,'; // zmeneno 24.7.2007

        $sql = $sql .
            ' OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' .
            ' Umisteni.popis as popisUmisteni, HistoriePruchodu.Rozdeleni, Rozdeleni.Popis, OperaceDochazky.Kod' .
            ' FROM ' .
            ' ( ( (HistoriePruchodu INNER JOIN OperaceDochazky' .
            '        ON OperaceDochazky.kod = HistoriePruchodu.KodOperace' .
            '     ) INNER JOIN Terminal' .
            '       ON Terminal.kod = HistoriePruchodu.terminal' .
            '   ) LEFT OUTER JOIN Umisteni' .
            '     ON Umisteni.kod = Terminal.KodUmisteni' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.Kod = HistoriePruchodu.Rozdeleni ' .
            '   AND Rozdeleni.KodTypuRozdeleni = HistoriePruchodu.KodTypuRozdeleni ' .
            ' WHERE (HistoriePruchodu.uspesny = 1)' .
            ' AND (HistoriePruchodu.DatumCasPruchodu <= ' . $database ->timestampToString($datum) . ')' .
            ' AND (HistoriePruchodu.DatumCasPruchodu > ' . $database ->timestampToString(DateUtils::incDay($datum, -1)) . ')' .
            ' AND ((Terminal.bezMonitorovani <> 1) OR (Terminal.bezMonitorovani IS NULL))' .
            ' AND OperaceDochazky.PritomnostProMonitorovani <= 2 ' . // nastaveni 3=nemonitorovat uplne ignoruje
            " AND HistoriePruchodu.RC='" . $database ->addSlashes($rc) . "'";

        if ($pouzitPP) {
            $sql .=
                " AND HistoriePruchodu.OsobniCislo='" . $database ->addSlashes($osobnicislo) . "'" .
                " AND HistoriePruchodu.DatumNastupu=" . $database ->dateToString($datumnastupu);
        }


        if ($database ->getDbType() == 'interbase') $sql = $sql . ' ORDER BY 1 DESC';
        else $sql = $sql . ' ORDER BY DatumCas DESC';

        $dochazka_row = null;

        $cursor = $database ->select($sql);

        $dochazka_row = $cursor ->fetch();

        if ($dochazka_row) {
            $OK = $dochazka_row[4] != 2;
            while (!$OK) {
                $pomrow = $cursor->fetch();
                if ($pomrow) {
                    $dochazka_row[4] = $pomrow[4];
                    $OK = $dochazka_row[4] != 2;
                } else {
                    $dochazka_row[4] = 0;
                    $OK = true;
                }
            }
        } else {
            $dochazka_row = [null, null, null, null, null, null, null, null, null];
        }

        $cursor ->close();

        // Nacte absenci
        $params = $database ->createParameters();
        $sql =
            'SELECT Absence.DatumOd, Absence.DatumDo, ' .
            ' OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani, OperaceDochazky.Kod, ' . $database->castWideMemo('Absence.Poznamka') .
            ' FROM Absence, OperaceDochazky' .
            ' WHERE (OperaceDochazky.Kod = Absence.OperaceDochazky)' .
            '  AND (Absence.DatumOd <= ' . $params ->addParameter($database ->timestampToParam($datum)) . ')' .
            '  AND (Absence.DatumDo >= ' . $params ->addParameter($database ->timestampToParam(DateUtils::incDay($datum, -1))) . ')' .
            '  AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 ' . // nezobrazuje zamitnute a zrusene
            ' AND Absence.RC = ' . $params ->addParameter($rc);

        if ($pouzitPP) {
            $sql .=
                ' AND Absence.OsobniCislo = ' . $params->addParameter($osobnicislo) .
                ' AND Absence.DatumNastupu = ' . $params->addParameter($database ->dateToParam($datumnastupu));
        }

        $sql .=
            ' ORDER BY DatumOd DESC';


        $absence_row = null;

        $cursor = $database ->getSqlResult($sql, $params, 'Spatny SQL vyraz pro vyber planovanych absenci');
        $absence_row = $cursor ->fetch();
        $cursor ->close();

        $datumdochazky = $database ->stringToTimestamp($dochazka_row[0]);

        if ($absence_row) {
            $datumabsence = $database ->stringToTimestamp($absence_row[0]);
            if ($datumabsence > $datumdochazky) {
                // Vraci absenci
                $result = array();
                $result[] = date('j.n.Y', $datumabsence) . ' - ' . DBUtils::getFormatedValue($database, 'date', $absence_row[1]);
                $result[] = ''; // terminal
                $result[] = $absence_row[5]; // poznamka
                $result[] = $absence_row[2]; // popis operace
                $result[] = $absence_row[3]; // pritomnost
                $result[] = ''; // umisteni
                $result[] = ''; // kod rozdeleni
                $result[] = ''; // popis rozdeleni
                $result[] = $absence_row[4]; // kod operace

                return $result;
            }
        }

        $dochazka_row[0] = DBUtils::getFormatedValue($database, 'datetime', $dochazka_row[0]);

        return $dochazka_row;
    }
}
