<?php
/**
 * stanislav.elbl, 16.03.2021
 * Formular pro hromadne schvaleni dochazky
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;

class HromadneSchvaleniFormular implements ViewObject
{
    /** @var Database */
    var $database;

    /** @var Obdobi */
    var $obdobi;
    var $datumZpracovaniDochazky;

    /** @var int        Kod typu rozdeleni*/
    var $typRozdeleni;
    /** @var string     Kod rozdeleni */
    var $kodRozdeleni;
    /** @var  Rozdeleni */
    var $rozdeleni;

    /** @var int        Kod typu rozdeleni 2 */
    var $typRozdeleni2;
    /** @var string     Kod rozdeleni 2 */
    var $kodRozdeleni2;
    /** @var  Rozdeleni */
    var $rozdeleni2;

    /** @var bool       pokud uzivatel muze editovat alespon jednu osobu, nastavi se na true */
    var $editovatZpracovano;
    /** @var int */
    var $zpracovanoDo;
    /** @var string */
    var $zpracoval;

    /** @var bool */
    var $editovatZkontrolovano;
    /** @var int */
    var $zkontrolovanoDo;
    /** @var string */
    var $zkontroloval;

    /** @var bool */
    var $editovatSchvaleno;
    /** @var int */
    var $schvalenoDo;
    /** @var string */
    var $schvalil;

    /** @var string */
    var $poznamka;


    function __construct($database)
    {
        $this ->database = $database;

        if (isset($_POST['rozdeleni'])) $this->kodRozdeleni = $_POST['rozdeleni'];
        else if (isset($_GET['rozdeleni'])) $this->kodRozdeleni = $_GET['rozdeleni'];
        else $this->kodRozdeleni = '';

        $this->typRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);

        if (isset($_POST['rozdeleni2'])) $this->kodRozdeleni2 = $_POST['rozdeleni2'];
        else if (isset($_GET['rozdeleni2'])) $this->kodRozdeleni2 = $_GET['rozdeleni2'];
        else $this->kodRozdeleni2 = '';

        $this->typRozdeleni2 = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP);
    }


    /**
     * @param int $obdobi
     * @throws \Exception
     */
    function nastavObdobi($obdobi)
    {
        $this->obdobi = new Obdobi();

        if (!$this->obdobi->nactiObdobi($this ->database, 'ADS', $obdobi)) {
            // nemelo by nastat
            throw new \Exception('Chyba! Nelze nacist obdobi!');
        }

        if ($this->obdobi->uzavreno) {
            throw new \Exception(\Ron\ronText('Obdobi_uzavreno'));
        }
        $this->obdobi->nastavSession();

        Prava::nastavObdobiProPrava($this->obdobi->zacatek, $this->obdobi->konec);
    }


    /**
     * @param Cursor $cursor
     * @return bool
     */
    function nactiZaznamySchvaleni($cursor)
    {
        $this->editovatZpracovano = false;
        $this->editovatZkontrolovano = false;
        $this->editovatSchvaleno = false;

        $prava1 = Prava::vratPrava('zpracovani');
        $prava2 = Prava::vratPrava('zkontrolovani');
        $prava3 = Prava::vratPrava('schvaleni');

        $first = true;
        while ($row = $cursor ->fetch()) {
            if ($first) {
                // prvni nacteny zaznam - inicializuje hodnoty
                if ($row[0]) $this->zpracovanoDo = $this->database ->stringToTimestamp($row[0]);
                $this->zpracoval = $row[1];

                if ($row[2]) $this->zkontrolovanoDo = $this->database ->stringToTimestamp($row[2]);
                $this->zkontroloval = $row[3];

                if ($row[4]) $this->schvalenoDo = $this->database ->stringToTimestamp($row[4]);
                $this->schvalil = $row[5];

                $this->poznamka = $row[6];

                $first = false;
            } else {
                // dalsi nacteny zaznam - porovna hodnoty a pokud nejsou stejne tak je "vypne"
                if (!$row[0] || $this->database ->stringToTimestamp($row[0]) != $this->zpracovanoDo) {
                    $this->zpracovanoDo = null;
                }
                if ($row[1] != $this->zpracoval) $this->zpracoval = '';


                if (!$row[2] || $this->database ->stringToTimestamp($row[2]) != $this->zkontrolovanoDo) {
                    $this->zkontrolovanoDo = null;
                }
                if ($row[3] != $this->zkontroloval) $this->zkontroloval = '';


                if (!$row[4] || $this->database ->stringToTimestamp($row[4]) != $this->schvalenoDo) {
                    $this->schvalenoDo = null;
                }
                if ($row[5] != $this->schvalil) $this->schvalil = '';

                if ($row[6] != $this->poznamka) $this->poznamka = '';
            }

            // kontrola prav
            if (!$this->editovatZpracovano) if ($prava1->lzeZobrazitPP($row[7])) $this->editovatZpracovano = true;
            if (!$this->editovatZkontrolovano) if ($prava2->lzeZobrazitPP($row[7])) $this->editovatZkontrolovano = true;
            if (!$this->editovatSchvaleno) if ($prava3->lzeZobrazitPP($row[7])) $this->editovatSchvaleno = true;
        }

        return true;
    }



    /**
     * Nacte seznam osob a prislusnych zaznamu schvaleni
     * @return Cursor
     * @throws \Ron\DB\DBException
     */
    function vratZaznamy()
    {
        $sql =
            'SELECT SchvaleniDochazky.ZpracovanoDo, SchvaleniDochazky.Zpracoval, ' . // 0,1
            '   SchvaleniDochazky.ZkontrolovanoDo, SchvaleniDochazky.Zkontroloval, ' . // 2,3
            '   SchvaleniDochazky.SchvalenoDo, SchvaleniDochazky.Schvalil, ' . // 4,5
            '   {fn_CASTWIDEMEMO(SchvaleniDochazky.Poznamka)}, PracovniPomer.Id, ' . // 6,7
            '   PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu ' . //8..10
            ' FROM PracovniPomer LEFT OUTER JOIN SchvaleniDochazky ' .
            '   ON SchvaleniDochazky.RC = PracovniPomer.RC ' .
            '  AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '  AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .

            " AND SchvaleniDochazky.Aplikace = 'ADS' " .
            ' AND SchvaleniDochazky.PocatekObdobi = ' . $this->database ->timestampToString($this->obdobi->zacatek) .

            ' WHERE PracovniPomer.Zamestnanec > 0 ' .
            '   AND PracovniPomer.ZpracovatDochazku = 1 ' .
            '   AND PracovniPomer.DatumNastupu < ' . $this->database ->timestampToString($this->obdobi->konec) . // jen platne PP
            '   AND ( PracovniPomer.DatumUkonceni >= ' . $this->database ->timestampToString($this->obdobi->zacatek) .
            ' OR PracovniPomer.DatumUkonceni IS NULL )';

        if ($this->kodRozdeleni) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                $this->typRozdeleni, $this->kodRozdeleni,
                $this->obdobi->zacatek, $this->obdobi->konec, true); // vcetnepodrizenych = true

            if ($filtr) $sql .= ' AND (' . $filtr . ')';
        }

        if ($this->kodRozdeleni2) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                $this->typRozdeleni2, $this->kodRozdeleni2,
                $this->obdobi->zacatek, $this->obdobi->konec, true); // vcetnepodrizenych = true

            if ($filtr) $sql .= ' AND (' . $filtr . ')';
        }

        return $this ->database ->getSqlResult($sql, null, 'Chyba pri nacitani schvaleni!');
    }


    /**
     * Nacte hodnoty pro osoby z rozdeleni a vybrane obdobi. Pokud jsou hodnoty ruzne, tak je nezobrazuje
     * @return bool
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nactiData()
    {
        $this->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        $data = $this->vratZaznamy();

        $this ->rozdeleni = ($this ->typRozdeleni  &&  $this ->kodRozdeleni)
            ? Rozdeleni::loadRozdeleni($this ->database, $this ->typRozdeleni, $this ->kodRozdeleni)
            : null;

        $this ->rozdeleni2 = ($this ->typRozdeleni2  &&  $this ->kodRozdeleni2)
            ? Rozdeleni::loadRozdeleni($this ->database, $this ->typRozdeleni2, $this ->kodRozdeleni2)
            : null;

        return $this->nactiZaznamySchvaleni($data);
    }



    function vlozPolickoSDatem($caption, $captionclass, $name, $value, $username, $editace, $datumPredplneni)
    {
        $checked = isset($_POST[$name . '_check']) ? $_POST[$name . '_check'] : false;
        $error = MessageManager::getInstance() ->isError($name);

        echo "<div class='form-row' id='hromadneschvaleni_$name'><div class='form-col1 caption $captionclass'><label>$caption";
        if ($editace) {
            $checkedstr = $checked ? "checked='checked'" : '';
            echo "<INPUT type='checkbox' name='{$name}_check' $checkedstr>";
        }
        echo "</label></div>";

        echo '<div class="form-col2">';
        if ($value) $valuestr = date('j.n.Y', $value);
        else $valuestr = '';

        if ($editace) {
            $classname = $error ? 'app_input_error' : 'app-input';

            if (isset($_POST[$name])) {
                $valuestr = StringUtils::htmlTextConvert($_POST[$name]);
            }

            echo
                '<input type="text" name="' . $name . '" value="' . $valuestr . '" class= "' . $classname . '" maxlength="10" size="10" '.
                ' data-roncalendar-mindate="' . date('j.n.Y', $this ->obdobi ->zacatek) . '" ' .
                ' data-roncalendar-maxdate="' . date('j.n.Y', $datumPredplneni) . '" ' .
                '>';
            echo
                '<i class="showcalendar inline-glyph fa glyph-calendar glyph-button"  data-roncalendar-inputname="' . $name . '"></i>';

            // odkaz pro predplneni do konce obdobi/posledniho zpracovani
            if ($datumPredplneni > 0) {
                $datumPredplneniStr = date('j.n.Y', $datumPredplneni);
                echo
                    " &nbsp;&nbsp; <a href='#' class='predplnit' data-inputname='$name' data-value='$datumPredplneniStr'>" . $datumPredplneniStr . '</a>';
            }
        } else {
            echo $valuestr;
        }

        echo ' ' . StringUtils::htmlTextConvert($username) . '</div>';

        echo "</div>\n";
    }



    function render()
    {
        require('view/schvalenidochazkyhromadne.php');
    }



    /**
     * @param string $postName
     * @param string $errorStr
     * @param int $maximalnidatum
     * @return int|null|false   Kontrola datumu zadaneho uzivatelem (format, rozsah) - vraci zadane datum nebo false v pripade chyby
     */
    function zkontrolujDatum($postName, $errorStr, $maximalnidatum) {
        if ($_POST[$postName]) {
            try {
                $datum = DateUtils::dateStringToTimestamp($_POST[$postName]);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($errorStr, $postName);
                return false;
            }

            // kontrola rozsahu
            if ($datum < $this->obdobi->zacatek || $this->zpracovanoDo > $maximalnidatum) {
                MessageManager::addErrorMessage(
                    $errorStr . ' ' .
                    sprintf(\Ron\ronText('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $this->obdobi->zacatek),
                        date('j.n.Y', $maximalnidatum)),
                    $postName
                );

                return false;
            }
        } else {
            $datum = null;
        }

        return $datum;
    }


    /**
     * Prevede zadane retezcove udaje ve formulari na timestampy. Pokud je neco zadano spatne tak vraci text false.
     * @return bool
     */
    function zkontrolujZadaneUdaje()
    {
        $maximalniDatum = $this->datumZpracovaniDochazky;
        if ($maximalniDatum > $this->obdobi->konec) $maximalniDatum = $this->obdobi->konec;
        $maximalniDatum = DateUtils::startOfTheDay($maximalniDatum);

        // nacte zadane hodnoty a zkontroluje je
        $this->poznamka = $_POST['poznamka'];

        $this->zpracovanoDo = $this ->zkontrolujDatum('zpracovanodo', \Ron\ronText('Nespravne_datum_schvaleni1'), $maximalniDatum);
        if ( $this->zpracovanoDo === false ) {
            return false;
        }

        // kontrola "odschvaleni" az u konkretni osoby!

        $this->zkontrolovanoDo = $this ->zkontrolujDatum('zkontrolovanodo', \Ron\ronText('Nespravne_datum_schvaleni2'), $maximalniDatum);
        if ( $this->zkontrolovanoDo === false ) {
            return false;
        }

        $this->schvalenoDo = $this ->zkontrolujDatum('schvalenodo', \Ron\ronText('Nespravne_datum_schvaleni1'), $maximalniDatum);
        if ( $this->schvalenoDo === false ) {
            return false;
        }

        return true;
    }


    /**
     * @param array $row
     * @return bool
     */
    function zapisSchvaleniOsoby($row)
    {
        $rec = new SchvaleniDochazkyRecord($this ->database, $this);

        $pp = PracovniPomer::nactiPP($this ->database, $row[7]);

        $exists = $rec ->loadDataProOsobu($pp, $this->obdobi);

        $rec ->addField('', '', true, 0, StringUtils::fromUTF8('   Hromadné schválení pro rozdělení: ') . $this->kodRozdeleni);
        if ($this ->typRozdeleni2) {
            $rec ->addField('', '', true, 0, StringUtils::fromUTF8('   rozdělení 2: ') . $this->kodRozdeleni2);
        }

        $prava1 = Prava::vratPrava('zpracovani');
        $editovatzpracovano = isset($_POST['zpracovanodo_check'])  &&  $_POST['zpracovanodo_check']  &&  $prava1 ->lzeZobrazitPP($row[7]);

        $prava2 = Prava::vratPrava('zkontrolovani');
        $editovatzkontrolovano = isset($_POST['zkontrolovanodo_check'])  &&  $_POST['zkontrolovanodo_check']  &&  $prava2 ->lzeZobrazitPP($row[7]);

        $prava3 = Prava::vratPrava('schvaleni');
        $editovatschvaleno = isset($_POST['schvalenodo_check'])  &&  $_POST['schvalenodo_check']  &&  $prava3 ->lzeZobrazitPP($row[7]);

        $zapsat = false;

        $msgPrefix = $pp->jmenoPrijmeni . ' (' . $pp->osobniCislo . '): ';

        if ($editovatzpracovano  &&  $this ->zpracovanoDo != $rec ->fZpracovano ->originalValue) {
            // doslo ke zmene schvaleni 1
            if (!$rec->nastavPoleSchvaleni(1, $this->zpracovanoDo, $msgPrefix)) return false;
            $zapsat = true;
        }


        if ($editovatzkontrolovano  &&  $this ->zkontrolovanoDo != $rec ->fZkontrolovano ->originalValue) {
            // doslo ke zmene schvaleni 2
            if (!$rec->nastavPoleSchvaleni(2, $this->zkontrolovanoDo, $msgPrefix)) return false;
            $zapsat = true;
        }

        if ($editovatschvaleno  &&  $this ->schvalenoDo != $rec ->fSchvaleno ->originalValue) {
            // doslo ke zmene schvaleni 3
            if (!$rec->nastavPoleSchvaleni(3, $this->schvalenoDo, $msgPrefix)) return false;
            $zapsat = true;
        }


        if (!$zapsat) return true; // neni co menit

        if (isset($_POST['poznamka_check'])) if ($_POST['poznamka_check']) {
            // poznamku meni jen v pripade, ze ma pravo alespon na jedno pole
            $rec ->fPoznamka ->setNewValue($_POST['poznamka']);
        }

        return $rec ->zkontrolujAZapisSchvaleni(!$exists, $msgPrefix);
    }



    // ----------------------------------------------------------------------------
    // Zapise zadane hodnoty. V pripade uspechu vraci true.
    function zapisData()
    {
        $this->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        if (!$this->zkontrolujZadaneUdaje()) return false;

        // nacte seznam osob (idpp)
        $cursor = $this->vratZaznamy();

        $result = true;

        $rows = [];
        while ($row = $cursor ->fetch()) {
            $rows[] = $row;
        }

        foreach ($rows as $row) {
            //$editovat = true;
            try {
                $this->zapisSchvaleniOsoby($row);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $result = false;
            }
        }

        return $result;
    }
}
