<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;


class MesicniVysledkyBrowser1 extends VysledkyBrowser
{
    var $showFilter; // ve zpracovane dochazce se filtr nezobrazi

    function __construct($database)
    {
        parent::__construct($database, 'mesicnivysledky');

        $this->showFilter = true;
        $this->code = 'browsermesicni';

        $this ->title = \Ron\ronText('Mesicni_vysledky');

        $this ->sqlSelectPart =
            'MesicniVysledekVypoctu.TypCasoveSlozkyMzdy, TypCasoveSlozkyMzdy.Popis, ' . // 0,1
            ' MesicniVysledekVypoctu.Hodiny, MesicniVysledekVypoctu.Dny, MesicniVysledekVypoctu.KalendarniDny,' . // 2..4
            ' MesicniVysledekVypoctu.Poznamka'; // 5

        $this->sqlFromPart = 'MesicniVysledekVypoctu, TypCasoveSlozkyMzdy';

        $this->sqlWherePart = 'TypCasoveSlozkyMzdy.Kod = MesicniVysledekVypoctu.TypCasoveSlozkyMzdy';

        $this->cssClass .= ' mesicnivysledky';

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.TypCasoveSlozkyMzdy', 0)
            ->setCode('col-kod')
            ->setHeaderText('Kod_casove_slozky') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('TypCasoveSlozkyMzdy.Popis', 1)
            ->setCode('col-popis')
            ->setHeaderText('Casova_slozka') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Hodiny', 2)
            ->setCode('col-hodiny')
            ->setHeaderText('Vysledek_Hodiny') ->setFormat('hodiny') ->setAlign('right') ->setOrdering(1) ->setClassName('hodiny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Dny', 3)
            ->setCode('col-dny')
            ->setHeaderText('Vysledek_Pracovni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1) ->setClassName('dny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.KalendarniDny', 4)
            ->setCode('col-kalendarni-dny')
            ->setHeaderText('Vysledek_Kalendarni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1) ->setClassName('kalendarnidny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Poznamka', 5)
            ->setCode('col-poznamka') ->setVisible(false) ->setHeaderText('Poznamka');

        $this->defaultSort = 1;
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        if ($this->showFilter) {
            parent::setFilterForm();

            $this->filterForm->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
            $this->filterForm ->nastavAkciGet('mesicnivysledky');
            $this->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        }
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->browserUrl = '?menuid=mesicnivysledky&idpp=' . $this ->filterForm ->vybranaOsoba . '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->vybraneObdobiOd);
    }



    function renderFilter()
    {
        if ($this->showFilter) {
            $this->odkazyMesicniVysledky('mesicnivysledky');
            parent::renderFilter();
        }
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        return new SqlStatement(
            'MesicniVysledekVypoctu.Mesic = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND MesicniVysledekVypoctu.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND MesicniVysledekVypoctu.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND MesicniVysledekVypoctu.DatumNastupu = ' . $params->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }


    function enableLine($values)
    {
        if (!RonPortalSettings::getIntValue('skrytnulovemesicni')) return true;

        return (float)$values[2] || (float)$values[3] || (float)$values[4];
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        // prida class mv_<kod_casove_slozky>
        $mvClass = 'mv_' . StringUtils::htmlTextConvert(str_replace(' ', '_', $values[0]));
        $className = ($className ? $className . ' ' : '') . $mvClass;

        return parent::getRowAttributes($values, $className, $rowNumber);
    }

}
