<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;
use RonPortal\RozdeleniUtils;
use RonPortal\User;

class MonitorovaniFilterForm extends RonPortalFilterForm
{
    const SETTINGS_JEN_HLAVNI_ROZDELENI = 'monitorovani_jennejvyssirozdeleni';

    public $aktualniStav;
    public $datumCas;

    public function __construct($database, $action, $kontrolovatprava)
    {
        parent::__construct($database, $action);

        $this ->nastavAkciGet($action);

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $jenvlastni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_JEN_VLASTNI);
            if (!User::getRC()) {
                $jenvlastni = 0; // pro monitorovani bez prihlaseni
            }

            $filtr = RonPortalSettings::getIntValue(self::SETTINGS_JEN_HLAVNI_ROZDELENI) ? 'Rodic IS NULL' : '';
            $this ->nastavRozdeleni(true, 0, $jenvlastni, $filtr); // typRozdeleni=0, jinak by se musel zadavat i v RonPortalSession::rozdeleni()!

            if ($kontrolovatprava) {
                $this->f_pravarozdeleni = Prava::vratPrava('monitorovani');
            }
        }

        if (isset($_GET['datum'])) {
            $this->aktualniStav = (isset($_GET['aktualnistav'])  &&  $_GET['aktualnistav']) ? 1 : 0;
        } else {
            $this->aktualniStav = 1;
        }

        // Pri prvnim zobrazeni stranky (ne po volbe "nastavit") nastavi vzdy aktualni stav na true
        // Nelze tady testovat $_GET["aktualnistav"] protoze pri vypnuti checkboxu je NULL,
        // proto testuji "datum"

        if (isset($_GET['datum'])) {
            try {
                $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
                $timeStamp = DateUtils::timeStringToTimestamp($_GET['cas'], $datum);
            } catch(Exception $e) {
                MessageManager::addErrorText( $e ->getMessage() );
                $timeStamp = time();
            }
        } else {
            $timeStamp = time();
        }

        $this ->datumCas = $timeStamp;
        $_SESSION['monitorovatcas'] = $timeStamp;

        if (isset($_GET['operace'])) {
            if ($_GET['operace'] !== '') {
                $_SESSION['monitorovat_operaci'] = $_GET['operace'];
            } else {
                unset($_SESSION['monitorovat_operaci']);
            }
        }

        if (isset($_GET['terminal'])) {
            if ($_GET['terminal'] !== '') {
                $_SESSION['monitorovat_terminal'] = $_GET['terminal'];
            } else {
                unset($_SESSION['monitorovat_terminal']);
            }
        }
    }


    protected function renderAktualniStav()
    {
        // Checkbox na zaskrtnuti "aktualni stav"
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . TextManager::_('Aktualni_stav') . '</span>';
        GuiUtils::renderCheckbox('aktualnistav', '', $this->aktualniStav);

        if (!$this->aktualniStav) {
            echo
                '<span class="margin-left-2">' .
                "<a href='" . $this ->f_formaction . "'>" . TextManager::_('Aktualni_stav') . "</a>" .
                '</span>';
        }

        echo "</div>\n";
    }


    protected function renderDatumCas()
    {
        $timeStamp = $this ->datumCas; // nastaveno v konstruktoru podle GET
        $cas = date('G:i', $timeStamp);

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . TextManager::_('Datum_a_cas') . '</span>';
        DateInput::renderDateSelect('datum', date('j.n.Y', $timeStamp), true); // submitonenter = true
        echo '&nbsp;&nbsp;';
        echo '<input type="text" name="cas" value="' . $cas . '" maxlength="8" size="5">';
        echo "</div>\n";
    }

    protected function renderOperace()
    {
        $sql = "SELECT Kod, Popis FROM OperaceDochazky WHERE Systemova <> 1 AND PritomnostProMonitorovani <> 3";

        $comboOperacePopis = new ComboBox('operace_popis');
        $comboOperacePopis->className = 'adsfilterforminput_popis';
        $comboOperacePopis->htmlParams = 'data-ron-select2="operace" data-ron-autosubmit="1"';
        $comboOperacePopis->emptyRow = TextManager::_('_vse_');

        $comboOperace = new ComboBox('operace');
        $comboOperace->captionIndex = 0;
        $comboOperace->className = 'adsfilterforminput_kod';
        $comboOperace->htmlParams = 'data-ron-select2="operace_popis" data-ron-autosubmit="1"';
        $comboOperace->emptyRow = TextManager::_('_vse_');

        try {
            $zvolenaOperace = isset($_SESSION['monitorovat_operaci']) ? $_SESSION['monitorovat_operaci'] : '';
            $comboOperacePopis->loadData($this->database, $sql . "ORDER BY Popis", $zvolenaOperace);
            $comboOperace->loadData($this->database, $sql . "ORDER BY Kod", $zvolenaOperace);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage("Chyba pri nacitani operaci. " . $exception->getMessage());
        }

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . TextManager::_('Operace') . '</span>';

        $comboOperacePopis->render();
        $comboOperace->render();

        echo "</div>\n";
    }

    protected function renderTerminal()
    {
        $sql = "SELECT Kod, Popis FROM Terminal WHERE Aktivni = 1";

        $comboTerminalPopis = new ComboBox('terminal_popis');
        $comboTerminalPopis->className = 'adsfilterforminput_popis';
        $comboTerminalPopis->htmlParams = 'data-ron-select2="terminal" data-ron-autosubmit="1"';
        $comboTerminalPopis->emptyRow = TextManager::_('_vse_');

        $comboTerminal = new ComboBox('terminal');
        $comboTerminal->captionIndex = 0;
        $comboTerminal->className = 'adsfilterforminput_kod';
        $comboTerminal->htmlParams = 'data-ron-select2="terminal_popis" data-ron-autosubmit="1"';
        $comboTerminal->emptyRow = TextManager::_('_vse_');

        try {
            $zvolenyTerminal = isset($_SESSION['monitorovat_terminal']) ? $_SESSION['monitorovat_terminal'] : '';
            $comboTerminalPopis->loadData($this->database, $sql . "ORDER BY Popis", $zvolenyTerminal);
            $comboTerminal->loadData($this->database, $sql . "ORDER BY Kod", $zvolenyTerminal);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage("Chyba pri nacitani terminalu. " . $exception->getMessage());
        }

        echo '<div class="filterrow">';
        echo '<span class="filtercaption">' . TextManager::_('Terminal') . '</span>';

        $comboTerminalPopis->render();
        $comboTerminal->render();

        echo "</div>\n";
    }

    public function renderFilters()
    {
        $this->renderAktualniStav();
        $this->renderDatumCas();
        $this->renderOperace();
        $this->renderTerminal();

        parent::renderFilters(); // rozdeleni
    }
}




class Monitorovani_FotoField extends BrowserField
{
    public $browserAction;

    /** @var  boolean true = zobrazi foto ve strance, false = zobrazi odkaz na zobrazeni fota */
    private $showFoto;

    /**
     * Monitorovani_FotoField constructor.
     * @param string $action menuid action - monitorovani muze byt zobrazeno s prihlasenim i bez
     * @param bool $showFoto
     */
    public function __construct($action, $showFoto = false)
    {
        parent::__construct();
        $this->align = 'center';

        $this->browserAction = $action;
        $this->showFoto = $showFoto;
    }


    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            if ($this->showFoto) {
                return '<div class="browserfoto ronFoto">
                            <img src="?menuid=' . $this->browserAction . '&fotopp=' . (int)$rowData[0] . '" alt="foto osoby"/>
                        </div>';
            }

            return '<a target="ads_foto" class="link_foto" href="?menuid=' . $this->browserAction . '&fotopp=' . (int)$rowData[0] . '">' . TextManager::_('Foto') . '</a>';
        }

        return '';
    }
}

class Email_BrowserField extends BrowserField
{
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            return "<a href='mailto:$value'>$value</a>";
        }

        return '';
    }
}

class MonitorovaniButton_BrowserField extends BrowserField
{
    private $osobaIDnumber;
    private $nastaveniSloupce;

    /**
     * @param int $osobaIDnumber
     * @param array|null $nastaveniSloupce
     */
    public function __construct($osobaIDnumber, $nastaveniSloupce)
    {
        parent::__construct();
        $this->osobaIDnumber = $osobaIDnumber;
        $this->nastaveniSloupce = $nastaveniSloupce;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $osoba = $rowData[$this->osobaIDnumber];
        $buttonID = $this->nastaveniSloupce['buttonid'];
        $action = $this->nastaveniSloupce['typ'];
        $nadpis = $this->nastaveniSloupce['nadpis'];

        $prava = null;

        if ($this->nastaveniSloupce['typ'] === 'vlozeniabsence') {
            $prava = Prava::vratPrava('absence', 'absence_' . $this->nastaveniSloupce['operace']);
        }
        if ($this->nastaveniSloupce['typ'] === 'vlozenidochazky') {
            $prava = Prava::vratPrava('vkladani');
        }
        if ($this->nastaveniSloupce['typprava']) {
            $prava = Prava::vratPrava($this->nastaveniSloupce['typprava']);
        }

        Prava::nastavObdobiProPrava(DateUtils::today());

        if ($prava && !$prava ->lzeZobrazitPP($osoba)) {
            return '';
        }

        return
            "<form class='submitWorkingInfo sendPosition' method='POST' action='?menuid=monitorovani'>
                <input type='hidden' name='action' value='$action' />
                <input type='hidden' name='buttonid' value='$buttonID' />
                <input type='hidden' name='osoba' value='$osoba' />
                <input type='hidden' name='x' />
                <input type='hidden' name='y' />
                <input type='submit' class='submitbutton' value='$nadpis' />
            </form>";
    }
}

class RozdeleniIMG_BrowserField extends BrowserField
{
    public $typRozdeleni;

    public function __construct($database, $kodTypuRozdeleni)
    {
        parent::__construct($database);

        $this->typRozdeleni = $kodTypuRozdeleni;
    }

    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idPP = $rowData[0];

        $ikony = RozdeleniOsob::nactiIkonyRozdeleni($this->database, $idPP, $this->typRozdeleni, time(), time());

        return RozdeleniOsob::getIkonyHtml($ikony);
    }
}


class MonitorovaniSloupecRozdeleni
{
    /** @var  int kod typu rozdeleni */
    public $kodTypu;
    /** @var  string kod, popis, vedoucipopis, vedouci, zastupce rozdeleni */
    public $typ;
    /** @var  int index sloupce v dotazu */
    public $indexSloupce;

    /**
     * MonitorovaniSloupecRozdeleni constructor.
     * @param int $kodTypu kod typu rozdeleni
     * @param string $typ kod, popis, vedoucipopis, vedouci, zastupce rozdeleni
     */
    public function __construct($kodTypu, $typ)
    {
        $this->kodTypu = $kodTypu;
        $this->typ = $typ;
    }
}


class MonitorovaniPocetDleOperace
{
    /** @var string kod operace */
    public $kod;
    /** @var string popis operace */
    public $popis;
    /** @var int */
    public $pocetPritomno;
    /** @var int */
    public $pocetNepritomno;
    /** @var int */
    public $pocetCelkem;

    /**
     * @param MonitorovaniPocetDleOperace[] $pole
     */
    public static function seradOperace(&$pole)
    {
        uasort($pole, 'RonPortal\Dochazka\MonitorovaniPocetDleOperace::porovnejOperace');
    }

    /**
     * @param MonitorovaniPocetDleOperace $operace1
     * @param MonitorovaniPocetDleOperace $operace2
     * @return int
     */
    public static function porovnejOperace($operace1, $operace2)
    {
        return strnatcasecmp($operace1->popis, $operace2->popis);
    }
}

/**
 * Class MonitorovaniOsobBrowser
 * @package Ron\RonPortal
 */
class MonitorovaniOsobBrowser extends RonPortalBrowser
{
    const SETTINGS_SLOUPCE = 'sloupcemonitorovani';
    const SETTINGS_TRIDIT = 'triditmonitorovani';
    const SETTINGS_POUZIT_RYCHLE = 'pouzitrychlemonitorovani';

    var $zobrazovaneSloupce;
    var $editovat;
    //var $aktualniStav;

    var $kontrolovatPrava;
    /** @var  boolean nastavi se na true, pokud se ma zobrazovat sloupec s rozdelenim osoby */
    var $zobrazitRozdeleni;

    /** @var  int  Pocet sloupcu, ktere jsou v zakladnim sql dotazu (osoba, pracovni pomer) <br/>
    U rychleho monitorovani jsou za nima dalsi sloupce - jejich pozice se
    pocita relativne vzhledem k teto hodnote */
    var $pocetSloupcuSql;


    var $indexPoleProTrideni;

    /** @var MonitorovaniFilterForm filterForm */
    var $filterForm;

    /** @var MonitorovaniSloupecRozdeleni[] seznam pouzitych kodu typu rozdeleni */
    private $kodyTypuRozdeleni = [];

    public $nastaveniSloupcuXML;

    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->title = TextManager::_('Monitorovani');

        $this->kontrolovatPrava = $kontrolovatprava = $action !== 'monitorovaninolog';
        $this ->cssClass .= ' monitorovani oddEvenRows';

        // Nastavi pole $this->f_sql_fields, $this->f_fieldsformat a $this->f_header
        // podle nastaveni ulozeneho v databazi
        $this->indexPoleProTrideni = -1;

        $this->nactiNastaveni();

        $this->sestavHlavicku();
    }

    /**
     * @throws DBException
     * @throws Exception
     */
    protected function nactiNastaveni()
    {
        $text = RonPortalSettings::getBvarSettings($this->database, 'RonPortal_monitorovani_nastaveni_xml');

        if ($text === '') {
            $str = RonPortalSettings::getValue(self::SETTINGS_SLOUPCE);
            $header = [];
            StringUtils::parseStringToArrays($str, $header, $sloupce, true);

            foreach ($sloupce as $i => $sloupec) {
                $sloupecNastaveni = [];
                $sloupecNastaveni['typ'] = $sloupec;
                $sloupecNastaveni['nadpis'] = $header[$i];

                $rozdeleni = explode('_', $sloupec);

                if (($rozdeleni[0] === 'r') || ($rozdeleni[0] === 'rkod') || ($rozdeleni[0] === 'rvedoucipopis') || ($rozdeleni[0] === 'rvedouci') || ($rozdeleni[0] === 'rzastupce')) {
                    $sloupecNastaveni['typrozdeleni'] = $rozdeleni[1];

                    switch ($rozdeleni[0]) {
                        case 'r':
                            $sloupecNastaveni['typ'] = 'rozdp';
                            break;
                        case 'rkod':
                            $sloupecNastaveni['typ'] = 'rozd';
                            break;
                        case 'rvedoucipopis':
                            $sloupecNastaveni['typ'] = 'rozdvedoucipopis';
                            break;
                        case 'rvedouci':
                            $sloupecNastaveni['typ'] = 'rozdvedouci';
                            break;
                        case 'rzastupce':
                            $sloupecNastaveni['typ'] = 'rozdzastupce';
                            break;
                    }
                }

                if (strpos($sloupec, 'uzivatelska') !== false) {
                    $sloupecNastaveni['typ'] = 'uzivatelska';
                    $sloupecNastaveni['index'] = substr($sloupec, -1);
                }

                $this->zobrazovaneSloupce[] = $sloupecNastaveni;
            }

            return;
        }

        $loader = new XmlLoader();
        $loader->includeAttributes = true;
        $loader->prefixToSubarray = true;
        $loader->ignorePrefixes = true;
        $loader->convertFromUtf8 = true;
        $this->nastaveniSloupcuXML = $loader->readXml($text);

        $this->zobrazovaneSloupce = $this->nastaveniSloupcuXML['sloupecmonitorovani'];
    }

    public function setFilterForm()
    {
        $this->filterForm = new MonitorovaniFilterForm($this->database, $this ->action, $this->kontrolovatPrava);
    }


    public function load($sqlFilter = '')
    {
        if (isset($_GET['monitorovatco'])) {
            $_SESSION['comonitorovat'] = $_GET['monitorovatco'];
        }

        parent::load();
        //$this->aktualniStav = $this ->filterForm ->aktualniStav;
    }


    protected function zobrazPosledniNacteni()
    {
        $sql = 'SELECT MAX(CasPoslednihoNacteni) FROM StavTerminalu';
        try {
            $cursor = $this->database->select($sql);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage('Chyba pri nacitani casu posledniho nacteni. ' . $exception->getMessage());
        }

        $row = isset($cursor) ? $cursor ->fetch() : null;

        if ($row) {
            echo
                '<div id="monitorovani_posledninacteni" class="align-right">' .
                TextManager::_('Cas_posledniho_nacteni') .
                ' <strong>' . DBUtils::getFormatedValue($this ->database, 'datetime', $row[0]) .
                "</strong></div>\n";
        }
    }


    protected function renderFilterPritomnost()
    {
        $data = [
            0 => TextManager::_('Pritomni'),
            1 => TextManager::_('Nepritomni'),
            2 => TextManager::_('Vsichni')
        ];

        LinksList::renderLinks($data,
            isset($_SESSION['comonitorovat']) ? $_SESSION['comonitorovat'] : 0,
            '?menuid=' . $this ->action . '&monitorovatco=');
    }


    public function renderFilter()
    {
        $this->zobrazPosledniNacteni();
        $this->renderFilterPritomnost();

        parent::renderFilter();
    }


    /**
     * Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka. Z nich vybere ten posledni.
     * @param int $datum
     * @param string $rc
     * @param string $osobnicislo
     * @param int $datumnastupu
     * @return array|bool|null
     * @throws DBException
     */
    protected function vratPosledniPruchodSQL($datum, $rc, $osobnicislo, $datumnastupu)
    {
        return DochazkaUtils::vratPosledniPruchodSQL($this->database, $datum, $rc, $osobnicislo, $datumnastupu);
    }


    /**
     *  Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka. Z nich vybere ten posledni.
     * @param int $datum
     * @param array $row
     * @return array|bool|null
     * @throws Exception
     */
    protected function vratPosledniPruchod($datum, $row)
    {
        if ($this->filterForm ->aktualniStav && RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE)) {

            $result = [];

            // datum - u absence vyplni i datum do
            if ($row[$this->pocetSloupcuSql + 8]) {
                // pokud je vyplneny datum do, je to absence
                $result[] = DBUtils::getFormatedValue($this ->database, 'date', $row[$this->pocetSloupcuSql]) . ' - ' . DBUtils::getFormatedValue($this ->database, 'date', $row[$this->pocetSloupcuSql + 8]);
            } else {
                $result[] = DBUtils::getFormatedValue($this ->database, 'datetime', $row[$this->pocetSloupcuSql]);
            }


            for ($i = 1; $i < 4; $i++) {
                $result[] = $row[$this->pocetSloupcuSql + $i]; // terminal, poznamka, popis operace
            }

            // pritomnost
            $result[] = ($this ->database ->stringToTimestamp($row[$this->pocetSloupcuSql]) < DateUtils::incDay($datum, -1))
                ? false
                : $row[$this->pocetSloupcuSql + 4];

            $result[] = $row[$this->pocetSloupcuSql + 5]; // umisteni

            $result[] = $row[$this->pocetSloupcuSql + 6]; // rozdeleni
            $result[] = $row[$this->pocetSloupcuSql + 7]; // popis rozdeleni
            $result[] = $row[$this->pocetSloupcuSql + 9]; //kod operace

            return $result;
        }

        return $this->vratPosledniPruchodSQL($datum, $row[0], $row[1], $this ->database ->stringToTimestamp($row[2]));
    }


    /**
     * Sestavi hlavicku a seznam poli podle nastaveni
     */
    protected function sestavHlavicku()
    {
        $this ->addField(BrowserField::FIELD_TYPE_LINE);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN) ->setSqlFieldName('PracovniPomer.Id');
        $hlpcnt = 1;

        $this ->zobrazitRozdeleni = false;

        foreach ($this->zobrazovaneSloupce as $i => $sloupec) {
            $newfield = null;

            $zobrazitSloupec = true;
            if (Prava::existujePravo('monitorovani_' . $sloupec['typ']) ) {
                $zobrazitSloupec = Prava::jsouNejakePrava('monitorovani_' . $sloupec['typ']);
            }
            if (isset($sloupec['typprava']) && $sloupec['typprava']) {
                $zobrazitSloupec = $zobrazitSloupec && Prava::jsouNejakePrava($sloupec['typprava']);
            }

            if ($zobrazitSloupec) {
                switch (strtolower($sloupec['typ'])) {
                    case 'rc' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.RC');
                        break;
                    case 'osobni' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo') ->setAlign('right');
                        break;
                    case 'datumnastupu' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.DatumNastupu') ->setFormat('date') ->setAlign('center');
                        break;
                    case 'cislosmlouvy' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.CisloPracovniSmlouvy') ->setAlign('right');
                        break;
                    case 'kodtypupomeru' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.TypPracovnihoPomeru');
                        break;
                    case 'datumukonceni' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.DatumUkonceni') ->setFormat('date') ->setAlign('center');
                        break;
                    case 'jmeno' :
                        $newfield = $this->addField() ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno');
                        break;
                    case 'prijmeni' :
                        $newfield = $this->addField() ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni');
                        //$this->f_defaultsort=$i + 2; // cislovani sloupcu v browseru je od 1, navic preskoci line
                        break;
                    case 'celejmeno' :
                        $newfield = $this->addField() ->setSqlFieldName('Osoba.Prijmeni, Osoba.Jmeno') ->setHeaderText('Jmeno_a_prijmeni');
                        //$this->f_defaultsort=$i + 2;
                        break;
                    case 'titul' :
                        $newfield = $this->addField() ->setSqlFieldName('Osoba.Titul');
                        break;
                    case 'telefon' :
                        $newfield = $this->addField() ->setSqlFieldName('Osoba.Telefon');
                        break;
                    case 'zamestnanec' :
                        $newfield = $this->addField() ->setSqlFieldName('PracovniPomer.Zamestnanec');
                        break;
                    case 'casoperace' :
                        $newfield = $this->addField() ->setSqlFieldName('1') ->setHeaderText('Cas_operace') ->setAlign('right');
                        break;
                    case 'terminal' :
                        $newfield = $this->addField() ->setSqlFieldName('2') ->setHeaderText('Terminal') ->setAlign('right');
                        break;
                    case 'operace' :
                        $newfield = $this->addField() ->setSqlFieldName('3');
                        break;
                    case 'pritomnost' :
                        $newfield = $this->addField() ->setSqlFieldName('4') ->setHeaderText('Pritomnost') ->setFormat('anone') ->setAlign('center');
                        break;
                    case 'umisteni' :
                        $newfield = $this->addField() ->setSqlFieldName('5');
                        break;
                    case 'poznamka' :
                        $newfield = $this->addField() ->setSqlFieldName('6') ->setHeaderText('Poznamka');
                        break;
                    case 'rozdeleni' :
                        $newfield = $this->addField() ->setSqlFieldName('7');
                        break;
                    case 'kodrozdeleni' :
                        $newfield = $this->addField() ->setSqlFieldName('8');
                        break;

                    case 'uzivatelska' :
                        $newfield = $this->addField() ->setSqlFieldName((10 + (int)$sloupec['index'] - 1));
                        break;

                    case 'uzivatelskapp' :
                        $newfield = $this->addField() ->setSqlFieldName((30 + (int)$sloupec['index'] - 1));
                        break;

                    case 'kodrozdeleniosoby' :
                        $newfield = $this->addField() ->setSqlFieldName('54');
                        $this->zobrazitRozdeleni = true;
                        break;
                    case 'rozdeleniosoby' :
                        $newfield = $this->addField() ->setSqlFieldName('55');
                        $this->zobrazitRozdeleni = true;
                        break;

                    case 'foto' :
                        $newfield = $this->addField(new Monitorovani_FotoField($this->action)) ->setSqlFieldName('51');
                        break;

                    case 'fotoimg' :
                        $newfield = $this->addField(new Monitorovani_FotoField($this->action, true)) ->setSqlFieldName('51');
                        break;

                    case 'email' :
                        $newfield = $this->addField(new Email_BrowserField()) ->setSqlFieldName('53');
                        break;

                    case 'rozdimg':
                        $newfield = $this->addField(new RozdeleniIMG_BrowserField($this->database, $sloupec['typrozdeleni']));
                        break;

                    case 'rozdp':
                    case 'rozd':
                    case 'rozdvedoucipopis':
                    case 'rozdvedouci':
                    case 'rozdzastupce':
                        $newfield = $this->addField() ->setSqlFieldName('18');
                        $this->kodyTypuRozdeleni[$i] = new MonitorovaniSloupecRozdeleni($sloupec['typrozdeleni'], $sloupec['typ']);
                        break;

                    case 'vlozeniabsence':
                        if (empty($sloupec['typprava'])) {
                            $prava = Prava::vratPrava('absence', 'absence_' . $sloupec['operace']);
                            $zobrazitSloupec = Prava::jsouNejakePrava($prava);
                        }
                        if ($zobrazitSloupec) {
                            $newfield = $this->addField(new MonitorovaniButton_BrowserField(0, $sloupec));
                        }
                        break;
                    case 'vlozenidochazky':
                        if (empty($sloupec['typprava'])) {
                            $zobrazitSloupec = Prava::jsouNejakePrava('vkladani');
                        }
                        if ($zobrazitSloupec) {
                            $newfield = $this->addField(new MonitorovaniButton_BrowserField(0, $sloupec));
                        }
                        break;

                    default :
                        $newfield = $this->addField(BrowserField::FIELD_TYPE_TEXT) ->setText($sloupec['typ']);
                }
            }

            if ($newfield) {
                if (isset($sloupec['nadpis'])) {
                    $newfield->header = TextManager::_($sloupec['nadpis']);
                }

                if (isset($sloupec['class'])) {
                    $newfield->className = $sloupec['class'];
                }

                if ($newfield ->sqlFieldName) {
                    $newfield->sqlFieldIndex = $hlpcnt;
                    $hlpcnt++;
                }
            }

        }
    }


    /**
     * @param array $row
     * @param array $poslednipruchod
     * @return array
     * @throws Exception
     */
    private function sestavRadek($row, $poslednipruchod)
    {
        $result = [ $row[50] ]; // id PP

        foreach ($this->zobrazovaneSloupce as $i => $sloupec) {

            $lzeZobrazit = true;
            if (Prava::existujePravo('monitorovani_' . $sloupec['typ'])) {
                $lzeZobrazit = Prava::lzeZobrazitPP('monitorovani_' . $sloupec['typ'], $row[50], time(), time());
            }
            if (isset($sloupec['typprava'])) {
                $lzeZobrazit = $lzeZobrazit && Prava::lzeZobrazitPP($sloupec['typprava'], $row[50], time(), time());
            }

            if ($lzeZobrazit) {
                switch (strtolower($sloupec['typ'])) {
                    case 'rc' :
                        $result[] = $row[0];
                        break;
                    case "osobni" :
                        $result[] = $row[1];
                        break;
                    case "datumnastupu" :
                        $result[] = $row[2];
                        break;
                    case "cislosmlouvy" :
                        $result[] = $row[3];
                        break;
                    case "kodtypupomeru" :
                        $result[] = $row[4];
                        break;
                    case "datumukonceni" :
                        $result[] = $row[5];
                        break;
                    case "jmeno" :
                        $result[] = $row[6];
                        break;
                    case "prijmeni" :
                        $result[] = $row[7];
                        break;
                    case "celejmeno" :
                        $result[] = $row[6] . " " . $row[7];
                        break;
                    case "titul" :
                        $result[] = $row[8];
                        break;
                    case "telefon" :
                        $result[] = $row[52];
                        break;
                    case "zamestnanec" :
                        $result[] = $row[9];
                        break;

                    case "uzivatelska" :
                        $result[] = $row[10 + $sloupec['index'] - 1];
                        break;
                    case "uzivatelskapp" :
                        $result[] = $row[30 + $sloupec['index'] - 1];
                        break;

                    case "fotoimg":
                    case "foto" :
                        $result[] = $row[51]; // id fotografie
                        break;

                    case 'email' :
                        $result[] = $row[53];
                        break;

                    case 'kodrozdeleniosoby' :
                        $result[] = $row[54];
                        break;
                    case 'rozdeleniosoby' :
                        $result[] = $row[55];
                        break;

                    case "casoperace" :
                        $result[] = $poslednipruchod[0];
                        break;
                    case "terminal" :
                        $result[] = $poslednipruchod[1];
                        break;
                    case "poznamka" :
                        $result[] = $poslednipruchod[2];
                        break;

                    case "operace" :
                        $result[] = $poslednipruchod[3];
                        break;
                    case "pritomnost" :
                        $result[] = $poslednipruchod[4];
                        break;
                    case "umisteni" :
                        $result[] = $poslednipruchod[5];
                        break;
                    case "kodrozdeleni" :
                        $result[] = $poslednipruchod[6];
                        break;
                    case "rozdeleni" :
                        $result[] = $poslednipruchod[7];
                        break;

                    default :
                        if ($this->kodyTypuRozdeleni[$i] instanceof MonitorovaniSloupecRozdeleni) {
                            $result[] = $row[$this->kodyTypuRozdeleni[$i]->indexSloupce];
                            break;
                        }

                } // switch
            } else {
                $result[] = null;
            }
        } // foreach

        return $result;
    }


    protected function vratPoleProTrideni()
    {
        $this->indexPoleProTrideni = -1;

        switch (RonPortalSettings::getValue(self::SETTINGS_TRIDIT)) {
            case "kodtypupomeru" :
                $this->indexPoleProTrideni = 4;
                break;
            case "titul"        :
                $this->indexPoleProTrideni = 8;
                break;
            case "zamestnanec"  :
                $this->indexPoleProTrideni = 9;
                break;
            case "uzivatelska1" :
                $this->indexPoleProTrideni = 10;
                break;
            case "uzivatelska2" :
                $this->indexPoleProTrideni = 11;
                break;
            case "uzivatelska3" :
                $this->indexPoleProTrideni = 12;
                break;
            case "uzivatelska4" :
                $this->indexPoleProTrideni = 13;
                break;
            case "uzivatelska5" :
                $this->indexPoleProTrideni = 14;
                break;
            case "uzivatelska6" :
                $this->indexPoleProTrideni = 15;
                break;
            case "uzivatelska7" :
                $this->indexPoleProTrideni = 16;
                break;
            case "uzivatelska8" :
                $this->indexPoleProTrideni = 17;
                break;
            case "uzivatelska9" :
                $this->indexPoleProTrideni = 18;
                break;
        }

        if (($this ->database ->getDbType() === 'interbase') || ($this ->database ->getDbType() === 'oracle')) {
            return ($this->indexPoleProTrideni >= 0)
                ? '' . ($this->indexPoleProTrideni + 1)
                : '';
        }

        switch (RonPortalSettings::getValue(self::SETTINGS_TRIDIT)) {
            case "kodtypupomeru" :
                return "PracovniPomer.TypPracovnihoPomeru";
            case "titul"        :
                return "osoba.titul";
            case "zamestnanec"  :
                return "PracovniPomer.Zamestnanec";
            case "uzivatelska1" :
                return "Osoba.Uzivatelska1";
            case "uzivatelska2" :
                return "Osoba.Uzivatelska2";
            case "uzivatelska3" :
                return "Osoba.Uzivatelska3";
            case "uzivatelska4" :
                return "Osoba.Uzivatelska4";
            case "uzivatelska5" :
                return "Osoba.Uzivatelska5";
            case "uzivatelska6" :
                return "Osoba.Uzivatelska6";
            case "uzivatelska7" :
                return "Osoba.Uzivatelska7";
            case "uzivatelska8" :
                return "Osoba.Uzivatelska8";
            case "uzivatelska9" :
                return "Osoba.Uzivatelska9";
        } // switch

        return '';
    }

    /**
     * @param int $datum
     * @param bool $nastavitIndexSloupcu
     * @return string
     */
    private function getSQLTextSloupceRozdeleni($datum, $nastavitIndexSloupcu = false)
    {
        $sql = '';

        foreach ($this->kodyTypuRozdeleni as $sloupecRozdeleni) {
            switch (strtoupper($sloupecRozdeleni->typ)) {
                case 'POPIS' :
                case 'ROZDP' :
                    $sql .=
                        ", (SELECT Popis FROM Rozdeleni
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod =
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS PopisRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'KOD' :
                case 'ROZD' :
                    $sql .=
                        ", (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ") AS KodRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);
                    break;
                case 'VEDOUCIPOPIS' :
                case 'ROZDVEDOUCIPOPIS' :
                    $sql .=
                        ", (SELECT Vedouci FROM Rozdeleni
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod =
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS VedouciRozdeleniPopis" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'VEDOUCI' :
                case 'ROZDVEDOUCI' :
                    $sql .=
                        ", (SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Rozdeleni JOIN Osoba ON Osoba.RC = Rozdeleni.VedouciRC
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod =
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS VedouciRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
                case 'ZASTUPCE' :
                case 'ROZDZASTUPCE' :
                    $sql .=
                        ", (SELECT Osoba.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Osoba.Jmeno,'') FROM Rozdeleni JOIN Osoba ON Osoba.RC = Rozdeleni.ZastupceRC
                            WHERE KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND Kod =
                            (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob
                            WHERE RozdeleniOsob.KodTypuRozdeleni = " . (int)$sloupecRozdeleni->kodTypu .
                        "     AND RozdeleniOsob.RC = PracovniPomer.RC
                              AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo
                              AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                              AND RozdeleniOsob.PlatnostOd <= " . $this->database->dateToString($datum) .
                        "     AND (RozdeleniOsob.PlatnostDo >= " . $this->database->timestampToString(DateUtils::startOfTheDay($datum) - 1) . " OR RozdeleniOsob.PlatnostDo IS NULL)
                        " .
                        ")) AS ZastupceRozdeleni" . (($sloupecRozdeleni->kodTypu > 0) ? $sloupecRozdeleni->kodTypu : 878);

                    break;
            }

            if ($nastavitIndexSloupcu) {
                $sloupecRozdeleni->indexSloupce = $this->pocetSloupcuSql;
                $this->pocetSloupcuSql++;
            }
        }

        return $sql;
    }

    private function getSQLTextRychle($datum)
    {
        $this->pocetSloupcuSql = 54;

        // Predelano 25.6.2007
        $sql =
            'SELECT PuvDochProMonitorovani.RC AS RC, PuvDochProMonitorovani.osobniCislo, PuvDochProMonitorovani.DatumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' . // 3..4
            'PracovniPomer.DatumUkonceni, osoba.jmeno, osoba.prijmeni, osoba.titul,' . // 5..8
            'PracovniPomer.Zamestnanec,' . // 9
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.idFotografie, Osoba.Telefon, Osoba.eMail '; // 50..53

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';  // 54,55
            $this->pocetSloupcuSql += 2;
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum, true);

        $sql .=
            // nasleduji sloupce jen pro rychle monitorovani (viz pocetSloupcuSql)
            ', PuvDochProMonitorovani.DatumCasOperace AS DatumCas,PuvDochProMonitorovani.Terminal, PuvDochProMonitorovani.Poznamka,' . // pocetSloupcuSql + 0..2
            'OperaceDochazky.popis as PopisOperace, PuvDochProMonitorovani.Pritomnost,' . // +3..4
            'Umisteni.popis, PuvDochProMonitorovani.Rozdeleni, Rozdeleni.Popis,  '; // +5..7

        $sql .= ($this ->database ->getDbType() === 'interbase')
            ? 'cast(null as timestamp)'
            : 'null'; // + 8

        $sql .= ' , OperaceDochazky.Kod'; // + 9

        $sql .=
            ' FROM '.
            ' ( ( ( ( (PuvDochProMonitorovani INNER JOIN PracovniPomer ' .
            '          ON (PracovniPomer.RC = PuvDochProMonitorovani.RC ' .
            '          AND PracovniPomer.OsobniCislo = PuvDochProMonitorovani.OsobniCislo ' .
            '          AND PracovniPomer.DatumNastupu = PuvDochProMonitorovani.DatumNastupu)' .
            '         ) INNER JOIN Osoba ' .
            '         ON Osoba.RC = PracovniPomer.RC ' .
            '       ) INNER JOIN OperaceDochazky ' .
            '       ON OperaceDochazky.kod = PuvDochProMonitorovani.KodOperace ' .
            '     ) LEFT OUTER JOIN Terminal ' .
            '     ON Terminal.kod = PuvDochProMonitorovani.terminal ' .
            '   ) LEFT OUTER JOIN Umisteni '.
            '   ON Umisteni.kod = Terminal.KodUmisteni ' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            ' ON Rozdeleni.KodTypuRozdeleni = PuvDochProMonitorovani.KodTypuRozdeleni' .
            ' AND Rozdeleni.Kod = PuvDochProMonitorovani.Rozdeleni ';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE PracovniPomer.DatumNastupu <= ' . $this ->database ->dateToString($datum) .
            ' AND ( (PracovniPomer.DatumUkonceni >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum) - 1) . ') OR (PracovniPomer.DatumUkonceni IS NULL) ) ';

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (isset($this ->filterForm ->kodRozdeleni)  &&  RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $sqlFilterRozdeleni =
                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                    RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this ->filterForm ->kodRozdeleni,
                    DateUtils::startOfTheDay($datum), $datum, $this ->filterForm ->vcetnePodrizenychRozdeleni);
            if ($sqlFilterRozdeleni) {
                $sql .= ' AND (' . $sqlFilterRozdeleni . ')';
            }

//            $sql .=
//                ' AND EXISTS ' .
//                '(SELECT * FROM RozdeleniOsob ' .
//                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
//                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
//                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
//                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
//                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
//                " AND ( (RozdeleniOsob.KodTypuRozdeleni = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
//                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
//            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
//                $sql .=
//                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
//                    " HierarchieRozdeleni.typNadrazene = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
//                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
//                    ' AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
//                    ' AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni' .
//                    ')'; // exists
//            }
//            $sql .=
//                ')' . // and
//                ')';

        }

        if (isset($_SESSION['monitorovat_operaci']) && $_SESSION['monitorovat_operaci']) {
            $sql .=
                " AND OperaceDochazky.Kod = '" . $this->database->addSlashes($_SESSION['monitorovat_operaci']) . "'";
        }

        // Prida do dotazu absence - pridano 25.7.2007
        $sql .=
            ' UNION ' .

            ' SELECT Absence.RC AS RC, Absence.osobniCislo, Absence.DatumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' .
            'PracovniPomer.DatumUkonceni, Osoba.jmeno, Osoba.prijmeni, Osoba.titul,' .
            'PracovniPomer.Zamestnanec,' .
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.idFotografie, Osoba.Telefon, Osoba.eMail ';

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';  // 54,55
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum);

        // nasleduji sloupce pro rychle monitorovani ...
        if ($this ->database ->getDbType() == 'interbase') {
            $sql .=
                ', Absence.DatumOd as DatumCas,  cast(null as varchar(10)), cast(SUBSTRING(Absence.Poznamka FROM 1 FOR 250) as varchar(250)),' .
                'OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' .
                'cast(null as varchar(40)), cast(null as varchar(10)), cast(null as varchar(40)), Absence.DatumDo, OperaceDochazky.Kod';
        } else {
            $sql .=
                ', Absence.DatumOd as DatumCas, null, CAST(Absence.Poznamka AS nvarchar(250)),' .
                'OperaceDochazky.popis as PopisOperace,OperaceDochazky.PritomnostProMonitorovani,' .
                'null, null, null, Absence.DatumDo, OperaceDochazky.Kod';
        }

        $sql .=
            ' FROM Absence INNER JOIN OperaceDochazky ' .
            '   ON (OperaceDochazky.kod = Absence.OperaceDochazky)' .
            ' INNER JOIN PracovniPomer ' .
            '   ON PracovniPomer.RC = Absence.RC' .
            '  AND PracovniPomer.OsobniCislo = Absence.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = Absence.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '   ON Osoba.RC = Absence.RC';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                ' LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE  (Absence.datumOd <= ' . $this ->database ->timestampToString($datum) . ')' .
            ' AND (Absence.datumDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum)) . ')' .
            ' AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 '; // nezobrazuje zamitnute a zrusene

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && isset($this ->filterForm ->kodRozdeleni)) {
            $sql .=
                ' AND EXISTS ' .
                '(SELECT * FROM RozdeleniOsob ' .
                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                " AND ( (RozdeleniOsob.KodTypuRozdeleni = " . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $sql .=
                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
                    " HierarchieRozdeleni.typNadrazene = " . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                    " AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni " .
                    " AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni" .
                    ')'; // exists
            }

            $sql .=
                ')' . // and
                ')';
        }

        if (isset($_SESSION['monitorovat_operaci']) && $_SESSION['monitorovat_operaci']) {
            $sql .=
                " AND OperaceDochazky.Kod = '" . $this->database->addSlashes($_SESSION['monitorovat_operaci']) . "'";
        }

        // Razeni
        $trideni = $this->vratPoleProTrideni();
        $sql .= ' ORDER BY ';
        if ($trideni) $sql .= $trideni . ", ";

        if ($this ->database ->getDbType() == 'interbase' || $this ->database ->getDbType() == 'oracle') $sql .= ' 8,7,1,21 DESC';
        else $sql .= ' Osoba.Prijmeni, Osoba.Jmeno, RC, DatumCas DESC';

        return $sql;
    } // GetSQLTextRychle


    function getSQLTextPuvodni($datum)
    {
        $sql =
            'SELECT DISTINCT PracovniPomer.RC, PracovniPomer.osobniCislo, PracovniPomer.datumNastupu,' .
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru,' .
            'PracovniPomer.DatumUkonceni, Osoba.jmeno, Osoba.prijmeni, Osoba.titul,' .
            'PracovniPomer.Zamestnanec,' .
            'Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5,  ' . // 10..
            'Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // ..19
            'Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 20..
            'Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . //   ..29
            'PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5,  ' . // 30..
            'PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // ..39
            'PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 40..
            'PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . //   ..49
            'PracovniPomer.Id, Osoba.IdFotografie, Osoba.Telefon, Osoba.eMail '; // 50 - 53

        //11.10.2016 RON-PR pouziva se pro zjisteni indexu sloupce rozdeleni
        $this->pocetSloupcuSql = 54;

        if ($this ->zobrazitRozdeleni) {
            $sql .= ', RozdeleniOsoby.Kod, RozdeleniOsoby.Popis ';
            $this->pocetSloupcuSql += 2;
        }

        //11.10.2016 RON-PR vybrane rozdeleni se pripoji k dotazu pomoci subselectu
        $sql .= $this->getSQLTextSloupceRozdeleni($datum, true);

        $sql .=
            ' FROM PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC ';

        if ($this ->zobrazitRozdeleni) {
            $sql .=
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->dateToString($datum) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->dateToString($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) .
                ' LEFT OUTER JOIN Rozdeleni RozdeleniOsoby ' .
                '  ON RozdeleniOsoby.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND RozdeleniOsoby.Kod = RozdeleniOsob.Rozdeleni';
        }

        $sql .=
            ' WHERE PracovniPomer.DatumNastupu <= ' . $this ->database ->dateToString($datum) .
            ' AND ( (PracovniPomer.DatumUkonceni >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum) - 1) . ') OR (PracovniPomer.DatumUkonceni IS NULL) ) ';

        if (RonPortalSettings::getIntValue('monitorovani_skryt_ext')) {
            $sql .= ' AND PracovniPomer.Zamestnanec <> 0 ';
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && isset($this ->filterForm ->kodRozdeleni)) {
            $sql .=
                ' AND EXISTS ' .
                '(SELECT * FROM RozdeleniOsob ' .
                ' WHERE RozdeleniOsob.RC = PracovniPomer.RC' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($datum) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString(DateUtils::startOfTheDay($datum)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                " AND ( (RozdeleniOsob.KodTypuRozdeleni = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                " AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "')";
            if (RonPortalSession::rozdeleniVcetnePodrizenych()) {
                $sql .=
                    " OR EXISTS (SELECT * FROM HierarchieRozdeleni WHERE" .
                    " HierarchieRozdeleni.typNadrazene = '" . (int)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) . "'" .
                    " AND HierarchieRozdeleni.kodNadrazene = '" . $this->database->addSlashes($this ->filterForm ->kodRozdeleni) . "'" .
                    ' AND HierarchieRozdeleni.typPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                    ' AND HierarchieRozdeleni.kodPodrizene = RozdeleniOsob.Rozdeleni' .
                    ')'; // exists
            }
            $sql .=
                ')' . // and
                ')';
        }

        $trideni = $this->vratPoleProTrideni();
        $sql .= ' ORDER BY ';
        if ($trideni) $sql .= $trideni . ', ';

        if ($this ->database ->getDbType() == 'interbase' || $this ->database ->getDbType() == 'oracle') $sql .= ' 8, 7, 1';
        else $sql .= ' Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.RC';

        return $sql;
    } // function GetSQLTextPuvodni


    function getSQL($sqlFilter = '')
    {

        $datumCas = $this ->filterForm ->datumCas;

        if ( RonPortalSettings::getIntValue(self::SETTINGS_POUZIT_RYCHLE) && $this->filterForm ->aktualniStav) {
            return $this->getSQLTextRychle($datumCas);
        } else {
            return $this->getSQLTextPuvodni($datumCas);
        }
    }


    function zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperace)
    {
        echo '<div id="monitorovani_pocty">';

        echo '<table class="browser">';

//        echo '<div>' . TextManager::_('Pritomno_osob') . "<strong>$pritomno</strong></div>";
//        echo '<div>' . TextManager::_('Nepritomno_osob') . "<strong>$nepritomno</strong></div>\n";

        MonitorovaniPocetDleOperace::seradOperace($poctyDleOperace);

        echo '<thead>';
        echo '<tr><th>' . TextManager::_('Operace') . '</th><th>' . TextManager::_('Pritomni') . '</th><th>' . TextManager::_('Nepritomni') . '</th><th>' . TextManager::_('Celkem') . '</th></tr>';
        echo '</thead>';

        echo '<tbody class="border-bottom">';
        /** @var MonitorovaniPocetDleOperace $prehledOperace */
        foreach ($poctyDleOperace as $index => $prehledOperace) {
            echo '<tr><td>' . StringUtils::htmlTextConvert($prehledOperace->popis) . " </td><td class='align-right'>" . $prehledOperace->pocetPritomno . "</td>
                        <td class='align-right'>" . $prehledOperace->pocetNepritomno . "</td><td class='align-right'>" . $prehledOperace->pocetCelkem . "</td></tr>";
        }
        echo '</tbody>';

        echo '<tbody>';
        echo '<tr class="browserfooter"><td>' . TextManager::_('Celkem') . "</td><td class='align-right'>$pritomno</td><td class='align-right'>$nepritomno</td><td class='align-right'>" . ($pritomno+$nepritomno) . "</td></tr>";
        echo '</tbody>';

        echo '</table>';

        echo "</div>\n";
    }


    function zobrazOddelovac($row)
    {
        $pocet = count($this->fields) - 1; // presah do dalsich bunek
        echo "<tr class='oddelovacmonitorovani border-bottom'><td></td><td colspan='$pocet'>";
        echo StringUtils::htmlTextConvert($row[$this->indexPoleProTrideni]);
        echo "</td></tr>";
    }


    /**
     * @throws Exception
     */
    function renderContent()
    {
        $datum = $_SESSION['monitorovatcas'];

        $pritomno = 0;
        $nepritomno = 0;

        $poctyDleOperaci = [];

        //$cursor = getSqlResult($this ->database, $sql);

        $this->renderTableStart();
        echo '<tbody>';

        $i = 1;

        // Promenne pro zapamatovani posledni osoby, aby se nezobrazovala vicekrat
        // (v pripade, ze ma nejakou operaci i planovanou absenci)
        $old_rc = '';
        $old_osobni = '';
        $old_datumnastupu = 0;

        $old_trideni = NULL;

        $prava = Prava::vratPrava('monitorovani');

        while ($row = $this ->cursor ->fetch()) {
            // Zkontroluje, jesli ma uzivatel pravo monitorovat osobu
            if ($this->kontrolovatPrava) {
                $lzezobrazit = $prava->lzeZobrazitPP($row[50], time(), time());
                // ? ? ? nastavit nejdrive obdobi pro prava ? A co prava v pripade divani na jiny cas nez aktualni ?
            } else {
                $lzezobrazit = true;
            }

            if ($lzezobrazit) {
                if ($old_rc != $row[0] || $old_osobni != $row[1] || $old_datumnastupu != $row[2]) {
                    // Osoba jeste nebyla zobrazena
                    $old_rc = $row[0];
                    $old_osobni = $row[1];
                    $old_datumnastupu = $row[2];

                    // Zjisti posledni pruchod a podle nej pritomnost osoby
                    $poslednipruchod = $this->vratPosledniPruchod($datum, $row);

                    if (((isset($_SESSION['monitorovat_operaci']) && ($_SESSION['monitorovat_operaci'] == $poslednipruchod[8])) || !isset($_SESSION['monitorovat_operaci'])) &&
                        ((isset($_SESSION['monitorovat_terminal']) && ($_SESSION['monitorovat_terminal'] == $poslednipruchod[1])) || !isset($_SESSION['monitorovat_terminal']))) {

                        if ($poslednipruchod[4]) $pritomno++;
                        else $nepritomno++;

                        if (isset($poctyDleOperaci[$poslednipruchod[8]]))
                            $pocetOperace = $poctyDleOperaci[$poslednipruchod[8]];
                        else
                            $pocetOperace = new MonitorovaniPocetDleOperace();

                        $pocetOperace->kod = $poslednipruchod[8];
                        $pocetOperace->popis = $poslednipruchod[3];
                        $pocetOperace->pocetCelkem++;
                        if ($poslednipruchod[4]) $pocetOperace->pocetPritomno++;
                        else $pocetOperace->pocetNepritomno++;

                        $poctyDleOperaci[$poslednipruchod[8]] = $pocetOperace;

                        if (($poslednipruchod[4] && $_SESSION['comonitorovat'] != 1) ||
                            (!$poslednipruchod[4] && $_SESSION['comonitorovat'])
                        ) {

                            // Trideni podle nektereho pole - nastaveno v aplikaci v nastaveni intrawebu
                            if ($this->indexPoleProTrideni >= 0) {
                                if ($old_trideni != $row[$this->indexPoleProTrideni]) {
                                    $old_trideni = $row[$this->indexPoleProTrideni];
                                    $this->zobrazOddelovac($row);
                                }
                            }

                            $radek = $this->sestavRadek($row, $poslednipruchod);
                            //$classname = ($i % 2) ? 'browsercolor1' : 'browsercolor2';
                            $classname = '';

                            if ($poslednipruchod[4]) {
                                $this->renderLine($radek, $classname . ' monitorovanipritomen', $i);
                            } else {
                                $this->renderLine($radek, $classname . ' monitorovaninepritomen', $i);
                            }

                            $i++;
                        }
                    }
                }
            }
        } // while

        echo "</tbody>\n";

        $this->renderTableEnd();

        $this->zobrazPoctyOsob($pritomno, $nepritomno, $poctyDleOperaci);
    }



    function render()
    {
        echo
            '<div id="monitorovani"' .
            ($this->filterForm ->aktualniStav ? " class='auto-refresh' data-timeout='180' data-url='?menuid={$this ->action}'" : '') .
            '>';

        parent::render();

        echo "</div>\n";
    }


}
