<?php
/**
 * stanislav.elbl, 08.01.2016
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\Parameters;


class OperaceDochazky
{
    var $kod;
    var $popis;
    var $popisProDochazku;
    var $autoSchvalovat;
    var $absence_LzeZrusit;
    var $povinneRozdeleniUDochazky;
    var $pocatekIntervalu;
    var $pritomnostProMonitorovani;
    var $lzeNastavitCas;
    var $barvaProVykresleni;

    /**
     * @param array $row
     * @return OperaceDochazky
     */
    private static function loadRow($row)
    {
        $result = new OperaceDochazky();

        $result ->kod = $row[0];
        $result ->popis = $row[1];
        $result ->popisProDochazku = $row[2];
        $result ->autoSchvalovat = $row[3];
        $result ->absence_LzeZrusit = $row[9];
        $result ->povinneRozdeleniUDochazky = $row[4];
        $result ->pocatekIntervalu = $row[5];
        $result ->pritomnostProMonitorovani = $row[6];
        $result ->lzeNastavitCas = (int)$row[7];
        $result ->barvaProVykresleni = isset($row[8]) ? (int)$row[8] : null;

        return $result;
    }


    /**
     * @param Database $database
     * @param string $filter
     * @param Parameters $filterParams
     * @param string $orderBy
     * @return OperaceDochazky[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiOperace($database, $filter, $filterParams = null, $orderBy = 'PopisProDochazku')
    {
        $sql =
            'SELECT Kod, Popis, PopisProDochazku, '. // 0..2
            '  Absence_AutoSchvalovat, PovinneRozdeleniUDochazky, PocatekIntervalu,' . // 3..5
            '  PritomnostProMonitorovani, LzeNastavitCas_Intraweb, BarvaProVykresleni, Absence_LzeZrusit' . // 6..9
            ' FROM OperaceDochazky';

        if ($filter) $sql .= ' WHERE ' . $filter;

        $sql .= ' ORDER BY ' . $orderBy;

        $cursor = $database ->getSqlResult($sql, $filterParams, 'Chyba pri nacitani seznamu operaci');

        $result = [];
        while ($row = $cursor ->fetch()) {
            $result[] = static::loadRow($row);
        }
        $cursor ->close();

        return $result;
    }

    /**
     * @param Database $database
     * @param string $kod
     * @return OperaceDochazky
     * @throws \Exception
     */
    public static function nactiOperaci($database, $kod)
    {
        $params = $database ->createParameters();
        $list = static::nactiOperace($database, 'Kod = ' . $params->addParameter($kod), $params);
        if (!$list  ||  !count($list)) {
            throw new \Exception('Pozadovana operace dochazky nebyla nalezena!');
        }
        return $list[0];
    }


    /**
     * @param Database $database
     * @param string $kod
     */
    static function rozdeleniDochazkyPovinneProOperaci($database, $kod)
    {
        $operace = self::nactiOperaci($database, $kod);
        return $operace ->povinneRozdeleniUDochazky;
    }


    static function lzeNastavitCasOperace($database, $kod)
    {
        $operace = self::nactiOperaci($database, $kod);
        return $operace ->lzeNastavitCas;
    }
}
