<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\StringUtils;

class PrirazeniKalendare
{
    public $kod;
    public $popis;

    public $dochazkaPodlePlanKapacit;

    public $platnostOd;
    public $platnostDo;


    /**
     * @param Database $database
     * @param int $idPP
     * @param $datumOd
     * @param $datumDo
     * @return PrirazeniKalendare[]
     */
    public static function nactiKalendarProObdobi($database, $idPP, $datumOd, $datumDo)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT PrirazeniKalendare.Kalendar, Kalendar.Popis, Kalendar.DochazkaPodlePlanKapacit, ' .
            '       PrirazeniKalendare.PlatnostOd, PrirazeniKalendare.PlatnostDo ' .
            '  FROM PrirazeniKalendare, Kalendar, PracovniPomer ' .
            ' WHERE PrirazeniKalendare.Kalendar = Kalendar.Kod ' .
            '   AND PrirazeniKalendare.PlatnostOd <= ' . $params->addParameter($database->dateToParam($datumDo)) .
            '   AND (PrirazeniKalendare.PlatnostDo IS NULL  OR  PrirazeniKalendare.PlatnostDo >= ' . $params->addParameter($database->dateToParam($datumOd)) . ')' .
            '   AND PrirazeniKalendare.RC = PracovniPomer.RC' .
            '   AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
            '   AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu' .
            '   AND PracovniPomer.Id = ' . $params->addParameter($idPP) .
            ' ORDER BY PrirazeniKalendare.PlatnostOd ';

        $cursor = $database->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor->fetch()) {
            $kal = new PrirazeniKalendare();
            $kal ->kod = $row[0];
            $kal ->popis = $row[1];
            $kal ->dochazkaPodlePlanKapacit = (int)$row[2];
            $kal ->platnostOd = $cursor ->stringToTimestamp($row[3]);
            $kal ->platnostDo = $cursor ->stringToTimestamp($row[4]);

            $result[] = $kal;
        }

        $cursor ->close();

        return $result;
    }

    /**
     * @param Database $database
     * @param int $idPP
     * @param int $datum
     * @return PrirazeniKalendare
     */
    public static function nactiKalendarProDen($database, $idPP, $datum)
    {
        $kal = static::nactiKalendarProObdobi($database, $idPP, $datum, $datum);
        if (!count($kal)) return null;
        return $kal[0];
    }


    public function format($format)
    {
        $result = str_replace('%KOD%', StringUtils::htmlTextConvert($this ->kod), $format);
        return str_replace('%POPIS%', StringUtils::htmlTextConvert($this ->popis), $result);
    }
}
