<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Kalendar;
use RonPortal\RonPortalSettings;

class PuvodniDochazkaBrowser extends DochazkaBrowser
{
    const SETTINGS_ZOBRAZIT_TERMINAL = 'terminalvpuvodni';

    function __construct($database)
    {
        parent::__construct($database, 'dochazkazamestnance');

        $this->title = TextManager::_('Dochazka_zamestnance');

        $this->cssClass = 'browser grid browserdenni puvodnidochazka mobile-table-rows';
    }



    function inicializujData($datumOd, $datumDo)
    {
        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
            $this ->filterForm ->vybranaOsobaRC, $this ->filterForm ->vybranaOsobaOsobniCislo, $this ->filterForm ->vybranaOsobaDatumNastupu,
            $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo);

        parent::inicializujData($datumOd, $datumDo);
    }


    function nactiData()
    {
        if ($this ->filterForm ->vybranaOsobaRC === null) { // kvuli firebirdu
            return null;
        }

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace, OperaceDochazky.PopisProDochazku,' . // 0,1
            ' PuvodniDochazka.Poznamka, PuvodniDochazka.Terminal, PuvodniDochazka.KodOperace, ' . // 2..4
            ' PuvodniDochazka.PopisPolohy, PuvodniDochazka.ZemSirka, PuvodniDochazka.ZemDelka, ' . // 5..7
            ' Terminal.Popis' . // 8
            ' FROM PuvodniDochazka INNER JOIN OperaceDochazky' .
            '  ON PuvodniDochazka.KodOperace = OperaceDochazky.Kod' .
            ' LEFT OUTER JOIN Terminal' .
            '  ON Terminal.Kod = PuvodniDochazka.Terminal' .
            ' WHERE PuvodniDochazka.DatumCasOperace >= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND PuvodniDochazka.DatumCasOperace <= ' . $params ->addParameter($this ->database ->timestampToParam(DateUtils::endOfTheDay($this ->filterForm ->vybraneObdobiDo)) ).
            ' AND PuvodniDochazka.RC = ' . $params ->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND PuvodniDochazka.OsobniCislo = ' . $params ->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND PuvodniDochazka.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)) .
            ' ORDER BY PuvodniDochazka.DatumCasOperace';

        return $this ->database ->getSqlResult($sql, $params);
    }


    function vratTextKZaznamu($zaznam, $posledni)
    {
        $time = $this ->database ->stringToTimestamp($zaznam[0]);

        $title = '';
        $poznamka = '';
        $data =
            "data-timestamp='$time' data-operace='" . StringUtils::htmlTextConvert($zaznam[4]) . "' " .
            "data-infostr='" . StringUtils::format('datetime', $time) . ", " . StringUtils::htmlTextConvert($zaznam[1]) . "' " .
            "data-terminal='" . StringUtils::htmlTextConvert($zaznam[3]) . "'";

        $result =
            "<div $data class='zaznamPuvodni'>" .
            date('H:i', $time) . ' &nbsp; ' . StringUtils::htmlTextConvert($zaznam[1]);

        if (RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_TERMINAL) && $zaznam[3]) {
            $result .= ' <DIV class="terminaldochazky" title="' . TextManager::_('Terminal') . ' ' . StringUtils::htmlTextConvert($zaznam[8]) . '">(' . StringUtils::htmlTextConvert($zaznam[3]) . ')</DIV>';
        }

        $result .= "</div>";

        if ($zaznam[5]  ||  ($zaznam[6]  &&  $zaznam[7])) {
            $dataPoloha = ($zaznam[6]  &&  $zaznam[7]) ? " data-latitude='{$zaznam[6]}' data-longitude='{$zaznam[7]}'" : '';

            if ($this ->zobrazitPoznamky) {
                $poznamka .=
                    "<div class='polohadochazky mapurl' $dataPoloha>" .
                    StringUtils::htmlTextConvert($zaznam[5]) .
                    '<i class="inline-glyph fa glyph-gps"></i>' .
                    '</div>';
            } else {
                $title .= $zaznam[5] . "\n";
            }
        }

        if ($zaznam[2]) {
            if ($this ->zobrazitPoznamky) {
                $poznamka .= '<div class="poznamka">' . StringUtils::htmlTextConvert($zaznam[2]) . '</div>';
            } else {
                $title .= $zaznam[2];
            }
        }

        if ($title) {
            return '<span title="' . StringUtils::htmlTextConvert($title) . '">'. $result . '</span>' . $poznamka;
        }

        return $result . $poznamka;
    }


    function render()
    {
        parent::render();

        if (RonPortalSettings::getIntValue('vkladani_zpusob_terminal') == 4) {  // povolen rucni vyber terminalu
            $comboTerminal = new ComboBox('terminal');
            $comboTerminal ->htmlParams = 'id="vyberTerminaluProVkladani"';
            $comboTerminal ->className .= ' ronform-field';
            $comboTerminal ->loadData($this ->database, "SELECT Kod, Popis FROM Terminal WHERE Aktivni = 1 AND KodUrceni IN (1, 6, 7)", null);

            require('ronportal/dochazka/view_puvodni_terminal.php');
        }
    }


}
