<?php
/**
 * stanislav.elbl, 16.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\HttpUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;



class Schvalovani_RozdeleniField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this ->code = 'col-rozdeleni';

        $this ->align = 'left';
        $this ->ordering = 1;
        $this ->header = RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_NADPIS);

        switch (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT)) {
            case 0 :
                $this->sqlFieldName = 'Rozdeleni.Popis';
                break;

            case 1 :
            case 2 :
                $this->sqlFieldName = 'Rozdeleni.Kod';
                break;
        }
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT)) {
            case 0 :
                // popis
                return $rowData[18];

            case 1 :
                // kod
                return $rowData[17];

            case 2 :
                if (isset($rowData[17])  &&  isset($rowData[18])) return $rowData[17] . ' - ' . $rowData[18];
                return $rowData[17] . $rowData[18];
        }

        return '';
    }

}



class SchvaleniDochazkyField extends BrowserField
{
    var $indexData;
    var $indexUzivatel;
    var $indexCas;

    /**
     * @param Database $database
     * @param int $indexData
     * @param int $indexUzivatel
     * @param int $indexCas
     */
    function __construct($database, $indexData, $indexUzivatel, $indexCas)
    {
        parent::__construct($database);

        $this -> align = 'right';
        $this -> ordering = 1;

        $this -> indexData = $indexData;
        $this -> indexUzivatel = $indexUzivatel;
        $this -> indexCas = $indexCas;
    }



    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ( !$rowData[$this -> indexData] ) return '';

        $result = DBUtils::getFormatedValue($this ->database, 'date', $rowData[$this -> indexData]);

        if ( $rowData[$this -> indexUzivatel] )
            $result =
                '<SPAN title="' . StringUtils::htmlTextConvert($rowData[$this -> indexUzivatel]) .
                ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this -> indexCas]).')">' . $result . '</SPAN>';

        return $result;
    }
}


class SchvaleniDochazkyEditaceField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this ->code = 'col-edit';
        $this -> header = '';
        $this -> align = 'left';
        $this -> ordering = 0;

        //$this -> f_prava = VratPrava('editace');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //if (!$this -> f_prava -> LzeZobrazitPP($rowdata[0], S_GetZacatekObdobi(), S_GetKonecObdobi())) return '';

        return
            '<A onclick="StopEventPropagation(event)" target="zpracovanadochazkawin" href="?menuid=zpracovanadochazka&osoba='.$rowData[0].
            '&obdobi=' . DateUtils::dateToUrlString(RonPortalSession::obdobi()).'">' .
            TextManager::_('dochazka') .
            '</A>';
    }
}


class SchvalovaniDochazkyBrowser extends Browser
{
    var $fieldJmeno;
    var $fieldPrijmeni;
    var $fieldOsobniCislo;

    var $filterForm;


    function __construct($database)
    {
        global $STR_ZpracovanoDo;   // kompat
        global $STR_ZkontrolovanoDo;// kompat
        global $STR_SchvalenoDo;    // kompat

        parent::__construct($database);

        $this ->title = TextManager::_('Schvalovani');
        $this ->code = 'browserschvaldoch';
        $this ->browserUrl = '?menuid=schvalovani&';

        $this ->tableId = 'schvalovani_dochazky_seznam';

        $this ->sqlSelectPart =
            'PracovniPomer.ID, Osoba.RC, ' . // 0,1
            'SchvaleniDochazky.ZpracovanoDo, SchvaleniDochazky.Zpracoval, SchvaleniDochazky.DT_Zpracovani, ' . // 2..4
            'SchvaleniDochazky.ZkontrolovanoDo, SchvaleniDochazky.Zkontroloval, SchvaleniDochazky.DT_Zkontrolovani, ' . // 5..7
            'SchvaleniDochazky.SchvalenoDo, SchvaleniDochazky.Schvalil, SchvaleniDochazky.DT_Schvaleni, '. // 8..10
            'Osoba.Titul, Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo, '. // 11..14
            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1) : 'null') . ', ' . // 15
            (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) ? RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2) : 'null'); // 16

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) ) {
            $this->sqlSelectPart .= ', Rozdeleni.Kod, Rozdeleni.Popis'; // 17,18
        }

        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'PracovniPomer.ID', BROWSER_FIELD_SQL_INDEX => 0] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'Osoba.RC', BROWSER_FIELD_SQL_INDEX => 1] );

        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.ZpracovanoDo', BROWSER_FIELD_SQL_INDEX => 2] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Zpracoval', BROWSER_FIELD_SQL_INDEX => 3] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Zpracovani', BROWSER_FIELD_SQL_INDEX => 4] );

        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.ZkontrolovanoDo', BROWSER_FIELD_SQL_INDEX => 5] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Zkontroloval', BROWSER_FIELD_SQL_INDEX => 6] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Zkontrolovani', BROWSER_FIELD_SQL_INDEX => 7] );

        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.SchvalenoDo', BROWSER_FIELD_SQL_INDEX => 8] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.Schvalil', BROWSER_FIELD_SQL_INDEX => 9] );
        //$this->addField( [BROWSER_FIELD_TYPE => BROWSER_FIELD_TYPE_HIDDEN, BROWSER_FIELD_SQL_NAME => 'SchvaleniDochazky.DT_Schvaleni', BROWSER_FIELD_SQL_INDEX => 10] );

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $this->addField()
                ->setSqlFieldName(RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1), 15)
                ->setCode('col-pole1') ->setOrdering(1)
                ->setHtmlConvert(false);
        }

        $this->addField()
            ->setSqlFieldName('Osoba.Titul', 11) ->setHeaderText('Titul') ->setOrdering(1)
            ->setCode('col-titul');

        // zapamatuje si pole, aby mohl spravne nastavit link podle zvoleneho obdobi (nutno nastavit az po zobrazeni filtru)
        $this->fieldPrijmeni = $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni', 12) ->setHeaderText('Prijmeni') ->setOrdering(1)
            ->setCode('col-prijmeni')
            ->setSortSQL('Prijmeni, Jmeno') ->setSortSQLDesc('Prijmeni DESC, Jmeno DESC');

        $this->defaultSort = count($this->fields);

        $this->fieldJmeno = $this->addField()
            ->setSqlFieldName('Osoba.Jmeno', 13) ->setHeaderText('Jmeno') ->setOrdering(1)
            ->setCode('col-jmeno');

        $this->fieldOsobniCislo = $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo', 14) ->setHeaderText('Osobni_cislo') ->setOrdering(1)
            ->setCode('col-osobnicislo');

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $this->addField()
                ->setSqlFieldName(RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2), 16) ->setOrdering(1)
                ->setCode('col-pole2')
                ->setHtmlConvert(false);
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            $this->addField(new Schvalovani_RozdeleniField());
        }


        $f = new SchvaleniDochazkyField($this ->database, 2, 3, 4);
        $f->sortSQL = 'SchvaleniDochazky.ZpracovanoDo';
        $f->code = 'col-schvaleni1';
        $f->header = $STR_ZpracovanoDo ?: TextManager::_('Dochazka_schvaleni_1_do');
        $f->className = 'textschvaleni1';
        $this->addField($f);

        $f = new SchvaleniDochazkyField($this ->database, 5, 6, 7);
        $f->sortSQL = 'SchvaleniDochazky.ZkontrolovanoDo';
        $f->code = 'col-schvaleni2';
        $f->header = $STR_ZkontrolovanoDo ?: TextManager::_('Dochazka_schvaleni_2_do');
        $f->className = 'textschvaleni2';
        $this->addField($f);

        $f = new SchvaleniDochazkyField($this ->database, 8, 9, 10);
        $f->sortSQL = 'SchvaleniDochazky.SchvalenoDo';
        $f->code = 'col-schvaleni3';
        $f->header = $STR_SchvalenoDo ?: TextManager::_('Dochazka_schvaleni_3_do');
        $f->className = 'textschvaleni3';
        $this->addField($f);

        $this->addField(new SchvaleniDochazkyEditaceField());

        $this->sqlWherePart =
            '   PracovniPomer.Zamestnanec > 0 ' .   // nezobrazuje externi osoby
            '   AND PracovniPomer.ZpracovatDochazku = 1 ';

        $this->sqlOrderPart = '';
        $this->primaryField = 1;

        $this -> filterForm = new SchvalovaniFilterForm($this ->database);

        $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }


    function load($sqlFilter = '')
    {
        $this->filterForm ->loadData();

        // sql dotaz se nastavi az po vyberu obdobi
        $this->sqlFromPart =
            '( Osoba INNER JOIN PracovniPomer ' .
            '  ON Osoba.RC = PracovniPomer.RC ' .
            ') ' .
            'LEFT OUTER JOIN SchvaleniDochazky ' .
            '  ON SchvaleniDochazky.RC = PracovniPomer.RC ' .
            ' AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            ' AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            " AND SchvaleniDochazky.Aplikace = 'ADS' " .
            ' AND SchvaleniDochazky.PocatekObdobi = ' . $this ->database ->timestampToString($this ->filterForm ->vybraneObdobiOd);

        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) ) {
            $this->sqlFromPart =
                '( ( ' .
                $this->sqlFromPart .
                ') ' .
                'LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = PracovniPomer.RC ' .
                ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $this ->database -> timestampToString($this ->filterForm ->vybraneObdobiDo) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database -> timestampToString($this ->filterForm ->vybraneObdobiDo) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ') ' .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();

        $params = '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->vybraneObdobiOd);

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            $params .= '&rozdeleni=' . HttpUtils::urlEncode($this ->filterForm ->kodRozdeleni);
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            $params .= '&rozdeleni2=' . HttpUtils::urlEncode($this ->filterForm ->kodRozdeleni2);
        }

        echo
            '<a href="?menuid=schvalovani&hromadneschvaleni=1' . $params . '">' .
            TextManager::_('Schvalit_hromadne') .
            '</a><br>';
    }


    function getSQLFilter()
    {
        $result =
            ' PracovniPomer.DatumNastupu <= ' . $this ->database -> dateToString($this ->filterForm ->vybraneObdobiDo) . // jen platne PP
            ' AND ( PracovniPomer.DatumUkonceni >= ' . $this ->database -> dateToString($this ->filterForm ->vybraneObdobiOd) .
            ' OR PracovniPomer.DatumUkonceni IS NULL )';

        $filtrprava = Prava::sqlFiltrPodlePrav(
            [Prava::vratPrava('zpracovani'), Prava::vratPrava('zkontrolovani'), Prava::vratPrava('schvaleni')],
            'PracovniPomer',
            $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo);

        if ($filtrprava) {
            $result .= ' AND (' . $filtrprava . ')';
        }


        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && $this ->filterForm ->kodRozdeleni !== null ) {
            $filtr = $this->filterForm->vratSQLFiltrRozdeleni('PracovniPomer', $this ->filterForm ->vcetnePodrizenychRozdeleni);

            if ($filtr) $result .= ' AND (' . $filtr . ')';
        }


        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) && $this ->filterForm ->kodRozdeleni2) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
                'PracovniPomer',
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this ->filterForm ->kodRozdeleni2,
                $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo,
                $this ->filterForm ->vcetnePodrizenychRozdeleni2); //

            if ($filtr) $result .= ' AND (' . $filtr . ')';
        }

        switch ((int)RonPortalSession::getValue('filtr_urovenschvaleni')) {
            case 1 :
                $filtruroven =
                    '(SchvaleniDochazky.ZpracovanoDo IS NULL  OR  SchvaleniDochazky.ZpracovanoDo < ' .
                    $this ->database ->dateToString($this ->filterForm ->vybraneObdobiDo) . ')';
                break;
            case 2 :
                $filtruroven =
                    '(SchvaleniDochazky.ZkontrolovanoDo IS NULL  OR  SchvaleniDochazky.ZkontrolovanoDo < ' .
                    $this ->database ->dateToString($this ->filterForm ->vybraneObdobiDo) . ')';
                break;
            case 3 :
                $filtruroven =
                    '(SchvaleniDochazky.SchvalenoDo IS NULL  OR  SchvaleniDochazky.SchvalenoDo < ' .
                    $this ->database ->dateToString($this ->filterForm ->vybraneObdobiDo) . ')';
                break;

            default :
                $filtruroven = null;
                break;
        }

        if ($filtruroven) $result .= ' AND ' . $filtruroven;


        return $result;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        return
            parent::getRowAttributes($values, 'schvalovani_row cursor-pointer ' . $className, $rowNumber) .
            ' data-idpp="' . (int)$values[0] . '"' .
            ' data-obdobi="' . date('d.m.Y', $this ->filterForm ->vybraneObdobiOd) . '"';

//        $onclick = "ronPortal.dochazka.zobrazSchvaleniDochazky({$values[0]}, '". date('d.m.Y', RonPortalSession::obdobi()) . "'); return false;";
//        $attrs .= ' onclick="' . $onclick;
//
//        return $attrs;
    }


    function enableLine($values)
    {
        return SchvaleniDochazky::lzeSchvalovatOsobu($values[0], $this ->filterForm ->vybraneObdobiOd, $this ->filterForm ->vybraneObdobiDo);
    }
}
