<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;

class VariantniVysledkyBrowser1 extends VysledkyBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'variantnivysledky');

        $this ->code = 'browservariabilni';
        $this->title = \Ron\ronText('Prehled_variantnich_vysledku');

        $this ->sqlSelectPart =
            'VariantniVysledek.TypCasoveSlozkyMzdy, TypCasoveSlozkyMzdy.Popis, ' . // 0,1
            ' VariantniVysledek.Hodiny, VariantniVysledek.Dny, VariantniVysledek.KalendarniDny'; // 2..4

        $this->sqlFromPart = 'VariantniVysledek, TypCasoveSlozkyMzdy';
        $this->sqlWherePart = 'TypCasoveSlozkyMzdy.Kod = VariantniVysledek.TypCasoveSlozkyMzdy';

        $this->addField()
            -> setSqlFieldName('VariantniVysledek.TypCasoveSlozkyMzdy', 0)
            -> setCode('col-kod')
            -> setHeaderText('Kod_casove_slozky') -> setOrdering(1);

        $this->addField()
            -> setSqlFieldName('TypCasoveSlozkyMzdy.Popis', 1)
            -> setCode('col-popis')
            -> setHeaderText('Casova_slozka') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.Hodiny', 2)
            ->setCode('col-hodiny')
            ->setHeaderText('Vysledek_Hodiny') ->setFormat('hodiny') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.Dny', 3)
            ->setCode('col-dny')
            ->setHeaderText('Vysledek_Pracovni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.KalendarniDny', 4)
            ->setCode('col-kalendarni-dny')
            ->setHeaderText('Vysledek_Kalendarni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->defaultSort = 0;
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action);

        $this->filterForm->nastavVariantni(true);

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava->zobrazitVyberRozdeleni()) {
            $this->filterForm->nastavRozdeleni(true);
            $this->filterForm->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;
        }

        $this->filterForm->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
        $this->filterForm->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';

        $this ->filterForm ->nastavAkciGet('variantnivysledky');
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->browserUrl = '?menuid=variantnivysledky&idpp=' . $this ->filterForm ->vybranaOsoba . '&variantni=' . DateUtils::dateToUrlString($this ->filterForm ->vybraneObdobiOd);
    }


    function renderFilter()
    {
        $this->odkazyVariabilniVysledky('variantnivysledky');
        parent::renderFilter();
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'VariantniVysledek.VariantniObdobi = ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::variantniObdobi())) .
            ' AND VariantniVysledek.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND VariantniVysledek.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND VariantniVysledek.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }
}
