<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\PracovniPomer;

class VysledekKontroly
{
    var $datum;
    var $ignorovat;
    var $kod;
    var $popis;
    var $kategorie;
    var $mesicni;
    var $blokovatSchvaleni1;
    var $blokovatSchvaleni2;
    var $blokovatSchvaleni3;
    var $barva;
    var $cssClass;

    var $poznamka;


    /**
     * @param Database $database
     * @param array $row
     */
    function loadData($database, $row) {
        $this ->datum = $database ->stringToTimestamp($row[0]);
        $this ->ignorovat = $row[1] == 1;
        $this ->kod = $row[2];
        $this ->popis = $row[3];
        $this ->kategorie = $row[4];
        $this ->mesicni = $row[5] == 1;
        $this ->blokovatSchvaleni1 = $row[6] == 1;
        $this ->blokovatSchvaleni2 = $row[7] == 1;
        $this ->blokovatSchvaleni3 = $row[8] == 1;
        $this ->barva = $row[9];
        $this ->cssClass = $row[10];
        $this ->poznamka = $row[11];
    }


    /**
     * Vraci seznam vysledku kontrol pro zadany pracovni pomer a obdobi - serazeno podle datumu
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return VysledekKontroly[]
     */
    static function nactiVysledkyKontrol($database, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT VysledekKontrolyDochazky.Datum, VysledekKontrolyDochazky.Ignorovat, TypKontrolyDochazky.Kod, ' . // 0..2
            ' TypKontrolyDochazky.Popis, TypKontrolyDochazky.Kategorie, TypKontrolyDochazky.Mesicni, ' .             // 3..5
            ' TypKontrolyDochazky.BlokovatSchvaleni1,  TypKontrolyDochazky.BlokovatSchvaleni2, TypKontrolyDochazky.BlokovatSchvaleni3, ' . // 6..8
            ' TypKontrolyDochazky.Barva, TypKontrolyDochazky.cssClass, ' . // 9,10
            $database ->castWideMemo('VysledekKontrolyDochazky.Poznamka') . // 11
            ' FROM VysledekKontrolyDochazky, TypKontrolyDochazky ' .
            ' WHERE VysledekKontrolyDochazky.TypKontroly = TypKontrolyDochazky.Kod ' .
            ' AND RC = ' . $params ->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)) .
            ' AND Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            ' AND Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' ORDER BY Datum, TypKontrolyDochazky.Kod';

        $result = [];

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani vysledku kontrol!');

        while ($row = $cursor ->fetch()) {
            $vysledek = new VysledekKontroly();
            $vysledek ->loadData($database, $row);
            $result[] = $vysledek;
        }

        $cursor ->close();

        return $result;
    }


    function htmlPriznak($index, $zobrazitDatum = false)
    {
        $datumStr = ($zobrazitDatum && !$this ->mesicni)
            ? StringUtils::format('date', $this ->datum) . ' '
            : '';
        return
            "<div class='inline-glyph kontrola_priznak fa kontrola_{$this ->kategorie}" . ($this ->cssClass ? ' ' . $this ->cssClass : '') . ($this ->ignorovat ? ' kontrola_ignorovat' : '') . "' " .
            " title='" . $datumStr . StringUtils::htmlTextConvert($this ->popis) . ($this ->poznamka ? "\n" . StringUtils::htmlTextConvert($this ->poznamka) : '') . "' data-ron-index='$index'></div>";
    }


    /**
     * Zobrazi dialog s vysledky kontrol
     * @param VysledekKontroly[] $kontroly
     */
    public static function zobrazDialogKontrol($kontroly)
    {
        if (!$kontroly) return;
        require('ronportal/dochazka/view_vysledek_kontroly.php');
    }
}
