<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Dokumenty;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Class ZruseniDokumentuField
 * Mazani dokumentu pomoci linku (GET)
 */
class ZruseniDokumentuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete"></i>',
            TextManager::_('Opravdu_odstranit_dokument'),
            $this->link . '&record=' . $rowData[0]);
    }
}


/**
 * Class ZruseniDokumentuPostField
 * Mazani dokumentu pomoci POST nadrazeneho formulare. Formular musi obsahovat input s id="deleteDocumentId"
 */
class ZruseniDokumentuPostField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $text = TextManager::_('Opravdu_odstranit_dokument');
        return "<i class='inline-glyph glyph-button fa glyph-delete delete-document-post' data-ron-confirm-text='$text' data-document-id='{$rowData[0]}'></i>";
    }
}



class DokumentyBrowser extends Browser
{
    var $downloadLink = '';

    var $deleteType = 0;
    var $deleteLink = null;

    var $datumField = null;

//    var $prvniDokument;  // pomocne promenne pro zobrazeni sekci
//    var $nadpisSekce;

    var $downloadField;

    // nactena data - jednotlive sekce
    //var $cursors = [];
    //var $sectionCaptions = [];


    /**
     * @param Database $database
     * @param int    $deleteType    Zpusob smazani (0=nic, 1=pres odkaz, 2=pres POST - nadrazeny formular musi obsahovat hidden pole deleteDocumentId)
     * @param string $downloadLink  Pouzije se pro stahnuti dokumentu
     * @param string $deleteLink    Pouzije se pro smazani, pokud je $deleteType=1
     */
    function __construct($database, $downloadLink = null, $deleteType = 0, $deleteLink = null)
    {
        parent::__construct($database);

        $this->downloadLink = $downloadLink;
        $this ->deleteType = $deleteType;
        $this ->deleteLink = $deleteLink;

        $this->title = TextManager::_('Dokumenty');
        $this->tableId = 'Dokumenty';
        $this->sqlFromPart = 'Dokument, TypDokumentu';
        $this->sqlWherePart = 'Dokument.Typ = TypDokumentu.Kod';
        $this->sqlSelectPart = // POZOR! V pripade zmeny se musi upravit showDokumentyDochazky!
            'Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru';

        $this ->primaryField = 0;
        $this->distinct = true;

        $this->cssClass = 'browser minwidth';

        //$this->prvniDokument = true;

        $this->defaultSort = 1; // podle datumu
        $this ->datumField = $this ->addField()
            ->setSqlFieldName('Dokument.Datum', 1) ->setHeaderText('Date') ->setFormat('date') ->setOrdering(1);

        $this ->addField()
            ->setSqlFieldName('TypDokumentu.Popis', 2) ->setHeaderText('Typ_dokumentu') ->setOrdering('ordering');

        $this ->addField()
            ->setSqlFieldName('Dokument.Popis', 3) ->setHeaderText('Popis') ->setOrdering(1);

        // odkaz na stahnuti souboru
        $this ->addField()
            ->setSqlFieldName('Dokument.JmenoSouboru', 4)
            ->setLink($this ->downloadLink) ->setLinkAttributes(['target' => '_blank']);

        switch ($this ->deleteType) {
            case 0 :
                break;

            case 1 :
                // odkaz na smazani souboru
                if ($this->deleteLink) {
                    $deleteField = new ZruseniDokumentuField();
                    $deleteField ->link = $this->deleteLink; // '?menuid=zrusitdokument&' . $this->downloadLink;
                    $this->addField($deleteField);
                }
                break;

            case 2 :
                $deleteField = new ZruseniDokumentuPostField();
                $this->addField($deleteField);
                break;
        }
    }

}
