<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use RonPortal\Dochazka\OperaceDochazky;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\XML\RonPortalXMLForm;

class EditaceUtils
{
    const SETTINGS_ROZDELENI_POVINNE = 'rozdelenieditacepovinne';

    /**
     * @param Database $database
     * @param string   $kodOperace
     * @return bool
     */
    public static function jeRozdeleniDochazkyPovinne($database, $kodOperace)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            return false;
        }

        if ( RonPortalSettings::getIntValue(self::SETTINGS_ROZDELENI_POVINNE)) {
            return true;
        }

        return OperaceDochazky::rozdeleniDochazkyPovinneProOperaci($database, $kodOperace);
    }


    /**
     * Pro zadany PP a den vraci vychozi smenu
     * - smenu, ktera je v kalendari pro dany den
     * - kdyz zadna neni, tak prvni ze seznamu pouzivanych smen daneho kalendare
     *
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int  $den
     * @return string|null
     */
    public static function vychoziSmenaProOsobu($database, $pp, $den)
    {
        $kal = PrirazeniKalendare::nactiKalendarProDen($database, $pp ->idPP, $den);
        if (!$kal) {
            return null;
        }

        if ($kal ->dochazkaPodlePlanKapacit) {
            // nacte naplanovanou smenu
            $smeny = SmenaProDen::nactiSmenyPodleKapacit($database, $pp, $den, $den);
            if (count($smeny)) {
                return $smeny[0] ->kod;
            }
        }

        $smeny = SmenaProDen::nactiSmenyPodleKalendare($database, $pp, $den, $den);
        if (count($smeny)) {
            return $smeny[0] ->kod;
        }

        // nenasel smenu pro dany den, zkusi najit povolenou smenu
        $params = $database ->createParameters();
        $sql =
            'SELECT PouzivaneSmeny.KodSmeny ' .
            '  FROM PouzivaneSmeny ' .
            ' WHERE PouzivaneSmeny.KodKalendare = ' . $params ->addParameter($kal ->kod) .
            ' ORDER BY PouzivaneSmeny.Poradi '; // kdyz jich je vic
        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0];
        }

        return null;
    }


    public static function jePravoEditovatOperaci($operace)
    {
        $prava = Prava::vratPrava('editace');
        if ($prava ->priznak3 == '') {
            return true;
        }

        return in_array($operace, explode(';', $prava ->priznak3));
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $context
     * @param bool $exceptionOnError
     */
    public static function editaceDochazkyEvent($database, $pp, $den, $context, $exceptionOnError = true)
    {
        $xmlParams = [
            'idPP' => $pp ->idPP,
            'RC' => $pp ->rc,
            'osobniCislo' => $pp ->osobniCislo,
            'datumNastupu' => $pp ->datumNastupu,
            'den' => $den
        ];

        RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::EDITACE_DOCHAZKY_EVENT,
            $xmlParams, $context, false, $exceptionOnError);

    }
}
