<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Editace;

use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\Profiler;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\Dochazka\SchvaleniView;
use RonPortal\Dochazka\ZaznamZpracovaneDochazky;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\Kapacity\KapacitySettings;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use RonPortal\System\Report;

class ZpracovanaDochazkaBrowser extends RonPortalBrowser
{
    /**
     * Nazvy polozek nastaveni
     */
    const SETTINGS_MESICNI1 = 'zpracovana_mesicni1';
    const SETTINGS_MESICNI2 = 'zpracovana_mesicni2';
    const SETTINGS_CO_ZOBRAZIT = 'zpracovana_cozobrazit'; // 0=podle volby uzivatele, 1=graficka, 2=text, 3=oboji, 4=nic

    var $povolitEditaci;
    var $zpracovanoDo; // datum zpracovani - nelze vkladat zaznamy pro dny po tomto datu

    /** @var DataZpracovaneDochazky */
    var $denniVysledky;

    var $grafickaWidth;

    /** @var PravaNaOsoby */
    var $pravaEditace;
    /** @var PravaNaOsoby */
    var $pravaAbsence;
    /** @var bool   priznak, ze formular obsahuje editovatelne prvky */
    var $jeEdit;

    /** @var EditaceMesicnich */
    var $editaceMesicnich1;
    /** @var EditaceMesicnich */
    var $editaceMesicnich2;

    var $pocetSloupcuDochazky; // pocet sloupcu zjisteny podle nejdelsiho radku
    var $zobrazitPoznamky;

    /** @var Report[] */
    var $reportList = [];

    var $jsonData;


    function __construct($database)
    {
        parent::__construct($database, 'zpracovanadochazka');

        $this ->jsonData = [];

        $this ->cssClass = 'browser grid zpracovanadochazka keyInputMove';
        $this ->tableId = 'zpracovanadochazkatable';
        $this ->noHeader = true;

        $this ->pravaEditace = Prava::vratPrava('editace');
        $this ->pravaAbsence = Prava::vratPrava('absence');

        $this ->title = \Ron\ronText('Zpracovana_dochazka_zamestnance');

        $this ->denniVysledky = new DataZpracovaneDochazky($this ->database);

        $settings = null;
        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA) ||  RonPortalSettings::getIntValue(RonPortalSettings::MODUL_PLUS)) {
            // od 11-2017 - se nacita z SYS_BVAR_GLOBAL, od 06-2018 se v aplikaci uklada do SYS_BVAR_GLOBAL)
            $settings = RonPortalSettings::getBvarSettings($this ->database, 'RonPortal_Nastaveni_zpracovane', RonPortalSettings::getMultilineValue('nastavenizpracovana'));
        }

        if ($settings) {
            $this ->denniVysledky ->nactiNastaveniXML( $settings );
        } else {
            // bez licence edit nebo kdyz nejsou sloupce nastaveny
            $this ->denniVysledky ->vychoziNastaveni(); // nactiNastaveni_OLD();
        }
    }



    function setFilterForm()
    {
        parent::setFilterForm(); // obdobi a osoba
        $this ->filterForm ->nastavAkciGet('zpracovanadochazka');

        $this ->filterForm ->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';

        if ($this ->zobrazitText()) {
            if (isset($_GET['zobrazitpoznamky'])) {
                $this ->zobrazitPoznamky = (int)$_GET['zobrazitpoznamky'];
                NastaveniUzivatele::setUserSettingsItem($this ->database, 'zobrazitpoznamkyzpracovane', $this ->zobrazitPoznamky, 0);
            } else {
                $this ->zobrazitPoznamky = NastaveniUzivatele::getUserSettingsItem($this ->database, 'zobrazitpoznamkyzpracovane', 0);
            }

            if ($this ->zobrazitPoznamky) {
                $this->filterForm->nastavLink('<a href="?menuid=zpracovanadochazka&zobrazitpoznamky=0">' . \Ron\ronText('Skryt_poznamky') . "</a>");
            } else {
                $this->filterForm->nastavLink('<a href="?menuid=zpracovanadochazka&zobrazitpoznamky=1">' . \Ron\ronText('Zobrazit_poznamky') . "</a>");
            }
        }
    }


    function renderFilter()
    {
        echo \Ron\ronText('Zpracovana_dochazka_info_text');
        parent::renderFilter();
    }

    function loadFilterData()
    {
        $this ->setFilterForm();
        $this ->filterForm ->loadData();
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['zpracovana_cozobrazit'])) {
            NastaveniUzivatele::setUserSettingsItem($this ->database, 'zpracovana_cozobrazit', (int)$_GET['zpracovana_cozobrazit'], 0);
        }

        $this ->loadFilterData();

        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $this ->filterForm ->vybraneObdobiOd);

        try {
            $this->nactiDataZpracovane($this->filterForm->vybranaOsoba, $obdobi);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        $this ->reportList = Report::loadFormReports($this ->database, 'ZpracovanaDochazka', 'ZpracovanaDochazka'); // 3. parametr je deprecated - jen pro kompatibilitu
    }


    private function lzeEditovatAbsence()
    {
        // TODO: doplnit kontrolu maximalniho poctu dnu pro vkladani absenci
        return
            RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  &&
            $this ->pravaAbsence ->lzeZobrazitPP($this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo);
    }


    function zobrazitTlacitkoSchvaleni($uroven)
    {
        if ($this ->denniVysledky ->obdobiUzavreno) {
            return false;
        }

        if (!$uroven) {
            return $this ->denniVysledky ->schvaleni ->lzeSchvalitNejvyssiLevel(
                $this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumDo,
                $this ->denniVysledky->datumOd, $this ->denniVysledky ->datumDo);
        }

        return $this ->denniVysledky ->schvaleni ->lzeSchvalitUroven(
            $this ->denniVysledky ->pp ->idPP, $uroven, $this ->denniVysledky ->datumDo,
            $this ->denniVysledky->datumOd, $this ->denniVysledky ->datumDo);
    }


    function zobrazitTlacitkoZrusitSchvaleni($uroven)
    {
        if ($this ->denniVysledky ->obdobiUzavreno) {
            return false;
        }

        if (!$uroven) {
            return $this ->denniVysledky ->schvaleni ->lzeOdschvalitNejvyssiLevel(
                $this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo );
        }

        return $this ->denniVysledky ->schvaleni ->lzeOdschvalitUroven( $this ->denniVysledky ->pp ->idPP, $uroven,
            $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo);
    }


    /**
     * @param TlacitkoZpracovaneDochazky $tlacitko
     * @param string[] $parametry
     */
    function zobrazTlacitko($tlacitko, $parametry)
    {
        // doplni parametry nastavene v tlacitku
        if (!isset($parametry['type'])) $parametry['type'] = 'submit';
        if (!isset($parametry['class'])) $parametry['class'] = $parametry['type'] == 'submit' ? 'submitbutton' : 'button';

        if (isset($tlacitko ->text)) $parametry['value'] = $tlacitko ->text;
        if (isset($tlacitko ->title)) $parametry['title'] = StringUtils::htmlTextConvert($tlacitko ->title);

        if ($tlacitko ->styl) $parametry['style'] = $tlacitko ->styl;
        if (isset($tlacitko ->dotaz)) $parametry['data-ron-question'] = $tlacitko ->dotaz;

        if ($tlacitko ->umisteni == 5) echo "<div>"; // v pravem postrannim panelu zabali tlacitka do divu
        echo '<input ';
        foreach ($parametry as $paramName => $paramValue) {
            echo $paramName . '="' . $paramValue . '" ';
        }
        echo '/>';

        if ($tlacitko ->umisteni == 5) echo "</div>";
    }

    /**
     * Zobrazi tlacitka, ktera jsou nakonfigurovana na zadanem umisteni
     * @param $umisteni
     * @throws \Exception
     */
    function zobrazTlacitka($umisteni)
    {
        foreach ($this ->denniVysledky ->tlacitka as $tlacitko) {
            if ($tlacitko ->umisteni == $umisteni  && $this ->denniVysledky ->lzePouzitTlacitko($tlacitko)) {
                switch ($tlacitko ->typ) {
                    case 'ulozit' :
                        if ($this ->povolitEditaci  &&  $this ->jeEdit) {
                            $this ->zobrazTlacitko($tlacitko, []);
                            //echo "<input class='submitbutton' type='submit' value='{$tlacitko ->text}' $stylstr />";
                        }
                        break;

                    case 'prepocitat' :
                    case 'prepocet' :
                        if ($this ->povolitEditaci && RonPortalSettings::getSpousteniADS()) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => 'prepocet']);
                        }
                        break;

                    case 'kontrola' :
                    case 'vlozitprestavky' :
                        if ($this ->povolitEditaci && RonPortalSettings::getSpousteniADS()) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ]);
                        }
                        break;

                    case 'rozdelenidv' :
                    case 'rozdelenimv' :
                    case 'kalendarosoby' :
                        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ]);
                        }
                        break;

                    case 'rozdelenivyroby' :
                        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  &&  RonPortalSettings::getIntValue('vyroba')) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ]);
                        }
                        break;

                    case 'schvaleni' :
                    case 'schvalit' :
                        if ($this ->zobrazitTlacitkoSchvaleni($tlacitko ->urovenSchvaleni)) {
                            $urovenstr = $tlacitko ->urovenSchvaleni ? (int)$tlacitko ->urovenSchvaleni : '';
                            $this ->zobrazTlacitko($tlacitko, ['name' => "schvalit$urovenstr", 'dotaz' => 'test']);
                        }
                        break;

                    case 'dialogschvaleni' :
                        if ( $this ->denniVysledky ->schvaleni ->lzeSchvalovatOsobu($this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo) ) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ]);
                        }
                        break;

                    case 'zrusitschvaleni' :
                        if ($this ->zobrazitTlacitkoZrusitSchvaleni($tlacitko ->urovenSchvaleni)) {
                            $urovenstr = $tlacitko ->urovenSchvaleni ? (int)$tlacitko ->urovenSchvaleni : '';
                            $this ->zobrazTlacitko($tlacitko, ['name' => "zrusitschvaleni$urovenstr"]);
                        }
                        break;

                    case 'podlepuvodni' :
                        if ($this ->povolitEditaci && RonPortalSettings::getSpousteniADS()) {
                            // pozor! pokud je tlacitko podlepuvodni vicekrat (napr. nahore a dole), tak se 2x vlozi i hidden input!
                            $this ->zobrazTlacitko($tlacitko, ['type' => 'button', 'name' => $tlacitko ->typ, 'data-ron-question' => \Ron\ronText('Opravdu_podle_puvodni')]);
                            echo "<input type='hidden' name='prepocitatpodlepuvodni' value='0' />\n";
                        }
                        break;

                    case 'doplnitpuvodni' :
                        if ($this ->povolitEditaci && RonPortalSettings::getSpousteniADS()) {
                            // pozor! pokud je tlacitko doplnitpuvodni vicekrat (napr. nahore a dole), tak se 2x vlozi i hidden input!
                            $this ->zobrazTlacitko($tlacitko, ['type' => 'button', 'name' => 'doplnitpuvodnibtn', 'data-ron-question' => \Ron\ronText('Opravdu_doplnit_puvodni')]);
                            echo "<input type='hidden' name='doplnitpuvodni' value='0' />\n";
                        }
                        break;

                    case 'vlozitdokument' :
                        $prava = Prava::vratPrava('editacedokument');
                        if ( $prava ->lzeZobrazitPP($this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo) ) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ]);
                        }
                        break;

                    case 'skript' :
                    case 'xml' :
                    case 'podlekalendare' :
                    case 'interval' :
                        if ($tlacitko ->id) $this ->zobrazTlacitko($tlacitko, ['name' => "{$tlacitko ->typ}_{$tlacitko ->id}"]); // echo "<input class='submitbutton' type='submit' name='skript_{$tlacitko ->id}' value='{$tlacitko ->text}' $stylstr />";
                        else echo $tlacitko ->text . ' - neni nastaveno ID tlacitka';
                        break;
                    //if ($tlacitko ->id) $this ->zobrazTlacitko($tlacitko, ['name' => "podlekalendare_{$tlacitko ->id}"); echo "<input class='submitbutton' type='submit' name='podlekalendare_{$tlacitko ->id}' value='{$tlacitko ->text}' $stylstr />";

                    case 'pk_zrusitsmenu' :
                        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  &&  RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_EDITACE)) {
                            $this ->zobrazTlacitko($tlacitko, ['name' => $tlacitko ->typ, 'data-ron-question' => \Ron\ronText('Opravdu_zrusit_naplanovanou_smenu')]);
                            $prepocitat = (int)$tlacitko ->prepocitat;
                            echo "<input type='hidden' name='pk_zrusitsmenu_prepocitat' value='{$prepocitat}' />\n";
                        }
                        break;

                    case 'pk_zmenitsmenu' :
                        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  &&  RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_EDITACE)) {
                            $this ->zobrazTlacitko($tlacitko, ['type' => 'button', 'name' => $tlacitko ->typ, 'data-ron-prepocitat' => (int)$tlacitko ->prepocitat]);
                        }
                }
            }
        }
    }



    function zobrazitGrafickou()
    {
        if (!$this ->denniVysledky ->filterLinks ->isEmpty()) return true; // ridi se to nastavenim v XML a volbou uzivatele

        switch ( RonPortalSettings::getIntValue(self::SETTINGS_CO_ZOBRAZIT) ) {
            case 4 : // nic
            case 2 : // jen text
                return false;

            case 1 : // jen graficka
            case 3 : // obe
                return true;
        }

        $userCoZobrazit = NastaveniUzivatele::getUserSettingsItem($this ->database, 'zpracovana_cozobrazit', 0);

        // podle volby uzivatele
        if ($userCoZobrazit == 4) return false; // nic
        if ($userCoZobrazit == 2) return false;

        return true;
    }


    function zobrazitText()
    {
        if (!$this ->denniVysledky ->filterLinks ->isEmpty()) return true; // ridi se to nastavenim v XML a volbou uzivatele

        switch ( RonPortalSettings::getIntValue(self::SETTINGS_CO_ZOBRAZIT) ) {
            case 4 : // nic
            case 1 : // jen graficka
                return false;

            case 2 : // jen text
            case 3 : // obe
                return true;
        }

        $userCoZobrazit = NastaveniUzivatele::getUserSettingsItem($this ->database, 'zpracovana_cozobrazit', 0);

        // podle volby uzivatele
        if ($userCoZobrazit == 4) return false; // nic
        if ($userCoZobrazit == 1) return false;

        return true;
    }


    /**
     * @param string $nastaveni
     * @param string $prefix
     * @return EditaceMesicnich
     */
    function createEditaceMesicnich($nastaveni, $prefix)
    {
        if (!$nastaveni) return null;

        $editace = new EditaceMesicnich($this ->database);
        $editace ->f_readonly = !RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  ||  $this ->denniVysledky ->obdobiUzavreno;

        // nacte nastaveni
        $editace ->prefixHTML = $prefix;
        $editace ->loadXML($nastaveni);

        return $editace;
    }


    function nastavMaximalniPocet()
    {
        $this ->pocetSloupcuDochazky = 0;

        foreach ($this ->denniVysledky ->f_data as $radek) {
            $cnt = 0;
            foreach ($radek ->zaznamy as $zaznam) {
                if (!$zaznam ->poPulnoci) $cnt++;
            }

            if ($cnt > $this ->pocetSloupcuDochazky) $this ->pocetSloupcuDochazky = $cnt;
        }
    }


    /**
     * @param int $id
     * @throws \Exception
     */
    function smazDokumentDochazky($id)
    {
        $pp = $this ->denniVysledky ->pp;

        $prava = Prava::vratPrava('editacedokument');
        if (!$prava ->lzeZobrazitPP($pp ->idPP, $this ->denniVysledky ->datumOd)) {
            throw new NeniPravoException();
        }

        // kontrola schvaleni dochazky
        // ... zjisti, ke kteremu dni je dokument pripojen
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Den FROM DokumentDochazky' .
            ' WHERE Id = ' . $params ->addParameter((int)$id) .
            ' AND RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ->datumNastupu)) .
            ' AND PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->denniVysledky ->datumOd)),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        $den = $this ->denniVysledky ->datumDo; // pokud neni pripojen k zadnemu dni, tak kontroluje konec mesice
        if ($row  &&  $row[0]) {
            $den = $this ->database ->stringToTimestamp($row[0]);
        }

        // ... a zkontroluje, jestli lze tento den editovat
        if (!$this ->denniVysledky ->lzeEditovatDen($den)) {
            throw new \Exception( sprintf(\Ron\ronText('Nelze_editovat_den_dochazka_schvalena'), StringUtils::format('date', $den)) );
        }

        $params = $this ->database ->createParameters();
        DokumentyUtils::smazDokument($this ->database, (int)$id, 'ZpracovanaDochazkaBrowser', 'DokumentDochazky',
            'RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ->datumNastupu)) .
            ' AND PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->denniVysledky ->datumOd)),
            $params
        );
    }



    //function nactiDataZpracovane($idPP, $obdobiOd, $obdobiDo)
    /**
     * @param int $idPP
     * @param Obdobi $obdobi
     * @throws \Exception
     */
    function nactiDataZpracovane($idPP, $obdobi)
    {
        if (!$idPP) {
            throw new \Exception(\Ron\ronText('Neni_pracovni_pomer'));
        }

        Profiler::addItem("");
        $item = Profiler::addItem('ZpracovanaDochazkaBrowser::nactiDataZpracovane ----------', 'APP');

        $this->grafickaWidth = NastaveniUzivatele::getUserSettingsItem($this->database, 'editace_graficka_width', 300);

        if ($this ->grafickaWidth < $this ->denniVysledky ->grafickaMinWidth) $this ->grafickaWidth = $this ->denniVysledky ->grafickaMinWidth;
        if ($this ->grafickaWidth > $this ->denniVysledky ->grafickaMaxWidth) $this ->grafickaWidth = $this ->denniVysledky ->grafickaMaxWidth;

        $pp = new OsobaProEditaci();
        $pp ->nactiPPPodleId($this ->database, $idPP);

        //$obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $obdobiOd);

        //$this ->denniVysledky ->nastavViditelnostSloupcu(); // zapne/vypne sloupce podle nastaveni uzivatele
        $this ->denniVysledky ->nastavOsobu($pp);

        $this ->denniVysledky ->nactiData($obdobi ->zacatek, $obdobi ->konec);

        $this ->povolitEditaci =
            RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA) && !$this ->denniVysledky ->obdobiUzavreno;

        if ($this ->povolitEditaci) {
            $this ->povolitEditaci = $this ->pravaEditace ->lzeZobrazitPP(
                $this ->denniVysledky ->pp ->idPP, $this ->denniVysledky ->datumOd, $this ->denniVysledky ->datumDo);
        }

        try {
            $this->zpracovanoDo = DochazkaUtils::datumCasZpracovaniDochazky($this->database);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $this->zpracovanoDo = 0;
        }

//        $refresh = false;

        $this ->editaceMesicnich1 = $this ->createEditaceMesicnich(
            RonPortalSettings::getBvarSettings($this ->database, 'RonPortal_Nastaveni_zpracovane_mv1', RonPortalSettings::getMultilineValue(self::SETTINGS_MESICNI1)), 'hodnotaMV_');
        $this ->editaceMesicnich2 = $this ->createEditaceMesicnich(
            RonPortalSettings::getBvarSettings($this ->database, 'RonPortal_Nastaveni_zpracovane_mv2', RonPortalSettings::getMultilineValue(self::SETTINGS_MESICNI2)), 'hodnotaMV2_');

        if ($this ->editaceMesicnich1) {
            $this ->editaceMesicnich1 ->tableId = 'zpracovanadochazka_mv1table';
            $this->editaceMesicnich1->nactiData($pp, $obdobi);
        }

        if ($this ->editaceMesicnich2) {
            $this ->editaceMesicnich2 ->tableId = 'zpracovanadochazka_mv2table';
            $this->editaceMesicnich2->nactiData($pp, $obdobi);
        }

        Profiler::endItem($item);
        Profiler::addItem("");
    }


    /**
     * Nacte data zadana uzivatelem. Vraci true, pokud data neobsahuji chybu (spatny format a podobne)
     * @return bool
     */
    function nactiPost()
    {
        $result = true;

        if ($this ->editaceMesicnich1) {
            if (!$this->editaceMesicnich1->nactiPost()) $result = false;
        }

        if ($this ->editaceMesicnich2) {
            if (!$this->editaceMesicnich2->nactiPost()) $result = false;
        }

        if (!$this ->denniVysledky ->nactiPost()) $result = false;

        return $result;
    }


    function zpracujPost()
    {
        Profiler::addItem('');
        $item = Profiler::addItem('ZpracovanaDochazkaBrowser::zpracujPost ----------', 'APP');

        if ( isset($_POST['deleteDocumentId'])  &&  (int)$_POST['deleteDocumentId'] ) {
            $this ->smazDokumentDochazky((int)$_POST['deleteDocumentId']);
        }

        if ($this ->editaceMesicnich1) {
            if ($this->editaceMesicnich1->zapisDataOsoby()) {
                $this ->denniVysledky ->jeZmenaVMesicnich = true;
            }
        }

        if ($this ->editaceMesicnich2) {
            if ($this->editaceMesicnich2->zapisDataOsoby()) {
                $this ->denniVysledky ->jeZmenaVMesicnich = true;
            };
        }

        $this ->denniVysledky ->zapisZmeneneHodnoty();

        // hromadna operace s vybranymi radky
        $this ->denniVysledky ->zpracujVybraneRadky();

        $this ->denniVysledky ->zpracujTlacitkoSkript();
        $this ->denniVysledky ->zpracujTlacitkoPodleKalendare();
        $this ->denniVysledky ->zpracujTlacitkoInterval();
        $this ->denniVysledky ->zpracujTlacitkoXML();

        if ($this ->podlePuvodni()) {
            // tlacitko "podle puvodni" nebo "doplnit puvodni"
        } else if ($this ->vlozitPrestavky()) {
            // tlacitko "vlozit prestavky"
        } else if ($this ->denniVysledky ->zpracujTlacitkoPrepocet()) {
            // tlacitka "prepocitat", "kontrola"
        } else {
            $this ->denniVysledky ->provedPrepocet();
        }

        $this ->denniVysledky ->zpracujTlacitkoSchvalit();
        $this ->denniVysledky ->zpracujTlacitkoZrusitSchvaleni();

        Profiler::endItem($item);
        Profiler::addItem('');
    }


    function zobrazHlavickuGraficka($tag = 'th')
    {
        echo "<$tag>";

        echo '<div class="graficka_cntr">';
        // casova osa
        echo '<div class="graficka_timediv">';

        $step =
            $this ->grafickaWidth < 100 ? 6 :
                ($this ->grafickaWidth < 150 ? 4 :
                    ($this ->grafickaWidth < 300 ? 3 :
                        2));
        for ($i = 1; $i < 24 / $step; $i++) {
            echo '<div class="graficka_cas" style="left: ' . ((int)($i * 10000 / 24 * $step) / 100) . '%;">' . ($i * $step) . '</div>';
        }

        echo '</div>';

        // splitter pro zmenu velikosti
        echo "<div class='graficka_splitter' data-min-width='{$this ->denniVysledky ->grafickaMinWidth}' data-max-width='{$this ->denniVysledky ->grafickaMaxWidth}'></div>";

        echo '</div>';

        echo "</$tag>";
    }


    function zobrazHlavickuSloupce($i)
    {
        $sloupec = $this ->denniVysledky ->sloupce[$i];

        $colspanstr = '';
        $dataStr = '';

        switch ($sloupec->typ) {
            case 'DOCH_G' :
                if ($this ->zobrazitGrafickou()) $this ->zobrazHlavickuGraficka();
                return; // nepokracuje!

            case 'DOCH_T' :
                if (!$this ->zobrazitText() || !$this ->pocetSloupcuDochazky) return;
                $colspanstr = ' colspan="' . ((int)$this ->pocetSloupcuDochazky) . '"';
                break;

            case 'EDIT' :
                if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) return;
                break;

            case 'ABSENCE_EDIT' :
                if (!$this ->lzeEditovatAbsence()) return;
                break;

            case 'DV_CHECK' :
                if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)  &&  $sloupec ->lzeEditovat)
                    $dataStr = '<input type="hidden" name="' . 'DV_CHECK_' . trim($sloupec ->kodCasoveSlozky) . '_' . trim($sloupec ->jednotka) . '" value="1"/>';
                break;

            default :
                break;
        }

        $classstr = $this ->denniVysledky ->sloupce[$i] ->headerClass ? " class='{$this ->denniVysledky ->sloupce[$i] ->headerClass}'" : '';
        $alignstr = $this ->denniVysledky ->sloupce[$i] ->align ? " style='text-align:{$this ->denniVysledky ->sloupce[$i] ->align}'" : '';

        echo "<TH{$classstr}{$alignstr}{$colspanstr}>{$this ->denniVysledky ->sloupce[$i] ->nadpis} $dataStr</TH>";
    }


    /**
     * @param ZaznamZpracovaneDochazky $zaznam
     * @param SloupecZpracovaneDochazky $sloupec
     */
    function zobrazZaznamGraficke($zaznam, $sloupec)
    {
        if ($zaznam ->datumCasPocatku > $this ->denniVysledky ->datumDo  &&  $zaznam ->datumCasPocatku >= DateUtils::incDayStart($this ->denniVysledky ->datumDo)) {
            // zaznam z nasledujiciho mesice - zapocitava se do posledniho dne, ale nezobrazuje se
            return;
        }

        $editData = '';

        if ($this ->povolitEditaci) {
            $editData = ($this ->denniVysledky ->editovatVzdyCelyDen)
                ? 'data-ron-datum="' . DateUtils::dateToUrlString($zaznam ->den) . '"'
                : 'data-ron-pocatek="' . $zaznam ->datumCasPocatku . '"';
        }

        echo $zaznam ->htmlGraficka($this ->grafickaWidth, $sloupec ->format, $editData);
    }


    /**
     * @param int $indexRadku
     * @param SloupecZpracovaneDochazky $sloupec
     */
    function zobrazGrafickouSmenu($indexRadku, $sloupec)
    {
        $smena = null;
        switch ($sloupec ->zobrazitSmenu) {
            case 'KAL' :
                $smena = $this ->denniVysledky ->f_data[$indexRadku] ->smenaPodleKalendare;
                break;

            case 'KAP' :
                $smena = $this ->denniVysledky ->f_data[$indexRadku] ->smenaPodleKapacit;
                break;

            case '1' :
            case 'AUTO' :
                if ($this ->denniVysledky ->f_data[$indexRadku] ->kalendarPodleKapacit) {
                    $smena = $this ->denniVysledky ->f_data[$indexRadku] ->smenaPodleKapacit;
                } else {
                    $smena = $this ->denniVysledky ->f_data[$indexRadku] ->smenaPodleKalendare;
                }
                break;
        }

        if ($smena) {
            $left = ((int)(10000 * $smena ->pocatekSmeny * 60 / 86400)) / 100;
            echo '<div ' . "style='left: {$left}%;' class='graficka_smena'></div>";

            $left = ((int)(10000 * $smena ->konecSmeny * 60 / 86400)) / 100;
            echo '<div ' . "style='left: {$left}%;' class='graficka_smena'></div>";

            if ($smena ->pocatekPevneCasti  ||  $smena ->konecPevneCasti) {
                $left = ((int)(10000 * $smena ->pocatekPevneCasti * 60 / 86400)) / 100;
                echo '<div ' . "style='left: {$left}%;' class='graficka_smena graficka_jadro'></div>";

                $left = ((int)(10000 * $smena ->konecPevneCasti * 60 / 86400)) / 100;
                echo '<div ' . "style='left: {$left}%;' class='graficka_smena graficka_jadro'></div>";
            }
        }
    }


    /**
     * @param int $indexRadku
     * @param SloupecZpracovaneDochazky $sloupec
     */
    function zobrazGrafickou($indexRadku, $sloupec)
    {
        $radek = $this ->denniVysledky ->f_data[$indexRadku];

        echo '<TD' . ($sloupec ->class ? ' class="' . $sloupec ->class . '"' : '') . '>';
        echo '<div class="graficka_cntr">';
        echo '<div class="graficka_timediv">';


        foreach ($radek ->zaznamy as $zaznam) {
            $this ->zobrazZaznamGraficke($zaznam, $sloupec);
        }

        $this ->zobrazGrafickouSmenu($indexRadku, $sloupec);

        echo '</div>';

        echo '</div>';
        echo '</TD>';
    }


    /**
     * Zobrazi obsah bunky s textovou dochazkou
     * @param ZaznamZpracovaneDochazky $zaznam
     * @param bool $posledni
     */
    function zobrazZaznamTextove($zaznam, $posledni)
    {
        // pruh nad zaznamem
        echo '<div class="barvadochazky" style="background-color:' . $zaznam ->barva . '"></div>';

        // rozliseni stylu pro editovany zaznam (podle pole "zpracoval")
        $class_str = $zaznam ->editovano ? 'zaznamdochazkyeditovany' : 'zaznamdochazky';
        if (isset($_POST['datumcaspocatku'])) {
            if ($_POST['datumcaspocatku'] == $zaznam ->datumCasPocatku) $class_str .= ' editovanyzaznam';
        }

        // cas od, operace
        echo '<span class="' . $class_str . '">';

        if ($this ->povolitEditaci) {
            if ($this ->denniVysledky ->editovatVzdyCelyDen) {
                echo '<a title="' . \Ron\ronText('Editovat_zaznam') . '" href="?menuid=editacedne&idpp=' . $this ->denniVysledky ->pp ->idPP . '&datum=' . DateUtils::dateToUrlString($zaznam ->den) . '">';
            } else {
                echo '<a title="' . \Ron\ronText('Editovat_zaznam') . '" href="?menuid=editacedochazky&idpp=' . $this ->denniVysledky ->pp ->idPP . '&datumcas=' . $zaznam ->datumCasPocatku . '">';
            }
        }

        $txt_od = date('G:i', $zaznam ->upravaOd + 1); // odecte delta pred, +1 kvuli zaokrouhlovani

        echo $txt_od . ' &nbsp;' . StringUtils::htmlTextConvert($zaznam ->popisOperace);

        if ($this ->povolitEditaci) echo '</a>';

        echo '</span>';

        // delka
        echo ' <span class="podrobnostidochazky">';
        echo '(' . $zaznam ->textDelka . ')';

        // rozdeleni
        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            if (isset($zaznam ->rozdeleni)) echo ' - <strong>' . StringUtils::htmlTextConvert($zaznam ->rozdeleni) . '</strong>'; // kod rozdeleni
        }

        echo '</span>';

        // u posledniho zaznamu konec
        if ($posledni) {
            echo ' <span class="podrobnostidochazky"> >> ' . $zaznam ->textCasDo . '</span>';
        }

        if ($this ->zobrazitPoznamky && $zaznam ->poznamka)
            echo '<div class="poznamka">' . StringUtils::htmlTextConvert($zaznam ->poznamka) . '</div>';
    }


    function zobrazTextovou($indexRadku)
    {
        $radek = $this ->denniVysledky ->f_data[$indexRadku];
        $indexsloupce = 0;

        for ($i = 0; $i < $this ->pocetSloupcuDochazky; $i++) {
            // preskoci zaznamy "po pulnoci"
            while ($indexsloupce < count($radek ->zaznamy) && $radek ->zaznamy[$indexsloupce] ->poPulnoci) $indexsloupce++;

            echo '<td>';
            if ($indexsloupce < count($radek ->zaznamy)) {
                $this ->zobrazZaznamTextove($radek ->zaznamy[$indexsloupce], $indexsloupce == count($radek ->zaznamy) - 1);
            }
            $indexsloupce++;
            echo '</td>';
        }
    }


    function lzeEditovatDen($datum, $kontrolovatPrava = true)
    {
        if ($datum > $this ->zpracovanoDo) return false; // editace jen ve zpracovanych

        if ($datum < $this ->denniVysledky ->pp ->datumNastupu) return false; // pred nastupem

        if ($this ->denniVysledky ->pp ->datumUkonceni  &&  $datum > $this ->denniVysledky ->pp ->datumUkonceni) return false; // po ukonceni PP

        if (!$kontrolovatPrava) return true;

        return
            (!$this ->pravaEditace ->priznak2 || DateUtils::incDay($datum, $this ->pravaEditace ->priznak2) >= DateUtils::startOfTheDay(time())) && // omezeni poctu dnu v pravech
            $this ->denniVysledky ->schvaleni ->lzeEditovatPP($this ->denniVysledky ->pp ->idPP, $datum, null, null);       // datumprav_od/do = null -> kontroluje jen tento den
    }


    /**
     * Pokud bylo stisknuto tlacitko "podle puvodni", provede pregenerovani
     * @return bool
     */
    function podlePuvodni()
    {
        $uloha = null;

        if (isset($_POST['prepocitatpodlepuvodni'])  &&  $_POST['prepocitatpodlepuvodni'] === '1') $uloha = 8;
        if (isset($_POST['doplnitpuvodni'])  &&  $_POST['doplnitpuvodni'] === '1') $uloha = 10;

        if (!$uloha) return false; // nic se neprovadi

        // najde obdobi, ktere je mozne editovat - tj. souvisly interval od posledniho editovatelneho dne
        $datumDo = $this ->zpracovanoDo;
        if ($datumDo > $this ->denniVysledky ->datumDo) {
            $datumDo = $this ->denniVysledky ->datumDo;
        }

        if ($datumDo < $this ->denniVysledky ->datumOd) {
            MessageManager::addDebugMessage(\Ron\ronText('Nelze_prepocitat_neni_zpracovano'));
            return false;
        }

        if ($this ->denniVysledky ->pp ->datumUkonceni  &&  $datumDo > $this ->denniVysledky ->pp ->datumUkonceni) {
            $datumDo = $this ->denniVysledky ->pp ->datumUkonceni;
        }
        if ($datumDo < $this ->denniVysledky ->datumOd) {
            MessageManager::addDebugMessage(\Ron\ronText('Nelze_prepocitat_pracovni_pomer_ukoncen'));
            return false;
        }

        if (!$this ->lzeEditovatDen($datumDo)) return false; // posledni den neni mozne editovat

        $datumOd = $datumDo;
        $pomDatum = DateUtils::incDay($datumDo, -1);

        while ($pomDatum >= $this ->denniVysledky ->datumOd && $this ->lzeEditovatDen($pomDatum)) {
            $datumOd = $pomDatum;
            $pomDatum = DateUtils::incDay($pomDatum, -1);
        }

        $this ->denniVysledky ->prepocetPodlePuvodni($uloha, $datumOd, $datumDo);

        return true; // pregenerovani bylo provedeno
    }


    function vlozitPrestavky()
    {
        if (!isset($_POST['vlozitprestavky'])) return false;
        if (!$_POST['vlozitprestavky']) return false;

        if (!RonPortalSettings::getSpousteniADS()) {
            return MessageManager::addErrorMessage(\Ron\ronText('Nelze_prepocet_nejsou_agenti'));
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            return MessageManager::addErrorMessage(\Ron\ronText('Editace_neni_povolena'));
        }

        if (!$this ->povolitEditaci) {
            return false;
        }

        $params = '7 ' . // uloha
            ' -interval:' . date('Y-n-j', $this ->denniVysledky ->datumOd) . ';' . date('Y-n-j', $this ->denniVysledky ->datumDo) .
            ' -IdPP:' . $this ->denniVysledky ->pp ->idPP .
            ' -alwaysLog ';

        try {
            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
            MessageManager::addInfoMessage( \Ron\ronText('Vlozeni_prestavek_OK') );
        } catch (\Exception $e) {
            MessageManager::addErrorMessage( $e ->getMessage() );
        }

        return true; // pregenerovani bylo provedeno
    }


    function zobrazIkony($indexRadku)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            return;
        }

        $datum = $radek = $this ->denniVysledky ->f_data[$indexRadku] ->datum;
        $lzeEditovatDen = $this ->lzeEditovatDen($datum);

        echo '<TD class="middle no-wrap">';

        if ($this ->povolitEditaci  &&  $lzeEditovatDen) {
            $maxdatumvkladani = DateUtils::startOfTheDay(time());
            if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADANI_DNU)) {
                $maxdatumvkladani = DateUtils::incDay($maxdatumvkladani, RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADANI_DNU));
            }
            if ($this->zpracovanoDo < $maxdatumvkladani) {
                $maxdatumvkladani = $this->zpracovanoDo;
            }

            // pridani noveho zaznamu - v nastaveni aplikace muze byt omezeni na pocet dnu od aktualniho dne
            if ($datum <= $maxdatumvkladani) {
                echo '<a title="' . \Ron\ronText('New_record') . '" href="?menuid=editacedochazky&action=novyzaznam&idpp=' . $this->denniVysledky->pp->idPP . '&datum=' . $datum . '">' .
                    '<span class="inline-glyph glyph-button fa glyph-new" title="' . \Ron\ronText('New_record') . '"></span>' .
                    '</a>';
            } else {
                /** @noinspection HtmlUnknownTarget */
                echo '<i class="fa inline-glyph glyph-transparent"></i>';
            }
        }

        if ($this ->lzeEditovatDen($datum, false)) { // nekontroluje prava - proklik na den zobrazi i kdyz nema prava
            // zobrazeni detailu dne najednou
            echo '<a title="' . \Ron\ronText('Editovat_den') . '" href="?menuid=editacedne&idpp=' . $this->denniVysledky->pp->idPP . '&datum=' . DateUtils::dateToUrlString($datum) . '">' .
                '<span class="inline-glyph glyph-button fa glyph-edit" title="' . \Ron\ronText('Editovat_den') . '"></span>' .
                '</a>';
        }

        if ($this ->povolitEditaci  &&  $lzeEditovatDen) {
            // smazani celeho radku
            echo "<span class='inline-glyph glyph-button fa glyph-delete smazatradekdochazky' title='" . \Ron\ronText('Smazat_cely_radek') .
                "' data-ron-question='" . \Ron\ronText('Opravdu_smazat_cely_radek') . "' data-ron-datum='$datum'></span>";
        }

        echo '</TD>';
    }


    private function zobrazIkonyAbsence($indexRadku)
    {
        if (!$this ->lzeEditovatAbsence()) {
            return;
        }

        $datum = $radek = $this ->denniVysledky ->f_data[$indexRadku] ->datum;
        $lzeEditovatAbsenci = $this ->pravaAbsence ->lzeZobrazitPP($this ->denniVysledky ->pp ->idPP, $datum, $datum);

        echo '<TD class="middle no-wrap">';

        if ($lzeEditovatAbsenci  && $this ->lzeEditovatDen($datum, false)) {
            echo '<a title="' . \Ron\ronText('New_record') . '" href="?menuid=planovani&idpp=' . $this->denniVysledky->pp->idPP . '&datum=' . DateUtils::dateToUrlString($datum) . '&backaction=zpracovanadochazka">' .
                '<span class="inline-glyph glyph-button fa glyph-new" title="' . \Ron\ronText('New_record') . '"></span>' .
                '</a>';
        }

        echo '</TD>';
    }


    /**
     * @param int $datum
     * @param SloupecZpracovaneDochazky $sloupec
     */
    function zobrazDatum($datum, $sloupec)
    {
        if (isset($_POST['datum'])) $zvyraznit = $datum == $_POST['datum'];
        else if (isset($_GET['datumcas'])) $zvyraznit = $datum == DateUtils::startOfTheDay($_GET['datumcas']);
        else if (isset($_GET['datum'])) $zvyraznit = $datum == DateUtils::startOfTheDay(DateUtils::urlStringToDate($_GET['datum'], true));
        else $zvyraznit = false;

        $class_str = 'browser_rowheader';
        // zvyrazneni poeditovaneho radku
        if ($zvyraznit) $class_str .= ' editovanyzaznam';

        if ($sloupec ->class) $class_str .= ' ' . $sloupec ->class;

        $stylestr = $sloupec ->style ? "style='{$sloupec ->style}'" : '';

        echo "<td class='$class_str' $stylestr>";

        if ($zvyraznit) echo '<a name="editovanyden" />';

        echo
            '<a href="?menuid=editacedne&idpp=' . $this ->denniVysledky ->pp ->idPP . '&datum=' . DateUtils::dateToUrlString($datum) . '">' .
            '<span class="zkratkadne">' . \Ron\ronText('Day_short_' .date('w', $datum)) . '</span> ' .
            date('j.n.Y', $datum) .
            '</a>';

        echo '</td>';
    }


    /**
     * @param int $indexSloupce
     * @param DenZpracovaneDochazky $radek
     * @param string $text
     */
    function zobrazEdit($indexSloupce, $radek, $text)
    {
        $sloupec = $this ->denniVysledky ->sloupce[$indexSloupce];
        $chyba = isset($radek ->f_chyby[$indexSloupce]) && $radek ->f_chyby[$indexSloupce];

        if ($sloupec ->typ == 'DV_CHECK') {
            $classstr = 'vysledekcheck';
            if ($chyba) $classstr .= ' app_input_error';

            echo
                '<input type="checkbox" value="1" name="' . $sloupec->getEditName($radek->datum) . '" ' .
                'class="' . $classstr . '"' . ((int)$text ? ' checked="checked"' : '' ) . '/>';
        } else if ($sloupec ->typ == 'P'  ||  $sloupec ->jednotka == 'P') {
            // poznamka ke dni nebo k dennimu vysledku
            $classstr = 'poznamkaedit';

            if ($chyba) $classstr .= ' app_input_error';

            echo '<input type="text" name="' . $sloupec ->getEditName($radek ->datum) . '" class="' . $classstr . '" size="30" value="' . StringUtils::htmlTextConvert($text) . '" />';
        } else {
            $classstr = 'vysledekedit';
            if ($chyba) $classstr .= ' app_input_error';

            if ($sloupec ->typ == 'DOCH1'  ||  $sloupec ->typ == 'DOCH2') $classstr .= ' inputTime';
            else if ($sloupec->jednotka == 'D' || $sloupec->jednotka == 'K') $classstr .= ' inputFloat';
            else if ($sloupec->jednotka == 'H') $classstr .= ' inputTime';

            // denni vysledek, doch1, doch2
            echo
                '<input type="text" name="' . $sloupec->getEditName($radek->datum) . '" ' .
                'class="' . $classstr . '" maxlength="6" size="5" value="' . StringUtils::htmlTextConvert($text) . '" />'; // 6 znaku, aby slo zadat napr. -15:00
        }

        $this ->jeEdit = true;
    }


    /**
     * @param DenZpracovaneDochazky $radek
     */
    function zobrazKontroly($radek)
    {
        foreach ($this ->denniVysledky ->kontroly as $index => $kontrola) {
            if ($kontrola ->datum > $radek ->datum) return; // serazeno podle datumu - nasleduji uz jen kontroly pro dalsi dny

            if (!$kontrola ->mesicni  &&  $kontrola ->datum == $radek ->datum) {
                echo $kontrola ->htmlPriznak($index);
            }
        }
    }


    /**
     * @param DenZpracovaneDochazky $radek
     * @param SloupecZpracovaneDochazky $sloupec
     * @param int $indexSloupce
     */
    function zobrazObsahSloupce($radek, $sloupec, $indexSloupce)
    {
        switch ($sloupec->typ) {
            case 'KONTROLY' :
                $this->zobrazKontroly($radek);
                break;

            default :
                if ($radek) {
                    if (isset($radek->f_texty[$indexSloupce])) $text = $radek->f_texty[$indexSloupce];
                    else $text = '';

                    if ($this->denniVysledky->lzeEditovat($radek, $sloupec)) {
                        $this->zobrazEdit($indexSloupce, $radek, $text);
                    } else {
                        $content = $sloupec->noHtmlConvert ? $text : StringUtils::htmlTextConvert($text);
                        // graficke zobrazeni odpracovane doby
                        if ($sloupec ->zobrazitGraf  &&  $sloupec ->typ == 'DV'  &&  isset($radek ->f_hodnoty[$indexSloupce])  &&  $radek ->f_hodnoty[$indexSloupce]) {
                            //$width = (int)(60 / 12 * $radek ->f_hodnoty[$indexSloupce]);
                            $hodnota = abs($radek ->f_hodnoty[$indexSloupce]);
                            $class = 'dv_graph' . ($radek ->f_hodnoty[$indexSloupce] < 0 ? ' minus' : '');
                            $width = $hodnota
                                ? (int)($hodnota / 24 * 100)
                                : 0;
                            if ($width > 100) $width = 100;
                            if ($width) {
                                echo "<div class='$class' style='width: $width%'></div>";
                            }

                            echo '<span class="dv_graph_text">';
                            echo $content;
                            echo '</span>';
                        } else {
                            echo $content;
                        }
                    }
                }
        }
    }


    /**
     * vraci text bunky TD se schvalenim pro dane datum
     * @param int $datum
     * @param int $index    Pokud neni zadan, vraci nejvyssi uroven schvaleni
     * @return string
     */
    function vratSloupecSchvaleni($datum, $index = null)
    {
        switch ($index) {
            case 1 :
                // zelena
                if ($datum <= $this ->denniVysledky ->schvaleni ->schvaleni1) return "<td class='schvaleni schvaleni1'></td>";
                break;

            case 2 :
                // zluta
                if ($datum <= $this ->denniVysledky ->schvaleni ->schvaleni2) return "<td class='schvaleni schvaleni2'></td>";
                break;

            case 3 :
                // cervena
                if ($datum <= $this ->denniVysledky ->schvaleni ->schvaleni3) return "<td class='schvaleni schvaleni3'></td>";
                break;

            default :
                // nejvyssi schvalena
                $uroven = $this ->denniVysledky ->schvaleni ->nejvyssiUrovenSchvaleni($datum);
                if ($uroven) return "<td class='schvaleni schvaleni{$uroven}'></td>";
        }

        return '<td class="schvaleni"></td>';
    }


    function zobrazSloupec($indexRadku, $indexSloupce)
    {
        $radek = $this ->denniVysledky ->f_data[$indexRadku];
        $sloupec = $this ->denniVysledky ->sloupce[$indexSloupce];

        switch ($sloupec ->typ) {
            case 'SCHVAL' :
                echo $this ->vratSloupecSchvaleni($radek ->datum);
                break;

            case 'SCHVAL1' :
                echo $this ->vratSloupecSchvaleni($radek ->datum, 1);
                break;

            case 'SCHVAL2' :
                echo $this ->vratSloupecSchvaleni($radek ->datum, 2);
                break;

            case 'SCHVAL3' :
                echo $this ->vratSloupecSchvaleni($radek ->datum, 3);
                break;

            case 'DATUM' :
                $this ->zobrazDatum($radek ->datum, $sloupec);
                break;

            case 'DOCH_G' :
                if ($this ->zobrazitGrafickou()) $this ->zobrazGrafickou($indexRadku, $sloupec);
                break;

            case 'DOCH_T' :
                //if ( $this -> ZobrazitText() ) DochazkaBrowser::ZobrazHodnoty($connection, $indexradku);
                if ($this ->zobrazitText()) $this ->zobrazTextovou($indexRadku);
                break;

            case 'EDIT' :
                $this ->zobrazIkony($indexRadku);
                break;

            case 'ABSENCE_EDIT' :
                $this ->zobrazIkonyAbsence($indexRadku);
                break;

            default :
                $classstr = $sloupec ->class;
                if ($radek && isset($radek ->f_hodnoty[$indexSloupce]) && $radek ->f_hodnoty[$indexSloupce] < 0) {
                    $classstr .= ($classstr ? ' ' : '' ) . 'minus';
                }
                if ($classstr) $classstr = "class='$classstr'";


                if ($sloupec ->style) $style = $sloupec ->style;
                else if ($sloupec ->align) $style = 'text-align:' . $sloupec ->align;
                else $style = '';

                $titlestr = $radek && isset($radek ->f_hinty[$indexSloupce]) && $radek ->f_hinty[$indexSloupce]
                    ? "title='" . StringUtils::htmlTextConvert($radek ->f_hinty[$indexSloupce]) . "'"
                    : '';

                echo "<td $classstr $titlestr style='$style'>";

                $this ->zobrazObsahSloupce($radek, $sloupec, $indexSloupce);


                echo '</td>';
                break;
        }
    }


    function zobrazPatickuSloupce($i)
    {
        $colspanstr = '';

        switch ($this ->denniVysledky ->sloupce[$i] ->typ) {
            case 'DOCH_G' :
                if ($this ->zobrazitGrafickou()) $this ->zobrazHlavickuGraficka('td');
                return; // nepokracuje!

            case 'DOCH_T' :
                if (!$this ->zobrazitText() || !$this ->pocetSloupcuDochazky) return;
                $colspanstr = ' colspan="' . ((int)$this ->pocetSloupcuDochazky) . '"';
                break;

            case 'EDIT' :
                if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) return;
                break;

            case 'ABSENCE_EDIT' :
                if (!$this ->lzeEditovatAbsence()) return;
                break;

            default :
                break;
        }

        $classstr = $this ->denniVysledky ->sloupce[$i] ->headerClass ? " class='{$this ->denniVysledky ->sloupce[$i] ->headerClass}'" : '';

        echo "<td{$classstr}{$colspanstr}>{$this ->denniVysledky ->sloupce[$i] ->soucet}</td>";
    }


    function renderMesicniKontroly()
    {
        $pocetSloupcu = 0;
        foreach ($this ->denniVysledky ->sloupce as $sloupec) {
            if ($sloupec->typ  &&  $sloupec->lzeZobrazit) {
                $pocetSloupcu++;
            }
        }

        // zobrazi mesicni kontroly
        foreach ($this ->denniVysledky ->kontroly as $index => $kontrola) {
            if ($kontrola ->mesicni) {
                $class = $kontrola ->cssClass ? $kontrola ->cssClass : ('kontrola_' . $kontrola ->kategorie);
                $styleStr = ($this ->denniVysledky ->obarvitKontroly  && isset($kontrola ->barva))
                    ? 'style="background-color: ' . GuiUtils::getHtmlColor($kontrola ->barva) . ' !important"'
                    : '';
                echo "<tr class='mesicnikontrola $class' $styleStr data-ron-index='$index'>";

                echo
                    "<td colspan='$pocetSloupcu'>" .
                    //"<div class='kontrola_priznak kontrola_" . $kontrola ->kategorie . "'></div>" .   // vypada blbe
                    "<div class='textkontroly'>" .
                    '<div>' . StringUtils::htmlTextConvert($kontrola ->popis) . '</div>' .
                    ($kontrola ->poznamka
                        ? '<div class="poznamka">' . StringUtils::longTextToHtml($kontrola ->poznamka, true) . '</div>'
                        : ''
                    ) .
                    '</div>' .
                    "</td>\n";
                echo "</tr>\n";
            }
        }
    }

    function renderTableFooter()
    {
        if ($this ->denniVysledky ->zobrazitMesicniKontroly == 1  ||  $this ->denniVysledky ->zobrazitMesicniKontroly == 3) {
            $this->renderMesicniKontroly();
        }

        echo '<tr class="browserfooter">';

        for ($i = 0; $i < count($this ->denniVysledky ->sloupce); $i++) {
            //$classstr = $this -> f_dennivysledky -> sloupce[$i] -> f_headerclass ? " class='{$this -> f_dennivysledky -> sloupce[$i] -> f_headerclass}'" : '';
            //echo "<TD $classstr style='text-align:right'>{$this -> f_dennivysledky -> sloupce[$i] -> f_Soucet}</TD>";
            if ($this ->denniVysledky ->sloupce[$i] ->typ  &&  $this ->denniVysledky ->sloupce[$i] ->lzeZobrazit) {
                $this->zobrazPatickuSloupce($i);
            }
        }

        echo "</tr>\n";
    }


    function zobrazRadek($indexRadku)
    {
        for ($i = 0; $i < count($this ->denniVysledky ->sloupce); $i++) {
            if ($this ->denniVysledky ->sloupce[$i] ->typ  &&  $this ->denniVysledky ->sloupce[$i] ->lzeZobrazit) {
                $this->zobrazSloupec($indexRadku, $i);
            }
        }
    }


    function volbaZobrazeni()
    {
        if ($this ->denniVysledky ->filterLinks ->isEmpty()) {
            // puvodni verze nic/text/graficka/vse
            if (!RonPortalSettings::getIntValue(self::SETTINGS_CO_ZOBRAZIT)) {
                // uzivatel si sam urcuje, co se zobrazi
                // prepnuti graficka/textova
                LinksList::renderLinks(
                    ['0' => \Ron\ronText('dochazka_graficka_i_text'), '1' => \Ron\ronText('dochazka_graficka'), '2' => \Ron\ronText('dochazka_textova'), '4' => \Ron\ronText('dochazka_nic')],
                    NastaveniUzivatele::getUserSettingsItem($this->database, 'zpracovana_cozobrazit', 0),
                    '?menuid=zpracovanadochazka&zpracovana_cozobrazit=', null, false);
            }
        } else {
            // filtry nastavene primo v XML pomoci filtr, kodfiltru, filtrgroup
            $this ->denniVysledky ->filterLinks ->url = '?menuid=zpracovanadochazka&';
            $this ->denniVysledky ->filterLinks ->render();
        }
    }


    function showDataDochazka()
    {
        $this ->nastavMaximalniPocet();

        require('ronportal/editace/view_zpracovana.php');
    }


    /**
     * Zobrazi obsah XML formularu nastavenych pomoci tagu <xml> pro danou pozici
     * @param string $position
     */
    function renderXml($position)
    {
        if (!$this ->denniVysledky ->xmlFormulare) {
            return;
        }

        foreach ($this ->denniVysledky ->xmlFormulare as $xml) {
            if ($xml ->umisteni == $position) {
                echo $xml ->obsah;
            }
        }
    }


    function render()
    {
        $this ->renderHeader(); // nadpis a filtr

        if ($this ->denniVysledky ->pp) {
            $this ->renderXml('top');

            echo '<div class="zpracovanadochazkabrowser" id="zpracovanadochazkabrowser">';

            $this->showDataDochazka();

            echo "</div>\n";

            $this ->renderXml('bottom');

            $_SESSION['editace_backaction'] = 'zpracovanadochazka';

            if (isset($_GET['dialogschvaleni']) && !$this->denniVysledky->jeChyba) {
                $schvaleniView = new SchvaleniView($this->database, true);
                $schvaleniView->redirectUrl = '?menuid=zpracovanadochazka&idpp=' . $this ->denniVysledky ->pp ->idPP . '&obdobi=' . DateUtils::dateToUrlString($this ->denniVysledky ->datumOd);
                $schvaleniView->render();

            } else if (isset($_GET['vlozitdokument'])) {
                // uzivatel klikl na tlacitko "vlozit dokument"
                $prava = Prava::vratPrava('editacedokument');
                if ($prava->lzeZobrazitPP($this->denniVysledky->pp->idPP, $this->denniVysledky->datumOd, $this->denniVysledky->datumDo)) {
                    // zobrazi dialog pro vlozeni dokumentu
                    $uploadForm = new DokumentDochazkyUploadForm($this->database, $this->denniVysledky);
                    $uploadForm->render();
                }

                $this->jsonData['showDokument'] = 1;

            } else {
                if (isset($_GET['x']) && (int)$_GET['x']) {
                    $this->jsonData['x'] = (int)$_GET['x'];
                }
                if (isset($_GET['y']) && (int)$_GET['y']) {
                    $this->jsonData['y'] = (int)$_GET['y'];
                }
            }


            $this->jsonData['idPP'] = $this->denniVysledky->pp->idPP;
            $this->jsonData['povolitEditaci'] = $this->povolitEditaci ? 1 : 0;
            $this->jsonData['editovatVzdyCelyDen'] = $this->denniVysledky->editovatVzdyCelyDen ? 1 : 0;
            $this->jsonData['obdobi'] = StringUtils::jsonDate($this ->denniVysledky ->datumOd);
            if (MessageManager::getInstance()->isError()) {
                $this->jsonData['showErrorsDialog'] = 1;
            }
            echo '<script type="text/javascript">ronPortal.zpracovanaDochazka(' . json_encode($this->jsonData) . ');</script>';
        }
    }

}
