<?php
/**
 * UTF8
 */

namespace RonPortal\ELearning;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Prava;


class KurzBrowser extends Browser
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->title = TextManager::_('Kurzy');

        $this->sqlSelectPart =
            'Id, Nazev, PlatnostOd, PlatnostDo, Stav, ' . $database ->castWideMemo('Popis'); // 0..5

        $this->sqlFromPart = 'EKurz';

        $this->defaultSort = 2;
        $this->primaryField = 0;
        $this->rowClickLink = '?menuid=kurzy&action=show';

        $this->code = 'browser_ekurzy';

        $this ->browserUrl = '?menuid=kurzy&';

        // Pole
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Id', 0)
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('EKurz.Nazev', 1) ->setHeaderText('Nazev') ->setLink('?menuid=kurzy&action=show')
            ->setCode('col-nazev')
            ->setOrdering(1);

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('EKurz.Stav', 4) ->setHeaderText('Stav_kurzu')
            ->setCode('col-stav')
            ->setOptions([0 => TextManager::_('Kurz_Stav_0'), 1 => TextManager::_('Kurz_Stav_1'), 2 => TextManager::_('Kurz_Stav_2')])
            ->setOrdering(1) ->setWrap(1);

        $this->addField()
            ->setSqlFieldName('EKurz.PlatnostOd', 2) ->setHeaderText('Platnost_od') ->setFormat('finedate')
            ->setCode('col-platnostod')
            ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('EKurz.PlatnostDo', 3) ->setHeaderText('Platnost_do') ->setFormat('finedate')
            ->setCode('col-platnostdo')
            ->setOrdering(1);
    }



    function renderFilter()
    {
        parent::renderFilter();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if ( Prava::vratHodnotuPrava('editacekurzu') ) {
            echo "<br/><div class='normaltext'><a href='?menuid=kurzy&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
        }
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        // zobrazi podrobnosti
        echo "<tr class='$className'>";
        echo '<td colspan="' . ($this->columnCount) . '"><div class="browserpoznamka">';
        echo $values[5];
        echo '</div></td>';
        echo '</tr>';
    }

}

