<?php
/**
 * stanislav.elbl, 13.11.2015
 */

namespace RonPortal\Hodnoceni;


use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;


class HodnoceniFilterForm extends RonPortalFilterForm
{
    protected $vysledky;
    public $zobrazitZnamky;

    /**
     * HodnoceniFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $vysledky
     * @throws Exception
     */
    function __construct($database, $action, $vysledky)
    {
        parent::__construct($database, $action);

        $this->setUseGet(true, $action);

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->vysledky = $vysledky; // jestli se jedna o zobrazeni vysledku

        if ($vysledky) {
            $datumPovinny = true;
        } else {
            $stav = self::getStavHodnoceni();
            $datumPovinny = ($stav == 2 || $stav == 3 || $stav == 100);
        }

        $this->nastavDatumOdDo(true, $datumPovinny, TextManager::_('Hodnocene_obdobi') . ' ' . TextManager::_('Od'));

        $prava = Prava::vratPrava('hodnoceni');
        if ($prava->jsouNejakePrava())
            $this->nastavOsoby(true, $prava, RonPortalSession::getDatumOd(), RonPortalSession::getDatumDo());

        $this->povolitVsechyOsoby = true;
    }


    public static function getStavHodnoceni()
    {
        return RonPortalSession::getValue('filtrstavhodnoceni');
    }

    public static function getJenTemaHodnoceni()
    {
        return RonPortalSession::getValue('filtrjentemahodnoceni');
    }

    public static function getTemaHodnoceni()
    {
        return RonPortalSession::getValue('filtrtemahodnoceni');
    }

    public static function getJenUcelHodnoceni()
    {
        return RonPortalSession::getValue('filtrjenucelhodnoceni');
    }

    public static function getUcelHodnoceni()
    {
        return RonPortalSession::getValue('filtrucelhodnoceni');
    }

    public static function getFiltrVysledku()
    {
        return RonPortalSession::getValue('filtrvysledekhodnoceni');
    }

    public static function getFiltrovatZnamky()
    {
        return RonPortalSession::getValue('filtrjenznamkyoddo');
    }

    public static function getZnamkaOd()
    {
        return RonPortalSession::getValue('filtrznamkaod');
    }

    public static function getZnamkaDo()
    {
        return RonPortalSession::getValue('filtrznamkado');
    }

    public static function getJenRozdeleniHodnoceni()
    {
        return RonPortalSession::getValue('filtrjenrozdelenihodnoceni');
    }

    public static function getRozdeleniHodnoceni()
    {
        return RonPortalSession::getValue('filtrrozdelenihodnoceni');
    }

    public static function getFiltrObdobi()
    {
        return RonPortalSession::getValue('filtrobdobihodnoceni');
    }


    function loadData()
    {
        parent::loadData();

        $_SESSION['filtrjenrozdelenihodnoceni'] = isset($_GET['jenrozdeleni']) &&  $_GET['jenrozdeleni'];
        if (isset($_GET['rozdeleni'])) {
            $_SESSION['filtrrozdelenihodnoceni'] = $_GET['rozdeleni'];
        }

        $_SESSION['filtrjentemahodnoceni'] = isset($_GET['jentemahodnoceni']) && $_GET['jentemahodnoceni'];
        if (isset($_GET['temahodnoceni'])) {
            $_SESSION['filtrtemahodnoceni'] = $_GET['temahodnoceni'];
        }

        $_SESSION['filtrjenucelhodnoceni'] = isset($_GET['jenucelhodnoceni'])  &&  $_GET['jenucelhodnoceni'];
        if (isset($_GET['ucelhodnoceni'])) {
            $_SESSION['filtrucelhodnoceni'] = (int)$_GET['ucelhodnoceni'];
        }

        $_SESSION['filtrjenznamkyoddo'] = isset($_GET['jenznamkaoddo'])  &&  $_GET['jenznamkaoddo'];

        if (isset($_GET['znamkaod'])) {
            $pom = StringUtils::stringToFloat($_GET['znamkaod']);
            $_SESSION['filtrznamkaod'] = $pom;
        }

        if (isset($_GET['znamkado'])) {
            $pom = StringUtils::stringToFloat($_GET['znamkado']);
            $_SESSION['filtrznamkado'] = $pom;
        }

        $nastavDatum = false;

        if (isset($_GET['obdobi'])) {
            $_SESSION['filtrobdobihodnoceni'] = $_GET['obdobi'];
            NastaveniUzivatele::setUserSettingsItem($this->database, 'hodnocenifiltrobdobi', $_GET['obdobi'], 0);

            $nastavDatum = true;
        } else {
            if (!isset($_SESSION['filtrobdobihodnoceni']))
                $nastavDatum = true;

            $_SESSION['filtrobdobihodnoceni'] = NastaveniUzivatele::getUserSettingsItem($this->database, 'hodnocenifiltrobdobi', 0);
        }

        if ($nastavDatum) {
            switch ((int)$_SESSION['filtrobdobihodnoceni']) {
                case 2:
                    $datumOd = DateUtils::incMonth(DateUtils::firstDayOfMonth(time()), -2);
                    $datumDo = DateUtils::lastDayOfMonth(time());
                    break;
                case 1:
                    $datumOd = DateUtils::firstDayOfYear(time());
                    $datumDo = DateUtils::lastDayOfMonth(DateUtils::incMonth($datumOd, 11));
                    break;
                case 0:
                default:
                    $datumOd = DateUtils::firstDayOfMonth(time());
                    $datumDo = DateUtils::lastDayOfMonth($datumOd);
            }
            RonPortalSession::setFiltrovatDatum(true, $datumOd, $datumDo);
        }
    }


    /**
     * @throws Exception
     */
    function renderRozdeleniHodnoceni()
    {
        echo '<DIV class="filterrow">';

        // checkbox "Jen rozdeleni"
        echo '<SPAN class="filtercaption">';
        global $STR_JenRozdeleni; // kompat.
        GuiUtils::renderCheckbox('jenrozdeleni',
            ($STR_JenRozdeleni ?: TextManager::_('Jen_rozdeleni')), self::getJenRozdeleniHodnoceni(),
            ['id' => 'filterform_jenrozdeleni'], 'clickAutoSubmit', 1);

        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =
            'SELECT Kod, Popis FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI));

        if (RonPortalSession::getFiltrovatDatum()) {
            $sql .=
                ' AND (DatumZruseni >= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumOd())) . ' OR DatumZruseni IS NULL) ' .
                ' AND DatumVytvoreni <= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumDo()));
        } else {
            $sql .= ' AND (DatumZruseni > ' . $params->addParameter($this ->database ->timestampToParam(time())) . ' OR DatumZruseni IS NULL) ';
        }

        $sql .=
            ' ORDER BY Popis';

        // vyber rozdeleni podle popisu
        $combo = new ComboBox('rozdeleni');
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo ->emptyRow = ' ';
        $combo->showCode = true;
        $combo ->loadData($this ->database, $sql, self::getRozdeleniHodnoceni(), $params);
        $combo->htmlParams = [
            'data-ron-autosubmit' => '1',
            'data-ron-autosubmit-check' => '#filterform_jenrozdeleni'
        ];
        $combo ->render();

        echo "</DIV>\n";
    }


    /**
     * @throws Exception
     */
    function renderFilterTemaHodnoceni()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        // checkbox "Jen tema hodnoceni"
        GuiUtils::renderCheckbox('jentemahodnoceni',
            (TextManager::_('Jen_tema_hodnoceni')), self::getJenTemaHodnoceni(),
            ['id' => 'filterform_jentemahodnoceni'], 'clickAutoSubmit', 1);

        echo '</SPAN>';

        $filtrovatdatum = $this->datumOdDoPovinne || RonPortalSession::getFiltrovatDatum();

        $params = $this->database->createParameters();

        $sql =
            'SELECT DISTINCT TemaHodnoceni.TemaCislo, TemaHodnoceni.PopisTematu ' .
            ' FROM TemaHodnoceni, PripraveneTemaHodnoceni ';
        if ($filtrovatdatum) $sql .= ', HodnoceniOsoby, VysledekHodnoceni ';

        $sql .=
            ' WHERE TemaHodnoceni.DruhHodnoceni = 8 ' .
            '   AND TemaHodnoceni.DruhHodnoceni = PripraveneTemaHodnoceni.DruhHodnoceni ' .
            '   AND TemaHodnoceni.TemaCislo = PripraveneTemaHodnoceni.TemaCislo ';

        if ($filtrovatdatum) {
            $sql .=
                ' AND HodnoceniOsoby.ID = PripraveneTemaHodnoceni.ID ' .
                ' AND VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku ' .
                ' AND VysledekHodnoceni.KonecObdobi >= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumOd())) .
                ' AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumDo()));
        } else {
            // odfiltruje neukoncene
            $sql .= ' AND (PripraveneTemaHodnoceni.Stav = 0  OR PripraveneTemaHodnoceni.Stav = 1  OR PripraveneTemaHodnoceni.Stav = 4) ';
        }

        $sql .= ' ORDER BY PopisTematu';

        // puvodni sql nefiltrovalo nepouzivane typy
        //    'SELECT DISTINCT TemaCislo, PopisTematu FROM TemaHodnoceni '.
        //    ' WHERE DruhHodnoceni = 8 '.
        //    ' ORDER BY PopisTematu',


        // combobox pro vyber (tema hodnoceni)
        $combo = new ComboBox('temahodnoceni');
        $combo ->className = 'app_combo adsfilterforminput_full';
        $combo ->emptyRow = ' ';
        $combo->showCode = true;
        //porovnani podle typu je nefunkcni, protoze hodnoty z databaze jsou nacteny jako string
        $combo ->dataTypeComparison = false;
        $combo ->loadData($this ->database, $sql, (int)self::getTemaHodnoceni(), $params); // $keynum, $captionnum, $emptyrow, $selectedvalue);
        $combo->htmlParams = [
            'data-ron-autosubmit' => '1',
            'data-ron-autosubmit-check' => '#filterform_jentemahodnoceni'
        ];
        $combo ->render();

        echo "</DIV>\n";
    }


    /**
     * @throws Exception
     */
    function renderFilterUcelHodnoceni()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        // checkbox "Jen rozdeleni"
        global $STR_UcelHodnoceni; // kompat.
        GuiUtils::renderCheckbox('jenucelhodnoceni',
            ($STR_UcelHodnoceni ?: TextManager::_('Ucel_hodnoceni')), HodnoceniFilterForm::getJenUcelHodnoceni(),
            ['id' => 'filterform_jenucelhodnoceni'], 'clickAutoSubmit', 1);

        echo '</SPAN>';

        $params = $this->database->createParameters();

        $sql =
            'SELECT DISTINCT UcelHodnoceni.Kod, UcelHodnoceni.Popis ' .
            ' FROM UcelHodnoceni, VysledekHodnoceni ' .
            ' WHERE VysledekHodnoceni.KodUcelu = UcelHodnoceni.Kod ' .
            // jen ucely hodnoceni pouzite v zobrazenem obdobi
            '   AND VysledekHodnoceni.KonecObdobi >= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumOd())) .
            '   AND VysledekHodnoceni.PocatekObdobi <= ' . $params->addParameter($this ->database ->timestampToParam(RonPortalSession::getDatumDo())) .
            ' ORDER BY UcelHodnoceni.Popis';

        $combo = new ComboBox('ucelhodnoceni');
        $combo ->className = 'app_combo adsfilterforminput_full';
        $combo ->emptyRow = ' ';
        $combo->showCode = true;
        //porovnani podle typu je nefunkcni, protoze hodnoty z databaze jsou nacteny jako string
        $combo ->dataTypeComparison = false;
        $combo ->loadData($this ->database, $sql, (int)HodnoceniFilterForm::getUcelHodnoceni(), $params);
        $combo->htmlParams = [
            'data-ron-autosubmit' => '1',
            'data-ron-autosubmit-check' => '#filterform_jenucelhodnoceni'
        ];
        $combo ->render();

        echo "</DIV>\n";
    }


    function renderFilterZnamka()
    {
        global $STR_Znamka; // kompat.

        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        //echo '<TR><TD>';

        GuiUtils::renderCheckbox('jenznamkaoddo',
            ($STR_Znamka ?: TextManager::_('Znamka')) . ' ' . TextManager::_('Od'),
            HodnoceniFilterForm::getFiltrovatZnamky(),
            [], 'clickAutoSubmit', 1);

        echo '</SPAN>';
        //echo '</TD><TD>';

        echo '<SPAN class="no-wrap">';
        echo '<INPUT type="text" name="znamkaod" size="5" value="' . self::getZnamkaOd() . '">';

        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
        echo '<INPUT type="text" name="znamkado" size="5" value="' . self::getZnamkaDo() . '">';
        echo '</SPAN>';

        echo "</DIV>\n";
        //echo "</TD></TR>\n";
    }

    function renderDatumOdDo()
    {
        if ($this->nadpisDatumOdDo) $nadpis = $this->nadpisDatumOdDo;
        else $nadpis = TextManager::_('Datum_od');

        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        if ($this->datumOdDoPovinne) {
            echo $nadpis; // .' &nbsp;&nbsp;';
        } else {
            GuiUtils::renderCheckbox('jendatumoddo', $nadpis, RonPortalSession::getFiltrovatDatum(), '', '', null, true);
        }
        echo '</SPAN>';

        echo '<SPAN class="no-wrap">';
        DateInput::renderDateSelect('datumod', date('j.n.Y', RonPortalSession::getDatumOd()), $this ->f_autosubmit); // submitonenter

        echo '&nbsp;&nbsp;&nbsp; ' . TextManager::_('Do') . ' &nbsp;&nbsp;';
        DateInput::renderDateSelect('datumdo', date('j.n.Y', RonPortalSession::getDatumDo()), $this ->f_autosubmit); // submitonenter

        echo '<div class="hodnoceni_nastav_datum_filtru">';
        LinksList::renderLinks(
            [0 => TextManager::_('hodnoceni_aktualni_mesic'), 1 => TextManager::_('hodnoceni_aktualni_rok'), 2 => TextManager::_('hodnoceni_posledni_3_mesice')],
            (int)HodnoceniFilterForm::getFiltrObdobi(), $this->f_formaction . '&obdobi=', '', false);
        echo '</div>';

        echo '</SPAN>';

        echo "</DIV>\n";
    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        if ($this->vysledky && (int)self::getFiltrVysledku() == 0) {
            $this->nastavOsoby(false);
        }

        parent::renderFilters(); // datum

        if (!$this->vysledky || (int)self::getFiltrVysledku() != 0) {
            $this->renderRozdeleniHodnoceni();
        }

        // ucel
        $this->renderFilterUcelHodnoceni();

        // tema (jen u hodnoceni)
        if (!$this->vysledky) $this->renderFilterTemaHodnoceni();

        if ($this->zobrazitZnamky) $this->renderFilterZnamka();

    }

}
