<?php

namespace RonPortal\Jidelna;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\PracovniPomer;


class JidelnaUtils
{
    /**
     * @param Database $database
     * @param int    $id        Pokud je zadano $id, tak vraci pocet objednavek teto polozky jidelnicku,
     * @param string $typ       ... jinak pokud je zadan typ tak vraci pocet objednavek pro zadany typ,
     * @param int    $datum     ... jinak vraci pocet vsech objednavek osoby na zadany datum
     * @param bool   $pouzeNevydane
     * @param PracovniPomer $pp
     * @param int    $stavBurza
     *      0 - bez ohledu na burzu
     *      3 - potvrzene v burze
     *      4 - jakekoliv v burze (nepotvrzene, prave posilane do terminalu i potvrzene)
     * @return int
     * @throws Exception
     */
    static function pocetObjednavek($database, $id, $typ, $datum, $pouzeNevydane = false, $pp = null, $stavBurza = 0)
    {
        $params = $database ->createParameters();

        if ($id)
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo ' .
                ' WHERE ObjednaneJidlo.IDPolozkyJidelnicku = ' . $params ->addParameter((int)$id);
        else if ($typ)
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo, PolozkaJidelnicku ' .
                ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
                ' AND PolozkaJidelnicku.typDennihoJidla  = ' . $params->addParameter($typ) .
                ' AND PolozkaJidelnicku.den  = ' . $params ->addParameter($database ->timestampToParam($datum));
        else
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo, PolozkaJidelnicku ' .
                ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
                ' AND PolozkaJidelnicku.Den = ' . $params ->addParameter($database ->timestampToParam($datum));

        $sql .= // zrusene nepocita
            ' AND ObjednaneJidlo.DatumCasZruseni IS NULL ';

        if ($pp)
            $sql .=
                ' AND ObjednaneJidlo.RC = ' . $params->addParameter($pp ->rc) .
                ' AND ObjednaneJidlo.OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
                ' AND ObjednaneJidlo.DatumNastupu = ' . $params ->addParameter($database ->timestampToParam($pp ->datumNastupu));

        if ($pouzeNevydane)
            $sql .=
                ' AND ObjednaneJidlo.DatumCasVydani IS NULL ';

        // burza
        switch ($stavBurza) {
            case 0 : // bez ohledu na burzu
                break;

            case 1 : // pouze nepotvrzene
            case 2 : // prave odesilane
            case 3 : // pouze potvrzene
                $sql .= ' AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL  AND  ObjednaneJidlo.StavVlozeniNaBurzu = ' . $params ->addParameter((int)$stavBurza);
                break;

            case 4 : // vsechny v burze, bez ohledu na stav
                $sql .= " AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL ";
                break;
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : 0;
    }


//    /**
//     * Nacte pocet objednavek osoby jednotlivych polozek jidelnicku
//     * @param Database $database
//     * @param int $datumOd
//     * @param int $datumDo
//     * @return array    vraci pole ve format IdPolozky => Pocet
//     */
//    public static function nactiPoctyObjednavekOsoby($database, $datumOd, $datumDo)
//    {
//        $sql =
//            'SELECT PolozkaJidelnicku.Id, COUNT(*)' .
//            '  FROM ObjednaneJidlo, PolozkaJidelnicku ' .
//            ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
//            '   AND PolozkaJidelnicku.den  >= ' . $database ->timestampToString($datumOd) .
//            '   AND PolozkaJidelnicku.den  <= ' . $database ->timestampToString($datumDo) .
//            "   AND ObjednaneJidlo.RC = '" . $database ->addSlashes(RonPortalSession::osobaRC()) . "' " .
//            "   AND ObjednaneJidlo.OsobniCislo = '" . $database ->addSlashes(RonPortalSession::osobaOsobniCislo()) . "' " .
//            '   AND ObjednaneJidlo.DatumNastupu = ' . $database ->timestampToString(RonPortalSession::osobaNastup()) .
//            '   AND ObjednaneJidlo.DatumCasZruseni IS NULL ' .
//            ' GROUP BY PolozkaJidelnicku.Id';
//        // jeste by se dal doplnit filtr na jidelnu, ale asi je to zbytecne
//
//        $result = [];
//
//        $cursor = $database ->getSqlResult($sql);
//        while ($row = $cursor ->fetch()) {
//            $result[$row[0]] = (int)$row[1]; // id => pocet
//        }
//
//        return $result;
//    }


//    /**
//     * Vraci pocet vsech objednavek osoby na zadany datum
//     * @param Database $database
//     * @param int $datum
//     * @return int
//     */
//    public static function pocetObjednavekOsobyNaDen($database, $datum)
//    {
//        return self::pocetObjednavek($database, 0, null, $datum, false);
//    }


//    /**
//     * Vraci pocet objednavek osoby pro zadany typ denniho jidla
//     * @param Database $database
//     * @param string $typ
//     * @param int $datum
//     * @return int
//     */
//    public static function pocetObjednavekOsobyProTypJidla($database, $typ, $datum)
//    {
//        return self::pocetObjednavek($database, 0, $typ, $datum);
//    }


//    /**
//     * Vraci pocet objednavek zadane polozky jidelnicku.
//     * @param Database $database
//     * @param int  $id
//     * @param bool $pouzeNevydane
//     * @return int
//     */
//    public static function pocetObjednavekOsobyProPolozku($database, $id, $pouzeNevydane = false)
//    {
//        return self::pocetObjednavek($database, $id, null, 0, $pouzeNevydane);
//    }


    /**
     * @param Database $database
     * @param int $id
     * @return int
     * @throws Exception
     */
    public static function pocetVBurzeCelkem($database, $id)
    {
        return self::pocetObjednavek($database, $id, null, 0, true, null, 3); // jen nevydane = true, pp = null, 3 = potvrzene
    }


    /**
     * @param Database $database
     * @param int $id               Id polozky jidelnicku
     * @param PracovniPomer $pp
     * @return int
     */
    public static function pocetVBurze_Osoba($database, $id, $pp)
    {
        return self::pocetObjednavek($database, $id, null, 0, true, $pp, 4); // jen nevydane, jen osoba, 4 = vsechny v burze
    }


    /**
     * @param $database
     * @param $id
     * @return int
     * @throws Exception
     */
    public static function pocetObjednavekPolozkyCelkovy($database, $id)
    {
        return self::pocetObjednavek($database, $id, null, 0, false);
    }


    /**
     * Vraci pole, ktere obsahuje casOd a casDo - cas vydeje zadane polozky jidelnicku (podle nastaveni u terminalu)
     * @param Database $database
     * @param int $idPolozkyJidelnicku
     * @return array
     * @throws DBException
     */
    public static function casVydejePolozkyJidelnicku($database, $idPolozkyJidelnicku)
    {
        $params = $database ->createParameters();
        $sql =
            "SELECT DISTINCT PolozkaJidelnicku.den, CasovaPredvolbaVydeje.CasOd, CasovaPredvolbaVydeje.CasDo".
            " FROM PolozkaJidelnicku, CasovaPredvolbaVydeje, Terminal, MapovaniJidlaVTerminalu".
            ' WHERE PolozkaJidelnicku.Id = ' . $params ->addParameter((int)$idPolozkyJidelnicku) .
            " AND Terminal.kodTypuRozdeleni = PolozkaJidelnicku.kodTypuRozdeleni".
            " AND Terminal.KodRozdeleni = PolozkaJidelnicku.jidelna".
            " AND MapovaniJidlaVTerminalu.Terminal = Terminal.kod".
            " AND MapovaniJidlaVTerminalu.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla".
            " AND CasovaPredvolbaVydeje.Terminal = Terminal.kod".
            " AND ( CasovaPredvolbaVydeje.KodTypuVTerminalu =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha1 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha2 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu )".
            " ORDER BY CasovaPredvolbaVydeje.CasOd";

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row)
        {
            return [
                'vydejOd' => $database ->stringToTimestamp($row[0]) + 3600 * $database ->stringToTimeOfDay($row[1]),
                'vydejDo' => $database ->stringToTimestamp($row[0]) + 3600 * $database ->stringToTimeOfDay($row[2])
            ];
        }

        return null;
    }
}

