<?php
/**
 * Seznam odberu jidel stravnika
 */

namespace RonPortal\Jidelna;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;


class FieldPopisNabidkyOdberu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // pokud neni zadan text primo u polozky, tak zobrazuje popis z jidelnicku
        return $rowData[4] ?: $rowData[5];
    }
}



class PuvodniOdberBrowser extends RonPortalBrowser
{

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'puvodniodber');

        $this->code = 'browserpuvodniodber';
        $this->title = TextManager::_('Puvodni_odbery');

        $this ->sqlSelectPart =
            'PuvodniOdber.DatumCasOperace, Terminal.Popis, PuvodniOdber.Cena, PuvodniOdber.Mnozstvi,' . // 0..3
            ' PuvodniOdber.Popis, PolozkaJidelnicku.PopisNabidky,' . // 4,5
            ' PuvodniOdber.Karta, PuvodniOdber.Uzivatel, PuvodniOdber.KodZbozi, PuvodniOdber.OdberBezObjednavky,' . // 6..9
            ' PuvodniOdber.DatumCasVlozeniZaznamu, PuvodniOdber.Poznamka'; // 10,11

        // definice dotazu a hlavicky
        $this->sqlFromPart =
            'PuvodniOdber LEFT OUTER JOIN Terminal ON (Terminal.kod = PuvodniOdber.Terminal)' .
            ' LEFT OUTER JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.Id = PuvodniOdber.IdPolozkyJidelnicku)';

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.DatumCasOperace', 0) ->setHeaderText('Datum_a_cas')
            ->setCode('col-datumcas') ->setFormat('datetime') ->setAlign('center') ->setOrdering(1);
        $this->defaultSort = 1;
        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('Terminal.Popis', 1)
            ->setCode('col-terminal') ->setHeaderText('Misto_odberu') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Cena', 2) ->setHeaderText('Cena')
            ->setCode('col-cena') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Mnozstvi', 3) ->setHeaderText('Mnozstvi')
            ->setCode('col-mnozstvi') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this ->addField(new FieldPopisNabidkyOdberu())
            ->setSqlFieldName('PuvodniOdber.Popis', 4)
            ->setCode('col-popis') ->setHeaderText('Popis_odberu') ->setOrdering(0);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Karta', 6)
            ->setCode('col-karta') ->setVisible(0) ->setHeaderText('Cislo_karty') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Uzivatel', 7)
            ->setCode('col-uzivatel') ->setVisible(0) ->setHeaderText('Uzivatel') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.KodZbozi', 8)
            ->setCode('col-kodzbozi') ->setVisible(0)
            ->setHeaderText('Kod_zbozi') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.OdberBezObjednavky', 9)
            ->setCode('col-bezobjednavky') ->setVisible(0) ->setFormat('anone') ->setHeaderText('Kod_zbozi') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.DatumCasVlozeniZaznamu', 10)
            ->setCode('col-casvlozeni') ->setVisible(0) ->setHeaderText('Cas_vlozeni')
            ->setFormat('datetime') ->setAlign('center') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PuvodniOdber.Poznamka', 11)
            ->setCode('col-poznamka') ->setVisible(0) ->setHeaderText('Poznamka') ->setOrdering(1);

        $this ->filterFormAppCode = 'JIDELNA';
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;
    }

    function setFilterForm()
    {
        parent::setFilterForm();
        $this ->filterForm ->nastavAkciGet('puvodniodber');
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        return new SqlStatement(
            'PuvodniOdber.DatumCasOperace >= ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybraneObdobiOd)) .
            ' AND PuvodniOdber.DatumCasOperace <= ' . $params ->addParameter($this ->database ->timestampToParam(DateUtils::endOfTheDay($this ->filterForm ->vybraneObdobiDo))) .
            ' AND PuvodniOdber.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND PuvodniOdber.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND PuvodniOdber.DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }
}
