<?php
/**
 * Roman.Szablatura RON Software 13.09.2019
 */

namespace RonPortal\Jidelna;


use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RozdeleniUtils;


/**
 * Class SeznamObjednavekJidlaFilter
 * @package RonPortal\Jidelna
 */
class SeznamObjednavekJidlaFilter extends RonPortalFilterForm
{

    /**
     * SeznamObjednavekJidlaFilter constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     * @param PravaNaOsoby $pravoZrusit
     */
    public function __construct($database, $pravoPrevedeni, $pravoZrusit)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this->f_classname = 'filtercontainer';
        $this->f_formaction = '?menuid=jidelnaobjednavky';

        $this ->setUseGet(true, 'jidelnaobjednavky');

        Prava::nastavObdobiProPrava(time(),time());
        $this->nastavDatumOdDo(true, true);
        $this->nastavOsoby(true, [$pravoPrevedeni, $pravoZrusit]);
        $this->povolitVsechyOsoby = true;

        $this ->nastavRozdeleni(true);
    }


    function renderDatumOdDoLinks()
    {
        $href =
            "?menuid=jidelnaobjednavky" .
            ($this ->vsechnyOsoby
                ? ''
                : '&osoba=' . $this ->vybranaOsoba
            ).
            (isset($_SESSION['jenTypJidla']) && $_SESSION['jenTypJidla']
                ? '&jenTypJidla=1&typJidla=' . StringUtils::htmlTextConvert($_SESSION['typJidla'])
                : ''
            );

        if ($this ->datumOd != DateUtils::today()  ||  $this ->datumDo != $this ->datumOd) {
            $todayStr = DateUtils::dateToUrlString(time());

            /** @noinspection HtmlUnknownTarget */
            echo
                "<span id='datum_od_do_today' class='margin-left-2'>" .
                "<a href='{$href}&datumod={$todayStr}&datumdo={$todayStr}'>" . TextManager::_('Today') . '</a>' .
                '</span>';
        }

        $monthStart = DateUtils::firstDayOfMonth(time());
        $monthEnd = DateUtils::lastDayOfMonth($monthStart);

        if ($this ->datumOd != $monthStart  ||  $this ->datumDo != $monthEnd) {
            echo
                "<span id='datum_od_do_today' class='margin-left-2'>" .
                " <a href='{$href}&datumod=" . DateUtils::dateToUrlString($monthStart) .
                "&datumdo=" . DateUtils::dateToUrlString($monthEnd) . "'>" . TextManager::_('Aktualni_mesic') . '</a>' .
                '</span>';

        }
    }


    /**
     * @throws \Ron\DB\DBException
     */
    public function renderPouzeTypJidla()
    {
        echo '<div class="filterrow" id="filterrow_typjidla">';
        echo '<span class="filtercaption">';

        GuiUtils::renderCheckbox('jenTypJidla', TextManager::_('Jidelna_pouze_typ_jidla'), isset($_SESSION['jenTypJidla'])  &&  $_SESSION['jenTypJidla'],
            ['id' => 'jen_typ_jidla_checkbox'], 'clickAutoSubmit', 1, true);
        echo '</span>';

        $sql =
            'SELECT Distinct TypDennihoJidla.Kod, TypDennihoJidla.Popis' .
             ' FROM TypDennihoJidla' .
             ' WHERE TypDennihoJidla.Aktivni = 1' .
             ' ORDER BY TypDennihoJidla.Popis';

        $combo = new ComboBox('typJidla');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_typ_jidla_checkbox'];
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('typJidla'));
        $combo->render();

        echo '</div>';
    }


    function loadTypJidla()
    {
        if (isset($_GET['typJidla'])){
            $_SESSION['jenTypJidla'] = isset($_GET['jenTypJidla']) ? (int)$_GET['jenTypJidla'] : 0;
            $_SESSION['typJidla'] = $_GET['typJidla'];
        }
    }


    function loadData()
    {
        $this->loadTypJidla();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderPouzeTypJidla();
    }
}


/**
 * Class DetailPolozkyJidelnicku
 * @package RonPortal\Jidelna
 */
class DetailPolozkyJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo_prevest */
    public $pravo_prevest;

    /** @var PravaNaOsoby $pravo_zrusit */
    public $pravo_zrusit;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_prevest
     * @param PravaNaOsoby $pravo_zrusit
     * @param null $options
     */
    public function __construct($database, $pravo_prevest, $pravo_zrusit, $options = null)
    {
        parent::__construct($database, $options);
        $this->pravo_prevest = $pravo_prevest;
        $this->pravo_zrusit = $pravo_zrusit;
        $this->header = TextManager::_('Detail');
        $this->code = 'col-popis';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        if ($this->pravo_prevest->lzeZobrazitPP($rowData[19]) || $this->pravo_zrusit->lzeZobrazitPP($rowData[19])) {
            $result .= "<a class='detail-objednavky' href='?menuid=jidelnaobjednavky&action=detailobjednavkyjidla&id=$rowData[0]'>";
            $result .= StringUtils::htmlTextConvert($rowData[16]);
            $result .= "</a>";
        } else {
            $result .= StringUtils::htmlTextConvert($rowData[16]);
        }

        return $result;
    }
}


/**
 * Class OdstranitPolozkuJidelnicku
 * @package RonPortal\Jidelna
 */
class OdstranitPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;


    /**
     * DetailPolozkyJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoZruseni
     */
    public function __construct($database, $pravoZruseni)
    {
        parent::__construct($database);
        $this->pravoZruseni = $pravoZruseni;
        $this->code = 'col-zrusit';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = "";

        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if (!$rowData[8]) {
            if ($this->pravoZruseni->lzeZobrazitRC($rowData[2])) {
                $confirmText = sprintf(TextManager::_('Jidelna_zrusit_objednavku_potvrzeni'), $rowData[15]);
                $result .= "<span class='rectangle confirmQuestion js zrusit-polozku-jidelnicku'  
                        data-ron-confirm-text='$confirmText' 
                        data-ron-href='?menuid=jidelnaobjednavky&action=objednavkuodstranitodkazem&osoba=$rowData[21]&id=$rowData[1]' 
                        data-ron-working-info='" . TextManager::_('Jidelna_odstranuji_objednavku') . "'>" . TextManager::_('jidelna_objednavku_odstranit') . "</span>";

            }
        } else {
            $result .= TextManager::_('Jidelna_objednavka_vydana');
        }
        return $result;
    }
}


/**
 * Class PrevestPolozkuJidelnicku
 * @package RonPortal\Jidelna
 */
class PrevestPolozkuJidelnicku extends BrowserField
{
    /** @var PravaNaOsoby $pravo */
    public $pravoPrevedeni;


    /**
     * PrevestPolozkuJidelnicku constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoPrevedeni
     */
    public function __construct($database, $pravoPrevedeni)
    {
        parent::__construct($database);
        $this->pravoPrevedeni = $pravoPrevedeni;
        $this->code = 'col-prevest';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud ma nastaveno datum vydani pak nezobrazit odkaz
        if (!$rowData[8]) {
            if ($this->pravoPrevedeni->lzeZobrazitRC($rowData[2])) {
                //Momentalne nelze pripojit poznamku prevedeni (neni vyreseno datove)
                return
                    "<span 
                        class='objednavku-prevest js' 
                        data-ron-akce='prevestobjednavku'
                        data-ron-idpp='$rowData[21]'
                        data-ron-id-polozky-jidelnicku='$rowData[1]'
                        data-ron-jidelna='" . StringUtils::htmlTextConvert($rowData[22]) . "'
                        data-ron-jidelna-popis='" . StringUtils::htmlTextConvert($rowData[18]) . "'
                        data-ron-jmeno='" . StringUtils::htmlTextConvert($rowData[15]) . "'
                        data-ron-popis-objednavky='" . StringUtils::htmlTextConvert($rowData[16]) . "'
                        data-ron-cislo-objednavky='$rowData[0]'>" . TextManager::_('Jidelna_objednavku_prevest') .
                    "</span>";
            }
        } else {
            return TextManager::_('Jidelna_objednavka_vydana');
        }

        return '';
    }
}


/**
 * Class SeznamObjednavekJidlaBrowser
 * @package RonPortal\Jidelna
 */
class SeznamObjednavekBrowser extends RonPortalBrowser
{

    /** @var PravaNaOsoby $pravoPrevedeni */
    public $pravoPrevedeni;

    /** @var PravaNaOsoby $pravoZruseni */
    public $pravoZruseni;

    /** @var SeznamObjednavekJidlaFilter */
    public $filterForm;


    /**
     * SeznamObjednavekJidlaBrowser constructor.
     * @param Database $database
     * @param $pravoPrevedeni
     * @param $pravoZruseni
     * @throws Exception
     */
    public function __construct($database, $pravoPrevedeni, $pravoZruseni)
    {
        parent::__construct($database, 'jidelnaobjednavky');

        $this->title = TextManager::_('Jidelna_seznam_objednavek');
        $this->code = 'browserjidelnaobjednavky';
        $this->tableId = 'browserjidelnaobjednavky';
        $this->browserUrl = '?menuid=jidelnaobjednavky';

        $this->pravoPrevedeni = $pravoPrevedeni;
        $this->pravoZruseni = $pravoZruseni;

        $this->sqlSelectPart =
            "OJ.ID, OJ.IDPolozkyJidelnicku, OJ.RC, OJ.OsobniCislo, OJ.DatumNastupu, OJ.PoradiVydani, OJ.DatumCasObjednani, " .               //0..6
            "OJ.DatumCasZruseni, OJ.DatumCasVydani, OJ.IDPuvodnihoOdberu, OJ.uzivatel, OJ.DatumCasVlozeniNaBurzu, " .                  //7..11
            "OJ.StavVlozeniNaBurzu, OJ.IDobjednavkyZBurzy, OJ.StavPrevzetiZBurzy, " .                        //12..14
            "COALESCE (Osoba.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} COALESCE (Osoba.Jmeno, '') AS Stravnik, PJ.popisNabidky, PJ.ID, Rozdeleni.Popis, " .       //15..18
            "TypDennihoJidla.Popis, PJ.Den, PracovniPomer.Id, PJ.Jidelna";        //19..22

        $this->sqlFromPart =
            'ObjednaneJidlo AS OJ' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = OJ.RC) AND (PracovniPomer.OsobniCislo = OJ.OsobniCislo) AND (PracovniPomer.DatumNastupu = OJ.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)' .
            ' JOIN PolozkaJidelnicku PJ ON (PJ.ID = OJ.IDPolozkyJidelnicku)' .
            ' JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = 8 AND Rozdeleni.Kod = PJ.jidelna)' .
            ' JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PJ.typDennihoJidla)';

        $this->sqlWherePart =
            'OJ.DatumCasZruseni IS NULL';

        $this->filterForm = new SeznamObjednavekJidlaFilter($this->database, $this->pravoPrevedeni, $this->pravoZruseni);
        $this->filterForm->loadData();

        $this->addField()
            ->setSqlFieldName('Stravnik', 15) ->setOrdering(1) ->setHeaderText('Osoba') ->setClassName('jmeno-stravnika')
            ->setCode('col-osoba') ->setVisible($this ->filterForm ->vsechnyOsoby);

        $this->addField()
            ->setSqlFieldName('OJ.ID', 0) ->setOrdering(1) ->setHeaderText('Jidelna_ID_polozky')
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('PJ.Den', 20) ->setHeaderText('Den') ->setOrdering(1) ->setFormat('finedate') ->setAlign('center')
            ->setCode('col-datum');

        $this->defaultSort = -count($this->fields);

        $this->addField()
            ->setSqlFieldName('TypDennihoJidla.Popis', 19) ->setOrdering(1) ->setAlign('center') ->setHeaderText('Jidelna_typ_jidla')
            ->setCode('col-typjidla');

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis', 18) ->setOrdering(1) ->setHeaderText('Jidelna')
            ->setCode('col-jidelna');

        $this->addField(new DetailPolozkyJidelnicku($database, $this->pravoPrevedeni, $this->pravoZruseni));

        $this->addField(new OdstranitPolozkuJidelnicku($database, $this->pravoZruseni));

        $this->addField(new PrevestPolozkuJidelnicku($database, $this->pravoPrevedeni));
    }


    function setFilterForm()
    {
        // nedela nic - filterform uz byl nastaven v konstruktoru
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result =
            ' PJ.Den >= ' . $params->addParameter($this->database->timestampToParam($this ->filterForm ->datumOd)) .
            ' AND PJ.Den <= ' . $params->addParameter($this->database->timestampToParam($this ->filterForm ->datumDo)) .
            "\n";

        if (!$this ->filterForm ->vsechnyOsoby) {
            $result .=
                ' AND PracovniPomer.Id = ' . $params->addParameter($this ->filterForm ->vybranaOsoba) .
                "\n";
        } else {

            if ($this ->filterForm ->kodRozdeleni != '') {
                $result .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                        $this ->filterForm ->f_typrozdeleni, $this ->filterForm ->kodRozdeleni,
                        time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni, $params) .
                    "\n";
            }
        }

        if (RonPortalSession::getValue('jenTypJidla')){
            $result .=
                ' AND ' .
                ' PJ.typDennihoJidla = ' . $params->addParameter($_SESSION['typJidla']) .
                "\n";
        }

        $filtrPrava = Prava::sqlFiltrPodlePrav([$this ->pravoPrevedeni, $this ->pravoZruseni, Prava::vratPrava('prohl_jidelna')], 'PracovniPomer');
        if ($filtrPrava) {
            $result .= ' AND (' . $filtrPrava . ')' . "\n";
        }

        return new SqlStatement($result, $params);
    }
}
