<?php

/**
 * Prehled vyuctovani stravy po mesicich
 */

namespace RonPortal\Jidelna;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;

class UcetStravnikaBrowser extends RonPortalBrowser
{
    /**
     * @param \Ron\DB\Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'ucetstravnika');

        $this->title = TextManager::_('Ucet_stravnika');
        $this->code = 'browserucetstravnika';

        $this->sqlSelectPart =
            'Obdobi.Popis, Obdobi.PocatekObdobi,' . // 0,1
            ' UcetStravnikaZaObdobi.Castka_Celkem, UcetStravnikaZaObdobi.Castka_Dotace, UcetStravnikaZaObdobi.Castka_SumaOperaciHotove,' . // 2..4
            ' UcetStravnikaZaObdobi.Castka_AktualniStav, UcetStravnikaZaObdobi.Pocet_NarokNaDotace,' . // 5,6
            ' UcetStravnikaZaObdobi.Pocet_Odebrano, UcetStravnikaZaObdobi.Pocet_Dotovano, UcetStravnikaZaObdobi.Pocet_Nedotovano'; // 7..9

        $this->sqlFromPart = 'UcetStravnikaZaObdobi, Obdobi';
        $this->sqlWherePart =
            'UcetStravnikaZaObdobi.Obdobi = Obdobi.PocatekObdobi' .
            " AND Obdobi.Aplikace = 'JIDELNA' ";

        $this->defaultSort = -2;
        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('Obdobi.Popis', 0)
            ->setCode('col-obdobi') ->setHeaderText('Obdobi') ->setOrdering(1);

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Obdobi.PocatekObdobi', 1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_Celkem', 2) ->setHeaderText('Celkem_za_stravu')
            ->setCode('col-castkacelkem') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_Dotace', 3)
            ->setCode('col-castkadotace') ->setHeaderText('Castka_dotace') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_SumaOperaciHotove', 4)
            ->setCode('col-sumahotove') ->setHeaderText('Hotovost') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Castka_AktualniStav', 5)
            ->setCode('col-aktualnistav') ->setHeaderText('Aktualni_stav_uctu') ->setFormat('curr') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_NarokNaDotace', 6)
            ->setCode('col-naroknadotace') ->setHeaderText('Narok_na_dotace') ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Odebrano', 7)
            ->setCode('col-pocetodebrano') ->setHeaderText('Pocet_odebrano') ->setVisible(0)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Dotovano', 8)
            ->setCode('col-pocetdotovano') ->setHeaderText('Pocet_dotovano') ->setVisible(0)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('UcetStravnikaZaObdobi.Pocet_Nedotovano', 9)
            ->setCode('col-pocetnedotovano') ->setHeaderText('Pocet_nedotovano') ->setVisible(0)
            ->setFormat('number_2') ->setAlign('right') ->setOrdering(1);

        $this ->filterFormAppCode = null; // vyber obdobi nezobrazuje
        $this ->filterFormRights = 'prohl_jidelna';
        $this ->filterFormVcetneExternich = true;
    }


    function setFilterForm()
    {
        parent::setFilterForm();
        $this ->filterForm ->nastavAkciGet('ucetstravnika');
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['rok'])) $_SESSION['rok'] = (int)$_GET['rok'];
        if (!isset($_SESSION['rok'])) $_SESSION['rok'] = date('Y');

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        parent::renderFilter();

        $rok = $_SESSION['rok'];

        echo
            '<div class="margin-top-2 margin-bottom-2">' .
            '<a href="?menuid=ucetstravnika&rok=' . ($rok - 1) . '"> << </a>' .
            '<span class="margin-all-2 padding-all-2">' . $rok . '</span>' .
            '<a href="?menuid=ucetstravnika&rok=' . ($rok + 1) . '"> >> </a>' .
            '</div>';
    }


    function getSQLFilter()
    {
        $rok = $_SESSION['rok'];

        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'UcetStravnikaZaObdobi.Obdobi >= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 1, 1, $rok))) .
            ' AND UcetStravnikaZaObdobi.Obdobi <= ' . $params ->addParameter($this ->database ->dateToParam(mktime(0, 0, 0, 12, 31, $rok))) .
            ' AND UcetStravnikaZaObdobi.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC) .
            ' AND UcetStravnikaZaObdobi.OsobniCislo = ' . $params->addParameter($this ->filterForm ->vybranaOsobaOsobniCislo) .
            ' AND UcetStravnikaZaObdobi.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->filterForm ->vybranaOsobaDatumNastupu)),
            $params);
    }
}

