<?php
/**
 * pavel.rudol, 29.4.2016 11:10
 * Seznam zadosti o zastup zamestnancu
 */

namespace RonPortal\Kapacity;


use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


class ZastupyFilterForm extends RonPortalFilterForm
{
    public $jenDuvodZadosti;
    public $duvodZadosti;

    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->nastavObdobi(true, 'ADS', false, false);
        $this->nastavDatumOdDo(true, false);

        $this->nastavAkciGet('zastup');

        $this->nastavRozdeleni(true, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), 0, '', true);
        $this->nastavRozdeleni2(true, RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1), false, '', true);
    }

    /**
     * filtr pro uroven schvaleni
     */
    protected function renderUrovenSchvaleni()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">' . TextManager::_('Zastup_uroven_schvaleni') . '</span>';

        $data = array(
            0 => TextManager::_('Vse'),
            -1 => TextManager::_('Zastup_ke_schvaleni'),
            -2 => TextManager::_('Zastup_bez_schvaleni'),
            1 => TextManager::_('Zastup_schvaleni'),
            2 => TextManager::_('Zastup_zamitnuti'),
            3 => TextManager::_('Zastup_zruseni'),
            100 => TextManager::_('Zastup_k_prirazeni')
        );

        LinksList::renderLinks($data,
            RonPortalSession::getValue('zastupyUrovenSchvaleni'), $this ->f_formaction .  '&urovenschvaleni=', '', false);

        echo '</div>';
    }

    /**
     * @throws Exception
     */
    protected function renderDuvodZadosti()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('jenDuvodZadosti',
            TextManager::_('Zastup_jen_duvod'), $this ->jenDuvodZadosti,
            ['class' => 'clickAutoSubmit', 'id' => 'jen_duvod_zadosti_checkbox']);
        echo '</span>';

        $sql = 'SELECT Kod, Popis FROM DuvodZastupu ORDER BY Popis';

        $combo = new ComboBox('duvodZadosti');
        $combo->loadData($this->database, $sql, $this ->duvodZadosti);
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1'];
        $combo->render();

        echo '</div>';
    }

    protected function loadUrovenSchvaleni()
    {
        if (isset($_GET['urovenschvaleni'])) {
            RonPortalSession::setValue('zastupyUrovenSchvaleni', $_GET['urovenschvaleni']);
        }
    }

    protected function loadDuvodZadosti()
    {
        if (isset($_GET['duvodZadosti'])) {
            $_SESSION['jenDuvodZadosti'] = $this ->jenDuvodZadosti = isset($_GET['jenDuvodZadosti']);
            $_SESSION['duvodZadosti'] = $this ->duvodZadosti = $_GET['duvodZadosti'];
        } else {
            $this ->jenDuvodZadosti = RonPortalSession::getValue('jenDuvodZadosti');
            $this ->duvodZadosti = RonPortalSession::getValue('duvodZadosti');
        }
    }


    function loadData()
    {
        parent::loadData();

        $this->loadUrovenSchvaleni();
        $this->loadDuvodZadosti();
    }

    /**
     * @throws Exception
     */
    public function renderFilters()
    {
        parent::renderFilters();
        $this->renderDuvodZadosti();
        $this->renderUrovenSchvaleni();
    }


}


class ZastupujiciZamestnanecField extends BrowserField
{
    private $zamestnanecRowId;
    private $schvaleniRowId;
    private $typRozdeleniRowId;
    private $rozdeleniRowId;
    private $datumRowId;

    /**
     * ZastupujiciZamestnanecField constructor.
     * @param null|Database $database
     * @param int $zamestnanecRowId
     * @param int $schvaleniRowId
     * @param int $typRozdeleniRowId
     * @param int $rozdeleniRowId
     * @param int $datumRowId
     */
    public function __construct($database, $zamestnanecRowId, $schvaleniRowId, $typRozdeleniRowId, $rozdeleniRowId, $datumRowId)
    {
        parent::__construct($database);

        $this->zamestnanecRowId = $zamestnanecRowId;
        $this->schvaleniRowId = $schvaleniRowId;
        $this->typRozdeleniRowId = $typRozdeleniRowId;
        $this->rozdeleniRowId = $rozdeleniRowId;
        $this->datumRowId = $datumRowId;

        $this->header = TextManager::_('Prijmeni');

        $this->ordering = true;
        $this->sortSQL = 'Osoba.Prijmeni, Osoba.Jmeno';
        $this->sortSQLDesc = 'Osoba.Prijmeni DESC, Osoba.Jmeno DESC';
        $this->code = 'col-prijmeni';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        //pokud je osoba prirazena, zobrazi prijmeni
        if ($rowData[$this->zamestnanecRowId]) {
            $result = $rowData[$this->zamestnanecRowId];
        }
        //pokud neni prirazena osoba a zaznam je schvalen, zobrazi odkaz na prirazeni osoby
        elseif ($rowData[$this->schvaleniRowId] == 1) {
            $prava = Prava::vratPrava('zastupy_prideleni');

            if ($prava->jePravoNaRozdeleni($rowData[$this->typRozdeleniRowId], $rowData[$this->rozdeleniRowId],
                $this ->database ->stringToTimestamp($rowData[$this->datumRowId]), $this ->database ->stringToTimestamp($rowData[$this->datumRowId]))) {
                $result = "<a href='?menuid=zastup&action=priradit&formaction=edit&record=" . $rowData[0] . "'>" . TextManager::_('Zastup_priradit') . "</a>";
            }
        }

        return $result;
    }

}

/**
 * Odkaz na schvaleni - podle prav
 * Class SchvaleniZastupuField
 * @package RonPortal\Kapacity
 */
class SchvaleniZastupuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[15] == 1) { //schvaleno
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textschvaleni';
            $hint = TextManager::_('Zastup_schvaleni');
        } else if ($rowData[15] == 2) { //zamitnuto
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textzamitnuti';
            $hint = TextManager::_('Zastup_zamitnuti');
        }
        else if ($rowData[15] == 3) { //zrusena zadost (smazano planovani kapacit prirazeneho zamestnance)
            $date = $rowData[4];
            $name = $rowData[5];

            $class = 'zastup_textzruseni';
            $hint = TextManager::_('Zastup_zruseni');
        } else {
            return '';
        }

        $result =
            "<SPAN class='$class' title='$hint'>" .
            ($date ? DBUtils::getFormatedValue($this ->database, 'date', $date) : '') .
            ($name ? ' (' . StringUtils::htmlTextConvert($name) . ')' : '') .
            "</SPAN>\n";

        return $result;
    }
}


class SchvaleniZastupuEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;
    var $indexSchvaleno;
    var $indexDatum;
    var $indexUzivatel;

    /** @var ZastupyBrowser */
    var $browser;

    var $existsEdit;


    /**
     * @param Browser $browser
     * @param PravaNaOsoby $prava
     * @param int $indexSchvaleno
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param string $caption
     */
    function __construct($browser, $prava, $indexSchvaleno, $indexDatum, $indexUzivatel, $caption)
    {
        parent::__construct($browser ->database);

        $this ->header =
            "<span id='schvaleniZastupuMenu_Header''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->prava = $prava;
        $this ->indexSchvaleno = $indexSchvaleno;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $schvaleno = isset($this ->indexSchvaleno)
            ? $rowData[$this ->indexSchvaleno] == 1
            : $rowData[$this ->indexDatum];

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) {
                $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
            }
        }

        if ($this ->prava ->jePravoNaRozdeleni($rowData[14], $rowData[6], $this ->database ->stringToTimestamp($rowData[1])) && $rowData[$this->indexSchvaleno] != 3) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}' value='1' />" .
                "<input type='checkbox' data-ronschvalenizastupu_index='{$this ->browser ->jsonRowIndex}' name='Schvaleni_{$rowData[0]}' " .
                "id='Schvaleni_{$rowData[0]}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        }

        return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . TextManager::_($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL' id='schvaleni_ALL' title='' />\n";
        }

        return '';
    }

}


class ZruseniZastupuField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /**
     * @param Database $database
     * @param PravaNaOsoby $prava
     */
    function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->prava = $prava;
        $this->code = 'col-delete';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[12]) {
            // je prirazena osoba, nejde smazat
            return '';
        }

        if ($rowData[15]) {
            // jiz je schvaleno/zamitnuto, nejde smazat
            return '';
        }

        // zkontroluje prava na editaci zastupu
        if ($this->prava->jePravoNaRozdeleni($rowData[14], $rowData[6], $this ->database ->stringToTimestamp($rowData[1]), $this ->database ->stringToTimestamp($rowData[1]))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete"></i>', TextManager::_('Zastup_opravdu_zrusit'),
                '?menuid=zastup&action=zrusitzastup&record=' . $rowData[0]);
        }

        return '';
    }
}

class ZastupyBrowser extends RonPortalBrowser
{
    private $schvalovani;

    private $pravaVlozit;
    private $pravaSchvalit;
    private $pravaPriradit;
    private $pravaZobrazit;
    private $pravaZrusit;

    public $jsonRowIndex;
    public $jsonObject;
    public $hiddenFields;

    /**
     * ZastupyBrowser constructor.
     * @param Database $database
     * @param bool $schvalovani
     * @throws Exception
     */
    public function __construct($database, $schvalovani = false)

    {
        parent::__construct($database, 'zastup');

        $this->title = TextManager::_('Zastup_zadosti');

        //$this->browserUrl = '?menuid=zastup';
        $this->code = 'browserzastupy';
        $this->tableId = 'seznamzastupu';

        $this->pravaVlozit = Prava::vratPrava('zastupy_vlozeni');
        $this->pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $this->pravaPriradit = Prava::vratPrava('zastupy_prideleni');
        $this->pravaZobrazit = Prava::vratPrava('zastupy_prohl');
        $this->pravaZrusit = Prava::vratPrava('zastupy_zruseni');

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'ZadostOZastup.ID, ZadostOZastup.Datum, ZadostOZastup.CasOd, ZadostOZastup.CasDo, ' . // 0 - 3
            'ZadostOZastup.Schvaleni, ZadostOZastup.Schvalil, ' . // 4, 5
            'ZadostOZastup.Rozdeleni, ZadostOZastup.Rozdeleni2, Rozdeleni.Popis, COALESCE(Pozice.Popis, PoziceKategorie.Popis) AS PopisPozice, ' . //6 - 9
            'Osoba.Jmeno, Osoba.Prijmeni, PracovniPomer.ID, ZadostOZastup.Poznamka, ' . // 10 - 13
            'ZadostOZastup.KodTypuRozdeleni, ZadostOZastup.TypSchvaleni, DuvodZastupu.Popis, ZadostOZastup.Specifikace '; //14 - 17

        $this->sqlFromPart =
            'ZadostOZastup LEFT JOIN Osoba ON (ZadostOZastup.RC = Osoba.RC) ' .
            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZadostOZastup.RC AND PracovniPomer.OsobniCislo = ZadostOZastup.OsobniCislo AND PracovniPomer.DatumNastupu = ZadostOZastup.DatumNastupu) ' .
            ' LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni AND Rozdeleni.Kod = ZadostOZastup.Rozdeleni) ' .
            ' LEFT JOIN Rozdeleni AS Pozice ON (Pozice.KodTypuRozdeleni = ZadostOZastup.KodTypuRozdeleni2 AND Pozice.Kod = ZadostOZastup.Rozdeleni2) ' .
            ' LEFT JOIN DuvodZastupu ON (DuvodZastupu.Kod = ZadostOZastup.DuvodZadosti)' .
            ' LEFT JOIN Kategorie AS PoziceKategorie ON (PoziceKategorie.Kod = ZadostOZastup.KodKategorie AND PoziceKategorie.TypKategorie = ZadostOZastup.TypKategorie)';

        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Datum', 1) ->setHeaderText('Zastup_datum')
            ->setCode('col-datum')
            ->setAlign('right') ->setOrdering(1) ->setFormat('date') ->setLink('?menuid=zastup&action=detail');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.CasOd', 2) ->setHeaderText('Zastup_cas_od')
            ->setCode('col-casod')
            ->setAlign('right') ->setOrdering(1) ->setFormat('time');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.CasDo', 3) ->setHeaderText('Zastup_cas_do')
            ->setCode('col-casdo')
            ->setAlign('right') ->setOrdering(1) ->setFormat('time');

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Rozdeleni', 6) ->setHeaderText('Kod')
            ->setCode('col-kodrozdeleni')
            ->setAlign('right') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis', 8) ->setHeaderText('Zastup_rozdeleni')
            ->setCode('col-popisrozdeleni')
            ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PopisPozice', 9) ->setHeaderText('Zastup_pozice')
            ->setCode('col-pozice')
            ->setOrdering(1);

        if ($this->schvalovani) {
            $this->addField(new SchvaleniZastupuEditField($this, $this->pravaSchvalit, 15, 4, 5, TextManager::_('Zastup_schvaleni')))
                ->setCode('col-schvaleniedit')
                ->setAlign('center');
        } else {
            $this->addField(new SchvaleniZastupuField($this ->database))
                ->setCode('col-schvaleni')
                ->setHeaderText('Zastup_schvaleni');
        }

        $this->addField(new ZastupujiciZamestnanecField($this->database, 11, 15, 14, 6, 1));

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno', 10) ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.DuvodZadosti', 16) ->setHeaderText('Zastup_duvod')
            ->setCode('col-duvod')
            ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('ZadostOZastup.Poznamka', 13) ->setHeaderText('Zastup_poznamka')
            ->setCode('col-poznamka')
            ->setOrdering(1);

        $this->addField(new ZruseniZastupuField($database, $this->pravaVlozit));

        $this->defaultSort = 1;
    }

    function setFilterForm()
    {
        $this->filterForm = new ZastupyFilterForm($this->database,'zastup');
    }


    public function renderFilter()
    {
        $this->filterForm->render();

        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava()) {
                echo "<div class='normaltext' id='zastup_links'><a href='?menuid=zastup&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
            }

            if ($this->pravaSchvalit->jsouNejakePrava()) {
                echo "<div id='poklk_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . TextManager::_('Zastup_schvalit_hromadne') . "'" .
                    " onClick='ron.zastup.hromadneSchvaleni.zobrazHromadne()'/></div>";
            }
        }
    }

    function getSQLFilter()
    {
        $result = '';

        // uroven schvaleni (vse, ke schvaleni, ...)
        switch (RonPortalSession::getValue('zastupyUrovenSchvaleni')) {
            case -2 : // bez schvaleni
                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
                break;

            case 1 : // schvalene
                $result .= 'ZadostOZastup.TypSchvaleni = 1 ';
                break;

            case 2 : // zamitnute
                $result .= 'ZadostOZastup.TypSchvaleni = 2 ';
                break;

            case 3 : // zrusene
                $result .= 'ZadostOZastup.TypSchvaleni = 3 ';
                break;

            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
                $result .= 'ZadostOZastup.TypSchvaleni = 0 ';
                break;

            case 100: //zaznamy, kde neni osoba, dale se filtruje v enable line podle prav
                $result .= 'ZadostOZastup.RC IS NULL ';
                break;

            default : // vse
                $result .= ' 1 = 1 ';
        }

        // obdobi nebo datum od-do
        if ($this ->filterForm ->jenObdobi) {
            $result .=
                ($result ? ' AND ' : '') .
                '  ZadostOZastup.Datum >= ' . $this->database->timestampToString($this ->filterForm ->vybraneObdobiOd) .
                ' AND ZadostOZastup.Datum <= ' . $this->database->timestampToString($this ->filterForm ->vybraneObdobiDo);
        } else if ($this ->filterForm ->filtrovatDatum) {
            $result .=
                ($result ? ' AND ' : '') .
                '  ZadostOZastup.Datum >= ' . $this->database->timestampToString($this ->filterForm ->datumOd) .
                ' AND ZadostOZastup.Datum <= ' . $this->database->timestampToString($this ->filterForm ->datumDo);
        }

        if ($this ->filterForm ->kodRozdeleni) {
            $result .=
                ($result ? ' AND ' : '') .
                RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
                    (int)$this->filterForm->f_typrozdeleni, $this ->filterForm ->kodRozdeleni,
                    $this ->filterForm ->vcetnePodrizenychRozdeleni);
        }

        if ($this ->filterForm ->kodRozdeleni2) {
            $result .=
                ($result ? ' AND ' : '') .
                RozdeleniUtils::sqlFilterRozdeleni($this ->database, 'ZadostOZastup',
                    (int)$this->filterForm->f_typrozdeleni2, $this ->filterForm ->kodRozdeleni2,
                    $this ->filterForm ->vcetnePodrizenychRozdeleni2,
                    null,
                    ['KodTypuRozdeleni2', 'Rozdeleni2']);
        }

        if ($this ->filterForm instanceof ZastupyFilterForm  &&  $this ->filterForm ->jenDuvodZadosti) {
            $result .=
                ($result ? ' AND ' : '') .
                " DuvodZadosti = '" . $this->database->addSlashes($this ->filterForm ->duvodZadosti) . "'";
        }

        return $result;
    }

    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function zastupEnableLine($values, $datumod, $datumdo)
    {
        $this ->jsonRowIndex = null;

        if (RonPortalSession::getValue('zastupyUrovenSchvaleni') == -1) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values[15] == 1) {
                // jiz je schvaleno nejvyssi urovni = neceka na nikoho
                return false;
            }

            return $this->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        if (RonPortalSession::getValue('zastupyUrovenSchvaleni') == 100) {
            // filtr K prirazeni - zkontroluje, jesti to ceka na me

            if ($values[15] != 1) {
                return false;
            }

            return $this->pravaPriradit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) {
                return true;
            }

            if ($this->pravaPriradit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) {
                return true;
            }

            //pokud ma uzivatel pravo zobrazit prirazenou osobu, nebo rozdeleni zobrazi se mu zaznam
            if (($this->pravaZobrazit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) ||
                ($this->pravaZobrazit->lzeZobrazitPP($values[12], $datumod, $datumdo))) {
                return true;
            }
        }

        if ($this->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) {
            return true;
        }

        //pravo zrusit musi mit zaroven na rozdeleni, kam je podana zadost, i na prirazeneho zamestnance
        if (($this->pravaZrusit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo)) &&
            ($this->pravaZrusit->lzeZobrazitPP($values[12], $datumod, $datumdo))) {
            return true;
        }

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $datumod = $this ->database ->stringToTimestamp($values[1]);
        $datumdo = $datumod;

        if (!$this ->zastupEnableLine($values, $datumod, $datumdo)) {
            return false;
        }

        if ($this ->schvalovani) {

            $this ->jsonRowIndex = count($this ->jsonObject['items']);

            $this ->hiddenFields .= "<input type='hidden' name='idZastup_{$this ->jsonRowIndex}' value='{$values[0]}'/>";

            $this ->jsonObject['items'][] = [
                'id' => $values[0],
                'pravo' => $this ->pravaSchvalit->jePravoNaRozdeleni($values[14], $values[6], $datumod, $datumdo) ? 1 : 0,
                'schvaleno' => ($values[15] == 1) ? 1 : 0,
            ];

        } else {
            $this ->jsonRowIndex = null;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values[15] == 1) {
            $className .= ' zastup_schvalen';
        }
        else if ($values[15] == 2) {
            $className .= ' zastup_zamitnut';
        }

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="zastup_row_' . $values[0] . '"';

        return $result;
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[17]) {
            // zobrazi specifikaci
            echo
            "<tr class='$className'>";
            echo    '<td></td><td colspan="' . ($this->columnCount - 1) . '"><div class="browserpoznamka">' . $values[17] . '</div></td>';
            echo '</tr>';
        }
    }

    function renderContent()
    {
        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        $this ->hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniZastupu" action="?menuid=zastup">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';
        }

        echo
            '<ul class="popupmenu"  id="schvaleniZastupuMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . TextManager::_('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . TextManager::_('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . TextManager::_('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->hiddenFields) {
            echo $this ->hiddenFields;
        }

        echo '<input type="hidden" name="pocet" value="' . count($this ->jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . TextManager::_('Provest') . "' id='provestHromadneSchvaleniZastupu' />" .
                '<input type="button" class="button" value="' . TextManager::_('Cancel') . '" id="stornoHromadneSchvaleniZastupu" />' .
                '</div>'.
                "</form>\n";

            echo
                '<script type="text/javascript">' .
                'ron.zastup.hromadneSchvaleni.init(' . json_encode($this ->jsonObject) . ');' .
                '</script>';
        } else {
            if ($this->pravaVlozit->jsouNejakePrava()) echo "<div class='normaltext'><a href='?menuid=zastup&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
        }
    }
}
