<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\SQLUtils;


class Log
{

    static function getLogRecordTimeStampAddValue()
    {
        /*
           // puvodni
           global $logrecordtimestampaddvalue;
           $logrecordtimestampaddvalue = (int)$logrecordtimestampaddvalue + 1;
           return $logrecordtimestampaddvalue;
        */
        $timestampAddValue = isset($_SESSION['g_timestampaddvalue'])
            ? $_SESSION['g_timestampaddvalue'] + 1
            : rand(1, 100);

        $_SESSION['g_timestampaddvalue'] = $timestampAddValue;

        return $timestampAddValue;
    }


    /**
     * @param Database $database
     * @param string $description
     * @param string $category
     * @param string $comment
     * @param string $userName
     * @throws \Exception
     */
    static function addRecordToLog($database, $description, $category, $comment = '', $userName = null)
    {
        if (!isset($userName)) {
            $userName = User::isLoggedAs()
                ? substr(User::getRealLoginName(), 0, 23) . '/z'
                : substr(User::getLoginName(), 0, 25);
        } else {
            $userName = substr($userName, 0, 25);
        }

        $category = $category
            ? substr($category, 0, 25)
            : null;

        $description = substr($description, 0, 120);

        $comment = $comment ?: null;

        $netUser = '';
        if (isset($_SERVER['REMOTE_USER'])) $netUser = substr(strrchr($_SERVER['REMOTE_USER'], '\\'), 1);
        $netUser = $netUser
            ? substr($netUser, 0, 25)
            : null;

        $computerName = isset($_SERVER['REMOTE_HOST'])
            ? substr($_SERVER['REMOTE_HOST'], 0, 25)
            : null;

        global $appVersion;

        $params = $database ->createParameters();
        $sql =
            'INSERT INTO SYS_LOG (' .
            'applicationID, GTimeStamp, GTimeStampAddValue,' .
            ' UserName, GComputerName, GNetUser,' .
            ' GCategory, GDecription, GComment, ApplicationVersion) ' .
            ' VALUES ( ' .
            "'INTRAWEB', " . $params ->addParameter($database ->timestampToParam(time())) . ', ' . $params ->addParameter(self::getLogRecordTimeStampAddValue()) . ', ' .
            $params ->addParameter($userName) . ', ' . $params ->addParameter($computerName) . ', ' . $params ->addParameter($netUser) . ', ' .
            $params ->addParameter($category) . ', ' . $params ->addParameter($description) . ', ' . $params ->addParameter($comment) . ', ' .
            $params ->addParameter(substr($appVersion, 0, 25)) .
            ')';

        try {
            $database->sql($sql, $params);
        } catch (\Exception $e) {
            // pripadnou chybu ignoruje a zaloguje
            error_log("RONPortal: Chyba pri zapisu do LOGu: " . $e ->getMessage() . "\n" . substr($sql, 0, 1024));
        }
    }


    /**
     * Zaloguje zapis/editaci($eventtype=UPDATE/INSERT/SELECT) do tabulky ($tablename)
     * @param Database $database
     * @param string $tableName
     * @param string $eventType
     * @param string|object $srcDescription     zdroj udalosti - muze to byt primo text nebo object a pak se pouzije nazev jeho tridy
     * @param string $comment
     */
    static function addRecordToDBLog($database, $tableName, $eventType, $srcDescription = '', $comment ='')
    {
        $username = User::isLoggedAs()
            ? substr(User::getRealLoginName(), 0, 23) . '/z'
            : substr(User::getLoginName(), 0, 25);

        $tableName = substr($tableName, 0, 25);

        $netUser = '';
        if (isset($_SERVER['REMOTE_USER'])) $netUser = substr(strrchr($_SERVER['REMOTE_USER'], '\\'), 1);
        $netUser = $netUser
            ? substr($netUser, 0, 25)
            : null;

        $computerName = isset($_SERVER['REMOTE_HOST'])
            ? substr($_SERVER['REMOTE_HOST'], 0, 25)
            : null;

        if (is_object($srcDescription)) {
            $srcDescription = substr(strrchr(get_class($srcDescription), "\\"), 1);
        }

        $srcDescription = $srcDescription
            ? substr($srcDescription, 0, 120)
            : null;

        $comment = $comment ?: null;

        $params = $database ->createParameters();
        $sql =
            'INSERT INTO SYS_DB_LOG ' .
            ' ( applicationID, GTimeStamp, GTimeStampAddValue,' .
            '  UserName, GComputerName, GNetUser,' .
            '  GTableName, GEventType, GSrcDescription, GComment )' .
            ' VALUES (' .
            " 'INTRAWEB', " . $params ->addParameter($database ->timestampToParam(time())) . ', ' . $params ->addParameter(self::getLogRecordTimeStampAddValue()) . ', ' .
            $params ->addParameter($username) . ', ' . $params ->addParameter($computerName) . ', ' . $params ->addParameter($netUser) . ', ' .
            $params ->addParameter($tableName) . ', ' . $params ->addParameter($eventType) . ', ' .
            $params ->addParameter($srcDescription) . ', ' . $params ->addParameter($comment) .
            ')';
        try {
            $database ->sql($sql, $params);
        } catch (\Exception $e) {
            // pripadnou chybu ignoruje a zaloguje
            error_log("RONPortal: Chyba pri zapisu do LOGu: " . $e ->getMessage() . "\n" . substr($sql, 0, 1024));
        }
    }


    static function logSQL($database, $sql, $srcDescription = '')
    {
        self::addRecordToDBLog($database, SQLUtils::getTableNameFromSQL($sql), SQLUtils::getEventTypeFromSQL($sql), $srcDescription, $sql);
    }

}
