<?php
/**
 * stanislav.elbl, 25.02.2021
 */

namespace RonPortal\Mzdy;

use Ron\Controls\Browser;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\RonPortalSession;

class SrazkyBrowser extends Browser
{
    function __construct($database)
    {
        parent::__construct($database);

        $this -> title = TextManager::_('Srazky_ze_mzdy');

        // definice dotazu a hlavicky
        $this -> sqlFromPart =
            "(NastaveniSrazky LEFT OUTER JOIN ProvedenaSrazka ".
            " ON ProvedenaSrazka.KodSrazky = NastaveniSrazky.KodSrazky ".
            " AND ProvedenaSrazka.SerioveCislo = NastaveniSrazky.SerioveCislo )".
            " INNER JOIN TypSrazky ON NastaveniSrazky.KodSrazky = TypSrazky.Kod ";


        $params = $this ->database ->createParameters();
        $this ->sqlWherePart = new SqlStatement(
            ' NastaveniSrazky.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
            ' AND ( NastaveniSrazky.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR NastaveniSrazky.PlatnostDo IS NULL ) ' .
            ' AND NastaveniSrazky.RC = ' . $params ->addParameter(RonPortalSession::osobaRC()) .
            ' AND NastaveniSrazky.OsobniCislo = ' . $params ->addParameter(RonPortalSession::osobaOsobniCislo()) .
            ' AND NastaveniSrazky.DatumNastupu = ' . $params ->addParameter($database ->dateToParam(RonPortalSession::osobaNastup())),
            $params);

        $this -> sqlGroupByPart =
            "NastaveniSrazky.SerioveCislo, NastaveniSrazky.KodSrazky, TypSrazky.Popis, NastaveniSrazky.PlatnostOd, NastaveniSrazky.PlatnostDo, ".
            "NastaveniSrazky.CastkaCelkova, NastaveniSrazky.CastkaMesicni ";

        $this ->sqlSelectPart =
            'NastaveniSrazky.SerioveCislo, NastaveniSrazky.KodSrazky, TypSrazky.Popis, ' .
            'NastaveniSrazky.PlatnostOd, NastaveniSrazky.PlatnostDo, ' .
            'NastaveniSrazky.CastkaCelkova, NastaveniSrazky.CastkaMesicni, SUM(ProvedenaSrazka.SrazenaCastka)';

        $this -> defaultSort = 0;
        $this -> primaryField = 0;

        $this ->addField()
            ->setSqlFieldName('TypSrazky.Popis', 2) ->setHeaderText('Typ_srazky');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.PlatnostOd', 3) ->setHeaderText('Platnost_od') ->setFormat('date');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.PlatnostDo', 4) ->setHeaderText('Platnost_do') ->setFormat('date');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.CastkaCelkova', 5) ->setHeaderText('Srazka_celkem') ->setFormat('curr') ->setAlign('right');

        $this ->addField()
            ->setSqlFieldName('NastaveniSrazky.CastkaMesicni', 6) ->setHeaderText('Srazka_mesicni') ->setFormat('curr') ->setAlign('right');

        $this ->addField()
            ->setSqlFieldName('SUM(ProvedenaSrazka.SrazenaCastka)', 7) ->setHeaderText('Jiz_srazeno') ->setFormat('curr') ->setAlign('right');
    }
}
