<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;

class Obdobi
{
    /** @var integer */
    var $zacatek;
    /** @var integer */
    var $konec;
    /** @var string */
    var $popis;

    var $uzavreno;


    function __construct()
    {
    }


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return bool
     * @throws \Exception
     */
    private function loadSQL($database, $sql, $params)
    {
        $cursor = $database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $this->zacatek = $database ->stringToTimestamp($row[1]);
            $this->konec = $database ->stringToTimestamp($row[2]);
            $this->popis = $row[0];
            $this->uzavreno = $row[3];

            // docasne nechavam deprecated pole
            $this->f_Zacatek = $database ->stringToTimestamp($row[1]);
            $this->f_Konec = $database ->stringToTimestamp($row[2]);
            $this->f_Popis = $row[0];
            $this->f_Uzavreno = $row[3];

            return true;
        }

        return false;
    }


    /**
     * @param Database $database
     * @param string $aplikace
     * @param int    $zacatek
     * @return bool
     */
    function nactiObdobi($database, $aplikace, $zacatek)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT Popis, PocatekObdobi, KonecObdobi, Uzavreno ' .
            ' FROM Obdobi WHERE Aplikace = ' . $params ->addParameter($aplikace) .
            ' AND PocatekObdobi = ' . $params ->addParameter($database ->dateToParam($zacatek));

        return $this->loadSQL($database, $sql, $params);
    }


    /**
     * @param Database $database
     * @param string $aplikace
     * @param int    $den
     * @return bool
     */
    function nactiObdobiProDen($database, $aplikace, $den)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT Popis, PocatekObdobi, KonecObdobi, Uzavreno ' .
            ' FROM Obdobi WHERE Aplikace = ' . $params ->addParameter($aplikace) .
            ' AND PocatekObdobi <= ' . $params->addParameter($database->dateToParam($den)) .
            ' AND KonecObdobi >= ' . $params ->addParameter($database ->dateToParam($den));

        return $this->loadSQL($database, $sql, $params);
    }


//    function NastavAktualniMesic()
//    {
//        $this->f_Zacatek = firstdayofmonth(time());
//        $this->f_Konec = lastdayofmonth($this->f_Zacatek);
//        $this->f_Popis = date('n.Y', $this->f_Zacatek);
//        $this->f_Uzavreno = 0;
//    }


    function nastavSession()
    {
        RonPortalSession::setObdobi($this ->zacatek, $this ->konec);
    }


    /**
     * Overi, jestli v zadanem rozsahu datumu nejake obdobi uzavreno
     * @param Database $database
     * @param string $aplikace
     * @param int $datumOd
     * @param int [$datumDo]    Pokud neni nastaveno, pouzije se stejne datum jako $datumOd
     * @return bool
     * @throws \Exception
     */
    public static function jeObdobiUzavreno($database, $aplikace, $datumOd, $datumDo = null)
    {
        if (!isset($datumDo)) $datumDo = $datumOd;

        $params = $database ->createParameters();
        $sql =
            'SELECT PocatekObdobi FROM Obdobi ' .
            ' WHERE Aplikace=' . $params ->addParameter($aplikace) .
            ' AND PocatekObdobi <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (KonecObdobi >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR KonecObdobi IS NULL)' .
            ' AND Uzavreno = 1';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri overovani uzavreneho obdobi!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) return true; // v overovanem rozsahu datumu existuje uzavrene obdobi

        return false;
    }


    /**
     * @param Database $database
     * @param string $aplikace
     * @param int    $datum
     * @return Obdobi
     * @throws \Exception
     */
    public static function vratObdobi($database, $aplikace, $datum)
    {
        $result = new Obdobi();
        if (!$result ->nactiObdobiProDen($database, $aplikace, $datum)) {
            throw new \Exception('Obdobi pro zadany den nebylo nalezeno!');
        }

        return $result;
    }
}
