<?php
/**
 * stanislav.elbl, 12.03.2021
 */

namespace RonPortal\Personalistika;

use Ron\Controls\FormField;
use Ron\DB\SqlStatement;
use RonPortal\Controls\RonPortalForm;
use RonPortal\PracovniPomer;

class NahlasenaZmenaForm extends RonPortalForm
{
    var $typZmeny = null;

    /** @var PracovniPomer */
    var $pp;

    function __construct($database)
    {
        parent::__construct($database);

        $this ->typZmeny = $_GET['typzmeny'];

        if ($this ->formAction == 'new') {
            $this ->formAction = 'edit';  // editace existujich hodnot z databaze
        }

        $this ->addFormFields();
    }


    function addFormFields()
    {
        switch ($this ->typZmeny) {
            case 'trvale' :
                $this ->addFieldsAdresa(true);
                break;

            case 'prechodne' :
                $this ->addFieldsAdresa(false);
                break;
        }
    }

    /**
     * Prida pole pro zmenu adresy osoby (trvale nebo prechodne)
     * @param $trvale
     */
    function addFieldsAdresa($trvale) {
        $code = $trvale ? 'trvale' : 'prechodne';

        $this ->sqlTableName = 'AdresaOsoby';

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostOd', 'AdresaOsoby') ->setCaption('Platnost_od')
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod");

        $this ->addField()
            ->setFieldName('Ulice', 'AdresaOsoby') ->setCaption('Ulice')
            ->setMaxLength(60)
            ->setCode("field-{$code}-ulice")
            ->setFieldId("zmena_{$code}_ulice");

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stat', 'AdresaOsoby')->setCaption('Stat')
            ->setCode("field-{$code}-stat")
            ->setFieldId("zmena_{$code}_stat")
            ->setSelectSql('SELECT Kod, Popis FROM Stat ORDER BY Popis')
            ->setFieldCssClass('form-control ronselect');
    }


    function getSqlFromPart()
    {
        switch (strtolower($this ->typZmeny)) {
            case 'trvale' :
            case 'prechodne' :
                return 'AdresaOsoby';
        }

        return parent::getSqlFromPart();
    }


    function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $trvale = 0;
        switch (strtolower($this ->typZmeny)) {
            case 'trvale' :
                $trvale = 1;
            case 'prechodne' :
                return
                    new SqlStatement(
                        'RC = ' . $params ->addParameter($this ->pp ->rc) .
                        ' AND TrvaleBydliste = ' . $params ->addParameter($trvale) .
                        ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')',
                        $params);
        }

        return parent::getWhereCondition();
    }


    function loadRecord()
    {
        $this ->pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['osoba']);

        return parent::loadRecord();
    }


    function loadOriginalValues()
    {
        //$this ->loadRecord();
        $this ->data = $this ->loadRecord();
    }


    function handleAction()
    {
        if ($this ->formAction == 'new') {
            $this ->loadOriginalValues();
        }

        $result = parent::handleAction();


    }
}
