<?php
/**
 * Roman.Szablatura, 18.2.2016
 */

namespace RonPortal\Pomucky;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\TextManager;
use RonPortal\Controls\FormReportsListView;
use RonPortal\Controls\OsobniCisloBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use Ron\StringUtils;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\System\Report;
use RonPortal\User;


/**
 * Class PracovniPomuckyFilterForm
 * @package Ron\Personalistika
 */
class PracovniPomuckyFilterForm extends RonPortalFilterForm
{
    /** @var PravaNaOsoby */
    protected $pravaVydatVratit;

    public $timestamp;

    /**
     * @param Database $database
     * @param PravaNaOsoby $pravaProhlizet
     * @param PravaNaOsoby $pravaPotvrdit
     * @param PravaNaOsoby $pravaVydatVratit
     */
    public function __construct($database, $pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit)
    {
        parent::__construct($database, 'pracovnipomucky');

        $this ->setUseGet(true, 'pracovnipomucky');

        $this->f_formaction = '?menuid=pracovnipomucky';
        $this->f_classname = 'filtercontainer leftcaptions';
        $this ->formAttributes['class'] = ' submitWorkingInfo';

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
        $this->nastavOsoby(true,[$pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit]);

        $this->povolitVsechyOsoby = 1;
        $this->pravaVydatVratit = $pravaVydatVratit;

        //$this->nastavDatumOdDo(true, true, TextManager::_('PracovniPomucky_obdobi') . ' ' . TextManager::_('Od'));
        $this->nastavDatum(true);
    }


    /**
     * @throws Exception
     */
    public function renderPomucka()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenPomucka', TextManager::_('pomucka'), RonPortalSession::getValue('jenPomucka'),
            ['id' => 'jen_pomucka_checkbox'], 'clickAutoSubmit', 1);
        echo '</SPAN>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis, TypPomucky FROM PracovniPomucka' .
            (RonPortalSession::getValue('jenTypPomucky')
                ? ' WHERE PracovniPomucka.TypPomucky = ' . $params ->addParameter(RonPortalSession::getValue('typPomucky'))
                : '') .
            ' ORDER BY Popis';

        $combo = new ComboBox('pomucka');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_pomucka_checkbox'];
        $combo->showCode = 1;
        $combo->loadData($this->database, $sql, RonPortalSession::getValue('pomucka'), $params);
        $combo->render();

        echo "</DIV>";
    }

    /**
     * @throws Exception
     */
    public function renderTypPomucky()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypPomucky', TextManager::_('typ_pomucky'), RonPortalSession::getValue('jenTypPomucky'),
            ['id' => 'jen_typ_pomucky_checkbox'], 'clickAutoSubmit', 1);
        echo '</SPAN>';

        $combo = new ComboBox('typPomucky');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_typ_pomucky_checkbox'];
        $combo->loadData($this->database,
            'SELECT Kod,TypPomucky.Popis FROM TypPomucky ORDER BY Popis',
            RonPortalSession::getValue('typPomucky'));
        $combo->render();
        echo "</DIV>";
    }

    public function renderVydatPracPomucku()
    {
        echo
            '<div class="filterrow">' .
            // vydej jedne pomucky
            '<span class="margin-right-2 padding-right-2">' .
            '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=new&vydanipomucky=2&individualni=1">' .
            TextManager::_('vydej_pomucky_zamestnanci') .
            '</a>' .
            '</span>' .
            // hromadny vydej naroku
            '<span class="margin-right-2 padding-right-2">' .
            '<a href="?menuid=pracovnipomucky&action=hromadnyvydej">' .
            TextManager::_('Hromadny_vydej_naroku') .
            '</a>' .
            '</span>' .

            "</div>\n";
    }


    public function renderLinkVydani($name, $caption, $value = null, $selected = null)
    {
        if (!isset($value)) {
            $value = (is_array(RonPortalSession::getValue('filtrVydanePomucky')) && isset($_SESSION['filtrVydanePomucky'][$name]))
                ? $_SESSION['filtrVydanePomucky'][$name]
                : 0;
        }

        $newValue = $value ? 0 : 1;

        $className =
            'setValue' .
            ( ($selected ||$value) ? ' font-weight-bold' : '');

        echo
            '<span class="margin-right-2 padding-right-2">' .
            "<input type='hidden' id='stav_{$name}' name='$name' value='$value'/>" .
            "<a href='#' id='filtrpomucky_{$name}' class='$className' data-submit='1' data-selector='#stav_{$name}' data-value='$newValue'>" .
            $caption .
            '</a>' .
            '</span>';
    }

    public function renderFiltrNevydanePracovniPomucky()
    {
        echo '<div>';
        echo '<span class="filtercaption padding-top-2">';

        $jeNejakyFiltr = !!RonPortalSession::getValue('filtrVydanePomucky');
        // aktualni stav - vzdy nastavi hodnotu "0" a vybrany je, kdyz neni vybrany zadny jiny
        // (zakliknuti tohoto filtru vypina vsechny ostatni a zakliknuti kterehokoliv jineho filtru vypina tento)
        $this ->renderLinkVydani('aktualniStav', TextManager::_('pomucky_stav_k_datu'), 0, !$jeNejakyFiltr);

        $this ->renderLinkVydani('jenNevydane', TextManager::_('nevydane_pomucky'));
        $this ->renderLinkVydani('jenPotvrzene', TextManager::_('vydane_potvrzene_pomucky'));
        $this ->renderLinkVydani('jenNepotvrzene', TextManager::_('vydane_nepotvrzene_pomucky'));
        $this ->renderLinkVydani('jenExpirovane', TextManager::_('vydane_expirovane_pomucky'));
        $this ->renderLinkVydani('jenVracene', TextManager::_('vracene_pomucky'));

        echo "</span>";
        echo '</div>';
    }

    private function renderReportList()
    {
        $filtrStav = is_array(RonPortalSession::getValue('filtrVydanePomucky'))
            ? RonPortalSession::getValue('filtrVydanePomucky')
            : [];

        $reportList = Report::loadFormReports($this->database, 'PracovniPomucky');
        FormReportsListView::renderReportList($reportList,
            '?menuid=pracovnipomucky&action=report&osoba=' . $this->vybranaOsoba .
            '&datumod=' . StringUtils::format('date', $this->datum) .
            '&datumdo=' . StringUtils::format('date', $this->datum) .
            '&rozdeleni=' . $this->kodRozdeleni .
            '&rozdelenipodrizene=' . $this->vcetnePodrizenychRozdeleni .
            '&rozdeleni2=' . $this->kodRozdeleni2 .
            '&rozdelenipodrizene2=' . $this->vcetnePodrizenychRozdeleni2 .
            '&vsechnyosoby=' . $this->vsechnyOsoby .
            '&jentyp=' . RonPortalSession::getValue('jenTypPomucky') .
            '&typpomucky=' . RonPortalSession::getValue('typPomucky') .
            '&jenpomucka=' . RonPortalSession::getValue('jenPomucka') .
            '&pomucka=' . RonPortalSession::getValue('pomucka') .
            '&nevydane=' . ((isset($filtrStav['jenNevydane']) && $filtrStav['jenNevydane']) ? 1 : 0) .
            //'&nevracene=' . $_SESSION['jenNevracene'] .
            '&vracene=' . ((isset($filtrStav['jenVracene']) && $filtrStav['jenVracene']) ? 1 : 0) .
            '&potvrzene=' . ((isset($filtrStav['jenPotvrzene']) && $filtrStav['jenPotvrzene']) ? 1 : 0) .
            '&expirovane=' . ((isset($filtrStav['jenExpirovane']) && $filtrStav['jenExpirovane']) ? 1 : 0) .
            '&nepotvrzene=' . ((isset($filtrStav['jenNepotvrzene']) && $filtrStav['jenNepotvrzene']) ? 1 : 0)
        );
    }

    private function loadVydaniCheckbox($name)
    {
        if (isset($_GET[$name])) {
            //$value = (is_array($_SESSION['filtrVydanePomucky']) && isset($_SESSION['filtrVydanePomucky'][$name])) ? 1 : 0;
            if (!is_array(RonPortalSession::getValue('filtrVydanePomucky'))) {
                RonPortalSession::setValue('filtrVydanePomucky', []);
            }

            if ((int)$_GET[$name]) {
                $_SESSION['filtrVydanePomucky'][$name] = 1;
            } else {
                unset($_SESSION['filtrVydanePomucky'][$name]);
            }
        }
    }

//    private function loadJenNevracene()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenNevracene'] = isset($_POST['jenNevracene']);
//        }
//    }

    private function loadTypPomucky()
    {
        if (isset($_GET['typPomucky'])) {
            $_SESSION['jenTypPomucky'] = isset($_GET['jenTypPomucky'])  &&  (int)$_GET['jenTypPomucky'];
            $_SESSION['typPomucky'] = $_GET['typPomucky'];
        }
    }


    /**
     * @throws Exception
     */
    private function loadPomucka()
    {
        if (isset($_GET['pomucka'])) {
            $_SESSION['jenPomucka'] = isset($_GET['jenPomucka']) && (int)$_GET['jenPomucka'];
            $_SESSION['pomucka'] = $_GET['pomucka'];
        }

        if (RonPortalSession::getValue('pomucka') != '' && RonPortalSession::getValue('jenPomucka')) {
            $params = $this->database->createParameters();
            $_SESSION['typPomucky'] = DBUtils::getSqlValue($this->database,
                'SELECT TypPomucky FROM PracovniPomucka' .
                ' WHERE Kod = ' . $params->addParameter(RonPortalSession::getValue('pomucka')), $params);
        }
    }

//    private function setVsechnyOsoby()
//    {
//        RonPortalSession::setVsechnyOsoby(true);
//    }

    function loadData()
    {
        parent::loadData();

//        //finta, po prihlaseni se nastavuje na aktu.den
//        $datumOd = RonPortalSession::getDatumOd();
//        if (!isset($datumOd)){
//            $this ->datumOd = DateUtils::today();
//            $this ->datumDo = DateUtils::today();
//
//            RonPortalSession::setFiltrovatDatum($this ->filtrovatDatum,
//                $this ->datumOd,
//                $this ->datumDo );
//        }

        //$this->loadDatumOdDo();
        if (isset($_GET['aktualniStav'])  &&  $_GET['aktualniStav']) {
            // uzivatel vybral filtr "aktualni stav" => vypne vsechny ostatni filtry
            $_SESSION['filtrVydanePomucky'] = [];
        } else {
            $this->loadVydaniCheckbox('jenNevydane');
            $this->loadVydaniCheckbox('jenNepotvrzene');
            $this->loadVydaniCheckbox('jenPotvrzene');
            $this->loadVydaniCheckbox('jenExpirovane');
            $this->loadVydaniCheckbox('jenVracene');
        }

//        $this->loadJenNevracene();
        //$this->loadJenVracene();
        //$this->loadJenNevydane();
        //$this->loadJenNepotvrzene();
        //$this->loadJenPotvrzene();
        //$this->loadJenExpirovane();

        $this->loadTypPomucky();
        $this->loadPomucka();
        //parent::loadData();
    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        parent::renderFilters();

//        $this->renderDatumOdDo();
//        if ($this->f_showrozdeleni) $this->renderRozdeleni();
//        if ($this->f_showrozdeleni2) $this->renderRozdeleni2();
//        if ($this->f_showosoba) $this->renderOsoby();
        $this->renderTypPomucky();
        $this->renderPomucka();
        if ($this->f_link) {
            $this->renderLink();
        }

        $this->renderFiltrNevydanePracovniPomucky();

        echo "<br />";
        if($this->pravaVydatVratit->jsouNejakePrava()) {
            $this->renderVydatPracPomucku();
        }
        $this->renderReportList();
    }
}

/**
 * Class PotvrzeniPrijetiField
 * @package Ron\Personalistika
 */
class PotvrzeniPrevzetiField extends BrowserField
{
    /**
     * PotvrzeniPrijetiField constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->header = TextManager::_('potvrzeni_prevzeti');
        $this->align = 'center';

        $this ->format = 'datetime';

        $this ->sqlFieldIndex = 11;
        $this ->titleFieldIndex = 12;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idosoby = $rowData[16];
//        $params = $this->database->createParameters();
//        $prihlasenaOsoba = DBUtils::getSqlValue($this->database,
//            "SELECT MAX(ID) FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);

        if ($rowData[11]) {
            // jiz potvrzeno - zobrazi cas potvrzeni
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        } elseif ($rowData[1] && !$rowData[2] && $rowData[17] && Prava::vratPrava('pomucky_potvrzeni')->lzeZobrazitRC($rowData[6])) {
            // je vydana, neni vracena a je vyzadovano potvrzeni
            return
                '<a class="confirmQuestion"' .
                ' href="?menuid=pracovnipomucky&action=potvrzeniprevzeti&osoba=' . $idosoby .
                '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) .
                '&poradi=' . $rowData[3] . '"' .
                (User::hasRC($rowData[6])
                    ? ''    // potvrzeni uzivatele - neni potreba dotaz
                    : ' data-ron-confirm-text="' . TextManager::_('dotaz_potvrdit_prevzeti_za_jinou_osobu') . '"' // potvrzeni za jineho zamestnance
                ) .
                '>' .
                TextManager::_('potvrdit') .
                '</a>';
        }

        return '';
    }
}


/**
 * Class DatumVraceniField
 * @package Ron\Personalistika
 */
class DatumVraceniField extends BrowserField
{
    var $pravaZapisPrevzeti;
    var $pomuckyPotvrzeni;
    /**
     * DatumVraceniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header= TextManager::_('datum_vraceni');

        $this->align = 'center';
        $this->ordering = 1;
        $this->sqlFieldName = 'PrirazenaPracPomucka.DatumVraceni';

        $this->pomuckyPotvrzeni = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[16];
        if ($rowData[2] != '') {
            $result = '<DIV title="'. $this->zjistiPocet(1,2,1,$rowData) .'">';
            $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[2]));
            $result .= '</DIV>';
        } else {
            if ($this->pomuckyPotvrzeni->lzeZobrazitRC($rowData[6])) {
                if ($rowData[1] != '') {
                    $result = '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                        '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&vydanipomucky=0">';
                    $result .= TextManager::_('vratit');
                    $result .= '</a>';
                } else {
                    $result = "";
                }
            }
        }
        return $result;
    }

    public function zjistiPocet ($jedna, $dva, $priznak, $rowData)
    {
        if ($priznak == 1) {
            $pocet = DateUtils::daysBetween($this->database->stringToTimestamp($rowData[$jedna]),$this->database->stringToTimestamp($rowData[$dva]));
            if ($pocet === 1) {
                return TextManager::_('zapujceno') . StringUtils::format('number0',$pocet) . TextManager::_('den');
            } else if ($pocet > 1 && $pocet < 5) {
                return TextManager::_('zapujceno') . StringUtils::format('number0',$pocet) . TextManager::_('dny');
            } else {
                return TextManager::_('zapujceno') . StringUtils::format('number0',$pocet) . TextManager::_('dni');
            }
        }
        return null;
    }
}

/**
 * Class DatumVydaniField
 * @package Ron\Personalistika
 */
class DatumVydaniField extends BrowserField
{
    var $pravaZapisPrevzeti;
    /**
     * DatumVydaniField constructor.
     * @param null|Database $database
     * @param array|null $options
     */
    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = TextManager::_('datum_vydani');

        $this->align = 'center';
        $this->ordering = 1;
        $this->sqlFieldName = 'PrirazenaPracPomucka.DatumVydani';

        $this->pravaZapisPrevzeti = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $idosoby = $rowData[16];

        if ($rowData[1]) {
            $result .= StringUtils::format('date', $this->database->stringToTimestamp($rowData[1]));
        } else {
            if ($this->pravaZapisPrevzeti->lzeZobrazitRC($rowData[6])) {
                $result = '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&vydanipomucky=1">';
                $result .= TextManager::_('vydat');
                $result.='</a>';
            }
        }
        return $result;
    }
}

/**
 * Odkaz na editaci data expirace
 * Class EditaceExpiraceField
 * @package Ron\Personalistika
 */
class EditaceExpiraceField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $pravaZapisPrevzeti;

    function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = TextManager::_('datum_expirace');
        $this->pravaZapisPrevzeti = Prava::vratPrava('pomucky_vydej');
        $this->sqlFieldName = 'PrirazenaPracPomucka.DatumExpirace';
        $this->align = 'center';
        $this->ordering = 1;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[1]  &&  !$rowData[2]  &&  $this->pravaZapisPrevzeti->lzeZobrazitRC($rowData[6])) { // je vydana, jeste neni vracena a jsou prava na vydej/vraceni
            $idosoby = $rowData[16];

            $result = '<a href="?menuid=pracovnipomucky&action=editaceexpirace&formaction=edit&osoba=' . $idosoby . '&pomucka=' . HttpUtils::urlEncode($rowData[0]) .
                '&datumnaroku=' . $this->database->stringToTimestamp($rowData[10]) . '&poradi=' . $rowData[3] . '&editexpirace=1">';
            if ($rowData[4] == '') {
                $result .= "<DIV title='" . TextManager::_('uprava_expirace_pomucky') . "'>" . TextManager::_('expirace_neuvedena');
                $result .= '</a>';
            } else {
                $result .= "<DIV title='" . TextManager::_('info_editace_expirace_pomucky') . "'>" . StringUtils::format('date', $this->database->stringToTimestamp($rowData[4]));
                $result .= '</a>';
            }

            return $result;
        }

        return StringUtils::format('date', $this->database->stringToTimestamp($rowData[4]));
    }
}

class ExpiraceVypoctenaField extends BrowserField
{

    function __construct($database, $options)
    {
        parent::__construct($database, $options);

        $this->header = TextManager::_('datum_expirace_vypoctene');
        $this->align = 'center';
        $this->ordering = 1;
        $this->format = 'date';
        $this->sqlFieldName = 'PrirazenaPracPomucka.DatumExpiraceVypoctene';
    }


    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if($rowData[1]  &&  !$rowData[$this ->sqlFieldIndex]) { // pomucka byla vydana a nema expiraci
            return TextManager::_('uprava_expirace_pomucky');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData[1]) {
            // pomucka nebyla vydana
            return '';
        }

        if (!$rowData[$this ->sqlFieldIndex]) {
            return TextManager::_('expirace_neuvedena');
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class PracovniPomuckyBrowser
 * @package Ron\Personalistika
 */
class PracovniPomuckyBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby  */
    var $pravaProhlizet;
    /** @var PravaNaOsoby  */
    var $pravaZapisPrevzetiVydej;
    /** @var PravaNaOsoby  */
    var $pomuckyPotvrzeni;

    var $typRozdeleni;
    var $rozdeleni;

    /**
     * PracovniPomuckyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'pracovnipomucky');

        $this->pravaProhlizet = Prava::vratPrava('pomucky_prohl');
        $this->pravaZapisPrevzetiVydej = Prava::vratPrava('pomucky_vydej');
        $this->pomuckyPotvrzeni = Prava::vratPrava('pomucky_potvrzeni');

        Prava::nastavObdobiProPrava(time(),time());

        $this->title = TextManager::_('pracovni_pomucky');
        $this->code = 'browserpracovnipomucky';
        $this->tableId = 'browserpracovnipomucky';

        $this->sqlSelectPart =
            'PracovniPomucka.Kod, PrirazenaPracPomucka.DatumVydani, PrirazenaPracPomucka.DatumVraceni, PrirazenaPracPomucka.Poradi,'. // 0 - 3
            'PrirazenaPracPomucka.DatumExpirace,  {fn_CASTWIDEMEMO(PrirazenaPracPomucka.Poznamka)}, PrirazenaPracPomucka.RC,'. // 4 - 6
            'TypPomucky.Popis, PracovniPomucka.Popis, PracovniPomucka.Hodnota, PrirazenaPracPomucka.DatumNaroku, '.   //7 - 10
            'PotvrzeniPrevzetiDatum, PotvrzeniPrevzetiPopis, TypPomucky.Kod, Osoba.Jmeno, Osoba.Prijmeni, '. //11 - 15
            '(SELECT MAX(ID) FROM PracovniPomer WHERE RC = Osoba.RC) AS IdPP, VyzadovatPotvrzeni, PrirazenaPracPomucka.DatumExpiraceVypoctene ';  //16 - 18

        $this->sqlFromPart =
            'PrirazenaPracPomucka, PracovniPomucka, StavPomucky, TypPomucky, Osoba';

        $this->sqlWherePart =
            'PrirazenaPracPomucka.PracovniPomucka=PracovniPomucka.Kod
             AND PrirazenaPracPomucka.RC = Osoba.RC
             AND PracovniPomucka.Stav = StavPomucky.Kod
             AND PracovniPomucka.TypPomucky = TypPomucky.Kod';

        $this->defaultSort = 5;
        $this->primaryField = 0;
        $this->browserUrl = '?menuid=pracovnipomucky&';

        $this->maxPageRows = 100;

        $this ->code = 'browserpracovnipomucky';

//        $this ->filterForm = new PracovniPomuckyFilterForm($this ->database, $this->pravaProhlizet, $this->pomuckyPotvrzeni, $this->pravaZapisPrevzetiVydej);
//        $this ->filterForm ->loadData();

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni', 15) ->setHeaderText('Prijmeni') ->setOrdering(1)
            ->setCode('col-prijmeni');

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno', 14) ->setHeaderText('Jmeno') ->setOrdering(1)
            ->setCode('col-jmeno');

        $this->addField(new OsobniCisloBrowserField($database, 6))
            ->setCode('col-osobni-cislo');

        $this->addField()   // new TypPomuckyField($database, null));
            ->setSqlFieldName('PracovniPomucka.TypPomucky', 7) ->setHeaderText('typ_pomucky') ->setOrdering(1)
            ->setCode('col-typ-pomucky');

        $this->addField()   // new PopisPomuckyField($database, null));
            ->setSqlFieldName('PracovniPomucka.Popis', 8) ->setHeaderText('Popis') ->setOrdering(1)
            ->setCode('col-popis-pomucky');

        $this->addField(BrowserField::FIELD_TYPE_TEXT)   // new DetailPomuckyField($database,null));
            ->setText(TextManager::_('detail')) //->setLink('?menuid=pracovnipomucky&action=detail&record=')
            ->setCode('col-detail')
            ->setClassName('js pracovni-pomucky-detail-link')
            ->setAttributeIndexes(['data-kod' => 0]);

        $this->addField(new DatumVydaniField($database, null))
            ->setCode('col-datum-vydani');

        $this->addField(new DatumVraceniField($database, null))
            ->setCode('col-datum-vraceni');

        $this->addField(new EditaceExpiraceField($database, null))
            ->setCode('col-expirace');

        $this->addField(new ExpiraceVypoctenaField($database, null))
            ->setSqlFieldIndex(18)
            ->setCode('col-expirace-vypoctena');


        /*        $this->addField([BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'PrirazenaPracPomucka.DatumExpirace',
                    BROWSER_FIELD_HEADER_TEXT => 'datum_expirace', 'format' => 'date' , 'align' => 'center', 'ordering' => 1]);*/

        $this->addField(new PotvrzeniPrevzetiField($database))
            ->setCode('col-potvrzeni-revzeti');
    }


    function setFilterForm()
    {
        $this ->filterForm = new PracovniPomuckyFilterForm($this ->database,
            $this->pravaProhlizet, $this->pomuckyPotvrzeni, $this->pravaZapisPrevzetiVydej);
    }


    /**
     * @return SqlStatement|string
     */
    public function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result =
            '(' .
            $this->pravaProhlizet->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ' OR ' .
            $this->pravaZapisPrevzetiVydej->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ' OR ' .
            $this->pomuckyPotvrzeni->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ')';

        $today = DateUtils::today();

        if (!$this->filterForm->vsechnyOsoby) {
            // filtr pro vybranou osobu
            $result .=
                ' AND PrirazenaPracPomucka.RC = ' . $params->addParameter($this->filterForm->vybranaOsobaRC);
        } else {
            if ($this->filterForm->kodRozdeleni != '') {
                $result .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PrirazenaPracPomucka',
                        RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                        $today, $today, $this->filterForm->vcetnePodrizenychRozdeleni, $params);
            }
            if ($this->filterForm->kodRozdeleni2 != '') {
                $result .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PrirazenaPracPomucka',
                        RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this->filterForm->kodRozdeleni2,
                        $today, $today, $this->filterForm->vcetnePodrizenychRozdeleni2, $params);
            }
        }

//        $result .=
//            ' AND ((PrirazenaPracPomucka.DatumVydani >= ' . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumOd())) .
//            ' AND PrirazenaPracPomucka.DatumVydani <= ' . $params->addParameter($this->database->timestampToParam(RonPortalSession::getDatumDo()));

//        $result .=
//            " ) OR PrirazenaPracPomucka.DatumVydani IS NULL )";


        $resultVydani = '';

        if (!isset($_SESSION['filtrVydanePomucky'])  ||  !$_SESSION['filtrVydanePomucky']  ||  !is_array($_SESSION['filtrVydanePomucky'])) {
            // filtr "stav k datu"
            $resultVydani .=
                '(' .
                '(PrirazenaPracPomucka.DatumVraceni IS NULL  OR  PrirazenaPracPomucka.DatumVraceni >=' . $params ->addParameter($this->database->dateToParam($this->filterForm->datum)) . ')';

            if ($this->filterForm->datum !== $today) {
                // jiny datum nez dnes (stav v minulosti)
                $resultVydani .=
                    'AND (' .
                    'PrirazenaPracPomucka.DatumVydani <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu vydana
                    ' OR PrirazenaPracPomucka.DatumNaroku <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu je na ni narok
                    ')';
            }

            $resultVydani .=
                ')';
        } else {

            //$_SESSION['filtrVydanePomucky'][$name]
            if (isset($_SESSION['filtrVydanePomucky']['jenNevydane'])  &&  $_SESSION['filtrVydanePomucky']['jenNevydane']) {
                $resultVydani .=
                    ($resultVydani ? ' OR ' : '') .
                    'PrirazenaPracPomucka.DatumVydani IS NULL';

                if ($this->filterForm->datum != $today) {
                    // je zvolen jiny nez aktualni datum
                    $resultVydani .= ' OR PrirazenaPracPomucka.DatumVydani > ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum));
                }
            }

            if (isset($_SESSION['filtrVydanePomucky']['jenPotvrzene'])  &&  $_SESSION['filtrVydanePomucky']['jenPotvrzene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($this->filterForm->datum == $today) {
                    // k aktualnimu datu
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani IS NOT NULL AND PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NOT NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL)';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu vydana
                        ' AND PrirazenaPracPomucka.PotvrzeniPrevzetiDatum < ' . $params->addParameter($this->database->dateToParam(DateUtils::incDay($this->filterForm->datum))) . // k zadanemu datu potvrzena
                        ' AND (PrirazenaPracPomucka.DatumVraceni > ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu nevracena
                        '  OR PrirazenaPracPomucka.DatumVraceni IS NULL)' .
                        ')';
                }
            }

            if (isset($_SESSION['filtrVydanePomucky']['jenNepotvrzene'])  &&  $_SESSION['filtrVydanePomucky']['jenNepotvrzene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($this->filterForm->datum == $today) {
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani IS NOT NULL AND  PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL)';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu vydana
                        ' AND (PrirazenaPracPomucka.PotvrzeniPrevzetiDatum >= ' . $params->addParameter($this->database->dateToParam(DateUtils::incDay($this->filterForm->datum))) . // k zadanemu datu nepotvrzena
                        '  OR PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NULL)' .
                        ' AND (PrirazenaPracPomucka.DatumVraceni > ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . // k zadanemu datu nevracena
                        '  OR PrirazenaPracPomucka.DatumVraceni IS NULL)' .
                        ')';
                }
            }


//        if ($_SESSION['jenNevracene']) {
//            if ($resultVydani != '') {
//                $resultVydani .= ' OR ';
//            }
//            $resultVydani .= " PrirazenaPracPomucka.DatumVraceni IS NULL";
//        }

            if (isset($_SESSION['filtrVydanePomucky']['jenVracene']) && $_SESSION['filtrVydanePomucky']['jenVracene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($this->filterForm->datum == $today) {
                    $resultVydani .=
                        'PrirazenaPracPomucka.DatumVraceni IS NOT NULL';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        'PrirazenaPracPomucka.DatumVraceni <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)); // k zadanemu datu vracena
                }
            }

            if (isset($_SESSION['filtrVydanePomucky']['jenExpirovane']) && $_SESSION['filtrVydanePomucky']['jenExpirovane']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                $resultVydani .=
                    '(PrirazenaPracPomucka.DatumExpirace <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum));

                if ($this->filterForm->datum == $today) {
                    $resultVydani .=
                        ' AND PrirazenaPracPomucka.DatumVraceni IS NULL';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        ' AND (PrirazenaPracPomucka.DatumVraceni IS NULL OR PrirazenaPracPomucka.DatumVraceni > ' . $params->addParameter($this->database->dateToParam($this->filterForm->datum)) . ')';
                }


                $resultVydani .= ')';
            }
        }

        if ($resultVydani != '') {
            $result .= ' AND (' . $resultVydani . ')';
        }


        if (RonPortalSession::getValue('jenTypPomucky')) {
            $result .= " AND PracovniPomucka.TypPomucky = " . $params->addParameter(RonPortalSession::getValue('typPomucky'));
        }

        if (RonPortalSession::getValue('jenPomucka')) {
            $result .= " AND PracovniPomucka.Kod = " . $params->addParameter(RonPortalSession::getValue('pomucka'));
        }
        return new SqlStatement($result, $params);
    }


//    function renderFilter()
//    {
//        parent::renderFilter();
//        $this->filterForm->render();
//    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        $colspan = ($this->columnCount - 1);
        if ($values[5]) {
            // poznamka
            echo
                "<tr class='$className'>" .
                '<td></td>' .
                "<td colspan='$colspan''>" .
                '<div class="poznamka browserpoznamka">' . StringUtils::htmlTextConvert($values[5] ). '</div>' .
                '</td>' .
                '</tr>';
        }

        $this->renderSeparator();
    }

    function render()
    {
        parent::render();

        echo
            "\n" .
            '<div id="detail_pracovni_pomucky" class="rondialog" title="' . TextManager::_('pracovni_pomucka') . '">';

        $pracovniPomucka = null;
        require 'ronportal/pomucky/view_detail_pomucky.php';

        echo
            '<div class="buttons">' .
            '<button type="button" class="button cancel">' . TextManager::_('OK') . '</button>' .
            '</div>';

        echo
        '</div>';
    }


}
