<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;

class RozdeleniUtils
{

    /**
     * Vraci SQL pro odfiltrovani zaznamu pro zadane rozdeleni (pripadne podrizenych a podobne)
     * Zadana tabulka musi mit pole KodTypuRozdeleni, Rozdeleni
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int $vcetnePodrizenych    0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @param null|Parameters $params
     * @param string[] $fieldNames      Nepovinne pole s jmeny poli pro typ rozdeleni a kod rozdeleni
     * @return string
     */
    public static function sqlFilterRozdeleni($database, $tabulka, $typRozdeleni, $kod, $vcetnePodrizenych, $params = null, $fieldNames = null)
    {
        $fieldNameTyp = $fieldNames ? $fieldNames[0] : 'KodTypuRozdeleni';
        $fieldNameKod = $fieldNames ? $fieldNames[1] : 'Rozdeleni';

        if ($vcetnePodrizenych === true) {
            $vcetnePodrizenych = 1;
        } else if ($vcetnePodrizenych === false) {
            $vcetnePodrizenych = 0;
        }

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM HierarchieRozdeleni ' .
                    '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    '    AND HierarchieRozdeleni.KodNadrazene = ' . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .

                    (($vcetnePodrizenych == 2) ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') . // filtr pro jen podrizene

                    "    AND $tabulka.$fieldNameTyp = HierarchieRozdeleni.typPodrizene" .
                    "    AND $tabulka.$fieldNameKod = HierarchieRozdeleni.kodPodrizene" .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM Rozdeleni ' .
                    '  WHERE Rozdeleni.RozdeleniRodice = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    '    AND Rozdeleni.Rodic = ' . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    "    AND $tabulka.$fieldNameTyp = Rozdeleni.KodTypuRozdeleni" .
                    "    AND $tabulka.$fieldNameKod = Rozdeleni.Kod" .
                    ')';
            default :
                // jen vybrane
                return
                    "$tabulka.$fieldNameTyp = " . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    " AND $tabulka.$fieldNameKod = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod));
        }
    }




    /**
     * Vraci SQL (EXISTS) pro odfiltrovani zaznamu pro pracovni pomery ze zadaneho rozdeleni (pripadne podrizenych a podobne)
     * Zadana tabulka musi mit pole RC, OsobniCislo, DatumNastupu
     * @param Database $database
     * @param string $tabulka
     * @param int $typRozdeleni
     * @param string $kod
     * @param int $datumOd
     * @param int $datumDo
     * @param int $vcetnePodrizenych   0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizne, 3=jen primo podrizene
     * @param null|Parameters $params
     * @return string
     */
    public static function sqlFilterRozdeleniPP($database, $tabulka, $typRozdeleni, $kod, $datumOd, $datumDo, $vcetnePodrizenych, $params = null, $jenRC = false)
    {
        if ($vcetnePodrizenych === true) {
            $vcetnePodrizenych = 1;
        } else if ($vcetnePodrizenych === false) {
            $vcetnePodrizenych = 0;
        }

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                    '  WHERE HierarchieRozdeleni.TypNadrazene = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND HierarchieRozdeleni.KodNadrazene = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .

                    (($vcetnePodrizenych == 2)
                        ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene'
                        : ''
                    ) . // filtr pro jen podrizene

                    '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene' .
                    '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.kodPodrizene' .

                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ) .
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob, Rozdeleni ' .
                    '  WHERE Rozdeleni.RozdeleniRodice = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND Rozdeleni.Rodic = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                    '    AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod' .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ).
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            default :
                // jen vybrane
                return
                    'EXISTS (' .
                    '  SELECT * FROM RozdeleniOsob ' .
                    '  WHERE RozdeleniOsob.KodTypuRozdeleni = ' . (is_null($params) ? (int)$typRozdeleni : $params->addParameter($typRozdeleni)) .
                    "    AND RozdeleniOsob.Rozdeleni = " . (is_null($params) ? "'" . $database ->addSlashes($kod) . "'" : $params->addParameter($kod)) .
                    '    AND RozdeleniOsob.RC = ' . $tabulka . '.RC ' .
                    ($jenRC
                        ? ''
                        : '    AND RozdeleniOsob.OsobniCislo = ' . $tabulka . '.OsobniCislo ' .
                        '    AND RozdeleniOsob.DatumNastupu = ' . $tabulka . '.DatumNastupu '
                    ).
                    '    AND RozdeleniOsob.PlatnostOd <= ' . (is_null($params) ? $database ->timestampToString($datumDo) : $params->addParameter($database->timestampToParam($datumDo))) .
                    '    AND (RozdeleniOsob.PlatnostDo >= ' . (is_null($params) ? $database ->timestampToString($datumOd) : $params->addParameter($database->timestampToParam($datumOd))) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
        }
    }


    /**
     * Vraci true, pokud je RCpodrizeny v danem obdobi v nejakem podrizenem rozdeleni daneho typu
     * Pokud je vcetneVlastnich true, tak muze byt i ve stejnem rozdeleni
     * Pokud neni nastaveno $OCpodrizeny, $nastupPodrizeny, kontroluje se jen RC
     *
     * @param Database $database
     * @param string $RCNadrizeny
     * @param string $RCPodrizeny
     * @param string $OCPodrizeny
     * @param int $nastupPodrizeny
     * @param int $typRozdeleni
     * @param int $vcetneVlastnich
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return bool
     */
    public static function jePodrizeny($database, $RCNadrizeny, $RCPodrizeny, $OCPodrizeny, $nastupPodrizeny, $typRozdeleni, $vcetneVlastnich, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT R_Ja.RC ' .
            '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
            '  WHERE R_Ja.RC = ' . $params->addParameter($RCNadrizeny) .
            '    AND R_JA.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '    AND R_Ja.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
            '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
            '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
            '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
            '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
            '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
            '    AND R_Hodnoceny.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '    AND (R_Hodnoceny.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
            (!$vcetneVlastnich ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
            '    AND R_Hodnoceny.RC = ' . $params->addParameter($RCPodrizeny);

        if ($OCPodrizeny && $nastupPodrizeny) {
            // kontroluje se i osobni cislo a datum nastupu
            $sql .=
                '  AND R_Hodnoceny.OsobniCislo = ' . $params ->addParameter($OCPodrizeny) .
                '  AND R_Hodnoceny.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($nastupPodrizeny));
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return (bool)$row;
    }
}
