<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use RonPortal\Controls\ComboBoxOsoba;
use RonPortal\Controls\FormReportsListView;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use function Ron\ronText;


class VydaneObjednavkyFilterForm extends RonPortalFilterForm
{
    /** @var PravaNaOsoby[] */
    private $prava;
    /** @var PravaNaOsoby */
    private $pravaVlozeni;

    private $indexUzivatelskehoFiltru;

    public $comboBoxesRozdeleni;
    public $comboBoxesRozdeleni2;
    public $comboBoxesRozdeleni3;

    protected $filterLinks;

    public $kodRozdeleni;
    public $kodRozdeleni2;
    public $kodRozdeleni3;
    public $kodRozdeleni4;
    public $kodRozdeleni5;

    public function __construct($database, $action, $pravaVlozeni, $prava)
    {
        parent::__construct($database, $action, true);

        $this->prava = $prava;
        $this->pravaVlozeni = $pravaVlozeni;

        $this->nastavDatumOdDo(true, true, ronText('Datum_vystaveni_filter'));

        $this->indexUzivatelskehoFiltru = ObjednavkyController::$indexUzivatelskehoFiltruDodavatele;

        $this->filterLinks = new LinksList();
        $this->filterLinks->itemsIdPrefix = 'vydane_objednavky_stav';

        $data = [
            'vse' => ronText('Vse'),
            'schvalene' => ronText('Schvalene'),
            'schvalene1' => ronText('Schvalene1'),
            'schvalene2' => ronText('Schvalene2'),
            'schvalene3' => ronText('Schvalene3'),
            'neschvalene' => ronText('Neschvalene'),
            'vracene' => ronText('Vracene'),
            'zamitnute' => ronText('Zamitnute'),
            'odeslane' => ronText('Odeslane'),
            'dodane' => ronText('Dodane'),
            'nedodane' => ronText('Nedodane'),
            'vyrizene' => ronText('Vyrizene'),
            'vyfakturovane' => ronText('Vyfakturovane'),
            'nevyfakturovane' => ronText('Nevyfakturovane'),
        ];

        $this->filterLinks->url = '?menuid=objednavky&';

        $this->filterLinks->addItemsFromArray($data, null, false);
    }

    private function loadUzivatelskyFiltr()
    {
        if (isset($_GET['uzivatelskyfiltr']))
            $_SESSION['objednavka_filtr'] = $_GET['uzivatelskyfiltr'];
    }

    private function loadDodavatel()
    {
        if (isset($_GET['dodavatel']))
            $_SESSION['objednavka_dodavatel'] = $_GET['dodavatel'];
    }

    private function loadStredisko()
    {
        if (isset($_GET['stredisko']))
            $_SESSION['objednavka_stredisko'] = $_GET['stredisko'];
    }

    private function loadVystavil()
    {
        if (isset($_GET['vystavil']))
            $_SESSION['objednavka_vystavil'] = $_GET['vystavil'];
    }

    private function loadSchvalovatel()
    {
        if (isset($_GET['schvalovatel1']))
            $_SESSION['objednavka_schvalovatel1'] = $_GET['schvalovatel1'];
        if (isset($_GET['schvalovatel2']))
            $_SESSION['objednavka_schvalovatel2'] = $_GET['schvalovatel2'];
        if (isset($_GET['schvalovatel3']))
            $_SESSION['objednavka_schvalovatel3'] = $_GET['schvalovatel3'];

    }

    private function loadCena()
    {
        if (isset($_GET['cenaod']))
            $_SESSION['objednavka_cenaod'] = $_GET['cenaod'];
        if (isset($_GET['cenado']))
            $_SESSION['objednavka_cenado'] = $_GET['cenado'];
        if (isset($_GET['mena']))
            $_SESSION['objednavka_mena'] = $_GET['mena'];
    }

    private function loadStavObjednavky()
    {
        $vybraneStavy = NastaveniUzivatele::getUserSettingsItem($this->database, 'vydana_objednavka_filtr_stav');
        $this ->filterLinks ->loadSettingString($vybraneStavy);
        $this ->filterLinks ->loadGet();

        //je vybrano vse
        if (isset($_GET['vse']) && ($_GET['vse'] == 1)) {
            foreach ($this ->filterLinks->items as $item) {
                if ($item ->key != 'vse') $item->active = false;
            }
        } else {
            $this->filterLinks->findItem('vse')->active = false;
        }

        $result = null;

        foreach ($this->filterLinks->items as $item) {
            if (isset($_GET['resetfilter'])) {
                $item->active = false;
            }
            if ((!(($item->key == -100) || ($item->key == 100))) && $item->active)
                $result[] = $item->key;
        }
        $_SESSION['objednavka_stav'] = $result;

        if (($vybraneStavy == '') || ($result == null))  {
            $this->filterLinks->activateItem('vse');
        }

        $vybraneNew = $this ->filterLinks ->getSettingsString();
        if ($vybraneNew != $vybraneStavy) NastaveniUzivatele::setUserSettingsItem($this->database, 'vydana_objednavka_filtr_stav', $vybraneNew, '');
    }

    private function loadRozdeleniObjednavky()
    {
        if (isset($_GET['rozdeleniobjednavky'])) {
            $_SESSION['objednavka_rozdeleni'] = $_GET['rozdeleniobjednavky'];
            $this->kodRozdeleni = $_SESSION['objednavka_rozdeleni'];
        }
        if (isset($_GET['rozdeleniobjednavky2'])) {
            $_SESSION['objednavka_rozdeleni2'] = $_GET['rozdeleniobjednavky2'];
            $this->kodRozdeleni2 = $_SESSION['objednavka_rozdeleni2'];
        }
        if (isset($_GET['rozdeleniobjednavky3'])) {
            $_SESSION['objednavka_rozdeleni3'] = $_GET['rozdeleniobjednavky3'];
            $this->kodRozdeleni3 = $_SESSION['objednavka_rozdeleni3'];
        }
        if (isset($_GET['rozdeleniobjednavky4'])) {
            $_SESSION['objednavka_rozdeleni4'] = $_GET['rozdeleniobjednavky4'];
            $this->kodRozdeleni4 = $_SESSION['objednavka_rozdeleni4'];
        }
        if (isset($_GET['rozdeleniobjednavky5'])) {
            $_SESSION['objednavka_rozdeleni5'] = $_GET['rozdeleniobjednavky5'];
            $this->kodRozdeleni5 = $_SESSION['objednavka_rozdeleni5'];
        }
    }

    private function loadVyhledaniTextu()
    {
        if (isset($_GET['objednavka_text']))
            $_SESSION['objednavka_text'] = $_GET['objednavka_text'];
        if (isset($_GET['pole_vyhledani']))
            $_SESSION['objednavka_typ_pole'] = $_GET['pole_vyhledani'];
    }

    private function loadJenSablona()
    {
        if (isset($_GET['jensablony'])) {
            $_SESSION['objednavka_jen_sablony'] = true;
        } else {
            unset($_SESSION['objednavka_jen_sablony']);
        }
        if (isset($_GET['bezsablon'])) {
            $_SESSION['objednavka_bez_sablon'] = true;
        } else {
            unset($_SESSION['objednavka_bez_sablon']);
        }
    }

    private function resetFilter()
    {
        if (isset($_GET['resetfilter'])) {
            $_SESSION['objednavka_filtr'] = "";
            $_SESSION['objednavka_dodavatel'] = "";
            $_SESSION['objednavka_stredisko'] = "";
            $_SESSION['objednavka_vystavil'] = "";
            $_SESSION['objednavka_schvalovatel1'] = "";
            $_SESSION['objednavka_schvalovatel2'] = "";
            $_SESSION['objednavka_schvalovatel3'] = "";
            $_SESSION['objednavka_cenaod'] = "";
            $_SESSION['objednavka_cenado'] = "";
            $_SESSION['objednavka_rozdeleni'] = "";
            $_SESSION['objednavka_rozdeleni2'] = "";
            $_SESSION['objednavka_rozdeleni3'] = "";
            $_SESSION['objednavka_filtr_obdobi'] = null;
            $_SESSION['objednavka_typ_pole'] = 0;
            $_SESSION['objednavka_text'] = "";
            $_SESSION['objednavka_jen_sablony'] = 0;
            $_SESSION['objednavka_bez_sablon'] = 0;
//            RonPortalSession::setFiltrovatDatum(true, DateUtils::firstDayOfYear(DateUtils::today()), DateUtils::today());
        }
    }

    /**
     * @throws DBException
     */
    public function renderStredisko()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Stredisko');
        echo '</SPAN>';

        // combobox pro vyber
        $typRozdeleniFiltrStredisek = RonPortalSettings::getIntValue('vydanaobjednavka_filtrovat_stredisko_tr');


        $pravaStredisko = Prava::vratPrava('objednavka_vydana_stredisko');
        $sql =
            "SELECT kod, Popis FROM Stredisko WHERE EXISTS(SELECT Kod FROM Rozdeleni WHERE Rozdeleni.Kod = Stredisko.kod AND Rozdeleni.KodTypuRozdeleni = " . (int)$typRozdeleniFiltrStredisek .
            ' AND ' . $pravaStredisko->sqlfiltrRozdeleni(DateUtils::today(), DateUtils::today(), 1) . ')';

        $sql .=
            "ORDER BY Popis";

        $combo = new ComboBox('stredisko');
        $combo->emptyRow = ronText('_vse_');
        $combo->htmlParams['data-ron-autosubmit'] = '1';
        $combo->className = 'app_combo adsfilterforminput_full ';
        $combo->loadData($this->database, $sql, isset($_SESSION['objednavka_stredisko']) ? $_SESSION['objednavka_stredisko'] : null);
        $combo->render();

        echo "</DIV>";
    }

    /**
     * @throws DBException
     */
    public function renderDodavatel()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo ronText('Dodavatel');
        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =
            "SELECT kodFirmy, nazev1 FROM Firma WHERE jeDodavatel = 1 AND Aktivni = 1 ";

        if (isset($_SESSION['objednavka_filtr']) && !empty($_SESSION['objednavka_filtr']))
            $sql .= " AND userData{$this->indexUzivatelskehoFiltru} = " . $params->addParameter($_SESSION['objednavka_filtr']);

        $sql .=
            " ORDER BY nazev1";

        $combo = new ComboBox('dodavatel');
        $combo->emptyRow = ronText('_vsichni_');
        $combo->htmlParams['data-ron-autosubmit'] = '1';
        $combo->className = 'app_combo adsfilterforminput_full ';
        $combo->loadData($this->database, $sql, isset($_SESSION['objednavka_dodavatel']) ? $_SESSION['objednavka_dodavatel'] : null, $params);
        $combo->render();

        echo "</DIV>";
    }

    /**
     * @param $index
     * @throws DBException
     */
    private function renderUzivatelskyFiltr($index)
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo RonPortalSettings::getValue('firma_uzivatelska'. $index);
        echo '</SPAN>';

        // combobox pro vyber
        $sql =
            "SELECT DISTINCT userData{$index} FROM Firma WHERE userData{$index} IS NOT NULL ";

        $sql .=
            "ORDER BY 1";

        $combo = new ComboBox('uzivatelskyfiltr');
        $combo->emptyRow = ronText('_vse_');
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->className = 'app_combo adsfilterforminput_full ';
        $combo->htmlParams['data-ron-autosubmit'] =  '1';
        $combo->loadData($this->database, $sql, $_SESSION['objednavka_filtr']);
        $combo->render();

        echo "</DIV>";
    }

    public function renderVystavil()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo ronText('Vystavil');
        echo '</span>';


        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('vystavil');
        $combo->filtr = ' EXISTS(SELECT Vystavil FROM ObjednavkaVydana WHERE ObjednavkaVydana.Vystavil = Osoba.RC)';
        $combo->htmlParams['data-ron-autosubmit'] = '1';
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->dataTypeComparison = false;
        $combo->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->prava,
            ronText('_vsichni_'),
            isset($_SESSION['objednavka_vystavil']) ? $_SESSION['objednavka_vystavil'] : null);
        $combo->render();
        echo '</div>';
    }

    public function renderSchvalovatel($uroven)
    {
        echo "<div class='filterrow vydana-objednavka-schvaleni{$uroven}'>";
        echo '<span class="filtercaption">';
        echo ronText('Schvalovatel' . $uroven);
        echo '</span>';


        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('schvalovatel' . $uroven);
        $combo->filtr = " EXISTS(SELECT 1 FROM ObjednavkaVydana WHERE ObjednavkaVydana.RC_Schvalovatele{$uroven} = Osoba.RC)";
        $combo->htmlParams['data-ron-autosubmit'] = '1';
        $combo->className = 'app_combo adsfilterforminput_full';
        $combo->dataTypeComparison = false;
        $combo->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->prava,
            ronText('_vsichni_'),
            isset($_SESSION['objednavka_schvalovatel' . $uroven]) ? $_SESSION['objednavka_schvalovatel' . $uroven] : null);
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    public function renderCena()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo ronText('Cena');
        echo '</span>';

        $cenaOd = isset($_SESSION['objednavka_cenaod']) ? $_SESSION['objednavka_cenaod'] : '';
        $cenaDo = isset($_SESSION['objednavka_cenado']) ? $_SESSION['objednavka_cenado'] : '';

        echo "<input type='text' class='inputInt' name='cenaod' size='10' value='$cenaOd' />";
        echo "&nbsp;";
        echo " - ";
        echo "&nbsp;";
        echo "<input type='text' class='inputInt' name='cenado' size='10' value='$cenaDo' />";

        echo "&nbsp;";

        $sql =
            "SELECT KodMeny_ISO FROM Mena WHERE Aktivni = 1 ORDER BY KodMeny_ISO";
        $combo = new ComboBox('mena');
        $combo->emptyRow = ronText('Vse');
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql, isset($_SESSION['objednavka_mena']) ? $_SESSION['objednavka_mena'] : null);
        $combo->htmlParams = null;

        $combo->render();

        echo '</div>';
    }

    public function renderNovaObjednavka()
    {
        echo '<div class="filterrow">';

        echo '<span class="filtercaption">';
        echo "<a class='submitbutton' href='?menuid=objednavky&formaction=new'>" . ronText('Objednavka_novy_zaznam') . "</a>";
        echo "</span>";

        echo "<a class='submitbutton' href='?menuid=objednavky&formaction=newtemplate'>" . ronText('Objednavka_vytvorit_novou_sablonu') . "</a>";

        echo "<span class='font-weight-bold'>" . ronText('Suma_objednano') . "</span>&nbsp;";
        echo "<span id='suma_objednano_filter'></span>&nbsp;";
        echo "<sapn class='font-weight-bold margin-left-2'>" . ronText('Suma_fakturovano') . "</sapn>&nbsp;";
        echo "<span id='suma_fakturovano_filter'></span>";

        echo "</div>";
    }

    public function renderFiltrStav()
    {
        echo '<DIV class="filterrow">';

        echo '<span class="filtercaption">' . ronText('Stav_objednavky') . '</span>';

        $this->filterLinks->render();

        echo "</DIV>\n";
    }

    /**
     * @param $typRozdeleni
     * @param null $index
     * @throws DBException
     * @throws Exception
     */
    function renderRozdeleniObjednavky($typRozdeleni, $index = null)
    {
        echo '<div class="filterrow" id="filterrow_rozdeleni">';
        echo '<span class="filtercaption">' . $this->loadPopisTypuRozdeleni($typRozdeleni) . '</span>';

        $params = $this->database->createParameters();
        // combobox pro vyber
        $sql =
            VydanaObjednavkaFormular::getSQLRozdeleni($this->database, $typRozdeleni, $index, $params, isset($_SESSION['objednavka_stredisko']) ? $_SESSION['objednavka_stredisko'] : null);

        $suffix = (($index == 1) ? '' : $index);

        $combo = new ComboBox("rozdeleniobjednavky$suffix");
        $combo->emptyRow = ronText('_vse_');
        $combo->keyIndex = 0;
        $combo->captionIndex = 1;
        $combo->cssClassIndex = 2;
        $combo->className = 'app_combo adsfilterforminput_full ronselect';
        $combo->htmlParams = ['data-ron-autosubmit' =>  '1',
            'data-ronselect-class' => 'adsfilterforminput_full ronselect'];
        $combo->loadData($this->database, $sql, isset($_SESSION["objednavka_rozdeleni$suffix"]) ? $_SESSION["objednavka_rozdeleni$suffix"] : null, $params);
        $combo->render();

        echo "</div>\n";
    }

    public function renderVyhledaniTextu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo ronText('Text');
        echo '</span>';

        $text = isset($_SESSION['objednavka_text']) ? $_SESSION['objednavka_text'] : '';

        echo "<input type='text' class='adsfilterforminput_popis' name='objednavka_text' size='10' value='$text' />";

        $vse = new Option(0, ronText('Vse'));
        $dodavatel = new Option(1, ronText('Dodavatel'));
        $popis = new Option(2, ronText('Popis'));
        $radek_popis = new Option(3, ronText('Radek_vydane_objednavky_popis_filtr'));
        $radek_rozsireny_popis = new Option(4, ronText('Rozsireny_popis'));
        $poznamka = new Option(5, ronText('Poznamka'));
        $cislo_objednavky = new Option(6, ronText('Cislo_objednavky'));

        $combo = new ComboBox("pole_vyhledani", [$vse, $dodavatel, $popis, $radek_popis, $radek_rozsireny_popis, $poznamka, $cislo_objednavky]);
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->htmlParams = ['data-ron-autosubmit' =>  '1'];
        $combo->setSelectedOption(isset($_SESSION['objednavka_typ_pole']) ? (int)$_SESSION['objednavka_typ_pole'] : 0);
        $combo->render();

        echo '</div>';
    }

    public function renderJenSablona()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';

        echo '</span>';

        GuiUtils::renderCheckbox('jensablony', ronText('Vydane_objednavky_jen_sablony'), isset($_SESSION['objednavka_jen_sablony']), 'onClick="submit()"');
        GuiUtils::renderCheckbox('bezsablon', ronText('Vydane_objednavky_bez_sablon'), isset($_SESSION['objednavka_bez_sablon']), 'onClick="submit()"');

        echo '</div>';
    }

    function renderFoto()
    {
        echo "<a class='submitbutton margin-left-2' href='?menuid=objednavky&resetfilter=1'>" . ronText('Objednavka_resetovat_filtr') . "</a>";
    }

    function renderReports()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo '</SPAN>';

        $reportList = Report::loadFormReports($this->database, 'VydaneObjednavky');

        $params = array_filter($_SESSION, function($k) {
            return strpos($k, 'objednavka_') === 0;
        }, ARRAY_FILTER_USE_KEY);

        $params['datumod'] = RonPortalSession::getDatumOd();
        $params['datumdo'] = RonPortalSession::getDatumDo();

        FormReportsListView::renderReportList($reportList, '?menuid=objednavky&action=report&' . http_build_query($params));

        echo "</DIV>";
    }

    function loadData()
    {
        parent::loadData();
        $this->loadStredisko();
        if ($this->indexUzivatelskehoFiltru)
            $this->loadUzivatelskyFiltr();
        $this->loadRozdeleniObjednavky();
        $this->loadDodavatel();
        $this->loadVystavil();
        $this->loadSchvalovatel();
        $this->loadCena();
        $this->loadStavObjednavky();
        $this->loadVyhledaniTextu();
        $this->loadJenSablona();
        $this->resetFilter();

        $nastavDatum = false;

        if (isset($_GET['datumod'])) {
            $_SESSION['objednavka_filtr_obdobi'] = true;
        } else {
            if (isset($_SESSION['objednavka_filtr_obdobi']))
                $nastavDatum = true;
        }

        if ($nastavDatum) {
            $datumOd = DateUtils::firstDayOfYear(time());
            $datumDo = DateUtils::today();

            RonPortalSession::setFiltrovatDatum(true, $datumOd, $datumDo);
        }
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderStredisko();
        if ($this->indexUzivatelskehoFiltru)
            $this->renderUzivatelskyFiltr($this->indexUzivatelskehoFiltru);

        if (ObjednavkyController::$typRozdeleniVydaneObjednavky1)
            $this->renderRozdeleniObjednavky(ObjednavkyController::$typRozdeleniVydaneObjednavky1, 1);
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky2)
            $this->renderRozdeleniObjednavky(ObjednavkyController::$typRozdeleniVydaneObjednavky2, 2);
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky3)
            $this->renderRozdeleniObjednavky(ObjednavkyController::$typRozdeleniVydaneObjednavky3, 3);
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky4)
            $this->renderRozdeleniObjednavky(ObjednavkyController::$typRozdeleniVydaneObjednavky4, 4);
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky5)
            $this->renderRozdeleniObjednavky(ObjednavkyController::$typRozdeleniVydaneObjednavky5, 5);
        $this->renderDodavatel();
        $this->renderCena();
        $this->renderVystavil();
        $this->renderSchvalovatel(1);
        $this->renderSchvalovatel(2);
        $this->renderSchvalovatel(3);
        $this->renderVyhledaniTextu();
        $this->renderJenSablona();
        $this->renderFiltrStav();
        $this->renderReports();
        if ($this->pravaVlozeni->jsouNejakePrava())
            $this->renderNovaObjednavka();
    }

}

class KopieObjednavkyField extends BrowserField
{
    /** @var PravaNaOsoby */
    private $prava;

    function __construct($prava)
    {
        parent::__construct();
        $this->align = 'left';
        $this->code = 'col-kopie';
        $this->prava = $prava;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($this->prava->jsouNejakePrava()) {
            $result =
                '<a href="?menuid=objednavky&formaction=newfromcopy&record=' . $rowData[0] .
                '"><i class="inline-glyph glyph-button fa glyph-copy" title="' . ronText('Kopirovat_zaznam') . '"></i></a>';
        }
        return $result;
    }
}

class StavSchvaleniObjednavkyField extends BrowserField
{
    private $indexPoleStav1;
    private $indexPoleStav2;
    private $indexPoleStav3;
    private $indexPoleDatum1;
    private $indexPoleDatum2;
    private $indexPoleDatum3;
    private $indexPoleSchvaleno;

    function __construct($indexPoleStav1, $indexPoleStav2, $indexPoleStav3, $indexPoleDatum1, $indexPoleDatum2, $indexPoleDatum3, $indexPoleSchvaleno)
    {
        parent::__construct();
        $this->align = 'left';
        $this->code = 'col-stav-schvaleni';
        $this->header = ronText('Stav_objednavky');
        $this->indexPoleStav1 = $indexPoleStav1;
        $this->indexPoleStav2 = $indexPoleStav2;
        $this->indexPoleStav3 = $indexPoleStav3;
        $this->indexPoleDatum1 = $indexPoleDatum1;
        $this->indexPoleDatum2 = $indexPoleDatum2;
        $this->indexPoleDatum3 = $indexPoleDatum3;
        $this->indexPoleSchvaleno = $indexPoleSchvaleno;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // schvaleni => odeslane -> dodane -> fakturovane -> vyrizene
        if ($rowData[VydaneObjednavkyBrowser::INDEX_VYRIZENO])
            return ronText('Stav_vyrizena');

        if ($rowData[VydaneObjednavkyBrowser::INDEX_FAKTUROVANA_CENA] > 0)
            return ronText('Stav_fakturovana');

        //pocet radku s nepotvrzenym dodanim musi byt 0
        if (($rowData[VydaneObjednavkyBrowser::INDEX_POCET_NEDODANYCH] == 0) && ($rowData[VydaneObjednavkyBrowser::INDEX_ODESLANO]))
            return ronText('Stav_dodana');

        if ($rowData[VydaneObjednavkyBrowser::INDEX_ODESLANO])
            return ronText('Stav_odeslana');

        if ((int)$rowData[$this->indexPoleSchvaleno] == 1)
            return ronText('Schvaleno');

        if ($rowData[$this->indexPoleDatum3]) {
            return VydanaObjednavka::vratPopisStavuSchvaleni($rowData[$this->indexPoleStav3], 3);
        } elseif ($rowData[$this->indexPoleDatum2]) {
            return VydanaObjednavka::vratPopisStavuSchvaleni($rowData[$this->indexPoleStav2], 2);
        } elseif ($rowData[$this->indexPoleStav1]) {
            return VydanaObjednavka::vratPopisStavuSchvaleni($rowData[$this->indexPoleStav1], 1);
        }

        return '';
    }
}

class DokumentyObjednavkyField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
        $this->code = 'col-dokumenty';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[VydaneObjednavkyBrowser::INDEX_POCET_DOKUMENTU]) {
            $text = '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . ronText('Pocet_priloh') . ': ' . $rowData[VydaneObjednavkyBrowser::INDEX_POCET_DOKUMENTU] . '"></i>';

            // pokud je jen jeden dokument, tak rovnou zobrazi proklik na stahnuti souboru
            if ($rowData[VydaneObjednavkyBrowser::INDEX_POCET_DOKUMENTU] == 1) {
                return "<a target='_blank' href='?menuid=objednavky&action=download&cisloobjednavky={$rowData[0]}&record={$rowData[VydaneObjednavkyBrowser::INDEX_ID_DOKUMENTU]}'>$text</a>";
            }

            return $text;
        }

        return '';
    }
}

class ZruseniObjednavkyField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava;

    /**
     * @param Database $database
     * @param $prava
     */
    function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->prava = $prava;
        $this->code = 'col-zrusit';

        $this->header = '<span title="' . ronText('Vyrizeno') . '/' .  ronText('Zrusit') . '">' . ronText('Sloupec_zrusit_objednavku') . '</span>';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[31]) {
            // zauctovano - nejde smazat, zobrazuje fajfku
            return '<i class="inline-glyph fa glyph-check" title="' . ronText('Vyrizeno') . '">';
        }

        if ($rowData[12] || $rowData[13] || $rowData[14]) return ''; // jiz je schvaleno, nejde smazat

        // zkontroluje prava na editaci
        if ($this->prava->lzeZobrazitRC($rowData[19], $this->database->stringToTimestamp($rowData[2])) && $this->prava->priznak) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('Zrusit_objednavku') . '"></i>', ronText('Opravdu_zrusit_objednavku'),
                '?menuid=objednavky&action=zrusitobjednavku&record=' . $rowData[0]);
        }

        return '';
    }
}

class CenaObjednavkyField extends BrowserField
{
    private $suma;
    private $count;
    private $htmlID;
    private $scitatZaMenu;

    public function __construct($database, $htmlID, $scitatZaMenu = true)
    {
        parent::__construct($database);
        $this->htmlID = $htmlID;
        $this->scitatZaMenu = $scitatZaMenu;

        $this->setFormat('number2')
            ->setAlign('right')
            ->setOrdering(true);
        return $this;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //jen pokud se nejedna o sablonu
        if (!$rowData[VydaneObjednavkyBrowser::INDEX_JE_SABLONA]) {
            if ($this->scitatZaMenu) {
                $this->suma[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]] =
                    (isset($this->suma[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]]) ? $this->suma[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]] : 0) + $value;
                $this->count[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]] =
                    (isset($this->count[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]]) ? $this->count[$rowData[VydaneObjednavkyBrowser::INDEX_MENA]] : 0) + 1;
            } else {
                $this->suma += $value;
                $this->count++;
            }
        }
        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }

    function getDataCellClass($value,$rowNumber,$recordId,$rowData)
    {
        $result = parent::getDataCellClass($value,$rowNumber,$recordId,$rowData);

        //jen pokud se nejedna o sablonu
        if (!$rowData[VydaneObjednavkyBrowser::INDEX_JE_SABLONA] && $this->scitatZaMenu && ($rowData[VydaneObjednavkyBrowser::INDEX_FAKTUROVANA_CENA] > 0) && ($rowData[VydaneObjednavkyBrowser::INDEX_CENA] != $rowData[VydaneObjednavkyBrowser::INDEX_FAKTUROVANA_CENA]))
            $result .= ' vydana_objednavka_pole_nesouhlasi_cena';

        return $result;
    }


    function getFooterText($fieldIndex, $url = null)
    {
        $result = '';
        if ($this->scitatZaMenu) {
            if (!empty($this->suma)) {
                foreach ($this->suma as $mena => $castka) {
                    if ($result != '')
                        $result .= "<br/>&nbsp;";
                    $result .= StringUtils::format($this->format, $castka) . ' ' . $mena . ' (' . $this->count[$mena] . ')';
                }
            }
        } else {
            $result = StringUtils::format($this->format, $this->suma) . ' ' . RonPortalSettings::getValue('vydanaobjednavka_domaci_mena');
        }

        return "<span id='{$this->htmlID}'>$result</span>";
    }


}

class VydaneObjednavkyBrowser extends RonPortalBrowser
{
    const INDEX_UZ_POLE_FIRMY = 43;
    const INDEX_POCET_DOKUMENTU = 33;
    const INDEX_ID_DOKUMENTU = 34;
    const INDEX_MENA = 32;
    const INDEX_CENA = 11;
    const INDEX_FAKTUROVANA_CENA = 27;

    const INDEX_VYRIZENO = 31;
    const INDEX_ODESLANO = 30;
    const INDEX_POCET_NEDODANYCH = 35;
    const INDEX_JE_SABLONA = 38;

    /** @var VydaneObjednavkyFilterForm */
    public $filterForm;
    /** @var PravaNaOsoby */
    private $pravaProhlizeni;
    /** @var PravaNaOsoby */
    private $pravaVlozeni;
    /** @var PravaNaOsoby */
    private $pravaSchvaleni1;
    /** @var PravaNaOsoby */
    private $pravaSchvaleni2;
    /** @var PravaNaOsoby */
    private $pravaSchvaleni3;
    /** @var PravaNaOsoby */
    private $pravaFakturace;

    private $indexUzPoleFirmy;

    /**
     * VydaneObjednavkyBrowser constructor.
     * @param Database $database
     * @param string $action
     * @param PravaNaOsoby $pravaProhlizeni
     * @param PravaNaOsoby $pravaVlozeni
     * @param PravaNaOsoby $pravaSchvaleni1
     * @param PravaNaOsoby $pravaSchvaleni2
     * @param PravaNaOsoby $pravaSchvaleni3
     * @param PravaNaOsoby $pravaFakturace
     * @throws Exception
     */
    public function __construct($database, $action, $pravaProhlizeni, $pravaVlozeni, $pravaSchvaleni1, $pravaSchvaleni2, $pravaSchvaleni3, $pravaFakturace)
    {
        parent::__construct($database, $action);

        $this->pravaProhlizeni = $pravaProhlizeni;
        $this->pravaVlozeni = $pravaVlozeni;
        $this->pravaSchvaleni1 = $pravaSchvaleni1;
        $this->pravaSchvaleni2 = $pravaSchvaleni2;
        $this->pravaSchvaleni3 = $pravaSchvaleni3;
        $this->pravaFakturace = $pravaFakturace;

        $this->code = 'browservydanaobjednavka';

        $this->title = ronText('Objednavka_vydana_seznam');

        $this->indexUzPoleFirmy = ObjednavkyController::$indexUzivatelskehoFiltruDodavatele;

        $this->sqlSelectPart =
            'cisloVydaneObjednavky, ObjednavkaVydana.stredisko, datumVystaveni, terminDodani, ObjednavkaVydana.popis, {fn_CASTWIDEMEMO(ObjednavkaVydana.poznamka)}, Stredisko.Popis, ' . // 0 - 6
            "Vystavil.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Vystavil.Jmeno,'') AS Vystavil, ". // 7
            "Schvalovatel1.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Schvalovatel1.Jmeno,'') AS Schvalovatel1, Schvalovatel2.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Schvalovatel2.Jmeno,'') AS Schvalovatel2, " . // 8 - 9
            "Schvalovatel3.Prijmeni {fn_concat()} ' ' {fn_concat()} COALESCE(Schvalovatel3.Jmeno,'') AS Schvalovatel3, ObjednavkaVydana.cenaCelkem, " . // 10 - 11
            " schvaleni1_DT, schvaleni2_DT, schvaleni3_DT, schvaleni1_stav, schvaleni2_stav, schvaleni3_stav, Firma.nazev1, ObjednavkaVydana.vystavil, " .  // 12 - 19
            "ObjednavkaVydana.Rozdeleni, ObjednavkaVydana.Rozdeleni2, ObjednavkaVydana.Rozdeleni3, R.Popis R1Popis, R2.Popis R2Popis, R3.Popis R3Popis, ObjednavkaVydana.schvaleno, " . // 20 - 26
            "(SELECT SUM(FakturaPrijata.castka) FROM FakturaPrijata WHERE FakturaPrijata.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky) FakturovanaCastka, " . // 27
            "(SELECT SUM(COALESCE(RadekVydaneObjednavky.mnozstvi,0)) FROM RadekVydaneObjednavky WHERE RadekVydaneObjednavky.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky) ObjednanoKusu, " . // 28
            "(SELECT SUM(COALESCE(RadekVydaneObjednavky.dodanoMnozstvi,0)) FROM RadekVydaneObjednavky WHERE RadekVydaneObjednavky.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky) DodanoKusu, " . // 29
            "odeslano, vyrizeno, ObjednavkaVydana.mena, (SELECT COUNT(ID) FROM DokumentObjednavkyVydane WHERE DokumentObjednavkyVydane.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky), " . // 30 - 33
            "(SELECT MIN(ID) FROM DokumentObjednavkyVydane WHERE DokumentObjednavkyVydane.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky), " . // 34
            "(SELECT COUNT(cisloVydaneObjednavky) FROM RadekVydaneObjednavky WHERE RadekVydaneObjednavky.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky AND souhlasSDodanymMnozstvim <> 1) PocetNedodanych, " . // 35
            "R4.Popis R4Popis, R5.Popis R5Popis, sablona, ObjednavkaVydana.Rozdeleni4, ObjednavkaVydana.Rozdeleni5, ObjednavkaVydana.DM_castka, " . // 36 - 41
            "(SELECT SUM(FakturaPrijata.DM_Castka) FROM FakturaPrijata WHERE FakturaPrijata.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky) FakturovanaCastka_DM " . // 42
            ($this->indexUzPoleFirmy ? ", Firma.userData{$this->indexUzPoleFirmy} " : '');

        $this->sqlFromPart =
            'ObjednavkaVydana JOIN Stredisko ON (Stredisko.Kod = ObjednavkaVydana.stredisko)' .
            ' JOIN Osoba Vystavil ON (Vystavil.RC = ObjednavkaVydana.Vystavil)'.
            ' JOIN Firma ON (Firma.kodFirmy = ObjednavkaVydana.dodavatel)' .
            ' LEFT OUTER JOIN Osoba Schvalovatel1 ON (Schvalovatel1.RC = ObjednavkaVydana.RC_Schvalovatele1)'.
            ' LEFT OUTER JOIN Osoba Schvalovatel2 ON (Schvalovatel2.RC = ObjednavkaVydana.RC_Schvalovatele2)'.
            ' LEFT OUTER JOIN Osoba Schvalovatel3 ON (Schvalovatel3.RC = ObjednavkaVydana.RC_Schvalovatele3)'.
            ' LEFT OUTER JOIN Rozdeleni R ON (R.KodTypuRozdeleni = ObjednavkaVydana.KodTypuRozdeleni AND R.Kod = ObjednavkaVydana.Rozdeleni)'.
            ' LEFT OUTER JOIN Rozdeleni R2 ON (R2.KodTypuRozdeleni = ObjednavkaVydana.KodTypuRozdeleni2 AND R2.Kod = ObjednavkaVydana.Rozdeleni2)'.
            ' LEFT OUTER JOIN Rozdeleni R3 ON (R3.KodTypuRozdeleni = ObjednavkaVydana.KodTypuRozdeleni2 AND R3.Kod = ObjednavkaVydana.Rozdeleni3)'.
            ' LEFT OUTER JOIN Rozdeleni R4 ON (R4.KodTypuRozdeleni = ObjednavkaVydana.KodTypuRozdeleni2 AND R4.Kod = ObjednavkaVydana.Rozdeleni4)'.
            ' LEFT OUTER JOIN Rozdeleni R5 ON (R5.KodTypuRozdeleni = ObjednavkaVydana.KodTypuRozdeleni2 AND R5.Kod = ObjednavkaVydana.Rozdeleni5)';

        $this->primaryField = 0;

        $this->addField()
            ->setHeaderText('Cislo_objednavky')
            ->setSqlFieldIndex(0)
            ->setSqlFieldName('cisloVydaneObjednavky')
            ->setLink('?menuid=objednavky&action=detail')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Stredisko')
            ->setSqlFieldIndex(6)
            ->setSqlFieldName('ObjednavkaVydana.stredisko')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Datum_vystaveni')
            ->setSqlFieldIndex(2)
            ->setSqlFieldName('datumVystaveni')
            ->setFormat('date')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Termin_dodani')
            ->setSqlFieldIndex(3)
            ->setSqlFieldName('terminDodani')
            ->setFormat('date')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Popis')
            ->setSqlFieldIndex(4)
            ->setSqlFieldName('ObjednavkaVydana.popis')
            ->setLink('?menuid=objednavky&action=detail')
            ->setOrdering(true);

        $this->addField(new CenaObjednavkyField($this->database, 'suma_objednano'))
            ->setHeaderText('Cena')
            ->setSqlFieldIndex(11)
            ->setSqlFieldName('ObjednavkaVydana.cenaCelkem');

        $this->addField()
            ->setHeaderText('Mena')
            ->setSqlFieldIndex(32)
            ->setSqlFieldName('ObjednavkaVydana.mena');

        $this->addField(new CenaObjednavkyField($this->database, 'suma_objednano_dm', false))
            ->setHeader(ronText('Cena') . ' ' . RonPortalSettings::getValue('vydanaobjednavka_domaci_mena'))
            ->setSqlFieldIndex(41)
            ->setSqlFieldName('ObjednavkaVydana.DM_castka');

        $this->addField()
            ->setHeaderText('Vystavil')
            ->setSqlFieldIndex(7)
            ->setSqlFieldName('ObjednavkaVydana.Vystavil')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Dodavatel')
            ->setSqlFieldIndex(18)
            ->setSqlFieldName('Firma.nazev1')
            ->setOrdering(true);

        if ($this->indexUzPoleFirmy) {
            $this->addField()
                ->setHeader(StringUtils::htmlTextConvert(ObjednavkyController::$nadpisUzivatelskehoFiltruDodavatele))
                ->setSqlFieldIndex(static::INDEX_UZ_POLE_FIRMY)
                ->setSqlFieldName("Firma.userData{$this->indexUzPoleFirmy} ")
                ->setOrdering(true);
        }

        $this->addField()
            ->setHeaderText('Mnozstvi')
            ->setSqlFieldIndex(28)
            ->setSqlFieldName('ObjednanoKusu')
            ->setFormat('int')
            ->setAlign('right')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Vydane_objednavky_sablona')
            ->setSqlFieldIndex(38)
            ->setSqlFieldName('sablona')
            ->setFormat('anone')
            ->setAlign('right');

        $this->addField(new StavSchvaleniObjednavkyField(15, 16, 17, 12, 13, 14, 26));
        
        $this->addField()
            ->setHeaderText('Schvalovatel1')
            ->setSqlFieldIndex(8)
            ->setSqlFieldName('Schvalovatel1')
            ->setOrdering(true)
            ->setClassName('vydana-objednavka-schvaleni1');
        $this->addField()
            ->setHeaderText('Schvalovatel2')
            ->setSqlFieldIndex(9)
            ->setSqlFieldName('Schvalovatel2')
            ->setOrdering(true)
            ->setClassName('vydana-objednavka-schvaleni2');
        $this->addField()
            ->setHeaderText('Schvalovatel3')
            ->setSqlFieldIndex(10)
            ->setSqlFieldName('Schvalovatel3')
            ->setOrdering(true)
            ->setClassName('vydana-objednavka-schvaleni3');

        $this->addField()
            ->setHeaderText('Odeslane')
            ->setSqlFieldIndex(30)
            ->setSqlFieldName('odeslano')
            ->setFormat('anone')
            ->setAlign('right');

        $this->addField(new CenaObjednavkyField($this->database, 'suma_fakturovano'))
            ->setHeaderText('Castka')
            ->setSqlFieldIndex(27)
            ->setSqlFieldName('FakturovanaCastka');

        $this->addField(new CenaObjednavkyField($this->database, 'suma_fakturovano_dm', false))
            ->setHeader(ronText('Castka') . ' ' . RonPortalSettings::getValue('vydanaobjednavka_domaci_mena'))
            ->setSqlFieldIndex(42)
            ->setSqlFieldName('FakturovanaCastka_DM');

        $this->addField()
            ->setHeaderText('Dodane')
            ->setSqlFieldIndex(29)
            ->setSqlFieldName('DodanoKusu')
            ->setFormat('int')
            ->setAlign('right')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Vyrizene')
            ->setSqlFieldIndex(31)
            ->setSqlFieldName('vyrizene')
            ->setFormat('anone')
            ->setAlign('right');

        if (ObjednavkyController::$typRozdeleniVydaneObjednavky1) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, ObjednavkyController::$typRozdeleniVydaneObjednavky1);
            $this->addField()
                ->setHeader($typRozdeleni->popis)
                ->setSqlFieldIndex(23)
                ->setSqlFieldName('R1Popis')
                ->setOrdering(true);
            $this->addField()
                ->setHeader($typRozdeleni->popis . " (" . ronText('Kod') . ")")
                ->setSqlFieldIndex(20)
                ->setSqlFieldName('ObjednavkaVydana.Rozdeleni')
                ->setOrdering(true)
                ->setCode('col-rozdelenikod')
                ->setVisible(false);
        }
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky2) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, ObjednavkyController::$typRozdeleniVydaneObjednavky2);
            $this->addField()
                ->setHeader($typRozdeleni->popis)
                ->setSqlFieldIndex(24)
                ->setSqlFieldName('R2Popis')
                ->setOrdering(true);
            $this->addField()
                ->setHeader($typRozdeleni->popis . " (" . ronText('Kod') . ")")
                ->setSqlFieldIndex(21)
                ->setSqlFieldName('ObjednavkaVydana.Rozdeleni2')
                ->setOrdering(true)
                ->setCode('col-rozdeleni2kod')
                ->setVisible(false);
        }
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky3) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, ObjednavkyController::$typRozdeleniVydaneObjednavky3);
            $this->addField()
                ->setHeader($typRozdeleni->popis)
                ->setSqlFieldIndex(25)
                ->setSqlFieldName('R3Popis')
                ->setOrdering(true);
            $this->addField()
                ->setHeader($typRozdeleni->popis . " (" . ronText('Kod') . ")")
                ->setSqlFieldIndex(22)
                ->setSqlFieldName('ObjednavkaVydana.Rozdeleni3')
                ->setOrdering(true)
                ->setCode('col-rozdeleni3kod')
                ->setVisible(false);
        }
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky4) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, ObjednavkyController::$typRozdeleniVydaneObjednavky4);
            $this->addField()
                ->setHeader($typRozdeleni->popis)
                ->setSqlFieldIndex(36)
                ->setSqlFieldName('R4Popis')
                ->setOrdering(true);
            $this->addField()
                ->setHeader($typRozdeleni->popis . " (" . ronText('Kod') . ")")
                ->setSqlFieldIndex(39)
                ->setSqlFieldName('ObjednavkaVydana.Rozdeleni4')
                ->setOrdering(true)
                ->setCode('col-rozdeleni4kod')
                ->setVisible(false);
        }
        if (ObjednavkyController::$typRozdeleniVydaneObjednavky5) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, ObjednavkyController::$typRozdeleniVydaneObjednavky5);
            $this->addField()
                ->setHeader($typRozdeleni->popis)
                ->setSqlFieldIndex(37)
                ->setSqlFieldName('R5Popis')
                ->setOrdering(true);
            $this->addField()
                ->setHeader($typRozdeleni->popis . " (" . ronText('Kod') . ")")
                ->setSqlFieldIndex(40)
                ->setSqlFieldName('ObjednavkaVydana.Rozdeleni5')
                ->setOrdering(true)
                ->setCode('col-rozdeleni5kod')
                ->setVisible(false);
        }

        $this->addField(new KopieObjednavkyField($this->pravaVlozeni));
        $this->addField(new DokumentyObjednavkyField());
        $this->addField(new ZruseniObjednavkyField($this->database, $this->pravaVlozeni));

        $this->showFooter = true;
    }

    function setFilterForm()
    {
        $this ->filterForm = new VydaneObjednavkyFilterForm($this ->database, $this ->action, $this->pravaVlozeni,
            [$this->pravaProhlizeni, $this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3, $this->pravaFakturace]);
    }

    /**
     * @return SqlStatement
     * @throws DBException
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql =
            '( ' .
            ' EXISTS(SELECT RC FROM PracovniPomer ' .
            'WHERE PracovniPomer.RC = ObjednavkaVydana.Vystavil ' .
            '  AND PracovniPomer.DatumNastupu <= ObjednavkaVydana.datumVystaveni ' .
            '  AND (PracovniPomer.DatumUkonceni >= ObjednavkaVydana.datumVystaveni OR PracovniPomer.DatumUkonceni IS NULL)' .
            '  AND (' . $this->pravaProhlizeni->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR' . $this->pravaVlozeni->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR' . $this->pravaSchvaleni1->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR' . $this->pravaSchvaleni2->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR' . $this->pravaSchvaleni3->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR' . $this->pravaFakturace->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       )' .
            ')';

        //zobrazi se i prirazenemu schvalovateli
        $sql .=
            " OR (" .
            "       ObjednavkaVydana.RC_Schvalovatele1 = " . $params->addParameter(User::getRC()) .
            "   OR  ObjednavkaVydana.RC_Schvalovatele2 = " . $params->addParameter(User::getRC()) .
            "   OR  ObjednavkaVydana.RC_Schvalovatele3 = " . $params->addParameter(User::getRC()) .
            " ))";

        $sql .=
            ' AND (ObjednavkaVydana.datumVystaveni >= ' . $params->addParameter($this->database->dateToParam(RonPortalSession::getDatumOd())) .
            ' AND ObjednavkaVydana.datumVystaveni < ' . $params->addParameter($this->database->dateToParam(DateUtils::incDay(RonPortalSession::getDatumDo()))) . ')';

        if (isset($_SESSION['objednavka_stredisko']) && !empty($_SESSION['objednavka_stredisko'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .= 'ObjednavkaVydana.stredisko = ' . $params->addParameter($_SESSION['objednavka_stredisko']);
        }

        if (isset($_SESSION['objednavka_vystavil']) && !empty($_SESSION['objednavka_vystavil'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $paramsRC = $this->database->createParameters();
            $vystavilRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . $paramsRC->addParameter($_SESSION['objednavka_vystavil']), $paramsRC);

            $sql .= 'ObjednavkaVydana.Vystavil = ' . $params->addParameter($vystavilRC);
        }

        for ($i = 1; $i <= 3; $i++) {
            if (isset($_SESSION['objednavka_schvalovatel' . $i]) && !empty($_SESSION['objednavka_schvalovatel' . $i])) {
                if ($sql != '')
                    $sql .= ' AND ';

                $paramsRC = $this->database->createParameters();
                $schvalovatelRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . $paramsRC->addParameter($_SESSION['objednavka_schvalovatel' . $i]), $paramsRC);

                $sql .= "ObjednavkaVydana.RC_Schvalovatele$i = " . $params->addParameter($schvalovatelRC);
            }
        }

        if ((isset($_SESSION['objednavka_cenaod']) && !empty($_SESSION['objednavka_cenaod'])) || (isset($_SESSION['objednavka_cenado']) && !empty($_SESSION['objednavka_cenado']))) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sqlCena = '';

            if (isset($_SESSION['objednavka_cenaod']) && !empty($_SESSION['objednavka_cenaod'])) {
                $sqlCena = "ObjednavkaVydana.cenaCelkem >= " . $params->addParameter($_SESSION['objednavka_cenaod']);
            }

            if (isset($_SESSION['objednavka_cenado']) && !empty($_SESSION['objednavka_cenado'])) {
                if ($sqlCena != '') {
                    $sqlCena .= ' AND ';
                }

                $sqlCena .= "ObjednavkaVydana.cenaCelkem <= " . $params->addParameter($_SESSION['objednavka_cenado']);
            }

            if ($_SESSION['objednavka_mena'] !== '')
                $sqlCena .= " AND ObjednavkaVydana.mena = " . $params->addParameter($_SESSION['objednavka_mena']);

            $sql .= $sqlCena;
        }

        if (isset($_SESSION['objednavka_dodavatel']) && !empty($_SESSION['objednavka_dodavatel'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .=
                ' ObjednavkaVydana.dodavatel = ' . $params->addParameter($_SESSION['objednavka_dodavatel']);
        }

        if (isset($_SESSION['objednavka_filtr']) && !empty($_SESSION['objednavka_filtr'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .=
                " Firma.userData{$this->indexUzPoleFirmy} = " . $params->addParameter($_SESSION['objednavka_filtr']);
        }

        if (isset($_SESSION['objednavka_stav'])) {
            $sqlStav = '';
            foreach ($_SESSION['objednavka_stav'] as $stav) {
                if ($sqlStav != '')
                    $sqlStav .= ' OR ';
                //TODO: zmenit OR na AND? doplnit stav Dodane-nevyfakturovane

                switch ($stav) {
                    case 'schvalene' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleno = 1)';
                        break;
                    case 'schvalene1' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni1_DT IS NOT NULL AND ObjednavkaVydana.schvaleni1_stav = 1)';
                        break;
                    case 'schvalene2' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni2_DT IS NOT NULL AND ObjednavkaVydana.schvaleni2_stav = 1)';
                        break;
                    case 'schvalene3' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni3_DT IS NOT NULL AND ObjednavkaVydana.schvaleni3_stav = 1)';
                        break;
                    case 'neschvalene' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni1_stav <> 1 AND ObjednavkaVydana.schvaleni2_stav <> 1 AND ObjednavkaVydana.schvaleni3_stav <> 1)';
                        break;
                    case 'vracene' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni1_stav = 2 OR ObjednavkaVydana.schvaleni2_stav = 2 OR ObjednavkaVydana.schvaleni3_stav = 2)';
                        break;
                    case 'zamitnute' :
                        $sqlStav .= ' (ObjednavkaVydana.schvaleni1_stav = 3 OR ObjednavkaVydana.schvaleni2_stav = 3 OR ObjednavkaVydana.schvaleni3_stav = 3)';
                        break;
                    case 'odeslane' :
                        $sqlStav .= ' (ObjednavkaVydana.odeslano = 1)';
                        break;
                    case 'dodane' :
                        $sqlStav .= ' (EXISTS(SELECT KodZbozi FROM RadekVydaneObjednavky RO WHERE RO.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky) ' .
                            ' AND NOT EXISTS(SELECT KodZbozi FROM RadekVydaneObjednavky RO WHERE RO.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky AND RO.mnozstvi > 0 AND RO.souhlasSDodanymMnozstvim <> 1))';
                        break;
                    case 'nedodane' :
                        $sqlStav .= ' EXISTS(SELECT KodZbozi FROM RadekVydaneObjednavky RO WHERE RO.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky AND RO.souhlasSDodanymMnozstvim <> 1)';
                        break;
                    case 'vyrizene' :
                        $sqlStav .= ' (ObjednavkaVydana.vyrizeno = 1)';
                        break;
                    case 'vyfakturovane' :
                        $sqlStav .= ' (EXISTS(SELECT cisloFakturyPrijate FROM FakturaPrijata WHERE cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky))';
                        break;
                    case 'nevyfakturovane' :
                        $sqlStav .= ' NOT (EXISTS(SELECT cisloFakturyPrijate FROM FakturaPrijata WHERE cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky))';
                        break;
                }
            }

            if ($sqlStav != '') {
                if ($sql != '')
                    $sql .= ' AND ';

                $sql .= "(" . $sqlStav . ")";
            }
        }

        if ($this->filterForm->kodRozdeleni) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= " ObjednavkaVydana.KodTypuRozdeleni = " . $params->addParameter(ObjednavkyController::$typRozdeleniVydaneObjednavky1) .
                " AND ObjednavkaVydana.Rozdeleni = " . $params->addParameter($this->filterForm->kodRozdeleni);
        }
        if ($this->filterForm->kodRozdeleni2) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= " ObjednavkaVydana.KodTypuRozdeleni2 = " . $params->addParameter(ObjednavkyController::$typRozdeleniVydaneObjednavky2) .
                " AND ObjednavkaVydana.Rozdeleni2 = " . $params->addParameter($this->filterForm->kodRozdeleni2);
        }
        if ($this->filterForm->kodRozdeleni3) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= " ObjednavkaVydana.KodTypuRozdeleni3 = " . $params->addParameter(ObjednavkyController::$typRozdeleniVydaneObjednavky3) .
                " AND ObjednavkaVydana.Rozdeleni3 = " . $params->addParameter($this->filterForm->kodRozdeleni3);
        }
        if ($this->filterForm->kodRozdeleni4) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= " ObjednavkaVydana.KodTypuRozdeleni4 = " . $params->addParameter(ObjednavkyController::$typRozdeleniVydaneObjednavky4) .
                " AND ObjednavkaVydana.Rozdeleni4 = " . $params->addParameter($this->filterForm->kodRozdeleni4);
        }
        if ($this->filterForm->kodRozdeleni5) {
            if ($sql != '')
                $sql .= ' AND ';

            $sql .= " ObjednavkaVydana.KodTypuRozdeleni5 = " . $params->addParameter(ObjednavkyController::$typRozdeleniVydaneObjednavky5) .
                " AND ObjednavkaVydana.Rozdeleni5 = " . $params->addParameter($this->filterForm->kodRozdeleni5);
        }

        if (isset($_SESSION['objednavka_text']) && !empty($_SESSION['objednavka_text'])) {
            $sqlTextSearch = '';
            //vse nebo dodavatel
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 1)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (Firma.nazev1 LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . " OR Firma.nazev2 LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ") ";
            }
            //vse nebo popis objednavky
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 2)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (ObjednavkaVydana.popis LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ") ";
            }
            //vse nebo popis radku
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 3)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (EXISTS(SELECT radekVydaneObjednavky FROM RadekVydaneObjednavky WHERE RadekVydaneObjednavky.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky AND RadekVydaneObjednavky.popis LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ")) ";
            }
            //vse nebo rozsireny popis radku
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 4)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (EXISTS(SELECT radekVydaneObjednavky FROM RadekVydaneObjednavky WHERE RadekVydaneObjednavky.cisloVydaneObjednavky = ObjednavkaVydana.cisloVydaneObjednavky AND RadekVydaneObjednavky.pokracovaniPopisu LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ")) ";
            }
            //vse nebo poznamka
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 5)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (ObjednavkaVydana.poznamka LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ") ";
            }
            //vse nebo cislo objednavky
            if (($_SESSION['objednavka_typ_pole'] == 0) || ($_SESSION['objednavka_typ_pole'] == 6)) {
                if ($sqlTextSearch != '')
                    $sqlTextSearch .= ' OR ';
                $sqlTextSearch .= " (ObjednavkaVydana.cisloVydaneObjednavky LIKE " . $params->addParameter("%{$_SESSION['objednavka_text']}%") . ") ";
            }

            if ($sqlTextSearch != '') {
                if ($sql != '')
                    $sql .= ' AND ';

                $sql .= " (" . $sqlTextSearch . ")";
            }
        }

        if (isset($_SESSION['objednavka_jen_sablony'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .= " ObjednavkaVydana.sablona = " . $params->addParameter(1);
        }
        if (isset($_SESSION['objednavka_bez_sablon'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .= " (ObjednavkaVydana.sablona = " . $params->addParameter(0) . " OR ObjednavkaVydana.sablona IS NULL)";
        }

        return new SqlStatement($sql, $params);
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        if (($values[static::INDEX_FAKTUROVANA_CENA] > 0) && ($values[static::INDEX_CENA] != $values[static::INDEX_FAKTUROVANA_CENA]))
            $className .= ' vydana_objednavka_radek_nesouhlasi_cena';

        parent::renderLine($values, $className, $rowNumber, $edit);
    }


}