<?php
/**
 * stanislav.elbl, 29.07.2015
 */

namespace RonPortal\SluzebniCesty;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;


/**
 * Zaznamy z tabulky PredpokladaneNakladySC
 * Class PredpokladanyVydajSC
 */
class PredpokladanyVydajSC
{
    public $sluzebniCesta_ID;
    public $typ;
    public $pocetDnu;
    public $castka;
    public $castkaStr;
    public $kodMeny;

    public $popisTypu; // tabulka TypRUctovaniSlCesty

    /**
     * Nacte z databaze zadane predpokladane vydaje sluzebni cesty
     *
     * @param Database $database
     * @param int $idSluzebniCesty
     * @return PredpokladanyVydajSC[]|false
     * @throws \Exception
     */
    static function nactiPredpokladaneVydaje($database, $idSluzebniCesty)
    {
        $params = $database ->createParameters();
        if ($idSluzebniCesty) {
            $sql =
                'SELECT TypRUctovaniSlCesty.Kod, TypRUctovaniSlCesty.Popis, ' .
                '       PredpokladaneNakladySC.PocetDnu, PredpokladaneNakladySC.Castka, PredpokladaneNakladySC.KodMeny ' .
                ' FROM SluzebniCesta INNER JOIN TypRUctovaniSlCesty' .
                '  ON 1 = 1' .
                ' LEFT OUTER JOIN PredpokladaneNakladySC ' .
                '  ON TypRUctovaniSlCesty.Kod = PredpokladaneNakladySC.Typ ' .
                '  AND PredpokladaneNakladySC.SluzebniCesta_ID = SluzebniCesta.Id' . // $params ->addParameter((int)$idSluzebniCesty) .
                ' WHERE TypRUctovaniSlCesty.PredpokladaneVydaje = 1 ' .
                '  AND SluzebniCesta.Id = ' . $params->addParameter((int)$idSluzebniCesty) . // kvuli ODBC nemuze byt v ON
                ' ORDER BY TypRUctovaniSlCesty.Popis';
        } else {
            $sql =
                'SELECT TypRUctovaniSlCesty.Kod, TypRUctovaniSlCesty.Popis, NULL, NULL, NULL' .
                ' FROM TypRUctovaniSlCesty' .
                ' WHERE TypRUctovaniSlCesty.PredpokladaneVydaje = 1 ' .
                ' ORDER BY TypRUctovaniSlCesty.Popis';
        }

        $cursor = $database ->select($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $rec = new PredpokladanyVydajSC();
            $rec ->sluzebniCesta_ID = $idSluzebniCesty;
            $rec ->typ = $row[0];
            $rec ->popisTypu = $row[1];
            $rec ->pocetDnu = $row[2];
            $rec ->castka = $row[3];
            $rec ->castkaStr = isset($rec ->castka)
                ? StringUtils::format('number2', $rec ->castka)
                : '';
            $rec ->kodMeny = $row[4];

            $result[] = $rec;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     * @return bool  Ulozi/Smaze zaznam do/z databaze
     * @throws \Exception
     */
    function store($database, $idSluzebniCesty)
    {
        $rec = new RonPortalDBRecord($database, 'PredpokladaneNakladySC');

        $rec ->addField('SluzebniCesta_ID', 'int', true, 0, $idSluzebniCesty);
        $rec ->addField('Typ', 'int', true, 0, $this ->typ);
        $rec ->addField('', '', true, 0, $this ->popisTypu); // jen pro zalogovani

        $rec ->addField('PocetDnu', 'int', false, 0, $this ->pocetDnu);
        $rec ->addField('Castka', 'float', false, 0, $this ->castka);
        $rec ->addField('KodMeny', '', false, 10, $this ->kodMeny);

        $params = $database ->createParameters();
        $rec ->setWhere(
            'SluzebniCesta_ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
            ' AND Typ = ' . $params ->addParameter($this ->typ),
            $params);

        $exists = $rec ->loadRecord();

        $this ->sluzebniCesta_ID = $idSluzebniCesty; // kvuli insert

        if ( $exists ) {
            if ( isset($this ->pocetDnu)  ||  isset($this ->castka) ) {
                return $rec ->updateRecord();
            }
            else {
                return $rec ->deleteRecord();
            }
        } else {
            if ( isset($this ->pocetDnu)  ||  isset($this ->castka) ) {
                $rec ->insertRecord();
            }

            return true;
        }

    }
}


class Spolucestujici
{
    var $idPP;

    var $rc;
    var $osobniCislo;
    var $datumNastupu;

    var $jmenoPrijmeni;

    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     * @return Spolucestujici[]
     */
    public static function nactiSpolucestujici($database, $idSluzebniCesty)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT PracovniPomer.id, Spolucestujici.RC, Spolucestujici.OsobniCislo, Spolucestujici.DatumNastupu,' .
            ' Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul' .
            ' FROM Spolucestujici, PracovniPomer, Osoba' .
            ' WHERE Spolucestujici.SluzebniCesta_ID = ' . $params ->addParameter($idSluzebniCesty) .
            '   AND PracovniPomer.RC = Spolucestujici.RC' .
            '   AND PracovniPomer.OsobniCislo = Spolucestujici.OsobniCislo' .
            '   AND PracovniPomer.DatumNastupu = Spolucestujici.DatumNastupu' .
            '   AND PracovniPomer.RC = Osoba.RC' .
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno, Spolucestujici.OsobniCislo';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani spolucestujicich!');
        while ($row = $cursor ->fetch()) {
            $spolucestujici = new Spolucestujici();
            $spolucestujici ->idPP = $row[0];
            $spolucestujici ->rc = $row[1];
            $spolucestujici ->osobniCislo = $row[2];
            $spolucestujici ->datumNastupu = $cursor ->stringToTimestamp($row[3]);
            $spolucestujici ->jmenoPrijmeni =
                ($row[6] ? $row[6] . ' ' : '') .
                $row[4] . ' ' . $row[5];

            $result[] = $spolucestujici;
        }
        $cursor ->close();

        return $result;
    }
}


class SluzebniCesta
{
    const SETTINGS_UZIVATELSKA1 = 'slcesty_uzivatelska1';
    const SETTINGS_UZIVATELSKA2 = 'slcesty_uzivatelska2';
    const SETTINGS_UZIVATELSKA3 = 'slcesty_uzivatelska3';
    const SETTINGS_UZIVATELSKA4 = 'slcesty_uzivatelska4';
    const SETTINGS_UZIVATELSKA5 = 'slcesty_uzivatelska5';
    const SETTINGS_ZACATEK = 'zacatekslcesty';
    const SETTINGS_ZACATEK_NEMENIT = 'zacatekslcesty_nemenit';
    const SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU = 'nepocitatzakladnisazbu';
    const SETTINGS_NEPOCITAT_ZS_NEMENIT = 'nepocitatzakladnisazbuzakazat';
    const SETTINGS_CASY_AUTOMATICKY = 'casyslcestyautomaticky';
    const SETTINGS_POVINNY_UCEL = 'povinnyucelslcesty';
    const SETTINGS_TYP_ROZDELENI_PRO_STAT = 'statslcesty_podletypurozdeleni';
    const SETTINGS_VYCHOZI_SAZBA_KAPESNEHO = 'vychozisazbakapesnehosc';     // nacita se podle nastaveni aplikace
    const SETTINGS_TYP_VYPOCTU = 'typvypoctucn';
    const SETTINGS_KONTROLA_DOKUMENTU = 'kontroladokumentunvv';             // 0=nekontrolovat, 1=jen kdyz je NVV, 2=dokument musi byt vzdy
    const SETTINGS_EDITACE_SCHVALENYCH = 'editaceschvalenychsc';
    const SETTINGS_EDITACE_V_UZAVRENEM = 'editacesluzebnichvuzavrenem';
    const SETTINGS_VKLADANI_POCET_DNU = 'editacesluzebnichpocetdnu';        // poznamka: funkcnost zmenena - kontroluje se pri vkladani, ne pri editaci
    const SETTINGS_KONTROLOVAT_PREKRYVANI = 'kontrolovatprekryvanisc';
    const SETTINGS_NESCHVALOVAT_ZAMESTNANCEM = 'neschvalovatsluzebnizam';
    const SETTINGS_EDITACE_SCHVALENE_NADRIZENYM = 'editacesc_schvalnad';    // jestli je mozne editovat SC schvalenou nadrizenym
    const SETTINGS_POSLOUPNOST_SCHVALOVANI = 'sc_posloupnostschvalovani';
    const SETTINGS_KOMPAKTNI_RADKY = 'kompaktniradkysc';
    const SETTINGS_CASY_PODLE_RADKU = 'casyscpodleradku';
    const SETTINGS_ZNAMENKO_UCTOVANI = 'znamenkouctovanisc';
    const SETTINGS_ZADAVAT_DATUM_KURZU =  'zadavatdatumkurzusc';
    const SETTINGS_PRENASET_DO_DOCHAZKY =  'prenasetsluzebnicesty';
    const SETTINGS_TYP_ROZDELENI = 'typrozdelenisc';
    const SETTINGS_ROZDELENI_POUZE_VLASTNI = 'rozdelenisc_pouzevlastni';
    const SETTINGS_ROZDELENI_POVINNE = 'rozdeleniscpovinne';
    const SETTINGS_ROZDELENI_VYCHOZI = 'rozdelenisc_vychozihodnota';
    const SETTINGS_TYP_ROZDELENI2 = 'typrozdelenisc2';
    const SETTINGS_TYP_ROZDELENI2Z = 'typrozdelenisc2_zahranicni';
    const SETTINGS_ROZDELENI2_POUZE_VLASTNI = 'rozdelenisc2_pouzevlastni';
    const SETTINGS_MENIT_SAZBU_KAPESNEHO = 'sazbakapesnehoslcest';
    const SETTINGS_ROZDELENI_POVOLIT_VYBER = 'rozdelenisc_povolitprirazeni';
    const SETTINGS_VYPOCET_VZDALENOSTI = 'povolitvypocetvzdalenosti';
    const SETTINGS_ZADAVAT_DPH = 'zadavatdph_sc';
    const SETTINGS_STAT = 'statslcesty';

    var $id;

    /** @var PracovniPomer */
    var $f_PP;

    var $datumCasOd;
    /** @var string     Pro formular */
    var $datumOdStr;
    var $casOdStr;

    var $datumCasDo;
    var $datumDoStr;
    var $casDoStr;

    var $zahranicni;
    var $datumKurzu;
    var $datumKurzuStr;
    var $sazbaKapesneho;
    var $sazbaKapesnehoStr;

    var $prenasetDoDochazky;
    var $typPrenosuDoDochazky;
    var $pocatekKonecPodleRadku;

    var $popis;
    var $poznamka;

    var $uzivatelska1;
    var $uzivatelska2;
    var $uzivatelska3;
    var $uzivatelska4;
    var $uzivatelska5;

    var $ucel;
    var $ucelPopis; // lookup

    var $cilovyStat;
    var $cilovyStatPopis;
    /** @var string     vychozi stat - urcuje legislativu */
    var $f_stat;

    var $mistoZacatku;
    /** @var string     lookup podle mistoZacatku */
    var $mistoZacatkuPopis;
    /** @var string     zadano uzivatelem */
    var $popisZacatku;

    var $cil;
    /** @var string     lookup podle pole cil */
    var $cilPopis;
    /** @var string     popis zadany uzivatelem */
    var $popisCile;
    /** @var string     prujezdni mista */
    var $pres;

    var $kodTypuRozdeleni;
    var $rozdeleni;

    var $kodTypuRozdeleni2;
    var $rozdeleni2;

    var $dopravniProstredek;
    var $registracniZnacka;
    var $popisVozidla;
    var $spotrebaVozidla;
    var $phmVozidla;

    /** @var Spolucestujici[] Seznam spolucestujicich */
    var $spolucestujici;

    var $schvalenoZamestnancem;
    var $sz_uzivatel;
    var $sz_datumcas;
    var $schvalenoNadrizenym;
    var $sn_uzivatel;
    var $sn_datumcas;
    var $schvalenoNadrizenym2;
    var $sn2_uzivatel;
    var $sn2_datumcas;
    var $schvalenoNadrizenym3;
    var $sn3_uzivatel;
    var $sn3_datumcas;
    var $schvalenoNadrizenym4;
    var $sn4_uzivatel;
    var $sn4_datumcas;

    var $schvaleni1;
    var $schvalil1;
    var $schvaleni2;
    var $schvalil2;
    var $schvaleni3;
    var $schvalil3;
    var $schvaleni4;
    var $schvalil4;
    var $datumVraceni1;
    var $datumVraceni2;
    var $datumVraceni3;
    //var $datumVraceni4;
    var $poznamkaKVraceni;

    /** @var float */
    var $vyuctovani;
    /** @var string */
    var $menaVyuctovani;
    /** @var int        Priznak, ze je zauctovana */
    var $zauctovano;
    var $zauctovanoStr;

    /** @var int        Ciselnik zpusobu vyplaceni */
    var $vyplatitHotove;
    /** @var string     Z tabulky S_ZpusobVyuctovaniSC */
    var $vyplatitHotoveStr;
    /** @var string */
    var $menaProVyplaceni;

    var $pozadovanaZaloha;
    var $pozadovanaZalohaHotovost;
    var $pozadovanaZalohaHotovostMena;
    var $pozadovanaZalohaUcet;
    var $pozadovanaZalohaUcetMena;
    var $pozadovanaZalohaIBAN;
    var $pozadovanaZalohaSWIFT;

    var $cisloUctuProZalohu;
    var $menaUctuProZalohu;

    var $urceneDenniStravne;
    var $urceneDenniStravneMena;
    var $urceneKapesne;
    var $urceneKapesneMena;

    var $historieSchvalovani;

    /** @var bool       Priznak, jestli je mozne s.cestu editovat. Pri inicializaci se nastavuje na false! */
    var $lzeEditovat;
    var $lzeEditovatHlavicku;

    /** @var PredpokladanyVydajSC[] */
    var $predpokladaneNaklady;



    function __construct()
    {
        $this ->lzeEditovat = false;
    }


    /**
     * @param Database $database
     * @param int $id
     * @return array|bool|false|null
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    private static function vratHlavicku($database, $id)
    {
        $id = (int)$id;
        //if (!$id) return false;

        $params = $database ->createParameters();

        $sql =
            'SELECT RC, OsobniCislo, DatumNastupu, ' . // 0..2
            '    DatumCasOd, DatumCasDo, Stat.Popis, ' . // 3,4,5
            '    SluzebniCesta.Popis, ' . $database ->castWideMemo('SluzebniCesta.Poznamka') . ', ' . // 6,7
            '    MistoSluzebniCesty.Popis, UcelSluzebniCesty.Popis, ' . // 8,9
            '    Schvaleni1, Schvalil1, Schvaleni2, Schvalil2, Schvaleni3, Schvalil3, ' . // 10..15
            '    SluzebniCesta.Id, MZ.Popis, ' . // 16,17
            '    VlastniVozidlo, PopisMistaZacatku, PopisCile, ' . // 18..20
            '    RadekUctovaniSCesty.Castka, ' . // 21
            $database ->castWideMemo('HistorieSchvalovani') . ', SluzebniCesta.Pres, ' . // 22,23
            '    SluzebniCesta.Zauctovano, SluzebniCesta.Zauctovano_Uzivatel, SluzebniCesta.Zauctovano_DatumCas, ' . // 24-26
            '    SluzebniCesta.Uzivatelska1, SluzebniCesta.Uzivatelska2, SluzebniCesta.Uzivatelska3, SluzebniCesta.Uzivatelska4, SluzebniCesta.Uzivatelska5, ' . // 27-31
            '    SluzebniCesta.SchvalenoZamestnancem, SluzebniCesta.SZ_Uzivatel, SluzebniCesta.SZ_DatumCas, ' . //32..34
            '    SluzebniCesta.SchvalenoNadrizenym, SluzebniCesta.SN_Uzivatel, SluzebniCesta.SN_DatumCas, ' . // 35..37
            '    SluzebniCesta.DatumKurzu, SluzebniCesta.SazbaKapesneho, SluzebniCesta.PrenasetDoDochazky, SluzebniCesta.TypPrenosuDoDochazky,  ' . // 38..41
            '    SluzebniCesta.Ucel, SluzebniCesta.MistoZacatku, SluzebniCesta.Cil, SluzebniCesta.DopravniProstredek, SluzebniCesta.CilovyStat, '. // 42..46
            // rozdeleni
            'SluzebniCesta.Zahranicni, SluzebniCesta.KodTypuRozdeleni, SluzebniCesta.Rozdeleni, ' . // 47..49
            'SluzebniCesta.KodTypuRozdeleni2, SluzebniCesta.Rozdeleni2, ' . // 50..51
            // ucetni zalezitosti
            '    SluzebniCesta.PozadovanaZaloha, SluzebniCesta.PozadovanaZalohaHotovost, SluzebniCesta.PozadovanaZalohaHotMena, ' . // 52..54
            '    SluzebniCesta.PozadovanaZalohaUcet, SluzebniCesta.PozadovanaZalohaUcetMena, SluzebniCesta.CisloUctuProZalohu, SluzebniCesta.MenaUctuProZalohu, ' . // 55..58
            '    SluzebniCesta.UrceneDenniStravne, SluzebniCesta.UrceneDenniStravneMena, SluzebniCesta.UrceneKapesne, SluzebniCesta.UrceneKapesneMena, ' . // 59..62
            '    SluzebniCesta.Stat, ' . // 63 - vychozi stat (pro legislativu)
            '    SluzebniCesta.SchvalenoNadrizenym2, SluzebniCesta.SN2_Uzivatel, SluzebniCesta.SN2_DatumCas,' . // 64..66
            '    Schvaleni4, Schvalil4, SluzebniCesta.RegistracniZnacka, Vozidlo.Popis,' . // 67..70
            '    SluzebniCesta.DatumVraceni1, SluzebniCesta.DatumVraceni2, SluzebniCesta.DatumVraceni3, SluzebniCesta.DatumVraceni4,' . // 71..74  ? ma datumvraceni4 smysl?
            '    SluzebniCesta.SchvalenoNadrizenym3, SluzebniCesta.SN3_Uzivatel, SluzebniCesta.SN3_DatumCas,' . // 75..77
            '    SluzebniCesta.SchvalenoNadrizenym4, SluzebniCesta.SN4_Uzivatel, SluzebniCesta.SN4_DatumCas,' . // 78..80
            '    SluzebniCesta.PoznamkaKVraceni,' . // 81
            '    Vozidlo.DruhPHM, Vozidlo.Spotreba,' . // 82,83
            '    SluzebniCesta.PocatekKonecPodleRadku, RadekUctovaniSCesty.KodMeny,' . // 84,85
            '    SluzebniCesta.IBANUctuProZalohu, SluzebniCesta.SWIFTUctuProZalohu,' . // 86,87
            '    SluzebniCesta.VyplatitHotove, ZV.Popis, SluzebniCesta.ManaProVyplaceni' . // 88..90

            ' FROM SluzebniCesta LEFT OUTER JOIN Stat' .
            '            ON Stat.Kod = SluzebniCesta.CilovyStat' .
            '          LEFT OUTER JOIN MistoSluzebniCesty' .
            '          ON MistoSluzebniCesty.Kod = SluzebniCesta.Cil' .
            '        LEFT OUTER JOIN MistoSluzebniCesty MZ' .
            '        ON MZ.Kod = SluzebniCesta.MistoZacatku' .
            '      LEFT OUTER JOIN UcelSluzebniCesty' .
            '      ON UcelSluzebniCesty.Kod = SluzebniCesta.Ucel' .
            '    LEFT OUTER JOIN RadekUctovaniSCesty' .
            '    ON RadekUctovaniSCesty.SluzebniCesta_Id = SluzebniCesta.Id' .
            '    AND RadekUctovaniSCesty.Typ = 3' .
            '  LEFT OUTER JOIN Vozidlo' .
            '  ON Vozidlo.RegistracniZnacka = SluzebniCesta.RegistracniZnacka' .
            '  LEFT OUTER JOIN S_ZpusobVyuctovaniSC ZV' .
            '  ON ZV.Kod = SluzebniCesta.VyplatitHotove' .

            ' WHERE SluzebniCesta.Id = ' . $params ->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Sluzebni cesta se zadanym ID neexistuje, nebo byla smazana!');
        }

        $cursor ->close();

        return $row;
    }


    /**
     * @param Database $database
     * @param array $row
     * @throws \Exception
     */
    private function nactiHlavicku($database, $row)
    {
        $this ->f_PP = new PracovniPomer();
        if (!$this ->f_PP ->nactiPPPodleRC($database, $row[0], $row[1], $database ->stringToTimestamp($row[2]))) {
            throw new \Exception('Nenalezen pracovni pomer ke sluzebni ceste!');
        }

        $this ->f_PP ->vyberNactenyPP($database, null);

        $this ->id = $row[16];

        $this ->datumCasOd = $database ->stringToTimestamp($row[3]);
        $this -> datumOdStr = date('j.n.Y', $this ->datumCasOd);
        $this -> casOdStr = date('G:i', $this ->datumCasOd);

        $this ->datumCasDo = $database ->stringToTimestamp($row[4]);
        $this -> datumDoStr = date('j.n.Y', $this ->datumCasDo);
        $this -> casDoStr = date('G:i', $this ->datumCasDo);


        $this ->popis = $row[6];
        $this ->poznamka = $row[7];
        $this ->uzivatelska1 = $row[27];
        $this ->uzivatelska2 = $row[28];
        $this ->uzivatelska3 = $row[29];
        $this ->uzivatelska4 = $row[30];
        $this ->uzivatelska5 = $row[31];

        $this ->ucel = $row[42];
        $this ->ucelPopis = $row[9]; // lookup

        $this ->cilovyStat = $row[46]; // lookup
        $this ->cilovyStatPopis = $row[5]; // lookup
        $this ->mistoZacatku = $row[43];
        $this ->mistoZacatkuPopis = $row[17]; // lookup
        $this ->popisZacatku = $row[19]; // zadano uzivatelem
        $this ->cil = $row[44];
        $this ->cilPopis = $row[8];
        $this ->popisCile = $row[20]; // popis zadany uzivatelem
        $this ->pres = $row[23];

        $this ->dopravniProstredek = $row[45];
        $this ->registracniZnacka = $row[69];
        $this ->popisVozidla = $row[70];
        $this ->phmVozidla = $row[82];
        $this ->spotrebaVozidla = $row[83];

        $this ->kodTypuRozdeleni = $row[48];
        $this ->rozdeleni = $row[49];

        $this ->kodTypuRozdeleni2 = $row[50];
        $this ->rozdeleni2 = $row[51];

        $this ->schvalenoZamestnancem = $row[32];
        $this ->sz_uzivatel = $row[33];
        $this ->sz_datumcas = $database ->stringToTimestamp($row[34]);

        $this ->schvalenoNadrizenym = $row[35];
        $this ->sn_uzivatel = $row[36];
        $this ->sn_datumcas = $database ->stringToTimestamp($row[37]);

        $this ->schvalenoNadrizenym2 = $row[64];
        $this ->sn2_uzivatel = $row[65];
        $this ->sn2_datumcas = $database ->stringToTimestamp($row[66]);

        $this ->schvalenoNadrizenym3 = $row[75];
        $this ->sn3_uzivatel = $row[76];
        $this ->sn3_datumcas = $database ->stringToTimestamp($row[77]);

        $this ->schvalenoNadrizenym4 = $row[78];
        $this ->sn4_uzivatel = $row[79];
        $this ->sn4_datumcas = $database ->stringToTimestamp($row[80]);

        $this ->schvaleni1 = $database ->stringToTimestamp($row[10]);
        $this ->schvalil1 = $row[11];
        $this ->schvaleni2 = $database ->stringToTimestamp($row[12]);
        $this ->schvalil2 = $row[13];
        $this ->schvaleni3 = $database ->stringToTimestamp($row[14]);
        $this ->schvalil3 = $row[15];
        $this ->schvaleni4 = $database ->stringToTimestamp($row[67]);
        $this ->schvalil4 = $row[68];
        $this ->datumVraceni1 = $database ->stringToTimestamp($row[71]);
        $this ->datumVraceni2 = $database ->stringToTimestamp($row[72]);
        $this ->datumVraceni3 = $database ->stringToTimestamp($row[73]);
        // datumVraceni4?
        $this ->poznamkaKVraceni = $row[81];

        $this ->vyuctovani = $row[21];
        $this ->menaVyuctovani = $row[85];

        $this ->zauctovano = $row[24];
        $this ->zauctovanoStr = $this ->zauctovano
            ? \Ron\ronText('SC_zauctovana') . ' (' . DBUtils::getFormatedValue($database, 'finedatetime', $row[26]) . ', ' . $row[25] . ')'
            : '';

        $this ->historieSchvalovani = $row[22];

        $this ->zahranicni = $row[47];
        $this ->datumKurzu = $database ->stringToTimestamp($row[38]);
        $this ->datumKurzuStr = $this ->datumKurzu ? date('j.n.Y', $this ->datumKurzu) : '';

        $this ->sazbaKapesneho = (float)$row[39];
        $this ->sazbaKapesnehoStr = $this ->sazbaKapesneho ? $this ->sazbaKapesneho : '';

        $this ->prenasetDoDochazky = (int)$row[40];
        $this ->typPrenosuDoDochazky = (int)$row[41];
        $this ->pocatekKonecPodleRadku = (int)$row[84];

        $this ->pozadovanaZaloha = (int)$row[52];
        $this ->pozadovanaZalohaHotovost = $row[53];
        $this ->pozadovanaZalohaHotovostMena = $row[54];
        $this ->pozadovanaZalohaUcet = $row[55];
        $this ->pozadovanaZalohaUcetMena = $row[56];
        $this ->pozadovanaZalohaIBAN = $row[86];
        $this ->pozadovanaZalohaSWIFT = $row[87];

        $this ->cisloUctuProZalohu = $row[57];
        $this ->menaUctuProZalohu = $row[58];

        $this ->urceneDenniStravne = $row[59];
        $this ->urceneDenniStravneMena = $row[60];
        $this ->urceneKapesne = $row[61];
        $this ->urceneKapesneMena = $row[62];

        $this ->f_stat = $row[63]; // vychozi stat (pro legislativu)

        $this ->vyplatitHotove = $row[88];
        $this ->vyplatitHotoveStr = $row[89];
        $this ->menaProVyplaceni = $row[90];

        $this ->predpokladaneNaklady = PredpokladanyVydajSC::nactiPredpokladaneVydaje($database, $this ->id);
        $this ->spolucestujici = Spolucestujici::nactiSpolucestujici($database, $this ->id);
    }

    /**
     * @param Database $database
     * @param int $id
     * @return SluzebniCesta
     * @throws \Exception
     */
    public static function nactiSluzebniCestu($database, $id)
    {
        $row = self::vratHlavicku($database, $id);

        $sluzebniCesta = new SluzebniCesta();
        $sluzebniCesta ->nactiHlavicku($database, $row);
        Prava::nastavObdobiProPrava($sluzebniCesta ->datumCasOd, $sluzebniCesta ->datumCasDo);

        return $sluzebniCesta;
    }


    /**
     * Vraci true, pokud je SC schvalena zadanym levelem nebo vyssim
     * @param $minimalniLevel
     * @return bool
     */
    function jeSchvalena($minimalniLevel = 1)
    {
        if ($minimalniLevel > 4) return false;

        if ($this ->schvaleni4) {
            return true;
        }

        if ($this ->schvaleni3) {
            return $minimalniLevel <= 3;
        }
        if ($this ->schvaleni2) {
            return $minimalniLevel <= 2;
        }
        if ($this ->schvaleni1) {
            return $minimalniLevel <= 1;
        }

        return false;
    }


    function loadDefaultValues($database)
    {
        $this->predpokladaneNaklady = PredpokladanyVydajSC::nactiPredpokladaneVydaje($database, 0);
        $this->spolucestujici = [];

        $this->zahranicni = RonPortalSettings::getIntValue('sc_zahranicni');
        $this->mistoZacatku = RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK);
        $this->datumCasOd = time();
        $this->datumCasDo = time();
        $this->datumOdStr = date('j.n.Y', time());
        $this->datumDoStr = $this->datumOdStr;
    }




    function jsouVyplnenePovinne($database)
    {
        if (RonPortalSettings::getIntValue(self::SETTINGS_POVINNY_UCEL) && !$this ->ucel) {
            return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_ucel_sc'), 'Ucel');
        }

        if (!$this ->cil && $this ->ucel && SluzebniCesta::jePovinnyCilProUcel($database, $this ->ucel)) {
            return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_cil_sc'), 'Cil');
        }

        return true;
    }


    /**
     * Vraci true, pokud ma uzivatel pravo na zobrazeni sluzebni cesty
     * @return boolean
     */
    function lzeZobrazitSluzebniCestu()
    {
        Prava::nastavObdobiProPrava($this ->datumCasOd, $this ->datumCasDo);

        $prava = Prava::vratPrava('prohlizeni');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('sluzebnicesty');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_prohl');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_ucet');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_nadriz');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_schval1');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_schval2');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_schval3');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        $prava = Prava::vratPrava('slcesty_zauct');
        if ($this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return true;
        }

        return false;
    }


    /**
     * Vraci true, pokud je sluzebni cesta schvalena nejvyssi urovni, na kterou ma uzivatel pravo, nebo vyssi urovni
     * @return bool
     */
    function jeSchvalenaProUzivatele()
    {
        if ( !RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENYCH, 1) ) {
            // nelze editovat schvalene - jakakoliv uroven zamyka editaci
            return $this ->schvaleni1 || $this ->schvaleni2 || $this ->schvaleni3 || $this ->schvaleni4;
        }

        if ($this ->schvaleni4) {
            // nejvyssi uroven zamyka vzdy
            return true;
        }

        if ($this ->f_PP ->lzeZobrazitPP(Prava::vratPrava('slcesty_schval4'), $this ->datumCasOd, $this ->datumCasDo)) {
            // ma pravo na uroven 4
            return false;
        }

        if ($this ->schvaleni3) {
            return true;
        }

        if ($this ->f_PP ->lzeZobrazitPP(Prava::vratPrava('slcesty_schval3'), $this ->datumCasOd, $this ->datumCasDo)) {
            // ma pravo na uroven 3
            return false;
        }

        if ($this ->schvaleni2) {
            return true;
        }

        if ($this ->f_PP ->lzeZobrazitPP(Prava::vratPrava('slcesty_schval2'), $this ->datumCasOd, $this ->datumCasDo)) {
            // ma pravo na uroven 2
            return false;
        }

        return $this ->schvaleni1;
    }


    function lzeEditovat($database, $raise)
    {
        // kontrola zauctovani - nelze editovat zauctovane
        if ($this ->zauctovano) {
            if ($raise) {
                throw new \Exception( \Ron\ronText('Sluzebka_je_jiz_zauctovana') );
            }

            return false;
        }

        // kontrola schvaleni (nelze editovat schvalene, ale pokud mam pravo na vyssi uroven, tak muzu)
        $schvalena = $this ->jeSchvalenaProUzivatele();

        if ($schvalena) {
            if ($raise) {
                throw new \Exception( \Ron\ronText('Sluzebka_je_jiz_schvalena') );
            }

            return false;
        }

        // kontrola uzavreneho obdobi
        if (!RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_V_UZAVRENEM) && Obdobi::jeObdobiUzavreno($database, 'ADS', $this ->datumCasOd, $this ->datumCasDo)) {
            if ($raise) {
                throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
            }

            return false;
        }

        // kontrola prav
        $prava = Prava::vratPrava('sluzebnicesty');
        if (!$this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            if ($raise) {
                throw new \Exception( \Ron\ronText('Neni_pravo_editovat_sc') );
            }

            return false;
        }

        return true;
    }


    /**
     * POZOR! Kontroluje jen blokovani SC schvalene nadrizenym! Je nutne zaroven zkontrolovat lzeEditovat()
     * @param Database $database
     * @return bool
     */
    function lzeEditovatHlavicku($database)
    {
        if (RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENE_NADRIZENYM, 1)) {
            // editace sluzebni cesty schvalene nadrizenym je povolena
            return true;
        }

        if (!$this ->schvalenoNadrizenym) {
            // jeste neni schvalena nadrizenym
            return true;
        }

        // sc schvalena nadrizenym - muzu editovat jen kdyz jsem nadrizeny
        $prava = Prava::vratPrava('slcesty_nadriz');
        return $this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo);
    }


    /**
     * @param Database $database
     * @param bool $showMessages   Pokud je true, zobrazi se chybove hlasky (neni pravo a podobne)
     * @return bool
     */
    function lzeEditovatSluzebniCestu($database, $showMessages)
    {
        try {
            $result = $this->lzeEditovat($database, $showMessages);
        } catch (\Exception $e) {
            return MessageManager::addErrorMessage($e ->getMessage());
        }

        return $result;
    }


    /**
     * @param Database $database
     * @throws DBException
     * @throws \Exception
     */
    function zkontrolujPrekryvaniSC($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT DatumCasOd, DatumCasDo' .
            ' FROM SluzebniCesta' .
            ' WHERE RC = ' . $params ->addParameter($this ->f_PP ->rc) .
            ' AND Id <> ' . $params ->addParameter((int)$this ->id ?: 0);

        if (RonPortalSettings::getIntValue(self::SETTINGS_CASY_AUTOMATICKY)  &&  $this ->datumCasOd == $this ->datumCasDo) {
            // casy se nastavuji automaticky a sluzebni cesta jeste nema nastaven cas
            // => nekontroluje se prekryvani s SC, ktere zacinaji nebo konci ve stejny den
            //    protoze po nastaveni casu se prekryvat nemusi
            $sql .=
                ' AND DatumCasOd < ' . $params->addParameter($database ->dateToParam($this->datumCasDo)) .
                ' AND DatumCasDo >= ' . $params->addParameter($database ->dateToParam(DateUtils::incDay($this->datumCasOd, 1)));

        } else {
            $sql .=
                ' AND DatumCasOd < ' . $params->addParameter($database ->timestampToParam($this->datumCasDo)) .
                ' AND DatumCasDo > ' . $params->addParameter($database ->timestampToParam($this->datumCasOd));
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new \Exception( \Ron\ronText('Chyba_prekryvani_SC') );
        }
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool
     */
    function lzeVlozitSluzebniCestu($database)
    {
        // kontrola uzavreneho obdobi
        if (!RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_V_UZAVRENEM)  &&  Obdobi::jeObdobiUzavreno($database, 'ADS', $this ->datumCasOd, $this ->datumCasDo)) {
            return MessageManager::addErrorMessage( \Ron\ronText('Obdobi_uzavreno'));
        }

        $pocetDnu = RonPortalSettings::getIntValue(self::SETTINGS_VKLADANI_POCET_DNU);
        if ( $pocetDnu  &&  DateUtils::incDay($this ->datumCasOd, $pocetDnu) < DateUtils::startOfTheDay(time()) ) {
            return MessageManager::addErrorMessage(sprintf( \Ron\ronText('Nelze_vkladat_stare_sc'), $pocetDnu));
        }

        // kontrola prav
        $prava = Prava::vratPrava('sluzebnicesty');
        if (!$this ->f_PP ->lzeZobrazitPP($prava, $this ->datumCasOd, $this ->datumCasDo)) {
            return MessageManager::addErrorMessage( \Ron\ronText('Neni_pravo_editovat_sc') );
        }

        // kontrola prekryvani
        if (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLOVAT_PREKRYVANI)) {
            try {
                $this->zkontrolujPrekryvaniSC($database);
            } catch (\Exception $e) {
                return MessageManager::addErrorMessage($e);
            }
        }

        return true;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws DBException
     */
    function obsahujeNahrady($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT CisloRadku FROM RadekUctovaniSCesty, TypRUctovaniSlCesty ' .
            ' WHERE SluzebniCesta_ID = ' . $params ->addParameter((int)$this ->id) .
            '   AND TypRUctovaniSlCesty.Kod = RadekUctovaniSCesty.Typ ' .
            '   AND TypRUctovaniSlCesty.NahradaZaVlastniVozidlo = 1';

        $cursor = $database ->getSqlResult($sql, $params);
        $result = !!$cursor ->fetch();
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws DBException
     */
    function obsahujeDokument($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Id FROM DokumentSluzebniCesty ' .
            ' WHERE SluzebniCesta_ID = ' . $params ->addParameter((int)$this ->id);

        $cursor = $database ->getSqlResult($sql, $params);
        $result = !!$cursor ->fetch();
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     */
    function kontrolaPredVyuctovanim($database)
    {
        if (!$this ->jsouVyplnenePovinne($database)) {
            return false;
        }

        if (!RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_DOKUMENTU)) {
            // kontrola vlozeni dokumentu se neprovadi
            return true;
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_DOKUMENTU) == 1  &&  !$this ->obsahujeNahrady($database)) {
            // neni radek NVV => dokument nemusi byt prilozen
            return true;
        }

        if ($this ->obsahujeDokument($database)) {
            return true; // kontrola OK, muze se vyuctovat
        }

        return MessageManager::addErrorText('Nelze_vyuctovat_chybi_dokument');
    }



    function zobrazZkracenouHlavicku()
    {
        require('sluzebnicesty/view_zkracenahlavicka.php');
    }



    function GetMistaProVzdalenost() {
        $startstr = null;
        $cilstr = null;

        switch (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI)) {
            case 1 : // prednostne podle lookupu
                $startstr = $this ->mistoZacatkuPopis; // lookup
                if (!$startstr) $startstr = $this ->popisZacatku;

                $cilstr = $this ->cilPopis; // lookup
                if (!$cilstr) $cilstr = $this ->popisCile;
                break;

            case 2 : // prednostne podle popisu
                $startstr = $this ->popisZacatku;
                if (!$startstr) $startstr = $this ->mistoZacatkuPopis; // lookup

                $cilstr = $this ->popisCile;
                if (!$cilstr) $cilstr = $this ->cilPopis; // lookup
                break;
        }

        if ($startstr  &&  $cilstr) {
            $result = array();

            $result[] = $startstr;
            $result[] = $cilstr;
            $result[] = $this ->pres;

            return $result;
        }

        return null;
    }


    /**
     * @param Database $database
     * @param string $ucel
     * @return bool
     */
    static function jePovinnyCilProUcel($database, $ucel)
    {
        if (!$ucel) return false;

        $params = $database ->createParameters();
        return DBUtils::getSqlValue($database,
            'SELECT PovinnyCil FROM UcelSluzebniCesty WHERE Kod = ' . $params ->addParameter($ucel),
            $params);
    }

    /**
     * @param Database $database
     * @param string $stat
     * @return null
     */
    static function sazbaKapesnehoProStat($database, $stat)
    {
        if (!$stat) return null;

        $params = $database ->createParameters();
        $sql = 'SELECT ImplicitniSazbaKapesneho FROM Stat WHERE Kod = ' . $params ->addParameter($stat);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if (!$row) return null;

        return $row[0];
    }


    /**
     * @return bool  Nacte predpokladane naklady zadane uzivatelem. Seznam predpokladanych nakladu jiz musi byt nacten!
     */
    function loadPostPredpokladaneNaklady()
    {
        $result = true;

        foreach ($this ->predpokladaneNaklady as $index => $predpokladanyvydaj) {
            if ( isset($_POST['PV_PocetDnu_' . $predpokladanyvydaj ->typ])  ||  isset($_POST['PV_Castka_' . $predpokladanyvydaj ->typ]) ) {
                $pocetstr = $_POST['PV_PocetDnu_' . $predpokladanyvydaj ->typ];
                $predpokladanyvydaj ->pocetDnu = (trim($pocetstr) === '') ? null : (int)$pocetstr;

                $predpokladanyvydaj ->castkaStr = $_POST['PV_Castka_' . $predpokladanyvydaj ->typ];
                $predpokladanyvydaj ->castka = (trim($predpokladanyvydaj ->castkaStr) === '')
                    ? null
                    : StringUtils::stringToFloat($predpokladanyvydaj ->castkaStr);

                $menastr = $_POST['PV_Mena_' . $predpokladanyvydaj ->typ];
                $predpokladanyvydaj ->kodMeny = (trim($menastr) === '') ? null : $menastr;

            } else {
                $predpokladanyvydaj ->castka = null;
                $predpokladanyvydaj ->castkaStr = '';
                $predpokladanyvydaj ->pocetDnu = null;
                $predpokladanyvydaj ->kodMeny = null;
            }
        }

        return $result;
    }


    function zkontroluZadaneUdaje($database)
    {
        // Provede kontroly
        // kontrola formatu casu
        //$datumod = null;
        try {
            $datumod = DateUtils::dateStringToTimestamp($this->datumOdStr);
        } catch (\Exception $e) {
            return MessageManager::addErrorMessage(\Ron\ronText('Bad_date_format') . " (" . \Ron\ronText('Zacatek') . ")", 'DatumCasOd');
        }

        if ( !RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) ) {
            try {
                $datumod = DateUtils::timeStringToTimestamp($this->casOdStr, $datumod);
            } catch (\Exception $e) {
                return MessageManager::addErrorMessage(\Ron\ronText('Bad_time_format') . ' (' . \Ron\ronText('Zacatek') . ')');
            }
        }

        $pocetDnu = RonPortalSettings::getIntValue(self::SETTINGS_VKLADANI_POCET_DNU);
        if ( $pocetDnu  &&   DateUtils::startOfTheDay($datumod) != DateUtils::startOfTheDay($this ->datumCasOd) ) {
            // je zakazano vkladat stare SC - zkontroluje, jestli uzivatel neposunul datum (z nebo do obdobi pred ...)
            $pomDatum = $datumod;
            if ($this ->datumCasOd  &&  $this ->datumCasOd < $pomDatum) $pomDatum = $this ->datumCasOd;

            if (DateUtils::incDay($pomDatum, $pocetDnu) < DateUtils::startOfTheDay(time()) ) {
                return MessageManager::addErrorMessage( sprintf(\Ron\ronText('Nelze_editovat_zacatek_sc'), $pocetDnu), 'DatumCasOd' );
            }
        }

        $this ->datumCasOd = $datumod;

        try {
            $datumdo = DateUtils::dateStringToTimestamp($this->datumDoStr);
        } catch (\Exception $e) {
            return MessageManager::addErrorMessage(\Ron\ronText('Bad_date_format') . ' (' . \Ron\ronText('Konec') . ")", 'DatumCasDo');
        }

        if ( !RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) ) {
            try {
                $datumdo = DateUtils::timeStringToTimestamp($this->casDoStr, $datumdo);
            } catch (\Exception $e) {
                return MessageManager::addErrorMessage(\Ron\ronText('Bad_time_format') . ' (' . \Ron\ronText('Konec') . ')');
            }
        }
        $this ->datumCasDo = $datumdo;

        if ($datumdo < $datumod) {
            return MessageManager::addErrorMessage( \Ron\ronText('Pocatecni_datum_vetsi_nez_koncove'), 'DatumCasDo');
        }

        if (!RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_V_UZAVRENEM)  &&  Obdobi::jeObdobiUzavreno($database, 'ADS', $datumod, $datumdo)) {
            return MessageManager::addErrorMessage( \Ron\ronText('Obdobi_uzavreno') );
        }

        Prava::nastavObdobiProPrava($datumod, $datumdo);

        if (!$this ->f_PP ->vyberNactenyPP($database, Prava::vratPrava('sluzebnicesty'))) {
            return MessageManager::getInstance() ->addError( \Ron\ronText('Nemate_pravo') );
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DATUM_KURZU, 1) ) {
            if ($this ->datumKurzuStr) {
                try {
                    $this->datumKurzu = DateUtils::dateStringToTimestamp($this->datumKurzuStr);
                } catch (\Exception $e) {
                    return MessageManager::addErrorMessage( \Ron\ronText('Bad_date_format'), 'DatumKurzu');
                }
            }
            else {
                $this ->datumKurzu = null;
            }
        }

        //return MessageManager::addErrorMessage( 'Povinny: ' . RonPortalSettings::getIntValue('povinnypopisslcesty'));
        if (RonPortalSettings::getIntValue('povinnypopisslcesty')  &&  !$this ->popis) {
            return MessageManager::addErrorMessage( \Ron\ronText('Neni_zadan_popis_sc'), 'Popis' );
        }

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI_POVINNE) && !isset($this ->rozdeleni)) {
            return MessageManager::addErrorMessage( \Ron\ronText('Neni_zadano_rozdeleni_sc'), 'Rozdeleni' );
        }

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_MENIT_SAZBU_KAPESNEHO)) {
            if ($this ->sazbaKapesnehoStr != '') {
                if (!StringUtils::isFloat($this ->sazbaKapesnehoStr)) {
                    return MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota_pro') . \Ron\ronText('Sazba_kapesneho'), 'SazbaKapesneho');
                }
                else $this ->sazbaKapesneho = StringUtils::stringToFloat($this ->sazbaKapesnehoStr);
            }
            else $this ->sazbaKapesneho = 0;
        }
        else if (RonPortalSettings::getIntValue('sazbakapesnehopodlestatu')) {
            // nacte sazbu kapesneho podle ciloveho statu

        }
        else $this ->sazbaKapesneho = 0;

        // zkontroluje, jestli jsou vyplnene vsechny povinne udaje
        if (!SluzebniCestyUtils::getVychoziStatOsoby($database, $this ->f_PP, $this ->datumCasOd)) {
            if (!$this ->f_stat) return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_stat'), 'Stat');
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_POVINNY_UCEL) && !$this ->ucel) {
            return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_ucel_sc'), 'Ucel');
        }

        if (!$this ->cil) {
            // cil neni zadan - zjisti, jestli je pro zadany ucel povinny a pripadne zobrazi chybu
            if (SluzebniCesta::jePovinnyCilProUcel($database, $this ->ucel)) {
                return MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_cil_sc'), 'Cil');
            }
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLOVAT_PREKRYVANI)) {
            try {
                $this->zkontrolujPrekryvaniSC($database);
            } catch (\Exception $e) {
                return MessageManager::addErrorMessage($e, 'DatumCasOd');
            }
        }

        return true;
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool
     * @throws DBException
     */
    function loadPost($database)
    {
        Prava::nastavObdobiProPrava(time(), time());

        // Nacte data z POST
        $this ->id = $_POST['Id'];

        $this ->zahranicni = isset($_POST['Zahranicni']) && $_POST['Zahranicni'];

        $this ->datumOdStr = $_POST['DatumCasOd'];
        if (!RonPortalSettings::getIntValue(self::SETTINGS_CASY_AUTOMATICKY)) {
            $this ->casOdStr = $_POST['DatumCasOd_time'];
        }

        $this ->datumDoStr = $_POST['DatumCasDo'];
        if (!RonPortalSettings::getIntValue(self::SETTINGS_CASY_AUTOMATICKY)) {
            $this ->casDoStr = $_POST['DatumCasDo_time'];
        }


        if (!RonPortalSettings::getValue(self::SETTINGS_STAT)) {
            $this ->f_stat = $_POST['Stat'];
        }

        if (  RonPortalSettings::getIntValue(self::SETTINGS_ZACATEK_NEMENIT) ) {
            $this ->mistoZacatku = RonPortalSettings::getValue(self::SETTINGS_ZACATEK);
        }
        else {
            $this ->mistoZacatku = $_POST['MistoZacatku'];
        }

        $this ->popisZacatku = $_POST['PopisMistaZacatku'];
        $this ->cil = $_POST['Cil'];
        $this ->popisCile = $_POST['PopisCile'];
        $this ->pres = $_POST['Pres'];

        if ($this ->zahranicni) {
            $this ->cilovyStat = $_POST['CilovyStat'];
        }
        else {
            // neni zahranicni => nastavi se stejne jako vychozi
            $this ->cilovyStat = $this ->f_stat;
        }

        if (trim($this ->cilovyStat) === '') {
            $this ->cilovyStat = null;
        } else {
            $params = $database ->createParameters();
            $this ->cilovyStatPopis = DBUtils::getSqlValue($database,
                'SELECT Popis FROM Stat WHERE Kod = ' . $params ->addParameter($this ->cilovyStat),
                $params);
        }

        if ( RonPortalSettings::getIntValue(self::SETTINGS_TYP_ROZDELENI) ) {
            $this ->rozdeleni = $_POST['Rozdeleni'];
            if ( isset($this ->rozdeleni)  &&  $this ->rozdeleni != '') {
                $this ->kodTypuRozdeleni = RonPortalSettings::getIntValue(self::SETTINGS_TYP_ROZDELENI);
            }
            else {
                $this ->rozdeleni = null;
                $this ->kodTypuRozdeleni = null;
            }
        }

        // rozdeleni 2 - muze byt ruzny typ rozdeleni pro tuzemske a zahranicni!
        if (  RonPortalSettings::getIntValue(self::SETTINGS_TYP_ROZDELENI2)  &&  (!$this ->zahranicni  ||  RonPortalSettings::getIntValue(self::SETTINGS_TYP_ROZDELENI2) == RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z))  ) {
            // sluzebni cesta neni zahranicni, nebo se pouziva pro zahranicni SC stejny typ rozdeleni jako pro tuzemske
            $this ->rozdeleni2 = $_POST['Rozdeleni2'] != '' ? $_POST['Rozdeleni2'] : null;
            if ( isset($this ->rozdeleni2)  &&  $this ->rozdeleni2 != '') {
                $this ->kodTypuRozdeleni2 = RonPortalSettings::getIntValue(self::SETTINGS_TYP_ROZDELENI2);
            }
            else {
                $this ->rozdeleni2 = null;
                $this ->kodTypuRozdeleni2 = null;
            }
        }
        else if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z)  &&  $this ->zahranicni ) {
            // sluzebni cesta je zahranicni a pouziva se jiny typ rozdeleni
            $this ->rozdeleni2 = $_POST['Rozdeleni2Z'];
            if ( isset($this ->rozdeleni2)  &&  $this ->rozdeleni2 != '') {
                $this ->kodTypuRozdeleni2 = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z);
            }
            else {
                $this ->rozdeleni2 = null;
                $this ->kodTypuRozdeleni2 = null;
            }
        }

        $this ->ucel = $_POST['Ucel'];

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DATUM_KURZU, 1) ) {
            $this ->datumKurzuStr = isset($_POST['DatumKurzu']) ? $_POST['DatumKurzu'] : '';
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) == 1  &&  isset($_POST['TypPrenosuDoDochazky'])) {
            $this ->prenasetDoDochazky = (int)$_POST['TypPrenosuDoDochazky'] >= 0;
            if ($this ->prenasetDoDochazky) {
                $this ->typPrenosuDoDochazky = (int)$_POST['TypPrenosuDoDochazky'];
            }
        }

        if ( RonPortalSettings::getIntValue(self::SETTINGS_CASY_PODLE_RADKU) == -1) {
            // nastaveni vypoctu podle radku urcuje primo uzivatel
            $this ->pocatekKonecPodleRadku = (isset($_POST['PocatekKonecPodleRadku'])  &&  $_POST['PocatekKonecPodleRadku']) ? 1 : 0;
        }

        $this ->sazbaKapesnehoStr = isset($_POST['SazbaKapesneho'])
            ? $_POST['SazbaKapesneho']
            : '';

        $this ->popis = $_POST['Popis'];

        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA1)  &&  isset($_POST['Uzivatelska1'])) {
            $this ->uzivatelska1 = $_POST['Uzivatelska1'];
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA2)  &&  isset($_POST['Uzivatelska2'])) {
            $this ->uzivatelska2 = $_POST['Uzivatelska2'];
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA3)  &&  isset($_POST['Uzivatelska3'])) {
            $this ->uzivatelska3 = $_POST['Uzivatelska3'];
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA4)  &&  isset($_POST['Uzivatelska4'])) {
            $this ->uzivatelska4 = $_POST['Uzivatelska4'];
        }
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_UZIVATELSKA5)  &&  isset($_POST['Uzivatelska5'])) {
            $this ->uzivatelska5 = $_POST['Uzivatelska5'];
        }

        if (isset($_POST['DopravniProstredek'])) {
            $this ->dopravniProstredek = $_POST['DopravniProstredek'];
        }
        if (isset($_POST['RegistracniZnacka'])) {
            $this ->registracniZnacka = $_POST['RegistracniZnacka'] != ''
                ? $_POST['RegistracniZnacka']
                : null;
        }

        $this ->poznamka = $_POST['Poznamka'];

        $this ->vyplatitHotove = (int)$_POST['VyplatitHotove'];
        $this ->menaProVyplaceni = ($_POST['MenaProVyplaceni'] != '')
            ? $_POST['MenaProVyplaceni']
            : null;

        $this ->pozadovanaZaloha = (isset($_POST['PozadovanaZaloha'])  &&  $_POST['PozadovanaZaloha']) ? 1 : 0;
        $this ->pozadovanaZalohaHotovost = (trim($_POST['PozadovanaZalohaHotovost']) != '')
            ? StringUtils::stringToFloat($_POST['PozadovanaZalohaHotovost'])
            : null;

        $this ->pozadovanaZalohaHotovostMena = ($_POST['PozadovanaZalohaHotMena'] != '')
            ? $_POST['PozadovanaZalohaHotMena']
            : null;

        $this ->pozadovanaZalohaUcet = (trim($_POST['PozadovanaZalohaUcet']) != '')
            ? StringUtils::stringToFloat($_POST['PozadovanaZalohaUcet'])
            : null;

        $this ->pozadovanaZalohaUcetMena = ($_POST['PozadovanaZalohaUcetMena'] != '')
            ? $_POST['PozadovanaZalohaUcetMena']
            : null;

        $this ->pozadovanaZalohaIBAN = ($_POST['PozadovanaZalohaIBAN'] != '')
            ? $_POST['PozadovanaZalohaIBAN']
            : null;

        $this ->pozadovanaZalohaSWIFT = ($_POST['PozadovanaZalohaSWIFT'] != '')
            ? $_POST['PozadovanaZalohaSWIFT']
            : null;

        $this ->cisloUctuProZalohu = ($_POST['CisloUctuProZalohu'] != '')
            ? $_POST['CisloUctuProZalohu']
            : null;

        $this ->menaUctuProZalohu = ($_POST['MenaUctuProZalohu'] != '')
            ? $_POST['MenaUctuProZalohu']
            : null;

        $this ->urceneDenniStravne = null;
        $this ->urceneDenniStravneMena = null;
        try {
            $poma = SluzebniCestyUtils::vratMaximalniDenniStravneProStat($database, $this->datumCasOd, $this->cilovyStat, $this->f_stat);

            if ($poma) {
                $this ->urceneDenniStravne = $poma['sazba']; // (trim($_POST['UrceneDenniStravne']) != '') ? stringtofloat($_POST['UrceneDenniStravne']) : null;
                $this ->urceneDenniStravneMena = $poma['mena'];
            }
        } catch (DBException $e) {
            throw $e;
        } catch (\Exception $e) {
            // nedela nic (sazba nenalezena)
        }

        $this ->urceneKapesne = (trim($_POST['UrceneKapesne']) != '')
            ? StringUtils::stringToFloat($_POST['UrceneKapesne'])
            : null;
        $this ->urceneKapesneMena = ($_POST['UrceneKapesneMena'] != null)
            ? $_POST['UrceneKapesneMena']
            : null;

        $this ->loadPostPredpokladaneNaklady();

        $this ->f_PP = new PracovniPomer();

        if (!isset($_POST['osoba']) || !$_POST['osoba']) {
            return MessageManager::addErrorMessage( \Ron\ronText('Nutno_vybrat_osobu'), 'osoba');
        }

        if (!$this ->f_PP ->nactiPPPodleId($database, $_POST['osoba'])) {
            return MessageManager::addErrorMessage('Chyba! Zadany pracovni pomer nebyl nalezen!', 'osoba');
        }

        return $this ->zkontroluZadaneUdaje($database);
    }


    /**
     * @param Database $database
     * @return RonPortalDBRecord
     */
    function createDBRecord($database)
    {
        $rec = new RonPortalDBRecord($database, 'SluzebniCesta');
        $rec ->logDescription = StringUtils::fromUTF8('Služební cesty');

        $rec ->addField('ID', 'int', true, 0, $this ->id); // readonly = true

        $rec ->addField('RC', '', false, 15, $this ->f_PP ->rc);
        $rec ->addField('OsobniCislo', '', false, 15, $this ->f_PP ->osobniCislo);
        $rec ->addField('DatumNastupu', 'date', false, 0, $this ->f_PP ->datumNastupu);

        $rec ->addField('', '', true, 0, '   ' . $this ->f_PP ->jmenoPrijmeni); // jen pro zalogovani

        $rec ->addField('DatumCasOd', 'datetime', false, 0, $this ->datumCasOd);
        $rec ->addField('DatumCasDo', 'datetime', false, 0, $this ->datumCasDo);

        $rec ->addField('MistoZacatku', '', false, 30, $this ->mistoZacatku);
        $rec ->addField('PopisMistaZacatku', '', false, 160, $this ->popisZacatku);
        $rec ->addField('Cil', '', false, 30, $this ->cil);
        $rec ->addField('PopisCile', '', false, 160, $this ->popisCile);
        $rec ->addField('Pres', '', false, 160, $this ->pres);
        $rec ->addField('CilovyStat', '', false, 10, $this ->cilovyStat);

        $rec ->addField('Ucel', '', false, 30, $this ->ucel);

        // zjisti, jestli je prednastaven stat (bud hodnotou, nebo podle rozdeleni)
        $vychoziStatOsoby = SluzebniCestyUtils::getVychoziStatOsoby($database, $this ->f_PP, $this ->datumCasOd);
        if (!$vychoziStatOsoby) {
            $rec->addField('Stat', '', false, 10, $this->f_stat);
        } else {
            // stat meni jen v pripade, ze neni prednastaven globalne a neni podle rozdeleni
            $rec->addField('Stat', '', true, 10, $vychoziStatOsoby);
        }

        $rec ->addField('Zahranicni', 'int', false, 0, (int)$this ->zahranicni);
        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DATUM_KURZU, 1) ) {
            $rec ->addField('DatumKurzu', 'date', false, 0, $this ->datumKurzu);
        }

        $rec ->addField('Popis', '', false, 160, $this ->popis);
        //$rec ->addField('VlastniVozidlo', 'int', false, 0, $_POST['VlastniVozidlo'] ? 1 : 0);
        $rec ->addField('DopravniProstredek', '', false, 30, $this ->dopravniProstredek);
        $rec ->addField('RegistracniZnacka', '', false, 30, $this ->registracniZnacka);

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI)) {
            $rec ->addField('KodTypuRozdeleni', 'int', false, 0, $this ->kodTypuRozdeleni);
            $rec ->addField('Rozdeleni', '', false, 30, $this ->rozdeleni);
        }

        if ( (!$this ->zahranicni  &&  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2)) ||
            ( $this ->zahranicni  &&  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z)) )  {
            $rec ->addField('KodTypuRozdeleni2', 'int', false, 0, $this ->kodTypuRozdeleni2);
            $rec ->addField('Rozdeleni2', '', false, 30, $this ->rozdeleni2);
        }

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) == 1) // podle volby uzivatele
        {
            $rec ->addField('PrenasetDoDochazky', 'int', false, 0, $this ->prenasetDoDochazky ? 1 : 0);
            if ($this ->typPrenosuDoDochazky >= 0) {
                $rec ->addField('TypPrenosuDoDochazky', 'int', false, 0, (int)$this ->typPrenosuDoDochazky);
            }
        } else {
            // pouze read only pro vkladani
            $rec ->addField('PrenasetDoDochazky', 'int', true, 0, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) >= 0 ? 1 : 0);
            if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) >= 2 ) {
                $rec ->addField('TypPrenosuDoDochazky', 'int', true, 0, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) - 2);
            }
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_CASY_PODLE_RADKU) == -1) {
            // nastaveni "pocatek konec podle radku" provede uzivatel
            $rec ->addField('PocatekKonecPodleRadku', 'int', false, 0, $this ->pocatekKonecPodleRadku);
        } else {
            $rec ->addField('PocatekKonecPodleRadku', 'int', true, 0, RonPortalSettings::getIntValue(self::SETTINGS_CASY_PODLE_RADKU));
        }

        // read-only - pouze pro vkladani
        $rec ->addField('TypVypoctuCN', 'int', true, 0, RonPortalSettings::getIntValue(self::SETTINGS_TYP_VYPOCTU));

        if (RonPortalSettings::getIntValue(self::SETTINGS_MENIT_SAZBU_KAPESNEHO)) {
            // Sazbu kapesneho urcuje uzivatel sam.
            $rec ->addField('SazbaKapesneho', 'float', false, 0, $this ->sazbaKapesneho);
        } else if (RonPortalSettings::getIntValue('sazbakapesnehopodlestatu')) {
            // Sazba kapesneho podle statu
            $sazba = $this ->cilovyStat ? self::sazbaKapesnehoProStat($database, $this ->cilovyStat) : 0;
            $rec ->addField('SazbaKapesneho', 'float', false, 0, $sazba);
        } else {
            $rec ->addField('SazbaKapesneho', 'float', true, 0, RonPortalSettings::getFloatValue(self::SETTINGS_VYCHOZI_SAZBA_KAPESNEHO));
        }

        if (RonPortalSettings::getValue(self::SETTINGS_UZIVATELSKA1)) {
            $rec ->addField('Uzivatelska1', '', false, 100, $this ->uzivatelska1);
        }
        if (RonPortalSettings::getValue(self::SETTINGS_UZIVATELSKA2)) {
            $rec ->addField('Uzivatelska2', '', false, 100, $this ->uzivatelska2);
        }
        if (RonPortalSettings::getValue(self::SETTINGS_UZIVATELSKA3)) {
            $rec ->addField('Uzivatelska3', '', false, 100, $this ->uzivatelska3);
        }
        if (RonPortalSettings::getValue(self::SETTINGS_UZIVATELSKA4)) {
            $rec ->addField('Uzivatelska4', '', false, 100, $this ->uzivatelska4);
        }
        if (RonPortalSettings::getValue(self::SETTINGS_UZIVATELSKA5)) {
            $rec ->addField('Uzivatelska5', '', false, 100, $this ->uzivatelska5);
        }

        $rec ->addIntField('VyplatitHotove', false, $this ->vyplatitHotove);
        $rec ->addStringField('ManaProVyplaceni', false, 10, $this ->menaProVyplaceni);

        $rec ->addField('PozadovanaZaloha', 'int', false, 0, $this ->pozadovanaZaloha);
        $rec ->addField('PozadovanaZalohaHotovost', 'float', false, 0, $this ->pozadovanaZalohaHotovost);
        $rec ->addField('PozadovanaZalohaHotMena', '', false, 10, $this ->pozadovanaZalohaHotovostMena);
        $rec ->addField('PozadovanaZalohaUcet', 'float', false, 0, $this ->pozadovanaZalohaUcet);
        $rec ->addField('PozadovanaZalohaUcetMena', '', false, 10, $this ->pozadovanaZalohaUcetMena);

        $rec ->addField('CisloUctuProZalohu', '', false, 30, $this ->cisloUctuProZalohu);
        $rec ->addField('MenaUctuProZalohu', '', false, 10, $this ->menaUctuProZalohu);
        $rec ->addField('IBANUctuProZalohu', '', false, 40, $this ->pozadovanaZalohaIBAN);
        $rec ->addField('SWIFTUctuProZalohu', '', false, 40, $this ->pozadovanaZalohaSWIFT);

        $rec ->addField('UrceneDenniStravne', 'float', false, 0, $this ->urceneDenniStravne);
        $rec ->addField('UrceneDenniStravneMena', '', false, 10, $this ->urceneDenniStravneMena);
        $rec ->addField('UrceneKapesne', 'float', false, 0, $this ->urceneKapesne);
        $rec ->addField('UrceneKapesneMena', '', false, 10, $this ->urceneKapesneMena);

        if (RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENE_NADRIZENYM, 1) == 2) {
            // pri editaci se vymaze schvaleni nadrizeneho (pokud needituje primo nadrizeny)
            $prava = Prava::vratPrava('slcesty_nadriz');
            if (!$prava ->lzeZobrazitPP($this ->f_PP ->idPP)) {
                $f = $rec ->addField('SchvalenoNadrizenym', 'int', false, 0, 0);
                $f ->updateLog = true;
            }
        }

        $rec ->addField('Poznamka', 'memo', false, 0, $this ->poznamka);

        return $rec;
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool
     * @throws \Ron\DB\DBException
     */
    function upravCasyPodleRadkuSC($database)
    {
        $sql =
            'SELECT MIN(DatumCasOd), MAX(DatumCasOd) FROM RadekSluzebniCesty '.
            ' WHERE SluzebniCesta_ID = ' . (int)$this ->id;

        $cursor = $database ->getSqlResult($sql);

        $row = $cursor ->fetch();
        if (!$row) {
            return true;
        }

        if (!$row[0]) {
            return true;
        }

        $casod = $database ->stringToTimestamp($row[0]);
        if (DateUtils::startOfTheDay($casod) == DateUtils::startOfTheDay($this ->datumCasOd)) {
            $this ->datumCasOd = $casod;
        }

        $casdo = $database ->stringToTimestamp($row[1]);
        if (DateUtils::startOfTheDay($casdo) == DateUtils::startOfTheDay($this ->datumCasDo)) {
            $this ->datumCasDo = $casdo;
        }

        return true;
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool|int
     */
    function updateCasy($database)
    {
        $this ->upravCasyPodleRadkuSC($database);

        $rec = new RonPortalDBRecord($database, 'SluzebniCesta');
        $rec ->logDescription = StringUtils::fromUTF8('Služební cesty');

        $rec ->addField('', '', true, 0, StringUtils::fromUTF8('Automatické nastavení času počátku a konce podle řádků'));
        $rec ->addField('ID', 'int', true, 0, $this ->id); // readonly = true

        $rec ->addField('RC', '', true, 15, $this ->f_PP ->rc);
        $rec ->addField('OsobniCislo', '', true, 15, $this ->f_PP ->osobniCislo);
        $rec ->addField('DatumNastupu', 'date', true, 0, $this ->f_PP ->datumNastupu);

        $rec ->addField('DatumCasOd', 'datetime', false, 0, $this ->datumCasOd);
        $rec ->addField('DatumCasDo', 'datetime', false, 0, $this ->datumCasDo);

        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($this ->id), $params);

        return $rec ->loadAndUpdateRecord();
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool Ulozi (update/insert/delete zadane predpokladane naklady
     */
    function zapisPredpokladaneNaklady($database) {
        $result = true;

        foreach ($this ->predpokladaneNaklady as $index => $predpokladanyvydaj) {
            if ( !$predpokladanyvydaj ->store($database, $this ->id) ) $result = false;
        }

        return $result;
    }


    /**
     * @param \Ron\DB\Database $database
     * @return bool
     * @throws \Exception
     */
    function updateRecord($database)
    {
        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) ) {
            // cas pocatku a konce nastavi automaticky podle existujich radku
            $this ->upravCasyPodleRadkuSC($database);
        }

        $rec = $this ->createDBRecord($database);

        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter((int)$this ->id), $params);

        if ( !$rec ->loadRecord() ) {
            throw new \Exception('Chyba! Nelze nacist puvodni zaznam sluzebni cesty!');
        }

        $changed = $rec ->isChanged(); // pozor - pokracuje dale i kdyz neni zmena - mohly se zmenit predpokladane naklady

        if ($changed) {
            if (!$rec ->executeXmlEvents(RonPortalXMLEvents::SC_BEFORE_POST, 'update')) {
                return false;
            }
        }

        $rec ->updateRecord();

        $result = $this ->zapisPredpokladaneNaklady($database);

        if ($changed) {
            if (!$rec->executeXmlEvents(RonPortalXMLEvents::SC_AFTER_POST, 'update')) {
                return false;
            }
        }

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function insert($database)
    {
        $rec = $this ->createDBRecord($database);

        $field_id = $rec ->findFieldByName('ID');
        $field_id ->setNewValue( SQLUtils::newSerialNumber($database, 'SluzebniCesta', 'ID') );

        if (!$rec ->executeXmlEvents(RonPortalXMLEvents::SC_BEFORE_POST, 'insert')) {
            return false;
        }

        $rec ->insertRecord();

        $this ->id = $field_id ->value;

        MessageManager::getInstance() ->addMessage( \Ron\ronText('Vlozeni_sc_OK') );

        try {
            $this->zapisPredpokladaneNaklady($database);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        try {
            $rec->executeXmlEvents(RonPortalXMLEvents::SC_AFTER_POST, 'insert'); // v pripade chyby nevraci false, protoze uzivatel by pak vkladal SC vicekrat
        } catch (\Exception $e) {
            if (!MessageManager::getInstance() ->isError()) {
                //MessageManager::addErrorMessage($e->getMessage());
            }
        }

        return true;
    }


    /**
     * @param Database $database
     * @param int  $idOriginal
     * @param int  $idNew
     * @param bool $radky
     * @param bool $uctovani
     * @return bool
     */
    public static function kopirujObsah($database, $idOriginal, $idNew, $radky, $uctovani)
    {
        $datumCasPuvodni = null;
        $datumCasNovy = null;

        $idOriginal = (int)$idOriginal;
        $idNew = (int)$idNew;

        // zjisti pocatek puvodni a nove sluzebni cesty
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, DatumCasOd FROM SluzebniCesta ' .
            ' WHERE Id = ' . $params ->addParameter($idOriginal) .
            ' OR Id = ' . $params ->addParameter($idNew);

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            if ($row[0] == $idOriginal) {
                $datumCasPuvodni = $database ->stringToTimestamp($row[1]);
            }
            if ($row[0] == $idNew) {
                $datumCasNovy = $database ->stringToTimestamp($row[1]);
            }
        }
        $cursor ->close();

        if (!$datumCasPuvodni) {
            return MessageManager::addErrorMessage("Chyba pri kopirovani radku - nelze nacist puvodni zaznam (Id=$idOriginal)");
        }

        if (!$datumCasNovy) {
            return MessageManager::addErrorMessage("Chyba pri kopirovani radku - nelze nacist novy záznam (Id=$idNew)");
        }

        if ($radky) {
            if ( RonPortalSettings::getIntValue(self::SETTINGS_CASY_AUTOMATICKY) ) {
                SluzebniCestyUtils::kopirujRadky($database, $idOriginal, $idNew, DateUtils::startOfTheDay($datumCasNovy) - DateUtils::startOfTheDay($datumCasPuvodni));
            }
            else {
                SluzebniCestyUtils::kopirujRadky($database, $idOriginal, $idNew, $datumCasNovy - $datumCasPuvodni);
            }
        }

        if ($uctovani) {
            SluzebniCestyUtils::kopirujRadkyUctovani($database, $idOriginal, $idNew, DateUtils::startOfTheDay($datumCasNovy) - DateUtils::startOfTheDay($datumCasPuvodni));
        }

        return true;
    }

}
