<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;

use Exception;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;
use Ron\StringUtils;
use RonPortal\RonPortalSession;

class PreparedReportOutputObject implements OutputObject
{
    /** @var string */
    private $reportKey;

    /** @var string */
    private $format;

    /** @var int minimalni platnost reportu (pocet minut) */
    const DEFAULT_VALIDITY = 60;

    /**
     * PreparedReportOutputObject constructor.
     * @param $reportKey
     * @param $format
     */
    public function __construct($reportKey, $format)
    {
        $this->reportKey = $reportKey;
        $this->format = $format;
    }


    /**
     * @throws Exception
     */
    public function output()
    {
        $preparedReports = RonPortalSession::getValue('PreparedReportsList');
        $preparedReportInfo = isset($preparedReports[$this ->reportKey])
            ? $preparedReports[$this ->reportKey]
            : null;

        if (!$preparedReportInfo) {
            throw new Exception('Neplatny_klic_reportu');
        }

        $server = ReportServer::createInstance();
        try {
//            $xml =
//                "  <printpreparedreport key='" . StringUtils::xmlTextConvert($this ->reportKey) ."' format='" . StringUtils::xmlTextConvert($this ->format ?: '') . "'/>\n";
            $xml =
                "  <printreport kod='" . StringUtils::xmlTextConvert($preparedReportInfo['reportName']) ."' format='" . StringUtils::xmlTextConvert($this ->format ?: '') . "'>\n" .
                $preparedReportInfo['variables'] .
                "  </printreport>\n";
            $server->sendXmlRequest($xml);

            HttpResponse::getInstance()->getHeaders()->addHeadersForContent('report.' . ReportServer::getFileExtension($this ->format), 0);
            HttpResponse::getInstance()->renderHeaders();

            $server->outputContent();

//            if ($this ->format) {
//                $server->sendMessage("FORMAT {$this ->format}\n");
//            }
//
//            $server ->sendMessage("PRINTPREPAREDREPORT {$this ->reportKey}\n");
//
//            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent('report.' . $this ->format, 0);
//            HttpResponse::getInstance() ->renderHeaders();
//
//            $server ->outputContent();
        } catch (Exception $e) {
            $server ->close();
            throw $e;
        }
    }


    /**
     * Prida do session informace o "prepared" reportu
     * @param $reportName
     * @param $variablesXml
     * @return string
     */
    public static function addPreparedReport($reportName, $variablesXml)
    {
        $preparedReports = RonPortalSession::getValue('PreparedReportsList');
        if (!$preparedReports) {
            $preparedReports = [];
        }

        $key = uniqid('', true);

        $newReportInfo = [
            'reportName' => $reportName,
            'variables' => $variablesXml,
            'validUntil' => time() + self::DEFAULT_VALIDITY * 60,
            'key' => $key
        ];

        $preparedReports[$key] = $newReportInfo;

        RonPortalSession::setValue('PreparedReportsList', $preparedReports);

        return $key;
    }
}
