<?php
/**
 * stanislav.elbl, 28.01.2019
 */

namespace RonPortal\XML\Classes;

use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\XML\XMLFormElement;


/**
 * implementace tagu <adsfilterform>
 * Class XMLElementADSFilterForm
 * @package Ron\RonPortal\XML
 */
class XMLElementADSFilterForm extends XMLFormElement
{
    /** @var RonPortalFilterForm */
    public $filterForm;


    /**
     * @throws DBException
     * @throws Exception
     */
    private function createFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->xmlForm->database, 'xml&record=' . $this->xmlForm->idSablony);
        if ($this ->getBoolAttribute('useget')) {
            $this ->filterForm ->setUseGet(true, ['menuid' => 'xml', 'record' => $this->xmlForm->idSablony]);
        }

        $autoSubmit = $this->getAttributeValue('autosubmit');
        $this->filterForm->f_autosubmit = (!isset($autoSubmit) || $autoSubmit === '')
            ? 1
            : self::getBoolValue($autoSubmit);

        // vnorene tagy adsfilterform_obdobi atd. vyhledaji tento element a nastavi filterForm pro zobrazeni
        $this->getElementsHTMLOutput();

        $this->filterForm->loadData();

        // doplni promenne do XML formulare
        if ($this->filterForm->showObdobi) {
            $this->xmlForm->nastavObdobiAplikace('OBDOBI', $this->filterForm->obdobiAplikace, RonPortalSession::obdobi());
        }

        if ($this->filterForm->f_showrozdeleni) {
            $this->xmlForm->nastavRozdeleni('ROZDELENI', $this->filterForm->f_typrozdeleni, $this->filterForm->kodRozdeleni);

            if ($this ->filterForm ->showVcetnePodrizenychRozdeleni) {
                $this->setVariableValue('ROZDELENI_vcetnepodrizenych', $this->filterForm->vcetnePodrizenychRozdeleni);
            }
        }

        if ($this->filterForm->f_showrozdeleni2) {
            $this->xmlForm->nastavRozdeleni2('ROZDELENI2', $this->filterForm->f_typrozdeleni2, $this->filterForm->kodRozdeleni2);

            if ($this ->filterForm ->showVcetnePodrizenychRozdeleni2) {
                $this->setVariableValue('ROZDELENI2_vcetnepodrizenych', $this->filterForm->vcetnePodrizenychRozdeleni2);
            }
        }

        if ($this->filterForm->showOsoby) {
            $this->xmlForm->nastavPracovniPomer('OSOBA',
                $this ->filterForm ->vsechnyOsoby ?
                    null :
                    $this->filterForm ->vybranaOsoba);
        }

        $this->setVariableValue('adsfilterformsubmit', isset($_POST['adsfilterform']) ? (int)$_POST['adsfilterform'] : 0);
    }


    /**
     * @throws DBException
     */
    public function getHTMLOutput()
    {
        $this ->createFilterForm();

        // zobrazeni filtru zajisti formular - vyrenderuje az zaroven s formularem
        $this ->xmlForm ->ronPortalFilterForm = $this ->filterForm;

        return '';
    }


    /**
     * Provadi se i v rezimu process, aby se spravne nastavily promenne
     * @throws DBException
     */
    public function process()
    {
        $this ->createFilterForm();
    }
}


/**
 * Implementace tagu <adsfilterform_...> pro nastaveni formulare (podrizene tagy <adsfilterform>)
 * Class XMLElementADSFilterFormPart
 * @package RonPortal\XML\Classes
 */
class XMLElementADSFilterFormPart extends XMLFormElement
{
    /** @var string Filtr pro seznam rozdeleni */
    private $sqlFiltr;

    public function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        switch ($this ->elementName) {
            case 'adsfilterform_rozdeleni' :
            case 'adsfilterform_rozdeleni2' :
            case 'adsfilterform_osoby' :
                $this ->sqlFiltr = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
                break;
        }
    }


    public function process()
    {
        // nastavi nejblizsi nadrazeny adsfilterform
        /* @var  XMLElementADSFilterForm $filterFormElement*/
        $filterFormElement = $this ->findParentByName('adsfilterform');
        if (!$filterFormElement) {
            return;
        }

        $sqlFiltr = trim($this ->nahradPromenneSQL($this ->sqlFiltr));

        switch ($this ->elementName) {
            case 'adsfilterform_obdobi' :
                $filterFormElement ->filterForm ->nastavObdobi(true,
                    $this ->nahradPromenne($this ->getAttributeValue('aplikace')),
                    self::getBoolValue($this ->getAttributeValue('jenotevrene')));
                break;

            case 'adsfilterform_rozdeleni' :
                $typRozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));

                $filterFormElement ->filterForm ->nastavRozdeleni(true,
                    $typRozdeleni, (int)$this ->getAttributeValue('pouzevlastni'),
                    $sqlFiltr,
                    $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
                $typPrava = $this ->nahradPromenne($this ->getAttributeValue('prava'));
                if ($typPrava) {
                    $filterFormElement ->filterForm ->f_pravarozdeleni = Prava::vratPrava($typPrava);
                }
                $filterFormElement->filterForm->showVcetnePodrizenychRozdeleni = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));
                break;

            case 'adsfilterform_rozdeleni2' :
                $typRozdeleni = (int)$this ->nahradPromenne($this ->getAttributeValue('typrozdeleni'));
                $filterFormElement ->filterForm ->nastavRozdeleni2(true,
                    $typRozdeleni, self::getBoolValue($this ->getAttributeValue('pouzevlastni')),
                    $sqlFiltr, $this ->getBoolAttribute('pouzitpopistypurozdeleni'));
                $filterFormElement->filterForm->showVcetnePodrizenychRozdeleni2 = self::getBoolValue($this ->getAttributeValue('showvcetnepodrizenych'));
                break;

            case 'adsfilterform_osoby' :
                $prava = $this ->nahradPromenne($this ->getAttributeValue('prava'));

                $datumOd = $this->getAttributeValue('datumod');
                $datumDo = $this->getAttributeValue('datumdo');

                $datumOd = (int)$this->nahradPromenne($datumOd);
                $datumDo = (int)$this->nahradPromenne($datumDo);

                if ($prava) {
                    $prava = Prava::vratPrava($prava);
                    $filterFormElement ->filterForm ->nastavOsoby(true, $prava, $datumOd, $datumDo);
                } else {
                    $predavaciproces = $this ->getAttributeValue('predavaciproces');
                    if ( !$predavaciproces ) {
                        $predavaciproces = $this ->xmlForm -> predavaciProces;
                    }
                    $filterFormElement ->filterForm ->nastavOsobyDleProcesu(true, $predavaciproces, $this ->xmlForm -> typRozdeleniProcesu);
                }

                $filterFormElement->filterForm->osobyFiltr = $sqlFiltr;

                if (self::getBoolValue($this ->getAttributeValue('vsechnyosoby'))) {
                    $filterFormElement ->filterForm ->povolitVsechyOsoby = true;
                }
        }
    }


    public function getHTMLOutput()
    {
        $this ->process();

        return '';
    }
}

/*
//DOC
PAGE adsfilterform
<pre>
PRE
  <adsfilterform autosubmit="1" useget="0">
      <adsfilterform_obdobi aplikace="ADS" jenotevrene="0"/>
      <adsfilterform_rozdeleni typrozdeleni="kodtypu" pouzevlastni="0/1/2" prava="typprav" pouzitpopistypurozdeleni="0" showvcetnepodrizenych="0">
         nepovinny_sql_filtr_pro_vyber_rozdeleni
      </adsfilterform_rozdeleni>
      <adsfilterform_rozdeleni2 typrozdeleni="kodtypu" pouzevlastni="0/1/2" pouzitpopistypurozdeleni="0" >
         nepovinny_sql_filtr_pro_vyber_rozdeleni
      </adsfilterform_rozdeleni2>
      <adsfilterform_osoby prava="prohlizeni" vsechnyosoby="1" datumod="" datumdo=""/>
  </adsfilterform>
PREEND
</pre>
//END
*/