<?php
/**
 * stanislav.elbl, 04.10.2018
 */

namespace RonPortal\XML\Classes;

use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Editace\EditaceMesicnichHromadna;
use RonPortal\XML\XMLElementAttribute;
use RonPortal\XML\XMLFormElement;

class XMLElementEditaceMesicnich extends XMLFormElement
{
    /** @var EditaceMesicnichHromadna */
    var $editace;

    var $settingsArray;


    function loadXML($xmlReader)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini'); // obsahuje chybove hlasky

        $loader = new XmlLoader();
        $loader ->convertFromUtf8 = true;
        $this ->settingsArray = $loader ->readXmlElement($xmlReader);

        $this ->attributes = [];    // doplni standardni pole atributu

        if ($this ->settingsArray['#attr']) {
            foreach ($this->settingsArray['#attr'] as $name => $value) {
                $this ->attributes[] = new XMLElementAttribute(null, $name, $value);
            }
        }

        // TODO: vyresit parametr content!
    }


    private function getEditace()
    {
        if ($this ->editace) {
            return $this ->editace;
        }

        // nahrazeni promennych v atributech hlavniho tagu
        if ($this ->settingsArray['#attr']) {
            foreach ($this->settingsArray['#attr'] as $name => $attr) {
                $this->settingsArray['#attr'][$name] = $this ->nahradPromenne($this->settingsArray['#attr'][$name]);
            }
        }
        // nahrazeni promennych v atributech sloupcu
        if ($this ->settingsArray['#children']) {
            foreach ($this ->settingsArray['#children'] as $index => $sloupec) { // => puvodne bylo by reference - to ale nefunguje pro ProxyArray
                if ($sloupec['element']['#attr']) {
                    foreach ($sloupec['element']['#attr'] as $name => $attr) {
                        $sloupec['element']['#attr'][$name] = $this ->nahradPromenne($sloupec['element']['#attr'][$name]);
                    }
                }

                $this ->settingsArray['#children'][$index] = $sloupec; // nahrada za puvodni kod by reference
            }
        }

        $this ->editace = new EditaceMesicnichHromadna($this -> xmlForm -> database, $this ->xmlForm ->kodTypuRozdeleni, $this ->xmlForm ->rozdeleni);
        $this ->editace ->f_readonly = $this ->xmlForm->readOnly;

        $this ->editace ->loadSettings($this ->settingsArray);

        return $this ->editace;
    }

    /**
     * Pouziva se jeste ve starych formularich bez nastaveneho mode
     * @deprecated
     */
    function zapisData()
    {
        $editace = $this ->getEditace();
        $editace->nactiSeznamOsobPodlePost($this ->xmlForm->pocatekObdobi, !$editace ->kontrolovatPrava);

        $editace->nactiLimityRozdeleni();

        if (!$editace->jeChyba) {
            if ($editace->kontrolaObdobi()) {
                if ($editace->zkontrolujLimity()) {
                    $editace->zapisData();
                    if ($editace->jeChyba) $this->xmlForm ->addErrorMsg('&nbsp;');
                } else {
                    $this->xmlForm->addErrorMsg(\Ron\ronText('Prekrocen_limit_MP'));
                }
            } else {
                $this ->xmlForm->addErrorMsg( \Ron\ronText('Obdobi_uzavreno') );
            }
        } else {
            $this ->xmlForm->addErrorMsg( \Ron\ronText('Spatna_hodnota') );
        }
    }


    /**
     * Nastavi promenne formulare podle nacteneho seznamu osob
     * @param EditaceMesicnichHromadna $editace
     */
    function nastavPromenne($editace)
    {
        $name = $this ->getAttributeValue('name');
        if (!$name) return;

        $osoby = [];
        if ($editace ->osoby) {
            foreach ($editace ->osoby as $osoba) {
                $osoby[] = $osoba ->idPP;
            }
        }

        $delimiter = $this ->getAttributeValue('delimiter', ',');

        $this ->setVariableValue($name, implode($delimiter, $osoby)); // retezec - seznam idpp oddelenych strednikem
        $this ->setVariableValue($name . '_pole', $osoby);
    }


    function getHTMLOutput()
    {
        $editace = $this ->getEditace();
        $editace->filtrOsoby = $this ->xmlForm->nahradPromenne($editace->filtrOsoby, 'sql');

        if ($this ->xmlForm->neukladat == 2  &&  $this ->xmlForm ->usePostValues) {
            // stary neukladaci formular
            $this ->zapisData();
        } else if ($this ->xmlForm->usePostValues) {
            // fill, process (pokud nastane chyba)
            $editace->nactiSeznamOsobXML($this ->xmlForm->kodTypuRozdeleni, $this ->xmlForm->rozdeleni, $this ->xmlForm->pocatekObdobi);
            $editace->nactiPost();
            //$this->editace->nactiSeznamOsobPodlePost($this ->xmlForm->pocatekObdobi, true); // povolitvse = true - lze editovat vsechny osoby

            $editace->zkontrolujLimity(); // aby se oznacily prekrocene hodnoty

            $editace->nactiLimityRozdeleni();
        } else {
            //$this ->editace -> f_vcetnepodrizenych = $vcetnepodrizenych;
            $editace->nactiSeznamOsobXML($this ->xmlForm->kodTypuRozdeleni, $this ->xmlForm->rozdeleni, $this ->xmlForm->pocatekObdobi);

            $editace->zkontrolujLimity(); // aby se oznacily prekrocene hodnoty
        }

        $this ->nastavPromenne($editace);

        return
            '<div>' .
            $editace->vratObsahFormulareH(false) . // tlacitkoulozit = false
            '</div>';
    }


    function writeXMLOutput($xmlWriter)
    {
        //$this ->zapisData();

        //if (!$this ->xmlForm ->neukladat) {
        throw new \Exception('Chyba - transformace tagu editacemesicnich neni implementovana!');

        // TODO: dodelat - parent nefunguje, protoze nejsou nacteny podrizene elementy
        parent::writeXMLOutput($xmlWriter);  // zkopiruje obsah do vystupu
        //}
    }


    function process()
    {
        $editace = $this ->getEditace();
        $editace->filtrOsoby = $this ->xmlForm->nahradPromenne($editace->filtrOsoby, 'sql');

        $editace->nactiSeznamOsobXML($this ->xmlForm->kodTypuRozdeleni, $this ->xmlForm->rozdeleni, $this ->xmlForm->pocatekObdobi);
        $editace->nactiPost();

        $this ->nastavPromenne($editace);

        if (!$editace->jeChyba) {
            if ($editace->kontrolaObdobi()) {
                if ($editace->zkontrolujLimity()) {
                    $editace->zapisData();
                    if ($editace->jeChyba) $this->xmlForm ->addErrorMsg('');
                } else {
                    $this->xmlForm->addErrorMsg(\Ron\ronText('Prekrocen_limit_MP'));
                }
            } else {
                $this ->xmlForm->addErrorMsg( \Ron\ronText('Obdobi_uzavreno') );
            }
        } else {
            $this ->xmlForm->addErrorMsg( \Ron\ronText('Spatna_hodnota') );
        }
    }
}
