<?php
/**
 * stanislav.elbl, 23.06.2021 17:45
 */

namespace RonPortal\XML\Classes;

use Ron\Http\HtmlHeaders;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\XML\XMLFormElementLoader;
use RonPortal\Zamestnanci\NovyZamestnanecForm;

class XMLElementNastupniFormular extends XMLFormElementLoader
{
    /** @var NovyZamestnanecForm */
    private static $form = null;

    /**
     * Vytvori formular
     * @return NovyZamestnanecForm
     * @throws \Exception
     */
    private function createForm()
    {
        TextManager::getInstance() ->loadStrings('zamestnanci.ini');
        HtmlHeaders::getInstance() ->addCss('css/person.css');

        $idPP = (int)$this ->nahradPromenne($this ->getAttributeValue('idpp'));
        $pp = $idPP
            ? PracovniPomer::nactiPP($this ->xmlForm ->database, $idPP)
            : '';

        $form = new NovyZamestnanecForm($this ->xmlForm ->database,
            $pp ? 'edit' : 'new',
            $pp ? $pp ->idPP : null,
            true);  // zobrazeno z XML = true

        $form ->title = ''; // zobrazuje se nadpis z XML formulare
        $form ->setNovyZamestnanecConfiguration($this ->content);

        return $form;
    }


    private function createVariables()
    {
        $name = $this ->getAttributeValue('name');

        if (!$name) {
            return;
        }

        $this ->setVariableValue($name . '_status', self::$form ->getStatus()); // 'rc' nebo 'all'
        $this ->setVariableValue($name . '_status_' . self::$form ->getStatus(), 1);
    }


    function getHTMLOutput()
    {
//        HtmlHeaders::getInstance()
//            ->addScript('script/vyroba.js')
//            ->addCss('css/dochazka.css');

        if (!self::$form) {
            self::$form = $this->createForm();
            self::$form ->handleAction();

            $this ->createVariables();
        }

        $this ->xmlForm ->viewObjects[] = self::$form;

        return $this ->xmlForm ->viewObjectPlaceholder;
    }


    function process()
    {
        self::$form = $this ->createForm();
        $result = self::$form ->handleAction();

        $this->createVariables();

        if (!$result) {
            $this ->xmlForm ->addErrorMsg(''); // aby pokracoval dale - formular jeste nebyl kompletne vyplnen
        } else {
            if (!$this ->xmlForm ->pp_RC  &&  self::$form ->getIdPP()) {
                // byl vlozen novy zaznam
                $this ->xmlForm ->nastavPracovniPomer('NASTUPNI_FORMULAR_PP', self::$form ->getIdPP());
            }
        }
    }
}

/*
//DOC
PAGE nastupniformular
<pre>
PRE
  <nastupniformular />
PREEND
</pre>

//END
*/
