<?php
/**
 * stanislav.elbl, 16.06.2021 15:48
 */

namespace RonPortal\XML\Classes;

use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\XML\XMLFormElementCommand;

class XMLElementRecord extends XMLFormElementCommand
{
    /** @var RonPortalDBRecord */
    var $dbRecord;

    function doCommand()
    {
        $logDescription = $this->getAttributeValue('logdescription');

        $this->dbRecord = new RonPortalDBRecord($this ->xmlForm ->database,
            $this->getAttributeValue('tablename'),
            $logDescription ?: 'XML ' . $this ->xmlForm ->idSablony);

        //parent::elementsEvent();
    }
}



class XMLElementRecordPart extends XMLFormElementCommand
{
    var $wherePart;
    var $valueText;

    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        switch ($this->elementName) {
            case 'recordwhere' :
                $this->wherePart = $this->getTextContent(false);
                break;

            case 'recordfield' :
                $this->valueText = $this->getAttributeValue('value');

                if (!isset($this ->valueText)) {
                    $this ->valueText = $this->getTextContent(false);
                } else {
                    $this ->valueText = StringUtils::fromUTF8($this ->valueText);
                }
        }
    }


    function doCommand()
    {
        /* @var  XMLElementRecord $recordElement */
        $recordElement = $this->findParentByName('record');

        if (!$recordElement) {
            MessageManager::addDebugMessage('Chyba v tagu ' . $this ->elementName . ': nenalezen nadrazeny tag record!');
            return;
        }

        switch ($this->elementName) {
            case 'recordfield' :
                $readonly = self::getBoolValue($this ->getAttributeValue('readonly'));
                $value = $this->nahradPromenne($this ->valueText);
                $name = $this->nahradPromenne($this ->getAttributeValue('name'));

                $field = $recordElement ->dbRecord ->addField(
                    $name,
                    $this ->getAttributeValue('format'),
                    $readonly,
                    (int)$this ->getAttributeValue('size'),
                    $value );

                // priznaky pro logovaci pole (napr. Update_UserName)
                $field ->updateLog = $this ->getAttributeValue('updatelog');

                break;

            case 'recordwhere' :
                $where = $this ->nahradPromenneSQL($this ->wherePart);
                $recordElement ->dbRecord ->where = $where;
                break;

            case 'loadrecord' :
                $varName = $this ->getAttributeValue('name');

                $this ->setVariableValue($varName, $recordElement ->dbRecord);

                $recordElement ->dbRecord ->loadRecord();
                foreach ( $recordElement ->dbRecord ->fields as $field ) {
                    $this ->setVariableValue($varName . '_' . ($field ->fieldName), $field ->originalValue);
                }
                break;

            case 'insertrecord' :
                $recordElement ->dbRecord ->insertRecord();
                break;

            case 'updateorinsertrecord' :
                $recordElement ->dbRecord ->updateOrInsert();
                break;

            case 'deleterecord' :
                $recordElement ->dbRecord ->deleteRecord();
                break;

            case 'updaterecord' :
            case 'editrecord' :
                if ($recordElement->dbRecord->loadRecord()) {
                    $recordElement->dbRecord->updateRecord();
                } else {
                    // TODO: vyhodit vyjimku?
                    MessageManager::addErrorMessage('Chyba pri nacitani puvodniho zaznamu!');
                }
        }
    }
}


/*
//DOC
PAGE record
<pre>
PRE
  <record tablename="JMENOTABULKY" logdescription="Popis pro zapis do logu">
    <recordfield name="JMENOPOLE1" format="FORMAT_POLE1" size="DELKA_TEXTOVEHO_POLE" readonly="0" updatelog="0" value="HODNOTA">JINAK_ZADANA_HODNOTA</recordfield>
    ...

    <recordwhere>SQL podminka pro SELECT a UPDATE</recordwhere>

    <insertrecord />
    <updaterecord />
    <updateorinsertrecord />

    <deleterecord />

    <loadrecord name="JMENO_PROMENNE" />

  </record>
PREEND
</pre>
//END
*/
