<?php
/**
 * stanislav.elbl, 26.07.2021 10:57
 */

namespace RonPortal\XML;

use Ron\DB\SqlStatement;
use Ron\Http\App;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PredavaciProcesyUtils;

class SablonaXMLFormulareBrowser extends RonPortalBrowser
{
    protected $procesyUzivatele; // seznam urovni predavani aktualniho uzivatele

    function __construct($database, $xmlMenuId)
    {
        parent::__construct($database, $xmlMenuId);

        $this->title = TextManager::_('Sablony_xml_formularu');

        $this->browserUrl = '?menuid=seznamxmlformularu';
        $this->code = 'browsersablonaxml';

        $this->sqlFromPart = 'SablonaXMLFormulare';
        $params = $database ->createParameters();
        $where =
            '(PlatnostOd IS NULL  OR  PlatnostOd <= ' . $params ->addParameter($database->dateToParam(time())) . ')' .
            ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($database->dateToParam(time())) . ')';

        $this->browserUrl = '?menuid=seznamxmlformularu&';
        if ($xmlMenuId) {
            $this->browserUrl .= 'xmlmenuid=' . $xmlMenuId . '&';
            $where .=
                ' AND MenuId = ' . $params->addParameter($xmlMenuId);
        }
        $this ->sqlWherePart = new SqlStatement($where, $params);

        $this->sqlSelectPart = 'Id, PredavaciProces, TypPrava, Popis, PlatnostDo';

        $this->addField()
            ->setSqlFieldName('Id')->setSqlFieldIndex(0)->setOrdering(true)->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('PredavaciProces')->setSqlFieldIndex(1)
            ->setCode('col-proces')
            ->setVisible(App::getDebugMode())
            ->setHeader('Proces') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('Popis') ->setSqlFieldIndex(3) ->setHeaderText('Nazev_xml_sablony')
            ->setOrdering(true)
            ->setCode('col-popis');

        $this->addField()
            ->setSqlFieldName('PlatnostDo') ->setSqlFieldIndex(4)->setHeaderText('Platnost_do')
            ->setFormat('date') ->setOrdering(true)
            -> setCode('col-platnostdo');

        $this->addField()
            ->setType('text') ->setText(\Ron\ronText('Vyplnit_xml_formular')) ->setLink('?menuid=xml') ->setOrdering(true)
            ->setCode('col-vyplnit'); // .($xmlmenuid ? '&xmlmenuid='.$xmlmenuid : '')

        $this->defaultSort = 1;
        $this->primaryField = 0;

        // nacte urovne predavani aktualniho uzivatele
        //$this -> f_urovneuzivatele = SeznamUrovniPredavaniUzivatele($connection);
        $this->procesyUzivatele = PredavaciProcesyUtils::seznamProcesuUzivatele($database, true); // jen prvni uroven = true
    }


    function setFilterForm()
    {
        // nedela nic (filtr se nepouziva)
    }


    function enableLine($values)
    {
        if ($values[2]) {
            // kontrola prav (pole TypPrava)
            $prava = Prava::vratPrava($values[2]);
            if (!$prava->jsouNejakePrava()) return false;
        }

        // odfiltruje jen formulare, ktere muze aktualni uzivatel vyplnit
        // (= je na prvni urovni v predavacim procesu)
        return in_array($values[1], $this->procesyUzivatele);
    }

}

