<?php
/**
 * stanislav.elbl, 26.07.2021 11:16
 */

namespace RonPortal\XML;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\App;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use Ron\StringUtils;
use RonPortal\User;


class XMLFilterForm extends RonPortalFilterForm
{
    /** @var ComboBox */
    protected $comboPopis;

    /** @var string */
    private $xmlMenuId;

    public static function getFilterStav()
    {
        return RonPortalSession::getValue('filtrstavxml');
    }

    public static function setFilterStav($value)
    {
        RonPortalSession::setValue('filtrstavxml', $value);
    }

    public static function getZobrazitVse()
    {
        return RonPortalSession::getValue('xmlzobrazitvse');
    }

    public static function setZobrazitVse($value)
    {
        RonPortalSession::setValue('xmlzobrazitvse', $value);
    }

    public static function getJenPopisXML()
    {
        return RonPortalSession::getValue('filtrjenpopisxml');
    }

    public static function setJenPopisXML($value)
    {
        RonPortalSession::setValue('filtrjenpopisxml', $value);
    }

    public static function getPopisXML()
    {
        return RonPortalSession::getValue('filtrpopisxml');
    }

    public static function setPopisXML($value)
    {
        RonPortalSession::setValue('filtrpopisxml', $value);
    }

    public static function getJenOsobaXML()
    {
        return RonPortalSession::getValue('filtrjenosobaxml');
    }

    public static function setJenOsobaXML($value)
    {
        RonPortalSession::setValue('filtrjenosobaxml', $value);
    }


    /**
     * @param Database $database
     * @param string $xmlMenuId
     */
    public function __construct($database, $xmlMenuId)
    {
        parent::__construct($database, 'seznamxmlformularu');

        $this ->xmlMenuId = $xmlMenuId;

        $typPravaOsoby = RonPortalSettings::getValue('vyplnenexml_osoby_pravo','prohlizeni');
        if ($typPravaOsoby) {
            $prava = Prava::vratPrava($typPravaOsoby);
            if ($prava->jsouNejakePrava())
                $this->nastavOsoby(true, $prava, DateUtils::today(), DateUtils::today());
        }

        $this->nastavAkciGet('seznamxmlformularu');
        $this->f_classname = 'filtercontainer';

        // datum od-do
        $stav = self::getFilterStav();
        //pri nastaveni stavu neukonceno, nebo vse, pro pripad, ze neni nastaveno obdobi, nastavi filtr na datum vytvoreni
        if ((isset($_GET['stavxml']) && !RonPortalSession::jenObdobi()) && (($stav == -2 || $stav == -3))) {
            RonPortalSession::setFiltrovatDatum(true);
        }
//        $filtrpovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
        $filtrpovinny = false;
        $this->nastavDatumOdDo(true, $filtrpovinny, TextManager::_('Datum_vytvoreni_xml') . ' ' . TextManager::_('Od'));

        //obdobi
        if (RonPortalUtils::jeNainstalovanaAplikace($this->database,'ADS')) {
            $this->nastavObdobi(true, 'ADS', 0, false);
        }

        // rozdeleni
        $this->nastavRozdeleni(true, RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI),
            RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_JEN_VLASTNI_ROZDELENI), '', true);

        // pouze vlastni/vse
        if (Prava::vratHodnotuPrava('xml') == 3) {
            // ma pravo na vsechny XML formulare - zobrazi se link vse/pouze vlastni
            if (isset($_GET['xmlzobrazitvse'])) self::setZobrazitVse((int)$_GET['xmlzobrazitvse']);

            if (self::getZobrazitVse()) {
                $this->nastavLink(
                    "<a href='?menuid=seznamxmlformularu&xmlzobrazitvse=0" .
                    ($xmlMenuId ? '&xmlmenuid=' . $xmlMenuId : '') .
                    "'>" .
                    TextManager::_('Zobrazit_pouze_vlastni_xml') .
                    "</a>");
            } else {
                $this->nastavLink(
                    "<a href='?menuid=seznamxmlformularu&xmlzobrazitvse=1" .
                    ($xmlMenuId ? '&xmlmenuid=' . $xmlMenuId : '') .
                    "'>" .
                    TextManager::_('Zobrazit_vsechny_xml') .
                    "</a>");
            }
        }
    }


    function loadData()
    {
        parent::loadData();

        if (isset($_GET['popisxml'])) {
            self::setJenPopisXML(isset($_GET['jenpopisxml'])  &&  (int)$_GET['jenpopisxml']);
            self::setPopisXML($_GET['popisxml']);
        }

        if (isset($_GET['osoba'])) {
            self::setJenOsobaXML(isset($_GET['jenosoba'])  &&  (int)$_GET['jenosoba']);
        }

        $params = $this ->database ->createParameters();
        if (self::getFilterStav() == -1) {
            // filtr pro vsechny neukoncene bez ohledu na obdobi
            $sql =
                'SELECT DISTINCT VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Popis'.
                ' FROM VyplnenyXMLFormular LEFT OUTER JOIN SablonaXMLFormulare' .
                ' ON SablonaXMLFormulare.ID = VyplnenyXMLFormular.IdSablony'.
                ' WHERE Stav < 128 ';
        } else {
            // obdobi je povinne => filtr podle obdobi
            $sql =
                'SELECT DISTINCT VyplnenyXMLFormular.Popis, VyplnenyXMLFormular.Popis'.
                ' FROM VyplnenyXMLFormular LEFT OUTER JOIN SablonaXMLFormulare' .
                ' ON SablonaXMLFormulare.ID = VyplnenyXMLFormular.IdSablony'.
                ' WHERE VyplnenyXMLFormular.DatumVytvoreni >= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::getDatumOd())) .
                ' AND VyplnenyXMLFormular.DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam(RonPortalSession::getDatumDo()));
        }

        if ($this->xmlMenuId) {
            $sql .=
                ' AND SablonaXMLFormulare.MenuId = ' . $params->addParameter($this->xmlMenuId);
        }

        $this ->comboPopis = new ComboBox('popisxml');
        $this ->comboPopis ->keyIndex = 0;
        $this ->comboPopis ->captionIndex = 1;
        $this ->comboPopis ->emptyRow = ' ';
        $this ->comboPopis ->dataTypeComparison = true;
        $this ->comboPopis ->className = 'form-input adsfilterforminput_full';
        $this ->comboPopis ->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jenpopisxml'];
        $this ->comboPopis ->loadData($this ->database, $sql, self::getPopisXML(), $params);
    }


    function showFilterPopis()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenpopisxml',
            TextManager::_('Jen_typ_xml'), self::getJenPopisXML(),
            ['id' => 'jenpopisxml'], 'clickAutoSubmit', 1, true);
        echo '</SPAN>';

        // combobox pro vyber
        $this ->comboPopis ->render();
        echo "</DIV>\n";
    }

    function renderOsoby()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenosoba', TextManager::_('Jen_osoba'), self::getJenOsobaXML(), '', 'clickAutoSubmit', 1, true);
        echo '</SPAN>';

        $this ->comboOsoba ->render();

        if ($this ->f_prava) {
            if ($this->comboOsoba->data && count($this->comboOsoba->data) > 1) {
                // tlacitka pro posun na dalsi a predchozi PP
                echo '<span id="adsfilterform_pp_fast" class="desktop-only">';
                echo '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_osoba"></i>';
                echo '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-submit="1" data-selector="#adsfilterform_osoba"></i>';
                echo '</span>';
            }
        }

        echo "</DIV>\n";
    }


    function renderFilters()
    {
        if ($this ->xmlMenuId) {
            echo "<input type='hidden' name='xmlmenuid' value='{$this ->xmlMenuId}'/>";
        }

        $this->renderDatumOdDo();

        if ($this->showObdobi) {
            $this->renderObdobi();
        }

        $this->showFilterPopis();
        $this->renderRozdeleni();

        if ($this->showOsoby) {
            $this->renderOsoby();
        }

        $this->renderLink();
    }
}




class EditaceXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this->type = 'text';
        $this->align = 'left';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=editovatdotaznik&record=' . $recordId . '">' .
            TextManager::_(($rowData[2] >= 128) ? 'Zobrazit_xml' : 'Editovat_xml') .
            '</a>';
    }
}



class RozdeleniXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        global $STR_Rozdeleni; // kompat.

        $this->header = $STR_Rozdeleni ?: TextManager::_('Rozdeleni');
        $this->type = BrowserField::FIELD_TYPE_DEFAULT;
        $this->align = 'left';
        $this->sqlFieldName = 'Rozdeleni.Kod';
        $this->ordering = 1;
        $this->sqlFieldIndex = 9;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = $value;
        if ($rowData[3]) {
            $result .= ' (' . $rowData[3] . ')';
        }

        return $result;
    }
}




class StavXMLField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this->header = TextManager::_('Stav');
        $this->type = BrowserField::FIELD_TYPE_DEFAULT;
        $this->align = 'left';
        $this->sqlFieldName = 'S_StavXMLFormulare.Popis';
        $this->ordering = 1;
        $this->sqlFieldIndex = 12;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = $value;

//        $backurl = 'seznamxmlformularu' . (isset($_GET['xmlmenuid']) ? "&xmlmenuid={$_GET['xmlmenuid']}" : '');
//        if ($rowData[4]) $result .= " <I>(<a href='?menuid=predavaciproces&xmlid={$rowData[0]}&backurl=$backurl'>"  .  $rowData[4] . '</a>)</I>';

        if ($rowData[4]) {
            $result .= ' <I>(' . $rowData[4] . ')</I>';
        }
        return $result;
    }
}




class VyplnenyXMLFormularBrowser extends Browser
{
    var $xmlMenuId;
    /** @var XMLFilterForm */
    var $filterForm;

    function __construct($database, $xmlMenuId)
    {
        parent::__construct($database);

        global $STR_VyplneneFormulare; // kompat.

        $this->xmlMenuId = $xmlMenuId;
        $this->browserUrl = '?menuid=seznamxmlformularu&' . ($xmlMenuId ? 'xmlmenuid=' . $xmlMenuId . '&' : '');

        $this->title = $STR_VyplneneFormulare ?: TextManager::_('Vyplnene_xml_formulare');

        $this->code = 'browservyplnenexml';

        $this->cssClass .= ' browservyplnenexml' . ($xmlMenuId ? '_' . $xmlMenuId : '') ;

        $this->sqlFromPart =
            '( ( ( ( VyplnenyXMLFormular LEFT OUTER JOIN Osoba ' .
            '          ON (VyplnenyXMLFormular.RC = Osoba.RC  OR   VyplnenyXMLFormular.PP_RC = Osoba.RC)' .
            '      ) LEFT OUTER JOIN PredavaciProces ' .
            '        ON PredavaciProces.Kod = VyplnenyXMLFormular.KodProcesu ' .
            '    ) LEFT OUTER JOIN PoradiPredavani ' .
            '      ON PoradiPredavani.KodProcesu = VyplnenyXMLFormular.KodProcesu ' .
            '         AND PoradiPredavani.PocatecniUroven = VyplnenyXMLFormular.PocatecniUroven ' .
            '         AND PoradiPredavani.Poradi = VyplnenyXMLFormular.PoradiPredavani ' .
            '  ) LEFT OUTER JOIN Rozdeleni ' .
            '    ON Rozdeleni.KodTypuRozdeleni = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '       AND Rozdeleni.Kod = VyplnenyXMLFormular.Rozdeleni ' .
            ') INNER JOIN S_StavXMLFormulare ' .
            '  ON S_StavXMLFormulare.Kod = VyplnenyXMLFormular.Stav ';

        // pokud je zadano $xmlmenuid, tak nacte jen formulare prirazene k sablonam se zadanym menuid
        if ($this->xmlMenuId)
            $this->sqlFromPart =
                '( ' . $this->sqlFromPart . ' ) ' .
                '  INNER JOIN SablonaXMLFormulare ON SablonaXMLFormulare.Id = VyplnenyXMLFormular.IdSablony' .
                "   AND SablonaXMLFormulare.MenuId = '" . $database ->addSlashes($this->xmlMenuId) . "'";

        //$this -> AddFieldDef( array('normal', 'RC', 'RC osoby ...', '', 'left', '', 0, 1) );
        $this ->sqlSelectPart =
            'VyplnenyXMLFormular.Id, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.Stav, ' . // 0..2
            'Rozdeleni.Popis, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.DodatecnyPopis, ' . // 3..5
            'VyplnenyXMLFormular.Popis, Osoba.Prijmeni, Osoba.Jmeno, Rozdeleni.Kod, ' . // 6..9
            'VyplnenyXMLFormular.DatumVytvoreni, VyplnenyXMLFormular.PosledniZmena, S_StavXMLFormulare.Popis, ' . // 10..12
            'VyplnenyXMLFormular.KodProcesu, VyplnenyXMLFormular.PocatecniUroven, VyplnenyXMLFormular.PoradiPredavani, ' . // 13..15
            'PoradiPredavani.Vlastni, PoradiPredavani.NekontrolovatHierarchii, VyplnenyXMLFormular.Obdobi, ' . // 16..18
            'VyplnenyXMLFormular.Rozliseni1, VyplnenyXMLFormular.Rozliseni2, VyplnenyXMLFormular.Rozliseni3, VyplnenyXMLFormular.Rozliseni4 , VyplnenyXMLFormular.Rozliseni5, ' . //19 .. 23
            'VyplnenyXMLFormular.Rozliseni6, VyplnenyXMLFormular.Rozliseni7, VyplnenyXMLFormular.Rozliseni8, VyplnenyXMLFormular.Rozliseni9, VyplnenyXMLFormular.Rozliseni10 '; //24 .. 28


        if (App::getDebugMode()) {
            $this->addField()
                ->setSqlFieldName('VyplnenyXMLFormular.Id')->setSqlFieldIndex(0)->setHeader('ID')->setAlign('right')->setOrdering(true)->setCode('col-id');
        }

        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.Popis')->setSqlFieldIndex(6)->setHeaderText('Typ')->setOrdering(true)->setCode('col-popis');

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni')->setSqlFieldIndex(7)->setHeaderText('Prijmeni')->setOrdering(true)->setCode('col-prijmeni');
        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno')->setSqlFieldIndex(8)->setHeaderText('Jmeno')->setOrdering(true)->setCode('col-jmeno');
        $this->addField( new RozdeleniXMLField() ); // Rozdeleni.Kod
        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.Obdobi')->setSqlFieldIndex(18)->setHeaderText('Obdobi')->setOrdering(true)
            ->setFormat('date')->setCode('col-obdobi');
        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.DatumVytvoreni')->setSqlFieldIndex(10)->setHeaderText('Cas_vytvoreni_xml')->setOrdering(true)
            ->setFormat('datetime')->setCode('col-datumvytvoreni');
        $this->addField()
            ->setSqlFieldName('VyplnenyXMLFormular.PosledniZmena')->setSqlFieldIndex(11)->setHeaderText('Posledni_zmena')->setOrdering(true)
            ->setFormat('datetime')->setCode('col-poslednizmena');

        $this ->defaultSort = count($this ->fields);
        $this->addField( new StavXMLField() ); // S_StavXMLFormulare.Popis

        for ($index = 1; $index <= 10; $index++) {
            if (RonPortalSettings::getValue("xml_rozliseni{$index}_nadpis") != '') {
                $this->addField()
                    ->setSqlFieldName("VyplnenyXMLFormular.Rozliseni{$index}")->setSqlFieldIndex(18+$index)->setHeader(RonPortalSettings::getValue("xml_rozliseni{$index}_nadpis"))
                    ->setOrdering(true)->setClassName("rozliseni{$index}")->setCode("col-rozliseni{$index}");
            }
        }

        if (App::getDebugMode()) {
            $this->addField()
                ->setSqlFieldName('VyplnenyXMLFormular.PocatecniUroven')->setSqlFieldIndex(14)->setHeader('Poc.uroven')->setOrdering(true)->setCode('col-pocatecni-uroven');
            $this->addField()
                ->setSqlFieldName('VyplnenyXMLFormular.PoradiPredavani')->setSqlFieldIndex(15)->setHeader('Por.predavani')->setOrdering(true)->setCode('col-poradi');
            $this->addField()
                ->setSqlFieldName('VyplnenyXMLFormular.Vlastni')->setSqlFieldIndex(16)->setHeader('Vlastni')->setOrdering(true)->setCode('col-vlastni');
            $this->addField()
                ->setSqlFieldName('VyplnenyXMLFormular.NekontrolovatHierarchii')->setSqlFieldIndex(17)->setHeader('NE hierarchie')->setOrdering(true)->setCode('col-hierarchie');
        }

        $this->addField(new EditaceXMLField());

        $this->primaryField = 0;

        $this->maxPageRows = 100;
    }


    function load($sqlFilter = '')
    {
        if (isset($_GET['stavxml'])) {
            XMLFilterForm::setFilterStav((int)$_GET['stavxml']);
        }

        $this ->filterForm = new XMLFilterForm($this ->database, $this->xmlMenuId);
        $this ->filterForm ->loadData();

        parent::load($sqlFilter);
    }


    function renderFilterStav()
    {
        $stav = XMLFilterForm::getFilterStav();
        if (!isset($stav)) {
            XMLFilterForm::setFilterStav(-1);
            $stav = -1; // vychozi hodnota - neukoncene
        }

        $data = [
            -1 => TextManager::_('Neukonceno'),
            -2 => TextManager::_('Ukonceno'),
            -3 => TextManager::_('Vse')
        ];

        $url = '?menuid=seznamxmlformularu' . ($this->xmlMenuId ? '&xmlmenuid=' . $this->xmlMenuId : '') . '&stavxml=';

        LinksList::renderLinks($data, $stav, $url);
    }


    function renderFilter()
    {
        $this->renderFilterStav();
        $this ->filterForm ->render();
    }



    /**
     * Vraci filtr pro vyber formularu podle procesu
     * @return string
     */
    function getSQLFilterXML()
    {
        // pokud ma prava na vse a ma zapnute vse, tak nic nefiltruje
        if (Prava::vratHodnotuPrava('xml') == 3  &&  XMLFilterForm::getZobrazitVse()) {
            return '';
        }

        //TODO: kontrola dle noveho nastaveni viditelnosti - provadet v enable line?

        // formulare ktere vlozil aktualni uzivatel - v puvodnich verzich se nepouzivala tabulka XMLFormularVyplnil
        $result =
            "( (RC_Vlozil = '" . $this ->database ->addSlashes(User::getRC()) . "') "; // prava zavorka bude dale !


        // formulare ktere vyplnil aktualni uzivatel (i kdyz jsou zrovna na jine urovni predavani)
        $result .=
            'OR ( EXISTS ( ' .
            ' SELECT * FROM XMLFormularVyplnil ' .
            " WHERE XMLFormularVyplnil.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
            '   AND XMLFormularVyplnil.IdFormulare = VyplnenyXMLFormular.Id' .
            ') ) ';

        // formulare na urovni oznacene "vlastni"
        $result .=
            'OR (PoradiPredavani.Vlastni = 1 ' .
            " AND (VyplnenyXMLFormular.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'  OR  VyplnenyXMLFormular.PP_RC = '" . $this ->database ->addSlashes(User::getRC()) . "') " .
            ')';


        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $filtrurovne = PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne');
        $kodtypurozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI); // pouzije se, pokud neni typ rozdeleni u procesu zadan

        global $_XML_jenPodrizene; //pro zachovani funkcnosti v appdbset, pokud budou chtit zmenit, musi se zrusit v appdbset
        $XML_jenPodrizene = $_XML_jenPodrizene == 1 ? true : RonPortalSettings::getIntValue('jenvlastnirozdelenihodnoceni') == 2;

        if ($filtrurovne) // mam pravo na nejake urovne hodnoceni
        {
            // vybere XML zamestnancu, na ktere mam pravo (podle pracovniho mista a hierarchie)
            $result .=
                'OR ( PoradiPredavani.Vlastni = 0 ' . // urovne s volbou "vlastni" jsou resene samostatne
                " AND ( $filtrurovne ) " .  // formular na "moji" urovni predavani
                ' AND  (' .
                // na aktualni urovni formulare se nekontroluje hierarchie
                'PoradiPredavani.NekontrolovatHierarchii = 1 '.
                '  OR ' .
                // nebo jsem ve stejnem nebo nadrazenem rozdeleni
                'EXISTS  ( SELECT * ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
                "  WHERE R_Ja.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
                '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodtypurozdeleni) " . // typ rozdeleni u procesu neni zadan
                ' )' .
                '    AND R_Ja.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                '    AND (R_Ja.PlatnostDo >= ' . $this ->database ->dateToString(time()) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                //"    AND R_Hodnoceny.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze - natvrdo jeden typ rozdeleni
                '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                '    AND R_Hodnoceny.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                '    AND (R_Hodnoceny.PlatnostDo >= ' . $this ->database ->dateToString(DateUtils::startOfTheDay(time())) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .

                '    AND ( ' .
                '      R_Hodnoceny.RC = VyplnenyXMLFormular.RC ' .
                '      OR ( ' .
                '        R_Hodnoceny.RC = VyplnenyXMLFormular.PP_RC ' .
                '        AND R_Hodnoceny.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
                '        AND R_Hodnoceny.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
                '      ) ' .
                '    ) ' .
                ')' .
                ')' .
                ')';

            // Vybere XML, ktere nejsou pro konkretni osobu, ale jsou pro moje a podrizene rozdeleni
            $result .=
                ' OR (' .
                "( $filtrurovne ) " .  // dokument na "moji" urovni predavani
                ' AND VyplnenyXMLFormular.RC IS NULL ' .    // neni pro konkretni osobu
                ' AND VyplnenyXMLFormular.PP_RC IS NULL ' .
                ' AND ('.
                '    PoradiPredavani.NekontrolovatHierarchii = 1 '.
                '   OR ' .
                '    EXISTS ( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                "  WHERE R_Ja.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'" .
                //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze

                '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = $kodtypurozdeleni) " . // typ rozdeleni u procesu neni zadan
                '    )' .

                '    AND R_Ja.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
                '    AND (R_Ja.PlatnostDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay(time())) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
                ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                ')' .
                ')' .
                ')';
        }

        $result .= ')';

        return $result;
    }


    /**
     * @return string
     */
    function getSQLFilterRozdeleni()
    {
        if (!$this->filterForm->kodRozdeleni) {
            return '';
        }

        $sqlPodrizene = '';

        switch ($this->filterForm->vcetnePodrizenychRozdeleni) {
            case 0:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene = HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 1:
                $sqlPodrizene = '';
                break;
            case 2:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodNadrazene <> HierarchieRozdeleni.kodPodrizene) ';
                break;
            case 3:
                $sqlPodrizene = 'and (HierarchieRozdeleni.kodPodrizene IN (SELECT Kod FROM Rozdeleni WHERE KodTypuRozdeleni = HierarchieRozdeleni.typNadrazene AND RozdeleniRodice = HierarchieRozdeleni.typNadrazene AND Rodic = HierarchieRozdeleni.kodNadrazene)) ';
                break;
        }

        // filtr podle rozdeleni formulare
        $result =
            '( ' .
            ' EXISTS ( ' .
            '   SELECT * FROM HierarchieRozdeleni ' .
            '   WHERE TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
            "     AND KodNadrazene = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "' " .
            '     AND TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '     AND KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
            $sqlPodrizene .
            ' )';


        $result .= ') OR (';

        // filtr podle osoby formulare
        $result .=
            'EXISTS (' .
            '  SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
            '  WHERE HierarchieRozdeleni.TypNadrazene = ' . RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI) .
            "    AND HierarchieRozdeleni.KodNadrazene = '" . $this ->database ->addSlashes(RonPortalSession::rozdeleni($this->filterForm->f_typrozdeleni)) . "'" .
            '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
            '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodPodrizene ' .
            $sqlPodrizene .
            '    AND ( RozdeleniOsob.RC = VyplnenyXMLFormular.RC ' .
            ' OR (' .
            '   RozdeleniOsob.RC = VyplnenyXMLFormular.PP_RC ' .
            '   AND RozdeleniOsob.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
            '   AND RozdeleniOsob.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
            ' )' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $this ->database ->timestampToString(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $this ->database ->timestampToString(DateUtils::startOfTheDay(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            '    ) ' .
            ')';

        $result .= ')';

        return $result;
    }


    function getSQLFilter()
    {
        // filtr pro stav
        switch (XMLFilterForm::getFilterStav()) {
            case -3 : // vsechny
                $result = '';
                break;

            case -2 : // ukoncene
                $result = ' (VyplnenyXMLFormular.Stav >= 128) ';
                break;

            case -1 : // neukoncene
            default :
                $result = ' (VyplnenyXMLFormular.Stav < 128) ';
                break;
        }

        // filtr podle predavaciho procesu
        $pom = $this->getSQLFilterXML();
        if ($pom && $result) {
            $result .= ' AND ';
        }
        $result .= $pom;


        // filtr jen typ (popis)
        if (XMLFilterForm::getJenPopisXML() && XMLFilterForm::getPopisXML()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result = $result . "VyplnenyXMLFormular.Popis = '" . $this ->database ->addSlashes(XMLFilterForm::getPopisXML()) . "'";
        }

        // filtr podle data vytvoreni
//        $stav = XMLFilterForm::getFilterStav();
//        $filtrpovinny = ($stav == -2 || $stav == -3); // -2 = ukonceno, -3 = vse
        $filtrpovinny = false;
        //30.9.2019 zrusena povinnost filtru na datum, nastaveno strankovani browseru
        if ($filtrpovinny || RonPortalSession::getFiltrovatDatum()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result = $result .
                ' VyplnenyXMLFormular.DatumVytvoreni >= ' . $this ->database ->dateToString(RonPortalSession::getDatumOd()) .
                '   AND VyplnenyXMLFormular.DatumVytvoreni < ' . $this ->database ->dateToString( DateUtils::incDay(RonPortalSession::getDatumDo(), 1) );

        }

        if (RonPortalSession::jenObdobi()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result .=
                ' (VyplnenyXMLFormular.Obdobi >= ' . $this->database->timestampToString(RonPortalSession::obdobi()) .
                ' AND VyplnenyXMLFormular.Obdobi <= ' . $this->database->timestampToString(RonPortalSession::konecObdobi()) . ')';
        }

        if (XMLFilterForm::getJenOsobaXML()) {
            if ($result) {
                $result = $result . ' AND ';
            }
            $result .=
                " ((VyplnenyXMLFormular.RC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "') OR " .
                " (VyplnenyXMLFormular.PP_RC = '" . $this->database->addSlashes(RonPortalSession::osobaRC()) . "' AND VyplnenyXMLFormular.PP_OsobniCislo = '" . $this->database->addSlashes(RonPortalSession::osobaOsobniCislo()) . "'" .
                "    AND VyplnenyXMLFormular.PP_DatumNastupu = " . $this->database->dateToString(RonPortalSession::osobaNastup()) . "))";
        }

        $filtrrozdeleni = $this->getSQLFilterRozdeleni();
        if ($filtrrozdeleni) {
            if ($result) {
                $result .= ' AND ';
            }
            $result .= '(' . $filtrrozdeleni . ')';
        }

        return $result;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[5]) {
            // dodatecny popis
            echo "<tr class='$className'>";
            echo '<td></td><td colspan="' . ($this->columnCount - 1) . '"><span class="browserpoznamka">' . StringUtils::htmlTextConvert($values[5]) . '</span></td>';
            echo '</tr>';

        }
    }
}

