
if(typeof String.prototype.trim !== 'function') {
  String.prototype.trim = function() {
    return this.replace(/^\s+|\s+$/g, ''); 
  }
}

if (!window.XMLHttpRequest) {
    window.XMLHttpRequest = function() {
        return new ActiveXObject('Microsoft.XMLHTTP');
    };
}

if (!window.console) {
    window.console = { log: function() {} };
}

if (!Function.prototype.bind) {
    Function.prototype.bind = function(oThis) {
        if (typeof this !== "function") {
            throw new TypeError('Function.prototype.bind - what is trying to be bound is not callable');
        }

        var aArgs   = Array.prototype.slice.call(arguments, 1),
            fToBind = this,
            fNOP    = function() {},
            fBound  = function() {
                return fToBind.apply(this instanceof fNOP && oThis
                        ? this
                        : oThis,
                    aArgs.concat(Array.prototype.slice.call(arguments)));
            };

        fNOP.prototype = this.prototype;
        fBound.prototype = new fNOP();

        return fBound;
    };
}


if(typeof FileReader !== "undefined") {
    if (FileReader && !FileReader.prototype.readAsBinaryString) {
        FileReader.prototype.readAsBinaryString = function (fileData) {
            var binary = "";
            var pt = this;
            var reader = new FileReader();
            reader.onload = function (e) {
                var bytes = new Uint8Array(reader.result);
                var length = bytes.byteLength;
                for (var i = 0; i < length; i++) {
                    binary += String.fromCharCode(bytes[i]);
                }
                pt.content = binary;
                $(pt).trigger('onload');
            };
            reader.readAsArrayBuffer(fileData);
        }
    }
}
var ron = {
    translations: [],

    translate : function(text) {
        var result = ron.translations[text];

        if (result == undefined) return text;
        return result;
    },
    Point : function(x, y) {
        this.x = x;
        this.y = y;
    },
    Rect : function(x1, y1, x2, y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    },


    utils: {
        lineBreaksToBr: function (str) {
            return str.replace(/(?:\r\n|\r|\n)/g, '<br />');
        },
        startsWith: function (str, prefix) {
            return str.substr(0, prefix.length) == prefix;
        },
        removePrefix: function (str, prefix) {
            if (!ron.utils.startsWith(str, prefix)) return false;

            return str.substring(prefix.length);
        },
        containsSubstring: function (str, substring, delimiter) {
            var a = str.split(delimiter);

            for (var i = 0; i < a.length; i++) {
                if (a[i] == substring) return true;
            }

            return false;
        },
        addSubstring: function (str, substring, delimiter) {
            if (ron.utils.containsSubstring(str, substring, delimiter)) return str;

            return str + (str ? delimiter : "") + substring;
        },
        removeSubstring: function (str, substring, delimiter) {
            var a = str.split(delimiter);
            var result = '';

            for (var i = 0; i < a.length; i++) {
                if (a[i] != substring) result += result ? (delimiter + a[i]) : a[i];
            }

            return result;
        },
        htmlTextConvert: function (text) {
            if (!text) return '';

            var map = {
                '&': '&amp;',
                '<': '&lt;',
                '>': '&gt;',
                '"': '&quot;',
                "'": '&#039;'
            };

            return text.replace(/[&<>"']/g, function (m) {
                return map[m];
            });
        },
        getUrlParam: function(paramName) {
            var params = location.search.split("&");

            for (var i = 0;  i < params.length;  i++) {
                var pom = params[i].split(paramName + "=");
                if (pom  &&  pom.length > 1) return pom[1];
            }

            return null;
        },
        stringToFloat : function(str) {
            if (!str) return null;

            str = str.replace(/,/g, ".");
            str = str.replace(/ /g, "");
            str = str.replace(/&nbsp;/g, "");

            return parseFloat(str);
        },
        formatFloat : function(number, digits) {
            if (typeof digits == "undefined" ) {
                digits = 2;
            }

            return window.Intl // kvuli IE <= 10
                ? Intl.NumberFormat('cs-cz', {minimumFractionDigits: digits, maximumFractionDigits: digits}).format(number)
                : number.toFixed(2);
        },
        stringToDate : function(str) {
            if (str == '') return null;
            if (str == undefined) return null;

            var now = new Date();
            var day = now.getDate();
            var month = now.getMonth();
            var year = now.getFullYear();

            if (str.match(/^(\d+).(\d+).(\d+)$/)) {
                var dateparts = str.split('.');
                day = parseInt(dateparts[0], 10);
                month = parseInt(dateparts[1], 10) - 1; // 0 - 11 !
                year = parseInt(dateparts[2], 10);
            } else if (str.match(/^(\d+).(\d+)[.]$/)) {
                dateparts = str.split('.');
                day = parseInt(dateparts[0], 10);
                month = parseInt(dateparts[1], 10) - 1; // 0 - 11 !
            } else if (str.match(/^(\d+)[.]$/)) {
                dateparts = str.split('.');
                day = parseInt(dateparts[0], 10);
            }
            else return null; // chyba

            return new Date(year, month, day);
        },
        stringToDateTime : function(str) {
            if (str == '') return null;
            if (str == undefined) return null;

            if (str.match(/^(\d+).(\d+).(\d+) (\d+):(\d+):(\d+)$/)) {
                var parts = str.split(' ');
                var dateParts = parts[0].split('.');
                var day = parseInt(dateParts[0], 10);
                var month = parseInt(dateParts[1], 10) - 1; // 0 - 11 !
                var year = parseInt(dateParts[2], 10);
                var timeParts = parts[1].split(':');
                var hour = parseInt(timeParts[0], 10);
                var min = parseInt(timeParts[1], 10);
                var sec = parseInt(timeParts[2], 10);

                return new Date(year, month, day, hour, min, sec);
            }

            return null; // chyba


        },
        dateToString : function(date) {
            if ( !date ) return '';

            return date.getDate() + '.' + (date.getMonth() + 1) + '.' + date.getFullYear();
        },
        stringToTime : function(str) {
            if (!str) return null;

            var timeparts = str.split(':');

            if (timeparts.length > 0) {
                var h = parseInt(timeparts[0], 10);
                var m = 0;
                var s = 0;
                if (timeparts.length == 1) {
                    if ( h > 100 ) {
                        m = h % 100;
                        h = Math.floor(h / 100);
                    }
                } else {
                    if (timeparts.length > 1) m = parseInt(timeparts[1], 10);
                    if (timeparts.length > 2) s = parseInt(timeparts[2], 10);
                }

                return new Date(1970, 0, 1, h, m, s, 0);
            }

            return null;
        },
        timeToString : function(date, utc, seconds) {
            if (!date) return '';

            var h = utc ? date.getUTCHours() : date.getHours();
            var m = utc ? date.getUTCMinutes() : date.getMinutes();

            var result = h + ':';
            if (m < 10) result += '0';
            result += m;

            if (seconds) {
                var s = utc ? date.getUTCSeconds() : date.getSeconds();
                result += ':';
                if (s < 10) result += '0';
                result += s;
            }

            return result;
        },


        secondsToString : function(seconds) {
            var s = seconds % 60;
            var min = (seconds - s) / 60;

            var m = min % 60;
            var h = (min - m) / 60;

            return h + ':' + ((m < 10) ? '0' : '') + m  + ':' + ((s < 10) ? '0' : '') + s;
        },
        timeStringToFloat : function(str) {
            if (!str) return 0;
            if (str == '') return 0;

            var timeparts = str.split(':');

            if (timeparts.length > 0) {
                var h = parseInt(timeparts[0], 10);
                if (isNaN(h)) return null;

                var m = 0;
                if (timeparts.length > 1) {
                    m = parseInt(timeparts[1], 10);
                    if (isNaN(m)) return null;
                }

                var s = 0;
                if (timeparts.length > 2) {
                    s = parseInt(timeparts[2], 10);
                    if (isNaN(s)) return null;
                }

                return h + m / 60 + s / 3600;
            }

            return 0;
        },
        floatToTimeString : function(value) {
            if (value == null) return '';

            var result = '';

            if (value < 0) {
                result = '-';
                value = -value;
            }

            var h = Math.floor(value);
            result += h + ':';


            var m = Math.round((value - h) * 60);
            if (m < 10) result += '0' + m;
            else result += m;

            return result;
        },


        today : function() {
            var datetime = new Date();
            return new Date(datetime.getFullYear(), datetime.getMonth(), datetime.getDate());
        },
        incMonth : function(date, cnt) {
            if (!date) return date;

            var
                month = date.getMonth(),
                year = date.getFullYear();

            month = month + cnt;

            while (month > 11) {
                month = month - 12;
                year = year + 1;
            }
            while (month < 0) {
                month = month + 12;
                year = year - 1;
            }

            return new Date(year, month, date.getDate(),
                date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds());
        },
        incDay : function(date, cnt) {
            var result = new Date(date.getTime());
            result.setDate(result.getDate() + cnt); // POZOR! Nefunguje pri posunu casu! (napr. 28.3.2020)
            return result;
        },
        incDayStart : function(date, cnt) {
            var result = new Date(date.getTime());
            result.setHours(0, 0, 0, 0);
            result.setTime(result.getTime() + (cnt * 24 + 2) * 60 * 60 * 1000); // hodiny navic kvuli posunu casu
            result.setHours(0, 0, 0, 0);
            return result;
        },
        daysBetween : function(date1, date2) {
            return Math.round( (date2.getTime() - date1.getTime()) / 86400000 );
        },
        copyArray : function(inputArray) {
            var result = [];
            var i = inputArray.length;
            while (i--) result[i] = inputArray[i]; // toto je dle benchmarku nejrychlejsi zpusob

            return result;
        },
        binarySearch : function(array, element, compareFunction) {
            var minIndex = 0;
            var maxIndex = array.length - 1;
            var currentIndex;
            var currentElement;

            while (minIndex <= maxIndex) {
                currentIndex = (minIndex + maxIndex) / 2 | 0;
                currentElement = array[currentIndex];

                if (compareFunction) {
                    var compare = compareFunction(currentElement, element);
                    if (compare < 0) { // currentElement < element
                        minIndex = currentIndex + 1;
                    }
                    else if (compare > 0) {
                        maxIndex = currentIndex - 1;
                    }
                    else {
                        return currentIndex;
                    }
                } else {
                    if (currentElement < element) {
                        minIndex = currentIndex + 1;
                    }
                    else if (currentElement > element) {
                        maxIndex = currentIndex - 1;
                    }
                    else {
                        return currentIndex;
                    }
                }
            }

            return -1;
        },
        windowSize : function() {
            var $window = $(window);
            return new ron.Point($window.width(), $window.height());
        },
        windowViewRect : function() {
            var size = ron.utils.windowSize();

            var $window = $(window);
            var left = $window.scrollLeft();
            var top = $window.scrollTop();

            return new ron.Rect(left, top, left + size.x - 1, top + size.y - 1);
        },
        getPositionInParent : function($element, parent) {
            var
                result = $element.offset(),
                parentOffset = $(parent).offset();

            result.top -= parentOffset.top;
            result.left -= parentOffset.left;

            return result;
        },
        getAbsoluteRect: function($obj) {
            var position = $obj.offset();

            return new ron.Rect(position.left, position.top, position.left + $obj.width() - 1, position.top + $obj.height() - 1);
        },
        hasFixedPosition : function($element) {
            var lastElement = null;
            while ($element.length  &&  $element.get(0) != lastElement) { // document.body  &&  $element.get(0) != ownerDocument) {
                lastElement = $element.get(0);
                if ( $element.css("position") == "fixed") return true;
                $element = $element.offsetParent();
            }

            return false;
        },
        scrollPage : function(options) {
            if (options.x) {
                $(window).scrollLeft(options.x);
            }

            if (options.y) {
                $(window).scrollTop(options.y);
            }
        },

        initialScrollPage : function(options) {
            $(function() {
                ron.utils.scrollPage(options);
            });
        },
        ensureVisible : function($element, $parent) {
            if (!$parent) {
                $parent = $element.scrollParent(); // jQuery-ui
            }

            var parentHeight = ($parent.get(0) == document) ? $(window).height() : $parent.get(0).clientHeight;

            if ($parent.get(0).scrollHeight <= parentHeight) {
                return;
            }

            var pos = ron.utils.getPositionInParent($element, $parent.get(0));

            if ( pos.top < 0 ) {  // ( pos.top < $parent.scrollTop() ) {
                var newPos = $parent.scrollTop() + pos.top;
                if (newPos < 0) newPos = 0;
                $parent.scrollTop( newPos );

            } else if ( pos.top + $element.height() > parentHeight /*+ $parent.scrollTop()*/ )  {
                newPos = $parent.scrollTop() + pos.top + $element.height() - parentHeight;
                if (newPos < 0) newPos = 0;
                $parent.scrollTop( newPos );
            }
        },
        isFullVisible : function($element, $parent) {
            if (!$parent) {
                $parent = $element.scrollParent(); // jQuery-ui
            }

            var parentHeight = ($parent.get(0) == document) ? $(window).height() : $parent.get(0).clientHeight;

            if ($parent.get(0).scrollHeight <= parentHeight) {
                return true;
            }

            var pos = ron.utils.getPositionInParent($element, $parent.get(0));

            if ( pos.top < 0 ) {
                return false;
            } else if ( pos.top + $element.height() > parentHeight )  {
                return false;
            }

            return true;
        },
        scrollElementToTop : function($element, $parent) {
            if (!$parent) {
                $parent = $element.scrollParent(); // jQuery-ui
            }

            var parentHeight = ($parent.get(0) == document) ? $(window).height() : $parent.get(0).clientHeight;

            if ($parent.get(0).scrollHeight <= parentHeight) {
                return;
            }

            var pos = ron.utils.getPositionInParent($element, $parent.get(0));
            var newPos = $parent.scrollTop() + pos.top;
            if (newPos < 0) newPos = 0;

            $parent.scrollTop( newPos );
        },
        scrollElementToBottom : function($element, $parent) {
            if (!$parent) {
                $parent = $element.scrollParent(); // jQuery-ui
            }

            var parentHeight = ($parent.get(0) == document) ? $(window).height() : $parent.get(0).clientHeight;

            if ($parent.get(0).scrollHeight <= parentHeight) {
                return;
            }

            var pos = ron.utils.getPositionInParent($element, $parent.get(0));
            newPos = $parent.scrollTop() + pos.top + $element.height() - parentHeight;
            if (newPos < 0) newPos = 0;
            $parent.scrollTop( newPos );
        },
        getCookie: function(cookieName) {
            var name = cookieName + "=";
            var ca = document.cookie.split(';');
            for (var i = 0; i < ca.length; i++) {
                var c = ca[i].trim();
                if (c.indexOf(name) == 0) return c.substring(name.length, c.length);
            }

            return "";
        }

    },
    inArray: function(needle, haystack) {
        for (var i = 0; i < haystack.length; i++) {
            if (haystack[i] == needle) return true;
        }
        return false;
    },
    removeFromArray: function(needle, haystack) {
        var result = 0;

        for (var i = haystack.length - 1; i >= 0; i--) {
            if (haystack[i] === needle) {
                haystack.splice(i, 1);
                result++;
            }
        }

        return result;
    }

};


ron._ = ron.translate;


ron.JsonError = function() {
    this.errorText = '';
    this.errorField = '';
    this.errorDescription = '';
};

ron.JsonResult = function() {
    this.result = '';
    this.errors = null;
};
ron.showDialog = function(selector) {
    var $element = $(selector);
    if (!$element.length) return;

    var $dialog = $("<div/>")
        .append($element.clone());

    var $buttons = $("<div/>")
        .attr("class", "buttons align-center")
        .appendTo($dialog);


    var closeDialog = function() {
        $dialog.dialog("destroy");
    };


    $("<input/>")
        .attr("type", "button")
        .attr("class", "button")
        .attr("value", ron.translate("OK"))
        .appendTo($buttons)
        .on("click", closeDialog );


    $dialog.dialog( {
            modal : true,
            resizable: true,
            width: "auto",
            height: "auto"
        }
    );
};


ron.errorsDialog = function(selector) {
    ron.showDialog(selector);
};
ron.setCaretPosition = function(ctrl, pos) {
    if (!ctrl) return false;

    if (ctrl.setSelectionRange) {
        ctrl.focus();
        ctrl.setSelectionRange(pos, pos);
    }
    else if (ctrl.createTextRange) {
        var range = ctrl.createTextRange();
        range.collapse(true);
        range.moveEnd('character', pos);
        range.moveStart('character', pos);
        range.select();
    }

    return true;
};


ron.dialog = {
    showWaitMessage : function(dialog) {
        $(dialog).find(".dialogwaitmessage").show();
    },

    hideWaitMessage : function(dialog) {
        $(dialog).find(".dialogwaitmessage").hide();
    }
};
ron.showPopupMenu = function($menu, options) {
    return $menu.ronPopupMenu(options);
};


ron.floatingMenu = (function() {

    var FloatingMenuOptions = function() {
        this.$originalElement = null;
        this.$parentElement = null;
        this.itemSelector = "li";
    };
    return function ($menu, options) {
        var menuItemMouseOver = function () {
            if (options && options.$parentElement && options.$parentElement.is(".expanded")) return;

            var $this = $(this);

            var itemId = $this.data("ron-itemid");
            if (!itemId) {
                $(document).ronPopup("closeAll");
                return;
            }

            var subMenuId = $menu.attr('id') + '_' + itemId;
            $("#" + subMenuId).ronPopupMenu({
                attachToElement: $this,
                wrap: true
            });
        };

        $menu.on("mouseover", (options && options.itemSelector) ? options.itemSelector : "li", menuItemMouseOver);

        var onScroll = function () {
            if ((document.documentElement.scrollTop || document.body.scrollTop) > options.$originalElement.position().top) {
                (options.$parentElement ? options.$parentElement : $menu).toggleClass("visible", true);
            } else {
                (options.$parentElement ? options.$parentElement : $menu).toggleClass("visible", false);
            }

        };

        if (options && options.$originalElement) {
            $(window).on("scroll", onScroll);
        }
    }
})();
ron.logAjaxError = function(jqXHR, errorMsg) {
    if (!errorMsg) errorMsg = "Chyba!";

    console.log(errorMsg, jqXHR.status + ' - ' + jqXHR.statusText);
    if (debugMode) console.log(errorMsg, jqXHR.responseText);
};
$( function() {
    var goToHref = function($element, $target) {
        var href = $element.data("ron-href");

        if (!href && $element.is("a")) {
            href = $element.attr("href");
        }


        if (href) {
            var workingInfo = $element.data("ron-working-info");
            if (workingInfo) {
                ron.showWorkingInfo(workingInfo);
            }

            var sendPosition = $element.data("ron-send-position");
            if (sendPosition) {
                href += "&y=" + $(window).scrollTop() + "&x=" + $(window).scrollLeft();
            }

            if (!$target) {
                $target = $element.data("ron-target");
            }

            if ($target) {
                window.open(href, $target);
            } else {
                window.location = href;
            }
            return false; // stopne defaultni akci pro <a>
        }

        return true;
    };


    var clickHref = function($e) {
        if (this != $e.target  && $($e.target).closest("a").length) {
            return;
        }

        return goToHref($(this), $e.ctrlKey ? '_blank' : null);
    };


    var confirmQuestionClick = function(event) {
        var $this = $(this);

        var
            isHref = !!$this.data("ron-href")  ||  ($this.is("a")  &&  $this.attr("href")),
            callbackFunc = null;

        if (isHref) {
            callbackFunc = function() {
                return goToHref($this);
            };
        }

        var result = ron.confirmQuestion( $this, callbackFunc );

        if (!isHref  &&  !result) {
            event.stopImmediatePropagation();
            return false;
        }

        return result;
    };


    var actionButtonClick = function(event) {
        var $this = $(this);

        var callbackFunc = function() {
            var
                inputSelector = $this.data("input-selector");


            if (inputSelector) {
                var $input = $(inputSelector);
                $input.val($this.data("new-value"));
            }

            $this.closest("form").submit();
        };

        ron.confirmQuestion( $this, callbackFunc );
    };


    var clickWorkingInfo = function() {
        var workingInfo = $(this).data("ron-working-info");
        ron.showWorkingInfo(workingInfo);
    };


    var sendPositionSubmit = function() {
        $(this).children("[name='x']").val( $(window).scrollLeft() );
        $(this).children("[name='y']").val( $(window).scrollTop() );
    };


    var submitClick = function() {
        var $that = $(this);
        return ron.confirmQuestion($that);
    },


    onFormSubmit = function() {
        $(this).find(".submit-indicator").show();

        var workingInfo = $(this).data("ron-working-info");
        if (workingInfo) {
            ron.showWorkingInfo(workingInfo);
        }
    },


    onActualTimeButtonClick = function() {
        var
            $this = $(this),
            dt = new Date(),
            $date = $('#' + $this.data('date-input')),
            $time = $('#' + $this.data('time-input'));

        $date.val(dt.getDate() + "." + (dt.getMonth() + 1) + "." + dt.getFullYear());
        $time.val(
            ((dt.getHours() < 10) ? ("0" + dt.getHours()) : dt.getHours()) + ":" +
            ((dt.getMinutes() < 10) ? ("0" + dt.getMinutes()) : dt.getMinutes())
        );

    };


    $("body")
        .on("click", "input:submit", submitClick )
        .on("click", ".confirmQuestion", confirmQuestionClick)
        .on("click", ".actionButton", actionButtonClick)
        .on("click", ".clickHref", clickHref)
        .on("click", ".clickWorkingInfo", clickWorkingInfo)
        .on("submit", ".submitWorkingInfo", clickWorkingInfo)
        .on("contextmenu", ".popupmenu", function() { return false; })
        .on("submit", "form.sendPosition", sendPositionSubmit)
        .on("submit", "form", onFormSubmit)
        .on("click", ".actual-time-button", onActualTimeButtonClick)
        .on("click", "input.clickAutoSubmit", function() { $(this.form).submit(); })
        ;


    $(".js-show").removeClass("hidden");
    $(".js-hide").addClass("hidden");
    (function() {
        var dialogCancel = function() {
            $(this).closest(".rondialog").dialog("close");
        };

        $("body")
            .on("click", "input.cancel,  button.cancel", dialogCancel);
    })();

});






ron.showWorkingInfo = function(text) {
    var $element = $("#workingInfo");
    if ( !$element.length ) {
        $element = $("<div/>");

        $element
            .attr("class", "workingInfo")
            .attr("id", "workingInfo")
            .appendTo( document.body );

        if (!text) {
            $element.html( ron._("Probiha_zpracovani_dat") );
        }
    }

    if (text) $element.html( text );

    $element.css("display", "block");
};


ron.hideWorkingInfo = function() {
    $("#workingInfo").css("display", "none");
};



ron.showImage = function(url) {
    var $imgDialog = $("#ron_image");

    var closeImage = function() {
        $imgDialog.dialog("close");
    };

    if (!$imgDialog.length) {
        $imgDialog = $("<div/>")
            .attr("id", "ron_image")
            .on("click", closeImage );
    }

    $imgDialog.html("");

    $("<img/>")
        .attr("src", url)
        .appendTo($imgDialog);

    $imgDialog
        .dialog( {
            modal : true,
            resizable: false,
            width: "auto",
            height: "auto",
            dialogClass: "ronimage",
            beforeClose : function () { $(".ui-widget-overlay").off("click", closeImage); }
        });

    $imgDialog.parent()
        .off("click")
        .on("click", closeImage);

    $(".ui-widget-overlay")
        .on("click", closeImage);
};


function StopEventPropagation(event)
{
  if (!event) event = window.event;

  if (event.stopPropagation) event.stopPropagation();
  else event.cancelBubble = true;
}


function ChangeLocationQuest(event, question, url, showWorkInfo)
{
    if ( ron.confirm(question)) {
        if (showWorkInfo) ron.showWorkingInfo();
        window.location = url;
    }

    StopEventPropagation(event);
}
ron.setSameCellWidth = function(tables, tables2) {
    var $cells = [];
    for (var i = 0;  i < tables.length;  i++) {
        if (tables[i].is("tr")) {
            var $row = tables[i]; // je zadan primo radek
        } else {
            $row = tables[i].children("tr").first();
            if (!$row.length) {
                $row = tables[i].children("tbody").children("tr").first();
            }
        }
        if ( $row.length ) {
            $cells.push($row.children("td,th"));
        }
    }
    if (!$cells.length) return;
    var widths = [];
    for (var colIndex = 0; colIndex < $cells[0].length;  colIndex++) {
        widths[colIndex] = $cells[0].eq(colIndex).outerWidth();
    }
    for (var tableIndex = 1;  tableIndex < $cells.length;  tableIndex++) {
        for (colIndex = 0; colIndex < $cells[tableIndex].length;  colIndex++) {
            var width = $cells[tableIndex].eq(colIndex).outerWidth();
            if (width > widths[colIndex]) widths[colIndex] = width;
        }
    }
    for (tableIndex = 0;  tableIndex < $cells.length;  tableIndex++) {
        for (colIndex = 0; colIndex < $cells[tableIndex].length; colIndex++) {
            $cells[tableIndex].setCellWidths(widths);
        }
    }

    if (tables2) {
        for (i = 0; i < tables2.length; i++) {
            if (tables2[i].is("tr")) {
                $row = tables2[i]; // je zadan primo radek
            } else {
                $row = tables2[i].children("tr").first();
                if (!$row.length) {
                    $row = tables2[i].children("tbody").children("tr").first();
                }
            }

            if ($row.length) {
                $row.children("td,th").setCellWidths(widths);
            }
        }
    }

    var result = 0;
    for (i = 0;  i < widths.length;  i++) {
        result += widths[i];
    }
    return result;
};
function DateTimeToStr(date)
{
   if ( !date ) return '';
   
   var result = date.getDate() + '.' + (date.getMonth() + 1) + '.' + date.getFullYear();
      
   if (date.getHours()  ||  date.getMinutes()) 
   {
     result += ' ' + date.getHours() + ':';
     if (date.getMinutes() <= 9) result += '0'; 
     result += date.getMinutes();
   }
      
   return result;
}
function StringToDateTime(str)
{
    var parts = str.split(' '); // rozdeli datum a cas

    if ( parts.length == 0 ) return null;
    var dateparts = parts[0].split('.');
    var day = parseInt(dateparts[0], 10);
    var month = parseInt(dateparts[1], 10) - 1; // 0 - 11 !
    var year = parseInt(dateparts[2], 10);

    var now = new Date();
    if ( !day ) day = now.getDate();
    if ( !dateparts[1] ) month = now.getMonth();
    if ( !year ) year = now.getFullYear();

    var hour = 0;
    var minute = 0;
    var seconds = 0;
    if ( parts.length > 1 ) {
        var timeparts = parts[1].split(':');
        hour = parseInt(timeparts[0], 10);
        minute = parseInt(timeparts[1], 10);
        if (timeparts.length > 2) seconds = parseInt(timeparts[2], 10);
    }

    return new Date(year, month, day, hour, minute, seconds, 0);
}



function dateOf(date) {
    return new Date(date.getFullYear(), date.getMonth(), date.getDate());
}
function GetXMLResult(xml)
{
   var dataxml = xml.getElementsByTagName("result");
   if ( !dataxml.length ) return false;
   
   return dataxml[0].textContent ? dataxml[0].textContent : dataxml[0].text; // druha verze pro IE
}
function ZobrazChybyXML(xml, showwarnings) {
    if ( !xml ) return 0;

    var result = 0;
    if (showwarnings) {
        var dataxml = xml.getElementsByTagName("warning");
        for (var i = 0; i < dataxml.length; i++) {
            alert(dataxml[i].textContent ? dataxml[i].textContent : dataxml[i].text); // druha verze pro IE
        }
    }


    dataxml = xml.getElementsByTagName("error");
    for (i = 0; i < dataxml.length; i++) {
        alert(dataxml[i].textContent ? dataxml[i].textContent : dataxml[i].text); // druha verze pro IE
        result++;
    }

    return result;
}
function focusJsonErrorField(json, setCssClass)
{
    if (json.errors) {
        for (var i = 0;  i < json.errors.length;  i++ ) {
            if ( json.errors[i].errorField ) {
                var fields = $("[name='" + json.errors[i].errorField + "']");
                if (fields.length) {
                    fields.addClass('error');
                    fields.first().focus();
                    return;
                }
            }
        }
    }
}
ron.delayedLaunch = function(timeOut, callable, storeTimerInData) {
    let timer = null;

    return function() {
        let
            args = arguments,
            that = this,
            $this = null;

        if (storeTimerInData) {
            $this = $(this);
            timer = $this.data("delayed-launch-timer");
            if (timer) {
                clearTimeout(timer);
                $this.data("delayed-launch-timer", null);
            }
        } else {
            if (timer) {
                clearTimeout(timer);
                timer = null;
            }
        }

        timer = setTimeout(
            function() {
                timer = null;
                callable.apply(that, args);
            },
            timeOut
        );

        if ($this  &&  storeTimerInData) {
            $this.data("delayed-launch-timer", timer);
        }
    }
};



ron.OptionItem = function() {
    this.key = null;
    this.value = null;
    this.cssClass = null;
    this.title = null;
};
ron.loadOptions = function($select, data) {
    if (!data) return;

    $.each(data, function(i) {
        $select.append(
            $('<option></option>')
                .val(data[i].key)
                .text(data[i].value)
                .addClass(data[i].cssClass)
        );
    });
};
ron.loadOptionsFromUrl = function($select, url) {
    $select.empty();

    $.ajax(url).done( function(data) {
        if (data.result == "OK") {
            ron.loadOptions($select, data.options);
        } else {
            ron.showError(data);
        }
    }).fail( function(jqXHR) {
        ron.showAjaxError(jqXHR, "Pri nacitani zaznamu doslo k chybe!");
    });
};


var dialog = null;
function showDialog(element, initcallback) {
    var form_back = document.getElementById("dialog_formback");
    if ( !form_back ) {
        var formback = document.createElement("div");
        formback.className = "dialog_formback";
    }

    form_back.style.display = "block";

    if (initcallback) {
        form_back.style.cursor = 'wait';
        initcallback();
        form_back.style.cursor = 'auto';
    }

    element.style.display = "inline-block";

    dialog = element;
}

function hideDialog() {
    if (!dialog) return;

    dialog.style.display = 'none';
    var form_back = document.getElementById("dialog_formback");
    if ( form_back ) form_back.style.display = "none";

    dialog = null;
}

function isDialog() {
    return !!(dialog);
}







ron.showProgressScreen = function(text, maximalniHodnota, prerusitCallback) {
    var prerusit = false;
    var floatDiv = $("<div/>")
        .attr("id", "ron_progress")
        .attr("class", "floatingdiv")
        .attr("tabindex", "1")
        .on("keydown", function(e) { e.stopPropagation(); } ) // aby fungoval clipboard
        .appendTo(document.body)
        .focus();

    var progressText = $("<div/>")
        .attr("id", "ron_progress_text")
        .attr("class", "progresstext")
        .html( text )
        .appendTo(floatDiv);
    var progressInfoDiv = $("<div/>")
        .attr("class", "progressinfotext")
        .attr("id", "ron_progress_infotext")
        .appendTo(floatDiv);

    var teplomerDiv = $("<div/>")
        .attr("class", "progressbar")
        .appendTo(floatDiv);

    var teplomer = $("<div/>")
        .attr("class", "progressbarcontent")
        .attr("id", "ron_progress_teplomer")
        .css("width", "0")
        .appendTo(teplomerDiv);
    if (prerusitCallback) {
        var buttonsDiv = $("<div/>")
            .append( $("<br/>") )
            .append( $("<input/>")
                .attr("type", "button")
                .attr("class", "button")
                .attr("id", "ron_progress_prerusit")
                .val( ron._('Prerusit') )
                .on("click", prerusitCallback)
             )
            .append( $("<br/>") )
            .append( $("<br/>") )
            .appendTo(floatDiv);
    }
    var progressMsgDiv = $("<div/>")
        .attr("class", "progressmessage")
        .attr("id", "ron_progress_message")
        .css("overflow", "auto")
        .appendTo(floatDiv);


    ron.progressScreen = {
        setProgress: function (hodnota, text, infoText) {
            if (text) {
                progressText.html(text);
            }
            progressInfoDiv.html(infoText ? infoText : '');

            if (!maximalniHodnota) return false;

            var widthStr = Math.round(100 * hodnota / maximalniHodnota) + '%';
            teplomer.css("width", widthStr);

            return this;
        },
        setInfoMessage: function (infoText) {
            progressInfoDiv.html(infoText);

            return this;
        },
        addMessage: function(text)
        {
            $("<div/>")
                .html(text)
                .appendTo(progressMsgDiv);

            return this;
        },
        setButton: function(text, callback) {
            $("#ron_progress_prerusit")
                .val(text)
                .removeAttr("disabled")
                .off("click")
                .on("click", callback);

            return this;
        }

    };

    return ron.progressScreen;
};


$(function() {
    var
        filterKey = function(event, allowedChars, forbiddenChars) {
            if (event.altKey || event.ctrlKey) {
                return true;
            }
            if (!event.which || event.which < 32) {
                return true;
            }

            if (allowedChars) {
                return allowedChars.includes(event.key);
            }

            if (forbiddenChars) {
                return !forbiddenChars.includes(event.key);
            }

            return true;
        },
        keyPressFilter = function(event) {
            let
                $this = $(this),
                allowed = $this.data('allowed-chars'),
                forbidden = $this.data('forbidden-chars');

            return filterKey(event, allowed, forbidden);
        },
        keyPressFilterFloat = function(event) {
            return filterKey(event, "0123456789.,+-");
        },
        keyPressFilterTime = function(event) {
            return filterKey(event, "0123456789:-");
        },
        keyPressFilterDate = function(event) {
            return filterKey(event, "0123456789.-");
        },
        keyPressFilterInt = function(event) {
            return filterKey(event, "0123456789+-");
        },


        keyDownDateInput = function(event) {
            let addDay = function ($input, count) {
                let
                    date = ron.utils.stringToDate($input.val());

                if (!date) return;

                date = ron.utils.incDay(date, count);
                $input.val(ron.utils.dateToString(date))
            };


            if (event.ctrlKey) {
                switch (event.which) {
                    case 38 : // up
                        addDay($(this), 1);
                        return false;

                    case 40 : // down
                        addDay($(this), -1);
                        return false;
                }
            }

            return true;
        },
        changeDateInput = function() {
            let
                $this = $(this),
                dateStr = $this.val();

            if (dateStr) {
                let
                    date = ron.utils.stringToDate(dateStr);

                $this.toggleClass('error', !date || isNaN(date.getTime()));
            }
        },
        changeTimeInput = function() {
            let
                $this = $(this),
                timeStr = $this.val();

            if (timeStr) {
                let
                    time = ron.utils.stringToTime(timeStr);

                $this.toggleClass('error', !time  ||  isNaN(time.getTime()));
            }
        },


        comboBoxChange = function() {
            var
                $this = $(this),
                select2 = $this.data('ron-select2');

            if (select2) {
                if (select2.charAt(0) != '#') {
                    select2 = "[name=" + select2 + "]";
                }

                var
                    $select2 = $(select2),
                    oldVal = $select2.val();

                if (oldVal != $this.val()) {
                    $(select2)
                        .val($this.val())
                        .trigger("change");
                }
            }

            if ($this.data('ron-autosubmit')) {
                var checkboxSelector = $this.data('ron-autosubmit-check');
                if (checkboxSelector) {
                    if (!$(checkboxSelector).prop("checked")) {
                        return;
                    }
                }

                $(this.form).submit(); // pres jQuery se vola on("submit"), jinak ne
            }
        },
        comboBoxMoveClick = function() {
            var
                $this = $(this),
                $select = $($this.data("selector")),
                value = $this.data("value"),
                optionSelector = $this.data("option-selector"),
                change = $this.data("step") ? parseInt($this.data("step")) : 1,
                submit = $this.data("submit") == "1",
                triggerChange = $this.data("trigger-change") == "1",
                $options = $select.children("option"),
                changed = false;

            if (value || value == "") {
                if (value != $select.val()) {
                    $select.val(value);
                    changed = true;
                }
            } else if (optionSelector) {
                $options = $select.find(optionSelector);
                if ($options.length) {
                    $select.val($options.attr("value"));
                    changed = true;
                }
            } else {
                var newindex = $select.prop("selectedIndex") + change;
                if (newindex < 0) newindex = 0;
                if (newindex >= $options.length) newindex = $options.length - 1;

                if (newindex != $select.prop("selectedIndex")) {
                    $select.prop("selectedIndex", newindex);
                    changed = true;
                }
            }

            if (changed) {
                if (triggerChange) {
                    $select.trigger("change");
                }

                if (submit) {
                    $select.closest("form").submit();
                }
            }
        },
        setValueClick = function() {
            var
                $this = $(this),
                $input = $($this.data("selector")),
                value = $this.data("value"),
                submit = $this.data("submit") == "1",
                triggerChange = $this.data("trigger-change") == "1",
                changed = false;

            if (value || value == "") {
                if (value != $input.val()) {
                    $input.val(value);
                    changed = true;
                }
            }

            if (changed) {
                if (triggerChange) {
                    $input.trigger("change");
                }

                if (submit) {
                    $input.closest("form").submit();
                }
            }
        },


        onColorInputChange = function() {
            var
                $this = $(this),
                indicatorSelector = $this.data("indicator");

            if (indicatorSelector) {
                $(indicatorSelector).attr("style", "background-color: " + $this.val() + " !important");
            }
        };



    $("body")
        .on("keypress", "input.inputDate", keyPressFilterDate)
        .on("keypress", "input.inputTime", keyPressFilterTime)
        .on("keypress", "input.inputFloat", keyPressFilterFloat)
        .on("keypress", "input.inputInt", keyPressFilterInt)
        .on("keypress", ".keyFilter", keyPressFilter)
        .on("keydown", "input.inputDate", keyDownDateInput)
        .on("change", "input.inputDate", changeDateInput)
        .on("change", "input.inputTime", changeTimeInput)
        .on("change", "select", comboBoxChange)
        .on("click", ".comboBoxMove", comboBoxMoveClick)
        .on("click", ".setValue", setValueClick)
        .on("change", ".color-input", onColorInputChange)
        ;


    setTimeout(function() {
            $(".defaultfocus").focus();     // nevola se hned, aby se provedla pripadna obsluha onfocus, ktera muze byt nastavena pozdeji
        },
        200
    );
});


(function($) {
    $.fn.toggleClassWithSiblings = function (className, state) {
        var
            $element = $(this),
            rowspanIndex = $element.data("rowspan-index");

        $element.toggleClass(className, state);

        if (rowspanIndex !== undefined) {
            var
                $prev = $element;

            for (var index = rowspanIndex - 1; index >= 0; index--) {
                $prev = $prev.prev();
                $prev.toggleClass(className, state);
            }
            var
                cnt = $prev.data("rowspan-count"),
                $next = $element;

            for (index = rowspanIndex + 1; index < cnt; index++) {
                $next = $next.next();
                $next.toggleClass(className, state);
            }
        }

        if (rowspanIndex !== undefined) {

        }
    };
})(jQuery);


$(function() {

    var
        toggleDisabled = function () {
            var
                $this = $(this);
            var $elements = $this;
            if ($this.is("input[type=radio]")) {
                if ($this.data("parent-selector")) {
                    $elements = $($this.data("parent-selector")).find("input[name=" + $this.attr("name") + "]");
                } else {
                    $elements = $this.parents("form").find("input[name=" + $this.attr("name") + "]");
                }
            }

            $elements.each(function () {
                var
                    $this = $(this), // checkbox/radio
                    $element = $($this.data("selector"));

                $element.prop("disabled", !$this.prop("checked"));
            });
        },
        toggleClass = function () {
            var
                $this = $(this),
                $element = $this;

            if ($this.data("selector")) {
                $element = $($this.data("selector"));
            }

            if ($this.data("add-class")) {
                $element.addClass($this.data("add-class"));
            }

            if ($this.data("remove-class")) {
                $element.removeClass($this.data("remove-class"));
            }

            if ($this.data("toggle-class")) {
                $element.toggleClass($this.data("toggle-class"));
            }
            var
                $elements = $this;

            if ($this.is("einput[type=radio]")) {
                if ($this.data("parent-selector")) {
                    $elements = $($this.data("parent-selector")).find("input[name=" + $this.attr("name") + "]");
                } else {
                    $elements = $this.closest("form").find("input[name=" + $this.attr("name") + "]");
                }
            }

            $elements.each(function () {
                var
                    $this = $(this), // checkbox/radio
                    $element = $this;

                if ($this.data("selector")) {
                    $element = $($this.data("selector"));
                }
                if ($this.data("check-class")) {
                    $element.toggleClass($this.data("check-class"), $this.prop("checked"));
                }
                if ($this.data("uncheck-class")) {
                    $element.toggleClass($this.data("uncheck-class"), !$this.prop("checked"));
                }
                if ($this.data("not-empty-class")) {
                    console.log($this.val());
                    $element.toggleClass($this.data("not-empty-class"), !!$this.val())
                }
                if ($this.data("empty-class")) {
                    $element.toggleClass($this.data("empty-class"), !$this.val())
                }
            });
        },

        autoExpand = function() {
            $(this).toggleClass("expanded");
        };



    $("body")
        .on("click", ".toggleClass", toggleClass)
        .on("click", ".toggleDisabled", toggleDisabled)
        .on("click", ".autoExpand", autoExpand)
        ;

    $(function() {
        $(".toggleOnLoad").each(toggleClass);
    });
});

$(function() {


    (function() {
        var
            $autoRefreshElements = $(".auto-refresh"),
            autoRefreshTimeout = null,
            autoRefreshUrl = null;
        $autoRefreshElements.each(function () {
            var
                $this = $(this),
                timeout = $this.data("timeout"),
                url = $this.data("url");

            if (timeout && url && (!autoRefreshTimeout || timeout < autoRefreshTimeout)) {
                autoRefreshTimeout = timeout;
                autoRefreshUrl = url;
            }
        });

        if (autoRefreshTimeout  &&  autoRefreshUrl) {
            console.log("Setting automatic refresh timeout", autoRefreshTimeout + "s", autoRefreshUrl);

            setTimeout(
                function() { location.replace(autoRefreshUrl); },
                1000 * autoRefreshTimeout);
        }
    })();


    var
        sendPositionLinkClick = function() {
            var href = $(this).attr("href");
            if (!href) return true;

            href += "&y=" + $(window).scrollTop() + "&x=" + $(window).scrollLeft();
            window.location.replace(href);
            return false;
        };


    $("body")
        .on("click", "a.sendPosition", sendPositionLinkClick)
        ;

});

(function() {
    $.fn.setCellWidths = function(widths) {
        var
            $cells = this,
            lastColspan = 0,
            cellIndex = 0,
            width = 0;

        for (var i = 0;  cellIndex < $cells.length  &&  i < widths.length;  i++) {
            if (lastColspan > 0) {
                width += widths[i];
                lastColspan --;

                if (!lastColspan) {
                    $cells.eq(cellIndex)
                        .outerWidth(width + "px")
                        .css("min-width", width + "px"); // bez toho to nekdy nefunguje (napr. soucty v paticce planovani kapacit)
                    width = 0;
                }
            } else {
                var $cell = $cells.eq(cellIndex);
                var colSpan = $cell.prop("colspan");
                if (colSpan > 1) {
                    lastColspan = colSpan - 1;
                    width = widths[i];
                } else {
                    $cells.eq(cellIndex).outerWidth(widths[i] + "px");
                    cellIndex++;
                }
            }
        }
    };

})(jQuery);


$(function() {

    var
        rowBodySelectClick = function() {
            $(this).toggleClass("browserselectedrow");
        },
        moveInTableVertical = function(sender, up) {
            var
                $cell = $(sender).closest("td");

            if (!$cell.length) {
                return false;
            }

            var
                $row = $cell.parent("tr");

            if (!$row.length) {
                return false;
            }

            while ($row.length) {
                $row = up ? $row.prev("tr") : $row.next("tr");

                var
                    $newcell = $row.children().eq($cell.index()),
                    $inputs = $newcell.find(sender.tagName + ":visible");

                if ($inputs.length) {
                    $inputs.eq(0).focus().select();
                    return true;
                }
            }

            return false;
        },
        keyDownVerticalMove = function(event) {
            if (event.altKey || event.ctrlKey) {
                return true;
            }

            switch (event.which) {
                case 38 : // up
                    return !moveInTableVertical(this, true);

                case 40 : // down
                    return !moveInTableVertical(this, false);
            }

            return true;
        };



    $("body")
        .on("dblclick", ".rowSelect>tr, .rowSelect>tbody>tr", rowBodySelectClick)
        .on("dblclick", ".bodySelect>tbody", rowBodySelectClick)
        .on("keydown", "table.keyInputMove input:text, table.keyInputMove input:checkbox", keyDownVerticalMove)
        ;

});



(function($) {
    $.fn.dndTarget = function() {
        return this.each(function() {
            var
                $that = $(this),
                counter = 0;

            var onDragEnter = function() {
                counter++;
                $that.addClass("dndTargetActive");
            };

            var onDragLeave = function() {
                if (counter > 0) counter--;
                if (!counter) {
                    $that.removeClass("dndTargetActive");
                }
            };

            var onDrop = function() {
                $that.removeClass("dndTargetActive");
                counter = 0;
            };

            $that
                .on("dragenter", onDragEnter )
                .on("dragleave", onDragLeave )
                .on("drop", onDrop)
                .on("dragover", function() { /*console.log($e);*/ return false; } )
        });
    }
})(jQuery);


(function($) {

    var createReplaceDiv = function($element) {
        return $("<div/>")
            .css("width", $element.outerWidth() + "px")
            .addClass("d-inline-block")
            .html('&nbsp;');
    };


    $.fn.ensureVisible = function(options) {
        var defaultOptions = {
            direction: "vertical",

            topWindowMargin: 30,
            $parentElement: null,
            $scrollElement: null
        };

        options = $.extend( defaultOptions, options );

        $(this).each(function() {
            var
                $element = $(this),
                $window = $(window),
                $parentElement = options.$parentElement || $element.parent(),
                $replaceDiv = null,

                fixedTop = false,
                fixedBottom = false,
                lastScrollTop = 0,
                lastScrollLeft = 0,
                originalLeft = 0
                ;

            if ($element.css("position") == "fixed") {
                return;
            }

            var
                showReplaceDiv = function() {
                    if ($replaceDiv) {
                        $replaceDiv.show();
                    } else {
                        $replaceDiv = createReplaceDiv($element);
                        $replaceDiv.insertBefore($element);
                        $element.parent().css("overflow", "hidden");
                    }
                },

                hideReplaceDiv = function() {
                    if ($replaceDiv) {
                        $replaceDiv.hide();
                    }
                },

                handleScroll = function () {
                    var
                        elementHeight = $element.outerHeight(),
                        parentHeight = $parentElement.height();

                    if (!fixedTop && !fixedBottom && elementHeight + 50 >= parentHeight) {
                        return;
                    }

                    if (!$element.is(":visible")) {
                        return;
                    }

                    if ($element.height() >= $parentElement.height()) {
                        return;
                    }

                    var
                        scrollTop = $window.scrollTop(),
                        scrollLeft = $window.scrollLeft(),
                        pos = $element.offset(),
                        parentPos = $parentElement.offset();

                    if (scrollTop > lastScrollTop) {
                        if (fixedBottom) {
                            if (pos.top + elementHeight >= parentPos.top + parentHeight) {
                                $element
                                    .css("position", "absolute")
                                    .css("top", "auto")
                                    .css("left", "auto")
                                    .css("bottom", 0);

                                fixedBottom = false;
                            }
                        } else if (fixedTop) {
                            $element
                                .css("position", "relative")
                                .css("top", pos.top - parentPos.top + "px")
                                .css("left", "auto")
                                .css("bottom", "auto")
                                .css("width", "auto");

                            hideReplaceDiv();

                            fixedTop = false;
                        } else {
                            var
                                windowHeight = $window.height();
                            if (parentPos.top < scrollTop && parentPos.top + parentHeight > scrollTop + windowHeight && pos.top + elementHeight < scrollTop + windowHeight) {
                                var
                                    pomRect = ron.utils.getAbsoluteRect($element),
                                    originalWidth = $element.outerWidth();

                                originalLeft = pomRect.x1;

                                if (windowHeight - options.topWindowMargin > elementHeight) {
                                    showReplaceDiv();
                                    $element
                                        .css("position", "fixed")
                                        .css("left", (originalLeft - scrollLeft) + "px")
                                        .css("top", options.topWindowMargin + "px")
                                        .css("width", originalWidth + "px")
                                        .css("bottom", "auto");
                                } else {
                                    showReplaceDiv();
                                    $element
                                        .css("position", "fixed")
                                        .css("left", (originalLeft - scrollLeft) + "px")
                                        .css("top", "auto")
                                        .css("width", originalWidth + "px")
                                        .css("bottom", 0);
                                }
                                fixedBottom = true;
                            }
                        }
                    } else if (scrollTop < lastScrollTop) {

                        if (fixedTop) {
                            if (pos.top <= parentPos.top) {
                                $element
                                    .css("position", "static")
                                    .css("top", "auto")
                                    .css("bottom", "auto")
                                    .css("left", "auto")
                                    .css("width", "auto");

                                hideReplaceDiv();

                                fixedTop = false;
                            }
                        } else if (fixedBottom) {
                            $element
                                .css("position", "relative")
                                .css("top", pos.top - parentPos.top)
                                .css("bottom", "auto")
                                .css("left", "auto")
                                .css("width", "auto");

                            hideReplaceDiv();

                            fixedBottom = false;
                        } else {
                            if (parentPos.top - scrollTop < 0 && pos.top - options.topWindowMargin >= scrollTop) {
                                pomRect = ron.utils.getAbsoluteRect($element);
                                originalLeft = pomRect.x1;
                                originalWidth = $element.outerWidth();

                                showReplaceDiv();

                                $element
                                    .css("position", "fixed")
                                    .css("left", (originalLeft - scrollLeft) + "px")
                                    .css("top", options.topWindowMargin)
                                    .css("width", originalWidth + "px")
                                    .css("bottom", "auto");

                                fixedTop = true;
                            }
                        }
                    }

                    lastScrollTop = scrollTop;

                    if (fixedTop || fixedBottom && lastScrollLeft != scrollLeft) {
                        $element.css("left", (originalLeft - scrollLeft) + "px");
                    }

                    lastScrollLeft = scrollLeft;
                },

                timer = null,

                onScroll = function() {
                    if (timer) return;

                    timer = setTimeout( function() {
                        timer = null;
                        handleScroll();
                    }, 50);
                };
            var $scrollElement = options.$scrollElement || $window;

            $scrollElement.on("scroll", onScroll);
        });
    }
})(jQuery);


$(function() {
    $(".ensure-visible").ensureVisible();
});


ron.floatingTableCols = function() {
    var floating_cols = [];

    var FloatingTableCol = function(table) {

        this.$div = null;
        this.$table = $(table);

        this.baseposition = null; // absolutni pozice zakladniho prvku v dokumentu (table)
        this.tableright = 0; // prava hrana tabulky - za ni se uz hlavicka nezobrazi
        this.createHeaderForCell = function ($cell) {
            if (!$cell.length) return;

            var pos = $cell.offset();
            pos.top -= this.baseposition.top;
            pos.left -= this.baseposition.left;
            var $div1 = $("<div/>")
                .addClass("floatingheadercontainer")
                .css("top", pos.top + "px")
                .css("left", pos.left + "px")
                .css("height", ($cell.height() + parseInt($cell.css('padding-top'))) + "px")
                .css("width", $cell.outerWidth())
                .css("text-align", $cell.css("text-align"))
                .appendTo(this.$div);

            $("<div/>")
                .attr("class", "floatingheadercell")
                .html($cell.html())
                .appendTo($div1);
        };


        this.createCells = function () {
            this.baseposition = this.$table.offset();
            this.tableright = this.baseposition.left + this.$table.width();

            this.$div
                .offset({
                    left: this.$div.offset().left,
                    top: this.baseposition.top
                })
                .height(this.$table.height());

            var $rows = this.$table.find("tr"); // nepouzivam each() kvuli this

            var colCount = parseInt(this.$table.data("ron-fixed-cols"));
            if (!colCount) colCount = 1;
            var width = 0;
            for (var colNum = 1;  colNum <= colCount; colNum++) {
                var rowSpan = 0;

                for (var i = 0; i < $rows.length; i++) {
                    if (rowSpan > 0) {
                        rowSpan --;
                    } else {
                        var $cell = $($rows[i]).find("td, th").filter(":nth-child(" + colNum + ")");
                        if (!i) {
                            width += $cell.outerWidth();
                        }
                        this.createHeaderForCell($cell);

                        rowSpan = $cell.attr('rowspan') - 1;
                    }
                }
            }

            this.$div.width(width);
        };
        this.doCreateHeader = function () {
            this.$div = $("<div/>")
                .attr("class", "floatingheadercol")
                .css("display", "none")
                .appendTo($(document.body));

            this.createCells();
        };


        this.createHeader = function () {
            this.doCreateHeader();
            floating_cols.push(this);
        };

        this.reCreateHeader = function () {
            this.$div.remove();

            this.$div = null;

            this.doCreateHeader();
        };


        this.onScroll = function () {
            if ((document.documentElement.scrollLeft || document.body.scrollLeft) > this.tableright - 40 || (document.documentElement.scrollLeft || document.body.scrollLeft) <= this.baseposition.left) {
                this.$div
                    .css("display", "none");
            }
            else {
                var left = 0;
                this.$div
                    .css("left", 0)
                    .css("top", (this.baseposition.top - (document.documentElement.scrollTop  ||  document.body.scrollTop)) + "px")
                    .css("display", "inline");
            }
        };
        this.createHeader();
    };


    $(window)
        .on("scroll", function () {
            for (var i = 0; i < floating_cols.length; i++) {
                floating_cols[i].onScroll();
            }
        });
    return function($obj) {
        $obj.each(function () {
            new FloatingTableCol(this);
        });
    }
}();



$(window).on("load", function () {
    ron.floatingTableCols( $("table.fixedCols") );
});

ron.floatingTableHeaders = function() {
    var floating_headers = [];
    var FloatingTableHeader = function (element) {
        this.$element = $(element);
        this.$div = null;

        this.$table = this.$element.closest("table"); // nadrazena tabulka

        this.baseposition = null; // absolutni pozice zakladniho prvku v dokumentu (thead)
        this.tablebottom = 0; // spodni hrana tabulky - pod ni se uz hlavicka nezobrazi
        this.createHeaderForCell = function (cell) {
            var
                $cell = $(cell),
                pos = $cell.offset();

            pos.left -= this.baseposition.left;
            var
                $div1 = $("<div/>")
                    .addClass("floatingheadercontainer")
                    .css("left", pos.left + "px")
                    .css("width", ($cell.width() + parseInt($cell.css('padding-right'))) + "px")
                    .height(this.$element.height())
                    .css("text-align", $cell.css("text-align"))
                    .appendTo(this.$div);

            $("<div/>")
                .attr("class", "floatingheadercell")
                .html($cell.html())
                .appendTo($div1);
        };


        this.createCells = function () {
            this.baseposition = this.$element.offset();

            if (this.$table) {
                var tablepos = this.$table.offset();
                this.tablebottom = tablepos.top + this.$table.height();
            }

            this.$div
                .offset({
                    top: this.$div.offset().top,
                    left: this.baseposition.left
                })
                .width(this.$element.width())
                .height(this.$element.height());

            var $cells = this.$element.find("th, td"); // nepouzivam each() kvuli this
            for (var i = 0; i < $cells.length; i++) {
                this.createHeaderForCell($cells[i]);
            }
        };
        this.doCreateHeader = function () {
            this.$div = $("<div/>")
                .attr("class", "floatingheader")
                .css("display", "none")
                .appendTo($(document.body));

            this.createCells();
        };


        this.createHeader = function () {
            this.doCreateHeader();
            floating_headers.push(this);
        };

        this.reCreateHeader = function () {
            this.$div.remove();

            this.$div = null;

            this.doCreateHeader();
        };


        this.onScroll = function () {
            this.baseposition = this.$element.offset();

            if ((document.documentElement.scrollTop || document.body.scrollTop) > this.tablebottom - 40 || (document.documentElement.scrollTop || document.body.scrollTop) <= this.baseposition.top) {
                this.$div
                    .css("display", "none");
            }
            else {
                var top = 0;
                $(".fixedheader").each( function() {
                    var $thisFixedHeader = $(this);
                    if ($thisFixedHeader.is(":visible")) {
                        var headerBottom = $thisFixedHeader.position().top + $thisFixedHeader.get(0).clientHeight;
                        if (headerBottom > top) top = headerBottom;
                    }
                } );
                this.$div
                    .css("top", top + "px")
                    .css("left", (this.baseposition.left - (document.documentElement.scrollLeft  ||  document.body.scrollLeft)) + "px") // left se muze menit pri horizontalnim scrolovani
                    .css("display", "inline");
            }
        };
        this.createHeader();
    };
    var findFloatingHeaderForTable = function (table) {
        for (var i = 0; i < floating_headers.length; i++) {
            if (floating_headers[i].$table.get(0) == table) return floating_headers[i];
        }

        return null;
    };


    window.ReCreateFloatingHeaderForTable = function(table)
    {
        var header = findFloatingHeaderForTable(table);
        if (header) header.reCreateHeader();
    };



    var recreateFloatingHeaders = function() {
        for (var i = 0; i < floating_headers.length; i++) {
            floating_headers[i].reCreateHeader();
        }
    };


    window.ReCreateFloatingHeaders = recreateFloatingHeaders; // kompat. - pouziva se v XML


    $(window)
        .on("scroll", function () {
            for (var i = 0; i < floating_headers.length; i++) {
                floating_headers[i].onScroll();
            }
        })
        .on("resize", function() {
            recreateFloatingHeaders();
        });
    return function($obj) {
        $obj.each(function () {
            new FloatingTableHeader(this);
        });
    };

}();


$(window).on("load", function () {
    ron.floatingTableHeaders( $(".floating-table-header-js") );
});

(function() {
    var
        $rootElement = null,
        $items = null,
        $thumbPanel = null,
        $fotoContainer = null,
        thumbnails = [],
        actualImage = 0;


    var
        selectThumbnail = function(index) {
            if (!thumbnails  ||  !thumbnails.length) {
                return;
            }

            if (index >= 0  &&  index < thumbnails.length) {
                if (actualImage >= 0) {
                    thumbnails[actualImage].toggleClass("selected", false);
                }

                thumbnails[index].toggleClass("selected", true);
                var newPos = $thumbPanel.width() / 2 - thumbnails[index].width() / 2;
                var newScroll = ( $thumbPanel.scrollLeft() + thumbnails[index].position().left - newPos );
                if (newScroll < 0) newScroll = 0;
                $thumbPanel.scrollLeft( newScroll );
            } else {
                console.log("selectThumbnail - spatny index: ", index);
            }
        },
        showVideo = function(src) {
            $("<video/>")
                .attr("width", 640)
                .attr("height", 480)
                .attr("controls", "controls")
                .append(
                $("<source/>")
                    .attr("src", src)
            )
                .appendTo($fotoContainer);
        },
        showImage = function(src) {
            $("<img/>")
                .attr("src", src)
                .appendTo($fotoContainer);
        },
        showLink = function(src) {
            $("<a/>")
                .attr("href", src)
                .attr("target", "_blank")
                .text(src)
                .appendTo($fotoContainer);
        },
        showFile = function(index) {
            $fotoContainer.empty();

            var
                $item = $($items[index]),
                src = $item.data("ron-url"),
                type = $item.data("ron-type") || "image";

            switch (type) {
                case "video" :
                    showVideo(src);
                    break;

                case "image" :
                    showImage(src);
                    break;

                default :
                    showLink(src);
                    break;
            }

            actualImage = index;
        },

        close = function() {
            $items = null;

            $rootElement.remove();
            $rootElement = null;
        },

        onThumbnailClick = function($e) {
            var $thumbnail = $(this); // $(this);

            selectThumbnail($thumbnail.data("ron-index"));

            showFile( $thumbnail.data("ron-index") );
        },


        afterDeleteCallback = function() {
            thumbnails[actualImage]
                .data("ron-deleted", "1")
                .empty()
                .append(
                    $("<i/>")
                        .addClass("fa inline-glyph glyph-delete")
                );
        },


        onKeyDown = function($e) {
            if (!$items) {
                return;
            }
            switch ($e.which) {
                case 27 :
                    close();
                    break;

                case 37 :
                    if (actualImage > 0) {
                        selectThumbnail(actualImage - 1);
                        showFile(actualImage - 1);
                    }
                    break;

                case 39 :
                    if (actualImage < thumbnails.length - 1) {
                        selectThumbnail(actualImage + 1);
                        showFile(actualImage + 1);
                    }
                    break;

                case 46 :
                    $($items[actualImage]).trigger("delete-media", afterDeleteCallback);
                    break;

                default :
            }
        };

    var
        thumbnailsDragStart = null,

        onThumbnailsMouseDown = function($e) {
            if ($e.which == 1) {
                var $this = $(this);
                thumbnailsDragStart = $e.pageX - this.offsetLeft; // + $this.scrollLeft();
                return false;
            } else {
                thumbnailsDragStart = null;
            }
        },

        onThumbnailsMouseUp = function($e) {
            thumbnailsDragStart = null;
        },

        onThumbnailsDrag = function($e) {
            if (thumbnailsDragStart  &&  $e.buttons == 1) {
                var $this = $(this);
                var newPos = $e.pageX - this.offsetLeft; // + $this.scrollLeft();
                $this.scrollLeft( $this.scrollLeft() + thumbnailsDragStart - newPos );
                thumbnailsDragStart = newPos;
                return false;
            } else if (thumbnailsDragStart) {
                thumbnailsDragStart = null;
            }
        };


    $.fn.mediaViewer = function(options) {
        var
            $this = $(this),
            defaultOptions = {
                initialImage: null,
                initialItem: null,
                thumbnailsHeight: 100
            };

        $items = $this.find(".media-item");

        if ($this.data("thumbnails-height")) {
            defaultOptions.thumbnailsHeight = $this.data("thumbnails-height");
        }

        options = $.extend({}, defaultOptions, options);

        if (options.initialImage != null) {
            actualImage = options.initialImage;
        } else if (options.initialItem) {
            actualImage = $items.index(options.initialItem);
        } else {
            actualImage = 0;
        }

        if (!actualImage  ||  actualImage < 0  ||  actualImage >= $items.length) {
            actualImage = 0;
        }


        $rootElement =
            $("<div/>")
                .attr("class", "fotoViewer")
                .appendTo( $(document.body) );

        $fotoContainer = $("<div/>")
            .attr("class", "fotoViewerContainer")
            .appendTo($rootElement);



        $thumbPanel =
            $("<div/>")
                .attr("class", "fotoViewerThumbnailsPanel")
                .on("mousedown", onThumbnailsMouseDown)
                .on("mouseup", onThumbnailsMouseUp)
                .on("mousemove", onThumbnailsDrag)
                .appendTo($rootElement);

        var $thumbnails =
            $("<div/>")
                .attr("class", "fotoViewerThumbnails")
                .on("click", ".fotoViewerThumbnail", onThumbnailClick)
                .appendTo($thumbPanel);


        var showThumbnails = function() {
            thumbnails = [];
            for (var i = 0;  i < $items.length;  i ++) {
                var
                    $item = $($items[i]),
                    src = $item.data("ron-url"),
                    type = $item.data("ron-type") || "image",
                    $container =
                        $("<div/>")
                            .attr("class", "fotoViewerThumbnail" + ( (i == actualImage) ? " selected" : "" ) )
                            .data("ron-index", i)
                            .appendTo($thumbnails)
                    ;



                switch (type) {
                    case "image" :
                        $("<img/>")
                            .attr("src", src)
                            .appendTo($container);
                        break;

                    case "video" :
                        $("<i/>")
                            .addClass("fa fa-3x glyph-video")
                            .appendTo($container);
                        break;

                    default :
                        $("<span/>")
                            .text("...")
                            .appendTo($container);
                }

                thumbnails[thumbnails.length] = $container;
            }
        };

        showFile(actualImage);
        showThumbnails();
    };

    var showMediaTimer = null;

    var mediaListClick = function($e) {
        if (showMediaTimer) {
            clearTimeout(showMediaTimer);
            showMediaTimer = null;
        }

        var $this = $(this);

        showMediaTimer = setTimeout( function() {
            showMediaTimer = null;

            $this.mediaViewer( {
                initialItem: $e.target
            });
        }, 300 );
    };

    var mediaListDblClick = function() {
        if (showMediaTimer) {
            clearTimeout(showMediaTimer);
            showMediaTimer = null;
        }
    };

    $(function() {
        $(document.body)
            .on("click", ".media-list", mediaListClick)
            .on("dblclick", ".media-list", mediaListDblClick)
            .on("keydown", onKeyDown);
    });
})();


(function() {
    var
        $dialog = null,
        $messageDiv = null;

    var createDialog = function() {
        $dialog = $("<div/>")
            .addClass("rondialog");

        $messageDiv = $("<div/>")
            .addClass("max-height-80vh")
            .addClass("overflow-auto")
            .appendTo($dialog);

        $("<div/>")
            .addClass("buttons")
            .append($("<button/>")
                .text("OK")
                .addClass("submitbutton")
                .addClass("cancel")
        )
            .appendTo($dialog);
    };

    ron.showMessage = function(options) {
        if (typeof options == "string") {
            if (options == "close") {
                if ($dialog) {
                    $dialog.dialog("close");
                }
                return;
            }
        }

        var defaultOptions = {
            title: document.title,
            content: null
        };

        if (typeof options == "string") {
            defaultOptions.content = $("<div/>").text(options);
            options = defaultOptions;
        } else if (typeof options == "object") {
            options = $.extend(defaultOptions, options);
        }

        if (!$dialog) {
            createDialog();
        }

        $messageDiv.empty();
        if (options.content) {
            $messageDiv.append(options.content);
        }

        $dialog.attr("title", options.title);

        $dialog.dialog( {
            modal: true,
            minHeight: "50px",
            position: { my: "center", at: "center", of: window },
            width: "auto"
        });

    };
})();


ron.hideMessage = function() {
    ron.showMessage("close");
};
    ron.showError = function (error, element, cssClass) {
        if (typeof(error) === "object" && (!error.errors || !error.errors.length)) return;

        if (cssClass === undefined) cssClass = "errortext";

        if (element) {
            var $element = $(element);
            $element.html("");

            if (error && error.errors) {
                for (var i = 0; i < error.errors.length; i++) {
                    $element.append($("<div/>").attr("class", cssClass).html(ron.utils.lineBreaksToBr(error.errors[i].errorText)));
                    if (error.errors[i].errorDescription) console.log(error.errors[i].errorText, error.errors[i].errorDescription);
                    else console.log(error.errors[i].errorText);
                }
            } else if (error) {
                $element.append($("<div/>").attr("class", cssClass).html(ron.utils.lineBreaksToBr(error)));
                console.log(error);
            }

        } else {
            var $content = $("<div/>");

            if (error && error.errors) {
                for (i = 0; i < error.errors.length; i++) {
                    var messages = error.errors[i].errorText.split("\n");
                    for (var msgIndex = 0; msgIndex < messages.length;  msgIndex++) {
                        if (messages[msgIndex]) {
                            $content.append($("<div/>").text(messages[msgIndex]).addClass(cssClass));
                        }
                    }
                    if (error.errors[i].errorDescription) {
                        console.log(error.errors[i].errorText, error.errors[i].errorDescription);
                    } else {
                        console.log(error.errors[i].errorText);
                    }
                }
            } else if (error) {
                $content.append(
                    $("<div/>")
                        .text(error)
                        .addClass(cssClass)
                );
                console.log(error);
            }

            ron.showMessage({
                title: ron._("Chyba"),
                content: $content
            });
        }
    };
ron.showAjaxError = function(jqXHR, errorMsg) {
    if (!errorMsg) errorMsg = "Chyba!";

    console.log( errorMsg, jqXHR.responseText);

    if (debugMode) {
        ron.showError(errorMsg + " (" + jqXHR.status + " - " + jqXHR.statusText + ")\n" + jqXHR.responseText);
    } else {
        ron.showError(errorMsg + " (" + jqXHR.status + " - " + jqXHR.statusText + ")" );
    }
};


ron.ajaxErrorFunction = function(errorMessage) {
    return function(jqXHR) {
        ron.showAjaxError(jqXHR, errorMessage);
    };
};



ron.showWaitMessage = function(options) {
    ron.showMessage(options);
};



ron.confirm = (function() {
    var
        $dialog = null,
        $messageDiv = null,

        $okBtn = null,
        $stornoBtn = null;

    var createDialog = function() {
        $dialog = $("<div/>")
            .addClass("rondialog")
            .attr("title", ron._("Dotaz"));

        $messageDiv = $("<div/>")
            .appendTo($dialog);

        $okBtn = $("<button/>").text("OK").addClass("submitbutton");
        $stornoBtn = $("<button/>").text("Storno").addClass("button");

        $("<div/>")
            .addClass("buttons")
            .append($okBtn)
            .append($stornoBtn)
            .appendTo($dialog);


    };
    return function(options, yesCallback, thisArg) {
        if (typeof options == "string") {
            options = { text: options };
        }

        if (!options) {
            return true;
        }

        if (!options.yesCallback) {
            options.yesCallback = yesCallback;
        }

        if (!options.thisArg) {
            options.thisArg = thisArg;
        }

        if (!options.yesCallback) {
            return confirm(options.text);
        }

        if (!options.text) {
            options.yesCallback.call(options.thisArg, options.callbackData);
            return false;
        }

        if (!$dialog) createDialog();

        if (options.html) $messageDiv.html(options.html);
        else $messageDiv.text(options.text);

        $stornoBtn
            .off("click")
            .on("click", function() {
                if (options.noCallback) {
                    options.noCallback.call(options.thisArg, options.callbackData);
                }
                $dialog.dialog("close");
            });

        $okBtn
            .off("click")
            .on("click", function() {
                if (options.yesCallback) {
                    options.yesCallback.call(options.thisArg, options.callbackData);
                }
                $dialog.dialog("close");
            });

        $dialog.dialog( {
            modal: true,
            minHeight: "50px",
            position: options.position ? options.position : { my: "center", at: "center", of: window }
        });

        return false; // zabrani defaultni akci napr. pro <a>
    }
})();
ron.confirmQuestion = function($element, yesCallback) {
    if ($element.data("ron-confirm-id")) {
        return ron.confirm( ron._($element.data("ron-confirm-id")), yesCallback, $element );
    } else if ($element.data("ron-confirm-text")) {
        return ron.confirm( $element.data("ron-confirm-text"), yesCallback, $element );
    }

    if (yesCallback) {
        yesCallback.call($element);
        return false;
    }

    return true;
};

(function($) {

    $.fn.progressDialog = function(options) {

        var
            that = this,
            action = null,
            defaultOptions = {
                title: null,
                text: null,
                min: 0,
                max: 100,
                value: 0,
                onCancel: null,
                onClose: null,
                buttons: ['cancel'], // cancel, close
                id: null
            };


        defaultOptions.id = this.attr("id");

        var originalOptions = this.data('progress-bar-options');
        if (!originalOptions) {
            originalOptions = defaultOptions;
        }

        if (typeof options === "string") {
            action = options;
            options = originalOptions;
        } else {
            options = $.extend(originalOptions, options);
        }

        this.data('progress-bar-options', options);

        var
            onCancel = function() {
                if (options  &&  options.onCancel) {
                    options.onCancel.call(that);
                }
            },

            onClose = function() {
                if (options  &&  options.onClose) {
                    options.onClose.call(that);
                }
            },


            updateOptions = function() {
                that.data('progress-bar-options', options);
                return that;
            },

            clearEvents = function() {
                that.off("click", "#" + options.id + "_cancel");
                that.off("click", "#" + options.id + "_close");
            },

            close = function() {
                that.dialog("close");
                clearEvents();

                return that;
            },
            setValue = function(value, text) {
                options.value = value;

                var $progressBar = that.find(".progress-bar");
                $progressBar.css("width", (100 / (options.max - options.min)) * value + "%");

                if (text != null  &&  typeof text != "undefined") {
                    $("#" + options.id + "_text").text(text);
                }

                return updateOptions();
            },

            addMessage = function(text, className) {
                if (text) {
                    $("<div/>")
                        .text(text)
                        .attr("class", className)
                        .appendTo( $("#" + options.id + "_messages") );
                }

                return that
            },

            setMessages = function(text) {
                $("#" + options.id + "_messages").text(text);
                return that;
            },

            setButtons = function(buttons) {
                console.log(buttons);
                if (buttons) {
                    var
                        btnCancel = false,
                        btnClose = false;

                    for (var i = 0;  i < buttons.length;  i++) {
                        switch (buttons[i]) {
                            case "cancel" :
                                btnCancel = true;
                                break;

                            case "close" :
                                btnClose = true;
                                break;
                        }
                    }

                    $("#" + options.id + "_cancel").toggle(btnCancel);
                    $("#" + options.id + "_close").toggle(btnClose);
                }

                return that;
            };

        if (action) {
            switch (action) {
                case "close" :
                    return close();

                case "setvalue" :
                    return setValue(arguments[1], arguments[2]);

                case "addmessage" :
                    return addMessage(arguments[1], arguments[2]);

                case "setmessages" :
                    return setMessages(arguments[1]);

                case "setbuttons" :
                    return setButtons(arguments[1]);
            }
        } else {
            if (typeof options.onCancel != "undefined") {
                that.off("click", "#" + options.id + "_cancel");
                that.on("click", "#" + options.id + "_cancel", onCancel);
            }

            if (typeof options.onClose != "undefined") {
                that.off("click", "#" + options.id + "_close");
                that.on("click", "#" + options.id + "_close", onClose);
            }

            setButtons(options.buttons);

            this
                .dialog({
                    modal: true,
                    resizable: true,
                    width: "auto",
                    height: "auto",
                    closeOnEscape: false
                });
        }

        return this;
    }
})(jQuery);
$(function() {
    var
        setDefaultValues = function($form) {

            $form.find(".ronform-field").each(function () {
                var
                    $field = $(this),
                    $defaultValue = $field.data("ronform-default-value");

                if ($defaultValue !== undefined) {
                    if ($field.is("input, select, textarea")) {
                        $field.val($defaultValue);
                    } else if ($field.is("checkbox")) {
                        console.log("CHECKBOX NENI IMPLEMENTOVAN!");
                        alert("Chyba v ronform: CHECKBOX NENI IMPLEMENTOVAN!");
                    } else {
                        $field.text($defaultValue);
                    }
                }
            });
        },
        setFieldValues = function($form, data) {
            $form.find(".ronform-field").each(function () {
                var
                    $field = $(this),
                    fieldName = $field.data("ronform-field-name");

                if (fieldName === undefined) {
                    fieldName = $field.attr("name");
                }

                if (fieldName && data[fieldName] !== undefined) {
                    if ($field.is("input, select, textarea")) {
                        $field.val(data[fieldName]);
                        $field.trigger("change");
                    } else if ($field.is("checkbox")) {
                        console.log("CHECKBOX NENI IMPLEMENTOVAN!");
                        alert("Chyba v ronform: CHECKBOX NENI IMPLEMENTOVAN!");
                    } else {
                        $field.text(data[fieldName]);
                    }
                }
            });
        },
        loadData = function($form, url) {
            $form.find(".ronform-submit").prop("disabled", true);
            $.ajax(url)
                .done(function (resultObject) {
                    console.log("ronform - data loaded", resultObject);
                    if (resultObject.data) {
                        setFieldValues($form, resultObject.data);
                    }

                    if (resultObject.result == "OK") {
                        $form.find(".ronform-submit").prop("disabled", false);
                    }
                });

        },


        beforeShow = function(event) {
            var
                $form = $(this),
                $button = $(event.relatedTarget); // Button that triggered the modal

            setDefaultValues($form);

            var
                url = $button.data("ronform-url");

            if (url === undefined) {
                url = $form.data("ronform-url");
            }

            if (url) {
                loadData($form, url);
            }
        },


        afterShow = function(event) {
            var
                $form = $(this),
                $button = $(event.relatedTarget); // Button that triggered the modal
            var
                fieldId = $button.data("ronform-focus-field");

            if (fieldId) {
                $("#" + fieldId).focus();
            } else {
                $form.find(".ronform-focus").focus();
            }
        },
        onDialogOpen = function(event) {
            var
                $form = $(this);

            setDefaultValues($form);

            var
                url = $form.data("ronform-url"),
                record = $form.data("ronform-record");

            $form.find("input[name=formaction]").val(record ? "save" : "insert");

            if (url  &&  record) {
                loadData($form, url + (record ? record : ''));
            }
        },


        showError = function(error) {
            if (ron && ron.showError) {
                ron.showError(error);
            } else {
                alert(error);
            }
        },


        handleErrors = function($form, errors) {
            var
                $fields = $form.find(".ronform-field");

            $fields.removeClass("alert-danger error");

            var msg = "";
            for (var i = 0;  i < errors.length;  i++) {
                if (errors[i].errorField) {
                    $fields.filter("[name='" + errors[i].errorField + "']").toggleClass("alert-danger error", true);
                    console.log("Chyba v poli " + errors[i].errorField, errors[i]);
                } else {
                    console.log("Chyba", errors[i])
                }

                msg = msg + errors[i].errorText + "\n";
            }

            showError(msg);
        },
        closeForm = function($form) {
            $form.find(".ronform-submit").prop("disabled", false);
            if ($form.is(".ui-dialog-content")) {
                $form.dialog("close");
            } else {
                $form.modal("hide"); // bootstrap
            }
        },
        enableSubmits = function($form, enable) {
            $form.find(".ronform-submit").prop("disabled", !enable); // zabrani druhemu submitu
        },


        submitForm = function($form, url) {
            enableSubmits($form, false);

            var
                data = {},
                fieldSelector = $form.data("ronform-field-selector") || ".ronform-field";

            $form.find(fieldSelector).each(function () {
                var
                    $field = $(this),
                    fieldName = $field.data("ronform-field-name") || $field.attr("name");

                if (fieldName) {
                    if ($field.is("input, select, textarea")) {
                        data[fieldName] = $field.val();
                    } else if ($field.is("checkbox")) {
                        data[fieldName] = this.checked ? 1 : 0;
                    }
                }
            });

            console.log("ronform - send data to url ", url, data);

            $.ajax({
                url: url,
                type: "post",
                dataType: "json",
                data: data
            }).fail(function (jqXHR) {
                showError("Error " + jqXHR.status + ": " + jqXHR.statusText);
            }).done(function (resultObject) {
                console.log(resultObject);
                if (resultObject.errors && resultObject.errors.length) {
                    handleErrors($form, resultObject.errors);
                    enableSubmits($form, true);
                } else {
                    switch ($form.data("ronform-after-submit")) {
                        case "reload" :
                            window.location.reload();
                            break;

                        default :
                            closeForm($form);
                    }
                }
            }).always(function () {
                enableSubmits($form, true);
            });
        },
        submitFormClick = function() {
            var
                $this = $(this),
                $form = $this.closest('form'),
                $ronform = $this.closest('.ronform');

            if (!$ronform.length) {
                return;
            }

            var
                url = $(this).data("ronform-submit-url");

            if (url === undefined) {
                url = $ronform.data("ronform-submit-url");
            }
            if (url === undefined) {
                url = $form.data("ronform-submit-url");
            }

            if (url) {
                submitForm($ronform, url);
                return false;
            } else {
                console.log("Chyba v ronform-submit - neni zadana URL!", this);
                closeForm($ronform);
            }

        },
        openDialogClick = function() {
            var
                $this = $(this),
                $dialog = $($this.data("dialog")),
                url = $this.data("ronform-url"),
                record = $this.data("ronform-record");

            if (url) {
                $dialog.data("ronform-url", url);
            }

            $dialog.data("ronform-record", (typeof(record) !== 'undefined') ? record : null);

            $dialog.dialog({
                modal: true,
                width: "auto"
            });

            return false;
        };


    $(document)
        .on("show.bs.modal", ".ronform", beforeShow)    // bootstrap
        .on("shown.bs.modal", ".ronform", afterShow)
        .on("dialogopen", ".ronform", onDialogOpen)
        .on("click", ".ronform-submit", submitFormClick)
        .on("click", ".open-dialog", openDialogClick);
});

(function($) {
    var
        $activePopup = null,
        activePopupOptions = null,
        $wrapElement = null,
        $originalParent = null;



    var closePopup = function() {
        if (!$activePopup) return;

        if (activePopupOptions  &&  activePopupOptions.onClose) {
            var closeResult = activePopupOptions.onClose.call($activePopup, activePopupOptions);

            if (closeResult === false) {
                return;
            }
        }

        var closeAction = activePopupOptions.closeAction  // muze byt hide / remove / none
            ? activePopupOptions.closeAction
            : "hide";

        switch (closeAction) {
            case "hide" :
                if (activePopupOptions.wrap && $originalParent) {
                    $activePopup.detach();
                    $originalParent.append($activePopup);

                    $wrapElement.hide();
                } else {
                    $activePopup.toggleClass("visible", false);
                }
                break;

            case "remove" :
                $activePopup.remove();
                break;
        }

        $activePopup = null;
        activePopupOptions = null;
        $originalParent = null;
    };

    $(document)
        .on("mousedown", function() {
            closePopup();
        })
        .on("keydown", function(e) {
            if ($activePopup  &&  activePopupOptions  &&  activePopupOptions.exitOnESC  &&  e.which == 27) {
                closePopup();
            }
        });

    $.fn.ronPopup = function(options) {
        var defaultOptions = {
            action : null,
            closeAction : "hide",
            exitOnESC : true,
            position : null,        // Pozice, na kterou se element umisti
            offset : null,
            attachToElement : null,
            positioning : 0,
            noFocus : false,
            wrap : false,
            onClose : null
        };

        options = $.extend( defaultOptions,
            typeof options === "string"
                ? { action: options }
                : options
        );

        if (options.action === "close") {
            if (this.is($activePopup)) {
                closePopup();
            }
            return this;
        } else if (options.action === "closeAll") {
            closePopup();
            return this;
        }


        if ($activePopup && $activePopup.is(this)) {
            return this;
        }

        if ($activePopup) {
            closePopup();
        }

        $activePopup = this;
        activePopupOptions = options;
        $originalParent = options.wrap
            ? this.parent()
            : null;

        if (options.wrap) {
            if (!$wrapElement) {
                $wrapElement = $("<div/>")
                    .addClass("popup")
                    .addClass("popup-wrapper")
                    .appendTo(document.body);
            }

            $wrapElement.append(this);
            $wrapElement.show();
        }

        var $popupElement = options.wrap ? $wrapElement : this;

        $popupElement
            .addClass("popup")
            .on("mousedown", function(event) {
                event.stopPropagation(); // aby to neproslo az na dokument, coz popup okno skryje!
            } )
            .on("closepopup", function() {
                closePopup(); // zajisti odregistrovani udalosti a skryti/zruseni elementu
            } )
            .toggleClass("visible", true)
        ;

        if (options.position) {
            $popupElement
                .css( {
                    top : options.position.top + "px",
                    left: options.position.left + "px"
                });

        } else if (options.attachToElement) {
            var
                position = options.attachToElement.position(),
                cssPosition = "";

            if (ron.utils.hasFixedPosition(options.attachToElement)) {
                var offset = $(options.attachToElement).offset();
                position.left = offset.left;

                cssPosition = "fixed";
            } else {
                var
                    $ofsParent = $popupElement.offsetParent(),
                    ofsPosition = $ofsParent.offset();

                position = $(options.attachToElement).offset();

                position.left -= ofsPosition.left;
                position.top -= ofsPosition.top;
            }

            position.top += options.attachToElement.outerHeight(true); // include margin

            $popupElement
                .css({
                    position: cssPosition,
                    top: position.top + "px",
                    left: position.left + "px"
                });

            setTimeout( function() {
                $popupElement
                    .css( {
                        top : position.top + "px"
                    });
            }, 10);
        }

        if (options.offset) {
            $popupElement.offset(options.offset);
        }
        var
            rect = ron.utils.getAbsoluteRect($popupElement), // pozor! pokud okno obsahuje hodne elementu, tak $().offset() v getAbsoluteRect trva dlouho!
            windowRect = ron.utils.windowViewRect();

        if (options.positioning == 1) {
            if (rect.y2 >= windowRect.y2) {
                var newTop = windowRect.y2 - (rect.y2 - rect.y1) - 10;
                if (newTop < 1) newTop = 1;
                $popupElement.css("top", newTop + "px");
            }
        } else {
            var size = rect.y2 - rect.y1 + 1;
            size -= (rect.y2 - windowRect.y2) + 10;

            $popupElement.css("max-height", size + "px");
        }

        if (!options.noFocus) {
            $popupElement.focus();
        }

        return this;
    };
})(jQuery);

(function($) {
    var hideMenu = function() {
        $(document).ronPopup("closeAll");
    };

    var
        $popupMenu = null,
        $selectedMenuItem = null,
        currentMenuOptions = null;

    var unselectItem = function() {
        if ($selectedMenuItem) {
            $selectedMenuItem.removeClass("selected");
            $selectedMenuItem = null;
        }
    };

    var selectItem = function($newItem) {
        if (!$newItem.length) {
            return;
        }

        unselectItem();

        $newItem.addClass("selected");
        $selectedMenuItem = $newItem;
    };


    var onKeyDown = function(e) {
        if (!$popupMenu) return;

        switch (e.which) {
            case 40:  // dolu
                if (!$selectedMenuItem) selectItem($popupMenu.children().first());
                else selectItem($selectedMenuItem.next());
                return false;

            case 38: // nahoru
                if (!$selectedMenuItem) selectItem($popupMenu.children().last());
                else selectItem($selectedMenuItem.prev());
                return false;

            case 34: // page down
                selectItem($popupMenu.children().last());
                return false;

            case 33: // page up
                selectItem($popupMenu.children().first());
                return false;

            case 13: // enter
                if (currentMenuOptions  &&  currentMenuOptions.onClick) {
                    var result = currentMenuOptions.onClick.call($selectedMenuItem.get(0), e, currentMenuOptions.data);
                    if (result === false) {
                        return false;
                    }
                }

                $selectedMenuItem.trigger("activate", currentMenuOptions ? currentMenuOptions.data : null);

                hideMenu();

                return false;
        }
    };


    var onMouseOver = function(e) {
        if (!$popupMenu) return;

        $popupMenu.children().removeClass("selected");
        $selectedMenuItem = null;
    };
    $.fn.ronPopupMenu = function(options) {
        if (!this.length) return;

        unselectItem();

        var that = this;

        if ($popupMenu) {
            $popupMenu
                .off("keydown", onKeyDown)
                .off("mouseover", onMouseOver)
            ;
        }

        var defaultOptions = {

            action : null,
            closeAction : "hide",
            exitOnESC : true,
            offset : null,
            position : null,
            attachToElement : null,
            mouseEvent : null,
            positioning : 1,
            noFocus : false,
            wrap : false,
            onClose : null,
            onClick : null,
            data : null

        };

        options = $.extend( defaultOptions, options );

        if (options.attachToElement) {
            this
                .css("min-width", ( options.attachToElement.outerWidth() - 2) + "px");
        }

        var position = null;
        if (options.position) {
            position = options.position;
        } else if (options.mouseEvent) {
            position = {
                top:  options.mouseEvent.clientY,
                left: options.mouseEvent.clientX
            }
        }

        $popupMenu = this;
        currentMenuOptions = options;

        this
            .off("click", ">")
            .on("click", ">", function(event) {
                hideMenu();

                if (options.onClick) {
                    var result = options.onClick.call(this, event, options.data);
                    if (result === false) {
                        return false;
                    }
                }

                $(this).trigger("activate", options.data);
            })
            .on("keydown", onKeyDown)
            .on("mouseover", onMouseOver)
        ;

        if (!this.attr("tabIndex")) {
            this.attr("tabIndex", -1);
        }
        this
            .ronPopup({
                attachToElement: options ? options.attachToElement : null,
                offset: options ? options.offset : null,
                position: position,
                positioning: 1,  // posunout, pokud se nevleze na obrazovku
                wrap: options ? options.wrap : false,
                onClose: options.onClose
            })
            .focus();
    };

})(jQuery);

$.widget("ron.ronselect", {
    options: {
        className: null,
        wrapElementTag: null,
        inputClass: null,
        items: null,
        itemsUrl: null,
        autoLoadItems: false,
        maxVisibleRows: null,
        dropDownAlign: null,
        columns: null,
        filterColumns: null,
        captions: null,
        textColumn: null,
        valueColumn: null,
        initialValue: null,
        onlyAutocomplete: false,

        caseSensitive: false,

        fullText: true,
        dropDownElement: null,
        parentElement: null,
        itemIndex: null,
        onDropDown: null,
        onChange: null,
        onItemActivate: null
    },
    $editElement: null,
    $dropDownElement: null,
    dropDownTable: null,
    $table: null,
    $captionsElement: null,
    $wrapElement: null,
    originalValue: null,

    dropDownFilter: null,


    _create: function () {
        this.options.itemIndex = null;

        this.options.className = (this.options.className || this.element.data("ronselect-class") || "") + " ronselect-wrapper";
        this.options.inputClass = this.options.inputClass || this.element.data("input-class") || "form-control";

        if (!this.options.maxVisibleRows) {
            var hlpStr = this.element.data("max-visible-rows");
            if (hlpStr) {
                this.options.maxVisibleRows = parseInt(hlpStr);
            }
        }

        this.options.onlyAutocomplete = this.options.onlyAutocomplete || this.element.data("only-autocomplete");
        this.options.itemsUrl = this.options.itemsUrl || this.element.data("items-url");
        this.options.autoLoadItems = this.options.autoLoadItems || this.element.data("auto-load-items");
        this.options.wrapElementTag = this.options.wrapElementTag || this.element.data("wrap-element-tag")  ||  "span";
        this.options.dropDownAlign = this.options.dropDownAlign || this.element.data("drop-down-align") || "left";
        if (!this.options.columns) {
            var columnsStr = this.element.data("columns");
            if (columnsStr) {
                this.options.columns = columnsStr.split(";");
            }
        }

        if (!this.options.filterColumns) {
            var filterColumnsStr = this.element.data("filter-columns");
            if (filterColumnsStr) {
                this.options.filterColumns = filterColumnsStr.split(";");
            }
        }

        this.options.textColumn = this.options.textColumn  ||  this.element.data("text-column");
        this.options.valueColumn = this.options.valueColumn  ||  this.element.data("value-column");

        if (!this.options.captions) {
            var captionsStr = this.element.data("captions");
            if (captionsStr) {
                this.options.captions = captionsStr.split(";");
            }
        }

        if (this.element.data('fulltext') === 0) {
            this.options.fullText = false;
        }

        if (this.element.is("select")) {
            this._createEdit();
        } else {
            this._wrapElement();
        }

        this._setEvents();

        if (this.options.initialValue) {
            var index = (this.options.initialValue === null)
                ? null
                : this.getItemIndex(this.options.initialValue);

            this._setElementValue(index);
            this.options.itemIndex = index;
        }

        if (this.options.autoLoadItems  &&  this.options.itemsUrl) {
            this.loadData(this.options.itemsUrl, false);
        }
    },
    _setWrapElement: function($wrapElement) {
        var that = this;

        if (!$wrapElement.attr("tabindex")) {
            $wrapElement
                .attr("tabindex", 1);
        }

        $("<span/>")
            .attr("class", "ronselect-loading fa fa-spin inline-glyph glyph-spinner loading-indicator")
            .appendTo($wrapElement);
        $("<span/>")
            .attr("class", "ronselect-button fa")
            .appendTo($wrapElement);

        var keyEvent = function(e) {
            if (that.originalValue === null) {
                that.originalValue = that.$editElement.val();
            }

            if (e.which == 13) {
                var newIndex = that.getItemIndex(that.$editElement.val(), that._getTextColumnIndex());
                if (newIndex !== null) {
                    that.setItemIndex(newIndex, true);
                }
            }

            if (that.dropDownVisible()) {
                if (e.which == 27) {
                    that.closeDropDown();
                    that.$editElement.val(that.originalValue);
                    return false;
                }
                if (that.$table) that.$table.triggerHandler(e);

                e.stopPropagation();
            } else {
                switch (e.which) {
                    case 38 : // up
                    case 40 : // down
                        that.dropDown();
                }
            }

        };


        var mouseDownEvent = function(e) {
            if (!that.dropDownVisible()) {
                that.dropDown();

                that.$editElement.select();

                return false; // pokud to tu neni tak hned popup okno zavre, protoze se provede onmousedown na dokumentu
            }
        };

        $wrapElement
            .on("keydown" , keyEvent)
            .on("keyup" , keyEvent)
            .on("keypress" , keyEvent)
            .on("mousedown", mouseDownEvent)
        ;
    },
    _wrapElement: function() {
        this.$editElement = this.element;

        this.element
            .wrap("<" + this.options.wrapElementTag +
            (this.options.className ? " class='" + this.options.className + "'" : "") +
            "></" + this.options.wrapElementTag + ">");

        this.$wrapElement = this.element.parent();

        this._setWrapElement(this.$wrapElement);
    },
    _createItems: function($options) {
        var
            result = [],
            that = this;

        $options.each( function() {
            var
                $option = $(this),
                item = null;

            if (that.options.columns) {
                item = {
                    value: $option.attr("value"),
                    text: "" + $option.text(),
                    title: $option.attr("title"),
                    class: $option.attr("class")
                };

                for (var i = 0;  i < that.options.columns.length;  i++) {
                    if (that.options.columns[i]  &&  that.options.columns[i] != "value"  &&  that.options.columns[i] != "text") {
                        item[that.options.columns[i]] = "" + ($option.data(that.options.columns[i]) ? $option.data(that.options.columns[i]) : "");
                    }
                }

            } else {
                item = [$option.attr("value"), $option.text()];
            }

            if (item) {
                result.push(item);
            }
        });

        if (!result.length  &&  that.options.itemsUrl) {
        } else {
            if (that.options.columns) {
                that.options.valueColumn = that.options.valueColumn || "value";
                if (that.options.textColumn == null || typeof that.options.textColumn == "undefined") {
                    that.options.textColumn = "text";
                }
            } else {
                that.options.valueColumn = 0;
                that.options.textColumn = 1;
            }

            that.options.items = result;
        }

        return this;
    },
    _createEdit: function() {
        if (!this.element.is("select")) {
            console.log("ronselect - error in _createEdit - element is not <select>")
        }

        this.$wrapElement = $("<" + this.options.wrapElementTag +
            (this.options.className ? " class='" + this.options.className + "'" : "") +
            "></" + this.options.wrapElementTag + ">");

        if (this.element.is(".form-control")) {
            this.$wrapElement.addClass("form-control");
        }

        this.$editElement = $("<input/>");

        this.$editElement
            .attr("autocomplete", "off")
            .addClass(this.options.inputClass)
            .appendTo(this.$wrapElement)
            .attr("title", this.element.attr("title"));

        if (this.element.attr("disabled")) {
            this.$editElement.attr("disabled", "disabled");
        }

        this.element.css("display", "none");

        if (!this.options.items) {
            this._createItems(this.element.children("option"));
        }

        this.element.after(this.$wrapElement);

        this._setWrapElement(this.$wrapElement);

        this._setElementValue(this.getItemIndex(this.element.val()));

        var that = this;
        this.element.on("change", function() {
            that._setElementValue(that.getItemIndex(that.element.val()));
        });
    },
    _setEvents: function() {
        var that = this;
        var blurTimer = null;

        var onBlur = function() {

            blurTimer = setTimeout(
                function () {
                    blurTimer = null;
                    that.closeDropDown();
                    if (!that.options.onlyAutocomplete) {
                        that.$editElement.val(that.originalValue);
                    }
                },
                200);
        };

        var onFocus = function() {
            if (blurTimer) {
                clearTimeout(blurTimer);
                blurTimer = null;
            }
        };
        var onTextChange = function(e) {
            if (!that.dropDownVisible()) {
                that.dropDown();
            }

            that.dropDownTable.filter(that.$editElement.val());
        };


        var onItemActivateEvent = function() {
            if (that.options.onItemActivate  ||  that.options.onChange) {
                var index = that.getItemIndex(that.$editElement.val(), that._getTextColumnIndex());
                if (that.options.onItemActivate) {
                    that.options.onItemActivate.call(that.element, index, that.options);
                }
                if (that.options.onChange) {
                    that.options.onChange.call(that.element, index, that.options);
                }
            }
        };


        this.$editElement
            .on( "focus", onFocus )
            .on( "blur", onBlur )
            .on("itemactivate", onItemActivateEvent)
            .on( "input", onTextChange)
            .attr("autocomplete", "off"); // zabrani zobrazeni drive zadanych hodnot pri kliknuti mysi

    },
    setItems: function(items) {
        this.options.items = items;

        if (this.dropDownTable) {
            this.dropDownTable.loadData(items);
        }
        this.value(null);
        return this;
    },


    setDropDownFilter: function(filter) {
        this.dropDownFilter = filter;
    },
    dropDownVisible: function() {
        return this.$dropDownElement  &&  this.$dropDownElement.is(":visible");
    },


    _doDropDown: function() {
        if (!this.$dropDownElement) {
            this._createDropDownElement();
        }

        var $parent = this.element.parent();

        this.$dropDownElement
            .css("min-width", $parent.outerWidth() + "px");

        var offset = $parent.offset();
        offset.top += $parent.outerHeight() - 1;

        if (this.options.dropDownAlign == "right") {
            offset.left = offset.left + $parent.outerWidth() - this.$dropDownElement.outerWidth();
            if (offset.left < 0) {
                offset.left = 0;
            }
        }

        if (this.$dropDownElement) {
            this.$dropDownElement.ronPopup({
                position: offset,
                noFocus: true
            });
        }

        if (this.$dropDownElement.width() > $parent.outerWidth()) {
            this.$dropDownElement
                .css("min-width", this.$dropDownElement.width() + "px"); // aby se pri vyhledavani podretezce nezmensoval (vadi to hlavne pri zarovnani doprava)
        }

        this.dropDownTable.filter(this.dropDownFilter);

        if (this.options.itemIndex !== null) {
            this.dropDownTable.setActiveRowIndex(this.options.itemIndex);
        } else if (this.dropDownTable  &&  this.options.textColumn) {
            this.dropDownTable.setActiveItem(this.options.textColumn, this.$editElement.val());
        }

        this.$editElement.focus();
    },


    _setItems: function(items) {
        this.options.items = items;

        if (this.element.is("select")) {
            this.element.empty();

            for (var i = 0;  i < items.length;  i++) {
                $("<option/>")
                    .attr("value", items[i][this._getValueColumnIndex()])
                    .text(items[i][this._getTextColumnIndex()])
                    .appendTo(this.element);
            }
        }
    },



    loadData: function(url, dropDown) {
        var
            that = this,
            originalValue = this.element.val();

        this.$wrapElement.addClass("loading");
        $.ajax(url)
            .done( function(data) {
                if (data.result = "OK") {
                    if (data.items) {
                        that._setItems(data.items);

                        if (that.dropDownTable) {
                            that.dropDownTable.loadData(data.items, that.options.columns);
                        }

                        if (originalValue) {
                            that.value(originalValue);
                        }
                        if (dropDown) {
                            that.dropDown();
                        }
                    }
                } else {
                    ron.showError(data);
                }
            })
            .fail( function(jqXHR) {
                ron.showAjaxError(jqXHR);
            })
            .always(function() {
                that.$wrapElement.removeClass("loading");
            });
    },
    dropDown: function() {
        if (this.$editElement.is("[readonly]")  ||  this.$editElement.is("[disabled]")) {
            return this;
        }

        if (this.options.onDropDown) {
            if (this.options.onDropDown.call(this.element, this.options) === false) {
                return this;
            }
        }

        if (!this.options.items  &&  this.options.itemsUrl) {
            this.loadData(this.options.itemsUrl, true);  // vola znovu dropDown a probehne znovu i onDropDown
        } else {
            this._doDropDown();
        }

        return this;
    },
    closeDropDown: function() {
        if (this.dropDownVisible()) {
            this.$dropDownElement.ronPopup("close");
        }

        if (this.dropDownTable) {
            this.dropDownTable.filter(true);
        }

        return this;
    },
    _getTextColumnIndex: function() {
        var colIndex = this.options.textColumn;
        if (typeof colIndex == "undefined"  &&  options.columns) colIndex = options.columns[0];
        if (typeof colIndex == "undefined") return 0;

        return colIndex;
    },
    _getValueColumnIndex: function() {
        if (typeof this.options.valueColumn != "undefined"  &&  this.options.valueColumn !== null) {
            return this.options.valueColumn;
        }

        return this._getTextColumnIndex();
    },
    getItemIndex: function(value, column) {
        if (!this.options.items) return null;

        var colIndex = column;
        if (typeof column == "undefined"  ||  column === null) {
            colIndex = this._getValueColumnIndex();
        }
        if (this.options.itemIndex !== null  &&  this.options.itemIndex >= 0  &&  this.options.itemIndex < this.options.items.length) {
            if (this.options.caseSensitive  ||  typeof value != 'string'  ||  this.options.items[this.options.itemIndex][colIndex] != 'string') {
                if (this.options.items[this.options.itemIndex][colIndex] == value) {
                    return this.options.itemIndex;
                }
            } else {
                if (value.toLowerCase() == this.options.items[this.options.itemIndex][colIndex].toLowerCase()) {
                    return this.options.itemIndex;
                }
            }
        }
        for (var i = 0;  i < this.options.items.length;  i++) {
            if (this.options.caseSensitive  ||  typeof value != 'string'  ||  typeof this.options.items[i][colIndex] != 'string') {
                if (this.options.items[i][colIndex] == value) {
                    return i;
                }
            } else {
                if (value.toLowerCase() == this.options.items[i][colIndex].toLowerCase()) {
                    return i;
                }
            }
        }

        return null;
    },

    _setElementValue: function(itemIndex) {
        if (itemIndex === null  ||  !this.options.items) {
            this.$editElement.val( "" );
        } else {
            if (itemIndex < 0 || itemIndex >= this.options.items.length) return false; // mimo rozsah
            var colIndex = this._getTextColumnIndex();
            this.originalValue = this.options.items[itemIndex][colIndex];
            this.$editElement.val( this.originalValue );
        }

        return true;
    },
    setItemIndex: function(index, callOnActivate) {
        if (this.options.itemIndex === index) return this; // jiz nastavena

        this._setElementValue(index);
        this.options.itemIndex = index;
        if (this.$editElement != this.element  &&  this.element.is("select")) {
            this.element.val(this.value());
        }

        var that = this;
        setTimeout( function() { that.element.change(); }, 100 );

        if (callOnActivate  &&  this.options.onItemActivate) {
            this.options.onItemActivate.call(this.element, index, this.options);
        }

        if (this.options.onChange) {
            this.options.onChange.call(this.element, index, this.options);
        }

        return this;
    },
    itemIndex: function(newIndex) {
        if (newIndex === undefined) {
            return this.options.itemIndex;
        }

        this.setItemIndex(newIndex);
    },



    value: function(newValue) {
        if (newValue === undefined) {
            return this.options.itemIndex === null
                ? null
                : this.options.items[this.options.itemIndex][this._getValueColumnIndex()];
        }

        var index = (newValue === null)
            ? null
            : this.getItemIndex(newValue);

        this.setItemIndex(index);
    },


    _createDropDownElement: function() {
        var that = this;
        var blurTimer = null;

        var onBlur = function() {
            if ($(this).filter(":hover").length) {
                return;
            }

            blurTimer = setTimeout(
                function() {
                    blurTimer = null;
                    that.closeDropDown();
                    that.$editElement.val(that.originalValue);
                },
                200 );
        };

        var onDropDownFocus = function() {
            that.$editElement.focus();
        };
        var onItemActivate = function(index, tableOptions) {
            that.setItemIndex(index, true);
            that.closeDropDown();

            that.element.trigger("itemactivate", {
                index: index,
                item: that.options.items[index]
            });
            return false; // aby se pri enter neodesilal formular
        };


        var elementMouseDown = function() {
            return false; // ???
        };

        that.$dropDownElement = $("<div/>")
            .addClass("popup ronselect-popup")
            .on("blur", onBlur)
            .on("mousedown", elementMouseDown)
            .on("focus", onDropDownFocus)
            .appendTo(that.options.parentElement || $(document.body));
        that.$table = $("<table/>")
            .attr("class", "ronselect-table rowOver")
            .appendTo(that.$dropDownElement)
            .on("keydown", function(e) { e.stopPropagation(); } );

        that.dropDownTable =
            that.$table
                .rontable({
                    data: that.options.items,
                    columns: that.options.columns,
                    filterColumns: that.options.filterColumns,
                    captions: that.options.captions,
                    onActivate: onItemActivate,
                    maxVisibleRows : that.options.maxVisibleRows,
                    fullText : that.options.fullText
                })
                .rontable("instance");
    },


    reloadItems : function() {
        this.options.itemIndex = null;

        if (this.element.is("select")) {
            this._createItems(this.element.children("option"));
        }

        if (this.$table) {
            this.dropDownTable.loadData(this.options.items, this.options.columns);
        }
    }
});


$(function() {
    $("select.ronselect").ronselect();
    $("input.ronselect").ronselect();
});
(function($) {
    var
        defaultOptions = {
            elementsSelector: null,
            data: null,
            columns: null,
            filterColumns: null,
            captions: null,
            maxVisibleRows: null,
            noFocus: false,
            onActivate: null,
            onFilter: null,
            fullText: true
        },
        filterByString = function (item, columns, str, fullText) {
            if (str == "") {
                return true;
            }

            if (str && typeof str != "string") {
                str = str.toString();
            }

            str = str.toLowerCase();

            if (columns) {
                for (var col = 0; col < columns.length; col++) {
                    var colText = item[columns[col]];

                    if (colText && colText.toLowerCase && colText.toLowerCase().indexOf(str) != -1) {
                        var pom = colText.toLowerCase().indexOf(str);

                        if (fullText) {
                            if (pom != -1) {
                                return true;
                            }
                        } else if (pom == 0) {
                            return true;
                        }
                    }
                }
            } else {
                for (var j = 0; j < item.length; j++) {
                    pom = item[j].toLowerCase().indexOf(str);

                    if (fullText) {
                        if (pom != -1) {
                            return true;
                        }
                    } else if (pom == 0) {
                        return true;
                    }
                }
            }

            return false;
        };



    var RonTable = function($element, options) {
        this.$element = $element;
        this.options = $.extend({}, defaultOptions, options);
        this.data = null;
        this.$activeRow = null;
        this.$threeDotsRow = null;
        this.lastLoadedIndex = null;

        this.loadedColumns = null;

        this.loadedFilterFunc = null;

        this.elementsSelector = options ? options.elementsSelector : null;

        if (!this.elementsSelector) {
            this.elementsSelector = $element.is("table")
                ? "tbody > tr, > tr"
                : ">";
        }

        var that = this;
        this.setActiveRow = function ($row) {
            if (this.$activeRow) {
                this.$activeRow.removeClass("active");
                this.$activeRow = null;
            }

            if ($row && $row.data("rontable-index") == "...") {
                this.loadNextData(true);
            } else {
                this.$activeRow = $row;
                if (this.$activeRow) {
                    this.$activeRow.addClass("active");
                }

                ron.utils.ensureVisible($row); // , $element.parent());
            }
        };
        this.setActiveRowIndex = function (index) {
            var $row = this.$element.find(this.elementsSelector).eq(index + (this.options.captions ? 1 : 0));

            if ($row.length) {
                this.setActiveRow($row);
            }
        };


        this.getSelectedRows = function() {
            if (this.$activeRow) {
                return [this.$activeRow.data("rontable-index")];
            }

            return [];
        };
        this.findItem = function(columnName, value) {
            if (!this.data) {
                return null;
            }

            value = ("" + value).toLocaleLowerCase();

            for (var i = 0;  i < this.data.length;  i++) {
                if ( !("" + this.data[i][columnName]).toLocaleLowerCase().localeCompare(value) ) {
                    return i;
                }
            }

            return null;
        };
        this.setActiveItem = function(columnName, value) {
            var itemIndex = this.findItem(columnName, value);

            if (itemIndex === null) return false;

            this.setActiveRowIndex(itemIndex);
            return true;
        };
        this.filter = function(filterFunc) {
            if (!this.data) {
                return;
            }

            if (this.options.maxVisibleRows) {
                this.loadData(this.data, this.loadedColumns, filterFunc);
            } else {
                var
                    $rows = $("tr", this.$element),
                    ronTable = this;

                $rows.each(function (index, row) {
                    var
                        $row = $(row),
                        i = $row.data("rontable-index");

                    var showRow = (typeof i != 'undefined'  &&  i !== "...")
                        ? filterItem(i, filterFunc)
                        : true;

                    $row.css("display", showRow ? "" : "none");
                });
            }
        };
        this.nextRow = function () {
            var
                $newRow = this.$activeRow;

            if ($newRow && !$newRow.is(":visible")) {
                $newRow = null;
            }

            do {
                $newRow = $newRow
                    ? $newRow.next()
                    : this.$element.find(this.elementsSelector).first();

                if ($newRow.is(":visible")) {
                    this.setActiveRow($newRow);
                    return true;
                }
            } while ($newRow && $newRow.length);

            return false;
        };
        this.prevRow = function () {
            var
                $newRow = this.$activeRow;

            if ($newRow && !$newRow.is(":visible")) {
                $newRow = null;
            }

            do {
                $newRow = $newRow
                    ? $newRow.prev()
                    : this.$element.find(this.elementsSelector).last();

                if ($newRow.is(":visible")) {
                    this.setActiveRow($newRow);
                    return true;
                }
            } while ($newRow && $newRow.length);

            return false;
        };
        this.pageUp = function () {
            var
                $newRow = null;

            if (this.$activeRow) {
                $newRow = this.$activeRow.prev();
                if (!$newRow.length) {
                    return false; // zadny dalsi radek
                }

                if (!ron.utils.isFullVisible($newRow)) {
                    ron.utils.scrollElementToBottom(this.$activeRow);
                }
                var OK = false;
                while (!OK) {
                    var $prevRow = $newRow.prev();
                    if ($prevRow.length && ron.utils.isFullVisible($prevRow)) {
                        $newRow = $prevRow;
                    } else {
                        OK = true;
                    }
                }
            } else {
                $newRow = this.$element.find(this.elementsSelector).last();
            }

            if ($newRow && $newRow.length) {
                this.setActiveRow($newRow);
                return true;
            }

            return false;
        };
        this.pageDown = function () {
            var $newRow = null;

            if (this.$activeRow) {
                $newRow = this.$activeRow.next();
                if (!$newRow.length) {
                    return false; // zadny dalsi radek
                }

                if (!ron.utils.isFullVisible($newRow)) {
                    ron.utils.scrollElementToTop(this.$activeRow);
                }
                var OK = false;
                while (!OK) {
                    var $nextRow = $newRow.next();
                    if ($nextRow.length && ron.utils.isFullVisible($nextRow)) {
                        $newRow = $nextRow;
                    } else {
                        OK = true;
                    }
                }

            } else {
                $newRow = this.$element.find(this.elementsSelector).first();
            }

            if ($newRow && $newRow.length) {
                this.setActiveRow($newRow);
                return true;
            }

            return false;
        };


        var
            filterItem = function (index, filterFunc) {
                if (typeof filterFunc == "boolean") {
                    return filterFunc;
                } else if (typeof filterFunc == "string") {
                    return filterByString(that.data[index],
                        that.options.filterColumns ? that.options.filterColumns : that.options.columns,
                        filterFunc, that.options.fullText);
                } else if (filterFunc) {
                    return filterFunc(index, that.data[index]);
                }

                return true;
            },


            renderHeader = function () {
                if (!that.options.captions) {
                    return;
                }

                var
                    $row = $("<tr/>")
                        .addClass('rontable-captions')
                        .appendTo(that.$element);

                for (var i = 0; i < that.options.captions.length; i++) {
                    $("<th/>")
                        .text(that.options.captions[i])
                        .appendTo($row);
                }
            },
            renderDataItems = function (columns, filterFunc, startIndex, selectFirstNewItem) {
                if (!that.data) {
                    return;
                }

                var
                    newRowsCount = 0,
                    colCount = 0,
                    $firstNewRow = null;
                for (var i = startIndex; i < that.data.length && (!that.options.maxVisibleRows || newRowsCount < that.options.maxVisibleRows); i++) {
                    var
                        showItem = filterFunc
                            ? filterItem(i, filterFunc)
                            : true;

                    if (showItem) {
                        that.lastLoadedIndex = i;
                        newRowsCount++;

                        var $row = $("<tr/>")
                            .data("rontable-index", i)
                            .appendTo(that.$element)
                            .attr("title", that.data[i]["title"]);

                        if (that.data[i]["class"]) {
                            $row.addClass(that.data[i]["class"]);
                        }

                        if (!$firstNewRow) {
                            $firstNewRow = $row;
                        }
                        if (columns) {
                            for (var col = 0; col < columns.length; col++) {
                                var
                                    text = that.data[i][columns[col]],
                                    $td = $("<td/>");

                                if (text != '') {
                                    $td.text(text);
                                } else {
                                    $td.html("&nbsp;");
                                }

                                $td.appendTo($row);

                                if (that.data[i].cssClass) {
                                    $td.attr("class", that.data[i].cssClass);
                                }
                            }

                            colCount = columns.length;
                        } else {
                            for (var j = 0; j < that.data[i].length; j++) {
                                $("<td/>")
                                    .text(that.data[i][j])
                                    .appendTo($row);
                            }

                            colCount = that.data[i].length;
                        }
                    }
                }

                if (i < that.data.length - 1) {
                    that.$threeDotsRow = $("<tr/>")
                        .data("rontable-index", "...")
                        .appendTo(that.$element);

                    $("<td/>")
                        .text("...")
                        .attr("colspan", colCount)
                        .appendTo(that.$threeDotsRow);
                }

                if (selectFirstNewItem && $firstNewRow) {
                    that.setActiveRow($firstNewRow);
                }
            };
        this.loadNextData = function (selectFirstNewItem) {
            if (this.$threeDotsRow) {
                this.$threeDotsRow.remove();
                this.$threeDotsRow = null;
            }

            if (this.lastLoadedIndex && this.lastLoadedIndex < this.data.length - 1) {
                renderDataItems(this.loadedColumns, this.loadedFilterFunc, this.lastLoadedIndex + 1, selectFirstNewItem);
            }
        };
        this.loadData = function (data, columns, filterFunc) {
            this.data = data;
            this.$element.empty();
            this.$activeRow = null;

            columns = columns || this.options.columns;

            renderHeader();
            renderDataItems(columns, filterFunc, 0);

            this.loadedColumns = columns;
            this.loadedFilterFunc = filterFunc;
        };
        var
            onKeyDown = function (event) {
                switch (event.which) {
                    case 38 : // up
                        return !that.prevRow();

                    case 40 : // down
                        return !that.nextRow();

                    case 33 : // page up
                        return !that.pageUp();

                    case 34 : // page down
                        return !that.pageDown();
                }
                return true;
            },


            onKeyPress = function (event) {
                if (event.which == 13) {
                    if (that.$activeRow && that.options.onActivate) {
                        var index = that.$activeRow.data("rontable-index");
                        if (index != "...") {
                            return that.options.onActivate.call(that.$element, index, that.options);
                        }
                    }
                }
            },
            onRowClick = function ($event) {
                var
                    $row = $(this),
                    index = $row.data("rontable-index");

                if ($row.is(".rontable-captions")) {
                    return false;
                }

                if (index === "...") {
                    that.loadNextData(true);
                } else {
                    that.setActiveRow($row);

                    if (that.options.onActivate) {
                        that.options.onActivate.call(that.$element, $row.data("rontable-index"), that.options);
                    }
                }
            };
        if (this.options.data) {
            this.loadData(this.options.data, this.options.columns, this.options.onFilter);
        }

        if (!this.options.noFocus) {
            this.$element
                .prop("tabindex", 1)
                .focus()
        }

        this.$element.addClass("rontable");
        this.$element
            .on("click", this.elementsSelector, onRowClick)
            .on("keydown", onKeyDown)
            .on("keypress", onKeyPress);
    };




    $.fn.rontable = function(options) {
        var
            action = null,
            originalArguments = arguments;

        if (typeof options === "string") {
            action = options;
        }

        if (action) {
            var
                $element = $(this),
                instance = $element.data("rontable-instance");

            if (!instance) {
                return;
            }

            switch (action) {
                case "setActiveRow" :
                    instance.setActiveRow(originalArguments[1]);
                    break;

                case "instance" :
                    return instance;

                case "filter" :
                    instance.filter(originalArguments[1]);
                    break;

                case "setActiveItem" :
                    instance.setActiveItem(originalArguments[1], originalArguments[2]);
                    break;

                case "findItem" :
                    instance.findItem(originalArguments[1], originalArguments[2]);
                    break;


                case "nextRow" :
                    instance.nextRow();
                    break;

                case "prevRow" :
                    instance.prevRow();
                    break;

                case "pageUp" :
                    instance.pageUp();
                    break;

                case "pageDown" :
                    instance.pageDown();
                    break;

                default :
                    console.log("ERROR in rontable: action is not implemented", action);
            }

            return;
        }

        this.each(function() {
            var
                $element = $(this),
                instance = $element.data("rontable-instance");
            if (!instance) {
                instance = new RonTable($element, options);
                $element
                    .data("rontable-instance", instance);
            }
        });

        return this;
    }
})(jQuery);

(function($) {
    var
        selectTab = function ($tabs, tab) {
            var
                result = null,
                $tab = (typeof tab === "string") ? $(tab) : tab;
            $tabs.children("li").toggleClass("active", false);
            $tab.toggleClass("active", true);
            $tabs.children("li").each(
                function (index, element) {
                    var
                        $element = $(element),
                        pageId = $element.data('ron-tab');

                    if (pageId) {
                        var $page = $("#" + pageId);

                        if ($element.hasClass("active")) {
                            $page.css("display", "block");
                            result = pageId;
                        } else {
                            $page.css("display", "none");
                        }
                    }
                }
            );

            return result;
        },
        findTab = function($tabs, page) {
            var
                result = null,
                id = (typeof page === "string")
                    ? page
                    : page.attr("id");

            $tabs.children("li").each(
                function (index, element) {
                    var
                        $tab = $(this);

                    if ($tab.data("ron-tab") == id) {
                        result = $tab;
                        return false; // nasel => konci each
                    }
                }
            );

            return result;
        },

        selectPage = function($tabs, page) {
            var $tab = findTab($tabs, page);
            if ($tab) {
                selectTab($tabs, $tab);
            }
        },

        onItemClick = function(e) {
            var
                $tabs = $(e.delegateTarget),
                $this = $(this);

            selectTab($tabs, $this.parent("li"));

            var $url = $this.attr("href");
            if ($url && $url != "" && $url != "#") {
                window.history.replaceState(null, null, $url);
            }

            return false;
        };
    $.fn.rontabs = function(options, param) {
        var
            action = null,
            defaultOptions = {
                className: "rontabs",
                initialPage: ""
            };

        if (typeof options === "string") {
            action = options;
        } else {
            options = $.extend(defaultOptions, options);
        }

        $(this).each(function() {
            var
                $element = $(this);

            if (action) {
                switch (action) {
                    case "selectPage" :
                        selectPage($element, param);
                        break;
                }

                return;
            }

            if (options.className) {
                $element.addClass(options.className);
            }

            $element
                .on("click", "a", onItemClick);

            if (options.initialPage) {
                selectPage($element, options.initialPage);
            } else {
                var $activeTab = $element.find("li.active").first();

                if (!$activeTab.length) {
                    $activeTab = $element.find("li").first();
                }

                if ($activeTab.length) {
                    selectTab($element, $activeTab);
                }
            }
        });
    }
})(jQuery);

$.widget("ron.scrollbar", {
    options: {
        vertical: true,
        min: 0,
        max: 100,
        size: 1,
        position: 0,
        $content: null,
        onScroll: null
    },

    _create: function () {
        var that = this;
        var startDrag = function () {
            that.element.addClass("slider_active");
        };

        var stopDrag = function () {
            that.element.removeClass("slider_active");
        };

        var onDrag = function (event) {
            if (!that.options.size) return;

            if (that.options.vertical) {
                that.options.position = (that.$slider.position().top / that.element.height() * (that.options.max - that.options.min));
            } else {
                that.options.position = (that.$slider.position().left / that.element.width() * (that.options.max - that.options.min));
            }
            if (that.options.onScroll) {
                if (!that._trigger("onScroll", event, {position: that.options.position})) {
                    return;
                }
            }

            if (that.options.$content) {
                that.options.$content.css("top", -that.options.position + "px");
            }
        };

        this.$slider = $("<div>")
            .attr("class", "slider")
            .css("top", "0")
            .css("left", "0")
            .appendTo(this.element)
            .on("mousedown", startDrag)
            .on("mouseup", stopDrag)
            .draggable({
                axis: this.options.vertical ? "y" : "x",
                containment: "parent",
                start: startDrag,
                stop: stopDrag,
                drag: onDrag
            });

        var onClick = function (event) {
            if (event.target != that.element.get(0)) {
                return;
            }
            var newPosition = that.options.position;

            if (that.options.vertical) {
                if (event.offsetY < that.$slider.position().top) {
                    newPosition = that.options.position - that.options.size;
                    if (newPosition < that.options.min) {
                        newPosition = that.options.min;
                    }
                } else if (event.offsetY > that.$slider.position().top + that.$slider.height()) {
                    newPosition = that.options.position + that.options.size;
                    if (newPosition > that.options.max) {
                        newPosition = that.options.max;
                    }
                }
            } else {
                if (event.offsetX < that.$slider.position().left) {
                    newPosition = that.options.position - that.options.size;
                    if (newPosition < that.options.min) {
                        newPosition = that.options.min;
                    }
                } else if (event.offsetX > that.$slider.position().left + that.$slider.width()) {
                    newPosition = that.options.position + that.options.size;
                    if (newPosition > that.options.max) {
                        newPosition = that.options.max;
                    }
                }
            }

            if (newPosition != that.options.position) {
                that.scrollToPosition(newPosition);
            }

        };

        this.element.on("click", onClick);
    },

    _setOption: function (key, value) {
        if (key === "position") {
            this.setPosition(value);
        } else if (key === "min") {
            this.setRange(value, this.options.max, this.options.size);
        } else if (key === "max") {
            this.setRange(this.options.min, value, this.options.size);
        } else if (key === "size") {
            this.setRange(this.options.min, this.options.max, value);
        } else {
            this._super(key, value);
        }

        return this;
    },


    position: function (value) {
        if (value === undefined) return this.options.position;
        this.setPosition(value);
    },
    setRange: function (min, max, size) {
        this.options.min = min;
        this.options.max = max;
        this.options.size = (size > 0) ? size : 1;

        if (this.options.vertical) {
            var newsize = Math.floor(this.element.height() * this.options.size / (this.options.max - this.options.min));
            if (newsize < this.$slider.width()) newsize = this.$slider.width(); // minimalni velikost stejna jako sirka

            if (newsize) this.$slider.css("height", newsize + "px"); // muze byt NaN!
        }  else {
            newsize = Math.floor(this.element.width() * this.options.size / (this.options.max - this.options.min));
            if (newsize < this.$slider.height()) newsize = this.$slider.height();
            if (newsize) this.$slider.css("width", newsize + "px"); // muze byt NaN!
        }

        if (this.options.position < this.options.min) this.setPosition(this.options.min);
        else if (this.options.position > this.options.max - this.options.size + 1) this.setPosition(this.options.max - this.options.size);
        else this.setPosition(this.options.position);
    },

    setPosition: function (value) {
        if (!this.options.size) return;

        if (this.options.vertical) {
            var top = Math.floor(value * this.element.height() / (this.options.max - this.options.min));

            if (top < 0) {
                top = 0;
                value = this.options.min;
            }
            else if (top >= this.element.height() - this.$slider.height()) {
                top = this.element.height() - this.$slider.height();
                value = this.options.max - this.options.size;
            }

            this.$slider.css("top", top + "px");
        }
        else {
            var left = Math.floor(value * this.element.width() / (this.options.max - this.options.min));

            if (left < 0) {
                left = 0;
                value = this.options.min;
            }
            else if (left >= this.element.width() - this.$slider.width()) {
                left = this.element.width() - this.$slider.width();
                value = this.options.max - this.options.size;
            }

            this.$slider.css("left", left + "px");
        }

        this.options.position = value;
    },
    scrollToPosition: function (value) {
        if (!this.options.size) return;

        if (this.options.position != value) {
            this.setPosition(value);

            if (this.options.onScroll) {
                if (!this._trigger("onScroll", null, {position: this.options.position})) {
                    return;
                }
            }

            if (this.options.$content) {
                this.options.$content.css("top", -this.options.position + "px");
            }
        }
    },
    adjust: function() {
        if (!this.options.$content) {
            console.log("ron.scrollbar: can't adjusct scrollbar without $content!");
            return;
        }

        var $parent = this.options.$content.parent();

        if (this.options.vertical) {
            this.setRange(0, this.options.$content.outerHeight() - 1, $parent.height());
            if (this.options.$content.outerHeight() + this.options.$content.position().top < $parent.height()) {
                this.options.$content.css("top", ($parent.height() - this.options.$content.outerHeight()) + "px");
            }
            this.setPosition(-this.options.$content.position().top);
        } else {
            this.setRange(0, this.options.$content.outerWidth() - 1, $parent.width());
            if (this.options.$content.outerWidth() + this.options.$content.position().left < $parent.width()) {
                this.options.$content.css("left", ($parent.width() - this.options.$content.outerWidth()) + "px");
            }
            this.setPosition(-this.options.$content.position().left);
        }
    }
});


(function() {
    var SelectionItem = function(x, y) {
        ron.Rect.call(this, x, y, x, y);
        this.$selectionDiv = null;

        this.$cell1 = null;
        this.$cell2 = null;
    };

    $.widget("ron.selectiontable", {
        options: {
            rowHeight: 0,
            colWidth: 0,
            disableTextSelection: true,
            processKeyEvents: true,
            $parentElement: null,
            $focusElement: null,
            beforeSelectCell: null,
            onSelectCell: null,
            onSelectionClick: null,
            onSelectionDblClick: null
        },

        _items: null,
        _selectionStart: null,
        _$selectionStartCell: null,
        _$activeCellDiv: null,
        _mouseSelection: false,
        _$selectedCell: null,



        _create: function () {
            this._items = [];

            if (!this.options.$parentElement) {
                this.options.$parentElement = this.element.parent();
            }

            if (this.options.processKeyEvents && !this.options.$focusElement) {
                if (!this.element.attr("tabindex") ) {
                    this.options.$focusElement = $("<a/>").appendTo(this.options.$parentElement);
                } else {
                    this.options.$focusElement = this.element;
                }
            }

            this._setEvents();
        },


        _destroy: function() {
            if (this._unsetEvents) {
                this._unsetEvents();
            }

            this.clear();
        },

        getSelectedCell : function() {
            return this._$selectedCell;
        },

        _unsetEvents: null,

        _setEvents: function () {
            var that = this;

            var onMouseUp = function() {
                that._mouseSelection = false;
            };


            var selectedMouseDown = function ($event) {
                if ($event.which == 1) that._mouseSelection = true;
            };



            var selectedClick = function($event) {
                if (that.options.onSelectionClick) {
                    var data = {
                        $cell : that._$selectedCell,
                        $event : $event,
                        shift : $event.shiftKey,
                        ctrl : $event.ctrlKey
                    };

                    that.options.onSelectionClick.call(that.element, data);
                }
            };


            var selectedDblClick = function($event) {
                if (that.options.onSelectionDblClick) {
                    var data = {
                        $cell : that._$selectedCell,
                        $event : $event,
                        shift : $event.shiftKey,
                        ctrl : $event.ctrlKey
                    };

                    that.options.onSelectionDblClick.call(that.element, data);
                }
            };
            var cellMouseDown = function ($event) {
                var $this = $(this);

                var pos = that.getCoordinates($this, $event);
                that.selectCell(pos.y, pos.x, $event.shiftKey, $event.ctrlKey);

                if ($event.which == 1) that._mouseSelection = true;

                if ($event.shiftKey) {
                    return false; // nutne v IE, aby se neoznacila cela stranka
                }

                if (that.options.disableTextSelection) {
                    if (that.options.$focusElement) {
                        that.options.$focusElement.focus();
                    }

                    return false;
                }
            };


            var cellMouseOver = function ($event) {
                if (that._mouseSelection) {
                    var pos = that.getCoordinates($(this), $event);
                    that.selectCell(pos.y, pos.x, true);
                }
            };


            var selectionMouseMove = function ($event) {
                if (that._mouseSelection) {
                    var pos = that.getCoordinates($(this), $event);

                    that.selectCell(pos.y, pos.x, true);
                }
            };
            var selectionMouseDown = function ($event) {
                if ($event.which == 1) {
                    var pos = that.getCoordinates($(this), $event);
                    that.selectCell(pos.y, pos.x, $event.shiftKey);

                    if ($event.shiftKey) return false; // nutne v IE, aby se neoznacila cela stranka
                    return false; // nutne kvuli
                }
            };


            var move = function(keyCode, shiftdown) {
                if (!that._$selectedCell) {
                    return false; // neni vybrana zadna bunka - nedela nic
                }

                var
                    col = that._$selectedCell.index(),
                    $row = that._$selectedCell.closest("tr"),
                    row = $row.index();

                switch (keyCode) {
                    case 37: // left
                        if (col > 0) {
                            that.selectCell(row, col - 1, shiftdown);
                            return true;
                        }
                        break;

                    case 38: // up
                        if (row > 0) {
                            that.selectCell(row - 1, col, shiftdown);
                            return true;
                        }
                        break;

                    case 39: // right
                        if (col < $row.children("td").length - 1) {
                            that.selectCell(row, col + 1, shiftdown);
                            return true;
                        }
                        break;

                    case 40: // down
                        if (row < $row.parent().children("tr").length - 1) {
                            that.selectCell(row + 1, col, shiftdown);
                            return true;
                        }
                        break;
                }

                return false;
            };
            var keyDown = function (e) {
                var $target = $(e.target);

                if ($target.is(".ui-dialog,.popup") || $target.parents(".ui-dialog,.popup").length) {
                    if (e.which != 27) { // ESC
                        e.stopPropagation();
                    }

                    return true;
                }

                if ($target.is("input,select")) {
                    return true;
                }
                switch (e.which) {
                    case 37 : // left
                    case 39 : // right
                    case 38 : // up
                    case 40 : // down
                    case 36 : // home
                    case 35 : // end
                    case 33 : // page up
                    case 34 : // page down
                        if (move(e.which, e.shiftKey)) {
                            return false;
                        }
                        break;
                }

                return true;
            };


            this._unsetEvents = function() {
                this.options.$parentElement
                    .off("mousedown", ".selectiontable-selected", selectedMouseDown)
                    .off("click", ".selectiontable-selected", selectedClick)
                    .off("dblclick", ".selectiontable-selected", selectedDblClick)
                    .off("mousemove", ".selectiontable-multiple", selectionMouseMove)
                    .off("mousedown", ".selectiontable-multiple", selectionMouseDown)
                    .off("mousedown", "td", cellMouseDown)
                    .off("mouseover", "td", cellMouseOver);

                $(document)
                    .off("mouseup", onMouseUp);
            };

            this.options.$parentElement
                .on("mousedown", ".selectiontable-selected", selectedMouseDown)
                .on("click", ".selectiontable-selected", selectedClick)
                .on("dblclick", ".selectiontable-selected", selectedDblClick)
                .on("mousemove", ".selectiontable-multiple", selectionMouseMove)
                .on("mousedown", ".selectiontable-multiple", selectionMouseDown)
                .on("mousedown", "td", cellMouseDown)
                .on("mouseover", "td", cellMouseOver)
                ;

            if (this.options.processKeyEvents) {
                this.options.$focusElement
                    .on("keydown", keyDown);
            }
            $(document)
                .on("mouseup", onMouseUp);
        },
        computeCoordinates: function($element, $event) {
            if ($element.is("td") || $element.is("th")) {
                return new ron.Point($element.index(), $element.closest("tr").index());
            }

            var
                pos = ron.utils.getPositionInParent($element, this.options.$parentElement),
                y = $event.offsetY + pos.top,
                x = $event.offsetX + pos.left;
            var
                OK = false,
                $items = this.element.find("tr"),
                row = 0, col = 0,
                min = 0, max = $items.length;

            while (min < max) {
                row = Math.floor((max + min) / 2);

                var
                    $tr = $items.eq(row),
                    rowPos = ron.utils.getPositionInParent($tr, this.options.$parentElement);

                if (y < rowPos.top) {
                    max = row - 1;
                    row = min;
                } else if (y > rowPos.top + $tr.outerHeight()) {
                    min = row + 1;
                    row = max;
                } else {
                    min = max = row; // found the row
                }
            }
            OK = false;
            $items = $items.eq(row).find("th,td");
            min = 0;
            max = $items.length;

            while (min < max) {
                col = Math.floor((max + min) / 2);

                var
                    $td = $items.eq(col),
                    colPos = ron.utils.getPositionInParent($td, this.options.$parentElement);

                if (x < colPos.left) {
                    max = col - 1;
                    col = min;
                } else if (x > colPos.left + $td.outerWidth()) {
                    min = col + 1;
                    col = max;
                } else {
                    min = max = col; // found the row
                }
            }

            return new ron.Point(col, row);
        },
        getCoordinates: function($element, $event) {
            if (!this.options.rowHeight  ||  !this.options.colWidth) {
                return this.computeCoordinates($element, $event);
            }

            var
                pos = $element.position(),
                row = Math.floor(($event.offsetY + pos.top) / (this.options.rowHeight)),
                col = Math.floor(($event.offsetX + pos.left) / (this.options.colWidth));

            return new ron.Point(col, row);
        },
        highlightActiveCell: function ($cell) {
            if (!this._$activeCellDiv) {
                this._$activeCellDiv = $("<div/>")
                    .attr("class", "selectiontable-selection selectiontable-selected")
                    .css("display", "inline")
                    .appendTo(this.options.$parentElement);
            }


            if (!$cell  ||  !$cell.length) {
                console.log("Error in selectiontable.highlightActiveCell", "$cell:", $cell);
                return this;
            }

            this._$selectedCell = $cell;

            var position = $cell.offset();
            this._$activeCellDiv
                .css("display", "inline")
                .width($cell.outerWidth())
                .height($cell.outerHeight())
                .offset({
                    left: position.left - 2,
                    top: position.top - 2
                });

            return this;
        },


        _setSelectionItemPosition: function(item, left, top, right, bottom) {
            item.$selectionDiv
                .css("display", "inline")   // nejdrive zobrazit !
                .width(right - left - 2)
                .height(bottom - top - 2)
                .offset({
                    left: left - 1,
                    top: top - 1
                });
        },
        _setSelectionDiv: function (item, $cell) {
            if (!item.$selectionDiv) {
                item.$selectionDiv = $("<div/>")
                    .attr("class", "selectiontable-selection selectiontable-multiple")
                    .appendTo(this.options.$parentElement);

                item.$cell1 = this._$selectionStartCell;
            }

            item.$cell2 = $cell;

            var
                position = $cell.offset(),
                selectionPos = this._$selectionStartCell.offset(),

                pomLeft = (selectionPos.left < position.left)
                    ? selectionPos.left
                    : position.left,

                pomRight = position.left + $cell.outerWidth(); // tablecell.offsetLeft + tablecell.offsetWidth;

            if (selectionPos.left + this._$selectionStartCell.outerWidth() > pomRight) {
                pomRight = selectionPos.left + this._$selectionStartCell.outerWidth();
            }

            var
                pomTop = (selectionPos.top < position.top)
                    ? selectionPos.top
                    : position.top,

                pomBottom = position.top + $cell.outerHeight();

            if (selectionPos.top + this._$selectionStartCell.outerHeight() > pomBottom) {
                pomBottom = selectionPos.top + this._$selectionStartCell.outerHeight();
            }

            this._setSelectionItemPosition(item, pomLeft, pomTop, pomRight, pomBottom);
        },
        _modifyLastSelection: function (x, y, $cell) {
            if (!this._selectionStart || !this._items.length) {
                console.log("Chyba ve funkci Selection.modifyLastSelection!");
                return;
            }

            var item = this._items[this._items.length - 1];
            item.x1 = x < this._selectionStart.x ? x : this._selectionStart.x;
            item.x2 = x > this._selectionStart.x ? x : this._selectionStart.x;

            item.y1 = y < this._selectionStart.y ? y : this._selectionStart.y;
            item.y2 = y > this._selectionStart.y ? y : this._selectionStart.y;

            if (item.x1 != item.x2 || item.y1 != item.y2) {
                this._setSelectionDiv(item, $cell);
            } else if (item.$selectionDiv) {
                item.$selectionDiv.remove();
                item.$selectionDiv = null;
            }
        },
        refreshSelection: function() {
            for (var i = 0;  i < this._items.length;  i++) {
                if (this._items[i].$cell1  &&  this._items[i].$cell2) {
                    var
                        $cell1 = this._items[i].$cell1,
                        $cell2 = this._items[i].$cell2;

                    var
                        position1 = $cell1.offset(),
                        position2 = $cell2.offset(),

                        pomLeft = (position1.left < position2.left)
                            ? position1.left
                            : position2.left,

                        pomRight = position1.left + $cell1.outerWidth(); // tablecell.offsetLeft + tablecell.offsetWidth;

                    if (position2.left + $cell2.outerWidth() > pomRight) {
                        pomRight = position2.left + $cell2.outerWidth();
                    }

                    var
                        pomTop = (position1.top < position2.top)
                            ? position1.top
                            : position2.top,

                        pomBottom = position1.top + $cell1.outerHeight();

                    if (position2.top + $cell2.outerHeight() > pomBottom) {
                        pomBottom = position2.top + $cell2.outerHeight();
                    }

                    this._setSelectionItemPosition(this._items[i], pomLeft, pomTop, pomRight, pomBottom);
                }
            }

            this.highlightActiveCell(this._$selectedCell);
        },
        clear: function () {
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].$selectionDiv) {
                    this._items[i].$selectionDiv.remove();
                    this._items[i].$selectionDiv = null;
                }
            }

            this._items = [];
            this._selectionStart = null;
            this._$selectionStartCell = null;

            return this;
        },
        selectCell: function (y, x, shift, ctrl) {
            var data = {
                x : x,
                y : y,
                $cell : null, // doplni se dale
                shift : shift,
                ctrl : ctrl
            };

            if (this.options.beforeSelectCell) {
                if (this.options.beforeSelectCell.call(this.element, data) === false ) {
                    return this;
                }
            }

            if (!data.$cell) {
                var $row = this.element.children("tbody").children("tr").eq(y);
                data.$cell = $row.children("td").eq(x);
            }

            if (!data.$cell  ||  !data.$cell.length) {
                return this;
            }

            if (shift && this._selectionStart) {
                this._modifyLastSelection(x, y, data.$cell);
            } else {
                if (!ctrl) {
                    this.clear();
                } else if (this._$selectionStartCell && !this._items[this._items.length - 1].$selectionDiv) {
                    this._setSelectionDiv(this._items[this._items.length - 1], this._$selectionStartCell);
                }
                this._selectionStart = new ron.Point(x, y);
                this._$selectionStartCell = data.$cell;

                var newItem = new SelectionItem(x, y);
                this._items.push(newItem);
            }

            this.highlightActiveCell(data.$cell);

            if (this.options.processKeyEvents) {
                if (this.options.$focusElement) {
                    this.options.$focusElement.focus();
                }
            }

            if (this.options.onSelectCell) {
                this.options.onSelectCell.call(this.element, data);
            }

            return this;
        },


        hideActiveCell: function () {
            if (this._$activeCellDiv) {
                this._$activeCellDiv.css("display", "none");
            }

            return this;
        },


        showActiveCell: function () {
            if (this._$activeCellDiv) {
                this._$activeCellDiv.css("display", "inline");
            }

            return this;
        },


        setActiveText: function (text) {
            if (this._$activeCellDiv) {
                this._$activeCellDiv.html(text);
            }

            return this;
        },


        isEmpty: function () {
            return !this._items.length;
        },
        getRect: function () {
            if (!this._items.length) {
                return this._selectionStart
                    ? new ron.Rect(this._selectionStart.x, this._selectionStart.y, this._selectionStart.x, this._selectionStart.y)
                    : new ron.Rect(-1, -1, -1, -1);
            }

            var result = new ron.Rect(this._items[0].x1, this._items[0].y1, this._items[0].x2, this._items[0].y2);
            for (var i = 1; i < this._items.length; i++) {
                if (this._items[i].x1 < result.x1) result.x1 = this._items[i].x1;
                if (this._items[i].x2 > result.x2) result.x2 = this._items[i].x2;
                if (this._items[i].y1 < result.y1) result.y1 = this._items[i].y1;
                if (this._items[i].y2 > result.y2) result.y2 = this._items[i].y2;
            }

            return result;
        },
        getActiveRect: function () {
            return this._items.length
                ? this._items[this._items.length - 1]
                : null;
        },


        isRowSelected: function (row) {
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].y1 <= row && this._items[i].y2 >= row) return true
            }

            return false;
        },


        isCellSelected: function (x, y) {
            for (var i = 0; i < this._items.length; i++) {
                if (this._items[i].y1 <= y && this._items[i].y2 >= y && this._items[i].x1 <= x && this._items[i].x2 >= x) return true;
            }

            return false;
        },
        forEach: function (callable) {
            for (var i = 0; i < this._items.length; i++) {
                callable(this._items[i]);
            }

            return this;
        },
        forEachInterval: function (callable) {
            for (var i = 0; i < this._items.length; i++) {
                for (var radek = this._items[i].y1; radek <= this._items[i].y2; radek++) {
                    var result = callable(radek, this._items[i].x1, this._items[i].x2, this._items[i]);
                    if (result === false) return false; // pokud vrati false, nepokracuje dale
                }
            }

            return true;
        },
        getIntervals: function () {
            var result = [];

            this.forEachInterval(function (radek, sloupecOd, sloupecDo) {
                result.push({radek: radek, sloupecOd: sloupecOd, sloupecDo: sloupecDo});
            });

            return result;
        },


        getSelectedRows: function () {
            var result = [];

            var rect = this.getRect();
            for (var i = rect.y1; i <= rect.y2; i++) {
                if (this.isRowSelected(i)) { // pri vicenasobnem hromadnem vyberu nemusi byt vsechny osoby v rect vybrane
                    result.push(i);
                }
            }

            return result;
        }

    });
})();

$.widget("ron.shiftform", {
    options: {
        $leftTable: null,
        $rightTable: null,
        leftItems: [],
        leftColumns: null,
        leftSubmitColumns: null,
        leftFilterFunc: null,
        rightItems: [],
        rightColumns: null,
        rightSubmitColumns: null,
        rightFilterFunc: null,
        url: '',

        data: null,
        afterLoadData: null
    },

    leftRonTable: null,
    rightRonTable: null,
    lastFilterFunc: null,

    _create: function () {
        this.leftRonTable = this.options.$leftTable
            .rontable({data: this.options.leftItems, columns: this.options.leftColumns})
            .rontable("instance");
        this.rightRonTable = this.options.$rightTable
            .rontable({data: this.options.rightItems, columns: this.options.rightColumns})
            .rontable("instance");

    },
    showData: function (filterFunc) {
        if (this.options.data) {
            this.leftRonTable.loadData(this.options.data.leftItems, this.options.leftColumns, filterFunc ? filterFunc : this.options.leftFilterFunc);
            this.rightRonTable.loadData(this.options.data.rightItems, this.options.rightColumns, filterFunc ? filterFunc : this.options.rightFilterFunc);
        }

        this.lastFilterFunc = filterFunc; // pro refresh
    },


    refresh: function () {
        this.showData(this.lastFilterFunc);
    },

    $loadRequest: null,
    loadData: function(parameters) {
        var that = this;

        if (this.$loadRequest) {
            this.$loadRequest.abort();
            this.$loadRequest = null;
        }

        this.$loadRequest = $.ajax(this.options.url + "&shiftformaction=loaddata" + (parameters ? "&" + parameters : ""))
            .done(
                function (responseData) {
                    that.$loadRequest = null;
                    if (responseData.data) {
                        that.options.data = responseData.data;

                        if (that.options.afterLoadData) {
                            that.options.afterLoadData.call(that, responseData);
                        }
                    } else if (responseData.errors) {
                        console.log(responseData.errors);
                        ron.showError(responseData);
                    } else {

                    }

                    that.showData();
                }
            )
            .fail(
                function (jqXHR, textStatus) {
                    that.$loadRequest = null;
                }
            );

    },
    getPost: function (items, selectedRows, columns)
    {
        if (!selectedRows  ||  !selectedRows.length) return null;

        var result = {};

        for (var i = 0;  i < selectedRows.length;  i++) {
            for (var col = 0;  col < columns.length;  col++) {
                var columnName = columns[col];
                result[columnName + '#' + i] = items[selectedRows[i]][columnName];
            }
        }

        result["count"] = selectedRows.length;

        return result;
    },

    removeRows: function(items, rows) {
        for (var i = rows.length - 1; i >= 0; i--) {
            items.splice(rows[i], 1);
        }
    },


    handleMoveResult: function(resultData, rows) {
        var i;

        if (resultData.leftAction == "add") {
            for (i = 0; i < rows.length; i++) {
                this.options.data.leftItems.push(this.options.data.rightItems[rows[i]]);
            }
        }

        if (resultData.rightAction == "add") {
            for (i = 0; i < rows.length; i++) {
                this.options.data.rightItems.push(this.options.data.leftItems[rows[i]]);
            }
        }

        switch (resultData.leftAction) {
            case "remove" :
                this.removeRows(this.options.data.leftItems, rows);
                break;
        }

        switch (resultData.rightAction) {
            case "remove" :
                this.removeRows(this.options.data.rightItems, rows);
                break;
        }

        this.refresh();
    },
    moveRight: function(rows, callBack) {
        var columns = this.options.leftSubmitColumns;
        if (!columns) columns = this.options.leftColumns;
        if (!columns) return;

        if (!rows) rows = this.leftRonTable.getSelectedRows();
        if (!rows || !rows.length) return; // zadny vybrany radek

        var dataObj = this.getPost(this.options.data.leftItems, rows, columns);

        if (!dataObj) return;

        var that = this;

        $.ajax(
            {
                url: this.options.url + "&shiftformaction=moveright",
                type: "POST",
                data: dataObj
            }
        ).done(function (result) {
                if (!result) alert("Chyba - nebyla prijata zadna data!");

                if (result.result == "OK") {
                    if (callBack) {
                        if (callBack() === false) return; // pokud callBack vrati false, nevola defaultni obsluhu
                    }
                    that.handleMoveResult(result.data, rows);
                } else {
                    ron.showError(result);
                }
            }
        ).fail(function (jqXHR) {
                ron.showAjaxError(jqXHR);
            }
        );
    },
    moveLeft: function(rows) {
        var columns = this.options.rightSubmitColumns;
        if (!columns) columns = this.options.rightColumns;
        if (!columns) return;

        if (!rows) rows = this.rightRonTable.getSelectedRows();
        if (!rows || !rows.length) return; // zadny vybrany radek

        var dataObj = this.getPost(this.options.data.rightItems, rows, columns);

        if (!dataObj) return;

        var that = this;

        $.ajax(
            {
                url: this.options.url + "&shiftformaction=moveleft",
                type: "POST",
                data: dataObj
            }
        ).done(function (result) {
                if (!result) alert("Chyba - nebyla prijata zadna data!");

                if (result.result == "OK") {
                    that.handleMoveResult(result.data, rows);
                } else {
                    ron.showError(result);
                }
            }
        ).fail(function (jqXHR) {
                ron.showAjaxError(jqXHR);
            }
        );
    }

});

$.widget("ron.splitter", {
    options : {
        vertical: true,
        position: null,
        minSize: null,
        maxSize: null,

        margin: 2,

        disabled: false,
        change: null,
        $parentElement: null
    },

    _create : function() {
        switch (this.options.position) {
            case "bottom" :
            case "top" :
                this.options.vertical = true;
                break;

            case "left" :
            case "right" :
                this.options.vertical = false;
                break;

            default :
                this.options.position = this.options.vertical ? "bottom" : "right";
        }

        this._createHandle();
    },
    _createHandle : function() {
        var minimalSize;
        var maximalSize;
        var that = this;

        var onStart = function(event, ui) {
            if (that.options.disabled) return;
            $(this).addClass("active");

            minimalSize = that.options.minSize;
            if (minimalSize === null) {
                minimalSize = that.element.data("splitter-minsize");
                if (minimalSize === undefined) {
                    minimalSize = parseInt(that.element.css(that.options.vertical ? "min-height" : "min-width"));
                }
            }

            maximalSize = that.options.maxSize;
            if (maximalSize === null) {
                maximalSize = that.element.data("splitter-maxsize");
                if (maximalSize === undefined) {
                    maximalSize = parseInt(that.element.css(that.options.vertical ? "max-height" : "max-width"));
                }
            }
        };

        var onStop = function(event, ui) {
            $(this).removeClass("active");
            if (that.options.vertical) {
                if (that.options.position == "top") {
                    var bottom = that.element.position().top + that.element.height();
                    ui["newHeight"] = bottom - ui.position.top - that.options.margin;
                } else {
                    ui["newHeight"] = ui.position.top + that.options.margin;
                }

                if (ui["newHeight"] < 0) ui["newHeight"] = 0;
                if (minimalSize  &&  ui["newHeight"] <  minimalSize) ui["newHeight"] = minimalSize;
                if (maximalSize  &&  ui["newHeight"] >  maximalSize) ui["newHeight"] = maximalSize;

            } else {
                if (that.options.position == "left") {
                    var right = that.element.position().left + that.element.width();
                    ui["newWidth"] = right - ui.position.left - that.options.margin;
                } else {
                    ui["newWidth"] = ui.position.left + that.options.margin;
                }

                if (ui["newWidth"] < 0) ui["newWidth"] = 0;
                if (minimalSize  &&  ui["newWidth"] <  minimalSize) ui["newWidth"] = minimalSize;
                if (maximalSize  &&  ui["newWidth"] >  maximalSize) ui["newWidth"] = maximalSize;
            }
            if (that.options.change) {
                if (!that._trigger("change", event, ui)) {
                    that.adjustSplitter();
                    return;
                }
            }


            if (that.options.vertical) {
                that.element.height(ui["newHeight"]);
            } else {
                that.element.height(ui["newWidth"]);
            }

            that.adjustSplitter();
        };

        var onDrag = function(event, ui) {
            if (minimalSize  ||  maximalSize) {
                if (that.options.vertical) {
                    var top = that.element.position().top;
                    if (that.options.position == "top") {
                        if (minimalSize) ui.position.top = Math.min(top + that.element.height() - minimalSize - that.options.margin, ui.position.top);
                        if (maximalSize) ui.position.top = Math.max(top + that.element.height() - maximalSize - that.options.margin, ui.position.top);
                    } else {
                        if (minimalSize) ui.position.top = Math.max(top + minimalSize, ui.position.top); // TODO: zapocitat margin
                        if (maximalSize) ui.position.top = Math.min(top + maximalSize, ui.position.top);
                    }
                } else {
                    var left = that.element.position().left;
                    if (that.options.position == "left") {
                        if (minimalSize) ui.position.left = Math.min(left + that.element.width() - minimalSize - that.options.margin, ui.position.left);
                    } else {
                        if (minimalSize) ui.position.left = Math.max(left + minimalSize, ui.position.left);
                    }
                }
            }
        };

        var $parent = this.options.$parentElement
            ? this.options.$parentElement
            : this.element.parent();

        this.$splitter = $("<div/>")
            .addClass("splitter")
            .addClass(this.options.vertical ? "vertical" : "horizontal" )
            .css("position", "absolute")    // dulezite pro IE a Edge - jinak se nastavi na "relative" a nezobrazi se
            .draggable({
                axis: this.options.vertical ? "y" : "x",
                start: onStart,
                drag: onDrag,
                stop: onStop
            })
            .appendTo($parent);

        this.adjustSplitter();
    },

    widget : function() {
        return this.$splitter;
    },

    _setOption: function( key, value ) {
        this._super( key, value );

        if ( key === "disabled" ) {
            if (value) this.$splitter.hide();
            else this.$splitter.show();
        }

        return this;
    },

    enable: function() {
        return this._setOptions({ disabled: false });
    },
    disable: function() {
        return this._setOptions({ disabled: true });
    },
    adjustSplitter : function() {
        if (this.options.vertical) {
            var pos = (this.options.position == "bottom")
                ? this.element.position().top + this.element.height() - this.options.margin
                : this.element.position().top - this.options.margin;
            this.$splitter.css("top", pos + "px");
        } else {
            pos = (this.options.position == "right")
                ? this.element.position().left + this.element.width() - this.options.margin
                : this.element.position().left - this.options.margin;
            this.$splitter.css("left", pos + "px");
        }
    }

});
function htmltext_convert(text) {
    return ron.utils.htmlTextConvert(text);
}
ron.showPopup = function($element, options) {
    $element.ronPopup(options);
};
ron.ronSelect = function($element, options) {
    console.log("ron.ronSelect je Deprecated - pouzijte $().ronselect!");

    $element.ronselect(options);
    var result = $element.ronselect("instance");
    result.$element = result.element;
    result.getActualItemIndex = function() {
        return result.options.itemIndex;
    };

    return result;
};
ron.ronTable = function( $element, options ) {
    console.log("ron.ronTable je Deprecated - pouzijte $().rontable!");
    $element.rontable(options);

    var result = $element.rontable("instance");
    result.$element = $element;
    result.clear = function() { this.element.empty(); };

    return result;
};
function IncMonth(date, cnt) {
    console.log('Funkce IncMonth je zastarala! Pouzivejte ron.utils.incMonth!');
    return ron.utils.incMonth(date, cnt);
}
function IncDay(date, cnt)
{
    console.log('Funkce IncDay je zastarala! Pouzivejte ron.utils.incDay!');
    return ron.utils.incDay(date,cnt);
}
function StringToDate(str)
{
    console.log('Funkce StringToDate je zastarala! Pouzivejte ron.utils.stringToDate!');
    return ron.utils.stringToDate(str);
}
function DateToStr(date)
{
    console.log('Funkce DateToStr je zastarala! Pouzivejte ron.utils.dateToString!');
    return ron.utils.dateToString(date);
}
function ComboBoxChange(sender, combobox2)
{
    console.log('Funkce ComboBoxChange je zastarala! Pouzijte atribut data-ron-select2!');
    var combo2 = document.getElementsByName(combobox2);
    if (combo2.length > 0)
    {
        combo2 = combo2[0];
        combo2.value = sender.value;
    }
}
function TimeStringToFloat(str) {
    console.log('Funkce TimeStringToFloat je zastarala! Pouzijte ron.utils.timeStringToFloat!');
    return ron.utils.timeStringToFloat(str);
}
function FloatToTimeString(value)
{
    console.log('Funkce FloatToTimeString je zastarala! Pouzijte ron.utils.floatToTimeString!');
    return ron.utils.floatToTimeString(value);
}
function StringToTime(str) {
    console.log('Funkce StringToTime je zastarala! Pouzijte ron.utils.stringToTime!');
    return ron.utils.stringToTime(str);
}
function StringToFloat(str) {
    console.log('Funkce StringToFloat je zastarala! Pouzijte ron.utils.stringToFloat!');
    return ron.utils.stringToFloat(str);
}
function TimeToString(date, utc, seconds) {
    console.log('Funkce TimeToString je zastarala! Pouzijte ron.utils.timeToString!');
    return ron.utils.timeToString(date, utc, seconds);
}
