<?php
/**
 * UTF8
 */

namespace Ron\RonPortal;

require_once 'ron/browser.php';
require_once 'ron/guiutils.php';
require_once 'adsapp/ronportalbrowser.php';

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;


class EditaceRSCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this -> align = 'left';
        $this ->code = 'col-edit';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a class="sendPosition" href="?menuid=sluzebnicesty&action=editovatradeksluzebky&record=' . $rowData[0] .
            '&radek=' . $rowData[1] . '"><i class="inline-glyph glyph-button fa glyph-edit" title="' . \Ron\ronText('Editovat_zaznam') . '"></i></a>';
    }
}


class KopieRSCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this -> align = 'left';
        $this ->code = 'col-copy';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a class="sendPosition" href="?menuid=sluzebnicesty&action=kopirovatradeksluzebky&record=' . $rowData[0] .
            '&radek=' . $rowData[1] . '"><i class="inline-glyph glyph-button fa glyph-copy" title="' . \Ron\ronText('Kopirovat_zaznam') . '"></i></a>';
    }
}


class ZruseniRSCField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this -> align = 'left';
        $this ->code = 'col-delete';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection HtmlUnknownTarget */
        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . \Ron\ronText('Zrusit_zaznam') . '"></i>',
            \Ron\ronText('SC_Opravdu_zrusit_radek'),
            '?menuid=sluzebnicesty&action=zrusitradeksluzebky&record=' . $rowData[0] . '&radek='. $rowData[1],
            'data-ron-send-position="1"'
        );
    }
}



class RadekSluzebniCestyBrowser extends Browser
{
    var $editovatRadek;
    var $kopirovatRadek; // cislo radku, ktery se ma kopirovat
    /** @var SluzebniCesta */
    var $sluzebniCesta;
    var $zobrazitNovyRadek;
    var $lzeEditovat;

    var $vychoziDatum;
    var $vychoziCas;
    var $vychoziStat;

    var $zobrazitMisto; // zobrazuje se jen kdyz nejake existuje


    /**
     * @param Database $database
     * @param bool $lzeEditovat
     * @param SluzebniCesta $sluzebniCesta
     * @param string $akce
     */
    function __construct($database, $lzeEditovat, $sluzebniCesta, $akce)
    {
        parent::__construct($database);

        $this->sluzebniCesta = $sluzebniCesta;
        $this->code = 'browserradeksc';

        $this->title = \Ron\ronText('Seznam_radku_sc');
        $this->tableId = 'tblRadekSluzebniCesty';
        $this ->sqlSelectPart =
            'RadekSluzebniCesty.SluzebniCesta_ID, RadekSluzebniCesty.CisloRadku, RadekSluzebniCesty.KodOperace, RadekSluzebniCesty.Stat, ' . // 0..3
            'RadekSluzebniCesty.DatumCasOd, OperaceDochazky.Popis, RadekSluzebniCesty.Popis, Stat.Popis, ' . // 4..7
            'PocetPJidel_S, PocetPJidel_O, PocetPJidel_V, Misto, MistoSluzebniCesty.Popis, RadekSluzebniCesty.Poznamka'; // 8..13

        $this->sqlFromPart =
            '( RadekSluzebniCesty LEFT OUTER JOIN OperaceDochazky ' .
            '  ON RadekSluzebniCesty.KodOperace = OperaceDochazky.Kod ' .
            ') ' .
            ' INNER JOIN Stat ON RadekSluzebniCesty.Stat = Stat.Kod' .
            ' LEFT OUTER JOIN MistoSluzebniCesty ON MistoSluzebniCesty.Kod = RadekSluzebniCesty.Misto';

        $this->sqlWherePart =
            'RadekSluzebniCesty.SluzebniCesta_ID = ' . (int)$sluzebniCesta ->id;

        $this->defaultSort = 1;
        $this->primaryField = 1;

        $this->lzeEditovat = $lzeEditovat;

        $this->showMessages = isset($_POST['EditaceRadkuSC']);

        $this->cssClass = 'browser minwidth';

        $this ->zobrazitMisto = false;

        $cursor = $database ->getSqlResult('SELECT COUNT(*) FROM MistoSluzebniCesty WHERE Aktivni = 1');
        if ($row = $cursor ->fetch()) {
            if ($row[0]) {
                $this ->zobrazitMisto = true;
            }
        }

        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'RadekSluzebniCesty.DatumCasOd',
            BROWSER_FIELD_CODE => 'col-datum', BROWSER_FIELD_HEADER_TEXT => 'Date', BROWSER_FIELD_FORMAT => 'date'] );
        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'RadekSluzebniCesty.DatumCasOd',
            BROWSER_FIELD_CODE => 'col-cas', BROWSER_FIELD_HEADER_TEXT => 'Cas', BROWSER_FIELD_FORMAT => 'time'] );

        $this->addField([
            BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'OperaceDochazky.Popis', BROWSER_FIELD_CLASS_NAME => 'radekSC_operace',
            BROWSER_FIELD_CODE => 'col-operace',
            BROWSER_FIELD_HEADER_TEXT => 'Operace_sc', BROWSER_FIELD_VISIBLE => SluzebniCestyController::getZobrazitOperaceSC(),
            ]);

        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 12, BROWSER_FIELD_SQL_NAME => 'MistoSluzebniCesty.Popis', BROWSER_FIELD_CLASS_NAME => 'radekSC_misto',
            BROWSER_FIELD_CODE => 'col-misto',
            BROWSER_FIELD_HEADER_TEXT => 'Misto_sc', BROWSER_FIELD_VISIBLE => $this ->zobrazitMisto // zobrazuje se jen kdyz jsou nejaka mista v databazi
            ] );

        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'RadekSluzebniCesty.Popis',
            BROWSER_FIELD_CODE => 'col-popis', BROWSER_FIELD_HEADER_TEXT => 'Popis', BROWSER_FIELD_CLASS_NAME => 'radekSC_popis'] );

        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_SQL_NAME => 'Stat.Popis',
            BROWSER_FIELD_CODE => 'col-stat', BROWSER_FIELD_HEADER_TEXT => 'Stat', BROWSER_FIELD_CLASS_NAME => 'radekSC_stat'] );

        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'PocetPJidel_S',
            BROWSER_FIELD_CODE => 'col-snidane', BROWSER_FIELD_HEADER_TEXT => 'Pocet_snidani', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'radekSC_snidane'] );
        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 9, BROWSER_FIELD_SQL_NAME => 'PocetPJidel_O',
            BROWSER_FIELD_CODE => 'col-obedy', BROWSER_FIELD_HEADER_TEXT => 'Pocet_obedu', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'radekSC_obed'] );
        $this->addField( [
            BROWSER_FIELD_SQL_INDEX => 10, BROWSER_FIELD_SQL_NAME => 'PocetPJidel_V',
            BROWSER_FIELD_CODE => 'col-vecere', BROWSER_FIELD_HEADER_TEXT => 'Pocet_veceri', BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_CLASS_NAME => 'radekSC_vecere'] );

        if ($lzeEditovat &&
            $akce != 'editovatradeksluzebky' &&
            $akce != 'kopirovatradeksluzebky' &&
            $akce != 'pridatradeksluzebky' &&
            $akce != 'ulozradeksluzebky'
        ) {
            $this->addField(new EditaceRSCField());
            $this->addField(new KopieRSCField());
            $this->addField(new ZruseniRSCField());
        }

        $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }


    function renderFieldEdit($index, $field, $value, $rowNumber, $recordId, $values)
    {
        switch ($field->sqlFieldName) {
            case 'RadekSluzebniCesty.DatumCasOd' :
                // pole pro datum nebo pro cas (rozlisi se podle formatu)
                if ($field->format == 'date') {
                    if (isset($_POST['DatumOd'])) $value = StringUtils::htmlTextConvert($_POST['DatumOd']);
                    else if ($value) $value = DBUtils::getFormatedValue($this ->database, $field->format, $value);
                    else $value = StringUtils::format($field->format, $this->vychoziDatum);

                    echo
                        '<td style="white-space:nowrap">' .

                        '<div class="input-group">' .
                        '<input type="text" name="DatumOd" class="form-input input-date inputDate ' . MessageManager::getClass('DatumOd') . '" value="' . $value . '" maxlength="10" size="10">' .
                        '<i class="showcalendar input-group-text fa glyph-calendar"  data-roncalendar-inputname="DatumOd"></i>' .
                        '</div>' .

                        '<input type="hidden" name="SluzebniCesta_ID" value="' . $this->sluzebniCesta ->id . '">' .
                        '<input type="hidden" name="CisloRadku" value="' . (isset($values[1]) ? $values[1] : '') . '">' .
                        '<input type="hidden" name="EditaceRadkuSC" value="1">' .

                        '</td>';
                } else {
                    $value = (isset($_POST['CasOd']))
                        ? StringUtils::htmlTextConvert($_POST['CasOd'])
                        : DBUtils::getFormatedValue($this ->database, $field->format, $value);
                    echo '<TD><INPUT type="text" name="CasOd" class="form-input ' . MessageManager::getClass('CasOd') . '" value="' . $value . '" maxlength="5" size="5"></TD>';
                }
                break;

            case 'OperaceDochazky.Popis' :
                echo '<TD class="radekSC_operace">';
                if (isset($_POST['KodOperace'])) $value = $_POST['KodOperace'];
                else $value = $values[2];

                require_once('ron/controls.php');
                $combo = new ComboBox('KodOperace');
                $combo ->emptyRow = ' ';
                $combo ->loadData($this ->database,
                    'SELECT Kod, Popis FROM OperaceDochazky WHERE ' . ($this ->sluzebniCesta ->zahranicni ? 'UdalostZahranicniSC' : 'UdalostSluzebniCesty') . ' = 1 ORDER BY Popis',
                    $value);
                $combo ->render();
                echo '</TD>';
                break;

            //case 'RadekSluzebniCesty.Misto' :
            case 'MistoSluzebniCesty.Popis' :
                echo '<TD class="radekSC_misto">';
                $value = isset($_POST['Misto'])
                    ? $_POST['Misto']
                    : (isset($values[11]) ? $values[11] : null);

                require_once('ron/controls.php');
                $combo = new ComboBox('Misto');
                $combo ->emptyRow = ' ';
                $combo ->loadData($this ->database,
                    'SELECT Kod, Popis FROM MistoSluzebniCesty WHERE Aktivni = 1 ORDER BY Popis',
                    $value);
                $combo ->render();
                echo '</TD>';
                break;

            case 'RadekSluzebniCesty.Popis' :
                if (isset($_POST['Popis'])) $value = $_POST['Popis'];

                echo '<td class="radekSC_popis"><input type="text" name="Popis" class="form-input" value="' . StringUtils::htmlTextConvert($value) . '" size="40" maxlength="160"/></td>';
                break;

            case 'Stat.Popis' :
                echo '<TD class="radekSC_stat">';

                if (isset($_POST['Stat'])) {
                    $value = $_POST['Stat'];
                } else {
                    $value = (isset($values[3])  && $values[3])
                        ? $values[3]
                        : $this->vychoziStat;
                }

                require_once('ron/controls.php');
                $combo = new ComboBox('Stat');
                $combo ->loadData($this ->database,
                    'SELECT Kod, Popis FROM Stat WHERE PovolitVeSluzCeste = 1 ORDER BY Popis', $value);
                $combo ->render();

                echo '</TD>';
                break;

            case 'PocetPJidel_S' :
            case 'PocetPJidel_O' :
            case 'PocetPJidel_V' :
                if (isset($_POST[$field->sqlFieldName])) $value = $_POST[$field->sqlFieldName];
                echo '<TD class="' . $field ->className . '"><INPUT type="text" class="form-input" name="' . $field->sqlFieldName . '" value="' . (int)$value . '" size="2" maxlength="2"></TD>';
                break;
        }
    }


    // Zobrazeni jednoho radku
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $edit = $values
            ? $this->editovatRadek == $values[1]
            : true; // novy radek

        parent::renderLine($values, $className, $rowNumber, $edit);

        $poznamkaStr = ($values && $values[13]) ? $values[13] : '';

        if ($edit) {
            if (isset($_POST['Poznamka'])) $poznamkaStr = $_POST['Poznamka'];
            echo
                "<tr class='$className poznamkaRadkuSC'>" .
                "<td class='align-right'>" . \Ron\ronText('Poznamka') ."</td>" .
                "<td colspan='" . (count($this ->fields) - 1) . "'><input name='Poznamka' class='poznamkaRadkuSC' maxlength='250' value='" . StringUtils::htmlTextConvert($poznamkaStr) . "'></td>" .
                "</tr>\n";
        } else if ($poznamkaStr) {
            echo
                "<tr class='$className poznamkaRadkuSC'>" .
                "<td></td><td class='poznamka browserpoznamka' colspan='" . (count($this ->fields) - 1) . "'>" . StringUtils::htmlTextConvert($poznamkaStr) . "</td>" .
                "</tr>\n";
        }

        if ($values && $this->kopirovatRadek == $values[1]) {
            $values[1] = null;
            $this->renderLine($values, '', 0, true);
        }
    }


    function nactiHodnotyProNovyRadek()
    {
        $sql =
            'SELECT SluzebniCesta.DatumCasOd, DatumCasDo, SluzebniCesta.Stat, MAX(RadekSluzebniCesty.DatumCasOd)' .
            '  FROM SluzebniCesta LEFT OUTER JOIN RadekSluzebniCesty ON RadekSluzebniCesty.SluzebniCesta_ID = SluzebniCesta.Id ' .
            ' WHERE SluzebniCesta.Id = ' . (int)$this->sluzebniCesta ->id .
            ' GROUP BY SluzebniCesta.DatumCasOd, DatumCasDo, SluzebniCesta.Stat ';

        $cursor = $this ->database ->getSqlResult($sql);

        if ($row = $cursor ->fetch()) {
            if ($row[3]) {
                // nejedna se o prvni radek - datum podle posledniho zaznamu, cas nevyplneny
                $this->vychoziCas = '';
                $this->vychoziDatum = $cursor ->stringToTimestamp($row[3]);
            } else {
                // jedna se o prvni radek - datum a cas podle zacatku sl. cesty
                $this->vychoziCas = $cursor ->stringToTimestamp($row[0]);
                $this->vychoziDatum = $this->vychoziCas;
            }

            $this->vychoziStat = $row[2];
        }
    }


    function renderData() // , $firstRow)
    {
        parent::renderData();

        // prida navic jeden novy radek pro editaci
        if ($this->zobrazitNovyRadek) {
            $this->nactiHodnotyProNovyRadek();
            $this ->renderLine(null, '', 0, true); // edit = true
        }
    }


    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     * @param int $cisloRadku
     * @return bool
     */
    public static function zrusRadekSluzebniCesty($database, $idSluzebniCesty, $cisloRadku)
    {
        require_once('sluzebnicesty/radeksc.php');
        $rec = new RadekSCDBRecord($database, $idSluzebniCesty, $cisloRadku);

        try {
            $rec->deleteRecord();
            MessageManager::addInfoMessage( \Ron\ronText('SC_Zruseni_radku_OK') );
            return true;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }
    }

    /**
     * @param RadekSCDBRecord $rec
     * @param SluzebniCesta $sluzebniCesta
     * @throws \Exception
     */
    private static function loadPost($rec, $sluzebniCesta)
    {
        try {
            $datum = DateUtils::dateStringToTimestamp($_POST['DatumOd']);
        } catch (\Exception $e) {
            throw new RonException($e ->getMessage(), 'DatumOd');
        }
        try {
            $cas = DateUtils::timeStringToTimestamp($_POST['CasOd'], $datum);
        } catch (\Exception $e) {
            throw new RonException($e ->getMessage(), 'CasOd');
        }


        if (!$sluzebniCesta ->pocatekKonecPodleRadku) {
            // Kontrola, jestli je zadany cas v rozsahu casu sluzebni cesty
            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY)) {
                // cas pocatku a konce se nastavuje automaticky - kontroluje jenom datum
                if ($cas < DateUtils::startOfTheDay($sluzebniCesta->datumCasOd) || $cas > DateUtils::endOfTheDay($sluzebniCesta->datumCasDo)) {
                    throw new \Exception(\Ron\ronText('Datum_mimo_rozsah_sc') . ' ' .
                        date('j.n.Y', $sluzebniCesta->datumCasOd) . ' - ' . date('j.n.Y', $sluzebniCesta->datumCasDo));
                }
            } else {
                if ($cas < $sluzebniCesta->datumCasOd || $cas > $sluzebniCesta->datumCasDo) {
                    throw new \Exception(
                        \Ron\ronText('Cas_mimo_rozsah_sc') . ' ' .
                        date('j.n.Y G:i', $sluzebniCesta->datumCasOd) . ' - ' . date('j.n.Y G:i', $sluzebniCesta->datumCasDo));
                }
            }
        }

        if ( SluzebniCestyController::getZobrazitOperaceSC() ) {
            if ( !$_POST['KodOperace'] ) {
                throw new \Exception(\Ron\ronText('Nutno_zadat_operaci'));
            }

            $rec->fKodOperace ->setNewValue($_POST['KodOperace']);
        }

        if (!$_POST['Stat']) {
            throw new \Exception( \Ron\ronText('Neni_zadan_stat') );
        }

        $rec->fPopis ->setNewValue($_POST['Popis']);
        if (isset($_POST['Misto'])) {
            $rec->fMisto->setNewValue($_POST['Misto']);
        }
        $rec->fStat ->setNewValue($_POST['Stat']);

        $rec->fDatumCasOd ->setNewValue($cas);

        $rec->fPocetPJidel_S ->setNewValue((int)$_POST['PocetPJidel_S']);
        $rec->fPocetPJidel_O ->setNewValue((int)$_POST['PocetPJidel_O']);
        $rec->fPocetPJidel_V ->setNewValue((int)$_POST['PocetPJidel_V']);

        $rec->fPoznamka ->setNewValue($_POST['Poznamka']);
    }

    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @return bool|int
     * @throws \Exception
     */
    public static function ulozRadekSluzebniCesty($database, $sluzebniCesta)
    {
        require_once('sluzebnicesty/radeksc.php');

        $rec = new RadekSCDBRecord($database, $sluzebniCesta ->id, (int)$_POST['CisloRadku']);
        self::loadPost($rec, $sluzebniCesta);

        if (!$rec->loadRecord()) {
            throw new \Exception('Chyba! Editovany radek nebyl nalezen!');
        }

        if ( !$rec->updateRecord() ) {
            return false;
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) ) {
            return $sluzebniCesta ->updateCasy($database);
        }

        return true;
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @return bool
     * @throws DBException
     */
    public static function vlozNovyRadekSluzebniCesty($database, $sluzebniCesta)
    {
        require_once('sluzebnicesty/radeksc.php');

        $rec = new RadekSCDBRecord($database, $sluzebniCesta ->id);
        self::loadPost($rec, $sluzebniCesta);

        $rec ->insertRadek();

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY) ) {
            return $sluzebniCesta->updateCasy($database);
        }

        return true;
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @param bool   $lzeEditovat
     * @param string $akce
     */
    public static function zobrazRadkySluzebniCesty($database, $sluzebniCesta, $lzeEditovat, $akce)
    {
        $b = new RadekSluzebniCestyBrowser($database, $lzeEditovat, $sluzebniCesta, $akce);

        $formaction = '';

        if ($lzeEditovat) {
            switch ($akce) {
                case 'editovatradeksluzebky' :
                    $formaction = 'ulozradeksluzebky';
                    if (isset($_POST['CisloRadku'])) $b->editovatRadek = (int)$_POST['CisloRadku'];
                    else $b->editovatRadek = (int)$_GET['radek'];
                    break;

                case 'kopirovatradeksluzebky' :
                    $formaction = 'vlozradeksluzebky';
                    $b->kopirovatRadek = $_GET['radek'];
                    break;

                case 'pridatradeksluzebky' :
                    $formaction = 'vlozradeksluzebky';
                    $b->zobrazitNovyRadek = true;
                    break;
            }
        }

        $b ->load();

        if ($formaction && $lzeEditovat) {
            echo '<FORM class="sendPosition" method="post" action="?menuid=sluzebnicesty&action=' . $formaction . '">' . "\n";
            echo '<INPUT type="hidden" name="x"><INPUT type="hidden" name="y">'; // pro zapamatovani scrollovani v prohlizeci
            echo "\n";
        }

        $b->render();

        if ($formaction && $lzeEditovat) {
            echo '<INPUT type="submit" class="submitbutton" value="' . \Ron\ronText('form.Store') . '">';

            $href = '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $sluzebniCesta ->id;
            echo '<INPUT type="button" class="button clickHref" value="' . \Ron\ronText('Cancel') . '" data-ron-href="' . $href . '" data-ron-send-position="1">';

            echo "</FORM>\n";
        }

        if (!$formaction && $lzeEditovat) {
            /** @noinspection HtmlUnknownAnchorTarget */
            echo
                "<div class='text_link' id='radekSCBrowserLinks'><ul class='linkslist'>\n" .
                "<li><a class='sendPosition' href='?menuid=sluzebnicesty&action=pridatradeksluzebky&record={$sluzebniCesta ->id}'>" . \Ron\ronText('Novy_radek') . "</a></li>\n" .
                "<li><a href='?menuid=sluzebnicesty&action=editovatradkysluzebky&record={$sluzebniCesta ->id}'>" . \Ron\ronText('Editovat_radky_sc') . "</a></li>\n" .
                "</ul></div>\n";
        }

        //echo '<script>ron.sc.browserRadekSluzebniCesty()</script>';
    }

}
