<?php
/**
 * UTF8
 */

namespace Ron\RonPortal;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxesRozdeleni;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\SluzebniCesty\SluzebniCestyUtils;
use RonPortal\SluzebniCesty\TypRadkuUctovaniSluzebniCesty;

require_once('sluzebnicesty/radekuctovanisc.php');



class EditaceUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;
    public $action;
    public $title;


    /**
     * @param SluzebniCesta $sc
     * @param string $action
     */
    function __construct($sc, $action, $code)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->action = $action;
        $this ->code = $code;
        $this ->align = 'left';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] >= 1 && $rowData[2] <= 3) {
            return '';
        }

        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData[2]);
        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

        switch ($this ->action) {
            case 'editovatuctovanisluzebky' :
                $glyph = 'glyph-edit';
                $title = TextManager::_('Editovat_zaznam');
                break;

            case 'kopirovatuctovanisluzebky' :
                $glyph = 'glyph-copy';
                $title = TextManager::_('Kopirovat_zaznam');
                break;

            default :
                $glyph = '';
                $title = '';
        }

        return
            "<a class='sendPosition' href='?menuid=sluzebnicesty&action={$this ->action}&record={$rowData[0]}&radek={$rowData[1]}'>" .
            "<i class='inline-glyph glyph-button fa $glyph' title='$title'></i>".
            "</a>";
    }
}


class ZruseniUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;

    /**
     * @param SluzebniCesta $sc
     */
    function __construct($sc)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] >= 1 && $rowData[2] <= 3) {
            return '';
        }

        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData[2]);
        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
            TextManager::_('SC_Opravdu_zrusit_radek'),
            "?menuid=sluzebnicesty&action=zrusituctovanisluzebky&record=" . $rowData[0] . "&radek=" . $rowData[1],
            'data-ron-send-position="1"');
    }
}


class CastkaUSCField extends BrowserField
{
    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData[17]) {
            // druh pohybu minus nebo plus
            case 1 :
            case 2 :
                // pokud se ma znamenko zobrazovat kladne (nastaveni 0) prevede na ABS
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
                    $value = abs($value);
                }
                break;
        }

        return parent::getDataCellClass($value, $rowNumber, $recordId, $rowData);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ( !isset($value) ) {
            return '';
        }

        switch ($rowData[17]) {
            // druh pohybu minus nebo plus
            case 1 :
            case 2 :
                // pokud se ma znamenko zobrazovat kladne (nastaveni 0) prevede na ABS
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
                    $value = abs($value);
                }
                break;
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


class DokumentyUSCField extends BrowserField
{
    var $lzeEditovat;

    /**
     * @param Database $database
     * @param bool $lzeEditovat
     */
    function __construct($database, $lzeEditovat)
    {
        parent::__construct($database);
        $this->code = 'col-dokumenty';
        $this->align = 'center';

        $this ->lzeEditovat = $lzeEditovat;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $sql =
            'SELECT DokumentSluzebniCesty.Id, Dokument.Popis' .
            '  FROM DokumentSluzebniCesty, Dokument ' .
            ' WHERE SluzebniCesta_ID = ' . (int)$rowData[0] .
            '   AND Dokument.Id = DokumentSluzebniCesty.Id ' .
            '   AND CisloRadku = ' . (int)$rowData[1];

        $result = '';

        $cursor = $this->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $result .=
                '<a target="_blank" href="?menuid=sluzebnicesty&action=download&idslcesty=' . (int)$rowData[0] . '&record=' . $row[0] . '">' .
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . StringUtils::htmlTextConvert($row[1]) . '"></i>' .
                '</a>';

        }
        $cursor ->close();

        if ($this ->lzeEditovat  &&  $rowData[2] > 3  ||  $rowData[2] < 1) {
            $result .=
                '<a href="#" title="' . TextManager::_('Vlozit_novy_dokument') . '">' .
                '<i class="inline-glyph fa glyph-upload show-dokument-dialog" data-fields="CisloRadku" data-field-cisloradku="' . $rowData[1] . '"></i>' .
                '</a>';
        }

        return $result;
    }
}


class CastkaDMField extends BrowserField
{
    private $indexMena;

    /**
     * CastkaDMField constructor.
     * @param null|Database $database
     * @param int $indexCastka
     * @param int $indexMena
     */
    public function __construct($database, $indexCastka, $indexMena)
    {
        parent::__construct($database);

        $this->sqlFieldIndex = $indexCastka;
        $this->indexMena = $indexMena;

        $this->format = 'number2';
        $this->align = 'right';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[$this->indexMena]) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}

class KurzDMField extends BrowserField
{

    /**
     * KurzDMField constructor.
     */
    public function __construct()
    {
        parent::__construct();

        $this ->code = 'col-kurz';
        $this ->header = TextManager::_('Kurz');
        $this ->sqlFieldName = 'RadekUctovaniSCesty.DM_Kurz'; // pouze pro funkci renderFieldEdit
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData[20]) {
            return '';
        }

        // je zaskrtnuto "vlastni kurz"
        return
            StringUtils::format('number', $rowData[21]) . ' ' .
            $rowData[13] . ' = ' .
            StringUtils::format('number', $rowData[22]) . ' ' .
            $rowData[19];
    }
}



class KontrolaUctovaniField extends BrowserField
{
    /** @var bool */
    protected $readOnly;

    /**
     * @param Database $database
     * @param bool $readOnly
     */
    function __construct($database, $readOnly)
    {
        parent::__construct($database);

        $this ->readOnly = $readOnly;
    }


    // 'RadekUctovaniSCesty.Zkontrolovano, RadekUctovaniSCesty.Zkontroloval, RadekUctovaniSCesty.DatumCasKontroly'; // 29..31
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $title = $rowData[31]
            ? (StringUtils::format('datetime', $this ->database ->stringToTimestamp($rowData[31])) . ' (' . StringUtils::htmlTextConvert($rowData[30]) . ')')
            : '';

        if ($this ->readOnly) {
            return $value
                ? "<i class='inline-glyph glyph-button fa glyph-check' title='$title'>"
                : '';
        }

        if (!$rowData) {
            return '';
        }

        return
            '<input type="checkbox" class="radek-uctovani-sc-kontrola"' .
            ($value ? ' checked="checked"' : '') .
            ($title ? " title='$title'" : '') .
            ' data-id-sc="' . $rowData[0] . '" data-cislo-radku="' . $rowData[1] . '"' .
            '/>';
    }
}



class RadekUctovaniSlCestyBrowser extends Browser
{
    /** @var SluzebniCesta */
    private $sluzebniCesta;

    var $editovatRadek;
    var $kopirovatRadek; // cislo radku, ktery se ma kopirovat
    var $idSluzebniCesty;
    var $zobrazitNovyRadek;
    var $lzeEditovat;

    var $f_datum; // datum pocatku sluzebni cesty pro pridani noveho zaznamu

    var $odkazyEditace; // priznak, jestli jsou zobrazeny odkazy pro editaci

    var $f_vzdalenost_start;
    var $f_vzdalenost_cil;
    var $f_vzdalenost_pres;


    // datum urcuje hodnotu pro datumu pro novy zaznam - mel by to byt zacatek SC
    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @param $akce
     */
    function __construct($database, $sluzebniCesta, $akce)
    {
        parent::__construct($database);

        $this ->sluzebniCesta = $sluzebniCesta;

        $this ->code = 'browseructovanisc';
        $this ->title = TextManager::_('Uctovani');
        $this ->tableId = 'tblRadekUctovaniSlCesty';

        $this -> sqlSelectPart =
            'RadekUctovaniSCesty.SluzebniCesta_ID, RadekUctovaniSCesty.CisloRadku, RadekUctovaniSCesty.Typ, ' . // 0..2
            'RadekUctovaniSCesty.NVV_SpotrebaVozidla, RadekUctovaniSCesty.NVV_PocetUjetychKM, RadekUctovaniSCesty.NVV_NepocitatZaklSazbu, ' . // 3..5
            'RadekUctovaniSCesty.NVV_NeprokazanaCenaPHM, RadekUctovaniSCesty.NVV_CenaPHM, RadekUctovaniSCesty.NVV_DruhPHM, RadekUctovaniSCesty.ZpusobPlatby, ' . //6..9

            'RadekUctovaniSCesty.Den, TypRUctovaniSlCesty.Popis, RadekUctovaniSCesty.Castka, RadekUctovaniSCesty.KodMeny, RadekUctovaniSCesty.CastkaDPH, '. // 10..14

            'ZpusobPlatbySC.Popis, RadekUctovaniSCesty.Popis, TypRUctovaniSlCesty.DruhPohybu, ' . // 15..17
            'RadekUctovaniSCesty.DM_Castka, RadekUctovaniSCesty.DM_KodMeny, ' . //18, 19
            'RadekUctovaniSCesty.DM_VlastniKurz, RadekUctovaniSCesty.DM_Kurz_Mnozstvi, RadekUctovaniSCesty.DM_Kurz, ' . // 20..22
            'RadekUctovaniSCesty.Rozdeleni, R1.Popis, TR1.Popis, RadekUctovaniSCesty.Rozdeleni2, R2.Popis, TR2.Popis,' .  // 23..28
            'RadekUctovaniSCesty.Zkontrolovano, RadekUctovaniSCesty.Zkontroloval, RadekUctovaniSCesty.DatumCasKontroly'; // 29..31

        $this ->sqlFromPart =
            '(RadekUctovaniSCesty INNER JOIN TypRUctovaniSlCesty ' .
            '  ON RadekUctovaniSCesty.Typ = TypRUctovaniSlCesty.Kod ' .
            ' ) ' .
            ' LEFT OUTER JOIN ZpusobPlatbySC ' .
            ' ON ZpusobPlatbySC.Kod = RadekUctovaniSCesty.ZpusobPlatby ' .
            ' LEFT OUTER JOIN Rozdeleni R1' .
            ' ON R1.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni' .
            ' AND R1.Kod = RadekUctovaniSCesty.Rozdeleni' .
            ' LEFT OUTER JOIN TypRozdeleni TR1' .
            ' ON TR1.Kod = R1.KodTypuRozdeleni' .
            ' LEFT OUTER JOIN Rozdeleni R2' .
            ' ON R2.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni2' .
            ' AND R2.Kod = RadekUctovaniSCesty.Rozdeleni2' .
            ' LEFT OUTER JOIN TypRozdeleni TR2' .
            ' ON TR2.Kod = R2.KodTypuRozdeleni';

        $this ->sqlWherePart = '';

        $this ->primaryField = 1;

        if ($sluzebniCesta ->lzeEditovat) {
            $prava = Prava::vratPrava('slcesty_ucet');
            $this ->lzeEditovat = $prava ->lzeZobrazitPP($sluzebniCesta ->f_PP ->idPP, $sluzebniCesta ->datumCasOd, $sluzebniCesta ->datumCasDo);
        } else {
            $this->lzeEditovat = false;
        }

        $lzeSchvalit = Prava::lzeZobrazitPP('slcesty_k_ucet', $this ->sluzebniCesta ->f_PP ->idPP);

        $this->showMessages = isset($_POST['EditaceUctovaniSC']);

        $this ->cssClass = 'browser minwidth';

        $this ->defaultSort = 1;
        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Den', 10) ->setCode('col-datum') ->setHeaderText('Datum') ->setFormat('date');

        $this ->addField()
            ->setSqlFieldName('TypRUctovaniSlCesty.Popis', 11) ->setCode('col-popis') ->setHeaderText('Typ');

        $this ->addField( new CastkaUSCField() )
            ->setSqlFieldName('RadekUctovaniSCesty.Castka', 12) -> setHeaderText('Castka')
            ->setCode('col-castka')
            ->setAlign('right') ->setFormat('number2');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.KodMeny', 13) ->setHeaderText('Mena')
            ->setCode('col-mena');

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this->addField(new KurzDMField());
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH) ) {
            $this ->addField()
                ->setSqlFieldName('RadekUctovaniSCesty.CastkaDPH', 14) ->setCode('col-dph') ->setHeaderText('SC_DPH')
                ->setFormat('number2') ->setAlign('right');
        }

        $this ->addField()
            ->setSqlFieldName('ZpusobPlatbySC.Popis', 15) ->setCode('col-zpusobplatby') ->setHeaderText('SC_Zpusob_platby');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Popis', 16) ->setCode('col-popis') ->setHeaderText('Popis');

        $this->addField(new CastkaDMField($database, 18, 19))
            ->setSqlFieldName('RadekUctovaniSCesty.DM_Castka') ->setHeaderText('Castka_DM')
            ->setCode('col-castkadm');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.DM_KodMeny', 19) ->setHeaderText('Mena_DM')
            ->setCode('col-menadm');

        if ($this ->lzeEditovat &&
            $akce != 'editovatuctovanisluzebky' &&
            $akce != 'kopirovatuctovanisluzebky' &&
            $akce != 'pridatuctovanisluzebky' &&
            $akce != 'ulozuctovanisluzebky'
        ) {
            $this ->addField(new EditaceUSCField($this ->sluzebniCesta, 'editovatuctovanisluzebky', 'col-edit'));
            $this ->addField(new EditaceUSCField($this ->sluzebniCesta, 'kopirovatuctovanisluzebky', 'col-copy'));
            $this ->addField(new ZruseniUSCField($this ->sluzebniCesta));
            $this ->odkazyEditace = true;
        } else {
            $this ->odkazyEditace = false;
        }

        $this ->addField(new DokumentyUSCField($database, $this ->lzeEditovat));

// TODO: dodelat!
//        // zkontrolovano
//        $this ->addField(new KontrolaUctovaniField($this ->database, !$lzeSchvalit))
//            ->setSqlFieldIndex(29) ->setHeaderText('Uctovani_SC_zkontrolovano')
//            ->setCode('col-zkontrolovano')
//            ->setVisible($lzeSchvalit) ->setAlign('center');


        $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }


    function renderFieldEdit($index, $field, $value, $rowNumber, $recordId, $values)
    {
        switch ($field ->sqlFieldName) {
            case 'RadekUctovaniSCesty.Den' :
                if (isset($_POST['Den'])) $value = StringUtils::htmlTextConvert($_POST['Den']);
                else if (!$values && $this ->f_datum) $value = date('j.n.Y', $this ->f_datum);
                else $value = DBUtils::getFormatedValue($this ->database, $field ->format, $value);

                echo
                    '<td style="white-space:nowrap">' .

                    '<div class="input-group">' .
                    '<input type="text" name="Den" class="form-input input-date appinput inputDate ' . MessageManager::getClass('Den') . '" value="' . $value . '" maxlength="10" size="10" />' .
                    '<i class="input-group-text fa glyph-calendar showcalendar"  data-roncalendar-inputname="Den"></i>' .
                    '</div>' .

                    '<input type="hidden" name="SluzebniCesta_ID" value="' . $this ->idSluzebniCesty . '" />' .
                    '<input type="hidden" name="CisloRadku" value="' . ($values ? $values[1] : '') . '" />' .
                    '<input type="hidden" name="EditaceUctovaniSC" value="1">' .

                    "</td>\n";

                break;

            case 'RadekUctovaniSCesty.Castka' :
                if (isset($_POST['Castka'])) $value = $_POST['Castka'];
                else {
                    $value = (float)$value;

                    if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)  &&  $values[17] >= 1) $value = abs($value);
                }

                $typ = isset($_POST['Typ']) ? $_POST['Typ'] : ($values ? $values[2] : null);

                $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);

                echo '<td><input type="text" name="Castka" class="form-input inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" ';
                if ($jenahrada) echo ' disabled="disabled" ';
                echo ' /></td>';
                break;

            case 'RadekUctovaniSCesty.KodMeny' :
                echo '<TD>';
                if (isset($_POST['KodMeny'])) {
                    $value = $_POST['KodMeny'];
                }
                else if (!$values) {
                    $value = SluzebniCestyUtils::vratVychoziMenu($this ->database, $this ->idSluzebniCesty);
                }

                $combo = new ComboBox('KodMeny');
                $combo ->captionIndex = 0;
                $combo ->emptyRow = ' ';
                $combo ->alwaysShowSelected = 1;
                $combo ->htmlParams = ['id' => 'KodMeny', 'class' => 'form-input'];
                $combo ->loadData($this ->database, 'SELECT KodMeny_ISO FROM Mena WHERE Aktivni = 1 ORDER BY KodMeny_ISO', $value);
                $combo ->render();

                echo "</TD>\n";
                break;

            case 'RadekUctovaniSCesty.DM_Kurz' :
                $checked = isset($_POST['DM_VlastniKurz'])
                    ? $_POST['DM_VlastniKurz']
                    : ($values ? (int)$values[20] : 0);
                $checkedStr = $checked
                    ? 'checked="checked"'
                    : '';
                $mnozstvi = isset($_POST['DM_Kurz_Mnozstvi'])
                    ? $_POST['DM_Kurz_Mnozstvi']
                    : StringUtils::format('number', $values ? $values[21] : null);
                $kurz = isset($_POST['DM_Kurz'])
                    ? $_POST['DM_Kurz']
                    : StringUtils::format('number', $values ? $values[22] : null);
                $styleStr = $checked ? '' : 'style="display:none"';
                echo
                    '<td>' .
                    '<div>' .
                    "<input type='checkbox' id='DM_VlastniKurz' name='DM_VlastniKurz' $checkedStr/> " .
                    TextManager::_('Vlastni_kurz') .
                    '</div>' .

                    "<div id='editace_uctovani_kurz' $styleStr>" .
                    '<input type="text" name="DM_Kurz_Mnozstvi" class="appinput inputFloat" value="' . $mnozstvi . '" size="4" />' .
                    ' <span id="editace_uctovani_mena">' . ($values ? $values[13] : '') . '</span> = ' .
                    '<input type="text" name="DM_Kurz" class="appinput inputFloat" value="' . $kurz . '" size="4" /> ' . ($values ? $values[19] : '') .
                    '</div>' .
                    "</td>\n";
                break;

            case 'RadekUctovaniSCesty.CastkaDPH' :
                $value = isset($_POST['CastkaDPH'])
                    ? $_POST['CastkaDPH']
                    : (float)$value;

                $typ = isset($_POST['Typ'])
                    ? $_POST['Typ']
                    : $values[2];

                $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);

                echo
                    '<td>' .
                    '<input type="text" name="CastkaDPH" class="form-input inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" ' .
                    ($jenahrada ? ' disabled="disabled" ' : '') .
                    '/>' .
                    '</td>';
                break;

            case 'RadekUctovaniSCesty.Popis' :
                if (isset($_POST['Popis'])) $value = $_POST['Popis'];
                echo '<td><input type="text" class="form-input" name="Popis" value="' . StringUtils::htmlTextConvert($value) . '" size="40" maxlength="160"></td>';
                break;

            case 'TypRUctovaniSlCesty.Popis' :
                echo '<td>';
                if (isset($_POST['Typ'])) {
                    $value = $_POST['Typ'];
                } else if ($values) {
                    $value = $values[2];
                } else if (RonPortalSettings::getValue('typradkuuctovanisc') !== ''  &&  RonPortalSettings::getValue('typradkuuctovanisc') !== null) {
                    $value = RonPortalSettings::getIntValue('typradkuuctovanisc');
                } else {
                    $value = null;
                }

                $options = TypRadkuUctovaniSluzebniCesty::loadOptions($this ->database, $this ->sluzebniCesta ->f_PP ->idPP);
                ComboBox::renderComboBox('Typ', $options, $value,
                        [//'emptyRow' => ' ',
                        'htmlParams' => ['id' => 'typRadkuUctovaniSC', 'class' => 'form-input'],
                        'dataTypeComparison' => false]);

                echo "</td>\n";
                break;

            case 'ZpusobPlatbySC.Popis' :
                echo '<td>';
                $value =  (isset($_POST['ZpusobPlatby'])) ? $_POST['ZpusobPlatby'] : ($values ? $values[9] : null);
                $combo = new ComboBox('ZpusobPlatby');
                $combo ->dataTypeComparison = false;
                $combo ->loadData($this ->database, 'SELECT Kod, Popis FROM ZpusobPlatbySC ORDER BY Popis', $value);
                $combo ->render();

                echo "</td>\n";
                break;

            default :
                echo "<td></td>\n";
        }
    }


    /**
     * Editace nahrad za sluzebni vozidlo
     * @param array $values
     * @param string $className
     */
    function showEditaceNahrad($values, $className)
    {
        $typ = isset($_POST['Typ'])
            ? $_POST['Typ']
            : ($values ? $values[2] : null);

        $jenahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $typ);
        $styleStr = $jenahrada ? '' : 'style="display:none"';

        echo "<tr class='$className'>";
        echo '<td></td>';
        $colspan = count($this ->fields);
        echo "<td colspan='$colspan'>";
        echo '<div id="nahradyzavozidlo" ' . $styleStr . '>';
        echo '<table class="middle">';

        // spotreba - litru
        if (isset($_POST['NVV_SpotrebaVozidla'])) {
            $value = $_POST['NVV_SpotrebaVozidla'];
        }
        else if ($values) {
            $value = StringUtils::format('number2', (float)$values[3]);
        }
        else {
            // novy radek
            $value = $this ->sluzebniCesta ->spotrebaVozidla ? StringUtils::format('number_2', $this ->sluzebniCesta ->spotrebaVozidla) : '';
        }

        echo '<tr><td>' . TextManager::_('Spotreba_TP') . '</td>';
        echo '<td><input type="text" name="NVV_SpotrebaVozidla" class="appinput inputFloat" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" /> ';
        echo TextManager::_('Litru_100') . "</td></tr>\n";


        $value = isset($_POST['NVV_PocetUjetychKM'])
            ? $_POST['NVV_PocetUjetychKM']
            : ($values ? (float)$values[4] : 0);

        echo '<tr><td>' . TextManager::_('Pocet_km') . '</td>';
        echo '<td><input type="text" name="NVV_PocetUjetychKM" class="appinput inputInt" value="' . StringUtils::htmlTextConvert($value) . '" size="8" maxlength="8" /> ';

        echo TextManager::_('Nepocitat_zakladni_sazbu'). ' <input name="NVV_NepocitatZaklSazbu" type="checkbox"';

        if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT) && isset($_POST['NVV_NepocitatZaklSazbu'])) {
            $checked = $_POST['NVV_NepocitatZaklSazbu'];
        }
        else if ($values) {
            $checked = $values[5];
        }
        else {
            // novy radek
            $checked = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZAKLADNI_SAZBU);
        }

        if ($checked) {
            echo ' checked="checked"';
        }
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) {
            echo ' disabled="disabled"';
        }

        echo '/>';

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI) && $this ->f_vzdalenost_start && $this ->f_vzdalenost_cil) {
            // proklik na zobrazeni trasy:
            echo
                "<span class='margin-left-2 trasa_link'>" .
                '<a target="_blank" href="https://www.google.com/maps/dir/' .
                StringUtils::htmlTextConvert($this ->f_vzdalenost_start) . '/' .
                ($this ->f_vzdalenost_pres ? StringUtils::htmlTextConvert($this ->f_vzdalenost_pres) . '/' : '') .
                StringUtils::htmlTextConvert($this ->f_vzdalenost_cil) . '/' .
                '">' . TextManager::_('Trasa') . '</a>' .
                '</span>';

            // vypocet vzdalenosti:
            // http://maps.google.com/maps/api/distancematrix/xml?origins=Brno&destinations=Karvin%C3%A1&sensor=true
            //echo '<script src="https://maps.googleapis.com/maps/api/js?sensor=false"></script>';
            //echo '<span style="display:inline-block; width:50px;"> </span>';
            //$vzdalenostscript = "GetDistance(this, '" . $this ->f_vzdalenost_start . "', '" . $this ->f_vzdalenost_cil . "', '" . $this ->f_vzdalenost_pres . "')";
            //echo '<input type="button" class="button" name="vypocetvzdalenosti" value="' . TextManager::_('Vzdalenost') . '" onclick="' . $vzdalenostscript . '" />';

            //echo '<input type="checkbox" name="NVV_DvojnasobekVzdalenosti" />' . TextManager::_('Zpatecni_cesta');
            //if ($this ->f_vzdalenost_pres) echo '<input type="checkbox" checked="checked" name="NVV_PocitatPres" />' . TextManager::_('Pres');
        }

        echo "</td></tr>\n";

        // Neprokaze cenu PHM => vyber druhu PHM
        $neprokazana = isset($_POST['NVV_NeprokazanaCenaPHM'])
            ? $_POST['NVV_NeprokazanaCenaPHM']
            : ($values ? $values[6] : null);

        echo "<tr><td>";
        echo '<input name="NVV_NeprokazanaCenaPHM" type="radio" value="1" ';
        if ($neprokazana) {
            echo " checked";
        }
        echo " />";
        echo TextManager::_('Neprokaze_cenu_PHM') . '</td>';

        echo "<td>";
        if (isset($_POST['NVV_DruhPHM'])) $value = $_POST['NVV_DruhPHM'];
        else if ($values) $value = $values[8];
        else $value = $this ->sluzebniCesta ->phmVozidla ?: null;

        $combo = new ComboBox('NVV_DruhPHM');
        $combo ->emptyRow = ' ';
        $combo ->htmlParams = ['id' => 'NVV_DruhPHM', 'class' => $neprokazana ? '' : 'd-none'];
        $combo ->loadData($this ->database, 'SELECT Kod, Popis FROM DruhPHM ORDER BY Popis', $value);
        $combo ->render();

        echo "</td></tr>";


        // Prokaze cenu PHM => zadani ceny
        echo '<tr>';
        echo
            '<td>' .
            '<input name="NVV_NeprokazanaCenaPHM" type="radio" value="0" ' .
            ($neprokazana ? '' : ' checked="checked"') .
            ' />' .
            TextManager::_('Prokaze_cenu_PHM') .
            '</td>';

        $value = isset($_POST['NVV_CenaPHM'])
            ? $_POST['NVV_CenaPHM']
            : ($values ? (float)$values[7] : 0);

        $classStr = $neprokazana ? 'class="d-none"' : '';

        echo
            '<td>' .
            '<input type="text" id="NVV_CenaPHM" name="NVV_CenaPHM" class="appInput inputFloat' . ($neprokazana ? ' d-none' : '') . '" value="' . StringUtils::htmlTextConvert($value) . '" ' . $classStr . ' size="8" maxlength="8" /> ' .
            TextManager::_('Kc_litr') .
            "</td></tr>\n";

        echo
            '</table>' .
            '</div>' .
            "</td></tr>\n";
    }


    /**
     * Zobrazeni detailu radku NVV
     * @param $values
     * @param $className
     */
    function showNVV($values, $className)
    {
        echo "<tr class='$className'>";
        echo '<td></td>';

        $colspan = count($this ->fields) - 1; // bez datumu
        echo "<td colspan='$colspan' class='poznamka browserpoznamka sc_nvv'>";
        echo TextManager::_('Spotreba_TP') . ': ' . StringUtils::format('number_2', $values[3]) . ' ' . TextManager::_('Litru_100') . ', ';
        echo "<strong>{$values[4]} " . TextManager::_('km') . '</strong>';

        if (!$values[6]) echo ", " . (float)$values[7] . ' ' . TextManager::_('Kc_litr'); // bez (float) to zobrazuje napr. 38.20000003

        echo ', ' . TextManager::_('Zakladni_sazba') . ': ';
        echo TextManager::_($values[5] ? 'Str_No' : 'Str_Yes');

        echo '</td>';

//        if ($this ->lzeEditovat && $this ->odkazyEditace) {
//            echo '<td></td><td></td><td></td>'; // doplni prazdne sloupce misto odkazu pro editaci
//        }

//        echo '<td></td>'; // dokumenty

        echo "</tr>\n";
    }


    function showRozdeleni($values, $className, $edit)
    {
        if ($edit) {
            if (!RonPortalSettings::getIntValue('typrozdeleniusc')  &&  !RonPortalSettings::getIntValue('typrozdeleniusc2')) {
                return;
            }
        } else {
            if (!$values) return;
            if (!($values[23] || $values[26])) return;
        }

        // radek pro rozdeleni (poznamka nebo editace)
        $colspan = count($this ->fields) - 1; // bez datumu

        echo "<tr class='$className'>";
        echo '<td></td>';
        echo "<td colspan='$colspan' class='poznamka browserpoznamka sc_rozdeleni'>";

        if ($edit) {
            // editace rozdeleni 1
            if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
                $comboBoxesRozdeleni1 = new ComboBoxesRozdeleni($this->database, 'Rozdeleni', 'rozdeleniusc');
                $comboBoxesRozdeleni1 ->loadRozdeleni(RonPortalSettings::getIntValue('rozdeleniusc_pouzevlastni'),
                    $this ->sluzebniCesta ->f_PP,
                    RonPortalSettings::getIntValue('typrozdeleniusc'),
                    isset($_POST['Rozdeleni']) ? $_POST['Rozdeleni'] : ($values ? $values[23] : null), false);

                echo '<div class="row" id="rozdeleni_usc_1">';

                echo '<div class="col col-2">' . StringUtils::htmlTextConvert($comboBoxesRozdeleni1 ->typRozdeleni ->popis) . '</div> ';

                echo '<div class="col col-4">';
                $comboBoxesRozdeleni1 ->combo1 ->render();
                echo '</div>';

                echo '<div class="col col-2">';
                $comboBoxesRozdeleni1 ->combo2 ->render();
                echo '</div>';

                echo '</div>';
            }

            // editace rozdeleni 2
            if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
                $comboBoxesRozdeleni2 = new ComboBoxesRozdeleni($this->database, 'Rozdeleni2', 'rozdeleniusc2');
                $comboBoxesRozdeleni2 ->loadRozdeleni(RonPortalSettings::getIntValue('rozdeleniusc2_pouzevlastni'),
                    $this ->sluzebniCesta ->f_PP,
                    RonPortalSettings::getIntValue('typrozdeleniusc2'),
                    isset($_POST['Rozdeleni2']) ? $_POST['Rozdeleni2'] : ($values ? $values[26] : null), false);

                echo '<div class="row" id="rozdeleni_usc_2">';

                echo '<div class="col col-2">' . StringUtils::htmlTextConvert($comboBoxesRozdeleni2 ->typRozdeleni ->popis) . '</div> ';

                echo '<div class="col col-4">';
                $comboBoxesRozdeleni2 ->combo1 ->render();
                echo '</div>';

                echo '<div class="col col-2">';
                $comboBoxesRozdeleni2 ->combo2 ->render();
                echo '</div>';

                echo '</div>';
            }
        } else {
            // rozdeleni 1
            if ($values[23]) {
                echo '<div>' . StringUtils::htmlTextConvert("{$values[25]}: {$values[24]} ({$values[23]})") . '</div>';
            }
            // rozdeleni 2
            if ($values[26]) {
                echo '<div>' . StringUtils::htmlTextConvert("{$values[28]}: {$values[27]} ({$values[26]})") . '</div>';
            }
        }

        echo '</td>';

        echo "</tr>\n";
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .= $values ? (' ' . 'typradkuuctovani_' . $values[2]) : '';
        return parent::getRowAttributes($values, $className, $rowNumber);
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        if ($values) {
            $edit = $this ->editovatRadek == $values[1]
                && ($values[2] <= 0 || $values[2] >= 3) // nelze editovat zaznamy s vyuctovanim
                && (TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $values[2])); // prava
        } else {
            // novy radek
            $edit = true;
        }

        if ($edit) $className .= ' editing';
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($edit) {
            $this ->showEditaceNahrad($values, $className);
        } else {
            // zobrazeni detailu nahrad v needitovatelnem rezimu
            //if (TypUctovaniSC_Je_NVV($connection, $values[2])) {
            if (TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $values[2])) {
                $this ->showNVV($values, $className);
            }
        }

        $this ->showRozdeleni($values, $className, $edit);

        // kopie aktualniho radku => zobrazi jej jeste jednou s nullovym cislem
        if ($values && $this ->kopirovatRadek == $values[1]) {
            $values[1] = null;
            $this ->renderLine($values, '', 0, true);
        }
    }


    function renderCursor($cursor)
    {
        parent::renderCursor($cursor);

        // prida navic jeden novy radek pro editaci
        if ($this ->zobrazitNovyRadek) {
            $this ->renderLine(null, '', 0, true); // edit = true
        }
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param int $cisloRadku
     * @param object|string $logDescription
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    static function zrusUctovaniSluzebniCesty($database, $sc, $cisloRadku, $logDescription)
    {
        $rec = new RadekUctovaniSCDBRecord($database, $sc ->id, (int)$cisloRadku, $logDescription);
        if ($rec->loadRecord()) {
            // kontrola prav
            if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $rec ->fTyp -> originalValue)) {
                throw new NeniPravoException();
            }

            $rec ->doDelete('');

            MessageManager::addInfoMessage(TextManager::_('SC_Zruseni_radku_OK'));
            return;
        }

        throw new DBException($database, null, 'Pri mazani radku uctovani sluzebni cesty doslo k chybe!');
    }


    /**
     * Zkontroluje zadana data. V pripade chyby zapise chybove hlaseni a vraci false
     * Pokud je vse spravne, tak vraci zadany cas (timestamp)
     * @param bool $jeNahrada
     * @param SluzebniCesta $sc
     * @return bool|int
     * @throws \Exception
     */
    static function zkontrolujUctovaniSluzebky($jeNahrada, $sc)
    {
        try {
            $datum = DateUtils::dateStringToTimestamp($_POST['Den']);
        } catch (\Exception $e) {
            return MessageManager::addErrorMessage( $e ->getMessage(), 'Den' );
        }

        if (!RonPortalSettings::getIntValue('datumuctovani_mimosc')) {
            // Kontrola, jestli je zadane datum v rozsahu datumu sluzebni cesty
            if ($datum < DateUtils::startOfTheDay($sc->datumCasOd) || $datum > $sc->datumCasDo) {
                return MessageManager::addErrorMessage(
                    sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), StringUtils::format('date', $sc->datumCasOd), StringUtils::format('date', $sc->datumCasDo)));
            }
        }

        if (!isset($_POST['Typ']) || trim($_POST['Typ']) == '') {
            return MessageManager::addErrorMessage(TextManager::_('Neni_zadan_typ'));
        }

        if ($jeNahrada) {
            $OK = true;

            // nahrada za sluzebni vozidlo - musi byt vyplnene dalsi hodnoty
            if (!$_POST['NVV_SpotrebaVozidla']) {
                $OK = MessageManager::addErrorMessage(TextManager::_('form.Missing_value') . TextManager::_('Spotreba_TP'));
            }

            if (!$_POST['NVV_PocetUjetychKM']) {
                $OK = MessageManager::addErrorMessage(TextManager::_('form.Missing_value') . TextManager::_('Pocet_km'));
            }

            if ($_POST['NVV_NeprokazanaCenaPHM']) {
                // neprokaze cenu PHM => musi vybrat druh PHM
                if (!$_POST['NVV_DruhPHM']) {
                    $OK = MessageManager::addErrorMessage(TextManager::_('form.Missing_value') . TextManager::_('Druh_PHM'));
                }
            } else {
                // prokaze cenu PHM => musi zadat cenu
                if (!$_POST['NVV_CenaPHM']) {
                    $OK = MessageManager::addErrorMessage(TextManager::_('form.Missing_value') . TextManager::_('Cena_PHM'));
                }
            }

            if (!$OK) return false;
        } else {
            // neni to nahrada za soukrome vozidlo - musi byt zadana castka
            if (!$_POST['Castka']) {
                return MessageManager::addErrorMessage(TextManager::_('Neni_zadana_castka'));
            } else {
                if (!StringUtils::isFloat($_POST['Castka'])) {
                    return MessageManager::addErrorMessage(TextManager::_('Spatna_hodnota_pro') . TextManager::_('Castka'));
                }
            }
        }

        if (!$_POST['KodMeny']) {
            return MessageManager::addErrorMessage(TextManager::_('Neni_zadana_mena'));
        }

        if (isset($_POST['DM_VlastniKurz']) && $_POST['DM_VlastniKurz']) {
            // zkontroluje zadane kurzy
            if (!StringUtils::isFloat($_POST['DM_Kurz_Mnozstvi'])) {
                return MessageManager::addErrorMessage(TextManager::_('Neni_zadano_mnozstvi_pro_kurz'));
            }

            if (!StringUtils::isFloat($_POST['DM_Kurz'])) {
                return MessageManager::addErrorMessage(TextManager::_('Neni_zadan_kurz'));
            }
        }

        return $datum;
    }


    /**
     * @param Database $database
     * @param RadekUctovaniSCDBRecord $rec
     * @param SluzebniCesta $sc
     * @return bool
     * @throws DBException
     * @throws \Exception
     */
    private static function loadPostToRecord($database, $rec, $sc)
    {
        $typUctovani = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($database, $_POST['Typ']);
        $jenahrada = $typUctovani ? $typUctovani ->nahrada : false;

        $datum = self::zkontrolujUctovaniSluzebky($jenahrada, $sc);
        if (!$datum) return false;

        $castka = StringUtils::stringToFloat($_POST['Castka']);
        if ($typUctovani && $typUctovani ->druhPohybu == 1) $castka = abs($castka); // prijem
        else if ($typUctovani && $typUctovani ->druhPohybu == 2) $castka = -abs($castka); // vydej

        $rec ->fPopis ->setNewValue($_POST['Popis']);
        $rec ->fDen ->setNewValue($datum);
        $rec ->fKodMeny ->setNewValue($_POST['KodMeny']);
        $rec ->fTyp ->setNewValue($_POST['Typ']);
        $rec ->fZpusobPlatby ->setNewValue((int)$_POST['ZpusobPlatby']);

        if ($jenahrada) {
            $rec ->fCastka ->setNewValue(0); // nastavi na 0, musi se preuctovat!
            $rec ->fNVV_SpotrebaVozidla ->setNewValue(StringUtils::stringToFloat($_POST['NVV_SpotrebaVozidla']));
            $rec ->fNVV_PocetUjetychKM ->setNewValue((int)$_POST['NVV_PocetUjetychKM']);
            if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) {
                $rec ->fNVV_NepocitatZaklSazbu ->setNewValue($_POST['NVV_NepocitatZaklSazbu'] ? 1 : 0);
            }
            $rec ->fNVV_NeprokazanaCenaPHM ->setNewValue($_POST['NVV_NeprokazanaCenaPHM'] ? 1 : 0);

            if ($_POST['NVV_NeprokazanaCenaPHM']) {
                $rec->fNVV_DruhPHM ->setNewValue($_POST['NVV_DruhPHM']);
            } else {
                $rec->fNVV_CenaPHM ->setNewValue(StringUtils::stringToFloat($_POST['NVV_CenaPHM']));
            }

        } else {
            $rec ->fCastka ->setNewValue($castka);
            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
                $rec ->fCastkaDPH ->setNewValue(StringUtils::stringToFloat($_POST['CastkaDPH']));
            }
        }

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $vlastniKurz = isset($_POST['DM_VlastniKurz']) ? ($_POST['DM_VlastniKurz'] ? 1 : 0) : 0;
            $rec ->fDM_VlastniKurz ->setNewValue($vlastniKurz);

            if ($vlastniKurz) {
                $rec ->fDM_Kurz_Mnozstvi ->setNewValue( $_POST['DM_Kurz_Mnozstvi'] == '' ? null : StringUtils::stringToFloat($_POST['DM_Kurz_Mnozstvi']) );
                $rec ->fDM_Kurz ->setNewValue( $_POST['DM_Kurz'] == '' ? null : StringUtils::stringToFloat($_POST['DM_Kurz']) );
            }
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc')) {
            $rozdeleni = (isset($_POST['Rozdeleni'])  &&  $_POST['Rozdeleni'] != '') ?  $_POST['Rozdeleni'] : null;
            $rec ->fKodTypuRozdeleni ->setNewValue($rozdeleni ? RonPortalSettings::getIntValue('typrozdeleniusc') : null);
            $rec ->fRozdeleni ->setNewValue($rozdeleni ?: null);
        }

        if (RonPortalSettings::getIntValue('typrozdeleniusc2')) {
            $rozdeleni2 = (isset($_POST['Rozdeleni2'])  &&  $_POST['Rozdeleni2'] != '') ?  $_POST['Rozdeleni2'] : null;
            $rec ->fKodTypuRozdeleni2 ->setNewValue($rozdeleni2 ? RonPortalSettings::getIntValue('typrozdeleniusc2') : null);
            $rec ->fRozdeleni2 ->setNewValue($rozdeleni2 ?: null);
        }


        return true;
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param object|string $logDescription
     * @return bool
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    static function ulozUctovaniSluzebniCesty($database, $sc, $logDescription)
    {
        // TODO: zkontrolovat, jestli se nezmenil typ z nepovoleneho na povoleny?
        $typ = (int)$_POST['Typ'];
        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $typ)) {
            RonPortalUtils::neniPravoException();
        }

        $rec = new RadekUctovaniSCDBRecord($database, (int)$_POST['SluzebniCesta_ID'], (int)$_POST['CisloRadku'], $logDescription);

        if ( !self::loadPostToRecord($database, $rec, $sc) ) {
            return false;
        }

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba pri aktualizaci radku uctovani sluzebni cesty - radek nebyl nalezen!');
        }

        return $rec ->updateRecord();
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param object|string $logDescription
     * @return bool
     * @throws NeniPravoException
     */
    static function vlozNoveUctovaniSlCesty($database, $sc, $logDescription)
    {
        $typ = (int)$_POST['Typ'];
        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($sc ->f_PP ->idPP, $typ)) {
            RonPortalUtils::neniPravoException();
        }

        $rec = new RadekUctovaniSCDBRecord($database, (int)$_POST['SluzebniCesta_ID'], null, $logDescription);

        if ( !self::loadPostToRecord($database, $rec, $sc) ) {
            return false;
        }

        $rec ->fCisloRadku ->setNewValue( SQLUtils::newSerialNumber($database, 'RadekUctovaniSCesty', 'CisloRadku', 'SluzebniCesta_ID = ' . $_POST['SluzebniCesta_ID']) );

        $rec ->insertRecord();
        return true;
    }


    /**
     * Vraci seznam typu uctovani SC, ktere jsou NVV (kody oddelene strednikem)
     * @param Database $database
     * @return string
     */
    private static function typyUctovaniNVV($database)
    {
        $sql =
            'SELECT Kod, DruhPohybu, NahradaZaVlastniVozidlo FROM TypRUctovaniSlCesty' .
            ' WHERE Aktivni = 1 AND NahradaZaVlastniVozidlo = 1';

        $cursor = $database ->select($sql);
        $result = '';

        while ($row = $cursor ->fetch()) {
            $result .= ($result ? ';' : '') . $row[0];
        }

        return $result;
    }


//    /**
//     * vypise javascript - promenna se seznamem radku, ktere jsou NVV
//     * @param Database $database
//     * @throws DBException
//     */
//    private function typUctovaniSC_NVV_Script($database)
//    {
//        $sql = 'SELECT Kod, DruhPohybu, NahradaZaVlastniVozidlo FROM TypRUctovaniSlCesty WHERE Aktivni = 1';
//
//        $nahrady = [];
//        $prijmy = '';
//        $vydaje = '';
//
//        $cursor = $database ->select($sql);
//
//        while ($row = $cursor ->fetch()) {
//            if ($row[2]) {
//                // je nahrada
//                $nahrady[] .= $row[0];
//            }
//
//            if ($row[1] == 1) {
//                // prijem
//                if ($prijmy) $prijmy .= ', ';
//                $prijmy .= $row[0];
//            } else if ($row[1] == 2) {
//                // vydej
//                if ($vydaje) $vydaje .= ', ';
//                $vydaje .= $row[0];
//            }
//        }
//
//        $cursor ->close();
//
//        echo '<script language="Javascript">';
//
//        $data = [
//            'idSC' => $this ->idSluzebniCesty,
//            'typUctovaniSC_NVV' => $nahrady
//        ];
//
//        echo 'ron.sc.seznamUctovaniSC(' . json_encode($data) . ')';
//
//        echo "</script>\n";
//    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param string $akce
     */
    public static function zobrazUctovani($database, $sc, $akce)
    {
        $startStr = '';
        $cilStr = '';

        switch (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI)) {
            case 1 : // prednostne podle lookupu
                $startStr = $sc ->mistoZacatkuPopis; // lookup
                if (!$startStr) $startStr = $sc ->popisZacatku;

                $cilStr = $sc ->cilPopis; // lookup
                if (!$cilStr) $cilStr = $sc ->popisCile;
                break;

            case 2 : // prednostne podle popisu
                $startStr = $sc ->popisZacatku;
                if (!$startStr) $startStr = $sc ->mistoZacatkuPopis; // lookup

                $cilStr = $sc ->popisCile;
                if (!$cilStr) $cilStr = $sc ->cilPopis; // lookup
                break;
        }

        $b = new RadekUctovaniSlCestyBrowser($database, $sc, $akce);
        $b ->idSluzebniCesty = $sc ->id;
        $b ->f_vzdalenost_start = $startStr;
        $b ->f_vzdalenost_cil = $cilStr;
        $b ->f_vzdalenost_pres = $sc ->pres;
        $b ->f_datum = DateUtils::startOfTheDay($sc ->datumCasOd);

        $formaction = '';

        if ($b ->lzeEditovat) {
            switch ($akce) {
                case 'editovatuctovanisluzebky' :
                    $formaction = 'ulozuctovanisluzebky';
                    if (isset($_POST['CisloRadku'])) $b ->editovatRadek = $_POST['CisloRadku'];
                    else $b ->editovatRadek = $_GET['radek'];
                    break;

                case 'kopirovatuctovanisluzebky' :
                    $formaction = 'vlozuctovanisluzebky';
                    //if ( isset($_POST['KopirovatRadek']) ) $b -> f_kopirovatradek = $_POST['KopirovatRadek'];
                    //else $b -> f_kopirovatradek = $_GET['radek'];
                    $b ->kopirovatRadek = $_GET['radek'];
                    break;


                case 'pridatuctovanisluzebky' :
                    $formaction = 'vlozuctovanisluzebky';
                    $b ->zobrazitNovyRadek = true;
                    break;
            }
        }

        $b ->load("RadekUctovaniSCesty.SluzebniCesta_ID = {$sc ->id}");

        if ($formaction && $b ->lzeEditovat) {
            echo
                '<FORM id="UctovaniSluzebniCestyForm" method="post" class="sendPosition" name="radky_uctovanisc_form"' .
                ' action="?menuid=sluzebnicesty&action=' . $formaction . '"' .
                ' data-typy-uctovani-nvv="' . self::typyUctovaniNVV($database) . '"' .
                '>' . "\n" .
                '<INPUT type="hidden" name="x"><INPUT type="hidden" name="y">' . // pro zapamatovani scrollovani v prohlizeci
                "\n";
        }

        $b ->render();

        if ($formaction && $b ->lzeEditovat) {
            echo
                '<INPUT type="submit" class="submitbutton" value="' . TextManager::_('form.Store') . '">' .
                '<INPUT type="button" class="button clickHref" value="' . TextManager::_('Cancel') . '" data-ron-href="' . '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $sc ->id . '" data-ron-send-position="1">' .
                "</FORM>\n";
        }

        //$b ->typUctovaniSC_NVV_Script($database);

        if (!$formaction && $b ->lzeEditovat) {
            /** @noinspection HtmlUnknownAnchorTarget */
            echo
                '<br/>' .
                "<div class='text-link'>" .
                "<a class='sendPosition' href='?menuid=sluzebnicesty&action=pridatuctovanisluzebky&record={$sc ->id}'>" . TextManager::_('Novy_radek') . '</a>' .
                "</div>\n";

            if (RonPortalSettings::getSpousteniADS() ) {
                echo
                    "<div id='link_vyuctovani_sc'>" .
                    "<a class='sendPosition' href='?menuid=sluzebnicesty&action=vyuctovatsluzebky&record={$sc ->id}'>" . TextManager::_('Vyuctovani_sc') . '</a>' .
                    "</div>\n";
            }
        }
    }

}
