<?php
/**
 * UTF8
 * Stanislav Elbl, 18.9.2014
 * Hromadna editace radku sluzbeni cesty
 */

namespace Ron\RonPortal;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;


class RadekSluzebniCesty
{
    var $idSluzebniCesty;

    var $cisloRadku;
    var $popis;
    var $datumCasOd;
    var $casStr; // text pro zobrazeni/zadany uzivatelem
    var $kodOperace;
    var $stat;
    var $dopravniProstredek;

    var $misto;
    //var $f_popismista;
    var $poznamka;

    var $pocetPJidel_S;
    var $pocetPJidel_O;
    var $pocetPJidel_V;

    var $error; // priznak chyby


    function __construct($idSluzebniCesty)
    {
        $this -> idSluzebniCesty = $idSluzebniCesty;
        $this -> error = false;
    }


    /**
     * @param RadekSluzebniCesty $radek1
     * @param RadekSluzebniCesty $radek2
     * @return int
     */
    public static function compareRadekSluzebniCesty($radek1, $radek2) {
        return $radek1 ->datumCasOd < $radek2 ->datumCasOd ? -1 : ($radek1 ->datumCasOd > $radek2 ->datumCasOd ? 1 : 0);
    }


    /**
     * @param RadekSluzebniCesty[] $array
     */
    public static function sort($array)
    {
        usort($array, '\Ron\RonPortal\RadekSluzebniCesty::compareRadekSluzebniCesty' );
    }
}




class RadekSluzebniCestyFormular implements ViewObject
{
    /** @var  $f_SluzebniCesta SluzebniCesta */
    var $sluzebniCesta;

    /** @var  Database */
    var $database;

    /** @var  $radky RadekSluzebniCesty[] */
    var $radky;

    var $readOnly;
    var $zobrazitMisto;  // nastavi se na true, pokud jsou v databazi nejaka aktivni mista


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     */
    function __construct($database, $sluzebniCesta) {
        $this ->database = $database;
        $this ->sluzebniCesta = $sluzebniCesta;
    }


    /**
     * @return bool
     */
    function loadRadkyDB() {
        $this ->readOnly = !$this ->sluzebniCesta ->lzeEditovatSluzebniCestu($this ->database, false);

        $this ->zobrazitMisto = false;
        $cursor = $this ->database ->getSqlResult('SELECT COUNT(*) FROM MistoSluzebniCesty WHERE Aktivni = 1');
        if ($row = $cursor ->fetch()) {
            if ($row[0]) {
                $this ->zobrazitMisto = true;
            }
        }

        $this ->radky = [];

        $sql =
            'SELECT SluzebniCesta_ID, CisloRadku, Popis, DatumCasOd, KodOperace, Stat, Misto, '. // 0..6
                  ' PocetPJidel_S, PocetPJidel_O, PocetPJidel_V, DopravniProstredek, Poznamka '. // 7..11
            '  FROM RadekSluzebniCesty '.
            ' WHERE SluzebniCesta_ID = '.(int)$this -> sluzebniCesta -> id.
            ' ORDER BY datumCasOd';

        $cursor = $this ->database ->select($sql);
        if (!$cursor) return !MessageManager::getInstance() ->addDbError($this ->database, $sql, null, null);

        while ($row  = $cursor ->fetch()) {
            $radek = new RadekSluzebniCesty($this -> sluzebniCesta -> id);

            $radek ->cisloRadku = $row[1];
            $radek ->popis = $row[2];
            $radek ->datumCasOd = $this ->database ->stringToTimestamp($row[3]);
            $radek ->casStr = date('G:i', $radek -> datumCasOd);
            $radek ->kodOperace = $row[4];
            $radek ->stat = $row[5];
            $radek ->misto = $row[6];
            $radek ->pocetPJidel_S = $row[7];
            $radek ->pocetPJidel_O = $row[8];
            $radek ->pocetPJidel_V = $row[9];
            $radek ->dopravniProstredek = $row[10];
            $radek ->poznamka = $row[11];
            $this ->radky[] = $radek;
        }

        return true;
    }


    function vratRadek($cisloRadku) {
        foreach($this ->radky as $radek) {
            if ( $radek ->cisloRadku == $cisloRadku ) return $radek;
        }

        return null;
    }


    /**
     * @param RadekSluzebniCesty $radek
     * @param bool     $delete
     * @return bool
     */
    function updateRadekDB($radek, $delete)
    {
        require_once('sluzebnicesty/radeksc.php');
        $rec = new RadekSCDBRecord($this->database, $radek->idSluzebniCesty, $radek->cisloRadku);
        $rec->fPopis->setNewValue($radek->popis);
        $rec->fPoznamka ->setNewValue($radek ->poznamka);
        $rec->fKodOperace->setNewValue($radek->kodOperace);
        if ($this ->zobrazitMisto) {
            $rec->fMisto->setNewValue($radek->misto);
        }
        $rec->fStat->setNewValue($radek->stat);
        $rec->fDatumCasOd->setNewValue($radek->datumCasOd);
        $rec->fDopravniProstredek->setNewValue($radek->dopravniProstredek);

        $rec->fPocetPJidel_S->setNewValue($radek->pocetPJidel_S);
        $rec->fPocetPJidel_O->setNewValue($radek->pocetPJidel_O);
        $rec->fPocetPJidel_V->setNewValue($radek->pocetPJidel_V);

        if ($radek->cisloRadku) {
            if ($delete) {
                return $rec->deleteRecord();
            } else {
                try {
                    if (!$rec->loadRecord()) {
                        return MessageManager::addErrorMessage('Chyba! Editovany radek nebyl nalezen!');
                    }

                    $rec->updateRecord();
                    return  true;
                } catch (\Exception $e) {
                    return MessageManager::addErrorMessage('xPri editaci zaznamu doslo k chybe! Informace o chybe byla zapsana do LOGu!' . $e ->getMessage());
                }
            }
        } else {
            // radek nema pridelene cislo => vlozi novy radek
            try {
                $radek->cisloRadku = $rec->insertRadek();
                return true;
            } catch (\Exception $e) {
                return MessageManager::addErrorMessage('Pri vkladani zaznamu doslo k chybe! Informace o chybe byla zapsana do LOGu!');
            }
        }
    }


    /**
     * Nacte zadana data. V pripade chyby vypise chybove hlaseni a vraci false.
     * @param RadekSluzebniCesty $radek
     * @param int $indexHtml
     * @return bool
     */
    function loadRadekFromPost($radek, $indexHtml)
    {
        try {
            $datum = DateUtils::dateStringToTimestamp($_POST["datum_$indexHtml"]);
        } catch (\Exception $e) {
            // nemelo by nastat, datum je hidden
            $radek ->error = true;
            MessageManager::addErrorMessage('Chyba ve funkci loadRadekFromPost: ' . TextManager::_('Bad_date_format') . ' (' . $_POST["datum_$indexHtml"] . ')');
            $datum = DateUtils::startOfTheDay($this ->sluzebniCesta ->datumCasOd); // jen kvuli warningum
        }

        try {
            if (trim($_POST["casod_$indexHtml"] == '')) {
                $cas = $datum;
            } else {
                $cas = DateUtils::timeStringToTimestamp($_POST["casod_$indexHtml"], $datum);
            }

            $radek ->datumCasOd = $cas;
            $radek ->casStr = date('G:i', $radek ->datumCasOd);
        } catch (\Exception $e) {
            $radek ->datumCasOd = $datum;
            $radek ->error = true;
            $radek ->casStr = $_POST["casod_$indexHtml"];
            MessageManager::addErrorMessage( TextManager::_('Bad_time_format') );
        }

        $radek ->kodOperace = isset($_POST["operace_$indexHtml"]) ? $_POST["operace_$indexHtml"] : null;
        if ($this ->zobrazitMisto) {
            $radek->misto = $_POST["misto_$indexHtml"];
        }

        $radek ->popis = $_POST["popis_$indexHtml"];
        $radek ->poznamka = $_POST["poznamka_$indexHtml"];

        // pokud sluzebni cesta neni zahranicni (zjisteno z POST kvuli kompatibilite se starou verzi),
        // tak stat nastavi vychozi stat SC
        if ($_POST['jezahranicni']) $radek ->stat = $_POST["stat_$indexHtml"];
        else $radek ->stat = $this ->sluzebniCesta ->f_stat;

        $radek ->dopravniProstredek = $_POST["dopravniProstredek_$indexHtml"];

        $radek ->pocetPJidel_S = isset($_POST["pocetPJidel_S_{$indexHtml}_E"]) ?
            ( (int)$_POST["pocetPJidel_S_{$indexHtml}_E"] ) :
            ( isset($_POST["pocetPJidel_S_{$indexHtml}"])  &&  $_POST["pocetPJidel_S_{$indexHtml}"] ? 1 : 0 );

        $radek ->pocetPJidel_O = isset($_POST["pocetPJidel_O_{$indexHtml}_E"]) ?
            ( (int)$_POST["pocetPJidel_O_{$indexHtml}_E"] ) :
            ( isset($_POST["pocetPJidel_O_{$indexHtml}"])  &&  $_POST["pocetPJidel_O_{$indexHtml}"] ? 1 : 0 );

        $radek ->pocetPJidel_V = isset($_POST["pocetPJidel_V_{$indexHtml}_E"]) ?
            ( (int)$_POST["pocetPJidel_V_{$indexHtml}_E"] ) :
            ( isset($_POST["pocetPJidel_V_{$indexHtml}"])  &&  $_POST["pocetPJidel_V_{$indexHtml}"] ? 1 : 0 );

        return !$radek -> error;
    }


    /**
     * Zkontroluje radek pred zapsanim
     * @param RadekSluzebniCesty $radek
     * @param int $indexhtml
     * @return bool
     */
    function zkontrolujRadek($radek, $indexhtml) {
        // zkontroluje, jestli neni zadan cas mimo rozsah sluzebni cesty
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY)) {
            // cas pocatku a konce se nastavuje automaticky - kontroluje jenom datum
            // chyba by nemela vubec nastat, protoze uzivatel cas nezadava - jedine, ze by mezitim zmenil datumy sl. cesty.

            if ($radek ->datumCasOd < DateUtils::startOfTheDay($this ->sluzebniCesta ->datumCasOd)  ||  $radek ->datumCasOd > DateUtils::endOfTheDay($this ->sluzebniCesta ->datumCasDo) ) {
                $radek ->error = true;

                MessageManager::addErrorMessage(
                    \Ron\ronText('Datum_mimo_rozsah_sc') . ' ' .
                    date('j.n.Y', $this ->sluzebniCesta ->datumCasOd) . ' - ' .
                    date('j.n.Y', $this ->sluzebniCesta ->datumCasDo),
                    "casod_$indexhtml");
            }
        }
        else {
            if ($radek ->datumCasOd < $this ->sluzebniCesta ->datumCasOd || $radek ->datumCasOd > $this ->sluzebniCesta ->datumCasDo) {
                $radek ->error = true;

                MessageManager::addErrorMessage(
                    \Ron\ronText('Cas_mimo_rozsah_sc') . ' ' .
                    date('j.n.Y G:i', $this ->sluzebniCesta ->datumCasOd) . ' - ' .
                    date('j.n.Y G:i', $this ->sluzebniCesta ->datumCasDo),
                    "casod_$indexhtml");
            }
        }

        return true;
    }


    /**
     * @param RadekSluzebniCesty $radek
     * @param int $indexhtml
     * @return bool|int
     */
    function updateRadek($radek, $indexhtml) {
        if ( !$this ->loadRadekFromPost($radek, $indexhtml) ) return false;

        if ( !$this ->zkontrolujRadek($radek, $indexhtml)) return false;

        return $this -> updateRadekDB($radek, false); // delete = false
    }


    /**
     * @param int $indexHtml
     * @return bool
     */
    function deleteRadek($indexHtml) {
        if (isset($_POST['cisloradku_' . $indexHtml])) {
            foreach($this ->radky as $index => $radek) {
                if ( $radek ->cisloRadku == $_POST['cisloradku_' . $indexHtml] ) {
                    // smaze radek z DB
                    if ( !$this -> updateRadekDB($radek, true) ) { // delete = true
                        return false;
                    }

                    array_splice($this ->radky, $index, 1);
                    return true;
                }
            }
        }

        return true;
    }


    /**
     * @param int $indexhtml
     * @return bool|null|RadekSluzebniCesty
     */
    function insertRadek($indexhtml) {
        $vlozit = false;

        $radek = new RadekSluzebniCesty($this ->sluzebniCesta ->id);

        if ( !$this -> loadRadekFromPost($radek, $indexhtml) ) return $radek; // v radku je chyba - nevklada ho

        if ($radek ->pocetPJidel_S  ||  $radek ->pocetPJidel_O  ||  $radek ->pocetPJidel_V) $vlozit = true;

        if (!$vlozit  &&  $_POST["casod_$indexhtml"] == '') return null; // neni zadan cas ani zaskrtnuto jidlo, radek neni vubec zobrazen, neuklada se

        if ($radek ->pocetPJidel_S  ||  $radek ->pocetPJidel_O  ||  $radek ->pocetPJidel_V) $vlozit = true;
        if ($radek ->stat) $vlozit = true;
        if ($this ->zobrazitMisto  &&  $radek ->misto) $vlozit = true;
        if ($radek ->popis) $vlozit = true;
        if ($radek ->kodOperace) $vlozit = true;

        if ( !$this ->zkontrolujRadek($radek, $indexhtml)) return false;

        if ($vlozit) {
            if (!$radek ->stat) {
                $radek ->error = true;
                MessageManager::addErrorMessage(\Ron\ronText('Neni_zadan_stat'), "stat_$indexhtml");
                return $radek;
            }
            $radek -> error = !$this -> updateRadekDB($radek, false);

            return $radek;
        }

        return null; // zadny novy radek nepridavam
    }


    function updateCasySC() {
        $this ->sluzebniCesta ->updateCasy($this ->database);
    }


    /**
     * @return bool
     */
    function loadPost() {
        if ( !isset($_POST['editace_radku_sluzebky'])  ||  !$_POST['editace_radku_sluzebky']  ||
             !isset($_POST['pocetradku'])  ||  !$_POST['pocetradku'] ) {
            return true;
        }

        $result = true;
        $noveradky = [];

        for ($i = 1;  $i <= $_POST['pocetradku'];  $i++) {
            if ( isset($_POST['smazat_radek_sluzebky']) &&  (int)$_POST['smazat_radek_sluzebky'] == $i ) {
                if ( !$this ->deleteRadek($i) ) $result = false;
            }
            else {
                /** @var $radek RadekSluzebniCesty */
                if (isset($_POST['cisloradku_' . $i])) $radek = $this ->vratRadek((int)$_POST['cisloradku_' . $i]);
                else $radek = null;

                // nacte postnuty radek a v pripade potreby provede update/insert
                if ($radek) {
                    if (!$this ->updateRadek($radek, $i)) $result = false;
                } else {
                    $novyradek = $this ->insertRadek($i);

                    if ($novyradek) {
                        $noveradky[] = $novyradek;
                        if ($novyradek ->error) $result = false;
                    }
                }
            }
        }


        foreach ($noveradky as $radek) $this ->radky[] = $radek;

        RadekSluzebniCesty::sort($this ->radky);

        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY)) $this ->updateCasySC();

        return $result;
    }


    /**
     * @return bool
     */
    function loadRadky() {
        if ( !$this -> loadRadkyDB() ) return false;

        if ( !$this -> loadPost() ) return false;

        return true;
    }



    function showInput_PocetJidel($name, $index, $value, $zobrazitVzdy) {
        $value = (int)$value;

        if ( !$zobrazitVzdy  &&  !$value ) return;

        if ($value > 1) echo "<input name='{$name}{$index}_E' size='2' value='$value'/>";
        else echo "<input name='$name$index' type='checkbox'". ($value ? " checked='checked'" : '') . " data-ron-index='$index'/>";
    }


    /**
     * @param RadekSluzebniCesty $radek
     * @param int  $index
     * @param bool $prvniRadekProDen    Urcuje, jestli se jedna o prvni radek pro zadany den
     * @param bool $jeZahranicni
     * @param bool $jinyStat
     */
    function zobrazRadek($radek, $index, $prvniRadekProDen, $jeZahranicni, $jinyStat)
    {
        $days = DateUtils::daysBetween($radek->datumCasOd, $this->sluzebniCesta->datumCasOd);
        $className = $days % 2 ? 'browsercolor1' : 'browsercolor2';

        if ($radek->error) $className .= ' error-back';
        if ($radek ->poznamka != '') $className .= ' jePoznamkaSC';

        $className1 = $prvniRadekProDen ? ' prvniradekprodenSC padding-top-2' : '';
        if (!$radek->cisloRadku && !$radek->error) $className1 .= ' novyradekSC';

        $pocetSloupcuPoznamka = 3;

        echo
            "<tr class='$className$className1' id='radekSC_$index'>" .
            '<td>' .
            '<span class="zkratkadne">' . \Ron\ronText('Day_short_' . date('w', $radek->datumCasOd)) . '</span>' . date('d.m.Y', $radek->datumCasOd) .
            '</td>';

        if ($this->readOnly) {
            echo
                '<td>' . date('G:i', $radek->datumCasOd) . '</td>' .
                // operace
                ( SluzebniCestyController::getZobrazitOperaceSC()
                    ? '<td>' .
                      ($radek->kodOperace
                          ? DBUtils::getSqlValue($this ->database, "SELECT Popis FROM OperaceDochazky WHERE Kod = '" . $this ->database ->addSlashes($radek->kodOperace) . "'")
                          : '') .
                      '</td>'
                    : '');

            if ($this ->zobrazitMisto) {
                // misto
                echo
                    '<td class="radekSC_misto">' .
                    ($radek->misto
                        ? DBUtils::getSqlValue($this->database, "SELECT Popis FROM MistoSluzebniCesty WHERE Kod = '" . $this->database->addSlashes($radek->misto) . "'")
                        : ''
                    ) .
                    '</td>';
            }

            echo
                '<td class="radekSC_popis">' . StringUtils::htmlTextConvert($radek->popis) . '</td>' .
                ($jeZahranicni ? '<td class="radekSC_stat">' . $radek->stat . '</td>' : '' ) .
                '<td class="radekSC_dopravniprostredek">' .
                    ($radek->dopravniProstredek
                        ? DBUtils::getSqlValue($this ->database, "SELECT Popis FROM DopravniProstredek WHERE Kod = '" . $this ->database ->addSlashes($radek->dopravniProstredek) . "'")
                        : ''
                    ).

                '</td>' .
                '<td class="radekSC_snidane align-center">' . ($radek->pocetPJidel_S ? $radek->pocetPJidel_S : '') . '</td>' .
                '<td class="radekSC_obed align-center">' . ($radek->pocetPJidel_O ? $radek->pocetPJidel_O : '') . '</td>' .
                '<td class="radekSC_vecere align-center">' . ($radek->pocetPJidel_V ? $radek->pocetPJidel_V : '') . '</td>';
        } else {
            //$keyup = $radek->f_cisloRadku ? '' : "onkeyup='return novyRadekSlCesty_Change($index)'";
            require_once('ron/controls.php');

            echo
                '<td class="radekSC_cas" >' .
                "<input type='text' class='appinput inputTime' name='casod_$index' size='5' value='{$radek->casStr}' data-ron-index='$index' />" .      // $keyup
                "<input type='hidden' name='cisloradku_$index' value='{$radek->cisloRadku}' />" .
                "<input type='hidden' name='datum_$index' value='" . date('d.m.Y', $radek->datumCasOd) . "' />" .
                '</td>';

            if ( SluzebniCestyController::getZobrazitOperaceSC() ) {
                // operace
                echo '<td class="radekSC_operace">';
                $combo = new ComboBox('operace_' . $index);
                $combo ->emptyRow = ' ';
                $combo ->loadData($this ->database,
                    'SELECT Kod, Popis FROM OperaceDochazky WHERE ' . ($this ->sluzebniCesta ->zahranicni ? 'UdalostZahranicniSC' : 'UdalostSluzebniCesty') . ' = 1 ORDER BY Popis',
                    $radek->kodOperace
                );
                $combo ->render();
                echo '</td>';

                $pocetSloupcuPoznamka ++;
            }

            // misto
            if ($this ->zobrazitMisto) {
                echo '<td class="radekSC_misto">';
                $combo = new ComboBox('misto_' . $index);
                $combo->emptyRow = ' ';
                $combo->htmlParams = "data-ron-index='$index'";
                $combo->loadData($this->database,
                    'SELECT Kod, Popis FROM MistoSluzebniCesty WHERE Aktivni = 1 ORDER BY Popis',
                    $radek->misto
                );
                $combo->render();
                echo '</td>';

                $pocetSloupcuPoznamka ++;
            }


            echo
                "<td class='radekSC_popis'><input name='popis_$index' size='40' value='" . StringUtils::htmlTextConvert($radek->popis) . "' /></td>";

            if ($jeZahranicni) {
                // stat
                echo '<td class="radekSC_stat">';
                $combo = new ComboBox('stat_' . $index);
                if (!$radek->stat) $combo ->emptyRow = ' ';
                $combo->loadData($this ->database,
                    'SELECT Kod, Popis FROM Stat WHERE PovolitVeSluzCeste = 1 ORDER BY Popis',
                    $radek->stat);
                $combo->render();
                echo '</td>';

                $pocetSloupcuPoznamka ++;
            }

            // dopravni prostredek
            echo '<td class="radekSC_dopravniprostredek">';
            //if ($prvniRadekProDen || $radek->f_dopravniProstredek) {  - zobrazit pro vsechny dny, zakaznici to chteji
                $combo = new ComboBox('dopravniProstredek_' . $index);
                $combo ->emptyRow = ' ';
                $combo->loadData($this ->database,
                    'SELECT Kod, Popis FROM DopravniProstredek ORDER BY Popis',
                    $radek->dopravniProstredek);
                $combo->render();
            //}
            echo '</td>';

            echo "<td class='radekSC_editovat_poznamku'><i class='radekSC_btn_poznamka inline-glyph glyph-button fa glyph-comment' id='radekSC_btn_poznamka_$index' data-ron-index='$index' title='" . \Ron\ronText('Editovat_poznamku_radku_sc') . "'></i></td>";

            echo '<td class="radekSC_snidane align-center">';
            $this->showInput_PocetJidel('pocetPJidel_S_', $index, $radek->pocetPJidel_S, $prvniRadekProDen || $jinyStat);
            echo '</td>';

            echo '<td class="radekSC_obed align-center">';
            $this->showInput_PocetJidel('pocetPJidel_O_', $index, $radek->pocetPJidel_O, $prvniRadekProDen || $jinyStat);
            echo '</td>';

            echo '<td class="radekSC_vecere align-center">';
            $this->showInput_PocetJidel('pocetPJidel_V_', $index, $radek->pocetPJidel_V, $prvniRadekProDen || $jinyStat);
            echo '</td>';

            echo '<td class="radekSC_editace align-center">';
            /** @noinspection HtmlUnknownTarget */
            echo "<i class='inline-glyph glyph-button fa glyph-delete' title='" . \Ron\ronText('Smazat_radek') . "' data-ron-deleterow='$index'></i>";
            echo '</td>';
        }

        echo '<td class="radekSC_vydaje">';

        if ($prvniRadekProDen) {
            $datum = DateUtils::startOfTheDay($radek ->datumCasOd);
            echo "<input type='button' class='button smallbutton' name='vydaje_$index' value='" . \Ron\ronText('Vydaje_sc') . "' data-ron-vydaje='$datum' />";
        }

        echo '</td>';

        echo "</tr>\n";

        // poznamka
        if (!$this->readOnly) {
            echo
                "<tr class='$className poznamka browserpoznamka radekSC_poznamka' id='radekSC_poznamka_$index'>" .
                "<td class='formtablecaption'>" . \Ron\ronText('Poznamka') . "</td>" . // datum
                "<td colspan='$pocetSloupcuPoznamka'><input class='poznamkaRadkuSC' name='poznamka_$index' maxlength='250' value='" . StringUtils::htmlTextConvert($radek ->poznamka) . "'></td><td class='radekSC_editovat_poznamku'></td>" .
                "<td class='radekSC_snidane'></td><td class='radekSC_obed'></td><td class='radekSC_vecere'></td>" .
                "<td class='radekSC_editace'></td><td class='radekSC_vydaje'></td>";
        }
    }


    /**
     * Vytvori seznam radku pro zobrazeni (zkopiruje existujici radky a doplni prazdne)
     * @return RadekSluzebniCesty[]
     */
    function radkyProZobrazeni() {
        $radkyprozobrazeni = array();

        $indexradku = 0;
        $indexhtml = 0; // index radku v html
        $datum = DateUtils::startOfTheDay($this -> sluzebniCesta -> datumCasOd);
        $zobrazenychradkuproden = 0;
        $prazdnychradku = 0;

        while ($datum <= $this -> sluzebniCesta -> datumCasDo  ||  $indexradku < count($this ->radky) ) {
            // pokud je to radek na aktualni den, tak jej zobrazi
            if ($indexradku < count($this ->radky)  &&  $datum >= DateUtils::startOfTheDay($this ->radky[$indexradku] ->datumCasOd) ) {
                $indexhtml ++;
                // aktualni radek patri do zobrazovaneho dne => zobrazi radek
                $radkyprozobrazeni[] = $this ->radky[$indexradku];

                $indexradku ++;
                $zobrazenychradkuproden ++;
            }

            // pokud je mezi radky mezera (dny bez zaznamu) nebo ma pro den jen jeden zaznam, tak zobrazi prazdne radky
            if ($indexradku >= count($this->radky) || $datum < DateUtils::startOfTheDay($this->radky[$indexradku]->datumCasOd)) {
                if ($this ->sluzebniCesta ->lzeEditovat) {
                    while ($zobrazenychradkuproden < 2 || $prazdnychradku < 1) {
                        $radek = new RadekSluzebniCesty($this->sluzebniCesta->id);
                        $radek->datumCasOd = $datum;
                        $indexhtml++;

                        $radkyprozobrazeni[] = $radek;
                        $zobrazenychradkuproden++;
                        $prazdnychradku++;
                    }
                }

                $datum = DateUtils::incDay($datum, 1);
                $zobrazenychradkuproden = 0;
                $prazdnychradku = 0;
            }
        }

        return $radkyprozobrazeni;
    }



    /**
     */
    public function render()
    {
        echo '<h1>' . \Ron\ronText('Radky_sc') . " <span id='cisloSC'>{$this ->sluzebniCesta ->id}</span></h1>";

        $this ->sluzebniCesta -> zobrazZkracenouHlavicku();

        require('sluzebnicesty/view_radekformular.php');
    }


}
