<?php
/**
 * UTF8
 * stanislav.elbl, 22.12.2015
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\DBRecordField;
use Ron\StringUtils;

require_once('adsapp/ronportaldbrecord.php');


class RadekSCDBRecord extends RonPortalDBRecord
{
    /** @var int */
    var $idSC;

    /** @var  DBRecordField */
    var $fCisloRadku;
    /** @var  DBRecordField */
    var $fPopis;
    /** @var  DBRecordField */
    var $fKodOperace;
    /** @var  DBRecordField */
    var $fMisto;
    /** @var  DBRecordField */
    var $fStat;
    /** @var  DBRecordField */
    var $fDatumCasOd;
    /** @var  DBRecordField */
    var $fDopravniProstredek;
    /** @var  DBRecordField */
    var $PocetPJidel_S;
    /** @var  DBRecordField */
    var $PocetPJidel_O;
    /** @var  DBRecordField */
    var $PocetPJidel_V;

    /** @var  DBRecordField */
    var $fPoznamka;

    /**
     * @param Database $database
     * @param int  $idSC
     * @param int  $cisloRadku
     */
    function __construct($database, $idSC, $cisloRadku = null)
    {
        parent::__construct($database, 'RadekSluzebniCesty');

        $this ->idSC = $idSC;

        $this ->logDescription = StringUtils::fromUTF8('Služební cesty');

        require_once 'adsapp/events.php';
        $this ->beforePostEvent = RonPortalXMLEvents::SC_RADEK_BEFORE_POST;
        $this ->afterPostEvent = RonPortalXMLEvents::SC_RADEK_AFTER_POST;

        $this ->addField('SluzebniCesta_ID', 'int', true, 0, $idSC); // readonly = true
        $this ->fCisloRadku = $this ->addField('CisloRadku', 'int', true, 0, $cisloRadku);

        $this ->fPopis = $this ->addField('Popis', '', true, 160);
        $this ->fKodOperace = $this ->addField('KodOperace', '', true, 10);
        $this ->fMisto = $this ->addField('Misto', '', true, 30);
        $this ->fStat = $this ->addField('Stat', '', true, 10);
        $this ->fDatumCasOd = $this ->addField('DatumCasOd', 'timestamp', true);
        $this ->fDopravniProstredek = $this -> addField('DopravniProstredek', '', true, 10);
        $this ->fPocetPJidel_S = $this -> addField('PocetPJidel_S', 'int', true);
        $this ->fPocetPJidel_O = $this -> addField('PocetPJidel_O', 'int', true);
        $this ->fPocetPJidel_V = $this -> addField('PocetPJidel_V', 'int', true);
        $this ->fPoznamka = $this ->addField('Poznamka', '', true, 250);

        $params = $database ->createParameters();
        $where =
            'SluzebniCesta_ID = ' . $params ->addParameter((int)$idSC);

        if ($cisloRadku) {
            $where .= ' AND CisloRadku = ' . $params ->addParameter((int)$cisloRadku);
        }

        $this ->setWhere($where, $params);
    }


    function insertRadek() {
        require_once('ron/sqlutils.php');

        $noveCislo = SQLUtils::newSerialNumber($this ->database,
            'RadekSluzebniCesty', 'CisloRadku', 'SluzebniCesta_ID = '.(int)$this ->idSC);

        $this ->fCisloRadku ->setNewValue( $noveCislo );

        if ( !$this ->insertRecord() ) {
            throw new \Exception('Pri vkladani zaznamu doslo k chybe! Informace o chybe byla zapsana do LOGu!');
        }

        return $noveCislo;
    }
}
