<?php
/**
 * stanislav.elbl, 24.7.2014
 */

namespace Ron\RonPortal;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\IcalOutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\RonPortal\XML\XmlFormException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\SluzebniCesty\PredpokladanyVydajSC;
use RonPortal\SluzebniCesty\SluzebniCestaBrowser;
use RonPortal\SluzebniCesty\SluzebniCestaCalItem;


require_once 'adsapp/ronportalutils.php';
require_once 'sluzebnicesty/sluzebnicesty.php';
require_once 'sluzebnicesty/sc_record.php';


class SluzebniCestyController extends HttpController
{
    //const LOG_DESCRIPTION = 'Sluzebni cesty';
    const SETTINGS_POVOLIT_SC = 'povolitsluzebnicesty';
    const SETTINGS_PORADI_OPERACI = 'poradioperacisc';
    const SETTINGS_POUZIVAT_VRACENI =  'vracenisluzebnich';

    /** @var  Database */
    protected $database;


    public function __construct($database)
    {
        $this->database = $database;
    }


    public static function getZobrazitOperaceSC()
    {
        return RonPortalSettings::getValue(self::SETTINGS_PORADI_OPERACI) != 'NULL';
    }

    /**
     * @param SluzebniCesta $sluzebniCesta
     * @param string $action
     * @return ViewObject
     * @throws NeniPravoException
     */
    function sluzebniCestaView($sluzebniCesta, $action = 'zobrazslcestu')
    {
        if (!$sluzebniCesta->lzeZobrazitSluzebniCestu()) {
            throw new NeniPravoException();
        }

        require_once('sluzebnicesty/sc_viewobject.php');
        return new SluzebniCestaView($this->database, $sluzebniCesta, $action);
    }


    function sluzebniCestaViewById($id, $action = 'zobrazslcestu')
    {
        $sluzebniCesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);
        return $this->sluzebniCestaView($sluzebniCesta, $action);
    }


    function sluzebniCestyBrowser($schvalovani = false)
    {
        $browser = new SluzebniCestaBrowser($this->database, $schvalovani); // schvalovani = false
        $browser->load();
        return $browser;
    }


    /**
     * @param int $id
     * @param bool $editaceHlavicky True, pokud se jedna o editaci hlavicky SC - v tom pripade se kontroluje i schvaleni nadrizeneho
     * @return SluzebniCesta
     * @throws \Exception
     */
    private function nactiSCProEditaci($id, $editaceHlavicky = false)
    {
        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);
        $cesta->lzeEditovat($this->database, true); // raise exception = true

        if ($editaceHlavicky) {
            if (!$cesta->lzeEditovatHlavicku($this->database)) {
                throw new \Exception(\Ron\ronText('Sluzebka_je_jiz_schvalena'));
            };
        }

        return $cesta;
    }


    private function sluzebniCestaFormView($sluzebniCesta, $formAction)
    {
        require_once('sluzebnicesty/f_sluzebnicesta.php');
        $form = new SluzebniCestaFormular($this ->database, $sluzebniCesta, $formAction);
        $form ->load();

        return $form;
//        Model::addData('database', $this->database);
//        Model::addData('sluzebniCesta', $sluzebniCesta);
//        Model::addData('formAction', $formAction);
//
//        return new BaseViewObject('sluzebnicesty/view_formular.php');
    }


    /**
     * @param string $formAction
     * @return ViewObject
     * @throws \Exception
     */
    function sluzebniCestaFormular($formAction)
    {
        $cesta = new SluzebniCesta();

        switch ($formAction) {
            case 'new' :
                Prava::nastavObdobiProPrava(time(), time());
                $cesta->loadDefaultValues($this->database);
                return $this->sluzebniCestaFormView($cesta, 'insert');

            case 'newfromcopy' :
                $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $_GET['record']);

                if (!$cesta->lzeZobrazitSluzebniCestu()) {
                    throw new NeniPravoException();
                }
                $cesta->id = null;
                return $this->sluzebniCestaFormView($cesta, 'copy');

            case 'insert' :
                try {
                    $cesta->predpokladaneNaklady = PredpokladanyVydajSC::nactiPredpokladaneVydaje($this->database, 0);
                    if ($cesta->loadPost($this->database)) {
                        if ($cesta->lzeVlozitSluzebniCestu($this->database)) {
                            if ($cesta->insert($this->database)) {
                                return $this->redirectToView($cesta->id);
                            }
                        }
                    }
                } catch (XmlFormException $e) {
                    if (!MessageManager::getInstance() ->isError()) {
                        MessageManager::addErrorMessage($e);
                    }
                } catch (\Exception $e) {
                    MessageManager::addErrorMessage($e);
                }
                return $this->sluzebniCestaFormView($cesta, 'insert');

            case 'copy' :
                $cesta->predpokladaneNaklady = PredpokladanyVydajSC::nactiPredpokladaneVydaje($this->database, 0);
                if ($cesta->loadPost($this->database)) {
                    if ($cesta->lzeVlozitSluzebniCestu($this->database)) {
                        if ($cesta->insert($this->database)) {
                            MessageManager::addDebugMessage('Probehlo vlozeni noveho zaznamu. Nasleduje kopirovani radku.');

                            SluzebniCesta::kopirujObsah($this->database,
                                $_POST['copyrecordid'], $cesta->id,
                                isset($_POST['kopirovatradky']) && $_POST['kopirovatradky'],
                                isset($_POST['kopirovatuctovani']) && $_POST['kopirovatuctovani']);

                            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY)) {
                                $cesta->updateCasy($this->database);
                            }

                            return $this->redirectToView($cesta->id);
                        }
                    }
                }
                return $this->sluzebniCestaFormView($cesta, 'copy');

            case 'edit' :
                $cesta = $this->nactiSCProEditaci($_GET['record'], true);
                return $this->sluzebniCestaFormView($cesta, 'save');

            case 'save' :
                $cesta = $this->nactiSCProEditaci($_POST['record'], true);

                try {
                    if ($cesta->loadPost($this->database)) {
                        if ($cesta->updateRecord($this->database)) {
                            return $this->redirectToView($cesta->id);
                        }
                    }
                } catch (\Exception $e) {
                    MessageManager::addErrorMessage($e);
                }

                return $this->sluzebniCestaFormView($cesta, 'save');


            default :
                throw new \Exception('Chyba v metode SluzebniCestyController::sluzebniCestaFormular! Neznama akce!');
        }
    }


    /**
     * @param int $id
     * @return Redirect
     */
    function redirectToView($id)
    {
        require_once('ron/guiutils.php');

        return new Redirect(null,
            '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $id . GuiUtils::scrollPageUrlParams());
    }


    /**
     * @param int $id
     * @param int $zauctovano
     * @return bool
     */
    function nastavZauctovaniSlCesty($id, $zauctovano)
    {
        $rec = new SluzebniCestaRecord($this->database, $id, $this);
        $rec->nastavZauctovaniSlCesty($zauctovano);

        return $this->redirectToView($id);
    }


    /**
     * @param int $id
     * @returns bool
     * @throws \Exception
     */
    function smazSluzebniCestu($id)
    {
        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);

        try {
            $cesta->lzeEditovat($this->database, true); // throw exception = true

            $OK = true;

            //kontrola jestli jsou radky - pokud ano, nelze smazat
            $sql = 'SELECT CisloRadku FROM RadekSluzebniCesty WHERE SluzebniCesta_ID = ' . (int)$id;

            if ($cursor = $this->database->getSqlResult($sql)) {
                if ($row = $cursor->fetch()) {
                    $OK = false;
                    MessageManager::addErrorMessage(\Ron\ronText('Sluzebka_obsahuje_radky'));
                }
            }

            //kontrola jestli je uctovani - pokud ano, nelze smazat
            $sql = 'SELECT CisloRadku FROM RadekUctovaniSCesty WHERE (Typ < 1  OR  Typ > 3)  AND SluzebniCesta_ID = ' . (int)$id;

            if ($cursor = $this->database->getSqlResult($sql)) {
                if ($row = $cursor->fetch()) {
                    $OK = false;
                    MessageManager::addErrorMessage(\Ron\ronText('Sluzebka_obsahuje_uctovani'));
                }
            }

            if ($OK) {
                $this ->database ->beginTransaction();
                try {
                    // smaze predpokladane naklady
                    $sql = 'DELETE FROM PredpokladaneNakladySC WHERE SluzebniCesta_ID = ' . (int)$id;

                    $this->database->sql($sql);

                    $rec = new SluzebniCestaRecord($this->database, $id, $this);
                    $rec ->setXmlEvents(RonPortalXMLEvents::SC_BEFORE_POST, RonPortalXMLEvents::SC_AFTER_POST,
                        ['sluzebnicesta' => $id]);

                    $rec->deleteRecord();

                    $this ->database ->commitTransaction();

                    MessageManager::addInfoMessage(\Ron\ronText('Zruseni_sluzebky_OK'));

                } catch (\Exception $e) {
                    $this ->database ->rollbackTransaction();

                    MessageManager::addErrorMessage($e);
                }
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->sluzebniCestyBrowser();
    }


    function ulozRadek()
    {
        $cesta = $this->nactiSCProEditaci((int)$_POST['SluzebniCesta_ID']);
        if (!$cesta) {
            throw new \Exception('Chyba pri editaci radku - nelze nacist pozadovanou sluzebni cestu!');
        }

        require_once('sluzebnicesty/b_radeksluzebnicesty.php');

        try {
            RadekSluzebniCestyBrowser::ulozRadekSluzebniCesty($this->database, $cesta);
        } catch (RonException $e) {
            MessageManager::addErrorMessage($e->getMessage(), $e->getFieldName());
            return $this->sluzebniCestaView($cesta, 'editovatradeksluzebky');
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
            return $this->sluzebniCestaView($cesta, 'editovatradeksluzebky');
        }

        return $this->redirectToView($cesta->id);
    }


    function vlozRadek()
    {
        $cesta = $this->nactiSCProEditaci((int)$_POST['SluzebniCesta_ID']);
        if (!$cesta) {
            throw new \Exception('Chyba pri mazani radku - nelze nacist pozadovanou sluzebni cestu!');
        }

        require_once('sluzebnicesty/b_radeksluzebnicesty.php');

        try {
            RadekSluzebniCestyBrowser::vlozNovyRadekSluzebniCesty($this->database, $cesta);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
            return $this->sluzebniCestaView($cesta, 'pridatradeksluzebky');
        }

        return $this->redirectToView($cesta->id);
    }


    function ulozRadekUctovani()
    {
        $cesta = $this->nactiSCProEditaci((int)$_POST['SluzebniCesta_ID']);

        require_once('sluzebnicesty/b_uctovanislcesty.php');
        if (!RadekUctovaniSlCestyBrowser::ulozUctovaniSluzebniCesty($this->database, $cesta, $this)) {
            return $this->sluzebniCestaView($cesta, 'editovatuctovanisluzebky');
        } else {
            return $this->redirectToView($cesta->id);
        }
    }


    function vlozRadekUctovani()
    {
        $cesta = $this->nactiSCProEditaci((int)$_POST['SluzebniCesta_ID']);
        if (!$cesta) {
            throw new \Exception('Chyba pri mazani radku - nelze nacist pozadovanou sluzebni cestu!');
        }

        require_once('sluzebnicesty/b_uctovanislcesty.php');
        if (!RadekUctovaniSlCestyBrowser::vlozNoveUctovaniSlCesty($this->database, $cesta, $this)) {
            return $this->sluzebniCestaView($cesta, 'pridatuctovanisluzebky');
        } else {
            return $this->redirectToView($cesta->id);
        }
    }

    /**
     * @param int $idSC
     * @param int $idRadku
     * @return bool
     * @throws \Exception
     */
    function smazRadek($idSC, $idRadku)
    {
        $cesta = $this->nactiSCProEditaci($idSC);
        if (!$cesta) {
            throw new \Exception('Chyba pri mazani radku - nelze nacist pozadovanou sluzebni cestu!');
        }

        require_once('sluzebnicesty/b_radeksluzebnicesty.php');

        if (RadekSluzebniCestyBrowser::zrusRadekSluzebniCesty($this->database, $cesta->id, $idRadku)) {
            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_CASY_AUTOMATICKY)) {
                if (!$cesta->updateCasy($this->database)) {
                    MessageManager::addErrorMessage('Pri aktualizaci casu pocatku a konce sluzebni cesty doslo k chybe!');
                }
            }
        }

        return $this->redirectToView($idSC);
    }


    /**
     * @param int $idSC
     * @param int $idRadku
     * @return bool
     * @throws \Exception
     */
    function smazRadekUctovani($idSC, $idRadku)
    {
        $cesta = $this->nactiSCProEditaci($idSC);
        if (!$cesta) {
            throw new \Exception('Chyba pri mazani radku uctovani - nelze nacist pozadovanou sluzebni cestu!');
        }

        try {
            require_once('sluzebnicesty/b_uctovanislcesty.php');
            RadekUctovaniSlCestyBrowser::zrusUctovaniSluzebniCesty($this->database, $cesta, $idRadku, $this);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $this->redirectToView($idSC);
    }


    function hromadnaEditaceUctovani($id)
    {
        if (isset($_POST['editace_radku_uctovani_sluzebky']) && $_POST['editace_radku_uctovani_sluzebky'] && $_POST['storno']) {
            // uzivatel kliknul na storno ve formulari uctovani
            return new Redirect(null, '?menuid=sluzebnicesty&action=editovatradkysluzebky&record=' . $id);
        }

        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);
        if (!$cesta->lzeZobrazitSluzebniCestu()) throw new NeniPravoException();

        $cesta->lzeEditovat = $cesta->lzeEditovatSluzebniCestu($this->database, false); // nezobrazovat chyby
        $datum = DateUtils::startOfTheDay($cesta->datumCasOd);
        if (isset($_POST['datum_uctovani_sluzebky'])) {
            $datum = (int)$_POST['datum_uctovani_sluzebky'];
        } else if (isset($_GET['datum'])) {
            $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
        }

        require_once('sluzebnicesty/f_radekuctovanisc.php');
        $uctovani = new RadekUctovaniSluzebniCestyFormular($this->database, $cesta, $datum);

        if ($uctovani->loadRadky()) {
            // nacteni OK (vcetne pripadneho POST)
            if (isset($_POST['novyRadekUctovani']) && $_POST['novyRadekUctovani']) {
                // uzivatel kliknul na "novy radek" => zobrazi formular

            } else if (isset($_POST['smazat_radek_uctovani_sluzebky'])  &&  (int)$_POST['smazat_radek_uctovani_sluzebky']) {
                // uzivatel smazal radek

            } else {
                if (isset($_POST['editace_radku_uctovani_sluzebky']) && $_POST['editace_radku_uctovani_sluzebky']) {
                    // uzivatel kliknul na "OK"
                    return new Redirect(null, '?menuid=sluzebnicesty&action=editovatradkysluzebky&record=' . $id);
                }
            }
        }

        return $uctovani;
    }


    /**
     * @param int $id
     * @return ViewObject
     * @throws \Exception
     */
    function hromadnaEditaceRadku($id)
    {
        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);

        if (!$cesta->lzeZobrazitSluzebniCestu()) throw new NeniPravoException();

        $cesta->lzeEditovat = $cesta->lzeEditovatSluzebniCestu($this->database, false); // nezobrazovat chyby

        require_once('sluzebnicesty/f_radeksluzebnicesty.php');

        $form = new RadekSluzebniCestyFormular($this->database, $cesta);
        $form->loadRadky();

        if (!MessageManager::getInstance()->isError()) {
            if (
                isset($_POST['OK']) && $_POST['OK'] &&
                isset($_POST['editace_radku_sluzebky']) && $_POST['editace_radku_sluzebky']
            ) {
                // stisknuto OK, vse v poradku => zobrazi sluzebni cestu
                return $this->redirectToView($id);
            }

            if (isset($_POST['zobrazitVydajeProDen'])  &&  $_POST['zobrazitVydajeProDen']) {
                return new Redirect(null, '?menuid=sluzebnicesty&action=editovatradkyuctovani&record=' . $id . '&datum=' . date('j.n.Y', (int)$_POST['zobrazitVydajeProDen']));
            }
        }

        return $form;
    }


    /**
     * @param int $id
     * @return bool
     * @throws \Exception
     */
    function vyuctovani($id)
    {
        if (!$id) throw new \Exception('Chyba - neni zadano ID sluzebni cesty!');

        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);

        try {
            if (!$cesta->kontrolaPredVyuctovanim($this->database)) {
                return $this->sluzebniCestaView($cesta);
            }

            require_once('adsapp/agentmanager.php');

            AgentManager::executeADS('ADS_Agent_BusinessTrips.exe', '0 -id:' . (int)$id . ' -alwaysLog', true);

            return $this->redirectToView($id);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return $this->sluzebniCestaView($cesta);
        }
    }


    /**
     * Schvaleni/zruseni schvaleni zamestnance
     * @param int $id
     * @param int $hodnota
     * @return Redirect|ViewObject
     */
    function schval_Z($id, $hodnota)
    {
        try {
            $rec = new SluzebniCestaRecord($this->database, $id, $this);
            $rec->schvalSlCestu_Z($hodnota);
            return $this->redirectToView($id);
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return $this->sluzebniCestaViewById($id);
        }
    }

    /**
     * Schvaleni/zamitnuti nadrizeneho
     * @param int $id
     * @param int $hodnota
     * @param int $level schvaleni nadrizeneho 1 nebo 2
     * @return Redirect|ViewObject
     */
    function schval_N($id, $hodnota, $level)
    {
        $rec = new SluzebniCestaRecord($this->database, $id, $this);
        try {
            if ($rec->schvalSlCestu_N($hodnota, $level)) {
                return $this->redirectToView($id);
            } else {
                return $this->sluzebniCestaViewById($id);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
            return $this->sluzebniCestaViewById($id);
        }
    }


    /**
     * Schvaleni s.c.
     * @param int $id
     * @param int $uroven
     * @return Redirect|ViewObject
     * @throws \Exception
     */
    function schvaleni($id, $uroven)
    {
        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $id);

        $result = $cesta->jsouVyplnenePovinne($this->database);

        if ($result) {
            $rec = new SluzebniCestaRecord($this->database, $id, $this);
            try {
                $result = $rec->schvalSlCestu($uroven);
            } catch (XmlFormException $e) {
                $result = false;
            } catch (\Exception $e) {
                $result = false;
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if ($result) {
            return $this->redirectToView($id);
        } else {
            return $this->sluzebniCestaView($cesta);
        }
    }


    /**
     * Schvaleni s.c.
     * @param int $id
     * @param int $uroven
     * @return Redirect|ViewObject
     * @throws \Exception
     */
    function zrusSchvaleni($id, $uroven)
    {
        $rec = new SluzebniCestaRecord($this->database, $id, $this);
        try {
            if ($rec->zrusSchvaleniSlCesty((int)$uroven)) {
                return $this->redirectToView($id);
            } else {
                return $this->sluzebniCestaViewById($id);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return $this->sluzebniCestaViewById($id);
        }
    }


    function vratSchvaleni($id)
    {
        if (!$id) throw new \Exception('Chyba - neni zadano ID sluzebni cesty!');

        if (!isset($_POST['poznamkakvraceni']) || !$_POST['poznamkakvraceni']) {
            return $this->sluzebniCestaViewById($id, 'vratslcestu');
        }

        $uroven = (int)$_POST['urovenvraceni'];
        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage('Chyba pri vraceni - neni zadana uroven schvaleni!');
            return $this->sluzebniCestaViewById($id, 'vratslcestu');
        }

        $rec = new SluzebniCestaRecord($this->database, $id, $this);
        if ($rec->vratSchvaleni($uroven, $_POST['poznamkakvraceni'])) {
            return $this->redirectToView($id);
        } else {
            return $this->sluzebniCestaViewById($id);
        }
    }

    /**
     * @param int $idSC
     * @param int $idDokumentu
     * @return ViewObject
     * @throws \Exception
     */
    function smazDokument($idSC, $idDokumentu)
    {
        $cesta = $this->nactiSCProEditaci($idSC);
        if (!$cesta) {
            throw new \Exception('Chyba pri mazani radku uctovani - nelze nacist pozadovanou sluzebni cestu!');
        }

        require_once('sluzebnicesty/sc_dokumenty.php');
        DokumentySCBrowser::zrusDokumentSluzebniCesty($this->database, $idSC, $idDokumentu, $this);

        return $this->redirectToView($cesta->id);
    }


    function downloadIcal($idSC)
    {
        $cal = SluzebniCestaCalItem::loadSluzebniCesty($this->database, $idSC);
        if (!$cal) throw new \Exception('Sluzebni cesta nebyla nalezena!');

        require_once 'sluzebnicesty/sc_utils.php';
        if (!SluzebniCestyUtils::jePravoNaSlCestuUzivatele($this->database, $cal->idPP, $cal->dateTimeFrom, $cal->dateTimeTo)) {
            throw new NeniPravoException();
        }

        return new IcalOutputObject($cal);
    }


    function report($idSC)
    {
        $sluzebniCesta = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);

        if (!$sluzebniCesta->lzeZobrazitSluzebniCestu()) {
            throw new NeniPravoException();
        }

        require_once('adsapp/reportserver.php');

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'SluzebniCesta', false, 'SluzebniCesta'); // treti parametr z duvodu kompatibility

        $result->nastavPP($sluzebniCesta->f_PP);

        if ($result ->report ->appReportName) {
            $result->integerParams['IdSluzebniCesty'] = $_GET['idslcesty'];
        } else {
            $result->stringParams['IdSluzebniCesty'] = $_GET['idslcesty'];  // stare sestavy pouzivaly stringovou promennou
        }

        return $result;
    }


    function odstranSpolucestujiciho($idSC, $idPP)
    {
        Prava::nastavObdobiProPrava(time(), time());

        $prava = Prava::vratPrava('slcesty_spolu');
        if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
            RonPortalUtils::neniPravoException();
        }

        $cesta = $this->nactiSCProEditaci($idSC);
        if (!$cesta) {
            throw new \Exception('Chyba pri odstranovani spolucestujiciho - nelze nacist pozadovanou sluzebni cestu!');
        }

        $pp = PracovniPomer::nactiPP($this->database, $idPP);

        require_once('adsapp/ronportaldbrecord.php');
        $rec = new RonPortalDBRecord($this->database, 'Spolucestujici');
        $rec->addField('SluzebniCesta_ID', 'int', true, 0, $idSC);
        $rec->addField('RC', '', true, 15, $pp->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pp->osobniCislo);
        $rec->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $rec->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeniKratke);
        $params = $this ->database ->createParameters();
        $rec->setWhere(
            'SluzebniCesta_ID = ' . $params ->addParameter($idSC) .
            ' AND RC = ' . $params->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this->database->timestampToParam($pp->datumNastupu)),
            $params);

        if ($rec->loadRecord()) {
            $rec ->deleteRecord();
        }

        return $this ->redirectToView($cesta->id);
    }


    function kopieProSpolucestujiciho($idSC, $idPP)
    {
        Prava::nastavObdobiProPrava(time(), time());

        // kontrola, jestli muze vytvaret kopie
        $prava = Prava::vratPrava('slcesty_spolu');
        if (!$prava ->priznak  ||  !$prava->lzeZobrazitPP($idPP, time(), time())) {
            RonPortalUtils::neniPravoException();
        }

        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
        if (!$cesta) {
            throw new \Exception('Chyba pri kopirovani zaznamu pro spolucestujiciho - nelze nacist pozadovanou sluzebni cestu!');
        }

        // kontrola, jestli existuje zadany PP
        //$pp = PracovniPomer::nactiPP($this->database, $idPP);

        // kontrola, jestli uz nema SC v danem terminu?

        require_once('adsapp/agentmanager.php');
        try {
            AgentManager::executeADS('ADS_Agent_BusinessTrips.exe', '1 -id:' . (int)$idSC . ' -idpp:' . $idPP . ' -alwaysLog', true);
            MessageManager::addInfoMessage(\Ron\ronText('Kopie_pro_spolucestujiciho_OK'));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $this->redirectToView($idSC);
    }



    /**
     * @param string $action
     * @return string|Redirect|ViewObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if ( !RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_SC) ) {
            throw new \Exception('Sluzebni cesty nejsou v RONPortalu povoleny!');
        }

        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');

        HtmlHeaders::getInstance() ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Prehled_sluzebnich_cest'));
        HtmlHeaders::getInstance() ->addCss('css/sc.css');
        $this ->addTranslationsScript('sluzebnicesty');
        HtmlHeaders::getInstance() ->addScript('script/sc.js');

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->sluzebniCestaFormular($formaction);
        }

        $action = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($action) {
            case 'zobrazslcestu' :
            case 'editovatradeksluzebky' :
            case 'kopirovatradeksluzebky' :
            case 'pridatradeksluzebky' :
            case 'pridatuctovanisluzebky' :
            case 'editovatuctovanisluzebky' :
            case 'kopirovatuctovanisluzebky' :
                return $this ->sluzebniCestaViewById((int)$_GET['record'], $action);

            case 'zrusitsluzebku' :
                return $this ->smazSluzebniCestu((int)$_GET['record']);

            // radky
            case 'ulozradeksluzebky' :
                return $this ->ulozRadek();
            case 'vlozradeksluzebky' :
                return $this ->vlozRadek();
            case 'zrusitradeksluzebky' :
                return $this ->smazRadek((int)$_GET['record'], (int)$_GET['radek']);

            case 'ulozuctovanisluzebky' :
                return $this ->ulozRadekUctovani();
            case 'vlozuctovanisluzebky' :
                return $this ->vlozRadekUctovani();
            case 'zrusituctovanisluzebky' :
                return $this ->smazRadekUctovani((int)$_GET['record'], (int)$_GET['radek']);

            // hromadna editace radku
            case 'editovatradkysluzebky' :
                return $this ->hromadnaEditaceRadku((int)$_GET['record']);

            case 'editovatradkyuctovani' :
                return $this ->hromadnaEditaceUctovani((int)$_GET['record']);

            // zauctovani
            case 'zauctovat' :
                return $this ->nastavZauctovaniSlCesty($_GET['record'], 1);

            case 'zrusitzauctovani' :
                return $this ->nastavZauctovaniSlCesty($_GET['record'], 0);

            case 'odstranspolucestujiciho' :
                return $this ->odstranSpolucestujiciho($_GET['record'], $_GET['idpp']);

            case 'kopieprospolucestujiciho' :
                return $this ->kopieProSpolucestujiciho($_GET['record'], $_GET['idpp']);

            // dokumenty
            case 'zrusitdokument' :
                return $this ->smazDokument((int)$_GET['idslcesty'], (int)$_GET['record']);

            case 'download' : // download dokumentu
                require_once('sluzebnicesty/sc_dokumenty.php');
                return DokumentySCBrowser::downloadDokumentSC($this ->database, (int)$_GET['idslcesty'], (int)$_GET['record']);

            // vyuctovani
            case 'vyuctovatsluzebky' :
                return $this ->vyuctovani((int)$_GET['record']);

            // schvalovani
            case 'schvalslcestuzam' :
                return $this ->schval_Z((int)$_GET['record'], 1);
            case 'zrusschvalenizam' :
                return $this ->schval_Z((int)$_GET['record'], 0);

            case 'schvalslcestunad1' :
                return $this ->schval_N((int)$_GET['record'], 1, 1);
            case 'zamitnislcestunad1' :
                return $this ->schval_N((int)$_GET['record'], 2, 1);
            case 'schvalslcestunad2' :
                return $this ->schval_N((int)$_GET['record'], 1, 2);
            case 'zamitnislcestunad2' :
                return $this ->schval_N((int)$_GET['record'], 2, 2);
            case 'schvalslcestunad3' :
                return $this ->schval_N((int)$_GET['record'], 1, 3);
            case 'zamitnislcestunad3' :
                return $this ->schval_N((int)$_GET['record'], 2, 3);
            case 'schvalslcestunad4' :
                return $this ->schval_N((int)$_GET['record'], 1, 4);
            case 'zamitnislcestunad4' :
                return $this ->schval_N((int)$_GET['record'], 2, 4);

            case 'schvalslcestu1' :
                return $this ->schvaleni((int)$_GET['record'], 1);
            case 'schvalslcestu2' :
                return $this ->schvaleni((int)$_GET['record'], 2);
            case 'schvalslcestu3' :
                return $this ->schvaleni((int)$_GET['record'], 3);
            case 'schvalslcestu4' :
                return $this ->schvaleni((int)$_GET['record'], 4);

            case 'zrusschvalenislcesty1' :
                return $this ->zrusSchvaleni((int)$_GET['record'], 1);
            case 'zrusschvalenislcesty2' :
                return $this ->zrusSchvaleni($_GET['record'], 2);
            case 'zrusschvalenislcesty3' :
                return $this ->zrusSchvaleni($_GET['record'], 3);
            case 'zrusschvalenislcesty4' :
                return $this ->zrusSchvaleni($_GET['record'], 4);

            case 'vratslcestu' :
                return $this ->vratSchvaleni($_GET['record']);

            // download
            case 'ical' :
                return $this ->downloadIcal($_GET['idslcesty']);

            case 'report' :
                return $this ->report($_GET['idslcesty']);

            case 'translations' :   // texty pro javascripty
                return $this ->translations();
        }

        if ( isset($_GET['record'])  &&  (int)$_GET['record'] ) {
            return $this ->sluzebniCestaViewById((int)$_GET['record']);
        }

        if (isset($_POST['hromadneSchvaleni'])  &&  $_POST['hromadneSchvaleni']) {
            SluzebniCestaRecord::hromadneSchvaleniSC($this ->database, $this);
        }

        return $this ->sluzebniCestyBrowser( isset($_GET['schvalovani'])  &&  $_GET['schvalovani'] );
    }

}