<?php
/**
 * UTF8
 * stanislav.elbl, 31.07.2015
 */

namespace Ron\RonPortal;

use Ron\Controls\ComboBox;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\StringUtils;

require_once('ron/controls.php');
require_once('adsapp/b_dokumenty.php');
require_once('adsapp/dokumentyutils.php');


class RadekUctovaniSCCombo extends ComboBox implements ICreateItem
{
    /** @var Database */
    var $database;

    /**
     * RadekUctovaniSCCombo constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('CisloRadku');
        $this->database = $database;
    }


    public function createItem($row)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[0]; // cislo radku
        $caption = DBUtils::getFormatedValue($this ->database, 'finedate', $row[1]) . ' ' . $row[2] . ' - ' . abs($row[3]) . ' ' . $row[4]; // datum  typ pohybu - castka mena
        if ($row[5]) $caption .= ' (' . $row[5]. ')'; // popis

        return new Option($key, $caption);
    }
}



class NovyDokumentSCForm extends DokumentUploadForm
{
    var $idSluzebniCesty = null;

    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     */
    function __construct($database, $idSluzebniCesty)
    {
        parent::__construct($database, '?menuid=sluzebnicestyjs&action=dokumentupload&idslcesty=' . (int)$idSluzebniCesty);

        $this ->idSluzebniCesty = (int)$idSluzebniCesty;
        $this ->kategorieDokumentu = 301; // dokument sluzebni cesty
        $this ->target = null;

        $this ->sqlPripojeniStarsiho =
            'SELECT Dokument.Id, Dokument.Popis, Dokument.Datum ' .
            '  FROM Dokument, DokumentSluzebniCesty, SluzebniCesta ' .
            ' WHERE Dokument.Id = DokumentSluzebniCesty.Id ' .
            '   AND DokumentSluzebniCesty.SluzebniCesta_Id = SluzebniCesta.Id ' .
            "   AND SluzebniCesta.RC = '" . $database ->addSlashes(RonPortalSession::osobaRC()) . "'" .
            '   AND NOT EXISTS (' .
            '        SELECT * FROM DokumentSluzebniCesty DS2' .
            '         WHERE DS2.Id = Dokument.Id ' .
            '           AND DS2.SluzebniCesta_Id = ' . $this ->idSluzebniCesty .
            '       )' .
            ' ORDER BY Dokument.Datum DESC ';
    }


    function dalsiRadkyKDokumentu()
    {
        echo '<div class="form-gap"></div>';

        // zobrazi vyber radku, ke kteremu se ma dokument pripojit
        echo '<div class="row" id="novydokument_radekscrow">';

        echo '<div class="col col-12 col-sm-4 caption">' . \Ron\ronText('Pripojit_dokument_k_radku_sc') . '</div>';
        echo '<div class="col col-12 col-sm-8">';
        $combo = new RadekUctovaniSCCombo($this ->database);

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT CisloRadku, Den, TypRUctovaniSlCesty.Popis, Castka, KodMeny, RadekUctovaniSCesty.Popis' .
            '  FROM RadekUctovaniSCesty, TypRUctovaniSlCesty' .
            ' WHERE TypRUctovaniSlCesty.Kod = RadekUctovaniSCesty.Typ' .
            '   AND RadekUctovaniSCesty.SluzebniCesta_ID = ' . $params ->addParameter((int)$this ->idSluzebniCesty) .
            '   AND Typ <> 1  AND  Typ <> 2  AND  Typ <> 3';

        $combo ->emptyRow = ' ';
        $combo ->dataTypeComparison = false;
        $combo ->htmlParams['id'] = 'Cislo_Radku_Uctovani_SC';
        $combo ->className = 'form-input documentUploadInput';
        $combo ->loadData($this ->database, $sql, null, $params);

        $combo ->render();
        echo '</div>';

        echo '</div>'; // row
    }

}



class DokumentySCBrowser extends DokumentyBrowser
{
    var $idSluzebniCesty;

    function __construct($database, $idSluzebniCesty) {
        parent::__construct($database, "?menuid=sluzebnicesty&action=download&idslcesty=$idSluzebniCesty", 1, "?menuid=sluzebnicesty&action=zrusitdokument&idslcesty=$idSluzebniCesty");

        $this ->idSluzebniCesty = $idSluzebniCesty;

        $this->sqlFromPart = 'Dokument, TypDokumentu, DokumentSluzebniCesty';
        $this->sqlWherePart =
            'Dokument.Id = DokumentSluzebniCesty.Id ' .
            ' AND TypDokumentu.Kod = Dokument.Typ ' .
            " AND DokumentSluzebniCesty.SluzebniCesta_ID = " . (int)$idSluzebniCesty;

        $this->showMessages = false;
    }


    /**
     * @param Database $database
     * @param int   $idSluzebniCesty
     * @param bool  $lzeEditovat
     */
    static function zobrazDokumentySlCesty($database, $idSluzebniCesty, $lzeEditovat)
    {
        echo "<A name='dokumenty'></A>\n";

        // Seznam dokumentu
        $b = new DokumentySCBrowser($database, $idSluzebniCesty);
        $b ->load();
        $b ->render();

        // Pokud lze editovat, zobrazi odkaz "vlozit novy"
        if ($lzeEditovat) {
            echo "<div class='padding-top-2'><a id='vlozit_dokument_sc' href='#' data-idSluzebniCesty='$idSluzebniCesty'>" . \Ron\ronText('Vlozit_novy_dokument') . "</a></div>\n";

            $form = new NovyDokumentSCForm($database, $idSluzebniCesty);
            $form ->render();
        }
    }

    /**
     * @param Database $database
     * @param int $idSC
     * @param int $idDokumentu
     * @param string|object $logDescription
     * @throws DBException
     */
    static function zrusDokumentSluzebniCesty($database, $idSC, $idDokumentu, $logDescription)
    {
        //require_once('adsapp/ronportallog.php');

        // smaze vazbu na dokument
        require_once('adsapp/ronportaldbrecord.php');

        $rec = new RonPortalDBRecord($database, 'DokumentSluzebniCesty');
        $rec ->addField('Id', 'int');
        $rec ->addField('SluzebniCesta_ID', 'int');

        $params = $database ->createParameters();
        $rec ->deleteRecord(
            'Id = ' . $params ->addParameter((int)$idDokumentu) .
            ' AND SluzebniCesta_ID = ' . $params ->addParameter((int)$idSC),
            $params);

        // zjisti jestli je pripojen jeste k dalsi sluzebni ceste (pomoci funkce "pripojit starsi dokument")
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult('SELECT COUNT(*) FROM DokumentSluzebniCesty WHERE Id = ' . $params ->addParameter((int)$idDokumentu), $params);
        $row = $cursor->fetch();
        $cursor ->close();
        if (!$row[0]) {
            // dokument neni pripojen k jine sluzebni ceste => smaze i dokument
            require_once('adsapp/dokumentyutils.php');
            try {
                DokumentyUtils::smazDokument($database, (int)$idDokumentu, $logDescription);
            } catch (\Exception $e) {
                // chybu ignoruje - zobrazi ji jen v debug rezimu - dokument totiz muze byt pripojen jeste v jine agende
                if (App::getDebugMode()) {
                    MessageManager::addErrorMessage($e);
                }
            }
        }
    }


    /**
     * @param Database $database
     * @param int $idSC
     * @param int $idDokumentu
     * @return DokumentOutputObject
     * @throws \Exception
     */
    static function downloadDokumentSC($database, $idSC, $idDokumentu)
    {
        $sql =
            'SELECT PracovniPomer.Id, SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo ' .
            '  FROM SluzebniCesta, PracovniPomer, DokumentSluzebniCesty ' .
            ' WHERE SluzebniCesta.Id = ' . (int)$idSC .
            '   AND DokumentSluzebniCesty.Id = ' . (int)$idDokumentu .               // toto je nutna kontrola, aby uzivatel
            '   AND DokumentSluzebniCesty.SluzebniCesta_Id = SluzebniCesta.Id ' .  // nemohl stahnout jakykoliv dokument!
            '   AND PracovniPomer.RC = SluzebniCesta.RC ' .
            '   AND PracovniPomer.OsobniCislo = SluzebniCesta.OsobniCislo ' .
            '   AND PracovniPomer.DatumNastupu = SluzebniCesta.DatumNastupu ';

        if (!$cursor = $database ->select($sql)) {
            throw new DBException($database, $sql);
        }

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Pozadovany dokument sluzebni cesty nebyl nalezen!');
        }

        require_once 'sluzebnicesty/sc_utils.php';
        if (!SluzebniCestyUtils::jePravoNaSlCestuUzivatele($database, $row[0], $database ->stringToTimestamp($row[1]), $database ->stringToTimestamp($row[2]))) {
            throw new NeniPravoException();
        }

        require_once('adsapp/dokumentyutils.php');
        return new DokumentOutputObject($database, $idDokumentu);
    }
}


