<?php
/**
 * UTF8
 * stanislav.elbl, 23.12.2016
 */

namespace Ron\RonPortal;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\Model;
use Ron\MVC\ViewObject;
use Ron\StringUtils;


class SluzebniCestaView implements ViewObject
{
    /** @var  Database */
    var $database;
    /** @var  SluzebniCesta */
    var $sluzebniCesta;
    /** @var  string */
    var $action;

    var $nazevStatu;

    /** @var TypRozdeleni */
    var $typRozdeleni;
    /** @var Rozdeleni */
    var $rozdeleni;

    /** @var TypRozdeleni */
    var $typRozdeleni2;
    /** @var Rozdeleni */
    var $rozdeleni2;

    /** @var string  vychozi stat osoby podle nastaveni nebo rozdeleni  */
    var $vychoziStatOsoby;


    function __construct($database, $sluzebniCesta, $action = 'zobrazslcestu')
    {
        $this ->database = $database;
        $this ->sluzebniCesta = $sluzebniCesta;
        $this ->action = $action;

        $this ->loadData();
    }


    function loadData()
    {
        $params = $this ->database ->createParameters();
        $sql = 'SELECT Popis FROM Stat WHERE Kod = ' . $params ->addParameter($this ->sluzebniCesta ->f_stat);
        $this ->nazevStatu = DBUtils::getSqlValue($this ->database, $sql, $params);

        require_once('adsapp/rozdeleni.php');

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI)  &&  $this ->sluzebniCesta ->rozdeleni) {
            $this ->typRozdeleni = new TypRozdeleni();
            $this ->typRozdeleni ->load($this ->database, (int)RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI));

            $this ->rozdeleni = new Rozdeleni();
            $this ->rozdeleni ->load($this ->database, (int)$this ->sluzebniCesta ->kodTypuRozdeleni, $this ->sluzebniCesta ->rozdeleni);
        }

        $kodTypu2 = $this ->sluzebniCesta ->zahranicni
            ? RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z)
            : RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2);

        if ($kodTypu2  &&  $this ->sluzebniCesta ->rozdeleni2) {
            $this ->typRozdeleni2 = new TypRozdeleni();
            $this ->typRozdeleni2 ->load($this ->database, $kodTypu2);

            $this ->rozdeleni2 = new Rozdeleni();
            $this ->rozdeleni2 ->load($this ->database, (int)$this ->sluzebniCesta ->kodTypuRozdeleni2, $this ->sluzebniCesta ->rozdeleni2);
        }

        require_once 'sluzebnicesty/sc_utils.php';
        $this ->vychoziStatOsoby = SluzebniCestyUtils::getVychoziStatOsoby($this ->database, $this ->sluzebniCesta ->f_PP, $this ->sluzebniCesta ->datumCasOd);
    }


    /**
     * @param bool $vraceni  Urcuje, jestli ma byt viditelny formular pro zadani poznamky k vraceni
     */
    function zobrazHlavicku($vraceni = false)
    {
        Model::addData('vraceni', $vraceni);

        require('sluzebnicesty/view_hlavicka.php');
    }


    function zobrazPovoleniNadrizeneho($uroven, $hodnota, $povolil, $datumPovoleni)
    {
        $pravaNadrizeny = Prava::vratPrava('slcesty_nadriz' . (($uroven > 1) ? $uroven : ''));
        $jeNadrizeny = $this ->sluzebniCesta ->f_PP ->lzeZobrazitPP($pravaNadrizeny, $this ->sluzebniCesta ->datumCasOd, $this ->sluzebniCesta ->datumCasDo);
        $class = 'schvaleni_sc_nadrizeny' . (($jeNadrizeny || $hodnota) ? '' : ' neschvaleno');

        echo "<div class='row $class' id='hlavickasc_schvaleninadrizeny{$uroven}'>";

        $suffix = $uroven > 1 ? '_' . $uroven : '';

        echo '<div class="col col-3 caption">' . \Ron\ronText('Nadrizeny' . $suffix) . '</div>';
        echo '<div class="col col-9">';
        if ($hodnota) {
            echo '<div class="sc_povoleno_nadrizenym">';
            // schvaleno/zamitnuto
            echo \Ron\ronText(($hodnota == 1) ? 'Povoleno_nadrizenym' . $suffix : 'Zamitnuto_nadrizenym' . $suffix);
            // datum a uzivatel
            echo ' ' . sprintf(\Ron\ronText('Datum_povoleni_nadrizenym'), date('j.n.Y', $datumPovoleni), StringUtils::htmlTextConvert($povolil));
            echo '</div>';
        }

        if ($jeNadrizeny) echo '<div class="sc_povoleni_nadrizeneho">';
        if ($jeNadrizeny  &&  $hodnota != 1) {    // &&  !$sluzebniCesta ->jeSchvalena() - od 5.5.2017 lze dodatecne povolit i kdyz je schvalena
            // muze nastavit povoleni => link povolit
            echo "<a href='?menuid=sluzebnicesty&action=schvalslcestunad{$uroven}&record={$this ->sluzebniCesta->id}'>" . \Ron\ronText('Povolit_nadrizenym') . '</a>';
        }

        if ($jeNadrizeny  &&  $hodnota != 2 &&  !$this ->sluzebniCesta ->jeSchvalena()) {
            // muze nastavit povoleni => link zamitnout
            echo "<a href='?menuid=sluzebnicesty&action=zamitnislcestunad{$uroven}&record={$this ->sluzebniCesta->id}'>" . \Ron\ronText('Zamitnout_nadrizenym') . '</a>';
        }
        if ($jeNadrizeny) echo '</div>';

        echo '</div>';

        echo "</div>\n"; // row
    }


    /**
     * @param int    $datum
     * @param string $schvalil
     * @param int    $uroven
     * @param bool   $lzeVratit
     * @param int    $datumVraceni
     */
    function zobrazSchvaleni($datum, $schvalil, $uroven, $lzeVratit, $datumVraceni) // , $jeVyssiSchvaleni
    {
        $jeVyssiSchvaleni = $this ->sluzebniCesta ->jeSchvalena($uroven + 1);
        $posloupnostZapnuta = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_POSLOUPNOST_SCHVALOVANI);

        $prava = Prava::vratPrava("slcesty_schval$uroven");

        global $STR_UrovenSchvaleniSC; // jen pro kompat.
        $nadpis = isset($STR_UrovenSchvaleniSC) ? $STR_UrovenSchvaleniSC[$uroven - 1] : \Ron\ronText('Schvaleni_sc_' . $uroven);

        echo "<div class='row schvaleni_sc_$uroven'><div class='col col-3 caption'><span class='textschvaleni$uroven'>{$nadpis}</span></div>";
        echo "<div class='col col-9'>";

        if ($datum) {
            echo '<span class="textschvaleni' . $uroven . '">' .
                date('j.n.Y', $datum) . ' ' . StringUtils::htmlTextConvert($schvalil) . '</span>';

            if (!$prava ->priznak) {
                // nelze zrusit datum schvaleni pokud je jiz obdobi (podle $datum) uzavreno
                // (napr. pokud se proplaceji cestovni nahrady v obdobi, kdy byla SC schvalena)
                if (!$jeVyssiSchvaleni && // pokud je vyssi uroven schvaleni, tak nesmi zrusit tuto
                    $this->sluzebniCesta ->f_PP->lzeZobrazitPP($prava, $this->sluzebniCesta ->datumCasDo, $this->sluzebniCesta ->datumCasDo) &&
                    (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_EDITACE_V_UZAVRENEM)  ||  !Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $datum, $datum))
                ) {
                    global $STR_ZrusitSchvaleniSC; // pouze pro kompatibilitu se starymi preklady
                    $strOdschvalit = (isset($STR_ZrusitSchvaleniSC)  &&  is_array($STR_ZrusitSchvaleniSC)  &&  isset($STR_ZrusitSchvaleniSC[$uroven - 1]))
                        ? $STR_ZrusitSchvaleniSC[$uroven - 1]
                        : \Ron\ronText('Zrusit_schvaleni_sc_' . $uroven);

                    echo "&nbsp;&nbsp;&nbsp;<A href='?menuid=sluzebnicesty&action=zrusschvalenislcesty$uroven&record={$this->sluzebniCesta ->id}'>$strOdschvalit</A>";
                }
            }
        } else {
            if ( (!$posloupnostZapnuta || $lzeVratit  ||  $uroven == 1)  &&  $this ->sluzebniCesta ->f_PP ->lzeZobrazitPP($prava, $this ->sluzebniCesta ->datumCasOd, $this ->sluzebniCesta ->datumCasDo)) {
                global $STR_SchvalitSC; // pouze pro kompatibilitu se starymi preklady
                $strSchvalit = (isset($STR_SchvalitSC)  &&  is_array($STR_SchvalitSC)  &&  isset($STR_SchvalitSC[$uroven - 1]))
                    ? $STR_SchvalitSC[$uroven - 1]
                    : \Ron\ronText('Schvalit_sc_' . $uroven);

                echo "<A href='?menuid=sluzebnicesty&action=schvalslcestu$uroven&record={$this ->sluzebniCesta ->id}'>$strSchvalit</A>";

                if (RonPortalSettings::getIntValue(SluzebniCestyController::SETTINGS_POUZIVAT_VRACENI) && $lzeVratit) {
                    echo '<span style="display:inline-block; width:50px;"></span>'; // mezera

                    echo "<a href='?menuid=sluzebnicesty&action=vratslcestu&record={$this ->sluzebniCesta ->id}' class='vraceni_sluzebni_cesty_link'>" . \Ron\ronText('Vratit_sc') . "</a>";
                }
            }

            if ($datumVraceni  &&  ($uroven == 1  || $this ->sluzebniCesta ->jeSchvalena($uroven - 1))) {
                if ($this ->sluzebniCesta ->poznamkaKVraceni) {
                    echo '<div class="poznamkaKVraceni">' . StringUtils::htmlTextConvert($this ->sluzebniCesta ->poznamkaKVraceni) . ' (' . StringUtils::format('datetime', $datumVraceni) . ')</div>';
                }
            }
        }

        echo "</div></div>\n";
    }


    function zobrazRadky()
    {
        echo "<div id='RadkySluzebniCestyView'>\n";
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_KOMPAKTNI_RADKY)) {
            require_once('sluzebnicesty/f_radeksckompaktni.php');

            $radky = KompaktniRadekSluzebniCesty::nactiKompaktniRadkySC($this ->database, $this ->sluzebniCesta);

            if ( $radky && sizeof($radky) ) {
                $sluzebniCesta = $this ->sluzebniCesta;
                require('sluzebnicesty/view_radkykompaktni.php');
            }
        } else {
            require_once('sluzebnicesty/b_radeksluzebnicesty.php');

            RadekSluzebniCestyBrowser::zobrazRadkySluzebniCesty($this ->database, $this ->sluzebniCesta, $this ->sluzebniCesta ->lzeEditovat, $this ->action);
        }
        echo "</div>\n";
    }


    function zobrazExterniOdkazy()
    {
        $text = RonPortalSettings::getMultilineValue('slcesty_dalsihtml');
        if (!$text) return;

        $text = str_ireplace('%id%', $this ->sluzebniCesta ->id, $text);
        $text = str_ireplace('%datumod%', date('dmY', $this ->sluzebniCesta ->datumCasOd), $text);
        $text = str_ireplace('%datumdo%', date('dmY', $this ->sluzebniCesta ->datumCasDo), $text);
        $text = str_ireplace('%idpp%', $this ->sluzebniCesta ->f_PP ->idPP, $text);
        $text = str_ireplace('%osobnicislo%', $this ->sluzebniCesta ->f_PP ->osobniCislo, $text);

        echo '<div id="sc_externi_odkazy">' .$text . '</div>';
    }


    public function render()
    {
        //$this ->sluzebniCesta ->zobrazSluzebniCestu($this ->database, $this ->action);

        $this ->sluzebniCesta ->lzeEditovat = $this ->sluzebniCesta->lzeEditovatSluzebniCestu($this ->database, false); // nezobrazovat chyby
        $this ->sluzebniCesta ->lzeEditovatHlavicku = $this ->sluzebniCesta ->lzeEditovatHlavicku($this ->database);

        $this ->zobrazHlavicku($this ->action == 'vratslcestu');

        // Sestavy
        require_once('adsapp/reportutils.php');

        //$reportList = RonPortalReport::loadFormReportsByName($this ->database, 'SluzebniCesta');
        $reportList = RonPortalReport::loadFormReports($this ->database, 'SluzebniCesta', 'SluzebniCesta'); // pro kompatibilitu zobrazuje i reporty s kodem SluzebniCesta...
        FormReportsListView::renderReportList($reportList, '?menuid=sluzebnicesty&action=report&idslcesty=' . $this ->sluzebniCesta ->id);

        $this ->zobrazRadky();

        echo "<div id='UctovaniSluzebniCestyView'>\n";
        require_once('sluzebnicesty/b_uctovanislcesty.php');
        RadekUctovaniSlCestyBrowser::zobrazUctovani($this ->database, $this ->sluzebniCesta, $this ->action);
        echo "</div>\n";

        // Dokumenty
        echo "<div id='DokumentySluzebniCestyView'>\n";
        require_once('sluzebnicesty/sc_dokumenty.php');
        echo "\n<br/>\n";
        DokumentySCBrowser::zobrazDokumentySlCesty($this ->database, $this ->sluzebniCesta ->id, $this ->sluzebniCesta ->lzeEditovat);
        echo "</div>\n";

        // Externi odkazy
        $this ->zobrazExterniOdkazy();

        echo "<br/><div class='normaltext'><a href='?menuid=sluzebnicesty'>" . \Ron\ronText('Zpet_na_seznam') . "</a></div>\n";

    }
}




