<?php
/**
 * stanislav.elbl, 13.05.2020
 */

namespace Ron\RonPortal\Vyroba;

use Ron\BrowserField;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\Database;
use Ron\RonPortal\Prava;
use Ron\RonPortal\RonPortalBrowser;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;

require_once 'adsapp/ronportalbrowser.php';


class VlozeniLinkField extends BrowserField
{
    var $prava;

    /**
     * VlozeniLinkField constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this ->type = BROWSER_FIELD_TYPE_TEXT;

        $this ->prava = Prava::vratPrava('vkladanivyroby');

        $this ->text = '<span class="js vlozit-vyrobu">' . \Ron\ronText("Vlozit_vyrobu") . '</span>';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this ->prava ->lzeZobrazitPP($rowData[0])) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}


class MonitorovaniVyrobyBrowserField extends BrowserField
{
    /**
     * @var MonitorovaniVyrobyBrowser
     */
    var $monitorovaniBrowser;

    /**
     * MonitorovaniVyrobyBrowserField constructor.
     * @param null|Database $database
     * @param MonitorovaniVyrobyBrowser $monitorovaniBrowser
     */
    public function __construct($database, $monitorovaniBrowser)
    {
        parent::__construct($database);
        $this->setType(BROWSER_FIELD_TYPE_DEFAULT);

        $this->monitorovaniBrowser = $monitorovaniBrowser;
    }


    function getLink($text, $recordId)
    {
        if (!$this ->link) {
            return $text;
        }
        switch ($this ->monitorovaniBrowser ->proklikEditace) {
            case 2 :
                $targetStr = ' target="monvyrobyeditace"';
                break;

            case 3 :
                $targetStr = ' target="_blank"';
                break;

            case 1 :
                $targetStr = '';
                break;

            case 0 :
            default :
                return $text;
        }

        return
            '<a href="' . $this ->link . $recordId . '"' .
            //(($recordId != '') ? $recordId : '') .
            $targetStr .
            '>' .
            $text .
            '</a>';
    }
}


class MonitorovaniVyrobyBrowser extends RonPortalBrowser
{
    var $zobrazitVkladani = false;

    /** @var int    Urcuje, kolik dnu zpetne se nacte */
    var $pocetDnu = 1;

    var $proklikEditace = 1;

    /**
     * MonitorovaniVyrobyBrowser constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'monitorovaniv');

        $this ->title = \Ron\ronText('Monitorovani_vyroby');
        $this ->code = 'browsermonitorovaniv';

        $this ->sqlSelectPart =
            'PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.DatumUkonceni,' . // 0..4
            'PracovniPomer.CisloPracovniSmlouvy, PracovniPomer.TypPracovnihoPomeru, PracovniPomer.Zamestnanec,' . // 5..7
            'Osoba.jmeno, Osoba.prijmeni, Osoba.titul, Osoba.IdFotografie, Osoba.Telefon, Osoba.eMail,' . // 8..13
            'PuvodniVyroba.DatumCas, PuvodniVyroba.KodVyrobniCinnosti, PuvodniVyroba.Vyrobek,' . // 14..16
            'PuvodniVyroba.Terminal, PuvodniVyroba.Karta, PuvodniVyroba.VyrobniZakazka, PuvodniVyroba.KodTypuRozdeleni, PuvodniVyroba.Rozdeleni,' . // 17..21
            'VyrobniCinnost.Popis, VyrobniCinnost.DruhZpracovani'; // 22,23
            // uzivatelska pole netreba - jdou doplnit v XML formulari config_browsermonitorovaniv, napriklad takto:
            // <column sqlFieldName="Osoba.Uzivatelska1" header="Mobil"/>

        $this ->primaryField = 0;

        // DruhZpracovani: 0=zacatek, 1=konec, 2=konec s odvedenim vyroby, 3=odvedeni vyroby, 4=ignorovat, 5=preruseni vyroby, 6=pokracovani prerusene vyroby

        $this ->distinct = true;

        $this ->cssClass .= ' monitorovani-vyroby';

        $this ->sqlFromPart = 'NASTAVENO AZ PO NACTENI KONFIGURACE!';

        $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('PracovniPomer.OsobniCislo', 2)
            ->setCode('col-osobnicislo')
            ->setHeaderText('Osobni_cislo');

        $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('Osoba.Prijmeni', 9)
            ->setCode('col-prijmeni')
            ->setHeaderText('Prijmeni')
            ->setLink('?menuid=vyrobaosoby&osoba=');

        $f = $this ->addField(new MonitorovaniVyrobyBrowserField($this ->database, $this))
            ->setSqlFieldName('Osoba.Jmeno', 8)
            ->setCode('col-jmeno')
            ->setHeaderText('Jmeno')
            ->setLink('?menuid=vyrobaosoby&osoba=');

        $this ->addField()
            ->setSqlFieldName('PuvodniVyroba.DatumCas', 14) ->setFormat('datetime')
            ->setCode('col-cas')
            ->setClassName('col-datum-cas')
            ->setHeaderText('Cas');

        $this ->addField()
            ->setSqlFieldName('VyrobniCinnost.Popis', 22)
            ->setCode('col-popiscinnosti')
            ->setHeaderText('Vyroba_cinnost');

        $this ->addField( new VlozeniLinkField($this ->database) );
    }


    function setFilterForm()
    {
        parent::setFilterForm();

        $this ->filterForm ->f_pravarozdeleni = Prava::vratPrava('vyroba');
        $this ->filterForm ->nastavOsoby(false, null); // nezobrazovat vyber osoby
        $this ->filterForm ->nastavObdobi(false, 'ADS', false);
        $this ->filterForm ->setUseGet(true, 'monitorovaniv');
    }


    function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'pocetDnu' :
                $this ->pocetDnu = (int)$valueText;
                break;

            case 'proklikeditace' :
                $this ->proklikEditace = (int)$valueText;
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function getSQL($sqlFilter = '')
    {
        $this ->sqlFromPart =
            'PracovniPomer INNER JOIN Osoba ON Osoba.RC = PracovniPomer.RC' .
            ' LEFT OUTER JOIN PuvodniVyroba' .
            ' ON PuvodniVyroba.RC = PracovniPomer.RC' .
            ' AND PuvodniVyroba.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND PuvodniVyroba.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND PuvodniVyroba.DatumCas > ' . $this ->database ->dateToString(DateUtils::incDayStart(time(), -$this ->pocetDnu)) . // nacita data od zacatku predchoziho dne
            ' LEFT OUTER JOIN VyrobniCinnost' .
            ' ON VyrobniCinnost.Kod = PuvodniVyroba.KodVyrobniCinnosti';

        return parent::getSQL($sqlFilter);
    }


    function getOrderSQLPart()
    {
        //return parent::getOrderSQLPart();
        return 'Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.Id, PuvodniVyroba.DatumCas';
    }

    function getSQLFilter()
    {
        $prava = Prava::vratPrava('vyroba');
        $result = $prava ->sqlFiltr('PracovniPomer', time(), time());

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && $this ->filterForm ->kodRozdeleni !== null) {
            $filtr = $this ->filterForm ->vratSQLFiltrRozdeleni('PracovniPomer', $this ->filterForm ->vcetnePodrizenychRozdeleni);

            if ($filtr) {
                if ($result) $result .= ' AND ';
                $result .= ' (' . $filtr . ') ';
            }
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) && $this ->filterForm ->kodRozdeleni2) {
            require_once('adsapp/rozdeleni.php');
            $filtr = Rozdeleni::sqlFilterRozdeleni($this ->database, 'PracovniPomer',
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this ->filterForm ->kodRozdeleni2,
                time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni2);

            if ($filtr) {
                if ($result) $result .= ' AND ';
                $result .= ' (' . $filtr . ') ';
            }
        }

        return $result;
    }


    function loadSQL($sql) // , $firstRow  * @param int  $firstRow          Prvni radek, ktery se ma zobrazit
    {
        parent::loadSQL($sql);

        $data = [];

        $lastRow = null;

        $pravaVkladani = Prava::vratPrava('vkladanivyroby');
        $this ->zobrazitVkladani = false; // pokud najde osobu, pro kterou muze vkladat, tak nastavi na true

        while ($row = $this ->cursor ->fetch()) {
            if ($this ->enableLine($row)) {
                if (!$this ->zobrazitVkladani  &&  $pravaVkladani ->lzeZobrazitPP($row[0])) {
                    $this ->zobrazitVkladani = true;
                }

                // $row[23] DruhZpracovani: 0=zacatek, 1=konec, 2=konec s odvedenim vyroby, 3=odvedeni vyroby, 4=ignorovat, 5=preruseni vyroby, 6=pokracovani prerusene vyroby
                if ($lastRow && $lastRow[0] !== $row[0]) {
                    $data[] = $lastRow;
                    $lastRow = null;
                }

                if (!$lastRow || $row[23] != 4) {  // v pripade, ze uz ma pro danou osobu nacteny nejaky radek, tak ignoruje radek s druhem zpracovani "ignorovat"
                    $lastRow = $row;
                }
            }
        }

        if ($lastRow) {
            $data[] = $lastRow;
        }

        $this ->cursor = new ArrayCursor($data);
        $this ->cursor ->setDatabase($this ->database);
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .= isset($values[23]) ? ' druh-zpracovani-' . $values[23] : ' druh-zpracovani-NULL';

        $time = $this ->database ->stringToTimestamp($values[14]);
        if ($time && $time <= time() - 12 * 60 * 60) {
            // zaznam starsi nez 12 hodin
            $className .= ' monitorovani-12';
        }

        return
            parent::getRowAttributes($values, $className, $rowNumber) .
            " data-id-pp='{$values[0]}'" .
            " data-jmeno='" . StringUtils::htmlTextConvert($values[8] . ' ' . $values[9]). "'" .
            " data-osobni-cislo='" . StringUtils::htmlTextConvert($values[2]) . "'";
    }

}
