<?php

/**
 * UTF8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 16.5.2018
 * Time: 8:41
 */

namespace Ron\RonPortal\Vyroba;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\RonPortal\AgentManager;
use Ron\RonPortal\DenniVysledek;
use Ron\RonPortal\MesicniVysledek;
use Ron\RonPortal\PracovniPomer;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalFilterForm;
use Ron\RonPortal\RonPortalSession;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\RonPortalUtils;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;
use function Ron\ronText;

require_once('adsapp/ronportalfilter.php');
require_once('dochazka/dennivysledek.php');
require_once('dochazka/mesicnivysledek.php');
require_once 'vyroba/zaznamvyroby.php';
require_once 'vyroba/vysledekvyroby.php';


class ZaznamEditaceVyroby extends ZaznamVyroby
{
    public $jmenoOsoby;

    public $vyrobekError = false;
    public $cinnostError = false;
    public $zakazkaError = false;
    public $casOdError = false;
    public $casDoError = false;
    public $jeChyba = false;

    /** @var DenniVysledek[] */
    public $denniVysledky;
    /** @var MesicniVysledek[] */
    public $mesicniVysledky;

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param string|string[] $typCSDenni
     * @param string|string[] $typCSMesicni
     * @param bool $nactiVysledkyVyroby
     * @return ZaznamEditaceVyroby[]
     * @throws Exception
     */
    public static function nactiOsobyProVyrobu($database, $den, $prava, $typCSDenni = null, $typCSMesicni = null, $nactiVysledkyVyroby = false)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Vyroba.serial, PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, Osoba.Prijmeni, Osoba.Jmeno, Vyroba.den, Vyroba.vyrobek, Vyroba.cinnost, Vyroba.hodiny, ". // 0 .. 10
            " Vyroba.mnozstviDobre, Vyroba.mnozstviSpatne, Vyroba.Premie, Vyroba.cisloObjednavkyPrijate, Vyroba.casOd, Vyroba.casDo, Vyroba.DatumCasPuvodniVyroby, " . // 11 .. 17
            " Vyroba.popis, Vyroba.rozsirujiciPopis, Vyroba.VyrobniZakazka, Vyroba.KodTypuRozdeleni, Vyroba.Rozdeleni, Vyroba.DatumSchvaleni, Vyroba.Zpracoval, Vyroba.kodFirmy, " . // 18 .. 25
            " Vyroba.Hmotnost, Vyroba.JednotekVCyklu, Vyroba.PocetCyklu, Vyroba.Insert_UserName, Vyroba.Insert_Timestamp, Vyroba.Update_UserName, Vyroba.Update_Timestamp \n" . // 26 .. 32
            "FROM PracovniPomer " .
            " INNER JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) \n".
            " LEFT OUTER JOIN Vyroba ON (PracovniPomer.RC = Vyroba.RC AND PracovniPomer.OsobniCislo = Vyroba.OsobniCislo AND PracovniPomer.DatumNastupu = Vyroba.DatumNastupu " .
            "   AND den = " . $params->addParameter($database->dateToParam($den)) . ") \n" .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $den);

        $sql .=
            " ORDER BY Osoba.Prijmeni, Osoba.Jmeno, Vyroba.serial";

        MessageManager::addDebugLog('Seznam osob: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $zaznamy = [];

        if (!empty($typCSDenni))
            $denniVysledky = static::nactiDenniVysledky($database, $den, $prava, $typCSDenni);
        if (!empty($typCSMesicni))
            $mesicniVysledky = static::nactiMesicniVysledky($database, DateUtils::firstDayOfMonth($den), $prava, $typCSMesicni);

        if ($nactiVysledkyVyroby)
            $vysledkyVyroby = VysledekVyroby::nactiVysledkyVyrobyProDenniEditaci($database, $den, $prava);

        while ($row = $cursor->fetch()) {
            $vyroba = new ZaznamEditaceVyroby();
            $vyroba->serialVyroba = (int)$row[0];
            $vyroba->idPP = $row[1];
            $vyroba->rc = $row[2];
            $vyroba->osobniCislo = $row[3];
            $vyroba->datumNastupu = $database->stringToTimestamp($row[4]);
            $vyroba->jmenoOsoby = $row[5] . ' ' . $row[6];
            $vyroba->den = $database->stringToTimestamp($row[7]);
            $vyroba->vyrobek = $row[8];
            $vyroba->cinnost = $row[9];
            $vyroba->hodiny = $row[10];
            $vyroba->mnozstviDobre = $row[11];
            $vyroba->mnozstviSpatne = $row[12];
            $vyroba->casOd = $database->stringToTimestamp($row[15]);
            $vyroba->casDo = $database->stringToTimestamp($row[16]);
            $vyroba->zakazka = $row[20];

            if (!empty($denniVysledky))
                $vyroba->denniVysledky = $denniVysledky[$row[1]];
            if (!empty($mesicniVysledky))
                $vyroba->mesicniVysledky = $mesicniVysledky[$row[1]];
            if (!empty($vysledkyVyroby))
                $vyroba->vysledkyVyroby = $vysledkyVyroby[$row[0]];

            $zaznamy[] = $vyroba;
        }

        return $zaznamy;
    }

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param int $prepocetVyroby 0 = neprepocitavat, 1 = prepocitat vzdy, 2 = prepocitat jen zmenene
     * @param int $datumCasZpracovaniVyroby
     * @param int $prepocetDochazky 0 = neprepocitavat, 1 = prepocitat vzdy, 2 = prepocitat jen zmenene
     * @param bool $kontrolaDochazky
     * @param array $nastaveni
     * @return ZaznamEditaceVyroby[]|null
     * @throws Exception
     */
    public static function zpracujPost($database, $den, $prava, $prepocetVyroby, $datumCasZpracovaniVyroby, $prepocetDochazky, $kontrolaDochazky, $nastaveni)
    {
        if ($den > $datumCasZpracovaniVyroby)
            throw new Exception(ronText('Vyroba_nesmi_prekrocit_cas_zpracovani'));

        $zaznamyVyroby = static::nactiOsobyProVyrobu($database, $den, $prava);

        require_once('adsapp/ronportaldbrecord.php');

        $vychoziCinnost = $_POST['vyroba_vychozi_cinnost'] ?: null;

        /** @var ZaznamEditaceVyroby[]|[] $chybneRadky */
        $chybneRadky = [];

        /** @var ZaznamEditaceVyroby[]|[] $prepocitatZaznamy */
        $prepocitatZaznamy = [];
        /** @var ZaznamEditaceVyroby[]|[] $prepocitatZaznamyDochazky */
        $prepocitatZaznamyDochazky = [];

        $pocetCyklu = 1;
        if ($_POST["pocet_chybnych_zaznamu"]) {
            $pocetCyklu = 1 + $_POST["pocet_chybnych_zaznamu"];
        }

        foreach ($zaznamyVyroby as $zaznamVyroby) {
            $idZaznamu = $zaznamVyroby->serialVyroba ?: $zaznamVyroby->idPP;
            $maxIndexZaznamu = $_POST["{$idZaznamu}_max_index"];

            for ($n = 1; $n <= $pocetCyklu; $n++) {
                //pro puvodne zadavany zaznam u ktereho doslo pri poslednim zapisu k chybe
                if ($n > 1) {
                    $zaznamVyroby->serialVyroba = null;

                    $indexChyby = $n - 1;
                    $idZaznamu = "vyroba_chyba_{$indexChyby}";
                    $maxIndexZaznamu = $_POST["{$idZaznamu}_max_index"];

                    $idpp = $_POST["vyroba_idpp_{$idZaznamu}"];
                    require_once 'adsapp/ppomer.php';
                    $pracPomer = new PracovniPomer();
                    $pracPomer->nactiPPPodleId($database, $idpp);

                    $zaznamVyroby->rc = $pracPomer->rc;
                    $zaznamVyroby->osobniCislo = $pracPomer->osobniCislo;
                    $zaznamVyroby->datumNastupu = $pracPomer->datumNastupu;

                    if ($n == $pocetCyklu)
                        $pocetCyklu = 1;
                }

                for ($i = 1; $i <= $maxIndexZaznamu; $i++) {
                    $idRadku = "{$idZaznamu}_{$i}";

                    $vyrobek = $_POST["vyroba_vyrobek_{$idRadku}"];
                    $cinnost = $_POST["vyroba_cinnost_{$idRadku}"];
                    $zakazka = $_POST["vyroba_zakazka_{$idRadku}"];

                    $pocetKs = null;
                    if (!empty($_POST["vyroba_pocetks_{$idRadku}"]))
                        $pocetKs = StringUtils::stringToFloat($_POST["vyroba_pocetks_{$idRadku}"]);

                    $hodiny = null;
                    if (!empty($_POST["vyroba_hodiny_{$idRadku}"]))
                        $hodiny = DateUtils::timeStringToFloat($_POST["vyroba_hodiny_{$idRadku}"], true);

                    $datumOd = $zaznamVyroby->den ?: $den;
                    //standardne se nastavi na den zaznamu vyroby, pripadne zvoleny den ve formulari, pokud vsak je nastavene i zadavani datumu, nacte se z POSTu
                    if (!empty($_POST["vyroba_datumod_{$idRadku}"]))
                        $datumOd = DateUtils::dateStringToTimestamp($_POST["vyroba_datumod_{$idRadku}"]);
                    $casOd = null;
                    if (!empty($_POST["vyroba_casod_{$idRadku}"]))
                        $casOd = DateUtils::timeStringToTimestamp($_POST["vyroba_casod_{$idRadku}"], $datumOd);

                    $datumDo = $zaznamVyroby->den ?: $den;
                    //standardne se nastavi na den zaznamu vyroby, pripadne zvoleny den ve formulari, pokud vsak je nastavene i zadavani datumu, nacte se z POSTu
                    if (!empty($_POST["vyroba_datumdo_{$idRadku}"]))
                        $datumDo = DateUtils::dateStringToTimestamp($_POST["vyroba_datumdo_{$idRadku}"]);
                    $casDo = null;
                    if (!empty($_POST["vyroba_casdo_{$idRadku}"]))
                        $casDo = DateUtils::timeStringToTimestamp($_POST["vyroba_casdo_{$idRadku}"], $datumDo);

                    if (is_null($hodiny) && !is_null($casOd) && !is_null($casDo))
                        $hodiny = ($casDo - $casOd) / 3600;

                    if (($vyrobek != '') || ($cinnost != '') || ($zakazka != '') || ($_POST["vyroba_pocetks_{$idRadku}"] != '')) {
                        $vlozit = true;

                        if (!SQLUtils::existsMasterRecord($database, $vyrobek, 'Zbozi', 'Kod')) {
                            MessageManager::addErrorText('Vyroba_vyrobek_neexistuje');
                            $zaznamVyroby->vyrobekError = true;
                            $vlozit = false;
                        }
                        if (!($vychoziCinnost || SQLUtils::existsMasterRecord($database, $cinnost, 'VyrobniCinnost', 'kod'))) {
                            MessageManager::addErrorText('Vyroba_cinnost_neexistuje');
                            $zaznamVyroby->cinnostError = true;
                            $vlozit = false;
                        }
                        if (!SQLUtils::existsMasterRecord($database, $zakazka, 'VyrobniZakazka', 'cisloZakazky')) {
                            MessageManager::addErrorText('Vyroba_zakazka_neexistuje');
                            $zaznamVyroby->zakazkaError = true;
                            $vlozit = false;
                        }

                        if ((!is_null($casOd) && ($casOd > $datumCasZpracovaniVyroby)) || (!is_null($casDo) && ($casDo > $datumCasZpracovaniVyroby))) {
                            MessageManager::addErrorText('Vyroba_nesmi_prekrocit_cas_zpracovani');

                            if (!is_null($casOd) && ($casOd > $datumCasZpracovaniVyroby))
                                $zaznamVyroby->casOdError = true;
                            if (!is_null($casDo) && ($casDo > $datumCasZpracovaniVyroby))
                                $zaznamVyroby->casDoError = true;

                            $vlozit = false;
                        }

                        if ($vlozit) {
                            $vyrobaRecord = new RonPortalDBRecord($database, 'Vyroba', 'Editace vyroby');
                            $vyrobaRecord->addField('serial', 'int', true, 0, (($zaznamVyroby->serialVyroba > 0) && ($i == 1) ? $zaznamVyroby->serialVyroba : SQLUtils::newSerialNumber($database, 'Vyroba', 'serial')));
                            $vyrobaRecord->addField('RC', 'string', true, 15, $zaznamVyroby->rc);
                            $vyrobaRecord->addField('OsobniCislo', 'string', true, 15, $zaznamVyroby->osobniCislo);
                            $vyrobaRecord->addField('DatumNastupu', 'timestamp', true, 0, $zaznamVyroby->datumNastupu);
                            $vyrobaRecord->addField('den', 'timestamp', true, 0, $zaznamVyroby->den ?: $den);

                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'VYROBEK')) {
                                $vyrobaRecord->addField('vyrobek', 'string', false, 30, $vyrobek);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CINNOST')) {
                                $vyrobaRecord->addField('cinnost', 'string', false, 30, $vychoziCinnost ?: $cinnost);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'ZAKAZKA')) {
                                $vyrobaRecord->addField('VyrobniZakazka', 'string', false, 30, $zakazka);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'MNOZSTVI')) {
                                $vyrobaRecord->addField('mnozstviDobre', 'float', false, 0, $pocetKs);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'HODINY')) {
                                $vyrobaRecord->addField('hodiny', 'float', false, 0, $hodiny);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CASOD')) {
                                $vyrobaRecord->addField('casOd', 'timestamp', false, 0, $casOd);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CASDO')) {
                                $vyrobaRecord->addField('casDo', 'timestamp', false, 0, $casDo);
                            }

                            $vyrobaRecord->addField('Zpracoval', '', false, 25, RonPortalUtils::getLogUserName())
                                ->updateLog = true;

                            $vyrobaRecord ->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
                            $vyrobaRecord ->addField('Insert_Timestamp', 'timestamp', true, 0, time());
                            $vyrobaRecord ->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
                                ->updateLog = true;
                            $vyrobaRecord ->addField('Update_Timestamp', 'timestamp', false, 0, time())
                                ->updateLog = true;

                            $prepocitat = false;
                            $prepocitatDochazku = false;
                            if (($zaznamVyroby->serialVyroba > 0) && ($i == 1)) {
                                $params = $database->createParameters();
                                $vyrobaRecord->setWhere('serial = ' . $params->addParameter($zaznamVyroby->serialVyroba), $params);
                                $vyrobaRecord->loadAndUpdateRecord();
                                if (($prepocetVyroby == 1) || ($prepocetVyroby == 2 && $vyrobaRecord->isChanged())) {
                                    $prepocitat = true;
                                }
                                if (($prepocetDochazky == 1) || ($prepocetDochazky == 2 && $vyrobaRecord->isChanged())) {
                                    $prepocitatDochazku = true;
                                }
                            } else {
                                $vyrobaRecord->insertRecord();
                                if ($prepocetVyroby > 0) {
                                    $prepocitat = true;
                                    $zaznamVyroby->den = $den;
                                }
                                if ($prepocetDochazky > 0) {
                                    $prepocitatDochazku = true;
                                    $zaznamVyroby->den = $den;
                                }
                            }
                            if ($prepocitat) {
                                if (!array_key_exists($zaznamVyroby->idPP, $prepocitatZaznamy)) {
                                    $prepocitatZaznamy[$zaznamVyroby->idPP] = $zaznamVyroby;
                                    $prepocitatZaznamy[$zaznamVyroby->idPP]->casOd = $vyrobaRecord['casOd'];
                                    $prepocitatZaznamy[$zaznamVyroby->idPP]->casDo = $vyrobaRecord['casDo'];
                                }
                            }
                            if ($prepocitatDochazku) {
                                if (!array_key_exists($zaznamVyroby->idPP, $prepocitatZaznamyDochazky)) {
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP] = $zaznamVyroby;
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP]->casOd = $vyrobaRecord['casOd'];
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP]->casDo = $vyrobaRecord['casDo'];
                                }
                            }
                        } else {
                            $chybnyZaznam = clone $zaznamVyroby;

                            $chybnyZaznam->serialVyroba = null;
                            $chybnyZaznam->vyrobek = $vyrobek;
                            $chybnyZaznam->cinnost = $vychoziCinnost ?: $cinnost;
                            $chybnyZaznam->zakazka = $zakazka;
                            $chybnyZaznam->mnozstviDobre = $pocetKs;
                            $chybnyZaznam->hodiny = $hodiny;
                            $chybnyZaznam->casOd = $casOd;
                            $chybnyZaznam->casDo = $casDo;
                            $chybnyZaznam->jeChyba = true;
                            $chybneRadky[] = $chybnyZaznam;
                        }
                    }
                }
            }
        }

        if ($prepocetVyroby > 0 && !empty($prepocitatZaznamy)) {
            $idPPList = [];
            $minDatumOd = 0;
            $maxDatumDo = 0;
            foreach ($prepocitatZaznamy as $zaznamVyroby) {
                $idPPList[] = $zaznamVyroby->idPP;

                // pokud se needituje cas od-do, pouzije se den
                if (!$minDatumOd) $minDatumOd = $zaznamVyroby->casOd ?: $zaznamVyroby ->den;
                if (!$maxDatumDo) $maxDatumDo = $zaznamVyroby->casDo ?: $zaznamVyroby ->den;

                $minDatumOd = min($minDatumOd, $zaznamVyroby->casOd ?: $zaznamVyroby ->den);
                $maxDatumDo = max($maxDatumDo, $zaznamVyroby->casDo ?: $zaznamVyroby ->den);
            }

            VyrobaController::prepocetVysledkuHromadne($idPPList, DateUtils::startOfTheDay($minDatumOd), DateUtils::endOfTheDay($maxDatumDo));
        }

        if ($prepocetDochazky > 0 && !empty($prepocitatZaznamyDochazky)) {
            require_once('adsapp/agentmanager.php');

            $idPPStr = '';
            $minDatumOd = 0;
            $maxDatumDo = 0;
            foreach ($prepocitatZaznamyDochazky as $zaznamVyroby) {
                if ($idPPStr != '') $idPPStr .= ';';
                $idPPStr .= $zaznamVyroby->idPP;

                if ($minDatumOd == 0)
                    $minDatumOd = $zaznamVyroby->casOd;
                if ($maxDatumDo == 0)
                    $maxDatumDo = $zaznamVyroby->casDo;

                $minDatumOd = min($minDatumOd, $zaznamVyroby->casOd);
                $maxDatumDo = max($maxDatumDo, $zaznamVyroby->casDo);
            }
            try {
                AgentManager::prepocetDochazky($idPPStr, DateUtils::startOfTheDay($minDatumOd), DateUtils::endOfTheDay($maxDatumDo), false, $kontrolaDochazky);
            } catch (Exception $exception) {
                MessageManager::addErrorMessage($exception);
            }
        }

        if (empty($chybneRadky)) return null;

        return $chybneRadky;
    }

    /**
     * @param Database $database
     * @param Parameters $params
     * @param PravaNaOsoby $prava
     * @param int $datumOd
     * @param int|null $datumDo
     * @return string
     */
    public static function sqlFiltrOsoby($database, $params, $prava, $datumOd, $datumDo = null)
    {
        if (is_null($datumDo)) {
            $datumDo = $datumOd;
        }

        $sql = $prava->sqlFiltr('PracovniPomer', $datumOd, DateUtils::lastDayOfMonth($datumDo), $params) ;

        require_once('adsapp/rozdeleni.php');
        if (RonPortalSession::rozdeleni(VyrobaFilterForm::getTypRozdeleni())) {
            $sql .= ' AND ' .
                Rozdeleni::sqlFilterRozdeleni($database, 'PracovniPomer', VyrobaFilterForm::getTypRozdeleni(), RonPortalSession::rozdeleni(VyrobaFilterForm::getTypRozdeleni()),
                    $datumOd, $datumDo, RonPortalSession::rozdeleniVcetnePodrizenych(VyrobaFilterForm::getTypRozdeleni()), $params);
        }

        if (VyrobaFilterForm::getTypRozdeleni2() && RonPortalSession::rozdeleni2(VyrobaFilterForm::getTypRozdeleni2())) {
            $sql .= ' AND ' .
                Rozdeleni::sqlFilterRozdeleni($database, 'PracovniPomer', VyrobaFilterForm::getTypRozdeleni2(), RonPortalSession::rozdeleni2(VyrobaFilterForm::getTypRozdeleni2()),
                    $datumOd, $datumDo, RonPortalSession::rozdeleni2VcetnePodrizenych(VyrobaFilterForm::getTypRozdeleni2()), $params);
        }

        if (!RonPortalSession::vsechnyOsoby()) {
            require_once('adsapp/ppomer.php');
            $pracPomer = PracovniPomer::nactiPP($database, RonPortalSession::vybranaOsoba());
            $sql .=
                " AND PracovniPomer.RC = " . $params->addParameter($pracPomer->rc) .
                " AND PracovniPomer.OsobniCislo = " . $params->addParameter($pracPomer->osobniCislo) .
                " AND PracovniPomer.DatumNastupu = " . $params->addParameter($database->dateToParam($pracPomer->datumNastupu));
        }

        return $sql;
    }

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param string|string[] $typCS
     * @return array
     * @throws Exception
     */
    private static function nactiDenniVysledky($database, $den, $prava, $typCS)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, ". // 0 .. 4
            " DenniVysledekVypoctu.Hodiny, DenniVysledekVypoctu.Dny, DenniVysledekVypoctu.KalendarniDny, DenniVysledekVypoctu.Poznamka " . // 5 .. 8
            "FROM PracovniPomer " .
            " LEFT OUTER JOIN DenniVysledekVypoctu ON (PracovniPomer.RC = DenniVysledekVypoctu.RC AND PracovniPomer.OsobniCislo = DenniVysledekVypoctu.OsobniCislo AND PracovniPomer.DatumNastupu = DenniVysledekVypoctu.DatumNastupu " .
            "   AND den = " . $params->addParameter($database->dateToParam($den)) . ") " .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $den);

        $sql .= " AND " . SQLUtils::getINCondition($database, 'TypCasoveSlozkyMzdy', $typCS, $params);

        $sql .=
            " ORDER BY PracovniPomer.ID, TypCasoveSlozkyMzdy";

        MessageManager::addDebugLog('Denni vysledky: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $vysledkyPP = [];
        $vysledky = [];
        $idPP = 0;

        while ($row = $cursor->fetch()) {
            if ($idPP  &&  $idPP != $row[0]) {
                $vysledkyPP[$idPP] = $vysledky;
                $vysledky = [];
            }

            $vysledek = new DenniVysledek();
            $vysledek->den = $den;
            $vysledek->typCasoveSlozkyMzdy = $row[4];
            $vysledek->hodiny = (float)$row[5];
            $vysledek->dny = (float)$row[6];
            $vysledek->kalendarniDny = (float)$row[7];
            $vysledek->poznamka = $row[8];

            $vysledky[$row[4]] = $vysledek;

            $idPP = $row[0];
        }

        if (!empty($vysledky))
            $vysledkyPP[$idPP] = $vysledky;

        return $vysledkyPP;
    }

    /**
     * @param Database $database
     * @param int $mesic
     * @param PravaNaOsoby $prava
     * @param string|string[] $typCS
     * @return array
     * @throws Exception
     */
    private static function nactiMesicniVysledky($database, $mesic, $prava, $typCS)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, MesicniVysledekVypoctu.TypCasoveSlozkyMzdy, ". // 0 .. 4
            " MesicniVysledekVypoctu.Hodiny, MesicniVysledekVypoctu.Dny, MesicniVysledekVypoctu.KalendarniDny, MesicniVysledekVypoctu.Poznamka " . // 5 .. 8
            "FROM PracovniPomer " .
            " LEFT OUTER JOIN MesicniVysledekVypoctu ON (PracovniPomer.RC = MesicniVysledekVypoctu.RC AND PracovniPomer.OsobniCislo = MesicniVysledekVypoctu.OsobniCislo AND PracovniPomer.DatumNastupu = MesicniVysledekVypoctu.DatumNastupu " .
            "   AND Mesic = " . $params->addParameter($database->dateToParam($mesic)) . ") " .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $mesic, DateUtils::lastDayOfMonth($mesic));

        $sql .= " AND " . SQLUtils::getINCondition($database, 'TypCasoveSlozkyMzdy', $typCS, $params);

        $sql .=
            " ORDER BY PracovniPomer.ID, TypCasoveSlozkyMzdy";

        MessageManager::addDebugLog('Mesicni vysledky: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $vysledkyPP = [];
        $vysledky = [];
        $idPP = 0;

        while ($row = $cursor->fetch()) {
            if ($idPP != $row[0]) {
                $vysledkyPP[$row[0]] = $vysledky;
                $vysledky = [];
            }

            $vysledek = new MesicniVysledek();
            $vysledek->mesic = $mesic;
            $vysledek->typCasoveSlozkyMzdy = $row[4];
            $vysledek->hodiny = (float)$row[5];
            $vysledek->dny = (float)$row[6];
            $vysledek->kalendarniDny = (float)$row[7];
            $vysledek->poznamka = $row[8];

            $vysledky[$row[4]] = $vysledek;

            $idPP = $row[0];
        }

        if (!empty($vysledky))
            $vysledkyPP[$idPP] = $vysledky;

        return $vysledkyPP;
    }


}

class VyrobaFilterForm extends RonPortalFilterForm
{
    private $datumZpracovaniVyroby;

    private static $typRozdeleni;
    private static $typRozdeleni2;

    /**
     * @param Database $database
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param bool $den
     * @param int $datumZpracovaniVyroby
     * @param int $typRozdeleni
     * @param int $typRozdeleni2
     */
    public function __construct(Database $database, $prava, $den, $datumZpracovaniVyroby, $typRozdeleni, $typRozdeleni2)
    {
        parent::__construct($database, 'vyroba', true);
        $this->hiddenFields['action'] = 'denni';

        if ($typRozdeleni !== RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP))
            $this->nastavRozdeleni(true, $typRozdeleni, 0, '', true);
        else
            $this->nastavRozdeleni(true);

        if (!empty($typRozdeleni2))
            $this->nastavRozdeleni2(true, $typRozdeleni2, 0, '', true);

        static::$typRozdeleni = $this->f_typrozdeleni;
        static::$typRozdeleni2 = $this->f_typrozdeleni2;

        $this->nastavOsoby(true, $prava, $den, $den);
        $this->povolitVsechyOsoby = true;

        $this->datumZpracovaniVyroby = $datumZpracovaniVyroby;
    }

    public function renderFilterDatum()
    {
        echo "<div class='filterrow'>";

        echo "<span class='filtercaption'>";
        echo ronText('Vyroba_den');
        echo "</span>";

        echo '<span style="white-space:nowrap;">';

        $submitstr = $this->f_autosubmit ? "data-roncalendar-submit='1'" : '';
        $valueStr = date('j.n.Y', static::getDen());
        $maxDatum = date('j.n.Y', $this->datumZpracovaniVyroby);

        echo "<input type='text' class='inputDate' size='10' maxlength='10' name='den' value='$valueStr' $submitstr data-roncalendar-maxdate='$maxDatum'>";
        echo "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='den'></i>";


        echo "</span>";
        echo "</div>";
    }

    function renderFilters()
    {
        $this->renderFilterDatum();
        parent::renderFilters();
    }

    /**
     * @return int
     */
    public static function getDen()
    {
        try {
            $den = isset($_POST['den']) ? DateUtils::dateStringToTimestamp($_POST['den']) : (isset($_GET['den']) ? DateUtils::dateStringToTimestamp($_GET['den']) : DateUtils::today());
        } catch (Exception $exception) {
            $den = DateUtils::today();
        }
        return $den;
    }

    public static function getTypRozdeleni()
    {
        return static::$typRozdeleni;
    }

    public static function getTypRozdeleni2()
    {
        return static::$typRozdeleni2;
    }
}