<?php
/**
 * UTF8
 * stanislav.elbl, 06.09.2018
 */

namespace Ron\RonPortal\Vyroba;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\RonPortal\PracovniPomer;

class ZaznamVyroby
{
    public $serialVyroba;

    public $idPP;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    public $den;

    /** @var string     Kod vyrobku (zbozi) */
    public $vyrobek;
    /** @var string     Kod cinnosti */
    public $cinnost;
    /** @var string     Cislo zakazky */
    public $zakazka;
    /** @var int        Id cinnosti vyrobni zakazky */
    public $cinnostVyrobniZakazky;

    public $hodiny;
    public $mnozstviDobre;
    public $mnozstviSpatne;
    public $casOd;
    public $casDo;

    public $popis;
    public $rozsirujiciPopis;

    /** @var VysledekVyroby[] */
    public $vysledkyVyroby;

    // Udaje z nadrazenych tabulek
    public $popisZakazky;
    public $platnostZakazkyOd;
    public $platnostZakazkyDo;
    // ... z CinnostVyrobniZakazky
    public $popisCinnosti;
    public $datumCasCinnostiOd;
    public $datumCasCinnostiDo;


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return ZaznamVyroby[]
     * @throws \Exception
     */
    private static function loadSql($database, $sql, $params)
    {
        $cursor = $database->getSqlResult($sql, $params);

        $zaznamy = [];

        while ($row = $cursor->fetch()) {
            $vyroba = new static();

            $vyroba->serialVyroba = $row[0];

            $vyroba->idPP = $row[1];
            $vyroba->rc = $row[2];
            $vyroba->osobniCislo = $row[3];
            $vyroba->datumNastupu = $database->stringToTimestamp($row[4]);

            $vyroba->den = $database->stringToTimestamp($row[5]);
            $vyroba->vyrobek = $row[6];
            $vyroba->cinnost = $row[7];
            $vyroba->zakazka = $row[18];
            $vyroba->cinnostVyrobniZakazky = $row[31];

            $vyroba->hodiny = $row[8] ? (float)$row[8] : 0;
            $vyroba->mnozstviDobre = $row[9];
            $vyroba->mnozstviSpatne = $row[10];
            $vyroba->casOd = $database->stringToTimeOfDay($row[13]);
            $vyroba->casDo = $database->stringToTimeOfDay($row[14]);

            $vyroba->popis = $row[32];
            $vyroba->rozsirujiciPopis = $row[33];

            // informace ze zakazky
            $vyroba ->popisZakazky = $row[34];
            $vyroba ->platnostZakazkyOd = $database->stringToTimestamp($row[35]);
            $vyroba ->platnostZakazkyDo = $database->stringToTimestamp($row[36]);

            $zaznamy[] = $vyroba;
        }

        $cursor ->close();

        return $zaznamy;
    }

    /**
     * Nacte zadany zaznam vyroby
     * @param Database $database
     * @param int $serial
     * @return ZaznamVyroby
     * @throws \Exception
     */
    public static function nactiZaznamVyroby($database, $serial)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Vyroba.serial, PracovniPomer.ID, Vyroba.RC, Vyroba.OsobniCislo, Vyroba.DatumNastupu, Vyroba.den, Vyroba.vyrobek, Vyroba.cinnost, Vyroba.hodiny, ". // 0 .. 8
            " Vyroba.mnozstviDobre, Vyroba.mnozstviSpatne, Vyroba.Premie, Vyroba.cisloObjednavkyPrijate, Vyroba.casOd, Vyroba.casDo, Vyroba.DatumCasPuvodniVyroby, " . // 9 .. 15
            " Vyroba.popis, Vyroba.rozsirujiciPopis, Vyroba.VyrobniZakazka, Vyroba.KodTypuRozdeleni, Vyroba.Rozdeleni, Vyroba.DatumSchvaleni, Vyroba.Zpracoval, Vyroba.kodFirmy, " . // 16 .. 23
            " Vyroba.Hmotnost, Vyroba.JednotekVCyklu, Vyroba.PocetCyklu, Vyroba.Insert_UserName, Vyroba.Insert_Timestamp, Vyroba.Update_UserName, Vyroba.Update_Timestamp, " . // 24 .. 30
            ' Vyroba.CinnostVyrobniZakazky, Vyroba.Popis, {fn_CASTWIDEMEMO(Vyroba.RozsirujiciPopis)},' . // 31..33
            ' VyrobniZakazka.Popis, VyrobniZakazka.PlatnostOd, VyrobniZakazka.PlatnostDo' . // 34..36
            ' FROM Vyroba JOIN PracovniPomer ON (PracovniPomer.RC = Vyroba.RC AND PracovniPomer.OsobniCislo = Vyroba.OsobniCislo AND PracovniPomer.DatumNastupu = Vyroba.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)'.
            ' LEFT OUTER JOIN VyrobniZakazka ON VyrobniZakazka.cisloZakazky = Vyroba.VyrobniZakazka' .
            ' WHERE Vyroba.Serial = ' . $params ->addParameter($serial);

        $zaznamy = self::loadSql($database, $sql, $params);

        if (!$zaznamy) {
            throw new \Exception('Pozadovany zaznam vyroby nebyl nalezen!');
        }

        return $zaznamy[0];
    }


    /**
     * Nacte zaznamy vyroby pro zadany pracovni pomer a/nebo obdobi
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $orderBy
     * @return array|ZaznamVyroby[]
     * @throws \Exception
     */
    public static function nactiVyrobu($database, $pp = null, $datumOd = 0, $datumDo = 0, $orderBy = null)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Vyroba.serial, PracovniPomer.ID, Vyroba.RC, Vyroba.OsobniCislo, Vyroba.DatumNastupu, Vyroba.den, Vyroba.vyrobek, Vyroba.cinnost, Vyroba.hodiny, ". // 0 .. 8
            " Vyroba.mnozstviDobre, Vyroba.mnozstviSpatne, Vyroba.Premie, Vyroba.cisloObjednavkyPrijate, Vyroba.casOd, Vyroba.casDo, Vyroba.DatumCasPuvodniVyroby, " . // 9 .. 15
            " Vyroba.popis, Vyroba.rozsirujiciPopis, Vyroba.VyrobniZakazka, Vyroba.KodTypuRozdeleni, Vyroba.Rozdeleni, Vyroba.DatumSchvaleni, Vyroba.Zpracoval, Vyroba.kodFirmy, " . // 16 .. 23
            " Vyroba.Hmotnost, Vyroba.JednotekVCyklu, Vyroba.PocetCyklu, Vyroba.Insert_UserName, Vyroba.Insert_Timestamp, Vyroba.Update_UserName, Vyroba.Update_Timestamp, " . // 24 .. 30
            ' Vyroba.CinnostVyrobniZakazky, Vyroba.Popis, {fn_CASTWIDEMEMO(Vyroba.RozsirujiciPopis)},' . // 31..33
            ' VyrobniZakazka.Popis, VyrobniZakazka.PlatnostOd, VyrobniZakazka.PlatnostDo' . // 34..36
            ' FROM Vyroba JOIN PracovniPomer ON (PracovniPomer.RC = Vyroba.RC AND PracovniPomer.OsobniCislo = Vyroba.OsobniCislo AND PracovniPomer.DatumNastupu = Vyroba.DatumNastupu)' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)'.
            ' LEFT OUTER JOIN VyrobniZakazka ON VyrobniZakazka.cisloZakazky = Vyroba.VyrobniZakazka' .
            ' WHERE 1 = 1';

        if ($datumOd) {
            $sql .= ' AND den >= ' . $params->addParameter($database->dateToParam($datumOd));
        }
        if ($datumDo ||  $datumOd) {
            $sql .= ' AND den <= ' . $params->addParameter($database->dateToParam($datumDo ?: $datumOd));
        }

        if ($pp) {
            $sql .= ' AND PracovniPomer.Id = ' . $params ->addParameter($pp ->idPP);
        }

        $sql .= ' ORDER BY ' . ($orderBy ?: 'PracovniPomer.ID, Vyroba.den, Vyroba.serial');

        return self::loadSql($database, $sql, $params);
    }


    /**
     * Provede update zaznamu v databazi podle aktualnich hodnot
     * @param Database $database
     * @param string $logDescription
     * @param bool $editovatHodiny
     * @param bool $editovatCasy
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public function update($database, $logDescription, $editovatHodiny = true, $editovatCasy = false)
    {
        require_once 'vyroba/vyrobarec.php';

        $rec = new VyrobaDbRecord($database, $this->serialVyroba, $logDescription);

        if ($rec->loadRecord()) {
            $rec->fCinnost ->setNewValue($this->cinnost);
            $rec->fCinnostVyrobniZakazky ->setNewValue($this->cinnostVyrobniZakazky);
            $rec->fVyrobniZakazka ->setNewValue($this->zakazka);

            if ($editovatHodiny) {
                $rec->fHodiny->setNewValue($this->hodiny);
            }

            if ($editovatCasy) {
                $rec->fCasOd->setNewValue($this->casOd);
                $rec->fCasDo->setNewValue($this->casDo);
            }

            $rec->fPopis->setNewValue($this->popis);
            $rec->fRozsirujiciPopis->setNewValue($this->rozsirujiciPopis);

            $rec->updateRecord();
        }
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $logDescription
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public function insert($database, $pp, $den, $logDescription)
    {
        if (!isset($this ->cinnost)  ||  $this ->cinnost === '') {
            // v zaznamu neni vyplnena cinnost (povinny udaj) - doplni ji podle cinnosti vyrobni zakazky
            if (!isset($this->cinnostVyrobniZakazky)) {
                throw new \Exception( \Ron\ronText('Neni_zadana_cinnost'));
            }

            require_once 'vyroba/cinnostdennivyroby.php';
            $this ->cinnost = CinnostDenniVyroby::vratKodCinnostiVyrobniZakazky($database, $this->cinnostVyrobniZakazky);
        }

        require_once 'vyroba/vyrobarec.php';
        $rec = new VyrobaDbRecord($database, $this->serialVyroba, $logDescription);

        $rec->fCinnost->setNewValue($this->cinnost);
        $rec->fCinnostVyrobniZakazky->setNewValue($this->cinnostVyrobniZakazky);
        $rec->fVyrobniZakazka->setNewValue($this->zakazka);
        $rec->fHodiny->setNewValue($this->hodiny);
        $rec->fPopis->setNewValue($this->popis);
        $rec->fRozsirujiciPopis->setNewValue($this->rozsirujiciPopis);

        $rec->setNewRecord($pp, $den);

        $rec->insertRecord();

        $this->serialVyroba = $rec ->fSerial ->value;
    }


    /**
     * Smaze zaznam z databaze
     * @param Database $database
     * @param string $logDescription
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public function delete($database, $logDescription)
    {
        require_once 'vyroba/vyrobarec.php';

        $rec = new VyrobaDbRecord($database, $this->serialVyroba, $logDescription);

        if ($rec->loadRecord()) {
            $rec ->deleteRecord();
        }
    }

}
