<?php
/**
 * UTF8
 * stanislav.elbl, 21.06.2017
 */

namespace Ron\RonPortal\XML;

use Ron\RonPortal\DokumentyUtils;
use Ron\StringUtils;

require_once 'xml/xml_commands.php';


/**
 * Spolecny predek pro tagy <dokument> a <pripojenydokument>
 * Class XMLFormElementDokumentData
 * @package Ron\RonPortal\XML
 */
class XMLFormElementDokumentData extends XMLFormElement
{
//    function getDownloadHtml($id, $popis, $jmenoSouboru)
//    {
//
//    }


    /**
     * Vraci HTML pro jiz pripojene soubory
     * @param string $idStr Seznam id dokumentu oddelenych strednikem
     * @param string $name
     * @return string
     * @throws \Ron\DB\DBException
     */
    function getFilesHtml($idStr, $name)
    {
        if (!$idStr) return '';

        $result = '';
        $ids = explode(';', $idStr);
        foreach ($ids as $id) {
            $id = (int)$id;
            $params = $this ->xmlForm ->database ->createParameters();
            $sql = 'SELECT Popis, JmenoSouboru FROM Dokument WHERE Id = ' . $params ->addParameter($id);

            $cursor = $this ->xmlForm ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if ($row) {
                if ($name) $this ->setVariableValue($name, $id);
                $result .=
                    "<div class='pripojenydokument'>" .
                    StringUtils::htmlTextConvert($row[0]) .
                    " <a target='_blank' href='?menuid=xmlform&amp;action=download&amp;idxml={$this ->xmlForm ->id}&amp;record=$id'>{$row[1]}" .
                    "</a></div>";
            }
        }

        return $result;
    }
}


class XMLFormElementDokument extends XMLFormElementDokumentData
{
    var $name;
    /** @var string Jestli se muze pridavat vice dokumentu no (default) /yes/append/update (prepise se existujici) */
    var $editing;
    /** @var bool   Jestli se maji zobrazit */
    var $showFiles;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->name = $this->getAttributeValue('name');
        $this->editing = $this ->getAttributeValue('editing'); // no (default) /yes/append
    }


    function getHTMLOutput()
    {
        $result = '';

        if ($this ->xmlForm ->dataMode()) {
            $result = $this ->getFilesHtml($this ->variableValue($this ->name . '_dokumenty'), $this ->name);

            if ($this ->variableValue($this ->name)  &&  $this ->editing != 'yes'  &&  $this ->editing != '1'  &&  $this ->editing != 'append'  &&  $this ->editing != 'update') {
                // uz bylo odeslano a pripojeni dalsiho neni povoleno
                return $result;
            }
        }

        $attrs = [
            'type' => 'file',
            'name' => $this ->name,
            'class' => ($this ->xmlForm ->isError($this ->name)) ? 'fileinput dotaznik_errinput' : 'fileinput'
        ];

        if ($this ->getAttributeValue('style')) $attrs['style'] = $this ->getAttributeValue('style');

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        return $result . "<input $attrs_str/>";
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception('XMLFormElementDokument::writeXMLOutput neni implementovano!');
    }


    function process()
    {
        require_once('adsapp/dokumentyutils.php');

        $popis = StringUtils::fromUTF8($this ->getAttributeValue('popis'));

        if (is_uploaded_file($_FILES[$this ->name]['tmp_name'])) {
            // uzivatel pripojil soubor
            $popisSouboru = $this->nahradPromenne($popis);
            $typ = (int)$this ->nahradPromenne($this ->getAttributeValue('typ'));

            try {
                // TODO: implementovat "update"
                $id = DokumentyUtils::vlozDokument($this ->xmlForm ->database, $this ->name, $typ, $popisSouboru, 'XML formular');

                $this ->xmlForm->variables[$this ->name] = $id;
                $this ->xmlForm->noveDokumenty[] = $id;

                $pripojeneDokumenty = $this ->variableValue($this ->name . '_dokumenty') ?: '';
                $pripojeneDokumenty .= ($pripojeneDokumenty ? ';' : '') . $id;
                $this ->xmlForm->variables[$this ->name . '_dokumenty'] = $pripojeneDokumenty;

                $this ->xmlForm ->addDataVariable($this ->name);
                $this ->xmlForm ->addDataVariable($this ->name . '_dokumenty');

                $this ->xmlForm ->addLogData($this ->name, StringUtils::fromUTF8('Nahrán soubor ') . $_FILES[$this ->name]['name'] . ', id dokumentu ' . $id . ', ' . $popisSouboru);
            } catch (\Exception $e) {
                $this ->xmlForm->addErrorMsg('Pri vkladani souboru doslo k chybe!', $this ->name);
            }
        }

        parent::process();
    }
}


class XMLFormElementPripojenyDokument extends XMLFormElementDokumentData
{
    function getHTMLOutput()
    {
        return $this ->getFilesHtml($this ->getAttributeValue('id'), $name = $this ->getAttributeValue('name'));
    }
}


class XMLFormElementDeleteDokument extends XMLFormElementCommand
{
    function jePripojenyDokument($id)
    {
        if (!$this ->xmlForm ->id) return false; // formular jeste neni vytvoren (napr. v pripade, ze je to jen sablona)

        $params = $this ->xmlForm ->database ->createParameters();
        $sql =
            'SELECT IdDokumentu FROM DokumentXMLFormulare' .
            ' WHERE IdDokumentu = ' . $params ->addParameter($id) .
            ' AND IdFormulare = ' . $params ->addParameter($this ->xmlForm ->id);

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    function doCommand()
    {
        // id dokumentu
        $id = $this ->nahradPromenne($this ->getAttributeValue('id'));
        // jmeno promenne
        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));

        $seznamDokumentu = [];

        if ($id  &&  $name) {
            // Je zadano id i name - zkontroluje, ze je dokument v zadane promenne. Pokud ne, tak je to chyba - ignoruje ji a neudela nic
            $pripojeneDokumenty = $this ->variableValue($name . '_dokumenty') ?: '';
            $seznamDokumentu = explode(';', $pripojeneDokumenty);
            if (!in_array("$id", $seznamDokumentu)) return;
        } else if ($name) {
            // je zadano jen name - id nacte z prislusne promenne
            $id = $this ->xmlForm->variables[$name];
        }

        if (!(int)$id) return;

        if  ($this ->jePripojenyDokument((int)$id)) {
            // dokument je pripojeny k aktualnimu XML formulari => muze ho smazat vcetne zaznamu z tabulky DokumentXMLFormulare
            require_once('adsapp/dokumentyutils.php');
            $params = $this ->xmlForm ->database ->createParameters();
            DokumentyUtils::smazDokument($this ->xmlForm ->database, (int)$id, 'XML formular',
                'DokumentXMLFormulare', // podrizena tabulka
                'IdFormulare = ' . $params ->addParameter($this ->xmlForm ->id), $params,
                'IdDokumentu');

            $this ->xmlForm ->addLogData($name, StringUtils::fromUTF8('Odstraněn dokument id = ') . $id);

            // odstrani dokument jeste ze zadane promenne
            if ($name) {
                // id dokumentu muze byt (ale nemusi) v promenne se zadanym jmenem
                if ($this ->variableValue($name) == $id) {
                    $this ->setVariableValue($name, null);
                }

                // a dale muze byt v seznamu vsech prilozenych dokumentu
                if (($key = array_search($id, $seznamDokumentu)) !== false) {
                    unset($seznamDokumentu[$key]);
                    $this ->setVariableValue($name . '_dokumenty', implode(';', $seznamDokumentu));
                    $this ->setVariableValue($name, end($seznamDokumentu));
                }
            }
        }
    }
}
