<?php
/*
 * UTF8
 * Stanislav Elbl, 08-2014
 */

namespace Ron\RonPortal\XML;

use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\TextViewObject;
use Ron\StringUtils;


/**
 * Class XMLFormElement
 * @package Ron\RonPortal\XML
 * @deprecated
 */
class XMLFormElement extends \RonPortal\XML\XMLFormElement
{
}


/**
 * Obecny predek pro prikazy (obvykle elementy, ktere negeneruji zadny obsah, ale provadi nejakou cinnost)
 * Class XMLFormElementCommand
 * @package Ron\RonPortal\XML
 * @deprecated
 */
abstract class XMLFormElementCommand extends \RonPortal\XML\XMLFormElementCommand
{
}



/**
 * zaznam logu primo v (datovem) XML, nebo prikaz pro zobrazeni logu
 * Class XMLLogItem
 * @package Ron\RonPortal\XML
 */
class XMLLogItem extends XMLFormElement
{
    /** @var string */
    var $userName;
    /** @var string */
    var $time;
    /** @var string */
    var $name;
    /** @var string */
    var $value;

    /** @var int */
    var $poradiPredavani;
    /** @var string  Kod urovne predavani, na ktere k udalosti doslo */
    var $urovenPredavani;


    function __construct($xmlForm, $parent)
    {
        parent::__construct($xmlForm, $parent);
        // TODO: tohle nejak predelat
        $this ->poradiPredavani = $xmlForm ->poradiPredavani;
        $this ->urovenPredavani = $xmlForm ->kodUrovne;
    }


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->userName = StringUtils::fromUTF8($this->getAttributeValue('username'));
        $this->time = $this->getAttributeValue('time');
        $this->name = StringUtils::fromUTF8($this->getAttributeValue('name'));

        $val = $this->getAttributeValue('value');
        $this->value = isset($val) ? StringUtils::fromUTF8($val) : null;

        $this->poradiPredavani = $this->getAttributeValue('poradipredavani');
        $uroven = $this->getAttributeValue('urovenpredavani');
        $this->urovenPredavani = isset($uroven) ? StringUtils::fromUTF8($uroven) : null;
    }

    function loadData()
    {
        $this ->xmlForm ->addLogItem($this);
    }

    function writeXMLOutput($xmlWriter)
    {
        // TODO: predelat - zapsat nactene atributy aby se tu nemusel resit kazdy atribut samostatne
        $xmlWriter ->startElement('xmllog');
        if ($this->userName) $xmlWriter ->writeAttribute('username', StringUtils::toUTF8($this ->userName) );
        if ($this->time) $xmlWriter ->writeAttribute('time', $this ->time);
        if ($this->name) $xmlWriter ->writeAttribute('name', StringUtils::toUTF8($this ->name) );
        if (isset($this->value)) $xmlWriter ->writeAttribute('value', StringUtils::toUTF8($this ->value) );
        // TODO: vyresit typ - napr. volanim metody formulare ...
        if (isset($this->poradiPredavani)) $xmlWriter ->writeAttribute('poradipredavani', $this ->poradiPredavani);
        if (isset($this->urovenPredavani)) $xmlWriter ->writeAttribute('urovenpredavani', StringUtils::toUTF8($this ->urovenPredavani));
        $xmlWriter ->endElement();
    }


    function getHTMLOutput()
    {
        return '';
    }


    function process()
    {
        $value = $this ->nahradPromenne($this ->value);
        $this ->xmlForm ->addLogData(null, $value);
        parent::process();
    }
}


class XMLFormElementDropThis extends XMLFormElement
{
    function getHTMLOutput()
    {
        return $this->getElementsHTMLOutput();
    }
}


class XMLFormElementShowXMLLog extends XMLFormElementDropThis
{
    function getHTMLOutput()
    {
        $result = '';
        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));

        foreach ($this ->xmlForm ->logItems as $logItem) {
            if (!$name  ||  $logItem ->name == $name) {
                if ($this->isEmpty && !$this->elements) {
                    $attrs = $this->setHTMLAttributes(['class' => 'dotaznik_starsiodpoved']);
                    $attrs_str = $this->getHTMLAttributes($attrs);

                    $result .=
                        "<div $attrs_str>" .
                        '<span class="dotaznik_jmeno">' . $logItem ->time . ' - ' . StringUtils::htmlTextConvert($logItem ->userName) . ': </span>' .
                        '<span class="dotaznik_xmllog">' . StringUtils::longTextToHtml($logItem ->value, true). '</span>' .
                        '</div>';
                } else {
                    // element ma vnoreny obsah = vlastni format zobrazeni historie
                    $this ->setVariableValue('xmllog_name', $logItem ->name);
                    $this ->setVariableValue('xmllog_time', $logItem ->time);
                    $this ->setVariableValue('xmllog_username', $logItem ->userName);
                    $this ->setVariableValue('xmllog_value', $logItem ->value);
                    $this ->setVariableValue('xmllog_value_br', StringUtils::longTextToHtml($logItem ->value));
                    $this ->setVariableValue('xmllog_poradipredavani', $logItem ->poradiPredavani);
                    $this ->setVariableValue('xmllog_urovenpredavani', $logItem ->urovenPredavani);

                    $result .=
                        parent::getHTMLOutput();
                }
            }
        }

        return $result;
    }
}


class XMLFormElementIgnore extends XMLFormElement
{
    function getHTMLOutput()
    {
    }

    function event()
    {
    }

    function process()
    {
    }
}


class XMLFormElementTextNode extends XMLFormElement
{
    // text nebo CDATA primo v XML
    var $f_text;

    /* @param \XMLReader $xmlReader */
    function loadXML($xmlReader)
    {
        //$this ->f_text = $xmlReader ->value;


        if ($xmlReader->nodeType == \XMLReader::CDATA) {
            $this->f_text = $xmlReader->value;
        } else {
            // TODO: otestovat!
            $this->f_text = $xmlReader->value;
            //$this ->f_text = $xmlReader ->readOuterXml(); // zajisti ponechani napr. &lt; - bohuzel nefunguje pro CDATA  - Poznamka: PROC? Pak nefunguji SQL Dotazy
        }


        //$this ->f_text = $xmlReader ->value;
        //$this ->f_text = $xmlReader ->readInnerXml();
        //$this ->f_text = $xmlReader ->readOuterXml(); - nefunguje pro CDATA
        $xmlReader->read();
    }


    function getTextContent($recursive)
    {
        return $this ->getOwnText();
    }


    /**
     * Vraci obsah
     * @return string
     */
    function getOwnText()
    {
        return StringUtils::fromUTF8($this->f_text);
    }


    function getHTMLOutput()
    {
        return $this->nahradPromenne( StringUtils::fromUTF8($this->f_text), 'html' );
    }


    function writeXMLOutput($xmlWriter)
    {
        $xmlWriter->text($this->f_text);
    }

    function copyXML($xmlWriter)
    {
        $xmlWriter->text($this->f_text);
    }
}


class XMLFormElementText extends XMLFormElement
{
    // tag text
    var $text;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);
        $this ->text = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );

    }


    function getHTMLOutput() {
        return $this -> nahradPromenne($this -> text);
    }
}




// TODO: ... Udelat parametry ... nebude to tak jednoduche, protoze se musi poskladat sql text dotazu
//class XMLFormElementSQLParam extends XMLFormElementCommand
//{
//
//    function doCommand()
//    {
//        $sqlElement = $this ->findParentByName('sqlaction');
//        if (!$sqlElement) return;
//
//        $valueText = $this ->getAttributeValue('value');
//        if (!$valueText) $valueText = $this ->getTextContent(false);
//
//        throw new \Exception('XMLFormElementSQLParam::doCommand neni implementovano!');
//
//    }
//}


class XMLFormElementTry extends XMLFormElement
{
    /** @var XMLFormElementCatch[] */
    var $catchElements = [];

    function getHTMLOutput()
    {
        try {
            return parent::getHTMLOutput();
        } catch (\Exception $e) {
            $result = '';

            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExeptionVariables($e);
                    $result .= $catchElement->getElementsHTMLOutput();
                }
            }

            return $result;
        }
    }

    function event()
    {
        try {
            parent::elementsEvent();
        } catch (\Exception $e) {
            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExeptionVariables($e);
                    $catchElement->elementsEvent();
                }
            }
        }
    }

    function process()
    {
        try {
            parent::processElements();
        } catch (\Exception $e) {
            foreach ($this ->catchElements as $catchElement) {
                if ($catchElement ->handlesException($e)) {
                    $catchElement ->addExeptionVariables($e);
                    $catchElement->processElements();
                }
            }
        }
    }
}


/**
 * Zpracovava se jen pri zachyceni prislusne podminky - proto je to potomek XMLFormElementIgnore
 * Class XMLFormElementCatch
 * @package Ron\RonPortal\XML
 */
class XMLFormElementCatch extends XMLFormElementIgnore
{
    function __construct($xmlForm, $parent)
    {
        parent::__construct($xmlForm, $parent);
        if ($parent instanceof XMLFormElementTry) {
            $parent ->catchElements[] = $this;
        }
    }


    /**
     * @param \Exception $e
     * @return bool
     */
    function handlesException($e) {
        // TODO: umoznit zachytit jen konkretni vyjimky!
        return true;
    }

    /**
     * @param \Exception $e
     */
    function addExeptionVariables($e) {
        $this ->setVariableValue('XML_EXCEPTION', $e);
        $this ->setVariableValue('XML_EXCEPTIONMESSAGE', $e ->getMessage());

        if ($e instanceof DBException) $type = 'dbexception';
        //else if ($e instanceof DBException) $type = 'dbexception';
        else $type = 'exception';

        $this ->setVariableValue('XML_EXCEPTIONTYPE', $type);
    }
}




/**
 * View object - nezobrazi se primo, ale prida k formulari View object. Lze pouzit pro zobrazeni view z PHP souboru.
 * Class XMLFormElementViewObject
 * @package Ron\RonPortal\XML
 */
class XMLFormElementViewObject extends XMLFormElement
{
    protected function getView() {
        if ($this ->elementName == 'errormessages') {
            $this ->xmlForm ->showErrorMessages = false; // uz jsou zobrazeny
            return new MessageManagerViewObject();
        }

        $src = $this ->getAttributeValue('viewsource');
        if ($src) {
            return new BaseViewObject($src);
        }

        return new TextViewObject(
                parent::getElementsHTMLOutput(), null, null); // zadny wrap Element - muze byt primo v XML
    }


    function getHTMLOutput()
    {
        $view = $this ->getView();
        $this ->xmlForm ->viewObjects[] = $view;
        return $this ->xmlForm ->viewObjectPlaceholder;
    }
}


