<?php
/**
 * UTF8
 * stanislav.elbl, 12.10.2018
 */

namespace Ron\RonPortal\XML;

use Ron\RonPortal\NastaveniUzivatele;
use Ron\RonPortal\User;
use Ron\StringUtils;

require_once 'xml/xml_commands.php';


/**
 * Umoznuje nacteni a zmenu uzivatelskeho nastaveni jakehokoliv uzivatele.
 * Atribut rc urcuje osobu - pokud neni zadan, tak pracuje s aktualni osobou
 * Class XMLFormElementUserSettings
 * @package Ron\RonPortal\XML
 */
class XMLFormElementUserSettings extends XMLFormElementCommand
{
    function doCommand()
    {
        require_once 'adsapp/usersettings.php';

        $rc = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('rc')));

        if ($rc) {
            // pracuje s nastavenim zadaneho uzivatele
            $nastaveni = new NastaveniUzivatele();
            $nastaveni->load($this->xmlForm->database, $rc);
        } else {
            // pracuje s nastavenim aktualniho uzivatele
            $nastaveni = NastaveniUzivatele::loadUserSettings($this ->xmlForm ->database);
        }

        $key = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('key')));
        $variableName = $this ->nahradPromenne($this ->getAttributeValue('name'));

        if ($variableName) {
            // nastavi promennou
            if ($key) {
                // vysledna promenna je retezec - hodnota konkretniho nastaveni
                $this ->setVariableValue($variableName, $nastaveni ->getItem($key));
            } else {
                // vysledna promenna je cele pole s nastavenim
                $this ->setVariableValue($variableName, $nastaveni);
            }
        }

        if ($key) {
            $newValue = $this->getAttributeValue('newvalue');
            if (isset($newValue)) {
                $newValue = $this ->nahradPromenne(StringUtils::fromUTF8($newValue));

                // nastavi novou hodnotu zadane promenne
                if ($nastaveni->getItem('index') != $newValue) {
                    $nastaveni ->setItem($key, $newValue);
                    $nastaveni ->save($this ->xmlForm ->database, $this ->getBoolAttribute('log'));
                }
            }
        }
    }
}


/**
 * Stary tag, pouze pro kompatibilitu - pracuje jen s nastavenim aktualniho uzivatele
 * Class XMLFormElementLoadUserSettings
 * @package Ron\RonPortal\XML
 */
class XMLFormElementLoadUserSettings extends XMLFormElementCommand
{
    function doCommand()
    {
        require_once('adsapp/usersettings.php');

        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));

        $key = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('key')));
        $defaultValue = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('defaultvalue')));

        $value = NastaveniUzivatele::getUserSettingsItem($this ->xmlForm ->database, strtolower($key), $defaultValue);

        $this ->setVariableValue($name, $value);
    }
}

/**
 * Stary tag, pouze pro kompatibilitu - pracuje jen s nastavenim aktualniho uzivatele
 * Class XMLFormElementStoreUserSettings
 * @package Ron\RonPortal\XML
 */
class XMLFormElementStoreUserSettings extends XMLFormElementCommand
{
    function doCommand()
    {
        require_once('adsapp/usersettings.php');

        $key = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('key')));
        $defaultValue = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('defaultvalue')));
        $value = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('value')));

        $save = self::getBoolValue($this ->getAttributeValue('save', '1'));

        NastaveniUzivatele::setUserSettingsItem($this ->xmlForm ->database, strtolower($key), $value, $defaultValue, $save);
    }
}
