<?php
/**
 * UTF8
 * Roman.Szablatura 17.1.2017
 */

namespace Ron\RonPortal\Dochazka;

use Ron\Browser;
use Ron\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\RonPortal\RonPortalSettings;
use Ron\RonPortal\Rozdeleni;
use Ron\StringUtils;
use Ron\MessageManager;
use Ron\RonPortal\Prava;
use Ron\RonPortal\PravaNaOsoby;
use Ron\RonPortal\RonPortalFilterForm;

require_once('adsapp/ronportalfilter.php');
require_once('adsapp/usersettings.php');
require_once('ron/browser.php');

/**
 * Class ZaznamSnimacuFilterForm
 * @package Ron\RonPortal\Dochazka
 */
class ZaznamSnimacuFilterForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby */
    public $prava;

    public $rozsahZobrazeni;
    public $timestamp;

    public $jedenDen;
    public $desetDni;
    public $dvacetDni;
    public $tricetDni;
    public $datumCasOdDatumCasDo;

    public $cas_od = '';
    public $cas_do = '';

    public $filtrUmisteni;


    function __construct(Database $database, $action, $useGet)
    {
        parent::__construct($database, $action, $useGet);

        $this->prava = Prava::vratPrava('prohl_terminaly');
        $this->f_formaction = '?menuid=zaznamysnimacu';
        $this->f_classname = 'filtercontainer leftcaptions';

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
        $this->nastavOsoby(true,$this->prava,time(),time(),true);
        Prava::nastavObdobiProPrava(time(), time());
        $this->povolitVsechyOsoby = true;

        $this->filtrUmisteni = RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitumisteni');

        $this->jedenDen = new Option(1,\Ron\ronText('1_den'));
        $this->desetDni = new Option(10,\Ron\ronText('10_dni'));
        $this->dvacetDni = new Option(20,\Ron\ronText('20_dni'));
        $this->tricetDni = new Option(30,\Ron\ronText('30_dni'));


        //Implicitně jsou zobrazovány záznamy za aktuální den
        if ($_SESSION['vyberpocetdnu'] == '') {
            $_SESSION['vyberpocetdnu'] = 1;
        }

        //Možnost uložení hodnoty ke každé přihlášené osobě - prozkoumat..
//        NastaveniUzivatele::getUserSettingsItem($database, 'datum', time(), false);

        //Nastavení timestamp (pokud je datum ve špatném formátu nebo není v session nebo není, nastaví se na aktu. datum)
        if (isset($_POST['datum'])){
            try {
                $this->timestamp = DateUtils::dateStringToTimestamp($_POST['datum']);
            } catch(\Exception $e) {
                MessageManager::addErrorText( 'Bad_time_format' );
                $this->timestamp = time();
            }
        } else {

            if ($_SESSION['datum']) {
                $this->timestamp = $_SESSION['datum'];
            } else {
                $this->timestamp = time();
            }

        }

        //Nastavení času pokud ještě není nastaveno od půlnoci do vteřiny před dvanáctou
        $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
        $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $_SESSION['vyberpocetdnu'])) - 1);   //Odečítám vteřinu aby se správně zobrazoval rozsah doby ze které zobrazuji záznamy

        if ($_SESSION['input_cas_od'] ==''){
            $_SESSION['input_cas_od'] = date("G:i", $_SESSION['datum']);
        }
        if ($_SESSION['input_cas_do'] ==''){
            $_SESSION['input_cas_do'] = date("G:i", $_SESSION['datumdodata']);
        }
    }

    /**
     * Funkce pro nastavení času v závislostech
     * @throws \Exception
     */
    function upresneniCasu(){
        //Jen a pouze pokud si uživatel přeje upravit čas - jinak se bere v potaz čas 00:00 - 23:59
        if (isset($_SESSION['volbacasu'])) {
            //Jestliže uživatel vybere v rámci jednoho dne např. čas od 18:00 a čas do 6:00 pak se to bere tak, že si přeje vidět rozsah přes noc

            //Převod na timestamp, kvůli správnému vyhodnocení při porovnání
            $timestampCasOd = DateUtils::timeStringToTimestamp($_SESSION['input_cas_od'], DateUtils::today());
            $timestampCasDo = DateUtils::timeStringToTimestamp($_SESSION['input_cas_do'], DateUtils::today());

            if (($timestampCasOd > $timestampCasDo) && ($_SESSION['vyberpocetdnu'] == 1)){
                //Upřesnění rozsahu pomocí zadaného času - nejdříve vložím do proměnných čas a datum a následně sestavím výsledný datum s časem pro sql
                $sCasOd = $timestampCasOd;
                $sDatum = $_SESSION["datum"];
                $tempDatum = mktime(date("H", $sCasOd), date("i", $sCasOd), date("s", $sCasOd), date("n", $sDatum), date("j", $sDatum), date("Y", $sDatum));//Sestavuji datum ze dvou složek
                $_SESSION['datum'] = $tempDatum;

                //Pro tyto účely se neodečítá vteřina a k vypočtenému dni se připojí čas Do
                $sCasDo = $timestampCasDo;
                $sDatumDo = ((DateUtils::incDay($_SESSION['datum'], $_SESSION['vyberpocetdnu'])));
                $tempDatumDo = mktime(date("H", $sCasDo), date("i", $sCasDo), date("s", $sCasDo), date("n", $sDatumDo), date("j", $sDatumDo), date("Y", $sDatumDo));//Sestavuji datumDo ze dvou složek
                $_SESSION['datumdodata'] = $tempDatumDo;

            } else {
                //Jestliže uživatel vybere v rámci jednoho dne čas od např 6:00 a čas do 18:00 pak mu to nabídne daný rozsah ve zvoleném dni
                //Upřesnění rozsahu pomocí zadaného času - nejdříve vložím do proměnných čas a datum a následně sestavím výsledný datum s časem pro sql
                $sCasOd = $timestampCasOd;
                $sDatum = $_SESSION["datum"];
                $tempDatum = mktime(date("H", $sCasOd), date("i", $sCasOd), date("s", $sCasOd), date("n", $sDatum), date("j", $sDatum), date("Y", $sDatum));//Sestavuji datum ze dvou složek
                $_SESSION['datum'] = $tempDatum;

                $sCasDo = $timestampCasDo;
                $sDatumDo = $_SESSION["datumdodata"];
                $tempDatumDo = mktime(date("H", $sCasDo), date("i", $sCasDo), date("s", $sCasDo), date("n", $sDatumDo), date("j", $sDatumDo), date("Y", $sDatumDo));//Sestavuji datumDo ze dvou složek
                $_SESSION['datumdodata'] = $tempDatumDo;
            }
        } else {
            //Pokud není vybráno upřesnění času pak se automaticky bere v potaz od půlnoci do vteřiny před půlnocí daného rozsahu
            $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
            $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $_SESSION['vyberpocetdnu'])) - 1);
        }

        $this->rozsahZobrazeni = '';
        $this->rozsahZobrazeni .= '&nbsp;&nbsp;' . date('j.n.Y H:i:s', $_SESSION['datum']);
        $this->rozsahZobrazeni .= '&nbsp;&nbsp;&nbsp;-&nbsp;&nbsp;&nbsp;' . date('j.n.Y H:i:s', $_SESSION['datumdodata']) . '&nbsp;&nbsp;';
        //Ukládám do session kvůli zobrazení jako tooltip u jednotlivých dnů
        $_SESSION['rozsahzaznamusnimacu'] = $this->rozsahZobrazeni;
    }

    function renderZaznamyOd()
    {
        echo '<a name="top"></a>';

        echo '<div class="filterrow" >';
            echo '<span class="filtercaption">';
            echo \Ron\ronText('zobrazit_data_za_pocet_dni');
            echo '</span>';
        echo '<span class="zaznamy_od">';
        echo '<div class="tooltipek">';
        echo '<span>' . \Ron\ronText('zobrazena_doba') . ': ' . $this->rozsahZobrazeni . '</span>';
        //Combo volby dní a datumu
        $combo = new ComboBox('vyberpocetdnu',[$this->jedenDen, $this->desetDni, $this->dvacetDni, $this->tricetDni]);
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->htmlParams = 'onChange="submit();"';
        $combo->selectedOption = $combo ->findOption((int)$_SESSION['vyberpocetdnu'] != '' ? (int)$_SESSION['vyberpocetdnu'] : 1);
        $combo->render();
        echo '</div>';

        //DateInput pro volbu datumu a případně úpravu času
        echo '<div class="tooltipek">';
        echo '&nbsp;&nbsp;' . \Ron\ronText('Od') . '&nbsp;&nbsp;&nbsp;';
        DateInput::renderDateSelect('datum', date('j.n.Y', $this->timestamp), true);
        echo '<span>' . \Ron\ronText('pouze_dnesni_den') . '</span>';
        echo '</div>';
        echo '</span>';

        echo '<span title="' . \Ron\ronText('info_cas_bez_upresneni') . '">';
            require_once('ron/guiutils.php');
            GuiUtils::renderCheckbox('volbacasu', \Ron\ronText('upresneni_casu') . '&nbsp;&nbsp;', isset($_SESSION['volbacasu']),
                'onClick="submit()"', "", 1);
        echo '</span>';

        if (isset($_SESSION['volbacasu'])) {
            $casOd = date('G:i', DateUtils::timeStringToTimestamp($_SESSION['input_cas_od'], DateUtils::today()));
            $casDo = date('G:i', DateUtils::timeStringToTimestamp($_SESSION['input_cas_do'], DateUtils::today()));
            echo '<input type="text" title="hh:mm" name="input_cas_od" placeholder="hh:mm" value="' . $casOd . '" length="6" size="5">';
            echo '&nbsp;-&nbsp;&nbsp;';
            echo '<input type="text" title="hh:mm" name="input_cas_do" placeholder="hh:mm" value="' . $casDo . '" length="6" size="5">';
        } else {
            echo '<input type="text" title="hh:mm" name="input_cas_od" placeholder="hh:mm" value="' . $_SESSION['input_cas_od'] . '" length="6" size="5" readonly>';
            echo '&nbsp;-&nbsp;&nbsp;';
            echo '<input type="text" title="hh:mm" name="input_cas_do" placeholder="hh:mm" value="' . $_SESSION['input_cas_do'] . '" length="6" size="5" readonly>';
        }
        echo '</div>';
    }


    /**
     * @throws \Exception
     */
    function renderUmisteni()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';

        if (!$this ->vsechnyOsoby) {
            require_once('ron/guiutils.php');
            GuiUtils::renderCheckbox('jenUmisteni', \Ron\ronText('umisteni') . '&nbsp;&nbsp;', $_SESSION['jenUmisteni'],
                'onClick="submit()"');
        } else {
            echo  \Ron\ronText('umisteni');
        }
        echo '</SPAN>';

        $sql = "SELECT Kod, Popis from Umisteni ";

        $combo = new ComboBox('umisteni');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->loadData($this->database, $sql . " ORDER BY Popis, Kod", $_SESSION['umisteni']);
        $combo->htmlParams = "data-ron-select2='umisteni_kod' data-ron-autosubmit='1'";
        $combo->render();

        $combo = new ComboBox('umisteni_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql . " ORDER BY Kod, Popis", $_SESSION['umisteni']);
        $combo->htmlParams = "data-ron-select2='umisteni' data-ron-autosubmit='1'";
        $combo->render();
        echo '</DIV>';
    }


    /**
     * @throws \Exception
     */
    function renderTerminal()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption">';
        if (!$this ->vsechnyOsoby || RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitumisteni')) {
            require_once('ron/guiutils.php');
            GuiUtils::renderCheckbox('jenTerminal', \Ron\ronText('terminal') . '&nbsp;&nbsp;', isset($_SESSION['jenTerminal']),
                'onClick="submit()"');
        } else {
            echo  \Ron\ronText('terminal');
        }
        echo '</SPAN>';

        $params = $this->database->createParameters();
        if(RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitumisteni')) {
            $sql = "SELECT Terminal.Kod, Terminal.Popis FROM Terminal LEFT JOIN Umisteni ON (Terminal.KodUmisteni = Umisteni.Kod) WHERE Terminal.KodUmisteni = " . $params->addParameter($_SESSION['umisteni']);
        } else {
            $sql = "SELECT Kod, Popis FROM Terminal ";
        }

        $zvolenyTerminal = isset($_SESSION['terminal']) ? $_SESSION['terminal'] : '';

        $combo = new ComboBox('terminal');
        $combo->className = 'app_combo adsfilterforminput_popis';
        $combo->showCode = true;
        $combo->loadData($this->database, $sql . " ORDER BY Popis, Kod", $zvolenyTerminal, $params);
        $combo->htmlParams = "data-ron-select2='terminal_kod' data-ron-autosubmit='1'";
        $combo->render();

        $combo = new ComboBox('terminal_kod');
        $combo->className = 'app_combo adsfilterforminput_kod';
        $combo->keyIndex = 0;
        $combo->captionIndex = 0;
        $combo->loadData($this->database, $sql . " ORDER BY Kod, Popis", $zvolenyTerminal, $params);
        $combo->htmlParams = "data-ron-select2='terminal' data-ron-autosubmit='1'";
        $combo->render();
        echo '</DIV>';
    }

    /**
     * @throws \Exception
     */
    function renderKodOperace()
    {
        echo '<div class="filterrow">';
            echo '<span class="filtercaption">';
                require_once('ron/guiutils.php');
                GuiUtils::renderCheckbox('jenKodOperace', \Ron\ronText('kod_operace'). '&nbsp;&nbsp;', isset($_SESSION['jenKodOperace']),
                    'onClick="submit()"');
            echo '</span>';

            //Na doporučení Pavla výpis všech bez filtrování
            $sql = "SELECT Kod, Popis FROM OperaceDochazky";

            $zvolenaOperace = isset($_SESSION['kodoperace']) ? $_SESSION['kodoperace'] : '';

            $combo = new ComboBox('kodoperace');
            $combo->alwaysShowSelected = true;
            $combo->showCode = true;
            $combo->className = 'app_combo adsfilterforminput_popis';
            $combo->loadData($this->database, $sql . " ORDER BY Popis, Kod", $zvolenaOperace);
            $combo->htmlParams = "data-ron-select2='kodoperace_kod' data-ron-autosubmit='1'";
            $combo->render();

            $combo = new ComboBox('kodoperace_kod');
            $combo->className = 'app_combo adsfilterforminput_kod';
            $combo->keyIndex = 0;
            $combo->captionIndex = 0;
            $combo->loadData($this->database, $sql . " ORDER BY Kod, Popis", $zvolenaOperace);
            $combo->htmlParams = "data-ron-select2='kodoperace' data-ron-autosubmit='1'";
            $combo->render();
        echo '</div>';
    }


    private function renderDalsiVolby()
    {
        require_once('ron/guiutils.php');
        echo '<DIV>';
        echo '<span class="filtercaption">';
        GuiUtils::renderCheckbox('zobrazitFotky', \Ron\ronText('foto_v_radku') . '&nbsp;&nbsp;&nbsp;&nbsp;', isset($_SESSION['zobrazitFotky']),
            'onClick="submit()"');

        //Zobrazení checkboxu vteřin
            GuiUtils::renderCheckbox('vteriny', \Ron\ronText('zobrazeni_vterin') . '&nbsp;&nbsp;&nbsp;&nbsp;', isset($_SESSION['vteriny']),
                'onClick="submit()"');

        // vybraná tabulka ukolu - puvodnidochazka /  historiepruchodu / obě
            LinksList::renderLinks(
                ['puvodnidochazka' => \Ron\ronText('puvodnidochazka'), 'historiepruchodu' => \Ron\ronText('historiepruchodu'), 'obe' => \Ron\ronText('obe')],
                $_SESSION['volbatabulky'], '?menuid=zaznamysnimacu&volbatabulky=','',false);

        echo '</span>';
        echo '</div>';
        echo "<input type='hidden' id='doba_pro_refresh_stranky' value='" . RonPortalSettings::getValue('zaznamysnimacu_dobaaktualizacestranky') . "'>";
    }

    //LOAD funkce -------------------
    private function loadVolbaTabulek()
    {
        if (isset($_GET['volbatabulky'])) {
            if (($_GET['volbatabulky'] == 'puvodnidochazka' )|| ($_GET['volbatabulky'] == 'historiepruchodu') || ($_GET['volbatabulky'] == 'obe')) {
                $_SESSION['volbatabulky'] = $_GET['volbatabulky'];
            }
        } else {
            //Při prvním zobrazeni stránky se implicitně nastaví puvodnidochazka
            if (!isset($_SESSION['volbatabulky'])) {
                $_SESSION['volbatabulky'] = 'puvodnidochazka';
            }
        }
    }

    private function loadFotoVeStrance()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['zobrazitFotky']))
                $_SESSION['zobrazitFotky'] = true;
            else
                unset($_SESSION['zobrazitFotky']);
        }
    }


    private function loadVolbaCasu()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['volbacasu']))
                $_SESSION['volbacasu'] = true;
            else
                unset($_SESSION['volbacasu']);
        }
    }

    /**
     * @throws \Exception
     */
    function loadCasOdDO()
    {
        if (isset($_POST['adsfilterform'])){
            if(($_POST["input_cas_od"]) != ''){
                $_SESSION['input_cas_od'] = $_POST['input_cas_od'];
            }

            if (($_POST["input_cas_do"]) != ''){
                $_SESSION['input_cas_do'] = $_POST['input_cas_do'];
            }
        } else {
            if(isset($_SESSION['volbacasu']) || isset($_GET['volbacasu'])) {
                if (($_POST['input_cas_od']) != '') {
                    $_SESSION['input_cas_od'] = $_POST['input_cas_od'] ? $_POST['input_cas_od'] : $_SESSION['input_cas_od'];

                } else {
                    $_SESSION['input_cas_od'] = $_SESSION['input_cas_od'] ? $_SESSION['input_cas_od'] : date("G:i", $_SESSION['datum']);

                }

                if (($_POST['input_cas_do']) != '') {
                    $_SESSION['input_cas_do'] = $_POST['input_cas_do'] ? $_POST['input_cas_do'] : $_SESSION['input_cas_do'];

                } else {
                    $_SESSION['input_cas_do'] = $_SESSION['input_cas_do'] ? $_SESSION['input_cas_do'] : date("G:i", $_SESSION['datumdodata']);

                }
            }
        }
        $this->upresneniCasu();
    }


    /**
     * @throws \Exception
     */
    function loadPocetDnu()
    {
        if (isset($_POST['adsfilterform'])){
            $_SESSION['vyberpocetdnu'] = $_POST['vyberpocetdnu'];
        }
        $this->upresneniCasu();
    }

    function loadVteriny()
    {
        if (isset($_POST['adsfilterform'])) {
            if (isset($_POST['vteriny']))
                $_SESSION['vteriny'] = true;
            else
                unset($_SESSION['vteriny']);
        }
    }

    function loadDatumFiltru()
    {
        if (isset($_POST['adsfilterform'])) {
            try{
                //V případě že se zadá neexistující den v měsíci tak skript skončil chybou, ze které se jen tak nedá odejít - proto jsem toto ošetřil
                // a nastavuji v případě chyby aktuální datum
                $_SESSION['datum'] = DateUtils::dateStringToTimestamp($_POST['datum']);
            } catch (\Exception $ex){
                $_SESSION['datum'] = time();
            }
        }
    }

    function loadUmisteni()
    {
        if (isset($_POST['adsfilterform']))
        {
            if ($this->filtrUmisteni) {
                $_SESSION['jenUmisteni'] = $this ->vsechnyOsoby ? true : isset($_POST['jenUmisteni']);
                $_SESSION['umisteni'] = $_POST['umisteni'];

            }
        }
    }

    function loadTerminal()
    {
        if (isset($_POST['adsfilterform']))
        {
            if (isset($_POST['jenTerminal']))
                $_SESSION['jenTerminal'] = true;
            else
                unset($_SESSION['jenTerminal']);
            $_SESSION['terminal'] = $_POST['terminal'];

            if ($this->filtrUmisteni) {
                //Došlo-li ke změně filtru Umístění pak vypne filtr Terminál
                if ($_SESSION['umisteni'] != $_POST['umisteni']) {
                    unset($_SESSION['jenTerminal']);
                }
            }
            elseif ($this ->vsechnyOsoby) {
                $_SESSION['jenTerminal'] = true;
            }
        }
    }

    function loadKodOperace()
    {
        if (isset($_POST['adsfilterform'])){
            if (isset($_POST['jenKodOperace']))
                $_SESSION['jenKodOperace'] = true;
            else
                unset($_SESSION['jenKodOperace']);
            $_SESSION['kodoperace'] = $_POST['kodoperace'];
        }
    }


    function loadZobrazenaDoba()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['rozsahzaznamusnimacu'] = $this->rozsahZobrazeni;
        } else {
            if (isset($_SESSION['volbacasu'])) {
                $_SESSION['rozsahzaznamusnimacu'] = $this->rozsahZobrazeni;
            }
        }
    }


    function loadData()
    {
        $this->loadFotoVeStrance();
        $this->loadDatumFiltru();
        $this->loadVolbaCasu();
        $this->loadVteriny();
        $this->loadPocetDnu();
        $this->loadVolbaTabulek();
        $this->loadTerminal();
        $this->loadUmisteni();
        $this->loadKodOperace();
        $this->loadZobrazenaDoba();
        $this->loadCasOdDO();
        parent::loadData();
    }

    /**
     * @throws \Exception
     */
    function renderFilters()
    {
        $this->renderZaznamyOd();
        parent::renderFilters();

        if($this->filtrUmisteni){
            $this->renderUmisteni();
        }

        $this->renderTerminal();
        $this->renderKodOperace();
        $this->renderDalsiVolby();
    }
}


/**
 * Class FieldFoto
 * @package Ron\RonPortal\Dochazka
 */
class FieldFoto extends BrowserField
{
    public $browserAction;

    private $fotoRowId;
    private $fotoAltID;
    private $tabulka;

    public function __construct($database, $options, $action, $fotoRowId, $fotoAltID = null, $tabulka = 'foto')
    {
        parent::__construct($database, $options);
        if (!isset($_SESSION['zobrazitFotky'])) {
            $this->header = '';
        }

        $this->fotoRowId = $fotoRowId;
        $this->fotoAltID = $fotoAltID;
        $this->tabulka = $tabulka;

        $this->browserAction = $action;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if (isset($_SESSION['zobrazitFotky'])) {
            if ($rowData[$this->fotoRowId]) {
                $result = '<div class="zaznamysnimacufoto ronFoto">
                             <img src="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$rowData[$this->fotoRowId] . '" alt="' . $this->tabulka . '"/>
                           </div>';
            }
        } else {
            if ($rowData[$this->fotoRowId]) {
                /** @noinspection HtmlUnknownTarget */
                $result = '<span class="ronInfoFoto" id="snimac_foto_' . $this->database->stringToTimestamp($rowData[1]) . '_' . $this->fotoRowId . '"
                                data-foto-url="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$rowData[$this->fotoRowId] . '" ' .
                    (!is_null($this->fotoAltID) ? 'data-foto-alt="' . $rowData[$this->fotoAltID] . '"' : '')
                    . '">
                             <img src="images/16_info.png"  alt="' . $this->tabulka . '" />    
                           </span>';

            }
        }
        return $result;
    }
}


class FieldOperace extends BrowserField
{
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('kod_operace');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = 'OperaceDochazky.Popis';
        $this->sortSQLDesc = 'OperaceDochazky.Popis DESC';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .=  StringUtils::htmlTextConvert($rowData[17]) . ' (' . StringUtils::htmlTextConvert($rowData[4]) . ') ';
        return $result;
    }
}


class FieldTerminal extends BrowserField
{
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('terminal');
        $this->ordering = 1;
        $this->sortSQL = 'Terminal.Popis';
        $this->sortSQLDesc = 'Terminal.Popis DESC';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= $rowData[18] . ' (' . $rowData[5] . ')' . '&nbsp;&nbsp;&nbsp;' ;
        return $result;
    }
}

/**
 * Class FieldOsoba
 * @property string header
 * @property string sortSQLDesc
 * @property int ordering
 * @property string sortSQL
 * @package Ron\RonPortal\Dochazka
 */
class FieldOsoba extends BrowserField
{
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('Osoba');
        $this->ordering = 1;
        $this->sortSQL = 'Osoba.Prijmeni, Zaznamysnimacu.DatumCas ';
        $this->sortSQLDesc = 'Osoba.Prijmeni DESC, Zaznamysnimacu.DatumCas  DESC ';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= StringUtils::htmlTextConvert($rowData[16]) . ' ' . StringUtils::htmlTextConvert($rowData[21]) . ' ' . StringUtils::htmlTextConvert($rowData[22]) . '&nbsp;&nbsp;';
        return $result;
    }
}

/**
 * Class FieldDatumCasDen
 * @property string header
 * @property string sortSQLDesc
 * @property int ordering
 * @property string sortSQL
 * @property string align
 * @property string className
 * @package Ron\RonPortal\Dochazka
 */
class FieldDatumCasDen extends BrowserField
{
    var $datumRadku;
//    var $oblastDatumu = 0;

    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('den_datum_cas');
        $this->ordering = 1;
        $this->align = 'center';
        $this->sortSQL = 'Zaznamysnimacu.DatumCas';
        $this->sortSQLDesc = 'Zaznamysnimacu.DatumCas DESC';
        $this->className = "zaznamysnimacu";
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $datum = StringUtils::format('date', $this->database->stringToTimestamp($rowData[1]));
        $den = \Ron\ronText('Day_short_' . date("w", DateUtils::dateStringToTimestamp( $datum)));

        if (isset($_SESSION['vteriny'])) {
            $cas = date('G:i:s', $this->database->stringToTimestamp($rowData[1]));
        } else {
            $cas = date('G:i', $this->database->stringToTimestamp($rowData[1]));
        }
        $datumProTitulek = $_SESSION['rozsahzaznamusnimacu'] .'  / ' . $_SESSION['vyberpocetdnu'] . 'd.';

        if ($this->datumRadku == '') {
//            $this->oblastDatumu++;
            $this->datumRadku = $datum;
                $result .= "<p title='" . $datumProTitulek . "'>" . $den . '</p><p>' . $datum . "</p><p >" .  $cas . '</p>';
            $result = '<strong>' . $result . '</strong>';
//            $result .= "<div class='datumOblasti' data-ron-name='datumOblasti' data-ron-datum='$datum' id='datumOblasti_$this->oblastDatumu'></div>";
        } else {
            if ($this->datumRadku == $datum) {
                $result = "<p title='" . $datumProTitulek . "'>" . $den . "</p><p>" . $result . "</p><p title='" . $datum . "'>" .  $cas . '</p>';
            } else {
                $this->datumRadku = $datum;
//                $this->oblastDatumu++;
                $result .= "<p title='" . $datumProTitulek . "'>" . $den . '</p><p>' . $datum . "</p><p >" .  $cas . '</p>';
                $result = '<strong>' . $result . '</strong>';
//            $result .= "<div class='datumOblasti' data-ron-name='datumOblasti' data-ron-datum='$datum' id='datumOblasti_$this->oblastDatumu'></div>";
            }
        }
        return $result;
    }
}

/**
 * Class FieldCisloRadku
 * @property string align
 * @package Ron\RonPortal\Dochazka
 */
class FieldCisloRadku extends BrowserField
{
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->align = 'left';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        /** @noinspection HtmlUnknownAnchorTarget */
        $result .= '<a' . ' href="#top">' . $rowNumber . '</a>';
        return $result;
    }
}


/**
 * Class FieldOsobniCislo
 * @property string header
 * @package Ron\RonPortal\Dochazka
 */
class FieldOsobniCislo extends BrowserField
{
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = \Ron\ronText('Osobni_cislo');
        $this->align = 'center';
        $this->ordering = 1;
        $this->sortSQL = 'Zaznamysnimacu.OsobniCislo, Zaznamysnimacu.DatumCas';
        $this->sortSQLDesc = 'Zaznamysnimacu.OsobniCislo DESC, Zaznamysnimacu.DatumCas DESC';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $result .= $rowData[0] . '&nbsp;&nbsp;';
        return $result;
    }
}

/**
 * Class BrowserZaznamySnimacu
 * @package Ron\RonPortal\Dochazka
 */
class BrowserZaznamySnimacu extends Browser
{
    /** @var  ZaznamSnimacuFilterForm */
    public $filterForm;

    public $from;

    public $action;

    /** @var PravaNaOsoby  */
    var $pravaProhlizet;


    /**
     * BrowserZaznamySnimacu constructor.
     * @param Database $database
     * @param $action
     * @throws \Exception
     */
    public function __construct(Database $database, $action)
    {
        parent::__construct($database);

        $this->title = \Ron\ronText('zaznamy_snimacu');
        $this->code = 'browserzaznamysnimacu';
        $this->tableId = 'browserzaznamysnimacu';
        $this->browserUrl = '?menuid=zaznamysnimacu&';
        $this->filterForm = new ZaznamSnimacuFilterForm($this->database, null, null);
        $this->filterForm->loadData();
        $this->action = $action;
        $this->pravaProhlizet = Prava::vratPrava('prohl_terminaly');
        Prava::nastavObdobiProPrava(time(),time());


        $this->sqlSelectPart =
            "Zaznamysnimacu.OsobniCislo, Zaznamysnimacu.DatumCas, Zaznamysnimacu.RC,  Zaznamysnimacu.DatumNastupu, Zaznamysnimacu.KodOperace, Zaznamysnimacu.Terminal, Zaznamysnimacu.Karta, " . //0 - 6
            "VlozenoManualne, VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, Zaznamysnimacu.KodTypuRozdeleni, " .      //7 - 12
            "Rozdeleni, Zaznamysnimacu.Uzivatel, " . $database ->castWideMemo('Zaznamysnimacu.Poznamka') . ", " .     //13 - 15
            "Osoba.Prijmeni, OperaceDochazky.Popis, " .    //16 - 17
            "Terminal.Popis, S_AnoNeVlozenoManualne.Popis, S_AnoNeVlozenoOnLine.Popis, Osoba.Jmeno, Osoba.Titul, " .        //18 - 22
            "FotoZTerminalu.ID, Fotografie.ID, Osoba.IdFotografie";     //23 - 24

        //Volba tabulky pro sqlFromPart
        if($_SESSION['volbatabulky'] == 'puvodnidochazka'){
            $this->from = 'DatumCasOperace DatumCas, RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, 
                            Karta, VlozenoManualne, VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka
                            FROM PuvodniDochazka';

        } elseif ($_SESSION['volbatabulky'] == 'historiepruchodu'){
            $this->from = 'DatumCasPruchodu DatumCas, RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta, VlozenoManualne, 
                            VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka
                            FROM HistoriePruchodu';

        } elseif ($_SESSION['volbatabulky'] == 'obe'){
            $this->from = " DatumCasOperace DatumCas, RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta, VlozenoManualne, 
                            VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka
                            FROM PuvodniDochazka
                                UNION ALL
                            SELECT DatumCasPruchodu DatumCas, RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta, 
                            VlozenoManualne, VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka
                            FROM HistoriePruchodu ";
        } else {
            $this->from = 'DatumCasOperace DatumCas, RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta, 
                            VlozenoManualne, VlozenoOnLine, ZemSirka, ZemDelka, PopisPolohy, KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka
                            FROM PuvodniDochazka';
        }

        $this->sqlFromPart =
            "( SELECT " . $this->from . " ) Zaznamysnimacu " .
            "LEFT JOIN Osoba ON (Osoba.RC = Zaznamysnimacu.RC)" .
            "LEFT JOIN OperaceDochazky ON (OperaceDochazky.Kod = Zaznamysnimacu.kodOperace)" .
            "LEFT JOIN Terminal ON (Terminal.Kod = Zaznamysnimacu.Terminal)" .
            "LEFT JOIN Umisteni ON (Terminal.KodUmisteni = Umisteni.Kod)" .
            "LEFT JOIN S_AnoNe S_AnoNeVlozenoManualne ON (Zaznamysnimacu.VlozenoManualne = S_AnoNeVlozenoManualne.Kod)" .
            "LEFT JOIN S_AnoNe S_AnoNeVlozenoOnLine ON (Zaznamysnimacu.VlozenoOnLine = S_AnoNeVlozenoOnLine.Kod)" .
            "LEFT JOIN FotoZTerminalu ON (Zaznamysnimacu.DatumCas = FotoZTerminalu.DatumCasOperace AND Zaznamysnimacu.Terminal = FotoZTerminalu.Terminal)" .
            "LEFT JOIN Fotografie ON (Osoba.IdFotografie = Fotografie.id)";

        $this->primaryField = 0;
        $this->defaultSort = -3;


        //Definice sloupců
        $this->addField(new FieldFoto($this->database, [BROWSER_FIELD_HEADER_TEXT => 'foto_z_karty'],
            $this->action, 24, 16));

        $this->addField(new FieldFoto($this->database, [BROWSER_FIELD_HEADER_TEXT => 'foto_z_terminalu', BROWSER_FIELD_ORDERING => 1,
            BROWSER_FIELD_SQL_NAME => 'FotoZTerminalu.ID'],
            $this->action, 23, 16, 'fotozterminalu'));

        $this->addField(new FieldDatumCasDen($this->database, null));

        $this->addField(new FieldOsoba($this->database, null));

        $this->addField(new FieldOsobniCislo($this->database, null));

        if ((int)RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitdatumnastupu')) {
            $this->addField([BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'Zaznamysnimacu.DatumNastupu', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('Datum_nastupu'),
                BROWSER_FIELD_FORMAT => 'date', BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'left']);
        }

        $this->addField(new FieldOperace($this->database, null));

        $this->addField(new FieldTerminal($this->database, null));

        $this->addField([BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME =>'Karta', BROWSER_FIELD_HEADER_TEXT => '&nbsp;' . \Ron\ronText('Karta'),
            BROWSER_FIELD_ORDERING => 1]);

        if ((int)RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitzpusobvlozeni')) {
            $this->addField([BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_SQL_NAME => 'S_AnoNeVlozenoManualne.Popis', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('vlozeno_manualne'),
                BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

            $this->addField([BROWSER_FIELD_SQL_INDEX => 20, BROWSER_FIELD_SQL_NAME => 'S_AnoNeVlozenoOnLine.Popis', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('vlozeno_online'),
                BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);
        }

        $this->addField([BROWSER_FIELD_SQL_INDEX => 14, BROWSER_FIELD_SQL_NAME =>'Uzivatel', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('uzivatel'),
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField([BROWSER_FIELD_SQL_INDEX => 15, BROWSER_FIELD_SQL_NAME =>'Zaznamysnimacu.Poznamka', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('Poznamka'),
            BROWSER_FIELD_ORDERING => 1]);

        $this->addField(new FieldCisloRadku($this->database, null));
    }


    function renderFilter()
    {
        parent::renderFilter();
        $this->filterForm->render();
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        //Zobrazí pokud má na osobu právo
        if ($this->pravaProhlizet->lzeZobrazitRC($values[2])){
            return true;
        }
        return false;
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $filtrUmisteni = RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitumisteni');

        $result = '';
        //timestamp -> string
        $result .= " Zaznamysnimacu.DatumCas >= " . $params->addParameter($this->database->timestampToParam($_SESSION['datum'])) .
                " AND Zaznamysnimacu.DatumCas <= " . $params->addParameter($this->database->timestampToParam($_SESSION['datumdodata']));

        if(!$this ->filterForm ->vsechnyOsoby){
            $result .= " AND Zaznamysnimacu.RC = " . $params->addParameter($this ->filterForm ->vybranaOsobaRC);
        } else {
            require_once('adsapp/rozdeleni.php');
            if ($this ->filterForm ->kodRozdeleni != '') {
                if($result){
                    $result .= ' AND ';
                }
                $result .= Rozdeleni::sqlFilterRozdeleni($this->database, 'Zaznamysnimacu', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this ->filterForm ->kodRozdeleni,
                    time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni);
            }
            if ($this ->filterForm ->kodRozdeleni2 != '') {
                if($result){
                    $result .= ' AND ';
                }
                $result .= Rozdeleni::sqlFilterRozdeleni($this->database, 'Zaznamysnimacu', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this ->filterForm ->kodRozdeleni2,
                    time(), time(), $this ->filterForm ->vcetnePodrizenychRozdeleni2);
            }
        }


        if ($filtrUmisteni) {
            if ($this->filterForm->vsechnyOsoby) {
                if ($result != '') {
                    $result .= ' AND ';
                }
                $result .= " Umisteni.Kod = " . $params->addParameter($_SESSION['umisteni']);

            } else {
                if (($_SESSION['jenUmisteni'])) {
                    if ($result != '') {
                        $result .= ' AND ';
                    }
                    $result .= " Umisteni.Kod = " . $params->addParameter($_SESSION['umisteni']);
                }
            }
        }



        if (!$filtrUmisteni) {
            if ($this->filterForm->vsechnyOsoby) {
                if ($result != '') {
                    $result .= ' AND ';
                }
                $result .= " Zaznamysnimacu.Terminal = " . $params->addParameter($_SESSION['terminal']);

            } else {
                if (isset($_SESSION['jenTerminal'])) {
                    if ($result != '') {
                        $result .= ' AND ';
                    }
                    $result .= " Zaznamysnimacu.Terminal = " . $params->addParameter($_SESSION['terminal']);
                }
            }
        } else {
            if (isset($_SESSION['jenTerminal'])) {
                if ($result != '') {
                    $result .= ' AND ';
                }
                $result .= " Zaznamysnimacu.Terminal = " . $params->addParameter($_SESSION['terminal']);
            }
        }



        if (isset($_SESSION['jenKodOperace'])) {
            if($result){
                $result .= ' AND ';
            }
            $result .= "Zaznamysnimacu.KodOperace = " . $params->addParameter($_SESSION['kodoperace']);
        }

        return new SqlStatement($result, $params);
    }
}